MASTER_COPY_ADDRESS = "0xd9Db270c1B5E3Bd161E8c8503c55cEABeE709552"
MASTER_COPY_L2_ADDRESSES = ["0xfb1bffC9d739B8D520DaF37dF666da4C687191EA", "0x3E5c63644E683549055b9Be8653de26E0B4CD36E"]
PROXY_FACTORY_ADDRESS = "0xa6B71E26C5e0845f74c812102Ca7114b6a896AB2"
MULTI_SEND_ADDRESS = "0xA238CBeb142c10Ef7Ad8442C6D1f9E89e07e7761"
GAS_PRICE_MULTIPLIER = 1.1
DEVNET_RPC_URL = "http://localhost:8545"
NATIVE_TOKEN_ADDRESS = "0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE"
