# THIS IS AN AUTOGENERATED FILE. DO NOT MODIFY MANUALLY, PLEASE CHECK autotx/load_tokens.py TO DO ANY MODIFICATIONS

token_list = [
  {
    "chainId": 1,
    "address": "0x4da08a1Bff50BE96bdeD5C7019227164b49C2bFc",
    "symbol": "Mononoke-Inu",
    "name": "Mononoke Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmRxpqFfzK6hBbFTagFymqM38RSytycTMNssPnfdaVcmFn"
  },
  {
    "chainId": 1,
    "address": "0xA67E9F021B9d208F7e3365B2A155E3C55B27de71",
    "symbol": "KLEE",
    "name": "KleeKai",
    "decimals": 9,
    "logoURI": "ipfs://QmPe794NnE7FWe4oBVjqMq8YMoihkV1SF6QaUS84XeoUiE"
  },
  {
    "chainId": 1,
    "address": "0x7F3Fe9D492A9a60aEBb06d82cBa23c6F32CAd10b",
    "symbol": "iYFI",
    "name": "Fulcrum YFI iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmUp5wdbAuGJUvpQfuk3NeXDSg6egDkkyQ7qNdV3fQTGFa"
  },
  {
    "chainId": 137,
    "address": "0xC168E40227E4ebD8C1caE80F7a55a4F0e6D66C97",
    "symbol": "DFYN",
    "name": "Dfyn Network",
    "decimals": 18,
    "logoURI": "ipfs://QmddZ8RhepPYAat2of5r6Faton7RbXhvEuusGmpSpwAtX4"
  },
  {
    "chainId": 137,
    "address": "0x6f8a06447Ff6FcF75d803135a7de15CE88C1d4ec",
    "symbol": "SHIB",
    "name": "Shiba Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmZoALsLtigZeqLq5nTAE6qZQPBmWK8pvamZYDkG9sXQ4R"
  },
  {
    "chainId": 1,
    "address": "0x5DE8ab7E27f6E7A1fFf3E5B337584Aa43961BEeF",
    "symbol": "SDEX",
    "name": "SmarDex",
    "decimals": 18,
    "logoURI": "ipfs://Qmc7vLdbMNJYojw82VQBZNvdC6BuXfsBpMyhDis65yRZKT"
  },
  {
    "chainId": 1,
    "address": "0xF4134146AF2d511Dd5EA8cDB1C4AC88C57D60404",
    "symbol": "SNC",
    "name": "SunContract",
    "decimals": 18,
    "logoURI": "ipfs://QmVmWWXiH5yUptGFz2h79oNcJWCFSXiRcGooMQNLE2c1wR"
  },
  {
    "chainId": 1,
    "address": "0x81f8f0bb1cB2A06649E51913A151F0E7Ef6FA321",
    "symbol": "VITA",
    "name": "VitaDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmU9NjPwMhV9kdtNX7M5yNFD1WMFJdSnvQ5Tz3DTsahHNX"
  },
  {
    "chainId": 1,
    "address": "0x2b1D36f5B61AdDAf7DA7ebbd11B35FD8cfb0DE31",
    "symbol": "ITP",
    "name": "Interport",
    "decimals": 18,
    "logoURI": "ipfs://QmTWPTMwXJWuwDgNmUxpdkZA7UASQfrye7t7MESNBuUJox"
  },
  {
    "chainId": 1,
    "address": "0x8C543AED163909142695f2d2aCd0D55791a9Edb9",
    "symbol": "VLX",
    "name": "Velas",
    "decimals": 18,
    "logoURI": "ipfs://QmNdjq9EAYX3GGArB6w7AHaPnHi3raNh9JY8Y5QFG9WUXv"
  },
  {
    "chainId": 1,
    "address": "0x6982508145454Ce325dDbE47a25d4ec3d2311933",
    "symbol": "PEPE",
    "name": "Pepe",
    "decimals": 18,
    "logoURI": "ipfs://QmQZ2V8ebiaTh789BWV4ZActPcYovYGuvceVDUvSRb4Yz9"
  },
  {
    "chainId": 56,
    "address": "0xAD29AbB318791D579433D831ed122aFeAf29dcfe",
    "symbol": "FTM",
    "name": "Fantom",
    "decimals": 18,
    "logoURI": "ipfs://QmUqjbFkDVsYFErRTxEdVbcFHxxGyJ4aT4Bac8fqY2yART"
  },
  {
    "chainId": 42161,
    "address": "0xfb9Fed8cB962548A11fE7F6F282949061395c7F5",
    "symbol": "NUON",
    "name": "Nuon",
    "decimals": 18,
    "logoURI": "ipfs://QmNmdgTVDYwspuCg4DSNN8DFJJNGFJqd4YRk91RVoMPGgm"
  },
  {
    "chainId": 1,
    "address": "0x255Aa6DF07540Cb5d3d297f0D0D4D84cb52bc8e6",
    "symbol": "RDN",
    "name": "Raiden",
    "decimals": 18,
    "logoURI": "ipfs://QmTGZCkJGBpPLvWC8EFBg8Bsks7H9owB3L3PX8ZTuJYwHy"
  },
  {
    "chainId": 1,
    "address": "0xf29e46887FFAE92f1ff87DfE39713875Da541373",
    "symbol": "UNC",
    "name": "UniCrypt",
    "decimals": 18,
    "logoURI": "ipfs://QmTmB5z7GjuFmwDNarwyVdk5W5xKZvhseACmed14timpLh"
  },
  {
    "chainId": 56,
    "address": "0xf275e1AC303a4C9D987a2c48b8E555A77FeC3F1C",
    "symbol": "DPS",
    "name": "DEEPSPACE",
    "decimals": 9,
    "logoURI": "ipfs://QmRLARb3VKqXLM1a96LHTNMt7vRtdFeNZjvVmi6e6i5NWt"
  },
  {
    "chainId": 137,
    "address": "0xE0B52e49357Fd4DAf2c15e02058DCE6BC0057db4",
    "symbol": "agEUR",
    "name": "agEUR",
    "decimals": 18,
    "logoURI": "ipfs://QmVyaJncpkGuDKsiD8oAPPT7RtKBgBaznFiKuBuzhCgdd6"
  },
  {
    "chainId": 56,
    "address": "0x42981d0bfbAf196529376EE702F2a9Eb9092fcB5",
    "symbol": "SFM",
    "name": "SafeMoon",
    "decimals": 9,
    "logoURI": "ipfs://QmYg28PUMFsG2zXC1zV56HxPuBJWZFkmZKdeT36Dth7z93"
  },
  {
    "chainId": 42161,
    "address": "0xDC8184ba488e949815d4AAfb35B3c56ad03B4179",
    "symbol": "ROSX",
    "name": "Roseon",
    "decimals": 18,
    "logoURI": "ipfs://QmT428sS7SKaKoEFnZTcPHUiYRLcBeXNCG4JoxTc7F3fNr"
  },
  {
    "chainId": 1,
    "address": "0xFca59Cd816aB1eaD66534D82bc21E7515cE441CF",
    "symbol": "RARI",
    "name": "Rarible",
    "decimals": 18,
    "logoURI": "ipfs://Qmcz5moUax8V14EWsj7GFycWGb9PiY6PciXt2aLNPYGwmC"
  },
  {
    "chainId": 1,
    "address": "0x903560b1CcE601794C584F58898dA8a8b789Fc5d",
    "symbol": "crKP3R",
    "name": "Cream Keep3rV1",
    "decimals": 8,
    "logoURI": "ipfs://Qmf9FJz1F4fLPwDHerGndFvJ2FXBAnsGS5n1b4yiGVpBhh"
  },
  {
    "chainId": 1,
    "address": "0x4Ddc2D193948926D02f9B1fE9e1daa0718270ED5",
    "symbol": "cETH",
    "name": "Compound Ether",
    "decimals": 8,
    "logoURI": "ipfs://QmNdML81DxWR8wo7QNGz7qr5dAYd5A3MxUVZDV5jYzTAzL"
  },
  {
    "chainId": 1,
    "address": "0xEE06A81a695750E71a662B51066F2c74CF4478a0",
    "symbol": "DG",
    "name": "Decentral Games",
    "decimals": 18,
    "logoURI": "ipfs://QmVWizj6NtHrmW8wUd1JJTZtRREXzNcVN3thpyLRBphdKF"
  },
  {
    "chainId": 1,
    "address": "0x4f5fa8f2d12e5eB780f6082Dd656C565C48E0f24",
    "symbol": "GUM",
    "name": "Gourmet Galaxy",
    "decimals": 18,
    "logoURI": "ipfs://QmW6tzbujJsqfm5xn4uAJ5odFMWm7TyZGQpyk5Bme5yXMj"
  },
  {
    "chainId": 1,
    "address": "0xdBdb4d16EdA451D0503b854CF79D55697F90c8DF",
    "symbol": "ALCX",
    "name": "Alchemix",
    "decimals": 18,
    "logoURI": "ipfs://QmQ2x1TC2yYMH9TVPcmuPUkg9bDiVLLrMhkWBqwWxfP8so"
  },
  {
    "chainId": 1,
    "address": "0x1A5F9352Af8aF974bFC03399e3767DF6370d82e4",
    "symbol": "OWL",
    "name": "Owl",
    "decimals": 18,
    "logoURI": "ipfs://QmaC5z7xDD8HHMBmeebz66ALBeBNu5zpCK84e45yzYpP28"
  },
  {
    "chainId": 1,
    "address": "0x5845Cd0205b5d43AF695412a79Cf7C1Aeddb060F",
    "symbol": "SHD",
    "name": "ShardingDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmYsCkGQPCZRfwx5ADUEVihY8yDWoKw1F4wL5DKdP9F4ft"
  },
  {
    "chainId": 1,
    "address": "0xd1d2Eb1B1e90B638588728b4130137D262C87cae",
    "symbol": "GALA",
    "name": "Gala",
    "decimals": 8,
    "logoURI": "ipfs://QmYpSWREsM6FFknVoLwEPfoKsLkRmyDLNy8n1LzdGRbrRF"
  },
  {
    "chainId": 1,
    "address": "0x9D65fF81a3c488d585bBfb0Bfe3c7707c7917f54",
    "symbol": "SSV",
    "name": "ssv.network",
    "decimals": 18,
    "logoURI": "ipfs://QmXUQ6PLhZscLf1sSqhZCy3a4JUFqsUwD6GHyisGL3cKBs"
  },
  {
    "chainId": 56,
    "address": "0x12f31B73D812C6Bb0d735a218c086d44D5fe5f89",
    "symbol": "agEUR",
    "name": "agEUR",
    "decimals": 18,
    "logoURI": "ipfs://QmWg22EVRTLpZ6hcPvgpFpZidKtXLrGkGkEveiFkBKAme8"
  },
  {
    "chainId": 42161,
    "address": "0xAeAeeD23478c3a4b798e4ed40D8B7F41366Ae861",
    "symbol": "ANKR",
    "name": "Ankr",
    "decimals": 18,
    "logoURI": "ipfs://QmbvErh2b3MSwt6TtWbLu25Hsz4tK3EghsLrfrb6igkbCy"
  },
  {
    "chainId": 42161,
    "address": "0xDCbf4CB83d27C408B30dD7F39bfcabD7176B1Ba3",
    "symbol": "OOE",
    "name": "OpenOcean",
    "decimals": 18,
    "logoURI": "ipfs://QmbySTwrepiT7a7sYn5ykeMyLnVw8NN8UbB5HBFemKbXFN"
  },
  {
    "chainId": 1,
    "address": "0x88dF592F8eb5D7Bd38bFeF7dEb0fBc02cf3778a0",
    "symbol": "TRB",
    "name": "Tellor",
    "decimals": 18,
    "logoURI": "ipfs://QmVTePgFymSvT8qthxiTPNbCgJfNKegRQeTFe78eKjpiAU"
  },
  {
    "chainId": 1,
    "address": "0xFA1a856Cfa3409CFa145Fa4e20Eb270dF3EB21ab",
    "symbol": "IOST",
    "name": "IOST",
    "decimals": 18,
    "logoURI": "ipfs://QmQQ7zmrEcjjQXw6dVUU6L8QPp5DVBw6S3FrGtBuv4tnnM"
  },
  {
    "chainId": 56,
    "address": "0x31471E0791fCdbE82fbF4C44943255e923F1b794",
    "symbol": "PVU",
    "name": "Plant vs Undead",
    "decimals": 18,
    "logoURI": "ipfs://Qmc7uoj3GkPpcVA8ycsZBfRGMThJhdE1TahQQsbWuAiHV9"
  },
  {
    "chainId": 56,
    "address": "0xe9e7CEA3DedcA5984780Bafc599bD69ADd087D56",
    "symbol": "BUSD",
    "name": "Binance USD",
    "decimals": 18,
    "logoURI": "ipfs://QmUCSx8qo7kSFcdPJ7gLp9X9pPkue13KUmnL6ZzBZ6ejaQ"
  },
  {
    "chainId": 42161,
    "address": "0x93b346b6BC2548dA6A1E7d98E9a421B42541425b",
    "symbol": "LUSD",
    "name": "Liquity USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPzBCxWo5xMA8Wif7wCQQmuZ5ajP5sw4JSHthMxvxY2uR"
  },
  {
    "chainId": 1,
    "address": "0x39AA39c021dfbaE8faC545936693aC917d5E7563",
    "symbol": "cUSDC",
    "name": "Compound USD Coin",
    "decimals": 8,
    "logoURI": "ipfs://QmcRswXCTrx6UJm4gT5AVUoeRoGbioTBG4udH6Q8BqJyGE"
  },
  {
    "chainId": 42220,
    "address": "0xcebA9300f2b948710d2653dD7B07f33A8B32118C",
    "symbol": "USDC",
    "name": "USDC",
    "decimals": 6,
    "logoURI": "ipfs://QmS7rV584ATvEk5Skugr78GyAEFBuEo5eMp2rDNRCRNSmr"
  },
  {
    "chainId": 137,
    "address": "0x1BFD67037B42Cf73acF2047067bd4F2C47D9BfD6",
    "symbol": "WBTC",
    "name": "Wrapped Bitcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmWabMBiaCBXDEu1Cc6A6NoLVJZCqd9GWwtTgxentKbeZ4"
  },
  {
    "chainId": 1,
    "address": "0x35fA164735182de50811E8e2E824cFb9B6118ac2",
    "symbol": "eETH",
    "name": "ether.fi Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmU8GPsG97AdsapmbCHKLGePr4L8aj9hGwmM5nW8PwsH5q"
  },
  {
    "chainId": 1,
    "address": "0xE66747a101bFF2dBA3697199DCcE5b743b454759",
    "symbol": "GT",
    "name": "GateToken",
    "decimals": 18,
    "logoURI": "ipfs://QmUvzLbhkdm14Ko9jt8L7qWwyCwgXzhbhWjh77XyeV9t2m"
  },
  {
    "chainId": 56,
    "address": "0x8AC76a51cc950d9822D68b83fE1Ad97B32Cd580d",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmUQp1wZzr4UREn4AVLkKH5GbwpUCwf9w5VQQJeh1bmThJ"
  },
  {
    "chainId": 1,
    "address": "0x81313f7c5c9C824236c9E4cba3AC4B049986E756",
    "symbol": "HIPPO",
    "name": "HippoFinance",
    "decimals": 18,
    "logoURI": "ipfs://QmWkkoAmR2RH2WuVMGJMTXTLGQ6o9jecpzF8ggEd42mDrk"
  },
  {
    "chainId": 1,
    "address": "0x4da27a545c0c5B758a6BA100e3a049001de870f5",
    "symbol": "stkAAVE",
    "name": "Staked Aave",
    "decimals": 18,
    "logoURI": "ipfs://QmcjgRdQpPPRKCdqf9WzNb3bWC6CPt5UUctFCFULffHwqt"
  },
  {
    "chainId": 1,
    "address": "0xd9b89EEe86B15634c70CaB51BAF85615A4AB91a1",
    "symbol": "LKT",
    "name": "Locklet",
    "decimals": 18,
    "logoURI": "ipfs://QmZYzfg6p95Rc8JymKsVm5tAc9punGBUnTBJEqTKGQPy8L"
  },
  {
    "chainId": 8453,
    "address": "0xF544251D25f3d243A36B07e7E7962a678f952691",
    "symbol": "TAROT",
    "name": "Tarot",
    "decimals": 18,
    "logoURI": "ipfs://QmenPX7UnjGeCrfGjsNUWX6tJj3o6QXXw9co5WL8aNP9UF"
  },
  {
    "chainId": 1,
    "address": "0x5BEfBB272290dD5b8521D4a938f6c4757742c430",
    "symbol": "XFI",
    "name": "Xfinance",
    "decimals": 18,
    "logoURI": "ipfs://QmXTbaHEMmT7HTMPUzZCBFKc5tzc9VRPbaeZYKsYuaHvDk"
  },
  {
    "chainId": 1,
    "address": "0xa21Af1050F7B26e0cfF45ee51548254C41ED6b5c",
    "symbol": "OSAK",
    "name": "Osaka Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUng1scXVJSCH1MpqK4RanCsW1aXYew8KUZphtj3WUpah"
  },
  {
    "chainId": 56,
    "address": "0x9840652DC04fb9db2C43853633f0F62BE6f00f98",
    "symbol": "CGPT",
    "name": "ChainGPT",
    "decimals": 18,
    "logoURI": "ipfs://QmaDRX8xF6Z6K6xCJgWZJkvNUK2q1d2f2Wkhbw45mdYsrp"
  },
  {
    "chainId": 1,
    "address": "0x826180541412D574cf1336d22c0C0a287822678A",
    "symbol": "FLIP",
    "name": "Chainflip",
    "decimals": 18,
    "logoURI": "ipfs://Qmcq8a72QgD8TaaTruLKcU1k7Vha34NtQQqLxcvLMTv6ws"
  },
  {
    "chainId": 137,
    "address": "0x6f7C932e7684666C9fd1d44527765433e01fF61d",
    "symbol": "MKR",
    "name": "Maker",
    "decimals": 18,
    "logoURI": "ipfs://QmfAnPxYQrWrEdjzaSi9D1Cn5XRwpPBKYov6JSbvtfcy1j"
  },
  {
    "chainId": 1,
    "address": "0x667102BD3413bFEaa3Dffb48fa8288819E480a88",
    "symbol": "TKX",
    "name": "Tokenize Xchange",
    "decimals": 8,
    "logoURI": "ipfs://QmZkydE3JecjmCTFL7FRBVbT9jhPAddmAPYy2SVWs1oXJq"
  },
  {
    "chainId": 1,
    "address": "0x767FE9EDC9E0dF98E07454847909b5E959D7ca0E",
    "symbol": "ILV",
    "name": "Illuvium",
    "decimals": 18,
    "logoURI": "ipfs://QmVbrxYmsddYUx9VS1sBXLconvN56gaYLzXbFz35om5ykX"
  },
  {
    "chainId": 1,
    "address": "0x3c9d6c1C73b31c837832c72E04D3152f051fc1A9",
    "symbol": "BOR",
    "name": "BoringDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmZimEzofo5RoBD4FJe1oZu46HdL47NpR6kqw2krW2fSf3"
  },
  {
    "chainId": 8453,
    "address": "0xbf1aeA8670D2528E08334083616dD9C5F3B087aE",
    "symbol": "MAI",
    "name": "MAI",
    "decimals": 18,
    "logoURI": "ipfs://QmW1f5GFZDXN98kBzESE64QrpEbNrB9RmHBUxFkiz6zNef"
  },
  {
    "chainId": 1,
    "address": "0xA7925aA2A6e4575ab0C74D169f3BC3e03D4C319A",
    "symbol": "BETTER",
    "name": "Better Money",
    "decimals": 4,
    "logoURI": "ipfs://QmQaqdaX1jVNH8LKKJvnS6Wp9XBvipeZLYW1RMXpcx15VJ"
  },
  {
    "chainId": 1,
    "address": "0x99D8a9C45b2ecA8864373A26D1459e3Dff1e17F3",
    "symbol": "MIM",
    "name": "Magic Internet Money",
    "decimals": 18,
    "logoURI": "ipfs://QmaNmAVohHYtCFn8H9Lk393KDyj1rhWLUamGggFXM2eGLT"
  },
  {
    "chainId": 1,
    "address": "0xeDF6568618A00C6F0908Bf7758A16F76B6E04aF9",
    "symbol": "ARIA20",
    "name": "Arianee",
    "decimals": 18,
    "logoURI": "ipfs://Qmb4oWrtBnQ5U9GEZo76xUiLi6bK1TRfBnxTzM7caBdXGk"
  },
  {
    "chainId": 1,
    "address": "0xB528edBef013aff855ac3c50b381f253aF13b997",
    "symbol": "AEVO",
    "name": "Aevo",
    "decimals": 18,
    "logoURI": "ipfs://QmUiqcE8nJRCxUUYqmNawsT3W8sbodqPG5gq35saAWsuiT"
  },
  {
    "chainId": 1,
    "address": "0xC28E27870558cF22ADD83540d2126da2e4b464c2",
    "symbol": "SASHIMI",
    "name": "Sashimi",
    "decimals": 18,
    "logoURI": "ipfs://QmSKsBZ7zj8KQbifkPgnTR35ts4yWwNFTLBRw9TTNciw7C"
  },
  {
    "chainId": 1,
    "address": "0x08d967bb0134F2d07f7cfb6E246680c53927DD30",
    "symbol": "MATH",
    "name": "MATH",
    "decimals": 18,
    "logoURI": "ipfs://QmfW3CMSBMTj4St9YEju5N5WBnkUUhtUKEAJHDUft54JV6"
  },
  {
    "chainId": 137,
    "address": "0x0169eC1f8f639B32Eec6D923e24C2A2ff45B9DD6",
    "symbol": "ALGB",
    "name": "Algebra",
    "decimals": 18,
    "logoURI": "ipfs://QmcD4tifQJXHE9Sabrhen3sJjptMFxzmuja5hHZvNKjmq2"
  },
  {
    "chainId": 100,
    "address": "0xE2e73A1c69ecF83F464EFCE6A5be353a37cA09b2",
    "symbol": "LINK",
    "name": "Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://Qmdn6E15MELheFNupKrMosD9KPDFZ9M3wVKkhCuacT1cVj"
  },
  {
    "chainId": 1,
    "address": "0x51d3e4C0b2c83E62f5d517D250b3e856897d2052",
    "symbol": "SPMK",
    "name": "Space Monkey",
    "decimals": 9,
    "logoURI": "ipfs://QmdLE532KK1tP9Nqpt2f73nNCJVqyyQhCiELWmo7vhV31U"
  },
  {
    "chainId": 1,
    "address": "0xDEf1CA1fb7FBcDC777520aa7f396b4E015F497aB",
    "symbol": "COW",
    "name": "CoW Protocol Token",
    "decimals": 18,
    "logoURI": "ipfs://QmVWGYizJ4eE1fiDjhyZ1ndwASHM91fL77cyM4tzgDy5aW"
  },
  {
    "chainId": 1,
    "address": "0x92eF4FFBfe0Df030837b65d7FcCFE1ABd6549579",
    "symbol": "SWG",
    "name": "Swirge",
    "decimals": 18,
    "logoURI": "ipfs://Qmbe2QqxAienYXdFpDvy8EvXEgvRdfd5aA925aNVyCA4dp"
  },
  {
    "chainId": 1,
    "address": "0xD33526068D116cE69F19A9ee46F0bd304F21A51f",
    "symbol": "RPL",
    "name": "Rocket Pool",
    "decimals": 18,
    "logoURI": "ipfs://QmNSj1BfWphbDaTStW6JvrmnzoEHpFJpPNNczNAheANF4f"
  },
  {
    "chainId": 1,
    "address": "0xCc80C051057B774cD75067Dc48f8987C4Eb97A5e",
    "symbol": "NEC",
    "name": "Nectar",
    "decimals": 18,
    "logoURI": "ipfs://QmNm49AmtkcuocTfQhsffBTCAsBB6NEz6EGwSpzV5e2NNz"
  },
  {
    "chainId": 1,
    "address": "0x16ECCfDbb4eE1A85A33f3A9B21175Cd7Ae753dB4",
    "symbol": "ROUTE",
    "name": "Router Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmQDpwU7MdSaiJUmtybfuzK8x8hquCotCZpRVt7d23Lwtj"
  },
  {
    "chainId": 100,
    "address": "0xcb3231aBA3b451343e0Fddfc45883c842f223846",
    "symbol": "stPNK",
    "name": "Staking PNK",
    "decimals": 18,
    "logoURI": "ipfs://QmT3MPe6E9LQLGMP5RJ8foUJS8eYp4vaYh3mSYW5JvwAZd"
  },
  {
    "chainId": 1,
    "address": "0x4E15361FD6b4BB609Fa63C81A2be19d873717870",
    "symbol": "FTM",
    "name": "Fantom",
    "decimals": 18,
    "logoURI": "ipfs://QmaxPoKBmZYJpXL9GcyT8QsaN7bjb4FJLxHWGTDbjFeTdC"
  },
  {
    "chainId": 100,
    "address": "0x703120F2f2011a0D03A03a531Ac0e84e81F15989",
    "symbol": "UNCL",
    "name": "UNCL",
    "decimals": 18,
    "logoURI": "ipfs://QmdWnNAzfwfmZX64H3Nt2DatWNkA3eJcKcPwYCykwZrWav"
  },
  {
    "chainId": 1,
    "address": "0x7ea9C63E216D5565c3940A2B3d150e59C2907Db3",
    "symbol": "crBBTC",
    "name": "Cream Binance Wrapped BTC",
    "decimals": 8,
    "logoURI": "ipfs://QmXx9iBDAaNs7MoEihWLLGq48FfezGrWSHkHQ5nJsPPWSF"
  },
  {
    "chainId": 1,
    "address": "0x0a50C93c762fDD6E56D86215C24AaAD43aB629aa",
    "symbol": "LGO",
    "name": "LGO",
    "decimals": 8,
    "logoURI": "ipfs://QmdqKZfHJF7rA7VcxDLM1FsFGmQTgwDHHGpLXGYVM3jePA"
  },
  {
    "chainId": 1,
    "address": "0xDDB3422497E61e13543BeA06989C0789117555c5",
    "symbol": "COTI",
    "name": "COTI",
    "decimals": 18,
    "logoURI": "ipfs://QmTGW8hKt4LrKDg5bdt2kGsQXf5FaRToDtBjnWfU4BgZid"
  },
  {
    "chainId": 1,
    "address": "0xeED9e4F2450035D6426276a8aA2084966EE3b1bb",
    "symbol": "STEAK",
    "name": "Steaks Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmdWyKVDHRWYj4citGPaHbk88hV1hhb96zhG8syX96UgA7"
  },
  {
    "chainId": 1,
    "address": "0xe541504417670FB76b612B41B4392d967a1956c7",
    "symbol": "BSC",
    "name": "Bitsonic",
    "decimals": 18,
    "logoURI": "ipfs://QmP2S5iJtb6aS6p3FpKXyENXDVaruwpCUEGK298sCE9rAd"
  },
  {
    "chainId": 137,
    "address": "0xD6DF932A45C0f255f85145f286eA0b292B21C90B",
    "symbol": "AAVE",
    "name": "Aave",
    "decimals": 18,
    "logoURI": "ipfs://QmWZmCpB1yJS3qzFVxRQDWB63zFKJq9UmBae25Y5ijj7ma"
  },
  {
    "chainId": 1,
    "address": "0x8888801aF4d980682e47f1A9036e589479e835C5",
    "symbol": "MPH",
    "name": "88mph",
    "decimals": 18,
    "logoURI": "ipfs://QmaUQBpvFAooSdmYme5sDEvbRZ3Bn3d23m5eX8jzLWofVe"
  },
  {
    "chainId": 1,
    "address": "0x609c711783295209d9f33F535A7cA55B8FF87Af2",
    "symbol": "TOLL",
    "name": "Toll Free Swap",
    "decimals": 18,
    "logoURI": "ipfs://QmTexmxQtH2SnaH5UT9Z2FaYcVniCUhtrvWBfydpwyA5Tp"
  },
  {
    "chainId": 100,
    "address": "0x532801ED6f82FFfD2DAB70A19fC2d7B2772C4f4b",
    "symbol": "SWPR",
    "name": "Swapr",
    "decimals": 18,
    "logoURI": "ipfs://QmXeUKr6JoBmNnyerR2pLMRJqM1Vibo7DjGaBFpjnpAMvq"
  },
  {
    "chainId": 1,
    "address": "0x147E3d644d53ADadD18a28201b91e2B98aEB7f94",
    "symbol": "USE",
    "name": "ElenaUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmSkicuotobqabcHBL3q5AFV2XNXUimeCER1nHLpaPk9tN"
  },
  {
    "chainId": 1,
    "address": "0xb78B3320493a4EFaa1028130C5Ba26f0B6085Ef8",
    "symbol": "DRC",
    "name": "Dracula Token",
    "decimals": 18,
    "logoURI": "ipfs://QmQksi9q89SjXpR7y9waLp9rANst3EvTE7d8BZ8PM17k4G"
  },
  {
    "chainId": 1,
    "address": "0x00c83aeCC790e8a4453e5dD3B0B4b3680501a7A7",
    "symbol": "SKL",
    "name": "SKALE",
    "decimals": 18,
    "logoURI": "ipfs://Qmby2JvN2h9XzGtF3e1awG8Zhfeuqrs1iR8krjBrs8WGuW"
  },
  {
    "chainId": 1,
    "address": "0xA8580F3363684d76055bdC6660CaeFe8709744e1",
    "symbol": "FOL",
    "name": "Folder Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmSMZia8aKTB5jv3T2Tc7V3RPhC8x4S4duK98WeZdXn8JJ"
  },
  {
    "chainId": 1,
    "address": "0x697eF32B4a3F5a4C39dE1cB7563f24CA7BfC5947",
    "symbol": "ISLA",
    "name": "Insula",
    "decimals": 18,
    "logoURI": "ipfs://QmYqhR1ALvH8iSN39qZurpkjTQ63aMD75c76e5fe8adv6A"
  },
  {
    "chainId": 1,
    "address": "0xA54C67bd320Da4F9725a6f585b7635a0c09B122e",
    "symbol": "TIME",
    "name": "TimeMiner",
    "decimals": 6,
    "logoURI": "ipfs://QmetutbaP8Xnfhba2MFHAx6nbmQpdimP7nb65CxzpZ4TPP"
  },
  {
    "chainId": 56,
    "address": "0xE9C803F48dFFE50180Bd5B01dC04DA939E3445Fc",
    "symbol": "VLX",
    "name": "Velas",
    "decimals": 18,
    "logoURI": "ipfs://QmNdjq9EAYX3GGArB6w7AHaPnHi3raNh9JY8Y5QFG9WUXv"
  },
  {
    "chainId": 56,
    "address": "0xacb2d47827C9813AE26De80965845D80935afd0B",
    "symbol": "MCRN",
    "name": "MacaronSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmcSmS5w8WtQFbR3Tm1HYeq9HDwWoJsRZMcdF8jvtYDSaM"
  },
  {
    "chainId": 1,
    "address": "0x71eebA415A523F5C952Cc2f06361D5443545Ad28",
    "symbol": "XDAO",
    "name": "XDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmYQZ8Qav3op7jpqYS3CAM541XKthqxNcVyYdpHthXPy2J"
  },
  {
    "chainId": 1,
    "address": "0x18240BD9C07fA6156Ce3F3f61921cC82b2619157",
    "symbol": "iBZRX",
    "name": "Fulcrum BZRX iToken",
    "decimals": 18,
    "logoURI": "ipfs://Qmcw5JiJzMpj9LfogA4WmVgkDFD1jY58zbMJJruuviQWG2"
  },
  {
    "chainId": 1,
    "address": "0x176C674Ee533C6139B0dc8b458D72A93dCB3e705",
    "symbol": "iAAVE",
    "name": "Synthetic Inverse Aave",
    "decimals": 18,
    "logoURI": "ipfs://QmfR8tQeZqQyuic4KBYiR94xmj1B1AocGRPN6bbndQNzpg"
  },
  {
    "chainId": 1,
    "address": "0xe3c408BD53c31C085a1746AF401A4042954ff740",
    "symbol": "GMT",
    "name": "STEPN",
    "decimals": 8,
    "logoURI": "ipfs://QmaXgp7LnvKbQ64Bb3P1e92Cju9KNbxzwzdLaLqb7nrUUW"
  },
  {
    "chainId": 1,
    "address": "0xaaAEBE6Fe48E54f431b0C390CfaF0b017d09D42d",
    "symbol": "CEL",
    "name": "Celsius",
    "decimals": 4,
    "logoURI": "ipfs://QmZh8NKheUQj7yojD8UyNz38Fxiqr5tdF6LPs4EEGqdCQ8"
  },
  {
    "chainId": 56,
    "address": "0xE3233fdb23F1c27aB37Bd66A19a1f1762fCf5f3F",
    "symbol": "ELMON",
    "name": "Elemon",
    "decimals": 18,
    "logoURI": "ipfs://QmekkAgSj62mWoboiShQmXeM2F874r7BfxQyMJzVoDYdBv"
  },
  {
    "chainId": 1,
    "address": "0x3F09400313e83d53366147e3ea0e4e2279D80850",
    "symbol": "KSEED",
    "name": "Kush Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmeuQRfdsMi6WxCdUkgug7g6zJwobQTRBdC5j6myTGbhKw"
  },
  {
    "chainId": 1,
    "address": "0xd9B48548DE7710519221207D70E40aa24D7d1985",
    "symbol": "BOOTY",
    "name": "CandyBooty",
    "decimals": 9,
    "logoURI": "ipfs://QmSB2a89HyhCBQBTzjNPkYsrRvA2GEpgdHgkcV5RKrzkbp"
  },
  {
    "chainId": 1,
    "address": "0x0316EB71485b0Ab14103307bf65a021042c6d380",
    "symbol": "HBTC",
    "name": "Huobi BTC",
    "decimals": 18,
    "logoURI": "ipfs://Qmch2xhmSnXqfhP3K3h9hueX7wWuK7whuC4PiXLEFvoj4B"
  },
  {
    "chainId": 1,
    "address": "0xdd690D8824c00C84d64606FFb12640e932C1AF56",
    "symbol": "TAVITT",
    "name": "Tavittcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmRowFTSR5xVjaeJHk3qnLBgY52J6Ak4M3sHQAugCKHvRE"
  },
  {
    "chainId": 1,
    "address": "0x8A8079c7149B8A1611e5C5d978DCA3bE16545F83",
    "symbol": "iADA",
    "name": "Synthetic Inverse Cardano",
    "decimals": 18,
    "logoURI": "ipfs://QmVW2g6Zk66n97StujQtcM2cwMDLiVRczrCwKUdNLTtgaE"
  },
  {
    "chainId": 1,
    "address": "0xd82Df0ABD3f51425Eb15ef7580fDA55727875f14",
    "symbol": "DAV",
    "name": "DAV",
    "decimals": 18,
    "logoURI": "ipfs://QmdBSV6NDmYbWs5T2PvN31JaP91uVtmLo7KX72AyASNFgQ"
  },
  {
    "chainId": 1,
    "address": "0xD850942eF8811f2A866692A623011bDE52a462C1",
    "symbol": "VEN",
    "name": "VeChain",
    "decimals": 18,
    "logoURI": "ipfs://QmYeY7UXjY44rb749N7hW335upa81CzoPAmsvKTfBcYJhc"
  },
  {
    "chainId": 1,
    "address": "0xe0cCa86B254005889aC3a81e737f56a14f4A38F5",
    "symbol": "ALTA",
    "name": "Alta Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmQYCjD2vHB5S3ZqXyQmR6MuJXSh78U4v2R4Q6YNhSfXgk"
  },
  {
    "chainId": 56,
    "address": "0xF2c96E402c9199682d5dED26D3771c6B192c01af",
    "symbol": "SCLP",
    "name": "Scallop",
    "decimals": 18,
    "logoURI": "ipfs://QmXxaLh94i43pZV1L8unjmFX6Xg9Zxfz2GDQAd8BrHH8xR"
  },
  {
    "chainId": 1,
    "address": "0xea6412Fb370e8d1605E6aEeAA21aD07C3C7e9F24",
    "symbol": "MUSH",
    "name": "Mushroom",
    "decimals": 18,
    "logoURI": "ipfs://QmQNxXCBwyqPfYS3vcZkQiCfQbtMeSy1WNFk1oSJvZ4bXb"
  },
  {
    "chainId": 1,
    "address": "0x6AA493050427c7AF81D38F19Fd6b4f5DD3a75E3B",
    "symbol": "GDOG",
    "name": "GoldenDog",
    "decimals": 18,
    "logoURI": "ipfs://Qmcg3pHXixTCpJgvAeZG1cbiPDtdAnGa2HvxY39ts7qjDK"
  },
  {
    "chainId": 1,
    "address": "0x19062190B1925b5b6689D7073fDfC8c2976EF8Cb",
    "symbol": "BZZ",
    "name": "Swarm",
    "decimals": 16,
    "logoURI": "ipfs://QmXvMXR4s3i7u9Ush2297AS3sN8d4KsjKzjyyLFFYg81yu"
  },
  {
    "chainId": 1,
    "address": "0xDE72a052cBB212eFf11A4A33d61dF9C5A2De8dd1",
    "symbol": "ShunaV2",
    "name": "Shuna Inuverse",
    "decimals": 9,
    "logoURI": "ipfs://QmYYy5tXZWRNSZVZmCAz61DKghEYoX2WwfqiH1kSLdEkvK"
  },
  {
    "chainId": 1,
    "address": "0xE33AE4E795114279721047484E5ad5cc7DF24fcB",
    "symbol": "MCF",
    "name": "MCFinance",
    "decimals": 10,
    "logoURI": "ipfs://QmXsYVwYb7p6TXPM2JgNK3aYr3U9L1sP6FgyNVNHEK6vGF"
  },
  {
    "chainId": 1,
    "address": "0x15Eabb7500E44B7Fdb6e4051cA8DecA430cF9FB8",
    "symbol": "DXF",
    "name": "Dexfin",
    "decimals": 18,
    "logoURI": "ipfs://QmNdMJjdsUj1CLDUfCZKER7LsAdPqUtUx6UHZK2kEu8mfe"
  },
  {
    "chainId": 56,
    "address": "0x55d398326f99059fF775485246999027B3197955",
    "symbol": "USDT",
    "name": "Tether",
    "decimals": 18,
    "logoURI": "ipfs://Qmd9YVdtaXDZuMmeChdpK4LfizyLkBbVijpLyXGSNwFGEz"
  },
  {
    "chainId": 8453,
    "address": "0x236aa50979D5f3De3Bd1Eeb40E81137F22ab794b",
    "symbol": "tBTC",
    "name": "tBTC",
    "decimals": 18,
    "logoURI": "ipfs://QmRMESJHxFZWSS5KuMj5fW7SqEdkVvEATYVWkRRk6y2VeX"
  },
  {
    "chainId": 1,
    "address": "0x9041Fe5B3FDEA0f5e4afDC17e75180738D877A01",
    "symbol": "PRA",
    "name": "ProChain",
    "decimals": 18,
    "logoURI": "ipfs://QmSAKnVdLhPEh1GsXA3G6WACWf1LrmRMPtqr6rgNsGmToy"
  },
  {
    "chainId": 100,
    "address": "0x7ecF26cd9A36990b8ea477853663092333f59979",
    "symbol": "PERP",
    "name": "Perpetual Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmYQ4fHJzGnifyXQBCdVLACBcRQitnp25CRpAtfYJkmDjX"
  },
  {
    "chainId": 1,
    "address": "0x048Fe49BE32adfC9ED68C37D32B5ec9Df17b3603",
    "symbol": "SKM",
    "name": "Skrumble Network",
    "decimals": 18,
    "logoURI": "ipfs://QmTT9k5rjbuXuWjbK3L9KEK5THdgbFiFDiinyQLBzoRnyM"
  },
  {
    "chainId": 1,
    "address": "0x515d7E9D75E2b76DB60F8a051Cd890eBa23286Bc",
    "symbol": "GDAO",
    "name": "Governor DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmdaK91bZskcoPAvT6qhQFWCqm1mngk6FTaXbmETuVq2kT"
  },
  {
    "chainId": 1,
    "address": "0xE5Dada80Aa6477e85d09747f2842f7993D0Df71C",
    "symbol": "DOCK",
    "name": "Dock",
    "decimals": 18,
    "logoURI": "ipfs://QmZsAFfsAnpBkT2wvBnBc63V95M723uXqewmhWnTuNTHHS"
  },
  {
    "chainId": 10,
    "address": "0xc55E93C62874D8100dBd2DfE307EDc1036ad5434",
    "symbol": "mooBIFI",
    "name": "Moo BIFI",
    "decimals": 18,
    "logoURI": "ipfs://QmceYPfvAigeiYaY9KRRryo72k3FvJaQW8LjgJNtA7mytr"
  },
  {
    "chainId": 1,
    "address": "0x6e36556B3ee5Aa28Def2a8EC3DAe30eC2B208739",
    "symbol": "BUILD",
    "name": "BUILD Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmdH5edd5GotRwmA3tG9ER15qMUSagVDiZZ5m5PXfr4eGa"
  },
  {
    "chainId": 10,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmVurySKQi4AcZg3wUXXaudowWcTve6vKMbTHpQU6WExat"
  },
  {
    "chainId": 1,
    "address": "0x27269b3e45A4D3E79A3D6BFeE0C8fB13d0D711A6",
    "symbol": "iXRP",
    "name": "Synthetic Inverse Ripple",
    "decimals": 18,
    "logoURI": "ipfs://QmTHQxS7VYckTXACuSV5jupBZM57MBZA9KTYQvGgT3W1tn"
  },
  {
    "chainId": 137,
    "address": "0x8A953CfE442c5E8855cc6c61b1293FA648BAE472",
    "symbol": "PolyDoge",
    "name": "PolyDoge",
    "decimals": 18,
    "logoURI": "ipfs://QmV14aX7LZe7y7nP1WMBoDmJpRLDak3Hq6b1KY2wx81qY1"
  },
  {
    "chainId": 1,
    "address": "0xe03B2642A5111aD0EFc0cbCe766498c2dd562Ae9",
    "symbol": "BC",
    "name": "Old Bitcoin",
    "decimals": 9,
    "logoURI": "ipfs://QmTxkif8cpxCdNY8oKPV5pjryWovn1PaAfo3A1atDywkzo"
  },
  {
    "chainId": 1,
    "address": "0xBC86727E770de68B1060C91f6BB6945c73e10388",
    "symbol": "XNK",
    "name": "Ink Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmSNXeiXeH5w9KDJoAio7qVHPQrNNZb7NEXHs1rjcYq4Xf"
  },
  {
    "chainId": 1,
    "address": "0x808507121B80c02388fAd14726482e061B8da827",
    "symbol": "PENDLE",
    "name": "Pendle",
    "decimals": 18,
    "logoURI": "ipfs://Qmb14CmURbbr29FNYXpQDi1yDNuPg6ASuHPhKJgce94pT5"
  },
  {
    "chainId": 1,
    "address": "0x86D49fbD3B6f989d641E700a15599d3b165002AB",
    "symbol": "HUH",
    "name": "HUH Token",
    "decimals": 9,
    "logoURI": "ipfs://QmfRQi5M3Ssxwnoj7rkc7Huu7rrU5uiEBH648AAxCQRSdB"
  },
  {
    "chainId": 1,
    "address": "0x23696914Ca9737466D8553a2d619948f548Ee424",
    "symbol": "stkTRU",
    "name": "Staked TrueFi",
    "decimals": 18,
    "logoURI": "ipfs://QmeL8UXYBDu28kT2o6Agacbtp39o6RutsdroLkJdota2sY"
  },
  {
    "chainId": 137,
    "address": "0xc6C855AD634dCDAd23e64DA71Ba85b8C51E5aD7c",
    "symbol": "ICE",
    "name": "Decentral Games ICE",
    "decimals": 18,
    "logoURI": "ipfs://QmTcgtGRzwgartdXdA25bPq4M47p7p1FqDMhnn1KXju4ac"
  },
  {
    "chainId": 56,
    "address": "0x0E09FaBB73Bd3Ade0a17ECC321fD13a19e81cE82",
    "symbol": "CAKE",
    "name": "PancakeSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmRkkScJSS849mYdwbvrSyoYUq2WDtofXH4NmPPN67Ut4A"
  },
  {
    "chainId": 1,
    "address": "0xa8006C4ca56F24d6836727D106349320dB7fEF82",
    "symbol": "INXT",
    "name": "Internxt",
    "decimals": 8,
    "logoURI": "ipfs://QmT7crc2aNYQbNNxaWjqqY9QynYpmJN6sQDdf749bnjcjh"
  },
  {
    "chainId": 137,
    "address": "0x5fe2B58c013d7601147DcdD68C143A77499f5531",
    "symbol": "GRT",
    "name": "The Graph",
    "decimals": 18,
    "logoURI": "ipfs://QmaCTcJ7pLUgdjdXeZkubcRsGnyT73fvPBsgPLWMZevCfd"
  },
  {
    "chainId": 1,
    "address": "0xb0dFd28d3CF7A5897C694904Ace292539242f858",
    "symbol": "LOTTO",
    "name": "Lotto",
    "decimals": 18,
    "logoURI": "ipfs://QmfUcCcLdWtA17bmsi5eUPKpBFztnfnHbUad5PHZruD4kq"
  },
  {
    "chainId": 56,
    "address": "0x9ed7E4B1BFF939ad473dA5E7a218C771D1569456",
    "symbol": "REUNI",
    "name": "Reunit Wallet",
    "decimals": 6,
    "logoURI": "ipfs://QmbvkXoR9DDtCvJuK7zfdLWQ196Vat4t8DXaV8UN8j6XJf"
  },
  {
    "chainId": 42161,
    "address": "0x93ca0d85837FF83158Cd14D65B169CdB223b1921",
    "symbol": "ECLIP",
    "name": "Eclipse Fi",
    "decimals": 6,
    "logoURI": "ipfs://Qmct5hkMtnBmdQJQHxhv6HwV99kqR1bsBGRSU37FSnVNMP"
  },
  {
    "chainId": 1,
    "address": "0x4355fC160f74328f9b383dF2EC589bB3dFd82Ba0",
    "symbol": "OPT",
    "name": "OPUS",
    "decimals": 18,
    "logoURI": "ipfs://Qmcm29pMYCpw3qptFdMa8hEiLXPb3Uazhu83JtPsKeiJ95"
  },
  {
    "chainId": 43114,
    "address": "0xCE1bFFBD5374Dac86a2893119683F4911a2F7814",
    "symbol": "SPELL",
    "name": "Spell",
    "decimals": 18,
    "logoURI": "ipfs://QmRQ87x9VSD3iyvoqYXqVAcKGUAmUwYu5RiuqExNkjYyfa"
  },
  {
    "chainId": 1,
    "address": "0x56015BBE3C01fE05bc30A8a9a9Fd9A88917e7dB3",
    "symbol": "CAT",
    "name": "Cat Token",
    "decimals": 18,
    "logoURI": "ipfs://QmULFVfSZeuisib8P6qxtCog62RbGGLejcNSYNHjv88KmR"
  },
  {
    "chainId": 137,
    "address": "0xf2f77FE7b8e66571E0fca7104c4d670BF1C8d722",
    "symbol": "jBRL",
    "name": "Jarvis Synthetic Brazilian Real",
    "decimals": 18,
    "logoURI": "ipfs://QmRxdK4TQ9NE92nZX9PkWZRx8Fe4qHsRpCng35fU8uRKrm"
  },
  {
    "chainId": 1,
    "address": "0x249e38Ea4102D0cf8264d3701f1a0E39C4f2DC3B",
    "symbol": "UFO",
    "name": "The Truth",
    "decimals": 18,
    "logoURI": "ipfs://QmWhqPfACRPAxN9zAB2TFfZk6xij6gNRQMPv4WMAMxqA9u"
  },
  {
    "chainId": 1,
    "address": "0x3c5DF3077BcF800640B5DAE8c91106575a4826E6",
    "symbol": "yvCurve-pBTC",
    "name": "Curve pBTC Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmdJiyzi3DSTeVg7beeRuau9wChpG5RQnvUptcqwzvH8Tj"
  },
  {
    "chainId": 1,
    "address": "0xDC59ac4FeFa32293A95889Dc396682858d52e5Db",
    "symbol": "BEAN",
    "name": "Bean",
    "decimals": 6,
    "logoURI": "ipfs://QmNdYKXQzrykpaJdAvn8syjiZA8pwjwQGgPDP4TvYtSuGx"
  },
  {
    "chainId": 250,
    "address": "0x04068DA6C83AFCFA0e13ba15A6696662335D5B75",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmWbGfqePP3uAvPNHdYSP69aDznHGknsypTPRaiJT9NZF8"
  },
  {
    "chainId": 1,
    "address": "0x4aF4C553b4d47Eb5A5810d44bC1998a6Fc273FF5",
    "symbol": "MINE",
    "name": "The Official MINE Token",
    "decimals": 9,
    "logoURI": "ipfs://QmUKN7viQKkLfAVW9ijJC8BBwi32atCYcvhT9G5WLeAHUY"
  },
  {
    "chainId": 1,
    "address": "0x4156D3342D5c385a87D264F90653733592000581",
    "symbol": "SALT",
    "name": "SALT",
    "decimals": 8,
    "logoURI": "ipfs://Qme82cayc5xaNNeUrNP5xCdNYQRU2eujtC52MDr3nz7BTF"
  },
  {
    "chainId": 43114,
    "address": "0xB97EF9Ef8734C71904D8002F8b6Bc66Dd9c48a6E",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmWbGfqePP3uAvPNHdYSP69aDznHGknsypTPRaiJT9NZF8"
  },
  {
    "chainId": 1,
    "address": "0xd2dF355C19471c8bd7D8A3aa27Ff4e26A21b4076",
    "symbol": "sAAVE",
    "name": "Synthetic Aave",
    "decimals": 18,
    "logoURI": "ipfs://QmNNGEuyuuq1fJQtGuwyQZM1MwTHGAwWhjzJ3EJKtz4ZDV"
  },
  {
    "chainId": 42161,
    "address": "0xEe9801669C6138E84bD50dEB500827b776777d28",
    "symbol": "O3",
    "name": "O3 Swap",
    "decimals": 18,
    "logoURI": "ipfs://QmeQutpbBzKJRBXbgEptRhzvibPDesyfKeSpDa3PNPxXt6"
  },
  {
    "chainId": 1,
    "address": "0x0198f46f520F33cd4329bd4bE380a25a90536CD5",
    "symbol": "PLA",
    "name": "PlayChip",
    "decimals": 18,
    "logoURI": "ipfs://QmUdFoChuEwwNAQdvQKMzyqAyfMiTnVwrtbGbbwmQkR1Ab"
  },
  {
    "chainId": 1,
    "address": "0x0f51bb10119727a7e5eA3538074fb341F56B09Ad",
    "symbol": "DAO",
    "name": "DAO Maker",
    "decimals": 18,
    "logoURI": "ipfs://QmXFYWGruYGapJjmBHUsLmycCzoSNWQTofUmfd2qbMUzvW"
  },
  {
    "chainId": 1,
    "address": "0x69af81e73A73B40adF4f3d4223Cd9b1ECE623074",
    "symbol": "MASK",
    "name": "Mask Network",
    "decimals": 18,
    "logoURI": "ipfs://QmZd3hATUb7nyMmmwVGkuMhkBQ97WgKRNv3Kgv3JitR6s4"
  },
  {
    "chainId": 1,
    "address": "0xaD22f63404f7305e4713CcBd4F296f34770513f4",
    "symbol": "AWC",
    "name": "Atomic Wallet Coin",
    "decimals": 8,
    "logoURI": "ipfs://QmUAbNRhG4f7wfaYYuRY44FTwTmk4Q3YY97CCGL5hpuUgb"
  },
  {
    "chainId": 1,
    "address": "0xD31533E8d0f3DF62060e94B3F1318137bB6E3525",
    "symbol": "sREN",
    "name": "Synthetic Ren",
    "decimals": 18,
    "logoURI": "ipfs://QmRfmd1cBAq9RrPxqWBGr6zxA6ewgiA6WbX1JwaCdquyS4"
  },
  {
    "chainId": 324,
    "address": "0xa995ad25Ce5eB76972ab356168f5e1D9257E4d05",
    "symbol": "KOI",
    "name": "Koi",
    "decimals": 18,
    "logoURI": "ipfs://QmVYUDx87XCMvpCUPj6HrVvZdNmjJ3r8vJWDH3G8akn6oe"
  },
  {
    "chainId": 1,
    "address": "0x0d438F3b5175Bebc262bF23753C1E53d03432bDE",
    "symbol": "wNXM",
    "name": "Wrapped NXM",
    "decimals": 18,
    "logoURI": "ipfs://QmYd9eTThkScxrskxfZRuH5k8TCKiYF27XCG8wDZHrujFM"
  },
  {
    "chainId": 56,
    "address": "0x0D8Ce2A99Bb6e3B7Db580eD848240e4a0F9aE153",
    "symbol": "FIL",
    "name": "Filecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmY9qcv14cy8XyEh6HnGXsYCpGF4DPjDWQ8Z6MMYibDbXg"
  },
  {
    "chainId": 42220,
    "address": "0x218c3c3D49d0E7B37aff0D8bB079de36Ae61A4c0",
    "symbol": "FTM",
    "name": "Fantom",
    "decimals": 18,
    "logoURI": "ipfs://QmSR99yc7jE8eVMmVhHVPnGRK4SSE1aZfZZvQGuL7hZ5it"
  },
  {
    "chainId": 1,
    "address": "0x80D55c03180349Fff4a229102F62328220A96444",
    "symbol": "OPUL",
    "name": "Opulous",
    "decimals": 18,
    "logoURI": "ipfs://QmbCckR4APFcStuAbLRPey3hxqpqHaLgoRxiYuXmWKA1vU"
  },
  {
    "chainId": 1,
    "address": "0x00a8b738E453fFd858a7edf03bcCfe20412f0Eb0",
    "symbol": "ALBT",
    "name": "AllianceBlock",
    "decimals": 18,
    "logoURI": "ipfs://QmbYVpAxw5YtVsZZ48eMRCt62Rpb3muzGGfigvLG81F5Q8"
  },
  {
    "chainId": 1,
    "address": "0x028171bCA77440897B824Ca71D1c56caC55b68A3",
    "symbol": "aDAI",
    "name": "Aave DAI",
    "decimals": 18,
    "logoURI": "ipfs://QmQ7mmAnpuACaZorNWmZ3yRjcxcR4y1Fqk9BAGJ2ZhMScz"
  },
  {
    "chainId": 1,
    "address": "0xEB9A4B185816C354dB92DB09cC3B50bE60b901b6",
    "symbol": "ORS",
    "name": "Origin Sport",
    "decimals": 18,
    "logoURI": "ipfs://QmSbwSCBMYKjG4MxiujCfThVYSmufMJhp4d4bi8HpUPhHh"
  },
  {
    "chainId": 1,
    "address": "0x30D20208d987713f46DFD34EF128Bb16C404D10f",
    "symbol": "SD",
    "name": "Stader",
    "decimals": 18,
    "logoURI": "ipfs://QmaUWWwauRDLLRY94d1wf1MikX4pyTy8P3qtCLsZzk1H34"
  },
  {
    "chainId": 42161,
    "address": "0xa61F74247455A40b01b0559ff6274441FAfa22A3",
    "symbol": "MGP",
    "name": "Magpie",
    "decimals": 18,
    "logoURI": "ipfs://Qmcmn9X1UPVr1xqoF3tJCN29qfCwLVKCyDPWa9p18ZzsiX"
  },
  {
    "chainId": 10,
    "address": "0x94b008aA00579c1307B0EF2c499aD98a8ce58e58",
    "symbol": "USDT",
    "name": "Tether",
    "decimals": 6,
    "logoURI": "ipfs://Qmb3xCbcXGs8pa1YMvmHvN4dyadcaUkW7ZDy9mZL7pZ4aF"
  },
  {
    "chainId": 42161,
    "address": "0xe685d3CC0be48BD59082eDe30C3b64CbFc0326e2",
    "symbol": "cFRM",
    "name": "Crucible Ferrum",
    "decimals": 18,
    "logoURI": "ipfs://QmeyjVJGLtMxuYVdXEiUCL7hZFNMChyNAgEtojfeRiPKGt"
  },
  {
    "chainId": 1,
    "address": "0x779492d3644dDF4495Aa2d80C468E1B7be6AF1d2",
    "symbol": "CAS",
    "name": "CAS Coin",
    "decimals": 2,
    "logoURI": "ipfs://QmTdB9WRbsy1DRJWAZjkMPsujBqeVXqwoeCTC2cBfdCHeM"
  },
  {
    "chainId": 1,
    "address": "0xBEEF8e0982874e0292E6C5751C5A4092b3e1BEEF",
    "symbol": "mooBIFI",
    "name": "Moo BIFI",
    "decimals": 18,
    "logoURI": "ipfs://QmceYPfvAigeiYaY9KRRryo72k3FvJaQW8LjgJNtA7mytr"
  },
  {
    "chainId": 1,
    "address": "0x1BaFfbc23f2246040cCE12F7a140C38c5DC73e46",
    "symbol": "PIB",
    "name": "Passive Income Bot",
    "decimals": 18,
    "logoURI": "ipfs://QmP3hk8jfKmXrLZiQVmSB7qtTq9f9q79rBBTmVy4ioyS1k"
  },
  {
    "chainId": 10,
    "address": "0x395Ae52bB17aef68C2888d941736A71dC6d4e125",
    "symbol": "POOL",
    "name": "PoolTogether",
    "decimals": 18,
    "logoURI": "ipfs://Qmd1f4NaF27puWeZ87fgymsr8co79Ue2obzH9QumYtjzkn"
  },
  {
    "chainId": 1,
    "address": "0xBA11D00c5f74255f56a5E366F4F77f5A186d7f55",
    "symbol": "BAND",
    "name": "Band Protocol",
    "decimals": 18,
    "logoURI": "ipfs://Qmf6DjJq81F4BVvXarrNvTai9zmeuPJL9E4mqV34so9DYr"
  },
  {
    "chainId": 100,
    "address": "0x7838796B6802B18D7Ef58fc8B757705D6c9d12b3",
    "symbol": "MANA",
    "name": "Decentraland",
    "decimals": 18,
    "logoURI": "ipfs://QmY18MUVBQbY22cM9gRxub76EqqdHMiNydGkfoR1fBarEo"
  },
  {
    "chainId": 1,
    "address": "0x0DB8D8b76BC361bAcbB72E2C491E06085A97Ab31",
    "symbol": "IQN",
    "name": "IQeon",
    "decimals": 18,
    "logoURI": "ipfs://Qmc5LATEukPUtLzFzGeLjVs2DUJkm7qvBMabJ53zSC6Lgb"
  },
  {
    "chainId": 1,
    "address": "0x763186eB8d4856D536eD4478302971214FEbc6A9",
    "symbol": "BETR",
    "name": "BetterBetting",
    "decimals": 18,
    "logoURI": "ipfs://QmYpNwc4kJCFMtNezuU6qzAf4VAuVqtPTJBRHHDp5Q3ii5"
  },
  {
    "chainId": 250,
    "address": "0x29b0Da86e484E1C0029B56e817912d778aC0EC69",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRQq6tqSDu6pvZPmYHhZsF9dvUqzudAnfn6cKPurWzQbF"
  },
  {
    "chainId": 1,
    "address": "0x74b1Af114274335598da72f5C6ed7b954a016EeD",
    "symbol": "HIT",
    "name": "HitBTC Token",
    "decimals": 18,
    "logoURI": "ipfs://QmfKUmsdWkrn2x13dbWKDZNcqtTvJjeUXp1AZqUnmwvu9g"
  },
  {
    "chainId": 1,
    "address": "0xcAfE001067cDEF266AfB7Eb5A286dCFD277f3dE5",
    "symbol": "PSP",
    "name": "ParaSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmNjaNg3ou6QUGQ1qqtpjPgAJAgxNmtumBYrpNyA3hqyrN"
  },
  {
    "chainId": 1,
    "address": "0x3432B6A60D23Ca0dFCa7761B7ab56459D9C964D0",
    "symbol": "FXS",
    "name": "Frax Share",
    "decimals": 18,
    "logoURI": "ipfs://QmTyd9iks8MwhuZjNaBMsZA22ihbiUCbzSo6s8q3LBZfwa"
  },
  {
    "chainId": 56,
    "address": "0x982E609643794a31A07F5C5b142DD3a9cF0690be",
    "symbol": "TAROT",
    "name": "Tarot",
    "decimals": 18,
    "logoURI": "ipfs://QmenPX7UnjGeCrfGjsNUWX6tJj3o6QXXw9co5WL8aNP9UF"
  },
  {
    "chainId": 1,
    "address": "0x049e119413aB1f1b0fd836e657e3e4BA3ad9CaB8",
    "symbol": "KSF",
    "name": "Kesef.Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmdXhruDLRbHKSnvXwy6Xf5wdj1nmF8ckREiz5aZdU8KeV"
  },
  {
    "chainId": 10,
    "address": "0xFE8B128bA8C78aabC59d4c64cEE7fF28e9379921",
    "symbol": "BAL",
    "name": "Balancer",
    "decimals": 18,
    "logoURI": "ipfs://QmVkdh1F8BoNDvZQ4EAw6kzeMSpPibEvtrJJLVbZyK1dx2"
  },
  {
    "chainId": 1,
    "address": "0x42891d10B550d0c5C4ac2e17bcD7E2A19C7230f0",
    "symbol": "DXH",
    "name": "Daxhund",
    "decimals": 18,
    "logoURI": "ipfs://QmZKhY7phBNTyNNrwZ4tbJJXWmDtffY3991tcmuZKGiTot"
  },
  {
    "chainId": 56,
    "address": "0x715D400F88C167884bbCc41C5FeA407ed4D2f8A0",
    "symbol": "AXS",
    "name": "Axie Infinity",
    "decimals": 18,
    "logoURI": "ipfs://Qma5gXpmfTK1Ja4AwaYNyAQrxsu3fEQXj8Kqa3W2jyD5uw"
  },
  {
    "chainId": 1,
    "address": "0x77777FeDdddFfC19Ff86DB637967013e6C6A116C",
    "symbol": "TORN",
    "name": "Tornado Cash",
    "decimals": 18,
    "logoURI": "ipfs://QmUsMaj5mG8wmWxrgVQnzKWvLBLYU48ZiCJK8L2HE2cj1i"
  },
  {
    "chainId": 1,
    "address": "0x37E8789bB9996CaC9156cD5F5Fd32599E6b91289",
    "symbol": "AID",
    "name": "AidCoin",
    "decimals": 18,
    "logoURI": "ipfs://QmQq4XpqC6JMt5DDsp1mN5nKxfAbnSXRKK5gPrsDfPLEV1"
  },
  {
    "chainId": 1,
    "address": "0xB3319f5D18Bc0D84dD1b4825Dcde5d5f7266d407",
    "symbol": "cZRX",
    "name": "Compound 0x",
    "decimals": 8,
    "logoURI": "ipfs://QmRJPdKidsYZd6opEv8nspUygzvE4QFR4ZnxfAUMV5KRDE"
  },
  {
    "chainId": 1,
    "address": "0x31f3D9D1BeCE0c033fF78fA6DA60a6048F3E13c5",
    "symbol": "EBC",
    "name": "EBCoin",
    "decimals": 18,
    "logoURI": "ipfs://QmPTbm1Vq8ecxYz5EXPWSnaPQ24T4ZFk4RBbqQ6NjdqcZf"
  },
  {
    "chainId": 42161,
    "address": "0x34229B3f16fBCDfA8d8d9d17C0852F9496f4C7BB",
    "symbol": "IPOR",
    "name": "IPOR",
    "decimals": 18,
    "logoURI": "ipfs://QmS48QnuzPgbAZSZ4RcnknAryLucXhmFVdaFujg7YTBFXz"
  },
  {
    "chainId": 56,
    "address": "0xb59490aB09A0f526Cc7305822aC65f2Ab12f9723",
    "symbol": "LIT",
    "name": "Litentry",
    "decimals": 18,
    "logoURI": "ipfs://QmcwjvacVmihnkn947niG4uee41St5DBvfBMBwGA4NVRCb"
  },
  {
    "chainId": 1,
    "address": "0xf911a7ec46a2c6fa49193212fe4a2a9B95851c27",
    "symbol": "XAMP",
    "name": "Antiample",
    "decimals": 9,
    "logoURI": "ipfs://QmX2E3cvuYNuYze5gVzGrATQKBmtQVbM1XUzAAu3Vtgy33"
  },
  {
    "chainId": 1,
    "address": "0xe3Cb486f3f5C639e98cCBaF57d95369375687F80",
    "symbol": "renDGB",
    "name": "renDGB",
    "decimals": 8,
    "logoURI": "ipfs://QmS2VLkPDD73ugtPLLsP2ZJt65pvGjR1v1D4DAprBdJ8P8"
  },
  {
    "chainId": 1,
    "address": "0xF57e7e7C23978C3cAEC3C3548E3D615c346e79fF",
    "symbol": "IMX",
    "name": "Immutable X",
    "decimals": 18,
    "logoURI": "ipfs://QmX9pbEUjpYc2ECCRqeqmJuHzdzdhC15Gob2VW1g9bcNhx"
  },
  {
    "chainId": 1,
    "address": "0x27054b13b1B798B345b591a4d22e6562d47eA75a",
    "symbol": "AST",
    "name": "AirSwap",
    "decimals": 4,
    "logoURI": "ipfs://QmVgPYTYTjSkdLVXswM3xM9baRtqAjE6LB57etqbkAkjho"
  },
  {
    "chainId": 56,
    "address": "0xB0D502E938ed5f4df2E681fE6E419ff29631d62b",
    "symbol": "STG",
    "name": "Stargate Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmNZ1QbyLSB716ycPc7orcFa1FaVc9P7SrG2Ef2TuZinmt"
  },
  {
    "chainId": 56,
    "address": "0xa050FFb3eEb8200eEB7F61ce34FF644420FD3522",
    "symbol": "ARB",
    "name": "Arbitrum",
    "decimals": 18,
    "logoURI": "ipfs://QmdzPaPoFE9o221RAKfjCRW7KzUQBeKohFE5MFQMvhGzjd"
  },
  {
    "chainId": 1,
    "address": "0xdAC17F958D2ee523a2206206994597C13D831ec7",
    "symbol": "USDT",
    "name": "Tether",
    "decimals": 6,
    "logoURI": "ipfs://QmfGmBRAtNuZ6RmUkqqu9Pk1Qb5psNzRmhDLKR5iy6ZGTS"
  },
  {
    "chainId": 1,
    "address": "0xa5Fd1A791C4dfcaacC963D4F73c6Ae5824149eA7",
    "symbol": "JNT",
    "name": "Jibrel Network",
    "decimals": 18,
    "logoURI": "ipfs://QmW6FsgbajFqoGBiRUwiDtKoiSHG6JP4eJfEtyDSGZpzUV"
  },
  {
    "chainId": 1,
    "address": "0xCb5f72d37685C3D5aD0bB5F982443BC8FcdF570E",
    "symbol": "ROOT",
    "name": "Rootkit",
    "decimals": 18,
    "logoURI": "ipfs://QmV2XDNb7cS9bpGnUM6CFAdBtmbYoT9JzTW7rmjNdZPLSK"
  },
  {
    "chainId": 1,
    "address": "0xa3BeD4E1c75D00fa6f4E5E6922DB7261B5E9AcD2",
    "symbol": "MTA",
    "name": "Meta",
    "decimals": 18,
    "logoURI": "ipfs://QmbJZEkVEHnSQtf1rWTKfaACZvXq7DnNt2WPh4peeyGZAa"
  },
  {
    "chainId": 1,
    "address": "0xA49d7499271aE71cd8aB9Ac515e6694C755d400c",
    "symbol": "MUTE",
    "name": "Mute",
    "decimals": 18,
    "logoURI": "ipfs://QmRxy2KhwDypiZHoDbdD6kXv8q5snhBDWgtfuPJ27TosU6"
  },
  {
    "chainId": 1,
    "address": "0x57B946008913B82E4dF85f501cbAeD910e58D26C",
    "symbol": "POND",
    "name": "Marlin",
    "decimals": 18,
    "logoURI": "ipfs://QmRY1eRQ5KcuFuk1wQ7cqgQAXaxBe1jr11KMpxmV9QpEb8"
  },
  {
    "chainId": 1,
    "address": "0x8E16bf47065Fe843A82f4399bAF5aBac4E0822B7",
    "symbol": "BFIL",
    "name": "Binance Wrapped FIL",
    "decimals": 18,
    "logoURI": "ipfs://QmfNBqidLq8LGW4khY85h41VHMoEE4LgPkMoRFWb3gxk6f"
  },
  {
    "chainId": 1,
    "address": "0xF29992D7b589A0A6bD2de7Be29a97A6EB73EaF85",
    "symbol": "DMST",
    "name": "DMScript",
    "decimals": 18,
    "logoURI": "ipfs://QmZPceC6dtsumQ9v9NG3V7nELmkZ1eC9QNmSCAH4m2LRyN"
  },
  {
    "chainId": 1,
    "address": "0x76BC677d444f1E9d57dAf5187ee2b7dC852745aE",
    "symbol": "XFT",
    "name": "Offshift",
    "decimals": 18,
    "logoURI": "ipfs://QmX2ew7WQE4rTkGuE6Do63EF2usfuwkRNuRENSeWnTAQWZ"
  },
  {
    "chainId": 1,
    "address": "0x77252494C25444F8598A0c74Ffc90ADc535291a9",
    "symbol": "NMS",
    "name": "Nemesis ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmZFVhMWfJ5Umfb4KyLQ1JVkgGWsmugasB1Xqc9D17A4GT"
  },
  {
    "chainId": 42161,
    "address": "0x82e3A8F066a6989666b031d916c43672085b1582",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRQq6tqSDu6pvZPmYHhZsF9dvUqzudAnfn6cKPurWzQbF"
  },
  {
    "chainId": 1,
    "address": "0x15874d65e649880c2614e7a480cb7c9A55787FF6",
    "symbol": "EMAX",
    "name": "EthereumMax",
    "decimals": 18,
    "logoURI": "ipfs://QmaPxrvSUyyV4j7fzyDSkikowii5UnzdjP1a3SYeHFgULD"
  },
  {
    "chainId": 100,
    "address": "0xCCF1279C3406Ad1181bee00daB4b28B23D17Ddb1",
    "symbol": "JRT",
    "name": "Jarvis Reward Token",
    "decimals": 18,
    "logoURI": "ipfs://QmQDvx5ZAN7A2fmnnxgCo3S2FRVLkxWq58EzojxiN8HMwr"
  },
  {
    "chainId": 1,
    "address": "0x80fB784B7eD66730e8b1DBd9820aFD29931aab03",
    "symbol": "LEND",
    "name": "ETHLend",
    "decimals": 18,
    "logoURI": "ipfs://QmXYtNWxEcVY7QA5SY2QWdzANdd6oPrSPGU9qGyrksJPMV"
  },
  {
    "chainId": 1,
    "address": "0x27b7b1ad7288079A66d12350c828D3C00A6F07d7",
    "symbol": "yvCurve-IronBank",
    "name": "Curve Iron Bank Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmNMCSUEvLK28T4tPq4Eqa9347n2zErCEbj5KKwuBdXtVN"
  },
  {
    "chainId": 56,
    "address": "0x170C84E3b1D282f9628229836086716141995200",
    "symbol": "OP",
    "name": "Optimism",
    "decimals": 18,
    "logoURI": "ipfs://Qmf6Rw34hCk8DUzKhtAXVcL8WVhWJ881rQn9YB88SAokRn"
  },
  {
    "chainId": 56,
    "address": "0x0112e557d400474717056C4e6D40eDD846F38351",
    "symbol": "PHA",
    "name": "Phala",
    "decimals": 18,
    "logoURI": "ipfs://QmcAZ5Um5oABpK4n5jr5egcG1TQdwDUz2DCyA2WkMxcbz6"
  },
  {
    "chainId": 56,
    "address": "0x43B35e89d15B91162Dea1C51133C4c93bdd1C4aF",
    "symbol": "SAKAI",
    "name": "Sakai Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmerwruuV7E1pnbphFY9gbxANHuZg3Gfvxd1s6gG9RK6Do"
  },
  {
    "chainId": 1,
    "address": "0xeb269732ab75A6fD61Ea60b06fE994cD32a83549",
    "symbol": "USDx",
    "name": "dForce USDx",
    "decimals": 18,
    "logoURI": "ipfs://QmdA1ChgBU1oGw33UMuXf32BsE8ALfKUdBXhYDS6aHFnyU"
  },
  {
    "chainId": 100,
    "address": "0x177127622c4A00F3d409B75571e12cB3c8973d3c",
    "symbol": "COW",
    "name": "CoW Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmPC2wbiuXcS2oirWYL6Ln5X4PrXCHY3tNsoc65FTGhFSR"
  },
  {
    "chainId": 1,
    "address": "0xdB25f211AB05b1c97D595516F45794528a807ad8",
    "symbol": "EURS",
    "name": "EURS",
    "decimals": 2,
    "logoURI": "ipfs://QmYwjLKULQFgkcmcFNKC9BFhBR1ZEuicTYYwboUVXA9nGp"
  },
  {
    "chainId": 1,
    "address": "0xb5b8F5616Fe42d5ceCA3e87F3FddbDd8F496d760",
    "symbol": "ZPR",
    "name": "ZPER",
    "decimals": 18,
    "logoURI": "ipfs://QmbPfB8wAXWS6mzxCEHqnptBPTELPS2z1HHZ5cFKT3Pvny"
  },
  {
    "chainId": 1,
    "address": "0x8f3470A7388c05eE4e7AF3d01D8C722b0FF52374",
    "symbol": "VERI",
    "name": "Veritaseum",
    "decimals": 18,
    "logoURI": "ipfs://QmfAmwEfBRNVEqv7T3BxuRrnwu6e8KAKaxoumtWCKPrZKG"
  },
  {
    "chainId": 1,
    "address": "0x65Be44C747988fBF606207698c944Df4442efE19",
    "symbol": "FUCK",
    "name": "FUCK Token",
    "decimals": 4,
    "logoURI": "ipfs://QmTd4ZwBKrrfSLrNFfs9BABTKNkcyz8qXJZu8D1RtZtBdj"
  },
  {
    "chainId": 137,
    "address": "0xbA777aE3a3C91fCD83EF85bfe65410592Bdd0f7c",
    "symbol": "CONE",
    "name": "BitCone",
    "decimals": 18,
    "logoURI": "ipfs://QmRyxPiwuxm7RSeDtWEwr63kPTUVdQjnhWkhc6MkQosGSc"
  },
  {
    "chainId": 1,
    "address": "0xf50B5e535F62a56A9BD2d8e2434204E726c027Fa",
    "symbol": "sFB",
    "name": "Synthetic Facebook",
    "decimals": 18,
    "logoURI": "ipfs://QmenGaXACwb8SZa5tNBSRE6kaibExT3WTitm63nFzwGEh5"
  },
  {
    "chainId": 56,
    "address": "0x686318000d982bc8dCC1cdCF8fFd22322F0960Ed",
    "symbol": "OPUL",
    "name": "Opulous",
    "decimals": 18,
    "logoURI": "ipfs://QmbCckR4APFcStuAbLRPey3hxqpqHaLgoRxiYuXmWKA1vU"
  },
  {
    "chainId": 10,
    "address": "0x1F514A61bcde34F94Bc39731235690ab9da737F7",
    "symbol": "TAROT",
    "name": "Tarot",
    "decimals": 18,
    "logoURI": "ipfs://QmP8gkWvZKQffn32AF85jfq6Yg17ELnMBVSLSrRzt2Rg5k"
  },
  {
    "chainId": 1,
    "address": "0x84BB61Eb0336b309Ccf14Cc68bD1888cFa4846eA",
    "symbol": "MINE",
    "name": "The Official MINE Token",
    "decimals": 9,
    "logoURI": "ipfs://QmPZ9GDYto4EeqRmcWfxhjcVfsZjVyexcTuYt9wuhQQDB3"
  },
  {
    "chainId": 1,
    "address": "0x269895a3dF4D73b077Fc823dD6dA1B95f72Aaf9B",
    "symbol": "sKRW",
    "name": "Synthetic South Korean Won",
    "decimals": 18,
    "logoURI": "ipfs://QmQ5x1cJ82uY1uhrTG4ZXabEucjENb8Db7xj4hj9gHPPK4"
  },
  {
    "chainId": 56,
    "address": "0x1AF3F329e8BE154074D8769D1FFa4eE058B1DBc3",
    "symbol": "DAI",
    "name": "Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmTswuPmisP64jxzZvGz6AaK2gosJyPeTwqMkNPob6pdaa"
  },
  {
    "chainId": 1,
    "address": "0xed0889F7E1c7C7267407222Be277e1f1Ef4d4892",
    "symbol": "MEL",
    "name": "Melalie",
    "decimals": 18,
    "logoURI": "ipfs://QmVUHTjZqv8qf1SEjfLhurKtnPDphYeEag4hLTMwvGjdya"
  },
  {
    "chainId": 56,
    "address": "0xd17479997F34dd9156Deef8F95A52D81D265be9c",
    "symbol": "USDD",
    "name": "Decentralized USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPtvMuo6UQwDXB7cTVz85Ms5noX6rnnNKi3HNmwKrPf8u"
  },
  {
    "chainId": 1,
    "address": "0x4563554284aA7148D6E6D0351519E954Ba3B6E02",
    "symbol": "RWA",
    "name": "Xend Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmZmWGvhWWoPmDdHKTjozuA9F3vvkCfA28M3iscCyQqAzr"
  },
  {
    "chainId": 43114,
    "address": "0x6e84a6216eA6dACC71eE8E6b0a5B7322EEbC0fDd",
    "symbol": "JOE",
    "name": "JOE",
    "decimals": 18,
    "logoURI": "ipfs://QmfHbKz1Xm96EpDwsZ6jw3VyxGnbjN8iqqza3kksBqW93E"
  },
  {
    "chainId": 1,
    "address": "0x92B767185fB3B04F881e3aC8e5B0662a027A1D9f",
    "symbol": "crDAI",
    "name": "Cream Dai",
    "decimals": 8,
    "logoURI": "ipfs://Qmb3W78NJHdFhJT5bWMVpQk9UNTzvUCrNWoaXdRHiUz8jx"
  },
  {
    "chainId": 1,
    "address": "0x87869A9789291A6cEC99f3c3Ef2fF71fcEb12a8e",
    "symbol": "CMOS",
    "name": "CoinMerge OS",
    "decimals": 9,
    "logoURI": "ipfs://QmbG1w15Ky3e7Lrwdozbfw7MzBQErPxRW8sKdFphGcS82v"
  },
  {
    "chainId": 1,
    "address": "0xcbd55D4fFc43467142761A764763652b48b969ff",
    "symbol": "ASTRO",
    "name": "AstroTools",
    "decimals": 18,
    "logoURI": "ipfs://QmaWzfAPxUqv1FoyiQGJfp8WX5C1jwJZaoK6ZJPDXdbmxv"
  },
  {
    "chainId": 56,
    "address": "0x27c073e8427aa493a90b8dC8b73A89e670FD77bB",
    "symbol": "RDP",
    "name": "Radpie",
    "decimals": 18,
    "logoURI": "ipfs://QmZxTu24YYCAVqeBi8cNLNoApZXA1D9W2MiYt4jnjf5KXU"
  },
  {
    "chainId": 1,
    "address": "0x43Dfc4159D86F3A37A5A4B3D4580b888ad7d4DDd",
    "symbol": "DODO",
    "name": "DODO",
    "decimals": 18,
    "logoURI": "ipfs://Qmf2bQc8uVSmsp2QJicH7JC9JyTX7APek93iE2jQBSNjoC"
  },
  {
    "chainId": 1,
    "address": "0xbEe9Cf658702527b0AcB2719c1FAA29EdC006a92",
    "symbol": "sETH",
    "name": "Strike ETH",
    "decimals": 8,
    "logoURI": "ipfs://Qmd3gcaYK2aHjV6wF3sbkTUcKT6uZ9AKcWVJ5mBX3BTvdR"
  },
  {
    "chainId": 42161,
    "address": "0xB688BA096b7Bb75d7841e47163Cd12D18B36A5bF",
    "symbol": "mPendle",
    "name": "mPendle",
    "decimals": 18,
    "logoURI": "ipfs://Qmdi1PC1TGnGg7uEVtM9tcrsb7DbcHDasEyrRm4jeaYJoL"
  },
  {
    "chainId": 1,
    "address": "0x3472A5A71965499acd81997a54BBA8D852C6E53d",
    "symbol": "BADGER",
    "name": "Badger DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmXJqHR9Wk5kxvd5VQ68WSwpiDd6BW1DmTgTJwYwreohwH"
  },
  {
    "chainId": 1,
    "address": "0x1a7e4e63778B4f12a199C062f3eFdD288afCBce8",
    "symbol": "agEUR",
    "name": "agEUR",
    "decimals": 18,
    "logoURI": "ipfs://QmZ5Z1xA6rkZPvGh8V8Ezv3vZp8gkL89QpgUzxYYaZUTjM"
  },
  {
    "chainId": 1,
    "address": "0x7537AAe01f3B218DAE75e10d952473823F961B87",
    "symbol": "sAAPL",
    "name": "Synthetic Apple",
    "decimals": 18,
    "logoURI": "ipfs://QmYgKpxFXEeubVaXkEgq9oRcuH55zy8AjNHqwN49Dj5iRz"
  },
  {
    "chainId": 42161,
    "address": "0x9DCE8e754913d928eB39Bc4fC3CF047e364f7f2C",
    "symbol": "BLOK",
    "name": "Bloktopia",
    "decimals": 18,
    "logoURI": "ipfs://QmZ9djJyGc1mg5ea5hMhatSNY9fQF7XWaQ5nanKZzCiJqn"
  },
  {
    "chainId": 1,
    "address": "0x954b890704693af242613edEf1B603825afcD708",
    "symbol": "CARD",
    "name": "Cardstack",
    "decimals": 18,
    "logoURI": "ipfs://QmXgun2BKcGKSoEoanqpsu4fzkqzGdSwBehhqwkFgRfT3C"
  },
  {
    "chainId": 100,
    "address": "0xaf204776c7245bF4147c2612BF6e5972Ee483701",
    "symbol": "sDAI",
    "name": "Savings xDAI",
    "decimals": 18,
    "logoURI": "ipfs://QmPFv2Xd74TZ2ncEkGw4iHP3Qz7cZSrLRV5fsyedH3ds8p"
  },
  {
    "chainId": 1,
    "address": "0x45804880De22913dAFE09f4980848ECE6EcbAf78",
    "symbol": "PAXG",
    "name": "PAX Gold",
    "decimals": 18,
    "logoURI": "ipfs://QmYjZhjXNSuj5LgFTnXyLAmTPapMs9e6qaxrcDU1GkSnJc"
  },
  {
    "chainId": 100,
    "address": "0xc5102fE9359FD9a28f877a67E36B0F050d81a3CC",
    "symbol": "HOP",
    "name": "Hop",
    "decimals": 18,
    "logoURI": "ipfs://QmcFNTzsHbeUeqiCmYdnZUsiWbNFDDx1qtF7n4MGn9YyJ1"
  },
  {
    "chainId": 1,
    "address": "0xA4f2fDB0a5842d62bbAa5B903f09687B85E4bf59",
    "symbol": "TBOT",
    "name": "TBOT Project",
    "decimals": 18,
    "logoURI": "ipfs://QmPfXLBKSmRntrudaX4TJYSVLvn7r4z57GM7nDNmxvz2D7"
  },
  {
    "chainId": 1,
    "address": "0x8A0cdfaB62eD35b836DC0633482798421C81b3Ec",
    "symbol": "SPHRI",
    "name": "Spherium",
    "decimals": 18,
    "logoURI": "ipfs://QmamE5JAezAkSdaLnKyJY7fWM4kXvmspgUwJoMQUKPjxhZ"
  },
  {
    "chainId": 1,
    "address": "0xAA19961b6B858D9F18a115f25aa1D98ABc1fdBA8",
    "symbol": "LCS",
    "name": "LocalCoinSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmPVr5MkP2vzhiTWfDtfgyxqkmLS56mobZgde8GqS3KYYu"
  },
  {
    "chainId": 1,
    "address": "0x5B7533812759B45C2B44C19e320ba2cD2681b542",
    "symbol": "AGIX",
    "name": "SingularityNET",
    "decimals": 8,
    "logoURI": "ipfs://QmUgnbM8fX7F6Ee2NgX5fiMrrytAT6pnVHmwyT3TxuEM1c"
  },
  {
    "chainId": 1,
    "address": "0xBbd900e05b4aF2124390D206F70bc4E583B1bE85",
    "symbol": "MINTYS",
    "name": "MintySwap",
    "decimals": 9,
    "logoURI": "ipfs://QmXfNJfsdFxZiTVkFyNS8UAEgPKPNRzWkYf2o2dYiP7UbN"
  },
  {
    "chainId": 1,
    "address": "0xAEcc217a749c2405b5ebC9857a16d58Bdc1c367F",
    "symbol": "PAWTH",
    "name": "Pawthereum",
    "decimals": 9,
    "logoURI": "ipfs://QmZckyar1gDZSo9XrNopbFn8GHpy6TMkuUY4MJLuCrncbx"
  },
  {
    "chainId": 1,
    "address": "0x1A434A4c16223fb48F8f835e2E277bf72B4b5D1A",
    "symbol": "NETHER",
    "name": "Netherworld",
    "decimals": 18,
    "logoURI": "ipfs://QmYT8qRxKu99UN3mGyFQEPa3uSz5kDWYKAJ6ie8xGUi2fX"
  },
  {
    "chainId": 56,
    "address": "0x36FE11B6d5c9421F68d235694Fe192B35e803903",
    "symbol": "RWA",
    "name": "Xend Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmZmWGvhWWoPmDdHKTjozuA9F3vvkCfA28M3iscCyQqAzr"
  },
  {
    "chainId": 1,
    "address": "0xA0b73E1Ff0B80914AB6fe0444E65848C4C34450b",
    "symbol": "CRO",
    "name": "Cronos",
    "decimals": 8,
    "logoURI": "ipfs://QmVbHBpxBWPECJbYNgyf2CdCktWB6X8waxgdnM4sZ6L1mh"
  },
  {
    "chainId": 1,
    "address": "0x9AB7bb7FdC60f4357ECFef43986818A2A3569c62",
    "symbol": "GOG",
    "name": "Guild of Guardians",
    "decimals": 18,
    "logoURI": "ipfs://QmbkjnfZ3Hdsk3d3M9i3aThbqLhBTmte3evkrNmxd7Brwm"
  },
  {
    "chainId": 1,
    "address": "0x85eBa557C06c348395fD49e35d860F58a4F7c95a",
    "symbol": "H3X",
    "name": "H3X",
    "decimals": 18,
    "logoURI": "ipfs://Qmdvf71enBCKjGKaTwePqWfSvSV1q74nSzb734KcsC3htt"
  },
  {
    "chainId": 1,
    "address": "0xcD39b5434a0A92cf47D1F567a7dF84bE356814F0",
    "symbol": "s1INCH",
    "name": "Synthetic 1inch",
    "decimals": 18,
    "logoURI": "ipfs://QmdkzcGwXC9Si1rgHYzyxA9eoNmq3RqRoYcAeD8N4bvCH6"
  },
  {
    "chainId": 1,
    "address": "0x6b093998D36f2C7F0cc359441FBB24CC629D5FF0",
    "symbol": "iDAI",
    "name": "Fulcrum DAI iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmZykNkn2647s53wjXfCJvZpT2bTtuzR77JejNMjCYxtZq"
  },
  {
    "chainId": 1,
    "address": "0x967da4048cD07aB37855c090aAF366e4ce1b9F48",
    "symbol": "OCEAN",
    "name": "Ocean",
    "decimals": 18,
    "logoURI": "ipfs://QmSmkzABGveatVaH41thVWdD1cgVAZ9SJe7Cet7YzpYWq6"
  },
  {
    "chainId": 1,
    "address": "0x6704B673c70dE9bF74C8fBa4b4bd748F0e2190E1",
    "symbol": "UBEX",
    "name": "Ubex",
    "decimals": 18,
    "logoURI": "ipfs://QmRQT9WdZQbsfy3JqURvF5zNiZ27fx2KyYztTPa9aqEx1w"
  },
  {
    "chainId": 1,
    "address": "0x1494CA1F11D487c2bBe4543E90080AeBa4BA3C2b",
    "symbol": "DPI",
    "name": "DeFi Pulse Index",
    "decimals": 18,
    "logoURI": "ipfs://QmZacYc39o7wE1JtF2qysEdwjTjN2zAgLWNEK6GjNQX54c"
  },
  {
    "chainId": 1,
    "address": "0x4D807509aECe24C0fa5A102b6a3B059Ec6E14392",
    "symbol": "ONE",
    "name": "Menlo One",
    "decimals": 18,
    "logoURI": "ipfs://QmPFRqWvdHwyzVf2vYYPLaYEDWuwax94TktomUEyzWCFqL"
  },
  {
    "chainId": 250,
    "address": "0x5Cc61A78F164885776AA610fb0FE1257df78E59B",
    "symbol": "SPIRIT",
    "name": "SpiritSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmTfaHewiPHucKyGUxjDUYpw1j87EvZSVDWxKLmCcKLL6T"
  },
  {
    "chainId": 1,
    "address": "0x5E8422345238F34275888049021821E8E08CAa1f",
    "symbol": "frxETH",
    "name": "Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmVmEupFmP81xAgZ6k5gVwW41zsNZCLVooHA6UVUEpetqG"
  },
  {
    "chainId": 1,
    "address": "0xcbE771323587EA16dACB6016e269D7F08A7ACC4E",
    "symbol": "SPO",
    "name": "Spores Network",
    "decimals": 18,
    "logoURI": "ipfs://QmXdBRJQ6YwD3qebuG6kbuBpD9B7e55xc696RLwfrvFusH"
  },
  {
    "chainId": 1,
    "address": "0x0a661F6AD63a1500D714ED1EeEdb64ec493a54A8",
    "symbol": "USDZ",
    "name": "Zedxion",
    "decimals": 9,
    "logoURI": "ipfs://QmXiG7hbnbLZYhnKckouWg3iN2FzrEadkC5LZSS3p75Aaj"
  },
  {
    "chainId": 1,
    "address": "0xf819d9Cb1c2A819Fd991781A822dE3ca8607c3C9",
    "symbol": "UNIBOT",
    "name": "UNIBOT",
    "decimals": 18,
    "logoURI": "ipfs://QmTgDf7ippBNy8q4YsnnnthgsRsAvDBEj5Dccbe8yZVXaH"
  },
  {
    "chainId": 1,
    "address": "0xCb94be6f13A1182E4A4B6140cb7bf2025d28e41B",
    "symbol": "TRST",
    "name": "WeTrust",
    "decimals": 6,
    "logoURI": "ipfs://QmUjN9gPtDZtJnFJaPTqqbjgUsN8sg5eX4NDg7cKeEN6a7"
  },
  {
    "chainId": 100,
    "address": "0x4f4F9b8D5B4d0Dc10506e5551B0513B61fD59e75",
    "symbol": "GIV",
    "name": "Giveth",
    "decimals": 18,
    "logoURI": "ipfs://QmfBkYQZVHewLGqUnDDSGmybtiCKSk4bkAoQjxqwdRxZiQ"
  },
  {
    "chainId": 42161,
    "address": "0xaf88d065e77c8cC2239327C5EDb3A432268e5831",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmWbGfqePP3uAvPNHdYSP69aDznHGknsypTPRaiJT9NZF8"
  },
  {
    "chainId": 1,
    "address": "0x00d8318E44780EdEEFcF3020A5448F636788883C",
    "symbol": "DAPPX",
    "name": "dAppstore",
    "decimals": 18,
    "logoURI": "ipfs://QmZiEY86MDDp9aY4bCa7VKQtoMoxJyWDmFPysoBU2EJRWe"
  },
  {
    "chainId": 1,
    "address": "0x12B19D3e2ccc14Da04FAe33e63652ce469b3F2FD",
    "symbol": "GRID",
    "name": "Grid+",
    "decimals": 12,
    "logoURI": "ipfs://QmXcGLt9ExAvgv5FJdrLiyyDM5Z4GLkxA7wCiZZZ7qV497"
  },
  {
    "chainId": 1,
    "address": "0xd7f0cC50AD69408aE58be033F4f85D2367C2e468",
    "symbol": "VERA",
    "name": "Vera",
    "decimals": 18,
    "logoURI": "ipfs://QmQ8RT7vmrnjqFK7e6UGVuR3AkYXDQJPnBC6EC6vAJLVfo"
  },
  {
    "chainId": 1,
    "address": "0x668DbF100635f593A3847c0bDaF21f0a09380188",
    "symbol": "BNSD",
    "name": "BNSD Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmTskfjpDak1f3qGxDFpd75DXinqyEj8tDfTzsFFANsVwB"
  },
  {
    "chainId": 1,
    "address": "0xaDB2437e6F65682B85F814fBc12FeC0508A7B1D0",
    "symbol": "UNCX",
    "name": "UniCrypt",
    "decimals": 18,
    "logoURI": "ipfs://QmVrrAQRHpLYXridCZ4EvDxDm1DUCq8EAWWSyzRPUE4Pn5"
  },
  {
    "chainId": 1,
    "address": "0x48bE867B240D2fFafF69e0746130F2c027d8d3d2",
    "symbol": "ELE",
    "name": "Elevate",
    "decimals": 9,
    "logoURI": "ipfs://QmZN2Armi5YYLMt1LVwwx1d4hzx8eGYw5KHFcZSDQ1CU4u"
  },
  {
    "chainId": 1,
    "address": "0x820802Fa8a99901F52e39acD21177b0BE6EE2974",
    "symbol": "EUROe",
    "name": "EUROe Stablecoin",
    "decimals": 6,
    "logoURI": "ipfs://QmU63WjFnFHgxJL4oRTamgEGqwXim1DjSV3DqA5xhR4Zqt"
  },
  {
    "chainId": 1,
    "address": "0x261b45D85cCFeAbb11F022eBa346ee8D1cd488c0",
    "symbol": "rDAI",
    "name": "rDAI",
    "decimals": 18,
    "logoURI": "ipfs://QmQqeSUv3G43hwbyhmrTQTFuAt52tmjYiGKU1EGemFKxLM"
  },
  {
    "chainId": 1,
    "address": "0xf3Db5Fa2C66B7aF3Eb0C0b782510816cbe4813b8",
    "symbol": "EVX",
    "name": "Everex",
    "decimals": 4,
    "logoURI": "ipfs://QmauBk4YSCcCL1UvCD8wr28LrmDtytFnjav1zpBGTEFJea"
  },
  {
    "chainId": 100,
    "address": "0x3a97704a1b25F08aa230ae53B352e2e72ef52843",
    "symbol": "AGVE",
    "name": "Agave",
    "decimals": 18,
    "logoURI": "ipfs://QmRVeKdtSCfV95nGMLsBpNDzT1pmFggvCSiWD2eJu7fqah"
  },
  {
    "chainId": 56,
    "address": "0x2b1D36f5B61AdDAf7DA7ebbd11B35FD8cfb0DE31",
    "symbol": "ITP",
    "name": "Interport",
    "decimals": 18,
    "logoURI": "ipfs://QmTWPTMwXJWuwDgNmUxpdkZA7UASQfrye7t7MESNBuUJox"
  },
  {
    "chainId": 1,
    "address": "0x8806926Ab68EB5a7b909DcAf6FdBe5d93271D6e2",
    "symbol": "UQC",
    "name": "Uquid",
    "decimals": 18,
    "logoURI": "ipfs://QmUsqe5FHFW5Lrt11NX2DmiC9nSeqzhDyYbgNpX8a4gtUJ"
  },
  {
    "chainId": 1,
    "address": "0x7420B4b9a0110cdC71fB720908340C03F9Bc03EC",
    "symbol": "JASMY",
    "name": "JasmyCoin",
    "decimals": 18,
    "logoURI": "ipfs://QmP5YjEjtj2QQXCNpYjEbd9L59hJtHQF2orh47kjcHqGHH"
  },
  {
    "chainId": 1,
    "address": "0x224DB5E6180761df4C3d8936585f6b8b83879770",
    "symbol": "OML",
    "name": "Omlira",
    "decimals": 18,
    "logoURI": "ipfs://QmeCfRRtyg5FtJi825jscEWCXFuiKaYXjJ9ZVdAYQ2ZiJo"
  },
  {
    "chainId": 1,
    "address": "0xc3771d47E2Ab5A519E2917E61e23078d0C05Ed7f",
    "symbol": "GTH",
    "name": "Gather",
    "decimals": 18,
    "logoURI": "ipfs://QmWcWP4va5EJyY1JxgyxjAFcoUWoj1sEpLNZbPkreJ7yfP"
  },
  {
    "chainId": 1,
    "address": "0x09e64c2B61a5f1690Ee6fbeD9baf5D6990F8dFd0",
    "symbol": "GRO",
    "name": "GROWTH DeFi",
    "decimals": 18,
    "logoURI": "ipfs://QmbzNJZUTfdQ9XumJHytWEt9DbN6pgrzbgw7vDnMD8HjZr"
  },
  {
    "chainId": 56,
    "address": "0x2dfF88A56767223A5529eA5960Da7A3F5f766406",
    "symbol": "ID",
    "name": "SPACE ID",
    "decimals": 18,
    "logoURI": "ipfs://QmaMA9qpmBogwzhZ8bor1R43kMksTT4kS8yMExqWs2aS3C"
  },
  {
    "chainId": 10,
    "address": "0x9e1028F5F1D5eDE59748FFceE5532509976840E0",
    "symbol": "PERP",
    "name": "Perpetual Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmVCa9zawdxxxXUUNZRJfVJBdQmzecrsxgUaGysjxVkJK9"
  },
  {
    "chainId": 1,
    "address": "0xB1f1F47061A7Be15C69f378CB3f69423bD58F2F8",
    "symbol": "FLASH",
    "name": "Flashstake",
    "decimals": 18,
    "logoURI": "ipfs://QmeaqsbSEjRSdEGsRewDNrWBbQirWoBzdEfMEjNsVAge8v"
  },
  {
    "chainId": 1,
    "address": "0xF938424F7210f31dF2Aee3011291b658f872e91e",
    "symbol": "VISR",
    "name": "Visor",
    "decimals": 18,
    "logoURI": "ipfs://QmV7S8htQ4WihNzs8tyFd1v3dEbejbJc3rhdAh78aCLDH8"
  },
  {
    "chainId": 1,
    "address": "0x64bbf62a855B24F45b3C7Fd3Aa8D34576E2e78Fe",
    "symbol": "CNX",
    "name": "Cleanex",
    "decimals": 9,
    "logoURI": "ipfs://QmRUcYnuBJ7pVtRwRgeFBatbLpqvpa8nJwxM9LouA1TW6i"
  },
  {
    "chainId": 1,
    "address": "0xc27A2F05fa577a83BA0fDb4c38443c0718356501",
    "symbol": "TAU",
    "name": "Lamden",
    "decimals": 18,
    "logoURI": "ipfs://QmQfu4zNZEJEc2SJ4NwCLepbHPwnJGgQmEsW4rnp1kUaYK"
  },
  {
    "chainId": 56,
    "address": "0x52CE071Bd9b1C4B00A0b92D298c512478CaD67e8",
    "symbol": "COMP",
    "name": "Compound",
    "decimals": 18,
    "logoURI": "ipfs://QmWZSaKRTLqDfF9BDG5HYrgc5fbr9uq3ksHct8c5MfjYUu"
  },
  {
    "chainId": 1,
    "address": "0x89303500a7Abfb178B274FD89F2469C264951e1f",
    "symbol": "REF",
    "name": "RefToken",
    "decimals": 8,
    "logoURI": "ipfs://QmTtPnJ2mFeAUpxtz3LGZqwGqbYzN6cDHg9Evfo7vir9VN"
  },
  {
    "chainId": 1,
    "address": "0x628eBC64A38269E031AFBDd3C5BA857483B5d048",
    "symbol": "LSETH",
    "name": "Liquid Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://Qmf4zwuZxks5fR9HLgNgM3GTF5iSsjUMBPZyc73K6ASApU"
  },
  {
    "chainId": 1,
    "address": "0x89d24A6b4CcB1B6fAA2625fE562bDD9a23260359",
    "symbol": "SAI",
    "name": "Sai",
    "decimals": 18,
    "logoURI": "ipfs://Qmbr5XtonKYPHPeJDBcguVGxrB5yr6wP69zKMtoKKCABDW"
  },
  {
    "chainId": 42161,
    "address": "0x82aF49447D8a07e3bd95BD0d56f35241523fBab1",
    "symbol": "WETH",
    "name": "Wrapped Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmPt2DEXhRSvbbFQep44XXtcKxRscshsijJ47Y1xUQwBZ1"
  },
  {
    "chainId": 1,
    "address": "0x8D75959f1E61EC2571aa72798237101F084DE63a",
    "symbol": "SUB",
    "name": "Substratum",
    "decimals": 18,
    "logoURI": "ipfs://QmR2YVnXnZKMXdAj51DmCfcRJyLkRJFPLgvyD46q4rC8QN"
  },
  {
    "chainId": 1,
    "address": "0x0954906da0Bf32d5479e25f46056d22f08464cab",
    "symbol": "INDEX",
    "name": "Index Cooperative",
    "decimals": 18,
    "logoURI": "ipfs://QmbybqnXCebXujqcMdyfXsiufkbkHJaPpxyDnQKHUhfFQa"
  },
  {
    "chainId": 100,
    "address": "0xdBF3Ea6F5beE45c02255B2c26a16F300502F68da",
    "symbol": "BZZ",
    "name": "Swarm",
    "decimals": 16,
    "logoURI": "ipfs://QmSiXeTDA3Q7Ht2RGKa2q4iRGBJUstrSfeRVzKBXvsigjw"
  },
  {
    "chainId": 1,
    "address": "0x797AAB1ce7c01eB727ab980762bA88e7133d2157",
    "symbol": "crUSDT",
    "name": "Cream USDT",
    "decimals": 8,
    "logoURI": "ipfs://Qmf8sPNYZpgW8sbREjt7e8H6NUfMiMrTYYux8QXdoZQBjw"
  },
  {
    "chainId": 1,
    "address": "0x2ccbFF3A042c68716Ed2a2Cb0c544A9f1d1935E1",
    "symbol": "DMT",
    "name": "DMarket",
    "decimals": 8,
    "logoURI": "ipfs://QmbsXfpttmxoLiKmV8mwHmdotDYG6izASxNzn7X3Tvopho"
  },
  {
    "chainId": 1,
    "address": "0x4AdF728E2Df4945082cDD6053869f51278fae196",
    "symbol": "iXMR",
    "name": "Synthetic Inverse Monero",
    "decimals": 18,
    "logoURI": "ipfs://QmTcWTK966oaRYXuQ76h51Nk9k5RA27t6RX7Hp1DmtAkLS"
  },
  {
    "chainId": 42161,
    "address": "0x2Ac2B254Bc18cD4999f64773a966E4f4869c34Ee",
    "symbol": "PNP",
    "name": "Penpie",
    "decimals": 18,
    "logoURI": "ipfs://QmSD3cGiAXwRujKBgGt5ZHTK9d1Dqj34GRB7jU6gMbwD2Y"
  },
  {
    "chainId": 1,
    "address": "0x6d16cF3EC5F763d4d99cB0B0b110eefD93B11B56",
    "symbol": "sOIL",
    "name": "Synthetic Perpetual Oil Futures",
    "decimals": 18,
    "logoURI": "ipfs://QmYoKXuSPnfLq8F6iVVHiHtNRaWzVUSeK6au7RnHhDiDD7"
  },
  {
    "chainId": 1,
    "address": "0x74ccbe53F77b08632ce0CB91D3A545bF6B8E0979",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 1,
    "address": "0x469eDA64aEd3A3Ad6f868c44564291aA415cB1d9",
    "symbol": "FLUX",
    "name": "FLUX",
    "decimals": 18,
    "logoURI": "ipfs://QmSjg1U1oWw3BfcCbbz9mzHwU9mPLWqzNo5ZdqLceJUGAH"
  },
  {
    "chainId": 1,
    "address": "0x8e1b448EC7aDFc7Fa35FC2e885678bD323176E34",
    "symbol": "EGT",
    "name": "Egretia",
    "decimals": 18,
    "logoURI": "ipfs://QmPFn1pmvDnYcxVwd5qe2x8MYUeD5u8yRG6qfg4zuQCoaX"
  },
  {
    "chainId": 1,
    "address": "0x220B71671b649c03714dA9c621285943f3cbcDC6",
    "symbol": "DIS",
    "name": "TosDis",
    "decimals": 18,
    "logoURI": "ipfs://Qmb5PtSs8fN37XT55oD5qfR6KRjQqXfkCZYUk1Xk8YT34n"
  },
  {
    "chainId": 1,
    "address": "0x1B073382E63411E3BcfFE90aC1B9A43feFa1Ec6F",
    "symbol": "BEST",
    "name": "Bitpanda Ecosystem Token",
    "decimals": 8,
    "logoURI": "ipfs://QmbZbZibWYcmhRhjDuvi24AzZfePitJMfV3oXyhCuFX9bk"
  },
  {
    "chainId": 56,
    "address": "0x5012c90F14d190607662CA8344120812Aaa2639D",
    "symbol": "PNP",
    "name": "Penpie",
    "decimals": 18,
    "logoURI": "ipfs://QmSD3cGiAXwRujKBgGt5ZHTK9d1Dqj34GRB7jU6gMbwD2Y"
  },
  {
    "chainId": 1,
    "address": "0xec67005c4E498Ec7f55E092bd1d35cbC47C91892",
    "symbol": "MLN",
    "name": "Enzyme",
    "decimals": 18,
    "logoURI": "ipfs://Qme7bxm7Wxnbx7ebz2KTPdaJ6KP48sZAYrmY7x1qjVurVr"
  },
  {
    "chainId": 10,
    "address": "0xc5102fE9359FD9a28f877a67E36B0F050d81a3CC",
    "symbol": "HOP",
    "name": "Hop Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmNPiY49XgSCf4eqmgSzTEiJpZrGJwwfvDzJQQBBxXybkM"
  },
  {
    "chainId": 1,
    "address": "0xA3865E64121537b5b59B5e239Db4aCBe6F36aa74",
    "symbol": "WXTZ",
    "name": "Wrapped Tezos",
    "decimals": 18,
    "logoURI": "ipfs://QmS1NpydbAb698dQWFQCBU4HNczWxnAfbK7aFqu5iKmZLn"
  },
  {
    "chainId": 1,
    "address": "0x6c5bA91642F10282b576d91922Ae6448C9d52f4E",
    "symbol": "PHA",
    "name": "Phala",
    "decimals": 18,
    "logoURI": "ipfs://QmcAZ5Um5oABpK4n5jr5egcG1TQdwDUz2DCyA2WkMxcbz6"
  },
  {
    "chainId": 56,
    "address": "0x47b19Af93d0bC33805269Af02B5CA953Aa145127",
    "symbol": "jMXN",
    "name": "Jarvis Synthetic Mexican Peso",
    "decimals": 18,
    "logoURI": "ipfs://QmTaASZKajcjmmQQeVkoYCje8se2FQYxGfKCuwu4xNxsmp"
  },
  {
    "chainId": 1,
    "address": "0xD520ddbdf81323F4B9a0Ea0AdBb692eBEf6cE280",
    "symbol": "HLD",
    "name": "Holdefi",
    "decimals": 18,
    "logoURI": "ipfs://QmQxWXih15rrAQD2swm1KfCuKdHCeYEiCdzf4awYf4XpfV"
  },
  {
    "chainId": 1,
    "address": "0xC382e04099A435439725BB40647e2B32dC136806",
    "symbol": "COGE",
    "name": "Cogecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmeDjhZSJ5x8uB7jwf9JZZejcvDpq6XKUaxsd4VpqgvYCy"
  },
  {
    "chainId": 250,
    "address": "0xD3b71117E6C1558c1553305b44988cd944e97300",
    "symbol": "YEL",
    "name": "YEL.Finance",
    "decimals": 18,
    "logoURI": "ipfs://Qmc9AXNryPwhTk5x9V3A6GSEzvJPiEjJdiomTwTXckhuVX"
  },
  {
    "chainId": 1,
    "address": "0xa10ae543dB5D967a73E9Abcc69c81a18A7Fc0A78",
    "symbol": "CLOUT",
    "name": "BLOCKCLOUT",
    "decimals": 18,
    "logoURI": "ipfs://QmQxgfFU19npztEaARu2gkLyq6Cv2HgPCZoZN48Sarts5r"
  },
  {
    "chainId": 1,
    "address": "0x0F02e27745e3b6e9e1310d19469e2b5D7B5eC99A",
    "symbol": "PCL",
    "name": "Peculium",
    "decimals": 8,
    "logoURI": "ipfs://QmbgBqSffmmJz7z9Ry3aqaLaZjL2LejREVXSLv1PQE9132"
  },
  {
    "chainId": 1,
    "address": "0x0f8c45B896784A1E408526B9300519ef8660209c",
    "symbol": "XMX",
    "name": "XMAX",
    "decimals": 8,
    "logoURI": "ipfs://Qmczph23JW9hshF9oHajLvqkrZaTnVC83kFjJM1VbdhQa4"
  },
  {
    "chainId": 1,
    "address": "0x10a3da2BB0Fae4D591476fd97D6636fd172923a8",
    "symbol": "crHEGIC",
    "name": "Cream Hegic",
    "decimals": 8,
    "logoURI": "ipfs://QmWJe7QQXDt4xJFN1j2ZiDVqxeuWnYB17UveLxq2CXifTH"
  },
  {
    "chainId": 1,
    "address": "0x1Ed460D149D48FA7d91703bf4890F97220C09437",
    "symbol": "tfBUSD",
    "name": "TrueFi Binance USD",
    "decimals": 18,
    "logoURI": "ipfs://QmThfRPR6AYrkDFNZ5rkudRmD3Xcu3WNhbQ5za3g3ueJVX"
  },
  {
    "chainId": 1,
    "address": "0x84cA8bc7997272c7CfB4D0Cd3D55cd942B3c9419",
    "symbol": "DIA",
    "name": "DIA",
    "decimals": 18,
    "logoURI": "ipfs://QmcJnRjr1t8AGjWrdzn6T69Ez4NrTSYT9KAD5GUTZ8kRVA"
  },
  {
    "chainId": 1,
    "address": "0x95aD61b0a150d79219dCF64E1E6Cc01f0B64C4cE",
    "symbol": "SHIB",
    "name": "Shiba Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmYc1GQpy4hTTEqpCtiBVpCKHthv4ZMpmUp6j9YZcEw3LH"
  },
  {
    "chainId": 56,
    "address": "0x6e4a971B81CA58045a2AA982EaA3d50C4Ac38F42",
    "symbol": "BRG",
    "name": "Bridge Oracle",
    "decimals": 18,
    "logoURI": "ipfs://QmQ55zH5SFAuzvxGHQjXW1tJjnNVvX27YVpzuwZd88ikA2"
  },
  {
    "chainId": 1,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmY6nVvrggMDphHtKA1FLuNNoF79W1SXYWY97Es2wr6jiR"
  },
  {
    "chainId": 1,
    "address": "0x21BfBDa47A0B4B5b1248c767Ee49F7caA9B23697",
    "symbol": "OVR",
    "name": "OVR",
    "decimals": 18,
    "logoURI": "ipfs://QmP4P8qym3QkNPqwzpNVEkjtoP1odz5PkkaBhR4Rrt9ArA"
  },
  {
    "chainId": 1,
    "address": "0x3506424F91fD33084466F402d5D97f05F8e3b4AF",
    "symbol": "CHZ",
    "name": "Chiliz",
    "decimals": 18,
    "logoURI": "ipfs://QmZBNTTkcAZ5GMoS4U9mHc8VE5P3o9LGcM5YSvVTUhQwvS"
  },
  {
    "chainId": 1,
    "address": "0xb439B8731ee047799019eF0b745a51d256B116Af",
    "symbol": "BULL",
    "name": "BullionFX",
    "decimals": 18,
    "logoURI": "ipfs://QmPFZNhmvvbwAegHENxqZYNfqG1mYWgG9MVzcooTEb6xSC"
  },
  {
    "chainId": 1,
    "address": "0xa0bEd124a09ac2Bd941b10349d8d224fe3c955eb",
    "symbol": "DEPAY",
    "name": "DePay",
    "decimals": 18,
    "logoURI": "ipfs://QmZKQdLJRBp9uqB8QizcdRtNxiTHFCb8RH4ieJYSfDn4j2"
  },
  {
    "chainId": 1,
    "address": "0xE5CAeF4Af8780E59Df925470b050Fb23C43CA68C",
    "symbol": "FRM",
    "name": "Ferrum Network",
    "decimals": 6,
    "logoURI": "ipfs://QmYBq6GWG5hFMn3xsNnyjY2kKJTwPsmVAAgBdhzwW34cNu"
  },
  {
    "chainId": 100,
    "address": "0xeEeEEb57642040bE42185f49C52F7E9B38f8eeeE",
    "symbol": "ELK",
    "name": "Elk Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYSmFMg8incix4Hzs85MPZzQJnb2W19zLTL3nXNEzY42m"
  },
  {
    "chainId": 1,
    "address": "0x9275e8386A5BDdA160c0e621e9A6067b8fd88ea2",
    "symbol": "NBNG",
    "name": "Nobunaga",
    "decimals": 18,
    "logoURI": "ipfs://QmXbeCY47VLATQNWQGBsRmB3DfXrVeQ2u1hoD5VLJ2Xpsm"
  },
  {
    "chainId": 1,
    "address": "0x7F3EDcdD180Dbe4819Bd98FeE8929b5cEdB3AdEB",
    "symbol": "XTK",
    "name": "xToken",
    "decimals": 18,
    "logoURI": "ipfs://QmdA3wu4jiYE7tCbNWMucyJrxuYmQapfxL3mAvMHg57ciZ"
  },
  {
    "chainId": 1,
    "address": "0x8C6bf16C273636523C29Db7DB04396143770F6A0",
    "symbol": "AAA",
    "name": "Moon Rabbit",
    "decimals": 18,
    "logoURI": "ipfs://QmZwVVXy9PeUeyirvnj4EaqaV8oeeG8sdrRKddMr1xvwki"
  },
  {
    "chainId": 1,
    "address": "0x00C2999c8B2AdF4ABC835cc63209533973718eB1",
    "symbol": "STATE",
    "name": "New World Order",
    "decimals": 18,
    "logoURI": "ipfs://QmY2dQ5ExX89VduFjvowWMdodfHKKfnL6LMFwQ8iYsd984"
  },
  {
    "chainId": 100,
    "address": "0x2d5563da42b06FbBF9c67b7DC073cF6A7842239e",
    "symbol": "jCHF",
    "name": "Jarvis Synthetic Swiss Franc",
    "decimals": 18,
    "logoURI": "ipfs://QmQYgf7dnsrcJPGjDgm1sQZ1n3Ugbk3cAvptuu7SV43KgQ"
  },
  {
    "chainId": 1,
    "address": "0x6D0F5149c502faf215C89ab306ec3E50b15e2892",
    "symbol": "PRT",
    "name": "Portion",
    "decimals": 18,
    "logoURI": "ipfs://Qmbm6fRYvQ8WpHeeWXv5sXPv7YqvP3SGcaAXeYbiex1C9F"
  },
  {
    "chainId": 10,
    "address": "0x0b2C639c533813f4Aa9D7837CAf62653d097Ff85",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmWbGfqePP3uAvPNHdYSP69aDznHGknsypTPRaiJT9NZF8"
  },
  {
    "chainId": 1,
    "address": "0x5963FD7cA9b17b85768476019F81CB43d9d1818E",
    "symbol": "YESTRUMP",
    "name": "Dai If Trump Wins The 2020 Election",
    "decimals": 18,
    "logoURI": "ipfs://QmQK7hBYsHLKkx1mR3pwzT5ueupZqcZaj541ZPmPEzqkRM"
  },
  {
    "chainId": 250,
    "address": "0x2F6F07CDcf3588944Bf4C42aC74ff24bF56e7590",
    "symbol": "STG",
    "name": "Stargate Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYCe3FShq8cRmJ9njM8eZhoNqkuohMm5hUjd4khBcEB8U"
  },
  {
    "chainId": 1,
    "address": "0xc5f0f7b66764F6ec8C8Dff7BA683102295E16409",
    "symbol": "FDUSD",
    "name": "First Digital USD",
    "decimals": 18,
    "logoURI": "ipfs://QmQJ2sw6ntJRrv3aGTKLA4MUYBt1KvxA6VqTgi9jshrs35"
  },
  {
    "chainId": 1,
    "address": "0xe3818504c1B32bF1557b16C238B2E01Fd3149C17",
    "symbol": "PLR",
    "name": "Pillar",
    "decimals": 18,
    "logoURI": "ipfs://QmW3v35P4TPQ1quWzpAVZo5pDwp5sG8M1pnCmGUQYd3D7k"
  },
  {
    "chainId": 1,
    "address": "0x4C19596f5aAfF459fA38B0f7eD92F11AE6543784",
    "symbol": "TRU",
    "name": "TrueFi",
    "decimals": 8,
    "logoURI": "ipfs://QmZS2DtdFXBfK4fmC5pgnVYiwgTPbxSYP2BHMrgKLx1BLT"
  },
  {
    "chainId": 1,
    "address": "0x0E22734e078d6e399BCeE40a549DB591C4EA46cB",
    "symbol": "STM",
    "name": "Streamity",
    "decimals": 18,
    "logoURI": "ipfs://QmPxi4vVAdniedFzskF9QKEPvByhqypu6kNTUrL43H2inq"
  },
  {
    "chainId": 1,
    "address": "0xA0b86991c6218b36c1d19D4a2e9Eb0cE3606eB48",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmSFwXFjMFh6E7PtXxwCwh2sSRs9sMdQLCkGM4bFXQshHY"
  },
  {
    "chainId": 1,
    "address": "0x68a3637bA6E75c0f66B61A42639c4e9fCD3D4824",
    "symbol": "MOON",
    "name": "MoonSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmV8vsoZUBqiQhaBRvah4fAGLzs12noYM5GBeaJkT2hDHN"
  },
  {
    "chainId": 1,
    "address": "0x0F5D2fB29fb7d3CFeE444a200298f468908cC942",
    "symbol": "MANA",
    "name": "Decentraland",
    "decimals": 18,
    "logoURI": "ipfs://QmQfMJvLMeo8kpCoKz37q7FoGC7U6nK7h9u6e9BZik3VEc"
  },
  {
    "chainId": 56,
    "address": "0xaEC945e04baF28b135Fa7c640f624f8D90F1C3a6",
    "symbol": "C98",
    "name": "Coin98",
    "decimals": 18,
    "logoURI": "ipfs://QmPtmkoaLbX3bdwcfFmuonRzA7xanbVt2xJ5RMfw1BtXxP"
  },
  {
    "chainId": 1,
    "address": "0x8E5610ab5E39d26828167640EA29823fe1dD5843",
    "symbol": "KNDC",
    "name": "KanadeCoin",
    "decimals": 8,
    "logoURI": "ipfs://Qmac6FqBrCf95FEAzqxVM16qSyUMJQSFhrRRunoJ7sHUes"
  },
  {
    "chainId": 1,
    "address": "0xFFa93Aacf49297D51E211817452839052FDFB961",
    "symbol": "DCC",
    "name": "Distributed Credit Chain",
    "decimals": 18,
    "logoURI": "ipfs://QmYxhcXuXH7fYc9ciCw5k5zgWV6yVUcQ6QognzNUakHtd6"
  },
  {
    "chainId": 1,
    "address": "0xF4FaEa455575354d2699BC209B0a65CA99F69982",
    "symbol": "NOBS",
    "name": "No BS Crypto",
    "decimals": 18,
    "logoURI": "ipfs://QmSiP4FEoMmzqmDvTcRwterScLvMKppSZ1F6UhAXLZLxne"
  },
  {
    "chainId": 1,
    "address": "0xCC12AbE4ff81c9378D670De1b57F8e0Dd228D77a",
    "symbol": "aREN",
    "name": "Aave REN",
    "decimals": 18,
    "logoURI": "ipfs://QmYKzY4MmEv96vCR42wfjA5cNC1vvQCLhq3JVHtrkk97t9"
  },
  {
    "chainId": 56,
    "address": "0x2aa69E8D25C045B659787BC1f03ce47a388DB6E8",
    "symbol": "KSM",
    "name": "Kusama",
    "decimals": 18,
    "logoURI": "ipfs://Qmf4Kphipscddo9mYtcqEFwurDxK695GMZho7d1p6LkiWh"
  },
  {
    "chainId": 1,
    "address": "0xE95A203B1a91a908F9B9CE46459d101078c2c3cb",
    "symbol": "ankrETH",
    "name": " Ankr Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmWpizC1p46quYHixpjZRjZFFrHvz8byAuZyzKtBdRxZUP"
  },
  {
    "chainId": 1,
    "address": "0x221657776846890989a759BA2973e427DfF5C9bB",
    "symbol": "REPv2",
    "name": "Augur",
    "decimals": 18,
    "logoURI": "ipfs://QmVg4teABAr1L1kUL7UxnSpvzF7YK1zCKW1kzJUPTB4cGj"
  },
  {
    "chainId": 1,
    "address": "0x7D1AfA7B718fb893dB30A3aBc0Cfc608AaCfeBB0",
    "symbol": "MATIC",
    "name": "Polygon",
    "decimals": 18,
    "logoURI": "ipfs://Qmc2zM8LQ2rsdJEk5ypHjGbVkreo3Q3KNpNTy5z73VTtuK"
  },
  {
    "chainId": 1,
    "address": "0xcFEB09C3c5F0f78aD72166D55f9e6E9A60e96eEC",
    "symbol": "VEMP",
    "name": "vEmpire DDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmP6FKs7HeEbATTmEXW6noZZkoqgJmUpF4XPx1U9S6Pqsz"
  },
  {
    "chainId": 10,
    "address": "0xC81D1F0EB955B0c020E5d5b264E1FF72c14d1401",
    "symbol": "RPL",
    "name": "Rocket Pool",
    "decimals": 18,
    "logoURI": "ipfs://QmNSj1BfWphbDaTStW6JvrmnzoEHpFJpPNNczNAheANF4f"
  },
  {
    "chainId": 1,
    "address": "0x49D09cDa1Deb8a1680F1270C5ed15218fc4B18f0",
    "symbol": "OVC",
    "name": "OVCODE",
    "decimals": 18,
    "logoURI": "ipfs://QmYABxSjCsBREPMz8MNWfs4V4kcLV2K6mFHG2zVEzvBMp1"
  },
  {
    "chainId": 42161,
    "address": "0x5575552988A3A80504bBaeB1311674fCFd40aD4B",
    "symbol": "SPA",
    "name": "Sperax",
    "decimals": 18,
    "logoURI": "ipfs://QmTZ5GpBAGfEZmSRVpeCRAEX8vQPHR6hnqXcWTmczqABdH"
  },
  {
    "chainId": 1,
    "address": "0x8A9C67fee641579dEbA04928c4BC45F66e26343A",
    "symbol": "JRT",
    "name": "Jarvis Reward Token",
    "decimals": 18,
    "logoURI": "ipfs://QmTZqMy43gyA5jbsfquBJfKUaChEK3PgLTgdYW6jyqQHnk"
  },
  {
    "chainId": 1,
    "address": "0xd4937682df3C8aEF4FE912A96A74121C0829E664",
    "symbol": "aFRAX",
    "name": "Aave FRAX",
    "decimals": 18,
    "logoURI": "ipfs://QmcakPXDgLifYDvd1Ue3GGeJmmDZsjjqNGC1fHCybZ4pGv"
  },
  {
    "chainId": 1,
    "address": "0x0AaCfbeC6a24756c20D41914F2caba817C0d8521",
    "symbol": "YAM",
    "name": "YAM",
    "decimals": 18,
    "logoURI": "ipfs://QmSzzEw8kYNGUQJzm2KFcxGhWM6oHAaA2Q3ztGeF38Ep8W"
  },
  {
    "chainId": 1,
    "address": "0xd3210F246Ae54C5a45A7b4A83315bf718F591bfc",
    "symbol": "ARKI",
    "name": "ArkiTech",
    "decimals": 9,
    "logoURI": "ipfs://QmcApbZ86hm8cfmeXPVP2xC8KJRTzGY9BnqJZxn1JFbPhv"
  },
  {
    "chainId": 1,
    "address": "0xa47c8bf37f92aBed4A126BDA807A7b7498661acD",
    "symbol": "UST",
    "name": "Wrapped UST",
    "decimals": 18,
    "logoURI": "ipfs://QmYUNLFNukHVS8MgA8fk4dyBCSWsE3pzENPzqXfKDCrSb6"
  },
  {
    "chainId": 137,
    "address": "0x61299774020dA444Af134c82fa83E3810b309991",
    "symbol": "RNDR",
    "name": "Render",
    "decimals": 18,
    "logoURI": "ipfs://Qmd4tMCvhQSdoBkxJbqBxVviGQfnMiuZWF1WA3XXRnarfa"
  },
  {
    "chainId": 1,
    "address": "0xAec7e1f531Bb09115103C53ba76829910Ec48966",
    "symbol": "BLANK",
    "name": "Blank",
    "decimals": 18,
    "logoURI": "ipfs://QmVdTaqYYG4f4CThEycYrSx5LAnH5m62oceqpw2SPdvR34"
  },
  {
    "chainId": 1,
    "address": "0x297D33e17e61C2Ddd812389C2105193f8348188a",
    "symbol": "$TRDL",
    "name": "Strudel Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYXo4MRF1SyeKQVzK1S3jCyAhxJRmUjC5rFHsdxRGFeZE"
  },
  {
    "chainId": 1,
    "address": "0xfA5047c9c78B8877af97BDcb85Db743fD7313d4a",
    "symbol": "ROOK",
    "name": "KeeperDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmUmFfKb9oeQE678FscZ5cQ5xDNmWEoFAaW6NPNrSDGTCG"
  },
  {
    "chainId": 1,
    "address": "0x430EF9263E76DAE63c84292C3409D61c598E9682",
    "symbol": "PYR",
    "name": "Vulcan Forged",
    "decimals": 18,
    "logoURI": "ipfs://QmaVmB72v5tqfhGCwGiNHrMtrM9DqJvSzs4HS92jJWhygH"
  },
  {
    "chainId": 1,
    "address": "0x40D16FC0246aD3160Ccc09B8D0D3A2cD28aE6C2f",
    "symbol": "GHO",
    "name": "GHO",
    "decimals": 18,
    "logoURI": "ipfs://QmZUaEQJkgxuxQFWMeTGSpuoSbJo1fSJgYXxQCrUYtdKRY"
  },
  {
    "chainId": 42161,
    "address": "0x9d2F299715D94d8A7E6F5eaa8E654E8c74a988A7",
    "symbol": "FXS",
    "name": "Frax Share",
    "decimals": 18,
    "logoURI": "ipfs://QmNRaYxGN6QHc6RKZfmaaKX6DJGtj6W7JeC4bDRroKGUCQ"
  },
  {
    "chainId": 1,
    "address": "0xD0660cD418a64a1d44E9214ad8e459324D8157f1",
    "symbol": "WOOFY",
    "name": "Woofy",
    "decimals": 12,
    "logoURI": "ipfs://QmZ3kxER9az7TsBsjKVfphU9s8urBJepmgdQ4qUgE72MFB"
  },
  {
    "chainId": 1,
    "address": "0x41545f8b9472D758bB669ed8EaEEEcD7a9C4Ec29",
    "symbol": "FORT",
    "name": "Forta",
    "decimals": 18,
    "logoURI": "ipfs://QmQGCw1mEn2WejiQck1V5GF4a9TVhwVH1s51okKRhufLhR"
  },
  {
    "chainId": 1,
    "address": "0xd26114cd6EE289AccF82350c8d8487fedB8A0C07",
    "symbol": "OMG",
    "name": "OMG Network",
    "decimals": 18,
    "logoURI": "ipfs://QmXGCW9GBZxLWU2kiqSy8oAisaJWhsQk7onmwMUbuwekfd"
  },
  {
    "chainId": 56,
    "address": "0xB248a295732e0225acd3337607cc01068e3b9c10",
    "symbol": "vXRP",
    "name": "Venus XRP",
    "decimals": 8,
    "logoURI": "ipfs://QmX5qTzbJm2KyMMxZ2U6xNFzYu9qJrV5dAhaAKtaqPUt3i"
  },
  {
    "chainId": 100,
    "address": "0xD3D47d5578e55C880505dC40648F7F9307C3e7A8",
    "symbol": "DPI",
    "name": "DeFi Pulse Index",
    "decimals": 18,
    "logoURI": "ipfs://QmZacYc39o7wE1JtF2qysEdwjTjN2zAgLWNEK6GjNQX54c"
  },
  {
    "chainId": 1,
    "address": "0xD8912C10681D8B21Fd3742244f44658dBA12264E",
    "symbol": "PLU",
    "name": "Pluton",
    "decimals": 18,
    "logoURI": "ipfs://QmUL1fzqVzyLH1jubyP6vw7S7YYga8Qp3jm1AjTPUTMVYD"
  },
  {
    "chainId": 56,
    "address": "0xd4ed60d8368a92b5F1ca33aF61eF2A94714B2d46",
    "symbol": "BAL",
    "name": "Balancer",
    "decimals": 18,
    "logoURI": "ipfs://QmbzDmJ2yboeFf45poTkzpqZkDbnsdL24x2BJQTs64yv5Z"
  },
  {
    "chainId": 1,
    "address": "0x6006FC2a849fEdABa8330ce36F5133DE01F96189",
    "symbol": "SHAKE",
    "name": "SHAKE",
    "decimals": 18,
    "logoURI": "ipfs://Qmf1EKhCNZb2z8kuXPTJ75UgixH9emKkxLJpx5oph8DKD5"
  },
  {
    "chainId": 1,
    "address": "0xdeFA4e8a7bcBA345F687a2f1456F5Edd9CE97202",
    "symbol": "KNC",
    "name": "Kyber Network",
    "decimals": 18,
    "logoURI": "ipfs://QmUV4eK6XWHuouhYAsxFqRm93JXCtkrCiwZur77nuWHq82"
  },
  {
    "chainId": 1,
    "address": "0x31d4Eb09a216e181eC8a43ce79226A487D6F0BA9",
    "symbol": "USDM",
    "name": "USDM",
    "decimals": 18,
    "logoURI": "ipfs://QmTZZhM31zF8kDgCExed6Q5JoMKwRghJzAVch7GLUVCyuY"
  },
  {
    "chainId": 100,
    "address": "0x9fB1d52596c44603198fB0aee434fac3a679f702",
    "symbol": "jEUR",
    "name": "Jarvis Synthetic Euro",
    "decimals": 18,
    "logoURI": "ipfs://Qmf5dn9opN4UiFuBisAjv95wrxsJvWGoDy6BUz2y8wZ7oq"
  },
  {
    "chainId": 1,
    "address": "0x7d3cb11f8c13730C24D01826d8F2005F0e1b348F",
    "symbol": "COS",
    "name": "COS",
    "decimals": 18,
    "logoURI": "ipfs://Qmby9N7YvEXWpd5jAKDri82exDF1ZRPmAFeALCYoZKRTuY"
  },
  {
    "chainId": 1,
    "address": "0x261EfCdD24CeA98652B9700800a13DfBca4103fF",
    "symbol": "sXAU",
    "name": "Synthetic Gold Ounce",
    "decimals": 18,
    "logoURI": "ipfs://QmQZndRVHLQWYJ3GQfWYq2ttkrbzKDD2ocMTwdx2ZFCagf"
  },
  {
    "chainId": 1,
    "address": "0xfffffffFf15AbF397dA76f1dcc1A1604F45126DB",
    "symbol": "FSW",
    "name": "FalconSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmeDZ7pSyCCGjNVNtY6DGiFUYBFRUE9CH6EkLQJSAn1cLb"
  },
  {
    "chainId": 42161,
    "address": "0xfb9E5D956D889D91a82737B9bFCDaC1DCE3e1449",
    "symbol": "LQTY",
    "name": "Liquity",
    "decimals": 18,
    "logoURI": "ipfs://QmV1bZrR5WNRC5mntEETpHX8sKMft1vtssk38FWWkkgocf"
  },
  {
    "chainId": 1,
    "address": "0x7D2D3688Df45Ce7C552E19c27e007673da9204B8",
    "symbol": "aLEND",
    "name": "Aave LEND",
    "decimals": 18,
    "logoURI": "ipfs://QmcPe96r5dNsNvyySfDsVskQAP4UWsukjbx8JzA4YtvcDR"
  },
  {
    "chainId": 137,
    "address": "0x6749441Fdc8650b5b5a854ed255C82EF361f1596",
    "symbol": "LUCHA",
    "name": "Lucha",
    "decimals": 18,
    "logoURI": "ipfs://QmYZLybVRoLGr5mjCpwxMZtCMxTeckmbbY7zwQhpSChN1Z"
  },
  {
    "chainId": 1,
    "address": "0xd55BD2C12B30075b325Bc35aEf0B46363B3818f8",
    "symbol": "ZOMBIE",
    "name": "Zombie.Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRu9QGFFW8GuWfUDkiDB768NVEtsqCyWsFzk7LAfeTuHF"
  },
  {
    "chainId": 1,
    "address": "0xf2E08356588EC5cd9E437552Da87C0076b4970B0",
    "symbol": "sTRX",
    "name": "Synthetic TRON",
    "decimals": 18,
    "logoURI": "ipfs://QmXcuA9VY1uGAcbAXJ4ZqSbAy4EafpY2Ji7VjW6D6WSdXH"
  },
  {
    "chainId": 137,
    "address": "0xdb95f9188479575F3F718a245EcA1B3BF74567EC",
    "symbol": "GTC",
    "name": "Gitcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmSMz1RrdcqKfUSFPqGFZMFNJm4gdqRWFohbYFgyL1cXaL"
  },
  {
    "chainId": 100,
    "address": "0xc60e38C6352875c051B481Cbe79Dd0383AdB7817",
    "symbol": "NODE",
    "name": "DAppNode",
    "decimals": 18,
    "logoURI": "ipfs://QmZ3hYnNuZV4fHHtWNeckZNDnfwtV7Y4cNJRxTHbke6W71"
  },
  {
    "chainId": 1,
    "address": "0xa4Cf2aFD3B165975afFFBf7e487CDd40C894Ab6B",
    "symbol": "SHIBAKEN",
    "name": "Shibaken Finance",
    "decimals": 0,
    "logoURI": "ipfs://QmVY61J4QPdEzqLPFcGY5eLfgk5SQqXrNsYg75cfAUnUkF"
  },
  {
    "chainId": 1,
    "address": "0xcAAfF72A8CbBfc5Cf343BA4e26f65a257065bFF1",
    "symbol": "KRT",
    "name": "Wrapped KRT",
    "decimals": 18,
    "logoURI": "ipfs://QmUPt5KdXmMGg7k64mZ4NY9icxPCuGcnmNCxQokbeKHCpz"
  },
  {
    "chainId": 1,
    "address": "0xd2Fa8f92Ea72AbB35dBD6DECa57173d22db2BA49",
    "symbol": "ORI",
    "name": "Origami Network",
    "decimals": 18,
    "logoURI": "ipfs://QmPFVPeSdSvA9hr8tULq4ro3mWUirydgozGW4PF3KrRqaF"
  },
  {
    "chainId": 137,
    "address": "0x3A58a54C066FdC0f2D55FC9C89F0415C92eBf3C4",
    "symbol": "stMATIC",
    "name": "Lido Staked MATIC",
    "decimals": 18,
    "logoURI": "ipfs://QmdDu8G9PRksN9qEjAA4Jioh7MKyVyjQdwmG9oh8GKiBRB"
  },
  {
    "chainId": 1,
    "address": "0xAf5191B0De278C7286d6C7CC6ab6BB8A73bA2Cd6",
    "symbol": "STG",
    "name": "Stargate Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmTj9QKW8byFvz6E4CE6n7wrxJRpoTrdU3opWG2AN8R3VK"
  },
  {
    "chainId": 1,
    "address": "0xE7C9C188138f7D70945D420d75F8Ca7d8ab9c700",
    "symbol": "BSDS",
    "name": "Basis Dollar Share",
    "decimals": 18,
    "logoURI": "ipfs://QmNygn8YL4qpwWGj5iTVqagnfSaWrUGnvznmK9S3feZxnP"
  },
  {
    "chainId": 56,
    "address": "0x8F0528cE5eF7B51152A59745bEfDD91D97091d2F",
    "symbol": "ALPACA",
    "name": "Alpaca Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmX52kb3uELLC5yagoxYdWqoqnVkvNjtJRUqh8v7vFsYgr"
  },
  {
    "chainId": 1,
    "address": "0xDDdddd4301A082e62E84e43F474f044423921918",
    "symbol": "DVF",
    "name": "DVF",
    "decimals": 18,
    "logoURI": "ipfs://QmREzt8rWC559YvKgSSqMuYw8moxCjUJXvuuTg7dJZL5nb"
  },
  {
    "chainId": 1,
    "address": "0xfAbA6f8e4a5E8Ab82F62fe7C39859FA577269BE3",
    "symbol": "ONDO",
    "name": "Ondo",
    "decimals": 18,
    "logoURI": "ipfs://QmSvGsTuUFySyWBiWX7Ebkyu8E8XH4zAeZ6FVrvhKDKH6J"
  },
  {
    "chainId": 1,
    "address": "0x2C537E5624e4af88A7ae4060C022609376C8D0EB",
    "symbol": "TRYB",
    "name": "BiLira",
    "decimals": 6,
    "logoURI": "ipfs://QmSYKiiwshu3cphtt7oqPuD2eEUjYPmWojmUkdJvGj8EFg"
  },
  {
    "chainId": 1,
    "address": "0x5d3a536E4D6DbD6114cc1Ead35777bAB948E3643",
    "symbol": "cDAI",
    "name": "Compound Dai",
    "decimals": 8,
    "logoURI": "ipfs://Qmc5fKgqYEgDvvTmtV194EJZuGpn8aMkyxAj6jCZXStieB"
  },
  {
    "chainId": 1,
    "address": "0x40FD72257597aA14C7231A7B1aaa29Fce868F677",
    "symbol": "XOR",
    "name": "Sora",
    "decimals": 18,
    "logoURI": "ipfs://QmeVWXzQiq2tC356xskRULfMYgSgMpMbEdpoFgfJ1baehV"
  },
  {
    "chainId": 1,
    "address": "0x7Fc66500c84A76Ad7e9c93437bFc5Ac33E2DDaE9",
    "symbol": "AAVE",
    "name": "Aave",
    "decimals": 18,
    "logoURI": "ipfs://QmWZmCpB1yJS3qzFVxRQDWB63zFKJq9UmBae25Y5ijj7ma"
  },
  {
    "chainId": 1,
    "address": "0xac3E018457B222d93114458476f3E3416Abbe38F",
    "symbol": "sfrxETH",
    "name": "Staked Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmX1r2JVToPDMkRUFsERHeeCheTXFZ9Hw1nuq1gDNjufxG"
  },
  {
    "chainId": 1,
    "address": "0xC63B8ECCE56aB9C46184eC6aB85e4771fEa4c8AD",
    "symbol": "sGOOG",
    "name": "Synthetic Alphabet",
    "decimals": 18,
    "logoURI": "ipfs://QmdKtL2M6pvCMqsQxywoRQWeuyWDp9S156BM9UKMFvwc2o"
  },
  {
    "chainId": 1,
    "address": "0xA31B1767e09f842ECFd4bc471Fe44F830E3891AA",
    "symbol": "ROOBEE",
    "name": "ROOBEE",
    "decimals": 18,
    "logoURI": "ipfs://QmSeX7GhUACY2mmcpjd5yL6pxjuHv9CarWvhzbAHHwsbag"
  },
  {
    "chainId": 1,
    "address": "0xF4EebDD0704021eF2a6Bbe993fdf93030Cd784b4",
    "symbol": "iEOS",
    "name": "Synthetic Inverse EOS",
    "decimals": 18,
    "logoURI": "ipfs://QmTPwH79fZJgZMR6Adyxhsrm89KBor4kTQJ9Tc3w4aT386"
  },
  {
    "chainId": 1,
    "address": "0x14094949152EDDBFcd073717200DA82fEd8dC960",
    "symbol": "iSAI",
    "name": "Fulcrum SAI iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmZykNkn2647s53wjXfCJvZpT2bTtuzR77JejNMjCYxtZq"
  },
  {
    "chainId": 1,
    "address": "0xDb05EA0877A2622883941b939f0bb11d1ac7c400",
    "symbol": "OPCT",
    "name": "Opacity",
    "decimals": 18,
    "logoURI": "ipfs://QmctShYZNUbjS5FNtNhJyPrPW3mFvKsd2dzbfuQu2ba87e"
  },
  {
    "chainId": 1,
    "address": "0x4688a8b1F292FDaB17E9a90c8Bc379dC1DBd8713",
    "symbol": "COVER",
    "name": "Cover Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmY4nwV5V3kzGM7hanKRGZi5qFHpcx2weik3T7h5imZ1j6"
  },
  {
    "chainId": 1,
    "address": "0x17EF75AA22dD5f6C2763b8304Ab24f40eE54D48a",
    "symbol": "RVP",
    "name": "Revolution Populi",
    "decimals": 18,
    "logoURI": "ipfs://QmNfPVrZd5TmGzHHd6nmF3eBXaSEGV5WmBNzHbFATCMCYR"
  },
  {
    "chainId": 1,
    "address": "0x2f4eb47A1b1F4488C71fc10e39a4aa56AF33Dd49",
    "symbol": "UNCL",
    "name": "UNCL",
    "decimals": 18,
    "logoURI": "ipfs://QmNVFw92RUYVYwDHQTk3CTn7xRPHu1TDkiQgamvo4c5hoD"
  },
  {
    "chainId": 1,
    "address": "0x1F573D6Fb3F13d689FF844B4cE37794d79a7FF1C",
    "symbol": "BNT",
    "name": "Bancor",
    "decimals": 18,
    "logoURI": "ipfs://QmYyiMwdk1BRxCUyXqA44KLDT1p571p81iipGoRmfRLf9a"
  },
  {
    "chainId": 1,
    "address": "0x8E6cd950Ad6ba651F6DD608Dc70e5886B1AA6B24",
    "symbol": "STARL",
    "name": "StarLink",
    "decimals": 18,
    "logoURI": "ipfs://QmYsqWqHeHL3q2beLrAc7KRurniEE9aSDqGSp4XsKeu5LH"
  },
  {
    "chainId": 1,
    "address": "0xe1030B48b2033314979143766d7dC1F40ef8CE11",
    "symbol": "PEEPS",
    "name": "The People's Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmXrRiq1MakbG3sjyZHNgJACnqJ4wAkBiY4ssvV6yX8CmK"
  },
  {
    "chainId": 1,
    "address": "0x57B49219614859176Ddb029298486B6c30193Cbd",
    "symbol": "ORACLE",
    "name": "Oracle AI",
    "decimals": 18,
    "logoURI": "ipfs://QmednDGset8JU9EU8wyCuxLtHp92Zs2bNF2CxJCvhb9emV"
  },
  {
    "chainId": 56,
    "address": "0x734548a9e43d2D564600b1B2ed5bE9C2b911c6aB",
    "symbol": "PEEL",
    "name": "Meta Apes PEEL",
    "decimals": 18,
    "logoURI": "ipfs://QmeCGD9AG5vd9LBjmArDjqKKbQsad86qfw3DGsNeopgSTk"
  },
  {
    "chainId": 56,
    "address": "0xEe9801669C6138E84bD50dEB500827b776777d28",
    "symbol": "O3",
    "name": "O3 Swap",
    "decimals": 18,
    "logoURI": "ipfs://QmeQutpbBzKJRBXbgEptRhzvibPDesyfKeSpDa3PNPxXt6"
  },
  {
    "chainId": 56,
    "address": "0x936B6659Ad0C1b244Ba8Efe639092acae30dc8d6",
    "symbol": "CO",
    "name": "Corite",
    "decimals": 6,
    "logoURI": "ipfs://Qmd6uQP2oabAqLBJD9FsaFRmTb42dmYpvizNU9JwK1gbb1"
  },
  {
    "chainId": 1,
    "address": "0x54C9EA2E9C9E8eD865Db4A4ce6711C2a0d5063Ba",
    "symbol": "BART",
    "name": "BarterTrade",
    "decimals": 18,
    "logoURI": "ipfs://QmRAr7GdwnttAEjsNzBjfoyRSwFfZUuX1yUjGQNPYNe1HH"
  },
  {
    "chainId": 1,
    "address": "0x6c3ea9036406852006290770BEdFcAbA0e23A0e8",
    "symbol": "PYUSD",
    "name": "PayPal USD",
    "decimals": 6,
    "logoURI": "ipfs://QmSgccL8xqACYF1n7WBioLdRRWkdi47ejFRuuaCkAMaDAX"
  },
  {
    "chainId": 43114,
    "address": "0xe0Ce60AF0850bF54072635e66E79Df17082A1109",
    "symbol": "ICE",
    "name": "Popsicle Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmSp3Vnq7VFsFrdcZZ35YkHHFLVF3nWvcDPKSDJgfkTiMD"
  },
  {
    "chainId": 1,
    "address": "0x2B89D32CfB9b28DFCc7FAcC22453c812F9097655",
    "symbol": "HAREM",
    "name": "Harem",
    "decimals": 18,
    "logoURI": "ipfs://QmPLQ3XQ7u3Lw8eAceZnUTnt3Hrc2HVJMgL7BtH1fqq12z"
  },
  {
    "chainId": 1,
    "address": "0xF5581dFeFD8Fb0e4aeC526bE659CFaB1f8c781dA",
    "symbol": "HOPR",
    "name": "HOPR",
    "decimals": 18,
    "logoURI": "ipfs://QmatpCk3VBM241MPEx8D6fMKSyHXpwTi7ua2z5MuzFdR7P"
  },
  {
    "chainId": 1,
    "address": "0xbC396689893D065F41bc2C6EcbeE5e0085233447",
    "symbol": "PERP",
    "name": "Perpetual Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmbrmbQViGSNaHX3MXbRoWRPniKT6QKzbn7zyKHAtK87iu"
  },
  {
    "chainId": 1,
    "address": "0xA67b8e40111A0EDD30C3210b77aadb86AD234c43",
    "symbol": "BANDEX",
    "name": "Banana Index",
    "decimals": 9,
    "logoURI": "ipfs://QmUXQC41rMyn3USxPWR3YfZnjgFZuasVLXVeLqLgmyWCXq"
  },
  {
    "chainId": 1,
    "address": "0xd44bb6663936CAb1310584A277f7DAa6943d4904",
    "symbol": "WCO",
    "name": "Winco",
    "decimals": 8,
    "logoURI": "ipfs://QmNoUNkfxpvXMMqHM2FT8bEepjSt6YwivNducwGAr3v8bF"
  },
  {
    "chainId": 137,
    "address": "0xdF7837DE1F2Fa4631D716CF2502f8b230F1dcc32",
    "symbol": "TEL",
    "name": "Telcoin",
    "decimals": 2,
    "logoURI": "ipfs://QmU4GXPewzzHK4pS7p53ZbGW3DNnmoX7C8jXLMHFXPG6Lv"
  },
  {
    "chainId": 1,
    "address": "0x3E9BC21C9b189C09dF3eF1B824798658d5011937",
    "symbol": "LINA",
    "name": "Linear Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmVdZp7XDu2kQofFhXM3sE5PrFB4rGj7WFEkYN9cb69qLx"
  },
  {
    "chainId": 1,
    "address": "0x43a89815F33747eDBeCC588d6BB7E1c10dDa5599",
    "symbol": "THUN",
    "name": "Thunder",
    "decimals": 18,
    "logoURI": "ipfs://Qma3axxieUnwWnWaWQagGHih4a3iG2G8o6XsYivtCPEgwM"
  },
  {
    "chainId": 1,
    "address": "0xe6828D65bf5023AE1851D90D8783Cc821ba7eeE1",
    "symbol": "ABOND",
    "name": "ApeBond",
    "decimals": 18,
    "logoURI": "ipfs://QmYfNLqbHYpe6rPE285f3XLNhYrqWvVpwv9CD4eqYto9pu"
  },
  {
    "chainId": 1,
    "address": "0x47bc01597798DCD7506DCCA36ac4302fc93a8cFb",
    "symbol": "CMCT",
    "name": "Crowd Machine",
    "decimals": 8,
    "logoURI": "ipfs://QmZkxwQEU11CY9SsZXZ4M1YXXzjwDhYyWvTkEMVLDGUT5b"
  },
  {
    "chainId": 56,
    "address": "0xc5f0f7b66764F6ec8C8Dff7BA683102295E16409",
    "symbol": "FDUSD",
    "name": "First Digital USD",
    "decimals": 18,
    "logoURI": "ipfs://QmQJ2sw6ntJRrv3aGTKLA4MUYBt1KvxA6VqTgi9jshrs35"
  },
  {
    "chainId": 56,
    "address": "0xD41FDb03Ba84762dD66a0af1a6C8540FF1ba5dfb",
    "symbol": "SFP",
    "name": "SafePal",
    "decimals": 18,
    "logoURI": "ipfs://QmVPts8pq7Wr3uehn66tmE8uHMMJ85pGhXjXVBdfRg12Ca"
  },
  {
    "chainId": 56,
    "address": "0xF8A0BF9cF54Bb92F17374d9e9A321E6a111a51bD",
    "symbol": "LINK",
    "name": "Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://QmNV8bCNTxP2ZeryqvtUFL4Z1ky6KVvr5FoicrPY8pECVM"
  },
  {
    "chainId": 1,
    "address": "0x6C8c6b02E7b2BE14d4fA6022Dfd6d75921D90E4E",
    "symbol": "cBAT",
    "name": "Compound Basic Attention Token",
    "decimals": 8,
    "logoURI": "ipfs://QmNVWktiGqs1b6fUBVGq27BC18BeCFqgkTrB1EHcFRZF3E"
  },
  {
    "chainId": 1,
    "address": "0x4c584CD339bdDE73B7F5210486dd8bbeEE3fDe6d",
    "symbol": "SHIBELON",
    "name": "ShibElon",
    "decimals": 9,
    "logoURI": "ipfs://QmWzu7iuneFKBTeG93LxjH15CJzDFh7gtoS7miCpG4Zo4k"
  },
  {
    "chainId": 1,
    "address": "0xe0d2F5C883d5d470D16fa6dD73610bB4253577fF",
    "symbol": "MCM",
    "name": "MemeCoinUniverse",
    "decimals": 9,
    "logoURI": "ipfs://QmTYWMVUp3zXknCo9CisMDSHaFHBVpFH8tWfUTb1KMiTjo"
  },
  {
    "chainId": 1,
    "address": "0x1cEB5cB57C4D4E2b2433641b95Dd330A33185A44",
    "symbol": "KP3R",
    "name": "Keep3rV1",
    "decimals": 18,
    "logoURI": "ipfs://QmQrxRC69mX9dVi1CL1GoVf9iooMaC4UNosTpSaszyg9vu"
  },
  {
    "chainId": 1,
    "address": "0x970B9bB2C0444F5E81e9d0eFb84C8ccdcdcAf84d",
    "symbol": "FUSE",
    "name": "Fuse network",
    "decimals": 18,
    "logoURI": "ipfs://QmWtpfZXtw4PZoZ91SW78CBu8YHD9tZ2EETQzNF9ArFyBv"
  },
  {
    "chainId": 1,
    "address": "0xD9016A907Dc0ECfA3ca425ab20B6b785B42F2373",
    "symbol": "GMEE",
    "name": "GAMEE",
    "decimals": 18,
    "logoURI": "ipfs://QmZtf65Yxq8UAKjGawBAFZBB5WMr9XTwEWcD8qw5aaeXjP"
  },
  {
    "chainId": 56,
    "address": "0x4e3cABD3AD77420FF9031d19899594041C420aeE",
    "symbol": "TITANO",
    "name": "Titano",
    "decimals": 18,
    "logoURI": "ipfs://QmU2ozonna5RmQApSmtj8XNkkZS5ppKL7dGDAia7MEP11C"
  },
  {
    "chainId": 1,
    "address": "0xf8d325872acbAed090056B4E0bf104B1C735F439",
    "symbol": "STC",
    "name": "STC",
    "decimals": 0,
    "logoURI": "ipfs://QmaNeGBhRXHrSfogZhZnF8iw9Xv4PrckBpAcD2YYwjppMt"
  },
  {
    "chainId": 1,
    "address": "0x44fbeBd2F576670a6C33f6Fc0B00aA8c5753b322",
    "symbol": "crUSDC",
    "name": "Cream USD Coin",
    "decimals": 8,
    "logoURI": "ipfs://QmRG1TnamVNvLPoGTHDFRzXL7LvihM6qSSthtJXhQefih5"
  },
  {
    "chainId": 137,
    "address": "0x2791Bca1f2de4661ED88A30C99A7a9449Aa84174",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmUQp1wZzr4UREn4AVLkKH5GbwpUCwf9w5VQQJeh1bmThJ"
  },
  {
    "chainId": 1,
    "address": "0x91fbB2503AC69702061f1AC6885759Fc853e6EaE",
    "symbol": "KNINE",
    "name": "K9 Finance DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmQJAmKz5xRh57fNUpt5nQQG2iEoVc9Z9wdyvxq57ai9XS"
  },
  {
    "chainId": 1,
    "address": "0xBd0793332e9fB844A52a205A233EF27a5b34B927",
    "symbol": "ZB",
    "name": "ZB Token",
    "decimals": 18,
    "logoURI": "ipfs://QmXQxWh8N9TPkRgcbBNhJVem3usoeUPBYLetmj9yhkpVrk"
  },
  {
    "chainId": 1,
    "address": "0x0258F474786DdFd37ABCE6df6BBb1Dd5dfC4434a",
    "symbol": "ORN",
    "name": "Orion Protocol",
    "decimals": 8,
    "logoURI": "ipfs://QmZpUYMDNfEghsDHqw9JnXi8YmD7pKworpxmU76QpmztQj"
  },
  {
    "chainId": 42161,
    "address": "0x71eebA415A523F5C952Cc2f06361D5443545Ad28",
    "symbol": "XDAO",
    "name": "XDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmYQZ8Qav3op7jpqYS3CAM541XKthqxNcVyYdpHthXPy2J"
  },
  {
    "chainId": 324,
    "address": "0xd03465338226ea0178337F4abb16FdD6df529F57",
    "symbol": "ITP",
    "name": "Interport",
    "decimals": 18,
    "logoURI": "ipfs://QmTWPTMwXJWuwDgNmUxpdkZA7UASQfrye7t7MESNBuUJox"
  },
  {
    "chainId": 1,
    "address": "0xC17c30e98541188614dF99239cABD40280810cA3",
    "symbol": "RISE",
    "name": "EverRise",
    "decimals": 18,
    "logoURI": "ipfs://QmU6dKNHUySjmVh9FJFMT1cRZ54aoQ7fpJ4q2a6qFPHsbU"
  },
  {
    "chainId": 1,
    "address": "0xbd31EA8212119f94A611FA969881CBa3EA06Fa3d",
    "symbol": "LUNC",
    "name": "Terra Classic",
    "decimals": 6,
    "logoURI": "ipfs://QmceuGzAgd2wcWiDvX9ygEohHnaMqWDpSok7ot1h7jahZt"
  },
  {
    "chainId": 1,
    "address": "0xae78736Cd615f374D3085123A210448E74Fc6393",
    "symbol": "rETH",
    "name": "Rocket Pool ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmfDu3ixQSNtUxYdRBMhCFSSKUX1H23dfY1dy1NHcF8ZQ2"
  },
  {
    "chainId": 42161,
    "address": "0x87AAfFdF26c6885f6010219208D5B161ec7609c0",
    "symbol": "EQU",
    "name": "Equation",
    "decimals": 18,
    "logoURI": "ipfs://QmQPfMmfNeWxsBiCrTn4XUNAzMVEzDDmn12Hu58PLmGvCf"
  },
  {
    "chainId": 1,
    "address": "0x13dB9034C9CA6cb739887288fcE790544a476f8c",
    "symbol": "TAIYO",
    "name": "Taiyo",
    "decimals": 9,
    "logoURI": "ipfs://QmQEP9PnmZ48qfxNuaxu28z71zjKdE7u3fEvtp3xdZHeoP"
  },
  {
    "chainId": 1,
    "address": "0xacE837f596029039E7eDd4789f9978259e1C3035",
    "symbol": "SHOP",
    "name": "CoinShop",
    "decimals": 10,
    "logoURI": "ipfs://QmVLtNETd6zQvYC36GKJfGdwJHWFsVhUgnE2T49CauEeeG"
  },
  {
    "chainId": 10,
    "address": "0x350a791Bfc2C21F9Ed5d10980Dad2e2638ffa7f6",
    "symbol": "LINK",
    "name": "Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://Qmdn6E15MELheFNupKrMosD9KPDFZ9M3wVKkhCuacT1cVj"
  },
  {
    "chainId": 1,
    "address": "0x7dE91B204C1C737bcEe6F000AAA6569Cf7061cb7",
    "symbol": "XRT",
    "name": "Robonomics Network",
    "decimals": 9,
    "logoURI": "ipfs://QmZD4hXZMe66SiiUwUVn5ayHHa2JcUi8Vzp9D3vCMYf99j"
  },
  {
    "chainId": 56,
    "address": "0xAD6cAEb32CD2c308980a548bD0Bc5AA4306c6c18",
    "symbol": "BAND",
    "name": "Band Protocol",
    "decimals": 18,
    "logoURI": "ipfs://Qmdg8BciXkxa6i9gp1nKBaVLzg1uBETvoJbryNegBWAW6x"
  },
  {
    "chainId": 137,
    "address": "0x62F594339830b90AE4C084aE7D223fFAFd9658A7",
    "symbol": "SPHERE",
    "name": "Sphere Finance",
    "decimals": 18,
    "logoURI": "ipfs://Qmcr1MJPYDeuoNRejs1S6LpQmhrQmvQU6wQ6CA93XwmUHL"
  },
  {
    "chainId": 1,
    "address": "0x41dBECc1cdC5517C6f76f6a6E836aDbEe2754DE3",
    "symbol": "MTN",
    "name": "Medicalchain",
    "decimals": 18,
    "logoURI": "ipfs://Qmdw9uCXAfsYpS1ZJiBZdmvJX9ftKEPZmXgQCBRh8q5Ap5"
  },
  {
    "chainId": 1,
    "address": "0x459086F2376525BdCebA5bDDA135e4E9d3FeF5bf",
    "symbol": "renBCH",
    "name": "renBCH",
    "decimals": 8,
    "logoURI": "ipfs://QmXvRk5pWAkB58A8QpZtsNHfzzy6iPzoAa1seDV1MFhCVC"
  },
  {
    "chainId": 1,
    "address": "0xC57d533c50bC22247d49a368880fb49a1caA39F7",
    "symbol": "PTF",
    "name": "PowerTrade Fuel",
    "decimals": 18,
    "logoURI": "ipfs://QmRCPkcofqoD4izP4Z2muvAdSv7Qy2bhbYSTEpjrpQod98"
  },
  {
    "chainId": 1,
    "address": "0x491604c0FDF08347Dd1fa4Ee062a822A5DD06B5D",
    "symbol": "CTSI",
    "name": "Cartesi",
    "decimals": 18,
    "logoURI": "ipfs://QmXwhy4L7SzzSPKntJSHEDfSL1P9pHjB5wmDk9z1j6pyfp"
  },
  {
    "chainId": 1,
    "address": "0xe1406825186D63980fd6e2eC61888f7B91C4bAe4",
    "symbol": "vBTC",
    "name": "vBTC",
    "decimals": 18,
    "logoURI": "ipfs://QmQYo7FoEhQ7SRYrBLGfu3sf4USAL31VVwKxtu1UhJjF7U"
  },
  {
    "chainId": 1,
    "address": "0x1B957Dc4aEfeed3b4A2351a6A6d5cbfbbA0CeCFa",
    "symbol": "HQX",
    "name": "HOQU",
    "decimals": 18,
    "logoURI": "ipfs://QmVB7dGPkvBpZ5ZozpK9J4p4KSNAbZTqVC6QtaHrQ5oiGt"
  },
  {
    "chainId": 1,
    "address": "0x1cBb83EbcD552D5EBf8131eF8c9CD9d9BAB342bC",
    "symbol": "NFY",
    "name": "Non-Fungible Yearn",
    "decimals": 18,
    "logoURI": "ipfs://QmQBkKBTax8YYjhdqRUPLNuhZXKrx23jK2FxhadRh7BSmG"
  },
  {
    "chainId": 1,
    "address": "0x374CB8C27130E2c9E04F44303f3c8351B9De61C1",
    "symbol": "BAO",
    "name": "Bao Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUcpxByj4HYHCnS28hQjuJjf8sGHtJg6RnSwgHBnY4oEz"
  },
  {
    "chainId": 1,
    "address": "0xcE4Fe9b4b8Ff61949DCfeB7e03bc9FAca59D2Eb3",
    "symbol": "crBAL",
    "name": "Cream Balancer",
    "decimals": 8,
    "logoURI": "ipfs://QmaN8eWm8xv8Fr36taxMrN35QAgN5xSRckwUCg3Smp4wxs"
  },
  {
    "chainId": 1,
    "address": "0xB04DFdb8271ed2d5e13858562C44A77D3CEb9e57",
    "symbol": "BUP",
    "name": "BuildUp",
    "decimals": 18,
    "logoURI": "ipfs://QmUndYD3UPaf41u2p7VS5SYXaPvKSLpf2iCDpyN1gNfcAz"
  },
  {
    "chainId": 1,
    "address": "0x56ba2Ee7890461f463F7be02aAC3099f6d5811A8",
    "symbol": "CAT",
    "name": "BlockCAT",
    "decimals": 18,
    "logoURI": "ipfs://QmRCNiZCPNswyF8XJBcwviW7sXDEwd2BLms5GP6BcmxHY6"
  },
  {
    "chainId": 1,
    "address": "0xf8C3527CC04340b208C854E985240c02F7B7793f",
    "symbol": "FRONT",
    "name": "Frontier",
    "decimals": 18,
    "logoURI": "ipfs://QmbPAXdg4yZb3Nq8NR2xGxc3d5W4Cv6ottd8HtozND6sgY"
  },
  {
    "chainId": 1,
    "address": "0x76BE3b62873462d2142405439777e971754E8E77",
    "symbol": "LL",
    "name": "Parallel Alpha",
    "decimals": 0,
    "logoURI": "ipfs://QmS1iEJdesEULnzwG985pmSHzq2GdKDppnMyjTCAkegNP8"
  },
  {
    "chainId": 1,
    "address": "0x73968b9a57c6E53d41345FD57a6E6ae27d6CDB2F",
    "symbol": "SDT",
    "name": "Stake DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmNoTzaDqVWb9oNsJsGGDrM4rnUikzMpTPzTxj4BUtt6kT"
  },
  {
    "chainId": 1,
    "address": "0xA017ac5faC5941f95010b12570B812C974469c2C",
    "symbol": "XES",
    "name": "Proxeus",
    "decimals": 18,
    "logoURI": "ipfs://QmRRipNdzX4dbm8e1i9ysPgnBz5MtWDKz6YLFy7pgEQsNR"
  },
  {
    "chainId": 1,
    "address": "0x6369c3DadfC00054A42BA8B2c09c48131dd4Aa38",
    "symbol": "MPH",
    "name": "Morpher",
    "decimals": 18,
    "logoURI": "ipfs://QmaEFtAe6wJttQk68dHdmesLCpfDLLuF1GBGsZCVTmQKC4"
  },
  {
    "chainId": 1,
    "address": "0x3774E825d567125988Fb293e926064B6FAa71DAB",
    "symbol": "sUSDC",
    "name": "Strike USDC",
    "decimals": 8,
    "logoURI": "ipfs://QmYepBL97YhLXFKLEoRH6FkbL6fWEReWycfe1mrPrqgna3"
  },
  {
    "chainId": 1,
    "address": "0x111111111117dC0aa78b770fA6A738034120C302",
    "symbol": "1INCH",
    "name": "1Inch Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdUjFYL2izDVe4PLKArg7GtoHmsSqX6Bcsv9fmaVMcbLF"
  },
  {
    "chainId": 10,
    "address": "0x9560e827aF36c94D2Ac33a39bCE1Fe78631088Db",
    "symbol": "VELO",
    "name": "Velodrome Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmQeHJ6fEgCVNo8ZGNdVEczMvRsguv9kYAasVyRLjqXyqm"
  },
  {
    "chainId": 1,
    "address": "0x056Fd409E1d7A124BD7017459dFEa2F387b6d5Cd",
    "symbol": "GUSD",
    "name": "Gemini Dollar",
    "decimals": 2,
    "logoURI": "ipfs://Qma5Z8akU9S9gfzhn2SnzAsVV6w5KLdYZbRcXF8wFjS7LG"
  },
  {
    "chainId": 137,
    "address": "0xc2132D05D31c914a87C6611C10748AEb04B58e8F",
    "symbol": "USDT",
    "name": "Tether",
    "decimals": 6,
    "logoURI": "ipfs://Qmd9YVdtaXDZuMmeChdpK4LfizyLkBbVijpLyXGSNwFGEz"
  },
  {
    "chainId": 1,
    "address": "0xf3db7560E820834658B590C96234c333Cd3D5E5e",
    "symbol": "CHP",
    "name": "CoinPoker",
    "decimals": 18,
    "logoURI": "ipfs://QmdDiPmT4UanudNH1HWX9odWph1hg3EeuJ4NB5Rmdu6uY4"
  },
  {
    "chainId": 1,
    "address": "0x3Ed3B47Dd13EC9a98b44e6204A523E766B225811",
    "symbol": "aUSDT",
    "name": "Aave USDT",
    "decimals": 6,
    "logoURI": "ipfs://QmZgK1xGpwDbMv6sHEgdpPumW89CYcd7ygBTuH5fngDm4w"
  },
  {
    "chainId": 1,
    "address": "0xB879DA8b24c9b8685dE8526cF492E954f165D74b",
    "symbol": "MBL",
    "name": "MovieBloc",
    "decimals": 18,
    "logoURI": "ipfs://QmVfPSLuw4wejEqiQYGSJq7aLRKYRjatWpCFRyc6JN16ab"
  },
  {
    "chainId": 100,
    "address": "0xFAdc59D012Ba3c110B08A15B7755A5cb7Cbe77D7",
    "symbol": "GRT",
    "name": "The Graph",
    "decimals": 18,
    "logoURI": "ipfs://QmaCTcJ7pLUgdjdXeZkubcRsGnyT73fvPBsgPLWMZevCfd"
  },
  {
    "chainId": 1,
    "address": "0x58b6A8A3302369DAEc383334672404Ee733aB239",
    "symbol": "LPT",
    "name": "Livepeer",
    "decimals": 18,
    "logoURI": "ipfs://QmaevUHKYe7k6tS3ovLXMipGZLY4FEKAhoe1EfP6RiVzDS"
  },
  {
    "chainId": 1,
    "address": "0xD9A12Cde03a86E800496469858De8581D3A5353d",
    "symbol": "YUP",
    "name": "Crowdholding",
    "decimals": 18,
    "logoURI": "ipfs://QmbUSMkcbx55r5THhs4EmunCmtWtgQNFQsxGx2cZPRniQ8"
  },
  {
    "chainId": 1,
    "address": "0x86Dd3002BF215082Ea43b0Bd2fD595EcE4341880",
    "symbol": "VIO",
    "name": "VIO",
    "decimals": 18,
    "logoURI": "ipfs://QmagBwPjLa7SFmRDRDNZrfvxZPbtWYKJW8ptBK1hr43rtG"
  },
  {
    "chainId": 1,
    "address": "0x26fb86579e371c7AEdc461b2DdEF0A8628c93d3B",
    "symbol": "BORA",
    "name": "BORA",
    "decimals": 18,
    "logoURI": "ipfs://QmZ5CwhwQhNmn4FKmFrhtfQsqGeeQcBYnfMj6hd11rWq7n"
  },
  {
    "chainId": 137,
    "address": "0x41b3966B4FF7b427969ddf5da3627d6AEAE9a48E",
    "symbol": "NEXO",
    "name": "Nexo",
    "decimals": 18,
    "logoURI": "ipfs://QmeQcA5VJM1ZoPUcEDtzeQ9Y8Qj4gHEbjEjk2e59x8vD3j"
  },
  {
    "chainId": 1,
    "address": "0x17d1823bD5E96e76e822ab42B61d348302E072d5",
    "symbol": "ROSA",
    "name": "ROSA",
    "decimals": 18,
    "logoURI": "ipfs://QmVdkAqLpXBXuoNQMBkuLKPJH2uU64KBvdrDYP4i2sL98o"
  },
  {
    "chainId": 1,
    "address": "0xa1faa113cbE53436Df28FF0aEe54275c13B40975",
    "symbol": "ALPHA",
    "name": "Alpha Finance Lab",
    "decimals": 18,
    "logoURI": "ipfs://Qmd2995jmAebU9StqzUaDSXSj3bc5UKBg3CFxQHYpMKzGq"
  },
  {
    "chainId": 250,
    "address": "0x56ee926bD8c72B2d5fa1aF4d9E4Cbb515a1E3Adc",
    "symbol": "SNX",
    "name": "Synthetix Network",
    "decimals": 18,
    "logoURI": "ipfs://QmX3ov1SgUvUJ64HuEn19RsDkVE91zbUuaCfozReqbbMx5"
  },
  {
    "chainId": 1,
    "address": "0x226f7b842E0F0120b7E194D05432b3fd14773a9D",
    "symbol": "UNN",
    "name": "UNION Protocol Governance Token",
    "decimals": 18,
    "logoURI": "ipfs://QmSnXiNhmfTMvGrrdC4WewzSfkASkTg4BQxwd82Q3JgGW4"
  },
  {
    "chainId": 100,
    "address": "0xC45b3C1c24d5F54E7a2cF288ac668c74Dd507a84",
    "symbol": "SYMM",
    "name": "Symmetric",
    "decimals": 18,
    "logoURI": "ipfs://QmaK238o194FtknhS9B2pNRKS9oZUnuW4keVYYqYyGXh7X"
  },
  {
    "chainId": 1,
    "address": "0x3E1d5A855aD9D948373aE68e4fe1f094612b1322",
    "symbol": "HQT",
    "name": "HyperQuant",
    "decimals": 18,
    "logoURI": "ipfs://QmU3HWjDAYhRA6Cpct21xMuG5QjCSHxAec5XJ9qUajvi1i"
  },
  {
    "chainId": 1,
    "address": "0xa44E5137293E855B1b7bC7E2C6f8cD796fFCB037",
    "symbol": "SENT",
    "name": "Sentinel",
    "decimals": 8,
    "logoURI": "ipfs://QmatciLMmg3tdtnLVjwHEwQyQY4tTMsSBH68zzaMvxAQ3p"
  },
  {
    "chainId": 43114,
    "address": "0x62AceEA3e666C5706Ce1c61055fAc1A669d31D93",
    "symbol": "KALM",
    "name": "KALM",
    "decimals": 18,
    "logoURI": "ipfs://QmZUhb1jzCscZULnbM1jz9KScbxFyMrBnSaawcTd6PmfXT"
  },
  {
    "chainId": 56,
    "address": "0x76F3CE6aF26de7A9854dBD153ACd8f46a2Cf5133",
    "symbol": "GLMR",
    "name": "Moonbeam",
    "decimals": 18,
    "logoURI": "ipfs://Qmb1rxnBpE8uaSrEzKEg3enn2XrMR3749XGw9xpTZ5YzLt"
  },
  {
    "chainId": 1,
    "address": "0xd4fa1460F537bb9085d22C7bcCB5DD450Ef28e3a",
    "symbol": "PPT",
    "name": "Populous",
    "decimals": 8,
    "logoURI": "ipfs://QmdyNccJeVxmaExKhW4SADmavKhsgQyJ7JVnRaSTcYYrMV"
  },
  {
    "chainId": 1,
    "address": "0x6aEDbF8dFF31437220dF351950Ba2a3362168d1b",
    "symbol": "DGS",
    "name": "Dragonglass",
    "decimals": 8,
    "logoURI": "ipfs://QmUqwt4PuKzg2DWn5jRQyTbPsu1eVgNFvEH4qkMYGjorwF"
  },
  {
    "chainId": 250,
    "address": "0x7d016eec9c25232b01F23EF992D98ca97fc2AF5a",
    "symbol": "FXS",
    "name": "Frax Share",
    "decimals": 18,
    "logoURI": "ipfs://QmNRaYxGN6QHc6RKZfmaaKX6DJGtj6W7JeC4bDRroKGUCQ"
  },
  {
    "chainId": 1,
    "address": "0x79ddbbCc9D7FbDc77b02A960DabD2577144FDD6E",
    "symbol": "CPX",
    "name": "Crypxie",
    "decimals": 18,
    "logoURI": "ipfs://QmRkf5VRnfezcP9Y5ei9wp5dzf9Qdw1N5fYSkzbLrJUr9m"
  },
  {
    "chainId": 1,
    "address": "0xaCbd826394189Cf2623C6DF98a18b41fC8fFC16D",
    "symbol": "N1",
    "name": "NFTify",
    "decimals": 18,
    "logoURI": "ipfs://QmS67N76KoczJVVgiEQxbYuYwWjskfbT74R1HKYCEXhVrv"
  },
  {
    "chainId": 100,
    "address": "0x2F0E755Efe6b58238A67DB420Ff3513Ec1fb31eF",
    "symbol": "RPL",
    "name": "Rocket Pool",
    "decimals": 18,
    "logoURI": "ipfs://QmNSj1BfWphbDaTStW6JvrmnzoEHpFJpPNNczNAheANF4f"
  },
  {
    "chainId": 1,
    "address": "0xaB456bDb0A373BBaC6C4A76176E9f159cAcD5752",
    "symbol": "SGE",
    "name": "Society of Galactic Exploration",
    "decimals": 9,
    "logoURI": "ipfs://QmcTLD8aWrMQW7q4SwE5P4uJyDdCXgvHDmuYRZfYeyuh1v"
  },
  {
    "chainId": 1,
    "address": "0xDb298285FE4C5410B05390cA80e8Fbe9DE1F259B",
    "symbol": "FOREX",
    "name": "handleFOREX",
    "decimals": 18,
    "logoURI": "ipfs://Qmcyx9EoC9BhToyzZLSCW47jqmVxdw42F2LXGwCVzZHLUG"
  },
  {
    "chainId": 137,
    "address": "0x3a9A81d576d83FF21f26f325066054540720fC34",
    "symbol": "DATA",
    "name": "Streamr",
    "decimals": 18,
    "logoURI": "ipfs://QmeBtBN63Ah7FkrZEF848tDbZKXeXApv165obUQdw7otk5"
  },
  {
    "chainId": 56,
    "address": "0x4d2d32d8652058Bf98c772953E1Df5c5c85D9F45",
    "symbol": "DAO",
    "name": "DAO Maker",
    "decimals": 18,
    "logoURI": "ipfs://QmXFYWGruYGapJjmBHUsLmycCzoSNWQTofUmfd2qbMUzvW"
  },
  {
    "chainId": 1,
    "address": "0xC18e7a03f8986798323658dD8645f93aa79Ac5C9",
    "symbol": "USKITA",
    "name": "American Akita",
    "decimals": 9,
    "logoURI": "ipfs://QmSKPe75db4tZxwiz2PdvcFDWfZsD8n7vxNR8cK3S9mZVh"
  },
  {
    "chainId": 1,
    "address": "0x0000000000004946c0e9F43F4Dee607b0eF1fA1c",
    "symbol": "CHI",
    "name": "Chi Gastoken",
    "decimals": 0,
    "logoURI": "ipfs://QmaELhudz2hajBYobUud6xWTXGB1hCis6eui7La56F7af3"
  },
  {
    "chainId": 1,
    "address": "0xb4058411967D5046f3510943103805be61f0600E",
    "symbol": "STONK",
    "name": "STONK",
    "decimals": 18,
    "logoURI": "ipfs://QmXho9sKKLVVAb9jt9LxPBzGcwFdaxHthabSJ9VnD5rdFJ"
  },
  {
    "chainId": 1,
    "address": "0x0C10bF8FcB7Bf5412187A595ab97a3609160b5c6",
    "symbol": "USDD",
    "name": "Decentralized USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPtvMuo6UQwDXB7cTVz85Ms5noX6rnnNKi3HNmwKrPf8u"
  },
  {
    "chainId": 56,
    "address": "0x11cd72f7A4B699c67f225ca8aBb20bC9F8DB90c7",
    "symbol": "OSAK",
    "name": "Osaka Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUng1scXVJSCH1MpqK4RanCsW1aXYew8KUZphtj3WUpah"
  },
  {
    "chainId": 42161,
    "address": "0x6694340fc020c5E6B96567843da2df01b2CE1eb6",
    "symbol": "STG",
    "name": "Stargate Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYCe3FShq8cRmJ9njM8eZhoNqkuohMm5hUjd4khBcEB8U"
  },
  {
    "chainId": 56,
    "address": "0x4B0F1812e5Df2A09796481Ff14017e6005508003",
    "symbol": "TWT",
    "name": "Trust Wallet",
    "decimals": 18,
    "logoURI": "ipfs://QmSfXgD1xZoZQa2VVAxVxHjaRG5Jb8vauneyUmBi1HNTMv"
  },
  {
    "chainId": 1,
    "address": "0x2d94AA3e47d9D5024503Ca8491fcE9A2fB4DA198",
    "symbol": "BANK",
    "name": "Bankless DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmU6UJuSbeRcxDaG8PVbuE74x2fud6dDq3yzSLLA7c2sqz"
  },
  {
    "chainId": 56,
    "address": "0xF77351D8f4eE853135961A936BB8d2e4fFa75f9D",
    "symbol": "ROOBEE",
    "name": "ROOBEE",
    "decimals": 18,
    "logoURI": "ipfs://QmSeX7GhUACY2mmcpjd5yL6pxjuHv9CarWvhzbAHHwsbag"
  },
  {
    "chainId": 56,
    "address": "0x84e9a6F9D240FdD33801f7135908BfA16866939A",
    "symbol": "GMEE",
    "name": "GAMEE",
    "decimals": 18,
    "logoURI": "ipfs://QmZtf65Yxq8UAKjGawBAFZBB5WMr9XTwEWcD8qw5aaeXjP"
  },
  {
    "chainId": 42161,
    "address": "0x0c880f6761F1af8d9Aa9C466984b80DAb9a8c9e8",
    "symbol": "PENDLE",
    "name": "Pendle",
    "decimals": 18,
    "logoURI": "ipfs://QmUgLTHTMw5TD3NmeH4L9VAcRH7EAevahe5AtP7vPVL4Gc"
  },
  {
    "chainId": 1,
    "address": "0xe28b3B32B6c345A34Ff64674606124Dd5Aceca30",
    "symbol": "INJ",
    "name": "Injective",
    "decimals": 18,
    "logoURI": "ipfs://QmbHj8s17pzLhzMBb9azQu5Gjn4ufJT3iZMm7vCefgJhBn"
  },
  {
    "chainId": 1,
    "address": "0x6226caA1857AFBc6DFB6ca66071Eb241228031A1",
    "symbol": "LAR",
    "name": "LinkArt",
    "decimals": 18,
    "logoURI": "ipfs://QmZBy8duR1KoUbLufsUFBngJKGqqkFJgstXwoT29bQkPD6"
  },
  {
    "chainId": 1,
    "address": "0x5fAa989Af96Af85384b8a938c2EdE4A7378D9875",
    "symbol": "GAL",
    "name": "Galxe",
    "decimals": 18,
    "logoURI": "ipfs://Qmbmw5JewVsNkzb2hfMgEgn9X19wejADFyKun8ef2KQqaH"
  },
  {
    "chainId": 250,
    "address": "0x2b1D36f5B61AdDAf7DA7ebbd11B35FD8cfb0DE31",
    "symbol": "ITP",
    "name": "Interport",
    "decimals": 18,
    "logoURI": "ipfs://QmTWPTMwXJWuwDgNmUxpdkZA7UASQfrye7t7MESNBuUJox"
  },
  {
    "chainId": 1,
    "address": "0x09395a2A58DB45db0da254c7EAa5AC469D8bDc85",
    "symbol": "SQT",
    "name": "SubQuery Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdjHWvgz6SUtKEUv2cFtYoqP1zRJEeC1FK9PVWXT1bQVH"
  },
  {
    "chainId": 1,
    "address": "0x1a7a8BD9106F2B8D977E08582DC7d24c723ab0DB",
    "symbol": "APPC",
    "name": "AppCoins",
    "decimals": 18,
    "logoURI": "ipfs://QmWtSbW1i94HiGjTx8GSNMhudoByafCuV1YLShwZVJCpcC"
  },
  {
    "chainId": 42161,
    "address": "0xB64E280e9D1B5DbEc4AcceDb2257A87b400DB149",
    "symbol": "LVL",
    "name": "Level",
    "decimals": 18,
    "logoURI": "ipfs://QmVGkU9X4SBvyErWHuiHW1wtpBWJBdFUeWgzVqTJEkxe74"
  },
  {
    "chainId": 1,
    "address": "0xB8fA12f8409DA31A4fc43D15c4c78C33d8213B9B",
    "symbol": "CALI",
    "name": "CaliCoin",
    "decimals": 18,
    "logoURI": "ipfs://QmV64d6qKqYioY2Hbq7KMEpDjQyx6zRaHDNiFE17gNGhHA"
  },
  {
    "chainId": 1,
    "address": "0xC011a73ee8576Fb46F5E1c5751cA3B9Fe0af2a6F",
    "symbol": "SNX",
    "name": "Synthetix Network Token",
    "decimals": 18,
    "logoURI": "ipfs://QmSb1MTUsbzKTBuRjBiAjCjJSYaWFNRwkD7LnxswVKyJZc"
  },
  {
    "chainId": 1,
    "address": "0x315dc1b524de57AE8e809A2e97699dbc895B8a21",
    "symbol": "PIST",
    "name": "Pist Trust",
    "decimals": 9,
    "logoURI": "ipfs://Qmdvz7FRMPP6r9Hnwu7LXTkXGc8DGZCSukwDvZ22QPW84C"
  },
  {
    "chainId": 100,
    "address": "0x97Edc0e345FbBBd8460847Fcfa3bc2a13bF8641F",
    "symbol": "RICE",
    "name": "DAOSquare",
    "decimals": 18,
    "logoURI": "ipfs://Qmf2DfENXpxg3AiNyuizMh38VnKcszgA46muZkjh75iG2z"
  },
  {
    "chainId": 137,
    "address": "0x50B728D8D964fd00C2d0AAD81718b71311feF68a",
    "symbol": "SNX",
    "name": "Synthetix Network",
    "decimals": 18,
    "logoURI": "ipfs://QmX3ov1SgUvUJ64HuEn19RsDkVE91zbUuaCfozReqbbMx5"
  },
  {
    "chainId": 42161,
    "address": "0xbB85D38fAA5064FAB8bf3E0a79583a2670F03DbC",
    "symbol": "FS",
    "name": "FantomStarter",
    "decimals": 18,
    "logoURI": "ipfs://QmeqPPX87Xj5FwRa18FNg5iEvtB3rDfQ5Gp3Yk8yb35C5r"
  },
  {
    "chainId": 1,
    "address": "0x888888848B652B3E3a0f34c96E00EEC0F3a23F72",
    "symbol": "TLM",
    "name": "Alien Worlds",
    "decimals": 4,
    "logoURI": "ipfs://QmWGNjuwADjssVS5LDUoJWVSj6wW5VxLUfrcedUDZE3quw"
  },
  {
    "chainId": 1,
    "address": "0x36A00FF9072570eF4B9292117850B8FE08d96cce",
    "symbol": "iUNI",
    "name": "Synthetic Inverse Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmNS237uaZzk376w96KYwzpZsCWYWJPtXqJ7nnWNQxo1Nv"
  },
  {
    "chainId": 1,
    "address": "0x992058B7DB08F9734d84485bfbC243C4ee6954A7",
    "symbol": "sYFI",
    "name": "Synthetic yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmXUyHpszkjY343mUEt2YbnGS3a2yb2qz7YCy9EBiopd9W"
  },
  {
    "chainId": 1,
    "address": "0xe53EC727dbDEB9E2d5456c3be40cFF031AB40A55",
    "symbol": "SUPER",
    "name": "SuperFarm",
    "decimals": 18,
    "logoURI": "ipfs://QmVqiHM2HPc8bdMwWei1DyJNX7Fv76E8NogMXuwAPGHh2F"
  },
  {
    "chainId": 1,
    "address": "0x99d36e97676A68313ffDc627fd6b56382a2a08B6",
    "symbol": "BABYTRUMP",
    "name": "Baby Trump",
    "decimals": 9,
    "logoURI": "ipfs://QmaZ5hYtuoPdCxhpYNUR1Tw1kUJNR7b9PvEFqDJWFB3vHt"
  },
  {
    "chainId": 42161,
    "address": "0xFb930D1A28990820C98144201637C99beA8cB91c",
    "symbol": "BUMP",
    "name": "Bumper",
    "decimals": 18,
    "logoURI": "ipfs://QmYxQVsZwu4GYsmphLLFhzwmJvn6DbVbjeAMCedjS3sqsh"
  },
  {
    "chainId": 1,
    "address": "0xBA50933C268F567BDC86E1aC131BE072C6B0b71a",
    "symbol": "ARPA",
    "name": "ARPA",
    "decimals": 18,
    "logoURI": "ipfs://QmZ8TDgRJBPP5a6LXoHc9DZ5igtPQY6w8mPCuSTXST1WhF"
  },
  {
    "chainId": 1,
    "address": "0x2ef52Ed7De8c5ce03a4eF0efbe9B7450F2D7Edc9",
    "symbol": "REV",
    "name": "Revain",
    "decimals": 6,
    "logoURI": "ipfs://QmQREivJ5WBdNaAZgyhPvfPbhHnzg3fKGQnxGTbWca1CVH"
  },
  {
    "chainId": 1,
    "address": "0x26B3038a7Fc10b36c426846a9086Ef87328dA702",
    "symbol": "YFT",
    "name": "Yield Farming Token",
    "decimals": 18,
    "logoURI": "ipfs://QmTWusK6hMYQy2oyfLW7wxmMuriRUMgN9U7tXTcjw8xALx"
  },
  {
    "chainId": 137,
    "address": "0xC17c30e98541188614dF99239cABD40280810cA3",
    "symbol": "RISE",
    "name": "EverRise",
    "decimals": 18,
    "logoURI": "ipfs://QmYyfc3fdEJCtKYAf4pWqssfpsuB2GfcsbwsstDmTwtCZK"
  },
  {
    "chainId": 1,
    "address": "0x8a6ACA71A218301c7081d4e96D64292D3B275ce0",
    "symbol": "SFG",
    "name": "S.Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmZu69jaqgNgsCSQYCV3xLV4bbDZrAuYbpdzDaMkrJ6Gd8"
  },
  {
    "chainId": 8453,
    "address": "0x22a2488fE295047Ba13BD8cCCdBC8361DBD8cf7c",
    "symbol": "SONNE",
    "name": "Sonne Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYtbpRyNJaMzVvZnyB4jRaWdBDgZqm5F5Q6ShPDQ9LPPv"
  },
  {
    "chainId": 56,
    "address": "0xaA9E582e5751d703F85912903bacADdFed26484C",
    "symbol": "HAI",
    "name": "Hacken",
    "decimals": 8,
    "logoURI": "ipfs://QmeJ3JYqTDyUVY8mPWNRrU8cKCPtLFoLGsjJwwkABzhh2o"
  },
  {
    "chainId": 1,
    "address": "0x73EE6D7e6b203125aDd89320E9F343d65ec7c39a",
    "symbol": "AXI",
    "name": "Axioms",
    "decimals": 18,
    "logoURI": "ipfs://QmUoQsmcH9rLCH6Fn181avGWRbdxVbdfiTC9f5kwNUYD7X"
  },
  {
    "chainId": 1,
    "address": "0x05Fb86775Fd5c16290f1E838F5caaa7342bD9a63",
    "symbol": "HAI",
    "name": "Hacken",
    "decimals": 8,
    "logoURI": "ipfs://QmeJ3JYqTDyUVY8mPWNRrU8cKCPtLFoLGsjJwwkABzhh2o"
  },
  {
    "chainId": 8453,
    "address": "0x22e6966B799c4D5B13BE962E1D117b56327FDa66",
    "symbol": "SNX",
    "name": "Synthetix Network",
    "decimals": 18,
    "logoURI": "ipfs://QmX3ov1SgUvUJ64HuEn19RsDkVE91zbUuaCfozReqbbMx5"
  },
  {
    "chainId": 56,
    "address": "0xAdBAF88B39D37Dc68775eD1541F1bf83A5A45feB",
    "symbol": "COTI",
    "name": "COTI",
    "decimals": 18,
    "logoURI": "ipfs://QmTGW8hKt4LrKDg5bdt2kGsQXf5FaRToDtBjnWfU4BgZid"
  },
  {
    "chainId": 1,
    "address": "0xEdc511cD6cf92877f755C3641141d416D65A4E10",
    "symbol": "CHRG",
    "name": "Charge",
    "decimals": 9,
    "logoURI": "ipfs://QmaXWTh1Jkrsqc7sLt6qjmFa2kdz2xzUMSoYE2ZVgnSY1t"
  },
  {
    "chainId": 1,
    "address": "0x85f6eB2BD5a062f5F8560BE93FB7147e16c81472",
    "symbol": "FLY",
    "name": "Franklin",
    "decimals": 4,
    "logoURI": "ipfs://Qmc1vGMcSD8TF9bDWLde6pXS1751LRuMc1gYU9hRBHYfY9"
  },
  {
    "chainId": 1,
    "address": "0x55a290f08Bb4CAe8DcF1Ea5635A3FCfd4Da60456",
    "symbol": "BITTO",
    "name": "BITTO",
    "decimals": 18,
    "logoURI": "ipfs://QmcXrUdjUbb8DjBb8xze35hDEzkLrMwEef4Ab7TZLsnvAA"
  },
  {
    "chainId": 1,
    "address": "0xDD16eC0F66E54d453e6756713E533355989040E4",
    "symbol": "TEN",
    "name": "Tokenomy",
    "decimals": 18,
    "logoURI": "ipfs://QmcdUHQj68mtAtHWMBi3M3rTvEEk32PwNwRVX9AU1saFCK"
  },
  {
    "chainId": 1,
    "address": "0xBF494F02EE3FdE1F20BEE6242bCe2d1ED0c15e47",
    "symbol": "WORLD",
    "name": "World Token",
    "decimals": 18,
    "logoURI": "ipfs://Qmekwss8MUKQDfRCZdqvg1rSVuxz7b6PHNYNkTULwtfb2N"
  },
  {
    "chainId": 1,
    "address": "0x333A4823466879eeF910A04D473505da62142069",
    "symbol": "NATION",
    "name": "Nation3",
    "decimals": 18,
    "logoURI": "ipfs://QmSuLhxya7ibLn9cM1m8EUJcjTvzHdRXauNaPCDKJmi3Rm"
  },
  {
    "chainId": 1,
    "address": "0x177d39AC676ED1C67A2b268AD7F1E58826E5B0af",
    "symbol": "CDT",
    "name": "Blox",
    "decimals": 18,
    "logoURI": "ipfs://QmbkMTdawLj1nvmEpSVUduprodLwvf7AdZYgf78HvNcABx"
  },
  {
    "chainId": 1,
    "address": "0x40d1F63B5D2048e67E9bEdB1B4c2F1A9fb4b6817",
    "symbol": "GOLD",
    "name": "Golden Goose",
    "decimals": 18,
    "logoURI": "ipfs://Qmf2mCiy3xZEePDgX215tYAZ3NMEjfhW5GxathDbnv2sGP"
  },
  {
    "chainId": 1,
    "address": "0x66fD97a78d8854fEc445cd1C80a07896B0b4851f",
    "symbol": "LMY",
    "name": "Lunch Money",
    "decimals": 18,
    "logoURI": "ipfs://QmTcNE3nWGzBegiumrSvQCf3U1y5YxezWvGk3YkPok9bzy"
  },
  {
    "chainId": 1,
    "address": "0x9d62526f5Ce701950c30F2cACa70Edf70f9fbf0F",
    "symbol": "BKT",
    "name": "Blocktanium",
    "decimals": 18,
    "logoURI": "ipfs://QmYUfQj7KrJqvBmZGxauRD9WcDtdHoiGk95kNpXg8HY5qb"
  },
  {
    "chainId": 1,
    "address": "0x491E136FF7FF03E6aB097E54734697Bb5802FC1C",
    "symbol": "KTN",
    "name": "Kattana",
    "decimals": 18,
    "logoURI": "ipfs://QmaAbjTN4GZCSswo73Der5pj7USvFzxmHkLu64dBoBQdMM"
  },
  {
    "chainId": 137,
    "address": "0xb33EaAd8d922B1083446DC23f610c2567fB5180f",
    "symbol": "UNI",
    "name": "Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmXw3PFpzLkvEjSouZiUvBWqasrL8NxHoEDGt87MqsPmtV"
  },
  {
    "chainId": 1,
    "address": "0x63f3B021B40a54D5a612b8dc3C0ff7DfA7da476D",
    "symbol": "EXEC",
    "name": "Executive Token",
    "decimals": 9,
    "logoURI": "ipfs://QmcnaTh2UBrSuw6KBt1mH3maCN8qSaYybucpL2Vu6RiHkE"
  },
  {
    "chainId": 1,
    "address": "0x13119E34E140097a507B07a5564bDe1bC375D9e6",
    "symbol": "IMT",
    "name": "MoneyToken",
    "decimals": 18,
    "logoURI": "ipfs://QmcFmvkE7xT27ECJsj3HPUyhrKAP4g26oKDfq7nwdSXA6A"
  },
  {
    "chainId": 100,
    "address": "0xdd96B45877d0E8361a4DDb732da741e97f3191Ff",
    "symbol": "BUSD",
    "name": "Binance USD",
    "decimals": 18,
    "logoURI": "ipfs://QmUCSx8qo7kSFcdPJ7gLp9X9pPkue13KUmnL6ZzBZ6ejaQ"
  },
  {
    "chainId": 56,
    "address": "0x14778860E937f509e651192a90589dE711Fb88a9",
    "symbol": "CYBER",
    "name": "CyberConnect",
    "decimals": 18,
    "logoURI": "ipfs://QmWsLPSKnBnhUzET5D5Y53QCL2j3HbPnQxssxtfjHF8Ahx"
  },
  {
    "chainId": 1,
    "address": "0x058Ef0Ba85E053e55d357C8A95BC6Ea7458Def8a",
    "symbol": "TKX",
    "name": "TradeKax",
    "decimals": 18,
    "logoURI": "ipfs://Qmdgv5NHY7k2Pas8RQgkwVozMSpLS5Sh1khKWZuywSgwLD"
  },
  {
    "chainId": 1,
    "address": "0x9248c485b0B80f76DA451f167A8db30F33C70907",
    "symbol": "DEBASE",
    "name": "Debase",
    "decimals": 18,
    "logoURI": "ipfs://Qmej6dRcRUWYZvc4ic4Dim4tjuESXrFuyMWrKvB6PxX3GN"
  },
  {
    "chainId": 137,
    "address": "0x7ceB23fD6bC0adD59E62ac25578270cFf1b9f619",
    "symbol": "WETH",
    "name": "Wrapped Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmeH11wxc6kpgAEpFRyteAJq9ksXzsmfHJcPWLQeCT1f8x"
  },
  {
    "chainId": 1,
    "address": "0xaB37e1358b639Fd877f015027Bb62d3ddAa7557E",
    "symbol": "LIEN",
    "name": "Lien",
    "decimals": 8,
    "logoURI": "ipfs://QmPftDGYxRaSM5NcVu55ZdMzhGbvadfUpQJjnqBZ3AxGB5"
  },
  {
    "chainId": 42161,
    "address": "0x95aB45875cFFdba1E5f451B950bC2E42c0053f39",
    "symbol": "sfrxETH",
    "name": "Staked Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmX1r2JVToPDMkRUFsERHeeCheTXFZ9Hw1nuq1gDNjufxG"
  },
  {
    "chainId": 1,
    "address": "0xD49ff13661451313cA1553fd6954BD1d9b6E02b9",
    "symbol": "ELEC",
    "name": "Electrify.Asia",
    "decimals": 18,
    "logoURI": "ipfs://QmcLiKhh6EwLKneVkx4V99ym9gRmyta9KGfAJWPm58HUa6"
  },
  {
    "chainId": 1,
    "address": "0x9AF4f26941677C706cfEcf6D3379FF01bB85D5Ab",
    "symbol": "DRT",
    "name": "DomRaider",
    "decimals": 8,
    "logoURI": "ipfs://QmYabsmRgfq6m3ew7bG6ev1wC5XTTK81Te6Hh9ywNeAK1Q"
  },
  {
    "chainId": 1,
    "address": "0x5BC25f649fc4e26069dDF4cF4010F9f706c23831",
    "symbol": "DUSD",
    "name": "DefiDollar",
    "decimals": 18,
    "logoURI": "ipfs://QmUZkq4BhHNiP7cNzNqHuHdXzTnA1kNLbCoMrGLNJgLZRE"
  },
  {
    "chainId": 1,
    "address": "0x467719aD09025FcC6cF6F8311755809d45a5E5f3",
    "symbol": "AXL",
    "name": "Axelar",
    "decimals": 6,
    "logoURI": "ipfs://QmdHe4ZsQ2DVvReLPGnE7S8fLwsDrfCSRngykTmQkugEmM"
  },
  {
    "chainId": 1,
    "address": "0x42bBFa2e77757C645eeaAd1655E0911a7553Efbc",
    "symbol": "BOBA",
    "name": "Boba Network",
    "decimals": 18,
    "logoURI": "ipfs://QmZrzc6DeCMZnGNXoPnLAZHM6wphS9uwWJGEZGWDxd1s9x"
  },
  {
    "chainId": 1,
    "address": "0x4672bAD527107471cB5067a887f4656D585a8A31",
    "symbol": "DROP",
    "name": "Dropil",
    "decimals": 18,
    "logoURI": "ipfs://QmQx6YYzfLey2vci1qNf1wPkLzaADQP7YbKu1vUnBefvGC"
  },
  {
    "chainId": 1,
    "address": "0xD6014EA05BDe904448B743833dDF07c3C7837481",
    "symbol": "iBTC",
    "name": "Synthetic Inverse Bitcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmYyfviMa2jhbT3A735kfa2i5ijADBeAkdEHqMeHvnryBL"
  },
  {
    "chainId": 56,
    "address": "0x101d82428437127bF1608F699CD651e6Abf9766E",
    "symbol": "BAT",
    "name": "Basic Attention Token",
    "decimals": 18,
    "logoURI": "ipfs://QmZPngVewTaGaXqN28CnZ9EJSBJ2jAEnMao87T9RbPUoGM"
  },
  {
    "chainId": 1,
    "address": "0xa3d58c4E56fedCae3a7c43A725aeE9A71F0ece4e",
    "symbol": "MET",
    "name": "Metronome",
    "decimals": 18,
    "logoURI": "ipfs://QmTvoTYEFefiC1bvSNViEF8NqvoFq8mrweEGRJTFhBr1qP"
  },
  {
    "chainId": 1,
    "address": "0xcB550A6D4C8e3517A939BC79d0c7093eb7cF56B5",
    "symbol": "yvWBTC",
    "name": "WBTC yVault",
    "decimals": 8,
    "logoURI": "ipfs://QmUZiFXhWsSYNAFxhXg65CQ6zuTUmtHN6mnjsJzZqR8BHB"
  },
  {
    "chainId": 1,
    "address": "0xCeD4E93198734dDaFf8492d525Bd258D49eb388E",
    "symbol": "EDO",
    "name": "Eidoo",
    "decimals": 18,
    "logoURI": "ipfs://QmZx9USmyqXt1DVf4MZAWciZ2wkScU7RxVM5G9EtMrgtC2"
  },
  {
    "chainId": 250,
    "address": "0x46E7628E8b4350b2716ab470eE0bA1fa9e76c6C5",
    "symbol": "BAND",
    "name": "Band Protocol",
    "decimals": 18,
    "logoURI": "ipfs://Qmdg8BciXkxa6i9gp1nKBaVLzg1uBETvoJbryNegBWAW6x"
  },
  {
    "chainId": 1,
    "address": "0x1e4746dC744503b53b4A082cB3607B169a289090",
    "symbol": "IPOR",
    "name": "IPOR",
    "decimals": 18,
    "logoURI": "ipfs://QmS48QnuzPgbAZSZ4RcnknAryLucXhmFVdaFujg7YTBFXz"
  },
  {
    "chainId": 56,
    "address": "0x111111111117dC0aa78b770fA6A738034120C302",
    "symbol": "1INCH",
    "name": "1inch Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdUjFYL2izDVe4PLKArg7GtoHmsSqX6Bcsv9fmaVMcbLF"
  },
  {
    "chainId": 1,
    "address": "0x5165d24277cD063F5ac44Efd447B27025e888f37",
    "symbol": "aYFI",
    "name": "Aave YFI",
    "decimals": 18,
    "logoURI": "ipfs://QmYvuK9FB9pSuusAwywdupJKJNviK1GH9t4GgUf2Szsqur"
  },
  {
    "chainId": 137,
    "address": "0x16ECCfDbb4eE1A85A33f3A9B21175Cd7Ae753dB4",
    "symbol": "ROUTE",
    "name": "Router Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmdjjKLtN8YkXVRQTxSYhKvnUa2Lk6qqQxRc1NDUiSxxAt"
  },
  {
    "chainId": 56,
    "address": "0xBf5140A22578168FD562DCcF235E5D43A02ce9B1",
    "symbol": "UNI",
    "name": "Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmbSptJsjBStRQ7tpzdq8DAnaT4cgaKbpHKTLLtsVN5n1Q"
  },
  {
    "chainId": 1,
    "address": "0xCE3f08e664693ca792caCE4af1364D5e220827B2",
    "symbol": "SAITAMA",
    "name": "SAITAMA",
    "decimals": 9,
    "logoURI": "ipfs://QmSBrUKL8q1mJk7HepJLskCJ8MJSXbmaQ38iF2nSpU5kZ4"
  },
  {
    "chainId": 1,
    "address": "0x1122B6a0E00DCe0563082b6e2953f3A943855c1F",
    "symbol": "CENNZ",
    "name": "Centrality",
    "decimals": 18,
    "logoURI": "ipfs://QmYLGu69GcmyLJRcnBz9NguvoUx6MSxHXWfWmpSnHLMcaa"
  },
  {
    "chainId": 1,
    "address": "0x8a83A0DE55C003A970070a8a0FA7CB24203fC13d",
    "symbol": "PAST",
    "name": "Past Project",
    "decimals": 9,
    "logoURI": "ipfs://QmdzNBY9uQr3EXAWVJDfLjysYGVX8ysw5V3bsfNhUb3UmH"
  },
  {
    "chainId": 1,
    "address": "0x1254E59712e6e727dC71E0E3121Ae952b2c4c3b6",
    "symbol": "MRS",
    "name": "Marginless",
    "decimals": 18,
    "logoURI": "ipfs://QmYgxpxDBtAUTwfemjPJZR4UDRaTj5PbVRkiEjAxjxkksX"
  },
  {
    "chainId": 1,
    "address": "0xB62132e35a6c13ee1EE0f84dC5d40bad8d815206",
    "symbol": "NEXO",
    "name": "Nexo",
    "decimals": 18,
    "logoURI": "ipfs://Qmd7LPax7FffjbtwiKTrXT3Nx8MHA2LzYM6gcuJ9wscubk"
  },
  {
    "chainId": 137,
    "address": "0x172370d5Cd63279eFa6d502DAB29171933a610AF",
    "symbol": "CRV",
    "name": "Curve DAO Token",
    "decimals": 18,
    "logoURI": "ipfs://QmNPpFtjU5kMFjz13Ysb3p6c2rZZQp3ysKzyXonjYCeKyT"
  },
  {
    "chainId": 1,
    "address": "0xfdb615d6A15F929dDabc6b83A4f1Cf9d361b064E",
    "symbol": "DEVE",
    "name": "Divert Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmcBybSFhSAQxQzzHds6xDR1jVKMF53hR8xvNFjQpn7KSu"
  },
  {
    "chainId": 1,
    "address": "0x6E2a43be0B1d33b726f0CA3b8de60b3482b8b050",
    "symbol": "ARKM",
    "name": "Arkham",
    "decimals": 18,
    "logoURI": "ipfs://QmVNTKrfi3pMB82hUd4csU8j7WJo6MPKqzm6XWsXyHm3oU"
  },
  {
    "chainId": 42161,
    "address": "0x2b1D36f5B61AdDAf7DA7ebbd11B35FD8cfb0DE31",
    "symbol": "ITP",
    "name": "Interport",
    "decimals": 18,
    "logoURI": "ipfs://QmTWPTMwXJWuwDgNmUxpdkZA7UASQfrye7t7MESNBuUJox"
  },
  {
    "chainId": 43114,
    "address": "0xd1c3f94DE7e5B45fa4eDBBA472491a9f4B166FC4",
    "symbol": "XAVA",
    "name": "Avalaunch",
    "decimals": 18,
    "logoURI": "ipfs://QmRAMCPeGwMgpXcaJsbRyyExKBHzAW7P5kwfqR9f3o82Ep"
  },
  {
    "chainId": 56,
    "address": "0xe7c9C6bc87B86f9E5b57072F907EE6460B593924",
    "symbol": "TOWER",
    "name": "TOWER",
    "decimals": 18,
    "logoURI": "ipfs://QmUDvzhLFf3t8cjweZgFFWeqjrprqE11ZyFrySNATaTWwy"
  },
  {
    "chainId": 1,
    "address": "0x30635297E450b930f8693297eBa160D9e6c8eBcf",
    "symbol": "sUNI",
    "name": "Synthetic Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmeTNs62nYbKnWQcsTTZwHVycdPTu9SHu28eFcq8cmScd1"
  },
  {
    "chainId": 1,
    "address": "0xE276056f924E15BEac26F7b10B6D2805F9c64Bb5",
    "symbol": "FIRU",
    "name": "Firulais",
    "decimals": 9,
    "logoURI": "ipfs://QmXswbpL42WxwW2WkJNyNhtGztAzwCP9GNWfU79t1tnbye"
  },
  {
    "chainId": 1,
    "address": "0x2D80f5F5328FdcB6ECeb7Cacf5DD8AEDaEC94e20",
    "symbol": "AGA",
    "name": "AGA Token",
    "decimals": 4,
    "logoURI": "ipfs://QmNdTc7KzfCpxVF3TUinzWnbjdE4jg1KFXyzc3K85Noo67"
  },
  {
    "chainId": 1,
    "address": "0xc7Fd8Dcee4697ceef5a2fd4608a7BD6A94C77480",
    "symbol": "crCRV",
    "name": "Cream CRV",
    "decimals": 8,
    "logoURI": "ipfs://QmVmbhZc84cWxbPKEWP5tfSnF7pgupkCCFxZy7JAQ3rE1D"
  },
  {
    "chainId": 1,
    "address": "0x09ce2B746C32528B7d864A1e3979Bd97d2f095AB",
    "symbol": "DFL",
    "name": "DeFIL",
    "decimals": 18,
    "logoURI": "ipfs://QmdDqjNmEU1VLPF1bDC2abfd6ayVQ4NQhoVctGEE2NJwUh"
  },
  {
    "chainId": 1,
    "address": "0x1715AC0743102BF5Cd58EfBB6Cf2dC2685d967b6",
    "symbol": "sDOT",
    "name": "Synthetic Polkadot",
    "decimals": 18,
    "logoURI": "ipfs://QmPkM1rfiAYMu8zubrkkVox8oDjEUjZzFyzGy5NMqVXxTN"
  },
  {
    "chainId": 1,
    "address": "0x1BE66C15731788BaE659B4bdD40d4AA9B3bbca94",
    "symbol": "PDOGE",
    "name": "PolkaDoge",
    "decimals": 9,
    "logoURI": "ipfs://QmbYbwdwy283Y1asK7nd2wQRiyMa53uNnuCwKWCKHsLzfg"
  },
  {
    "chainId": 43114,
    "address": "0x5C09A9cE08C4B332Ef1CC5f7caDB1158C32767Ce",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 43114,
    "address": "0x130966628846BFd36ff31a822705796e8cb8C18D",
    "symbol": "MIM",
    "name": "Magic Internet Money",
    "decimals": 18,
    "logoURI": "ipfs://QmaNmAVohHYtCFn8H9Lk393KDyj1rhWLUamGggFXM2eGLT"
  },
  {
    "chainId": 1,
    "address": "0x80D186B4C786Ea66592b2c52e2004AB10CfE4CF3",
    "symbol": "LYNX",
    "name": "Lynx Tech",
    "decimals": 18,
    "logoURI": "ipfs://QmRsuWiZjTNTd8WrAZ6BT16W15JxrHuKprVaUmbXjsJePj"
  },
  {
    "chainId": 1,
    "address": "0x6B3595068778DD592e39A122f4f5a5cF09C90fE2",
    "symbol": "SUSHI",
    "name": "Sushi",
    "decimals": 18,
    "logoURI": "ipfs://QmT48vXNDWa4KUUk6ZsYk8Z4QwNqjk9xaqpbpQvMFz1xFC"
  },
  {
    "chainId": 1,
    "address": "0x4a9D8B8FCe0B6ec033932B13c4e24d24Dc4113cd",
    "symbol": "UDOG",
    "name": "United Doge Finance",
    "decimals": 9,
    "logoURI": "ipfs://QmfCGMuoxzyKGcxsSk42yrsUPQtgoug5kpeDVfStqN1g4z"
  },
  {
    "chainId": 56,
    "address": "0x3203c9E46cA618C8C1cE5dC67e7e9D75f5da2377",
    "symbol": "MBOX",
    "name": "MOBOX",
    "decimals": 18,
    "logoURI": "ipfs://Qmapq3q5CM5yrk9F2gWD48yf5eyizxqUhvDtST9LUqaMvF"
  },
  {
    "chainId": 56,
    "address": "0x40fEd5691e547885cABd7A2990De719DCc8497FC",
    "symbol": "SHA",
    "name": "Safe Haven",
    "decimals": 18,
    "logoURI": "ipfs://QmZXfqMBack4aDf2iSUv3F21ZjMjAhYpQvfN7u1Tw8VKDj"
  },
  {
    "chainId": 1,
    "address": "0xC0c293ce456fF0ED870ADd98a0828Dd4d2903DBF",
    "symbol": "AURA",
    "name": "Aura",
    "decimals": 18,
    "logoURI": "ipfs://QmRoL6fUzR2diwEMoqXvj6E1jQfFAc3VR1tc2Pu9ecdB6S"
  },
  {
    "chainId": 1,
    "address": "0x19D1666f543D42ef17F66E376944A22aEa1a8E46",
    "symbol": "crCOMP",
    "name": "Cream Compound",
    "decimals": 8,
    "logoURI": "ipfs://QmSawSndMER3pnh4yLBKUxtDGA82zkEyPDcrdeL8Ma9zg6"
  },
  {
    "chainId": 1,
    "address": "0x9ed7E4B1BFF939ad473dA5E7a218C771D1569456",
    "symbol": "REUNI",
    "name": "Reunit Wallet",
    "decimals": 6,
    "logoURI": "ipfs://QmbvkXoR9DDtCvJuK7zfdLWQ196Vat4t8DXaV8UN8j6XJf"
  },
  {
    "chainId": 1,
    "address": "0x05f4a42e251f2d52b8ed15E9FEdAacFcEF1FAD27",
    "symbol": "ZIL",
    "name": "Zilliqa",
    "decimals": 12,
    "logoURI": "ipfs://QmYaLRGTgwzLe7NUFGCrFsbXmDrSibus87TEhgKdAgiZar"
  },
  {
    "chainId": 1,
    "address": "0x9d1233cc46795E94029fDA81aAaDc1455D510f15",
    "symbol": "ZAI",
    "name": "Zero Collateral Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmTsHXErPsZFoeFLhkc8h1kaPPRhebioiThNb3W6p9jGQa"
  },
  {
    "chainId": 10,
    "address": "0xdFA46478F9e5EA86d57387849598dbFB2e964b02",
    "symbol": "MAI",
    "name": "MAI",
    "decimals": 18,
    "logoURI": "ipfs://QmW1f5GFZDXN98kBzESE64QrpEbNrB9RmHBUxFkiz6zNef"
  },
  {
    "chainId": 1,
    "address": "0x280f76a218DDC8d56B490B5835e251E55a2e8F8d",
    "symbol": "sUNI",
    "name": "Strike UNI",
    "decimals": 8,
    "logoURI": "ipfs://QmPrmUgkBUzyzEYXs6UMSN8QuA3rgNufZyTsHT3Dj5SRU3"
  },
  {
    "chainId": 1,
    "address": "0x3225E3C669B39C7c8B3e204a8614bB218c5e31BC",
    "symbol": "crAAVE",
    "name": "Cream AAVE",
    "decimals": 8,
    "logoURI": "ipfs://Qma2sPDf8Lbg1FNBBvJXirDJb5CfaNDhMt1fPB3SYTnYku"
  },
  {
    "chainId": 1,
    "address": "0x3496B523e5C00a4b4150D6721320CdDb234c3079",
    "symbol": "NUM",
    "name": "Numbers Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmciBuPZ2QW5TXQMiQVkiqxgkNomJaZenzxDoxnYZLL6sR"
  },
  {
    "chainId": 1,
    "address": "0xf6537FE0df7F0Cc0985Cf00792CC98249E73EFa0",
    "symbol": "GIV",
    "name": "GIV",
    "decimals": 8,
    "logoURI": "ipfs://QmPNx3vb83ZA7G7HS3eSTCrrTVmrHu95eLquzW2p1pjCEU"
  },
  {
    "chainId": 1,
    "address": "0x949D48EcA67b17269629c7194F4b727d4Ef9E5d6",
    "symbol": "MC",
    "name": "Merit Circle",
    "decimals": 18,
    "logoURI": "ipfs://QmWYsFCcKCfWMhvYWuxkHpBQF9beNC5MJmjDEnhbjuB3ba"
  },
  {
    "chainId": 10,
    "address": "0x74ccbe53F77b08632ce0CB91D3A545bF6B8E0979",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 1,
    "address": "0x21aB6c9fAC80C59D401b37cB43F81ea9DDe7Fe34",
    "symbol": "BRC",
    "name": "Baer Chain",
    "decimals": 8,
    "logoURI": "ipfs://QmbKnyhpSGDhi1xukacAWDn2edbGXeXP1HuHWsVWHveC7T"
  },
  {
    "chainId": 1,
    "address": "0x51DB5Ad35C671a87207d88fC11d593AC0C8415bd",
    "symbol": "MDA",
    "name": "Moeda Loyalty Points",
    "decimals": 18,
    "logoURI": "ipfs://QmfKxpxrPJ7XzgTvr9T9YucEK7JCpvHd6zWPKLvwsLYcqy"
  },
  {
    "chainId": 1,
    "address": "0x5c872500c00565505F3624AB435c222E558E9ff8",
    "symbol": "COT",
    "name": "CoTrader",
    "decimals": 18,
    "logoURI": "ipfs://QmSAX6SGV6oebR9DMtwECHPKh4JxEGUvYAd49vpXo8kQ4c"
  },
  {
    "chainId": 1,
    "address": "0x18aAA7115705e8be94bfFEBDE57Af9BFc265B998",
    "symbol": "AUDIO",
    "name": "Audius",
    "decimals": 18,
    "logoURI": "ipfs://QmZWwBpdazSbczP6mEYaATUkdSELFyqHnjzosatqJm4He5"
  },
  {
    "chainId": 1,
    "address": "0x630d98424eFe0Ea27fB1b3Ab7741907DFFEaAd78",
    "symbol": "PEAK",
    "name": "PEAKDEFI",
    "decimals": 8,
    "logoURI": "ipfs://QmPgpiwofrTsHtfRd9bHLSf6CCiXEqjiMFvpHN1skch6ZL"
  },
  {
    "chainId": 100,
    "address": "0xb0C5f3100A4d9d9532a4CfD68c55F1AE8da987Eb",
    "symbol": "HAUS",
    "name": "DAOhaus",
    "decimals": 18,
    "logoURI": "ipfs://QmZU6FKTUgtn1cGWV4KWbnQ5skJsbmDKodqAMLmGo7pGeH"
  },
  {
    "chainId": 1,
    "address": "0x8064d9Ae6cDf087b1bcd5BDf3531bD5d8C537a68",
    "symbol": "OBTC",
    "name": "BoringDAO BTC",
    "decimals": 18,
    "logoURI": "ipfs://QmdAPhNe9aab1X53T2tQQ7LcB63c9wiB3jq9a98MavojuD"
  },
  {
    "chainId": 1,
    "address": "0xD70240Dd62F4ea9a6A2416e0073D72139489d2AA",
    "symbol": "GLYPH",
    "name": "GLYPH Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmNUHdujzHQTcg4GVFyixAVVLNiE2MqBboYs7JPQmjfmF9"
  },
  {
    "chainId": 1,
    "address": "0xEd04915c23f00A313a544955524EB7DBD823143d",
    "symbol": "ACH",
    "name": "Alchemy Pay",
    "decimals": 8,
    "logoURI": "ipfs://QmS1WAXrydaJwyQDgXQUe1XcDmgAc79M1NMzsRgekbR9YM"
  },
  {
    "chainId": 100,
    "address": "0x712b3d230F3C1c19db860d80619288b1F0BDd0Bd",
    "symbol": "CRV",
    "name": "Curve DAO Token",
    "decimals": 18,
    "logoURI": "ipfs://QmNPpFtjU5kMFjz13Ysb3p6c2rZZQp3ysKzyXonjYCeKyT"
  },
  {
    "chainId": 1,
    "address": "0x722f97A435278B7383a1e3c47F41773bebF3232C",
    "symbol": "UCM",
    "name": "UCROWDME",
    "decimals": 18,
    "logoURI": "ipfs://QmdnCUhHbAhtxtVAW3ex4KKYjoa2VjfsUNSLygRwYSjsG6"
  },
  {
    "chainId": 56,
    "address": "0x151B1e2635A717bcDc836ECd6FbB62B674FE3E1D",
    "symbol": "vXVS",
    "name": "Venus XVS",
    "decimals": 8,
    "logoURI": "ipfs://QmVUqBxA43QRz6zkeLaTPojxB3k9N9Lt6kKD7JZyALLF6m"
  },
  {
    "chainId": 43114,
    "address": "0xC891EB4cbdEFf6e073e859e987815Ed1505c2ACD",
    "symbol": "EURC",
    "name": "EURC",
    "decimals": 6,
    "logoURI": "ipfs://QmSyyMD1CUc17Rj2Cn4vV9wSLuHsWDLN2zDBaGozC5rdaF"
  },
  {
    "chainId": 1,
    "address": "0xbf2179859fc6D5BEE9Bf9158632Dc51678a4100e",
    "symbol": "ELF",
    "name": "aelf",
    "decimals": 18,
    "logoURI": "ipfs://QmSqDf5a7nNdomWsbhbUmfdy4ftjAYEZA2RfgsCoTCa6zN"
  },
  {
    "chainId": 1,
    "address": "0x8207c1FfC5B6804F6024322CcF34F29c3541Ae26",
    "symbol": "OGN",
    "name": "Origin Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmRL9tBVUevwCPe61WqETXZLhRGQFiibxikBSCwFYjVbtn"
  },
  {
    "chainId": 1,
    "address": "0xD417144312DbF50465b1C641d016962017Ef6240",
    "symbol": "CQT",
    "name": "Covalent",
    "decimals": 18,
    "logoURI": "ipfs://QmcUCFM8Ss4NAMyRWasRiZbdVoUQAweUh8U8dbf9mt4geQ"
  },
  {
    "chainId": 1,
    "address": "0x72E5390EDb7727E3d4e3436451DADafF675dBCC0",
    "symbol": "HANU",
    "name": "Hanu Yokia",
    "decimals": 12,
    "logoURI": "ipfs://QmbS2FENWimC68QXoQwZQU9h1kC7wYamSx8uo5k3aGRZTc"
  },
  {
    "chainId": 1,
    "address": "0xf8aD7dFe656188A23e89da09506Adf7ad9290D5d",
    "symbol": "BLY",
    "name": "Blocery",
    "decimals": 18,
    "logoURI": "ipfs://QmZVw2GX9JpjdQ2W6dxpGkKQ8gTacuexmoxgvU9An4Yoh5"
  },
  {
    "chainId": 1,
    "address": "0x9B39A0B97319a9bd5fed217c1dB7b030453bac91",
    "symbol": "TCH",
    "name": "TigerCash",
    "decimals": 18,
    "logoURI": "ipfs://Qmc16BJCGk4XUhK1EZLXWtmSTZK3nzAwh4caE3Ce1Sr4a6"
  },
  {
    "chainId": 42161,
    "address": "0x9ed7E4B1BFF939ad473dA5E7a218C771D1569456",
    "symbol": "REUNI",
    "name": "Reunit Wallet",
    "decimals": 6,
    "logoURI": "ipfs://QmbvkXoR9DDtCvJuK7zfdLWQ196Vat4t8DXaV8UN8j6XJf"
  },
  {
    "chainId": 1,
    "address": "0xfF20817765cB7f73d4bde2e66e067E58D11095C2",
    "symbol": "AMP",
    "name": "Amp",
    "decimals": 18,
    "logoURI": "ipfs://QmWsHrwcEYgLhdbdtMcpTy8wXTsN2RLMztDDndu1eMaycP"
  },
  {
    "chainId": 1,
    "address": "0x865ec58b06bF6305B886793AA20A2da31D034E68",
    "symbol": "MOC",
    "name": "Moss Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmUoQPT4zvy4y7ervms7TMEXqo9Xo7bvN3Ju5946b4gaNr"
  },
  {
    "chainId": 8453,
    "address": "0x1C7a460413dD4e964f96D8dFC56E7223cE88CD85",
    "symbol": "SEAM",
    "name": "Seamless",
    "decimals": 18,
    "logoURI": "ipfs://QmWjEcdM1eqyCC6ZnMHWYSG9oWMhzBuNzoZhJUTvk1P4XM"
  },
  {
    "chainId": 1,
    "address": "0xaA17A236F2bAdc98DDc0Cf999AbB47D47Fc0A6Cf",
    "symbol": "ankrCrv",
    "name": "ankrCurve",
    "decimals": 18,
    "logoURI": "ipfs://QmaALf59vDB3orPpF2DW1gkg7ubrGTSH8Xnkx3auZvyy3p"
  },
  {
    "chainId": 42161,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmY6nVvrggMDphHtKA1FLuNNoF79W1SXYWY97Es2wr6jiR"
  },
  {
    "chainId": 1,
    "address": "0x0000000000095413afC295d19EDeb1Ad7B71c952",
    "symbol": "LON",
    "name": "Tokenlon Network",
    "decimals": 18,
    "logoURI": "ipfs://QmYvxJKT7TgVTL75LtJRzs11ugZNCZV6YfXdeFYd74249E"
  },
  {
    "chainId": 1,
    "address": "0x69bBC3F8787d573F1BBDd0a5f40C7bA0Aee9BCC9",
    "symbol": "YUP",
    "name": "Yup",
    "decimals": 18,
    "logoURI": "ipfs://QmR58B5bcLuBN8RmqYJzh7pddF2tY4zPiHLyqpZ7SamJ9f"
  },
  {
    "chainId": 1,
    "address": "0xcA75C43f8c9AfD356c585ce7AA4490B48A95C466",
    "symbol": "INARI",
    "name": "Inari",
    "decimals": 9,
    "logoURI": "ipfs://QmSEHENQq3dUit2LoGQY96eknvSQreGWThM2eFsJQ3qeGR"
  },
  {
    "chainId": 1,
    "address": "0x785c34312dfA6B74F6f1829f79ADe39042222168",
    "symbol": "BUMP",
    "name": "Bumper",
    "decimals": 18,
    "logoURI": "ipfs://QmYxQVsZwu4GYsmphLLFhzwmJvn6DbVbjeAMCedjS3sqsh"
  },
  {
    "chainId": 1,
    "address": "0x4123a133ae3c521FD134D7b13A2dEC35b56c2463",
    "symbol": "QRDO",
    "name": "Qredo",
    "decimals": 8,
    "logoURI": "ipfs://QmV3msj8wtsRkgxuKm1qyNsG2GRJL5SYau5Kkuq9xKfdxK"
  },
  {
    "chainId": 10,
    "address": "0xbFd5206962267c7b4b4A8B3D76AC2E1b2A5c4d5e",
    "symbol": "OSAK",
    "name": "Osaka Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUng1scXVJSCH1MpqK4RanCsW1aXYew8KUZphtj3WUpah"
  },
  {
    "chainId": 1,
    "address": "0xeca82185adCE47f39c684352B0439f030f860318",
    "symbol": "PERL",
    "name": "Perlin",
    "decimals": 18,
    "logoURI": "ipfs://QmQoQkzHqtsj8TG9jWba2mowpjXSY1XEnPdFkBebqaqcWz"
  },
  {
    "chainId": 1,
    "address": "0x804ea14b08dEc488e5B0bC408f23EEf107fE3717",
    "symbol": "VNM",
    "name": "Venom",
    "decimals": 18,
    "logoURI": "ipfs://QmdvnVqkj3bg57BEFK8ZpNJjibWb7qV5Bj1fZAFXFA2dBv"
  },
  {
    "chainId": 56,
    "address": "0xecA88125a5ADbe82614ffC12D0DB554E2e2867C8",
    "symbol": "vUSDC",
    "name": "Venus USDC",
    "decimals": 8,
    "logoURI": "ipfs://Qmd54DjFY5vCwpDkAfd4joLpycEgNwGNkTWbgWNouXF5wt"
  },
  {
    "chainId": 1,
    "address": "0xAec7d1069e3a914a3EB50f0BFB1796751f2ce48a",
    "symbol": "S4F",
    "name": "S4FE",
    "decimals": 18,
    "logoURI": "ipfs://QmfKU8hNykGiVMt6yrHH2X9w8ichYQ19b8bHfVuvA4TJWw"
  },
  {
    "chainId": 1,
    "address": "0x37F04d2C3AE075Fad5483bB918491F656B12BDB6",
    "symbol": "VEST",
    "name": "VestChain",
    "decimals": 8,
    "logoURI": "ipfs://QmbfTfY7t6pvP5pkph8yXAMmzHZzG1toEjtYNr5J7wek3E"
  },
  {
    "chainId": 1,
    "address": "0xfa05A73FfE78ef8f1a739473e462c54bae6567D9",
    "symbol": "LUN",
    "name": "Lunyr",
    "decimals": 18,
    "logoURI": "ipfs://QmPPxYa27X5Ktqv7M9h86535zvys6MX7nwo47Q33rUoPaH"
  },
  {
    "chainId": 137,
    "address": "0x1B815d120B3eF02039Ee11dC2d33DE7aA4a8C603",
    "symbol": "WOO",
    "name": "WOO Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdJjzzpWuzQAN6sj2j5FRy3SnFYgwMPQCx5ZNrFdVpcFt"
  },
  {
    "chainId": 1,
    "address": "0x2A8e1E676Ec238d8A992307B495b45B3fEAa5e86",
    "symbol": "OUSD",
    "name": "Origin Dollar",
    "decimals": 18,
    "logoURI": "ipfs://QmbGwuoU6Fd6WDxgZFtEKNazjb464zskSpnU4s6s8dGo2t"
  },
  {
    "chainId": 1,
    "address": "0xda23d301761E4e2bF474951f978f6DFB6F3C9F14",
    "symbol": "TKINU",
    "name": "Tsuki Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmcDyRG3T5qmigXM9kBxS57FmRVZJMixePyxAoD6TXpyNh"
  },
  {
    "chainId": 1,
    "address": "0xf3dcbc6D72a4E1892f7917b7C43b74131Df8480e",
    "symbol": "BDP",
    "name": "Big Data Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmVjLtDANWVp9MU9TyYML4YwVzT7ibP7oNAtYx99Wshzfc"
  },
  {
    "chainId": 1,
    "address": "0x653430560bE843C4a3D143d0110e896c2Ab8ac0D",
    "symbol": "MOF",
    "name": "Molecular Future",
    "decimals": 16,
    "logoURI": "ipfs://QmSsNtx5eB3nyeqpU89QR8oai6ZNYvMEycWbNAnMAv5CN2"
  },
  {
    "chainId": 1,
    "address": "0x4d224452801ACEd8B2F0aebE155379bb5D594381",
    "symbol": "APE",
    "name": "ApeCoin",
    "decimals": 18,
    "logoURI": "ipfs://QmcrCvzkazoVbVtNYZcWATusYiK3CXv8WV2wMWb4cCNRN4"
  },
  {
    "chainId": 56,
    "address": "0x2eD9a5C8C13b93955103B9a7C167B67Ef4d568a3",
    "symbol": "MASK",
    "name": "Mask Network",
    "decimals": 18,
    "logoURI": "ipfs://QmZd3hATUb7nyMmmwVGkuMhkBQ97WgKRNv3Kgv3JitR6s4"
  },
  {
    "chainId": 1,
    "address": "0x9a7a4C141a3BCCE4A31e42C1192Ac6Add35069b4",
    "symbol": "XMM",
    "name": "Momentum",
    "decimals": 10,
    "logoURI": "ipfs://QmcVjFFQAqp3JwXKpEBM5X92tJHFZxd4t73GU5hnLJaEca"
  },
  {
    "chainId": 1,
    "address": "0x9f8F72aA9304c8B593d555F12eF6589cC3A579A2",
    "symbol": "MKR",
    "name": "Maker",
    "decimals": 18,
    "logoURI": "ipfs://QmPwdJwZoBDHWVDTxQmnYzCiYQkToMzunGbCryjUgqVxbc"
  },
  {
    "chainId": 1,
    "address": "0xC5bBaE50781Be1669306b9e001EFF57a2957b09d",
    "symbol": "GTO",
    "name": "Gifto",
    "decimals": 5,
    "logoURI": "ipfs://QmeSTjCWbDow8EmqHboRUAXSz9GhZadGthjgDgJnVYYLPa"
  },
  {
    "chainId": 1,
    "address": "0xb4bebD34f6DaaFd808f73De0d10235a92Fbb6c3D",
    "symbol": "YETI",
    "name": "Yearn Ecosystem Token Index",
    "decimals": 18,
    "logoURI": "ipfs://QmeiPPTviJFrkcV9g8SJaTtMtvVUeScLZKS9WnaxhJ5j51"
  },
  {
    "chainId": 1,
    "address": "0xCbfef8fdd706cde6F208460f2Bf39Aa9c785F05D",
    "symbol": "KINE",
    "name": "KINE",
    "decimals": 18,
    "logoURI": "ipfs://QmP2vS7zfaH5HUMs8Swfh1yu7jYRu84a53k3kJ1LyZ634T"
  },
  {
    "chainId": 1,
    "address": "0x92CC36D66e9d739D50673d1f27929a371FB83a67",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 1,
    "address": "0x256845e721C0c46d54E6afBD4FA3B52CB72353EA",
    "symbol": "UNIUSD",
    "name": "UniDollar",
    "decimals": 18,
    "logoURI": "ipfs://QmRrjb7RdBKfuvXW9kij64yZHP99RuK8x94XFjB5Nuv5cG"
  },
  {
    "chainId": 56,
    "address": "0xe05A08226c49b636ACf99c40Da8DC6aF83CE5bB3",
    "symbol": "ankrETH",
    "name": "Ankr Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://Qmbe4jVgB3fQZeJa166jodSYNaK9pq9CfVUZrnvvJFfL5h"
  },
  {
    "chainId": 1,
    "address": "0x9Ed8e7C9604790F7Ec589F99b94361d8AAB64E5E",
    "symbol": "UNISTAKE",
    "name": "Unistake",
    "decimals": 18,
    "logoURI": "ipfs://QmbZUgYddDvCMB4mxGSxvgDYQrGoabCXooUF1ZSpRaoUAQ"
  },
  {
    "chainId": 1,
    "address": "0x163f8C2467924be0ae7B5347228CABF260318753",
    "symbol": "WLD",
    "name": "Worldcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmU3gcamx4qmHv7eajFssTy3XJniD5G3ZJ9itZzpaLYoSV"
  },
  {
    "chainId": 1,
    "address": "0x476c5E26a75bd202a9683ffD34359C0CC15be0fF",
    "symbol": "SRM",
    "name": "Serum",
    "decimals": 6,
    "logoURI": "ipfs://QmQmD8Vsfu9sH1UHu1Cjpi6opmoW7AWNnZ1VAAvfbYNC8T"
  },
  {
    "chainId": 1,
    "address": "0xb892249939AdBf6D7851864CA9A5c7D2d537af97",
    "symbol": "GSHIBA",
    "name": "Gambler Shiba",
    "decimals": 18,
    "logoURI": "ipfs://Qmdwhn64dFg7uyporce8H99S4awuFPZ9xYv5MEkcFY7DEU"
  },
  {
    "chainId": 1,
    "address": "0xd2d6158683aeE4Cc838067727209a0aAF4359de3",
    "symbol": "BNTY",
    "name": "Bounty0x",
    "decimals": 18,
    "logoURI": "ipfs://QmZpjQQu9DfCmLjd7k7GLY9Uw3ukYJSCSE3gnwCcSdgjZ2"
  },
  {
    "chainId": 1,
    "address": "0x49184E6dAe8C8ecD89d8Bdc1B950c597b8167c90",
    "symbol": "LIBERTAS",
    "name": "Libertas",
    "decimals": 2,
    "logoURI": "ipfs://QmdadkMLqZdrSWNrairTp6bo1Kp75zL2YPv6KKspPwbT1t"
  },
  {
    "chainId": 1,
    "address": "0xe25b0BBA01Dc5630312B6A21927E578061A13f55",
    "symbol": "SHIP",
    "name": "ShipChain",
    "decimals": 18,
    "logoURI": "ipfs://QmZfq8vZCW8N8aw1uJ3RbtsJZNoWh96Sqzox7UPZqriS81"
  },
  {
    "chainId": 1,
    "address": "0x943c5F4F54509d1e78B1fCD93B92c43ce83d3141",
    "symbol": "AMPH",
    "name": "Amphora Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmVnfM7LLCJfUv5n7ZwxzaChBiRkKqQs5wgC3aFD7fgiVq"
  },
  {
    "chainId": 1,
    "address": "0xf45f6c8bB3d77eA762175b8f7ca4D251941649fA",
    "symbol": "LEMD",
    "name": "Lemond",
    "decimals": 18,
    "logoURI": "ipfs://QmTgphJ2p2Qano5nXhnPdKeuFM72mZbhHyhS7w74AuLHjK"
  },
  {
    "chainId": 1,
    "address": "0x1B2a76Da77D03b7FC21189d9838f55Bd849014aF",
    "symbol": "DFC",
    "name": "DeFiScale",
    "decimals": 8,
    "logoURI": "ipfs://QmP3aLRoxFW9K76xGYYAjePx9kaYExjJmLaeiWY6w7N3dS"
  },
  {
    "chainId": 250,
    "address": "0x9E73F99EE061C8807F69f9c6CCc44ea3d8c373ee",
    "symbol": "frxETH",
    "name": "Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmQqQhtwApaT4X9FAaAzBRvyeE2iaSDH4ahD6exWXcszZU"
  },
  {
    "chainId": 1,
    "address": "0xfFffFffF2ba8F66D4e51811C5190992176930278",
    "symbol": "COMBO",
    "name": "Furucombo",
    "decimals": 18,
    "logoURI": "ipfs://QmdNni3K958dC66ivU33AjAu23yCSzDAyXTkgwEP6EghDM"
  },
  {
    "chainId": 1,
    "address": "0xC72ED4445B3fe9f0863106E344E241530d338906",
    "symbol": "MLC",
    "name": "FLOGmall",
    "decimals": 18,
    "logoURI": "ipfs://QmYbbsAJQ2R8cgQicpuXXx8nV6dF3ftuw22Dmq2jTwEwVu"
  },
  {
    "chainId": 324,
    "address": "0xD0eA21ba66B67bE636De1EC4bd9696EB8C61e9AA",
    "symbol": "OT",
    "name": "Onchain Trade",
    "decimals": 18,
    "logoURI": "ipfs://QmNRsdAqQrgJMwPN29aXHuWDnGyYF9w4HvQF3gCJeMHcWz"
  },
  {
    "chainId": 1,
    "address": "0x9aD45E692350BA56e2583671C4Bd76C5331a249e",
    "symbol": "PeepsPay",
    "name": "PeepsPay",
    "decimals": 9,
    "logoURI": "ipfs://QmVLPefg6VqVNEWwkdDgtFWbobjGQz3uRcLHnrgNmzEid3"
  },
  {
    "chainId": 1,
    "address": "0x846C66cf71C43f80403B51fE3906B3599D63336f",
    "symbol": "PMA",
    "name": "PumaPay",
    "decimals": 18,
    "logoURI": "ipfs://Qmer6xJ8NeZ6gBYFVSe5TQLhgRY8EBiv5aYYEz9uVzCFHD"
  },
  {
    "chainId": 1,
    "address": "0x75231F58b43240C9718Dd58B4967c5114342a86c",
    "symbol": "OKB",
    "name": "OKB",
    "decimals": 18,
    "logoURI": "ipfs://QmaGqT6SFuUxGQuVNWEgPbXnNa5iTqAWDpiPxUBPrf8w2o"
  },
  {
    "chainId": 1,
    "address": "0xB72B31907C1C95F3650b64b2469e08EdACeE5e8F",
    "symbol": "vBZRX",
    "name": "bZx Vesting Token",
    "decimals": 18,
    "logoURI": "ipfs://QmYySgHorsUsCUch9F75MF76SiEAH6YAVJr4nHzEPpj3yE"
  },
  {
    "chainId": 56,
    "address": "0x95c78222B3D6e262426483D42CfA53685A67Ab9D",
    "symbol": "vBUSD",
    "name": "Venus BUSD",
    "decimals": 8,
    "logoURI": "ipfs://QmXrrGZJptUEsV1Kf2cuE5LNAEU1LsBXawtqJYEPVoArgo"
  },
  {
    "chainId": 1,
    "address": "0x13339fD07934CD674269726EdF3B5ccEE9DD93de",
    "symbol": "CUR",
    "name": "Curio",
    "decimals": 18,
    "logoURI": "ipfs://QmckooqCrRpp5bbzxPcuT1S3LsxbE1HCudVaqrcXc1mMTT"
  },
  {
    "chainId": 1,
    "address": "0x4Cf89ca06ad997bC732Dc876ed2A7F26a9E7f361",
    "symbol": "MYST",
    "name": "Mysterium",
    "decimals": 18,
    "logoURI": "ipfs://QmUAiqZqYheHJuxzPeDfAGTWGQsZkMKoWXsJtH6pgg8otP"
  },
  {
    "chainId": 1,
    "address": "0x43044f861ec040DB59A7e324c40507adDb673142",
    "symbol": "CAP",
    "name": "Cap",
    "decimals": 18,
    "logoURI": "ipfs://QmVohHQPQpYubek9wmkRbJ1oNo9zbBeYLcB1SoHKVVnjYN"
  },
  {
    "chainId": 1,
    "address": "0x6D6fc83225AD1b8373357b10d61bA6d77090C5a6",
    "symbol": "NAUINU",
    "name": "Naughty Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmV3kH2af1S4d8RxHWSmuiGufMwoeYRxZLts66YLuDXauW"
  },
  {
    "chainId": 1,
    "address": "0xA9859874e1743A32409f75bB11549892138BBA1E",
    "symbol": "iETH",
    "name": "Synthetic Inverse Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmNW4mzHpdb2qFgwEdsLwyv1h3vLv1pMFyagjN5nF63ZAW"
  },
  {
    "chainId": 1,
    "address": "0x3757232B55E60da4A8793183aC030CfCE4c3865d",
    "symbol": "YDR",
    "name": "YDragon",
    "decimals": 18,
    "logoURI": "ipfs://QmePEKB4JeJ4i4D7sAvBeb4KPRk1PxmtqfWNb2TvcmzjPo"
  },
  {
    "chainId": 1,
    "address": "0x7B0C06043468469967DBA22d1AF33d77d44056c8",
    "symbol": "MRPH",
    "name": "Morpheus.Network",
    "decimals": 4,
    "logoURI": "ipfs://Qmf1SF7kYtZw8AsyDBuz23FFRHzsvvMmicDxyqh2Hzma8c"
  },
  {
    "chainId": 1,
    "address": "0x6cAcDB97e3fC8136805a9E7c342d866ab77D0957",
    "symbol": "SWPR",
    "name": "Swapr",
    "decimals": 18,
    "logoURI": "ipfs://QmXeUKr6JoBmNnyerR2pLMRJqM1Vibo7DjGaBFpjnpAMvq"
  },
  {
    "chainId": 1,
    "address": "0x2B1Fe2cea92436E8C34B7C215Af66Aaa2932a8b2",
    "symbol": "SAHU",
    "name": "Sakhalin Husky",
    "decimals": 9,
    "logoURI": "ipfs://QmPCr5CqgzDP8gMNPLQBE9raGvzE9hbk66tAdMtxsVfrix"
  },
  {
    "chainId": 1,
    "address": "0x918dA91Ccbc32B7a6A0cc4eCd5987bbab6E31e6D",
    "symbol": "sTSLA",
    "name": "Synthetic Tesla",
    "decimals": 18,
    "logoURI": "ipfs://QmWbuSqVkZ5wo76WA1BbTak6eBMhjZzkhC1VZXqpm2aZ9j"
  },
  {
    "chainId": 1,
    "address": "0x2cAd4991f62fc6Fcd8EC219f37E7DE52B688B75A",
    "symbol": "SCHA",
    "name": "Schain Wallet",
    "decimals": 0,
    "logoURI": "ipfs://QmenxWqKfXxLcjsnrkL2ZTButCx6T881FjvVNK1fBWDCD3"
  },
  {
    "chainId": 1,
    "address": "0x88EF27e69108B2633F8E1C184CC37940A075cC02",
    "symbol": "DEGO",
    "name": "Dego Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmcJiJseBZVHQ6iJwF4FNQnc7Frq4SJJPJXTyLrmQec8z9"
  },
  {
    "chainId": 10,
    "address": "0xfDeFFc7Ad816BF7867C642dF7eBC2CC5554ec265",
    "symbol": "BeVELO",
    "name": "Beefy Velo",
    "decimals": 18,
    "logoURI": "ipfs://Qmcjev8T5UytGgkFHfK2NKoz5SFy23gKmxuyoNzMjYcFzQ"
  },
  {
    "chainId": 100,
    "address": "0x4291F029B9e7acb02D49428458cf6fceAC545f81",
    "symbol": "WATER",
    "name": "1Hive Water",
    "decimals": 18,
    "logoURI": "ipfs://QmXdSXfCKYufdPTRXupDDim797sdfeu3iKixWmwX7uj1zi"
  },
  {
    "chainId": 56,
    "address": "0xC17c30e98541188614dF99239cABD40280810cA3",
    "symbol": "RISE",
    "name": "EverRise",
    "decimals": 18,
    "logoURI": "ipfs://QmU6dKNHUySjmVh9FJFMT1cRZ54aoQ7fpJ4q2a6qFPHsbU"
  },
  {
    "chainId": 100,
    "address": "0x0939a7c3f8D37C1ce67fAda4963aE7E0bd112ff3",
    "symbol": "VITA",
    "name": "VitaDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmXjsi9mvEsTP1AoSCjdyZuE44WfDtcbMNHxZUkRXhLqce"
  },
  {
    "chainId": 1,
    "address": "0x16980b3B4a3f9D89E33311B5aa8f80303E5ca4F8",
    "symbol": "KEX",
    "name": "KIRA Network",
    "decimals": 6,
    "logoURI": "ipfs://QmWas8fH4Th2k1n1jUXQLSzYxnRKttvz5JuGhXv8nB8hBL"
  },
  {
    "chainId": 1,
    "address": "0x1FF8CDB51219a8838b52E9cAc09b71e591BC998e",
    "symbol": "crBUSD",
    "name": "Cream Binance USD",
    "decimals": 8,
    "logoURI": "ipfs://QmXuYN57yqAKD2eCMiAERs4nj6v2bRKvMPrEuXwtR4NC81"
  },
  {
    "chainId": 1,
    "address": "0x18a3563c21062897950BB09339C82b9686a35667",
    "symbol": "ENXS",
    "name": "EtherNexus",
    "decimals": 9,
    "logoURI": "ipfs://QmXW7B4uqUAbxKRXwXtH1QEXU3hJWDgjEYuYcd6Te5c7yL"
  },
  {
    "chainId": 1,
    "address": "0x900dB999074d9277c5DA2A43F252D74366230DA0",
    "symbol": "GIV",
    "name": "Giveth",
    "decimals": 18,
    "logoURI": "ipfs://QmfBkYQZVHewLGqUnDDSGmybtiCKSk4bkAoQjxqwdRxZiQ"
  },
  {
    "chainId": 137,
    "address": "0x1379E8886A944d2D9d440b3d88DF536Aea08d9F3",
    "symbol": "MYST",
    "name": "Mysterium",
    "decimals": 18,
    "logoURI": "ipfs://QmfKqbcvm76L68VkUqWxLF5TWJfnstXcTMTYEnKS8Lk4jj"
  },
  {
    "chainId": 56,
    "address": "0x1935C03bB7DbC3623bD992CE599bC26bD30A43EB",
    "symbol": "FS",
    "name": "FantomStarter",
    "decimals": 18,
    "logoURI": "ipfs://QmeqPPX87Xj5FwRa18FNg5iEvtB3rDfQ5Gp3Yk8yb35C5r"
  },
  {
    "chainId": 56,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmY6nVvrggMDphHtKA1FLuNNoF79W1SXYWY97Es2wr6jiR"
  },
  {
    "chainId": 1,
    "address": "0x1337DEF16F9B486fAEd0293eb623Dc8395dFE46a",
    "symbol": "ARMOR",
    "name": "Armor",
    "decimals": 18,
    "logoURI": "ipfs://QmWZWyYaY1tzsxQoEpaUNeU3FkLoETwncG8qLYMRXKPAgm"
  },
  {
    "chainId": 1,
    "address": "0xBb1f24C0c1554b9990222f036b0AaD6Ee4CAec29",
    "symbol": "SOUL",
    "name": "CryptoSoul",
    "decimals": 18,
    "logoURI": "ipfs://QmR1d5bL3rx5idA879xNDuA6tsHo5KnkbYm6zJ7Uq9AuAE"
  },
  {
    "chainId": 1,
    "address": "0x2F57430a6ceDA85a67121757785877b4a71b8E6D",
    "symbol": "DFP2",
    "name": "DefiPlaza",
    "decimals": 18,
    "logoURI": "ipfs://QmNsW3u6wrYhs2HzZVW5ZNtp6s7AiPuTjHqHCn9sE7RHXw"
  },
  {
    "chainId": 1,
    "address": "0x5e74C9036fb86BD7eCdcb084a0673EFc32eA31cb",
    "symbol": "sETH",
    "name": "Synthetic Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmPMPctu8ZpantzEjiGfjBJXxAX8gPL7W5xAvF9LDt1pvM"
  },
  {
    "chainId": 1,
    "address": "0xB98d4C97425d9908E66E53A6fDf673ACcA0BE986",
    "symbol": "ABT",
    "name": "ArcBlock",
    "decimals": 18,
    "logoURI": "ipfs://QmSG8bjXUXF5FMGoxonen4B1PozJPtf11fFN8Lj2LoWqvB"
  },
  {
    "chainId": 1,
    "address": "0x18A908eD663823C908A900b934D6249d4befbE44",
    "symbol": "sBUSD",
    "name": "Strike BUSD",
    "decimals": 8,
    "logoURI": "ipfs://QmfB27GDcU3r15h1cX2abL7hQ7c6iEhve3SUvGWvBtLY79"
  },
  {
    "chainId": 1,
    "address": "0x580c8520dEDA0a441522AEAe0f9F7A5f29629aFa",
    "symbol": "DAWN",
    "name": "Dawn Protocol",
    "decimals": 18,
    "logoURI": "ipfs://Qmez8SftYbG3HYtyZL8u536iszdYXrrLu4msdQQoRVrXtm"
  },
  {
    "chainId": 100,
    "address": "0xb7D311E2Eb55F2f68a9440da38e7989210b9A05e",
    "symbol": "STAKE",
    "name": "STAKE",
    "decimals": 18,
    "logoURI": "ipfs://QmWT1MhsiZ1SFuBQj5taSaW73tvbn2Prqcnm1NXHpcMRCb"
  },
  {
    "chainId": 1,
    "address": "0x51BC0DeaF7bBE82bC9006b0c3531668a4206D27F",
    "symbol": "RAKU",
    "name": "RAKUN",
    "decimals": 18,
    "logoURI": "ipfs://QmdHWVL5vd9Vh3dw57Jcd5vvf2HqdFX3HedDMrDpPGBGCK"
  },
  {
    "chainId": 250,
    "address": "0xb7C2ddB1EBAc1056231ef22c1b0A13988537a274",
    "symbol": "TAROT",
    "name": "Tarot",
    "decimals": 18,
    "logoURI": "ipfs://QmenPX7UnjGeCrfGjsNUWX6tJj3o6QXXw9co5WL8aNP9UF"
  },
  {
    "chainId": 1,
    "address": "0x69D9905B2e5f6F5433212b7F3c954433F23C1572",
    "symbol": "OOKS",
    "name": "Onooks",
    "decimals": 18,
    "logoURI": "ipfs://QmZDAyUwMGEP9zUuMqs1Rnj2EzdCFsLtYVsP2BbHSCssvw"
  },
  {
    "chainId": 137,
    "address": "0x2F6F07CDcf3588944Bf4C42aC74ff24bF56e7590",
    "symbol": "STG",
    "name": "Stargate Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUpS8PCxJpsC8Ly8btjp7LXKueEdmWdZJUHk6e9zhzhnK"
  },
  {
    "chainId": 1,
    "address": "0x9189c499727f88F8eCC7dC4EEA22c828E6AaC015",
    "symbol": "iMKR",
    "name": "Fulcrum MKR iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmSdvfpZhy4DV4kh2i59NGQ7Kpr7wgsBndq5taWkYvK1NJ"
  },
  {
    "chainId": 1,
    "address": "0xe96EdD48CF0c6e930cE55F171A721017b28E0F08",
    "symbol": "NexusAI",
    "name": "NexusAI",
    "decimals": 9,
    "logoURI": "ipfs://QmYN7Xe1AFZuEMGcSgZMb2WS6NfbYEseAh6YVCiwo915LT"
  },
  {
    "chainId": 56,
    "address": "0x7c869b5A294b1314E985283d01C702B62224a05f",
    "symbol": "jCHF",
    "name": "Jarvis Synthetic Swiss Franc",
    "decimals": 18,
    "logoURI": "ipfs://QmQYgf7dnsrcJPGjDgm1sQZ1n3Ugbk3cAvptuu7SV43KgQ"
  },
  {
    "chainId": 1,
    "address": "0x96F6eF951840721AdBF46Ac996b59E0235CB985C",
    "symbol": "USDY",
    "name": "Ondo US Dollar Yield",
    "decimals": 18,
    "logoURI": "ipfs://QmYzRhqDFYy9f5C8JgYsA2sWNZdwukxr2qsgrbSmhxkV4o"
  },
  {
    "chainId": 1,
    "address": "0x115eC79F1de567eC68B7AE7eDA501b406626478e",
    "symbol": "CRE",
    "name": "Carry",
    "decimals": 18,
    "logoURI": "ipfs://QmfUihmQgAJ1nQZnJwzZmVNB3xprqYPJfDWvowCKDeFaDC"
  },
  {
    "chainId": 42161,
    "address": "0x74ccbe53F77b08632ce0CB91D3A545bF6B8E0979",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 1,
    "address": "0x85c4EdC43724e954e5849cAAab61A26a9CB65F1B",
    "symbol": "BBCH",
    "name": "Binance Wrapped BCH",
    "decimals": 8,
    "logoURI": "ipfs://Qmb6EW2YSNCH4w2saZ2MS3MqjGrQKkn9WR8HcfHb11NA5d"
  },
  {
    "chainId": 1,
    "address": "0x8dfc8cc3201425669FaE803e1eB125cddd4189eC",
    "symbol": "OKAGE",
    "name": "Okage Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmNpnay866iZ4yjSe1CcRnH1fJvbvQKJkj4DPmqFR7MiHf"
  },
  {
    "chainId": 10,
    "address": "0x217D47011b23BB961eB6D93cA9945B7501a5BB11",
    "symbol": "THALES",
    "name": "Thales",
    "decimals": 18,
    "logoURI": "ipfs://QmeGZ9fRMeNJP9MnMNDQSBGdQzAJL5GUAWWqTd254NUuQJ"
  },
  {
    "chainId": 56,
    "address": "0x76A797A59Ba2C17726896976B7B3747BfD1d220f",
    "symbol": "TONCOIN",
    "name": "Wrapped TON Coin",
    "decimals": 9,
    "logoURI": "ipfs://QmRwD6j7bvxWU6dhvYR84KaW1xXWgu5ZwqpYJbtZsJri6z"
  },
  {
    "chainId": 1,
    "address": "0x4B4701f3f827E1331fb22FF8e2BEaC24b17Eb055",
    "symbol": "DISTX",
    "name": "DistX",
    "decimals": 18,
    "logoURI": "ipfs://QmVWindfxKK3ZJ192dRfLJmxdtTRkyGmkJ9mwDKjtkER9C"
  },
  {
    "chainId": 1,
    "address": "0xe8e8486228753E01Dbc222dA262Aa706Bd67e601",
    "symbol": "WEB3",
    "name": "Arch Ethereum Web3",
    "decimals": 18,
    "logoURI": "ipfs://Qmdup9K1iKcZyznP4E2PZc8mVCUbqpv7CGS4QQVgwWib1m"
  },
  {
    "chainId": 1,
    "address": "0x1829aA045E21E0D59580024A951DB48096e01782",
    "symbol": "FXT",
    "name": "FuzeX",
    "decimals": 18,
    "logoURI": "ipfs://QmQfxhxUE8JnpwsJVRsQeEDD8US9Q1nGkAFfzPfowRfTS5"
  },
  {
    "chainId": 56,
    "address": "0x650b940a1033B8A1b1873f78730FcFC73ec11f1f",
    "symbol": "vLINK",
    "name": "Venus LINK",
    "decimals": 8,
    "logoURI": "ipfs://QmQXfFD1NnN1WQrS64xpknpFpVxXfps7Z7bJ25kDWPy371"
  },
  {
    "chainId": 1,
    "address": "0xE884cc2795b9c45bEeac0607DA9539Fd571cCF85",
    "symbol": "ULT",
    "name": "Ultiledger",
    "decimals": 18,
    "logoURI": "ipfs://QmWg6rsVQ7UtXpNTVF2u2YU6cQw6x8zRoK3MyNuKUcwBjF"
  },
  {
    "chainId": 250,
    "address": "0xfB98B335551a418cD0737375a2ea0ded62Ea213b",
    "symbol": "miMATIC",
    "name": "MAI",
    "decimals": 18,
    "logoURI": "ipfs://QmW1f5GFZDXN98kBzESE64QrpEbNrB9RmHBUxFkiz6zNef"
  },
  {
    "chainId": 43114,
    "address": "0x62edc0692BD897D2295872a9FFCac5425011c661",
    "symbol": "GMX",
    "name": "GMX",
    "decimals": 18,
    "logoURI": "ipfs://QmaJyhkXJfRPxAoSYn6mXvi535zFvhzj166uB7ssXHNcUq"
  },
  {
    "chainId": 56,
    "address": "0x9C65AB58d8d978DB963e63f2bfB7121627e3a739",
    "symbol": "MDX",
    "name": "Mdex",
    "decimals": 18,
    "logoURI": "ipfs://QmSsXejTPYQgFgL7RSS8By6ec2QKfX1BdKW8V5GYGcb2aC"
  },
  {
    "chainId": 42161,
    "address": "0xf18C263EC50CC211EF3F172228549B6618F10613",
    "symbol": "COLLAB",
    "name": "Collab.Land",
    "decimals": 18,
    "logoURI": "ipfs://QmUmjee4H8CdtkHJrCXGCLXiLCoasYcs7fJB6hr75Wz2LX"
  },
  {
    "chainId": 10,
    "address": "0xe453d6649643F1F460C371dC3D1da98F7922fe51",
    "symbol": "FUSE",
    "name": "Fuse",
    "decimals": 18,
    "logoURI": "ipfs://QmYiyLPWFmwxTv43Jsx4qpPcKu2YmqGX6XxwvBjTRVPKhC"
  },
  {
    "chainId": 1,
    "address": "0x48C3399719B582dD63eB5AADf12A40B4C3f52FA2",
    "symbol": "SWISE",
    "name": "StakeWise",
    "decimals": 18,
    "logoURI": "ipfs://QmWWwqSTweQtGHsWvBaoAnfTDuzspwR811Moyw2YnfAkRH"
  },
  {
    "chainId": 1,
    "address": "0xEEaa34aF95b034BaDa4baF565063132C765b1fA5",
    "symbol": "OLCF",
    "name": "OLCF",
    "decimals": 18,
    "logoURI": "ipfs://QmQ9Jj5mm54oSEwE2FJknrfteXwLLv5PVdYREPvAMnYVms"
  },
  {
    "chainId": 250,
    "address": "0x7f620d7d0b3479b1655cEFB1B0Bc67fB0EF4E443",
    "symbol": "nICE",
    "name": "Staked Ice",
    "decimals": 18,
    "logoURI": "ipfs://QmWYeq6cyC6Cthagmqn9dCst62t2vuCu3Fm8UJi85VYFpC"
  },
  {
    "chainId": 1,
    "address": "0xbf900809f4C73e5a3476eb183d8b06a27e61F8E5",
    "symbol": "VNL",
    "name": "Vanilla",
    "decimals": 12,
    "logoURI": "ipfs://QmWDLHDpqJ79iy2fwu5ZuJxguUo6QNCpcTkaZhMLQ2qs5Z"
  },
  {
    "chainId": 42161,
    "address": "0xFEa7a6a0B346362BF88A9e4A88416B77a57D6c2A",
    "symbol": "MIM",
    "name": "Magic Internet Money",
    "decimals": 18,
    "logoURI": "ipfs://QmaNmAVohHYtCFn8H9Lk393KDyj1rhWLUamGggFXM2eGLT"
  },
  {
    "chainId": 1,
    "address": "0x957c30aB0426e0C93CD8241E2c60392d08c6aC8e",
    "symbol": "MOD",
    "name": "Modum",
    "decimals": 0,
    "logoURI": "ipfs://QmcYPPqJadNufT7Vpgw3bGi4AbcfT7k5LzibW1p1ESC8ET"
  },
  {
    "chainId": 1,
    "address": "0x5Ca135cB8527d76e932f34B5145575F9d8cbE08E",
    "symbol": "FPI",
    "name": "Frax Price Index",
    "decimals": 18,
    "logoURI": "ipfs://QmTf1B4mMgr9bdBH4KGWWnew9SkJF4DK9M7FAEnyvr9tKr"
  },
  {
    "chainId": 1,
    "address": "0xB705268213D593B8FD88d3FDEFF93AFF5CbDcfAE",
    "symbol": "IDEX",
    "name": "IDEX",
    "decimals": 18,
    "logoURI": "ipfs://QmawT5xDG7Rr6GLRVk8NDW9HwejiHQe8phdoq5R7TR8Cwg"
  },
  {
    "chainId": 1,
    "address": "0x4a6058666cf1057eaC3CD3A5a614620547559fc9",
    "symbol": "BBK",
    "name": "Brickblock",
    "decimals": 18,
    "logoURI": "ipfs://Qme2KTdyfNAdhrwQkwUgrBsX3Xw5sMEt33txrTW7beEX51"
  },
  {
    "chainId": 1,
    "address": "0xF5DCe57282A584D2746FaF1593d3121Fcac444dC",
    "symbol": "cSAI",
    "name": "Compound Sai",
    "decimals": 8,
    "logoURI": "ipfs://QmVHEHmSPnxaaXEVtjrfm4B9uemuJhDZ2mfA81dERE4gXn"
  },
  {
    "chainId": 1,
    "address": "0x897Abf83f0C44b86cb67ec56c006c00d56659517",
    "symbol": "ARTEON",
    "name": "Arteon",
    "decimals": 18,
    "logoURI": "ipfs://QmSPzZoTVAbUJoapX4APyQYcsh7WQYbdEsyh5wPNLv4xfs"
  },
  {
    "chainId": 1,
    "address": "0x62786Eeacc9246b4018e0146cb7a3efeACD9459D",
    "symbol": "LESS",
    "name": "Less Network",
    "decimals": 18,
    "logoURI": "ipfs://QmeRVznaEV3MnPLrjXhrYZCV7rkt9tft7VKhJEDCPXmuKS"
  },
  {
    "chainId": 1,
    "address": "0xE17f017475a709De58E976081eB916081ff4c9d5",
    "symbol": "RMPL",
    "name": "RMPL",
    "decimals": 9,
    "logoURI": "ipfs://Qmecs5GQzzifmyNGCVL4WZVzA9WeNrEncw2CDoSPbrt2EX"
  },
  {
    "chainId": 42161,
    "address": "0x6c84a8f1c29108F47a79964b5Fe888D4f4D0dE40",
    "symbol": "tBTC",
    "name": "tBTC",
    "decimals": 18,
    "logoURI": "ipfs://QmRMESJHxFZWSS5KuMj5fW7SqEdkVvEATYVWkRRk6y2VeX"
  },
  {
    "chainId": 1,
    "address": "0x3F3B3B269d9f7088B022290906acff8710914be1",
    "symbol": "sLINK",
    "name": "Strike LINK",
    "decimals": 8,
    "logoURI": "ipfs://QmbjDyKGdyYjaSfsKcL2uhN4cY3XvhUN2zms8uvZorHDz6"
  },
  {
    "chainId": 1,
    "address": "0x8B40761142B9aa6dc8964e61D0585995425C3D94",
    "symbol": "TRIO",
    "name": "Tripio",
    "decimals": 18,
    "logoURI": "ipfs://QmYKz3e2UzHdEGJhu3Q8tijqcCsrzDWue241TUTvtH2LnM"
  },
  {
    "chainId": 137,
    "address": "0xBD1fe73e1f12bD2bc237De9b626F056f21f86427",
    "symbol": "jMXN",
    "name": "Jarvis Synthetic Mexican Peso",
    "decimals": 18,
    "logoURI": "ipfs://QmUcsBRdQM5HQKoiKMMFCjhTpYuggPJXF72pgygunCH4Tc"
  },
  {
    "chainId": 1,
    "address": "0xdF574c24545E5FfEcb9a659c229253D4111d87e1",
    "symbol": "HUSD",
    "name": "HUSD",
    "decimals": 8,
    "logoURI": "ipfs://QmbJh5GxoM1BHtovY56MC7pz9zqALddYdgxfA42tebkNxt"
  },
  {
    "chainId": 1,
    "address": "0x2604FA406Be957E542BEb89E6754fCdE6815e83f",
    "symbol": "PKT",
    "name": "Playkey",
    "decimals": 18,
    "logoURI": "ipfs://QmdESRiDtSXDKj69GJbVudCrfVdVJkUCtNeThuDTnMAkxE"
  },
  {
    "chainId": 1,
    "address": "0xa704fCe7b309Ec09DF16e2F5Ab8cAf6Fe8A4BAA9",
    "symbol": "AGRI",
    "name": "AgriChain",
    "decimals": 18,
    "logoURI": "ipfs://QmXX3pJjaQG3RufVKuwsQAAaH6iYMPAncQWDxk628rxNxq"
  },
  {
    "chainId": 1,
    "address": "0xB16e967ff83DE3F1e9FCeAfbc2C28c1c5c56eF91",
    "symbol": "PDOG",
    "name": "Polkadog",
    "decimals": 18,
    "logoURI": "ipfs://QmTUQKfUgSP1V1KvYky2Mn5u89uiHaJPaqXP8paudGYCpt"
  },
  {
    "chainId": 1,
    "address": "0xc7283b66Eb1EB5FB86327f08e1B5816b0720212B",
    "symbol": "TRIBE",
    "name": "Tribe",
    "decimals": 18,
    "logoURI": "ipfs://QmPwBcAsUcpFsfiSJrQSMXa8zZ8ma35Uzgu78NC48c6764"
  },
  {
    "chainId": 1,
    "address": "0x50DE6856358Cc35f3A9a57eAAA34BD4cB707d2cd",
    "symbol": "RAZOR",
    "name": "Razor Network",
    "decimals": 18,
    "logoURI": "ipfs://QmXLoX8YsccFxxxFQJ8kTFLVy8oC7LhptUVG3RznzwZJC8"
  },
  {
    "chainId": 1,
    "address": "0x83d60E7aED59c6829fb251229061a55F35432c4d",
    "symbol": "INFT",
    "name": "Infinito",
    "decimals": 6,
    "logoURI": "ipfs://QmfAELrARAsCrB1vzwgAgAhaw36x6YKnxqw6XE2nmiUoXq"
  },
  {
    "chainId": 42220,
    "address": "0x29dFce9c22003A4999930382Fd00f9Fd6133Acd1",
    "symbol": "SUSHI",
    "name": "Sushi",
    "decimals": 18,
    "logoURI": "ipfs://QmWmSKdZMf2kjte9Drd7AUX5VTj9FF9bSvbctgawzAcxet"
  },
  {
    "chainId": 1,
    "address": "0xA10A210C27C86f8C63825096E34f671fD61041b1",
    "symbol": "JEFF",
    "name": "Jeff in Space",
    "decimals": 9,
    "logoURI": "ipfs://QmYH7WJzk8ebhP7e6qxLyN45tPtCVdxHCsnm6LFDxoD9gm"
  },
  {
    "chainId": 1,
    "address": "0xCd7492db29E2ab436e819b249452EE1bbDf52214",
    "symbol": "SMI",
    "name": "SafeMoon Inu",
    "decimals": 8,
    "logoURI": "ipfs://QmTcmMyvRrF86UNNGSnAuWnL3AXJKd65tpNdM49CCYvBpo"
  },
  {
    "chainId": 1,
    "address": "0x14da230d6726C50F759Bc1838717F8CE6373509C",
    "symbol": "KAT",
    "name": "Kambria",
    "decimals": 18,
    "logoURI": "ipfs://QmbDoNz75ZHCvDdmmvYCu1Q4Up6ygaJV1zQJtRGG7m2YaC"
  },
  {
    "chainId": 10,
    "address": "0xdC6fF44d5d932Cbd77B52E5612Ba0529DC6226F1",
    "symbol": "WLD",
    "name": "Worldcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmZPdBuzk8HxZ1sH5sqr7ezn3T1AXuUgXqcVv1WMemwcYx"
  },
  {
    "chainId": 1,
    "address": "0xad3E3Fc59dff318BecEaAb7D00EB4F68b1EcF195",
    "symbol": "WCUSD",
    "name": "Wrapped Celo USD",
    "decimals": 18,
    "logoURI": "ipfs://QmTieLb8SjmvpbnBFMqZpo3rzwDXZoQ2nFUfMScZEKx7Sj"
  },
  {
    "chainId": 56,
    "address": "0xEd8c8Aa8299C10f067496BB66f8cC7Fb338A3405",
    "symbol": "PROS",
    "name": "Prosper",
    "decimals": 18,
    "logoURI": "ipfs://QmewE3Kv1SjWchsXj6iRPBL2Nh5qKHvR6ivb4R3D3nWQo5"
  },
  {
    "chainId": 1,
    "address": "0x91Af0fBB28ABA7E31403Cb457106Ce79397FD4E6",
    "symbol": "AERGO",
    "name": "Aergo",
    "decimals": 18,
    "logoURI": "ipfs://QmehSYj1x4w4ZmhhxdMFJ6WW5m8iBnvR7oqcEmiEwUzFXV"
  },
  {
    "chainId": 1,
    "address": "0x61cDb66e56FAD942a7b5cE3F419FfE9375E31075",
    "symbol": "RAIN",
    "name": "RAIN Network",
    "decimals": 18,
    "logoURI": "ipfs://Qmc56YktVQyG1rGeLAkMTwcvUR1pqcHtkuAivXxTZkZ9Cg"
  },
  {
    "chainId": 56,
    "address": "0xA0d96fD642156FC7E964949642257b3572f10cD6",
    "symbol": "BLOK",
    "name": "Bloktopia",
    "decimals": 18,
    "logoURI": "ipfs://QmZ9djJyGc1mg5ea5hMhatSNY9fQF7XWaQ5nanKZzCiJqn"
  },
  {
    "chainId": 1,
    "address": "0xB37a769B37224449d92AAc57dE379E1267Cd3B00",
    "symbol": "COVA",
    "name": "COVA",
    "decimals": 18,
    "logoURI": "ipfs://QmWPs1dnj43pWjSXHeG5MDnuh2Lmz6wderof28MkNdGeaV"
  },
  {
    "chainId": 8453,
    "address": "0x858c50C3AF1913b0E849aFDB74617388a1a5340d",
    "symbol": "SQT",
    "name": "SubQuery Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdjHWvgz6SUtKEUv2cFtYoqP1zRJEeC1FK9PVWXT1bQVH"
  },
  {
    "chainId": 1,
    "address": "0x0f7F961648aE6Db43C75663aC7E5414Eb79b5704",
    "symbol": "XIO",
    "name": "XIO",
    "decimals": 18,
    "logoURI": "ipfs://QmT9Kh5PdEXD7qvpXhTmjLUKESMALZtF3qm8RxL6kqUNva"
  },
  {
    "chainId": 1,
    "address": "0x035bfe6057E15Ea692c0DfdcaB3BB41a64Dd2aD4",
    "symbol": "ULU",
    "name": "Universal Liquidity Union",
    "decimals": 18,
    "logoURI": "ipfs://QmZGmzCPuffxAfAZAhi3zuq86jqKPQt7tD8hjVba25e8dW"
  },
  {
    "chainId": 100,
    "address": "0xAAd66432d27737ecf6ED183160Adc5eF36aB99f2",
    "symbol": "TRB",
    "name": "Tellor",
    "decimals": 18,
    "logoURI": "ipfs://QmWvmGB7NhEcfLrqCM6dsRozFytqY2wBC3ordycpzgjQqU"
  },
  {
    "chainId": 1,
    "address": "0x2ffa85f655752fB2aCB210287c60b9ef335f5b6E",
    "symbol": "iWBTC",
    "name": "Fulcrum WBTC iToken",
    "decimals": 8,
    "logoURI": "ipfs://Qmei61Bx2gEf8L7W6d5CpXZprqXcqSiKH5XFH5D4MxwrsR"
  },
  {
    "chainId": 1,
    "address": "0xD54F76a4F66d6016A7FF169FacE62C14746e25b7",
    "symbol": "EWOM",
    "name": "EWOM",
    "decimals": 18,
    "logoURI": "ipfs://QmfT6iRi1sDPG6am9BaKDHHpVd6RbDFiZZ878537mNR9eT"
  },
  {
    "chainId": 1,
    "address": "0x8003C49f6ebACDdC493EA47CAB45e892d1B638a1",
    "symbol": "AMPLG",
    "name": "AmpleGold",
    "decimals": 9,
    "logoURI": "ipfs://QmcMXaVwiU5v7o3dBqod1TiTbGumAbd1XneayUT4r6fH2w"
  },
  {
    "chainId": 56,
    "address": "0xf508fCD89b8bd15579dc79A6827cB4686A3592c8",
    "symbol": "vETH",
    "name": "Venus ETH",
    "decimals": 8,
    "logoURI": "ipfs://Qmahymf3y51ti19BmQEMuQ6SCM25DoYg6EFFGQCNoVwr4Q"
  },
  {
    "chainId": 1,
    "address": "0x4104b135DBC9609Fc1A9490E61369036497660c8",
    "symbol": "APW",
    "name": "APWine",
    "decimals": 18,
    "logoURI": "ipfs://QmUt2wDb8N5TXq5f19NSVaT5yAMwADDsUfaC1fYrzhRj4i"
  },
  {
    "chainId": 1,
    "address": "0xAC9ce326e95f51B5005e9fE1DD8085a01F18450c",
    "symbol": "VSF",
    "name": "VeriSafe",
    "decimals": 18,
    "logoURI": "ipfs://QmWxok3zaKgMMTYGEvLj62veiqC68hURcvLqzUudBrUH5a"
  },
  {
    "chainId": 1,
    "address": "0x7a2Bc711E19ba6aff6cE8246C546E8c4B4944DFD",
    "symbol": "WAXE",
    "name": "WAXE",
    "decimals": 8,
    "logoURI": "ipfs://QmeMyuXyMYpqYa6Tq6CttTcYKUvaCHaz6pkMQvzT6tB8Cx"
  },
  {
    "chainId": 1,
    "address": "0x62D0A8458eD7719FDAF978fe5929C6D342B0bFcE",
    "symbol": "BEAM",
    "name": "Beam",
    "decimals": 18,
    "logoURI": "ipfs://QmPfGrjzfmzCevK86kK5ZvD9E7S88wLw1t7Sq1X6yPAJUB"
  },
  {
    "chainId": 1,
    "address": "0xB1f66997A5760428D3a87D68b90BfE0aE64121cC",
    "symbol": "LUA",
    "name": "Lua Token",
    "decimals": 18,
    "logoURI": "ipfs://QmVop2FbFZs6iCnk6sExARFPyTVkhCVuRwyS4LLx1tKfS1"
  },
  {
    "chainId": 1,
    "address": "0x8290333ceF9e6D528dD5618Fb97a76f268f3EDD4",
    "symbol": "ANKR",
    "name": "Ankr",
    "decimals": 18,
    "logoURI": "ipfs://QmXjiygak4dUVX2m1W4i6V6vFoUvDKgtJBbQseFcVNJZ94"
  },
  {
    "chainId": 1,
    "address": "0xB6eD7644C69416d67B522e20bC294A9a9B405B31",
    "symbol": "0xBTC",
    "name": "0xBitcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmX1V76wwDjKPAdcsS8MptpDugJGE3cVfrMkFndbSpMCeq"
  },
  {
    "chainId": 1,
    "address": "0x83e817E1574e2201a005EC0f7e700ED5606F555E",
    "symbol": "mPendleOFT",
    "name": "mPendle",
    "decimals": 18,
    "logoURI": "ipfs://Qmdi1PC1TGnGg7uEVtM9tcrsb7DbcHDasEyrRm4jeaYJoL"
  },
  {
    "chainId": 137,
    "address": "0xdAb529f40E671A1D4bF91361c21bf9f0C9712ab7",
    "symbol": "BUSD",
    "name": "Binance USD",
    "decimals": 18,
    "logoURI": "ipfs://QmUCSx8qo7kSFcdPJ7gLp9X9pPkue13KUmnL6ZzBZ6ejaQ"
  },
  {
    "chainId": 1,
    "address": "0xF256CC7847E919FAc9B808cC216cAc87CCF2f47a",
    "symbol": "aXSUSHI",
    "name": "Aave XSUSHI",
    "decimals": 18,
    "logoURI": "ipfs://QmUT4HXThbM7VLS94mVTvQQnFufz6hBSYE6Tjss4KWSrTE"
  },
  {
    "chainId": 42161,
    "address": "0xdA661fa59320B808c5a6d23579fCfEdf1FD3cf36",
    "symbol": "MBOX",
    "name": "MOBOX",
    "decimals": 18,
    "logoURI": "ipfs://QmPm94nfJA7itJHxUWWJr7QiLLuNKSzr5xUyWyD4AanhTm"
  },
  {
    "chainId": 1,
    "address": "0xE46F290Cd59195a83e757891430d8D517d16b334",
    "symbol": "AFN",
    "name": "AltaFin",
    "decimals": 18,
    "logoURI": "ipfs://Qmeqdxd8G8jCodtsYPkovWPPAEoQrM17wrSWZSAxH6jgAy"
  },
  {
    "chainId": 1,
    "address": "0x607F4C5BB672230e8672085532f7e901544a7375",
    "symbol": "RLC",
    "name": "iExec RLC",
    "decimals": 9,
    "logoURI": "ipfs://QmSZkBRpRX4oSqKjscgfqRDgwjtXPCkwmNdryhmQS3YKhb"
  },
  {
    "chainId": 1,
    "address": "0x761D38e5ddf6ccf6Cf7c55759d5210750B5D60F3",
    "symbol": "ELON",
    "name": "Dogelon Mars",
    "decimals": 18,
    "logoURI": "ipfs://QmaHs61uW9nZYiHVSQuZgnQWBPSwNokcnNgZn7FGMhSHVk"
  },
  {
    "chainId": 1,
    "address": "0x646707246D7d5C2a86d7206f41CA8199ea9CED69",
    "symbol": "CHOP",
    "name": "Porkchop",
    "decimals": 18,
    "logoURI": "ipfs://QmW1h4vfn7Z533bwKA3wBJv2magsWSzsXACqLKxpfkFFpd"
  },
  {
    "chainId": 1,
    "address": "0xDf7FF54aAcAcbFf42dfe29DD6144A69b629f8C9e",
    "symbol": "aZRX",
    "name": "Aave ZRX",
    "decimals": 18,
    "logoURI": "ipfs://QmczZhrtoXqcDm1YCxCc4RVFQhqyPU7URRrEXExVxqHfKz"
  },
  {
    "chainId": 1,
    "address": "0x8798249c2E607446EfB7Ad49eC89dD1865Ff4272",
    "symbol": "xSUSHI",
    "name": "xSUSHI",
    "decimals": 18,
    "logoURI": "ipfs://QmeXt3g3sRF2dh6kosC3ubBN9tSw9YC7oJRA5kttdddrra"
  },
  {
    "chainId": 1,
    "address": "0x4a57E687b9126435a9B19E4A802113e266AdeBde",
    "symbol": "FXC",
    "name": "Flexacoin",
    "decimals": 18,
    "logoURI": "ipfs://QmVfe2f4TZLM43noUgyR2UWaNWNt47xYxqq69iALY5H2tR"
  },
  {
    "chainId": 1,
    "address": "0x78B039921E84E726EB72E7b1212bb35504c645cA",
    "symbol": "SETH",
    "name": "Sether",
    "decimals": 18,
    "logoURI": "ipfs://Qma5qjKV4v2tQRZSqopw3gS2St7mkaQqsg2MjAqGayP7HP"
  },
  {
    "chainId": 56,
    "address": "0x75E8ddB518bB757b4282cd5b83bb70d4101D12FB",
    "symbol": "NFP",
    "name": "NFPrompt",
    "decimals": 18,
    "logoURI": "ipfs://QmPMYiypjKpmRaEsYqP8W7g1yWmE5HKfKRCwyjNUzh5aHC"
  },
  {
    "chainId": 1,
    "address": "0x08d32b0da63e2C3bcF8019c9c5d849d7a9d791e6",
    "symbol": "DCN",
    "name": "Dentacoin",
    "decimals": 0,
    "logoURI": "ipfs://QmbhNhGY6QJChiUposVxdvKFQgjHDfTDfb2sTg48WiTmLF"
  },
  {
    "chainId": 324,
    "address": "0x0e97C7a0F8B2C9885C8ac9fC6136e829CbC21d42",
    "symbol": "MUTE",
    "name": "Mute",
    "decimals": 18,
    "logoURI": "ipfs://QmRxy2KhwDypiZHoDbdD6kXv8q5snhBDWgtfuPJ27TosU6"
  },
  {
    "chainId": 1,
    "address": "0xC5807183a9661A533CB08CbC297594a0B864dc12",
    "symbol": "iTRX",
    "name": "Synthetic Inverse TRON",
    "decimals": 18,
    "logoURI": "ipfs://QmPjn51ZqPZkroZ4p446tKm8NBL9kBUbtFduUTPhjnELeL"
  },
  {
    "chainId": 1,
    "address": "0xc50EF449171a51FbeAFd7c562b064B6471C36caA",
    "symbol": "ZINU",
    "name": "Zombie Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmcEAoRnFPvxbPhud29D78WT7EX2mZW8cFpFxXMtkhhH9G"
  },
  {
    "chainId": 10,
    "address": "0x4200000000000000000000000000000000000042",
    "symbol": "OP",
    "name": "Optimism",
    "decimals": 18,
    "logoURI": "ipfs://Qmf6Rw34hCk8DUzKhtAXVcL8WVhWJ881rQn9YB88SAokRn"
  },
  {
    "chainId": 137,
    "address": "0xD1f9c58e33933a993A3891F8acFe05a68E1afC05",
    "symbol": "SFL",
    "name": "Sunflower Land",
    "decimals": 18,
    "logoURI": "ipfs://Qmbintub8EwRSzCUT7hn4NKswtYXXaXJD7ZgbzPBXLEzkV"
  },
  {
    "chainId": 1,
    "address": "0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2",
    "symbol": "WETH",
    "name": "WETH",
    "decimals": 18,
    "logoURI": "ipfs://QmRvUDB8isthbwNEWtCvWTJ9kxZsc1VZ4mNmVmT7BBVUS9"
  },
  {
    "chainId": 1,
    "address": "0x3623387773010d9214B10C551d6e7fc375D31F58",
    "symbol": "crMTA",
    "name": "Cream Meta",
    "decimals": 8,
    "logoURI": "ipfs://QmdRJdoP5VZexUsAUxC43Vh1o8qtBUs6MG5t55Zq7bEu5J"
  },
  {
    "chainId": 1,
    "address": "0xDea67845A51E24461D5fED8084E69B426AF3D5Db",
    "symbol": "HTRE",
    "name": "HodlTree",
    "decimals": 18,
    "logoURI": "ipfs://Qmc4BagUiZCBYe7HvZkdLtAEGqThZu8FsdYfptXjSGV1Fa"
  },
  {
    "chainId": 8453,
    "address": "0x23ee2343B892b1BB63503a4FAbc840E0e2C6810f",
    "symbol": "AXL",
    "name": "Axelar",
    "decimals": 6,
    "logoURI": "ipfs://QmcSRWc1LN1mk4tjy38kyTMqqVx7j4Dmnkw3dsW7efPMTL"
  },
  {
    "chainId": 1,
    "address": "0x1E4EDE388cbc9F4b5c79681B7f94d36a11ABEBC9",
    "symbol": "X2Y2",
    "name": "X2Y2",
    "decimals": 18,
    "logoURI": "ipfs://QmP5sg2XbsUhWE4fVHmVE2wZmfYf6EjnhtYRULj6UFHsrD"
  },
  {
    "chainId": 1,
    "address": "0x501acE9c35E60f03A2af4d484f49F9B1EFde9f40",
    "symbol": "SOLACE",
    "name": "Solace",
    "decimals": 18,
    "logoURI": "ipfs://QmaTdZw8DQ2mMcZK5G6ipeQajBUUcLUjpaXsV7eSvnCj7C"
  },
  {
    "chainId": 1,
    "address": "0x06012c8cf97BEaD5deAe237070F9587f8E7A266d",
    "symbol": "CK",
    "name": "CryptoKitties",
    "decimals": 0,
    "logoURI": "ipfs://QmXW3H7ZT6CRCWXEhgg64HiGUFTvwbQ6fRN44kVAHwUHj2"
  },
  {
    "chainId": 1,
    "address": "0x0947b0e6D821378805c9598291385CE7c791A6B2",
    "symbol": "LND",
    "name": "Lendingblock",
    "decimals": 18,
    "logoURI": "ipfs://QmTiXXzyvQjocvLaPLmx5gfksHJEwZHnJ73QZS9Ha6qcGc"
  },
  {
    "chainId": 1,
    "address": "0xCdCFc0f66c522Fd086A1b725ea3c0Eeb9F9e8814",
    "symbol": "AURA",
    "name": "Aurora DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmUN2ugVSLsrcwY6kLC62aPXDXrt71k1U5KTXTknLsSRGS"
  },
  {
    "chainId": 1,
    "address": "0xB1F1ee126e9c96231Cc3d3fAD7C08b4cf873b1f1",
    "symbol": "BIFI",
    "name": "Beefy",
    "decimals": 18,
    "logoURI": "ipfs://QmX8mm6KZbGmqDzZKTfWbby4XUn4ox8MDv5bYKedobBZLo"
  },
  {
    "chainId": 1,
    "address": "0xbb1EE07d6c7BAeB702949904080eb61f5D5e7732",
    "symbol": "DINU",
    "name": "Dogey-Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmfA1H8Et8bqhB1JDbHWfZfwsGMpPnyxTQHUbiGNyWAqF7"
  },
  {
    "chainId": 1,
    "address": "0xb3999F658C0391d94A37f7FF328F3feC942BcADC",
    "symbol": "HFT",
    "name": "Hashflow",
    "decimals": 18,
    "logoURI": "ipfs://QmNobp6XesVAXdTTGuiL1moNkjxEHFgVcWu7iLAzWQcJWs"
  },
  {
    "chainId": 1,
    "address": "0x5283D291DBCF85356A21bA090E6db59121208b44",
    "symbol": "BLUR",
    "name": "Blur",
    "decimals": 18,
    "logoURI": "ipfs://QmbccKEk1pb7p2Be8kPhKsoxjj7gAnU4TdtU3EaaKUf8QU"
  },
  {
    "chainId": 1,
    "address": "0xBe9895146f7AF43049ca1c1AE358B0541Ea49704",
    "symbol": "cbETH",
    "name": "Coinbase Wrapped Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmTPxtJKnMmY8XrJ5jMSwdmkVB5htNK9AXWu5Tbmw9K5ux"
  },
  {
    "chainId": 1,
    "address": "0xA682Ee16b497afcEEDF47e4820Fc2af3845FD2D2",
    "symbol": "PIKA",
    "name": "Pika",
    "decimals": 18,
    "logoURI": "ipfs://QmX4BdxaJEitiV5cvqKxTWo3oy1jLbWsFrr94UfCDgvReq"
  },
  {
    "chainId": 1,
    "address": "0x0AbdAce70D3790235af448C88547603b945604ea",
    "symbol": "DNT",
    "name": "district0x",
    "decimals": 18,
    "logoURI": "ipfs://QmNn4nv7oFMKPeZMqFvBHebx8LhoJQXsPzp1R9LwFGbFGy"
  },
  {
    "chainId": 1,
    "address": "0xd5103AfcD0B3fA865997Ef2984C66742c51b2a8b",
    "symbol": "crHFIL",
    "name": "Cream Huobi FIL",
    "decimals": 8,
    "logoURI": "ipfs://QmYVD2NVgiXjxuLD13JYxeem9wFeRj4zNLRbuTHkBVX7kE"
  },
  {
    "chainId": 1,
    "address": "0x66a0f676479Cee1d7373f3DC2e2952778BfF5bd6",
    "symbol": "WISE",
    "name": "Wise",
    "decimals": 18,
    "logoURI": "ipfs://QmauZH81qdTanmyJXQ4moGP8Q8GDX3HfixwQnS3fJWrvFV"
  },
  {
    "chainId": 1,
    "address": "0x757de3ac6B830a931eF178C6634c5C551773155c",
    "symbol": "sNIKKEI",
    "name": "Synthetic Nikkei 225 Index",
    "decimals": 18,
    "logoURI": "ipfs://QmYutUNRuRuoJMEQhRrw3fvmGbiiaX9fsdJEzeCXH3x5GX"
  },
  {
    "chainId": 1,
    "address": "0x4e3FBD56CD56c3e72c1403e103b45Db9da5B9D2B",
    "symbol": "CVX",
    "name": "Convex Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmWwi4dKqrdaxmdYLmutkxNSGdQBanvsfwCjh7W5BgBg9F"
  },
  {
    "chainId": 1,
    "address": "0xAaEf88cEa01475125522e117BFe45cF32044E238",
    "symbol": "GF",
    "name": "GuildFi",
    "decimals": 18,
    "logoURI": "ipfs://QmRyUBSyH7GqH9w1mDBCwJ8op69vCXfhNNiA3veF7yDLJY"
  },
  {
    "chainId": 1,
    "address": "0xcE1d3dA32e3a45d27dC841781f09E40C41CAC677",
    "symbol": "DLB",
    "name": "Data Link Base",
    "decimals": 18,
    "logoURI": "ipfs://Qmevh6JpcbBaSqvG7CKH9GEfWbbk4uzaQ8RVdNR7jWUoXZ"
  },
  {
    "chainId": 1,
    "address": "0xA7Eb2bc82df18013ecC2A6C533fc29446442EDEe",
    "symbol": "iZRX",
    "name": "Fulcrum ZRX iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmeLNWUVUAJKwjewbEo9hgdTHKotzmrFYGpN4gzFBwQnVU"
  },
  {
    "chainId": 43114,
    "address": "0x2F6F07CDcf3588944Bf4C42aC74ff24bF56e7590",
    "symbol": "STG",
    "name": "Stargate Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYCe3FShq8cRmJ9njM8eZhoNqkuohMm5hUjd4khBcEB8U"
  },
  {
    "chainId": 1,
    "address": "0x0b38210ea11411557c13457D4dA7dC6ea731B88a",
    "symbol": "API3",
    "name": "API3",
    "decimals": 18,
    "logoURI": "ipfs://QmSiLEcUpBdFPkwbkwgEVNnW6vSToiNeroGY9zQVfxNk7w"
  },
  {
    "chainId": 1,
    "address": "0xe1aFe1Fd76Fd88f78cBf599ea1846231B8bA3B6B",
    "symbol": "sDEFI",
    "name": "Synthetic DeFi Index",
    "decimals": 18,
    "logoURI": "ipfs://QmcgvPrDuXA8F6gyQgV9ysHMEpo1WpGK2phskoe9oHzXum"
  },
  {
    "chainId": 250,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmVurySKQi4AcZg3wUXXaudowWcTve6vKMbTHpQU6WExat"
  },
  {
    "chainId": 1,
    "address": "0x468ab3b1f63A1C14b361bC367c3cC92277588Da1",
    "symbol": "YELD",
    "name": "Yeld Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRqcpSEEWeiwPya8nW7WRqFBD1JvBgkdHhddDjGbGpaMf"
  },
  {
    "chainId": 1,
    "address": "0x5A235C0b4cB8d0e80a5c3bF4d2faD5c32E440884",
    "symbol": "KONGZ",
    "name": "KONGZ Vault",
    "decimals": 18,
    "logoURI": "ipfs://Qmd6LjkBk5xfKEhtz4h1kgi4jzTWzKnPH8uQLp9Yf5Yjuj"
  },
  {
    "chainId": 1,
    "address": "0x0000000000085d4780B73119b644AE5ecd22b376",
    "symbol": "TUSD",
    "name": "TrueUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmSuucvXBjBTJyfwfY77wLBsJ1NWUPKTnJT9B4dsJP9mMp"
  },
  {
    "chainId": 1,
    "address": "0x7047F90229a057C13BF847C0744D646CFb6c9E1A",
    "symbol": "yvCurve-renBTC",
    "name": "Curve renBTC Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmNy5Hn2sC9rNB99wJAT9rFWyFo4Q1c9cVD6kgefyWT1AN"
  },
  {
    "chainId": 1,
    "address": "0xEABB8996eA1662cAd2f7fB715127852cd3262Ae9",
    "symbol": "CNFI",
    "name": "Connect Financial",
    "decimals": 18,
    "logoURI": "ipfs://QmZTR5RMa3uEu1LF8hB9xAN1jPrmWZ79daTyAeCKYp8P1X"
  },
  {
    "chainId": 1,
    "address": "0x7Da96a3891Add058AdA2E826306D812C638D87a7",
    "symbol": "yvUSDT",
    "name": "USDT yVault",
    "decimals": 6,
    "logoURI": "ipfs://QmWTxXoBfmxtpobVrQVwtyEHcm8APCG8xpw7b9zEtmJ8YU"
  },
  {
    "chainId": 1,
    "address": "0xD5147bc8e386d91Cc5DBE72099DAC6C9b99276F5",
    "symbol": "renFIL",
    "name": "renFIL",
    "decimals": 18,
    "logoURI": "ipfs://QmPXhBhEvEuK5kdJaeLBqaUBQqLcS4iZmELFXu1SaWb2cR"
  },
  {
    "chainId": 1,
    "address": "0xDd1Ad9A21Ce722C151A836373baBe42c868cE9a4",
    "symbol": "UBI",
    "name": "Universal Basic Income",
    "decimals": 18,
    "logoURI": "ipfs://QmZgm1mythRBiEksoSFiouniMEZZXHG4NezaSvVKMUPA16"
  },
  {
    "chainId": 1,
    "address": "0xD8Bd3958725F216Eb236E9DC65B169DE48101C6A",
    "symbol": "GBT",
    "name": "Globatalent",
    "decimals": 8,
    "logoURI": "ipfs://QmVE1tyd1BkRBh9MvBqb6p3FwAqkPiWae7dkAVGJUui4dD"
  },
  {
    "chainId": 1,
    "address": "0x3a1Bda28AdB5B0a812a7CF10A1950c920F79BcD3",
    "symbol": "FLP",
    "name": "FLIP",
    "decimals": 18,
    "logoURI": "ipfs://QmYCcNTc39eVog5dywPY3uouUbpWCpQ4NYhBkCTBsM2aXV"
  },
  {
    "chainId": 1,
    "address": "0xa1d0E215a23d7030842FC67cE582a6aFa3CCaB83",
    "symbol": "YFII",
    "name": "YFII.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmR22geJeLtoUzBp6n82BRpyZoRoGjXcY2c1FdCSmgnRko"
  },
  {
    "chainId": 42161,
    "address": "0xa684cd057951541187f288294a1e1C2646aA2d24",
    "symbol": "VSTA",
    "name": "Vesta Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmdHXSxskXWEfaAcHRQwCsDJS9wasYtDgCc7HtTFtDKSWu"
  },
  {
    "chainId": 1,
    "address": "0x17525E4f4Af59fbc29551bC4eCe6AB60Ed49CE31",
    "symbol": "YPIE",
    "name": "PieDAO Yearn Ecosystem Pie",
    "decimals": 18,
    "logoURI": "ipfs://QmaNqgJzk9xn6nMkhHwbYAZJdo5nmwSNu3WWNovgp5nLeh"
  },
  {
    "chainId": 137,
    "address": "0x40379a439D4F6795B6fc9aa5687dB461677A2dBa",
    "symbol": "USDR",
    "name": "Real USD",
    "decimals": 9,
    "logoURI": "ipfs://QmNPpu9zxYu4d6WjAgUFX8Ej3GQvZ3CYVUApXPPMWKw971"
  },
  {
    "chainId": 1,
    "address": "0xCbaE0A83f4f9926997c8339545fb8eE32eDc6b76",
    "symbol": "crYFI",
    "name": "Cream YFI",
    "decimals": 8,
    "logoURI": "ipfs://QmdsXZWcnh6q72qKtHYWnNhQPNsgJbHAdAuCjp8PXXBpC3"
  },
  {
    "chainId": 1,
    "address": "0x1C5db575E2Ff833E46a2E9864C22F4B22E0B37C2",
    "symbol": "renZEC",
    "name": "renZEC",
    "decimals": 8,
    "logoURI": "ipfs://QmSajvEWSby7XqXYie7fXqVzdJSmNdWhFnRwU72eAB3Ymv"
  },
  {
    "chainId": 43114,
    "address": "0x8eBAf22B6F053dFFeaf46f4Dd9eFA95D89ba8580",
    "symbol": "UNI",
    "name": "Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmbSptJsjBStRQ7tpzdq8DAnaT4cgaKbpHKTLLtsVN5n1Q"
  },
  {
    "chainId": 1,
    "address": "0x1712Aad2C773ee04bdC9114b32163c058321CD85",
    "symbol": "LIMIT",
    "name": "LimitSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmdcQz6Jicse4WB5KRSk1jv2RfYjnVgSLE5EojZPxs5swv"
  },
  {
    "chainId": 1,
    "address": "0x5F64Ab1544D28732F0A24F4713c2C8ec0dA089f0",
    "symbol": "DEXTF",
    "name": "DEXTF Protocol",
    "decimals": 18,
    "logoURI": "ipfs://Qmbza32G6DgYrD3dthzS2uogKfooX3yB9FZQHfmgLcpyse"
  },
  {
    "chainId": 1,
    "address": "0x60EF10EDfF6D600cD91caeCA04caED2a2e605Fe5",
    "symbol": "MOCHI",
    "name": "Mochi Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmXfpDvEHsaF9sU2nkffmWCf5bHkU7HdGZVdrKZfU4GshW"
  },
  {
    "chainId": 1,
    "address": "0xCdF7028ceAB81fA0C6971208e83fa7872994beE5",
    "symbol": "T",
    "name": "Threshold Network",
    "decimals": 18,
    "logoURI": "ipfs://QmSGMD29fCZELn3JY6yCkkf5TLhhamhNmp5jZSy1VfV9nk"
  },
  {
    "chainId": 1,
    "address": "0x3aFfCCa64c2A6f4e3B6Bd9c64CD2C969EFd1ECBe",
    "symbol": "DSLA",
    "name": "DSLA Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmRbiUXswRVfHKM6fCBfFgn7h89nczoA1KWartGHMwgPZQ"
  },
  {
    "chainId": 1,
    "address": "0x00000000441378008EA67F4284A57932B1c000a5",
    "symbol": "TGBP",
    "name": "TrueGBP",
    "decimals": 18,
    "logoURI": "ipfs://QmS4iZiNcUK7SYdjenQwQdXcg8Tv5iXWQTqumCcKdBYSJB"
  },
  {
    "chainId": 43114,
    "address": "0x9eAaC1B23d935365bD7b542Fe22cEEe2922f52dc",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRQq6tqSDu6pvZPmYHhZsF9dvUqzudAnfn6cKPurWzQbF"
  },
  {
    "chainId": 1,
    "address": "0x62359Ed7505Efc61FF1D56fEF82158CcaffA23D7",
    "symbol": "CORE",
    "name": "cVault.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmTHv4rUgNF3hBQepAVj3mLoJfYuYxmh1qR9YQgnVtgLUf"
  },
  {
    "chainId": 1,
    "address": "0xd559f20296FF4895da39b5bd9ADd54b442596a61",
    "symbol": "FTX",
    "name": "FintruX Network",
    "decimals": 18,
    "logoURI": "ipfs://QmVsbB1WX6sCCety1YQHFLyqqPUjZJ5tYcYWGD4Szn9HuS"
  },
  {
    "chainId": 100,
    "address": "0xfa57AA7beED63D03Aaf85fFd1753f5f6242588fb",
    "symbol": "MPS",
    "name": "Mt Pelerin Shares",
    "decimals": 0,
    "logoURI": "ipfs://QmW8CL9WN1JRgRfSVitXyBTajUvnenGoaxEVtZmRYmCBSt"
  },
  {
    "chainId": 100,
    "address": "0x7eF541E2a22058048904fE5744f9c7E4C57AF717",
    "symbol": "BAL",
    "name": "Balancer",
    "decimals": 18,
    "logoURI": "ipfs://QmbzDmJ2yboeFf45poTkzpqZkDbnsdL24x2BJQTs64yv5Z"
  },
  {
    "chainId": 1,
    "address": "0x156B36ec68FdBF84a925230BA96cb1Ca4c4bdE45",
    "symbol": "MNT",
    "name": "Wrapped MNT",
    "decimals": 18,
    "logoURI": "ipfs://QmTNFJTGoZs7R18gWCZY9zMrcGAJ78c8Yr9qQY6WAbocXk"
  },
  {
    "chainId": 1,
    "address": "0x798D1bE841a82a273720CE31c822C61a67a601C3",
    "symbol": "DIGG",
    "name": "DIGG",
    "decimals": 9,
    "logoURI": "ipfs://QmRP97y1TntiTdBUZVXtr2BYUfdVsWxWkN2kpzff9NZwvq"
  },
  {
    "chainId": 137,
    "address": "0x714DB550b574b3E927af3D93E26127D15721D4C2",
    "symbol": "GMT",
    "name": "STEPN",
    "decimals": 8,
    "logoURI": "ipfs://QmaXgp7LnvKbQ64Bb3P1e92Cju9KNbxzwzdLaLqb7nrUUW"
  },
  {
    "chainId": 1,
    "address": "0xF629cBd94d3791C9250152BD8dfBDF380E2a3B9c",
    "symbol": "ENJ",
    "name": "Enjin Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmRZrrgVwxuSyC8YpF3KHwAGzPEAQYbCsZ5YUBC8gCpHvT"
  },
  {
    "chainId": 1,
    "address": "0xc3dD23A0a854b4f9aE80670f528094E9Eb607CCb",
    "symbol": "TRND",
    "name": "Trendering",
    "decimals": 18,
    "logoURI": "ipfs://Qmf4CuwSyRWLEK4q3ftpZN5N1pEeUKYSTLRcRdo3rN81ZL"
  },
  {
    "chainId": 1,
    "address": "0xA91ac63D040dEB1b7A5E4d4134aD23eb0ba07e14",
    "symbol": "BEL",
    "name": "Bella Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmZQK5hofUHyVgwwxNYjcJRUmqH8HDkUdTKA3z9Mf5esw4"
  },
  {
    "chainId": 1,
    "address": "0x6345728B1ccE16E6f8C509950b5c84FFF88530d9",
    "symbol": "iCOMP",
    "name": "Synthetic Inverse Compound",
    "decimals": 18,
    "logoURI": "ipfs://QmdNRQvYCijqiE7T8TEELG4K1T1VVFXGt58kzcmF5V43Jd"
  },
  {
    "chainId": 1,
    "address": "0x3A9FfF453d50D4Ac52A6890647b823379ba36B9E",
    "symbol": "SHUF",
    "name": "Shuffle Monster",
    "decimals": 18,
    "logoURI": "ipfs://QmbthYP46q6dHF7iATqBsp4wgjqmff2X4u8xSQVDWifmzL"
  },
  {
    "chainId": 1,
    "address": "0xb131f4A55907B10d1F0A50d8ab8FA09EC342cd74",
    "symbol": "MEME",
    "name": "Memecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmTZR4yGfMfESAoEvf8P4vJmyoSZ4gEfAuRWGciddmfzCa"
  },
  {
    "chainId": 1,
    "address": "0xCB56b52316041A62B6b5D0583DcE4A8AE7a3C629",
    "symbol": "CIG",
    "name": "Cigarettes",
    "decimals": 18,
    "logoURI": "ipfs://Qme4tPMxap81it6fTZ8WbJV3dJ24VUgLzL4UZpVNLxDmCs"
  },
  {
    "chainId": 137,
    "address": "0x0B220b82F3eA3B7F6d9A1D8ab58930C064A2b5Bf",
    "symbol": "GLM",
    "name": "Golem",
    "decimals": 18,
    "logoURI": "ipfs://QmTXdjxTZjiPgtBDArBu4qRWRLPyG7UqF9wyXyPM3qYu9Q"
  },
  {
    "chainId": 1,
    "address": "0x56A86d648c435DC707c8405B78e2Ae8eB4E60Ba4",
    "symbol": "STACK",
    "name": "StackOS",
    "decimals": 18,
    "logoURI": "ipfs://Qmbw5eXFt5Erm8eQBQPMkmRuWJUuqRu49nEsPmTgZR6Y13"
  },
  {
    "chainId": 250,
    "address": "0xf16e81dce15B08F326220742020379B855B87DF9",
    "symbol": "ICE",
    "name": "Popsicle Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmSp3Vnq7VFsFrdcZZ35YkHHFLVF3nWvcDPKSDJgfkTiMD"
  },
  {
    "chainId": 1,
    "address": "0x4a220E6096B25EADb88358cb44068A3248254675",
    "symbol": "QNT",
    "name": "Quant",
    "decimals": 18,
    "logoURI": "ipfs://QmXBB2ZMiQ9r4sC2oDrQSVsXmeyABFemqLYMguYQzQqLKU"
  },
  {
    "chainId": 56,
    "address": "0x6679eB24F59dFe111864AEc72B443d1Da666B360",
    "symbol": "ARV",
    "name": "Ariva",
    "decimals": 8,
    "logoURI": "ipfs://Qmc3XiAuPWcNJWhZym4SvY5HbHhU1y6czQLbjTEHZ1ygHW"
  },
  {
    "chainId": 56,
    "address": "0xF21768cCBC73Ea5B6fd3C687208a7c2def2d966e",
    "symbol": "REEF",
    "name": "Reef",
    "decimals": 18,
    "logoURI": "ipfs://QmSGTkTMTbFUnUoDmisKxyD96yKBeLRHXwHz64zohA5d9W"
  },
  {
    "chainId": 1,
    "address": "0x4208D8d500B1643DcA98dD27bA6C0060BcA311c5",
    "symbol": "REBASE",
    "name": "REBASE",
    "decimals": 9,
    "logoURI": "ipfs://QmfZodh3xugZii2TTSMdraRB6qKh99abtn5bzZWwkRmLQv"
  },
  {
    "chainId": 42161,
    "address": "0xbFd5206962267c7b4b4A8B3D76AC2E1b2A5c4d5e",
    "symbol": "OSAK",
    "name": "Osaka Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUng1scXVJSCH1MpqK4RanCsW1aXYew8KUZphtj3WUpah"
  },
  {
    "chainId": 42161,
    "address": "0x27f485b62C4A7E635F561A87560Adf5090239E93",
    "symbol": "DFX",
    "name": "DFX Finance",
    "decimals": 18,
    "logoURI": "ipfs://Qme93F8UUpLjxyLgHveAst6E6E6C4opbqchzSVYj52ADt7"
  },
  {
    "chainId": 1,
    "address": "0x7025baB2EC90410de37F488d1298204cd4D6b29d",
    "symbol": "XRA",
    "name": "Xriba",
    "decimals": 18,
    "logoURI": "ipfs://QmXBW1WwYrrNEmWoWKRM6bxowEd7tLhF6DQsRsppiC7X9q"
  },
  {
    "chainId": 42161,
    "address": "0x539bdE0d7Dbd336b79148AA742883198BBF60342",
    "symbol": "MAGIC",
    "name": "MAGIC",
    "decimals": 18,
    "logoURI": "ipfs://QmX9mZzzH1uooK4V4TpaXKXYrBVKYacBqigPsTiKLpSS3d"
  },
  {
    "chainId": 1,
    "address": "0x4B5f49487ea7B3609b1aD05459BE420548789f1f",
    "symbol": "LEVER",
    "name": "LeverFi",
    "decimals": 18,
    "logoURI": "ipfs://QmWhcCzSZyyqXmaz6aMDfN2wtWzACmvNDABMHszXTpK7uP"
  },
  {
    "chainId": 42161,
    "address": "0x3B475F6f2f41853706afc9Fa6a6b8C5dF1a2724c",
    "symbol": "ZYB",
    "name": "Zyberswap",
    "decimals": 18,
    "logoURI": "ipfs://QmWi5S6GhsyV3RKifS1Nyyqtagn9tZQ717Ww1XdoAaNMMs"
  },
  {
    "chainId": 1,
    "address": "0x26607aC599266b21d13c7aCF7942c7701a8b699c",
    "symbol": "PIPT",
    "name": "Power Index Pool Token",
    "decimals": 18,
    "logoURI": "ipfs://QmSBU8exG3LMgnNY62dve1T2Sh6ydQuRxCiBSXps9WVWhU"
  },
  {
    "chainId": 1,
    "address": "0x9ff58f4fFB29fA2266Ab25e75e2A8b3503311656",
    "symbol": "aWBTC",
    "name": "Aave WBTC",
    "decimals": 8,
    "logoURI": "ipfs://QmXbn8baBoJGaH7TtwtL1si9fU7XVfHYfzHoyyz2gBxDY2"
  },
  {
    "chainId": 1,
    "address": "0xb62d18DeA74045E822352CE4B3EE77319DC5ff2F",
    "symbol": "EVC",
    "name": "EventChain",
    "decimals": 18,
    "logoURI": "ipfs://QmaxoR4vkeB7ZXaHcMiuQreipznvHpvh53S6juXncCWw3d"
  },
  {
    "chainId": 1,
    "address": "0x5B322514FF727253292637D9054301600c2C81e8",
    "symbol": "DAD",
    "name": "DAD",
    "decimals": 9,
    "logoURI": "ipfs://QmXGmi876uyWrXRAew6WM4Xx2LtWnXqjVwUqmoG2k67hsu"
  },
  {
    "chainId": 1,
    "address": "0x1f19f83fC9a25F3C861260143E36c17706257986",
    "symbol": "VEST",
    "name": "DAO Invest",
    "decimals": 18,
    "logoURI": "ipfs://QmeKVjY3YpD3SoCKmDJDVwn6vSUpym8GWLFdwbRsCgPQi7"
  },
  {
    "chainId": 1,
    "address": "0x0fF6ffcFDa92c53F615a4A75D982f399C989366b",
    "symbol": "LAYER",
    "name": "UniLayer",
    "decimals": 18,
    "logoURI": "ipfs://QmQMUdNydwtfCYt8BZWfsB2q9vuKdewbYBh9qB7dVG32G4"
  },
  {
    "chainId": 1,
    "address": "0xb0280743b44bF7db4B6bE482b2Ba7b75E5dA096C",
    "symbol": "TNS",
    "name": "Transcodium",
    "decimals": 18,
    "logoURI": "ipfs://Qma8ADzgRinshWoiLteVSVa97C2RgxSYGJ5EuzChA8pfsb"
  },
  {
    "chainId": 1,
    "address": "0x04Fa0d235C4abf4BcF4787aF4CF447DE572eF828",
    "symbol": "UMA",
    "name": "UMA",
    "decimals": 18,
    "logoURI": "ipfs://QmQfkyWmsAoWjz5j9KASDWtxQF5hWAyfkd6BKNo6GjsHpv"
  },
  {
    "chainId": 1,
    "address": "0x9469D013805bFfB7D3DEBe5E7839237e535ec483",
    "symbol": "RING",
    "name": "Darwinia Network",
    "decimals": 18,
    "logoURI": "ipfs://QmboC9D5SSYSzhPXX5ovGrMAxi7tm1uBJzLFcEs4vCckyd"
  },
  {
    "chainId": 1,
    "address": "0x1776e1F26f98b1A5dF9cD347953a26dd3Cb46671",
    "symbol": "NMR",
    "name": "Numeraire",
    "decimals": 18,
    "logoURI": "ipfs://QmPRgkASPadSb2GXksWt2CQTpmByFtPrdisuWELirQofVG"
  },
  {
    "chainId": 56,
    "address": "0x39d4549908e7Adcee9b439429294eEb4c65c2C9e",
    "symbol": "HORD",
    "name": "Hord",
    "decimals": 18,
    "logoURI": "ipfs://QmRWLNz4pjL6uxUJEv28vTtu7YF5qse8XNc6sEuvemHuc3"
  },
  {
    "chainId": 1,
    "address": "0x0Cf0Ee63788A0849fE5297F3407f701E122cC023",
    "symbol": "DATA",
    "name": "Streamr",
    "decimals": 18,
    "logoURI": "ipfs://QmXxq8zYZ5hQ5FadqHbRb6cQVGqcSJvn3Kg9PEDNcatreu"
  },
  {
    "chainId": 100,
    "address": "0x5fd896D248fbfa54d26855C267859eb1b4DAEe72",
    "symbol": "MKR",
    "name": "Maker",
    "decimals": 18,
    "logoURI": "ipfs://QmfAnPxYQrWrEdjzaSi9D1Cn5XRwpPBKYov6JSbvtfcy1j"
  },
  {
    "chainId": 1,
    "address": "0xfe9A29aB92522D14Fc65880d817214261D8479AE",
    "symbol": "SNOW",
    "name": "SnowSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmXitqRfiqemKJa3iDiTgHjbCfFfRz7siKy4riqVCfrUSE"
  },
  {
    "chainId": 1,
    "address": "0x054B7ed3F45714d3091e82aAd64A1588dC4096Ed",
    "symbol": "crHBTC",
    "name": "Cream Huobi BTC",
    "decimals": 8,
    "logoURI": "ipfs://QmZUJPXPEsGYH4dAFgTFWyRaEjZHCc3obYgAygPNrzZnrb"
  },
  {
    "chainId": 1,
    "address": "0x321C2fE4446C7c963dc41Dd58879AF648838f98D",
    "symbol": "CTX",
    "name": "Cryptex Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUiXwnqasriSR9zBuGi8X1e2KE6c9V1HHnDEbKRenJEiK"
  },
  {
    "chainId": 100,
    "address": "0x524B969793a64a602342d89BC2789D43a016B13A",
    "symbol": "DONUT",
    "name": "Donut",
    "decimals": 18,
    "logoURI": "ipfs://QmcBL7KqNYdeUuojuhq1Pgb9F9cpi3s6Vjyd8nAE6d8Wa1"
  },
  {
    "chainId": 1,
    "address": "0x1045F5cCb01DAEA4f8eaB055F5FcBB7C0E7c89F0",
    "symbol": "DFIAT",
    "name": "DeFiato",
    "decimals": 18,
    "logoURI": "ipfs://QmVdCKa4U5hP4y82CoEPDiTJkKoQFLY6JwnmbmGZxGSvrb"
  },
  {
    "chainId": 1,
    "address": "0xF0939011a9bb95c3B791f0cb546377Ed2693a574",
    "symbol": "ZERO",
    "name": "Zero Exchange",
    "decimals": 18,
    "logoURI": "ipfs://QmUuVk2vibthq3bZxtanW6Jzk9XfoiNkirutpV9iyWKK3h"
  },
  {
    "chainId": 1,
    "address": "0x0A913beaD80F321E7Ac35285Ee10d9d922659cB7",
    "symbol": "DOS",
    "name": "DOS Network",
    "decimals": 18,
    "logoURI": "ipfs://QmWedgYjEoujYKT95jSkJCNuUVEP6VDr82gRsThBouCw38"
  },
  {
    "chainId": 1,
    "address": "0xB81D70802a816B5DacBA06D708B5acF19DcD436D",
    "symbol": "DEXG",
    "name": "Dextoken",
    "decimals": 18,
    "logoURI": "ipfs://QmQJNrLBmy7pSHok5ySjAMLnS1qs5GopiZDULC3XiJty4A"
  },
  {
    "chainId": 1,
    "address": "0x9992eC3cF6A55b00978cdDF2b27BC6882d88D1eC",
    "symbol": "POLY",
    "name": "Polymath",
    "decimals": 18,
    "logoURI": "ipfs://Qmcdw3smknaNvqgpTduiGDA1gBA8S9CHmxDK7d3pQnzUA8"
  },
  {
    "chainId": 250,
    "address": "0x6c021Ae822BEa943b2E66552bDe1D2696a53fbB7",
    "symbol": "TOMB",
    "name": "Tomb",
    "decimals": 18,
    "logoURI": "ipfs://QmT6E71oE8AHugjW4YduJcM7s44MUqAEE2NVEzEKXgNJHC"
  },
  {
    "chainId": 1,
    "address": "0x11eeF04c884E24d9B7B4760e7476D06ddF797f36",
    "symbol": "MX",
    "name": "MX",
    "decimals": 18,
    "logoURI": "ipfs://QmVGbN7wBk6D4jTkq6YoCYQtJcJT8TcfkqLi3rFBygtamU"
  },
  {
    "chainId": 10,
    "address": "0x1DB2466d9F5e10D7090E7152B68d62703a2245F0",
    "symbol": "SONNE",
    "name": "Sonne Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYtbpRyNJaMzVvZnyB4jRaWdBDgZqm5F5Q6ShPDQ9LPPv"
  },
  {
    "chainId": 1,
    "address": "0xb683D83a532e2Cb7DFa5275eED3698436371cc9f",
    "symbol": "BTU",
    "name": "BTU Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmT4ydZAthdsCZaFZ4opS4GBMouHeEJLppCrLEFq4Ssmhf"
  },
  {
    "chainId": 1,
    "address": "0xeF8bA8cBa86f81B3108f60186FCe9c81B5096D5c",
    "symbol": "YFIIG",
    "name": "YFII Gold",
    "decimals": 18,
    "logoURI": "ipfs://QmYYLcfz2pJBrU8qzKV9zRYBpD9XxCyPHCVtiYb2zZFkwa"
  },
  {
    "chainId": 1,
    "address": "0x2C4e8f2D746113d0696cE89B35F0d8bF88E0AEcA",
    "symbol": "OST",
    "name": "OST",
    "decimals": 18,
    "logoURI": "ipfs://QmR61q5RC2XDMfqJiv83qy6Cb3b3CfKEw9cn46dTo48u69"
  },
  {
    "chainId": 1,
    "address": "0x558EC3152e2eb2174905cd19AeA4e34A23DE9aD6",
    "symbol": "BRD",
    "name": "Bread",
    "decimals": 18,
    "logoURI": "ipfs://QmP6h2J3XL4nLyJST7W92q7YF7npewT1PC1M1m43dYmNuB"
  },
  {
    "chainId": 10,
    "address": "0xb4Bc46bc6cB217B59ea8F4530BaE26Bf69F677f0",
    "symbol": "BEETS",
    "name": "Beethoven X",
    "decimals": 18,
    "logoURI": "ipfs://QmZUxkMhnBeVyFwYWbK9TvbLEmvJVboFCzadNanUXcpDVD"
  },
  {
    "chainId": 1,
    "address": "0xAAf37055188Feee4869dE63464937e683d61b2a1",
    "symbol": "UCN",
    "name": "UChain",
    "decimals": 18,
    "logoURI": "ipfs://QmciaviyMitHtdQPsTZW2cMQ6J8qmkNo312MfjPjuvPy6J"
  },
  {
    "chainId": 56,
    "address": "0x8ea5219a16c2dbF1d6335A6aa0c6bd45c50347C5",
    "symbol": "OOE",
    "name": "OpenOcean",
    "decimals": 18,
    "logoURI": "ipfs://QmbySTwrepiT7a7sYn5ykeMyLnVw8NN8UbB5HBFemKbXFN"
  },
  {
    "chainId": 56,
    "address": "0x3EE2200Efb3400fAbB9AacF31297cBdD1d435D47",
    "symbol": "ADA",
    "name": "Cardano",
    "decimals": 18,
    "logoURI": "ipfs://QmdVJVXbbocbjmnkV38XeFiPp4xjwZvQnvxGSyYzUgnNCD"
  },
  {
    "chainId": 1,
    "address": "0x8dAE6Cb04688C62d939ed9B68d32Bc62e49970b1",
    "symbol": "aCRV",
    "name": "Aave CRV",
    "decimals": 18,
    "logoURI": "ipfs://QmfNsWuRqaNbkWYdsnGvTLzvQLgYGTFB9Gxm7WqwBhmfHA"
  },
  {
    "chainId": 1,
    "address": "0xb1dC9124c395c1e97773ab855d66E879f053A289",
    "symbol": "YAX",
    "name": "yAxis",
    "decimals": 18,
    "logoURI": "ipfs://QmY497J73cPtTkUogAaEEgcFP19CkWpaFE8yNitbYj2TdC"
  },
  {
    "chainId": 56,
    "address": "0x031b41e504677879370e9DBcF937283A8691Fa7f",
    "symbol": "FET",
    "name": "Fetch.ai",
    "decimals": 18,
    "logoURI": "ipfs://Qmf2x78m2kyhE46x8g3Z7eZW8FyRhR8oAVUB9C9JGAwmfT"
  },
  {
    "chainId": 1,
    "address": "0x7c9D8Fb3bDe3D9Ea6e89170618C2dC3d16695D36",
    "symbol": "WRC",
    "name": "WhiteRockCasino",
    "decimals": 18,
    "logoURI": "ipfs://QmfQtmBt7RderkjPLKe1mUfUfGQgPd8S3S2ruozeieq1DR"
  },
  {
    "chainId": 56,
    "address": "0x9Ac983826058b8a9C7Aa1C9171441191232E8404",
    "symbol": "SNX",
    "name": "Synthetix Network",
    "decimals": 18,
    "logoURI": "ipfs://QmX3ov1SgUvUJ64HuEn19RsDkVE91zbUuaCfozReqbbMx5"
  },
  {
    "chainId": 1,
    "address": "0x30680AC0a8A993088223925265fD7a76bEb87E7F",
    "symbol": "ARAW",
    "name": "ARAW",
    "decimals": 18,
    "logoURI": "ipfs://QmNeQNjnuvpiMTQh45YFE5uZxdimMbhEBFbg9xRg7Mk9ZC"
  },
  {
    "chainId": 1,
    "address": "0xc5102fE9359FD9a28f877a67E36B0F050d81a3CC",
    "symbol": "HOP",
    "name": "Hop",
    "decimals": 18,
    "logoURI": "ipfs://QmcFNTzsHbeUeqiCmYdnZUsiWbNFDDx1qtF7n4MGn9YyJ1"
  },
  {
    "chainId": 1,
    "address": "0xc813EA5e3b48BEbeedb796ab42A30C5599b01740",
    "symbol": "NIOX",
    "name": "Autonio",
    "decimals": 4,
    "logoURI": "ipfs://QmVtWDkFBX7zFv2a9Aqrmzn7V4Rpq2gbLN7fQz4Sj8WEsb"
  },
  {
    "chainId": 1,
    "address": "0xA967Dd943B336680540011536E7D8c3d33333515",
    "symbol": "EURL",
    "name": "Lugh",
    "decimals": 6,
    "logoURI": "ipfs://QmYt8MvZYroocRVq1RgazfoS17gDY6WBBTJ1kHsi44HHSH"
  },
  {
    "chainId": 56,
    "address": "0x8FFf93E810a2eDaaFc326eDEE51071DA9d398E83",
    "symbol": "BRISE",
    "name": "Bitgert",
    "decimals": 9,
    "logoURI": "ipfs://QmVUkGuZshiuC4hXbFsGWkR51ND9ANMXQTrwbM6eXrn2o9"
  },
  {
    "chainId": 1,
    "address": "0x0fEd38108bdb8e62ef7b5680E8E0726E2F29e0De",
    "symbol": "iREN",
    "name": "Synthetic Inverse Ren",
    "decimals": 18,
    "logoURI": "ipfs://QmfWQ3RjU7bqjN4vjuyrDgr6MwxwNqqzkEUyRe9Dpv9921"
  },
  {
    "chainId": 56,
    "address": "0x6E88056E8376Ae7709496Ba64d37fa2f8015ce3e",
    "symbol": "DEXE",
    "name": "DeXe",
    "decimals": 18,
    "logoURI": "ipfs://QmZrmTvwVZFmjhZRBdjh1reCPtNenyaQhNS2cD4U18rd2B"
  },
  {
    "chainId": 1,
    "address": "0x7a545Ed3863221A974F327199Ac22F7f12535F11",
    "symbol": "BGTT",
    "name": "Baguette Token",
    "decimals": 18,
    "logoURI": "ipfs://QmdkRKXpSDxNiwvxnvTXK6vB46SZd89Cy7XThfmG8UzoaB"
  },
  {
    "chainId": 56,
    "address": "0x40af3827F39D0EAcBF4A168f8D4ee67c121D11c9",
    "symbol": "TUSD",
    "name": "TrueUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmdcuKBFVn8KEoN6QW7mmGgd38dtF4aL6qTGTqMiDeQfRz"
  },
  {
    "chainId": 1,
    "address": "0xef58b2d5A1b8D3cDE67b8aB054dC5C831E9Bc025",
    "symbol": "crSRM",
    "name": "Cream Serum",
    "decimals": 8,
    "logoURI": "ipfs://QmaaRXzFw2pLDzBdcTS25wPrSHbTvWFcfWHJkQ9RPgSMU7"
  },
  {
    "chainId": 1,
    "address": "0xdB11F6Fc8E16c02719e9c2Eb3c4C762feE8F5C5b",
    "symbol": "NIA",
    "name": "Nydronia",
    "decimals": 18,
    "logoURI": "ipfs://QmeeStVYgifVWzpSbtN963fVK5sUZLEB8UowHy4B3ZwQni"
  },
  {
    "chainId": 1,
    "address": "0x94d863173EE77439E4292284fF13fAD54b3BA182",
    "symbol": "ADEL",
    "name": "Akropolis Delphi",
    "decimals": 18,
    "logoURI": "ipfs://QmQo4PC5st5YmGHsEX3JQmVz28FHBCdrzAP61zySvfU3C4"
  },
  {
    "chainId": 8453,
    "address": "0xf7C1CEfCf7E1dd8161e00099facD3E1Db9e528ee",
    "symbol": "TOWER",
    "name": "TOWER",
    "decimals": 18,
    "logoURI": "ipfs://QmUDvzhLFf3t8cjweZgFFWeqjrprqE11ZyFrySNATaTWwy"
  },
  {
    "chainId": 1,
    "address": "0x45245bc59219eeaAF6cD3f382e078A461FF9De7B",
    "symbol": "BKX",
    "name": "BANKEX",
    "decimals": 18,
    "logoURI": "ipfs://Qma4az7edTUpr8oZJZJvq3RNKLbShovzb4DReubikGjSYM"
  },
  {
    "chainId": 1,
    "address": "0x3ebb4A4e91Ad83BE51F8d596533818b246F4bEe1",
    "symbol": "SATA",
    "name": "Signata",
    "decimals": 18,
    "logoURI": "ipfs://QmU19EwajSFjMRiPdkQ9tYqqjRXPbtTcMs3zadLTYACjwN"
  },
  {
    "chainId": 1,
    "address": "0x71BEFF5533dd4c0a47B739dA50C56A2dd28633fa",
    "symbol": "SHINU",
    "name": "Sheltie Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmfLswrT9R85Buip89nNCDffmMhEPWSRJAbrqPcLYahhWV"
  },
  {
    "chainId": 43114,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmVurySKQi4AcZg3wUXXaudowWcTve6vKMbTHpQU6WExat"
  },
  {
    "chainId": 42161,
    "address": "0xd4d42F0b6DEF4CE0383636770eF773390d85c61A",
    "symbol": "SUSHI",
    "name": "Sushi",
    "decimals": 18,
    "logoURI": "ipfs://QmWmSKdZMf2kjte9Drd7AUX5VTj9FF9bSvbctgawzAcxet"
  },
  {
    "chainId": 1,
    "address": "0xd0929d411954c47438dc1d871dd6081F5C5e149c",
    "symbol": "RFR",
    "name": "Refereum",
    "decimals": 4,
    "logoURI": "ipfs://QmNj5Q3Y2wb9MEj3P3vnnGRuKsWrzcnYjcLfMtmXjbyF5s"
  },
  {
    "chainId": 1,
    "address": "0xfF1D3d9ddfF8fA05c064349f118bb3D95BA65c85",
    "symbol": "LVLUP",
    "name": "Level Up",
    "decimals": 18,
    "logoURI": "ipfs://QmRfYWDfZQqLSynPJ3ELt4Vzj5dK9fcQUmXqqo2MZ3Nqk7"
  },
  {
    "chainId": 1,
    "address": "0x15bCDFAd12498DE8a922E62442Ae4CC4bd33bd25",
    "symbol": "WALT",
    "name": "Walletreum",
    "decimals": 18,
    "logoURI": "ipfs://QmXdvbP45BoCfErNHbx5AF7pSGcYQ2e9KaTGNdediiWD5p"
  },
  {
    "chainId": 1,
    "address": "0x22dE9912cd3D74953B1cd1F250B825133cC2C1b3",
    "symbol": "DREP",
    "name": "DREP",
    "decimals": 18,
    "logoURI": "ipfs://QmQ4aqFVgUXBqigWWXEhiEk9Czdq9rLdEyj9sSTEEJb83k"
  },
  {
    "chainId": 1,
    "address": "0x22222C03318440305aC3e8a7820563d6A9FD777F",
    "symbol": "CLV",
    "name": "Clover",
    "decimals": 6,
    "logoURI": "ipfs://QmSQTbUVitujrnbnLdMvn3QfNdq1XN5Ci7jJgoNrH6spYJ"
  },
  {
    "chainId": 1,
    "address": "0xF48e200EAF9906362BB1442fca31e0835773b8B4",
    "symbol": "sAUD",
    "name": "Synthetic Australian Dollars",
    "decimals": 18,
    "logoURI": "ipfs://Qmf84wPWrp8WA7NP72VCvABhPjrvj1qGFPmUx8GKc4bQrF"
  },
  {
    "chainId": 1,
    "address": "0x72dD4b6bd852A3AA172Be4d6C5a6dbEc588cf131",
    "symbol": "NGC",
    "name": "NAGA",
    "decimals": 18,
    "logoURI": "ipfs://QmcfYyvqP453WtmcLnNu425KLPo3XZW25hySWjdfPrhCKF"
  },
  {
    "chainId": 10,
    "address": "0xe0BB0D3DE8c10976511e5030cA403dBf4c25165B",
    "symbol": "0xBTC",
    "name": "0xBitcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmTGhWrUFnhEC6zSsjpioqYVSTD7sMAc1VVWadjny46DDX"
  },
  {
    "chainId": 1,
    "address": "0x57e114B691Db790C35207b2e685D4A43181e6061",
    "symbol": "ENA",
    "name": "Ethena",
    "decimals": 18,
    "logoURI": "ipfs://QmZhXmHH2NRn39BQqC42BKZ7ADCDYT4jRmnPsduhUuQL1H"
  },
  {
    "chainId": 1,
    "address": "0x536381a8628dBcC8C70aC9A30A7258442eAb4c92",
    "symbol": "PAN",
    "name": "Pantos",
    "decimals": 8,
    "logoURI": "ipfs://QmSn1hwenF39cZGx5on9rs3kBaSYHJiGF6GuXSpZa7Y7GF"
  },
  {
    "chainId": 1,
    "address": "0xff56Cc6b1E6dEd347aA0B7676C85AB0B3D08B0FA",
    "symbol": "ORBS",
    "name": "Orbs",
    "decimals": 18,
    "logoURI": "ipfs://QmQ5aWfNnNHknkBkwWh8wceWcbpH7shyZhLdZ25BLDmgYV"
  },
  {
    "chainId": 1,
    "address": "0xF6b1C627e95BFc3c1b4c9B825a032Ff0fBf3e07d",
    "symbol": "sJPY",
    "name": "Synthetic Japanese Yen",
    "decimals": 18,
    "logoURI": "ipfs://QmcjJVo1CtmkPQExVUL91u8rRn4iX2aj6RhPAbZ3NRmi7p"
  },
  {
    "chainId": 1,
    "address": "0xB9e7F8568e08d5659f5D29C4997173d84CdF2607",
    "symbol": "SWT",
    "name": "Swarm City",
    "decimals": 18,
    "logoURI": "ipfs://QmTRuupe2YWjcGpCEjKRMLQwiokib2X5UiJ2bzzzeKDjwa"
  },
  {
    "chainId": 1,
    "address": "0xb6c4267C4877BB0D6b1685Cfd85b0FBe82F105ec",
    "symbol": "REL",
    "name": "Relevant",
    "decimals": 18,
    "logoURI": "ipfs://QmP24eMbf8Mb5LD6GBwYWJ9TCSmNwDyAksW6BWTwLkpMk1"
  },
  {
    "chainId": 1,
    "address": "0xD06527D5e56A3495252A528C4987003b712860eE",
    "symbol": "crETH",
    "name": "Cream Ether",
    "decimals": 8,
    "logoURI": "ipfs://QmeJVCLRE8ZG1DKT3w3qGWDrEBGWaFGedHcoh8q66TaPK8"
  },
  {
    "chainId": 1,
    "address": "0x2dFAc6327Dce9F88E31fF46de13049eca5a7DfcB",
    "symbol": "SHIBERUS",
    "name": "Shiberus Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmVpQ74df2joAvC5q9ZiZgYsN6ztNvM5z3jr438mT4aEBQ"
  },
  {
    "chainId": 1,
    "address": "0x6De037ef9aD2725EB40118Bb1702EBb27e4Aeb24",
    "symbol": "RNDR",
    "name": "Render",
    "decimals": 18,
    "logoURI": "ipfs://Qmd4tMCvhQSdoBkxJbqBxVviGQfnMiuZWF1WA3XXRnarfa"
  },
  {
    "chainId": 1,
    "address": "0xCB98f42221b2C251A4E74A1609722eE09f0cc08E",
    "symbol": "iDASH",
    "name": "Synthetic Inverse Dash",
    "decimals": 18,
    "logoURI": "ipfs://QmXATer7itBiwRzy3SVD7M9puJKwNUCyfgnDkmcR41ejcE"
  },
  {
    "chainId": 1,
    "address": "0xf3AFDC2525568Ffe743801C8C54bdeA1704c9adb",
    "symbol": "TURTLE",
    "name": "Turtle",
    "decimals": 18,
    "logoURI": "ipfs://Qmf4J5EitTc9iH47PNaJzc4Tq6QeDmtbSZQkm5aHcZd9qE"
  },
  {
    "chainId": 250,
    "address": "0xaD996A45fd2373ed0B10Efa4A8eCB9de445A4302",
    "symbol": "ALPACA",
    "name": "Alpaca Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmX52kb3uELLC5yagoxYdWqoqnVkvNjtJRUqh8v7vFsYgr"
  },
  {
    "chainId": 1,
    "address": "0xfd0205066521550D7d7AB19DA8F72bb004b4C341",
    "symbol": "LIT",
    "name": "Liquidity Incentive Token",
    "decimals": 18,
    "logoURI": "ipfs://QmedXC9TiwpFe8w91zk6TaWdTnZi2YsHu5W9xawRSXqxuE"
  },
  {
    "chainId": 1,
    "address": "0x0327112423F3A68efdF1fcF402F6c5CB9f7C33fd",
    "symbol": "BTC++",
    "name": "PieDAO BTC++",
    "decimals": 18,
    "logoURI": "ipfs://QmVh2f9SF2Z2Jd3yQ9CzCP2vMwzk547pRWtDJEuuzv6pPL"
  },
  {
    "chainId": 1,
    "address": "0x6DEA81C8171D0bA574754EF6F8b412F2Ed88c54D",
    "symbol": "LQTY",
    "name": "Liquity",
    "decimals": 18,
    "logoURI": "ipfs://Qmd7h4JMCtYvAk1mN25dYu2UuoS9ukQkBSuxErv92s1JKh"
  },
  {
    "chainId": 137,
    "address": "0xE5417Af564e4bFDA1c483642db72007871397896",
    "symbol": "GNS",
    "name": "Gains Network",
    "decimals": 18,
    "logoURI": "ipfs://QmbGho8kZ8QDJMLXYvejvvGpnJnMXqhHn4QomceTy4zsVi"
  },
  {
    "chainId": 137,
    "address": "0xcB059C5573646047D6d88dDdb87B745C18161d3b",
    "symbol": "POLY",
    "name": "Polymath",
    "decimals": 18,
    "logoURI": "ipfs://Qmcdw3smknaNvqgpTduiGDA1gBA8S9CHmxDK7d3pQnzUA8"
  },
  {
    "chainId": 1,
    "address": "0x246908BfF0b1ba6ECaDCF57fb94F6AE2FcD43a77",
    "symbol": "DIVI",
    "name": "Divi",
    "decimals": 8,
    "logoURI": "ipfs://Qmb6evvEXe8pzd8MWetc34v3mpb5eqVNc6z66z8x5Y3uwD"
  },
  {
    "chainId": 1,
    "address": "0xa06bC25B5805d5F8d82847D191Cb4Af5A3e873E0",
    "symbol": "aLINK",
    "name": "Aave LINK",
    "decimals": 18,
    "logoURI": "ipfs://QmUShEVQkaihBEgrAHx5qbEewpWuVTCShPvKpCb9XCLtM6"
  },
  {
    "chainId": 100,
    "address": "0x9C58BAcC331c9aa871AFD802DB6379a98e80CEdb",
    "symbol": "GNO",
    "name": "Gnosis",
    "decimals": 18,
    "logoURI": "ipfs://QmZBqLt3N7zvakLJS7jYx3D9D1TpjdPfwYhZhhGtikoiRv"
  },
  {
    "chainId": 1,
    "address": "0xeABACD844A196D7Faf3CE596edeBF9900341B420",
    "symbol": "sCEX",
    "name": "Synthetic Centralised Exchange Index",
    "decimals": 18,
    "logoURI": "ipfs://QmT9RXFQPHFqngcE2tHNCYM2ydG4ZqaRnX4GGmjTzm4cvB"
  },
  {
    "chainId": 1,
    "address": "0x99a3a5333b4Eb80192149F9eDd42d1D57AC591ee",
    "symbol": "CELT",
    "name": "CELT",
    "decimals": 18,
    "logoURI": "ipfs://QmWAqix1xFE8WB34Mbw3FrbX6kzyeN34CrLigqEJXZoqwB"
  },
  {
    "chainId": 1,
    "address": "0xc00e94Cb662C3520282E6f5717214004A7f26888",
    "symbol": "COMP",
    "name": "Compound",
    "decimals": 18,
    "logoURI": "ipfs://QmZnPax5bdmAKY4zYgcQHwP1xQxVVqNQQwFqhPRmyY3F84"
  },
  {
    "chainId": 1,
    "address": "0x02D3A27Ac3f55d5D91Fb0f52759842696a864217",
    "symbol": "IONX",
    "name": "Charged Particles",
    "decimals": 18,
    "logoURI": "ipfs://QmbpH9YQ6rmT4GTNk2B4pbEFX5rJdAxKW1uuwgFSpWz9vk"
  },
  {
    "chainId": 324,
    "address": "0x3355df6D4c9C3035724Fd0e3914dE96A5a83aaf4",
    "symbol": "USDC",
    "name": "USDC",
    "decimals": 6,
    "logoURI": "ipfs://QmS7rV584ATvEk5Skugr78GyAEFBuEo5eMp2rDNRCRNSmr"
  },
  {
    "chainId": 56,
    "address": "0xbF7c81FFF98BbE61B40Ed186e4AfD6DDd01337fe",
    "symbol": "EGLD",
    "name": "MultiversX",
    "decimals": 18,
    "logoURI": "ipfs://QmVSMNvzzMZHJUhQQeYkuzgmh989XiBEdLZqhRgXmTeeSU"
  },
  {
    "chainId": 1,
    "address": "0x3212b29E33587A00FB1C83346f5dBFA69A458923",
    "symbol": "imBTC",
    "name": "The Tokenized Bitcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmTSDst4wxoar338wNt7mB62XjtvfuXvxus2mTCL4NSJE9"
  },
  {
    "chainId": 56,
    "address": "0xfb5B838b6cfEEdC2873aB27866079AC55363D37E",
    "symbol": "FLOKI",
    "name": "FLOKI",
    "decimals": 9,
    "logoURI": "ipfs://QmRiPbpWgyXiCPCvR3qKCMpDaEeeZyCGJ9LXrEYM8PH4ad"
  },
  {
    "chainId": 1,
    "address": "0xBA5F11b16B155792Cf3B2E6880E8706859A8AEB6",
    "symbol": "ARN",
    "name": "Aeron",
    "decimals": 8,
    "logoURI": "ipfs://QmQHw7zmjmofgk5WVr8SbK3JquopvfdLhVmXwPSmzMMtYw"
  },
  {
    "chainId": 1,
    "address": "0xdcDc1c1cC33AA817CbDBe8F5E2390BF7cc43dc4B",
    "symbol": "AVASTR",
    "name": "AVASTR Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmZDbbwc2LferUBrnhj5hMUh3jSSKQmovaYoQc9qnUmdiR"
  },
  {
    "chainId": 1,
    "address": "0xF5cFBC74057C610c8EF151A439252680AC68c6DC",
    "symbol": "OCT",
    "name": "Octopus Network",
    "decimals": 18,
    "logoURI": "ipfs://QmfMH7XmPdSUNnbxneVz2uJQDBbAerAyPR42kozLDWT6HQ"
  },
  {
    "chainId": 56,
    "address": "0x471Ea49dd8E60E697f4cac262b5fafCc307506e4",
    "symbol": "KOM",
    "name": "Kommunitas",
    "decimals": 8,
    "logoURI": "ipfs://QmP7hhJjTh4iwnJAvYBUb2cizKjjk3GiCAjuEuYby5fkn9"
  },
  {
    "chainId": 56,
    "address": "0xD06716E1Ff2E492Cc5034c2E81805562dd3b45fa",
    "symbol": "MGP",
    "name": "Magpie",
    "decimals": 18,
    "logoURI": "ipfs://Qmcmn9X1UPVr1xqoF3tJCN29qfCwLVKCyDPWa9p18ZzsiX"
  },
  {
    "chainId": 1,
    "address": "0xfF69e48af1174Da7F15D0c771861c33d3f19eD8a",
    "symbol": "TESINU",
    "name": "TESLA INU",
    "decimals": 9,
    "logoURI": "ipfs://QmYCRVJ5tRqmgaULiaGJo2urv5B3ei5mFMUAL1vbUQQxsu"
  },
  {
    "chainId": 1,
    "address": "0xFE3E6a25e6b192A42a44ecDDCd13796471735ACf",
    "symbol": "REEF",
    "name": "Reef",
    "decimals": 18,
    "logoURI": "ipfs://QmQ1fDAcrwBbbzrLNaj5nysDDudf5PdaW7RS3DM6yJyXE3"
  },
  {
    "chainId": 1,
    "address": "0x3c3a81e81dc49A522A592e7622A7E711c06bf354",
    "symbol": "MNT",
    "name": "Mantle",
    "decimals": 18,
    "logoURI": "ipfs://QmT52GuneX7YJcb9HL9e5yoRjQwSVGPKpXQqE4gMJkvN4G"
  },
  {
    "chainId": 42161,
    "address": "0x3E6648C5a70A150A88bCE65F4aD4d506Fe15d2AF",
    "symbol": "SPELL",
    "name": "Spell",
    "decimals": 18,
    "logoURI": "ipfs://QmRQ87x9VSD3iyvoqYXqVAcKGUAmUwYu5RiuqExNkjYyfa"
  },
  {
    "chainId": 56,
    "address": "0xd691d9a68C887BDF34DA8c36f63487333ACfD103",
    "symbol": "MAV",
    "name": "Maverick Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmYkDjrskzi8aUwXCMgy7LhQdUttqi3y9yzZjxG5Zx238t"
  },
  {
    "chainId": 1,
    "address": "0xaD6A626aE2B43DCb1B39430Ce496d2FA0365BA9C",
    "symbol": "DEFI+S",
    "name": "PieDAO DEFI Small Cap",
    "decimals": 18,
    "logoURI": "ipfs://QmVmiuffy7AZ3x2hdByLsJWNVfm2f3Ed4aMyHUB6nASCjy"
  },
  {
    "chainId": 1,
    "address": "0x1c4481750daa5Ff521A2a7490d9981eD46465Dbd",
    "symbol": "BCPT",
    "name": "Blockmason Credit Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmQanYu4muPqpACuHJN4ffdWeqd1yZTjAJ3Gj2eSZXcqqu"
  },
  {
    "chainId": 56,
    "address": "0x2723522702093601e6360CAe665518C4f63e9dA6",
    "symbol": "LINK3",
    "name": "Link3",
    "decimals": 0,
    "logoURI": "ipfs://Qmcuey2f8F3R57rw6a9aVK9bSqBxDZoFMPMXBKPaKCBGCf"
  },
  {
    "chainId": 1,
    "address": "0x47daC6BD80f024575a6d367aF5Ba8e89202A09fc",
    "symbol": "OXO",
    "name": "OXO.Farm",
    "decimals": 18,
    "logoURI": "ipfs://QmZRf9Aww2CK6NYeFkUiLVZdL4wb6DC2AFdmoGdNQTJ7zM"
  },
  {
    "chainId": 42161,
    "address": "0x0c5fa0E07949F941A6c2C29a008252db1527d6EE",
    "symbol": "OPUL",
    "name": "Opulous",
    "decimals": 18,
    "logoURI": "ipfs://QmbCckR4APFcStuAbLRPey3hxqpqHaLgoRxiYuXmWKA1vU"
  },
  {
    "chainId": 56,
    "address": "0x88D7e9B65dC24Cf54f5eDEF929225FC3E1580C25",
    "symbol": "JMPT",
    "name": "JumpToken",
    "decimals": 18,
    "logoURI": "ipfs://QmZnPm3t5ECmafB4Ppiu5aH8Rd8XDirRBe9czovVoUiM5c"
  },
  {
    "chainId": 42161,
    "address": "0x23ee2343B892b1BB63503a4FAbc840E0e2C6810f",
    "symbol": "AXL",
    "name": "Axelar",
    "decimals": 6,
    "logoURI": "ipfs://QmcSRWc1LN1mk4tjy38kyTMqqVx7j4Dmnkw3dsW7efPMTL"
  },
  {
    "chainId": 1,
    "address": "0xa685a61171bb30d4072B338c80Cb7b2c865c873E",
    "symbol": "aMANA",
    "name": "Aave MANA",
    "decimals": 18,
    "logoURI": "ipfs://QmaW48tqvvGypz5Y4PnU8eUaMstcshujRuUrpFVSngmUdh"
  },
  {
    "chainId": 1,
    "address": "0xBd56E9477Fc6997609Cf45F84795eFbDAC642Ff1",
    "symbol": "iREP",
    "name": "Fulcrum REP iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmSdtc2i8aLKi4MZpVbXdsxmETArt4DVWQf9k1E5XLSbjN"
  },
  {
    "chainId": 1,
    "address": "0x4A42d2c580f83dcE404aCad18dab26Db11a1750E",
    "symbol": "RLX",
    "name": "Relex",
    "decimals": 18,
    "logoURI": "ipfs://QmcHJejRH8EkvFiP8N6oXGvXx4mwtfzBwnLXXVYsLUR9TJ"
  },
  {
    "chainId": 100,
    "address": "0xD057604A14982FE8D88c5fC25Aac3267eA142a08",
    "symbol": "HOPR",
    "name": "HOPR",
    "decimals": 18,
    "logoURI": "ipfs://QmatpCk3VBM241MPEx8D6fMKSyHXpwTi7ua2z5MuzFdR7P"
  },
  {
    "chainId": 56,
    "address": "0x316622977073BBC3dF32E7d2A9B3c77596a0a603",
    "symbol": "jBRL",
    "name": "Jarvis Synthetic Brazilian Real",
    "decimals": 18,
    "logoURI": "ipfs://QmRxdK4TQ9NE92nZX9PkWZRx8Fe4qHsRpCng35fU8uRKrm"
  },
  {
    "chainId": 42161,
    "address": "0x509FD25EE2AC7833a017f17Ee8A6Fb4aAf947876",
    "symbol": "mWOM",
    "name": "Magpie WOM",
    "decimals": 18,
    "logoURI": "ipfs://QmcLq2WDJcR8FPSw2guBFpCJaA3PPd8rPvdquCUHVwF48u"
  },
  {
    "chainId": 56,
    "address": "0xd21d29B38374528675C34936bf7d5Dd693D2a577",
    "symbol": "PRQ",
    "name": "Parsiq",
    "decimals": 18,
    "logoURI": "ipfs://QmahEpMtGqvbmzfn33yjRNuJdivmJfAmJcyURi5bX88D2J"
  },
  {
    "chainId": 1,
    "address": "0x26CE25148832C04f3d7F26F32478a9fe55197166",
    "symbol": "DEXT",
    "name": "DEXTools",
    "decimals": 18,
    "logoURI": "ipfs://QmXUKMaBYXngZgD56Aae4aBrTGGuJHb7EqsReFftZBhKkt"
  },
  {
    "chainId": 1,
    "address": "0xBE0C826f17680d8Da620855bE89DD6544C034cA1",
    "symbol": "BANKETH",
    "name": "BankEth",
    "decimals": 18,
    "logoURI": "ipfs://QmXMUBE7xSKthwAYkXbvTkSsZ5eefCHgsMzV19PnjTod3a"
  },
  {
    "chainId": 1,
    "address": "0x524Ebc93BeEf838F70b4Ae54b675d3e971D5884E",
    "symbol": "HERO",
    "name": "Challenge Coin",
    "decimals": 9,
    "logoURI": "ipfs://QmemZEWnmnUPaEhTjXWSiDEjS3eZxAE4xUuMR9s1yLkTcE"
  },
  {
    "chainId": 10,
    "address": "0x67CCEA5bb16181E7b4109c9c2143c24a1c2205Be",
    "symbol": "FXS",
    "name": "Frax Share",
    "decimals": 18,
    "logoURI": "ipfs://QmNRaYxGN6QHc6RKZfmaaKX6DJGtj6W7JeC4bDRroKGUCQ"
  },
  {
    "chainId": 1,
    "address": "0xcce63fD31e9053c110c74CEbc37C8e358A6AA5bD",
    "symbol": "BTR",
    "name": "Bitriver",
    "decimals": 4,
    "logoURI": "ipfs://QmUs9byhhXLEB7kQGZpKqdLCewybNEp6jD6Jfn3pNpn544"
  },
  {
    "chainId": 137,
    "address": "0x0b3F868E0BE5597D5DB7fEB59E1CADBb0fdDa50a",
    "symbol": "SUSHI",
    "name": "Sushi",
    "decimals": 18,
    "logoURI": "ipfs://QmT48vXNDWa4KUUk6ZsYk8Z4QwNqjk9xaqpbpQvMFz1xFC"
  },
  {
    "chainId": 1,
    "address": "0xBcca60bB61934080951369a648Fb03DF4F96263C",
    "symbol": "aUSDC",
    "name": "Aave USDC",
    "decimals": 6,
    "logoURI": "ipfs://QmSJZVFFZJDbhaJFqEE3VBfSMCmT476y6qcevLVDgiCBUo"
  },
  {
    "chainId": 42161,
    "address": "0x3269a3C00AB86c753856fD135d97b87FACB0d848",
    "symbol": "FFM",
    "name": "Florence Finance Medici",
    "decimals": 18,
    "logoURI": "ipfs://QmWaNX4CxQrMNqqJQaJjGXXb3WdBAd9Kv4muipTXSkAQH3"
  },
  {
    "chainId": 10,
    "address": "0x9Bcef72be871e61ED4fBbc7630889beE758eb81D",
    "symbol": "rETH",
    "name": "Rocket Pool ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmRqLBdZaA2oSr7yWVmwdSyj5HwixHMWRpyKw8QyAQTgRz"
  },
  {
    "chainId": 137,
    "address": "0xEe327F889d5947c1dc1934Bb208a1E792F953E96",
    "symbol": "frxETH",
    "name": "Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmQqQhtwApaT4X9FAaAzBRvyeE2iaSDH4ahD6exWXcszZU"
  },
  {
    "chainId": 1,
    "address": "0xf6c0aA7eBFE9992200C67E5388E4F42da49E1783",
    "symbol": "USD1",
    "name": "Psyche",
    "decimals": 2,
    "logoURI": "ipfs://QmXq4YyV7jLGDyvXUAp6webBAtGAGTasu4Gx41Li1geGq4"
  },
  {
    "chainId": 1,
    "address": "0x4B4Eb5C44D50Bfd44124688c6754633f7e258b01",
    "symbol": "SGB",
    "name": "SubGame",
    "decimals": 8,
    "logoURI": "ipfs://QmQwh4xGrWTh7Q1UYxNArPs4fzQ5H5i5bRAUp4n9TBGZbZ"
  },
  {
    "chainId": 137,
    "address": "0x8343091F2499FD4b6174A46D067A920a3b851FF9",
    "symbol": "jJPY",
    "name": "Jarvis Synthetic Japanese Yen",
    "decimals": 18,
    "logoURI": "ipfs://QmechN91MXJW1tX3pks96uSNAvRbP3npPJhiyadChV8wsX"
  },
  {
    "chainId": 56,
    "address": "0x3aD9594151886Ce8538C1ff615EFa2385a8C3A88",
    "symbol": "SAFEMARS",
    "name": "SafeMars",
    "decimals": 9,
    "logoURI": "ipfs://QmSJPNnETWhdSfF1JvTsQwZoWzT3W1ZQvrUyzWrVgS4NGA"
  },
  {
    "chainId": 1,
    "address": "0x4E0fCa55a6C3A94720ded91153A27F60E26B9AA8",
    "symbol": "BOOST",
    "name": "Boost",
    "decimals": 18,
    "logoURI": "ipfs://QmYhLZYf9TdCRJVCB9bzhSaLfpGZfVy5NRYctBAetBcZ23"
  },
  {
    "chainId": 10,
    "address": "0x6c84a8f1c29108F47a79964b5Fe888D4f4D0dE40",
    "symbol": "tBTC",
    "name": "tBTC",
    "decimals": 18,
    "logoURI": "ipfs://QmRMESJHxFZWSS5KuMj5fW7SqEdkVvEATYVWkRRk6y2VeX"
  },
  {
    "chainId": 1,
    "address": "0x30365Ed4Ca8173013ad948b9842f34ac71d01f7C",
    "symbol": "DHS",
    "name": "Dirham Crypto",
    "decimals": 18,
    "logoURI": "ipfs://QmS2LRag2hd12KwxZQ2eaJ7pUfXYRFsZRwztTkcWDPJ2w1"
  },
  {
    "chainId": 42161,
    "address": "0x912CE59144191C1204E64559FE8253a0e49E6548",
    "symbol": "ARB",
    "name": "Arbitrum",
    "decimals": 18,
    "logoURI": "ipfs://QmZTMxDWvgVfYhuvVaE1opHmgknz26AcauHU3k2wGVDYkv"
  },
  {
    "chainId": 56,
    "address": "0x88f1A5ae2A3BF98AEAF342D26B30a79438c9142e",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRQq6tqSDu6pvZPmYHhZsF9dvUqzudAnfn6cKPurWzQbF"
  },
  {
    "chainId": 1,
    "address": "0xC14103C2141E842e228FBaC594579e798616ce7A",
    "symbol": "sLTC",
    "name": "Synthetic Litecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmScVEbRzwJy95EyH84y4q2GunZbm97yKqwsAzcwSCcZb9"
  },
  {
    "chainId": 1,
    "address": "0xeEeEEb57642040bE42185f49C52F7E9B38f8eeeE",
    "symbol": "ELK",
    "name": "Elk Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYSmFMg8incix4Hzs85MPZzQJnb2W19zLTL3nXNEzY42m"
  },
  {
    "chainId": 1,
    "address": "0x9b53E429B0baDd98ef7F01F03702986c516a5715",
    "symbol": "HY",
    "name": "hybrix",
    "decimals": 18,
    "logoURI": "ipfs://QmXQRrS4kPPyQCK8vFhj9s4eShpbdkX8Jv2Lc68QP9YqTT"
  },
  {
    "chainId": 1,
    "address": "0xADE00C28244d5CE17D72E40330B1c318cD12B7c3",
    "symbol": "ADX",
    "name": "AdEx",
    "decimals": 18,
    "logoURI": "ipfs://QmWZ1tZ6itV3HTGMFoAMbPJk1D3rqcW1XP8xBxWm4DdAJf"
  },
  {
    "chainId": 1,
    "address": "0x030bA81f1c18d280636F32af80b9AAd02Cf0854e",
    "symbol": "aWETH",
    "name": "Aave WETH",
    "decimals": 18,
    "logoURI": "ipfs://QmfKQx4eRXG6JdtdTBr9qTroHLAhrEWbCphocpeMwKh1Dz"
  },
  {
    "chainId": 1,
    "address": "0x79da1431150C9b82D2E5dfc1C68B33216846851e",
    "symbol": "iLTC",
    "name": "Synthetic Inverse Litecoin",
    "decimals": 18,
    "logoURI": "ipfs://Qmc62QgS8ugkbLtBwDoFdH84g3yXLKrJPLYVFbi1WjCTsy"
  },
  {
    "chainId": 1,
    "address": "0xd7631787B4dCc87b1254cfd1e5cE48e96823dEe8",
    "symbol": "SCL",
    "name": "Sociall",
    "decimals": 8,
    "logoURI": "ipfs://QmZdJfXBxHgHExYsMDKr5CzcTB5SpoefeKRrC6xeCqF4qt"
  },
  {
    "chainId": 1,
    "address": "0xaA7a9CA87d3694B5755f213B5D04094b8d0F0A6F",
    "symbol": "TRAC",
    "name": "OriginTrail",
    "decimals": 18,
    "logoURI": "ipfs://QmbiR2dTfi3UeHGtUuLH37gyc6AVqaFsYq9RbZXywBzvGV"
  },
  {
    "chainId": 137,
    "address": "0x23D29D30e35C5e8D321e1dc9A8a61BFD846D4C5C",
    "symbol": "HEX",
    "name": "HEX",
    "decimals": 8,
    "logoURI": "ipfs://QmTWVWF3xe5UHtZpdeMVbvTg6Brk184ngqzErei8CcMF3x"
  },
  {
    "chainId": 1,
    "address": "0xD533a949740bb3306d119CC777fa900bA034cd52",
    "symbol": "CRV",
    "name": "Curve DAO Token",
    "decimals": 18,
    "logoURI": "ipfs://QmVaPBFT3DHqGXXEpBuj8CKcb56vyKzgNBUXoTkyMToXsw"
  },
  {
    "chainId": 137,
    "address": "0x4E8dc2149EaC3f3dEf36b1c281EA466338249371",
    "symbol": "INJ",
    "name": "Injective",
    "decimals": 18,
    "logoURI": "ipfs://QmWJAE68L5VZgJnn3QPj9UdckFPJjZX6inTVdHs6CQB1hX"
  },
  {
    "chainId": 1,
    "address": "0xcA2796F9F61dc7b238Aab043971e49c6164DF375",
    "symbol": "YEED",
    "name": "YGGDRASH",
    "decimals": 18,
    "logoURI": "ipfs://QmSTQWvMT3xYs3L9tZPgowWNsfWveVNgvVfvZtV8m5cjcg"
  },
  {
    "chainId": 137,
    "address": "0xE06Bd4F5aAc8D0aA337D13eC88dB6defC6eAEefE",
    "symbol": "IXT",
    "name": "Planet IX",
    "decimals": 18,
    "logoURI": "ipfs://QmdhrPzZ6Qis17qHVwGN3zk7GYa2dVUmAvVdENUYpHzUrJ"
  },
  {
    "chainId": 1,
    "address": "0xe0955F26515d22E347B17669993FCeFcc73c3a0a",
    "symbol": "STACK",
    "name": "Stacker Ventures",
    "decimals": 18,
    "logoURI": "ipfs://Qmbr8EdZru9NaP4w6CS2HbCPFv3o1foo9oQdUvgGETZMZw"
  },
  {
    "chainId": 1,
    "address": "0x1c48f86ae57291F7686349F12601910BD8D470bb",
    "symbol": "USDK",
    "name": "USDK",
    "decimals": 18,
    "logoURI": "ipfs://QmQz8NuhLJsh3xkYo5zYVn1EYDKZJTBpzwYFsoLgRGwvim"
  },
  {
    "chainId": 1,
    "address": "0xC631bE100F6Cf9A7012C23De5a6ccb990EAFC133",
    "symbol": "BTFA",
    "name": "Banana Task Force Ape",
    "decimals": 9,
    "logoURI": "ipfs://QmQVZ5uTPMcFf5Nmi2GAJufehXeCoQfbadY6zNSd7qYySc"
  },
  {
    "chainId": 10,
    "address": "0xC4a65A93dD6CD9717551ebE827E8baEe025d1d7E",
    "symbol": "PNP",
    "name": "Penpie",
    "decimals": 18,
    "logoURI": "ipfs://QmSD3cGiAXwRujKBgGt5ZHTK9d1Dqj34GRB7jU6gMbwD2Y"
  },
  {
    "chainId": 56,
    "address": "0x56b6fB708fC5732DEC1Afc8D8556423A2EDcCbD6",
    "symbol": "EOS",
    "name": "EOS",
    "decimals": 18,
    "logoURI": "ipfs://QmT2F2AT537N5nvCTkwWwvy5DDY5xPP5nkdpiTtJbj5KTc"
  },
  {
    "chainId": 1,
    "address": "0xEB9951021698B42e4399f9cBb6267Aa35F82D59D",
    "symbol": "LIF",
    "name": "L\u00edf",
    "decimals": 18,
    "logoURI": "ipfs://QmaP4vZYZ5kej8uSyBMAyzMXcYgJ7snVhFA2NgGQp3Nkqz"
  },
  {
    "chainId": 1,
    "address": "0xaf9f549774ecEDbD0966C52f250aCc548D3F36E5",
    "symbol": "RFuel",
    "name": "Rio DeFi",
    "decimals": 18,
    "logoURI": "ipfs://QmfSjt6fcu5Dj3K7zDbDrdioYUWC89X61pauuyabfW4r2j"
  },
  {
    "chainId": 1,
    "address": "0x103c3A209da59d3E7C4A89307e66521e081CFDF0",
    "symbol": "GVT",
    "name": "Genesis Vision",
    "decimals": 18,
    "logoURI": "ipfs://QmQhTtVGR9qiPCr3tSMDJVhxZiHc5hU1YoGXiaC11mLAPL"
  },
  {
    "chainId": 100,
    "address": "0x4ECaBa5870353805a9F068101A40E0f32ed605C6",
    "symbol": "USDT",
    "name": "Tether ",
    "decimals": 6,
    "logoURI": "ipfs://Qmd9YVdtaXDZuMmeChdpK4LfizyLkBbVijpLyXGSNwFGEz"
  },
  {
    "chainId": 42161,
    "address": "0xFa7F8980b0f1E64A2062791cc3b0871572f1F7f0",
    "symbol": "UNI",
    "name": "Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmbSptJsjBStRQ7tpzdq8DAnaT4cgaKbpHKTLLtsVN5n1Q"
  },
  {
    "chainId": 1,
    "address": "0xF433089366899D83a9f26A773D59ec7eCF30355e",
    "symbol": "MTL",
    "name": "Metal",
    "decimals": 8,
    "logoURI": "ipfs://QmPKro6jBGT8VpmsjSigZBjE6Hu1Z1X4zXRpAWMfUYzcw2"
  },
  {
    "chainId": 137,
    "address": "0xBbba073C31bF03b8ACf7c28EF0738DeCF3695683",
    "symbol": "SAND",
    "name": "The Sandbox",
    "decimals": 18,
    "logoURI": "ipfs://QmUNcno8wSiqwGunX6TkLYdQKs9PybBJ7t1kTJBkg9qstL"
  },
  {
    "chainId": 1,
    "address": "0x1602af2C782cC03F9241992E243290Fccf73Bb13",
    "symbol": "QBIT",
    "name": "Qubitica",
    "decimals": 18,
    "logoURI": "ipfs://QmX15siiH4JGdakKfagABfjeQwashpuzrqNZp5Hd2Qsvq9"
  },
  {
    "chainId": 1,
    "address": "0x1BBf25e71EC48B84d773809B4bA55B6F4bE946Fb",
    "symbol": "VOW",
    "name": "Vow",
    "decimals": 18,
    "logoURI": "ipfs://QmREAWi8E9DecQCn64FwVbMrJPHBFo3zMGaKVRoDTpZFnq"
  },
  {
    "chainId": 56,
    "address": "0xa3f020a5C92e15be13CAF0Ee5C95cF79585EeCC9",
    "symbol": "ELF",
    "name": "aelf",
    "decimals": 18,
    "logoURI": "ipfs://QmTTuFgr4oiBwq5g77rFTCDYC2xEQ9wNTvcu5JqdiGbWvi"
  },
  {
    "chainId": 1,
    "address": "0x4236f8aaF2B1f3A28420Eb15B8e0ddF63201a95E",
    "symbol": "BMDA",
    "name": "Bermuda",
    "decimals": 18,
    "logoURI": "ipfs://QmTioxEnmwASSYBUtRmnwyzwieSgWtVXSD77ThLvdvj4cM"
  },
  {
    "chainId": 1,
    "address": "0x584bC13c7D411c00c01A62e8019472dE68768430",
    "symbol": "HEGIC",
    "name": "Hegic",
    "decimals": 18,
    "logoURI": "ipfs://QmYoqzEahXEuy4w5YEDcNTBw89J95y8r5vn7RhqyEyCZpL"
  },
  {
    "chainId": 137,
    "address": "0x0d500B1d8E8eF31E21C99d1Db9A6444d3ADf1270",
    "symbol": "WMATIC",
    "name": "Wrapped Matic",
    "decimals": 18,
    "logoURI": "ipfs://QmYrwmiGyxhWd7YREgYrgkehuWZEzzoXMUWUDcHcWX72We"
  },
  {
    "chainId": 1,
    "address": "0x297E4e5e59Ad72B1B0A2fd446929e76117be0E0a",
    "symbol": "VALOR",
    "name": "SMART VALOR",
    "decimals": 18,
    "logoURI": "ipfs://QmaQpHVBidXUcK4e7zP8TMzV4YW5sYnqeLuRXNQsM6NScx"
  },
  {
    "chainId": 56,
    "address": "0xE37DbF20A4fFf3B88233e456355dc49B76B6fe19",
    "symbol": "LEE",
    "name": "Love Earn Enjoy",
    "decimals": 18,
    "logoURI": "ipfs://QmTLL5D5qQfj5t5nhMHdxKddD3FmYuy6F39n5WuxejcV4B"
  },
  {
    "chainId": 1,
    "address": "0x83F873388Cd14b83A9f47FabDe3C9850b5C74548",
    "symbol": "ZUT",
    "name": "Zero Utility Token",
    "decimals": 18,
    "logoURI": "ipfs://QmRyLfXJcUg8gkYGubtw3HvPw3LJBHR3SHVRcCf57URePP"
  },
  {
    "chainId": 1,
    "address": "0xf55BbE0255f7f4E70f63837Ff72A577fbDDbE924",
    "symbol": "crBOND",
    "name": "Cream BarnBridge",
    "decimals": 8,
    "logoURI": "ipfs://QmcqLdcsig4CMr8ezoxNSrepQrQrtSpjnc3NqzZeqcW8Eb"
  },
  {
    "chainId": 1,
    "address": "0x1b980e05943dE3dB3a459C72325338d327B6F5a9",
    "symbol": "GEAR",
    "name": "Bitgear",
    "decimals": 18,
    "logoURI": "ipfs://QmV6AFtWDwGNj7JR96up1f6rB68jzbmXXZnj6avXzxs9fD"
  },
  {
    "chainId": 1,
    "address": "0x892B14321a4FCba80669aE30Bd0cd99a7ECF6aC0",
    "symbol": "crCREAM",
    "name": "Cream Cream",
    "decimals": 8,
    "logoURI": "ipfs://QmUcVZcc4ABq3L1HPzMB8cnMAKpXSk45quCRidDMPdiRVZ"
  },
  {
    "chainId": 100,
    "address": "0x21a42669643f45Bc0e086b8Fc2ed70c23D67509d",
    "symbol": "FOX",
    "name": "ShapeShift FOX",
    "decimals": 18,
    "logoURI": "ipfs://QmRFkU5X6HLcFdeHEwGEJ4NhxyjDWsSP2i81trLYsY4eRG"
  },
  {
    "chainId": 10,
    "address": "0x2E3D870790dC77A83DD1d18184Acc7439A53f475",
    "symbol": "FRAX",
    "name": "Frax",
    "decimals": 18,
    "logoURI": "ipfs://QmRdVg4RRw7JXdM5enAAC7xkZ1BfgyMxNNqh7q8bhYmjJ7"
  },
  {
    "chainId": 1,
    "address": "0x543Ff227F64Aa17eA132Bf9886cAb5DB55DCAddf",
    "symbol": "GEN",
    "name": "DAOstack",
    "decimals": 18,
    "logoURI": "ipfs://QmUTjTLhDb5W28k7sFFgtyQEfu9YH2euD2CqGQa6JSa4Jk"
  },
  {
    "chainId": 1,
    "address": "0xB26631c6dda06aD89B93C71400D25692de89c068",
    "symbol": "MINDS",
    "name": "Minds",
    "decimals": 18,
    "logoURI": "ipfs://QmS3K2PP6mokMLjFnFonuQyQ4AMoh6RnSoW7hmRN1v8wEQ"
  },
  {
    "chainId": 1,
    "address": "0xdfbc9050F5B01DF53512DCC39B4f2B2BBaCD517A",
    "symbol": "JOB",
    "name": "Jobchain",
    "decimals": 8,
    "logoURI": "ipfs://QmaYEq19XQjwWEqsCptpuN3anxSWMvhEHZ43SHbjzruWBd"
  },
  {
    "chainId": 56,
    "address": "0xeCEB87cF00DCBf2D4E2880223743Ff087a995aD9",
    "symbol": "NUM",
    "name": "Numbers Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmWfPGFZqsLV1XJ1gjTxgQ5RLVMnhDybSSUF2wx4zfywQP"
  },
  {
    "chainId": 1,
    "address": "0xBbE460dC4ac73f7C13A2A2feEcF9aCF6D5083F9b",
    "symbol": "WICK",
    "name": "Wick Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmeE7qFarszc5DonKk4yvwGMfmLc1YkCavLs1Qp2hBBsHP"
  },
  {
    "chainId": 1,
    "address": "0xD45740aB9ec920bEdBD9BAb2E863519E59731941",
    "symbol": "SNOOD",
    "name": "Schnoodle",
    "decimals": 18,
    "logoURI": "ipfs://QmeeKtqfrvfgySgqgMSsLcQvVf6tVpo2uMam4LzTx4v5Gs"
  },
  {
    "chainId": 1,
    "address": "0x97fe22E7341a0Cd8Db6F6C021A24Dc8f4DAD855F",
    "symbol": "sGBP",
    "name": "Synthetic Pound Sterling",
    "decimals": 18,
    "logoURI": "ipfs://QmPofncKSNxDVRUdzWitaRYT1vqUr81Rskq54jEQcyNVpK"
  },
  {
    "chainId": 1,
    "address": "0x469E66e06fEc34839E5eB1273ba85A119B8D702F",
    "symbol": "DEGOV",
    "name": "Degov",
    "decimals": 18,
    "logoURI": "ipfs://QmVccZ3TynVb43qcWN7xCDJ58UmBbrycwZXXFsGAYcS68R"
  },
  {
    "chainId": 1,
    "address": "0x23B608675a2B2fB1890d3ABBd85c5775c51691d5",
    "symbol": "SOCKS",
    "name": "Unisocks",
    "decimals": 18,
    "logoURI": "ipfs://QmRr6rbmZgGbqwdpxJfDXKJnSd697sWkKsv3CfTSjtGgmt"
  },
  {
    "chainId": 1,
    "address": "0xB64ef51C888972c908CFacf59B47C1AfBC0Ab8aC",
    "symbol": "STORJ",
    "name": "Storj",
    "decimals": 8,
    "logoURI": "ipfs://QmVEjJkoaXq1QYMfyRLhE7gj4iQ4bKPFCJ69qYcwuum8aA"
  },
  {
    "chainId": 1,
    "address": "0x5Fc6DE61258e63706543bb57619b99cC0E5a5A1F",
    "symbol": "WAN",
    "name": "Wanchain",
    "decimals": 18,
    "logoURI": "ipfs://QmdDHC5AunrTwBprHNFnizfGGQc4c5yLfMyoyZehydbXS6"
  },
  {
    "chainId": 1,
    "address": "0x0Ae055097C6d159879521C384F1D2123D1f195e6",
    "symbol": "STAKE",
    "name": "STAKE",
    "decimals": 18,
    "logoURI": "ipfs://QmaCDAUrdGqK64cWWrnNcuejkfS5dLNJUmERcuXV7AH3QB"
  },
  {
    "chainId": 100,
    "address": "0x4b1E2c2762667331Bc91648052F646d1b0d35984",
    "symbol": "agEUR",
    "name": "agEUR",
    "decimals": 18,
    "logoURI": "ipfs://QmZ26fGJwCYoZYzkBrxnRmkBT41VhF7V1M7VDoPGJCNVVJ"
  },
  {
    "chainId": 1,
    "address": "0x5d60d8d7eF6d37E16EBABc324de3bE57f135e0BC",
    "symbol": "MYB",
    "name": "MyBit",
    "decimals": 18,
    "logoURI": "ipfs://QmckbQYaMKTW28agWE93UoHBsTsRYgMyaXqQf34i8BQrRk"
  },
  {
    "chainId": 1,
    "address": "0x3C6C553A95910F9FC81c98784736bd628636D296",
    "symbol": "crESD",
    "name": "Cream Empty Set Dollar",
    "decimals": 8,
    "logoURI": "ipfs://QmbNixeWCfMRxQmouX9RuZEFgoBPYwRtjFTcwBmaZEhaDo"
  },
  {
    "chainId": 100,
    "address": "0x6C76971f98945AE98dD7d4DFcA8711ebea946eA6",
    "symbol": "wstETH",
    "name": "Wrapped Staked Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmPLyg2t9yytzoezeSMUQkGuRifstCJdm9HGhSKL2zbkTE"
  },
  {
    "chainId": 137,
    "address": "0xA3fB72CBF2e07dC1b34AEA569Ed40755fd978BD8",
    "symbol": "OPX",
    "name": "OPEX",
    "decimals": 6,
    "logoURI": "ipfs://QmbrZ4tFVzuXZzfenUXPHNpkvUNuBnvW38TbUM3HY4U3L7"
  },
  {
    "chainId": 1,
    "address": "0x853d955aCEf822Db058eb8505911ED77F175b99e",
    "symbol": "FRAX",
    "name": "Frax",
    "decimals": 18,
    "logoURI": "ipfs://QmauyBhQ9TYWwHMLMLoLjCPyCftWbCui5LwzSLVokJ2QL2"
  },
  {
    "chainId": 1,
    "address": "0x6B175474E89094C44Da98b954EedeAC495271d0F",
    "symbol": "DAI",
    "name": "Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmTswuPmisP64jxzZvGz6AaK2gosJyPeTwqMkNPob6pdaa"
  },
  {
    "chainId": 1,
    "address": "0x9f48b2f14517770F2d238c787356F3b961a6616F",
    "symbol": "BSDB",
    "name": "Basis Dollar Bond",
    "decimals": 18,
    "logoURI": "ipfs://QmPovqeuXKL4DPLyE3uzZh4xF6g2xxPTUWrAKdeP6ex47A"
  },
  {
    "chainId": 56,
    "address": "0xc748673057861a797275CD8A068AbB95A902e8de",
    "symbol": "BABYDOGE",
    "name": "Baby Doge Coin",
    "decimals": 9,
    "logoURI": "ipfs://QmWJ5hff93N53MPdpjMLhNoeSMPusLdCfhYw2JaBkb7bBH"
  },
  {
    "chainId": 137,
    "address": "0xFAD65Eb62a97fF5Ed91B23aFD039956aaCa6e93b",
    "symbol": "HT",
    "name": "Huobi",
    "decimals": 18,
    "logoURI": "ipfs://QmScJttJGSYVKYyNFhDrVxM7AUDS3bAj9wfCbeMhBBzBsT"
  },
  {
    "chainId": 1,
    "address": "0xC11b1268C1A384e55C48c2391d8d480264A3A7F4",
    "symbol": "cWBTC",
    "name": "Compound Wrapped BTC",
    "decimals": 8,
    "logoURI": "ipfs://QmRjbcnWJoj1cS6qbwReAHaq58bb1Rwt3V8mWHLcY8gfhs"
  },
  {
    "chainId": 1,
    "address": "0x93ED3FBe21207Ec2E8f2d3c3de6e058Cb73Bc04d",
    "symbol": "PNK",
    "name": "Pinakion",
    "decimals": 18,
    "logoURI": "ipfs://QmbC6rKFGa6xTn7QEXDiDtoCi69fon8nvjtzMoXRLD4uvv"
  },
  {
    "chainId": 1,
    "address": "0xa1d65E8fB6e87b60FECCBc582F7f97804B725521",
    "symbol": "DXD",
    "name": "DXdao",
    "decimals": 18,
    "logoURI": "ipfs://QmbUccYTyR3cMzkVxX1srVAVPCRgCTmDBnGnsU2XKeP2ik"
  },
  {
    "chainId": 1,
    "address": "0x37Ee79E0B44866876de2fB7F416d0443DD5ae481",
    "symbol": "TAT",
    "name": "Tatcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmWkozeF35yGeCKvgVZ1TG2P5woR5WXSCMAX4jMjLf2yz7"
  },
  {
    "chainId": 100,
    "address": "0x256eb8a51f382650B2A1e946b8811953640ee47D",
    "symbol": "DATA",
    "name": "Streamr",
    "decimals": 18,
    "logoURI": "ipfs://QmXxq8zYZ5hQ5FadqHbRb6cQVGqcSJvn3Kg9PEDNcatreu"
  },
  {
    "chainId": 1,
    "address": "0x35A18000230DA775CAc24873d00Ff85BccdeD550",
    "symbol": "cUNI",
    "name": "Compound Uniswap",
    "decimals": 8,
    "logoURI": "ipfs://QmNQyg95g5Hv564hne6MXDKYnPdQbse1FwxzifscT7AD5R"
  },
  {
    "chainId": 1,
    "address": "0x8f8221aFbB33998d8584A2B05749bA73c37a938a",
    "symbol": "REQ",
    "name": "Request",
    "decimals": 18,
    "logoURI": "ipfs://QmWExJk6p2LhYYvaixBYw51ZWSTrzPaHgE5EiAQVFY72Eg"
  },
  {
    "chainId": 1,
    "address": "0x7884F51dC1410387371ce61747CB6264E1dAeE0B",
    "symbol": "BDOT",
    "name": "Binance Wrapped DOT",
    "decimals": 10,
    "logoURI": "ipfs://QmXXfRrcuiVTrqcYdRCbLvpPUcqoadstB96BkAF86EKCNp"
  },
  {
    "chainId": 1,
    "address": "0x9AAb071B4129B083B01cB5A0Cb513Ce7ecA26fa5",
    "symbol": "HUNT",
    "name": "HUNT",
    "decimals": 18,
    "logoURI": "ipfs://QmTjfYboCz5gqhm1WEhPkvMTTy84Se9ncoifTogk4CZHUX"
  },
  {
    "chainId": 1,
    "address": "0x0c90C57aaf95A3A87eadda6ec3974c99D786511F",
    "symbol": "HAN",
    "name": "HanChain",
    "decimals": 18,
    "logoURI": "ipfs://QmXiJTDpk8JHxgBFnYsqJJTaqj7cQnSXtGteQxW6dhs7oJ"
  },
  {
    "chainId": 10,
    "address": "0x484c2D6e3cDd945a8B2DF735e079178C1036578c",
    "symbol": "sfrxETH",
    "name": "Staked Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmX1r2JVToPDMkRUFsERHeeCheTXFZ9Hw1nuq1gDNjufxG"
  },
  {
    "chainId": 56,
    "address": "0xbb4CdB9CBd36B01bD1cBaEBF2De08d9173bc095c",
    "symbol": "WBNB",
    "name": "Wrapped BNB",
    "decimals": 18,
    "logoURI": "ipfs://QmPJvxkPRPnyN1tLAmeCTjPLG13RPTxpzaoE1qTaJ4FAGW"
  },
  {
    "chainId": 1,
    "address": "0x64aa3364F17a4D01c6f1751Fd97C2BD3D7e7f1D5",
    "symbol": "OHM",
    "name": "Olympus",
    "decimals": 9,
    "logoURI": "ipfs://QmS9CqU3gEoq1H8yMoNwzUSywaQNZexC2m1arkuRxtFMkQ"
  },
  {
    "chainId": 1,
    "address": "0x19D3364A399d251E894aC732651be8B0E4e85001",
    "symbol": "yvDAI",
    "name": "DAI yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmZ1EjTzpYpcDGHeT8PgFZs33WV3BwEfTGWQkwGnhgi56P"
  },
  {
    "chainId": 1,
    "address": "0x368BF9F1A1CA767935E39F20439D9041707E2634",
    "symbol": "CFXT",
    "name": "Chainflix",
    "decimals": 18,
    "logoURI": "ipfs://QmXoXDAWEaFZkkGo3BzpDNrywnjfUmSSCWCre2f7Hi3kch"
  },
  {
    "chainId": 1,
    "address": "0x120a3879da835A5aF037bB2d1456beBd6B54d4bA",
    "symbol": "RVST",
    "name": "Revest Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmbkNv59B7Su6nEqEZBu1zHc415GfvR7qeZwcjsvR2HF5m"
  },
  {
    "chainId": 10,
    "address": "0x50Bce64397C75488465253c0A034b8097FeA6578",
    "symbol": "HAN",
    "name": "HanChain",
    "decimals": 18,
    "logoURI": "ipfs://QmXiJTDpk8JHxgBFnYsqJJTaqj7cQnSXtGteQxW6dhs7oJ"
  },
  {
    "chainId": 1,
    "address": "0x5732046A883704404F284Ce41FfADd5b007FD668",
    "symbol": "BLZ",
    "name": "Bluzelle",
    "decimals": 18,
    "logoURI": "ipfs://QmRAbZnEifiyUx7LpDfVraeo69vc2oJTSYJ4U4EhTKs32R"
  },
  {
    "chainId": 1,
    "address": "0xb6EE9668771a79be7967ee29a63D4184F8097143",
    "symbol": "CXO",
    "name": "CargoX",
    "decimals": 18,
    "logoURI": "ipfs://QmUhvPazqW2hkQkfNCyNF7oxRhkeQ2qKbX1ZFrGgmHnup9"
  },
  {
    "chainId": 1,
    "address": "0x2ba592F78dB6436527729929AAf6c908497cB200",
    "symbol": "CREAM",
    "name": "Cream",
    "decimals": 18,
    "logoURI": "ipfs://QmSMwsQc4vLC8cq8faWX6ysuks5XL4bFf7kcAGTLgu7eQf"
  },
  {
    "chainId": 1,
    "address": "0x29502fE4d233EF0b45C3647101Fa1252cE0634BD",
    "symbol": "FROGE",
    "name": "Froge Finance",
    "decimals": 9,
    "logoURI": "ipfs://QmZqgu6mXLMEUVWc64WXueKG2YB4GtJUvpmkBK3jtt7Hhu"
  },
  {
    "chainId": 1,
    "address": "0xf6E9b246319ea30e8C2fA2d1540AAEBF6f9E1B89",
    "symbol": "iBCH",
    "name": "Synthetic Inverse Bitcoin Cash",
    "decimals": 18,
    "logoURI": "ipfs://QmYHdiTSv9mnse66vHuCkymQyj7M7Pyyb5CgDwsAR9ZHfN"
  },
  {
    "chainId": 1,
    "address": "0x4e352cF164E64ADCBad318C3a1e222E9EBa4Ce42",
    "symbol": "MCB",
    "name": "MCDEX",
    "decimals": 18,
    "logoURI": "ipfs://QmVyW3GPeVEShDebghWPZQtYN8DKME8U7xinhBYCEVFbQf"
  },
  {
    "chainId": 250,
    "address": "0x74b23882a30290451A17c44f4F05243b6b58C76d",
    "symbol": "WETH",
    "name": "Wrapped Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmPt2DEXhRSvbbFQep44XXtcKxRscshsijJ47Y1xUQwBZ1"
  },
  {
    "chainId": 42161,
    "address": "0x54BDBF3cE36f451Ec61493236b8E6213ac87c0f6",
    "symbol": "RDP",
    "name": "Radpie",
    "decimals": 18,
    "logoURI": "ipfs://QmZxTu24YYCAVqeBi8cNLNoApZXA1D9W2MiYt4jnjf5KXU"
  },
  {
    "chainId": 1,
    "address": "0x9fcF4497ED9fBb8D8B46259e4f2bB4749953ab8B",
    "symbol": "DeFi",
    "name": "DeFiant Token",
    "decimals": 9,
    "logoURI": "ipfs://QmVmoBTt12JHJZULVt9QpVohoAAKZktCanThcrYqWxb2nK"
  },
  {
    "chainId": 1,
    "address": "0x17Aa18A4B64A55aBEd7FA543F2Ba4E91f2dcE482",
    "symbol": "INB",
    "name": "Insight Chain",
    "decimals": 18,
    "logoURI": "ipfs://QmeHwxooUDSioGfpR8hxaEtTPecYNcH7GkMxeH31b7rEEn"
  },
  {
    "chainId": 1,
    "address": "0x9d1C2A187cf908aEd8CFAe2353Ef72F06223d54D",
    "symbol": "sWBTC",
    "name": "Strike WBTC",
    "decimals": 8,
    "logoURI": "ipfs://QmdYL6iiUpSkGxbsbSL5cKZJDbtkc1eumgcgpYvSXMmwiC"
  },
  {
    "chainId": 1,
    "address": "0xBdea5bB640DBFC4593809DEeC5CdB8F99b704Cd2",
    "symbol": "DIREWOLF",
    "name": "Direwolf",
    "decimals": 2,
    "logoURI": "ipfs://QmQQneKbGnCoAyqx3R1PzMCwdz923mqcbwAfJuj7pQspcS"
  },
  {
    "chainId": 56,
    "address": "0x26433c8127d9b4e9B71Eaa15111DF99Ea2EeB2f8",
    "symbol": "MANA",
    "name": "Decentraland",
    "decimals": 18,
    "logoURI": "ipfs://QmY18MUVBQbY22cM9gRxub76EqqdHMiNydGkfoR1fBarEo"
  },
  {
    "chainId": 56,
    "address": "0xd0c4BC1B89BbD105EeCb7EBa3f13E7648c0De38F",
    "symbol": "WEB3",
    "name": "Web3 Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmcnL775KhSxyke8LewJ96Eskq1HTzXEUjfQvQaYS1uCLN"
  },
  {
    "chainId": 1,
    "address": "0x5Ca381bBfb58f0092df149bD3D243b08B9a8386e",
    "symbol": "MXC",
    "name": "MXC",
    "decimals": 18,
    "logoURI": "ipfs://QmUE65y8HZzD3ZGeKsojqJxzuVf3E65EwXSo4hp4qAExPw"
  },
  {
    "chainId": 1,
    "address": "0x9B70740e708a083C6fF38Df52297020f5DfAa5EE",
    "symbol": "DAN",
    "name": "Daneel",
    "decimals": 10,
    "logoURI": "ipfs://QmZ3MjDBaAvNMUG9xsRgjCstqhyufZJskspHZuaZ5C4GDP"
  },
  {
    "chainId": 1,
    "address": "0xA361718326c15715591c299427c62086F69923D9",
    "symbol": "aBUSD",
    "name": "Aave BUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmNQwpgbcQQaXYuLUpNMqBg9ajVswF9buhv6FVAxbX9mVc"
  },
  {
    "chainId": 42161,
    "address": "0xcAFcD85D8ca7Ad1e1C6F82F651fA15E33AEfD07b",
    "symbol": "WOO",
    "name": "WOO Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdJjzzpWuzQAN6sj2j5FRy3SnFYgwMPQCx5ZNrFdVpcFt"
  },
  {
    "chainId": 1,
    "address": "0xFFC97d72E13E01096502Cb8Eb52dEe56f74DAD7B",
    "symbol": "aAAVE",
    "name": "Aave AAVE",
    "decimals": 18,
    "logoURI": "ipfs://Qmb3odzj42eDcDkQvR5TNpPk5osKR2fqCN1YU2Ek1gFn1b"
  },
  {
    "chainId": 1,
    "address": "0xE14d13d8B3b85aF791b2AADD661cDBd5E6097Db1",
    "symbol": "yvYFI",
    "name": "YFI yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmY4AVY22dDaK7SGv4huDf1w5QaEVPTEbrttgYcuvyvDma"
  },
  {
    "chainId": 56,
    "address": "0x33A3d962955A3862C8093D1273344719f03cA17C",
    "symbol": "SPORE",
    "name": "Spore",
    "decimals": 9,
    "logoURI": "ipfs://QmXkfNjECJ6DsWNCGi6HcM7R87WVkqHXzeS4Tms7sMvSXD"
  },
  {
    "chainId": 1,
    "address": "0x5dc60C4D5e75D22588FA17fFEB90A63E535efCE0",
    "symbol": "DKA",
    "name": "dKargo",
    "decimals": 18,
    "logoURI": "ipfs://QmaCf2Zse9BVAtAN4QV4LH3N4mL9rQ8aAxEmDS7x1ta2tA"
  },
  {
    "chainId": 1,
    "address": "0xE41d2489571d322189246DaFA5ebDe1F4699F498",
    "symbol": "ZRX",
    "name": "0x",
    "decimals": 18,
    "logoURI": "ipfs://QmR9MP9Ng17aSn7DxXTxMSaRhsAQJYPTmBHZzp2HjrDphi"
  },
  {
    "chainId": 1,
    "address": "0x4954Db6391F4feB5468b6B943D4935353596aEC9",
    "symbol": "USDQ",
    "name": "USDQ",
    "decimals": 18,
    "logoURI": "ipfs://Qmcj4LbGxoia2tpHiHTyz1td7jZtatgPFyhRt2u3cNRpPB"
  },
  {
    "chainId": 1,
    "address": "0x7090a6e22c838469c9E67851D6489ba9c933a43F",
    "symbol": "ZBUX",
    "name": "Zuck Bucks",
    "decimals": 0,
    "logoURI": "ipfs://Qmc6q1dsGdfTta2Xz2NhepfULLJUyFomwFwN8ACqUXKJTj"
  },
  {
    "chainId": 43114,
    "address": "0x96E1056a8814De39c8c3Cd0176042d6ceCD807d7",
    "symbol": "OSAK",
    "name": "Osaka Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUng1scXVJSCH1MpqK4RanCsW1aXYew8KUZphtj3WUpah"
  },
  {
    "chainId": 250,
    "address": "0xdc301622e621166BD8E82f2cA0A26c13Ad0BE355",
    "symbol": "FRAX",
    "name": "Frax",
    "decimals": 18,
    "logoURI": "ipfs://QmRdVg4RRw7JXdM5enAAC7xkZ1BfgyMxNNqh7q8bhYmjJ7"
  },
  {
    "chainId": 1,
    "address": "0xEEF9f339514298C6A857EfCfC1A762aF84438dEE",
    "symbol": "HEZ",
    "name": "Hermez Network",
    "decimals": 18,
    "logoURI": "ipfs://QmcvpV2MStZ33KPZ3SQ4Fk2q8qWdNV1XirXpRrCAMiaaTG"
  },
  {
    "chainId": 1,
    "address": "0xeFF039C3c1D668f408d09dD7B63008622a77532C",
    "symbol": "crWNXM",
    "name": "Cream Wrapped NXM",
    "decimals": 8,
    "logoURI": "ipfs://QmZ55P4HDWn838vzv8afwqE69GoCvZNMu4icM4VXLamFxS"
  },
  {
    "chainId": 56,
    "address": "0x62D0A8458eD7719FDAF978fe5929C6D342B0bFcE",
    "symbol": "BEAM",
    "name": "Beam",
    "decimals": 18,
    "logoURI": "ipfs://QmPfGrjzfmzCevK86kK5ZvD9E7S88wLw1t7Sq1X6yPAJUB"
  },
  {
    "chainId": 1,
    "address": "0x9AFb950948c2370975fb91a441F36FDC02737cD4",
    "symbol": "HFIL",
    "name": "Huobi FIL",
    "decimals": 18,
    "logoURI": "ipfs://QmVzuHUxcgpccANDwTneHc4kqv9XBmx1SLSZwcq1L8xXfX"
  },
  {
    "chainId": 1,
    "address": "0x35f6B052C598d933D69A4EEC4D04c73A191fE6c2",
    "symbol": "aSNX",
    "name": "Aave SNX",
    "decimals": 18,
    "logoURI": "ipfs://QmfHv6w8H3StBYCnRKJAVmwgeRFAdCHggBDxThASbrRq3p"
  },
  {
    "chainId": 250,
    "address": "0x85dec8c4B2680793661bCA91a8F129607571863d",
    "symbol": "BRUSH",
    "name": "PaintSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmbkAXk8qDPkEHQE62GZxYMqzhE3m8Ay1cXpidaYR2qost"
  },
  {
    "chainId": 1,
    "address": "0x9e96604445Ec19fFed9a5e8dd7B50a29C899A10C",
    "symbol": "COSS",
    "name": "COSS",
    "decimals": 18,
    "logoURI": "ipfs://QmYwhBdUwZHuEoE59163Tz5n1gPoMkVG8s6qEuXkd5293B"
  },
  {
    "chainId": 1,
    "address": "0x7D48FBe0A877bB1f511fcf9B57F12420C75841e9",
    "symbol": "DOOR",
    "name": "DOOR",
    "decimals": 18,
    "logoURI": "ipfs://QmQ7jd3rW3i2Zqq8znfwCF5RQuXJb3n7f6rzYbqCCJ1vcM"
  },
  {
    "chainId": 1,
    "address": "0xDe30da39c46104798bB5aA3fe8B9e0e1F348163F",
    "symbol": "GTC",
    "name": "Gitcoin",
    "decimals": 18,
    "logoURI": "ipfs://Qmaxb89DFcf5B2AmuJ3cnhHcCVKMseP2eB4219kw5Vkhid"
  },
  {
    "chainId": 100,
    "address": "0xdfc20AE04ED70bd9c7D720F449eEDAe19F659D65",
    "symbol": "BADGER",
    "name": "Badger DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmfRYPsbRLjenLyGEsTVJHd5TXdxx5Z11KEefC6692i3gZ"
  },
  {
    "chainId": 1,
    "address": "0x463538705E7d22aA7f03Ebf8ab09B067e1001B54",
    "symbol": "iLINK",
    "name": "Fulcrum LINK iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmWZ6WR6Q77BPSnqXeYCYzLfmjJWxXUCYQx2zi2fnLJEFj"
  },
  {
    "chainId": 1,
    "address": "0x9a794Dc1939F1d78fa48613b89B8f9d0A20dA00E",
    "symbol": "ABX",
    "name": "Arbidex",
    "decimals": 18,
    "logoURI": "ipfs://QmaZeuMHv5TJkgPtbMpPQXD2gW2bFmuWbdJnG8DswAHYgp"
  },
  {
    "chainId": 100,
    "address": "0xDDAfbb505ad214D7b80b1f830fcCc89B60fb7A83",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmUQp1wZzr4UREn4AVLkKH5GbwpUCwf9w5VQQJeh1bmThJ"
  },
  {
    "chainId": 1,
    "address": "0x6399C842dD2bE3dE30BF99Bc7D1bBF6Fa3650E70",
    "symbol": "PREMIA",
    "name": "Premia",
    "decimals": 18,
    "logoURI": "ipfs://Qmbef33x648zGE9v9rthiyz4w9jpGsRSsy39UFGBjhpgts"
  },
  {
    "chainId": 1,
    "address": "0xB8647e90C0645152Fccf4d9AbB6B59Eb4AA99052",
    "symbol": "KEYFI",
    "name": "KeyFi",
    "decimals": 18,
    "logoURI": "ipfs://QmRWvD3vV6pAaZ4qjQ1FQGJ4oEYKEyuJYQC7unDeB2N8H9"
  },
  {
    "chainId": 56,
    "address": "0xB64E280e9D1B5DbEc4AcceDb2257A87b400DB149",
    "symbol": "LVL",
    "name": "Level",
    "decimals": 18,
    "logoURI": "ipfs://QmVGkU9X4SBvyErWHuiHW1wtpBWJBdFUeWgzVqTJEkxe74"
  },
  {
    "chainId": 1,
    "address": "0x8Ab7404063Ec4DBcfd4598215992DC3F8EC853d7",
    "symbol": "AKRO",
    "name": "Akropolis",
    "decimals": 18,
    "logoURI": "ipfs://QmQcYTDb1pwMvF31sfrkNEfHqW4AKvjwNHYHcNCdbh1cgg"
  },
  {
    "chainId": 43114,
    "address": "0xD24C2Ad096400B6FBcd2ad8B24E7acBc21A1da64",
    "symbol": "FRAX",
    "name": "Frax",
    "decimals": 18,
    "logoURI": "ipfs://QmRdVg4RRw7JXdM5enAAC7xkZ1BfgyMxNNqh7q8bhYmjJ7"
  },
  {
    "chainId": 137,
    "address": "0xa9a6A3626993D487d2Dbda3173cf58cA1a9D9e9f",
    "symbol": "UD",
    "name": "Unstoppable Domains",
    "decimals": 0,
    "logoURI": "ipfs://QmZieLVqfExjdEx3HQHSNT1CVC3YnXGtkKGnQzYA1MQAtD"
  },
  {
    "chainId": 56,
    "address": "0x2222227E22102Fe3322098e4CBfE18cFebD57c95",
    "symbol": "TLM",
    "name": "Alien Worlds",
    "decimals": 4,
    "logoURI": "ipfs://QmWGNjuwADjssVS5LDUoJWVSj6wW5VxLUfrcedUDZE3quw"
  },
  {
    "chainId": 1,
    "address": "0xF970b8E36e23F7fC3FD752EeA86f8Be8D83375A6",
    "symbol": "RCN",
    "name": "Ripio Credit Network",
    "decimals": 18,
    "logoURI": "ipfs://Qmav4Ey6bJchvqWK6ZFuAg6yRTdMUK9AVwkGuXNmsMFeS4"
  },
  {
    "chainId": 1,
    "address": "0xc713e5E149D5D0715DcD1c156a020976e7E56B88",
    "symbol": "aMKR",
    "name": "Aave MKR",
    "decimals": 18,
    "logoURI": "ipfs://QmXVkKM6n9jagTbPse6jbMsiWpPJRbdZnm5WrWkmzcF8Uq"
  },
  {
    "chainId": 1,
    "address": "0x84294FC9710e1252d407d3D80A84bC39001bd4A8",
    "symbol": "NUTS",
    "name": "Squirrel Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmQy8PWiC8yN6xaWgYdEL2bwkcMEmhtjLC83KfY5MFufAN"
  },
  {
    "chainId": 1,
    "address": "0xE277aC35F9D327A670c1A3F3eeC80a83022431e4",
    "symbol": "PUX",
    "name": "PolypuX",
    "decimals": 8,
    "logoURI": "ipfs://QmV9kNbcnak4Ho3dZWh1XrzMHbeeNHmbCvQgqST27DKdoK"
  },
  {
    "chainId": 1,
    "address": "0x817bbDbC3e8A1204f3691d14bB44992841e3dB35",
    "symbol": "CUDOS",
    "name": "Cudos",
    "decimals": 18,
    "logoURI": "ipfs://QmTmECvBQGz6FRHv3zDj4kbLtPEVnPNtUt9ryTLDidYeyy"
  },
  {
    "chainId": 1,
    "address": "0xfE18be6b3Bd88A2D2A7f928d00292E7a9963CfC6",
    "symbol": "sBTC",
    "name": "Synthetic Bitcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmYp8Gbr7f3RQaaB5vsRgRyoZtR5Dq648u3nvFyvKhnztQ"
  },
  {
    "chainId": 1,
    "address": "0x137dDB47Ee24EaA998a535Ab00378d6BFa84F893",
    "symbol": "RDNT",
    "name": "Radiant",
    "decimals": 18,
    "logoURI": "ipfs://QmPFwFqf6KCqR6fBxYffw1FtPGVG6AUACK1dKu79cFvpHt"
  },
  {
    "chainId": 1,
    "address": "0x05D3606d5c81EB9b7B18530995eC9B29da05FaBa",
    "symbol": "TOMOE",
    "name": "TomoChain ERC-20",
    "decimals": 18,
    "logoURI": "ipfs://Qmd8aunGkByP46L6hJgcPGYZPs3i15Qes4YtgvB64mnhHh"
  },
  {
    "chainId": 1,
    "address": "0x9BE89D2a4cd102D8Fecc6BF9dA793be995C22541",
    "symbol": "BBTC",
    "name": "Binance Wrapped BTC",
    "decimals": 8,
    "logoURI": "ipfs://QmVGRju6vMmwohv4aHG33AMB3qav5hUZ7k4igvGn1fr5WZ"
  },
  {
    "chainId": 1,
    "address": "0x888888435FDe8e7d4c54cAb67f206e4199454c60",
    "symbol": "DFX",
    "name": "DFX Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYEukBdcnQaJig9Vg92rhFmP3buofwaA6b3UM6oN9ienC"
  },
  {
    "chainId": 1,
    "address": "0x5A7E3c07604EB515C16b36cd51906a65f021F609",
    "symbol": "sNFLX",
    "name": "Synthetic Netflix",
    "decimals": 18,
    "logoURI": "ipfs://QmYSngJGkEihhyVnSSkrDhZ9cAchCR2ooZQt2R679iRLUp"
  },
  {
    "chainId": 43114,
    "address": "0xC17c30e98541188614dF99239cABD40280810cA3",
    "symbol": "RISE",
    "name": "EverRise",
    "decimals": 18,
    "logoURI": "ipfs://QmU6dKNHUySjmVh9FJFMT1cRZ54aoQ7fpJ4q2a6qFPHsbU"
  },
  {
    "chainId": 56,
    "address": "0x6B8b9AE0627a7622c593A1696859ca753c61A670",
    "symbol": "jZAR",
    "name": "Jarvis Synthetic South African Rand",
    "decimals": 18,
    "logoURI": "ipfs://QmVNPjNmBSk1oR9cVJUNMmvVzmxxj9eNXjPhMAboxynBaV"
  },
  {
    "chainId": 1,
    "address": "0xeEAA40B28A2d1b0B08f6f97bB1DD4B75316c6107",
    "symbol": "GOVI",
    "name": "Govi",
    "decimals": 18,
    "logoURI": "ipfs://QmSJL7yi682TPGt69VecM1HCgQLWUYCPLzzxuGRihjH52h"
  },
  {
    "chainId": 1,
    "address": "0x06AF07097C9Eeb7fD685c692751D5C66dB49c215",
    "symbol": "CHAI",
    "name": "Chai",
    "decimals": 18,
    "logoURI": "ipfs://QmeQsr7VfsYe46p9ibwxrj33pJhQWKM3PrEk4tedM14oJZ"
  },
  {
    "chainId": 56,
    "address": "0xf307910A4c7bbc79691fD374889b36d8531B08e3",
    "symbol": "ANKR",
    "name": "Ankr",
    "decimals": 18,
    "logoURI": "ipfs://QmbvErh2b3MSwt6TtWbLu25Hsz4tK3EghsLrfrb6igkbCy"
  },
  {
    "chainId": 56,
    "address": "0xfD5840Cd36d94D7229439859C0112a4185BC0255",
    "symbol": "vUSDT",
    "name": "Venus USDT",
    "decimals": 8,
    "logoURI": "ipfs://QmXEdyrqjhfYeQB6VGnVKBowtG4abTNhhfRqh7BsoDCzVF"
  },
  {
    "chainId": 56,
    "address": "0x2170Ed0880ac9A755fd29B2688956BD959F933F8",
    "symbol": "ETH",
    "name": "Ethereum",
    "decimals": 18,
    "logoURI": "ipfs://QmShSisJHEC56LHmX7mxKAeUwH4utCHtmqeoGR2QonZQZ7"
  },
  {
    "chainId": 1,
    "address": "0x9EeF4CA7aB9fa8bc0650127341C2d3F707a40f8A",
    "symbol": "sCOIN",
    "name": "Synthetic Coinbase",
    "decimals": 18,
    "logoURI": "ipfs://QmUzTwNH5QMXpGB2F4jU2dvqVTvQb6yZeY5FRFR4fxgz6f"
  },
  {
    "chainId": 324,
    "address": "0xAFe4cA0Bbe6215cBdA12857e723134Bc3809F766",
    "symbol": "MCRN",
    "name": "MacaronSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmcSmS5w8WtQFbR3Tm1HYeq9HDwWoJsRZMcdF8jvtYDSaM"
  },
  {
    "chainId": 1,
    "address": "0x744d70FDBE2Ba4CF95131626614a1763DF805B9E",
    "symbol": "SNT",
    "name": "Status",
    "decimals": 18,
    "logoURI": "ipfs://QmewBQAyAd81xRJyfghTQ6aQ5G6NnrdDiCdB91h8MwSRgp"
  },
  {
    "chainId": 100,
    "address": "0x2995D1317DcD4f0aB89f4AE60F3f020A4F17C7CE",
    "symbol": "SUSHI",
    "name": "Sushi",
    "decimals": 18,
    "logoURI": "ipfs://QmT48vXNDWa4KUUk6ZsYk8Z4QwNqjk9xaqpbpQvMFz1xFC"
  },
  {
    "chainId": 1,
    "address": "0x5299d6F7472DCc137D7f3C4BcfBBB514BaBF341A",
    "symbol": "sXMR",
    "name": "Synthetic Monero",
    "decimals": 18,
    "logoURI": "ipfs://QmQKrxK93rADExK29FXr9FRb9BitFM6kxddb7FVhDUsddh"
  },
  {
    "chainId": 1,
    "address": "0xf16e81dce15B08F326220742020379B855B87DF9",
    "symbol": "ICE",
    "name": "Popsicle Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmSp3Vnq7VFsFrdcZZ35YkHHFLVF3nWvcDPKSDJgfkTiMD"
  },
  {
    "chainId": 137,
    "address": "0xFFA4D863C96e743A2e1513824EA006B8D0353C57",
    "symbol": "USDD",
    "name": "Decentralized USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPtvMuo6UQwDXB7cTVz85Ms5noX6rnnNKi3HNmwKrPf8u"
  },
  {
    "chainId": 1,
    "address": "0x2791BfD60D232150Bff86b39B7146c0eaAA2BA81",
    "symbol": "BIFI",
    "name": "BiFi",
    "decimals": 18,
    "logoURI": "ipfs://QmVUWkVWgkoryJpan9TygturFdBAy1FfoNWe97cETwFZcw"
  },
  {
    "chainId": 137,
    "address": "0x70c006878a5A50Ed185ac4C87d837633923De296",
    "symbol": "REVV",
    "name": "REVV",
    "decimals": 18,
    "logoURI": "ipfs://QmZc39HdhcUMJFMnLnwb4RtYoSxcKUjAGpGK6puYnGvfU4"
  },
  {
    "chainId": 1,
    "address": "0x22B243B96495C547598D9042B6f94B01C22B2e9E",
    "symbol": "crSWAG",
    "name": "Cream Swag",
    "decimals": 8,
    "logoURI": "ipfs://Qman9m4VwLJ1e4RpBoGpGxBzwWSBm8dXWRRsym952kZEEP"
  },
  {
    "chainId": 1,
    "address": "0x878CF148ccBb50426043a9AFFe54Ba408221C7fA",
    "symbol": "KOMBAT",
    "name": "Crypto Kombat",
    "decimals": 8,
    "logoURI": "ipfs://QmawBAVNbTtHn3XkrjG6rx1WqZZhmz4x3r4edNaMjbG3LS"
  },
  {
    "chainId": 56,
    "address": "0x4B5C23cac08a567ecf0c1fFcA8372A45a5D33743",
    "symbol": "FARM",
    "name": "Harvest Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUcvATxahU75MoEAEE8BGVNr2XFLeLd2chohtATcEoZW5"
  },
  {
    "chainId": 1,
    "address": "0x0e0989b1f9B8A38983c2BA8053269Ca62Ec9B195",
    "symbol": "POE",
    "name": "Po.et",
    "decimals": 8,
    "logoURI": "ipfs://QmebVEomNBTXiAyG8xM1PynJM3bsuDznfEzabG5JUmyhyW"
  },
  {
    "chainId": 1,
    "address": "0x5150956E082C748Ca837a5dFa0a7C10CA4697f9c",
    "symbol": "ZDEX",
    "name": "Zeedex",
    "decimals": 18,
    "logoURI": "ipfs://QmV9c635vDbWfuKz8Nn7AAX8BF8atwNovrJGom7Yp5gy5A"
  },
  {
    "chainId": 1,
    "address": "0x69702cfd7DAd8bCcAA24D6B440159404AAA140F5",
    "symbol": "sUSDT",
    "name": "Strike USDT",
    "decimals": 8,
    "logoURI": "ipfs://QmUqXRqRnpobmz2KBzyCBsriLcLkCEGkuUG9ET8sRZn8V6"
  },
  {
    "chainId": 1,
    "address": "0x31c8EAcBFFdD875c74b94b077895Bd78CF1E64A3",
    "symbol": "RAD",
    "name": "Radicle",
    "decimals": 18,
    "logoURI": "ipfs://QmVzppE6wdafQtApnGkdSHJqFcd6w1uJQBPyEgdxqBRTMB"
  },
  {
    "chainId": 1,
    "address": "0x8Ee04E1e9400d4e2091Ff3A46509E246E2F34542",
    "symbol": "MUSUBI",
    "name": "Musubi",
    "decimals": 9,
    "logoURI": "ipfs://QmZ3V5h6tX4P2pFPa4rfc7NEqH9WonAQAYJ1tN4Pu9DQS1"
  },
  {
    "chainId": 1,
    "address": "0xeCe8531550Fc599ccB2c7734494FBc358d0a9483",
    "symbol": "PP",
    "name": "PP",
    "decimals": 18,
    "logoURI": "ipfs://QmT71iXeBXHoJRffSuVL1fStXdRGZxq4zVt2bVJff7SdJP"
  },
  {
    "chainId": 1,
    "address": "0xc690F7C7FcfFA6a82b79faB7508c466FEfdfc8c5",
    "symbol": "LYM",
    "name": "Lympo",
    "decimals": 18,
    "logoURI": "ipfs://QmVkikQsGp3S1TtTA2a7UE8HNUH1mLkYV6e8BS9CKH5VwD"
  },
  {
    "chainId": 137,
    "address": "0x23001f892c0C82b79303EDC9B9033cD190BB21c7",
    "symbol": "LUSD",
    "name": "Liquity USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPzBCxWo5xMA8Wif7wCQQmuZ5ajP5sw4JSHthMxvxY2uR"
  },
  {
    "chainId": 1,
    "address": "0xbaA70614C7AAfB568a93E62a98D55696bcc85DFE",
    "symbol": "UCAP",
    "name": "Unicap.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmSc3Q78LfQ8egt8ZLUtFTcJguigwNYixZFJWzJfBWtsvT"
  },
  {
    "chainId": 8453,
    "address": "0xB6fe221Fe9EeF5aBa221c348bA20A1Bf5e73624c",
    "symbol": "rETH",
    "name": "Rocket Pool ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmRqLBdZaA2oSr7yWVmwdSyj5HwixHMWRpyKw8QyAQTgRz"
  },
  {
    "chainId": 1,
    "address": "0xBB0E17EF65F82Ab018d8EDd776e8DD940327B28b",
    "symbol": "AXS",
    "name": "Axie Infinity",
    "decimals": 18,
    "logoURI": "ipfs://Qma5gXpmfTK1Ja4AwaYNyAQrxsu3fEQXj8Kqa3W2jyD5uw"
  },
  {
    "chainId": 1,
    "address": "0x5cAf454Ba92e6F2c929DF14667Ee360eD9fD5b26",
    "symbol": "DEV",
    "name": "Dev Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmdKdvH32cnqv3aNGTwDoJ6ygkcQ9vzQcTiEogkFjNDzUa"
  },
  {
    "chainId": 1,
    "address": "0x467Bccd9d29f223BcE8043b84E8C8B282827790F",
    "symbol": "TEL",
    "name": "Telcoin",
    "decimals": 2,
    "logoURI": "ipfs://QmdRM767SbowAoxgb1i327MjQxMvoenfeSWWNYtnZFTLgd"
  },
  {
    "chainId": 1,
    "address": "0x737F98AC8cA59f2C68aD658E3C3d8C8963E40a4c",
    "symbol": "AMN",
    "name": "Amon",
    "decimals": 18,
    "logoURI": "ipfs://QmemZYjfBwHT3p5XyUp6GifN5FzPtLn9ZPuJLx3zFJ39JJ"
  },
  {
    "chainId": 1,
    "address": "0x03ab458634910AaD20eF5f1C8ee96F1D6ac54919",
    "symbol": "RAI",
    "name": "Rai Reflex Index",
    "decimals": 18,
    "logoURI": "ipfs://QmPy2zFxZbcmKBfZhg6YhNpfUjcBeqdRX83wFoELTZBz69"
  },
  {
    "chainId": 1,
    "address": "0xC18360217D8F7Ab5e7c516566761Ea12Ce7F9D72",
    "symbol": "ENS",
    "name": "Ethereum Name Service",
    "decimals": 18,
    "logoURI": "ipfs://QmXtigPW4eXE48EMJqVjGQud84NWrPptAutV7gstsb6TX8"
  },
  {
    "chainId": 1,
    "address": "0xeE0f286776639cD363Da810DAF3e0623F82576b0",
    "symbol": "L2P",
    "name": "Lung Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmRtMKXhHYHGPMswVAetJFpBkENxKZoxo9jeMn5edsZ7ZH"
  },
  {
    "chainId": 56,
    "address": "0x8b1f4432F943c465A973FeDC6d7aa50Fc96f1f65",
    "symbol": "AXL",
    "name": "Axelar",
    "decimals": 6,
    "logoURI": "ipfs://QmcSRWc1LN1mk4tjy38kyTMqqVx7j4Dmnkw3dsW7efPMTL"
  },
  {
    "chainId": 1,
    "address": "0x616ef40D55C0D2c506f4d6873Bda8090b79BF8fC",
    "symbol": "KTO",
    "name": "Kounotori",
    "decimals": 9,
    "logoURI": "ipfs://QmQ6BPyfGTJG7GoJvqYfVEGGiKf4toaaRmNpcA8PHYojja"
  },
  {
    "chainId": 1,
    "address": "0x50D1c9771902476076eCFc8B2A83Ad6b9355a4c9",
    "symbol": "FTT",
    "name": "FTX Token",
    "decimals": 18,
    "logoURI": "ipfs://QmYWkTfPjEpyBPWaMo5nkA7SsDG3LvhxuRysDQGvaD1Qas"
  },
  {
    "chainId": 1,
    "address": "0x9c306A78b1a904e83115c05Ac67c1Ef07C653651",
    "symbol": "ODOGE",
    "name": "BoringDAO DOGE",
    "decimals": 18,
    "logoURI": "ipfs://QmWv3YwHReWWoGR378NVUYAeADqfTV2Cf1eCpKLipbYWKZ"
  },
  {
    "chainId": 1,
    "address": "0x3D3D35bb9bEC23b06Ca00fe472b50E7A4c692C30",
    "symbol": "VIDYA",
    "name": "Vidya",
    "decimals": 18,
    "logoURI": "ipfs://QmYgchhaGM1Mm93uZG8eFMTTqdw191ScHfyDP6GKuTP6su"
  },
  {
    "chainId": 1,
    "address": "0x39Bb259F66E1C59d5ABEF88375979b4D20D98022",
    "symbol": "WAX",
    "name": "WAX",
    "decimals": 8,
    "logoURI": "ipfs://QmY7xYvWNGnqoEw1rAH75gdgyY2PBQ5AerNoyccj2qmNJN"
  },
  {
    "chainId": 1,
    "address": "0xAE12C5930881c53715B369ceC7606B70d8EB229f",
    "symbol": "C98",
    "name": "Coin98",
    "decimals": 18,
    "logoURI": "ipfs://QmRQbHrMyudBLwQWkC1GHgyYBDzmgDGbEDWLZZ9ZqNL8dF"
  },
  {
    "chainId": 1,
    "address": "0xbCC8EBABfd6ab52CaA883ed8eBc45b7973700A29",
    "symbol": "YBI",
    "name": "Yubiai",
    "decimals": 18,
    "logoURI": "ipfs://QmX9iyy2jcWzitkqDJkcjE6A3P2RsW7dbjTSEK2itLWNan"
  },
  {
    "chainId": 1,
    "address": "0xE50365f5D679CB98a1dd62D6F6e58e59321BcdDf",
    "symbol": "LA",
    "name": "LATOKEN",
    "decimals": 18,
    "logoURI": "ipfs://Qmf8nXnsZMdLCQvwpvqLxw5jcBcYqyJgNbCMpEoHexKttA"
  },
  {
    "chainId": 42161,
    "address": "0xfc5A1A6EB076a2C7aD06eD22C90d7E710E35ad0a",
    "symbol": "GMX",
    "name": "GMX",
    "decimals": 18,
    "logoURI": "ipfs://QmaJyhkXJfRPxAoSYn6mXvi535zFvhzj166uB7ssXHNcUq"
  },
  {
    "chainId": 42161,
    "address": "0x3082CC23568eA640225c2467653dB90e9250AaA0",
    "symbol": "RDNT",
    "name": "Radiant",
    "decimals": 18,
    "logoURI": "ipfs://QmPFwFqf6KCqR6fBxYffw1FtPGVG6AUACK1dKu79cFvpHt"
  },
  {
    "chainId": 56,
    "address": "0xd32d01A43c869EdcD1117C640fBDcfCFD97d9d65",
    "symbol": "NMX",
    "name": "Nominex",
    "decimals": 18,
    "logoURI": "ipfs://QmRJVuaDvV7dxahY7of5UWyQnDv2Er6kMq1NNRF1sZnbLJ"
  },
  {
    "chainId": 1,
    "address": "0xfa9C3dC54baA9eefBe9453B1f3B3B93aD2AF0A77",
    "symbol": "DST",
    "name": "Dynamic Supply",
    "decimals": 18,
    "logoURI": "ipfs://QmVJTAFfYHfQ8Y5rA5gWzzGtaxS3qcRJtjHbXhjduXzS3Y"
  },
  {
    "chainId": 1,
    "address": "0xC76FB75950536d98FA62ea968E1D6B45ffea2A55",
    "symbol": "COL",
    "name": "Unit Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmdYojc5iV37oowX3QQR5HroYg5igesr9GUsCjJbf55GgX"
  },
  {
    "chainId": 8453,
    "address": "0x2598c30330D5771AE9F983979209486aE26dE875",
    "symbol": "AI",
    "name": "Any Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmVurySKQi4AcZg3wUXXaudowWcTve6vKMbTHpQU6WExat"
  },
  {
    "chainId": 1,
    "address": "0xf04a8ac553FceDB5BA99A64799155826C136b0Be",
    "symbol": "FLIXX",
    "name": "Flixxo",
    "decimals": 18,
    "logoURI": "ipfs://QmVKvyAL1fQKMpycE1XbfJ2Y19i21T2DLkzS7cGdBjMQRj"
  },
  {
    "chainId": 1,
    "address": "0x6f259637dcD74C767781E37Bc6133cd6A68aa161",
    "symbol": "HT",
    "name": "Huobi",
    "decimals": 18,
    "logoURI": "ipfs://QmScJttJGSYVKYyNFhDrVxM7AUDS3bAj9wfCbeMhBBzBsT"
  },
  {
    "chainId": 1,
    "address": "0x92D6C1e31e14520e676a687F0a93788B716BEff5",
    "symbol": "DYDX",
    "name": "dYdX",
    "decimals": 18,
    "logoURI": "ipfs://QmbqcAJHjX4U3DiZkrx2LrvK5PLYejuLTrUs9FXAyY4jMN"
  },
  {
    "chainId": 1,
    "address": "0xdF5e0e81Dff6FAF3A7e52BA697820c5e32D806A8",
    "symbol": "yCrv",
    "name": "yCurve",
    "decimals": 18,
    "logoURI": "ipfs://QmVG1AJgQ1NsHz4NhFwsL5W8M3zZkGFzAdR8J1jdjcEtME"
  },
  {
    "chainId": 137,
    "address": "0x6aBB753C1893194DE4a83c6e8B4EadFc105Fd5f5",
    "symbol": "SXP",
    "name": "Swipe",
    "decimals": 18,
    "logoURI": "ipfs://QmQyhHJ92pNm8RbezaeeNtfUqLyv8sQZ62GfBN5RZqADro"
  },
  {
    "chainId": 56,
    "address": "0x16939ef78684453bfDFb47825F8a5F714f12623a",
    "symbol": "XTZ",
    "name": "Tezos",
    "decimals": 18,
    "logoURI": "ipfs://QmcHtFxQy3TiBRjkMmVxdbgR2dYe16R9B482YFSVUkp3qY"
  },
  {
    "chainId": 1,
    "address": "0x05Ec93c0365baAeAbF7AefFb0972ea7ECdD39CF1",
    "symbol": "aBAT",
    "name": "Aave BAT",
    "decimals": 18,
    "logoURI": "ipfs://QmTmy1fbqwjtaUM55me4Dp7Cxi5fajgzqB7bN2AtjZfEw8"
  },
  {
    "chainId": 1,
    "address": "0x42d6622deCe394b54999Fbd73D108123806f6a18",
    "symbol": "SPANK",
    "name": "SpankChain",
    "decimals": 18,
    "logoURI": "ipfs://QmcZFbGd5z8Gb7Hvmeng2Tr4FxWPoV4GFtLkV6VAHaPRuD"
  },
  {
    "chainId": 56,
    "address": "0x336f5A68fD46A25056a6C1D9c06072c691486acC",
    "symbol": "MIMIR",
    "name": "Mimir",
    "decimals": 18,
    "logoURI": "ipfs://QmZm3fuCFNaE1s3tH6NrgHBFRjbVEAu5XaZ4ASZ8oRyt6f"
  },
  {
    "chainId": 1,
    "address": "0x174aFE7A032b5A33a3270a9f6C30746E25708532",
    "symbol": "HUM",
    "name": "Humanscape",
    "decimals": 18,
    "logoURI": "ipfs://QmcamcWnuD7WsQQRgBszujdijxt3caSptHQqFW4YUTNUjo"
  },
  {
    "chainId": 1,
    "address": "0x5dE83a4b2CA7D85476bb5cA5435E1BF1bbFdE9a2",
    "symbol": "NUTRA",
    "name": "Nutraceutical Finance",
    "decimals": 9,
    "logoURI": "ipfs://QmciBbcysuuGnjDH722j593eL1kmVrBKQx5gAGvchYxMeE"
  },
  {
    "chainId": 1,
    "address": "0x3832d2F059E55934220881F831bE501D180671A7",
    "symbol": "renDOGE",
    "name": "renDOGE",
    "decimals": 8,
    "logoURI": "ipfs://QmWUUKtf1JWi4uGQwRwJ3xBEnT8gNSLGeYmrNKyS36cz6z"
  },
  {
    "chainId": 1,
    "address": "0xD71eCFF9342A5Ced620049e616c5035F1dB98620",
    "symbol": "sEUR",
    "name": "Synthetic Euros",
    "decimals": 18,
    "logoURI": "ipfs://QmSGm9XRV7SFTQYQv7QBbTyzeSo8MsGq6aqgDhyGgT3odN"
  },
  {
    "chainId": 1,
    "address": "0xB97048628DB6B661D4C2aA833e95Dbe1A905B280",
    "symbol": "PAY",
    "name": "TenX",
    "decimals": 18,
    "logoURI": "ipfs://QmQVXvrqzXaEeAGEBAL4AbKqnL6LacnwJVVaYvmqRSB8i2"
  },
  {
    "chainId": 1,
    "address": "0x5adc961D6AC3f7062D2eA45FEFB8D8167d44b190",
    "symbol": "DTH",
    "name": "Dether",
    "decimals": 18,
    "logoURI": "ipfs://QmdihZeKqZtUXab3YEonhwLnPYW5KmYsKfudfWT3vkdV8x"
  },
  {
    "chainId": 137,
    "address": "0x282d8efCe846A88B159800bd4130ad77443Fa1A1",
    "symbol": "OCEAN",
    "name": "Ocean Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmSai6eFJgLb6g3LmnaBRzxc5GKNQE1kLRCLGfmLnBuPE4"
  },
  {
    "chainId": 1,
    "address": "0x875773784Af8135eA0ef43b5a374AaD105c5D39e",
    "symbol": "IDLE",
    "name": "IDLE",
    "decimals": 18,
    "logoURI": "ipfs://QmfSFLnBjTuLoXRFYEuwYAtg24rKcMGpP4uo33ErG7f8SX"
  },
  {
    "chainId": 1,
    "address": "0x4164e5b047842Ad7dFf18fc6A6e63a1e40610f46",
    "symbol": "sSTRK",
    "name": "Strike STRK",
    "decimals": 8,
    "logoURI": "ipfs://QmYLeTvGrfDLyVw1defauXwXZu6UF6RBZ7rAKpWrJ9RvdS"
  },
  {
    "chainId": 1,
    "address": "0x44108f0223A3C3028F5Fe7AEC7f9bb2E66beF82F",
    "symbol": "ACX",
    "name": "Across Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUTywZ71uqSEuG24c1hHAFn3nchW2T3m7KygPAkxJ5Yxk"
  },
  {
    "chainId": 1,
    "address": "0xa8B0F154A688c22142E361707df64277e0A0bE66",
    "symbol": "RAK",
    "name": "Rake Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmS4FC5r2GT4WqXCifBTuDzYvbs3JEzy9eGipfSDoeUZky"
  },
  {
    "chainId": 1,
    "address": "0xe0AeE745b22c522DE621F826a9453f534808DFD0",
    "symbol": "MCU",
    "name": "Meme Coin Universe",
    "decimals": 9,
    "logoURI": "ipfs://QmWhVbMxnEcThmVQ6Th3sFGKiRNsmYMQTQDXxR8Z9yCavL"
  },
  {
    "chainId": 42161,
    "address": "0xE85B662Fe97e8562f4099d8A1d5A92D4B453bF30",
    "symbol": "THALES",
    "name": "Thales",
    "decimals": 18,
    "logoURI": "ipfs://QmeGZ9fRMeNJP9MnMNDQSBGdQzAJL5GUAWWqTd254NUuQJ"
  },
  {
    "chainId": 1,
    "address": "0x1b890fD37Cd50BeA59346fC2f8ddb7cd9F5Fabd5",
    "symbol": "NEWO",
    "name": "New Order",
    "decimals": 18,
    "logoURI": "ipfs://Qmf15uTtmviGrSsiqwsoVKGHUVXVEXx5EKXm1bYeg9kgEi"
  },
  {
    "chainId": 10,
    "address": "0x8700dAec35aF8Ff88c16BdF0418774CB3D7599B4",
    "symbol": "SNX",
    "name": "Synthetix Network",
    "decimals": 18,
    "logoURI": "ipfs://QmX3ov1SgUvUJ64HuEn19RsDkVE91zbUuaCfozReqbbMx5"
  },
  {
    "chainId": 1,
    "address": "0x514cdb9cd8A2fb2BdCf7A3b8DDd098CaF466E548",
    "symbol": "REDPANDA",
    "name": "RedPanda Earth",
    "decimals": 9,
    "logoURI": "ipfs://QmRCNYMxqhQpZ7m4zX1qNr8rpVCtzaezCfP51WeiKS69GM"
  },
  {
    "chainId": 42161,
    "address": "0x3F56e0c36d275367b8C502090EDF38289b3dEa0d",
    "symbol": "MAI",
    "name": "MAI",
    "decimals": 18,
    "logoURI": "ipfs://QmW1f5GFZDXN98kBzESE64QrpEbNrB9RmHBUxFkiz6zNef"
  },
  {
    "chainId": 1,
    "address": "0xDB5C3C46E28B53a39C255AA39A411dD64e5fed9c",
    "symbol": "NCR",
    "name": "Neos Metaverse Credits",
    "decimals": 18,
    "logoURI": "ipfs://QmbsKzFvfDCRbVp7FAFaVED6EQK4mWVaNNZRsb4FbNDbS2"
  },
  {
    "chainId": 1,
    "address": "0xf1f955016EcbCd7321c7266BccFB96c68ea5E49b",
    "symbol": "RLY",
    "name": "Rally",
    "decimals": 18,
    "logoURI": "ipfs://QmTA7bGTi6gBeFMVuWALYCAyWZjX3YZHSAXD8nVwNbciNk"
  },
  {
    "chainId": 1,
    "address": "0xd15eCDCF5Ea68e3995b2D0527A0aE0a3258302F8",
    "symbol": "MCX",
    "name": "Machi X",
    "decimals": 18,
    "logoURI": "ipfs://QmQEGknGnUZabgNr23E3pofVn7ct5KLn6s2kv8aveCMN5i"
  },
  {
    "chainId": 1,
    "address": "0x69692D3345010a207b759a7D1af6fc7F38b35c5E",
    "symbol": "CHADS",
    "name": "CHADS VC",
    "decimals": 18,
    "logoURI": "ipfs://QmcvK2WZuxrALQjPeCnEojcrRKxmN1dAsS5qnMcpZVu9fm"
  },
  {
    "chainId": 1,
    "address": "0x456D8f0D25A4e787eE60c401F8B963a465148f70",
    "symbol": "CAVA",
    "name": "Cavapoo",
    "decimals": 9,
    "logoURI": "ipfs://QmUo9W9sJjsxHjMBGGHTcpS3mAjsDRCcmhLCnp8AqmN32P"
  },
  {
    "chainId": 1,
    "address": "0x8B3192f5eEBD8579568A2Ed41E6FEB402f93f73F",
    "symbol": "SAITAMA",
    "name": "SAITAMA INU",
    "decimals": 9,
    "logoURI": "ipfs://QmWeTkuvpA234u5GpqxwLTmzgJUhWSpcET7Zdwfi7bpgeo"
  },
  {
    "chainId": 1,
    "address": "0x196f4727526eA7FB1e17b2071B3d8eAA38486988",
    "symbol": "RSV",
    "name": "Reserve",
    "decimals": 18,
    "logoURI": "ipfs://QmX38Axj8iBnrVhywMf8DRrdxoZCQi8LMPXSfTbn8N2Lta"
  },
  {
    "chainId": 1,
    "address": "0xfD107B473AB90e8Fbd89872144a3DC92C40Fa8C9",
    "symbol": "LALA",
    "name": "LALA World",
    "decimals": 18,
    "logoURI": "ipfs://QmVDELVWhSukrJjEcKQHCujc4FyY2b8xeTUgXVPmKqaGKk"
  },
  {
    "chainId": 250,
    "address": "0x3028b4395F98777123C7da327010c40f3c7Cc4Ef",
    "symbol": "PREMIA",
    "name": "Premia",
    "decimals": 18,
    "logoURI": "ipfs://Qmbef33x648zGE9v9rthiyz4w9jpGsRSsy39UFGBjhpgts"
  },
  {
    "chainId": 1,
    "address": "0xCDB7eCFd3403Eef3882c65B761ef9B5054890a47",
    "symbol": "HUR",
    "name": "Hurify",
    "decimals": 18,
    "logoURI": "ipfs://QmWABfSc3L4soRim6rKrUMoDoUAkwqgxatWqa8H7mPUpBf"
  },
  {
    "chainId": 1,
    "address": "0x3DE7148c41e3B3233f3310E794F68d8E70Ca69AF",
    "symbol": "SLINK",
    "name": "ShibaLink",
    "decimals": 9,
    "logoURI": "ipfs://QmSLV5QxSAgcyYsJLw2oGsRgmkZ7v1iDKTDjdN9ndHn2Vt"
  },
  {
    "chainId": 1,
    "address": "0xEfaB7248D36585e2340E5d25F8a8D243E6e3193F",
    "symbol": "DACXI",
    "name": "Dacxi",
    "decimals": 18,
    "logoURI": "ipfs://QmXEqiin4UebhJhT6Af23gjyLGLVnqPaDcBzur2fRfrddz"
  },
  {
    "chainId": 1,
    "address": "0xe89a6D0509faF730BD707bf868d9A2A744a363C7",
    "symbol": "crUNI",
    "name": "Cream Uniswap",
    "decimals": 8,
    "logoURI": "ipfs://QmWEEJ14Hi7hmgoD5BsZ7xXNsjmzgJdTEidrnuYM53jZ6V"
  },
  {
    "chainId": 1,
    "address": "0x1A4b46696b2bB4794Eb3D4c26f1c55F9170fa4C5",
    "symbol": "BIT",
    "name": "BitDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmYf2gVcvyyxyVwCncEQeHqjcZLDmEFaAbdZ61bp57QKiH"
  },
  {
    "chainId": 1,
    "address": "0x1681bcB589b3cFCF0c0616B0cE9b19b240643dc1",
    "symbol": "ISLE",
    "name": "Island",
    "decimals": 9,
    "logoURI": "ipfs://QmYNQBF9aif8U3tZnpckXHSCGhA1kgCnAWYDRXm5xzsXCc"
  },
  {
    "chainId": 250,
    "address": "0x74ccbe53F77b08632ce0CB91D3A545bF6B8E0979",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 250,
    "address": "0x15b7c0c907e4C6b9AdaAaabC300C08991D6CEA05",
    "symbol": "GEL",
    "name": "Gelato",
    "decimals": 18,
    "logoURI": "ipfs://QmQe8BUGdw7YRmwVHYYQ1VeQGpCPj4twgCsv22jK23wQmB"
  },
  {
    "chainId": 1,
    "address": "0x0000852600CEB001E08e00bC008be620d60031F2",
    "symbol": "THKD",
    "name": "TrueHKD",
    "decimals": 18,
    "logoURI": "ipfs://QmWqqBjuWoDQ5Ctpyo33tbFbRyV4VtvBhQtPesYQvjKBFe"
  },
  {
    "chainId": 1,
    "address": "0x72B886d09C117654aB7dA13A14d603001dE0B777",
    "symbol": "XDEFI",
    "name": "XDEFI",
    "decimals": 18,
    "logoURI": "ipfs://QmSH3Yoaxr8t1QchiyreXGebwfwS4AgL3STswEefa5TwCd"
  },
  {
    "chainId": 1,
    "address": "0xe9Dc63083c464d6EDcCFf23444fF3CFc6886f6FB",
    "symbol": "yvCurve-oBTC",
    "name": "Curve oBTC Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmZNCb4tDrPCBxvRNgJ5kiSZ4UwNnScyCfp4t4AbbBD6y9"
  },
  {
    "chainId": 250,
    "address": "0xb1F795776cB9DdAC6E7e162f31C7419Dd3d48297",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 1,
    "address": "0x83e2BE8d114F9661221384B3a50d24B96a5653F5",
    "symbol": "ZXC",
    "name": "0xcert",
    "decimals": 18,
    "logoURI": "ipfs://QmSv4MF5DokShCRBTbg8xp1f8q1TXEuUPn8bQ3dj2uSSJw"
  },
  {
    "chainId": 56,
    "address": "0xA719b8aB7EA7AF0DDb4358719a34631bb79d15Dc",
    "symbol": "FRM",
    "name": "Ferrum Network",
    "decimals": 18,
    "logoURI": "ipfs://QmYBq6GWG5hFMn3xsNnyjY2kKJTwPsmVAAgBdhzwW34cNu"
  },
  {
    "chainId": 1,
    "address": "0x9Cb2f26A23b8d89973F08c957C4d7cdf75CD341c",
    "symbol": "DZAR",
    "name": "Digital Rand",
    "decimals": 6,
    "logoURI": "ipfs://QmY3BBPjGtsCaHzd7oMJsPy5s7e1br4Qfv4JGKXwnP8tjy"
  },
  {
    "chainId": 1,
    "address": "0x6C5024Cd4F8A59110119C56f8933403A539555EB",
    "symbol": "aSUSD",
    "name": "Aave SUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmYVWq5z2fpBS4viMQHsFHvr6saQxV62qefrKCsANjBysV"
  },
  {
    "chainId": 1,
    "address": "0x69FE11b042789bC6E2FDaad064AfD32bb298aE05",
    "symbol": "ORDR",
    "name": "The Red Order",
    "decimals": 18,
    "logoURI": "ipfs://QmZcL2WLoBRFchx3EgGZQpwyjAN5sWm3RxtUxsEAPiyGmv"
  },
  {
    "chainId": 1,
    "address": "0xf2051511B9b121394FA75B8F7d4E7424337af687",
    "symbol": "HAUS",
    "name": "DAOhaus",
    "decimals": 18,
    "logoURI": "ipfs://QmP9U4RZVhRUAYdvGirwjVEcqddsQbUScfBpw3xwCSRpqs"
  },
  {
    "chainId": 1,
    "address": "0x6243d8CEA23066d098a15582d81a598b4e8391F4",
    "symbol": "FLX",
    "name": "Reflexer Ungovernance Token",
    "decimals": 18,
    "logoURI": "ipfs://QmcQgBY943BzJ2vPtBZ5EBhnCq2R6xRF7Z1thXF3Qq7wW6"
  },
  {
    "chainId": 1,
    "address": "0x186eF81fd8E77EEC8BfFC3039e7eC41D5FC0b457",
    "symbol": "INSP",
    "name": "Inspect",
    "decimals": 18,
    "logoURI": "ipfs://QmXHbrRkcscRkQfeXtCckvgpUwdYY2pcCtPJ5N4SUv3fre"
  },
  {
    "chainId": 10,
    "address": "0xb12c13e66AdE1F72f71834f2FC5082Db8C091358",
    "symbol": "ROOBEE",
    "name": "ROOBEE",
    "decimals": 18,
    "logoURI": "ipfs://QmSeX7GhUACY2mmcpjd5yL6pxjuHv9CarWvhzbAHHwsbag"
  },
  {
    "chainId": 42161,
    "address": "0x9623063377AD1B27544C965cCd7342f7EA7e88C7",
    "symbol": "GRT",
    "name": "The Graph",
    "decimals": 18,
    "logoURI": "ipfs://Qmf8sMWoxGHRThqGaHKBTsSndkZdbsEyp8DXQvisj6QGH7"
  },
  {
    "chainId": 1,
    "address": "0xb9871cB10738eADA636432E86FC0Cb920Dc3De24",
    "symbol": "PRIA",
    "name": "PRIA",
    "decimals": 18,
    "logoURI": "ipfs://QmVZKUhNnnsai7795PVFPocC79J72M4Byo17ifZAM5PGUZ"
  },
  {
    "chainId": 1,
    "address": "0xAf30D2a7E90d7DC361c8C4585e9BB7D2F6f15bc7",
    "symbol": "1ST",
    "name": "FirstBlood",
    "decimals": 18,
    "logoURI": "ipfs://QmYai96YfRnsvEfSh9Q42WtSbPeMocQg6tNwhEquEq9da1"
  },
  {
    "chainId": 1,
    "address": "0xaff84e86d72EDb971341a6A66eb2dA209446FA14",
    "symbol": "TCAT",
    "name": "The Currency Analytics",
    "decimals": 18,
    "logoURI": "ipfs://QmTihrnEAX8HBko7XzS7cNfrkQYJo78v7FcwduswjMqDC5"
  },
  {
    "chainId": 1,
    "address": "0x7968bc6a03017eA2de509AAA816F163Db0f35148",
    "symbol": "HGET",
    "name": "Hedget",
    "decimals": 6,
    "logoURI": "ipfs://QmNe2qUgYEBYuRTgUkvBN1VzvCsZeisTvBVNfiVNQXTDmb"
  },
  {
    "chainId": 1,
    "address": "0x4c1C4957D22D8F373aeD54d0853b090666F6F9De",
    "symbol": "SLV",
    "name": "Silverway",
    "decimals": 18,
    "logoURI": "ipfs://QmcHJAC2CiQx5Jm1RkKRMYFHfuJXds33qShVq39dGkfSf1"
  },
  {
    "chainId": 1,
    "address": "0x152649eA73beAb28c5b49B26eb48f7EAD6d4c898",
    "symbol": "CAKE",
    "name": "PancakeSwap",
    "decimals": 18,
    "logoURI": "ipfs://Qmci7V6yNrA1wqMbFapLHRMMM7dpdKz2GVCiSypbDUctXf"
  },
  {
    "chainId": 1,
    "address": "0xb47e3cd837dDF8e4c57F05d70Ab865de6e193BBB",
    "symbol": "C",
    "name": "CryptoPunks",
    "decimals": 0,
    "logoURI": "ipfs://QmS7nb3msDTW58evR7iLES1D5QCWEQUBMLuNiaJoBcDkX4"
  },
  {
    "chainId": 100,
    "address": "0xFbdd194376de19a88118e84E279b977f165d01b8",
    "symbol": "DHV",
    "name": "DeHive",
    "decimals": 18,
    "logoURI": "ipfs://QmeWfbxG4G3H8LiWM8cFxZDd5PiYoztAa7xJWB9NrVCdki"
  },
  {
    "chainId": 1,
    "address": "0x9B62513c8a27290CF6A7A9e29386e600245EA819",
    "symbol": "CPT",
    "name": "Contents Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmPb8mveJjSeSmCTUAbShA6qnNGzAfEzrCBP1LJQ2dPJAw"
  },
  {
    "chainId": 1,
    "address": "0x95172ccBe8344fecD73D0a30F54123652981BD6F",
    "symbol": "LOCK",
    "name": "Meridian Network",
    "decimals": 18,
    "logoURI": "ipfs://QmZjq9ftKwCEVuAciXyAV4Ywk9CTGXDgRbPuwJTNtapiRw"
  },
  {
    "chainId": 8453,
    "address": "0xf34e0cff046e154CAfCae502C7541b9E5FD8C249",
    "symbol": "THALES",
    "name": "Thales",
    "decimals": 18,
    "logoURI": "ipfs://QmeGZ9fRMeNJP9MnMNDQSBGdQzAJL5GUAWWqTd254NUuQJ"
  },
  {
    "chainId": 1,
    "address": "0x1d37986F252d0e349522EA6C3B98Cb935495E63E",
    "symbol": "CHART",
    "name": "ChartEx",
    "decimals": 18,
    "logoURI": "ipfs://QmfQLuuN7yf4WEhzzwJ6t2X1tLnGpd5Yk9S542j4uradWz"
  },
  {
    "chainId": 43114,
    "address": "0xaf20f5f19698f1D19351028cd7103B63D30DE7d7",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 1,
    "address": "0x33E07f5055173cF8FeBedE8B21B12D1e2b523205",
    "symbol": "ELAND",
    "name": "Etherland",
    "decimals": 18,
    "logoURI": "ipfs://QmTFFLoGK1qqmDcmBRZCrfxA9xkL5WZVvA7wAZ7DZykiTL"
  },
  {
    "chainId": 1,
    "address": "0x9813037ee2218799597d83D4a5B6F3b6778218d9",
    "symbol": "BONE",
    "name": "BONE SHIBASWAP",
    "decimals": 18,
    "logoURI": "ipfs://Qmf9AQKzGFre6EAkghhfbX6JEepCQV9ArrwmjyopH82iL6"
  },
  {
    "chainId": 1,
    "address": "0x1788430620960F9a70e3DC14202a3A35ddE1A316",
    "symbol": "OAP",
    "name": "OpenAlexa Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUaVsbZLdugd3cBNivA3qXBWPGdz9dsaob4FdEcgAnm4x"
  },
  {
    "chainId": 56,
    "address": "0x965F527D9159dCe6288a2219DB51fc6Eef120dD1",
    "symbol": "BSW",
    "name": "Biswap",
    "decimals": 18,
    "logoURI": "ipfs://Qmc3UdghzHNGbgqdf9EV5hseptxupi5vcgMxM3z5SLFEuP"
  },
  {
    "chainId": 137,
    "address": "0x9a71012B13CA4d3D0Cdc72A177DF3ef03b0E76A3",
    "symbol": "BAL",
    "name": "Balancer",
    "decimals": 18,
    "logoURI": "ipfs://QmbzDmJ2yboeFf45poTkzpqZkDbnsdL24x2BJQTs64yv5Z"
  },
  {
    "chainId": 1,
    "address": "0x2C540c3c7BE7aF98278Dc6963e092cD450009d1F",
    "symbol": "SPARKO",
    "name": "Sparko",
    "decimals": 18,
    "logoURI": "ipfs://Qmdj23X9LKyop8dCo8Hvj8Gv69fHTiCjMSQvCwNWrcmdkA"
  },
  {
    "chainId": 43114,
    "address": "0x2b1D36f5B61AdDAf7DA7ebbd11B35FD8cfb0DE31",
    "symbol": "ITP",
    "name": "Interport",
    "decimals": 18,
    "logoURI": "ipfs://QmTWPTMwXJWuwDgNmUxpdkZA7UASQfrye7t7MESNBuUJox"
  },
  {
    "chainId": 8453,
    "address": "0x74ccbe53F77b08632ce0CB91D3A545bF6B8E0979",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 56,
    "address": "0x9B69b06272980FA6BAd9D88680a71e3c3BeB32c6",
    "symbol": "vlMGP",
    "name": "Vote-Locked MGP",
    "decimals": 18,
    "logoURI": "ipfs://QmTKsJTYqdwFJNMBRyi6kUFAjtB2fCRaac3LV8jiNo1zHK"
  },
  {
    "chainId": 1,
    "address": "0xF0FAC7104aAC544e4a7CE1A55ADF2B5a25c65bD1",
    "symbol": "PAMP",
    "name": "Pamp Network",
    "decimals": 18,
    "logoURI": "ipfs://QmTDYoeBc58x3sLNG1hCqvBrgfRdA1dShhSmo8jfYMph49"
  },
  {
    "chainId": 1,
    "address": "0x0649Cef6D11ed6F88535462E147304d3FE5ae14D",
    "symbol": "KUB",
    "name": "Bitkub",
    "decimals": 18,
    "logoURI": "ipfs://QmNhTMToh21vrGzfqgZWShSRrMAtF3Pis9uEgFwH17p86p"
  },
  {
    "chainId": 56,
    "address": "0x79eBC9A2ce02277A4b5b3A768b1C0A4ed75Bd936",
    "symbol": "CATGIRL",
    "name": "Catgirl",
    "decimals": 9,
    "logoURI": "ipfs://QmVUrwfYfZYv5qvUxv9kbGer4Kyf3nPDtu2Sx7shRaAzSn"
  },
  {
    "chainId": 1,
    "address": "0x3ac2AB91dDF57e2385089202Ca221C360CED0062",
    "symbol": "SWSH",
    "name": "SwapShip",
    "decimals": 18,
    "logoURI": "ipfs://QmSUiMcxsyfot7UsG3NKvnCVSXsT18mhPeAwrenqNpfva4"
  },
  {
    "chainId": 1,
    "address": "0xc12d099be31567add4e4e4d0D45691C3F58f5663",
    "symbol": "AUC",
    "name": "Auctus",
    "decimals": 18,
    "logoURI": "ipfs://QmQs1uUJWxMvtC18CdEZnAxg4Wf9cQF84ZfWp3RxqQDHjA"
  },
  {
    "chainId": 1,
    "address": "0xDF2C7238198Ad8B389666574f2d8bc411A4b7428",
    "symbol": "MFT",
    "name": "Hifi Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmXhErVaPdPZr2DqgXSpxdrySrmPPrmYp7F8bkYT2faJ3o"
  },
  {
    "chainId": 1,
    "address": "0x9c405acf8688AfB61B3197421cDeeC1A266c6839",
    "symbol": "DOGY",
    "name": "DogeYield",
    "decimals": 18,
    "logoURI": "ipfs://QmYjK4jSMfZxqo95HB9U7dsL7D4B7bmrgBoKxmffp3Ej6g"
  },
  {
    "chainId": 1,
    "address": "0x910Dfc18D6EA3D6a7124A6F8B5458F281060fa4c",
    "symbol": "X8X",
    "name": "X8X Token",
    "decimals": 18,
    "logoURI": "ipfs://QmVjVR8TribFRbrpXV5D1dW9R3LMv7dVMKVFFaNsV5ZDXo"
  },
  {
    "chainId": 1,
    "address": "0xCF8335727B776d190f9D15a54E6B9B9348439eEE",
    "symbol": "WHACKD",
    "name": "WHACKD",
    "decimals": 18,
    "logoURI": "ipfs://QmYtmcJu1pAq2a8MiR3xWt5deBvD9ZcLDYRo2bYoyWiWsk"
  },
  {
    "chainId": 1,
    "address": "0xE0e05c43c097B0982Db6c9d626c4eb9e95C3b9ce",
    "symbol": "USF",
    "name": "Unslashed Finance",
    "decimals": 18,
    "logoURI": "ipfs://Qmas75ppaL24asqMJwRWKYbSUjBxWx7KtP2pyj6SvabHpK"
  },
  {
    "chainId": 1,
    "address": "0x2Fb12bccF6f5Dd338b76Be784A93ade072425690",
    "symbol": "BEAT",
    "name": "BEAT",
    "decimals": 18,
    "logoURI": "ipfs://QmXXfW5GX4zncbbQ2ymZJkmzBD1PVf2Aspd2nLi52nSRWt"
  },
  {
    "chainId": 56,
    "address": "0x4e7f408be2d4E9D60F49A64B89Bb619c84C7c6F5",
    "symbol": "PERP",
    "name": "Perpetual Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmVCa9zawdxxxXUUNZRJfVJBdQmzecrsxgUaGysjxVkJK9"
  },
  {
    "chainId": 1,
    "address": "0xc770EEfAd204B5180dF6a14Ee197D99d808ee52d",
    "symbol": "FOX",
    "name": "ShapeShift FOX ",
    "decimals": 18,
    "logoURI": "ipfs://QmRFkU5X6HLcFdeHEwGEJ4NhxyjDWsSP2i81trLYsY4eRG"
  },
  {
    "chainId": 1,
    "address": "0x090185f2135308BaD17527004364eBcC2D37e5F6",
    "symbol": "SPELL",
    "name": "Spell",
    "decimals": 18,
    "logoURI": "ipfs://QmRQ87x9VSD3iyvoqYXqVAcKGUAmUwYu5RiuqExNkjYyfa"
  },
  {
    "chainId": 1,
    "address": "0x1aBaEA1f7C830bD89Acc67eC4af516284b1bC33c",
    "symbol": "EUROC",
    "name": "Euro Coin",
    "decimals": 6,
    "logoURI": "ipfs://Qmcot1uZuiGa6HtXZqs6feW6rx6koECZfca6x5Z5on3GP7"
  },
  {
    "chainId": 1,
    "address": "0xc944E90C64B2c07662A292be6244BDf05Cda44a7",
    "symbol": "GRT",
    "name": "The Graph",
    "decimals": 18,
    "logoURI": "ipfs://QmaCTcJ7pLUgdjdXeZkubcRsGnyT73fvPBsgPLWMZevCfd"
  },
  {
    "chainId": 1,
    "address": "0xD9A947789974bAD9BE77E45C2b327174A9c59D71",
    "symbol": "YSR",
    "name": "Ystar",
    "decimals": 18,
    "logoURI": "ipfs://QmXs6FqwBkrMFaL8Sfiazn6yJroTgm3Cco4Xayw827PyEn"
  },
  {
    "chainId": 1,
    "address": "0xAbeA7663c472648d674bd3403D94C858dFeEF728",
    "symbol": "PUDGY",
    "name": "PUDGY Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmUyMdim1BDzQL9pkjEys62P8DR5oPQkZvdCES54aBMC21"
  },
  {
    "chainId": 1,
    "address": "0xe6fd75ff38Adca4B97FBCD938c86b98772431867",
    "symbol": "ELA",
    "name": "ELA on Ethereum",
    "decimals": 18,
    "logoURI": "ipfs://Qme4Gz4WdCAuCmaZSMED7kQL8YyjVUmyFM5VHY2nUxAzai"
  },
  {
    "chainId": 1,
    "address": "0x0E192d382a36De7011F795Acc4391Cd302003606",
    "symbol": "FST",
    "name": "Futureswap",
    "decimals": 18,
    "logoURI": "ipfs://QmTxTqTehArrC7VjDzbgXqtmd5VQaD59sMRvCNiVnCbR5n"
  },
  {
    "chainId": 1,
    "address": "0x299698B4B44BD6d023981A7317798dEe12860834",
    "symbol": "NFP",
    "name": "New Frontier Presents",
    "decimals": 9,
    "logoURI": "ipfs://QmQSPb3Zhj4Dc2mS4GbZbUUSS5dXr8jayCB9WD3s5cvqu4"
  },
  {
    "chainId": 1,
    "address": "0xfDBc1aDc26F0F8f8606a5d63b7D3a3CD21c22B23",
    "symbol": "1WO",
    "name": "1World",
    "decimals": 8,
    "logoURI": "ipfs://QmeZcCDpg44XS7soXkizvzoLSWjTnX1ivTixztaJCB6ULM"
  },
  {
    "chainId": 56,
    "address": "0x2859e4544C4bB03966803b044A93563Bd2D0DD4D",
    "symbol": "SHIB",
    "name": "Shiba Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmZGG16WF6T62rhJgY7yWExBNNaYo4CiXrdesdxnbfbYfR"
  },
  {
    "chainId": 10,
    "address": "0xFdb794692724153d1488CcdBE0C56c252596735F",
    "symbol": "LDO",
    "name": "Lido DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmTdQtLaPqXoXbHgbjVgYbXbbukvQAsNZtzq9bs14oVknY"
  },
  {
    "chainId": 1,
    "address": "0xfec0cF7fE078a500abf15F1284958F22049c2C7e",
    "symbol": "ART",
    "name": "Maecenas",
    "decimals": 18,
    "logoURI": "ipfs://QmQzHwEULCqDGd3hCMxmxdh6U4HewxoWbUGZyRBeRxfAHV"
  },
  {
    "chainId": 1,
    "address": "0xde4EE8057785A7e8e800Db58F9784845A5C2Cbd6",
    "symbol": "DEXE",
    "name": "DeXe",
    "decimals": 18,
    "logoURI": "ipfs://QmZrmTvwVZFmjhZRBdjh1reCPtNenyaQhNS2cD4U18rd2B"
  },
  {
    "chainId": 1,
    "address": "0x1E8E29CA51363D923725aB9DaC73Bd7e9C440f71",
    "symbol": "MTAO",
    "name": "MEME TAO",
    "decimals": 9,
    "logoURI": "ipfs://QmcefEaa4R1xfDxkwBhbXMiG6qToaECB4fkZx1wosdDmEz"
  },
  {
    "chainId": 1,
    "address": "0x2260FAC5E5542a773Aa44fBCfeDf7C193bc2C599",
    "symbol": "WBTC",
    "name": "Wrapped Bitcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmW8TQjyF7y1BefmvtbFUuMzo6iPL3hmanNeRrb3CSrfuh"
  },
  {
    "chainId": 1,
    "address": "0xEd40834A13129509A89be39a9bE9C0E96A0DDd71",
    "symbol": "WARP",
    "name": "Warp Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYt78dKoPA8toh5D9f9Eh6GfkdeaAVefeKTbe1NDTART7"
  },
  {
    "chainId": 137,
    "address": "0x8505b9d2254A7Ae468c0E9dd10Ccea3A837aef5c",
    "symbol": "COMP",
    "name": "Compound",
    "decimals": 18,
    "logoURI": "ipfs://QmWZSaKRTLqDfF9BDG5HYrgc5fbr9uq3ksHct8c5MfjYUu"
  },
  {
    "chainId": 1,
    "address": "0x5f98805A4E8be255a32880FDeC7F6728C6568bA0",
    "symbol": "LUSD",
    "name": "Liquity USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPzBCxWo5xMA8Wif7wCQQmuZ5ajP5sw4JSHthMxvxY2uR"
  },
  {
    "chainId": 1,
    "address": "0xB8BAa0e4287890a5F79863aB62b7F175ceCbD433",
    "symbol": "SWRV",
    "name": "Swerve",
    "decimals": 18,
    "logoURI": "ipfs://QmRaX4hMrirv57aTbnfCNGUUdCm1bvXujSpLnqhUHs7pX7"
  },
  {
    "chainId": 1,
    "address": "0x96B0bF939D9460095C15251F71Fda11e41DcBddB",
    "symbol": "S",
    "name": "Sharpay",
    "decimals": 18,
    "logoURI": "ipfs://QmYaMEMqaVF6o4aWk175W7VULwVXU9orze8GFs71G1yiFQ"
  },
  {
    "chainId": 137,
    "address": "0x0000000000000000000000000000000000001010",
    "symbol": "MATIC",
    "name": "Polygon",
    "decimals": 18,
    "logoURI": "ipfs://QmcMPwSeDvVa1K8hXKPbU4diCwGC3BXYK4dsNtPNMVFHJQ"
  },
  {
    "chainId": 10,
    "address": "0x374Ad0f47F4ca39c78E5Cc54f1C9e426FF8f231A",
    "symbol": "PREMIA",
    "name": "Premia",
    "decimals": 18,
    "logoURI": "ipfs://Qmbef33x648zGE9v9rthiyz4w9jpGsRSsy39UFGBjhpgts"
  },
  {
    "chainId": 1,
    "address": "0x81AFa8ACa19d5693883B0B1e10ae1Ccaf2F42781",
    "symbol": "FOREVERUP",
    "name": "ForeverUp",
    "decimals": 9,
    "logoURI": "ipfs://QmXJarPqP2wXxx2xWKx3k9pywaqSDbtZ4ocguQr5KUKZuU"
  },
  {
    "chainId": 1,
    "address": "0x0488401c3F535193Fa8Df029d9fFe615A06E74E6",
    "symbol": "SRK",
    "name": "SparkPoint",
    "decimals": 18,
    "logoURI": "ipfs://Qma8YABwskCW9qzp7X1mbXiB9Smqb6Q7CNdbi5SwJq2vDE"
  },
  {
    "chainId": 56,
    "address": "0x6Ba7a8f9063c712C1c8CabC776B1dA7126805f3b",
    "symbol": "TRADE",
    "name": "Polytrade",
    "decimals": 18,
    "logoURI": "ipfs://QmZ8VAAw7szkxXAg8qLCYQo5iCafwYB7GWicam4zyMomzA"
  },
  {
    "chainId": 56,
    "address": "0x9573c88aE3e37508f87649f87c4dd5373C9F31e0",
    "symbol": "MONI",
    "name": "Monsta Infinite",
    "decimals": 18,
    "logoURI": "ipfs://QmcFEdZofPM25C9Da32r5kxwocddhcEDVZUhhc84hxLLtU"
  },
  {
    "chainId": 1,
    "address": "0x00000100F2A2bd000715001920eB70D229700085",
    "symbol": "TCAD",
    "name": "TrueCAD",
    "decimals": 18,
    "logoURI": "ipfs://QmVQDTPJD36wAbynhUt8W4MJVoZKSbdVN9W58vFsJEwdYS"
  },
  {
    "chainId": 1,
    "address": "0xD64809f5F7d772D9112A6BD379De00A77188199E",
    "symbol": "LSILVER",
    "name": "Lyfe Silver",
    "decimals": 18,
    "logoURI": "ipfs://QmfYGbDaqvZrFyoysoFAgwNVfxFeeuqwQ3Y5PZYtz4NvS6"
  },
  {
    "chainId": 1,
    "address": "0xba100000625a3754423978a60c9317c58a424e3D",
    "symbol": "BAL",
    "name": "Balancer",
    "decimals": 18,
    "logoURI": "ipfs://QmYfge9JvkcP16d8RNETBTu4m54JirNH5mzwC8EBb17Dea"
  },
  {
    "chainId": 56,
    "address": "0x1D2F0da169ceB9fC7B3144628dB156f3F6c60dBE",
    "symbol": "XRP",
    "name": "XRP",
    "decimals": 18,
    "logoURI": "ipfs://QmQVRk5TGJWjTWmrsczSryDiDpXNiZc58VwyVVLB2tJUHi"
  },
  {
    "chainId": 1,
    "address": "0x66186008C1050627F979d464eABb258860563dbE",
    "symbol": "MDS",
    "name": "MediShares",
    "decimals": 18,
    "logoURI": "ipfs://QmZWhBuUCJdia2VyTBUL6TZ7i3YTs36eo6MRm3t154Jw3X"
  },
  {
    "chainId": 137,
    "address": "0xE3322702BEdaaEd36CdDAb233360B939775ae5f1",
    "symbol": "TRB",
    "name": "Tellor",
    "decimals": 18,
    "logoURI": "ipfs://QmWvmGB7NhEcfLrqCM6dsRozFytqY2wBC3ordycpzgjQqU"
  },
  {
    "chainId": 1,
    "address": "0x9c354503C38481a7A7a51629142963F98eCC12D0",
    "symbol": "OGV",
    "name": "Origin Dollar Governance",
    "decimals": 18,
    "logoURI": "ipfs://QmaA6TSeLEabCGFVWszBCsAn11H16TvjyhC8f71ZnAfKn9"
  },
  {
    "chainId": 56,
    "address": "0x28A8EceCdf311C0a71231775d6ff4B8481C4ef21",
    "symbol": "XVR",
    "name": "Xover",
    "decimals": 9,
    "logoURI": "ipfs://QmevaUu5Pw172RYsGib17REfz5VzdRorjctPnCec4MR8TD"
  },
  {
    "chainId": 1,
    "address": "0x40395044Ac3c0C57051906dA938B54BD6557F212",
    "symbol": "MGO",
    "name": "MobileGo",
    "decimals": 8,
    "logoURI": "ipfs://QmRDbRmBJNYcBJX4RCywnWRW1HgTGneQeZhJ8Tzy4ZRn6N"
  },
  {
    "chainId": 1,
    "address": "0x5d65D971895Edc438f465c17DB6992698a52318D",
    "symbol": "NAS",
    "name": "Nebulas",
    "decimals": 18,
    "logoURI": "ipfs://QmbjdykdhKo82z8H8KjxNWFwXr665FjCGurN6N4bR4Zmjb"
  },
  {
    "chainId": 1,
    "address": "0x6aEB95F06CDA84cA345c2dE0F3B7f96923a44f4c",
    "symbol": "BERRY",
    "name": "Rentberry",
    "decimals": 14,
    "logoURI": "ipfs://QmesEiNjwX5u36pqzqTpkzTaS8RWZn6JJegMbseQ5UEMeN"
  },
  {
    "chainId": 1,
    "address": "0xB987D48Ed8f2C468D52D6405624EADBa5e76d723",
    "symbol": "STBZ",
    "name": "Stabilize",
    "decimals": 18,
    "logoURI": "ipfs://QmPDKrXZyJrY2zTiSLjG5usyNZvgyy6HxC9Dj3wDQt4Tfo"
  },
  {
    "chainId": 1,
    "address": "0xE5F166c0D8872B68790061317BB6CcA04582C912",
    "symbol": "TFD",
    "name": "TE-FOOD",
    "decimals": 18,
    "logoURI": "ipfs://Qmf2Yi9T2bUU7W7d1DcAMGytGYZJwj3GbTn2TiMBwNZGiP"
  },
  {
    "chainId": 100,
    "address": "0xb90D6bec20993Be5d72A5ab353343f7a0281f158",
    "symbol": "DXD",
    "name": "DXdao",
    "decimals": 18,
    "logoURI": "ipfs://QmSwZkCFvsk7rKRdW8du2rzWBpFf4WWRaJ1jgXeUqcR3ve"
  },
  {
    "chainId": 1,
    "address": "0x14778860E937f509e651192a90589dE711Fb88a9",
    "symbol": "CYBER",
    "name": "CyberConnect",
    "decimals": 18,
    "logoURI": "ipfs://QmWsLPSKnBnhUzET5D5Y53QCL2j3HbPnQxssxtfjHF8Ahx"
  },
  {
    "chainId": 1,
    "address": "0x6be61833FC4381990e82D7D4a9F4c9B3F67eA941",
    "symbol": "HTB",
    "name": "Hotbit Token",
    "decimals": 18,
    "logoURI": "ipfs://QmXD8Zd75PjZEFSPgZ2vS6qyqmSXXNs8Ato4rNfipK8zsD"
  },
  {
    "chainId": 56,
    "address": "0x603c7f932ED1fc6575303D8Fb018fDCBb0f39a95",
    "symbol": "BANANA",
    "name": "ApeSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmYLFQ6Z7Da5zNmZ7sSWmLLdt3yc6sBx15Sqopq9WRnkpS"
  },
  {
    "chainId": 1,
    "address": "0xfb62AE373acA027177D1c18Ee0862817f9080d08",
    "symbol": "DPET",
    "name": "My DeFi Pet",
    "decimals": 18,
    "logoURI": "ipfs://QmSxaPY4voDhTwvu8MqLC4CXdXLimRXwCFcixqfQmjDQWY"
  },
  {
    "chainId": 42161,
    "address": "0x289ba1701C2F088cf0faf8B3705246331cB8A839",
    "symbol": "LPT",
    "name": "Livepeer",
    "decimals": 18,
    "logoURI": "ipfs://QmXtJTcmWE1VEzB94VVwHBvVXpEyW97no84PNvxr3iSiY5"
  },
  {
    "chainId": 1,
    "address": "0x1f9840a85d5aF5bf1D1762F925BDADdC4201F984",
    "symbol": "UNI",
    "name": "Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmXw3PFpzLkvEjSouZiUvBWqasrL8NxHoEDGt87MqsPmtV"
  },
  {
    "chainId": 1,
    "address": "0x1E26b3D07E57F453caE30F7DDd2f945f5bF3EF33",
    "symbol": "XCLR",
    "name": "ClearCoin",
    "decimals": 8,
    "logoURI": "ipfs://QmaWgK5mLag1qVyAX46hxfjjRiHNXrQq9cM3kPUq9mkj9p"
  },
  {
    "chainId": 1,
    "address": "0x1D4cCC31dAB6EA20f461d329a0562C1c58412515",
    "symbol": "TALAO",
    "name": "Talao",
    "decimals": 18,
    "logoURI": "ipfs://QmUL47fZLFqQwUcmHUhcxYFKfbZC1ubvF6XYwDpZ59HwJz"
  },
  {
    "chainId": 137,
    "address": "0x385Eeac5cB85A38A9a07A70c73e0a3271CfB54A7",
    "symbol": "GHST",
    "name": "Aavegotchi",
    "decimals": 18,
    "logoURI": "ipfs://QmNk1f63RgBJ1oWMwM13tAx78HBLwh3oVE9QR1Tf79tFCs"
  },
  {
    "chainId": 42161,
    "address": "0x17FC002b466eEc40DaE837Fc4bE5c67993ddBd6F",
    "symbol": "FRAX",
    "name": "Frax",
    "decimals": 18,
    "logoURI": "ipfs://QmRdVg4RRw7JXdM5enAAC7xkZ1BfgyMxNNqh7q8bhYmjJ7"
  },
  {
    "chainId": 1,
    "address": "0x33c2DA7Fd5B125E629B3950f3c38d7f721D7B30D",
    "symbol": "SEAL",
    "name": "Seal Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmQA7CTqLbkMbpFejLJcSQxzA7vw3knsSZ7DGpH4s6evrK"
  },
  {
    "chainId": 1,
    "address": "0x6fe56C0bcdD471359019FcBC48863d6c3e9d4F41",
    "symbol": "PROPS",
    "name": "Props Token",
    "decimals": 18,
    "logoURI": "ipfs://QmR3hftpW1kLLqCM3R3n1tRARLuDeCYwL573bPEYuUnPiT"
  },
  {
    "chainId": 1,
    "address": "0x67c597624B17b16fb77959217360B7cD18284253",
    "symbol": "MARK",
    "name": "Benchmark Protocol",
    "decimals": 9,
    "logoURI": "ipfs://QmVbQg3qYs2qGvN9KdFK7dEQtMniXY4w5boryTz913BtpE"
  },
  {
    "chainId": 42161,
    "address": "0x371c7ec6D8039ff7933a2AA28EB827Ffe1F52f07",
    "symbol": "JOE",
    "name": "JOE",
    "decimals": 18,
    "logoURI": "ipfs://QmfHbKz1Xm96EpDwsZ6jw3VyxGnbjN8iqqza3kksBqW93E"
  },
  {
    "chainId": 1,
    "address": "0x39C6b3e42d6A679d7D776778Fe880BC9487C2EDA",
    "symbol": "aKNC",
    "name": "Aave KNC",
    "decimals": 18,
    "logoURI": "ipfs://QmNhdckCZu5ihZ9oVVDiYGKZUMT1xpuoAMvfvyqDrv8z7U"
  },
  {
    "chainId": 137,
    "address": "0xE0339c80fFDE91F3e20494Df88d4206D86024cdF",
    "symbol": "ELON",
    "name": "Dogelon Mars",
    "decimals": 18,
    "logoURI": "ipfs://QmaHs61uW9nZYiHVSQuZgnQWBPSwNokcnNgZn7FGMhSHVk"
  },
  {
    "chainId": 1,
    "address": "0x2467AA6B5A2351416fD4C3DeF8462d841feeecEC",
    "symbol": "QBX",
    "name": "qiibee",
    "decimals": 18,
    "logoURI": "ipfs://QmZdMe97hEhuymSDRafGnNT5s1WR6W2rHszMFD6Zzz7fxr"
  },
  {
    "chainId": 1,
    "address": "0x436F0F3a982074c4a05084485D421466a994FE53",
    "symbol": "RTE",
    "name": "Rate3",
    "decimals": 18,
    "logoURI": "ipfs://QmT8JSaJ6PNdvHUucZuDJx9nA9iV2BuK4AhLjtqrDcLKyD"
  },
  {
    "chainId": 1,
    "address": "0xEeEeeeeEe2aF8D0e1940679860398308e0eF24d6",
    "symbol": "ETHV",
    "name": "Ethverse",
    "decimals": 18,
    "logoURI": "ipfs://QmRca5AkNCrU6aJBXcM6nS32ywFotKV9aWRBsBgB7pmV8Q"
  },
  {
    "chainId": 1,
    "address": "0xAFD870F32CE54EfdBF677466B612bf8ad164454B",
    "symbol": "iBNB",
    "name": "Synthetic Inverse Binance Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmW2R8rhF9xEvAwWeUxo2mhaG4KqiBQTuDkWSpceUHP7ng"
  },
  {
    "chainId": 56,
    "address": "0x1610bc33319e9398de5f57B33a5b184c806aD217",
    "symbol": "vDOT",
    "name": "Venus DOT",
    "decimals": 8,
    "logoURI": "ipfs://QmWFncFFkQNSBRggDk3ku52ayiVM5kdWHwDTRCVB4Wx963"
  },
  {
    "chainId": 1,
    "address": "0xeF9Cd7882c067686691B6fF49e650b43AFBBCC6B",
    "symbol": "FNX",
    "name": "FinNexus",
    "decimals": 18,
    "logoURI": "ipfs://QmWCrRCg1J1z9R2BguhsJoFkvqra1iZFV2HSLAtnfRtcfU"
  },
  {
    "chainId": 1,
    "address": "0xAFCdd4f666c84Fed1d8BD825aA762e3714F652c9",
    "symbol": "VINU",
    "name": "Vita Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmbxesZM37FGfPdDRFn6AC9bmxnFAqn5RrmVUrxVXNKjpH"
  },
  {
    "chainId": 1,
    "address": "0x362bc847A3a9637d3af6624EeC853618a43ed7D2",
    "symbol": "PRQ",
    "name": "PARSIQ",
    "decimals": 18,
    "logoURI": "ipfs://QmeAoT4dcbEaXbU3XtWKALQQcx5gRTgLVrfhszfXqCxhMV"
  },
  {
    "chainId": 1,
    "address": "0x5Cf04716BA20127F1E2297AdDCf4B5035000c9eb",
    "symbol": "NKN",
    "name": "NKN",
    "decimals": 18,
    "logoURI": "ipfs://QmaY3NdkWRaoo8x5pzuohkwqtVCGSFK172sJ7AxjSnA9MQ"
  },
  {
    "chainId": 1,
    "address": "0x1961B3331969eD52770751fC718ef530838b6dEE",
    "symbol": "BDG",
    "name": "BitDegree",
    "decimals": 18,
    "logoURI": "ipfs://QmYGWD2G1f54WrSvZSDoUFFKUU7CV8Ty8FxgESNCuMWvCS"
  },
  {
    "chainId": 1,
    "address": "0x24E89bDf2f65326b94E36978A7EDeAc63623DAFA",
    "symbol": "TKING",
    "name": "Tiger King Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmRjABRn4VFXkDA8gcsc64yjrxeUwZzKujTpz9K63PxZYj"
  },
  {
    "chainId": 56,
    "address": "0xaf329a957653675613D0D98f49fc93326AeB36Fc",
    "symbol": "cFRM",
    "name": "Crucible Ferrum",
    "decimals": 18,
    "logoURI": "ipfs://QmeyjVJGLtMxuYVdXEiUCL7hZFNMChyNAgEtojfeRiPKGt"
  },
  {
    "chainId": 56,
    "address": "0x4BA0057f784858a48fe351445C672FF2a3d43515",
    "symbol": "KALM",
    "name": "KALM",
    "decimals": 18,
    "logoURI": "ipfs://QmZUhb1jzCscZULnbM1jz9KScbxFyMrBnSaawcTd6PmfXT"
  },
  {
    "chainId": 56,
    "address": "0xbA2aE424d960c26247Dd6c32edC70B295c744C43",
    "symbol": "DOGE",
    "name": "Dogecoin",
    "decimals": 8,
    "logoURI": "ipfs://QmYHueBwGTHStyjfZqQ83g1YQ1MxDtms7dv29x22MTtFUR"
  },
  {
    "chainId": 1,
    "address": "0x0D8775F648430679A709E98d2b0Cb6250d2887EF",
    "symbol": "BAT",
    "name": "Basic Attention Token",
    "decimals": 18,
    "logoURI": "ipfs://QmcE1QB66Hw6o27brVBvbHiR4EmiyWfpVxfhV4LNvDWL5B"
  },
  {
    "chainId": 1,
    "address": "0x9aeB50f542050172359A0e1a25a9933Bc8c01259",
    "symbol": "OIN",
    "name": "OIN Finance",
    "decimals": 8,
    "logoURI": "ipfs://QmemDSt71fz4AtfGo9TiXYTz6JY4VPZLM8okFLZpFwwtbQ"
  },
  {
    "chainId": 250,
    "address": "0xF24Bcf4d1e507740041C9cFd2DddB29585aDCe1e",
    "symbol": "BEETS",
    "name": "Beethoven X",
    "decimals": 18,
    "logoURI": "ipfs://QmZUxkMhnBeVyFwYWbK9TvbLEmvJVboFCzadNanUXcpDVD"
  },
  {
    "chainId": 1,
    "address": "0x003e0af2916e598Fa5eA5Cb2Da4EDfdA9aEd9Fde",
    "symbol": "BSD",
    "name": "Basis Dollar",
    "decimals": 18,
    "logoURI": "ipfs://QmaSeb6XofaJuw4SNLueyG5J5vgbMeAnqiVjbedmGKB4PX"
  },
  {
    "chainId": 1,
    "address": "0x9E46A38F5DaaBe8683E10793b06749EEF7D733d1",
    "symbol": "NCT",
    "name": "PolySwarm",
    "decimals": 18,
    "logoURI": "ipfs://QmbPmSLsxqk2tjBwkpAUc6WEZcJrwWLe33PF34FBDVJFvu"
  },
  {
    "chainId": 1,
    "address": "0xEe9801669C6138E84bD50dEB500827b776777d28",
    "symbol": "O3",
    "name": "O3 Swap",
    "decimals": 18,
    "logoURI": "ipfs://QmeQutpbBzKJRBXbgEptRhzvibPDesyfKeSpDa3PNPxXt6"
  },
  {
    "chainId": 56,
    "address": "0x334b3eCB4DCa3593BCCC3c7EBD1A1C1d1780FBF1",
    "symbol": "vDAI",
    "name": "Venus DAI",
    "decimals": 8,
    "logoURI": "ipfs://QmYxPe8urZb78gwYtmUxfdxbMsMs2wjoCbLxN4PgNAAHBd"
  },
  {
    "chainId": 1,
    "address": "0xD80F72a6558ec337E0d4CF76b8752B17FA770860",
    "symbol": "RWD",
    "name": "Rewards",
    "decimals": 18,
    "logoURI": "ipfs://Qmemj5biKik1bX2zxmnrT26hPkrfciUv6RZrgHwbam9zcG"
  },
  {
    "chainId": 56,
    "address": "0xFd7B3A77848f1C2D67E05E54d78d174a0C850335",
    "symbol": "ONT",
    "name": "Ontology",
    "decimals": 18,
    "logoURI": "ipfs://QmXWcBanCvWmAuV2Jz1zboBKAKPLLcj8gnH6RxeUmvZ3aL"
  },
  {
    "chainId": 1,
    "address": "0xb59490aB09A0f526Cc7305822aC65f2Ab12f9723",
    "symbol": "LIT",
    "name": "Litentry",
    "decimals": 18,
    "logoURI": "ipfs://QmeKqKoAHZKzgpDYz57ySgvvFS5YJgZYHniB1VAcyqCdAq"
  },
  {
    "chainId": 1,
    "address": "0x00006100F7090010005F1bd7aE6122c3C2CF0090",
    "symbol": "TAUD",
    "name": "TrueAUD",
    "decimals": 18,
    "logoURI": "ipfs://QmX2EgpvnCLLUGGVCQBk1jNDJWY3gQ1FUa273BrfWYEfz3"
  },
  {
    "chainId": 1,
    "address": "0x9355372396e3F6daF13359B7b607a3374cc638e0",
    "symbol": "WHALE",
    "name": "WHALE",
    "decimals": 4,
    "logoURI": "ipfs://QmbxcpkL2nNyjXszvmxte15WyJmoykx5XA7kxnzbRN7Eeh"
  },
  {
    "chainId": 1,
    "address": "0x4575f41308EC1483f3d399aa9a2826d74Da13Deb",
    "symbol": "OXT",
    "name": "Orchid",
    "decimals": 18,
    "logoURI": "ipfs://QmWGLZYzVETzBV6vk1YeSkaFyaTQp2WSUTkuDm5sfWLGoA"
  },
  {
    "chainId": 1,
    "address": "0x6B4c7A5e3f0B99FCD83e9c089BDDD6c7FCe5c611",
    "symbol": "MM",
    "name": "Million",
    "decimals": 18,
    "logoURI": "ipfs://QmT11zD7MsKo1mGrjNXxWJJFyueQD8xwRf32BVBxDoSLb9"
  },
  {
    "chainId": 1,
    "address": "0x1F8A626883d7724DBd59eF51CBD4BF1Cf2016D13",
    "symbol": "STAK",
    "name": "Jigstack",
    "decimals": 18,
    "logoURI": "ipfs://QmXAibTyn9C6fBeYdawDXkvTjXPAcuJ5tfEE3LexXczMYx"
  },
  {
    "chainId": 1,
    "address": "0x7C5fF719a6c76fe643e9eCd0F11F146a2de05F14",
    "symbol": "GNSH",
    "name": "Ganesha Token",
    "decimals": 18,
    "logoURI": "ipfs://QmbEiY5WnTRrgFrtKinFGfc946GbBK2oDecLsj44CyeULN"
  },
  {
    "chainId": 250,
    "address": "0xE992bEAb6659BFF447893641A378FbbF031C5bD6",
    "symbol": "WIGO",
    "name": "WigoSwap",
    "decimals": 18,
    "logoURI": "ipfs://Qmf9B74SsC3dJeQk6xmRokQNEWzaQfr3G5EgjnnkLZmYah"
  },
  {
    "chainId": 1,
    "address": "0x990f341946A3fdB507aE7e52d17851B87168017c",
    "symbol": "STRONG",
    "name": "Strong",
    "decimals": 18,
    "logoURI": "ipfs://QmZbtrthpGiMSNcXkDgpJhmD8Ctet5JtAJ8vgvsA88sSxM"
  },
  {
    "chainId": 1,
    "address": "0x236d53148F83706C3d670064809577385f923a75",
    "symbol": "SHUSKY",
    "name": "Siberian Husky",
    "decimals": 9,
    "logoURI": "ipfs://QmZuzsYxZ4fbmNBPtXvwgrSaVz9umkh62RhrczWF76uLuo"
  },
  {
    "chainId": 42161,
    "address": "0xE22c452bd2AdE15DfC8ad98286bc6bDF0c9219b7",
    "symbol": "TRADE",
    "name": "Polytrade",
    "decimals": 18,
    "logoURI": "ipfs://QmZ8VAAw7szkxXAg8qLCYQo5iCafwYB7GWicam4zyMomzA"
  },
  {
    "chainId": 1,
    "address": "0xd50c1746D835d2770dDA3703B69187bFfeB14126",
    "symbol": "iETC",
    "name": "Synthetic Inverse Ethereum Classic",
    "decimals": 18,
    "logoURI": "ipfs://QmQumUMVEtVHCpM3dwxfAt5WubGRKy4ankUHjy6fF8wAss"
  },
  {
    "chainId": 250,
    "address": "0xCfC785741Dc0e98ad4c9F6394Bb9d43Cd1eF5179",
    "symbol": "ankrFTM",
    "name": "Ankr Staked FTM",
    "decimals": 18,
    "logoURI": "ipfs://QmRCNQL8d2gb2mFq1vuLkEnJFQE8H5hDpvVp4xaGMESWZc"
  },
  {
    "chainId": 1,
    "address": "0x72e4f9F808C49A2a61dE9C5896298920Dc4EEEa9",
    "symbol": "BITCOIN",
    "name": "HarryPotterObamaSonic10Inu",
    "decimals": 8,
    "logoURI": "ipfs://QmZq6VNLg1aaHYNcKUDwWEGgbEC2SDCouasu6Cwn4RvV4P"
  },
  {
    "chainId": 1,
    "address": "0x57Ab1ec28D129707052df4dF418D58a2D46d5f51",
    "symbol": "sUSD",
    "name": "Synthetic US Dollars",
    "decimals": 18,
    "logoURI": "ipfs://QmdFHhBRXu4rp9NW7qBAdunM4HkbtaSR4D2EzEwFozXaD1"
  },
  {
    "chainId": 1,
    "address": "0x48FF53777F747cFB694101222a944dE070c15D36",
    "symbol": "IMP",
    "name": "Ether Kingdoms",
    "decimals": 7,
    "logoURI": "ipfs://QmYzULe8z71wvav2qLDZtFxB6yinrRiQiHBhfJeRBBZzBV"
  },
  {
    "chainId": 1,
    "address": "0x61107a409fFFe1965126aa456Af679719695C69C",
    "symbol": "UMI",
    "name": "Umi Digital",
    "decimals": 18,
    "logoURI": "ipfs://QmRmB9NdyQGocpmWVZkJEABSa2cFDatvnEetQyqfvzkZji"
  },
  {
    "chainId": 1,
    "address": "0x49d716DFe60b37379010A75329ae09428f17118d",
    "symbol": "plDai",
    "name": "Pool Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmZLtsc4ctdFnPF7NgKmpuLo3Cc2RP6Ph8EgRkpGuG5ie4"
  },
  {
    "chainId": 1,
    "address": "0xC7a8B45E184138114E6085C82936A8Db93DD156a",
    "symbol": "MASK",
    "name": "MASK Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmPs2By1jcxFHm8HKUaVxSbDVihfa2ZvhBqNGpjtzPB7rq"
  },
  {
    "chainId": 1,
    "address": "0x9f4F1C679FC8479Cd71482A8D8C88Dc60c1403f9",
    "symbol": "DAA",
    "name": "Double Ace",
    "decimals": 8,
    "logoURI": "ipfs://QmcfmdG6w3epZWDPY12kAzZmgWJbweyrpcAZArMz6j1VPZ"
  },
  {
    "chainId": 1,
    "address": "0x55696EfC7c9779d868Ac34aC6b4a4C5FeD61aC12",
    "symbol": "DSTR",
    "name": "Dynamic Supply Tracker",
    "decimals": 18,
    "logoURI": "ipfs://QmZ7nWXGx3q5tddhXxKgxE5e17m5fmYWuUmkT7q5KgCtmW"
  },
  {
    "chainId": 1,
    "address": "0xD478161C952357F05f0292B56012Cd8457F1cfbF",
    "symbol": "POLK",
    "name": "Polkamarkets",
    "decimals": 18,
    "logoURI": "ipfs://QmRz7vRdug8ZcKcMt2Ch6i4Vr3v4hmAGoWPEMUTUkFQ9Qk"
  },
  {
    "chainId": 1,
    "address": "0x34Be5b8C30eE4fDe069DC878989686aBE9884470",
    "symbol": "SENATE",
    "name": "SENATE",
    "decimals": 18,
    "logoURI": "ipfs://QmQ1femQVjaoTwL4N2513D5rx1dBoW7ZGRpnVqJdeWhwEf"
  },
  {
    "chainId": 1,
    "address": "0xF80D589b3Dbe130c270a69F1a69D050f268786Df",
    "symbol": "DAM",
    "name": "Datamine",
    "decimals": 18,
    "logoURI": "ipfs://QmdXNjxyrV2p3v8JapZ4VnVMmu8GNqDpsoCYk1KGfY6E72"
  },
  {
    "chainId": 1,
    "address": "0xFd2a8fA60Abd58Efe3EeE34dd494cD491dC14900",
    "symbol": "a3Crv",
    "name": "a3Curve",
    "decimals": 18,
    "logoURI": "ipfs://QmYwuAvfwoycX6nSG98tYVF1mg4J8eCpnM941NTtpSR3kK"
  },
  {
    "chainId": 10,
    "address": "0x6806411765Af15Bddd26f8f544A34cC40cb9838B",
    "symbol": "frxETH",
    "name": "Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmQqQhtwApaT4X9FAaAzBRvyeE2iaSDH4ahD6exWXcszZU"
  },
  {
    "chainId": 1,
    "address": "0x5228a22e72ccC52d415EcFd199F99D0665E7733b",
    "symbol": "pBTC",
    "name": "pTokens BTC",
    "decimals": 18,
    "logoURI": "ipfs://QmQ64jVm7aBCQ16bSbTBZyL8AZKhHiA1BKRWRqXFL46tAe"
  },
  {
    "chainId": 1,
    "address": "0xEB4C2781e4ebA804CE9a9803C67d0893436bB27D",
    "symbol": "renBTC",
    "name": "renBTC",
    "decimals": 8,
    "logoURI": "ipfs://QmchFjgxwBKwokrsRyTJhENAcFhLnBrEujtAeCvqEhoNB9"
  },
  {
    "chainId": 56,
    "address": "0xfEbe8C1eD424DbF688551D4E2267e7A53698F0aa",
    "symbol": "VINU",
    "name": "Vita Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmZ4SxmMu7JJP9tXi6wCDtncNqTTUt691Vr7v1sYR6eQhV"
  },
  {
    "chainId": 1,
    "address": "0xA823E6722006afe99E91c30FF5295052fe6b8E32",
    "symbol": "NEU",
    "name": "Neumark",
    "decimals": 18,
    "logoURI": "ipfs://QmPqPhFGsKydWqrU2cu8sRFApoE8d6BiWYTWqGn1eCJVnn"
  },
  {
    "chainId": 1,
    "address": "0xA9B1Eb5908CfC3cdf91F9B8B3a74108598009096",
    "symbol": "AUCTION",
    "name": "Bounce",
    "decimals": 18,
    "logoURI": "ipfs://QmP95V5miG6w9hNetFeTSbMpTmUZFbcc3fwHG5gp72QUuC"
  },
  {
    "chainId": 1,
    "address": "0x3F382DbD960E3a9bbCeaE22651E88158d2791550",
    "symbol": "GHST",
    "name": "Aavegotchi",
    "decimals": 18,
    "logoURI": "ipfs://QmNk1f63RgBJ1oWMwM13tAx78HBLwh3oVE9QR1Tf79tFCs"
  },
  {
    "chainId": 1,
    "address": "0x4463e6A3dEd0dBE3F6e15bC8420dFc55e5FeA830",
    "symbol": "TXA",
    "name": "TXA",
    "decimals": 18,
    "logoURI": "ipfs://QmdEGwAAccqRudQUZgeWr5RbYaE5DR5fWTe8CiAMpdun1C"
  },
  {
    "chainId": 10,
    "address": "0xa3B615667CBd33cfc69843Bf11Fbb2A1D926BD46",
    "symbol": "mPendle",
    "name": "mPendle",
    "decimals": 18,
    "logoURI": "ipfs://Qmdi1PC1TGnGg7uEVtM9tcrsb7DbcHDasEyrRm4jeaYJoL"
  },
  {
    "chainId": 1,
    "address": "0x9783B81438C24848f85848f8df31845097341771",
    "symbol": "COLLAR",
    "name": "Dog Collar",
    "decimals": 18,
    "logoURI": "ipfs://QmWpX1Fg49v73h4YR1XTSbK9gjuqTD6bddqP42aoqUPYBD"
  },
  {
    "chainId": 1,
    "address": "0x7448c7456a97769F6cD04F1E83A4a23cCdC46aBD",
    "symbol": "MAV",
    "name": "Maverick Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmYkDjrskzi8aUwXCMgy7LhQdUttqi3y9yzZjxG5Zx238t"
  },
  {
    "chainId": 1,
    "address": "0x272F97b7a56a387aE942350bBC7Df5700f8a4576",
    "symbol": "aBAL",
    "name": "Aave BAL",
    "decimals": 18,
    "logoURI": "ipfs://QmW5aXqc8LGgiBnH997P5uqAq7zYhr1KSg4wVyJtiMj5TV"
  },
  {
    "chainId": 1,
    "address": "0xE452E6Ea2dDeB012e20dB73bf5d3863A3Ac8d77a",
    "symbol": "WCELO",
    "name": "Wrapped Celo",
    "decimals": 18,
    "logoURI": "ipfs://QmZuuzQUgfGxk433KFLQ8zEZ5ffrDSNDnGuoH2e2p2HdWN"
  },
  {
    "chainId": 1,
    "address": "0x8A1E3930FDe1f151471c368fDBb39F3F63A65B55",
    "symbol": "BEZ",
    "name": "Bezop",
    "decimals": 18,
    "logoURI": "ipfs://QmfDveL1p6GkZWAt879PKw1RRnBz7JwuPJPjZjYSaVEkr2"
  },
  {
    "chainId": 1,
    "address": "0xb2F7EB1f2c37645bE61d73953035360e768D81E6",
    "symbol": "COB",
    "name": "Cobinhood",
    "decimals": 18,
    "logoURI": "ipfs://QmUPWsYFf8i6WfME9DdgdMhbBfkQq92DFADtdtmzwob7Gn"
  },
  {
    "chainId": 1,
    "address": "0x2A537Fa9FFaea8C1A41D3C2B68a9cb791529366D",
    "symbol": "crDPI",
    "name": "Cream DefiPulse Index",
    "decimals": 8,
    "logoURI": "ipfs://QmfPUT1NHKeFFNnNePdLFj4s8GPG5vaHzHi5kPSN8Eypwd"
  },
  {
    "chainId": 250,
    "address": "0xae75A438b2E0cB8Bb01Ec1E1e376De11D44477CC",
    "symbol": "SUSHI",
    "name": "Sushi",
    "decimals": 18,
    "logoURI": "ipfs://QmWmSKdZMf2kjte9Drd7AUX5VTj9FF9bSvbctgawzAcxet"
  },
  {
    "chainId": 1,
    "address": "0x8947da500Eb47F82df21143D0C01A29862a8C3c5",
    "symbol": "THALES",
    "name": "Thales",
    "decimals": 18,
    "logoURI": "ipfs://QmeGZ9fRMeNJP9MnMNDQSBGdQzAJL5GUAWWqTd254NUuQJ"
  },
  {
    "chainId": 1,
    "address": "0xb4d0FDFC8497AEF97d3c2892AE682eE06064A2BC",
    "symbol": "FMF",
    "name": "Formosa Financial",
    "decimals": 18,
    "logoURI": "ipfs://QmWNtE8cQba7Tb5ZSPkQsmAbD6QT3ZhQChEW7f3wV3ZRMV"
  },
  {
    "chainId": 1,
    "address": "0xE1Fc4455f62a6E89476f1072530C20CF1A0622dA",
    "symbol": "PHTR",
    "name": "Phuture",
    "decimals": 18,
    "logoURI": "ipfs://QmWvPPzHXfsXQfD5vCoGiy9LCWfazzuct7E8SejbzWfqd7"
  },
  {
    "chainId": 1,
    "address": "0x7fc2a3a998E8a80dA7F2962C5A8803b641711e6c",
    "symbol": "EPARA",
    "name": "Paralism-EPARA",
    "decimals": 9,
    "logoURI": "ipfs://QmWbCUjpe2tnuqo7NhhUomyLNxMxUvzvhV4kwSeJpahWno"
  },
  {
    "chainId": 1,
    "address": "0x0E29e5AbbB5FD88e28b2d355774e73BD47dE3bcd",
    "symbol": "HAKKA",
    "name": "Hakka Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmVirPaAizCmuCg8ZgsQpTPaBhSFcwdFjDGZtewhUjipmp"
  },
  {
    "chainId": 1,
    "address": "0x6810e776880C02933D47DB1b9fc05908e5386b96",
    "symbol": "GNO",
    "name": "Gnosis",
    "decimals": 18,
    "logoURI": "ipfs://QmWPGom11TShpqTs2sKytGCaYgnXDgK8RbXmhR83JLwzMY"
  },
  {
    "chainId": 1,
    "address": "0xd26fb114401Ec86887CD09F62eccd95fCf20B571",
    "symbol": "BCP",
    "name": "Bitcoin Platinums",
    "decimals": 8,
    "logoURI": "ipfs://QmdWuPy34VCBVci6bpYL2oQmjPnSnNLfNX18oyPKVmaFZd"
  },
  {
    "chainId": 1,
    "address": "0x763Fa6806e1acf68130D2D0f0df754C93cC546B2",
    "symbol": "LIT",
    "name": "Lition",
    "decimals": 18,
    "logoURI": "ipfs://QmXh8PagSs9rkACT6r8CBWqTvtM6dMZgcAz1MVTZH8PpzR"
  },
  {
    "chainId": 137,
    "address": "0x45c32fA6DF82ead1e2EF74d17b76547EDdFaFF89",
    "symbol": "FRAX",
    "name": "Frax",
    "decimals": 18,
    "logoURI": "ipfs://QmbE2HVRj4eL1pEivnRYTHB4WjuBfzyFzT7ZLDgA8oYYj8"
  },
  {
    "chainId": 137,
    "address": "0xD85d1e945766Fea5Eda9103F918Bd915FbCa63E6",
    "symbol": "CEL",
    "name": "Celsius",
    "decimals": 4,
    "logoURI": "ipfs://QmTpTS9D6SruWLzWADgQ5y9XoXxyaf8uEzFuWbhdbMdSB4"
  },
  {
    "chainId": 1,
    "address": "0xB983E01458529665007fF7E0CDdeCDB74B967Eb6",
    "symbol": "iETH",
    "name": "Fulcrum ETH iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmcH39MQG9uLEsKWZgxAMpN1zJWwEfMxf1pPFreobiPMtZ"
  },
  {
    "chainId": 1,
    "address": "0x408e41876cCCDC0F92210600ef50372656052a38",
    "symbol": "REN",
    "name": "Ren",
    "decimals": 18,
    "logoURI": "ipfs://QmX7veANMSVWJNQVNiHcfahZiKSASiX7iVzS2C6ExQSeVV"
  },
  {
    "chainId": 1,
    "address": "0xB8c77482e45F1F44dE1745F52C74426C631bDD52",
    "symbol": "BNB",
    "name": "Binance Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmaL9gA7a2kEYabFzHpWWf2b9f1UDeDihQon4hFmgtNesR"
  },
  {
    "chainId": 1,
    "address": "0xCbd0595113c173Ab7746347D0600D390a85a7EEa",
    "symbol": "ACE",
    "name": "Ascend",
    "decimals": 9,
    "logoURI": "ipfs://QmNcNDMaobMCgoxGg2ks8svsuvnVXRqyiGwR6fyikH5DJv"
  },
  {
    "chainId": 1,
    "address": "0x8dB253a1943DdDf1AF9bcF8706ac9A0Ce939d922",
    "symbol": "UNB",
    "name": "Unbound",
    "decimals": 18,
    "logoURI": "ipfs://QmTi7Gkc4Yex5NG2sDUR59TZX62XJSugvY8ibeYX4qmETn"
  },
  {
    "chainId": 56,
    "address": "0x1F1C90aEb2fd13EA972F0a71e35c0753848e3DB0",
    "symbol": "CHEEL",
    "name": "Cheelee",
    "decimals": 18,
    "logoURI": "ipfs://Qmb4ydXnxVXDpcXDZec9ZGFg5BKqDQuudUirEdoyLphE89"
  },
  {
    "chainId": 1,
    "address": "0x3845badAde8e6dFF049820680d1F14bD3903a5d0",
    "symbol": "SAND",
    "name": "The Sandbox",
    "decimals": 18,
    "logoURI": "ipfs://Qmenn3oa4uM3St91dH4QUTpwT1ium5pwfcNiTjCywrzCLj"
  },
  {
    "chainId": 1,
    "address": "0x08f5a9235B08173b7569F83645d2c7fB55e8cCD8",
    "symbol": "TNT",
    "name": "Tierion",
    "decimals": 8,
    "logoURI": "ipfs://QmZr9T31z5XjN7MkN6rbudqGVX8k1kWmFkuWxMoYq7Qw3v"
  },
  {
    "chainId": 1,
    "address": "0xd49EFA7BC0D339D74f487959C573d518BA3F8437",
    "symbol": "SHLD",
    "name": "Shield Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmNw5uXqXj6dqDBn1ACppkgx6Dwsih7372bY7zV4eA7Zew"
  },
  {
    "chainId": 1,
    "address": "0x8CE9137d39326AD0cD6491fb5CC0CbA0e089b6A9",
    "symbol": "SXP",
    "name": "Swipe",
    "decimals": 18,
    "logoURI": "ipfs://QmcrAPgDkqroEd6mjoGaYsF18cAp7bM8ZKLMfL42qurv1P"
  },
  {
    "chainId": 1,
    "address": "0x97Aa8e14db0bc073cC7e2d42AC715427717d6042",
    "symbol": "SPUNK",
    "name": "SPUNK Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmT5HicDCbyzPQdbUCJ41QAQhKv65VNQ1vzjY8W3DAfpaa"
  },
  {
    "chainId": 1,
    "address": "0x4DC3643DbC642b72C158E7F3d2ff232df61cb6CE",
    "symbol": "AMB",
    "name": "Ambrosus",
    "decimals": 18,
    "logoURI": "ipfs://QmeG8zDU4UvgyccdVQnyG2HTSq5kn8d9f4EqiPUaYDyPFF"
  },
  {
    "chainId": 1,
    "address": "0xBBbbCA6A901c926F240b89EacB641d8Aec7AEafD",
    "symbol": "LRC",
    "name": "Loopring",
    "decimals": 18,
    "logoURI": "ipfs://QmV2PzSr5cYpbZqZpnRK5uks8GePKmXYGyv8Zu7LRNbdy2"
  },
  {
    "chainId": 56,
    "address": "0xD3b71117E6C1558c1553305b44988cd944e97300",
    "symbol": "YEL",
    "name": "YEL.Finance",
    "decimals": 18,
    "logoURI": "ipfs://Qmc9AXNryPwhTk5x9V3A6GSEzvJPiEjJdiomTwTXckhuVX"
  },
  {
    "chainId": 1,
    "address": "0x8CB924583681cbFE487A62140a994A49F833c244",
    "symbol": "SWAPP",
    "name": "Swapp",
    "decimals": 18,
    "logoURI": "ipfs://QmSTrWkt9DyFdQsWQkjZMBdEVHEewckZrGvkzZ6x8UeKER"
  },
  {
    "chainId": 1,
    "address": "0x97D0CfEB4FdE54B430307c9482d6f79C761Fe9B6",
    "symbol": "EPOCH",
    "name": "Epoch",
    "decimals": 18,
    "logoURI": "ipfs://QmPLKVGX5NXPo9f52HjKaBaXUhAxMmyJbSZRzW98qdQikC"
  },
  {
    "chainId": 1,
    "address": "0xdCD90C7f6324cfa40d7169ef80b12031770B4325",
    "symbol": "yvCurve-stETH",
    "name": "Curve stETH Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmZgahRFnsGgFQNJmhRpW6FxREj5DQtuikiYswkRA5gY3t"
  },
  {
    "chainId": 1,
    "address": "0x8414Db07a7F743dEbaFb402070AB01a4E0d2E45e",
    "symbol": "yvCurve-sBTC",
    "name": "Curve sBTC Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://Qmb23DnVPSAokuZJk83KF9XiiVgT3iB9dfDqrQ39pbHx8D"
  },
  {
    "chainId": 1,
    "address": "0xaC6Df26a590F08dcC95D5a4705ae8abbc88509Ef",
    "symbol": "aENJ",
    "name": "Aave ENJ",
    "decimals": 18,
    "logoURI": "ipfs://QmeSANptmBuCqiXtyCfaniuuUQJ9tobmWxzxTaFcLUjsco"
  },
  {
    "chainId": 1,
    "address": "0x42476F744292107e34519F9c357927074Ea3F75D",
    "symbol": "LOOM",
    "name": "Loom Network",
    "decimals": 18,
    "logoURI": "ipfs://QmTwKnc9313STRaDtWvQZ5Y5RbreaKxEX5thDWGwhDbc3n"
  },
  {
    "chainId": 100,
    "address": "0x4537e328Bf7e4eFA29D05CAeA260D7fE26af9D74",
    "symbol": "UNI",
    "name": "Uniswap",
    "decimals": 18,
    "logoURI": "ipfs://QmSxMQhDUfjuLVwThzkiijQkMH2JsjJiEoRX4DLxeWN1h7"
  },
  {
    "chainId": 1,
    "address": "0xEa319e87Cf06203DAe107Dd8E5672175e3Ee976c",
    "symbol": "SURF",
    "name": "SURF Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmXo9HwGr6EvRU5jpaVJ3izSViGKHyvWfcTGdqbsy8ihUo"
  },
  {
    "chainId": 43114,
    "address": "0xAfE3d2A31231230875DEe1fa1eEF14a412443d22",
    "symbol": "DFIAT",
    "name": "DeFiato",
    "decimals": 18,
    "logoURI": "ipfs://QmVdCKa4U5hP4y82CoEPDiTJkKoQFLY6JwnmbmGZxGSvrb"
  },
  {
    "chainId": 56,
    "address": "0x71eebA415A523F5C952Cc2f06361D5443545Ad28",
    "symbol": "XDAO",
    "name": "XDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmYQZ8Qav3op7jpqYS3CAM541XKthqxNcVyYdpHthXPy2J"
  },
  {
    "chainId": 1,
    "address": "0x3B3Ac5386837Dc563660FB6a0937DFAa5924333B",
    "symbol": "bCrv",
    "name": "bCurve",
    "decimals": 18,
    "logoURI": "ipfs://QmTd665kZGY7kN2hRD5Ubix1y8vbTNqpVkgFt5CgdKdv46"
  },
  {
    "chainId": 56,
    "address": "0x0465aad9da170798433F4aB7fa7Ec8b9b9bf0Bb1",
    "symbol": "mPendle",
    "name": "mPendle",
    "decimals": 18,
    "logoURI": "ipfs://Qmdi1PC1TGnGg7uEVtM9tcrsb7DbcHDasEyrRm4jeaYJoL"
  },
  {
    "chainId": 1,
    "address": "0xf0f9D895aCa5c8678f706FB8216fa22957685A13",
    "symbol": "CULT",
    "name": "Cult DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmZ2SNcaKSXWjN1eYXw5zWfqtz73BQzwbwPqJdKJAZerza"
  },
  {
    "chainId": 1,
    "address": "0xE4f726Adc8e89C6a6017F01eadA77865dB22dA14",
    "symbol": "BCP",
    "name": "PieDAO Balanced Crypto Pie",
    "decimals": 18,
    "logoURI": "ipfs://QmT5HcWcsKuFY8c5Fa4PpMPL6BV5pjWB8KwWETSakZSa7K"
  },
  {
    "chainId": 1,
    "address": "0x3DA434F76226b9489ED4277eb6fDccdf03FF190e",
    "symbol": "ZION",
    "name": "ZION",
    "decimals": 18,
    "logoURI": "ipfs://QmZedF8s8kTASnXTUXGk6g1KXK1WopGaiSLsXAPptVNqG7"
  },
  {
    "chainId": 1,
    "address": "0xb339FcA531367067e98d7c4f9303Ffeadff7B881",
    "symbol": "ALD",
    "name": "Aludra Network",
    "decimals": 18,
    "logoURI": "ipfs://QmP2GPust8uZZJqcdHF23eRvB5toiu1zqmPWbbUZnanLai"
  },
  {
    "chainId": 1,
    "address": "0x4946Fcea7C692606e8908002e55A582af44AC121",
    "symbol": "FOAM",
    "name": "FOAM",
    "decimals": 18,
    "logoURI": "ipfs://Qmcxop712UqLd53NLxMVYnJmfVLVfN7ZpQtrzVnicyeyfb"
  },
  {
    "chainId": 1,
    "address": "0x5A98FcBEA516Cf06857215779Fd812CA3beF1B32",
    "symbol": "LDO",
    "name": "Lido DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmRnKWB78HG1kxPnWHr7QWAvp92AripDcXsrbdm2kR5dPS"
  },
  {
    "chainId": 1,
    "address": "0x32c319AC8d2f4A79b2fBDeA543473cC03a986AaF",
    "symbol": "AVENGER",
    "name": "AVENGER",
    "decimals": 9,
    "logoURI": "ipfs://QmaoR7tfPTFA3HSKPH4SsjBDrG3fKXJM4eZvqpeKUBUCQM"
  },
  {
    "chainId": 1,
    "address": "0x5f18C75AbDAe578b483E5F43f12a39cF75b973a9",
    "symbol": "yvUSDC",
    "name": "USDC yVault",
    "decimals": 6,
    "logoURI": "ipfs://QmRaBnw3sPymc1ACxyg7T6fXwAJqX3qFe4EYXDzw3pMoTn"
  },
  {
    "chainId": 1,
    "address": "0x814e0908b12A99FeCf5BC101bB5d0b8B5cDf7d26",
    "symbol": "MDT",
    "name": "Measurable Data Token",
    "decimals": 18,
    "logoURI": "ipfs://QmWZftLSRfGDNiGB4EnXQ8fF1uLNQFKPTcJ1biaoD7UHqp"
  },
  {
    "chainId": 1,
    "address": "0xCC4304A31d09258b0029eA7FE63d032f52e44EFe",
    "symbol": "SWAP",
    "name": "TrustSwap",
    "decimals": 18,
    "logoURI": "ipfs://QmSLDx9kmX7Nse913qSncfPrv4Q6H6UaTpZheJjHNDsY3L"
  },
  {
    "chainId": 1,
    "address": "0x53c4871322Bb47e7A24136fce291a6dcC832a294",
    "symbol": "WLTC",
    "name": " Wrapped Litecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmPAhZD8aEpQuHsNaserL3vHqGDaojFY1r3bRbqVFfrwhv"
  },
  {
    "chainId": 1,
    "address": "0xcBc1065255cBc3aB41a6868c22d1f1C573AB89fd",
    "symbol": "CRETH2",
    "name": "Cream ETH 2",
    "decimals": 18,
    "logoURI": "ipfs://QmWwemSuwVacr5orYXi7Du6pWQNh17wV1rqbmxpJr28Juo"
  },
  {
    "chainId": 1,
    "address": "0xD7EFB00d12C2c13131FD319336Fdf952525dA2af",
    "symbol": "XPR",
    "name": "XPR Network",
    "decimals": 4,
    "logoURI": "ipfs://QmczGHjAz5TBonjAuMaaWM8rdHBSKBVj8QhxnAvEq5qF3m"
  },
  {
    "chainId": 1,
    "address": "0xae7ab96520DE3A18E5e111B5EaAb095312D7fE84",
    "symbol": "stETH",
    "name": "Staked Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmXmFBTH2DnBUnfyMvkNEJPLzkmWWmraY8rPDzQc5LbbM3"
  },
  {
    "chainId": 1,
    "address": "0x155040625D7ae3e9caDA9a73E3E44f76D3Ed1409",
    "symbol": "REVO",
    "name": "Revomon",
    "decimals": 18,
    "logoURI": "ipfs://QmT7vRUrKz8TsNQDgrpyG37Wbm3faSKXrf5V5uSJZAMMhF"
  },
  {
    "chainId": 1,
    "address": "0x2de72aDa48BDF7Bac276256D3F016fE058490C34",
    "symbol": "FLOKI",
    "name": "Floki Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmYPJNZqcS8vKiEVT46Z4d2SyupA14uCeZspjjrQdJusXX"
  },
  {
    "chainId": 43114,
    "address": "0xC7f4debC8072e23fe9259A5C0398326d8EfB7f5c",
    "symbol": "HeC",
    "name": "Heroes Chained",
    "decimals": 18,
    "logoURI": "ipfs://QmUK8LYQJS675pjT7u3Uez49MZVGNJDbMZbb7QqeF7d1ve"
  },
  {
    "chainId": 42161,
    "address": "0x6609BE1547166D1C4605F3A243FDCFf467e600C3",
    "symbol": "NEU",
    "name": "Neutra Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmcqNtb1kWHbYNBQcnqotwmXJ4FJyFv6VT4HNRWjoXRcy5"
  },
  {
    "chainId": 1,
    "address": "0xd9d01D4Cb824219A8F482a0FAd479cB971Fd0628",
    "symbol": "ENTRC",
    "name": "EnterCoin",
    "decimals": 8,
    "logoURI": "ipfs://QmQBeu98iSzp6kdeaZupyriJ2ATzBVqkDaDjhyk3GKEEsG"
  },
  {
    "chainId": 1,
    "address": "0x6A22e5e94388464181578Aa7A6B869e00fE27846",
    "symbol": "sXAG",
    "name": "Synthetic Silver Ounce",
    "decimals": 18,
    "logoURI": "ipfs://QmcRBVUVDxvSgaxfmr9g9rtxa1KvXGYdbXe1agKccDKRX4"
  },
  {
    "chainId": 56,
    "address": "0x5857c96DaE9cF8511B08Cb07f85753C472D36Ea3",
    "symbol": "FUSE",
    "name": "Fuse",
    "decimals": 18,
    "logoURI": "ipfs://QmYiyLPWFmwxTv43Jsx4qpPcKu2YmqGX6XxwvBjTRVPKhC"
  },
  {
    "chainId": 1,
    "address": "0x43A96962254855F16b925556f9e97BE436A43448",
    "symbol": "HORD",
    "name": "Hord",
    "decimals": 18,
    "logoURI": "ipfs://QmRWLNz4pjL6uxUJEv28vTtu7YF5qse8XNc6sEuvemHuc3"
  },
  {
    "chainId": 1,
    "address": "0xd4c435F5B09F855C3317c8524Cb1F586E42795fa",
    "symbol": "CND",
    "name": "Cindicator",
    "decimals": 18,
    "logoURI": "ipfs://QmWYuHJdFmQ5bum7Q6gop2WAGPXMHSw9LDJXn3fcYU3qPk"
  },
  {
    "chainId": 1,
    "address": "0x0557E0d15aeC0b9026dD17aA874fDf7d182A2cEB",
    "symbol": "CFXQ",
    "name": "CFX Quantum",
    "decimals": 6,
    "logoURI": "ipfs://QmXt4SvqcxYHo5vcLZPLCvANV5MThRFUCy1gxbEAZkto3d"
  },
  {
    "chainId": 10,
    "address": "0x50c5725949A6F0c72E6C4a641F24049A917DB0Cb",
    "symbol": "LYRA",
    "name": "Lyra",
    "decimals": 18,
    "logoURI": "ipfs://QmaijTpgJwSbSg49ZDYh2FJcG4cSbckxn4ySFsCJviz25L"
  },
  {
    "chainId": 1,
    "address": "0xA01199c61841Fce3b3daFB83FeFC1899715c8756",
    "symbol": "CIRUS",
    "name": "Cirus",
    "decimals": 18,
    "logoURI": "ipfs://Qmc3EMxMCuAUZ7DfVNRLH6P9vim83mB1zy7m9XcWRBGhy8"
  },
  {
    "chainId": 1,
    "address": "0x0202Be363B8a4820f3F4DE7FaF5224fF05943AB1",
    "symbol": "UFT",
    "name": "UniLend Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmdV8s7tXyepQ2zNTqbfiShTJ9rMnFbn6Wxqg86FwrrQ4L"
  },
  {
    "chainId": 137,
    "address": "0x03cf5d93cA7c70Ce0A21a09F4D70779D2c66B25A",
    "symbol": "JMPT",
    "name": "JumpToken",
    "decimals": 18,
    "logoURI": "ipfs://QmZnPm3t5ECmafB4Ppiu5aH8Rd8XDirRBe9czovVoUiM5c"
  },
  {
    "chainId": 1,
    "address": "0xAf91E8aFbe87642dC628786188a54B78580A4d76",
    "symbol": "FOY",
    "name": "Fund Of Yours",
    "decimals": 18,
    "logoURI": "ipfs://QmZfB8yJwiAaHMpbEQDXG13i8SQXFbNxffvjdfRYwszbdK"
  },
  {
    "chainId": 100,
    "address": "0x63e62989D9EB2d37dfDB1F93A22f063635b07d51",
    "symbol": "MIVA",
    "name": "Minerva Wallet",
    "decimals": 18,
    "logoURI": "ipfs://QmX7kvDt6F32tJifxDPDs9aymAnAzE9b46g4jgzahgJrRn"
  },
  {
    "chainId": 1,
    "address": "0x6fDb90535C09B82825e38D41edF5e66211D4B442",
    "symbol": "MAGNET",
    "name": "Yield Magnet",
    "decimals": 18,
    "logoURI": "ipfs://QmfCBEHud3xA2sXUiWZpa5nXpXJ1YGEjF5gFYTXRsrukWk"
  },
  {
    "chainId": 1,
    "address": "0xFF0a024B66739357c4ED231fB3DBC0c8C22749F5",
    "symbol": "BWRX",
    "name": "Binance Wrapped WRX",
    "decimals": 8,
    "logoURI": "ipfs://QmQVQSDMsnet1RBzrbERWzuaZKLVHrDDVcDjhuHgPckz1Y"
  },
  {
    "chainId": 1,
    "address": "0xD692ac3245bb82319A31068D6B8412796eE85d2c",
    "symbol": "crHUSD",
    "name": "Cream HUSD",
    "decimals": 8,
    "logoURI": "ipfs://QmcMp5f4csYd6GNBKEYvzBJbgpUawJyCRpsKtiJ2RT94LV"
  },
  {
    "chainId": 1,
    "address": "0xe2f2a5C287993345a840Db3B0845fbC70f5935a5",
    "symbol": "mUSD",
    "name": "mStable USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPLntQBAPgByTpFCYarAsmaNmWdsQ5Ta7hQPf1hVDqsz7"
  },
  {
    "chainId": 1,
    "address": "0xD13c7342e1ef687C5ad21b27c2b65D772cAb5C8c",
    "symbol": "UOS",
    "name": "Ultra",
    "decimals": 4,
    "logoURI": "ipfs://Qme6hkk13dDrU6tEyMkfXHUAfSctauxmbN1VUs4HbMYR1s"
  },
  {
    "chainId": 1,
    "address": "0xA4Bdb11dc0a2bEC88d24A3aa1E6Bb17201112eBe",
    "symbol": "USDS",
    "name": "Stably USD",
    "decimals": 6,
    "logoURI": "ipfs://QmZdrfQMkDRGBAgfbYvPezUp4f5RpEkA5iNi6XYwPCcaDQ"
  },
  {
    "chainId": 1,
    "address": "0x61fd1c62551850D0c04C76FcE614cBCeD0094498",
    "symbol": "IDK",
    "name": "IDK",
    "decimals": 8,
    "logoURI": "ipfs://QmPEi8VBTLLwgBJ56HnAxYxKc46aPcNouqhHeG6w9BfACU"
  },
  {
    "chainId": 137,
    "address": "0x2e1AD108fF1D8C782fcBbB89AAd783aC49586756",
    "symbol": "TUSD",
    "name": "TrueUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmdcuKBFVn8KEoN6QW7mmGgd38dtF4aL6qTGTqMiDeQfRz"
  },
  {
    "chainId": 1,
    "address": "0x24DCc881E7Dd730546834452F21872D5cb4b5293",
    "symbol": "XD",
    "name": "Data Transaction Token",
    "decimals": 18,
    "logoURI": "ipfs://QmUMwVxeAMKV7VsSC6Puj1qTNZWdztMr18ENYBgQiohdiU"
  },
  {
    "chainId": 42161,
    "address": "0x13Ad51ed4F1B7e9Dc168d8a00cB3f4dDD85EfA60",
    "symbol": "LDO",
    "name": "Lido DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmTdQtLaPqXoXbHgbjVgYbXbbukvQAsNZtzq9bs14oVknY"
  },
  {
    "chainId": 1,
    "address": "0x55296f69f40Ea6d20E478533C15A6B08B654E758",
    "symbol": "XYO",
    "name": "XYO",
    "decimals": 18,
    "logoURI": "ipfs://QmeESPGkF7qQzsCyRkxWSNtvrNNWCDjqZ2eQFuate4cWRR"
  },
  {
    "chainId": 1,
    "address": "0x2b6fF53Fc2493CcD5202D80a6C439741414C5Ff2",
    "symbol": "TWEE",
    "name": "Tweebaa",
    "decimals": 18,
    "logoURI": "ipfs://QmVSqL4w89JgfVJkSzq2rSi7iMppYYLXa4ZZe1uYs9aHM1"
  },
  {
    "chainId": 1,
    "address": "0x2C974B2d0BA1716E644c1FC59982a89DDD2fF724",
    "symbol": "VIB",
    "name": "Viberate",
    "decimals": 18,
    "logoURI": "ipfs://QmXhYPBSAaUiRym9GyGMM1ZxMWBx8R7r5ATMh3fjZjC9G1"
  },
  {
    "chainId": 1,
    "address": "0xb753428af26E81097e7fD17f40c88aaA3E04902c",
    "symbol": "SFI",
    "name": "saffron.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUXAQXih7g6pEgQVDrx5GfQAP2iKx6rJ3f2am9ZWTXNfo"
  },
  {
    "chainId": 1,
    "address": "0x4946583c5b86E01cCD30c71a05617D06E3E73060",
    "symbol": "PTON",
    "name": "Foresting",
    "decimals": 18,
    "logoURI": "ipfs://QmTQe2oheVbSuhkhdX4R6Lcdhn3yvXjNXjVkyL31YLRwWk"
  },
  {
    "chainId": 1,
    "address": "0xFA2562da1Bba7B954f26C74725dF51fb62646313",
    "symbol": "ASSY",
    "name": "ASSY Index",
    "decimals": 18,
    "logoURI": "ipfs://QmWsykkkGwZpy5ZxNGavbRExrFKcn1iRDhKE8LLYQtiPTv"
  },
  {
    "chainId": 1,
    "address": "0x70c621f949b6556c4545707a2d5d73A776b98359",
    "symbol": "SKCH",
    "name": "Skychain",
    "decimals": 6,
    "logoURI": "ipfs://QmNw6N4nrjczfXhSUmD3kGFnuMnmmWLqkiBGxoJewFUJLs"
  },
  {
    "chainId": 250,
    "address": "0x7381eD41F6dE418DdE5e84B55590422a57917886",
    "symbol": "beFTM",
    "name": "Beefy Escrowed Fantom",
    "decimals": 18,
    "logoURI": "ipfs://QmcbBezs5xfzUr368A7jvjwuj6vk25KXGewkqqXdBQVK2H"
  },
  {
    "chainId": 1,
    "address": "0xB8C3B7A2A618C552C23B1E4701109a9E756Bab67",
    "symbol": "yv1INCH",
    "name": "1INCH yVault",
    "decimals": 18,
    "logoURI": "ipfs://Qmb29UqaYaMcEeZr2WfmW3v9zfEDBa8hcACEWHmR3G69aD"
  },
  {
    "chainId": 1,
    "address": "0x595832F8FC6BF59c85C527fEC3740A1b7a361269",
    "symbol": "POWR",
    "name": "Power Ledger",
    "decimals": 6,
    "logoURI": "ipfs://QmQgXguskPqfnmyTzphkWQQtSLASTiqkVG4tL64dPiV1Bz"
  },
  {
    "chainId": 137,
    "address": "0x9c2C5fd7b07E95EE044DDeba0E97a665F142394f",
    "symbol": "1INCH",
    "name": "1inch Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdUjFYL2izDVe4PLKArg7GtoHmsSqX6Bcsv9fmaVMcbLF"
  },
  {
    "chainId": 1,
    "address": "0xaF4DcE16Da2877f8c9e00544c93B62Ac40631F16",
    "symbol": "MTH",
    "name": "Monetha",
    "decimals": 5,
    "logoURI": "ipfs://QmSbXe9TsvGZLuCHZvH8oj845NjGCRn6WcBcVY74uqNYcf"
  },
  {
    "chainId": 100,
    "address": "0x1509706a6c66CA549ff0cB464de88231DDBe213B",
    "symbol": "AURA",
    "name": "Aura Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmeFwBoJY8mMLCiyc6EDqoUgbZRrjvga8cC7gMdgnbx1vJ"
  },
  {
    "chainId": 1,
    "address": "0x557B933a7C2c45672B610F8954A3deB39a51A8Ca",
    "symbol": "REVV",
    "name": "REVV",
    "decimals": 18,
    "logoURI": "ipfs://QmPRMgpp7FGfU4Tebx9W4uNTGJXaZKzrrzzn7op9C6Ctvd"
  },
  {
    "chainId": 1,
    "address": "0x6C2adC2073994fb2CCC5032cC2906Fa221e9B391",
    "symbol": "DPY",
    "name": "Delphy",
    "decimals": 18,
    "logoURI": "ipfs://QmfGb7KnEX3yrV4jWWwppNjTE9aLJbcfVUVEEd2FTnHvmj"
  },
  {
    "chainId": 137,
    "address": "0x3c499c542cEF5E3811e1192ce70d8cC03d5c3359",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmUQp1wZzr4UREn4AVLkKH5GbwpUCwf9w5VQQJeh1bmThJ"
  },
  {
    "chainId": 100,
    "address": "0x0116e28B43A358162B96f70B4De14C98A4465f25",
    "symbol": "UNCX",
    "name": "UNCX Network",
    "decimals": 18,
    "logoURI": "ipfs://QmR3EL7BBuw64gNiebFSq6i4BcysGuB83zL7VWBuSWkC1w"
  },
  {
    "chainId": 137,
    "address": "0xa3Fa99A148fA48D14Ed51d610c367C61876997F1",
    "symbol": "MIMATIC",
    "name": "MAI",
    "decimals": 18,
    "logoURI": "ipfs://Qmf7QpnQj6bj4Aa5YWJKjK5FbrsvxmAy22YSgqczX4pejm"
  },
  {
    "chainId": 1,
    "address": "0xD46bA6D942050d489DBd938a2C909A5d5039A161",
    "symbol": "AMPL",
    "name": "Ampleforth",
    "decimals": 9,
    "logoURI": "ipfs://QmdbipJGgq8hX5BQc9pEam84QVc7JCBxhKnMCfEFPrF9qC"
  },
  {
    "chainId": 1,
    "address": "0x71F85B2E46976bD21302B64329868fd15eb0D127",
    "symbol": "AXN",
    "name": "Axion",
    "decimals": 18,
    "logoURI": "ipfs://QmVAsSTJbWUbHEQEwABZhgE6NtanmZYVmTtdxjE4Tmczve"
  },
  {
    "chainId": 1,
    "address": "0xAa9D866666C2A3748d6B23Ff69E63E52f08d9AB4",
    "symbol": "FMTA",
    "name": "Fundamenta",
    "decimals": 18,
    "logoURI": "ipfs://QmahbtgXyoTZj69LU5pSjBkqYgiswQwHTc2sFP44mbU211"
  },
  {
    "chainId": 8453,
    "address": "0x356177B452dbae18B5a4b43b7B5324bFbae235Be",
    "symbol": "DFD",
    "name": "DeFido",
    "decimals": 18,
    "logoURI": "ipfs://QmdG6L1HSvrLHgYMp62iMaY5EYtxKaehpWwZfuHEfkenov"
  },
  {
    "chainId": 1,
    "address": "0x0F4CA92660Efad97a9a70CB0fe969c755439772C",
    "symbol": "LEV",
    "name": "Leverj",
    "decimals": 9,
    "logoURI": "ipfs://QmXzrpT6ewm2kCP1uEAcF77bGogj1ds6Kh2fa22JsWptBw"
  },
  {
    "chainId": 1,
    "address": "0xf278c1CA969095ffddDED020290cf8B5C424AcE2",
    "symbol": "RUFF",
    "name": "Ruff",
    "decimals": 18,
    "logoURI": "ipfs://QmTViNWMo9GheMCBVH69EmAzwdafN2tCtru9g7Hpv3ZT2q"
  },
  {
    "chainId": 56,
    "address": "0x4691937a7508860F876c9c0a2a617E7d9E945D4B",
    "symbol": "WOO",
    "name": "WOO Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdJjzzpWuzQAN6sj2j5FRy3SnFYgwMPQCx5ZNrFdVpcFt"
  },
  {
    "chainId": 1,
    "address": "0x887168120cb89Fb06F3E74Dc4AF20D67dF0977f6",
    "symbol": "SKRT",
    "name": "Sekuritance",
    "decimals": 18,
    "logoURI": "ipfs://QmX9DS3kMfSCBa1hbKwpouLw8RqV9GtC9kcZuCKRuuXVku"
  },
  {
    "chainId": 56,
    "address": "0x371c7ec6D8039ff7933a2AA28EB827Ffe1F52f07",
    "symbol": "JOE",
    "name": "JOE",
    "decimals": 18,
    "logoURI": "ipfs://QmfHbKz1Xm96EpDwsZ6jw3VyxGnbjN8iqqza3kksBqW93E"
  },
  {
    "chainId": 1,
    "address": "0xfF836A5821E69066c87E268bC51b849FaB94240C",
    "symbol": "SMURFCAT",
    "name": "Real Smurf Cat",
    "decimals": 18,
    "logoURI": "ipfs://Qmf48uuctiTub3zDPQCxejUZeqsknYHP1hdKEgn41AqxJN"
  },
  {
    "chainId": 1,
    "address": "0xaAAf91D9b90dF800Df4F55c205fd6989c977E73a",
    "symbol": "TKN",
    "name": "Monolith",
    "decimals": 8,
    "logoURI": "ipfs://QmV1hU6mYPV1vDGtopYv1Mgrs27WGncitUzGeDem6p8VVB"
  },
  {
    "chainId": 56,
    "address": "0xb3Ed0A426155B79B898849803E3B36552f7ED507",
    "symbol": "PENDLE",
    "name": "Pendle",
    "decimals": 18,
    "logoURI": "ipfs://QmUgLTHTMw5TD3NmeH4L9VAcRH7EAevahe5AtP7vPVL4Gc"
  },
  {
    "chainId": 1,
    "address": "0x08c32b0726C5684024ea6e141C50aDe9690bBdcc",
    "symbol": "STOS",
    "name": "Stratos",
    "decimals": 18,
    "logoURI": "ipfs://QmV5xdGQYxMCvX9GVZiBCPKtuN4yBPDBEEuCTbDYhK2xhV"
  },
  {
    "chainId": 56,
    "address": "0xe4Cc45Bb5DBDA06dB6183E8bf016569f40497Aa5",
    "symbol": "GAL",
    "name": "Galxe",
    "decimals": 18,
    "logoURI": "ipfs://Qmbmw5JewVsNkzb2hfMgEgn9X19wejADFyKun8ef2KQqaH"
  },
  {
    "chainId": 100,
    "address": "0xB714654e905eDad1CA1940b7790A8239ece5A9ff",
    "symbol": "TUSD",
    "name": "TrueUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmdcuKBFVn8KEoN6QW7mmGgd38dtF4aL6qTGTqMiDeQfRz"
  },
  {
    "chainId": 1,
    "address": "0xef3A930e1FfFFAcd2fc13434aC81bD278B0ecC8d",
    "symbol": "FIS",
    "name": "StaFi",
    "decimals": 18,
    "logoURI": "ipfs://QmfXhahJUpGT8RJ41biy3SJuvAUVEQYo5WKqbaCBY4Zuty"
  },
  {
    "chainId": 1,
    "address": "0x37A15C92e67686aA268df03d4C881A76340907E8",
    "symbol": "PIXIU",
    "name": "Pixiu Finance",
    "decimals": 0,
    "logoURI": "ipfs://QmZyK5aCZCGu9VMoQMnYL29WgbpWfacaF1gi6YMT44YskR"
  },
  {
    "chainId": 1,
    "address": "0x73D9E335669462Cbdd6aa3AdaFe9efeE86a37Fe9",
    "symbol": "DAIQ",
    "name": "Daiquilibrium",
    "decimals": 18,
    "logoURI": "ipfs://QmW7LNmm8a8v3zBZJLkTRdnMxcDZ6rYw8vjjFipNkrbSJB"
  },
  {
    "chainId": 56,
    "address": "0x8b303d5BbfBbf46F1a4d9741E491e06986894e18",
    "symbol": "WOOP",
    "name": "Woonkly Power",
    "decimals": 18,
    "logoURI": "ipfs://Qma9CmXTqtZibdh7gSUDjZLyDyX1GJMTYecW3xSzL8wvW2"
  },
  {
    "chainId": 56,
    "address": "0x570A5D26f7765Ecb712C0924E4De545B89fD43dF",
    "symbol": "SOL",
    "name": "Solana",
    "decimals": 18,
    "logoURI": "ipfs://QmQXRpCY4VSH5ZuJeu9FzUwUnwe1E9YzPdvNKHqRvmeMZn"
  },
  {
    "chainId": 1,
    "address": "0x9e3319636e2126e3c0bc9e3134AEC5e1508A46c7",
    "symbol": "UTNP",
    "name": "Universa",
    "decimals": 18,
    "logoURI": "ipfs://QmVC73XpDUb5ohCoUKjsTCiqgAywWpqUooBZWDLf3HdfEd"
  },
  {
    "chainId": 1,
    "address": "0x85516e8862aB543EA15972B7809256efEC0696ea",
    "symbol": "ROCK",
    "name": "ROCK",
    "decimals": 18,
    "logoURI": "ipfs://QmeuyJjx6MYYZAc5Uu64dz5yjZV2aZM6d13bcCQRDuH4bY"
  },
  {
    "chainId": 1,
    "address": "0xEd025A9Fe4b30bcd68460BCA42583090c2266468",
    "symbol": "RPC",
    "name": "Ripio Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmUvCk5SEGzozDoR5NjksPjoUVR6oEMdDHuh72kUkSRbMA"
  },
  {
    "chainId": 100,
    "address": "0x71850b7E9Ee3f13Ab46d67167341E4bDc905Eef9",
    "symbol": "HNY",
    "name": "Honey",
    "decimals": 18,
    "logoURI": "ipfs://QmYxsVgovmspp3BwuSUrRyicW2XwnVy1JbQnHkEdy4YsGH"
  },
  {
    "chainId": 1,
    "address": "0xf05a9382A4C3F29E2784502754293D88b835109C",
    "symbol": "REX",
    "name": "imbrex",
    "decimals": 18,
    "logoURI": "ipfs://QmWRcXy5DZrfiCtSPg9VFAPrnc5ZUgXHYGm4QeBwKgka3g"
  },
  {
    "chainId": 1,
    "address": "0xEb029507d3e043DD6C87F2917C4E82B902c35618",
    "symbol": "sCOMP",
    "name": "Synthetic Compound",
    "decimals": 18,
    "logoURI": "ipfs://QmUcSJ1XKmSEcir989LwNMoL5LTmcGgzQENjmQTVCQiBjz"
  },
  {
    "chainId": 1,
    "address": "0xB4272071eCAdd69d933AdcD19cA99fe80664fc08",
    "symbol": "XCHF",
    "name": "CryptoFranc",
    "decimals": 18,
    "logoURI": "ipfs://QmVLshQxKAgJ29ixYCpXy9zvd5nvdUqWYYV5GtJsT9kSzy"
  },
  {
    "chainId": 1,
    "address": "0x49f4592E641820e928F9919Ef4aBd92a719B4b49",
    "symbol": "iSUSD",
    "name": "Fulcrum sUSD iToken",
    "decimals": 18,
    "logoURI": "ipfs://Qmdt4bG2x93U8V85PQAithCMv6H9gTco48JDAEDfR76L7u"
  },
  {
    "chainId": 1,
    "address": "0x780116D91E5592E58a3b3c76A351571b39abCEc6",
    "symbol": "BOXX",
    "name": "Blockparty",
    "decimals": 15,
    "logoURI": "ipfs://QmNqw7dvT6Pda5eAKevWEZwx9uEfqPVyV3V9jbHXHdmtu6"
  },
  {
    "chainId": 42161,
    "address": "0xB1bC21f748AE2bE95674876710bc6D78235480e0",
    "symbol": "HORD",
    "name": "Hord",
    "decimals": 18,
    "logoURI": "ipfs://QmRWLNz4pjL6uxUJEv28vTtu7YF5qse8XNc6sEuvemHuc3"
  },
  {
    "chainId": 1,
    "address": "0x1563D521ba309e2Ad9f4aFfD6f4dE9759E8d4F21",
    "symbol": "VNX",
    "name": "VisionX",
    "decimals": 18,
    "logoURI": "ipfs://QmfH8AgkKh7No1yweNDsXFXwNwHYoVVXv6QUV4HTFVFZVM"
  },
  {
    "chainId": 56,
    "address": "0xDf7952B35f24aCF7fC0487D01c8d5690a60DBa07",
    "symbol": "PB",
    "name": "Pancake Bunnies",
    "decimals": 0,
    "logoURI": "ipfs://QmVVhGQCsvNzH3mP6ZbHGoiMmcAAwFWiUSo2JzYxTs1LDw"
  },
  {
    "chainId": 1,
    "address": "0x4EbE70CB942D5AF0A18b9126762637e7098Ff5fD",
    "symbol": "G",
    "name": "G Revolution",
    "decimals": 9,
    "logoURI": "ipfs://QmYHXZuyUJpwyfo1e6UwPp3CgkegvsX3HVohfTjBfDrUR4"
  },
  {
    "chainId": 1,
    "address": "0xd957E08ac5421E2C28510586B57d095E5094836a",
    "symbol": "VTX",
    "name": "VorteX Network",
    "decimals": 18,
    "logoURI": "ipfs://QmY1dyHuc56piABqqEYexKxGbptkjMoyPW7BunN6tjhWJR"
  },
  {
    "chainId": 1,
    "address": "0xa0246c9032bC3A600820415aE600c6388619A14D",
    "symbol": "FARM",
    "name": "Harvest Finance",
    "decimals": 18,
    "logoURI": "ipfs://Qmd2Ub6FQEU8G8d9CPwsskqyfccQrTRMNCpaAdrKC5H6SE"
  },
  {
    "chainId": 1,
    "address": "0xbBC455cb4F1B9e4bFC4B73970d360c8f032EfEE6",
    "symbol": "sLINK",
    "name": "Synthetic Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://QmekS5oTKi7GimHcCk8iX9GjRNfnGt3NEaWv8VQe7EJTdp"
  },
  {
    "chainId": 1,
    "address": "0x2b591e99afE9f32eAA6214f7B7629768c40Eeb39",
    "symbol": "HEX",
    "name": "HEX",
    "decimals": 8,
    "logoURI": "ipfs://QmNVqEMueiS6YaoFm7ZhkTGw1F1xKTqz9YoxFPiHr2c6VP"
  },
  {
    "chainId": 1,
    "address": "0xFbC4f3f645C4003a2E4F4e9b51077d2DaA9a9341",
    "symbol": "ZEDXION",
    "name": "ZEDXION",
    "decimals": 18,
    "logoURI": "ipfs://QmYAc8hYn8A3yQGtsLyFizexFkNNtcaYP3yVhhijTqU4R2"
  },
  {
    "chainId": 56,
    "address": "0xaf20f5f19698f1D19351028cd7103B63D30DE7d7",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 1,
    "address": "0x4F9254C83EB525f9FCf346490bbb3ed28a81C667",
    "symbol": "CELR",
    "name": "Celer Network",
    "decimals": 18,
    "logoURI": "ipfs://QmVTxqkF5viA8xYcanFQJLyir4bpLX8iGXS39MrfVV7yfj"
  },
  {
    "chainId": 10,
    "address": "0xc40F949F8a4e094D1b49a23ea9241D289B7b2819",
    "symbol": "LUSD",
    "name": "Liquity USD",
    "decimals": 18,
    "logoURI": "ipfs://QmPzBCxWo5xMA8Wif7wCQQmuZ5ajP5sw4JSHthMxvxY2uR"
  },
  {
    "chainId": 1,
    "address": "0x8E870D67F660D95d5be530380D0eC0bd388289E1",
    "symbol": "USDP",
    "name": "Pax Dollar",
    "decimals": 18,
    "logoURI": "ipfs://QmerxAPnW2PWxDejrLRU2aFX9NCJgEMGdHebGXoS2jHC3c"
  },
  {
    "chainId": 1,
    "address": "0x47da42696A866CDC61A4C809A515500a242909C1",
    "symbol": "BIT",
    "name": "BitRewards",
    "decimals": 18,
    "logoURI": "ipfs://QmZPAVEkwydtCdnM25jU9Mpv7idqpDPHiU9HZGBRbtn5fn"
  },
  {
    "chainId": 1,
    "address": "0x41e5560054824eA6B0732E656E3Ad64E20e94E45",
    "symbol": "CVC",
    "name": "Civic",
    "decimals": 8,
    "logoURI": "ipfs://QmZuZz7KAtnByuizThmeDC7hkH2oF2kPxp35Xp7FXYpADi"
  },
  {
    "chainId": 1,
    "address": "0xF5238462E7235c7B62811567E63Dd17d12C2EAA0",
    "symbol": "CGT",
    "name": "CACHE Gold",
    "decimals": 8,
    "logoURI": "ipfs://QmZ5ngH3qbdGvGRsTKYfo1oeNdtcpyidqZk1zuFCSDhoSX"
  },
  {
    "chainId": 1,
    "address": "0x7DD9c5Cba05E151C895FDe1CF355C9A1D5DA6429",
    "symbol": "GLM",
    "name": "Golem",
    "decimals": 18,
    "logoURI": "ipfs://QmTXdjxTZjiPgtBDArBu4qRWRLPyG7UqF9wyXyPM3qYu9Q"
  },
  {
    "chainId": 1,
    "address": "0xC343f099d3E41aA5C1b59470450e21E92E2d840b",
    "symbol": "VME",
    "name": "VeriME",
    "decimals": 18,
    "logoURI": "ipfs://QmcRDTc2hoJN2aU2gLu7ouXE9h6Dyj99TKZfUxiNW8wA6Q"
  },
  {
    "chainId": 1,
    "address": "0x5919DeA604631016c15c805e3D948A0384879892",
    "symbol": "PEEP$",
    "name": "The Peoples Coin",
    "decimals": 9,
    "logoURI": "ipfs://QmTCQbjCmuBBjR6yJzBwE5Hyz1S7okBAPdX7PCajabajGk"
  },
  {
    "chainId": 1,
    "address": "0x927402ab67c0CDA3c187E9DFE34554AC581441f2",
    "symbol": "SAITABIT",
    "name": "SaitaBit",
    "decimals": 18,
    "logoURI": "ipfs://QmerW9zrozMvYqf1x2J435oQMw21nLLFNfag2i75aNQa7b"
  },
  {
    "chainId": 1,
    "address": "0x4CC19356f2D37338b9802aa8E8fc58B0373296E7",
    "symbol": "KEY",
    "name": "SelfKey",
    "decimals": 18,
    "logoURI": "ipfs://QmVUsF49ZfdkFmTzEm6rrxYvFCUnYtSmiYHyEwyMKKDVYf"
  },
  {
    "chainId": 1,
    "address": "0x31D457E7bcFf5Bc9A5Ef86E6a5eA1DB5b5C3BFB0",
    "symbol": "FOXX",
    "name": "Star Foxx",
    "decimals": 18,
    "logoURI": "ipfs://QmQUUzyAFiwv6YpfqrESNyW9DvAGntibG5RGRJAzV2mhNT"
  },
  {
    "chainId": 56,
    "address": "0x47BEAd2563dCBf3bF2c9407fEa4dC236fAbA485A",
    "symbol": "SXP",
    "name": "Swipe",
    "decimals": 18,
    "logoURI": "ipfs://QmQyhHJ92pNm8RbezaeeNtfUqLyv8sQZ62GfBN5RZqADro"
  },
  {
    "chainId": 1,
    "address": "0xB50721BCf8d664c30412Cfbc6cf7a15145234ad1",
    "symbol": "ARB",
    "name": "Arbitrum",
    "decimals": 18,
    "logoURI": "ipfs://QmdzPaPoFE9o221RAKfjCRW7KzUQBeKohFE5MFQMvhGzjd"
  },
  {
    "chainId": 250,
    "address": "0xC758295Cd1A564cdb020a78a681a838CF8e0627D",
    "symbol": "FS",
    "name": "FantomStarter",
    "decimals": 18,
    "logoURI": "ipfs://QmeqPPX87Xj5FwRa18FNg5iEvtB3rDfQ5Gp3Yk8yb35C5r"
  },
  {
    "chainId": 1,
    "address": "0x15B543e986b8c34074DFc9901136d9355a537e7E",
    "symbol": "STC",
    "name": "Student Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmfT4temychqeDpv2GpZiUCB3eDxHWJGPtVLh9iM9vZjZf"
  },
  {
    "chainId": 1,
    "address": "0x120d3d081CDfc62CfAF99638f12703E4B9A6C4Bc",
    "symbol": "SAITAMA",
    "name": "Saitama Classic",
    "decimals": 18,
    "logoURI": "ipfs://QmQLRXsGo7WoTwNdmnqz7muyshsQ6YzSe11KrLpEmpq5ke"
  },
  {
    "chainId": 1,
    "address": "0x0391D2021f89DC339F60Fff84546EA23E337750f",
    "symbol": "BOND",
    "name": "BarnBridge",
    "decimals": 18,
    "logoURI": "ipfs://QmR1zLuhbjkYiG6ePSvyo9waqxbAsk23SbKnbwjd649wVQ"
  },
  {
    "chainId": 1,
    "address": "0x38e4adB44ef08F22F5B5b76A8f0c2d0dCbE7DcA1",
    "symbol": "CVP",
    "name": "PowerPool Concentrated Voting Power",
    "decimals": 18,
    "logoURI": "ipfs://QmYYsVEkhWHRrXqjHFD1nhZDk5Xj4sRe9EdUEjDNXxdi6M"
  },
  {
    "chainId": 1,
    "address": "0x88C8Cf3A212c0369698D13FE98Fcb76620389841",
    "symbol": "sEOS",
    "name": "Synthetic EOS",
    "decimals": 18,
    "logoURI": "ipfs://QmSKoQkUu4fZMCxYWXtxJebWqzQjiGihTyRQbMHPkNXNF7"
  },
  {
    "chainId": 42161,
    "address": "0x13278cD824D33A7aDB9f0a9A84ACA7C0D2DEEBf7",
    "symbol": "TAROT",
    "name": "Tarot",
    "decimals": 18,
    "logoURI": "ipfs://QmenPX7UnjGeCrfGjsNUWX6tJj3o6QXXw9co5WL8aNP9UF"
  },
  {
    "chainId": 1,
    "address": "0x0E5f00DA8AAef196a719d045DB89b5DA8F371b32",
    "symbol": "CNTM",
    "name": "Connectome",
    "decimals": 18,
    "logoURI": "ipfs://QmVcrnK6NXkjWy1WUFmMSzJYtqXmcsnL2WBCbuQV4b5L2F"
  },
  {
    "chainId": 250,
    "address": "0x4f851750a3e6f80f1E1f89C67B56960Bfc29A934",
    "symbol": "KALM",
    "name": "KALM",
    "decimals": 18,
    "logoURI": "ipfs://QmZUhb1jzCscZULnbM1jz9KScbxFyMrBnSaawcTd6PmfXT"
  },
  {
    "chainId": 100,
    "address": "0x84E2C67CBEfae6B5148fcA7d02B341B12ff4b5Bb",
    "symbol": "SWASH",
    "name": "Swash",
    "decimals": 18,
    "logoURI": "ipfs://QmP5jabdBdR9Xcp2R2TqJgc3RfSJVqvFQJTg61WWL4oCfF"
  },
  {
    "chainId": 56,
    "address": "0x4a0A3902e091cdb3AEc4279a6BFAC50297F0A79e",
    "symbol": "VERA",
    "name": "Vera",
    "decimals": 18,
    "logoURI": "ipfs://QmQ8RT7vmrnjqFK7e6UGVuR3AkYXDQJPnBC6EC6vAJLVfo"
  },
  {
    "chainId": 137,
    "address": "0x53E0bca35eC356BD5ddDFebbD1Fc0fD03FaBad39",
    "symbol": "LINK",
    "name": "Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://Qmdn6E15MELheFNupKrMosD9KPDFZ9M3wVKkhCuacT1cVj"
  },
  {
    "chainId": 1,
    "address": "0x626E8036dEB333b408Be468F951bdB42433cBF18",
    "symbol": "AIOZ",
    "name": "AIOZ Network",
    "decimals": 18,
    "logoURI": "ipfs://QmSvSawrMj86bFBPF6thS3En54hm5PNSxDPJSTPBJsDweS"
  },
  {
    "chainId": 1,
    "address": "0x87d73E916D7057945c9BcD8cdd94e42A6F47f776",
    "symbol": "NFTX",
    "name": "NFTX",
    "decimals": 18,
    "logoURI": "ipfs://QmRw5KHczxpT5aBDVnwRah1tth1anV72tDFKpYHb4ysFKA"
  },
  {
    "chainId": 1,
    "address": "0xf650C3d88D12dB855b8bf7D11Be6C55A4e07dCC9",
    "symbol": "cUSDT",
    "name": "Compound USDT",
    "decimals": 8,
    "logoURI": "ipfs://Qmbi3ANqzbU4A3gcnu5y1TnJfdmcWS4Fn2x1qx5HubG97i"
  },
  {
    "chainId": 1,
    "address": "0x33349B282065b0284d756F0577FB39c158F935e6",
    "symbol": "MPL",
    "name": "Maple",
    "decimals": 18,
    "logoURI": "ipfs://QmdYUv9tq7e5W9v4eVFvVvAfApmfcprJrf4CGeVZXKydbH"
  },
  {
    "chainId": 1,
    "address": "0x1C9922314ED1415c95b9FD453c3818fd41867d0B",
    "symbol": "TOWER",
    "name": "TOWER",
    "decimals": 18,
    "logoURI": "ipfs://QmUDvzhLFf3t8cjweZgFFWeqjrprqE11ZyFrySNATaTWwy"
  },
  {
    "chainId": 1,
    "address": "0x0F83287FF768D1c1e17a42F44d644D7F22e8ee1d",
    "symbol": "sCHF",
    "name": "Synthetic Swiss Franc",
    "decimals": 18,
    "logoURI": "ipfs://Qmce8NBY7ie22DgXqu7pmvZD7X9RWHttgoJaQWYUAL9aTu"
  },
  {
    "chainId": 1,
    "address": "0xE0B7927c4aF23765Cb51314A0E0521A9645F0E2A",
    "symbol": "DGD",
    "name": "DigixDAO",
    "decimals": 9,
    "logoURI": "ipfs://QmbUWtGWpsQ8iDCoo2HPzYbWXsfYuuk7wDqeKphWSxFZjR"
  },
  {
    "chainId": 1,
    "address": "0xF87E31492Faf9A91B02Ee0dEAAd50d51d56D5d4d",
    "symbol": "LAND",
    "name": "Decentraland LAND",
    "decimals": 0,
    "logoURI": "ipfs://QmVfReUZqV7bWuF7X83XMrU7spnV2ZFfigSdduXHe8jwaH"
  },
  {
    "chainId": 1,
    "address": "0x107c4504cd79C5d2696Ea0030a8dD4e92601B82e",
    "symbol": "BLT",
    "name": "Bloom",
    "decimals": 18,
    "logoURI": "ipfs://QmSCJ453JWLYv4n4rJS16QWqtrUsLUPMeE96EC13a11sjR"
  },
  {
    "chainId": 56,
    "address": "0xf7DE7E8A6bd59ED41a4b5fe50278b3B7f31384dF",
    "symbol": "RDNT",
    "name": "Radiant",
    "decimals": 18,
    "logoURI": "ipfs://QmPFwFqf6KCqR6fBxYffw1FtPGVG6AUACK1dKu79cFvpHt"
  },
  {
    "chainId": 1,
    "address": "0x4Fabb145d64652a948d72533023f6E7A623C7C53",
    "symbol": "BUSD",
    "name": "Binance USD",
    "decimals": 18,
    "logoURI": "ipfs://QmarL9ewxDAz5hpRJq2Skr2oMP5CX5KMX1fPkgXfSUHrEx"
  },
  {
    "chainId": 1,
    "address": "0xfd609a03B393F1A1cFcAcEdaBf068CAD09a924E2",
    "symbol": "crCRETH2",
    "name": "Cream Cream ETH 2",
    "decimals": 8,
    "logoURI": "ipfs://QmPjw3JC69YAYs9CQdnWqTaL94BWVxQgKTxAVXUEuWWmCq"
  },
  {
    "chainId": 1,
    "address": "0x8A2279d4A90B6fe1C4B30fa660cC9f926797bAA2",
    "symbol": "CHR",
    "name": "Chromia",
    "decimals": 6,
    "logoURI": "ipfs://QmSkpvRfbSVKxsZjnFmdcHcZrcEMiya4KTGKowmUV8UZBR"
  },
  {
    "chainId": 1,
    "address": "0xaea46A60368A7bD060eec7DF8CBa43b7EF41Ad85",
    "symbol": "FET",
    "name": "Fetch.ai",
    "decimals": 18,
    "logoURI": "ipfs://QmeR4XrGNJFoEnScmuKpaQxTsu6ySPxmmXzaHk6C7ejaM9"
  },
  {
    "chainId": 1,
    "address": "0x8762db106B2c2A0bccB3A80d1Ed41273552616E8",
    "symbol": "RSR",
    "name": "Reserve Rights",
    "decimals": 18,
    "logoURI": "ipfs://QmVudzh7roJ7j3kFBWe33jvF9P6eYdyUnJuxu2XeYnGwTn"
  },
  {
    "chainId": 1,
    "address": "0x4a5B3D0004454988C50e8dE1bCFC921EE995ADe3",
    "symbol": "wXEQ",
    "name": "wXEQ v2",
    "decimals": 18,
    "logoURI": "ipfs://QmQoHikAxtfEEjyrqen2ZvWMJwq1UP9Wb16pr6e2pnAEvk"
  },
  {
    "chainId": 56,
    "address": "0x156ab3346823B651294766e23e6Cf87254d68962",
    "symbol": "LUNC",
    "name": "Terra Classic",
    "decimals": 6,
    "logoURI": "ipfs://QmceuGzAgd2wcWiDvX9ygEohHnaMqWDpSok7ot1h7jahZt"
  },
  {
    "chainId": 100,
    "address": "0x431D5dfF03120AFA4bDf332c61A6e1766eF37BDB",
    "symbol": "JPYC",
    "name": "JPY Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmX5M9oHm5KexoSF5hYDKbykYbceg5hvvi1ioho1YrBsyP"
  },
  {
    "chainId": 324,
    "address": "0x3613AD277DF1d5935D41400A181Aa9ec1DC2Dc9e",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 1,
    "address": "0x6c3F90f043a72FA612cbac8115EE7e52BDe6E490",
    "symbol": "3Crv",
    "name": "3Curve",
    "decimals": 18,
    "logoURI": "ipfs://QmdpGMVZF7bcfpNuEEQHQXoDXQ1QE99kjiYXZDdz1UqsU7"
  },
  {
    "chainId": 1,
    "address": "0xA4eA687A2A7F29cF2dc66B39c68e4411C0D00C49",
    "symbol": "IVY",
    "name": "Ivy",
    "decimals": 18,
    "logoURI": "ipfs://QmQ1A1XDFsHDWdLewic15KPnkvbCxCgEYhUJqszDd1mu3J"
  },
  {
    "chainId": 1,
    "address": "0x986b4AFF588a109c09B50A03f42E4110E29D353F",
    "symbol": "yvCurve-sETH",
    "name": "Curve sETH Pool yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmWBxxb9RYCLTVa9xg64j7saoqJuGhUqaeMo9M2HJorYqR"
  },
  {
    "chainId": 137,
    "address": "0x5FFD62D3C3eE2E81C00A7b9079FB248e7dF024A8",
    "symbol": "GNO",
    "name": "Gnosis",
    "decimals": 18,
    "logoURI": "ipfs://QmTVco7p4VxCMHJAKDNmqYAveVhww11r5Z3hKqbgg6ShRB"
  },
  {
    "chainId": 1,
    "address": "0x1c8266A4369aF6d80Df2659Ba47B3c98f35cB8bE",
    "symbol": "KOJI",
    "name": "Koji",
    "decimals": 18,
    "logoURI": "ipfs://QmamxMJQtvMMuCUE5YPUiBvLJeHUHjf4RX4hwK1PW7296W"
  },
  {
    "chainId": 1,
    "address": "0xB0c7a3Ba49C7a6EaBa6cD4a96C55a1391070Ac9A",
    "symbol": "MAGIC",
    "name": "MAGIC",
    "decimals": 18,
    "logoURI": "ipfs://QmX9mZzzH1uooK4V4TpaXKXYrBVKYacBqigPsTiKLpSS3d"
  },
  {
    "chainId": 1,
    "address": "0x209C1808FeBf6c1aB7C65764bb61AD67d3923fcc",
    "symbol": "APE",
    "name": "APEcoin",
    "decimals": 18,
    "logoURI": "ipfs://QmULCg8MyfEQbZo5dCPAHiskLUExUQqiS6nkMqd7WDPReB"
  },
  {
    "chainId": 100,
    "address": "0x6A023CCd1ff6F2045C3309768eAd9E68F978f6e1",
    "symbol": "WETH",
    "name": "Wrapped Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmeH11wxc6kpgAEpFRyteAJq9ksXzsmfHJcPWLQeCT1f8x"
  },
  {
    "chainId": 1,
    "address": "0x68037790A0229e9Ce6EaA8A99ea92964106C4703",
    "symbol": "PAR",
    "name": "Parallel",
    "decimals": 18,
    "logoURI": "ipfs://QmSp3xELVa8NXo3XVL2BC7zGyyswgR723mwTiyRyL4doCb"
  },
  {
    "chainId": 1,
    "address": "0x7865af71cf0b288b4E7F654f4F7851EB46a2B7F8",
    "symbol": "SNTVT",
    "name": "Sentivate",
    "decimals": 18,
    "logoURI": "ipfs://QmZPFARvsdw8CBPhBi1fti9bGgKrtjkX1XFpUUWWP5f6nq"
  },
  {
    "chainId": 1,
    "address": "0x1cA02DD95f3F1e33DA7F5AfE15EA866dAb07aF04",
    "symbol": "BKING",
    "name": "King Arthur",
    "decimals": 9,
    "logoURI": "ipfs://QmWp5TW8G671GXoHoLxjRh8v8M462DjQ9sFwFKrNokvLQM"
  },
  {
    "chainId": 56,
    "address": "0x882C173bC7Ff3b7786CA16dfeD3DFFfb9Ee7847B",
    "symbol": "vBTC",
    "name": "Venus BTC",
    "decimals": 8,
    "logoURI": "ipfs://QmVcmLWcq3u93KCsXxMmbrFBEinEXxThqniJBMRxEJrJpT"
  },
  {
    "chainId": 1,
    "address": "0x36a2422a863D5B950882190Ff5433E513413343a",
    "symbol": "sBCH",
    "name": "Synthetic Bitcoin Cash",
    "decimals": 18,
    "logoURI": "ipfs://QmYWbK7CxHKSDuAefriPi2uaNrf7hskd8jKvvcPCPbiAjW"
  },
  {
    "chainId": 1,
    "address": "0x151202C9c18e495656f372281F493EB7698961D5",
    "symbol": "DEB",
    "name": "Debitum Network",
    "decimals": 18,
    "logoURI": "ipfs://QmXGVsfiRLtFTQrSDMRALjfPsxNXECqKDMH7tmZAsK9ti2"
  },
  {
    "chainId": 137,
    "address": "0xbD1463F02f61676d53fd183C2B19282BFF93D099",
    "symbol": "jCHF",
    "name": "Jarvis Synthetic Swiss Franc",
    "decimals": 18,
    "logoURI": "ipfs://QmdvXu6Ly98GP2XSc6jVcyXi8UVd8UHAzEDdWKrFV2zaob"
  },
  {
    "chainId": 1,
    "address": "0xD85AD783cc94bd04196a13DC042A3054a9B52210",
    "symbol": "HAKA",
    "name": "TribeOne",
    "decimals": 18,
    "logoURI": "ipfs://QmcNkZUyKvZrvwpU2CpjwUA1A4UWFa8r5VL5TeWiL6r1ie"
  },
  {
    "chainId": 1,
    "address": "0x554FFc77F4251a9fB3c0E3590a6a205f8d4e067D",
    "symbol": "ZMN",
    "name": "ZMINE",
    "decimals": 18,
    "logoURI": "ipfs://QmZqytjD7WW9dpwSe57d9b8DLe9jcWTd7RmGVEEzJb8mDE"
  },
  {
    "chainId": 1,
    "address": "0x78F225869c08d478c34e5f645d07A87d3fe8eb78",
    "symbol": "DEFI+L",
    "name": "PieDAO DEFI Large Cap",
    "decimals": 18,
    "logoURI": "ipfs://Qmc8BrCxfGgvoAu9bmCTXengzWUKX5PEEKTkSkP5EWkJyR"
  },
  {
    "chainId": 100,
    "address": "0x8e5bBbb09Ed1ebdE8674Cda39A0c169401db4252",
    "symbol": "WBTC",
    "name": "Wrapped Bitcoin",
    "decimals": 8,
    "logoURI": "ipfs://QmWabMBiaCBXDEu1Cc6A6NoLVJZCqd9GWwtTgxentKbeZ4"
  },
  {
    "chainId": 1,
    "address": "0xCC8Fa225D80b9c7D42F96e9570156c65D6cAAa25",
    "symbol": "SLP",
    "name": "Smooth Love Potion",
    "decimals": 0,
    "logoURI": "ipfs://QmcGQbVKiruHmKrVTrjsPa7DNqBbmd7PXB26sPQaw3Um4v"
  },
  {
    "chainId": 1,
    "address": "0x83F20F44975D03b1b09e64809B757c47f942BEeA",
    "symbol": "sDAI",
    "name": "Savings DAI",
    "decimals": 18,
    "logoURI": "ipfs://Qma2GoCXE7RN7uarcujTmJHXxpxyCBvUBUKR6eJKPWjvC5"
  },
  {
    "chainId": 1,
    "address": "0x89Ab32156e46F46D02ade3FEcbe5Fc4243B9AAeD",
    "symbol": "PNT",
    "name": "pNetwork",
    "decimals": 18,
    "logoURI": "ipfs://Qmb5cdSaXP3MYFdcCMqdLdZCr8hNhzPrb5x7HM2UPd2SxC"
  },
  {
    "chainId": 1,
    "address": "0xB9D7CB55f463405CDfBe4E90a6D2Df01C2B92BF1",
    "symbol": "aUNI",
    "name": "Aave UNI",
    "decimals": 18,
    "logoURI": "ipfs://QmdzG2wx3oRtPzZfyVLW8dMw26oxyRcSBEdZ1wDCFVKcgr"
  },
  {
    "chainId": 1,
    "address": "0x71dc40668682a124231301414167E4CF7f55383C",
    "symbol": "MIMIR",
    "name": "Mimir",
    "decimals": 18,
    "logoURI": "ipfs://QmZm3fuCFNaE1s3tH6NrgHBFRjbVEAu5XaZ4ASZ8oRyt6f"
  },
  {
    "chainId": 250,
    "address": "0x468003B688943977e6130F4F68F23aad939a1040",
    "symbol": "SPELL",
    "name": "Spell",
    "decimals": 18,
    "logoURI": "ipfs://QmRQ87x9VSD3iyvoqYXqVAcKGUAmUwYu5RiuqExNkjYyfa"
  },
  {
    "chainId": 1,
    "address": "0xcb86c6A22CB56B6cf40CaFEDb06BA0DF188a416E",
    "symbol": "SURE",
    "name": "inSure DeFi",
    "decimals": 18,
    "logoURI": "ipfs://QmdXbCyP11ThyvYVrNVmdSmAeQFTkqJ8z9sQK3VgAWcgYw"
  },
  {
    "chainId": 42161,
    "address": "0x180F7cF38805d1be95C7632F653E26B0838e2969",
    "symbol": "XDEFI",
    "name": "XDEFI",
    "decimals": 18,
    "logoURI": "ipfs://QmSH3Yoaxr8t1QchiyreXGebwfwS4AgL3STswEefa5TwCd"
  },
  {
    "chainId": 56,
    "address": "0x64048A7eEcF3a2F1BA9e144aAc3D7dB6e58F555e",
    "symbol": "frxETH",
    "name": "Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmQqQhtwApaT4X9FAaAzBRvyeE2iaSDH4ahD6exWXcszZU"
  },
  {
    "chainId": 1,
    "address": "0xbc16da9df0A22f01A16BC0620a27e7D6d6488550",
    "symbol": "PCT",
    "name": "Percent",
    "decimals": 18,
    "logoURI": "ipfs://QmV8BhAsiScgk2Rc8hR5VHpEBMADiRtwvFZs3LUDMCHAgQ"
  },
  {
    "chainId": 10,
    "address": "0xDA10009cBd5D07dd0CeCc66161FC93D7c9000da1",
    "symbol": "DAI",
    "name": "Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmTswuPmisP64jxzZvGz6AaK2gosJyPeTwqMkNPob6pdaa"
  },
  {
    "chainId": 1,
    "address": "0x6e5970DBd6fc7eb1f29C6D2eDF2bC4c36124C0C1",
    "symbol": "TRADE",
    "name": "Polytrade",
    "decimals": 18,
    "logoURI": "ipfs://QmZ8VAAw7szkxXAg8qLCYQo5iCafwYB7GWicam4zyMomzA"
  },
  {
    "chainId": 1,
    "address": "0xf485C5E679238f9304D986bb2fC28fE3379200e5",
    "symbol": "UGC",
    "name": "ugChain",
    "decimals": 18,
    "logoURI": "ipfs://QmSsiZEQwjM8dAECknzpoEycEw98W58pEVV6k1Nd28578g"
  },
  {
    "chainId": 1,
    "address": "0x80f222a749a2e18Eb7f676D371F19ad7EFEEe3b7",
    "symbol": "MGN",
    "name": "Magnolia",
    "decimals": 18,
    "logoURI": "ipfs://QmWoA3rTV1PqdLdqiivN22LTMZPK3u8AozE5sxXBganHCL"
  },
  {
    "chainId": 56,
    "address": "0x7bd6FaBD64813c48545C9c0e312A0099d9be2540",
    "symbol": "ELON",
    "name": "Dogelon Mars",
    "decimals": 18,
    "logoURI": "ipfs://QmaHs61uW9nZYiHVSQuZgnQWBPSwNokcnNgZn7FGMhSHVk"
  },
  {
    "chainId": 1,
    "address": "0x4fa000dF40C06FC8c7D9179661535846B7Cd4f87",
    "symbol": "ZNT",
    "name": "Zenswap Network",
    "decimals": 18,
    "logoURI": "ipfs://QmUjzQTjwdXZvDcjXShJRdC877GVcTqn4zu1KiJUDhDoZp"
  },
  {
    "chainId": 1,
    "address": "0xFA57F00D948bb6a28072f5416fCbF7836C3d62dD",
    "symbol": "FRIES",
    "name": "friesDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmfL5Nsbjhf4JY3u5RAxnZkTrr4K4YiFCr6Eo7vCd5pa24"
  },
  {
    "chainId": 1,
    "address": "0x52d87F22192131636F93c5AB18d0127Ea52CB641",
    "symbol": "renLUNA",
    "name": "renLUNA",
    "decimals": 6,
    "logoURI": "ipfs://QmRKGynzRpQj5DPRWUs9n7VRtZpzvYXJ3FgUnNnDd2CULX"
  },
  {
    "chainId": 1,
    "address": "0x6911F552842236bd9E8ea8DDBB3fb414e2C5FA9d",
    "symbol": "SNP",
    "name": "Synapse Network",
    "decimals": 18,
    "logoURI": "ipfs://QmVBYsG6RmVCno8Sim8GiXw32pYFHgox4i57kwWBj2ixqX"
  },
  {
    "chainId": 1,
    "address": "0x1350Fbe8Ce27762ec19134bf8FC405a427fe9Bf1",
    "symbol": "TRIPS",
    "name": "TRIPS",
    "decimals": 18,
    "logoURI": "ipfs://QmXUFjoJx4mp3RbyXnvGSwLFo9hsdFeFU2WMk6SiAXbWgw"
  },
  {
    "chainId": 56,
    "address": "0xf16e81dce15B08F326220742020379B855B87DF9",
    "symbol": "ICE",
    "name": "Popsicle Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmSp3Vnq7VFsFrdcZZ35YkHHFLVF3nWvcDPKSDJgfkTiMD"
  },
  {
    "chainId": 56,
    "address": "0x8443f091997f06a61670B735ED92734F5628692F",
    "symbol": "BEL",
    "name": "Bella Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmPNxN9GWvwgTg4r72cX3vENWxPPJVnFvHxjVAB95REA5y"
  },
  {
    "chainId": 1,
    "address": "0x3155BA85D5F96b2d030a4966AF206230e46849cb",
    "symbol": "RUNE",
    "name": "THORChain",
    "decimals": 18,
    "logoURI": "ipfs://Qmc62WwaGG6jDw94msaiHe9QkU2shpAEXmMSkWmNjDzYmo"
  },
  {
    "chainId": 56,
    "address": "0xA07c5b74C9B40447a954e1466938b865b6BBea36",
    "symbol": "vBNB",
    "name": "Venus BNB",
    "decimals": 8,
    "logoURI": "ipfs://QmS8dd9K38HKP72mqEnEPNcX4uZr5wB9eg4hZr3qR8wHbt"
  },
  {
    "chainId": 1,
    "address": "0xbdEB4b83251Fb146687fa19D1C660F99411eefe3",
    "symbol": "SVD",
    "name": "savedroid",
    "decimals": 18,
    "logoURI": "ipfs://QmNhq6SkP6o4FML4AfWARryWn3bFMm5iySxRSR4XEnDsRM"
  },
  {
    "chainId": 1,
    "address": "0x687642347a9282Be8FD809d8309910A3f984Ac5a",
    "symbol": "iKNC",
    "name": "Fulcrum KNC iToken",
    "decimals": 18,
    "logoURI": "ipfs://QmWAbwq559hBwBBKYgjNR6SwaujQFKjcvqHKGbd3Ehpz9f"
  },
  {
    "chainId": 1,
    "address": "0xfF3Ac80c1caA08Cbd43a7e90d20c398d54C7342f",
    "symbol": "nICE",
    "name": "Staked Ice",
    "decimals": 18,
    "logoURI": "ipfs://QmWYeq6cyC6Cthagmqn9dCst62t2vuCu3Fm8UJi85VYFpC"
  },
  {
    "chainId": 56,
    "address": "0xF64eD9Ad397A1Ae657F31131d4B189220A7f1cc7",
    "symbol": "DFIAT",
    "name": "DeFiato",
    "decimals": 18,
    "logoURI": "ipfs://QmVdCKa4U5hP4y82CoEPDiTJkKoQFLY6JwnmbmGZxGSvrb"
  },
  {
    "chainId": 1,
    "address": "0x429881672B9AE42b8EbA0E26cD9C73711b891Ca5",
    "symbol": "PICKLE",
    "name": "Pickle Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRo1htyxY5mMwuzQzmE2f3yvorxTLqmXjNA4YmR4gJCWR"
  },
  {
    "chainId": 137,
    "address": "0xa22f6bc96f13bcC84dF36109c973d3c0505a067E",
    "symbol": "jKRW",
    "name": "Jarvis Synthetic South Korean Won",
    "decimals": 18,
    "logoURI": "ipfs://QmNWbvcuMDv7xiemxJDmYJS5byjvMFE9tWQVvvcYJ1crvv"
  },
  {
    "chainId": 1,
    "address": "0xC741f06082AA47F93729070aD0dD95E223Bda091",
    "symbol": "LEDU",
    "name": "Education Ecosystem",
    "decimals": 8,
    "logoURI": "ipfs://QmQgyLd2GZWJxS97xExPpuuqyogdaVdxDtooVit4qUx17K"
  },
  {
    "chainId": 1,
    "address": "0x25377ddb16c79C93B0CBf46809C8dE8765f03FCd",
    "symbol": "SBREE",
    "name": "CBDAO",
    "decimals": 18,
    "logoURI": "ipfs://QmbJVusZHgicX9g42uYykCEhe8VgPZxyhToeBXDScjedCk"
  },
  {
    "chainId": 1,
    "address": "0xbdab72602e9AD40FC6a6852CAf43258113B8F7a5",
    "symbol": "eSOV",
    "name": "Sovryn",
    "decimals": 18,
    "logoURI": "ipfs://QmPwahMuMXsKuK7C4YKcoMjugRrZ8ams6JqJXY734qPYwc"
  },
  {
    "chainId": 1,
    "address": "0x960b236A07cf122663c4303350609A66A7B288C0",
    "symbol": "ANT",
    "name": "Aragon",
    "decimals": 18,
    "logoURI": "ipfs://QmY8qL4UouA8u2ZeNfJj26MMi98DCwY7a9fshEmHhh475p"
  },
  {
    "chainId": 1,
    "address": "0x38A94e92A19E970c144DEd0B2DD47278CA11CC1F",
    "symbol": "F9",
    "name": "Falcon Nine",
    "decimals": 9,
    "logoURI": "ipfs://QmYVd8MYZe9fBPwgfRGJksDmz5xTiiJFQsDXgTeaT5B9yi"
  },
  {
    "chainId": 1,
    "address": "0x19810559dF63f19cfE88923313250550eDADB743",
    "symbol": "HOUSE",
    "name": "Toast.finance",
    "decimals": 0,
    "logoURI": "ipfs://QmXykNAbiMdmHytv45GmthsHWW7nvuK1CZrUPccxfAKM4o"
  },
  {
    "chainId": 137,
    "address": "0xA8b1E0764f85f53dfe21760e8AfE5446D82606ac",
    "symbol": "BAND",
    "name": "Band Protocol",
    "decimals": 18,
    "logoURI": "ipfs://Qma3oFfhCHJqvLu9E2v9REMGZJnsGJd2BVUi94C54HFLmT"
  },
  {
    "chainId": 43114,
    "address": "0x9702230A8Ea53601f5cD2dc00fDBc13d4dF4A8c7",
    "symbol": "USDT",
    "name": "Tether",
    "decimals": 6,
    "logoURI": "ipfs://Qmb3xCbcXGs8pa1YMvmHvN4dyadcaUkW7ZDy9mZL7pZ4aF"
  },
  {
    "chainId": 1,
    "address": "0x745a824D6aBBD236AA794b5530062778A6Ad7523",
    "symbol": "sMSFT",
    "name": "Synthetic Microsoft",
    "decimals": 18,
    "logoURI": "ipfs://QmaGcVN1cgNBLo9vhRtGvWPcceVWpirgtCcvEkwSSSzSbq"
  },
  {
    "chainId": 1,
    "address": "0x2Ebd53d035150f328bd754D6DC66B99B0eDB89aa",
    "symbol": "MET",
    "name": "Metronome",
    "decimals": 18,
    "logoURI": "ipfs://QmXEa85dGiXuCeVgFGKvHpprY9E7XECS6RbPmQCwMRLaPh"
  },
  {
    "chainId": 1,
    "address": "0xbE9375C6a420D2eEB258962efB95551A5b722803",
    "symbol": "STMX",
    "name": "StormX",
    "decimals": 18,
    "logoURI": "ipfs://QmUidHFkPZuEWJBKqYKGy84wypwPVUEkdzVC8QARAWc3xH"
  },
  {
    "chainId": 42161,
    "address": "0x876Ec6bE52486Eeec06bc06434f3E629D695c6bA",
    "symbol": "FLUID",
    "name": "FluidFi",
    "decimals": 18,
    "logoURI": "ipfs://QmafyVs5JP5iaejmfVFjr1myFToLL8BdGyJdA6oxMwd4LZ"
  },
  {
    "chainId": 1,
    "address": "0x17107f40d70f4470d20CB3f138a052cAE8EbD4bE",
    "symbol": "crRENBTC",
    "name": "Cream renBTC",
    "decimals": 8,
    "logoURI": "ipfs://QmUAXvA6qwHtxFjjNQz8arc8jm4zS5jiJjVmr3zzSAfaES"
  },
  {
    "chainId": 1,
    "address": "0x97b65710D03E12775189F0D113202cc1443b0aa2",
    "symbol": "ELONONE",
    "name": "AstroElon",
    "decimals": 9,
    "logoURI": "ipfs://QmdP3eVJrsGTG3UXxkfBEVmSi5BZbhCQonuSztPV1yRtJW"
  },
  {
    "chainId": 1,
    "address": "0x101cc05f4A51C0319f570d5E146a8C625198e636",
    "symbol": "aTUSD",
    "name": "Aave TUSD",
    "decimals": 18,
    "logoURI": "ipfs://QmQhjFcKLaEoocDduDcpmgVA9JdLEsUq4HUTzdRwSTkDUF"
  },
  {
    "chainId": 1,
    "address": "0xA8b919680258d369114910511cc87595aec0be6D",
    "symbol": "LYXe",
    "name": "LUKSO",
    "decimals": 18,
    "logoURI": "ipfs://QmP1j6XNEBgvVNWpi6uFSUz2Pc7GGBzrta9hpS3bKfHdaZ"
  },
  {
    "chainId": 8453,
    "address": "0x2Ae3F1Ec7F1F5012CFEab0185bfc7aa3cf0DEc22",
    "symbol": "cbETH",
    "name": "Coinbase Wrapped Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://Qmd4LULSCJvNUcnSuYrXnPvhibUR6SNDSuAYAE2YTjSN64"
  },
  {
    "chainId": 250,
    "address": "0x6626c47c00F1D87902fc13EECfaC3ed06D5E8D8a",
    "symbol": "WOO",
    "name": "WOO Network",
    "decimals": 18,
    "logoURI": "ipfs://QmdJjzzpWuzQAN6sj2j5FRy3SnFYgwMPQCx5ZNrFdVpcFt"
  },
  {
    "chainId": 56,
    "address": "0x048E9b1ddF9EBbb224812372280e94Ccac443f9e",
    "symbol": "jGBP",
    "name": "Jarvis Synthetic British Pound",
    "decimals": 18,
    "logoURI": "ipfs://QmaTirQDCtHCjoNCnHyQJTB46nxFWq4b75cidRURRKhaYW"
  },
  {
    "chainId": 1,
    "address": "0x641927E970222B10b2E8CDBC96b1B4F427316f16",
    "symbol": "MEEB",
    "name": "MEEB Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmczD2QxDB4qGu75CGncc3vAfrt5trWWgExq5rhc5nDGYt"
  },
  {
    "chainId": 1,
    "address": "0x8ED2Fc62D6850eAaDcB717465752dab591286839",
    "symbol": "$ELY",
    "name": "Elyssa AR",
    "decimals": 18,
    "logoURI": "ipfs://QmRJ97GH3vZJu6D7KMBp7US2Ad34K773xNnP3fXjckocK3"
  },
  {
    "chainId": 137,
    "address": "0x9cd6746665D9557e1B9a775819625711d0693439",
    "symbol": "LUNC",
    "name": "Terra Classic",
    "decimals": 6,
    "logoURI": "ipfs://QmceuGzAgd2wcWiDvX9ygEohHnaMqWDpSok7ot1h7jahZt"
  },
  {
    "chainId": 1,
    "address": "0xd33D0EB1c0c0295c3A2409a9101dd511823Bf217",
    "symbol": "FANS",
    "name": "Unique Fans",
    "decimals": 18,
    "logoURI": "ipfs://QmeCa3q2QFKELvcjvGc9PUnJ7oRQrzfnrzC2hG1apn16Xo"
  },
  {
    "chainId": 1,
    "address": "0x5b558564B57E4ff88c6b8D8E7EeEe599bF79B368",
    "symbol": "MMM",
    "name": "MultiMillion",
    "decimals": 18,
    "logoURI": "ipfs://QmNdesYg1cUmHDhqBCbeHxHpdgp32MrGKoRzHwqzAUBaSX"
  },
  {
    "chainId": 1,
    "address": "0x2a3bFF78B79A009976EeA096a51A948a3dC00e34",
    "symbol": "WILD",
    "name": "Wilder World",
    "decimals": 18,
    "logoURI": "ipfs://QmPTtQX3LjnnijiTqRaECUJVBzxwfERSLBmZRiPe4iqtjY"
  },
  {
    "chainId": 1,
    "address": "0xF411903cbC70a74d22900a5DE66A2dda66507255",
    "symbol": "VRA",
    "name": "Verasity",
    "decimals": 18,
    "logoURI": "ipfs://QmNsxQGWZ6UsbRWKiGgmHGVZnKa4Yk4YuhWYqvYvBtQR2z"
  },
  {
    "chainId": 1,
    "address": "0x6c6EE5e31d828De241282B9606C8e98Ea48526E2",
    "symbol": "HOT",
    "name": "Holo",
    "decimals": 18,
    "logoURI": "ipfs://QmeNKZj7zozcViKgmfLKNDKohwtzhMdih8RSj22bNU7XPp"
  },
  {
    "chainId": 1,
    "address": "0xfAd45E47083e4607302aa43c65fB3106F1cd7607",
    "symbol": "HOGE",
    "name": "Hoge Finance",
    "decimals": 9,
    "logoURI": "ipfs://QmXDBwp7ksasenfhiWKQeNZuW7zoLsXZNA7dVFo3WPDXQD"
  },
  {
    "chainId": 1,
    "address": "0x158079Ee67Fce2f58472A96584A73C7Ab9AC95c1",
    "symbol": "cREP",
    "name": "Compound Augur",
    "decimals": 8,
    "logoURI": "ipfs://QmcYFwayw7U6HvW5SssBELZiX1iyzpBDfQFTMJ3TywGKD3"
  },
  {
    "chainId": 1,
    "address": "0x7DEdBce5a2E31E4c75f87FeA60bF796C17718715",
    "symbol": "PNP",
    "name": "Penpie",
    "decimals": 18,
    "logoURI": "ipfs://QmSD3cGiAXwRujKBgGt5ZHTK9d1Dqj34GRB7jU6gMbwD2Y"
  },
  {
    "chainId": 1,
    "address": "0xc58467b855401EF3FF8FdA9216F236e29f0d6277",
    "symbol": "GASG",
    "name": "Gasgains",
    "decimals": 18,
    "logoURI": "ipfs://QmcccQYN4jDQx9U4en8PcstcvMhTzZbq9XhvWW3HiEwNL4"
  },
  {
    "chainId": 1,
    "address": "0xa2B0fDe6D710e201d0d608e924A484d1A5fEd57c",
    "symbol": "sXRP",
    "name": "Synthetic Ripple",
    "decimals": 18,
    "logoURI": "ipfs://QmPz17iKSnxGdFTMK72iLKVFXYVczfkTz7EPcUxjXsKFDy"
  },
  {
    "chainId": 1,
    "address": "0x330839EF82D34801bd96e75a4Ee778ac56Fa1ED8",
    "symbol": "ICH",
    "name": "ICOStart",
    "decimals": 18,
    "logoURI": "ipfs://QmZfHRVgctZQDwbXX734E3bgQgeEckmfwwoqHmG9oK3GQL"
  },
  {
    "chainId": 1,
    "address": "0x546AED37d202d607F45Cbd2b8C0CaD0D25fBe339",
    "symbol": "KAWA",
    "name": "Kawakami",
    "decimals": 18,
    "logoURI": "ipfs://QmZyLwwtNhXtFk6ftujff8Fmyj16kAA2JZtDHA7sr3vZTD"
  },
  {
    "chainId": 1,
    "address": "0x998FFE1E43fAcffb941dc337dD0468d52bA5b48A",
    "symbol": "IDRT",
    "name": "Rupiah Token",
    "decimals": 2,
    "logoURI": "ipfs://QmNfbzNAsdGD2iya3PgcBsEvFt3WGBqZhA4E67twPnpCoq"
  },
  {
    "chainId": 56,
    "address": "0x74ccbe53F77b08632ce0CB91D3A545bF6B8E0979",
    "symbol": "fBOMB",
    "name": "fBomb",
    "decimals": 18,
    "logoURI": "ipfs://QmaZGK72QKQJDmkfxA4CTmche9BTmgBgQiViZ3rhyxkZ2n"
  },
  {
    "chainId": 1,
    "address": "0xA6446D655a0c34bC4F05042EE88170D056CBAf45",
    "symbol": "CSP",
    "name": "Caspian",
    "decimals": 18,
    "logoURI": "ipfs://QmSBuGQ1vF317jQpKM3wUJ38uUVTYsWgRUAEUcyGETyTVs"
  },
  {
    "chainId": 1,
    "address": "0x4c9BBFc1FbD93dFB509E718400978fbEedf590E9",
    "symbol": "RAI",
    "name": "Rai",
    "decimals": 18,
    "logoURI": "ipfs://QmNQFDN6HAqEwkp6Un97xfWc91WEDJ5hfGAA5hgPt5UYyw"
  },
  {
    "chainId": 1,
    "address": "0x09617F6fD6cF8A71278ec86e23bBab29C04353a7",
    "symbol": "ULT",
    "name": "Shardus",
    "decimals": 18,
    "logoURI": "ipfs://QmYZHdz5JzpXnoqgdcQmwJzoGUedJx9bnrt9SJMibqvVM1"
  },
  {
    "chainId": 42161,
    "address": "0xe05A08226c49b636ACf99c40Da8DC6aF83CE5bB3",
    "symbol": "ankrETH",
    "name": "Ankr Staked ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmWpizC1p46quYHixpjZRjZFFrHvz8byAuZyzKtBdRxZUP"
  },
  {
    "chainId": 1,
    "address": "0xB4d930279552397bbA2ee473229f89Ec245bc365",
    "symbol": "MAHA",
    "name": "MahaDAO",
    "decimals": 18,
    "logoURI": "ipfs://Qma97nj1LF9c1CwteQHHDBJuPdr4JiAfYpbeiiXsUfc6CJ"
  },
  {
    "chainId": 1,
    "address": "0xfc05987bd2be489ACCF0f509E44B0145d68240f7",
    "symbol": "ESS",
    "name": "Essentia",
    "decimals": 18,
    "logoURI": "ipfs://QmRHzxLdPxQudupVQ6QJzezTnae8t8RJc9PZeFK6oA72LL"
  },
  {
    "chainId": 1,
    "address": "0xf2Ef3551C1945A7218fc4eC0a75c9eCFDF012A4F",
    "symbol": "C4G3",
    "name": "Cage",
    "decimals": 18,
    "logoURI": "ipfs://QmSdr6HJ7WAJ4HS9CbR8LvvCKtkDvj3YVWKGAuS58Avjyg"
  },
  {
    "chainId": 1,
    "address": "0x8deef89058090ac5655A99EEB451a4f9183D1678",
    "symbol": "iXTZ",
    "name": "Synthetic Inverse Tezos",
    "decimals": 18,
    "logoURI": "ipfs://QmcFvvRzmrWHmBv8uFziQH3P1WpB6RXCLJJCb3UeWYAHxX"
  },
  {
    "chainId": 1,
    "address": "0x4691937a7508860F876c9c0a2a617E7d9E945D4B",
    "symbol": "WOO",
    "name": "WOO Network",
    "decimals": 18,
    "logoURI": "ipfs://QmP3qXoD1NyGiHHsRPcEkmuasZj1cQPivy5KV93SEjgGVZ"
  },
  {
    "chainId": 56,
    "address": "0xCC42724C6683B7E57334c4E856f4c9965ED682bD",
    "symbol": "MATIC",
    "name": "Matic",
    "decimals": 18,
    "logoURI": "ipfs://QmcMPwSeDvVa1K8hXKPbU4diCwGC3BXYK4dsNtPNMVFHJQ"
  },
  {
    "chainId": 1,
    "address": "0x9E32b13ce7f2E80A01932B42553652E053D6ed8e",
    "symbol": "METIS",
    "name": "Metis",
    "decimals": 18,
    "logoURI": "ipfs://QmSifbmJ5Jms4vAPKhtNwHqjzyEosdZvbjmjcYZMW3jBb4"
  },
  {
    "chainId": 1,
    "address": "0x33e18a092a93ff21aD04746c7Da12e35D34DC7C4",
    "symbol": "PLAY",
    "name": "Metaverse NFT Index",
    "decimals": 18,
    "logoURI": "ipfs://QmVNJQrJLdbzHUV1m8cxfmLeX5FEG9qjR6cBbJanVLnPge"
  },
  {
    "chainId": 1,
    "address": "0x08711D3B02C8758F2FB3ab4e80228418a7F8e39c",
    "symbol": "EDG",
    "name": "Edgeless",
    "decimals": 0,
    "logoURI": "ipfs://QmS8etH8X4mmdVnzFnhJ7yjjnvp3NZJSnWzfXL3EGgXThV"
  },
  {
    "chainId": 1,
    "address": "0x3DB6Ba6ab6F95efed1a6E794caD492fAAabF294D",
    "symbol": "LTO",
    "name": "LTO Network",
    "decimals": 8,
    "logoURI": "ipfs://QmU9h5yk7wfJHCS65Hng9qLuUVDPBDCsxP6CFiEwkEQom7"
  },
  {
    "chainId": 1,
    "address": "0x076C97e1c869072eE22f8c91978C99B4bcB02591",
    "symbol": "CBT",
    "name": "CommerceBlock",
    "decimals": 18,
    "logoURI": "ipfs://QmTqKxhQziBer8AitBdYX3MARSxiHjAGrtKf9iw3L6KRwp"
  },
  {
    "chainId": 56,
    "address": "0xeEeEEb57642040bE42185f49C52F7E9B38f8eeeE",
    "symbol": "ELK",
    "name": "Elk Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYSmFMg8incix4Hzs85MPZzQJnb2W19zLTL3nXNEzY42m"
  },
  {
    "chainId": 1,
    "address": "0xA974c709cFb4566686553a20790685A47acEAA33",
    "symbol": "XIN",
    "name": "Mixin",
    "decimals": 18,
    "logoURI": "ipfs://QmfSwqgnnDxYEGPyftYoSx7FQn8zqh9dPFmiGzNcUUKooX"
  },
  {
    "chainId": 1,
    "address": "0xD5525D397898e5502075Ea5E830d8914f6F0affe",
    "symbol": "MEME",
    "name": "Meme",
    "decimals": 8,
    "logoURI": "ipfs://QmeFWqQo2vN7JMoefQ3RYuAiHfZZU6487edGyfYSrxzQkg"
  },
  {
    "chainId": 42161,
    "address": "0xEC70Dcb4A1EFa46b8F2D97C310C9c4790ba5ffA8",
    "symbol": "rETH",
    "name": "Rocket Pool ETH",
    "decimals": 18,
    "logoURI": "ipfs://QmRqLBdZaA2oSr7yWVmwdSyj5HwixHMWRpyKw8QyAQTgRz"
  },
  {
    "chainId": 56,
    "address": "0x3019BF2a2eF8040C242C9a4c5c4BD4C81678b2A1",
    "symbol": "GMT",
    "name": "STEPN",
    "decimals": 8,
    "logoURI": "ipfs://QmaXgp7LnvKbQ64Bb3P1e92Cju9KNbxzwzdLaLqb7nrUUW"
  },
  {
    "chainId": 1,
    "address": "0xc2544A32872A91F4A553b404C6950e89De901fdb",
    "symbol": "FPIS",
    "name": "Frax Price Index Share",
    "decimals": 18,
    "logoURI": "ipfs://QmWzJ6WF973aNnrMxk1d7GVmEvjZH4PBXWuHhHopnMniuo"
  },
  {
    "chainId": 1,
    "address": "0x856c4Efb76C1D1AE02e20CEB03A2A6a08b0b8dC3",
    "symbol": "OETH",
    "name": "Origin Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmdSRVuw27BYmyHNAjH3hanuZUbfCiByhtiUK1siX6LaBT"
  },
  {
    "chainId": 10,
    "address": "0x4200000000000000000000000000000000000006",
    "symbol": "WETH",
    "name": "Wrapped Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmPt2DEXhRSvbbFQep44XXtcKxRscshsijJ47Y1xUQwBZ1"
  },
  {
    "chainId": 100,
    "address": "0x6A8cb6714B1EE5b471a7D2eC4302cb4f5Ff25eC2",
    "symbol": "EWT",
    "name": "Energy Web Token",
    "decimals": 18,
    "logoURI": "ipfs://QmPx8qAq3NifpADHkDKJb8qyTT8DdsDH5N36wbSYMWkMRF"
  },
  {
    "chainId": 56,
    "address": "0x2fF3d0F6990a40261c66E1ff2017aCBc282EB6d0",
    "symbol": "vSXP",
    "name": "Venus SXP",
    "decimals": 8,
    "logoURI": "ipfs://QmVXhSid5JyyBuhP4fvM6VpMpmtNh5mHiQcR8X7A54qJxF"
  },
  {
    "chainId": 1,
    "address": "0x243cACb4D5fF6814AD668C3e225246efA886AD5a",
    "symbol": "SHI",
    "name": "Shina Inu",
    "decimals": 18,
    "logoURI": "ipfs://QmewMHz4rCXYVcHRbYhp4qz8KMuP6KVfSGh54JUnNGNMh3"
  },
  {
    "chainId": 56,
    "address": "0x027a9d301FB747cd972CFB29A63f3BDA551DFc5c",
    "symbol": "mWOM",
    "name": "Magpie WOM",
    "decimals": 18,
    "logoURI": "ipfs://QmcLq2WDJcR8FPSw2guBFpCJaA3PPd8rPvdquCUHVwF48u"
  },
  {
    "chainId": 1,
    "address": "0x514910771AF9Ca656af840dff83E8264EcF986CA",
    "symbol": "LINK",
    "name": "Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://QmZ6xMzTmuuMBXGXDxsqYzscFxeCeyLPzTbYDcENuuxn6t"
  },
  {
    "chainId": 1,
    "address": "0x219F4a1D142DFC564bD6e80c022cD29f3394A999",
    "symbol": "WXLM",
    "name": "Wrapped Stellar",
    "decimals": 18,
    "logoURI": "ipfs://QmaVomWHZrtVrLtvdkcUHKt9HNTdWL38QrYSWvfUYk8Yto"
  },
  {
    "chainId": 1,
    "address": "0x1b40183EFB4Dd766f11bDa7A7c3AD8982e998421",
    "symbol": "VSP",
    "name": "Vesper Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmX3NmrFqtuJNaDDWMpnFXaEh3g3dZzmbjBN6L1i2C59HK"
  },
  {
    "chainId": 1,
    "address": "0x40ce0A1D8F4999807b92ec266a025F071814b15d",
    "symbol": "NOTRUMP",
    "name": "Dai If Trump Loses The 2020 Election",
    "decimals": 18,
    "logoURI": "ipfs://QmaHpCUg5SJbVwWRsvwNk3upqs8fBjz9yu3exbEWb7dhHf"
  },
  {
    "chainId": 1,
    "address": "0x7240aC91f01233BaAf8b064248E80feaA5912BA3",
    "symbol": "OCTO",
    "name": "OctoFi",
    "decimals": 18,
    "logoURI": "ipfs://QmbGHwDkfGjC8v56NqrfL2wkCmJjq3Wg9byebEDpkY8aG9"
  },
  {
    "chainId": 1,
    "address": "0x44709a920fCcF795fbC57BAA433cc3dd53C44DbE",
    "symbol": "RADAR",
    "name": "DappRadar",
    "decimals": 18,
    "logoURI": "ipfs://QmevnqiqZ872GfyFjV7HCvCLfegzdb63BTvTaip6w6aGz1"
  },
  {
    "chainId": 42161,
    "address": "0x6C2C06790b3E3E3c38e12Ee22F8183b37a13EE55",
    "symbol": "DPX",
    "name": "Dopex",
    "decimals": 18,
    "logoURI": "ipfs://QmYmKcbBgVJnfJQrUmU7GM28pAG1ZV1d5wT2AKbM3Bm3mZ"
  },
  {
    "chainId": 1,
    "address": "0x041fdd6637eCfD96af8804278AC12660ac2D12c0",
    "symbol": "SDX",
    "name": "SwapDEX",
    "decimals": 7,
    "logoURI": "ipfs://QmaAvzz7nUuCXxy8R7bnc53ydmhU1bokpHFPgbpkJ5tHvW"
  },
  {
    "chainId": 100,
    "address": "0xbf65bfcb5da067446CeE6A706ba3Fe2fB1a9fdFd",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUqUt4Z4ftUcKUKpT4RMWJUTcRyoTu1ni9tCno1nXwHPR"
  },
  {
    "chainId": 1,
    "address": "0x01da76DEa59703578040012357b81ffE62015C2d",
    "symbol": "crYETH",
    "name": "Cream yETH",
    "decimals": 8,
    "logoURI": "ipfs://QmT4fQBW315Hd9NgPD9NAE3hHgZyAaDFL3VjPCnAgbWa19"
  },
  {
    "chainId": 42161,
    "address": "0x20547341E58fB558637FA15379C92e11F7b7F710",
    "symbol": "MOZ",
    "name": "Mozaic",
    "decimals": 18,
    "logoURI": "ipfs://QmXC2NamcsJE5ZUTpRGujnBoNDUL4is2xuTNAZQ1qCYtm4"
  },
  {
    "chainId": 137,
    "address": "0x3Cef98bb43d732E2F285eE605a8158cDE967D219",
    "symbol": "BAT",
    "name": "Basic Attention Token",
    "decimals": 18,
    "logoURI": "ipfs://QmZPngVewTaGaXqN28CnZ9EJSBJ2jAEnMao87T9RbPUoGM"
  },
  {
    "chainId": 8453,
    "address": "0x9EaF8C1E34F05a589EDa6BAfdF391Cf6Ad3CB239",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmRQq6tqSDu6pvZPmYHhZsF9dvUqzudAnfn6cKPurWzQbF"
  },
  {
    "chainId": 1,
    "address": "0x2e98A6804E4b6c832ED0ca876a943abD3400b224",
    "symbol": "BELA",
    "name": "Bela",
    "decimals": 18,
    "logoURI": "ipfs://QmXRXMsoQHM6nZaDYtB5HujCnojYW6ngKL8Z5uSRds8UUn"
  },
  {
    "chainId": 1,
    "address": "0x56d811088235F11C8920698a204A5010a788f4b3",
    "symbol": "BZRX",
    "name": "bZx Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmY1PvCUwosCACycBjKdy2HM3XS1HSUaYYJUMU1C4WKMNw"
  },
  {
    "chainId": 1,
    "address": "0x10Bae51262490B4f4AF41e12eD52A0E744c1137A",
    "symbol": "SLINK",
    "name": "Soft Link",
    "decimals": 9,
    "logoURI": "ipfs://QmbxeRCr8KeAoedAWftc3pXwdy83u7wUbMYx1dggTxw7u6"
  },
  {
    "chainId": 1,
    "address": "0x0bc529c00C6401aEF6D220BE8C6Ea1667F6Ad93e",
    "symbol": "YFI",
    "name": "yearn.finance",
    "decimals": 18,
    "logoURI": "ipfs://QmUqUt4Z4ftUcKUKpT4RMWJUTcRyoTu1ni9tCno1nXwHPR"
  },
  {
    "chainId": 1,
    "address": "0xEA26c4aC16D4a5A106820BC8AEE85fd0b7b2b664",
    "symbol": "QKC",
    "name": "QuarkChain",
    "decimals": 18,
    "logoURI": "ipfs://QmdNmZ5fZuk9uj68JceJm9yWSLfu24Qm29WDiZbeY5ETcY"
  },
  {
    "chainId": 56,
    "address": "0x34294AfABCbaFfc616ac6614F6d2e17260b78BEd",
    "symbol": "ABOND",
    "name": "ApeBond",
    "decimals": 18,
    "logoURI": "ipfs://QmYfNLqbHYpe6rPE285f3XLNhYrqWvVpwv9CD4eqYto9pu"
  },
  {
    "chainId": 1,
    "address": "0xfE33ae95A9f0DA8A845aF33516EDc240DCD711d6",
    "symbol": "sDASH",
    "name": "Synthetic Dash",
    "decimals": 18,
    "logoURI": "ipfs://QmdgTFhcBZKmNUX4Ru9Q1fZzgD991qen935MCcSfoJTzPB"
  },
  {
    "chainId": 1,
    "address": "0x8b3FF1ed4F36C2c2be675AFb13CC3AA5d73685a5",
    "symbol": "crCEL",
    "name": "Cream Celsius",
    "decimals": 8,
    "logoURI": "ipfs://QmNZ3cHiQstDKpf1gDDyWcx4SyKgKe54eH3hLexu7KGcN4"
  },
  {
    "chainId": 137,
    "address": "0x77f56cf9365955486B12C4816992388eE8606f0E",
    "symbol": "C98",
    "name": "Coin98",
    "decimals": 18,
    "logoURI": "ipfs://QmRQbHrMyudBLwQWkC1GHgyYBDzmgDGbEDWLZZ9ZqNL8dF"
  },
  {
    "chainId": 137,
    "address": "0xA1c57f48F0Deb89f569dFbE6E2B7f46D33606fD4",
    "symbol": "MANA",
    "name": "Decentraland",
    "decimals": 18,
    "logoURI": "ipfs://QmY18MUVBQbY22cM9gRxub76EqqdHMiNydGkfoR1fBarEo"
  },
  {
    "chainId": 56,
    "address": "0x43C934A845205F0b514417d757d7235B8f53f1B9",
    "symbol": "XLM",
    "name": "Stellar",
    "decimals": 18,
    "logoURI": "ipfs://QmXEkrYLhmVJCGJ9AhxQypF3eS4aUUQX3PTef31gmEfyJo"
  },
  {
    "chainId": 1,
    "address": "0x9556f8ee795D991fF371F547162D5efB2769425F",
    "symbol": "DMME",
    "name": "DMme",
    "decimals": 18,
    "logoURI": "ipfs://QmSau4kiADix48qCdBTWbufW5RvvasphX8P81NvwZhFTJX"
  },
  {
    "chainId": 1,
    "address": "0x5abFd418AdB35e89c68313574eB16BdfFc15e607",
    "symbol": "TMV",
    "name": "Timvi",
    "decimals": 18,
    "logoURI": "ipfs://QmRLx2pp9G2HCvjQS5f2Wp34fzX2Py6QiEPCCVC4JW32eG"
  },
  {
    "chainId": 56,
    "address": "0xcF6BB5389c92Bdda8a3747Ddb454cB7a64626C63",
    "symbol": "XVS",
    "name": "Venus",
    "decimals": 18,
    "logoURI": "ipfs://QmXTx1VXDTzAavzk6TV81XW61iLMRaNx2DF2JTdsJxXAzD"
  },
  {
    "chainId": 1,
    "address": "0x10FDBD1e48eE2fD9336a482D746138AE19e649Db",
    "symbol": "crFTT",
    "name": "Cream FTX Token",
    "decimals": 8,
    "logoURI": "ipfs://QmUAjxyf38antvrNBaeqBGuzwPZdAvhzmGR2Sp72jiuTAA"
  },
  {
    "chainId": 1,
    "address": "0xD82fd4D6D62f89A1E50b1db69AD19932314aa408",
    "symbol": "LIQ",
    "name": "Liquis",
    "decimals": 18,
    "logoURI": "ipfs://QmNy9DZPYkjBUUwXvYZT3gDuo1njYcbHSBVvdejy1UUjcX"
  },
  {
    "chainId": 1,
    "address": "0x3FD8f39A962eFDA04956981C31AB89FAB5FB8bC8",
    "symbol": "RTH",
    "name": "Rotharium",
    "decimals": 18,
    "logoURI": "ipfs://QmT57bje4H2QJwdtEwkhURRimnsRHneh4939PqLbaws9G4"
  },
  {
    "chainId": 1,
    "address": "0x3893b9422Cd5D70a81eDeFfe3d5A1c6A978310BB",
    "symbol": "MITH",
    "name": "Mithril",
    "decimals": 18,
    "logoURI": "ipfs://QmRsyP8dkfdcVBQ1Qp27gbuo3YZxvFTkw7bi3oCAGWFk1z"
  },
  {
    "chainId": 1,
    "address": "0x431ad2ff6a9C365805eBaD47Ee021148d6f7DBe0",
    "symbol": "DF",
    "name": "dForce",
    "decimals": 18,
    "logoURI": "ipfs://QmNZqSfgPn2B3VhrefqLYGtDzmyLy1TnrbTarGwyPcBig3"
  },
  {
    "chainId": 100,
    "address": "0xCa8d20f3e0144a72C6B5d576e9Bd3Fd8557E2B04",
    "symbol": "WBNB",
    "name": "Wrapped BNB",
    "decimals": 18,
    "logoURI": "ipfs://QmSqnCR4xndFFVeUrUSdVmV7zmysYgcr1oGRAjmJmBrtJd"
  },
  {
    "chainId": 1,
    "address": "0x99ea4dB9EE77ACD40B119BD1dC4E33e1C070b80d",
    "symbol": "QSP",
    "name": "Quantstamp",
    "decimals": 18,
    "logoURI": "ipfs://QmdLczacUT3w5RG5KnFKDg53U5Gp4Wp2TuaqtvN4JJYE7w"
  },
  {
    "chainId": 1,
    "address": "0x0E3cC2c4FB9252d17d07C67135E48536071735D9",
    "symbol": "ARTH",
    "name": "ARTH",
    "decimals": 18,
    "logoURI": "ipfs://QmRFpRmLYTyc8EGy6FrjgpVYRs4EHyEh46CtGXGHiyyewZ"
  },
  {
    "chainId": 56,
    "address": "0x301AF3Eff0c904Dc5DdD06FAa808f653474F7FcC",
    "symbol": "UNB",
    "name": "Unbound",
    "decimals": 18,
    "logoURI": "ipfs://QmTi7Gkc4Yex5NG2sDUR59TZX62XJSugvY8ibeYX4qmETn"
  },
  {
    "chainId": 1,
    "address": "0x15b7c0c907e4C6b9AdaAaabC300C08991D6CEA05",
    "symbol": "GEL",
    "name": "Gelato",
    "decimals": 18,
    "logoURI": "ipfs://QmQe8BUGdw7YRmwVHYYQ1VeQGpCPj4twgCsv22jK23wQmB"
  },
  {
    "chainId": 1,
    "address": "0x2008e3057BD734e10AD13c9EAe45Ff132aBc1722",
    "symbol": "ZCO",
    "name": "Zebi",
    "decimals": 8,
    "logoURI": "ipfs://QmVSTWZDULQ1ZYzEkXxEGFRgH5DGyhaHyvDzJt5HDd5xCj"
  },
  {
    "chainId": 1,
    "address": "0x22602469d704BfFb0936c7A7cfcD18f7aA269375",
    "symbol": "sETC",
    "name": "Synthetic Ethereum Classic",
    "decimals": 18,
    "logoURI": "ipfs://QmTU7BWvndLxAWqb4rP6zrX448MeNiieihaty52ndNNgQb"
  },
  {
    "chainId": 1,
    "address": "0x0c7D5ae016f806603CB1782bEa29AC69471CAb9c",
    "symbol": "BFC",
    "name": "Bifrost",
    "decimals": 18,
    "logoURI": "ipfs://QmcbEMgcCc3RaW5sronwftdoaNV9yoeAhXeEeJKiLXMipp"
  },
  {
    "chainId": 1,
    "address": "0x419D0d8BdD9aF5e606Ae2232ed285Aff190E711b",
    "symbol": "FUN",
    "name": "FunFair",
    "decimals": 8,
    "logoURI": "ipfs://QmUqA14PWyqWcnu3m2Y1Be4xnP9TCYrZ5ieR2G5eNbbfB1"
  },
  {
    "chainId": 1,
    "address": "0xD9Ec3ff1f8be459Bb9369b4E79e9Ebcf7141C093",
    "symbol": "KAI",
    "name": "KardiaChain",
    "decimals": 18,
    "logoURI": "ipfs://QmbtaRhGo5uPQQwzEKHG7mYwk1aU8R7GeqddqagyNRXqyn"
  },
  {
    "chainId": 1,
    "address": "0xdA1E53E088023Fe4D1DC5a418581748f52CBd1b8",
    "symbol": "AIDI",
    "name": "Aidi Inu",
    "decimals": 9,
    "logoURI": "ipfs://QmXS8D6QgX3fuvo72uCG61rRcBnKgsyonQ7K9hQ1NVeaMu"
  },
  {
    "chainId": 1,
    "address": "0x0417912b3a7AF768051765040A55BB0925D4DDcF",
    "symbol": "LID",
    "name": "Liquidity Dividends Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmRXTo4wPvygFwT49AC6vj1YVKqkd5JLqcZWKoz711p77o"
  },
  {
    "chainId": 1,
    "address": "0x9F284E1337A815fe77D2Ff4aE46544645B20c5ff",
    "symbol": "KTON",
    "name": "Darwinia Commitment Token",
    "decimals": 18,
    "logoURI": "ipfs://QmPND3auCEeiueSoqZ5agoB2QyTphnUDqswS1xffXL65z3"
  },
  {
    "chainId": 1,
    "address": "0x1c4b7d0e1885bd7667Af3378E0c538F74E712006",
    "symbol": "IOG",
    "name": "Playgroundz",
    "decimals": 18,
    "logoURI": "ipfs://QmQJiSKhg9V7V9BtmHUV1VtoUUug8RhMzuujB5g7Wf3TGD"
  },
  {
    "chainId": 1,
    "address": "0xB1191F691A355b43542Bea9B8847bc73e7Abb137",
    "symbol": "KIRO",
    "name": "Kirobo",
    "decimals": 18,
    "logoURI": "ipfs://QmVQFhHdAeLWQWBsF7X1e7iN6KpNaAPS6TBqbKFAQbRkBZ"
  },
  {
    "chainId": 1,
    "address": "0x6c936D4AE98E6d2172dB18c16C4b601C99918EE6",
    "symbol": "LIFE",
    "name": "Life Crypto",
    "decimals": 18,
    "logoURI": "ipfs://QmRWZpVkNZQxj5W6pCWayN6goVZn51K3fA3XgfREjhe58g"
  },
  {
    "chainId": 1,
    "address": "0xEd91879919B71bB6905f23af0A68d231EcF87b14",
    "symbol": "DMG",
    "name": "DMM: Governance",
    "decimals": 18,
    "logoURI": "ipfs://QmSdSiwF4vCRBbqeXmvcsiU2fxknQHLmM56tbh2mA4Bpnd"
  },
  {
    "chainId": 1,
    "address": "0x537edD52ebcb9F48ff2f8a28c51FCdB9D6a6E0D4",
    "symbol": "SDOG",
    "name": "Small Doge",
    "decimals": 18,
    "logoURI": "ipfs://Qma3UvxaNDGniJsWqKDx7RHijuQquRvj3yLwH3pKQ1GNo8"
  },
  {
    "chainId": 42161,
    "address": "0xA58663FAEF461761e44066ea26c1FCddF2927B80",
    "symbol": "KOM",
    "name": "Kommunitas",
    "decimals": 8,
    "logoURI": "ipfs://QmP7hhJjTh4iwnJAvYBUb2cizKjjk3GiCAjuEuYby5fkn9"
  },
  {
    "chainId": 1,
    "address": "0xd5F7838F5C461fefF7FE49ea5ebaF7728bB0ADfa",
    "symbol": "mETH",
    "name": "Mantle Staked Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmPzCHNB2PoDN8tGv3iTfZAAuxL8NASp9uHmpE1LTYBTRK"
  },
  {
    "chainId": 1,
    "address": "0xbCdfE338D55c061C084D81fD793Ded00A27F226D",
    "symbol": "DML",
    "name": "Decentralized Machine Learning",
    "decimals": 18,
    "logoURI": "ipfs://QmRWsgxw5awcMcGACbk136e1YEhqCud7HqfQhndgZWT4Ts"
  },
  {
    "chainId": 1,
    "address": "0xc3589F56B6869824804A5EA29F2c9886Af1B0FcE",
    "symbol": "HNY",
    "name": "Honey",
    "decimals": 18,
    "logoURI": "ipfs://QmYxsVgovmspp3BwuSUrRyicW2XwnVy1JbQnHkEdy4YsGH"
  },
  {
    "chainId": 56,
    "address": "0x23b8683Ff98F9E4781552DFE6f12Aa32814924e8",
    "symbol": "jEUR",
    "name": "Jarvis Synthetic Euro",
    "decimals": 18,
    "logoURI": "ipfs://QmPGP57ZrLr8KAD5Q9m9C8z5x7FzTEMuLo1hp8vcb4wfFx"
  },
  {
    "chainId": 10,
    "address": "0xFf733b2A3557a7ed6697007ab5D11B79FdD1b76B",
    "symbol": "ACX",
    "name": "Across Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmZ2HLSy4k7SzTxGrSX3AJiyY5CiwdV9MPVstk6N9HrhnX"
  },
  {
    "chainId": 56,
    "address": "0x154A9F9cbd3449AD22FDaE23044319D6eF2a1Fab",
    "symbol": "SKILL",
    "name": "CryptoBlades",
    "decimals": 18,
    "logoURI": "ipfs://QmfHKRLnY43AEfUCAivyMhrV9pv2maskCYu3zF8tanWEVH"
  },
  {
    "chainId": 1,
    "address": "0xfc82bb4ba86045Af6F327323a46E80412b91b27d",
    "symbol": "PROM",
    "name": "Prometeus",
    "decimals": 18,
    "logoURI": "ipfs://QmRVpirJhazUJkvo6jUNQLZoMavmJCKazECiNEX1cJjBg6"
  },
  {
    "chainId": 1,
    "address": "0x9A48BD0EC040ea4f1D3147C025cd4076A2e71e3e",
    "symbol": "USD++",
    "name": "PieDAO USD++",
    "decimals": 18,
    "logoURI": "ipfs://QmbEyp1qWnxeTvDN5Ucdk2yTgKwuCjTgvRbyfzz71bKu1y"
  },
  {
    "chainId": 137,
    "address": "0xC3C7d422809852031b44ab29EEC9F1EfF2A58756",
    "symbol": "LDO",
    "name": "Lido DAO",
    "decimals": 18,
    "logoURI": "ipfs://QmTdQtLaPqXoXbHgbjVgYbXbbukvQAsNZtzq9bs14oVknY"
  },
  {
    "chainId": 1,
    "address": "0x676Ad1b33ae6423c6618C1AEcf53BAa29cf39EE5",
    "symbol": "SDT",
    "name": "Wrapped SDT",
    "decimals": 18,
    "logoURI": "ipfs://QmPJ1r1Ja9U2XcbKFRhExGksTr8xyA2oSeUzAwKk4e1NbE"
  },
  {
    "chainId": 137,
    "address": "0xeEeEEb57642040bE42185f49C52F7E9B38f8eeeE",
    "symbol": "ELK",
    "name": "Elk Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmYSmFMg8incix4Hzs85MPZzQJnb2W19zLTL3nXNEzY42m"
  },
  {
    "chainId": 1,
    "address": "0x4EcE5C5CfB9B960A49aAe739E15cDB6cfDcc5782",
    "symbol": "DBUY",
    "name": "Doont Buy",
    "decimals": 9,
    "logoURI": "ipfs://QmdjxhwAUmS3rc5MS2Xb4ok77K7DupzfR3JFtBXGMEBEZx"
  },
  {
    "chainId": 1,
    "address": "0x106238737b85b4220C7A7DdFD5c4cAF8597281fe",
    "symbol": "BURNX",
    "name": "BurnX",
    "decimals": 18,
    "logoURI": "ipfs://QmdaRvD2GZbYgu7PVKBR5CTKiQ1HwRhsXkEjTPSnXB1d1W"
  },
  {
    "chainId": 10,
    "address": "0xaf20f5f19698f1D19351028cd7103B63D30DE7d7",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 1,
    "address": "0x2d7aC061fc3db53c39fe1607fB8cec1B2C162B01",
    "symbol": "iLINK",
    "name": "Synthetic Inverse Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://QmVRE93cpXdFpEAtUMErSs52fzKq9gXrfDsweFEHDdC7TN"
  },
  {
    "chainId": 1,
    "address": "0xd42debE4eDc92Bd5a3FBb4243e1ecCf6d63A4A5d",
    "symbol": "C8",
    "name": "Carboneum",
    "decimals": 18,
    "logoURI": "ipfs://QmdZcrPWGJthiiNayHs2NiswQ6XettLy8Pwy15LrUutd5V"
  },
  {
    "chainId": 1,
    "address": "0xE5a3229CCb22b6484594973A03a3851dCd948756",
    "symbol": "RAE",
    "name": "Receive Access Ecosystem",
    "decimals": 18,
    "logoURI": "ipfs://QmUAuHjZ5ruQdKH2WuPy8CY3oQYe9iVRT34jxRFQwpYnus"
  },
  {
    "chainId": 56,
    "address": "0x5f0Da599BB2ccCfcf6Fdfd7D81743B6020864350",
    "symbol": "MKR",
    "name": "Maker",
    "decimals": 18,
    "logoURI": "ipfs://QmfAnPxYQrWrEdjzaSi9D1Cn5XRwpPBKYov6JSbvtfcy1j"
  },
  {
    "chainId": 1,
    "address": "0x46DceB1bB2c55531D12A4E769E8088F4aF64D410",
    "symbol": "XBX",
    "name": "XBX",
    "decimals": 18,
    "logoURI": "ipfs://QmeEw2Y3zpyqJh9zKb1xW4v4hH1dNCRUpyboULKgrAYTyC"
  },
  {
    "chainId": 1,
    "address": "0xe0E4839E0c7b2773c58764F9Ec3B9622d01A0428",
    "symbol": "ENCORE",
    "name": "EnCore",
    "decimals": 18,
    "logoURI": "ipfs://QmSoyhEXtjuzNXQ1ovf7q8Gdc6pKupaj53hXgnFYe5Ta5W"
  },
  {
    "chainId": 1,
    "address": "0x4Fb721eF3Bf99e0f2c193847afA296b9257d3C30",
    "symbol": "TOK",
    "name": "Tokenplace",
    "decimals": 8,
    "logoURI": "ipfs://QmPsuSmit2qKsc8x2ojTxVioVPWuK3cehrkqsMPJDSPrqi"
  },
  {
    "chainId": 100,
    "address": "0xD589F00Fa2Eb83367f732AB3CDA92eE0940389cF",
    "symbol": "FUSE",
    "name": "Fuse",
    "decimals": 18,
    "logoURI": "ipfs://QmQkbt5kPE1q2362xZKErG1onw9RDfapPrdiuuPNymGci2"
  },
  {
    "chainId": 137,
    "address": "0x97C415D4c8ea982aaabCBb65506D37FA6631C351",
    "symbol": "UNCL",
    "name": "UNCL",
    "decimals": 18,
    "logoURI": "ipfs://QmdWnNAzfwfmZX64H3Nt2DatWNkA3eJcKcPwYCykwZrWav"
  },
  {
    "chainId": 1,
    "address": "0x5979F50f1D4c08f9A53863C2f39A7B0492C38d0f",
    "symbol": "pLTC",
    "name": "pTokens LTC",
    "decimals": 18,
    "logoURI": "ipfs://QmcDfKGb6tZpKL5aejE5pdKMzW2txAJJqQKYwsX9j3wUfu"
  },
  {
    "chainId": 10,
    "address": "0x8B21e9b7dAF2c4325bf3D18c1BeB79A347fE902A",
    "symbol": "COLLAB",
    "name": "Collab.Land",
    "decimals": 18,
    "logoURI": "ipfs://QmUmjee4H8CdtkHJrCXGCLXiLCoasYcs7fJB6hr75Wz2LX"
  },
  {
    "chainId": 8453,
    "address": "0xbFd5206962267c7b4b4A8B3D76AC2E1b2A5c4d5e",
    "symbol": "OSAK",
    "name": "Osaka Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmUng1scXVJSCH1MpqK4RanCsW1aXYew8KUZphtj3WUpah"
  },
  {
    "chainId": 1,
    "address": "0x33B919F54692dDbf702065763EA2b50Ca02e6bfF",
    "symbol": "MCW",
    "name": "Mocrow",
    "decimals": 18,
    "logoURI": "ipfs://QmSSh93o6AY5XKUGvi7sFJLWT7wYagqdv3sFzjPzonkJdy"
  },
  {
    "chainId": 56,
    "address": "0x3Cd55356433C89E50DC51aB07EE0fa0A95623D53",
    "symbol": "sfrxETH",
    "name": "Staked Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmX1r2JVToPDMkRUFsERHeeCheTXFZ9Hw1nuq1gDNjufxG"
  },
  {
    "chainId": 137,
    "address": "0x0621d647cecbFb64b79E44302c1933cB4f27054d",
    "symbol": "AMP",
    "name": "Amp",
    "decimals": 18,
    "logoURI": "ipfs://QmQjvLscdUTvwZQ5Qmka1g6oNjBPwjEf3igNn4a1VL54iN"
  },
  {
    "chainId": 1,
    "address": "0xDf6Ef343350780BF8C3410BF062e0C015B1DD671",
    "symbol": "BMC",
    "name": "Blackmoon",
    "decimals": 8,
    "logoURI": "ipfs://QmYJyC17923N7xVwJbXzAfNzMtoYvZDBXMwCLp7UFGbkSB"
  },
  {
    "chainId": 1,
    "address": "0xDe5eA375FFBDc8b25a80fe13D631E8Ba0AB4BB02",
    "symbol": "GERA",
    "name": "Gera Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmWRYiu9u1NG2FM63d3UQT3DK7bq9eQUcn2fJUxprGtxny"
  },
  {
    "chainId": 1,
    "address": "0x85F17Cf997934a597031b2E18a9aB6ebD4B9f6a4",
    "symbol": "NEAR",
    "name": "NEAR Protocol",
    "decimals": 24,
    "logoURI": "ipfs://QmVPvVH7de6BnNxFkhzjmdCZxVME1StXViP26SMcxjad1d"
  },
  {
    "chainId": 250,
    "address": "0x82f0B8B456c1A451378467398982d4834b6829c1",
    "symbol": "MIM",
    "name": "Magic Internet Money",
    "decimals": 18,
    "logoURI": "ipfs://QmaNmAVohHYtCFn8H9Lk393KDyj1rhWLUamGggFXM2eGLT"
  },
  {
    "chainId": 1,
    "address": "0x6f40d4A6237C257fff2dB00FA0510DeEECd303eb",
    "symbol": "INST",
    "name": "Instadapp",
    "decimals": 18,
    "logoURI": "ipfs://QmVvPHqUg4p2axX5ggQpyVa1V39EefHcM4XaVNQxtjRTjF"
  },
  {
    "chainId": 42161,
    "address": "0x153516904bc7E28aE7C526E8aEe4EC5EaE878eDB",
    "symbol": "PLX",
    "name": "PLEXUS",
    "decimals": 18,
    "logoURI": "ipfs://QmeCaJ3m2PmULVq5wbX6mF2PiSfx69873NAXbdGMzSVnxk"
  },
  {
    "chainId": 100,
    "address": "0x37b60f4E9A31A64cCc0024dce7D0fD07eAA0F7B3",
    "symbol": "PNK",
    "name": "Pinakion",
    "decimals": 18,
    "logoURI": "ipfs://QmbC6rKFGa6xTn7QEXDiDtoCi69fon8nvjtzMoXRLD4uvv"
  },
  {
    "chainId": 100,
    "address": "0xe91D153E0b41518A2Ce8Dd3D7944Fa863463a97d",
    "symbol": "WXDAI",
    "name": "Wrapped XDAI",
    "decimals": 18,
    "logoURI": "ipfs://QmUQfPdRURXsHDfW5gpqJuz5LKCrwszK6v4eLRJT4myF9x"
  },
  {
    "chainId": 1,
    "address": "0xDe7D85157d9714EADf595045CC12Ca4A5f3E2aDb",
    "symbol": "STPT",
    "name": "STP",
    "decimals": 18,
    "logoURI": "ipfs://Qmf4CK7Ug95EWnACFBr7n9dkxL2WxRhuJk5A5LobxrevQ3"
  },
  {
    "chainId": 56,
    "address": "0x8e17ed70334C87eCE574C9d537BC153d8609e2a3",
    "symbol": "WRX",
    "name": "WazirX",
    "decimals": 8,
    "logoURI": "ipfs://QmepHRW6R35R8ZkNB9yEXowXZ2KFPZCcQsFKUh9pXajUeV"
  },
  {
    "chainId": 137,
    "address": "0xBFA35599c7AEbb0dAcE9b5aa3ca5f2a79624D8Eb",
    "symbol": "RETRO",
    "name": "Retro Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmfAVNA35HwGvwJXDYATorBoKBPCiHzkQxCenHNpfJPmQz"
  },
  {
    "chainId": 1,
    "address": "0x582d872A1B094FC48F5DE31D3B73F2D9bE47def1",
    "symbol": "TONCOIN",
    "name": "Wrapped TON Coin",
    "decimals": 9,
    "logoURI": "ipfs://QmQDnCpnW6QHWj6NVd3br8KpHsoAwxfQthC6EFXCCJ7Tsy"
  },
  {
    "chainId": 1,
    "address": "0x4a22A69E45AB29F9F7276B0267797474DAf1F27c",
    "symbol": "SUNI",
    "name": "SUNI",
    "decimals": 18,
    "logoURI": "ipfs://QmWsDUv5dEorjGoJxtF4DerYcLvyhfmXf6Q8wasxhjLwQb"
  },
  {
    "chainId": 137,
    "address": "0x4e3Decbb3645551B8A19f0eA1678079FCB33fB4c",
    "symbol": "jEUR",
    "name": "Jarvis Synthetic Euro",
    "decimals": 18,
    "logoURI": "ipfs://QmPGP57ZrLr8KAD5Q9m9C8z5x7FzTEMuLo1hp8vcb4wfFx"
  },
  {
    "chainId": 1,
    "address": "0xBD05CeE8741100010D8E93048a80Ed77645ac7bf",
    "symbol": "CYTR",
    "name": "Cyclops Treasure",
    "decimals": 18,
    "logoURI": "ipfs://QmW2Piu5MWaZtUNyc9cdmxfQ7ep3TGLAHLhDPy8oH8x1ZX"
  },
  {
    "chainId": 1,
    "address": "0xD0D6D6C5Fe4a677D343cC433536BB717bAe167dD",
    "symbol": "ADT",
    "name": "adToken",
    "decimals": 9,
    "logoURI": "ipfs://QmdG1K1AMwXznQh9yWddFe6BbEjKsDawmTJ9Zat9ekX734"
  },
  {
    "chainId": 1,
    "address": "0xEA097A2b1dB00627B2Fa17460Ad260c016016977",
    "symbol": "UFR",
    "name": "Upfiring",
    "decimals": 18,
    "logoURI": "ipfs://QmW9qyrWnDJFT5XL26YBgNcjY7zLSmEMyD58UBVKcsB4je"
  },
  {
    "chainId": 1,
    "address": "0x0E8d6b471e332F140e7d9dbB99E5E3822F728DA6",
    "symbol": "ABYSS",
    "name": "Abyss",
    "decimals": 18,
    "logoURI": "ipfs://QmePf7N7HwqmKT8aYZMph6s9bncxKj3R2TQ16cjEo8LwvZ"
  },
  {
    "chainId": 1,
    "address": "0x6C4522F0035bED2180B40f4c5d9DbAab64B41325",
    "symbol": "PASS",
    "name": "Passport Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmXnqCL23UcV7KpQvXQcAQsggJSqkx3uKjog4U99hy5u3c"
  },
  {
    "chainId": 42161,
    "address": "0x1A5B0aaF478bf1FDA7b934c76E7692D722982a6D",
    "symbol": "BFR",
    "name": "Buffer Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmVAqqNBrDGskVvVs1Ueqi41UdreSff4VtC1s9rcNSS2bW"
  },
  {
    "chainId": 1,
    "address": "0x8400D94A5cb0fa0D041a3788e395285d61c9ee5e",
    "symbol": "UBT",
    "name": "Unibright",
    "decimals": 8,
    "logoURI": "ipfs://QmQgC8EoYL3yvvM1MjEFncecJkAN3Zs7Sx4Qafc76m4NzZ"
  },
  {
    "chainId": 1,
    "address": "0x630bC7dD0aBfc2d196289cE09Db947DD2CAfae7C",
    "symbol": "NABOX",
    "name": "Nabox",
    "decimals": 18,
    "logoURI": "ipfs://QmbNNeZg8NpdvoMYcThnsat8qhfgZJhXMgbFoQBpXSSZjT"
  },
  {
    "chainId": 1,
    "address": "0x217ddEad61a42369A266F1Fb754EB5d3EBadc88a",
    "symbol": "DON",
    "name": "Don-key",
    "decimals": 18,
    "logoURI": "ipfs://QmUmNBTEk3Cn8WgqXvnkxBXJdiNsNiPGb7MS65TCqiHssk"
  },
  {
    "chainId": 1,
    "address": "0xDa007777D86AC6d989cC9f79A73261b3fC5e0DA0",
    "symbol": "NODE",
    "name": "DAppNode",
    "decimals": 18,
    "logoURI": "ipfs://QmdVxDsdvm5MkgxJckmKuntxFx8umGQ2nV4t9qBbegvyqV"
  },
  {
    "chainId": 56,
    "address": "0x52F24a5e03aee338Da5fd9Df68D2b6FAe1178827",
    "symbol": "ankrBNB",
    "name": "Ankr Staked BNB",
    "decimals": 18,
    "logoURI": "ipfs://QmQs3FW18L4UqumZBjWh81M9svXcqkq8x7DmxUeELjC2GX"
  },
  {
    "chainId": 1,
    "address": "0xdC386452F9FFDa7F0d2940e5c60Dc0F9469b1097",
    "symbol": "xKAWA",
    "name": "xKAWA",
    "decimals": 18,
    "logoURI": "ipfs://QmaziqAwXh5fkyonMZdHNDky81xhJDaVSjz8zgHkijdhGr"
  },
  {
    "chainId": 42161,
    "address": "0xDA10009cBd5D07dd0CeCc66161FC93D7c9000da1",
    "symbol": "DAI",
    "name": "Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmWcx6mu7PtPNM6eiUHwLwGuPsjXLoXiQ8ayBYteGJ7WFf"
  },
  {
    "chainId": 1,
    "address": "0x3d1BA9be9f66B8ee101911bC36D3fB562eaC2244",
    "symbol": "RVT",
    "name": "Rivetz",
    "decimals": 18,
    "logoURI": "ipfs://QmUjEG6pFRnv6y5BtCGyyKM4GbQqCvfZ9wFEENRfX4YSGT"
  },
  {
    "chainId": 8453,
    "address": "0x833589fCD6eDb6E08f4c7C32D4f71b54bdA02913",
    "symbol": "USDC",
    "name": "USD Coin",
    "decimals": 6,
    "logoURI": "ipfs://QmWbGfqePP3uAvPNHdYSP69aDznHGknsypTPRaiJT9NZF8"
  },
  {
    "chainId": 137,
    "address": "0x2C89bbc92BD86F8075d1DEcc58C7F4E0107f286b",
    "symbol": "AVAX",
    "name": "Avalanche",
    "decimals": 18,
    "logoURI": "ipfs://QmeJKVZCwbKeB6zzDanjcYRLLeaJe2rDY4f5ij5Uak7EC8"
  },
  {
    "chainId": 1,
    "address": "0x6e1A19F235bE7ED8E3369eF73b196C07257494DE",
    "symbol": "WFIL",
    "name": "Wrapped Filecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmbKfADJcTcCSGDh7sz5FZnuUnJQiSKhYPjaB2QXz4yGXN"
  },
  {
    "chainId": 1,
    "address": "0x10bA8C420e912bF07BEdaC03Aa6908720db04e0c",
    "symbol": "RAISE",
    "name": "Raise",
    "decimals": 18,
    "logoURI": "ipfs://QmSGmkJiNexqJqGP55ijfDNmxAnXJMFcB2f4QtMw9W3DCG"
  },
  {
    "chainId": 1,
    "address": "0xfFED56a180f23fD32Bc6A1d8d3c09c283aB594A8",
    "symbol": "FL",
    "name": "Freeliquid",
    "decimals": 18,
    "logoURI": "ipfs://QmenQLmVNs5aGYYBuVJMqKwgA7MpTGzNsdaKesMMKBsUf5"
  },
  {
    "chainId": 137,
    "address": "0x2B9E7ccDF0F4e5B24757c1E1a80e311E34Cb10c7",
    "symbol": "MASK",
    "name": "Mask Network",
    "decimals": 18,
    "logoURI": "ipfs://QmRoLnh8xZA9Aw8ddiA2E22cwqyq6ZSxvKyH25wC8X87kL"
  },
  {
    "chainId": 1,
    "address": "0x8888889213DD4dA823EbDD1e235b09590633C150",
    "symbol": "MBC",
    "name": "Marblecoin",
    "decimals": 18,
    "logoURI": "ipfs://QmTSRzr81casaxv5Q8oNNuC5sw6x4NWPPiQfNJEYPpmcFN"
  },
  {
    "chainId": 1,
    "address": "0x55d1d16fB42fCE47B899010c996a3a31f6db8fD6",
    "symbol": "BINU",
    "name": "Bully Inu",
    "decimals": 18,
    "logoURI": "ipfs://Qmbk9WQAb6JJysy5GpuYGW5bo2Py8Rd1jjz5aVEfdX8ZCb"
  },
  {
    "chainId": 1,
    "address": "0x423b5F62b328D0D6D44870F4Eee316befA0b2dF5",
    "symbol": "GOT",
    "name": "GoNetwork",
    "decimals": 18,
    "logoURI": "ipfs://QmPZVpCyS8NVoPR33vKsJ9HkmbSRZPpAqHAsKbWLZxDBEU"
  },
  {
    "chainId": 1,
    "address": "0x956F47F50A910163D8BF957Cf5846D573E7f87CA",
    "symbol": "FEI",
    "name": "Fei Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmcwNoQdrMEbixYhqJzE6U97qwoKkX4iGJFtnEb4f2yeoQ"
  },
  {
    "chainId": 1,
    "address": "0xA5a5DF41883Cdc00c4cCC6E8097130535399d9a3",
    "symbol": "iOIL",
    "name": "Synthetic Inverse Perpetual Oil Futures",
    "decimals": 18,
    "logoURI": "ipfs://Qmdu2y6AzNv2KHTPLjHFdq4Wqewtnu5DwjUG7YfJ6eh45G"
  },
  {
    "chainId": 1,
    "address": "0xAcfa209Fb73bF3Dd5bBfb1101B9Bc999C49062a5",
    "symbol": "BCDT",
    "name": "Blockchain Certified Data Token",
    "decimals": 18,
    "logoURI": "ipfs://QmYA56kw69GeMxHfBwkhdmMKr1XT9awE9GSucPk4BoLnTd"
  },
  {
    "chainId": 1,
    "address": "0xb052F8A33D8bb068414EaDE06AF6955199f9f010",
    "symbol": "ECOREAL",
    "name": "Ecoreal Estate ",
    "decimals": 18,
    "logoURI": "ipfs://QmV1jBDVNLMouwAdike2xke55JzQXYpz1YdS21Xfswgepw"
  },
  {
    "chainId": 1,
    "address": "0x33EbD390d986EB21Fb91a5B72Ee5AD8a1F40e43e",
    "symbol": "CLIFFC",
    "name": "Clifford Classic",
    "decimals": 18,
    "logoURI": "ipfs://QmdRr5YRjqCZwD24EhpFhKKukLkG89C97BG1VMb1jUxWoJ"
  },
  {
    "chainId": 1,
    "address": "0x8689D850CdF3b74A1F6A5eB60302c785B71c2fc7",
    "symbol": "CPHX",
    "name": "Crypto Phoenix",
    "decimals": 18,
    "logoURI": "ipfs://QmPL4Dk4eZeAANhmKSXRnp9Z7nCCuB7TZa2jYmd8G2zgQK"
  },
  {
    "chainId": 1,
    "address": "0xB2279B6769CFBa691416F00609b16244c0cF4b20",
    "symbol": "WAIF",
    "name": "Waifu Token",
    "decimals": 18,
    "logoURI": "ipfs://QmX4z2Qdou1h6MFASVkeShYu2UhaGwafqMs4aXbF9Us27R"
  },
  {
    "chainId": 1,
    "address": "0xe76C6c83af64e4C60245D8C7dE953DF673a7A33D",
    "symbol": "RAIL",
    "name": "RAILGUN",
    "decimals": 18,
    "logoURI": "ipfs://QmQHocvkazBM3eGiStPMeceYANdZJx9fjtvXnGrpX8fsm3"
  },
  {
    "chainId": 1,
    "address": "0xA35923162C49cF95e6BF26623385eb431ad920D3",
    "symbol": "TURBO",
    "name": "Turbo",
    "decimals": 18,
    "logoURI": "ipfs://QmP4BNUEp7jb5txpJbvjBec2y5tGrDNYBkVLYYWWZ6EHu5"
  },
  {
    "chainId": 1,
    "address": "0x32E4c68B3A4a813b710595AebA7f6B7604Ab9c15",
    "symbol": "iUSDC",
    "name": "Fulcrum USDC iToken",
    "decimals": 6,
    "logoURI": "ipfs://QmXaGXjRpFhf2NgoCwzYp4fWFoN8hY2hmUnCUD9rKytBYM"
  },
  {
    "chainId": 137,
    "address": "0xAdA58DF0F643D959C2A47c9D4d4c1a4deFe3F11C",
    "symbol": "CRO",
    "name": "Cronos",
    "decimals": 8,
    "logoURI": "ipfs://QmQCyvKSpQRcokX61YQKW1tE7aX1cZV9QS8EAuFJEVtTFV"
  },
  {
    "chainId": 1,
    "address": "0xad32A8e6220741182940c5aBF610bDE99E737b2D",
    "symbol": "DOUGH",
    "name": "PieDAO DOUGH v2",
    "decimals": 18,
    "logoURI": "ipfs://QmY3pJEgtRk8yNYjP8eic1ZbJZ6HVULQbPxhBbkb8hD86j"
  },
  {
    "chainId": 1,
    "address": "0xf34960d9d60be18cC1D5Afc1A6F012A723a28811",
    "symbol": "KCS",
    "name": "KuCoin Token",
    "decimals": 6,
    "logoURI": "ipfs://QmYPgbZNK5r6DrZ5KsJxgcinmrJSf6Bkb5wLDNvVdhUK9b"
  },
  {
    "chainId": 1,
    "address": "0x5CA9a71B1d01849C0a95490Cc00559717fCF0D1d",
    "symbol": "AE",
    "name": "Aeternity",
    "decimals": 18,
    "logoURI": "ipfs://QmTwBRpQ3E5FPFaD81cXxxFqEmuq6vjQxj3RvKzwoHVr9B"
  },
  {
    "chainId": 1,
    "address": "0x9F8eef61b1Ad834B44C089DBF33eb854746a6bf9",
    "symbol": "DAWGS",
    "name": "SpaceDawgs",
    "decimals": 9,
    "logoURI": "ipfs://QmYceUUW97Tzba78gT9BKpj8YoU3WAFSgHEpm5H4m7yGKm"
  },
  {
    "chainId": 1,
    "address": "0x0f8C793DbC30A185138d5Da00f106290B26E36C5",
    "symbol": "ASTAX",
    "name": "Ape Stax",
    "decimals": 9,
    "logoURI": "ipfs://QmXg8uXagNzEUPX6o7yCgFZAcZGfiJeavBDK9z9dygYsmk"
  },
  {
    "chainId": 1,
    "address": "0xEec2bE5c91ae7f8a338e1e5f3b5DE49d07AfdC81",
    "symbol": "DPX",
    "name": "Dopex",
    "decimals": 18,
    "logoURI": "ipfs://QmWYUf3cdbRyZytf3qvCg7aMrb6Mj8co2CNwQymeqfQicn"
  },
  {
    "chainId": 1,
    "address": "0xB7B1570e26315BAAD369b8EA0a943b7F140DB9eB",
    "symbol": "DPS",
    "name": "DEEPSPACE",
    "decimals": 9,
    "logoURI": "ipfs://QmRLARb3VKqXLM1a96LHTNMt7vRtdFeNZjvVmi6e6i5NWt"
  },
  {
    "chainId": 1,
    "address": "0x6781a0F84c7E9e846DCb84A9a5bd49333067b104",
    "symbol": "ZAP",
    "name": "Zap",
    "decimals": 18,
    "logoURI": "ipfs://QmfGQAScapgZKAMxWL1Ge7WbCf76RoCBZePKScqcVjT8up"
  },
  {
    "chainId": 1,
    "address": "0xF68df6dF642E8387AfC9D03214B78f3087ef8a99",
    "symbol": "RELOADED",
    "name": "Doge Reloaded",
    "decimals": 9,
    "logoURI": "ipfs://QmVjGha8782bikX2PcWUzCA11nBgvNjNJjrjaxwvbhK5uT"
  },
  {
    "chainId": 1,
    "address": "0x92E187a03B6CD19CB6AF293ba17F2745Fd2357D5",
    "symbol": "DUCK",
    "name": "Unit Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmRyFHbLwrZXh7Yn5wCkfn2CuRzhtF2deQHL7HCiMjCnyd"
  },
  {
    "chainId": 1,
    "address": "0x037A54AaB062628C9Bbae1FDB1583c195585fe41",
    "symbol": "LCX",
    "name": "LCX",
    "decimals": 18,
    "logoURI": "ipfs://QmQUKTYwwp6woMuyRnqFSgg3jhyTaVKVxJAyfdNKn6Gix3"
  },
  {
    "chainId": 1,
    "address": "0x76851A93977bEA9264C32255b6457882035C7501",
    "symbol": "FOGE",
    "name": "Fat Doge",
    "decimals": 9,
    "logoURI": "ipfs://QmXNKDmgovNHRFxc1qeRvcSFaYZQ9faxpC72HWP659kXWy"
  },
  {
    "chainId": 56,
    "address": "0x4e6415a5727ea08aAE4580057187923aeC331227",
    "symbol": "FINE",
    "name": "Refinable",
    "decimals": 18,
    "logoURI": "ipfs://QmXxBkrfMZsYE39JeqF9LBkLKWurLTK8wryMuRFtWJ8Dzm"
  },
  {
    "chainId": 1,
    "address": "0xBa21Ef4c9f433Ede00badEFcC2754B8E74bd538A",
    "symbol": "SWFL",
    "name": "Swapfolio",
    "decimals": 18,
    "logoURI": "ipfs://Qmf1DVFojCK8wbLhaLJvesSPKJ9rpRfzxGPYbumcB1tuZn"
  },
  {
    "chainId": 1,
    "address": "0xbbFF862d906E348E9946Bfb2132ecB157Da3D4b4",
    "symbol": "SS",
    "name": "Sharder",
    "decimals": 18,
    "logoURI": "ipfs://QmUGCkmZi7frnZ9DyzKnKnPAAKYLijRQkD414Jdhy1FFBi"
  },
  {
    "chainId": 1,
    "address": "0xdab396cCF3d84Cf2D07C4454e10C8A6F5b008D2b",
    "symbol": "GFI",
    "name": "Goldfinch",
    "decimals": 18,
    "logoURI": "ipfs://QmdyUTPzy9FVDMPJzT64iiYgDUoP6UUuACKswXXzSHCyDA"
  },
  {
    "chainId": 8453,
    "address": "0x64b88c73A5DfA78D1713fE1b4c69a22d7E0faAa7",
    "symbol": "MAV",
    "name": "Maverick Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmYkDjrskzi8aUwXCMgy7LhQdUttqi3y9yzZjxG5Zx238t"
  },
  {
    "chainId": 56,
    "address": "0xCE7de646e7208a4Ef112cb6ed5038FA6cC6b12e3",
    "symbol": "TRX",
    "name": "TRON",
    "decimals": 18,
    "logoURI": "ipfs://QmejfwdibxibEpygxVAeqRVFkAecVSMJSR1yu4LXu2wtAm"
  },
  {
    "chainId": 1,
    "address": "0xa9fE4601811213c340e850ea305481afF02f5b28",
    "symbol": "yvWETH",
    "name": "WETH yVault",
    "decimals": 18,
    "logoURI": "ipfs://QmceTmZ7T6QXrVV6KEjEdrmyg3A21XrWtRPbQXv6TzxNqz"
  },
  {
    "chainId": 1,
    "address": "0xe36E2D3c7c34281FA3bC737950a68571736880A1",
    "symbol": "sADA",
    "name": "Synthetic Cardano",
    "decimals": 18,
    "logoURI": "ipfs://QmeSRea3pa5kJxaGvWVsf9u1nwSEQyY1Uv1WmmPh88oNLN"
  },
  {
    "chainId": 1,
    "address": "0xEA47B64e1BFCCb773A0420247C0aa0a3C1D2E5C5",
    "symbol": "BAYC",
    "name": "BAYC Vault",
    "decimals": 18,
    "logoURI": "ipfs://QmdSv6MebYPQ99p4ExtU2Mgq7bAEiBYaWndzattpFmLSwt"
  },
  {
    "chainId": 1,
    "address": "0x269616D549D7e8Eaa82DFb17028d0B212D11232A",
    "symbol": "PUNK",
    "name": "PUNK Vault",
    "decimals": 18,
    "logoURI": "ipfs://Qmb7xRSAvcZhURBnahLNNjPcsyyFEMer23NsCSdfnW2m7i"
  },
  {
    "chainId": 1,
    "address": "0x16B0a1a87ae8aF5C792faBC429C4FE248834842B",
    "symbol": "ALG",
    "name": "Algory",
    "decimals": 18,
    "logoURI": "ipfs://Qmb24huyb1UZiNhajqxrbVF9A7QnFC5MsBsUm9xRZaB4C8"
  },
  {
    "chainId": 42161,
    "address": "0xaf20f5f19698f1D19351028cd7103B63D30DE7d7",
    "symbol": "WAGMI",
    "name": "Wagmi",
    "decimals": 18,
    "logoURI": "ipfs://QmYF9ngLNPqFAyGUXiq4UMJxH96Bs4mccxd57mJ9B6ZPF5"
  },
  {
    "chainId": 56,
    "address": "0xD40bEDb44C081D2935eebA6eF5a3c8A31A1bBE13",
    "symbol": "HERO",
    "name": "Metahero",
    "decimals": 18,
    "logoURI": "ipfs://QmYoSCsjyJ6aqb3SQxETSkjZuP8oMt36rYTNw2s9KVrmzF"
  },
  {
    "chainId": 1,
    "address": "0x419c4dB4B9e25d6Db2AD9691ccb832C8D9fDA05E",
    "symbol": "DRGN",
    "name": "Dragonchain",
    "decimals": 18,
    "logoURI": "ipfs://QmRS6KSpNrmwatJo2yvQyVPEixxBbMmSdwJUpQfVqDUvN8"
  },
  {
    "chainId": 1,
    "address": "0x32b86b99441480a7E5BD3A26c124ec2373e3F015",
    "symbol": "BAD",
    "name": "BAD IDEA AI",
    "decimals": 18,
    "logoURI": "ipfs://QmbVTW11yZmY7wmKB2tHVB9rEfCy4dwvJs3DmF58FeXKpt"
  },
  {
    "chainId": 1,
    "address": "0x12e2b8033420270db2F3b328E32370Cb5B2Ca134",
    "symbol": "SFP",
    "name": "SafePal",
    "decimals": 18,
    "logoURI": "ipfs://QmVPts8pq7Wr3uehn66tmE8uHMMJ85pGhXjXVBdfRg12Ca"
  },
  {
    "chainId": 1,
    "address": "0x2494a68C1484376fEf880b4c24D91f049d29B02A",
    "symbol": "TTT",
    "name": "The Transfer Token",
    "decimals": 18,
    "logoURI": "ipfs://QmRvgoR4EimrLQxEMTBrrA2XBKL2dGuKZHHTK1EP5vQYgU"
  },
  {
    "chainId": 1,
    "address": "0x18084fbA666a33d37592fA2633fD49a74DD93a88",
    "symbol": "tBTC",
    "name": "tBTC",
    "decimals": 18,
    "logoURI": "ipfs://QmRMESJHxFZWSS5KuMj5fW7SqEdkVvEATYVWkRRk6y2VeX"
  },
  {
    "chainId": 1,
    "address": "0x37fE0f067FA808fFBDd12891C0858532CFE7361d",
    "symbol": "CIV",
    "name": "Civilization",
    "decimals": 18,
    "logoURI": "ipfs://QmS8yfu2hut7qEqu86nx9hi35epd2KPt35ELMkBVf8dFWS"
  },
  {
    "chainId": 1,
    "address": "0x14feE680690900BA0ccCfC76AD70Fd1b95D10e16",
    "symbol": "PAAL",
    "name": "PAAL AI",
    "decimals": 9,
    "logoURI": "ipfs://QmNjdmLc1q62EFNn1fSMLEBa3vbDvUaYzpLdLWcGNLsR3o"
  },
  {
    "chainId": 56,
    "address": "0x2B5D9ADea07B590b638FFc165792b2C610EdA649",
    "symbol": "CKP",
    "name": "Cakepie",
    "decimals": 18,
    "logoURI": "ipfs://QmSDQLp7HVG4asBdCHbRNMBvt5hbgby15VsKbCavuftSxX"
  },
  {
    "chainId": 1,
    "address": "0xC567bca531992352166252ea5121e535432E81eD",
    "symbol": "TAR",
    "name": "Tartarus",
    "decimals": 8,
    "logoURI": "ipfs://QmSJKZnXPLupCgtrwssgbkXSSAeJFbmX6dFeSoxLVD9D6V"
  },
  {
    "chainId": 1,
    "address": "0x6123B0049F904d730dB3C36a31167D9d4121fA6B",
    "symbol": "RBN",
    "name": "Ribbon Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmVePjvBD1j6STJBypANRZiuRFXURcmWoMtrRajuHXo9y3"
  },
  {
    "chainId": 1,
    "address": "0x6fB3e0A217407EFFf7Ca062D46c26E5d60a14d69",
    "symbol": "IOTX",
    "name": "IoTeX",
    "decimals": 18,
    "logoURI": "ipfs://QmbCems7dusffBC21hovSehgvVYGmZQvL6Vb46wSknuNVn"
  },
  {
    "chainId": 1,
    "address": "0xA2cd3D43c775978A96BdBf12d733D5A1ED94fb18",
    "symbol": "XCN",
    "name": "Onyxcoin",
    "decimals": 18,
    "logoURI": "ipfs://Qme5AUweKBvvTYQGVSwuEZdSXwggYqgADVrzAMxhWbXfio"
  },
  {
    "chainId": 1,
    "address": "0xf4d2888d29D722226FafA5d9B24F9164c092421E",
    "symbol": "LOOKS",
    "name": "LooksRare",
    "decimals": 18,
    "logoURI": "ipfs://QmRGoQJaybuVdF2vReK9EThK796ULyxVMvaACY7eDkFqAc"
  },
  {
    "chainId": 1,
    "address": "0x54a3017754BFba73F71F37d893A368814CbFf457",
    "symbol": "WWCN",
    "name": "Wrapped Widecoin",
    "decimals": 8,
    "logoURI": "ipfs://QmZdkurMfUgKWRG1nRp38wDF8B8ADKxNkniQG1MetKDhjj"
  },
  {
    "chainId": 1,
    "address": "0x0000000000b3F879cb30FE243b4Dfee438691c04",
    "symbol": "GST2",
    "name": "GasToken",
    "decimals": 2,
    "logoURI": "ipfs://QmSpdxTUBZxyg1J3yZjiKfE8HCZRLmGmqgkUEZCLgxjfbm"
  },
  {
    "chainId": 42161,
    "address": "0x178412e79c25968a32e89b11f63B33F733770c2A",
    "symbol": "frxETH",
    "name": "Frax Ether",
    "decimals": 18,
    "logoURI": "ipfs://QmQqQhtwApaT4X9FAaAzBRvyeE2iaSDH4ahD6exWXcszZU"
  },
  {
    "chainId": 43114,
    "address": "0xE5CAeF4Af8780E59Df925470b050Fb23C43CA68C",
    "symbol": "FRM",
    "name": "Ferrum Network",
    "decimals": 18,
    "logoURI": "ipfs://QmYBq6GWG5hFMn3xsNnyjY2kKJTwPsmVAAgBdhzwW34cNu"
  },
  {
    "chainId": 42161,
    "address": "0xDb298285FE4C5410B05390cA80e8Fbe9DE1F259B",
    "symbol": "FOREX",
    "name": "handleFOREX",
    "decimals": 18,
    "logoURI": "ipfs://Qmcyx9EoC9BhToyzZLSCW47jqmVxdw42F2LXGwCVzZHLUG"
  },
  {
    "chainId": 56,
    "address": "0xa2B726B1145A4773F68593CF171187d8EBe4d495",
    "symbol": "INJ",
    "name": "Injective",
    "decimals": 18,
    "logoURI": "ipfs://QmbHj8s17pzLhzMBb9azQu5Gjn4ufJT3iZMm7vCefgJhBn"
  },
  {
    "chainId": 1,
    "address": "0x8642A849D0dcb7a15a974794668ADcfbe4794B56",
    "symbol": "PROS",
    "name": "Prosper",
    "decimals": 18,
    "logoURI": "ipfs://QmewE3Kv1SjWchsXj6iRPBL2Nh5qKHvR6ivb4R3D3nWQo5"
  },
  {
    "chainId": 1,
    "address": "0x23348160D7f5aca21195dF2b70f28Fce2B0be9fC",
    "symbol": "sFTSE",
    "name": "Synthetic FTSE 100 Index",
    "decimals": 18,
    "logoURI": "ipfs://QmTGnrhN18nUuUij8gxo4WnGNy2HiPBBBBEYULT3kZhWjx"
  },
  {
    "chainId": 1,
    "address": "0x4Ba6dDd7b89ed838FEd25d208D4f644106E34279",
    "symbol": "VETH",
    "name": "Vether",
    "decimals": 18,
    "logoURI": "ipfs://QmSoBqGE3sa6vmK1tZXFguJ5LKg8sTGuWTVzKuRdxDuFt5"
  },
  {
    "chainId": 137,
    "address": "0x8f3Cf7ad23Cd3CaDbD9735AFf958023239c6A063",
    "symbol": "DAI",
    "name": "Dai",
    "decimals": 18,
    "logoURI": "ipfs://QmboEaSgdZWMar6bTtvs1kEJ2eoQwonTzayNs8ugpmKFwK"
  },
  {
    "chainId": 56,
    "address": "0xBe1a001FE942f96Eea22bA08783140B9Dcc09D28",
    "symbol": "BETA",
    "name": "Beta Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmVAbMc8uoUCjcm9WA3J4LdFuFVuK5ZtKdUuitHo5CxoQq"
  },
  {
    "chainId": 137,
    "address": "0x15b7c0c907e4C6b9AdaAaabC300C08991D6CEA05",
    "symbol": "GEL",
    "name": "Gelato",
    "decimals": 18,
    "logoURI": "ipfs://QmQe8BUGdw7YRmwVHYYQ1VeQGpCPj4twgCsv22jK23wQmB"
  },
  {
    "chainId": 1,
    "address": "0x5d4ABC77B8405aD177d8ac6682D584ecbFd46CEc",
    "symbol": "PST",
    "name": "Primas",
    "decimals": 18,
    "logoURI": "ipfs://Qmdky3mAhPeoyvs1i3orBoDf86aM9NJvV49GL9xfrhTX28"
  },
  {
    "chainId": 42161,
    "address": "0x18c11FD286C5EC11c3b683Caa813B77f5163A122",
    "symbol": "GNS",
    "name": "Gains Network",
    "decimals": 18,
    "logoURI": "ipfs://QmbGho8kZ8QDJMLXYvejvvGpnJnMXqhHn4QomceTy4zsVi"
  },
  {
    "chainId": 1,
    "address": "0x74e1698344eC7563EEE5bCB4D1eF894f9bA78e82",
    "symbol": "MCU",
    "name": "MemeCoin Universe",
    "decimals": 9,
    "logoURI": "ipfs://QmPshkjACUnqDAMheciGCuDdDC6u3tifWQBkJH8xNdyXmP"
  },
  {
    "chainId": 1,
    "address": "0xadd5E881984783dD432F80381Fb52F45B53f3e70",
    "symbol": "VITE",
    "name": "Vite",
    "decimals": 18,
    "logoURI": "ipfs://QmU1kD1sU9YJSotL4C8TvAeN2tokWX6sacwDvUwjKPwCrw"
  },
  {
    "chainId": 1,
    "address": "0xdBee1d8C452c781C17Ea20115CbaD0d5f627a680",
    "symbol": "sSXP",
    "name": "Strike SXP",
    "decimals": 8,
    "logoURI": "ipfs://QmceoCJCtNMASmSt3KYxY7XiPUQtq3dgrvbASBVxDW3Jk4"
  },
  {
    "chainId": 1,
    "address": "0x0352557B007A4Aae1511C114409b932F06F9E2f4",
    "symbol": "sRUNE",
    "name": "Synthetic THORChain",
    "decimals": 18,
    "logoURI": "ipfs://QmULP7aGKQB7j8QLoWipQaY4T53vEZzqv5yRKn7u7F8yu6"
  },
  {
    "chainId": 10,
    "address": "0xBC7B1Ff1c6989f006a1185318eD4E7b5796e66E1",
    "symbol": "PENDLE",
    "name": "Pendle",
    "decimals": 18,
    "logoURI": "ipfs://QmUgLTHTMw5TD3NmeH4L9VAcRH7EAevahe5AtP7vPVL4Gc"
  },
  {
    "chainId": 42161,
    "address": "0xf97f4df75117a78c1A5a0DBb814Af92458539FB4",
    "symbol": "LINK",
    "name": "Chainlink",
    "decimals": 18,
    "logoURI": "ipfs://Qmdn6E15MELheFNupKrMosD9KPDFZ9M3wVKkhCuacT1cVj"
  },
  {
    "chainId": 1,
    "address": "0x579CEa1889991f68aCc35Ff5c3dd0621fF29b0C9",
    "symbol": "IQ",
    "name": "IQ",
    "decimals": 18,
    "logoURI": "ipfs://QmZQFbKhmbBRXAzLuBJRGNKdWuow7Knpidcjj8vBtsKUz6"
  },
  {
    "chainId": 1,
    "address": "0x617aeCB6137B5108D1E7D4918e3725C8cEbdB848",
    "symbol": "sBNB",
    "name": "Synthetic Binance Coin",
    "decimals": 18,
    "logoURI": "ipfs://QmauvAXsTmRLTSAx6oFREEmkk387HwKjnBWu9MfQ8AJjXq"
  },
  {
    "chainId": 250,
    "address": "0xC17c30e98541188614dF99239cABD40280810cA3",
    "symbol": "RISE",
    "name": "EverRise",
    "decimals": 18,
    "logoURI": "ipfs://QmU6dKNHUySjmVh9FJFMT1cRZ54aoQ7fpJ4q2a6qFPHsbU"
  },
  {
    "chainId": 1,
    "address": "0x14d10003807AC60d07BB0ba82cAeaC8d2087c157",
    "symbol": "iDEFI",
    "name": "Synthetic Inverse DeFi Index",
    "decimals": 18,
    "logoURI": "ipfs://QmQGnnZPGWk1z9uQWL7yYbcAyHa4Yy8kVpFemFhsS26aKB"
  },
  {
    "chainId": 1,
    "address": "0x9e18d5BAB2FA94a6A95F509Ecb38F8F68322AbD3",
    "symbol": "OMIKAMI",
    "name": "Amaterasu Omikami",
    "decimals": 9,
    "logoURI": "ipfs://QmWNE7TyMTjACCsBFQ2iN6Vzunr1NqBSGZRBaiWKqwy11v"
  },
  {
    "chainId": 1,
    "address": "0xB4A3B0Faf0Ab53df58001804DdA5Bfc6a3D59008",
    "symbol": "SPA",
    "name": "Sperax",
    "decimals": 18,
    "logoURI": "ipfs://QmTZ5GpBAGfEZmSRVpeCRAEX8vQPHR6hnqXcWTmczqABdH"
  },
  {
    "chainId": 1,
    "address": "0x70401dFD142A16dC7031c56E862Fc88Cb9537Ce0",
    "symbol": "BIRD",
    "name": "Bird.Money",
    "decimals": 18,
    "logoURI": "ipfs://QmRVjZAVdgsFBSnXwv1P3p121Mqc6A1MNPWF4qAo5HNe41"
  },
  {
    "chainId": 1,
    "address": "0x1B19C19393e2d034D8Ff31ff34c81252FcBbee92",
    "symbol": "OUSG",
    "name": "OUSG",
    "decimals": 18,
    "logoURI": "ipfs://QmQ1HqtG2e5ceJtc3Cb7vrzvtkYGxZ9neSw8HTk1q6Mnbv"
  },
  {
    "chainId": 1,
    "address": "0x9358e3a79D428c7708da22a5bd085159F6818D12",
    "symbol": "TENSHI",
    "name": "Tenshi",
    "decimals": 9,
    "logoURI": "ipfs://QmZ21QN19rocGFxLywy6SaHyBw1Qko7D6ToZjLPCuzYjHp"
  },
  {
    "chainId": 1,
    "address": "0xb9E6BDcBa653b5f768dD2E2fAF6288df9054e554",
    "symbol": "DAH",
    "name": "Dirham",
    "decimals": 18,
    "logoURI": "ipfs://QmYi4eCEwvELnUNwFSM33JFNKhNg6EuiLAGyEv2Aa4QL4p"
  },
  {
    "chainId": 1,
    "address": "0x839e71613f9aA06E5701CF6de63E303616B0DDE3",
    "symbol": "VVS",
    "name": "VVS Finance",
    "decimals": 18,
    "logoURI": "ipfs://QmVvZiQuKRwHRP2iiQq7aWi6MaDkdccnX4d5KibyzMwSt5"
  },
  {
    "chainId": 1,
    "address": "0x8a854288a5976036A725879164Ca3e91d30c6A1B",
    "symbol": "GET",
    "name": "GET Protocol",
    "decimals": 18,
    "logoURI": "ipfs://QmVHERSsfvp44eLyJPZ8dDxLxAi3gjsVmgBujwSVxaCToZ"
  },
  {
    "chainId": 1,
    "address": "0xFF603F43946A3A28DF5E6A73172555D8C8b02386",
    "symbol": "RNT",
    "name": "OneRoot Network",
    "decimals": 18,
    "logoURI": "ipfs://QmZ73VGPURtajb8DeLHM4o9yCrHYsx7Y8QcDzczSuZvwGK"
  },
  {
    "chainId": 56,
    "address": "0x86aC3974e2BD0d60825230fa6F355fF11409df5c",
    "symbol": "vCAKE",
    "name": "Venus CAKE",
    "decimals": 8,
    "logoURI": "ipfs://QmZkkuku9oXKRSq2YT8DyZNMAR7dMr6eMQmCQXELppERkP"
  },
  {
    "chainId": 137,
    "address": "0xd0258a3fD00f38aa8090dfee343f10A9D4d30D3F",
    "symbol": "VOXEL",
    "name": "Voxies",
    "decimals": 18,
    "logoURI": "ipfs://Qmb2wg9BXjnuwohrWva4zN4a4ywWePr4uaFPV9HX5yEcmw"
  },
  {
    "chainId": 1,
    "address": "0x0cEC1A9154Ff802e7934Fc916Ed7Ca50bDE6844e",
    "symbol": "POOL",
    "name": "PoolTogether",
    "decimals": 18,
    "logoURI": "ipfs://Qmd1f4NaF27puWeZ87fgymsr8co79Ue2obzH9QumYtjzkn"
  },
  {
    "chainId": 56,
    "address": "0x7E35D0e9180bF3A1fc47b0d110bE7a21A10B41Fe",
    "symbol": "OVR",
    "name": "OVR",
    "decimals": 18,
    "logoURI": "ipfs://QmP4P8qym3QkNPqwzpNVEkjtoP1odz5PkkaBhR4Rrt9ArA"
  },
  {
    "chainId": 1,
    "address": "0x6Ba460AB75Cd2c56343b3517ffeBA60748654D26",
    "symbol": "UP",
    "name": "UpToken",
    "decimals": 8,
    "logoURI": "ipfs://QmVN2FipRy8iuXNQBPxx28ym98gA7xSdy6ZX5eaDDyWdjY"
  },
  {
    "chainId": 1,
    "address": "0x799ebfABE77a6E34311eeEe9825190B9ECe32824",
    "symbol": "BTRST",
    "name": "Braintrust",
    "decimals": 18,
    "logoURI": "ipfs://QmVmh9bdbj8XKgE76wgsDb3iLbVc6CMsBxTMmCZHSn2JHC"
  },
  {
    "chainId": 1,
    "address": "0xA9639160481b625ba43677bE753E0A70bf58c647",
    "symbol": "RBXS",
    "name": "RBXSamurai",
    "decimals": 18,
    "logoURI": "ipfs://QmcbhHDvZyE1MEpZxGSwspkpRr1SLzzwKp1s6Lvt4jKXEp"
  },
  {
    "chainId": 1,
    "address": "0x03042482d64577A7bdb282260e2eA4c8a89C064B",
    "symbol": "CNTR",
    "name": "Centaur",
    "decimals": 18,
    "logoURI": "ipfs://QmWXACyGhSBewgQe97PA9bv4qCiBBUPXuFp6sEBkN4ewuS"
  },
  {
    "chainId": 42161,
    "address": "0x51fC0f6660482Ea73330E414eFd7808811a57Fa2",
    "symbol": "PREMIA",
    "name": "Premia",
    "decimals": 18,
    "logoURI": "ipfs://Qmbef33x648zGE9v9rthiyz4w9jpGsRSsy39UFGBjhpgts"
  },
  {
    "chainId": 1,
    "address": "0x5B2e4a700dfBc560061e957edec8F6EeEb74a320",
    "symbol": "INS",
    "name": "Insolar",
    "decimals": 10,
    "logoURI": "ipfs://QmQ1dpmYqCqAQbskgwVcSvJkBUBpFwyJHfwPHZ2TvWC8wX"
  },
  {
    "chainId": 1,
    "address": "0x46a97629C9C1F58De6EC18C7F536e7E6d6A6ecDe",
    "symbol": "iDOT",
    "name": "Synthetic Inverse Polkadot",
    "decimals": 18,
    "logoURI": "ipfs://QmR65PEeabRdLWvLAGtmQrxWNf4mUQfGjEqrGRN4VxWPiF"
  },
  {
    "chainId": 1,
    "address": "0x1fC52f1ABade452Dd4674477D4711951700b3d27",
    "symbol": "NOKU",
    "name": "Noku",
    "decimals": 18,
    "logoURI": "ipfs://QmWJGd6BftkEPesNhr5mN4H7TutptNCb355EFMRbBEEbPe"
  },
  {
    "chainId": 1,
    "address": "0x7475C42f8BF2c19F4EAF12fEaabaBA859fdC8914",
    "symbol": "ACCEL",
    "name": "ACCEL",
    "decimals": 18,
    "logoURI": "ipfs://QmREBWcWdsN9yvaLL8oRkC7igE9UpVFAVJ3TGUFtAUMc3B"
  },
  {
    "chainId": 42161,
    "address": "0x9f6AbbF0Ba6B5bfa27f4deb6597CC6Ec20573FDA",
    "symbol": "FRM",
    "name": "Ferrum Network",
    "decimals": 18,
    "logoURI": "ipfs://QmYBq6GWG5hFMn3xsNnyjY2kKJTwPsmVAAgBdhzwW34cNu"
  },
  {
    "chainId": 1,
    "address": "0xd2877702675e6ceb975b4a1dff9fb7baf4c91ea9",
    "name": "Wrapped Terra Classic",
    "symbol": "LUNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13628/thumb/wluna.png?1696513376"
  },
  {
    "chainId": 1,
    "address": "0x5bb29c33c4a3c29f56f8aca40b4db91d8a5fe2c5",
    "name": "One Share",
    "symbol": "ONS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13531/thumb/bss.a1671c75.png?1696513292"
  },
  {
    "chainId": 1,
    "address": "0xd49ff13661451313ca1553fd6954bd1d9b6e02b9",
    "name": "Electrify Asia",
    "symbol": "ELEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3415/thumb/d45b1d82743c749d05697da200179874.jpg?1696504111"
  },
  {
    "chainId": 1,
    "address": "0x77c6e4a580c0dce4e5c7a17d0bc077188a83a059",
    "name": "Swerve fi USD",
    "symbol": "SWUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12918/thumb/swerve.png?1696512706"
  },
  {
    "chainId": 1,
    "address": "0x9cb2f26a23b8d89973f08c957c4d7cdf75cd341c",
    "name": "Digital Rand",
    "symbol": "DZAR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/9841/thumb/logo200_%281%29.png?1696509896"
  },
  {
    "chainId": 1,
    "address": "0xa58a4f5c4bb043d2cc1e170613b74e767c94189b",
    "name": "UTU Coin",
    "symbol": "UTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12831/thumb/Aa5nmbkJ_400x400.png?1696512622"
  },
  {
    "chainId": 1,
    "address": "0x3212b29e33587a00fb1c83346f5dbfa69a458923",
    "name": "The Tokenized Bitcoin",
    "symbol": "IMBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10702/thumb/imbtc.png?1696510670"
  },
  {
    "chainId": 1,
    "address": "0xc3eb2622190c57429aac3901808994443b64b466",
    "name": "ORO",
    "symbol": "ORO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13114/thumb/oro_logo.png?1696512899"
  },
  {
    "chainId": 1,
    "address": "0xdcd85914b8ae28c1e62f1c488e1d968d5aaffe2b",
    "name": "TOP Network",
    "symbol": "TOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7058/thumb/topnetwork-logo.png?1696507364"
  },
  {
    "chainId": 1,
    "address": "0xa14ea0e11121e6e951e87c66afe460a00bcd6a16",
    "name": "IdleDAI  Risk Adjusted ",
    "symbol": "IDLEDAISAFE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11929/thumb/idledai-safe.png?1696511790"
  },
  {
    "chainId": 1,
    "address": "0x634239cfa331df0291653139d1a6083b9cf705e3",
    "name": "DeSpace Protocol",
    "symbol": "DES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18517/thumb/NQQu-EsT_400x400.jpg?1696517998"
  },
  {
    "chainId": 1,
    "address": "0x431d5dff03120afa4bdf332c61a6e1766ef37bdb",
    "name": "JPY Coin",
    "symbol": "JPYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25971/thumb/2023jpyc.png?1696525049"
  },
  {
    "chainId": 1,
    "address": "0x4ec1b60b96193a64acae44778e51f7bff2007831",
    "name": "Edge",
    "symbol": "EDGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1848/thumb/EDGE.png?1696502846"
  },
  {
    "chainId": 1,
    "address": "0x3391bc034f2935ef0e1e41619445f998b2680d35",
    "name": "IdleUSDC  Risk Adjusted ",
    "symbol": "IDLEUSDCSAFE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11930/thumb/idleusdc-safe.png?1696511791"
  },
  {
    "chainId": 1,
    "address": "0x22b6c31c2beb8f2d0d5373146eed41ab9ede3caf",
    "name": "The Cocktailbar",
    "symbol": "COC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13121/thumb/Cocktail_Bar_logo_%281%29_%281%29.png?1696512909"
  },
  {
    "chainId": 1,
    "address": "0x2370f9d504c7a6e775bf6e14b3f12846b594cd53",
    "name": "JPY Coin v1",
    "symbol": "JPYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17277/thumb/WoZ8rruL_400x400.png?1696516831"
  },
  {
    "chainId": 1,
    "address": "0xb6ca7399b4f9ca56fc27cbff44f4d2e4eef1fc81",
    "name": "Muse DAO",
    "symbol": "MUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13230/thumb/muse_logo.png?1696513008"
  },
  {
    "chainId": 1,
    "address": "0x246908bff0b1ba6ecadcf57fb94f6ae2fcd43a77",
    "name": "Divi",
    "symbol": "DIVI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1273/thumb/red_logomark.png?1696502345"
  },
  {
    "chainId": 1,
    "address": "0xdf35988d795d90711e785b488bb2127692e6f956",
    "name": "BabyFloki",
    "symbol": "BABYFLOKI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16712/thumb/floki.jpg?1696516286"
  },
  {
    "chainId": 1,
    "address": "0x28fac5334c9f7262b3a3fe707e250e01053e07b5",
    "name": "IdleUSDT  Risk Adjusted ",
    "symbol": "IDLEUSDTSAFE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11931/thumb/idleusdt-safe.png?1696511792"
  },
  {
    "chainId": 1,
    "address": "0x77607588222e01bf892a29abab45796a2047fc7b",
    "name": "Unagii ETH",
    "symbol": "UETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14624/thumb/uETH.png?1696514302"
  },
  {
    "chainId": 1,
    "address": "0xdf801468a808a32656d2ed2d2d80b72a129739f4",
    "name": "Somnium Space CUBEs",
    "symbol": "CUBE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10687/thumb/CUBE_icon.png?1696510656"
  },
  {
    "chainId": 1,
    "address": "0xc5fdf3569af74f3b3e97e46a187a626352d2d508",
    "name": "Zombie Inu",
    "symbol": "ZINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28562/thumb/zinu_new.png?1696527552"
  },
  {
    "chainId": 1,
    "address": "0xb478c6245e3d85d6ec3486b62ea872128d562541",
    "name": "LootBot",
    "symbol": "LOOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30985/thumb/K7dMTTKi_400x400.png?1696529824"
  },
  {
    "chainId": 1,
    "address": "0xdab396ccf3d84cf2d07c4454e10c8a6f5b008d2b",
    "name": "Goldfinch",
    "symbol": "GFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19081/thumb/GOLDFINCH.png?1696518531"
  },
  {
    "chainId": 1,
    "address": "0x19d97d8fa813ee2f51ad4b4e04ea08baf4dffc28",
    "name": "Badger Sett Badger",
    "symbol": "BBADGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14502/thumb/BADGER_DAO.png?1696514188"
  },
  {
    "chainId": 1,
    "address": "0x7051faed0775f664a0286af4f75ef5ed74e02754",
    "name": "Changex",
    "symbol": "CHANGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26487/thumb/Changex-mark-200x200.png?1696525560"
  },
  {
    "chainId": 1,
    "address": "0x471a202f69d6e975da55e363dab1bdb2e86e0c0f",
    "name": "Geke",
    "symbol": "GEKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30332/thumb/geke.jpg?1696529233"
  },
  {
    "chainId": 1,
    "address": "0x35bd01fc9d6d5d81ca9e055db88dc49aa2c699a8",
    "name": "Friends With Benefits Pro",
    "symbol": "FWB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14391/thumb/xRGEXmQN_400x400.png?1696514082"
  },
  {
    "chainId": 1,
    "address": "0x30cf203b48edaa42c3b4918e955fed26cd012a3f",
    "name": "MetaGame",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13099/thumb/V8phEz8V.png?1696512885"
  },
  {
    "chainId": 1,
    "address": "0xfad45e47083e4607302aa43c65fb3106f1cd7607",
    "name": "Hoge Finance",
    "symbol": "HOGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14360/thumb/hoge.jpg?1696514046"
  },
  {
    "chainId": 1,
    "address": "0x3af5ba94c29a8407785f5f6d90ef5d69a8eb2436",
    "name": "Unagii Wrapped Bitcoin",
    "symbol": "UWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14625/thumb/uBTC.png?1696514303"
  },
  {
    "chainId": 1,
    "address": "0x5274891bec421b39d23760c04a6755ecb444797c",
    "name": "IdleUSDC  Yield ",
    "symbol": "IDLEUSDCYIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11932/thumb/idleusdcv3_32.png?1696511793"
  },
  {
    "chainId": 1,
    "address": "0x723cbfc05e2cfcc71d3d89e770d32801a5eef5ab",
    "name": "Bitcoin Pro",
    "symbol": "BTCP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/3545/thumb/DSiD9ZhWsAE_8cS.png?1696504231"
  },
  {
    "chainId": 1,
    "address": "0x6fa5e1c43b5a466cbd1cae7993b67c982400d481",
    "name": "CoinBot",
    "symbol": "COINBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31234/thumb/1-removebg-preview_%282%29.png?1696530060"
  },
  {
    "chainId": 1,
    "address": "0x54d1c019c53501f76361bb656542fec427bf13aa",
    "name": "Peer to Peer",
    "symbol": "P2P",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33123/thumb/IMG_7077.PNG?1700794851"
  },
  {
    "chainId": 1,
    "address": "0x20c64dee8fda5269a78f2d5bdba861ca1d83df7a",
    "name": "Backed HIGH   High Yield Corp Bond",
    "symbol": "BHIGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31868/thumb/b-HIGH-200x200.png?1696530680"
  },
  {
    "chainId": 1,
    "address": "0xbe9895146f7af43049ca1c1ae358b0541ea49704",
    "name": "Coinbase Wrapped Staked ETH",
    "symbol": "CBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27008/thumb/cbeth.png?1709186989"
  },
  {
    "chainId": 1,
    "address": "0xf34842d05a1c888ca02769a633df37177415c2f8",
    "name": "IdleUSDT  Yield ",
    "symbol": "IDLEUSDTYIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11933/thumb/idleusdtv3maxyield_32.png?1696511794"
  },
  {
    "chainId": 1,
    "address": "0x5dfc78c4d073fd343bc6661668948178522a0de5",
    "name": "Derp",
    "symbol": "DERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33069/thumb/derpdex_%281%29.png?1700793428"
  },
  {
    "chainId": 1,
    "address": "0x7f3edcdd180dbe4819bd98fee8929b5cedb3adeb",
    "name": "xToken",
    "symbol": "XTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14089/thumb/xtk.png?1696513811"
  },
  {
    "chainId": 1,
    "address": "0xd7c1eb0fe4a30d3b2a846c04aa6300888f087a5f",
    "name": "Points",
    "symbol": "POINTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33666/thumb/points_%281%29.png?1703050357"
  },
  {
    "chainId": 1,
    "address": "0xf52cdcd458bf455aed77751743180ec4a595fd3f",
    "name": "IdleSUSD  Yield ",
    "symbol": "IDLESUSDYIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11934/thumb/idlesusdv3mmaxyield_32.png?1696511795"
  },
  {
    "chainId": 1,
    "address": "0x5f0e628b693018f639d10e4a4f59bd4d8b2b6b44",
    "name": "Whiteheart",
    "symbol": "WHITE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13484/thumb/whiteheart.png?1696513245"
  },
  {
    "chainId": 1,
    "address": "0x1beef31946fbbb40b877a72e4ae04a8d1a5cee06",
    "name": "Parachute",
    "symbol": "PAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7590/thumb/Parachute_Logo.png?1696507850"
  },
  {
    "chainId": 1,
    "address": "0x70e8de73ce538da2beed35d14187f6959a8eca96",
    "name": "XSGD",
    "symbol": "XSGD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/12832/thumb/StraitsX_Singapore_Dollar_%28XSGD%29_Token_Logo.png?1696512623"
  },
  {
    "chainId": 1,
    "address": "0xc278041fdd8249fe4c1aad1193876857eea3d68c",
    "name": "IdleTUSD  Best Yield ",
    "symbol": "IDLETUSDYIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11935/thumb/idletusdv3mMaxyield_32.png?1696511796"
  },
  {
    "chainId": 1,
    "address": "0x5732046a883704404f284ce41ffadd5b007fd668",
    "name": "Bluzelle",
    "symbol": "BLZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2848/thumb/ColorIcon_3x.png?1696503607"
  },
  {
    "chainId": 1,
    "address": "0xec681f28f4561c2a9534799aa38e0d36a83cf478",
    "name": "YVS Finance",
    "symbol": "YVS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13471/thumb/cu0LSzE.png?1696513232"
  },
  {
    "chainId": 1,
    "address": "0x7a73839bd0e5cded853cb01aa9773f8989381065",
    "name": "UpBots",
    "symbol": "UBXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12476/thumb/UBXT.png?1696512294"
  },
  {
    "chainId": 1,
    "address": "0x89ab32156e46f46d02ade3fecbe5fc4243b9aaed",
    "name": "pNetwork",
    "symbol": "PNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11659/thumb/pNetwork.png?1696511550"
  },
  {
    "chainId": 1,
    "address": "0xf03a7eb46d01d9ecaa104558c732cf82f6b6b645",
    "name": "Stader MaticX",
    "symbol": "MATICX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25383/thumb/maticx.png?1696524516"
  },
  {
    "chainId": 1,
    "address": "0x72630b1e3b42874bf335020ba0249e3e9e47bafc",
    "name": "Paypolitan",
    "symbol": "EPAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13192/thumb/ava3.png?1696512973"
  },
  {
    "chainId": 1,
    "address": "0x0eb3032bcac2be1fa95e296442f225edb80fc3cd",
    "name": "Aster",
    "symbol": "ATC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17325/thumb/atc.png?1696516878"
  },
  {
    "chainId": 1,
    "address": "0xdb25f211ab05b1c97d595516f45794528a807ad8",
    "name": "STASIS EURO",
    "symbol": "EURS",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/5164/thumb/EURS_300x300.png?1696505680"
  },
  {
    "chainId": 1,
    "address": "0xfbeea1c75e4c4465cb2fccc9c6d6afe984558e20",
    "name": "DuckDaoDime",
    "symbol": "DDIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12146/thumb/token_DDIM-01.png?1696511987"
  },
  {
    "chainId": 1,
    "address": "0x8ae4bf2c33a8e667de34b54938b0ccd03eb8cc06",
    "name": "Patientory",
    "symbol": "PTOY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/746/thumb/patientory.png?1696501901"
  },
  {
    "chainId": 1,
    "address": "0xd337382da15d12bb6e56498e91df64f86c8f1ea8",
    "name": "Aelysir",
    "symbol": "AEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12955/thumb/ael_logo.png?1696512743"
  },
  {
    "chainId": 1,
    "address": "0xca0e7269600d353f70b14ad118a49575455c0f2f",
    "name": "AMLT Network",
    "symbol": "AMLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2703/thumb/amlt.png?1696503486"
  },
  {
    "chainId": 1,
    "address": "0x5394794be8b6ed5572fcd6b27103f46b5f390e8f",
    "name": "Aave AMM UniYFIWETH",
    "symbol": "AAMMUNIYFIWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17258/thumb/aAmmUniYFIWETH.png?1696516814"
  },
  {
    "chainId": 1,
    "address": "0x90de74265a416e1393a450752175aed98fe11517",
    "name": "Unlock Protocol",
    "symbol": "UDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14545/thumb/unlock.jpg?1696514228"
  },
  {
    "chainId": 1,
    "address": "0x8d75959f1e61ec2571aa72798237101f084de63a",
    "name": "Substratum",
    "symbol": "SUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/950/thumb/substratum.png?1696502066"
  },
  {
    "chainId": 1,
    "address": "0xf9fb4ad91812b704ba883b11d2b576e890a6730a",
    "name": "Aave AMM WETH",
    "symbol": "AAMMWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17259/thumb/aAMMWETH_2x.png?1696516814"
  },
  {
    "chainId": 1,
    "address": "0xc4d5545392f5fc57eba3af8981815669bb7e2a48",
    "name": "HEdpAY",
    "symbol": "HDP",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/7496/thumb/icon_hedpay.png?1696507765"
  },
  {
    "chainId": 1,
    "address": "0x2de509bf0014ddf697b220be628213034d320ece",
    "name": "Don t Buy Inu",
    "symbol": "DBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28058/thumb/1Artboard_1_%282%29.png?1696527070"
  },
  {
    "chainId": 1,
    "address": "0xd01409314acb3b245cea9500ece3f6fd4d70ea30",
    "name": "LTO Network",
    "symbol": "LTO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/6068/thumb/lto.png?1696506473"
  },
  {
    "chainId": 1,
    "address": "0x0e0989b1f9b8a38983c2ba8053269ca62ec9b195",
    "name": "Po et",
    "symbol": "POE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/910/thumb/poet.png?1696502031"
  },
  {
    "chainId": 1,
    "address": "0x2edf094db69d6dcd487f1b3db9febe2eec0dd4c5",
    "name": "ZeroSwap",
    "symbol": "ZEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12861/thumb/logo.?1696512650"
  },
  {
    "chainId": 1,
    "address": "0x24b47299e756af0571f512232a3629e0dabb52ed",
    "name": "concertVR",
    "symbol": "CVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2914/thumb/cvt.png?1696503662"
  },
  {
    "chainId": 1,
    "address": "0x358bd0d980e031e23eba9aa793926857703783bd",
    "name": "Aave AMM BptWBTCWETH",
    "symbol": "AAMMBPTWBTCWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17260/thumb/aAmmBptWBTCWETH.png?1696516815"
  },
  {
    "chainId": 1,
    "address": "0x378cb52b00f9d0921cb46dfc099cff73b42419dc",
    "name": "LUSD yVault",
    "symbol": "YVLUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28788/thumb/yvLUSD-128-0x378cb52b00F9D0921cb46dFc099CFf73b42419dC.png?1696527767"
  },
  {
    "chainId": 1,
    "address": "0xc0c293ce456ff0ed870add98a0828dd4d2903dbf",
    "name": "Aura Finance",
    "symbol": "AURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25942/thumb/logo.png?1696525021"
  },
  {
    "chainId": 1,
    "address": "0xcc6f15be8573cb8243c42d300565566d328213dd",
    "name": "OWN Token",
    "symbol": "OWN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10950/thumb/cropped-WhatsApp-Image-2020-04-04-at-9.04.01-PM.jpeg?1696510899"
  },
  {
    "chainId": 1,
    "address": "0xf3db5fa2c66b7af3eb0c0b782510816cbe4813b8",
    "name": "Everex",
    "symbol": "EVX",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/997/thumb/everex.png?1696502109"
  },
  {
    "chainId": 1,
    "address": "0xd31e53966bf212e860d48a3a8651a23d09a7fdc3",
    "name": "DogeAi",
    "symbol": "DOGEAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33023/thumb/DogeAI_logo_200x200.png?1700316738"
  },
  {
    "chainId": 1,
    "address": "0x7a41e0517a5eca4fdbc7fbeba4d4c47b9ff6dc63",
    "name": "Zeusshield",
    "symbol": "ZSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1009/thumb/zeusshield.png?1696502121"
  },
  {
    "chainId": 1,
    "address": "0x6d765cbe5bc922694afe112c140b8878b9fb0390",
    "name": "SUSHI yVault",
    "symbol": "YVSUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28789/thumb/yvSUSHI-128-0x6d765CbE5bC922694afE112C140b8878b9FB0390.png?1696527767"
  },
  {
    "chainId": 1,
    "address": "0xd2d6158683aee4cc838067727209a0aaf4359de3",
    "name": "Bounty0x",
    "symbol": "BNTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1444/thumb/bounty0x.png?1696502494"
  },
  {
    "chainId": 1,
    "address": "0xf51092fe93b4e9282f42c459f05d93d2d079549e",
    "name": "RociFi",
    "symbol": "ROCI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31960/thumb/Icon-03.png?1696530766"
  },
  {
    "chainId": 1,
    "address": "0x9c78ee466d6cb57a4d01fd887d2b5dfb2d46288f",
    "name": "Must",
    "symbol": "MUST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13688/thumb/must_logo.png?1696513436"
  },
  {
    "chainId": 1,
    "address": "0x7533d63a2558965472398ef473908e1320520ae2",
    "name": "INTEXCOIN",
    "symbol": "INTX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/11854/thumb/INTX.png?1696511724"
  },
  {
    "chainId": 1,
    "address": "0x220b71671b649c03714da9c621285943f3cbcdc6",
    "name": "TosDis",
    "symbol": "DIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13745/thumb/Tosdis-black.png?1696513488"
  },
  {
    "chainId": 1,
    "address": "0xd109b2a304587569c84308c55465cd9ff0317bfb",
    "name": "Aave AMM BptBALWETH",
    "symbol": "AAMMBPTBALWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17261/thumb/aAmmBptBALWETH.png?1696516816"
  },
  {
    "chainId": 1,
    "address": "0x5dfe42eea70a3e6f93ee54ed9c321af07a85535c",
    "name": "Union Finance",
    "symbol": "UNION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30556/thumb/Mark.png?1712560901"
  },
  {
    "chainId": 1,
    "address": "0xba11d00c5f74255f56a5e366f4f77f5a186d7f55",
    "name": "Band Protocol",
    "symbol": "BAND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9545/thumb/Band_token_blue_violet_token.png?1696509627"
  },
  {
    "chainId": 1,
    "address": "0xb1f66997a5760428d3a87d68b90bfe0ae64121cc",
    "name": "LuaSwap",
    "symbol": "LUA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12627/thumb/Screenshot_2020-09-28_at_6.24.59_PM.jpg?1696512435"
  },
  {
    "chainId": 1,
    "address": "0x8f0921f30555624143d427b340b1156914882c10",
    "name": "FlypMe",
    "symbol": "FYP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1098/thumb/flypme.png?1696502196"
  },
  {
    "chainId": 1,
    "address": "0xec67005c4e498ec7f55e092bd1d35cbc47c91892",
    "name": "Enzyme",
    "symbol": "MLN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/605/thumb/Enzyme_Icon_Secondary.png?1696501803"
  },
  {
    "chainId": 1,
    "address": "0xfd0877d9095789caf24c98f7cce092fa8e120775",
    "name": "TUSD yVault",
    "symbol": "YVTUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28790/thumb/yvTUSD-128-0xFD0877d9095789cAF24c98F7CCe092fa8E120775.png?1696527768"
  },
  {
    "chainId": 1,
    "address": "0x6c2adc2073994fb2ccc5032cc2906fa221e9b391",
    "name": "Delphy",
    "symbol": "DPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1133/thumb/delphy.png?1696502226"
  },
  {
    "chainId": 1,
    "address": "0xed40834a13129509a89be39a9be9c0e96a0ddd71",
    "name": "Warp Finance",
    "symbol": "WARP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13910/thumb/601ed0ac35c687c6e07d17c2_warp_token.png?1696513651"
  },
  {
    "chainId": 1,
    "address": "0x1500205f50bf3fd976466d0662905c9ff254fc9c",
    "name": "BitBoost",
    "symbol": "BBT",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/1194/thumb/bitboost.jpg?1696502276"
  },
  {
    "chainId": 1,
    "address": "0x3893b9422cd5d70a81edeffe3d5a1c6a978310bb",
    "name": "Mithril",
    "symbol": "MITH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3484/thumb/mithril.png?1696504172"
  },
  {
    "chainId": 1,
    "address": "0xedbf98724a86f92baefac101082c366e96f1e9d9",
    "name": "BiFinance Exchange",
    "symbol": "BFT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x4db2c02831c9ac305ff9311eb661f80f1df61e07",
    "name": "oneICHI",
    "symbol": "ONEICHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25908/thumb/74116843.png?1696524989"
  },
  {
    "chainId": 1,
    "address": "0xf4134146af2d511dd5ea8cdb1c4ac88c57d60404",
    "name": "SunContract",
    "symbol": "SNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1166/thumb/suncontract.png?1696502255"
  },
  {
    "chainId": 1,
    "address": "0x71fc860f7d3a592a4a98740e39db31d25db65ae8",
    "name": "Aave USDT v1",
    "symbol": "AUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/11725/thumb/aUSDT.png?1696511607"
  },
  {
    "chainId": 1,
    "address": "0x0000000000095413afc295d19edeb1ad7b71c952",
    "name": "Tokenlon",
    "symbol": "LON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13454/thumb/lon_logo.png?1696513217"
  },
  {
    "chainId": 1,
    "address": "0xb1f1ee126e9c96231cc3d3fad7c08b4cf873b1f1",
    "name": "Beefy",
    "symbol": "BIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12704/thumb/bifi.png?1698202580"
  },
  {
    "chainId": 1,
    "address": "0xfc05987bd2be489accf0f509e44b0145d68240f7",
    "name": "Essentia",
    "symbol": "ESS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2483/thumb/Essentia-token.jpg?1696503302"
  },
  {
    "chainId": 1,
    "address": "0x20c3fa331a385b63ee39137e99d0cf2db142fce1",
    "name": "Shila Inu",
    "symbol": "SHIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28847/thumb/rsz_logo1.png?1696527823"
  },
  {
    "chainId": 1,
    "address": "0x1f16d41f9b3db03b462bdd6c92245ee708d1c103",
    "name": "Rangers Protocol Gas",
    "symbol": "RPG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18791/thumb/tO8MlqiM_400x400.png?1633421196"
  },
  {
    "chainId": 1,
    "address": "0x3810a4ddf41e586fa0dba1463a7951b748cecfca",
    "name": "Menapay",
    "symbol": "MPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6360/thumb/Menapay_ico.png?1696506732"
  },
  {
    "chainId": 1,
    "address": "0x35a18000230da775cac24873d00ff85bccded550",
    "name": "cUNI",
    "symbol": "CUNI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12696/thumb/compound-uni.png?1696512499"
  },
  {
    "chainId": 1,
    "address": "0x679badc551626e01b23ceecefbc9b877ea18fc46",
    "name": "Ccore",
    "symbol": "CCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1287/thumb/LOGO_CCO.png?1696502358"
  },
  {
    "chainId": 1,
    "address": "0x70efdc485a10210b056ef8e0a32993bc6529995e",
    "name": "Blaze Network",
    "symbol": "BLZN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12248/thumb/A8JOuPSJ_400x400.jpg?1696512079"
  },
  {
    "chainId": 1,
    "address": "0x1e797ce986c3cff4472f7d38d5c4aba55dfefe40",
    "name": "BlockCDN",
    "symbol": "BCDN",
    "decimals": 15,
    "logoURI": "https://assets.coingecko.com/coins/images/1301/thumb/blockcdn.png?1696502370"
  },
  {
    "chainId": 1,
    "address": "0x441761326490cacf7af299725b6292597ee822c2",
    "name": "Unifi Protocol DAO",
    "symbol": "UNFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13152/thumb/logo-2.png?1696512937"
  },
  {
    "chainId": 1,
    "address": "0x6ff2241756549b5816a177659e766eaf14b34429",
    "name": "AQTIS",
    "symbol": "AQTIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29910/thumb/Screenshot_2023-04-14_at_17.23.46.png?1696528839"
  },
  {
    "chainId": 1,
    "address": "0xbdab72602e9ad40fc6a6852caf43258113b8f7a5",
    "name": "Sovryn",
    "symbol": "SOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16248/thumb/sovryn_logo_orange.png?1709174299"
  },
  {
    "chainId": 1,
    "address": "0x66186008c1050627f979d464eabb258860563dbe",
    "name": "MediShares",
    "symbol": "MDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1343/thumb/medishares.png?1696502404"
  },
  {
    "chainId": 1,
    "address": "0x21f15966e07a10554c364b988e91dab01d32794a",
    "name": "SmartMesh",
    "symbol": "SMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1346/thumb/smartmesh.png?1696502406"
  },
  {
    "chainId": 1,
    "address": "0x3505f494c3f0fed0b594e01fa41dd3967645ca39",
    "name": "Swarm Network",
    "symbol": "SWM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/197/thumb/swarm.jpg?1696501567"
  },
  {
    "chainId": 1,
    "address": "0x8b1f49491477e0fb46a29fef53f1ea320d13c349",
    "name": "MicroMoney",
    "symbol": "AMM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/1356/thumb/micromoney.png?1696502415"
  },
  {
    "chainId": 1,
    "address": "0x2dca19e944453e46d9130950ca135461b3bc0c30",
    "name": "EYES Protocol",
    "symbol": "EYES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9734/thumb/pTaAZUI6_400x400.jpg?1696509797"
  },
  {
    "chainId": 1,
    "address": "0x6ec8a24cabdc339a06a172f8223ea557055adaa5",
    "name": "Genaro Network",
    "symbol": "GNX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/1361/thumb/genaro-network.png?1696502420"
  },
  {
    "chainId": 1,
    "address": "0xc86d054809623432210c107af2e3f619dcfbf652",
    "name": "Sentinel Protocol",
    "symbol": "UPP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3369/thumb/Sentinel_Protocol.jpg?1696504069"
  },
  {
    "chainId": 1,
    "address": "0x16b0a1a87ae8af5c792fabc429c4fe248834842b",
    "name": "Algory",
    "symbol": "ALG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12231/thumb/logo-2.png?1696512064"
  },
  {
    "chainId": 1,
    "address": "0xa5f2211b9b8170f694421f2046281775e8468044",
    "name": "THORSwap",
    "symbol": "THOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19292/thumb/THORSwap_Logo-removebg-preview.png?1696518735"
  },
  {
    "chainId": 1,
    "address": "0xb44b653f147569d88a684cbf6549e1968e8b2a1d",
    "name": "2DAI io",
    "symbol": "2DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30542/thumb/2dai.jpg?1696529414"
  },
  {
    "chainId": 1,
    "address": "0x0c93b616933b0cd03b201b29cd8a22681dd9e0d9",
    "name": "HollyGold",
    "symbol": "HGOLD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13652/thumb/QnfGCea3aO_Qb-Gd9n6MSFE_eBVAr87WPoxAzf0gKdcGOEL4K5J6A5cpGinzLvHclj_UHkCnw9XTlGoNYd0H8T_Ebe8voLS49MAZO2NIknCxytaYFtt5_u12RQg-kpmWHcETzAFlDgk9iNXAL-88fB_l-DRi8WEsgSJzXXjztKzQlpXuET6d4_98pykED6uyq1-4PMVg05P7tUN.jpg?1696513401"
  },
  {
    "chainId": 1,
    "address": "0x7968bc6a03017ea2de509aaa816f163db0f35148",
    "name": "Hedget",
    "symbol": "HGET",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/12453/thumb/Hedget.png?1696512273"
  },
  {
    "chainId": 1,
    "address": "0x671a912c10bba0cfa74cfc2d6fba9ba1ed9530b2",
    "name": "LINK yVault",
    "symbol": "YVLINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28791/thumb/yvLINK-128-0x671a912C10bba0CFA74Cfc2d6Fba9BA1ed9530B2.png?1696527769"
  },
  {
    "chainId": 1,
    "address": "0x873fb544277fd7b977b196a826459a69e27ea4ea",
    "name": "RAI yVault",
    "symbol": "YVRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28792/thumb/yvRAI-128-0x873fB544277FD7b977B196a826459a69E27eA4ea.png?1696527770"
  },
  {
    "chainId": 1,
    "address": "0xafa42b8ba6ba9dace46dae129a2a1ef54b73fa8b",
    "name": "MODAI",
    "symbol": "MODAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35625/thumb/p5200x200.jpg?1709290896"
  },
  {
    "chainId": 1,
    "address": "0x6f222e04f6c53cc688ffb0abe7206aac66a8ff98",
    "name": "Roko Network",
    "symbol": "ROKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29427/thumb/1.618_200.jpg?1696528376"
  },
  {
    "chainId": 1,
    "address": "0xa5b947687163fe88c3e6af5b17ae69896f4abccf",
    "name": "Poseidon",
    "symbol": "PSDN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25316/thumb/psdn_32.png?1696524451"
  },
  {
    "chainId": 1,
    "address": "0xb753428af26e81097e7fd17f40c88aaa3e04902c",
    "name": "saffron finance",
    "symbol": "SFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13117/thumb/sfi_red_250px.png?1696512904"
  },
  {
    "chainId": 1,
    "address": "0xdcb5645eda1ed34c5641d81b927d33ebae9cf2a4",
    "name": "PayB",
    "symbol": "PAYB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25805/thumb/logo_payb_200_x_200.png?1696524891"
  },
  {
    "chainId": 1,
    "address": "0x24d73bca2bd9c3a61e99dfc7cb86d3c379ebded7",
    "name": "Micro AI",
    "symbol": "MAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32811/thumb/mai.png?1699537844"
  },
  {
    "chainId": 1,
    "address": "0x6f40d4a6237c257fff2db00fa0510deeecd303eb",
    "name": "Instadapp",
    "symbol": "INST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14688/thumb/30hFM0-n_400x400.jpg?1696514361"
  },
  {
    "chainId": 1,
    "address": "0xa258c4606ca8206d8aa700ce2143d7db854d168c",
    "name": "WETH yVault",
    "symbol": "YVWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28793/thumb/yvWETH-128-0xa258C4606Ca8206D8aA700cE2143D7db854D168c.png?1696527771"
  },
  {
    "chainId": 1,
    "address": "0xa5ca62d95d24a4a350983d5b8ac4eb8638887396",
    "name": "sUSD yVault",
    "symbol": "YVSUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28794/thumb/yvsUSD-128-0xa5cA62D95D24A4a350983D5B8ac4EB8638887396.png?1696527772"
  },
  {
    "chainId": 1,
    "address": "0x89fb927240750c1b15d4743cd58440fc5f14a11c",
    "name": "Attila",
    "symbol": "ATT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11337/thumb/LOGO_%2874%29.png?1696511258"
  },
  {
    "chainId": 1,
    "address": "0xb26631c6dda06ad89b93c71400d25692de89c068",
    "name": "Minds",
    "symbol": "MINDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11517/thumb/Minds.png?1696511420"
  },
  {
    "chainId": 1,
    "address": "0xe5867608b51a2c9c78b9587355cc093140a49b0a",
    "name": "Speed Mining Service",
    "symbol": "SMS",
    "decimals": 3,
    "logoURI": "https://assets.coingecko.com/coins/images/1807/thumb/speed-mining-service.png?1548610827"
  },
  {
    "chainId": 1,
    "address": "0x6251e725cd45fb1af99354035a414a2c0890b929",
    "name": "MixTrust",
    "symbol": "MXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12148/thumb/MXT_LOGO_200_200.png?1696511988"
  },
  {
    "chainId": 1,
    "address": "0xfa3118b34522580c35ae27f6cf52da1dbb756288",
    "name": "Linkeye",
    "symbol": "LET",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/1814/thumb/linkeye.png?1696502815"
  },
  {
    "chainId": 1,
    "address": "0xb8c3b7a2a618c552c23b1e4701109a9e756bab67",
    "name": "1INCH yVault",
    "symbol": "YV1INCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28795/thumb/yv1INCH-128-0xB8C3B7A2A618C552C23B1E4701109a9E756Bab67.png?1696527773"
  },
  {
    "chainId": 1,
    "address": "0xa696a63cc78dffa1a63e9e50587c197387ff6c7e",
    "name": "WBTC yVault",
    "symbol": "YVWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28796/thumb/yvWBTC-128-0xA696a63cc78DfFa1a63E9E50587C197387FF6C7E.png?1696527774"
  },
  {
    "chainId": 1,
    "address": "0xf29ae508698bdef169b89834f76704c3b205aedf",
    "name": "SNX yVault",
    "symbol": "YVSNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28797/thumb/yvSNX-128-0xF29AE508698bDeF169B89834F76704C3B205aedf.png?1696527775"
  },
  {
    "chainId": 1,
    "address": "0x88d50b466be55222019d71f9e8fae17f5f45fca1",
    "name": "Cryptaur",
    "symbol": "CPT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2481/thumb/n3w6Z9xW_400x400.jpg?1696503301"
  },
  {
    "chainId": 1,
    "address": "0xf406f7a9046793267bc276908778b29563323996",
    "name": "APY vision",
    "symbol": "VISION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13288/thumb/apyvisionlogo200circle.png?1696513060"
  },
  {
    "chainId": 1,
    "address": "0xea01906843ea8d910658a2c485ffce7c104ab2b6",
    "name": "Qtoken",
    "symbol": "QTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26024/thumb/8QyOrTCm_400x400.jpg?1696525101"
  },
  {
    "chainId": 1,
    "address": "0xdffa3a7f5b40789c7a437dbe7b31b47f9b08fe75",
    "name": "CryptoPunk  7171",
    "symbol": "HOODIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17842/thumb/cryptopunk-hoodie.png?1696517365"
  },
  {
    "chainId": 1,
    "address": "0x411099c0b413f4feddb10edf6a8be63bd321311c",
    "name": "HELLO",
    "symbol": "HELLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27891/thumb/uFXN2S1N_400x400.jpeg?1696526908"
  },
  {
    "chainId": 1,
    "address": "0x4a8f5f96d5436e43112c2fbc6a9f70da9e4e16d4",
    "name": "Internxt",
    "symbol": "INXT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/986/thumb/inxt.png?1696502099"
  },
  {
    "chainId": 1,
    "address": "0x3fd8f39a962efda04956981c31ab89fab5fb8bc8",
    "name": "Rotharium",
    "symbol": "RTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2030/thumb/L7kpRMda_400x400.jpg?1696503000"
  },
  {
    "chainId": 1,
    "address": "0x4cf488387f035ff08c371515562cba712f9015d4",
    "name": "WePower",
    "symbol": "WPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1951/thumb/wpr.png?1696502934"
  },
  {
    "chainId": 1,
    "address": "0xea38eaa3c86c8f9b751533ba2e562deb9acded40",
    "name": "Etherparty",
    "symbol": "FUEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1082/thumb/etherparty.png?1696502182"
  },
  {
    "chainId": 1,
    "address": "0x3d658390460295fb963f54dc0899cfb1c30776df",
    "name": "Circuits of Value",
    "symbol": "COVAL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/588/thumb/coval-logo.png?1696501792"
  },
  {
    "chainId": 1,
    "address": "0x5cac718a3ae330d361e39244bf9e67ab17514ce8",
    "name": "Cosplay Token",
    "symbol": "COT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21294/thumb/brave_ZxsjzUc8xn.png?1696520664"
  },
  {
    "chainId": 1,
    "address": "0x36905fc93280f52362a1cbab151f25dc46742fb5",
    "name": "Bottos",
    "symbol": "BTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2048/thumb/bottos.png?1696503016"
  },
  {
    "chainId": 1,
    "address": "0x4ddc2d193948926d02f9b1fe9e1daa0718270ed5",
    "name": "cETH",
    "symbol": "CETH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10643/thumb/ceth.png?1696510617"
  },
  {
    "chainId": 1,
    "address": "0xb533687ef77459093368c43e95f8df1c2b5a1f7a",
    "name": "Nekoverse  City of Greed Anima Spirit G",
    "symbol": "ASG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32655/thumb/asg.png?1698895535"
  },
  {
    "chainId": 1,
    "address": "0xee573a945b01b788b9287ce062a0cfc15be9fd86",
    "name": "Exeedme",
    "symbol": "XED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13518/thumb/exeedme.png?1696513279"
  },
  {
    "chainId": 1,
    "address": "0x5c6ee304399dbdb9c8ef030ab642b10820db8f56",
    "name": "Balancer 80 BAL 20 WETH",
    "symbol": "B-80BAL-20WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26452/thumb/0x5c6ee304399dbdb9c8ef030ab642b10820db8f56.png?1696525525"
  },
  {
    "chainId": 1,
    "address": "0xbe428c3867f05dea2a89fc76a102b544eac7f772",
    "name": "CyberVein",
    "symbol": "CVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3886/thumb/CyberVein_LOGO.png?1696504547"
  },
  {
    "chainId": 1,
    "address": "0xd0929d411954c47438dc1d871dd6081f5c5e149c",
    "name": "Refereum",
    "symbol": "RFR",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/2102/thumb/refereum.png?1696503069"
  },
  {
    "chainId": 1,
    "address": "0xa64dfe8d86963151e6496bee513e366f6e42ed79",
    "name": "Goku",
    "symbol": "GOKU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17947/thumb/goku.png?1696517467"
  },
  {
    "chainId": 1,
    "address": "0xa15c7ebe1f07caf6bff097d8a589fb8ac49ae5b3",
    "name": "Pundi X  OLD ",
    "symbol": "NPXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2170/thumb/pundi-x.png?1696503130"
  },
  {
    "chainId": 1,
    "address": "0x515d7e9d75e2b76db60f8a051cd890eba23286bc",
    "name": "Governor DAO",
    "symbol": "GDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13140/thumb/GDAOlogo2-bird.png?1696512926"
  },
  {
    "chainId": 1,
    "address": "0xf5aed4f6a1ad00f39dd21febb6f400ea020030c2",
    "name": "Hodless Bot",
    "symbol": "HBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32489/thumb/logo-200-200.png?1698476298"
  },
  {
    "chainId": 1,
    "address": "0xe11ba472f74869176652c35d30db89854b5ae84d",
    "name": "HEGIC yVault",
    "symbol": "YVHEGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28798/thumb/yvHEGIC-128-0xe11ba472F74869176652C35D30dB89854b5ae84D.png?1696527776"
  },
  {
    "chainId": 1,
    "address": "0x4da9b813057d04baef4e5800e36083717b4a0341",
    "name": "Aave TUSD v1",
    "symbol": "ATUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11724/thumb/aTUSD.png?1696511606"
  },
  {
    "chainId": 1,
    "address": "0xef53462838000184f35f7d991452e5f25110b207",
    "name": "Knit Finance",
    "symbol": "KFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15632/thumb/knit.jpg?1696515264"
  },
  {
    "chainId": 1,
    "address": "0xb40c535c8899f95e3b722df2f0619ebd28c4a4ea",
    "name": "Kenda",
    "symbol": "KNDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31692/thumb/kenda_200_200.png?1696530510"
  },
  {
    "chainId": 1,
    "address": "0x0ebb614204e47c09b6c3feb9aaecad8ee060e23e",
    "name": "Cryptopay",
    "symbol": "CPAY",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/2216/thumb/cryptopay.png?1696503169"
  },
  {
    "chainId": 1,
    "address": "0xe477292f1b3268687a29376116b0ed27a9c76170",
    "name": "HEROcoin",
    "symbol": "PLAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2221/thumb/herocoin.png?1696503173"
  },
  {
    "chainId": 1,
    "address": "0x9d86b1b2554ec410eccffbf111a6994910111340",
    "name": "Open Platform",
    "symbol": "OPEN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2953/thumb/open-platform.png?1696503697"
  },
  {
    "chainId": 1,
    "address": "0xd578779dbc9252218e12d18d628e3cb27e4a56f2",
    "name": "Britto",
    "symbol": "BRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22830/thumb/britto-coin-logo.jpg?1696522131"
  },
  {
    "chainId": 1,
    "address": "0x2bba3cf6de6058cc1b4457ce00deb359e2703d7f",
    "name": "HashCoin",
    "symbol": "HSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4989/thumb/hashfuture-logo.png?1696505523"
  },
  {
    "chainId": 1,
    "address": "0x3a1bda28adb5b0a812a7cf10a1950c920f79bcd3",
    "name": "Gameflip",
    "symbol": "FLP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2173/thumb/flp.png?1696503133"
  },
  {
    "chainId": 1,
    "address": "0x1571ed0bed4d987fe2b498ddbae7dfa19519f651",
    "name": "iFARM",
    "symbol": "IFARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14472/thumb/ifarm.png?1696514159"
  },
  {
    "chainId": 1,
    "address": "0x4740735aa98dc8aa232bd049f8f0210458e7fca3",
    "name": "Ridotto",
    "symbol": "RDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18671/thumb/200x200_%2832%29.png?1696518141"
  },
  {
    "chainId": 1,
    "address": "0x370adc71f67f581158dc56f539df5f399128ddf9",
    "name": "Aave AMM UniMKRWETH",
    "symbol": "AAMMUNIMKRWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17222/thumb/aAmmUniMKRWETH.png?1696516777"
  },
  {
    "chainId": 1,
    "address": "0x275f5ad03be0fa221b4c6649b8aee09a42d9412a",
    "name": "Monavale",
    "symbol": "MONA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13298/thumb/monavale_logo.jpg?1696513070"
  },
  {
    "chainId": 1,
    "address": "0xf921ae2dac5fa128dc0f6168bf153ea0943d2d43",
    "name": "Fire Protocol",
    "symbol": "FIRE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13495/thumb/fire.jpg?1696513256"
  },
  {
    "chainId": 1,
    "address": "0xf8483e2d6560585c02d46bf7b3186bf154a96166",
    "name": "IdeaChain",
    "symbol": "ICH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11053/thumb/4fJlFzca_400x400.jpg?1696510995"
  },
  {
    "chainId": 1,
    "address": "0xc96df921009b790dffca412375251ed1a2b75c60",
    "name": "Ormeus Coin",
    "symbol": "ORME",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/964/thumb/ORMEUS_logo.png?1696502079"
  },
  {
    "chainId": 1,
    "address": "0x0abdace70d3790235af448c88547603b945604ea",
    "name": "district0x",
    "symbol": "DNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/849/thumb/district0x.png?1696501985"
  },
  {
    "chainId": 1,
    "address": "0xaa99199d1e9644b588796f3215089878440d58e0",
    "name": "Alphr",
    "symbol": "ALPHR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15029/thumb/alphr.jpg?1696514689"
  },
  {
    "chainId": 1,
    "address": "0xa6a840e50bcaa50da017b91a0d86b8b2d41156ee",
    "name": "EchoLink",
    "symbol": "EKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2437/thumb/echolink.png?1696503269"
  },
  {
    "chainId": 1,
    "address": "0x4824a7b64e3966b0133f4f4ffb1b9d6beb75fff7",
    "name": "TokenClub",
    "symbol": "TCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2455/thumb/tokenclub.png?1696503285"
  },
  {
    "chainId": 1,
    "address": "0x9d91be44c06d373a8a226e1f3b146956083803eb",
    "name": "Aave KNC v1",
    "symbol": "AKNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11727/thumb/aKNC.png?1696511608"
  },
  {
    "chainId": 1,
    "address": "0x712db54daa836b53ef1ecbb9c6ba3b9efb073f40",
    "name": "Aave ENJ v1",
    "symbol": "AENJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12484/thumb/aENJ.png?1696512301"
  },
  {
    "chainId": 1,
    "address": "0xf7413489c474ca4399eee604716c72879eea3615",
    "name": "APYSwap",
    "symbol": "APYS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14163/thumb/apys.png?1696513881"
  },
  {
    "chainId": 1,
    "address": "0x7deb5e830be29f91e298ba5ff1356bb7f8146998",
    "name": "Aave MKR v1",
    "symbol": "AMKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11731/thumb/aMKR.png?1696511612"
  },
  {
    "chainId": 1,
    "address": "0x5d3a536e4d6dbd6114cc1ead35777bab948e3643",
    "name": "cDAI",
    "symbol": "CDAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9281/thumb/cDAI.png?1696509390"
  },
  {
    "chainId": 1,
    "address": "0x79650799e7899a802cb96c0bc33a6a8d4ce4936c",
    "name": "AICHAIN",
    "symbol": "AIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2525/thumb/aichain.png?1696503338"
  },
  {
    "chainId": 1,
    "address": "0xfe3e6a25e6b192a42a44ecddcd13796471735acf",
    "name": "Reef",
    "symbol": "REEF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13504/thumb/Group_10572.png?1696513266"
  },
  {
    "chainId": 1,
    "address": "0x92e52a1a235d9a103d970901066ce910aacefd37",
    "name": "U CASH",
    "symbol": "UCASH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2901/thumb/ucash.png?1696503651"
  },
  {
    "chainId": 1,
    "address": "0x23894dc9da6c94ecb439911caf7d337746575a72",
    "name": "Geojam",
    "symbol": "JAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24648/thumb/200px_Transparent_Bubble_Token_Icon.png?1696523818"
  },
  {
    "chainId": 1,
    "address": "0xdfbc9050f5b01df53512dcc39b4f2b2bbacd517a",
    "name": "Jobchain",
    "symbol": "JOB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9352/thumb/Logo_%283%29.png?1696509452"
  },
  {
    "chainId": 1,
    "address": "0x39aa39c021dfbae8fac545936693ac917d5e7563",
    "name": "cUSDC",
    "symbol": "CUSDC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9442/thumb/Compound_USDC.png?1696509534"
  },
  {
    "chainId": 1,
    "address": "0xbf776e4fca664d791c4ee3a71e2722990e003283",
    "name": "Smoothy",
    "symbol": "SMTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15039/thumb/dDxKgwPN_400x400.jpg?1696514698"
  },
  {
    "chainId": 1,
    "address": "0x9f5f3cfd7a32700c93f971637407ff17b91c7342",
    "name": "Scry info",
    "symbol": "DDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2569/thumb/Scry.info.jpg?1696503374"
  },
  {
    "chainId": 1,
    "address": "0x63f584fa56e60e4d0fe8802b27c7e6e3b33e007f",
    "name": "ContentBox",
    "symbol": "BOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4929/thumb/contentbox.png?1696505470"
  },
  {
    "chainId": 1,
    "address": "0x1a57367c6194199e5d9aea1ce027431682dfb411",
    "name": "MatrixETF",
    "symbol": "MDF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18538/thumb/MDF.png?1696518018"
  },
  {
    "chainId": 1,
    "address": "0x72108a8cc3254813c6be2f1b77be53e185abfdd9",
    "name": "Era Swap",
    "symbol": "ES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6031/thumb/era_swap.PNG?1696506442"
  },
  {
    "chainId": 1,
    "address": "0x29cbd0510eec0327992cd6006e63f9fa8e7f33b7",
    "name": "Tidal Finance",
    "symbol": "TIDAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14460/thumb/Tidal-mono.png?1696514147"
  },
  {
    "chainId": 1,
    "address": "0x9a64977ebf739dff35ed4281a4b5e833bfdb1314",
    "name": "NFTFN",
    "symbol": "NFTFN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36624/thumb/NFTFN_TokenPng_%281%29.png?1711982088"
  },
  {
    "chainId": 1,
    "address": "0xf278c1ca969095ffddded020290cf8b5c424ace2",
    "name": "Ruff",
    "symbol": "RUFF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2726/thumb/ruff.png?1696503504"
  },
  {
    "chainId": 1,
    "address": "0xa2847348b58ced0ca58d23c7e9106a49f1427df6",
    "name": "Convex FPIS",
    "symbol": "CVXFPIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29678/thumb/photo_2023-03-18_21-37-17.jpg?1696528612"
  },
  {
    "chainId": 1,
    "address": "0xf1f508c7c9f0d1b15a76fba564eef2d956220cf7",
    "name": "Pepedex",
    "symbol": "PPDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13022/thumb/output-onlinepngtools-1.png?1696512811"
  },
  {
    "chainId": 1,
    "address": "0x2001f2a0cf801ecfda622f6c28fb6e10d803d969",
    "name": "CoinLoan",
    "symbol": "CLT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1713/thumb/blue-logo-transparent-200x200.png?1696502730"
  },
  {
    "chainId": 1,
    "address": "0x40955d77f87123b71b145098358a60573ac7be96",
    "name": "Daisy Protocol",
    "symbol": "DAISY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17844/thumb/daisy.PNG?1696517367"
  },
  {
    "chainId": 1,
    "address": "0x1e87d63d11d1c16052bbca06d43ba4ceb4ee686c",
    "name": "BRC on the ERC",
    "symbol": "BRC20",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33534/thumb/brcontheerc.jpg?1702354054"
  },
  {
    "chainId": 1,
    "address": "0x0aa1582bebf8d96ea384b6829a5d41278579cd88",
    "name": "Lyve Finance",
    "symbol": "LYVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34627/thumb/e-lox52v_400x400.jpeg?1705546422"
  },
  {
    "chainId": 1,
    "address": "0x60c24407d01782c2175d32fe7c8921ed732371d1",
    "name": "LemoChain",
    "symbol": "LEMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3108/thumb/lemochain.png?1696503836"
  },
  {
    "chainId": 1,
    "address": "0x1b6e9c73bee68102d9dd4a2627f97bff4183ab0a",
    "name": "OpenLeverage",
    "symbol": "OLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26098/thumb/256x256_OLE_Token_Logo.png?1696525189"
  },
  {
    "chainId": 1,
    "address": "0x06a01a4d579479dd5d884ebf61a31727a3d8d442",
    "name": "Skey Network",
    "symbol": "SKEY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13425/thumb/SKEY_Network_logo_black.png?1696513185"
  },
  {
    "chainId": 1,
    "address": "0xdf347911910b6c9a4286ba8e2ee5ea4a39eb2134",
    "name": "Bob s Repair",
    "symbol": "BOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3550/thumb/PNG_BOB_Token_Large.png?1696504237"
  },
  {
    "chainId": 1,
    "address": "0x0b4bdc478791897274652dc15ef5c135cae61e60",
    "name": "DAEX",
    "symbol": "DAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4063/thumb/daex.png?1696504696"
  },
  {
    "chainId": 1,
    "address": "0x0557e0d15aec0b9026dd17aa874fdf7d182a2ceb",
    "name": "CFX Quantum",
    "symbol": "CFXQ",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/15031/thumb/CFXQ_TOKEN_LOGO_200x200.png?1696514691"
  },
  {
    "chainId": 1,
    "address": "0x4c6ec08cf3fc987c6c4beb03184d335a2dfc4042",
    "name": "MurAll",
    "symbol": "PAINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14103/thumb/paint_logo_200x200.png?1696513825"
  },
  {
    "chainId": 1,
    "address": "0x0335a7610d817aeca1bebbefbd392ecc2ed587b8",
    "name": "Nitro League",
    "symbol": "NITRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21668/thumb/_X6vYBDM_400x400.jpg?1696521025"
  },
  {
    "chainId": 1,
    "address": "0x6a969d379700b2e5ea4e684d273d63c1c050ba49",
    "name": "Pacific",
    "symbol": "PAF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20915/thumb/paf.png?1696520306"
  },
  {
    "chainId": 1,
    "address": "0xd71ecff9342a5ced620049e616c5035f1db98620",
    "name": "sEUR",
    "symbol": "SEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13637/thumb/sEUR.png?1696513385"
  },
  {
    "chainId": 1,
    "address": "0x78132543d8e20d2417d8a07d9ae199d458a0d581",
    "name": "Luna Inu",
    "symbol": "LINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25623/thumb/output-onlinepngtools.png?1696524757"
  },
  {
    "chainId": 1,
    "address": "0x6d614686550b9e1c1df4b2cd8f91c9d4df66c810",
    "name": "Skeb",
    "symbol": "SKEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27155/thumb/skebcoin-cmc.png?1696526206"
  },
  {
    "chainId": 1,
    "address": "0x46d0dac0926fa16707042cadc23f1eb4141fe86b",
    "name": "SONM",
    "symbol": "SNM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/861/thumb/sonm.png?1696501995"
  },
  {
    "chainId": 1,
    "address": "0xc221b7e65ffc80de234bbb6667abdd46593d34f0",
    "name": "Wrapped Centrifuge",
    "symbol": "WCFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17106/thumb/WCFG.jpg?1696516667"
  },
  {
    "chainId": 1,
    "address": "0xcda2f16c6aa895d533506b426aff827b709c87f5",
    "name": "Fairum",
    "symbol": "FAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13894/thumb/logo-65ce48ea8d2b64b6478a42c0050214e2.png?1696513637"
  },
  {
    "chainId": 1,
    "address": "0xaf9f549774ecedbd0966c52f250acc548d3f36e5",
    "name": "RioDeFi",
    "symbol": "RFUEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12623/thumb/RFUEL_SQR.png?1696512430"
  },
  {
    "chainId": 1,
    "address": "0x444444444444c1a66f394025ac839a535246fcc8",
    "name": "Genius",
    "symbol": "GENI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28621/thumb/GENI200x200.png?1696527606"
  },
  {
    "chainId": 1,
    "address": "0x85b6acaba696b9e4247175274f8263f99b4b9180",
    "name": "Spiral",
    "symbol": "SPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33300/thumb/logo6s1_4_%281%29.png?1701403013"
  },
  {
    "chainId": 1,
    "address": "0x29ceddcf0da3c1d8068a7dfbd0fb06c2e438ff70",
    "name": "Freela",
    "symbol": "FREL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15856/thumb/freela.PNG?1696515472"
  },
  {
    "chainId": 1,
    "address": "0x50de6856358cc35f3a9a57eaaa34bd4cb707d2cd",
    "name": "Razor Network",
    "symbol": "RAZOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13797/thumb/icon.png?1696513545"
  },
  {
    "chainId": 1,
    "address": "0x9b00e6e8d787b13756eb919786c9745054db64f9",
    "name": "Sienna  ERC 20 ",
    "symbol": "WSIENNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15420/thumb/sienna.jpeg?1696515066"
  },
  {
    "chainId": 1,
    "address": "0xa13f0743951b4f6e3e3aa039f682e17279f52bc3",
    "name": "Sentinel Chain",
    "symbol": "SENC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2960/thumb/sentinel-chain.jpg?1696503703"
  },
  {
    "chainId": 1,
    "address": "0xa1e349fac47e50b42cd323c4285ef4622b60a5e0",
    "name": "Pepy coin",
    "symbol": "PEPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36966/thumb/pepy_cr_%281%29.jpg?1712908045"
  },
  {
    "chainId": 1,
    "address": "0x6c8c6b02e7b2be14d4fa6022dfd6d75921d90e4e",
    "name": "cBAT",
    "symbol": "CBAT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10645/thumb/cBAT1.JPG?1696510619"
  },
  {
    "chainId": 1,
    "address": "0x15b543e986b8c34074dfc9901136d9355a537e7e",
    "name": "Student Coin",
    "symbol": "STC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6260/thumb/logo_in_circle_%281%29.png?1696506642"
  },
  {
    "chainId": 1,
    "address": "0xaae3cf9968d26925bdb73ce3864e0084a20f4687",
    "name": "Farmland Protocol",
    "symbol": "FAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13497/thumb/70059912.jpg?1696513260"
  },
  {
    "chainId": 1,
    "address": "0xc25a3a3b969415c80451098fa907ec722572917f",
    "name": "LP sCurve",
    "symbol": "SCURVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11899/thumb/Curvefi_sCrv_32.png?1696511765"
  },
  {
    "chainId": 1,
    "address": "0xb3319f5d18bc0d84dd1b4825dcde5d5f7266d407",
    "name": "c0x",
    "symbol": "CZRX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10644/thumb/czrx1.JPG?1696510618"
  },
  {
    "chainId": 1,
    "address": "0xa8b0f154a688c22142e361707df64277e0a0be66",
    "name": "Rake Finance",
    "symbol": "RAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13078/thumb/rake-logo-200x200.png?1696512866"
  },
  {
    "chainId": 1,
    "address": "0x23352036e911a22cfc692b5e2e196692658aded9",
    "name": "Friendz",
    "symbol": "FDZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3685/thumb/download_%2812%29.png?1696504361"
  },
  {
    "chainId": 1,
    "address": "0xab846fb6c81370327e784ae7cbb6d6a6af6ff4bf",
    "name": "Paladin",
    "symbol": "PAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24558/thumb/Circle-Logo.png?1696523734"
  },
  {
    "chainId": 1,
    "address": "0xa64bd6c70cb9051f6a9ba1f163fdc07e0dfb5f84",
    "name": "Aave LINK v1",
    "symbol": "ALINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11729/thumb/aLINK.png?1696511610"
  },
  {
    "chainId": 1,
    "address": "0xad5fe5b0b8ec8ff4565204990e4405b2da117d8e",
    "name": "TronClassic",
    "symbol": "TRXC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/4626/thumb/trxc.png?1696505197"
  },
  {
    "chainId": 1,
    "address": "0xe5b826ca2ca02f09c1725e9bd98d9a8874c30532",
    "name": "ZEON Network",
    "symbol": "ZEON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4247/thumb/XZqXYc2j_400x400.jpg?1696504861"
  },
  {
    "chainId": 1,
    "address": "0x1f8a626883d7724dbd59ef51cbd4bf1cf2016d13",
    "name": "Jigstack",
    "symbol": "STAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14978/thumb/jigstack.PNG?1696514641"
  },
  {
    "chainId": 1,
    "address": "0x6a4c76874e686a7d080d173987a35a9c48905583",
    "name": "Luxurious Pro Network",
    "symbol": "LPNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14126/thumb/logo_%2894%29.png?1696513847"
  },
  {
    "chainId": 1,
    "address": "0xcbcc0f036ed4788f63fc0fee32873d6a7487b908",
    "name": "Humaniq",
    "symbol": "HMQ",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/647/thumb/humaniq.jpg?1696501841"
  },
  {
    "chainId": 1,
    "address": "0xad78d154baec2e9b4e78182d02388981b5093f80",
    "name": "Soyjak",
    "symbol": "SOY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33041/thumb/soyjak256_%282%29.png?1700428317"
  },
  {
    "chainId": 1,
    "address": "0x05d3606d5c81eb9b7b18530995ec9b29da05faba",
    "name": "TomoChain ERC 20",
    "symbol": "TOMOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12646/thumb/tomoe_logo.png?1696512453"
  },
  {
    "chainId": 1,
    "address": "0xb2617246d0c6c0087f18703d576831899ca94f01",
    "name": "Zignaly",
    "symbol": "ZIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14796/thumb/zignaly.jpg?1704292004"
  },
  {
    "chainId": 1,
    "address": "0x03042482d64577a7bdb282260e2ea4c8a89c064b",
    "name": "Centaur",
    "symbol": "CNTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12743/thumb/logo_%2898%29.png?1696512542"
  },
  {
    "chainId": 1,
    "address": "0xb67718b98d52318240c52e71a898335da4a28c42",
    "name": "Innovative Bioresearch Coin",
    "symbol": "INNBC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/4043/thumb/INNBC.png?1696504678"
  },
  {
    "chainId": 1,
    "address": "0x0cdf9acd87e940837ff21bb40c9fd55f68bba059",
    "name": "Public Mint",
    "symbol": "MINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14106/thumb/Public_Mint_Icon_200x200.png?1696513827"
  },
  {
    "chainId": 1,
    "address": "0x0fd10b9899882a6f2fcb5c371e17e70fdee00c38",
    "name": "Pundi X",
    "symbol": "PUNDIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14571/thumb/vDyefsXq_400x400.jpg?1696514252"
  },
  {
    "chainId": 1,
    "address": "0x4bcea5e4d0f6ed53cf45e7a28febb2d3621d7438",
    "name": "Modex",
    "symbol": "MODEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2851/thumb/LhWIeAg.png?1696503608"
  },
  {
    "chainId": 1,
    "address": "0xf0d33beda4d734c72684b5f9abbebf715d0a7935",
    "name": "NuNet",
    "symbol": "NTX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/20950/thumb/8Zb2W2Wi_400x400.png?1696520338"
  },
  {
    "chainId": 1,
    "address": "0x3f5294df68f871241c4b18fcf78ebd8ac18ab654",
    "name": "99Starz",
    "symbol": "STZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21467/thumb/stz.png?1696520828"
  },
  {
    "chainId": 1,
    "address": "0x6aeb95f06cda84ca345c2de0f3b7f96923a44f4c",
    "name": "Rentberry",
    "symbol": "BERRY",
    "decimals": 14,
    "logoURI": "https://assets.coingecko.com/coins/images/1787/thumb/rentberry.png?1696502792"
  },
  {
    "chainId": 1,
    "address": "0x7e9e431a0b8c4d532c745b1043c7fa29a48d4fba",
    "name": "eosDAC",
    "symbol": "EOSDAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3878/thumb/2644.png?1696504539"
  },
  {
    "chainId": 1,
    "address": "0xaf91e8afbe87642dc628786188a54b78580a4d76",
    "name": "Fund Of Yours",
    "symbol": "FOY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16648/thumb/foy200.png?1696516209"
  },
  {
    "chainId": 1,
    "address": "0x159a1dfae19057de57dfffcbb3da1ae784678965",
    "name": "Reflex",
    "symbol": "RFX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12745/thumb/2MKGMRCT_400x400.png?1696512543"
  },
  {
    "chainId": 1,
    "address": "0xc7bba5b765581efb2cdd2679db5bea9ee79b201f",
    "name": "Gems",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3945/thumb/Webp.net-resizeimage_%2817%29.jpg?1696504590"
  },
  {
    "chainId": 1,
    "address": "0x9e10f61749c4952c320412a6b26901605ff6da1d",
    "name": "Theos",
    "symbol": "THEOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18150/thumb/theos_logo.png?1696517652"
  },
  {
    "chainId": 1,
    "address": "0xada86b1b313d1d5267e3fc0bb303f0a2b66d0ea7",
    "name": "Covesting",
    "symbol": "COV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1950/thumb/covesting.png?1696502933"
  },
  {
    "chainId": 1,
    "address": "0x4a220e6096b25eadb88358cb44068a3248254675",
    "name": "Quant",
    "symbol": "QNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3370/thumb/5ZOu7brX_400x400.jpg?1696504070"
  },
  {
    "chainId": 1,
    "address": "0x9d9e399e5385e2b9a58d4f775a1e16441b571afb",
    "name": "Metano",
    "symbol": "METANO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28183/thumb/logo200x.jpeg?1696527186"
  },
  {
    "chainId": 1,
    "address": "0xadd5e881984783dd432f80381fb52f45b53f3e70",
    "name": "Vite",
    "symbol": "VITE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4513/thumb/VITE_new_logo_200px.png?1709175957"
  },
  {
    "chainId": 1,
    "address": "0x5b322514ff727253292637d9054301600c2c81e8",
    "name": "DAD",
    "symbol": "DAD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/9833/thumb/q_Vt0ajV_400x400.jpg?1696509888"
  },
  {
    "chainId": 1,
    "address": "0x006bea43baa3f7a6f765f14f10a1a1b08334ef45",
    "name": "Stox",
    "symbol": "STX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1230/thumb/stox-token.png?1696502307"
  },
  {
    "chainId": 1,
    "address": "0xb81408a1cc2f4be70a6a3178d351ca95a77c5a06",
    "name": "Xodex",
    "symbol": "XODEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25551/thumb/XODEX_Avatar_02.png?1711716939"
  },
  {
    "chainId": 1,
    "address": "0x31e6506751437428ae8f2e15ec4a3d2e33d4f8a4",
    "name": "AePos Network",
    "symbol": "AEPOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35402/thumb/logo_%284%29.png?1708494833"
  },
  {
    "chainId": 1,
    "address": "0x23ddbd36547d43627afa9b42d4e9fb0515f48b09",
    "name": "hiBEANZ",
    "symbol": "HIBEANZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29281/thumb/hibeanz.png?1696528234"
  },
  {
    "chainId": 1,
    "address": "0x228ba514309ffdf03a81a205a6d040e429d6e80c",
    "name": "Global Social Chain",
    "symbol": "GSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4304/thumb/global-social-chain.png?1696504911"
  },
  {
    "chainId": 1,
    "address": "0x7eaf9c89037e4814dc0d9952ac7f888c784548db",
    "name": "Royale",
    "symbol": "ROYA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13602/thumb/roya.png?1696513353"
  },
  {
    "chainId": 1,
    "address": "0x9b4e2b4b13d125238aa0480dd42b4f6fc71b37cc",
    "name": "MyToken",
    "symbol": "MT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4109/thumb/2712.png?1696504736"
  },
  {
    "chainId": 1,
    "address": "0x3ebb4a4e91ad83be51f8d596533818b246f4bee1",
    "name": "Signata",
    "symbol": "SATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14704/thumb/logo.png?1696514375"
  },
  {
    "chainId": 1,
    "address": "0xa8b12cc90abf65191532a12bb5394a714a46d358",
    "name": "pBTC35A",
    "symbol": "PBTC35A",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13653/thumb/pBTC35A.png?1696513402"
  },
  {
    "chainId": 1,
    "address": "0xa891cf72aeda692570928efe1a832342e9783cdc",
    "name": "Interfinex Bills",
    "symbol": "IFEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13346/thumb/U6uYZry.png?1696513113"
  },
  {
    "chainId": 1,
    "address": "0x8b937af714ac7e2129bd33d93641f52b665ca352",
    "name": "JizzRocket",
    "symbol": "JIZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28985/thumb/bLQvXuh_%281%29.png?1696527958"
  },
  {
    "chainId": 1,
    "address": "0xc08512927d12348f6620a698105e1baac6ecd911",
    "name": "GYEN",
    "symbol": "GYEN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14191/thumb/icon_gyen_200_200.png?1696513909"
  },
  {
    "chainId": 1,
    "address": "0x47481c1b44f2a1c0135c45aa402ce4f4dde4d30e",
    "name": "Meetple",
    "symbol": "MPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14323/thumb/Tqlsvhn1_400x400.png?1696514012"
  },
  {
    "chainId": 1,
    "address": "0x7240ac91f01233baaf8b064248e80feaa5912ba3",
    "name": "OctoFi",
    "symbol": "OCTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12594/thumb/octofi-256x256-radius-22percent.png?1696512403"
  },
  {
    "chainId": 1,
    "address": "0xbc4171f45ef0ef66e76f979df021a34b46dcc81d",
    "name": "Dora Factory  OLD ",
    "symbol": "DORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14478/thumb/dora_logo.jpg?1696514164"
  },
  {
    "chainId": 1,
    "address": "0x3a8d5bc8a8948b68dfc0ce9c14ac4150e083518c",
    "name": "Paralink Network",
    "symbol": "PARA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15789/thumb/para.PNG?1696515412"
  },
  {
    "chainId": 1,
    "address": "0x4abb9cc67bd3da9eb966d1159a71a0e68bd15432",
    "name": "KelVPN",
    "symbol": "KEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14851/thumb/logokelvpn200x200.png?1696514517"
  },
  {
    "chainId": 1,
    "address": "0xa21af1050f7b26e0cff45ee51548254c41ed6b5c",
    "name": "Osaka Protocol",
    "symbol": "OSAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30911/thumb/osak_logo.png?1696529756"
  },
  {
    "chainId": 1,
    "address": "0xdb2f2bcce3efa95eda95a233af45f3e0d4f00e2a",
    "name": "Aegis",
    "symbol": "AGS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12391/thumb/logo-3.png?1696512214"
  },
  {
    "chainId": 1,
    "address": "0xe8e06a5613dc86d459bc8fb989e173bb8b256072",
    "name": "Feyorra",
    "symbol": "FEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13689/thumb/1_XiKKk5_400x400.jpg?1696513436"
  },
  {
    "chainId": 1,
    "address": "0x62199b909fb8b8cf870f97bef2ce6783493c4908",
    "name": "pTokens BTC",
    "symbol": "PBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25861/thumb/wMTpRljt_400x400.png?1696524945"
  },
  {
    "chainId": 1,
    "address": "0xd945d2031b4c63c0e363304fb771f709b502dc0a",
    "name": "BountyMarketCap",
    "symbol": "BMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13472/thumb/bmc-logo-200.png?1696513233"
  },
  {
    "chainId": 1,
    "address": "0xc834fa996fa3bec7aad3693af486ae53d8aa8b50",
    "name": "Convergence",
    "symbol": "CONV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14519/thumb/Convergence_Logo_%28Final%29.png?1696514204"
  },
  {
    "chainId": 1,
    "address": "0x624d520bab2e4ad83935fa503fb130614374e850",
    "name": "Smartshare",
    "symbol": "SSP",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/4642/thumb/smartshare.png?1696505210"
  },
  {
    "chainId": 1,
    "address": "0x5228a22e72ccc52d415ecfd199f99d0665e7733b",
    "name": "pTokens BTC  OLD ",
    "symbol": "PBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10805/thumb/J51iIea.png?1696510764"
  },
  {
    "chainId": 1,
    "address": "0xaec7d1069e3a914a3eb50f0bfb1796751f2ce48a",
    "name": "S4FE",
    "symbol": "S4F",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7405/thumb/logo_%284%29.png?1696507681"
  },
  {
    "chainId": 1,
    "address": "0x76960dccd5a1fe799f7c29be9f19ceb4627aeb2f",
    "name": "Red",
    "symbol": "RED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4523/thumb/red.png?1696505107"
  },
  {
    "chainId": 1,
    "address": "0xadb2437e6f65682b85f814fbc12fec0508a7b1d0",
    "name": "UNCX Network",
    "symbol": "UNCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12871/thumb/uncx.png?1696512659"
  },
  {
    "chainId": 1,
    "address": "0x62359ed7505efc61ff1d56fef82158ccaffa23d7",
    "name": "cVault finance",
    "symbol": "CORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12635/thumb/cvault.finance_logo.png?1601353499"
  },
  {
    "chainId": 1,
    "address": "0x068e3563b1c19590f822c0e13445c4fa1b9eefa5",
    "name": "Wrapped USD",
    "symbol": "WUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28854/thumb/wusd.png?1696527828"
  },
  {
    "chainId": 1,
    "address": "0x20fcefa41045080764c48c2b9429e44c644e5dea",
    "name": "Foox  Ordinals ",
    "symbol": "FOOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33692/thumb/Foox_Logo.png?1702796923"
  },
  {
    "chainId": 1,
    "address": "0x0f8c45b896784a1e408526b9300519ef8660209c",
    "name": "XMax",
    "symbol": "XMX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4643/thumb/xmx.png?1696505211"
  },
  {
    "chainId": 1,
    "address": "0x446c9033e7516d820cc9a2ce2d0b7328b579406f",
    "name": "SOLVE",
    "symbol": "SOLVE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1768/thumb/Solve.Token_logo_200_200_wiyhout_BG.png?1696502776"
  },
  {
    "chainId": 1,
    "address": "0x5afff9876c1f98b7d2b53bcb69eb57e92408319f",
    "name": "metavisa",
    "symbol": "MESA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23246/thumb/twitteravatar_circle.png?1696522466"
  },
  {
    "chainId": 1,
    "address": "0x8793fb615eb92822f482f88b3137b00aad4c00d2",
    "name": "revoAI",
    "symbol": "REVOAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28952/thumb/roveAI1_%283%29.png?1696527925"
  },
  {
    "chainId": 1,
    "address": "0x0b9ae6b1d4f0eeed904d1cef68b9bd47499f3fff",
    "name": "IlluminatiCoin",
    "symbol": "NATI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32399/thumb/CMCIcon.png?1698055172"
  },
  {
    "chainId": 1,
    "address": "0xb2e96a63479c2edd2fd62b382c89d5ca79f572d3",
    "name": "Zenon",
    "symbol": "ZNN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30753/thumb/zenon.jpg?1696529622"
  },
  {
    "chainId": 1,
    "address": "0xe530441f4f73bdb6dc2fa5af7c3fc5fd551ec838",
    "name": "GSENetwork",
    "symbol": "GSE",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/4776/thumb/gsenetwork.png?1696505333"
  },
  {
    "chainId": 1,
    "address": "0xccf4429db6322d5c611ee964527d42e5d685dd6a",
    "name": "cWBTC",
    "symbol": "CWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10823/thumb/cwbtc.png?1696510780"
  },
  {
    "chainId": 1,
    "address": "0x62b9c7356a2dc64a1969e19c23e4f579f9810aa7",
    "name": "Convex CRV",
    "symbol": "CVXCRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15586/thumb/convex-crv.png?1696515222"
  },
  {
    "chainId": 1,
    "address": "0x3a4cab3dcfab144fe7eb2b5a3e288cc03dc07659",
    "name": "OnGo",
    "symbol": "FTG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20154/thumb/IMG-20220728-WA0004.jpg?1696519568"
  },
  {
    "chainId": 1,
    "address": "0xe1ba0fb44ccb0d11b80f92f4f8ed94ca3ff51d00",
    "name": "Aave BAT v1",
    "symbol": "ABAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11726/thumb/aBAT.png?1696511608"
  },
  {
    "chainId": 1,
    "address": "0xe1bad922f84b198a08292fb600319300ae32471b",
    "name": "Firmachain",
    "symbol": "FCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9892/thumb/6mHcLurm_400x400.jpg?1696509941"
  },
  {
    "chainId": 1,
    "address": "0xd559f20296ff4895da39b5bd9add54b442596a61",
    "name": "FintruX",
    "symbol": "FTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1928/thumb/token-500x500.png?1696502917"
  },
  {
    "chainId": 1,
    "address": "0xbea0000029ad1c77d3d5d23ba2d8893db9d1efab",
    "name": "Bean",
    "symbol": "BEAN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/18447/thumb/bean-logo-coingecko.png?1696517934"
  },
  {
    "chainId": 1,
    "address": "0x38e68a37e401f7271568cecaac63c6b1e19130b4",
    "name": "Banana Gun",
    "symbol": "BANANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31744/thumb/bgg.jpeg?1696530563"
  },
  {
    "chainId": 1,
    "address": "0x69948cc03f478b95283f7dbf1ce764d0fc7ec54c",
    "name": "Aave REN v1",
    "symbol": "AREN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12483/thumb/aREN.png?1696512300"
  },
  {
    "chainId": 1,
    "address": "0xca3ea3061d638e02113aa960340c98343b5acd62",
    "name": "Bittwatt",
    "symbol": "BWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4777/thumb/bittwatt.png?1547274101"
  },
  {
    "chainId": 1,
    "address": "0x4e4a47cac6a28a62dcc20990ed2cda9bc659469f",
    "name": "I will poop it NFT",
    "symbol": "SHIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25945/thumb/PCNoOl8.png?1696525024"
  },
  {
    "chainId": 1,
    "address": "0xfc1e690f61efd961294b3e1ce3313fbd8aa4f85d",
    "name": "Aave DAI v1",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10843/thumb/aDAI.png?1696510799"
  },
  {
    "chainId": 1,
    "address": "0x6fb0855c404e09c47c3fbca25f08d4e41f9f062f",
    "name": "Aave ZRX v1",
    "symbol": "AZRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11735/thumb/aZRX.png?1696511616"
  },
  {
    "chainId": 1,
    "address": "0x328c4c80bc7aca0834db37e6600a6c49e12da4de",
    "name": "Aave SNX v1",
    "symbol": "ASNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11733/thumb/aSNX.png?1696511614"
  },
  {
    "chainId": 1,
    "address": "0x996229d0c6a485c7f4b52e092eaa907cb2def5c6",
    "name": "BuckHath Coin",
    "symbol": "BHIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7821/thumb/logstex.PNG?1585121237"
  },
  {
    "chainId": 1,
    "address": "0x916c5de09cf63f6602d1e1793fb41f6437814a62",
    "name": "JACY",
    "symbol": "JACY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19740/thumb/tBwbvjZK_400x400.jpg?1696519164"
  },
  {
    "chainId": 1,
    "address": "0x30f271c9e86d2b7d00a6376cd96a1cfbd5f0b9b3",
    "name": "Decentr",
    "symbol": "DEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11816/thumb/Decentr.png?1696511689"
  },
  {
    "chainId": 1,
    "address": "0x03ab458634910aad20ef5f1c8ee96f1d6ac54919",
    "name": "Rai Reflex Index",
    "symbol": "RAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14004/thumb/RAI-logo-coin.png?1696513733"
  },
  {
    "chainId": 1,
    "address": "0xe8ff5c9c75deb346acac493c463c8950be03dfba",
    "name": "VIBE",
    "symbol": "VIBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/948/thumb/vibe.png?1696502064"
  },
  {
    "chainId": 1,
    "address": "0x471d113059324321749e097705197a2b44a070fc",
    "name": "Kanga Exchange",
    "symbol": "KNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21188/thumb/KNG_logo-200.png?1696520564"
  },
  {
    "chainId": 1,
    "address": "0x8edc6f7d2f23c10653972e611f707ce0562d61b1",
    "name": "MultiDEX AI",
    "symbol": "MDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35403/thumb/Multidex.jpg?1708499023"
  },
  {
    "chainId": 1,
    "address": "0xb8baa0e4287890a5f79863ab62b7f175cecbd433",
    "name": "Swerve",
    "symbol": "SWRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12361/thumb/swerve.png?1696512186"
  },
  {
    "chainId": 1,
    "address": "0x4fe83213d56308330ec302a8bd641f1d0113a4cc",
    "name": "NuCypher",
    "symbol": "NU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3318/thumb/photo1198982838879365035.jpg?1696504026"
  },
  {
    "chainId": 1,
    "address": "0xda31d0d1bc934fc34f7189e38a413ca0a5e8b44f",
    "name": "BitStable Finance",
    "symbol": "BSSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33545/thumb/07LKCLMU_400x400.jpg?1705046428"
  },
  {
    "chainId": 1,
    "address": "0x4db26817c88a57d2ef84cfd90168f535ee83bda9",
    "name": "hiAZUKI",
    "symbol": "HIAZUKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28565/thumb/hiazuki.png?1696527554"
  },
  {
    "chainId": 1,
    "address": "0x2dd1b9410c73e16b60240e529e38e30425f00d4e",
    "name": "Kay Pacha",
    "symbol": "PACHA",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/27194/thumb/0_Lzf-8HxwOnPjYWi-.jpeg?1696526243"
  },
  {
    "chainId": 1,
    "address": "0x5dc60c4d5e75d22588fa17ffeb90a63e535efce0",
    "name": "dKargo",
    "symbol": "DKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11875/thumb/bVD0g0dlmrEOPIkt943KZIBZ086eCshyY0jIQFti4zxYdOlFltU8tKa6uJlcA14HvNjX4bc7dxdMvlpoW5NFMND85oG4aiiCbFRhI6eowDfKEBY3BoSVY0IrBbA9SFGIxh_IYrkNC5uNdG-roZ0_TlGO3098now6Tbzga0p4IDqVk6lnaX3TuRC7pgnAYWZM15RD-uEIHr3O_3OoIIWP-.jpg?1696511743"
  },
  {
    "chainId": 1,
    "address": "0x943ed852dadb5c3938ecdc6883718df8142de4c8",
    "name": "FansTime",
    "symbol": "FTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4952/thumb/fanstime.png?1696505490"
  },
  {
    "chainId": 1,
    "address": "0x3802c218221390025bceabbad5d8c59f40eb74b8",
    "name": "Guarded Ether",
    "symbol": "GETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14001/thumb/guarda-shield-logo.png?1696513730"
  },
  {
    "chainId": 1,
    "address": "0xf6ce4be313ead51511215f1874c898239a331e37",
    "name": "Bird Dog",
    "symbol": "BIRDDOG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36707/thumb/200x.jpg?1712126632"
  },
  {
    "chainId": 1,
    "address": "0x990e081a7b7d3ccba26a2f49746a68cc4ff73280",
    "name": "KStarCoin",
    "symbol": "KSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1630/thumb/ksc.png?1696502658"
  },
  {
    "chainId": 1,
    "address": "0xbc0e2969d23a084a0bb65a2b9a4242e7f9ea65b6",
    "name": "Clippy AI",
    "symbol": "CLIPPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33124/thumb/PHOTO-2023-11-10-23-46-27.jpg?1700794900"
  },
  {
    "chainId": 1,
    "address": "0x5c59a5b139b0538cb106d775a022cad98dd14b5a",
    "name": "XREATORS",
    "symbol": "ORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25678/thumb/XREATORS_ORT_Logo.png?1696524806"
  },
  {
    "chainId": 1,
    "address": "0xd26114cd6ee289accf82350c8d8487fedb8a0c07",
    "name": "OMG Network",
    "symbol": "OMG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/776/thumb/OMG_Network.jpg?1696501928"
  },
  {
    "chainId": 1,
    "address": "0xf8c3527cc04340b208c854e985240c02f7b7793f",
    "name": "Frontier",
    "symbol": "FRONT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12479/thumb/frontier_logo.png?1696512296"
  },
  {
    "chainId": 1,
    "address": "0x28fda76721a8077a5de802ab0212849b8c38429e",
    "name": "Artemis Vision",
    "symbol": "ARV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23716/thumb/uUMU6_Cr_400x400.jpg?1696522916"
  },
  {
    "chainId": 1,
    "address": "0x6e98e5401adcb0d76f4debfc3d794b3031f48790",
    "name": "Aurix",
    "symbol": "AUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14372/thumb/gold-brown-2.png?1696514064"
  },
  {
    "chainId": 1,
    "address": "0x6cacdb97e3fc8136805a9e7c342d866ab77d0957",
    "name": "Swapr",
    "symbol": "SWPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18740/thumb/swapr.jpg?1696518206"
  },
  {
    "chainId": 1,
    "address": "0xebb4c56dd5490156ae94a96e11903a2d5ff81deb",
    "name": "GMBot",
    "symbol": "GMBT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32823/thumb/SD6IB_md_400x400.jpg?1699582313"
  },
  {
    "chainId": 1,
    "address": "0x6ad2d2c22bb58ea94be18cff11ef67e8bb97b652",
    "name": "Trestle Wrapped TIA",
    "symbol": "WTIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36385/thumb/trestle.jpeg?1711349480"
  },
  {
    "chainId": 1,
    "address": "0x299a1503e88433c0fd1bd68625c25c5a703eb64f",
    "name": "TEAR",
    "symbol": "TEAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35416/thumb/tearcoin.png?1708507106"
  },
  {
    "chainId": 1,
    "address": "0x419777d3e39aa9b00405724eace5ea57620c9062",
    "name": "PAW",
    "symbol": "PAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32118/thumb/photo1701025138-removebg-preview__5_-removebg-preview.png?1701386426"
  },
  {
    "chainId": 1,
    "address": "0xe3818504c1b32bf1557b16c238b2e01fd3149c17",
    "name": "Pillar",
    "symbol": "PLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/809/thumb/v2logo-1.png?1696501958"
  },
  {
    "chainId": 1,
    "address": "0xef89c37fe9e5c906b404cd7edae4a2992b5d25fa",
    "name": "Time Alliance Guild Time",
    "symbol": "TIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32423/thumb/fav.png?1698118383"
  },
  {
    "chainId": 1,
    "address": "0x0fcbc31c503b4a9ed90e87f8ff46c318a4a14260",
    "name": "Quantfury",
    "symbol": "QTF",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14033/thumb/FtSOX9Vy_400x400.jpg?1696513760"
  },
  {
    "chainId": 1,
    "address": "0x37e1160184f7dd29f00b78c050bf13224780b0b0",
    "name": "Yuan Chain Coin",
    "symbol": "YCC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/5328/thumb/G5gCEo-J_400x400.jpg?1696505823"
  },
  {
    "chainId": 1,
    "address": "0x4e12eb8e506ccd1427f6b8f7faa3e88fb698eb28",
    "name": "Jack Token",
    "symbol": "JACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10991/thumb/Jacktoken200x200.png?1696510939"
  },
  {
    "chainId": 1,
    "address": "0x8cc379a292a47cb8406fb1bd8a6d98f442275f0e",
    "name": "Uranium3o8",
    "symbol": "U",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32571/thumb/Logo_Black_Background.png?1698608787"
  },
  {
    "chainId": 1,
    "address": "0x39ea10e507720783c27edd5f96bf2d6e199579b8",
    "name": "DeltaFlare",
    "symbol": "HONR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28059/thumb/DF_Logo_200x200.png?1696527071"
  },
  {
    "chainId": 1,
    "address": "0x31c2415c946928e9fd1af83cdfa38d3edbd4326f",
    "name": "MADworld",
    "symbol": "UMAD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/20939/thumb/UMAD_.png?1696520328"
  },
  {
    "chainId": 1,
    "address": "0xd377f28245bc505190c8f34d2bfe5f215754f634",
    "name": "0xLiquidity",
    "symbol": "0XLP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35367/thumb/Logo_MAIN.jpg?1708400938"
  },
  {
    "chainId": 1,
    "address": "0x2565ae0385659badcada1031db704442e1b69982",
    "name": "Assemble Protocol",
    "symbol": "ASM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11605/thumb/gpvrlkSq_400x400_%281%29.jpg?1696511500"
  },
  {
    "chainId": 1,
    "address": "0x7ecbb21346c501fd07eb165e406120fa32381c16",
    "name": "Ecoreal Estate",
    "symbol": "ECOREAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6431/thumb/golden_leave_in_200_x_200.png?1696506796"
  },
  {
    "chainId": 1,
    "address": "0x07e3c70653548b04f0a75970c1f81b4cbbfb606f",
    "name": "Agrello",
    "symbol": "DLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/900/thumb/delta_200x200.png?1696502023"
  },
  {
    "chainId": 1,
    "address": "0x23fa3aa82858e7ad1f0f04352f4bb7f5e1bbfb68",
    "name": "Frictionless",
    "symbol": "FRIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36783/thumb/Fric_logo_200X200.png?1712331864"
  },
  {
    "chainId": 1,
    "address": "0xe02f72be83855c9e400fd9cd9f83158abfc87053",
    "name": "CHECKMATE",
    "symbol": "CMBOT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/33348/thumb/checkmate200x200.png?1701561194"
  },
  {
    "chainId": 1,
    "address": "0xcdbb2498fa9e7b5849bed5d3661386d0ce2733b2",
    "name": "FuturesAI",
    "symbol": "FAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36830/thumb/photo_2024-04-03_22.01.57.jpeg?1712557619"
  },
  {
    "chainId": 1,
    "address": "0x7d8daff6d70cead12c6f077048552cf89130a2b1",
    "name": "ARCS",
    "symbol": "ARX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10068/thumb/arcs.png?1696510099"
  },
  {
    "chainId": 1,
    "address": "0x6d10e0194400a04600881057787eb9e9c063dca0",
    "name": "Proton Project",
    "symbol": "PRTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14888/thumb/prtn_200_px_.png?1710000932"
  },
  {
    "chainId": 1,
    "address": "0xc7ff1e126cc81e816915ff48c940ed9d4e6d05d6",
    "name": "IjasCoin",
    "symbol": "IJC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14099/thumb/Ijascode-Flat-Logo-2-24-2021.png?1696513821"
  },
  {
    "chainId": 1,
    "address": "0x9aeb50f542050172359a0e1a25a9933bc8c01259",
    "name": "OIN Finance",
    "symbol": "OIN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12339/thumb/OIN_FInance_-_cLogo-01.png?1696512166"
  },
  {
    "chainId": 1,
    "address": "0x0e573ce2736dd9637a0b21058352e1667925c7a8",
    "name": "Verified USD",
    "symbol": "USDV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32948/thumb/usdv_%281%29.png?1699933314"
  },
  {
    "chainId": 1,
    "address": "0xff19138b039d938db46bdda0067dc4ba132ec71c",
    "name": "Snetwork",
    "symbol": "SNET",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/5382/thumb/snetwork.png?1696505867"
  },
  {
    "chainId": 1,
    "address": "0xe8663a64a96169ff4d95b4299e7ae9a76b905b31",
    "name": "DPRating",
    "symbol": "RATING",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/5383/thumb/dprating.png?1696505868"
  },
  {
    "chainId": 1,
    "address": "0x177ba0cac51bfc7ea24bad39d81dcefd59d74faa",
    "name": "KittenFinance",
    "symbol": "KIF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12340/thumb/SnQPkABT_400x400.png?1696512167"
  },
  {
    "chainId": 1,
    "address": "0xfc226294dafb6e69905b3e7635b575d0508a42c5",
    "name": "0xBlack",
    "symbol": "0XB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33301/thumb/oxb_metal.png?1701403803"
  },
  {
    "chainId": 1,
    "address": "0xb1f871ae9462f1b2c6826e88a7827e76f86751d4",
    "name": "GNY",
    "symbol": "GNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5300/thumb/GNY_LOGO_NEW_TRANS.png?1696505799"
  },
  {
    "chainId": 1,
    "address": "0x6468e79a80c0eab0f9a2b574c8d5bc374af59414",
    "name": "e Radix",
    "symbol": "EXRD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13145/thumb/exrd_logo.png?1696512931"
  },
  {
    "chainId": 1,
    "address": "0xec213f83defb583af3a000b1c0ada660b1902a0f",
    "name": "Presearch",
    "symbol": "PRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1299/thumb/presearch.png?1696502369"
  },
  {
    "chainId": 1,
    "address": "0x6eff556748ee452cbdaf31bcb8c76a28651509bd",
    "name": "TiUSD",
    "symbol": "TIUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30697/thumb/TIUDS.png?1696529566"
  },
  {
    "chainId": 1,
    "address": "0x4fb721ef3bf99e0f2c193847afa296b9257d3c30",
    "name": "Tokenplace",
    "symbol": "TOK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/15779/thumb/output-onlinepngtools_%283%29.png?1696515403"
  },
  {
    "chainId": 1,
    "address": "0x7995ab36bb307afa6a683c24a25d90dc1ea83566",
    "name": "HitChain",
    "symbol": "HIT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/5555/thumb/hitchain.png?1696506024"
  },
  {
    "chainId": 1,
    "address": "0xaf05ce8a2cef336006e933c02fc89887f5b3c726",
    "name": "Lockheed Martin Inu",
    "symbol": "LMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32484/thumb/photo_2023-10-11_22-41-52_%281%29_%281%29.jpg?1698290653"
  },
  {
    "chainId": 1,
    "address": "0x408e41876cccdc0f92210600ef50372656052a38",
    "name": "Ren",
    "symbol": "REN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3139/thumb/REN.png?1696503862"
  },
  {
    "chainId": 1,
    "address": "0x3e70f6806171873d17d4bfc984a6f9d20f5a9018",
    "name": "BrianArmstrongTrumpYellenGTA6",
    "symbol": "COIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33839/thumb/0x3e70f6806171873d17d4bfc984a6f9d20f5a9018.png?1703108075"
  },
  {
    "chainId": 1,
    "address": "0x22fc5a29bd3d6cce19a06f844019fd506fce4455",
    "name": "Equilibria Finance ePENDLE",
    "symbol": "EPENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33379/thumb/photo_2023-12-02_19-45-14.jpg?1701665986"
  },
  {
    "chainId": 1,
    "address": "0xa2a54f1ec1f09316ef12c1770d32ed8f21b1fb6a",
    "name": "DigiFinex",
    "symbol": "DFT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4819/thumb/digifinextoken.png?1696505369"
  },
  {
    "chainId": 1,
    "address": "0xd7f5cabdf696d7d1bf384d7688926a4bdb092c67",
    "name": "DRC Mobility",
    "symbol": "DRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13457/thumb/drc_symbol.png?1696513220"
  },
  {
    "chainId": 1,
    "address": "0x80ee5c641a8ffc607545219a3856562f56427fe9",
    "name": "Brett ETH",
    "symbol": "BRETT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36832/thumb/zD21_tw__400x400.jpg?1712559083"
  },
  {
    "chainId": 1,
    "address": "0x474021845c4643113458ea4414bdb7fb74a01a77",
    "name": "Uno Re",
    "symbol": "UNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15073/thumb/c0vbqVE.png?1696514732"
  },
  {
    "chainId": 1,
    "address": "0x07ef9e82721ac16809d24dafbe1792ce01654db4",
    "name": "Chimpion",
    "symbol": "BNANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7522/thumb/banana-token-logo.jpg?1696507789"
  },
  {
    "chainId": 1,
    "address": "0x1a23a6bfbadb59fa563008c0fb7cf96dfcf34ea1",
    "name": "CoFiX",
    "symbol": "COFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12781/thumb/dnPnSkfa_400x400.png?1696512576"
  },
  {
    "chainId": 1,
    "address": "0x93dfaf57d986b9ca77df9376c50878e013d9c7c8",
    "name": "Unique One",
    "symbol": "RARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13458/thumb/Logo_Unique.png?1696513221"
  },
  {
    "chainId": 1,
    "address": "0xb6ed7644c69416d67b522e20bc294a9a9b405b31",
    "name": "0xBitcoin",
    "symbol": "0XBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4454/thumb/0xbtc.png?1696505045"
  },
  {
    "chainId": 1,
    "address": "0x3aada3e213abf8529606924d8d1c55cbdc70bf74",
    "name": "XMON",
    "symbol": "XMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14008/thumb/xmon_logo.png?1696513737"
  },
  {
    "chainId": 1,
    "address": "0x625ae63000f46200499120b906716420bd059240",
    "name": "Aave SUSD v1",
    "symbol": "ASUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11723/thumb/aSUSD.png?1696511605"
  },
  {
    "chainId": 1,
    "address": "0xc07a150ecadf2cc352f5586396e344a6b17625eb",
    "name": "Bio Passport",
    "symbol": "BIOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14167/thumb/logo_%2895%29.png?1696513885"
  },
  {
    "chainId": 1,
    "address": "0x10bc518c32fbae5e38ecb50a612160571bd81e44",
    "name": "VeraOne",
    "symbol": "VRO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11112/thumb/wsBaVF.png?1696511051"
  },
  {
    "chainId": 1,
    "address": "0x3506424f91fd33084466f402d5d97f05f8e3b4af",
    "name": "Chiliz",
    "symbol": "CHZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8834/thumb/CHZ_Token_updated.png?1696508986"
  },
  {
    "chainId": 1,
    "address": "0xb893a8049f250b57efa8c62d51527a22404d7c9a",
    "name": "American Shiba",
    "symbol": "USHIBA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15650/thumb/american_shiba.PNG?1696515282"
  },
  {
    "chainId": 1,
    "address": "0x3b544e6fcf6c8dce9d8b45a4fdf21c9b02f9fda9",
    "name": "Giftedhands",
    "symbol": "GHD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12540/thumb/K-8uHktJ.png?1696512351"
  },
  {
    "chainId": 1,
    "address": "0x8f8221afbb33998d8584a2b05749ba73c37a938a",
    "name": "Request",
    "symbol": "REQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1031/thumb/Request_icon_green.png?1696502140"
  },
  {
    "chainId": 1,
    "address": "0xbed4ab0019ff361d83ddeb74883dac8a70f5ea1e",
    "name": "MerchDAO",
    "symbol": "MRCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14540/thumb/logo_256x256.png?1696514224"
  },
  {
    "chainId": 1,
    "address": "0x52928c95c4c7e934e0efcfab08853a0e4558861d",
    "name": "Hara",
    "symbol": "HART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5138/thumb/hara.jpg?1696505657"
  },
  {
    "chainId": 1,
    "address": "0x3449fc1cd036255ba1eb19d65ff4ba2b8903a69a",
    "name": "Basis Cash",
    "symbol": "BAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13246/thumb/BAC.png?1696513021"
  },
  {
    "chainId": 1,
    "address": "0x75ecb52e403c617679fbd3e77a50f9d10a842387",
    "name": "CSR",
    "symbol": "CSR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25585/thumb/CSR-LOGO-200x200-1.png?1696524717"
  },
  {
    "chainId": 1,
    "address": "0xba745513acebcbb977497c569d4f7d340f2a936b",
    "name": "Mainstream For The Underground",
    "symbol": "MFTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5519/thumb/Mainstream_for_the_underground.png?1696505989"
  },
  {
    "chainId": 1,
    "address": "0x01ff50f8b7f74e4f00580d9596cd3d0d6d6e326f",
    "name": "BnkToTheFuture",
    "symbol": "BFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3468/thumb/bnktothefuture.png?1696504157"
  },
  {
    "chainId": 1,
    "address": "0xa5ef74068d04ba0809b7379dd76af5ce34ab7c57",
    "name": "LunaChow",
    "symbol": "LUCHOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18805/thumb/J-MwYfhD_400x400.jpg?1696518267"
  },
  {
    "chainId": 1,
    "address": "0x70e36f6bf80a52b3b46b3af8e106cc0ed743e8e4",
    "name": "cCOMP",
    "symbol": "CCOMP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12840/thumb/1_z8UrVtod3bme4-J_pXAQQA_2x.png?1696512631"
  },
  {
    "chainId": 1,
    "address": "0x607f4c5bb672230e8672085532f7e901544a7375",
    "name": "iExec RLC",
    "symbol": "RLC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/646/thumb/pL1VuXm.png?1696501840"
  },
  {
    "chainId": 1,
    "address": "0x697ef32b4a3f5a4c39de1cb7563f24ca7bfc5947",
    "name": "Insula",
    "symbol": "ISLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10884/thumb/isla.PNG?1696510837"
  },
  {
    "chainId": 1,
    "address": "0xc64500dd7b0f1794807e67802f8abbf5f8ffb054",
    "name": "Locus Chain",
    "symbol": "LOCUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6903/thumb/32169759_2050428658545017_7287666225423843328_n.jpg?1696507221"
  },
  {
    "chainId": 1,
    "address": "0x08389495d7456e1951ddf7c3a1314a4bfb646d8b",
    "name": "Crypterium",
    "symbol": "CRPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1901/thumb/crypt.png?1696502894"
  },
  {
    "chainId": 1,
    "address": "0xc4c2614e694cf534d407ee49f8e44d125e4681c4",
    "name": "Chain Games",
    "symbol": "CHAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12257/thumb/chain-pfp-logo.png?1696512087"
  },
  {
    "chainId": 1,
    "address": "0xe308a6a13a74fae24f542a6c4c3d6fc2cd8e664a",
    "name": "The Roman Empire",
    "symbol": "ROME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32708/thumb/rome.png?1698986439"
  },
  {
    "chainId": 1,
    "address": "0x2e85ae1c47602f7927bcabc2ff99c40aa222ae15",
    "name": "Katana Inu",
    "symbol": "KATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21872/thumb/Katana_Inu512.png?1696521226"
  },
  {
    "chainId": 1,
    "address": "0x26946ada5ecb57f3a1f91605050ce45c482c9eb1",
    "name": "BitcoinSoV",
    "symbol": "BSOV",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9205/thumb/bsov.png?1696509323"
  },
  {
    "chainId": 1,
    "address": "0xfbcb5cbedeebcc55dcd136d34db1daaf74cf67e8",
    "name": "Animalia",
    "symbol": "ANIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22706/thumb/1_rHlkmYoebSJ_wn0iE_-d-g_2x.png?1696522016"
  },
  {
    "chainId": 1,
    "address": "0x024b6e7dc26f4d5579bdd936f8d7bc31f2339999",
    "name": "Mithril Share",
    "symbol": "MIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13516/thumb/MIS.png?1696513277"
  },
  {
    "chainId": 1,
    "address": "0x41c21693e60fc1a5dbb7c50e54e7a6016aa44c99",
    "name": "SO COL",
    "symbol": "SIMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28760/thumb/Frame_427318605.png?1696527739"
  },
  {
    "chainId": 1,
    "address": "0xa3ad8c7ab6b731045b5b16e3fdf77975c71abe79",
    "name": "Dinerobet",
    "symbol": "DINERO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28248/thumb/CABA1DE1-2CB1-46D3-97BC-20305EA67FBB.png?1696527251"
  },
  {
    "chainId": 1,
    "address": "0x9dfad1b7102d46b1b197b90095b5c4e9f5845bba",
    "name": "Botto",
    "symbol": "BOTTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18892/thumb/bottos_logo.jpg?1696518350"
  },
  {
    "chainId": 1,
    "address": "0xff56cc6b1e6ded347aa0b7676c85ab0b3d08b0fa",
    "name": "Orbs",
    "symbol": "ORBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4630/thumb/Orbs.jpg?1696505200"
  },
  {
    "chainId": 1,
    "address": "0xebd9d99a3982d547c5bb4db7e3b1f9f14b67eb83",
    "name": "Everest",
    "symbol": "ID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5209/thumb/Everest.jpg?1696505719"
  },
  {
    "chainId": 1,
    "address": "0xbadff0ef41d2a68f22de21eabca8a59aaf495cf0",
    "name": "Kabosu Inu",
    "symbol": "KABOSU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28619/thumb/Kabosu_Inu.png?1696527604"
  },
  {
    "chainId": 1,
    "address": "0xf8ad7dfe656188a23e89da09506adf7ad9290d5d",
    "name": "Blocery",
    "symbol": "BLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12095/thumb/Blocery_logo_200x200_.png?1696511941"
  },
  {
    "chainId": 1,
    "address": "0xeed3ae7b0f8b5b9bb8c035a9941382b1822671cd",
    "name": "EveryCoin",
    "symbol": "EVY",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/5629/thumb/TutkUC1x_400x400_%281%29.jpg?1696506092"
  },
  {
    "chainId": 1,
    "address": "0x340d2bde5eb28c1eed91b2f790723e3b160613b7",
    "name": "BLOCKv",
    "symbol": "VEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1266/thumb/blockv.png?1696502339"
  },
  {
    "chainId": 1,
    "address": "0x130914e1b240a7f4c5d460b7d3a2fd3846b576fa",
    "name": "Aureus Nummus Gold",
    "symbol": "ANG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12134/thumb/AureusNummusGold-logo.png?1696511976"
  },
  {
    "chainId": 1,
    "address": "0xd23ac27148af6a2f339bd82d0e3cff380b5093de",
    "name": "Siren",
    "symbol": "SI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13955/thumb/siren_token_200x200.png?1696513692"
  },
  {
    "chainId": 1,
    "address": "0x6adb2e268de2aa1abf6578e4a8119b960e02928f",
    "name": "ShibaDoge",
    "symbol": "SHIBDOGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22018/thumb/5HMWoFW0_400x400.jpg?1696521364"
  },
  {
    "chainId": 1,
    "address": "0x000000007a58f5f58e697e51ab0357bc9e260a04",
    "name": "Concave",
    "symbol": "CNV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24492/thumb/concave.jpg?1696523672"
  },
  {
    "chainId": 1,
    "address": "0xa1a36d3537bbe375cc9694795f663ddc8d516db9",
    "name": "Polinate",
    "symbol": "POLI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18096/thumb/Polinate_Games___Guilds_Elements-05.png?1696517601"
  },
  {
    "chainId": 1,
    "address": "0x32e6c34cd57087abbd59b5a4aecc4cb495924356",
    "name": "BitBase Token",
    "symbol": "BTBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17414/thumb/btbs.PNG?1696516962"
  },
  {
    "chainId": 1,
    "address": "0x4f14cdbd815b79e9624121f564f24685c6b1211b",
    "name": "Angry Doge",
    "symbol": "ANFD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25921/thumb/angry-doge-anfd.png?1696525001"
  },
  {
    "chainId": 1,
    "address": "0x5d43b66da68706d39f6c97f7f1415615672b446b",
    "name": "ROGin AI",
    "symbol": "ROG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25922/thumb/20490.png?1696525002"
  },
  {
    "chainId": 1,
    "address": "0x3b604747ad1720c01ded0455728b62c0d2f100f0",
    "name": "WAGMI Games",
    "symbol": "WAGMIGAMES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25906/thumb/WAGMI_DEFENSE_LOGO24_coingecko.png?1708591845"
  },
  {
    "chainId": 1,
    "address": "0x4c9edd5852cd905f086c759e8383e09bff1e68b3",
    "name": "Ethena USDe",
    "symbol": "USDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33613/thumb/3466ef_3c088c66c7d941e8856339d0bddf33cc_mv2.png?1702514458"
  },
  {
    "chainId": 1,
    "address": "0xe66747a101bff2dba3697199dcce5b743b454759",
    "name": "Gate",
    "symbol": "GT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8183/thumb/gate.png?1696508395"
  },
  {
    "chainId": 1,
    "address": "0xe5a733681bbe6cd8c764bb8078ef8e13a576dd78",
    "name": "Devour",
    "symbol": "DPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27191/thumb/DPAY_-_Icon_%28200_%C3%97_200_px%29.png?1696526240"
  },
  {
    "chainId": 1,
    "address": "0x92d529163c5e880b9de86f01de0cb8924d790357",
    "name": "Eyeverse",
    "symbol": "EYE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28623/thumb/EYE.png?1696527608"
  },
  {
    "chainId": 1,
    "address": "0x66a0f676479cee1d7373f3dc2e2952778bff5bd6",
    "name": "Wise",
    "symbol": "WISE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13552/thumb/Wise.jpg.jpg?1696513309"
  },
  {
    "chainId": 1,
    "address": "0xe51b8ab09008285a0380dd2680cd9dd5e13924d3",
    "name": "BallSwap",
    "symbol": "BSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14050/thumb/bsp.png?1696513775"
  },
  {
    "chainId": 1,
    "address": "0x83984d6142934bb535793a82adb0a46ef0f66b6d",
    "name": "Remme",
    "symbol": "REM",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/2152/thumb/semme.png?1696503115"
  },
  {
    "chainId": 1,
    "address": "0xb683d83a532e2cb7dfa5275eed3698436371cc9f",
    "name": "BTU Protocol",
    "symbol": "BTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3697/thumb/logo-btu-white-background_%281%29.png?1707407754"
  },
  {
    "chainId": 1,
    "address": "0xeadc218ac4cb7895a5a56e6484646b48f841c45a",
    "name": "MetaFinance",
    "symbol": "MFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17365/thumb/meta.PNG?1696516916"
  },
  {
    "chainId": 1,
    "address": "0x298d492e8c1d909d3f63bc4a36c66c64acb3d695",
    "name": "PolkaBridge",
    "symbol": "PBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13744/thumb/symbol-whitebg200x200.png?1696513488"
  },
  {
    "chainId": 1,
    "address": "0x3ef3b555842cdaff0f4f0b79c9dd65096d60ba63",
    "name": "CryptoPawCoin",
    "symbol": "CPRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32162/thumb/20231001_232853.png?1696722479"
  },
  {
    "chainId": 1,
    "address": "0xb13de094cc5cee6c4cc0a3737bf0290166d9ca5d",
    "name": "GoWithMi",
    "symbol": "GMAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7162/thumb/qXNbxVny_400x400.png?1696507459"
  },
  {
    "chainId": 1,
    "address": "0xa1d6df714f91debf4e0802a542e13067f31b8262",
    "name": "RFOX",
    "symbol": "RFOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12956/thumb/rfox.png?1696512744"
  },
  {
    "chainId": 1,
    "address": "0x68749665ff8d2d112fa859aa293f07a622782f38",
    "name": "Tether Gold",
    "symbol": "XAUT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/10481/thumb/Tether_Gold.png?1696510471"
  },
  {
    "chainId": 1,
    "address": "0x1b40183efb4dd766f11bda7a7c3ad8982e998421",
    "name": "Vesper Finance",
    "symbol": "VSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13527/thumb/vesper_logo.jpg?1696513288"
  },
  {
    "chainId": 1,
    "address": "0x06e0feb0d74106c7ada8497754074d222ec6bcdf",
    "name": "Bitball",
    "symbol": "BTB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7039/thumb/B_Logo.png?1697617213"
  },
  {
    "chainId": 1,
    "address": "0xc57d533c50bc22247d49a368880fb49a1caa39f7",
    "name": "PowerTrade Fuel",
    "symbol": "PTF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12590/thumb/powertrade_logo.jpg?1696512400"
  },
  {
    "chainId": 1,
    "address": "0x6fce4a401b6b80ace52baaefe4421bd188e76f6f",
    "name": "Aave MANA v1",
    "symbol": "AMANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11730/thumb/aMANA.png?1696511611"
  },
  {
    "chainId": 1,
    "address": "0x923b83c26b3809d960ff80332ed00aa46d7ed375",
    "name": "Creator Platform",
    "symbol": "CTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18252/thumb/logo_%281%29.png?1696517748"
  },
  {
    "chainId": 1,
    "address": "0x3f9bec82c776c47405bcb38070d2395fd18f89d3",
    "name": "Phantom Protocol",
    "symbol": "PHM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18253/thumb/phm.PNG?1696517748"
  },
  {
    "chainId": 1,
    "address": "0x793786e2dd4cc492ed366a94b88a3ff9ba5e7546",
    "name": "Axia",
    "symbol": "AXIAV3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12906/thumb/axia_200x200.png?1696512694"
  },
  {
    "chainId": 1,
    "address": "0x048fe49be32adfc9ed68c37d32b5ec9df17b3603",
    "name": "Skrumble Network",
    "symbol": "SKM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3826/thumb/skrumble-network.png?1696504495"
  },
  {
    "chainId": 1,
    "address": "0x14da7b27b2e0fedefe0a664118b0c9bc68e2e9af",
    "name": "Blockchain Cuties Universe Governance",
    "symbol": "BCUG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14425/thumb/bcug_logo.png?1696514115"
  },
  {
    "chainId": 1,
    "address": "0xbb1ee07d6c7baeb702949904080eb61f5d5e7732",
    "name": "Dogey Inu",
    "symbol": "DINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16374/thumb/512x512_dinu_logo.jpg?1696515973"
  },
  {
    "chainId": 1,
    "address": "0x9ba00d6856a4edf4665bca2c2309936572473b7e",
    "name": "Aave USDC v1",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/11674/thumb/aUSDC.png?1696511564"
  },
  {
    "chainId": 1,
    "address": "0xbb97e381f1d1e94ffa2a5844f6875e6146981009",
    "name": "WiBX",
    "symbol": "WBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11009/thumb/wibx_new_logo.png?1696510957"
  },
  {
    "chainId": 1,
    "address": "0xbcca60bb61934080951369a648fb03df4f96263c",
    "name": "Aave v2 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14318/thumb/aUSDC.e260d492.png?1696514006"
  },
  {
    "chainId": 1,
    "address": "0xbddc20ed7978b7d59ef190962f441cd18c14e19f",
    "name": "Crypto Asset Governance Alliance",
    "symbol": "CAGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32846/thumb/200px_caga_transparent.png?1707469892"
  },
  {
    "chainId": 1,
    "address": "0xc0f1728d9513efc316d0e93a0758c992f88b0809",
    "name": "SWTCoin",
    "symbol": "SWAT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/5985/thumb/swtcoin.jpg?1696506402"
  },
  {
    "chainId": 1,
    "address": "0xeeee2a2e650697d2a8e8bc990c2f3d04203be06f",
    "name": "Forgotten Playland",
    "symbol": "FP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35339/thumb/200.png?1708515043"
  },
  {
    "chainId": 1,
    "address": "0x07b701ac44aacb03d8bed42eb85ec38210bdf513",
    "name": "Taproot",
    "symbol": "TAPROOT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35368/thumb/200.png?1708401189"
  },
  {
    "chainId": 1,
    "address": "0xf8b358b3397a8ea5464f8cc753645d42e14b79ea",
    "name": "Airbloc",
    "symbol": "ABL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4393/thumb/airbloc-protocol-logo.png?1696504991"
  },
  {
    "chainId": 1,
    "address": "0x05bd6b9bc125fee6e670420f11d4e1817c0fc132",
    "name": "404WHEELS",
    "symbol": "404WHEELS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35340/thumb/d02e6933-7b6a-48c4-b84a-6e3383bdef8f.jpeg?1708316971"
  },
  {
    "chainId": 1,
    "address": "0x00059ae69c1622a7542edc15e8d17b060fe307b6",
    "name": "AmonD",
    "symbol": "AMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8952/thumb/AmonD_512x512.png?1696509099"
  },
  {
    "chainId": 1,
    "address": "0x115ec79f1de567ec68b7ae7eda501b406626478e",
    "name": "Carry",
    "symbol": "CRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3994/thumb/Carry.png?1696504635"
  },
  {
    "chainId": 1,
    "address": "0x7103ebdbf1f89be2d53eff9b3cf996c9e775c105",
    "name": "X7103",
    "symbol": "X7103",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28421/thumb/X7103_LOGO_black_back_200_x_200_px.png?1696527418"
  },
  {
    "chainId": 1,
    "address": "0xc53342fd7575f572b0ff4569e31941a5b821ac76",
    "name": "Ethereum Volatility Index Token",
    "symbol": "ETHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16716/thumb/logo_-_2021-06-28T092549.772.png?1696516290"
  },
  {
    "chainId": 1,
    "address": "0x98c23e9d8f34fefb1b7bd6a91b7ff122f4e16f5c",
    "name": "Aave v3 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32847/thumb/usdc_%281%29.png?1699619355"
  },
  {
    "chainId": 1,
    "address": "0xde4ee8057785a7e8e800db58f9784845a5c2cbd6",
    "name": "DeXe",
    "symbol": "DEXE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12713/thumb/DEXE_token_logo.png?1696512514"
  },
  {
    "chainId": 1,
    "address": "0x2e2364966267b5d7d2ce6cd9a9b5bd19d9c7c6a9",
    "name": "Voice",
    "symbol": "VOICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12926/thumb/OjCQtdL.png?1696512714"
  },
  {
    "chainId": 1,
    "address": "0xd714d91a169127e11d8fab3665d72e8b7ef9dbe2",
    "name": "BlackHole Protocol",
    "symbol": "BLACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15124/thumb/blackhole.PNG?1696514781"
  },
  {
    "chainId": 1,
    "address": "0xa23c1194d421f252b4e6d5edcc3205f7650a4ebe",
    "name": "LaunchBlock",
    "symbol": "LBP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24181/thumb/S6z_-9GB_400x400.jpg?1696523370"
  },
  {
    "chainId": 1,
    "address": "0x55a380d134d722006a5ce2d510562e1239d225b1",
    "name": "Marvin Inu",
    "symbol": "MARVIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22039/thumb/lVshyCp.png?1696521384"
  },
  {
    "chainId": 1,
    "address": "0xd2be3722b17b616c51ed9b8944a227d1ce579c24",
    "name": "Dtube Coin",
    "symbol": "DTUBE",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/13126/thumb/dtube_logo.png?1696512913"
  },
  {
    "chainId": 1,
    "address": "0x23b608675a2b2fb1890d3abbd85c5775c51691d5",
    "name": "Unisocks",
    "symbol": "SOCKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10717/thumb/qFrcoiM.png?1696510685"
  },
  {
    "chainId": 1,
    "address": "0x67954768e721fad0f0f21e33e874497c73ed6a82",
    "name": "KeKChain",
    "symbol": "KEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27061/thumb/20220816_155427.png?1696526111"
  },
  {
    "chainId": 1,
    "address": "0xc5d27f27f08d1fd1e3ebbaa50b3442e6c0d50439",
    "name": "Moon App",
    "symbol": "APP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34492/thumb/moon-app.jpeg?1705131046"
  },
  {
    "chainId": 1,
    "address": "0xadc3f2c3d728202658930860158c726d8180a38f",
    "name": "StarkMeta",
    "symbol": "SMETA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25851/thumb/YcUYwy5f_400x400.jpg?1696524935"
  },
  {
    "chainId": 1,
    "address": "0xa2881f7f441267042f9778ffa0d4f834693426be",
    "name": "The HUSL",
    "symbol": "HUSL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18619/thumb/W8ZIHKU.png?1696518092"
  },
  {
    "chainId": 1,
    "address": "0xab37e1358b639fd877f015027bb62d3ddaa7557e",
    "name": "Lien",
    "symbol": "LIEN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12224/thumb/Lien.png?1696512057"
  },
  {
    "chainId": 1,
    "address": "0x033e223870f766644f7f7a4b7dc2e91573707d06",
    "name": "Zin",
    "symbol": "ZIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13250/thumb/coingecko_logo.png?1696513025"
  },
  {
    "chainId": 1,
    "address": "0xb4a677b0e363c3815d46326954a4e4d2b1ace357",
    "name": "THENODE",
    "symbol": "THE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10292/thumb/thenode_logo.png?1696510297"
  },
  {
    "chainId": 1,
    "address": "0x40fd72257597aa14c7231a7b1aaa29fce868f677",
    "name": "Sora",
    "symbol": "XOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11093/thumb/sora_logo_cg_white.png?1696511034"
  },
  {
    "chainId": 1,
    "address": "0x106538cc16f938776c7c180186975bca23875287",
    "name": "Basis Share",
    "symbol": "BAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13251/thumb/BAS.png?1696513026"
  },
  {
    "chainId": 1,
    "address": "0xd90e69f67203ebe02c917b5128629e77b4cd92dc",
    "name": "One Cash",
    "symbol": "ONC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13529/thumb/onc_logo.png?1696513290"
  },
  {
    "chainId": 1,
    "address": "0xfeeeef4d7b4bf3cc8bd012d02d32ba5fd3d51e31",
    "name": "Tail",
    "symbol": "TAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28260/thumb/IMG_20221210_122215_723.jpg?1696527262"
  },
  {
    "chainId": 1,
    "address": "0x6c6ee5e31d828de241282b9606c8e98ea48526e2",
    "name": "Holo",
    "symbol": "HOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3348/thumb/Holologo_Profile.png?1696504052"
  },
  {
    "chainId": 1,
    "address": "0x076641af1b8f06b7f8c92587156143c109002cbe",
    "name": "SoPay",
    "symbol": "SOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6736/thumb/sopay.png?1696507071"
  },
  {
    "chainId": 1,
    "address": "0x0b4c2708f052dca413600e237675e4d6778a9375",
    "name": "CoinClaim",
    "symbol": "CLM",
    "decimals": 16,
    "logoURI": "https://assets.coingecko.com/coins/images/6687/thumb/coinclaim.jpg?1696507028"
  },
  {
    "chainId": 1,
    "address": "0x09617f6fd6cf8a71278ec86e23bbab29c04353a7",
    "name": "Shardus",
    "symbol": "ULT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8383/thumb/final_logo_photoshop.png?1696508575"
  },
  {
    "chainId": 1,
    "address": "0x487d62468282bd04ddf976631c23128a425555ee",
    "name": "UPCX",
    "symbol": "UPC",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/33308/thumb/02_UPC_logo_RGB_black_200_200.png?1701409035"
  },
  {
    "chainId": 1,
    "address": "0xdc5864ede28bd4405aa04d93e05a0531797d9d59",
    "name": "Falcon Project",
    "symbol": "FNT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/11579/thumb/falcon_ava_black.png?1696511476"
  },
  {
    "chainId": 1,
    "address": "0x009c43b42aefac590c719e971020575974122803",
    "name": "Bibox",
    "symbol": "BIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1441/thumb/bibox-token.png?1696502491"
  },
  {
    "chainId": 1,
    "address": "0x954b890704693af242613edef1b603825afcd708",
    "name": "Cardstack",
    "symbol": "CARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3247/thumb/cardstack.png?1696503962"
  },
  {
    "chainId": 1,
    "address": "0x3597bfd533a99c9aa083587b074434e61eb0a258",
    "name": "Dent",
    "symbol": "DENT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1152/thumb/gLCEA2G.png?1696502243"
  },
  {
    "chainId": 1,
    "address": "0x2341dd0a96a0dab62aa1efb93d59ff7f3bdb8932",
    "name": "ProStarter",
    "symbol": "PROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15259/thumb/rpot.PNG?1696514912"
  },
  {
    "chainId": 1,
    "address": "0x6c5024cd4f8a59110119c56f8933403a539555eb",
    "name": "Aave SUSD",
    "symbol": "ASUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14266/thumb/aSUSD.9b00ffc6.png?1696513967"
  },
  {
    "chainId": 1,
    "address": "0x6524b87960c2d573ae514fd4181777e7842435d4",
    "name": "Benzene",
    "symbol": "BZN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17368/thumb/bzn-logo-200.png?1696516919"
  },
  {
    "chainId": 1,
    "address": "0x1788430620960f9a70e3dc14202a3a35dde1a316",
    "name": "OpenAlexa Protocol",
    "symbol": "OAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12400/thumb/256x256-OAP.png?1696512222"
  },
  {
    "chainId": 1,
    "address": "0xcf3c8be2e2c42331da80ef210e9b1b307c03d36a",
    "name": "Bepro",
    "symbol": "BEPRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10251/thumb/logo.png?1696510261"
  },
  {
    "chainId": 1,
    "address": "0x6b4c7a5e3f0b99fcd83e9c089bddd6c7fce5c611",
    "name": "Million",
    "symbol": "MM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16825/thumb/logo200x200.png?1696516393"
  },
  {
    "chainId": 1,
    "address": "0x39c6b3e42d6a679d7d776778fe880bc9487c2eda",
    "name": "Aave KNC",
    "symbol": "AKNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14250/thumb/aKNC.42bcd2e3.png?1696513964"
  },
  {
    "chainId": 1,
    "address": "0x6b9f031d718dded0d681c20cb754f97b3bb81b78",
    "name": "GEEQ",
    "symbol": "GEEQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11953/thumb/GeeqLogoPNGTransparent-1.png?1696511812"
  },
  {
    "chainId": 1,
    "address": "0x4f3afec4e5a3f2a6a1a411def7d7dfe50ee057bf",
    "name": "Digix Gold",
    "symbol": "DGX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/4171/thumb/DGX_Token.png?1696504794"
  },
  {
    "chainId": 1,
    "address": "0xc8871267e07408b89aa5aecc58adca5e574557f8",
    "name": "Instadapp USDC",
    "symbol": "IUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25820/thumb/iUSDC_100x100.png?1696524905"
  },
  {
    "chainId": 1,
    "address": "0x4af5ff1a60a6ef6c7c8f9c4e304cd9051fca3ec0",
    "name": "Rigel Protocol",
    "symbol": "RGP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15837/thumb/A_qRYvB2_400x400.png?1696515455"
  },
  {
    "chainId": 1,
    "address": "0x40a9d39aa50871df092538c5999b107f34409061",
    "name": "Instadapp DAI",
    "symbol": "IDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25821/thumb/iDAI_100x100.png?1696524906"
  },
  {
    "chainId": 1,
    "address": "0x93ed3fbe21207ec2e8f2d3c3de6e058cb73bc04d",
    "name": "Kleros",
    "symbol": "PNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3833/thumb/kleros.png?1696504500"
  },
  {
    "chainId": 1,
    "address": "0x5d285f735998f36631f678ff41fb56a10a4d0429",
    "name": "MixMarvel",
    "symbol": "MIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8222/thumb/8878caf93b1e3b6cfb3b414bda3b5250.png?1696508428"
  },
  {
    "chainId": 1,
    "address": "0x009178997aff09a67d4caccfeb897fb79d036214",
    "name": "1Sol",
    "symbol": "1SOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21615/thumb/YyGDie9f_400x400.jpg?1696520976"
  },
  {
    "chainId": 1,
    "address": "0x0e6fa9c050c8a707e7f56a2b3695665e4f9eac9b",
    "name": "Robo Inu Finance",
    "symbol": "RBIF",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/20821/thumb/O68Gs5SE_400x400.jpg?1696520214"
  },
  {
    "chainId": 1,
    "address": "0x946551dd05c5abd7cc808927480225ce36d8c475",
    "name": "One",
    "symbol": "ONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4960/thumb/Screenshot_39.png?1696505497"
  },
  {
    "chainId": 1,
    "address": "0x7104d1f179cc9cc7fb5c79be6da846e3fbc4c105",
    "name": "X7104",
    "symbol": "X7104",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28422/thumb/X7104_LOGO_black_back_200_x_200_px.png?1696527419"
  },
  {
    "chainId": 1,
    "address": "0xa7a5c1058194af8f00c187adb7fcc0c95f1c6c2d",
    "name": "SPACE iZ",
    "symbol": "SPIZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12213/thumb/Frame_1000002608.png?1703058029"
  },
  {
    "chainId": 1,
    "address": "0x99295f1141d58a99e939f7be6bbe734916a875b8",
    "name": "LinkPool",
    "symbol": "LPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14548/thumb/linkpool-logo-256x256.png?1696514231"
  },
  {
    "chainId": 1,
    "address": "0x056fd409e1d7a124bd7017459dfea2f387b6d5cd",
    "name": "Gemini Dollar",
    "symbol": "GUSD",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/5992/thumb/gemini-dollar-gusd.png?1696506408"
  },
  {
    "chainId": 1,
    "address": "0x25e1474170c4c0aa64fa98123bdc8db49d7802fa",
    "name": "Bidao",
    "symbol": "BID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12596/thumb/bidao.png?1696512405"
  },
  {
    "chainId": 1,
    "address": "0x73c9275c3a2dd84b5741fd59aebf102c91eb033f",
    "name": "Bitball Treasure",
    "symbol": "BTRS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7273/thumb/btrs.png?1696507565"
  },
  {
    "chainId": 1,
    "address": "0xf74751c07c92b668f02527d0e1384ee6d68ac90e",
    "name": "AltCTRL",
    "symbol": "CTRL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35472/thumb/200x200.png?1708770112"
  },
  {
    "chainId": 1,
    "address": "0xf7970499814654cd13cb7b6e7634a12a7a8a9abc",
    "name": "TOM Finance",
    "symbol": "TOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13173/thumb/TOM_logo.png?1696512956"
  },
  {
    "chainId": 1,
    "address": "0x4527a3b4a8a150403090a99b87effc96f2195047",
    "name": "P2P solutions foundation",
    "symbol": "P2PS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2798/thumb/p2ps.png?1696503565"
  },
  {
    "chainId": 1,
    "address": "0x6d1dc3928604b00180bb570bdae94b9698d33b79",
    "name": "UnitedCrowd",
    "symbol": "UCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14956/thumb/eUvRU9wm.png?1696514614"
  },
  {
    "chainId": 1,
    "address": "0x8ce55da273fb5cf4b1ffc5d4066bd62445a43543",
    "name": "Burners",
    "symbol": "BRNR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33911/thumb/Burners.jpeg?1703310206"
  },
  {
    "chainId": 1,
    "address": "0x2b89bf8ba858cd2fcee1fada378d5cd6936968be",
    "name": "Secret  ERC20 ",
    "symbol": "WSCRT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13767/thumb/Secret_S_Black_Coingecko.png?1696513509"
  },
  {
    "chainId": 1,
    "address": "0x2fc246aa66f0da5bb1368f688548ecbbe9bdee5d",
    "name": "TEMCO",
    "symbol": "TEMCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6210/thumb/bSZ7HUuS_400x400.jpg?1696506595"
  },
  {
    "chainId": 1,
    "address": "0x9ceb84f92a0561fa3cc4132ab9c0b76a59787544",
    "name": "Doki Doki",
    "symbol": "DOKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12759/thumb/doki_logo.png?1696512556"
  },
  {
    "chainId": 1,
    "address": "0x6781a0f84c7e9e846dcb84a9a5bd49333067b104",
    "name": "Zap",
    "symbol": "ZAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2180/thumb/zap.png?1696503138"
  },
  {
    "chainId": 1,
    "address": "0x7105faa4a26ed1c67b8b2b41bec98f06ee21d105",
    "name": "X7105",
    "symbol": "X7105",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28423/thumb/X7105_LOGO_black_back_200_x_200_px.png?1696527420"
  },
  {
    "chainId": 1,
    "address": "0x1ceb5cb57c4d4e2b2433641b95dd330a33185a44",
    "name": "Keep3rV1",
    "symbol": "KP3R",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12966/thumb/kp3r_logo.jpg?1696512754"
  },
  {
    "chainId": 1,
    "address": "0xd585aaafa2b58b1cd75092b51ade9fa4ce52f247",
    "name": "peg eUSD",
    "symbol": "PEUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31521/thumb/peUSD_200x200.png?1696530331"
  },
  {
    "chainId": 1,
    "address": "0xbd356a39bff2cada8e9248532dd879147221cf76",
    "name": "WOM Protocol",
    "symbol": "WOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4559/thumb/wom_logo_small.png?1696505139"
  },
  {
    "chainId": 1,
    "address": "0x120a3879da835a5af037bb2d1456bebd6b54d4ba",
    "name": "Revest Finance",
    "symbol": "RVST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18622/thumb/Qma94n6waADECpde1ukBBS8iNiECcdVcxjfgubnWPE9ZT7.png?1696518094"
  },
  {
    "chainId": 1,
    "address": "0x97a9bac06f90940bce9caec2b880ff17707519e4",
    "name": "Minato",
    "symbol": "MNTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24622/thumb/MNTO_200x200.png?1696523794"
  },
  {
    "chainId": 1,
    "address": "0x83e6f1e41cdd28eaceb20cb649155049fac3d5aa",
    "name": "Polkastarter",
    "symbol": "POLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12648/thumb/polkastarter.png?1696512455"
  },
  {
    "chainId": 1,
    "address": "0xbf52f2ab39e26e0951d2a02b49b7702abe30406a",
    "name": "ODEM",
    "symbol": "ODE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3752/thumb/odem.png?1696504428"
  },
  {
    "chainId": 1,
    "address": "0x5b7533812759b45c2b44c19e320ba2cd2681b542",
    "name": "SingularityNET",
    "symbol": "AGIX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2138/thumb/singularitynet.png?1696503103"
  },
  {
    "chainId": 1,
    "address": "0xef6344de1fcfc5f48c30234c16c1389e8cdc572c",
    "name": "EncrypGen",
    "symbol": "DNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1215/thumb/encrypgen.png?1696502295"
  },
  {
    "chainId": 1,
    "address": "0xbd3de9a069648c84d27d74d701c9fa3253098b15",
    "name": "EQIFi",
    "symbol": "EQX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17490/thumb/EQIFI_Logo_Color.png?1696517031"
  },
  {
    "chainId": 1,
    "address": "0x630d98424efe0ea27fb1b3ab7741907dffeaad78",
    "name": "PEAKDEFI",
    "symbol": "PEAK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9626/thumb/PEAKDEFI_Logo_250x250.png?1696509698"
  },
  {
    "chainId": 1,
    "address": "0xf2051511b9b121394fa75b8f7d4e7424337af687",
    "name": "DAOhaus",
    "symbol": "HAUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14551/thumb/jN3kkqke_400x400.png?1696514234"
  },
  {
    "chainId": 1,
    "address": "0xc8c424b91d8ce0137bab4b832b7f7d154156ba6c",
    "name": "apM Coin",
    "symbol": "APM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10069/thumb/PvDCvh5h_400x400.png?1696510100"
  },
  {
    "chainId": 1,
    "address": "0x2391b8147caef113787972abbdaf77f4bae0cf22",
    "name": "ArcaneDEX",
    "symbol": "ARC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34575/thumb/ARC_%281%29.png?1705411582"
  },
  {
    "chainId": 1,
    "address": "0x6293bbe52c3682057163172850002f18f993da6a",
    "name": "TruthGPT  ETH ",
    "symbol": "TRUTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30993/thumb/LOGO_200.png?1696529831"
  },
  {
    "chainId": 1,
    "address": "0x8f43ee50942e96d84052253ab13f59c1d942fb98",
    "name": "Paragon Network",
    "symbol": "PARA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36967/thumb/0x8f43ee50942e96d84052253ab13f59c1d942fb98.png?1712908143"
  },
  {
    "chainId": 1,
    "address": "0x1e0b2992079b620aa13a7c2e7c88d2e1e18e46e9",
    "name": "KOMPETE",
    "symbol": "KOMPETE",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/24298/thumb/KOMPETE_LOGO.?1696523480"
  },
  {
    "chainId": 1,
    "address": "0x47e67ba66b0699500f18a53f94e2b9db3d47437e",
    "name": "PlayGame",
    "symbol": "PXG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7242/thumb/pxg-coin_3x.png?1696507536"
  },
  {
    "chainId": 1,
    "address": "0x8de5b80a0c1b02fe4976851d030b36122dbb8624",
    "name": "Vanar Chain",
    "symbol": "VANRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33466/thumb/apple-touch-icon.png?1701942541"
  },
  {
    "chainId": 1,
    "address": "0x86772b1409b61c639eaac9ba0acfbb6e238e5f83",
    "name": "Indexed Finance",
    "symbol": "NDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13546/thumb/indexed-light.74bb5471.png?1696513306"
  },
  {
    "chainId": 1,
    "address": "0x0c9c7712c83b3c70e7c5e11100d33d9401bdf9dd",
    "name": "Wombat",
    "symbol": "WOMBAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26430/thumb/Project_Page_Icon.png?1696525504"
  },
  {
    "chainId": 1,
    "address": "0x0198f46f520f33cd4329bd4be380a25a90536cd5",
    "name": "PlayChip",
    "symbol": "PLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4567/thumb/pla.png?1696505147"
  },
  {
    "chainId": 1,
    "address": "0xb97048628db6b661d4c2aa833e95dbe1a905b280",
    "name": "TenX",
    "symbol": "PAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/775/thumb/TenX-Icon-CircleBlack.png?1696501927"
  },
  {
    "chainId": 1,
    "address": "0x0f71b8de197a1c84d31de0f1fa7926c365f052b3",
    "name": "Arcona",
    "symbol": "ARCONA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4312/thumb/icon_ARCONA_%281%29.png?1696504918"
  },
  {
    "chainId": 1,
    "address": "0x3543638ed4a9006e4840b105944271bcea15605d",
    "name": "U Network",
    "symbol": "UUU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3877/thumb/2645.png?1696504538"
  },
  {
    "chainId": 1,
    "address": "0x543ff227f64aa17ea132bf9886cab5db55dcaddf",
    "name": "DAOstack",
    "symbol": "GEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3479/thumb/gen.png?1696504167"
  },
  {
    "chainId": 1,
    "address": "0x99d8a9c45b2eca8864373a26d1459e3dff1e17f3",
    "name": "Magic Internet Money",
    "symbol": "MIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16786/thumb/mimlogopng.png?1696516358"
  },
  {
    "chainId": 1,
    "address": "0x3136ef851592acf49ca4c825131e364170fa32b3",
    "name": "CoinFi",
    "symbol": "COFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1709/thumb/coinfi.png?1696502727"
  },
  {
    "chainId": 1,
    "address": "0x4092678e4e78230f46a1534c0fbc8fa39780892b",
    "name": "Odyssey",
    "symbol": "OCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2559/thumb/ocn.png?1696503367"
  },
  {
    "chainId": 1,
    "address": "0x45080a6531d671ddff20db42f93792a489685e32",
    "name": "Finance Vote",
    "symbol": "FVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13181/thumb/finance.png?1696512964"
  },
  {
    "chainId": 1,
    "address": "0x501e8726d06cdef66f3e0cb67f54924cca1cc894",
    "name": "BRMV",
    "symbol": "BRMV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14389/thumb/brmv-logo-256px.png?1696514080"
  },
  {
    "chainId": 1,
    "address": "0xc4cb5793bd58bad06bf51fb37717b86b02cbe8a4",
    "name": "PROXI DeFi",
    "symbol": "CREDIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12212/thumb/_credit.png?1696512046"
  },
  {
    "chainId": 1,
    "address": "0x1a7a8bd9106f2b8d977e08582dc7d24c723ab0db",
    "name": "AppCoins",
    "symbol": "APPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1876/thumb/appcoins.png?1696502872"
  },
  {
    "chainId": 1,
    "address": "0x33c04bed4533e31f2afb8ac4a61a48eda38c4fa0",
    "name": "Gorilla",
    "symbol": "GORILLA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33006/thumb/IMG_20231110_124057_942.jpg?1700123171"
  },
  {
    "chainId": 1,
    "address": "0x73484a262730d1d422610729e828346f9b2ff480",
    "name": "BRCP",
    "symbol": "BRCP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14388/thumb/brcp-500x500-verde.png?1696514079"
  },
  {
    "chainId": 1,
    "address": "0x221657776846890989a759ba2973e427dff5c9bb",
    "name": "Augur",
    "symbol": "REP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/309/thumb/REP.png?1696501648"
  },
  {
    "chainId": 1,
    "address": "0x544c42fbb96b39b21df61cf322b5edc285ee7429",
    "name": "InsurAce",
    "symbol": "INSUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14226/thumb/insur.png?1696513941"
  },
  {
    "chainId": 1,
    "address": "0xe3fedaecd47aa8eab6b23227b0ee56f092c967a9",
    "name": "Primas",
    "symbol": "PST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/917/thumb/primas.png?1696502039"
  },
  {
    "chainId": 1,
    "address": "0x89d24a6b4ccb1b6faa2625fe562bdd9a23260359",
    "name": "Sai",
    "symbol": "SAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1442/thumb/dai.png?1696502492"
  },
  {
    "chainId": 1,
    "address": "0x04a020325024f130988782bd5276e53595e8d16e",
    "name": "Herbalist",
    "symbol": "HERB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/6892/thumb/herbalist-token-twitter.png?1696507211"
  },
  {
    "chainId": 1,
    "address": "0x41dbecc1cdc5517c6f76f6a6e836adbee2754de3",
    "name": "Medicalchain",
    "symbol": "MTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2850/thumb/medicalchain.png?1696503608"
  },
  {
    "chainId": 1,
    "address": "0x46d473a0b3eeec9f55fade641bc576d5bc0b2246",
    "name": "SurfExUtilityToken",
    "symbol": "SURF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10783/thumb/200x200-logo-blu-grey-bkg-4-e1583512409629.png?1696510744"
  },
  {
    "chainId": 1,
    "address": "0xb98d4c97425d9908e66e53a6fdf673acca0be986",
    "name": "Arcblock",
    "symbol": "ABT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2341/thumb/arcblock.png?1696503220"
  },
  {
    "chainId": 1,
    "address": "0x20910e5b5f087f6439dfcb0dda4e27d1014ac2b8",
    "name": "BananaTok",
    "symbol": "BNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10289/thumb/sMCxdYBa_400x400.jpg?1696510295"
  },
  {
    "chainId": 1,
    "address": "0x558ec3152e2eb2174905cd19aea4e34a23de9ad6",
    "name": "Bread",
    "symbol": "BRD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1440/thumb/bread.png?1696502490"
  },
  {
    "chainId": 1,
    "address": "0x16eccfdbb4ee1a85a33f3a9b21175cd7ae753db4",
    "name": "Router Protocol",
    "symbol": "ROUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13709/thumb/route_token_200x200-19.png?1696513454"
  },
  {
    "chainId": 1,
    "address": "0xf0939011a9bb95c3b791f0cb546377ed2693a574",
    "name": "0 exchange",
    "symbol": "ZERO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13706/thumb/0.exchange_%28logo%29.jpg?1696513452"
  },
  {
    "chainId": 1,
    "address": "0xff931a7946d2fa11cf9123ef0dc6f6c7c6cb60c4",
    "name": "Dancing Baby",
    "symbol": "BABY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32641/thumb/IMG_20231101_135519_737.jpg?1698889030"
  },
  {
    "chainId": 1,
    "address": "0xf3dcbc6d72a4e1892f7917b7c43b74131df8480e",
    "name": "Big Data Protocol",
    "symbol": "BDP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14222/thumb/logo_BDP_200.png?1696513937"
  },
  {
    "chainId": 1,
    "address": "0x6b785a0322126826d8226d77e173d75dafb84d11",
    "name": "Bankroll Vault",
    "symbol": "VLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11638/thumb/vlt-200.png?1696511531"
  },
  {
    "chainId": 1,
    "address": "0x6149c26cd2f7b5ccdb32029af817123f6e37df5b",
    "name": "Launchpool",
    "symbol": "LPOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14041/thumb/dGUvV0HQ_400x400.jpg?1696513767"
  },
  {
    "chainId": 1,
    "address": "0x668dbf100635f593a3847c0bdaf21f0a09380188",
    "name": "BNSD Finance",
    "symbol": "BNSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12368/thumb/bnsd.png?1696512193"
  },
  {
    "chainId": 1,
    "address": "0x72f020f8f3e8fd9382705723cd26380f8d0c66bb",
    "name": "PlotX",
    "symbol": "PLOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12795/thumb/PlotX.png?1696512589"
  },
  {
    "chainId": 1,
    "address": "0x5a666c7d92e5fa7edcb6390e4efd6d0cdd69cf37",
    "name": "Unmarshal",
    "symbol": "MARSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14554/thumb/img_circle_256x256.png?1696514237"
  },
  {
    "chainId": 1,
    "address": "0xce2b26bec3c6945f0922a983f3cfa11cd79176f4",
    "name": "Elonium",
    "symbol": "ELON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32642/thumb/ELONIUM.png?1698892015"
  },
  {
    "chainId": 1,
    "address": "0x40d1f63b5d2048e67e9bedb1b4c2f1a9fb4b6817",
    "name": "Golden Goose",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7892/thumb/tsRcntSR_400x400.png?1696508124"
  },
  {
    "chainId": 1,
    "address": "0x0563dce613d559a47877ffd1593549fb9d3510d6",
    "name": "SuperBid",
    "symbol": "SUPERBID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14179/thumb/sign_super_bid.png?1696513897"
  },
  {
    "chainId": 1,
    "address": "0x2942e3b38e33123965bfbc21e802be943a76bbc6",
    "name": "EHash",
    "symbol": "EHASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14180/thumb/ehash.png?1696513898"
  },
  {
    "chainId": 1,
    "address": "0xd076c4ba62c57b3fa10800bcfd8da66742110e0e",
    "name": "HAVAH",
    "symbol": "HVH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29156/thumb/hvh.png?1696528115"
  },
  {
    "chainId": 1,
    "address": "0x53884b61963351c283118a8e1fc05ba464a11959",
    "name": "Monnos",
    "symbol": "MNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11640/thumb/logo_%2880%29.png?1696511533"
  },
  {
    "chainId": 1,
    "address": "0x75231f58b43240c9718dd58b4967c5114342a86c",
    "name": "OKB",
    "symbol": "OKB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4463/thumb/WeChat_Image_20220118095654.png?1696505053"
  },
  {
    "chainId": 1,
    "address": "0x1900e8b5619a3596745f715d0427fe617c729ba9",
    "name": "Chainbing",
    "symbol": "CBG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18052/thumb/j3saoPU.png?1696517564"
  },
  {
    "chainId": 1,
    "address": "0x2c4e8f2d746113d0696ce89b35f0d8bf88e0aeca",
    "name": "OST",
    "symbol": "OST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1367/thumb/ost.jpg?1696502425"
  },
  {
    "chainId": 1,
    "address": "0x4dd672e77c795844fe3a464ef8ef0faae617c8fb",
    "name": "CONUN",
    "symbol": "CON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7480/thumb/conun.png?1696507750"
  },
  {
    "chainId": 1,
    "address": "0x7a5ce6abd131ea6b148a022cb76fc180ae3315a6",
    "name": "bAlpha",
    "symbol": "BALPHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14224/thumb/logo_bAlpha_200.png?1696513939"
  },
  {
    "chainId": 1,
    "address": "0x4159862bcf6b4393a80550b1ed03dffa6f90533c",
    "name": "One Hundred Million Inu",
    "symbol": "OHMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28625/thumb/ohmi-shield-200.png?1696527610"
  },
  {
    "chainId": 1,
    "address": "0x7ae0d42f23c33338de15bfa89c7405c068d9dc0a",
    "name": "Shibaverse",
    "symbol": "VERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18407/thumb/logo_200.png?1696517897"
  },
  {
    "chainId": 1,
    "address": "0xbc19712feb3a26080ebf6f2f7849b417fdd792ca",
    "name": "BoringDAO",
    "symbol": "BORING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16429/thumb/Tjq3pXEH_400x400.jpg?1696516025"
  },
  {
    "chainId": 1,
    "address": "0x6f87d756daf0503d08eb8993686c7fc01dc44fb1",
    "name": "Unitrade",
    "symbol": "TRADE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11982/thumb/unitrade.PNG?1696511840"
  },
  {
    "chainId": 1,
    "address": "0x55c08ca52497e2f1534b59e2917bf524d4765257",
    "name": "UwU Lend",
    "symbol": "UWU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27408/thumb/uwu_logo_no-bg_1024.png?1696526449"
  },
  {
    "chainId": 1,
    "address": "0x5c872500c00565505f3624ab435c222e558e9ff8",
    "name": "CoTrader",
    "symbol": "COT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4205/thumb/logo_black.png?1696504825"
  },
  {
    "chainId": 1,
    "address": "0xd6a5ab46ead26f49b03bbb1f9eb1ad5c1767974a",
    "name": "Ethermon",
    "symbol": "EMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15889/thumb/LtET0reH_400x400.jpg?1696515504"
  },
  {
    "chainId": 1,
    "address": "0x35f6b052c598d933d69a4eec4d04c73a191fe6c2",
    "name": "Aave SNX",
    "symbol": "ASNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14265/thumb/aSNX.3238f3e7.png?1696513966"
  },
  {
    "chainId": 1,
    "address": "0x27054b13b1b798b345b591a4d22e6562d47ea75a",
    "name": "AirSwap",
    "symbol": "AST",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/1019/thumb/Airswap.png?1696502130"
  },
  {
    "chainId": 1,
    "address": "0xeef9f339514298c6a857efcfc1a762af84438dee",
    "name": "Hermez Network",
    "symbol": "HEZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12826/thumb/hermez_logo.png?1696512617"
  },
  {
    "chainId": 1,
    "address": "0x038a68ff68c393373ec894015816e33ad41bd564",
    "name": "Glitch Protocol",
    "symbol": "GLCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13712/thumb/glitch_logo.jpeg?1696513457"
  },
  {
    "chainId": 1,
    "address": "0x73374ea518de7addd4c2b624c0e8b113955ee041",
    "name": "Juggernaut",
    "symbol": "JGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12761/thumb/juggernaut_logo.png?1696512558"
  },
  {
    "chainId": 1,
    "address": "0xc0ec8caec55f37d47fbfa595727418868a21fd48",
    "name": "EcoG9coin",
    "symbol": "EGC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/8038/thumb/Screenshot_2019-12-05_at_7.14.34_AM.png?1696508259"
  },
  {
    "chainId": 1,
    "address": "0xb1e9157c2fdcc5a856c8da8b2d89b6c32b3c1229",
    "name": "Zenfuse",
    "symbol": "ZEFU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12796/thumb/zenfuse.jpg?1696512590"
  },
  {
    "chainId": 1,
    "address": "0xddb3422497e61e13543bea06989c0789117555c5",
    "name": "COTI",
    "symbol": "COTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2962/thumb/Coti.png?1696503705"
  },
  {
    "chainId": 1,
    "address": "0xfb7b4564402e5500db5bb6d63ae671302777c75a",
    "name": "DexTools",
    "symbol": "DEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11603/thumb/dext.png?1696511498"
  },
  {
    "chainId": 1,
    "address": "0x6afcff9189e8ed3fcc1cffa184feb1276f6a82a5",
    "name": "PolkaPet World",
    "symbol": "PETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19409/thumb/pets_polka.PNG?1696518848"
  },
  {
    "chainId": 1,
    "address": "0x5150956e082c748ca837a5dfa0a7c10ca4697f9c",
    "name": "Zeedex",
    "symbol": "ZDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12412/thumb/Untitled-design-4.png?1696512234"
  },
  {
    "chainId": 1,
    "address": "0x4d2ee5dae46c86da2ff521f7657dad98834f97b8",
    "name": "Pepemon Pepeballs",
    "symbol": "PPBLZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12823/thumb/200pepebball-BIG.png?1696512615"
  },
  {
    "chainId": 1,
    "address": "0x217ddead61a42369a266f1fb754eb5d3ebadc88a",
    "name": "Don key",
    "symbol": "DON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15482/thumb/donkey_logo.jpg?1696515127"
  },
  {
    "chainId": 1,
    "address": "0xb4fbed161bebcb37afb1cb4a6f7ca18b977ccb25",
    "name": "Dogeswap",
    "symbol": "DOGES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12763/thumb/20200926-220107.png?1696512560"
  },
  {
    "chainId": 1,
    "address": "0x91af0fbb28aba7e31403cb457106ce79397fd4e6",
    "name": "Aergo",
    "symbol": "AERGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4490/thumb/aergo.png?1696505079"
  },
  {
    "chainId": 1,
    "address": "0x9506d37f70eb4c3d79c398d326c871abbf10521d",
    "name": "Media Licensing Token",
    "symbol": "MLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15659/thumb/milc_200x200.png?1696515290"
  },
  {
    "chainId": 1,
    "address": "0xd26a9c3437f7d121098c8c05c7413f5cc70bb070",
    "name": "Azuma Coin",
    "symbol": "AZUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10483/thumb/WDncHJ5.png?1696510473"
  },
  {
    "chainId": 1,
    "address": "0x08711d3b02c8758f2fb3ab4e80228418a7f8e39c",
    "name": "Edgeless",
    "symbol": "EDG",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/635/thumb/Edgeless-Token.jpg?1696501830"
  },
  {
    "chainId": 1,
    "address": "0x3b484b82567a09e2588a13d54d032153f0c0aee0",
    "name": "OpenDAO",
    "symbol": "SOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21956/thumb/fo42wXI6_400x400.jpg?1696521305"
  },
  {
    "chainId": 1,
    "address": "0xfb559ce67ff522ec0b9ba7f5dc9dc7ef6c139803",
    "name": "Probit",
    "symbol": "PROB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6765/thumb/ProBit-Exchange-logo.png?1696507099"
  },
  {
    "chainId": 1,
    "address": "0xc713e5e149d5d0715dcd1c156a020976e7e56b88",
    "name": "Aave MKR",
    "symbol": "AMKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14248/thumb/aMKR.ac965178.png?1696513962"
  },
  {
    "chainId": 1,
    "address": "0x6e0dade58d2d89ebbe7afc384e3e4f15b70b14d8",
    "name": "QuiverX",
    "symbol": "QRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12916/thumb/qrx_logo.png?1696512704"
  },
  {
    "chainId": 1,
    "address": "0x8e964e35a76103af4c7d7318e1b1a82c682ae296",
    "name": "Fellaz",
    "symbol": "FLZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26169/thumb/NUERy1Wy_400x400.png?1696525257"
  },
  {
    "chainId": 1,
    "address": "0x7707aada3ce7722ac63b91727daf1999849f6835",
    "name": "Bankera",
    "symbol": "BNK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1949/thumb/bankera.png?1696502933"
  },
  {
    "chainId": 1,
    "address": "0xf94b5c5651c888d928439ab6514b93944eee6f48",
    "name": "Yield App",
    "symbol": "YLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13365/thumb/Google_Play_Store_Icon.png?1696513130"
  },
  {
    "chainId": 1,
    "address": "0x5cf04716ba20127f1e2297addcf4b5035000c9eb",
    "name": "NKN",
    "symbol": "NKN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3375/thumb/nkn.png?1696504074"
  },
  {
    "chainId": 1,
    "address": "0xf058501585023d040ea9493134ed72c083553eed",
    "name": "Dymmax",
    "symbol": "DMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13476/thumb/dmmx.png?1696513237"
  },
  {
    "chainId": 1,
    "address": "0xe64b47931f28f89cc7a0c6965ecf89eadb4975f5",
    "name": "Ludos Protocol",
    "symbol": "LUD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5118/thumb/Ludos_Protocol.jpg?1557216263"
  },
  {
    "chainId": 1,
    "address": "0x7c8155909cd385f120a56ef90728dd50f9ccbe52",
    "name": "Nahmii",
    "symbol": "NII",
    "decimals": 15,
    "logoURI": "https://assets.coingecko.com/coins/images/9786/thumb/nahmii-sm_icon-full-color.png?1696509845"
  },
  {
    "chainId": 1,
    "address": "0xe0ad1806fd3e7edf6ff52fdb822432e847411033",
    "name": "OnX Finance",
    "symbol": "ONX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13445/thumb/onxlogo-1.png?1696513209"
  },
  {
    "chainId": 1,
    "address": "0x909e34d3f6124c324ac83dcca84b74398a6fa173",
    "name": "Panther Protocol",
    "symbol": "ZKP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18611/thumb/panther.jpg?1696518084"
  },
  {
    "chainId": 1,
    "address": "0xf434908dcf8206691bb99cae9232d4833ec257d4",
    "name": "Blackjack fun",
    "symbol": "JACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32499/thumb/logo_J.png?1698309198"
  },
  {
    "chainId": 1,
    "address": "0x965d79f1a1016b574a62986e13ca8ab04dfdd15c",
    "name": "M2",
    "symbol": "M2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18239/thumb/m2_%281%29.png?1696517735"
  },
  {
    "chainId": 1,
    "address": "0x8e0fe2947752be0d5acf73aae77362daf79cb379",
    "name": "NFTrade",
    "symbol": "NFTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18578/thumb/nftrade.png?1696518055"
  },
  {
    "chainId": 1,
    "address": "0x34bdf48a8f753de4822a6cfb1fee275f9b4d662e",
    "name": "FACTS",
    "symbol": "BKC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8461/thumb/sFNOkmVx_400x400.jpg?1696508647"
  },
  {
    "chainId": 1,
    "address": "0x72e203a17add19a3099137c9d7015fd3e2b7dba9",
    "name": "BlockchainPoland",
    "symbol": "BCP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8143/thumb/43Stz1Tw.png?1696508359"
  },
  {
    "chainId": 1,
    "address": "0x64a60493d888728cf42616e034a0dfeae38efcf0",
    "name": "OneLedger",
    "symbol": "OLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3422/thumb/One_Ledger.jpeg?1696504117"
  },
  {
    "chainId": 1,
    "address": "0x6c3f90f043a72fa612cbac8115ee7e52bde6e490",
    "name": "LP 3pool Curve",
    "symbol": "3CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12972/thumb/3pool_128.png?1696512759"
  },
  {
    "chainId": 1,
    "address": "0x8c4e7f814d40f8929f9112c5d09016f923d34472",
    "name": "XCELTOKEN PLUS",
    "symbol": "XLAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8508/thumb/CXQJPNyO_400x400.jpg?1696508687"
  },
  {
    "chainId": 1,
    "address": "0x4297394c20800e8a38a619a243e9bbe7681ff24e",
    "name": "Hot Cross",
    "symbol": "HOTCROSS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15706/thumb/Hotcross.png?1696515334"
  },
  {
    "chainId": 1,
    "address": "0x667102bd3413bfeaa3dffb48fa8288819e480a88",
    "name": "Tokenize Xchange",
    "symbol": "TKX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4984/thumb/TKX_-_Logo_-_RGB-15.png?1696505519"
  },
  {
    "chainId": 1,
    "address": "0x8ef47555856f6ce2e0cd7c36aef4fab317d2e2e2",
    "name": "PayAccept",
    "symbol": "PAYT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12513/thumb/logo.png?1696512328"
  },
  {
    "chainId": 1,
    "address": "0xd32641191578ea9b208125ddd4ec5e7b84fcab4c",
    "name": "MDsquare",
    "symbol": "TMED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8558/thumb/bAdnHTWUIl4rFB4EfbH2L50BW7xi2T9ubci7v6xfYDipXkrG8Wb4LS8g6eAiMsSKqLJfV-FjH5TrYALqACw4MM2A9NfK_L6Ujnddhw1uEgL0hmVQnY96xZeL0C4kkFRJalreK9wUBzjZ4pRhZLdQf9h1drSTF7NE9moAdQ7KtuOvJqommn_q-TjQpF8GUB7J2Ejs7zIvhXXsUHGI1.jpg?1696508730"
  },
  {
    "chainId": 1,
    "address": "0x4d953cf077c0c95ba090226e59a18fcf97db44ec",
    "name": "Mini",
    "symbol": "MINI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12298/thumb/IrTAVc_GqZ7iQucAa3fNYlh_Cqt3tY9wM_pmzOl5SifscRMpuzrp_dizMzGTiMr_NxDJPCKigBgz8THrzvO_DqT3JLzqZIYeytDBRw3qKI73dljS0BnFaaI2aLadpdCZah4RkhydddLIDDbQlGit77farlQRaq7qEgxdjVe0aqEeh4phE-DWAKi_KS_Yz-fFdDfjWgifVCKkZRBeNSWWQEplxxl.jpg?1696512125"
  },
  {
    "chainId": 1,
    "address": "0x0954906da0bf32d5479e25f46056d22f08464cab",
    "name": "Index Cooperative",
    "symbol": "INDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12729/thumb/index.png?1696512528"
  },
  {
    "chainId": 1,
    "address": "0x9565c2036963697786705120fc59310f747bcfd0",
    "name": "PoorPleb",
    "symbol": "PP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28063/thumb/PP_TOKEN_LOGO.png?1696527075"
  },
  {
    "chainId": 1,
    "address": "0x1e18821e69b9faa8e6e75dffe54e7e25754beda0",
    "name": "KIMCHI finance",
    "symbol": "KIMCHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12302/thumb/VBa2Z60o_400x400.png?1696512132"
  },
  {
    "chainId": 1,
    "address": "0x39795344cbcc76cc3fb94b9d1b15c23c2070c66d",
    "name": "Seigniorage Shares",
    "symbol": "SHARE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/12306/thumb/logo_%281%29.png?1696512136"
  },
  {
    "chainId": 1,
    "address": "0x4a621d9f1b19296d1c0f87637b3a8d4978e9bf82",
    "name": "CyberFM",
    "symbol": "CYFM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5476/thumb/cyberfm.png?1696505952"
  },
  {
    "chainId": 1,
    "address": "0x07150e919b4de5fd6a63de1f9384828396f25fdc",
    "name": "Base Protocol",
    "symbol": "BASE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/13265/thumb/200x200green.png?1696513039"
  },
  {
    "chainId": 1,
    "address": "0x054d64b73d3d8a21af3d764efd76bcaa774f3bb2",
    "name": "Plasma Finance",
    "symbol": "PPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13340/thumb/Hi9sEGAD.png?1696513107"
  },
  {
    "chainId": 1,
    "address": "0x41a08648c3766f9f9d85598ff102a08f4ef84f84",
    "name": "Aave Balancer Pool Token",
    "symbol": "ABPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16788/thumb/ABPT_2x.png?1696516360"
  },
  {
    "chainId": 1,
    "address": "0x4fbb350052bca5417566f188eb2ebce5b19bc964",
    "name": "RigoBlock",
    "symbol": "GRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1532/thumb/Symbol-RigoblockRGB.png?1696502572"
  },
  {
    "chainId": 1,
    "address": "0x31fdd1c6607f47c14a2821f599211c67ac20fa96",
    "name": "Burency",
    "symbol": "BUY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12216/thumb/Ddphl9BD.png?1696512050"
  },
  {
    "chainId": 1,
    "address": "0xdf7ff54aacacbff42dfe29dd6144a69b629f8c9e",
    "name": "Aave ZRX",
    "symbol": "AZRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14264/thumb/aZRX.9be555a9.png?1696513965"
  },
  {
    "chainId": 1,
    "address": "0xa1116930326d21fb917d5a27f1e9943a9595fb47",
    "name": "Staked Aave Balancer Pool Token",
    "symbol": "STKABPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16789/thumb/StkABPT_2x.png?1696516360"
  },
  {
    "chainId": 1,
    "address": "0x32a7c02e79c4ea1008dd6564b35f131428673c41",
    "name": "Crust Network",
    "symbol": "CRU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12549/thumb/sAB3KVzD_400x400.jpg?1696512360"
  },
  {
    "chainId": 1,
    "address": "0x87e75025d6607564ad005088ab549dd76ca2ce48",
    "name": "Pier Protocol",
    "symbol": "PIER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35305/thumb/Design_sans_titre__42_-removebg-preview.png?1708154849"
  },
  {
    "chainId": 1,
    "address": "0xf418588522d5dd018b425e472991e52ebbeeeeee",
    "name": "Push Protocol",
    "symbol": "PUSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14769/thumb/aiOxYOJI_400x400.jpeg?1696514438"
  },
  {
    "chainId": 1,
    "address": "0x2904b9b16652d7d0408eccfa23a19d4a3358230f",
    "name": "Puriever",
    "symbol": "PURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12600/thumb/200x200_pure_logo.png?1696512409"
  },
  {
    "chainId": 1,
    "address": "0x4156d3342d5c385a87d264f90653733592000581",
    "name": "SALT",
    "symbol": "SALT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/962/thumb/salt.png?1696502077"
  },
  {
    "chainId": 1,
    "address": "0x8dc2906de0d7409c3cace9482fbffd566639c593",
    "name": "Pepeinu",
    "symbol": "PEPEINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32295/thumb/IMG_20231010_122020_094.jpg?1697184910"
  },
  {
    "chainId": 1,
    "address": "0x940bdcb99a0ee5fb008a606778ae87ed9789f257",
    "name": "JFIN Coin",
    "symbol": "JFIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13370/thumb/JFin-Coin-Logo.png?1696513134"
  },
  {
    "chainId": 1,
    "address": "0x6d4ca1177087924edfe0908ef655169ea766fdc3",
    "name": "Hedgehog",
    "symbol": "HEDGEHOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28552/thumb/hedgie.png?1696527543"
  },
  {
    "chainId": 1,
    "address": "0x7a771a90b8f2a10afd9a75b8e49d646d1139bb98",
    "name": "Stealth Deals",
    "symbol": "DEAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33562/thumb/Stealth-Deals-2.png?1702443551"
  },
  {
    "chainId": 1,
    "address": "0xdc47f2ba852669b178699449e50682d6ceaf8c07",
    "name": "Ston",
    "symbol": "STON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13001/thumb/ston-v2-200x200.png?1696512790"
  },
  {
    "chainId": 1,
    "address": "0x3d3d35bb9bec23b06ca00fe472b50e7a4c692c30",
    "name": "Vidya",
    "symbol": "VIDYA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12219/thumb/VIDYA_TOKEN.png?1696512053"
  },
  {
    "chainId": 1,
    "address": "0x0aacfbec6a24756c20d41914f2caba817c0d8521",
    "name": "YAM",
    "symbol": "YAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12530/thumb/YAM-icon.png?1696512343"
  },
  {
    "chainId": 1,
    "address": "0x0a6c7d5b442ddf53d963581d87932f25743dd3a0",
    "name": "SimpleHub",
    "symbol": "SHUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33024/thumb/imgonline-com-ua-Resize-1akYG0UVinM7FOR4.png?1700317445"
  },
  {
    "chainId": 1,
    "address": "0x65ccd72c0813ce6f2703593b633202a0f3ca6a0c",
    "name": "Nestree",
    "symbol": "EGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9362/thumb/nestree.png?1696509461"
  },
  {
    "chainId": 1,
    "address": "0x0a913bead80f321e7ac35285ee10d9d922659cb7",
    "name": "DOS Network",
    "symbol": "DOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7991/thumb/DOS.png?1696508217"
  },
  {
    "chainId": 1,
    "address": "0x38a2fdc11f526ddd5a607c1f251c065f40fbf2f7",
    "name": "PhoenixDAO",
    "symbol": "PHNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11523/thumb/Token_Icon.png?1696511425"
  },
  {
    "chainId": 1,
    "address": "0xd9016a907dc0ecfa3ca425ab20b6b785b42f2373",
    "name": "GAMEE",
    "symbol": "GMEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14716/thumb/gmee-200x200.png?1696514386"
  },
  {
    "chainId": 1,
    "address": "0x2c537e5624e4af88a7ae4060c022609376c8d0eb",
    "name": "BiLira",
    "symbol": "TRYB",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/10119/thumb/JBs9jiXO_400x400.jpg?1696510144"
  },
  {
    "chainId": 1,
    "address": "0xa8262eb913fccea4c3f77fc95b8b4043b384cfbb",
    "name": "Krypton Galaxy Coin",
    "symbol": "KGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9152/thumb/krypton.PNG?1696509275"
  },
  {
    "chainId": 1,
    "address": "0x49849c98ae39fff122806c06791fa73784fb3675",
    "name": "LP renBTC Curve",
    "symbol": "RENBTCCURVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11957/thumb/Curvefi_renCrv_32.png?1696511817"
  },
  {
    "chainId": 1,
    "address": "0x84ca8bc7997272c7cfb4d0cd3d55cd942b3c9419",
    "name": "DIA",
    "symbol": "DIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11955/thumb/Token_Logo.png?1696511815"
  },
  {
    "chainId": 1,
    "address": "0x525794473f7ab5715c81d06d10f52d11cc052804",
    "name": "12Ships",
    "symbol": "TSHP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9351/thumb/12ships.png?1696509451"
  },
  {
    "chainId": 1,
    "address": "0x593114f03a0a575aece9ed675e52ed68d2172b8c",
    "name": "BidiPass",
    "symbol": "BDP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3890/thumb/AxiFAoHc_400x400.jpg?1696504552"
  },
  {
    "chainId": 1,
    "address": "0x8806926ab68eb5a7b909dcaf6fdbe5d93271d6e2",
    "name": "Uquid Coin",
    "symbol": "UQC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1341/thumb/uquid-coin.png?1696502403"
  },
  {
    "chainId": 1,
    "address": "0x7a939bb714fd2a48ebeb1e495aa9aaa74ba9fa68",
    "name": "Electric Vehicle Zone",
    "symbol": "EVZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9456/thumb/kLohzILUIln6mHFYOlecpWjINVIH-BVghP2vRTeuD0XteaQa7Lpn4sLcuPN4gHw8MU2pKWZCJRNwBmyyl1CYxplCLDcgSVihMC7vvfmkepY-_O_ImWBA27s4pKNlhcBnBYrc8y5WH0ZB2CjmqPh-32nPslrv329tqFWr2DAR8dl4R5LZGgeZ1ubCdtMoUua6gEL3umYShHBxrYLto.jpg?1696509547"
  },
  {
    "chainId": 1,
    "address": "0xaa2ce7ae64066175e0b90497ce7d9c190c315db4",
    "name": "Suterusu",
    "symbol": "SUTER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9830/thumb/p-NFlBlw_400x400.jpg?1696509886"
  },
  {
    "chainId": 1,
    "address": "0xface851a4921ce59e912d19329929ce6da6eb0c7",
    "name": "cLINK",
    "symbol": "CLINK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/23943/thumb/iShot2022-02-25_14.35.54.png?1696523139"
  },
  {
    "chainId": 1,
    "address": "0x2fd61567c29e7adb4ca17e60e1f4a3fcfe68acb8",
    "name": "SymVerse",
    "symbol": "SYM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9449/thumb/Picture1.png?1696509540"
  },
  {
    "chainId": 1,
    "address": "0x8e30ea2329d95802fd804f4291220b0e2f579812",
    "name": "Decentralized Vulnerability Platform",
    "symbol": "DVP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9424/thumb/4520.png?1696509517"
  },
  {
    "chainId": 1,
    "address": "0x20c36f062a31865bed8a5b1e512d9a1a20aa333a",
    "name": "DefiDollar DAO",
    "symbol": "DFD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12959/thumb/DFD.jpg?1696512747"
  },
  {
    "chainId": 1,
    "address": "0xffe510a92434a0df346c5e72a3494b043cf249eb",
    "name": "LUX BIO EXCHANGE COIN",
    "symbol": "LBXC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9482/thumb/sam7Fy3A_400x400.jpg?1696509569"
  },
  {
    "chainId": 1,
    "address": "0xc77b230f31b517f1ef362e59c173c2be6540b5e8",
    "name": "VIDY",
    "symbol": "VIDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5088/thumb/vidy-logo.jpg?1696505610"
  },
  {
    "chainId": 1,
    "address": "0x37f74e99794853777a10ea1dc08a64c86958f06a",
    "name": "D Community",
    "symbol": "DILI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9485/thumb/BYf_d6kekNWOY34VLGPvf7RcVj8SeRi7P65Nk0W7ji1jir_qSLfssffnNW0ZzQSWzhhsNRkMCL7qcX4CCaMTbgKb0D7b5yNeStKVMm2deZd9cONMuItgoKUXxIYkHElfk1PW2uiNZbg6a2K0OHtZTvkt1xo8YVKuthskXaryOkM6Ax-MXQlHOfbjwCcnQQjHxAucufrCZJfDgrY.jpg?1696509571"
  },
  {
    "chainId": 1,
    "address": "0x7fbec0bb6a7152e77c30d005b5d49cbc08a602c3",
    "name": "disBalancer",
    "symbol": "DDOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14791/thumb/communityIcon_o2yriheuszk61.png?1696514460"
  },
  {
    "chainId": 1,
    "address": "0x6fe56c0bcdd471359019fcbc48863d6c3e9d4f41",
    "name": "Props",
    "symbol": "PROPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6735/thumb/photo-2017-10-10-03-32-02.png?1696507070"
  },
  {
    "chainId": 1,
    "address": "0xfd957f21bd95e723645c07c48a2d8acb8ffb3794",
    "name": "Ethereum Meta",
    "symbol": "ETHM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6586/thumb/ethereum-meta.png?1696506937"
  },
  {
    "chainId": 1,
    "address": "0x05bbe7240de66f6480c9aeda77c1376b13393f83",
    "name": "Xeno",
    "symbol": "XNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13830/thumb/XNO_logo_200x200.png?1696513573"
  },
  {
    "chainId": 1,
    "address": "0x808632424440f6483d9be7509812b53ed0f00344",
    "name": "Mechazilla",
    "symbol": "MECHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33025/thumb/MECHA.jpg?1700318938"
  },
  {
    "chainId": 1,
    "address": "0xb19dd661f076998e3b0456935092a233e12c2280",
    "name": "Continuum World",
    "symbol": "UM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18798/thumb/Moneda.png?1696518260"
  },
  {
    "chainId": 1,
    "address": "0xf2cdf38e24738ba379ffa38d47bc88a941df5627",
    "name": "Ally",
    "symbol": "ALY",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/9619/thumb/BTf_sTXi_400x400.jpg?1696509692"
  },
  {
    "chainId": 1,
    "address": "0x6c3be406174349cfa4501654313d97e6a31072e1",
    "name": "CNNS",
    "symbol": "CNNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8362/thumb/cnns.png?1696508555"
  },
  {
    "chainId": 1,
    "address": "0x62dc4817588d53a056cbbd18231d91ffccd34b2a",
    "name": "DeHive",
    "symbol": "DHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14926/thumb/logo_200x200.png?1696514587"
  },
  {
    "chainId": 1,
    "address": "0x3e8203e0b1d56984abc66f183a8d0b1a09a7e607",
    "name": "Liquid Protocol",
    "symbol": "LP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32078/thumb/5000x5000.png?1696530875"
  },
  {
    "chainId": 1,
    "address": "0xe0b9bcd54bf8a730ea5d3f1ffce0885e911a502c",
    "name": "ZUM",
    "symbol": "ZUM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9721/thumb/zum256x256.png?1696509787"
  },
  {
    "chainId": 1,
    "address": "0x467bccd9d29f223bce8043b84e8c8b282827790f",
    "name": "Telcoin",
    "symbol": "TEL",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/1899/thumb/tel.png?1696502892"
  },
  {
    "chainId": 1,
    "address": "0x5d60d8d7ef6d37e16ebabc324de3be57f135e0bc",
    "name": "MyBit",
    "symbol": "MYB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1240/thumb/mybit.png?1696502315"
  },
  {
    "chainId": 1,
    "address": "0x8a854288a5976036a725879164ca3e91d30c6a1b",
    "name": "GET Protocol",
    "symbol": "GET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1927/thumb/GET_Protocol.png?1696502916"
  },
  {
    "chainId": 1,
    "address": "0x65a8fba02f641a13bb7b01d5e1129b0521004f52",
    "name": "Amasa",
    "symbol": "AMAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18799/thumb/agmqWjv8_400x400.png?1696518261"
  },
  {
    "chainId": 1,
    "address": "0x5fbc2ffe91ac74e3e286bd7504b233f0e5291c69",
    "name": "EarnBet",
    "symbol": "EBET",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32032/thumb/logo.png?1696530829"
  },
  {
    "chainId": 1,
    "address": "0xd7d05bda4bf5876ba1254b3eaaf8b47d2f5676eb",
    "name": "STABLE ASSET",
    "symbol": "STA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12313/thumb/stable_200x200.png?1696512142"
  },
  {
    "chainId": 1,
    "address": "0xfae4ee59cdd86e3be9e8b90b53aa866327d7c090",
    "name": "CPChain",
    "symbol": "CPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2760/thumb/cpchain.png?1696503533"
  },
  {
    "chainId": 1,
    "address": "0x33d0568941c0c64ff7e0fb4fba0b11bd37deed9f",
    "name": "RAMP  OLD ",
    "symbol": "RAMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12837/thumb/RAMP-Logo-v2-1000pxsq.png?1696512628"
  },
  {
    "chainId": 1,
    "address": "0x249e38ea4102d0cf8264d3701f1a0e39c4f2dc3b",
    "name": "UFO Gaming",
    "symbol": "UFO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16801/thumb/ufo.png?1696516371"
  },
  {
    "chainId": 1,
    "address": "0xaac679720204aaa68b6c5000aa87d789a3ca0aa5",
    "name": "Huobi Bitcoin Cash",
    "symbol": "HBCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14105/thumb/HBCh.png?1696513826"
  },
  {
    "chainId": 1,
    "address": "0xb705268213d593b8fd88d3fdeff93aff5cbdcfae",
    "name": "IDEX",
    "symbol": "IDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2565/thumb/idexlogo.png?1696503371"
  },
  {
    "chainId": 1,
    "address": "0x24da31e7bb182cb2cabfef1d88db19c2ae1f5572",
    "name": "Shikoku",
    "symbol": "SHIK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28560/thumb/shikoku_inu02.png?1696527550"
  },
  {
    "chainId": 1,
    "address": "0xc1abb8c93be6811affc70675b0432926c4bfbb5d",
    "name": "UERII",
    "symbol": "UERII",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29388/thumb/Avatar512.png?1696528334"
  },
  {
    "chainId": 1,
    "address": "0x6c936d4ae98e6d2172db18c16c4b601c99918ee6",
    "name": "Life Crypto",
    "symbol": "LIFE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18245/thumb/communityIcon_t3kzc5skazh81.png?1696517741"
  },
  {
    "chainId": 1,
    "address": "0xf80d589b3dbe130c270a69f1a69d050f268786df",
    "name": "Datamine",
    "symbol": "DAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11695/thumb/qxsFH8W.png?1696511584"
  },
  {
    "chainId": 1,
    "address": "0x03806ce5ef69bd9780edfb04c29da1f23db96294",
    "name": "Tesla TSL",
    "symbol": "TSL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1222/thumb/energo.png?1696502301"
  },
  {
    "chainId": 1,
    "address": "0x2c974b2d0ba1716e644c1fc59982a89ddd2ff724",
    "name": "Viberate",
    "symbol": "VIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/983/thumb/Viberate.png?1696502096"
  },
  {
    "chainId": 1,
    "address": "0x5d5e244660ca05c42073c9a526616d99f2c99516",
    "name": "Game Tree",
    "symbol": "GTCOIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24719/thumb/hUEmunGU_400x400.jpg?1696523883"
  },
  {
    "chainId": 1,
    "address": "0xf5717f5df41ea67ef67dfd3c1d02f9940bcf5d08",
    "name": "SeChain",
    "symbol": "SNN",
    "decimals": 3,
    "logoURI": "https://assets.coingecko.com/coins/images/11961/thumb/2AsGMTeI_400x400.jpg?1696511821"
  },
  {
    "chainId": 1,
    "address": "0x0ff6ffcfda92c53f615a4a75d982f399c989366b",
    "name": "UniLayer",
    "symbol": "LAYER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12164/thumb/logo-layer.jpg?1696512002"
  },
  {
    "chainId": 1,
    "address": "0xaec2e87e0a235266d9c5adc9deb4b2e29b54d009",
    "name": "SingularDTV",
    "symbol": "SNGLS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/521/thumb/singulardtv.png?1696501742"
  },
  {
    "chainId": 1,
    "address": "0x604026696fdb3c6720ae3049c46d59ac604dea0a",
    "name": "eXciting Japan Coin",
    "symbol": "XJP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11963/thumb/XJP_LOGO.png?1696511822"
  },
  {
    "chainId": 1,
    "address": "0x301c755ba0fca00b1923768fffb3df7f4e63af31",
    "name": "Global Digital Content",
    "symbol": "GDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10122/thumb/Vp4L1x4Hu4HEuEPUl0gSocDaVifkXlCqSd8A72ml_Q0uwEc9uBWBTwjS43_NP8eiiZVveJDIQXguWUXYk3MLZiFOER-8CGO120I0NGU_L8aTJ8c50a-O_AbkY-4nUkPYhUTNgf3rkcqkXnKecixQwLfkJG2XTCLwNvo_ox05x0_hU1Z__FOT4O7AeU-PC6skc85NCVS68cnZjxcUBOY9K.jpg?1696510147"
  },
  {
    "chainId": 1,
    "address": "0x0a7b89e66a1dc16633abdfd132bae05827d3bfa5",
    "name": "hiMOONBIRDS",
    "symbol": "HIMOONBIRDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28663/thumb/himoonbirds.png?1696527648"
  },
  {
    "chainId": 1,
    "address": "0xa3e54198998d800703399f3bfa38be4e14045036",
    "name": "REVIVAL",
    "symbol": "REVIVAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32500/thumb/revival.jpg?1698311388"
  },
  {
    "chainId": 1,
    "address": "0xfca47962d45adfdfd1ab2d972315db4ce7ccf094",
    "name": "iXledger",
    "symbol": "IXT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/814/thumb/ixledger.png?1696501963"
  },
  {
    "chainId": 1,
    "address": "0x79be75ffc64dd58e66787e4eae470c8a1fd08ba4",
    "name": "Aave AMM DAI",
    "symbol": "AAMMDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17197/thumb/aAMMDAI_2x.png?1696516754"
  },
  {
    "chainId": 1,
    "address": "0x17a79792fe6fe5c95dfe95fe3fcee3caf4fe4cb7",
    "name": "Aave AMM USDT",
    "symbol": "AAMMUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17200/thumb/aAMMUSDT_2x.png?1696516757"
  },
  {
    "chainId": 1,
    "address": "0x98968f0747e0a261532cacc0be296375f5c08398",
    "name": "MOONCAT Vault  NFTX ",
    "symbol": "MOONCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17055/thumb/Mooncats.png?1696516618"
  },
  {
    "chainId": 1,
    "address": "0x2c8ea636345a231e4b1a28f6eeb2072ed909c406",
    "name": "Meme Elon Doge Floki",
    "symbol": "MEMELON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32609/thumb/MEMELON.jpg?1698733326"
  },
  {
    "chainId": 1,
    "address": "0xacbd826394189cf2623c6df98a18b41fc8ffc16d",
    "name": "NFTify",
    "symbol": "N1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16095/thumb/n1-token-logo-circle-200x200.png?1696515703"
  },
  {
    "chainId": 1,
    "address": "0x89509aa1d14a8e1e5364ec4c3b041213bcdbe08d",
    "name": "ZURRENCY",
    "symbol": "ZURR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22240/thumb/logo.png?1696521585"
  },
  {
    "chainId": 1,
    "address": "0x757c67cb79a4dc8af386cb3abf32359c9a99f652",
    "name": "Money Laundering Protocol",
    "symbol": "MLP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32502/thumb/IMG_6964.jpeg?1698312926"
  },
  {
    "chainId": 1,
    "address": "0x2e95cea14dd384429eb3c4331b776c4cfbb6fcd9",
    "name": "Throne",
    "symbol": "THN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16965/thumb/aXhD3i_g_400x400.jpg?1696516531"
  },
  {
    "chainId": 1,
    "address": "0xe67f943af5eb6051ef56f05979cc30b732717fa6",
    "name": "WATTTON",
    "symbol": "WATT",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/27215/thumb/CG_WATT_LOGO.png?1696526262"
  },
  {
    "chainId": 1,
    "address": "0x2fe39f22eac6d3c1c86dd9d143640ebb94609fce",
    "name": "JD Coin",
    "symbol": "JDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9958/thumb/Cv-2ca6p_400x400.jpg?1696509998"
  },
  {
    "chainId": 1,
    "address": "0xdbdb4d16eda451d0503b854cf79d55697f90c8df",
    "name": "Alchemix",
    "symbol": "ALCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14113/thumb/Alchemix.png?1696513834"
  },
  {
    "chainId": 1,
    "address": "0x6a7ef4998eb9d0f706238756949f311a59e05745",
    "name": "Keysians Network",
    "symbol": "KEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12141/thumb/Keysians_logo.jpg?1696511982"
  },
  {
    "chainId": 1,
    "address": "0x48fb253446873234f2febbf9bdeaa72d9d387f94",
    "name": "Bancor Governance",
    "symbol": "VBNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14053/thumb/Bancor_Token.png?1710479583"
  },
  {
    "chainId": 1,
    "address": "0xbc6da0fe9ad5f3b0d58160288917aa56653660e9",
    "name": "Alchemix USD",
    "symbol": "ALUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14114/thumb/Alchemix_USD.png?1696513835"
  },
  {
    "chainId": 1,
    "address": "0xdfe691f37b6264a90ff507eb359c45d55037951c",
    "name": "Karma DAO",
    "symbol": "KARMA",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/11884/thumb/Karma.png?1696511751"
  },
  {
    "chainId": 1,
    "address": "0x3301ee63fb29f863f2333bd4466acb46cd8323e6",
    "name": "Akita Inu",
    "symbol": "AKITA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14115/thumb/akita.png?1696513836"
  },
  {
    "chainId": 1,
    "address": "0xac6df26a590f08dcc95d5a4705ae8abbc88509ef",
    "name": "Aave ENJ",
    "symbol": "AENJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14251/thumb/aENJ.ed76293b.png?1696513964"
  },
  {
    "chainId": 1,
    "address": "0x7866e48c74cbfb8183cd1a929cd9b95a7a5cb4f4",
    "name": "DexKit",
    "symbol": "KIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13187/thumb/7739.png?1696512969"
  },
  {
    "chainId": 1,
    "address": "0x08d967bb0134f2d07f7cfb6e246680c53927dd30",
    "name": "MATH",
    "symbol": "MATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11335/thumb/2020-05-19-token-200.png?1696511257"
  },
  {
    "chainId": 1,
    "address": "0x035df12e0f3ac6671126525f1015e47d79dfeddf",
    "name": "0xMonero",
    "symbol": "0XMR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11035/thumb/0xmnr.PNG?1696510979"
  },
  {
    "chainId": 1,
    "address": "0xdf574c24545e5ffecb9a659c229253d4111d87e1",
    "name": "HUSD",
    "symbol": "HUSD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9567/thumb/HUSD.jpg?1696509647"
  },
  {
    "chainId": 1,
    "address": "0xc7283b66eb1eb5fb86327f08e1b5816b0720212b",
    "name": "Tribe",
    "symbol": "TRIBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14575/thumb/tribe.PNG?1696514256"
  },
  {
    "chainId": 1,
    "address": "0x50bc2ecc0bfdf5666640048038c1aba7b7525683",
    "name": "carVertical",
    "symbol": "CV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2098/thumb/carvertical.png?1696503065"
  },
  {
    "chainId": 1,
    "address": "0x70881d5c8a5950ceedf1f1b4b5d4105718642548",
    "name": "Bagholder",
    "symbol": "BAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30503/thumb/rsz_bagholdrpepecut-sq%281%29.png?1696529389"
  },
  {
    "chainId": 1,
    "address": "0xa685a61171bb30d4072b338c80cb7b2c865c873e",
    "name": "Aave MANA",
    "symbol": "AMANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14267/thumb/aMANA.f89a1098.png?1696513968"
  },
  {
    "chainId": 1,
    "address": "0xb53ecf1345cabee6ea1a65100ebb153cebcac40f",
    "name": "Childhoods End",
    "symbol": "O",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25670/thumb/photo_2022-05-27_12-45-29.jpg?1696524798"
  },
  {
    "chainId": 1,
    "address": "0xdd16ec0f66e54d453e6756713e533355989040e4",
    "name": "Tokenomy",
    "symbol": "TEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2286/thumb/logo_%281%29.png?1696503190"
  },
  {
    "chainId": 1,
    "address": "0x0d15009896efe9972f8e086bdd3bcba5c1f74bf3",
    "name": "SonoCoin",
    "symbol": "SONO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7230/thumb/MeWTs09-_400x400.jpg?1696507526"
  },
  {
    "chainId": 1,
    "address": "0x9ee91f9f426fa633d227f7a9b000e28b9dfd8599",
    "name": "Lido Staked Matic",
    "symbol": "STMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24185/thumb/stMATIC.png?1696523373"
  },
  {
    "chainId": 1,
    "address": "0x85614a474dbeed440d5bbdb8ac50b0f22367f997",
    "name": "Verge  ETH ",
    "symbol": "XVG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32585/thumb/Verge_Token_%28erc-20%29_icon_200x200.jpg?1699220724"
  },
  {
    "chainId": 1,
    "address": "0x74faab6986560fd1140508e4266d8a7b87274ffd",
    "name": "HyperDAO",
    "symbol": "HDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10780/thumb/B7-ppPfE_400x400.png?1696510742"
  },
  {
    "chainId": 1,
    "address": "0xedee9cc3bf9cfd86a493a23a0775508e0f826644",
    "name": "Atalis",
    "symbol": "ALS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35857/thumb/logo200.jpg?1709896205"
  },
  {
    "chainId": 1,
    "address": "0x2af5d2ad76741191d15dfe7bf6ac92d4bd912ca3",
    "name": "LEO Token",
    "symbol": "LEO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8418/thumb/leo-token.png?1696508607"
  },
  {
    "chainId": 1,
    "address": "0x666d875c600aa06ac1cf15641361dec3b00432ef",
    "name": "BTSE Token",
    "symbol": "BTSE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10807/thumb/BTSE_logo_Square.jpeg?1696510765"
  },
  {
    "chainId": 1,
    "address": "0x95e40e065afb3059dcabe4aaf404c1f92756603a",
    "name": "King DAG",
    "symbol": "KDAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10809/thumb/3xcLUorv_400x400.jpg?1696510767"
  },
  {
    "chainId": 1,
    "address": "0x4e9fcd48af4738e3bf1382009dc1e93ebfce698f",
    "name": "TAO INU",
    "symbol": "TAONU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33145/thumb/656b24d78397f73bfbadb752_TAONU_Logo_100x100.jpg?1701861274"
  },
  {
    "chainId": 1,
    "address": "0x8400d94a5cb0fa0d041a3788e395285d61c9ee5e",
    "name": "Unibright",
    "symbol": "UBT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2707/thumb/UnibrightLogo_colorful_500x500_preview.png?1696503489"
  },
  {
    "chainId": 1,
    "address": "0x62431de84c503e152a8957ff51c8945aaaa7d929",
    "name": "BlastAI",
    "symbol": "BLAST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35858/thumb/Design_sans_titre_%2818%29_15.51.23.png?1709896264"
  },
  {
    "chainId": 1,
    "address": "0xcb2b9b5b136dc29feb0548dff315021b9b6c2ba0",
    "name": "GigaToken",
    "symbol": "GIGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34899/thumb/Giga_Logo.png?1708020799"
  },
  {
    "chainId": 1,
    "address": "0xc626e0619ac79afea9281c8eb9b1a9f9d3fab532",
    "name": "Freedom Reserve",
    "symbol": "FR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13342/thumb/J6uNL2FS_400x400.jpg?1696513109"
  },
  {
    "chainId": 1,
    "address": "0xd7efb00d12c2c13131fd319336fdf952525da2af",
    "name": "XPR Network",
    "symbol": "XPR",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/10941/thumb/XPR.jpg?1696510891"
  },
  {
    "chainId": 1,
    "address": "0xe38149abc673a117abeb8af7d1ff3d0d1aa5af15",
    "name": "Astrid Restaked cbETH",
    "symbol": "RCBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32268/thumb/astridlogo_copy.png?1697166282"
  },
  {
    "chainId": 1,
    "address": "0x57b946008913b82e4df85f501cbaed910e58d26c",
    "name": "Marlin",
    "symbol": "POND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8903/thumb/200x200.png?1706115827"
  },
  {
    "chainId": 1,
    "address": "0x8578530205cecbe5db83f7f29ecfeec860c297c2",
    "name": "smARTOFGIVING",
    "symbol": "AOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6050/thumb/logo_%286%29.png?1696506459"
  },
  {
    "chainId": 1,
    "address": "0x979aca85ba37c675e78322ed5d97fa980b9bdf00",
    "name": "FUSION",
    "symbol": "FSN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2515/thumb/Fusion_200x200.png?1696503329"
  },
  {
    "chainId": 1,
    "address": "0x9cb1aeafcc8a9406632c5b084246ea72f62d37b6",
    "name": "LBK",
    "symbol": "LBK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9492/thumb/lbk.jpeg?1696509578"
  },
  {
    "chainId": 1,
    "address": "0x79c9e0410b6615e7ba9dd69614b0519325a2b047",
    "name": "hiFLUF",
    "symbol": "HIFLUF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28602/thumb/hifluf.png?1696527588"
  },
  {
    "chainId": 1,
    "address": "0x8290333cef9e6d528dd5618fb97a76f268f3edd4",
    "name": "Ankr Network",
    "symbol": "ANKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4324/thumb/U85xTl2.png?1696504928"
  },
  {
    "chainId": 1,
    "address": "0x42726d074bba68ccc15200442b72afa2d495a783",
    "name": "Isiklar Coin",
    "symbol": "ISIKC",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/10992/thumb/logo_%2866%29.png?1696510941"
  },
  {
    "chainId": 1,
    "address": "0x2ef52ed7de8c5ce03a4ef0efbe9b7450f2d7edc9",
    "name": "Revain",
    "symbol": "REV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/1107/thumb/256x256.png?1696502203"
  },
  {
    "chainId": 1,
    "address": "0xc237868a9c5729bdf3173dddacaa336a0a5bb6e0",
    "name": "Wagerr",
    "symbol": "WGR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/759/thumb/syGKmAT.png?1696501912"
  },
  {
    "chainId": 1,
    "address": "0xa0cf46eb152656c7090e769916eb44a138aaa406",
    "name": "Spheroid Universe",
    "symbol": "SPH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11968/thumb/Frame_36.png?1696511827"
  },
  {
    "chainId": 1,
    "address": "0xde2f7766c8bf14ca67193128535e5c7454f8387c",
    "name": "Metadium",
    "symbol": "META",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5247/thumb/META_Logo_black.png?1696505751"
  },
  {
    "chainId": 1,
    "address": "0x72dd4b6bd852a3aa172be4d6c5a6dbec588cf131",
    "name": "NAGA",
    "symbol": "NGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1439/thumb/naga.jpg?1696502489"
  },
  {
    "chainId": 1,
    "address": "0xd31a59c85ae9d8edefec411d448f90841571b89c",
    "name": "SOL  Wormhole ",
    "symbol": "SOL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22876/thumb/SOL_wh_small.png?1696522175"
  },
  {
    "chainId": 1,
    "address": "0x69cbaf6c147086c3c234385556f8a0c6488d3420",
    "name": "69420",
    "symbol": "69420",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32830/thumb/bg_f8f8f8-flat_750x_075_f-pad_750x1000_f8f8f8.jpg?1699585611"
  },
  {
    "chainId": 1,
    "address": "0xb1a30851e3f7d841b231b086479608e17198363a",
    "name": "Homeros",
    "symbol": "HMR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11048/thumb/g1jAE5knZSlNUdLN8fEAQFMI4mTXCuNdrjj3i77rWhsIQADp3VWH2BHTkH7-oWwExxJk7Fu545LPeT3gMNzGlCZ63oX3-9sTjiqlSDo3fRLwHmhUgjD_jtKTl1AJ_F_UOOpsTqob9CxiWHXsSTlz0zODxOrjnqpMe4_cPT_C4EskSEVee_oooTasTQ6ONrv85Zcvc8Eeb8cHLsV.jpg?1696510991"
  },
  {
    "chainId": 1,
    "address": "0x0e4188bda0a92ff4fff62392211b076c0119198d",
    "name": "0xSnipeProAi",
    "symbol": "0XSPAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34331/thumb/0XSPAI.jpg?1704668642"
  },
  {
    "chainId": 1,
    "address": "0xe5acbb03d73267c03349c76ead672ee4d941f499",
    "name": "BEAM",
    "symbol": "BEAM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7339/thumb/BEAM.png?1696507623"
  },
  {
    "chainId": 1,
    "address": "0xc4de189abf94c57f396bd4c52ab13b954febefd8",
    "name": "B20",
    "symbol": "B20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13803/thumb/b20.png?1696513550"
  },
  {
    "chainId": 1,
    "address": "0x8fc9b6354e839ab1c8b31f4afa53607092b8c2e5",
    "name": "ECOSC",
    "symbol": "ECU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10989/thumb/ecosc-logo.png?1696510938"
  },
  {
    "chainId": 1,
    "address": "0x3d382228c54736d831fac2748f4734d9177c7332",
    "name": "Aniverse",
    "symbol": "ANV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16348/thumb/anv.PNG?1696515948"
  },
  {
    "chainId": 1,
    "address": "0xb8fa12f8409da31a4fc43d15c4c78c33d8213b9b",
    "name": "CaliCoin",
    "symbol": "CALI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14998/thumb/cropped-Logo-transparent-background-1.png?1696514662"
  },
  {
    "chainId": 1,
    "address": "0x8716fc5da009d3a208f0178b637a50f4ef42400f",
    "name": "Ultrain",
    "symbol": "UGAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4684/thumb/ultrain.png?1696505251"
  },
  {
    "chainId": 1,
    "address": "0x8a9c67fee641579deba04928c4bc45f66e26343a",
    "name": "Jarvis Reward",
    "symbol": "JRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10390/thumb/cfeii0y.png?1696510389"
  },
  {
    "chainId": 1,
    "address": "0x3affcca64c2a6f4e3b6bd9c64cd2c969efd1ecbe",
    "name": "DSLA Protocol",
    "symbol": "DSLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6694/thumb/dsla_logo-squared_200x200.png?1696507035"
  },
  {
    "chainId": 1,
    "address": "0x9343e24716659a3551eb10aff9472a2dcad5db2d",
    "name": "STFX",
    "symbol": "STFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28631/thumb/stfx.png?1696527615"
  },
  {
    "chainId": 1,
    "address": "0xf293d23bf2cdc05411ca0eddd588eb1977e8dcd4",
    "name": "Sylo",
    "symbol": "SYLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6430/thumb/SYLO.svg?1696506795"
  },
  {
    "chainId": 1,
    "address": "0xfd6c31bb6f05fc8db64f4b740ab758605c271fd8",
    "name": "Contracoin",
    "symbol": "CTCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10801/thumb/Contracoin-symbol.png?1696510760"
  },
  {
    "chainId": 1,
    "address": "0x05079687d35b93538cbd59fe5596380cae9054a9",
    "name": "BitSong",
    "symbol": "BTSG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5041/thumb/logo_-_2021-01-10T210801.390.png?1696505572"
  },
  {
    "chainId": 1,
    "address": "0xe0bceef36f3a6efdd5eebfacd591423f8549b9d5",
    "name": "Defactor",
    "symbol": "FACTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19201/thumb/jFLSu4U9_400x400.png?1696518648"
  },
  {
    "chainId": 1,
    "address": "0x52498f8d9791736f1d6398fe95ba3bd868114d10",
    "name": "Netvrk",
    "symbol": "NETVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15721/thumb/netvrk_icon.png?1696515348"
  },
  {
    "chainId": 1,
    "address": "0x73d7c860998ca3c01ce8c808f5577d94d545d1b4",
    "name": "IX Swap",
    "symbol": "IXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18069/thumb/ixswap.PNG?1696517577"
  },
  {
    "chainId": 1,
    "address": "0xa7de087329bfcda5639247f96140f9dabe3deed1",
    "name": "Statera",
    "symbol": "STA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11472/thumb/Statera.png?1696511379"
  },
  {
    "chainId": 1,
    "address": "0x82a77710495a35549d2add797412b4a4497d33ef",
    "name": "Dogz",
    "symbol": "DOGZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7544/thumb/dogz.png?1696507809"
  },
  {
    "chainId": 1,
    "address": "0x6fb3e0a217407efff7ca062d46c26e5d60a14d69",
    "name": "IoTeX",
    "symbol": "IOTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3334/thumb/iotex-logo.png?1696504041"
  },
  {
    "chainId": 1,
    "address": "0xc4c346edc55504574cceb00aa1091d22404a4bc3",
    "name": "MEZZ",
    "symbol": "MEZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29344/thumb/20230307_234212.png?1696528293"
  },
  {
    "chainId": 1,
    "address": "0x506b8f75bdef0edac36b0a6f9cf313485e4341b0",
    "name": "WCAPES",
    "symbol": "WCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29361/thumb/wca.png?1696528309"
  },
  {
    "chainId": 1,
    "address": "0x580c8520deda0a441522aeae0f9f7a5f29629afa",
    "name": "Dawn Protocol",
    "symbol": "DAWN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11555/thumb/dawn_protocol.png?1696511454"
  },
  {
    "chainId": 1,
    "address": "0x66fd97a78d8854fec445cd1c80a07896b0b4851f",
    "name": "Lunch Money",
    "symbol": "LMY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4149/thumb/IMG_2073.png?1696504774"
  },
  {
    "chainId": 1,
    "address": "0x3a880652f47bfaa771908c07dd8673a787daed3a",
    "name": "DerivaDAO",
    "symbol": "DDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13453/thumb/ddx_logo.png?1696513216"
  },
  {
    "chainId": 1,
    "address": "0x05030203674173fa6df6f9f7e34d6e70e9a761d7",
    "name": "Muverse",
    "symbol": "MU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28857/thumb/20230130_100626.jpg?1696527831"
  },
  {
    "chainId": 1,
    "address": "0x5e3346444010135322268a4630d2ed5f8d09446c",
    "name": "LockTrip",
    "symbol": "LOC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1357/thumb/LOC_Transparent_New.png?1696502416"
  },
  {
    "chainId": 1,
    "address": "0x1c95b093d6c236d3ef7c796fe33f9cc6b8606714",
    "name": "BOMB",
    "symbol": "BOMB",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/7713/thumb/Bomb-token.png?1549944422"
  },
  {
    "chainId": 1,
    "address": "0xc538143202f3b11382d8606aae90a96b042a19db",
    "name": "Coinsbit Token",
    "symbol": "CNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10340/thumb/cbicon.png?1696510342"
  },
  {
    "chainId": 1,
    "address": "0xc06d9013a1d3f25f76ee5291bbc04a181985814e",
    "name": "Yertle The Turtle",
    "symbol": "YERTLE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32518/thumb/IMG_6911.jpeg?1698406852"
  },
  {
    "chainId": 1,
    "address": "0x9ed8e7c9604790f7ec589f99b94361d8aab64e5e",
    "name": "Unistake",
    "symbol": "UNISTAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12813/thumb/unistake.png?1696512606"
  },
  {
    "chainId": 1,
    "address": "0x58b6a8a3302369daec383334672404ee733ab239",
    "name": "Livepeer",
    "symbol": "LPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7137/thumb/logo-circle-green.png?1696507437"
  },
  {
    "chainId": 1,
    "address": "0xe69d699cbcd79cb1b55f82cf36bf1a2836053562",
    "name": "DropCoin",
    "symbol": "DROP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32835/thumb/Frame_5_%2848%29.jpg?1699586509"
  },
  {
    "chainId": 1,
    "address": "0x55296f69f40ea6d20e478533c15a6b08b654e758",
    "name": "XYO Network",
    "symbol": "XYO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4519/thumb/XYO_Network-logo.png?1696505103"
  },
  {
    "chainId": 1,
    "address": "0xfa14fa6958401314851a17d6c5360ca29f74b57b",
    "name": "Saito",
    "symbol": "SAITO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14750/thumb/SAITO.png?1696514419"
  },
  {
    "chainId": 1,
    "address": "0x8c543aed163909142695f2d2acd0d55791a9edb9",
    "name": "Velas",
    "symbol": "VLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9651/thumb/logo_blue.png?1696509720"
  },
  {
    "chainId": 1,
    "address": "0xc9eb61ffb66d5815d643bbb8195e17c49687ae1e",
    "name": "Morpheus Labs",
    "symbol": "MIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3164/thumb/mitx.png?1696503886"
  },
  {
    "chainId": 1,
    "address": "0x7c5d5100b339fe7d995a893af6cb496b9474373c",
    "name": "Loon Network",
    "symbol": "LOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11673/thumb/ErUvUA2D4Q8hPWXxZBIuiI8MD0nrU2rDEY56d6EtVzJDTqlflAFXa2bgJ_kFwb8OtotfAvESvm-dik7d_SeJJh23f6RosUJ0AzU64QcKk5H3fu300KOJjFDFsntpRWARFKI5S4Cc8F3pcRPBkFIw6oPF58mKi83sa2DJPl-E5JMqmj6rJh5acb7Fvi80kfziWv3DHhhpg1YJYMP.jpg?1696511563"
  },
  {
    "chainId": 1,
    "address": "0x27702a26126e0b3702af63ee09ac4d1a084ef628",
    "name": "Aleph im",
    "symbol": "ALEPH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11676/thumb/Aleph-Logo-BW.png?1696511566"
  },
  {
    "chainId": 1,
    "address": "0xcc12abe4ff81c9378d670de1b57f8e0dd228d77a",
    "name": "Aave REN",
    "symbol": "AREN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14249/thumb/aREN.382be386.png?1696513963"
  },
  {
    "chainId": 1,
    "address": "0xa19f5264f7d7be11c451c093d8f92592820bea86",
    "name": "Neo Tokyo",
    "symbol": "BYTES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28858/thumb/bytes.png?1696527832"
  },
  {
    "chainId": 1,
    "address": "0x4dfae3690b93c47470b03036a17b23c1be05127c",
    "name": "The Original Pepe",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35702/thumb/logo.png?1709567988"
  },
  {
    "chainId": 1,
    "address": "0x0aee8703d34dd9ae107386d3eff22ae75dd616d1",
    "name": "Tranche Finance",
    "symbol": "SLICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14119/thumb/slice.png?1696513839"
  },
  {
    "chainId": 1,
    "address": "0x5aaefe84e0fb3dd1f0fcff6fa7468124986b91bd",
    "name": "Evedo",
    "symbol": "EVED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7721/thumb/Variations-09.png?1696507968"
  },
  {
    "chainId": 1,
    "address": "0x03042ae6fcfd53e3a0baa1fab5ce70e0cb74e6fb",
    "name": "Ten Best Coins",
    "symbol": "TBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27312/thumb/tbc.png?1696526362"
  },
  {
    "chainId": 1,
    "address": "0x701c244b988a513c945973defa05de933b23fe1d",
    "name": "OAX",
    "symbol": "OAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/853/thumb/OAXlogo.png?1696501987"
  },
  {
    "chainId": 1,
    "address": "0x1c9922314ed1415c95b9fd453c3818fd41867d0b",
    "name": "Tower",
    "symbol": "TOWER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14134/thumb/tower-circular-1000.png?1696513854"
  },
  {
    "chainId": 1,
    "address": "0x662b67d00a13faf93254714dd601f5ed49ef2f51",
    "name": "Orbit Chain",
    "symbol": "ORC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9782/thumb/-p1Br7oh_400x400.png?1696509841"
  },
  {
    "chainId": 1,
    "address": "0xd6bd97a26232ba02172ff86b055d5d7be789335b",
    "name": "Ormeus Cash",
    "symbol": "OMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11798/thumb/Vooo8SX.png?1696511674"
  },
  {
    "chainId": 1,
    "address": "0xd3e4ba569045546d09cf021ecc5dfe42b1d7f6e4",
    "name": "Morpheus Network",
    "symbol": "MNW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2379/thumb/MRPH_CoinGecko.png?1696503245"
  },
  {
    "chainId": 1,
    "address": "0xf57e7e7c23978c3caec3c3548e3d615c346e79ff",
    "name": "Immutable",
    "symbol": "IMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17233/thumb/immutableX-symbol-BLK-RGB.png?1696516787"
  },
  {
    "chainId": 1,
    "address": "0x213c53c96a01a89e6dcc5683cf16473203e17513",
    "name": "Defi Shopping Stake",
    "symbol": "DSS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12602/thumb/DSS.png?1696512411"
  },
  {
    "chainId": 1,
    "address": "0x0ad896863ce4cd84f10a9d30d4f509ceffd53c84",
    "name": "Chocolate Like Butterfly",
    "symbol": "CLB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28859/thumb/clb.png?1696527833"
  },
  {
    "chainId": 1,
    "address": "0x6226e00bcac68b0fe55583b90a1d727c14fab77f",
    "name": "MultiVAC",
    "symbol": "MTV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4937/thumb/MultiVAC.png?1696505477"
  },
  {
    "chainId": 1,
    "address": "0x400b1d8a7dd8c471026b2c8cbe1062b27d120538",
    "name": "Limestone Network",
    "symbol": "LIMEX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11907/thumb/nw1FE_f4_400x400.png?1696511771"
  },
  {
    "chainId": 1,
    "address": "0xd8b90d2e680ea535eacce1b025c998b347892f68",
    "name": "Chatter Shield",
    "symbol": "SHIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31140/thumb/CABDF194-1-FCF-4418-8-BA1-F537-D227288-F.png?1696529969"
  },
  {
    "chainId": 1,
    "address": "0xc05d14442a510de4d3d71a3d316585aa0ce32b50",
    "name": "LINA",
    "symbol": "LINA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5412/thumb/3083.png?1696505898"
  },
  {
    "chainId": 1,
    "address": "0x7e77dcb127f99ece88230a64db8d595f31f1b068",
    "name": "Escrowed Illuvium 2",
    "symbol": "SILV2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28860/thumb/silv2.png?1696527834"
  },
  {
    "chainId": 1,
    "address": "0x32c4adb9cf57f972bc375129de91c897b4f364f1",
    "name": "Flowchain",
    "symbol": "FLC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7446/thumb/logo_%2889%29.png?1597459811"
  },
  {
    "chainId": 1,
    "address": "0x7199b5a15c7fb79aa861780230adc65fff99ec73",
    "name": "0xAnon",
    "symbol": "0XANON",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32836/thumb/IMG_20231108_161217_519.jpg?1699587007"
  },
  {
    "chainId": 1,
    "address": "0x6f80310ca7f2c654691d1383149fa1a57d8ab1f8",
    "name": "Silo Finance",
    "symbol": "SILO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21454/thumb/y0iYKZOv_400x400.png?1696520816"
  },
  {
    "chainId": 1,
    "address": "0xe6fd75ff38adca4b97fbcd938c86b98772431867",
    "name": "Elastos",
    "symbol": "ELA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2780/thumb/Elastos.png?1696503549"
  },
  {
    "chainId": 1,
    "address": "0x722f97a435278b7383a1e3c47f41773bebf3232c",
    "name": "UCROWDME",
    "symbol": "UCM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12079/thumb/UKm2qXh.png?1696511926"
  },
  {
    "chainId": 1,
    "address": "0x990f341946a3fdb507ae7e52d17851b87168017c",
    "name": "Strong",
    "symbol": "STRONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12092/thumb/STRONG-Token-256x256.png?1696511938"
  },
  {
    "chainId": 1,
    "address": "0xe63684bcf2987892cefb4caa79bd21b34e98a291",
    "name": "Chicken",
    "symbol": "KFC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12569/thumb/download.png?1696512378"
  },
  {
    "chainId": 1,
    "address": "0x70d2b7c19352bb76e4409858ff5746e500f2b67c",
    "name": "Pawtocol",
    "symbol": "UPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12186/thumb/pawtocol.jpg?1696512023"
  },
  {
    "chainId": 1,
    "address": "0x5d65d971895edc438f465c17db6992698a52318d",
    "name": "Nebulas",
    "symbol": "NAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2431/thumb/193394331.png?1696503264"
  },
  {
    "chainId": 1,
    "address": "0xc813ea5e3b48bebeedb796ab42a30c5599b01740",
    "name": "Autonio",
    "symbol": "NIOX",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/1122/thumb/NewLogo.png?1696502216"
  },
  {
    "chainId": 1,
    "address": "0x9f7fc686cfd64aa5ae15b351d03071e91533094b",
    "name": "Trace Network Labs",
    "symbol": "TRACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23266/thumb/Token_Icon_02.png?1696522486"
  },
  {
    "chainId": 1,
    "address": "0xd8912c10681d8b21fd3742244f44658dba12264e",
    "name": "Pluton",
    "symbol": "PLU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1241/thumb/pluton.png?1696502316"
  },
  {
    "chainId": 1,
    "address": "0xcb8d1260f9c92a3a545d409466280ffdd7af7042",
    "name": "NFT Protocol",
    "symbol": "NFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12174/thumb/nftprotocol_32.png?1696512011"
  },
  {
    "chainId": 1,
    "address": "0x56015bbe3c01fe05bc30a8a9a9fd9a88917e7db3",
    "name": "Mooncat CAT",
    "symbol": "CAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12177/thumb/happy2-2.jpg?1696512014"
  },
  {
    "chainId": 1,
    "address": "0xf0ee6b27b759c9893ce4f094b49ad28fd15a23e4",
    "name": "Enigma",
    "symbol": "ENG",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1007/thumb/enigma-logo.png?1696502119"
  },
  {
    "chainId": 1,
    "address": "0xc975342a95ccb75378ddc646b8620fa3cd5bc051",
    "name": "MetaStreet V2 mwstETH WPUNKS 20",
    "symbol": "PUNKETH-20",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xecc0f1f860a82ab3b442382d93853c02d6384389",
    "name": "Axis DeFi",
    "symbol": "AXIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12199/thumb/YeLWZ3V.jpg?1696512035"
  },
  {
    "chainId": 1,
    "address": "0x8a6aca71a218301c7081d4e96d64292d3b275ce0",
    "name": "S Finance",
    "symbol": "SFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12605/thumb/Z7D8B4b.png?1696512414"
  },
  {
    "chainId": 1,
    "address": "0x2a79324c19ef2b89ea98b23bc669b7e7c9f8a517",
    "name": "WAX",
    "symbol": "WAXP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1372/thumb/WAX_Coin_Tickers_P_512px.png?1696502430"
  },
  {
    "chainId": 1,
    "address": "0x04e0af0af1b7f0023c6b12af5a94df59b0e8cf59",
    "name": "Sensitrust",
    "symbol": "SETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15035/thumb/SETS-token-logo-200.png?1696514695"
  },
  {
    "chainId": 1,
    "address": "0x2be5e8c109e2197d077d13a82daead6a9b3433c5",
    "name": "Tokamak Network",
    "symbol": "TON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12260/thumb/D919x5-s_400x400.png?1696512091"
  },
  {
    "chainId": 1,
    "address": "0x6a6c2ada3ce053561c2fbc3ee211f23d9b8c520a",
    "name": "TON Community",
    "symbol": "TON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12334/thumb/ton.jpg?1696512162"
  },
  {
    "chainId": 1,
    "address": "0xcbd55d4ffc43467142761a764763652b48b969ff",
    "name": "AstroTools",
    "symbol": "ASTRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12345/thumb/AT-flat-purple_logo.png?1696512172"
  },
  {
    "chainId": 1,
    "address": "0x3a8cccb969a61532d1e6005e2ce12c200caece87",
    "name": "TitanSwap",
    "symbol": "TITAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12606/thumb/nqGlQzdz_400x400.png?1696512414"
  },
  {
    "chainId": 1,
    "address": "0xf6c0aa7ebfe9992200c67e5388e4f42da49e1783",
    "name": "Psyche",
    "symbol": "USD1",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/13661/thumb/Psyche-logo-256.png?1696513411"
  },
  {
    "chainId": 1,
    "address": "0x464ebe77c293e473b48cfe96ddcf88fcf7bfdac0",
    "name": "KRYLL",
    "symbol": "KRL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2807/thumb/krl.png?1696503571"
  },
  {
    "chainId": 1,
    "address": "0xc28e27870558cf22add83540d2126da2e4b464c2",
    "name": "Sashimi",
    "symbol": "SASHIMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12427/thumb/SashimiSwap-200x200.png?1696512248"
  },
  {
    "chainId": 1,
    "address": "0x035bfe6057e15ea692c0dfdcab3bb41a64dd2ad4",
    "name": "Universal Liquidity Union",
    "symbol": "ULU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12376/thumb/ulu_finance_logo.ico?1696512200"
  },
  {
    "chainId": 1,
    "address": "0xa1afffe3f4d611d252010e3eaf6f4d77088b0cd7",
    "name": "reflect finance",
    "symbol": "RFI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/13189/thumb/reflect_finance_logo.png?1696512971"
  },
  {
    "chainId": 1,
    "address": "0x488e0369f9bc5c40c002ea7c1fe4fd01a198801c",
    "name": "Golff",
    "symbol": "GOF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12445/thumb/_x-AmLBv_400x400.jpg?1696512264"
  },
  {
    "chainId": 1,
    "address": "0x5befbb272290dd5b8521d4a938f6c4757742c430",
    "name": "Xfinance",
    "symbol": "XFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12449/thumb/go.png?1696512268"
  },
  {
    "chainId": 1,
    "address": "0x31429d1856ad1377a8a0079410b297e1a9e214c2",
    "name": "ANGLE",
    "symbol": "ANGLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19060/thumb/ANGLE_Token-light.png?1696518509"
  },
  {
    "chainId": 1,
    "address": "0xeb986da994e4a118d5956b02d8b7c3c7ce373674",
    "name": "Gather",
    "symbol": "GTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12458/thumb/Gather-Logo-Working-File.png?1696512278"
  },
  {
    "chainId": 1,
    "address": "0x0e8d6b471e332f140e7d9dbb99e5e3822f728da6",
    "name": "Abyss",
    "symbol": "ABYSS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2105/thumb/NrFmpxs.png?1696503072"
  },
  {
    "chainId": 1,
    "address": "0xc3071803b9d23460820b516673fd3cec0415d0ed",
    "name": "KuKu",
    "symbol": "KUKU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31235/thumb/dssad_%281%29.png?1696530061"
  },
  {
    "chainId": 1,
    "address": "0x0e5f00da8aaef196a719d045db89b5da8f371b32",
    "name": "Connectome",
    "symbol": "CNTM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8528/thumb/200_200_CNTM-LOGO-07.png?1696508705"
  },
  {
    "chainId": 1,
    "address": "0xedeec5691f23e4914cf0183a4196bbeb30d027a0",
    "name": "Wrapped Statera",
    "symbol": "WSTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13599/thumb/wsta_logo.png?1696513350"
  },
  {
    "chainId": 1,
    "address": "0xf2f9a7e93f845b3ce154efbeb64fb9346fcce509",
    "name": "UniPower",
    "symbol": "POWER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11618/thumb/unipower.png?1696511512"
  },
  {
    "chainId": 1,
    "address": "0xc71d8baaa436aa7e42da1f40bec48f11ab3c9e4a",
    "name": "iEthereum",
    "symbol": "IETH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/1065/thumb/ieth.png?1696502169"
  },
  {
    "chainId": 1,
    "address": "0x5d942f9872863645bcb181aba66c7d9646a91378",
    "name": "AltSignals",
    "symbol": "ASI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35709/thumb/ASIlogo.png?1709625768"
  },
  {
    "chainId": 1,
    "address": "0xd13c7342e1ef687c5ad21b27c2b65d772cab5c8c",
    "name": "Ultra",
    "symbol": "UOS",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/4480/thumb/Ultra.png?1696505069"
  },
  {
    "chainId": 1,
    "address": "0x9e78b8274e1d6a76a0dbbf90418894df27cbceb5",
    "name": "Covenants",
    "symbol": "UNIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12258/thumb/Unifi.png?1696512090"
  },
  {
    "chainId": 1,
    "address": "0x0c37bcf456bc661c14d596683325623076d7e283",
    "name": "Aeron",
    "symbol": "ARNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1124/thumb/ARNX-token-logo-256x256.png?1696502218"
  },
  {
    "chainId": 1,
    "address": "0x329c6e459ffa7475718838145e5e85802db2a303",
    "name": "MaidSafeCoin",
    "symbol": "EMAID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/80/thumb/maidsafecoin.png?1696501470"
  },
  {
    "chainId": 1,
    "address": "0x9be89d2a4cd102d8fecc6bf9da793be995c22541",
    "name": "Binance Wrapped BTC",
    "symbol": "BBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14867/thumb/binance-btc_32.png?1696514532"
  },
  {
    "chainId": 1,
    "address": "0xa4ef4b0b23c1fc81d3f9ecf93510e64f58a4a016",
    "name": "1MillionNFTs",
    "symbol": "1MIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14990/thumb/4kQ8hRnU_400x400.jpg?1696514654"
  },
  {
    "chainId": 1,
    "address": "0xcc7ab8d78dba187dc95bf3bb86e65e0c26d0041f",
    "name": "Spacelens",
    "symbol": "SPACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16979/thumb/SPACE___Spacelens.png?1696516545"
  },
  {
    "chainId": 1,
    "address": "0xb2f79d891f11bc8e9805db135defc04ead8d780e",
    "name": "All In One Wallet",
    "symbol": "AIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32643/thumb/AIO.jpeg?1698892673"
  },
  {
    "chainId": 1,
    "address": "0x8e57c27761ebbd381b0f9d09bb92ceb51a358abb",
    "name": "extraDNA",
    "symbol": "XDNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12667/thumb/Logo_coin_black.png?1696512473"
  },
  {
    "chainId": 1,
    "address": "0x6369c3dadfc00054a42ba8b2c09c48131dd4aa38",
    "name": "Morpher",
    "symbol": "MPH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12619/thumb/morpher_200_200.png?1696512426"
  },
  {
    "chainId": 1,
    "address": "0x998ffe1e43facffb941dc337dd0468d52ba5b48a",
    "name": "Rupiah Token",
    "symbol": "IDRT",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/9441/thumb/57421944_1371636006308255_3647136573922738176_n.jpg?1696509533"
  },
  {
    "chainId": 1,
    "address": "0x7122985656e38bdc0302db86685bb972b145bd3c",
    "name": "StakeStone ETH",
    "symbol": "STONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33103/thumb/200_200.png?1702602672"
  },
  {
    "chainId": 1,
    "address": "0x80fb784b7ed66730e8b1dbd9820afd29931aab03",
    "name": "Aave  OLD ",
    "symbol": "LEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1365/thumb/ethlend.png?1696502424"
  },
  {
    "chainId": 1,
    "address": "0xaac41ec512808d64625576eddd580e7ea40ef8b2",
    "name": "Gameswap",
    "symbol": "GSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13026/thumb/gameswap.jpg?1696512814"
  },
  {
    "chainId": 1,
    "address": "0x0391d2021f89dc339f60fff84546ea23e337750f",
    "name": "BarnBridge",
    "symbol": "BOND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12811/thumb/barnbridge.jpg?1696512604"
  },
  {
    "chainId": 1,
    "address": "0x177d39ac676ed1c67a2b268ad7f1e58826e5b0af",
    "name": "Blox",
    "symbol": "CDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1231/thumb/Blox_Staking_Logo_2.png?1696502308"
  },
  {
    "chainId": 1,
    "address": "0x418d75f65a02b3d53b2418fb8e1fe493759c7605",
    "name": "Binance Coin  Wormhole ",
    "symbol": "BNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22884/thumb/BNB_wh_small.png?1696522182"
  },
  {
    "chainId": 1,
    "address": "0x16587cf43f044aba0165ffa00acf412631194e4b",
    "name": "Simracer Coin",
    "symbol": "SRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13027/thumb/SimRacer-500px.png?1696512815"
  },
  {
    "chainId": 1,
    "address": "0x25c7b64a93eb1261e130ec21a3e9918caa38b611",
    "name": "Wrapped Virgin Gen 0 CryptoKittties",
    "symbol": "WVG0",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12875/thumb/wvg0.png?1696512663"
  },
  {
    "chainId": 1,
    "address": "0x0202be363b8a4820f3f4de7faf5224ff05943ab1",
    "name": "UniLend Finance",
    "symbol": "UFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12819/thumb/UniLend_Finance_logo_PNG.png?1696512611"
  },
  {
    "chainId": 1,
    "address": "0xe0b9a2c3e9f40cf74b2c7f591b2b0cca055c3112",
    "name": "Genesis Shards",
    "symbol": "GS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14784/thumb/gs.png?1696514453"
  },
  {
    "chainId": 1,
    "address": "0xbbff34e47e559ef680067a6b1c980639eeb64d24",
    "name": "Leverj Gluon",
    "symbol": "L2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12950/thumb/Gluon256x256.png?1696512739"
  },
  {
    "chainId": 1,
    "address": "0xdc580d4dc6b702b72a3511ac1802fa700ad182bb",
    "name": "Pumpkin",
    "symbol": "PUMPKIN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32842/thumb/IMG_20231108_220727_983.jpg?1699589128"
  },
  {
    "chainId": 1,
    "address": "0xc5bddf9843308380375a611c18b50fb9341f502a",
    "name": "veCRV DAO yVault",
    "symbol": "YVE-CRVDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13065/thumb/yearn_veCRV.png?1696512854"
  },
  {
    "chainId": 1,
    "address": "0xc3c221fe28c33814c28c822b631fd76047ef1a63",
    "name": "Millimeter",
    "symbol": "MM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12941/thumb/mm%EB%A1%9C%EA%B3%A0-2.png?1696512730"
  },
  {
    "chainId": 1,
    "address": "0x74e5e53056526b2609d82e85486005ef2a2db001",
    "name": "TruMATIC MATIC Stable Pool",
    "symbol": "TRUMATIC-MATIC",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xdacd69347de42babfaecd09dc88958378780fb62",
    "name": "Atari",
    "symbol": "ATRI",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/12992/thumb/AtariLogoPS_200x200_%281%29.png?1696512782"
  },
  {
    "chainId": 1,
    "address": "0xd849882983f1ba8a3c23b16b65bb0173a7f63b63",
    "name": "De Layer",
    "symbol": "DEAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35879/thumb/Frame_1_%2816%29.png?1709983222"
  },
  {
    "chainId": 1,
    "address": "0x725c263e32c72ddc3a19bea12c5a0479a81ee688",
    "name": "Bridge Mutual",
    "symbol": "BMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13808/thumb/bmi_logo.png?1696513554"
  },
  {
    "chainId": 1,
    "address": "0x11613b1f840bb5a40f8866d857e24da126b79d73",
    "name": "Cappasity",
    "symbol": "CAPP",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/1302/thumb/cappasity.png?1696502371"
  },
  {
    "chainId": 1,
    "address": "0x10633216e7e8281e33c86f02bf8e565a635d9770",
    "name": "Dvision Network",
    "symbol": "DVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13020/thumb/WGAhHOLv_400x400.png?1696512809"
  },
  {
    "chainId": 1,
    "address": "0x84c722e6f1363e8d5c6db3ea600bef9a006da824",
    "name": "Misbloc",
    "symbol": "MSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13021/thumb/xREV4lXV.png?1696512810"
  },
  {
    "chainId": 1,
    "address": "0x50b0696468f42cab1ddc76413a1312aff3cabdf6",
    "name": "ClosedAI",
    "symbol": "CLOSEDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36143/thumb/CLOSEDAI.jpg?1710689243"
  },
  {
    "chainId": 1,
    "address": "0x37cfc2e83665d49364670dfea6d2dd4cb1215f22",
    "name": "Astrid Restaked rETH",
    "symbol": "RRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32267/thumb/astridlogo_copy.png?1697166279"
  },
  {
    "chainId": 1,
    "address": "0xfe9a29ab92522d14fc65880d817214261d8479ae",
    "name": "Snowswap",
    "symbol": "SNOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12751/thumb/uQBJL3A.png?1696512549"
  },
  {
    "chainId": 1,
    "address": "0x6cb7d80bdefef3f820a3a77ee56f06afcb20ea7e",
    "name": "HUNDRED  ETH ",
    "symbol": "HUNDRED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31992/thumb/mQM7_dCk_400x400.jpg?1696530793"
  },
  {
    "chainId": 1,
    "address": "0x1fc5ef0337aea85c5f9198853a6e3a579a7a6987",
    "name": "ReapChain",
    "symbol": "REAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13109/thumb/REAP.jpg?1696512894"
  },
  {
    "chainId": 1,
    "address": "0x63b4f3e3fa4e438698ce330e365e831f7ccd1ef4",
    "name": "CyberFi",
    "symbol": "CFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13112/thumb/cyberfi_logo.jpeg?1696512897"
  },
  {
    "chainId": 1,
    "address": "0x26a604dffe3ddab3bee816097f81d3c4a2a4cf97",
    "name": "CorionX",
    "symbol": "CORX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13129/thumb/x_log.png?1696512916"
  },
  {
    "chainId": 1,
    "address": "0x67b66c99d3eb37fa76aa3ed1ff33e8e39f0b9c7a",
    "name": "Interest Bearing ETH",
    "symbol": "IBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13131/thumb/7675.png?1696512918"
  },
  {
    "chainId": 1,
    "address": "0x2c2f7e7c5604d162d75641256b80f1bf6f4dc796",
    "name": "Polkarare",
    "symbol": "PRARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15388/thumb/Image_from_iOS.png?1696515035"
  },
  {
    "chainId": 1,
    "address": "0x4946fcea7c692606e8908002e55a582af44ac121",
    "name": "FOAM",
    "symbol": "FOAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3345/thumb/K51lJQc.png?1696504049"
  },
  {
    "chainId": 1,
    "address": "0x910524678c0b1b23ffb9285a81f99c29c11cbaed",
    "name": "Azuki",
    "symbol": "AZUKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13091/thumb/bdUBSCo.png?1696512878"
  },
  {
    "chainId": 1,
    "address": "0x191557728e4d8caa4ac94f86af842148c0fa8f7e",
    "name": "Ormeus Ecosystem",
    "symbol": "ECO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/8923/thumb/logo_eco_low.png?1696509072"
  },
  {
    "chainId": 1,
    "address": "0x581911b360b6eb3a14ef295a83a91dc2bce2d6f7",
    "name": "MileVerse",
    "symbol": "MVC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13146/thumb/kXSdwuxD_400x400.jpg?1696512931"
  },
  {
    "chainId": 1,
    "address": "0x3175df0976dfa876431c2e9ee6bc45b65d3473cc",
    "name": "Curve fi FRAX USDC",
    "symbol": "CRVFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26829/thumb/W1sQNVWo_400x400.jpeg?1696525888"
  },
  {
    "chainId": 1,
    "address": "0xb7cb1c96db6b22b0d3d9536e0108d062bd488f74",
    "name": "Waltonchain",
    "symbol": "WTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1093/thumb/ggx6nnW.png?1696502192"
  },
  {
    "chainId": 1,
    "address": "0x2ab6bb8408ca3199b8fa6c92d5b455f820af03c4",
    "name": "TE FOOD",
    "symbol": "TONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2325/thumb/tec.png?1696503214"
  },
  {
    "chainId": 1,
    "address": "0xcbe7142f5c16755d8683ba329efa1abf7b54482d",
    "name": "MedicalVeda",
    "symbol": "MVEDA",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13069/thumb/medicalveda_new_logo_final_%281%29.png?1696512858"
  },
  {
    "chainId": 1,
    "address": "0xd794dd1cada4cf79c9eebaab8327a1b0507ef7d4",
    "name": "Hyve",
    "symbol": "HYVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13072/thumb/bAe1G-lD_400x400.png?1696512861"
  },
  {
    "chainId": 1,
    "address": "0xae0585a259a3bcab258d6ee02fb583f7b33c2a12",
    "name": "TemDAO",
    "symbol": "TEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28617/thumb/77163809-BDA6-4C9C-8387-4DBB33F06265.jpeg?1696527602"
  },
  {
    "chainId": 1,
    "address": "0xcb8fb2438a805664cd8c3e640b85ac473da5be87",
    "name": "ClinTex CTi",
    "symbol": "CTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13266/thumb/CTI.png?1696513039"
  },
  {
    "chainId": 1,
    "address": "0x485d17a6f1b8780392d53d64751824253011a260",
    "name": "chrono tech",
    "symbol": "TIME",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/604/thumb/time-32x32.png?1696501802"
  },
  {
    "chainId": 1,
    "address": "0xf0655dcee37e5c0b70fffd70d85f88f8edf0aff6",
    "name": "UniDex",
    "symbol": "UNIDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13178/thumb/unidx.png?1696512961"
  },
  {
    "chainId": 1,
    "address": "0xb04dfdb8271ed2d5e13858562c44a77d3ceb9e57",
    "name": "BuildUp",
    "symbol": "BUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13206/thumb/BUP_-_LOGO.png?1696512987"
  },
  {
    "chainId": 1,
    "address": "0x86ed939b500e121c0c5f493f399084db596dad20",
    "name": "SpaceChain  ERC 20 ",
    "symbol": "SPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6659/thumb/Spacechain.jpg?1696507004"
  },
  {
    "chainId": 1,
    "address": "0x3a810ff7211b40c4fa76205a14efe161615d0385",
    "name": "AI Network",
    "symbol": "AIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13211/thumb/AI_Network_Logo_200x200.png?1696512991"
  },
  {
    "chainId": 1,
    "address": "0xce593a29905951e8fc579bc092eca72577da575c",
    "name": "GROM",
    "symbol": "GR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13216/thumb/gr.png?1696512995"
  },
  {
    "chainId": 1,
    "address": "0xa1d65e8fb6e87b60feccbc582f7f97804b725521",
    "name": "DXdao",
    "symbol": "DXD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11148/thumb/dxdao.png?1696511083"
  },
  {
    "chainId": 1,
    "address": "0x69bbc3f8787d573f1bbdd0a5f40c7ba0aee9bcc9",
    "name": "Yup",
    "symbol": "YUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12322/thumb/photo_2021-10-26_00-47-35.jpg?1696512151"
  },
  {
    "chainId": 1,
    "address": "0xcaeaf8381d4b20b43afa42061d6f80319a8881f6",
    "name": "R34P",
    "symbol": "R34P",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13393/thumb/r34p_logo.png?1696513155"
  },
  {
    "chainId": 1,
    "address": "0x6876eba317272fe221c67405c5e8eb3b24535547",
    "name": "MicroTuber",
    "symbol": "MCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15489/thumb/mct.PNG?1696515134"
  },
  {
    "chainId": 1,
    "address": "0x1de852cd18323bc5ebda842b8eef46c6e551aa47",
    "name": "hiRENGA",
    "symbol": "HIRENGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28999/thumb/hirenga.png?1696527971"
  },
  {
    "chainId": 1,
    "address": "0x0789dbae94fb18e5789b8e4489bcb7a1adb58622",
    "name": "FISCO Coin",
    "symbol": "FSCC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/6507/thumb/AnyConv.com__fscc_icon_fin.png?1696506865"
  },
  {
    "chainId": 1,
    "address": "0x07ac55797d4f43f57ca92a49e65ca582cc287c27",
    "name": "Rebasing TBT",
    "symbol": "TBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29792/thumb/TBT.png?1696528722"
  },
  {
    "chainId": 1,
    "address": "0x6628606c321faf52b7230a57b26c01b19aa68e82",
    "name": "BitHash",
    "symbol": "BT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13224/thumb/bt.png?1696513003"
  },
  {
    "chainId": 1,
    "address": "0xb6ff96b8a8d214544ca0dbc9b33f7ad6503efd32",
    "name": "Sync Network",
    "symbol": "SYNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13231/thumb/sync_network.png?1696513009"
  },
  {
    "chainId": 1,
    "address": "0xd36306a5d6bfde4f57b5159c6518d93f171fe755",
    "name": "Astrid Restaked stETH",
    "symbol": "RSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32266/thumb/astridlogo_copy.png?1697166275"
  },
  {
    "chainId": 1,
    "address": "0x44f5909e97e1cbf5fbbdf0fc92fd83cde5d5c58a",
    "name": "Acria AI",
    "symbol": "ACRIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28598/thumb/image002.png?1696527585"
  },
  {
    "chainId": 1,
    "address": "0x34950ff2b487d9e5282c5ab342d08a2f712eb79f",
    "name": "Efforce",
    "symbol": "WOZX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13310/thumb/rZ6Oe3dm_400x400.jpg?1696513080"
  },
  {
    "chainId": 1,
    "address": "0xc12d099be31567add4e4e4d0d45691c3f58f5663",
    "name": "Auctus",
    "symbol": "AUC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2165/thumb/Auc_Discord_Avatar1.png?1696503126"
  },
  {
    "chainId": 1,
    "address": "0xbcd4b7de6fde81025f74426d43165a5b0d790fdd",
    "name": "SpiderDAO",
    "symbol": "SPDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13378/thumb/spiderdao_logo.png?1696513142"
  },
  {
    "chainId": 1,
    "address": "0xdf49c9f599a0a9049d97cff34d0c30e468987389",
    "name": "SaTT",
    "symbol": "SATT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4070/thumb/r3s9XKk.png?1696504702"
  },
  {
    "chainId": 1,
    "address": "0x755be920943ea95e39ee2dc437b268917b580d6e",
    "name": "VersoView",
    "symbol": "VVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13380/thumb/HkfxEtWh_400x400.jpg?1696513143"
  },
  {
    "chainId": 1,
    "address": "0xedf6568618a00c6f0908bf7758a16f76b6e04af9",
    "name": "Arianee",
    "symbol": "ARIA20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5054/thumb/Aria_Logo_256.png?1696505581"
  },
  {
    "chainId": 1,
    "address": "0x374cb8c27130e2c9e04f44303f3c8351b9de61c1",
    "name": "Bao Finance",
    "symbol": "BAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13547/thumb/BaoLogo.png?1696513307"
  },
  {
    "chainId": 1,
    "address": "0xc014186cf1ba36032aaec7f96088f09eb3934347",
    "name": "WeCoOwn",
    "symbol": "WCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14497/thumb/WCX-Token-Logo.png?1696514184"
  },
  {
    "chainId": 1,
    "address": "0xbd434a09191d401da3283a5545bb3515d033b8c4",
    "name": "GoldFinX",
    "symbol": "GIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13292/thumb/KAB_611E_400x400.jpg?1696513064"
  },
  {
    "chainId": 1,
    "address": "0x27c70cd1946795b66be9d954418546998b546634",
    "name": "Doge Killer",
    "symbol": "LEASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15802/thumb/Leash.png?1696515425"
  },
  {
    "chainId": 1,
    "address": "0x0488401c3f535193fa8df029d9ffe615a06e74e6",
    "name": "SparkPoint",
    "symbol": "SRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8371/thumb/SRK.png?1696508564"
  },
  {
    "chainId": 1,
    "address": "0xff44b937788215eca197baaf9af69dbdc214aa04",
    "name": "Rocki",
    "symbol": "ROCKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13465/thumb/rocki_logo.png?1696513227"
  },
  {
    "chainId": 1,
    "address": "0xe65cdb6479bac1e22340e4e755fae7e509ecd06c",
    "name": "cAAVE",
    "symbol": "CAAVE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17527/thumb/caave.PNG?1696517065"
  },
  {
    "chainId": 1,
    "address": "0x185ece9bc75164f9fc0fbe44738e8dd1863f8464",
    "name": "UNODEX",
    "symbol": "UNDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34115/thumb/UNODEX_LOGO_PNG_200X200.png?1704020683"
  },
  {
    "chainId": 1,
    "address": "0xf16e81dce15b08f326220742020379b855b87df9",
    "name": "Popsicle Finance",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14586/thumb/ice.png?1696514266"
  },
  {
    "chainId": 1,
    "address": "0xf0acf8949e705e0ebb6cb42c2164b0b986454223",
    "name": "Barter",
    "symbol": "BRTR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13401/thumb/BRTR.png?1696513163"
  },
  {
    "chainId": 1,
    "address": "0x96c645d3d3706f793ef52c19bbace441900ed47d",
    "name": "Mt Pelerin Shares",
    "symbol": "MPS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/11471/thumb/MPS.png?1696511378"
  },
  {
    "chainId": 1,
    "address": "0x7a2bc711e19ba6aff6ce8246c546e8c4b4944dfd",
    "name": "WAXE",
    "symbol": "WAXE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13508/thumb/waxe_logo.png?1696513270"
  },
  {
    "chainId": 1,
    "address": "0x853d955acef822db058eb8505911ed77f175b99e",
    "name": "Frax",
    "symbol": "FRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13422/thumb/FRAX_icon.png?1696513182"
  },
  {
    "chainId": 1,
    "address": "0x3432b6a60d23ca0dfca7761b7ab56459d9c964d0",
    "name": "Frax Share",
    "symbol": "FXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13423/thumb/Frax_Shares_icon.png?1696513183"
  },
  {
    "chainId": 1,
    "address": "0x6522f491df42651cf5e6636b7261adaa096d095f",
    "name": "Asap Sniper Bot",
    "symbol": "ASAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30707/thumb/LOGO_%281%29.jpeg?1696529578"
  },
  {
    "chainId": 1,
    "address": "0x5ea82c27efc7634f1c5ad20a3561c453433a2f3a",
    "name": "BTRIPS",
    "symbol": "BTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19273/thumb/btr.png?1696518716"
  },
  {
    "chainId": 1,
    "address": "0x5a338f22b798490ec3e70cd5e64d0cc0754241c0",
    "name": "Null Social Finance",
    "symbol": "NSF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32485/thumb/logo-1.png?1698291028"
  },
  {
    "chainId": 1,
    "address": "0x0c7d5ae016f806603cb1782bea29ac69471cab9c",
    "name": "Bifrost",
    "symbol": "BFC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4639/thumb/BFC_Symbol.png?1696505208"
  },
  {
    "chainId": 1,
    "address": "0x7f0c8b125040f707441cad9e5ed8a8408673b455",
    "name": "CSP DAO Network",
    "symbol": "NEBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13436/thumb/csp_dao.png?1696513200"
  },
  {
    "chainId": 1,
    "address": "0xcbfef8fdd706cde6f208460f2bf39aa9c785f05d",
    "name": "Kine Protocol",
    "symbol": "KINE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14339/thumb/kine.jpg?1696514026"
  },
  {
    "chainId": 1,
    "address": "0x73968b9a57c6e53d41345fd57a6e6ae27d6cdb2f",
    "name": "Stake DAO",
    "symbol": "SDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13724/thumb/stakedao_logo.jpg?1696513468"
  },
  {
    "chainId": 1,
    "address": "0x28cb7e841ee97947a86b06fa4090c8451f64c0be",
    "name": "YF Link",
    "symbol": "YFL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12081/thumb/YFLink.png?1696511928"
  },
  {
    "chainId": 1,
    "address": "0x65f9a292f1aeed5d755aa2fd2fb17ab2e9431447",
    "name": "SoMee Social",
    "symbol": "SOMEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16406/thumb/SoMeeBallLogo200x200.png?1696516003"
  },
  {
    "chainId": 1,
    "address": "0x94d863173ee77439e4292284ff13fad54b3ba182",
    "name": "Delphi",
    "symbol": "ADEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12300/thumb/adel_on_white_10x.png?1696512130"
  },
  {
    "chainId": 1,
    "address": "0x8798249c2e607446efb7ad49ec89dd1865ff4272",
    "name": "xSUSHI",
    "symbol": "XSUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13725/thumb/xsushi.png?1696513469"
  },
  {
    "chainId": 1,
    "address": "0x4b0181102a0112a2ef11abee5563bb4a3176c9d7",
    "name": "cSUSHI",
    "symbol": "CSUSHI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17529/thumb/csushi.PNG?1696517067"
  },
  {
    "chainId": 1,
    "address": "0xbbc2ae13b23d715c30720f079fcd9b4a74093505",
    "name": "Ethernity Chain",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14238/thumb/LOGO_HIGH_QUALITY.png?1696513952"
  },
  {
    "chainId": 1,
    "address": "0x6d0f5149c502faf215c89ab306ec3e50b15e2892",
    "name": "Portion",
    "symbol": "PRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13617/thumb/OKeO2FI.png?1696513366"
  },
  {
    "chainId": 1,
    "address": "0xd1ba9bac957322d6e8c07a160a3a8da11a0d2867",
    "name": "HUMAN Protocol",
    "symbol": "HMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16412/thumb/human_protocol.PNG?1696516009"
  },
  {
    "chainId": 1,
    "address": "0x80a2ae356fc9ef4305676f7a3e2ed04e12c33946",
    "name": "cYFI",
    "symbol": "CYFI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17530/thumb/cyfi.PNG?1696517068"
  },
  {
    "chainId": 1,
    "address": "0x7690202e2c2297bcd03664e31116d1dffe7e3b73",
    "name": "Cat in a Box Ether",
    "symbol": "BOXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29927/thumb/boxETH.png?1696528855"
  },
  {
    "chainId": 1,
    "address": "0x0590cc9232ebf68d81f6707a119898219342ecb9",
    "name": "BananaCat",
    "symbol": "BCAT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32557/thumb/imresizer-1698351889115.jpg?1698503559"
  },
  {
    "chainId": 1,
    "address": "0xeabb8996ea1662cad2f7fb715127852cd3262ae9",
    "name": "Connect Financial",
    "symbol": "CNFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13592/thumb/cf-logo-iconic-black.png?1696513344"
  },
  {
    "chainId": 1,
    "address": "0xc0f9bd5fa5698b6505f643900ffa515ea5df54a9",
    "name": "Donut",
    "symbol": "DONUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7538/thumb/Donut.png?1696507804"
  },
  {
    "chainId": 1,
    "address": "0x5c2975269e74cb3a8514e5b800a1e66c694d4df8",
    "name": "Caroline",
    "symbol": "HER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30146/thumb/caroline.jpeg?1696529067"
  },
  {
    "chainId": 1,
    "address": "0x8642a849d0dcb7a15a974794668adcfbe4794b56",
    "name": "Prosper",
    "symbol": "PROS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13668/thumb/heD6cg22l3sF5VgPh4G1xC6lnKEWXJif-jbaqUpv8CDP6jbWaqn9UjBdkXWNrw1CewaQOxb8zXRdNeNJWWiUDjfsEl_d7E3bPLg4cFoilQF5TGKHfWyJlnpm3UYc9ytvRvOjxOevMuiu8-lusnNoOcwgsJpMkYWHqe322GAxLt0_30kFMVAcjEDUrOlkK6hUYi0m9P433mvNlOm.jpg?1696513418"
  },
  {
    "chainId": 1,
    "address": "0x87de305311d5788e8da38d19bb427645b09cb4e5",
    "name": "Verox",
    "symbol": "VRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13669/thumb/IMG-20210115-000024.png?1696513419"
  },
  {
    "chainId": 1,
    "address": "0x49e833337ece7afe375e44f4e3e8481029218e5c",
    "name": "Value DeFi",
    "symbol": "VALUE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12525/thumb/value_logo_-_500x500.png?1696512338"
  },
  {
    "chainId": 1,
    "address": "0x2791bfd60d232150bff86b39b7146c0eaaa2ba81",
    "name": "BiFi",
    "symbol": "BIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13671/thumb/ysYIu7Q.png?1696513420"
  },
  {
    "chainId": 1,
    "address": "0xaa19961b6b858d9f18a115f25aa1d98abc1fdba8",
    "name": "LocalCoinSwap",
    "symbol": "LCS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3998/thumb/LocalCoinSwap.png?1696504639"
  },
  {
    "chainId": 1,
    "address": "0x7d4b8cce0591c9044a22ee543533b72e976e36c3",
    "name": "Change",
    "symbol": "CAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1021/thumb/200x200.png?1696502132"
  },
  {
    "chainId": 1,
    "address": "0x5caf454ba92e6f2c929df14667ee360ed9fd5b26",
    "name": "Dev Protocol",
    "symbol": "DEV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11560/thumb/Dev_Protocol__CoinGecko_Logo___Jan.18.2021_.png?1696511459"
  },
  {
    "chainId": 1,
    "address": "0x6e5970dbd6fc7eb1f29c6d2edf2bc4c36124c0c1",
    "name": "Polytrade",
    "symbol": "TRADE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16416/thumb/Logo_colored_200.png?1696516012"
  },
  {
    "chainId": 1,
    "address": "0x26c8afbbfe1ebaca03c2bb082e69d0476bffe099",
    "name": "Cellframe",
    "symbol": "CELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14465/thumb/cellframe-coingecko.png?1696514152"
  },
  {
    "chainId": 1,
    "address": "0x8666cb197af5103f7a3a0295b50efea47f3df78b",
    "name": "Ducks",
    "symbol": "DUCKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30147/thumb/ducks.png?1696529068"
  },
  {
    "chainId": 1,
    "address": "0xca1207647ff814039530d7d35df0e1dd2e91fa84",
    "name": "dHEDGE DAO",
    "symbol": "DHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12508/thumb/dht.png?1696512323"
  },
  {
    "chainId": 1,
    "address": "0x12e2b8033420270db2f3b328e32370cb5b2ca134",
    "name": "SafePal",
    "symbol": "SFP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13905/thumb/sfp.png?1696513647"
  },
  {
    "chainId": 1,
    "address": "0x178bf8fd04b47d2de3ef3f6b3d112106375ad584",
    "name": "Unagii Tether USD",
    "symbol": "UUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13778/thumb/uUSDT.png?1696513519"
  },
  {
    "chainId": 1,
    "address": "0x0d02755a5700414b26ff040e1de35d337df56218",
    "name": "BendDAO",
    "symbol": "BEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22829/thumb/benddao.PNG?1696522130"
  },
  {
    "chainId": 1,
    "address": "0xe9b076b476d8865cdf79d1cf7df420ee397a7f75",
    "name": "Unification",
    "symbol": "FUND",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/7845/thumb/DV80FOp.png?1696508079"
  },
  {
    "chainId": 1,
    "address": "0xaecc217a749c2405b5ebc9857a16d58bdc1c367f",
    "name": "Pawthereum",
    "symbol": "PAWTH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19275/thumb/pawth.png?1696518718"
  },
  {
    "chainId": 1,
    "address": "0xdb05ea0877a2622883941b939f0bb11d1ac7c400",
    "name": "Opacity",
    "symbol": "OPCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7237/thumb/Opacity.jpg?1696507532"
  },
  {
    "chainId": 1,
    "address": "0xbc5991ccd8caceba01edc44c2bb9832712c29cab",
    "name": "Unagii USD Coin",
    "symbol": "UUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13781/thumb/uUSDC.png?1696513522"
  },
  {
    "chainId": 1,
    "address": "0x4ad0b81f92b16624bbcf46fc0030cfbbf8d02376",
    "name": "Unagii Dai",
    "symbol": "UDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13782/thumb/uDAI.png?1696513523"
  },
  {
    "chainId": 1,
    "address": "0xf4172630a656a47ece8616e75791290446fa41a0",
    "name": "PEPPA",
    "symbol": "PEPPA",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/30492/thumb/200_x_200_JPG.jpg?1696529379"
  },
  {
    "chainId": 1,
    "address": "0xc691bc298a304d591ad9b352c7a8d216de9f2ced",
    "name": "Polaris Share",
    "symbol": "POLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13817/thumb/tHQrxdYQ_400x400.png?1696513563"
  },
  {
    "chainId": 1,
    "address": "0x9d7b68970d2be6dc93124477b4e2e1c9a6b180aa",
    "name": "Dede",
    "symbol": "DEDE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30976/thumb/dede_png_%281%29.png?1696529815"
  },
  {
    "chainId": 1,
    "address": "0xec5483804e637d45cde22fa0869656b64b5ab1ab",
    "name": "Acent",
    "symbol": "ACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15211/thumb/acent-logo.png?1696514867"
  },
  {
    "chainId": 1,
    "address": "0x081f67afa0ccf8c7b17540767bbe95df2ba8d97f",
    "name": "CoinEx",
    "symbol": "CET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4817/thumb/coinex-token.png?1696505367"
  },
  {
    "chainId": 1,
    "address": "0x8f693ca8d21b157107184d29d398a8d082b38b76",
    "name": "Streamr",
    "symbol": "DATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17869/thumb/DATA_new_symbol_3x.png?1696517392"
  },
  {
    "chainId": 1,
    "address": "0x4f5fa8f2d12e5eb780f6082dd656c565c48e0f24",
    "name": "Gourmet Galaxy",
    "symbol": "GUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13846/thumb/gum.png?1696513592"
  },
  {
    "chainId": 1,
    "address": "0x59e9261255644c411afdd00bd89162d09d862e38",
    "name": "ETHA Lend",
    "symbol": "ETHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14141/thumb/etha_logo200x200.png?1696513861"
  },
  {
    "chainId": 1,
    "address": "0x9d5963ba32e877871dff3e2e697283dc64066271",
    "name": "Edcoin",
    "symbol": "EDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13872/thumb/EDCOIN-1.png?1696513617"
  },
  {
    "chainId": 1,
    "address": "0x7420b4b9a0110cdc71fb720908340c03f9bc03ec",
    "name": "JasmyCoin",
    "symbol": "JASMY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13876/thumb/JASMY200x200.jpg?1696513620"
  },
  {
    "chainId": 1,
    "address": "0xf680429328caaacabee69b7a9fdb21a71419c063",
    "name": "Butterfly Protocol",
    "symbol": "BFLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13882/thumb/ButterflyProtocolNoText_sm.png?1696513626"
  },
  {
    "chainId": 1,
    "address": "0x6fc13eace26590b80cccab1ba5d51890577d83b2",
    "name": "Umbrella Network",
    "symbol": "UMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13913/thumb/Apple_Watch_45mm_-_3.png?1705486680"
  },
  {
    "chainId": 1,
    "address": "0x3fe7940616e5bc47b0775a0dccf6237893353bb4",
    "name": "IdleDAI  Best Yield ",
    "symbol": "IDLEDAIYIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11901/thumb/idledai_32.png?1696511767"
  },
  {
    "chainId": 1,
    "address": "0x8c81121b15197fa0eeaee1dc75533419dcfd3151",
    "name": "IdleWBTC  Best Yield ",
    "symbol": "IDLEWBTCYIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11936/thumb/idlewbtcv3maxyield_32.png?1696511797"
  },
  {
    "chainId": 1,
    "address": "0x93b1e78a3e652cd2e71c4a767595b77282344932",
    "name": "BITO Coin",
    "symbol": "BITO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7796/thumb/bitopro_28.png?1696508036"
  },
  {
    "chainId": 1,
    "address": "0xc5fb36dd2fb59d3b98deff88425a3f425ee469ed",
    "name": "Dejitaru Tsuka",
    "symbol": "TSUKA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26405/thumb/Dejitaru_Tsuka_Logo.jpg?1696525481"
  },
  {
    "chainId": 1,
    "address": "0x196c81385bc536467433014042788eb707703934",
    "name": "CryptoTask",
    "symbol": "CTASK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13965/thumb/cryptotask_logo.png?1696513700"
  },
  {
    "chainId": 1,
    "address": "0xd4342a57ecf2fe7ffa37c33cb8f63b1500e575e6",
    "name": "Apron",
    "symbol": "APN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14916/thumb/1_GOjoDhGzzpqnMPGpHGeWhg.png?1696514578"
  },
  {
    "chainId": 1,
    "address": "0xe516d78d784c77d479977be58905b3f2b1111126",
    "name": "Bitspawn",
    "symbol": "SPWN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16513/thumb/token_logo.png?1696516077"
  },
  {
    "chainId": 1,
    "address": "0xb49fa25978abf9a248b8212ab4b87277682301c0",
    "name": "RAI Finance",
    "symbol": "SOFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14686/thumb/sofi.png?1696514359"
  },
  {
    "chainId": 1,
    "address": "0xe75ad3aab14e4b0df8c5da4286608dabb21bd864",
    "name": "Double A Chain",
    "symbol": "AAC",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/2577/thumb/acute-angle-cloud.png?1696503381"
  },
  {
    "chainId": 1,
    "address": "0x3d3af44cf092a49280e316f09c8f20ecf97bc933",
    "name": "UCX",
    "symbol": "UCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10131/thumb/UCX_200200.png?1696510155"
  },
  {
    "chainId": 1,
    "address": "0x5d3a4f62124498092ce665f865e0b38ff6f5fbea",
    "name": "Ideaology",
    "symbol": "IDEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13938/thumb/idea_logo.png?1696513676"
  },
  {
    "chainId": 1,
    "address": "0x14cc8dfaf2258e1b8b2869300dba1b734dc0fe43",
    "name": "K Tune",
    "symbol": "KTT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13939/thumb/KakaoTalk_20220928_162525453.png?1696513677"
  },
  {
    "chainId": 1,
    "address": "0xfdb15e5e6799be72798b1ccfaecbf186bf73a0c4",
    "name": "NitroEX",
    "symbol": "NTX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13976/thumb/ntx-icon.png?1696513709"
  },
  {
    "chainId": 1,
    "address": "0xea1ea0972fa092dd463f2968f9bb51cc4c981d71",
    "name": "Modefi",
    "symbol": "MOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13980/thumb/modefi_logo.png?1696513713"
  },
  {
    "chainId": 1,
    "address": "0xd13cfd3133239a3c73a9e535a5c4dadee36b395c",
    "name": "Vaiot",
    "symbol": "VAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13981/thumb/VAIOT_logo.png?1696513714"
  },
  {
    "chainId": 1,
    "address": "0xf5581dfefd8fb0e4aec526be659cfab1f8c781da",
    "name": "HOPR",
    "symbol": "HOPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14061/thumb/Shared_HOPR_logo_512px.png?1696513786"
  },
  {
    "chainId": 1,
    "address": "0xd478161c952357f05f0292b56012cd8457f1cfbf",
    "name": "Polkamarkets",
    "symbol": "POLK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14084/thumb/polkamarkets.jpg?1696513807"
  },
  {
    "chainId": 1,
    "address": "0xaaca86b876ca011844b5798eca7a67591a9743c8",
    "name": "0x nodes",
    "symbol": "BIOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15600/thumb/BIOS_01.png?1696515235"
  },
  {
    "chainId": 1,
    "address": "0xad22f63404f7305e4713ccbd4f296f34770513f4",
    "name": "Atomic Wallet Coin",
    "symbol": "AWC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7430/thumb/Atomic_Wallet_Aeron_%281%29.png?1696507703"
  },
  {
    "chainId": 1,
    "address": "0x1d6405138a335ce5fd7364086334efb3e4f28b59",
    "name": "ClearCryptos",
    "symbol": "CCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27900/thumb/3e88b61451995da1a5aa572cd4f48e78023e010c0bf984cd5456b2a7bd85f7a5.png?1696526921"
  },
  {
    "chainId": 1,
    "address": "0x2fda8c6783aa36bed645bad28a4cdc8769dcd252",
    "name": "Deuterium",
    "symbol": "D2O",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28732/thumb/LinkedIn_d2o_Icon_300x300px.png?1696527712"
  },
  {
    "chainId": 1,
    "address": "0x63f88a2298a5c4aee3c216aa6d926b184a4b2437",
    "name": "GameCredits",
    "symbol": "GAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/193/thumb/XlQmXoU.png?1696501564"
  },
  {
    "chainId": 1,
    "address": "0x89bd2e7e388fab44ae88bef4e1ad12b4f1e0911c",
    "name": "Peanut",
    "symbol": "NUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13958/thumb/2sAMZXpO_400x400.jpg?1696513694"
  },
  {
    "chainId": 1,
    "address": "0xb9d99c33ea2d86ec5ec6b8a4dd816ebba64404af",
    "name": "K21",
    "symbol": "K21",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14466/thumb/k21-token.png?1696514153"
  },
  {
    "chainId": 1,
    "address": "0x852e5427c86a3b46dd25e5fe027bb15f53c4bcb8",
    "name": "NiiFi",
    "symbol": "NIIFI",
    "decimals": 15,
    "logoURI": "https://assets.coingecko.com/coins/images/16033/thumb/niifi.PNG?1696515643"
  },
  {
    "chainId": 1,
    "address": "0x9355372396e3f6daf13359b7b607a3374cc638e0",
    "name": "WHALE",
    "symbol": "WHALE",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/11797/thumb/WHALE.png?1696511673"
  },
  {
    "chainId": 1,
    "address": "0x8f3470a7388c05ee4e7af3d01d8c722b0ff52374",
    "name": "Veritaseum",
    "symbol": "VERI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/695/thumb/veritaseum.png?1696501884"
  },
  {
    "chainId": 1,
    "address": "0x469eda64aed3a3ad6f868c44564291aa415cb1d9",
    "name": "Datamine FLUX",
    "symbol": "FLUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11756/thumb/fluxres.png?1696511637"
  },
  {
    "chainId": 1,
    "address": "0x6399c842dd2be3de30bf99bc7d1bbf6fa3650e70",
    "name": "Premia",
    "symbol": "PREMIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13962/thumb/apple-touch-icon.png?1696513698"
  },
  {
    "chainId": 1,
    "address": "0xcadc0acd4b445166f12d2c07eac6e2544fbe2eef",
    "name": "CAD Coin",
    "symbol": "CADC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14149/thumb/cadc_2.png?1696513868"
  },
  {
    "chainId": 1,
    "address": "0x850aab69f0e0171a9a49db8be3e71351c8247df4",
    "name": "Konomi Network",
    "symbol": "KONO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14406/thumb/konomi.jpg?1696514098"
  },
  {
    "chainId": 1,
    "address": "0x252b9f56359901a0bde52d0675b1f1130d86f471",
    "name": "Pando",
    "symbol": "PANDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14150/thumb/pando_logo.png?1696513869"
  },
  {
    "chainId": 1,
    "address": "0x53c8395465a84955c95159814461466053dedede",
    "name": "DeGate",
    "symbol": "DG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14415/thumb/DG_token_brand_web_rgb_blue.png?1696514106"
  },
  {
    "chainId": 1,
    "address": "0xfc4b8ed459e00e5400be803a9bb3954234fd50e3",
    "name": "Aave WBTC v1",
    "symbol": "AWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11734/thumb/aWBTC.png?1696511615"
  },
  {
    "chainId": 1,
    "address": "0x044d078f1c86508e13328842cc75ac021b272958",
    "name": "Peercoin",
    "symbol": "PPC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/4/thumb/peercoin-icon-green-transparent_6x.png?1696501402"
  },
  {
    "chainId": 1,
    "address": "0x037a54aab062628c9bbae1fdb1583c195585fe41",
    "name": "LCX",
    "symbol": "LCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9985/thumb/zRPSu_0o_400x400.jpg?1696510023"
  },
  {
    "chainId": 1,
    "address": "0x6ee0f7bb50a54ab5253da0667b0dc2ee526c30a8",
    "name": "Aave BUSD v1",
    "symbol": "ABUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11736/thumb/aBUSD.png?1696511617"
  },
  {
    "chainId": 1,
    "address": "0xfc98e825a2264d890f9a1e68ed50e1526abccacd",
    "name": "Moss Carbon Credit",
    "symbol": "MCO2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14414/thumb/ENtxnThA_400x400.jpg?1696514105"
  },
  {
    "chainId": 1,
    "address": "0x3ed3b47dd13ec9a98b44e6204a523e766b225811",
    "name": "Aave USDT",
    "symbol": "AUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14243/thumb/aUSDT.78f5faae.png?1696513958"
  },
  {
    "chainId": 1,
    "address": "0x8c15ef5b4b21951d50e53e4fbda8298ffad25057",
    "name": "Function X",
    "symbol": "FX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8186/thumb/47271330_590071468072434_707260356350705664_n.jpg?1696508396"
  },
  {
    "chainId": 1,
    "address": "0x05ec93c0365baaeabf7aeffb0972ea7ecdd39cf1",
    "name": "Aave BAT",
    "symbol": "ABAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14246/thumb/aBAT.1fe436f7_-_Copy.png?1696513961"
  },
  {
    "chainId": 1,
    "address": "0x9ff58f4ffb29fa2266ab25e75e2a8b3503311656",
    "name": "Aave WBTC",
    "symbol": "AWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14244/thumb/aWBTC.41453c2a.png?1696513959"
  },
  {
    "chainId": 1,
    "address": "0xa361718326c15715591c299427c62086f69923d9",
    "name": "Aave BUSD",
    "symbol": "ABUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14245/thumb/aBUSD.346a65d7_-_Copy.png?1696513960"
  },
  {
    "chainId": 1,
    "address": "0x028171bca77440897b824ca71d1c56cac55b68a3",
    "name": "Aave DAI",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14242/thumb/aDAI.84b6c41f.png?1696513957"
  },
  {
    "chainId": 1,
    "address": "0xd69f306549e9d96f183b1aeca30b8f4353c2ecc3",
    "name": "MCH Coin",
    "symbol": "MCHC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15399/thumb/MCHC.jpg?1696515045"
  },
  {
    "chainId": 1,
    "address": "0x3e2c956b4ab4807b2f942235c9074d9bd069e9f0",
    "name": "IPVERSE",
    "symbol": "IPV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25778/thumb/hhQtNx1X_400x400.jpeg?1696524865"
  },
  {
    "chainId": 1,
    "address": "0xfbbe9b1142c699512545f47937ee6fae0e4b0aa9",
    "name": "EDDASwap",
    "symbol": "EDDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14368/thumb/edda.png?1696514061"
  },
  {
    "chainId": 1,
    "address": "0x19ac2659599fd01c853de846919544276ad26f50",
    "name": "Covenant",
    "symbol": "COVN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22280/thumb/16837.png?1696521627"
  },
  {
    "chainId": 1,
    "address": "0x79126d32a86e6663f3aaac4527732d0701c1ae6c",
    "name": "Dark Matter",
    "symbol": "DMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14223/thumb/dmt.jpg?1696513938"
  },
  {
    "chainId": 1,
    "address": "0x86efc496dca70bcfd92d19194290e8457a375773",
    "name": "Silent Notary",
    "symbol": "UBSN",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/1599/thumb/silent-notary.png?1696502631"
  },
  {
    "chainId": 1,
    "address": "0x101cc05f4a51c0319f570d5e146a8c625198e636",
    "name": "Aave TUSD",
    "symbol": "ATUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14317/thumb/aTUSD.08592b49.png?1696514005"
  },
  {
    "chainId": 1,
    "address": "0x5c8c8d560048f34e5f7f8ad71f2f81a89dbd273e",
    "name": "CryptoArt Ai",
    "symbol": "CART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17583/thumb/logo-64.png?1696517117"
  },
  {
    "chainId": 1,
    "address": "0x8185bc4757572da2a610f887561c32298f1a5748",
    "name": "Aluna",
    "symbol": "ALN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14379/thumb/uaLoLU8c_400x400_%281%29.png?1696514071"
  },
  {
    "chainId": 1,
    "address": "0x442bc47357919446eabc18c7211e57a13d983469",
    "name": "BeeChat",
    "symbol": "CHAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14386/thumb/ezgif-6-b78b8423d870.png?1696514077"
  },
  {
    "chainId": 1,
    "address": "0xa06bc25b5805d5f8d82847d191cb4af5a3e873e0",
    "name": "Aave LINK",
    "symbol": "ALINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14315/thumb/aLINK.412c6589.png?1696514004"
  },
  {
    "chainId": 1,
    "address": "0x1796ae0b0fa4862485106a0de9b654efe301d0b2",
    "name": "Polychain Monsters",
    "symbol": "PMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14604/thumb/polkamon.png?1696514282"
  },
  {
    "chainId": 1,
    "address": "0x5218e472cfcfe0b64a064f055b43b4cdc9efd3a6",
    "name": "Residual Token",
    "symbol": "ERSDL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12975/thumb/ERSDL.jpg?1706998787"
  },
  {
    "chainId": 1,
    "address": "0x1fe24f25b1cf609b9c4e7e12d802e3640dfa5e43",
    "name": "Chain Guardians",
    "symbol": "CGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14326/thumb/cgg_logo.png?1696514015"
  },
  {
    "chainId": 1,
    "address": "0x657b83a0336561c8f64389a6f5ade675c04b0c3b",
    "name": "Playcent",
    "symbol": "PCNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14335/thumb/1_B5bFcgBld5poUj_c-_K1Jw.png?1696514023"
  },
  {
    "chainId": 1,
    "address": "0xb5fe099475d3030dde498c3bb6f3854f762a48ad",
    "name": "Fnk com",
    "symbol": "FNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13483/thumb/fnk.png?1696513243"
  },
  {
    "chainId": 1,
    "address": "0xb6c3dc857845a713d3531cea5ac546f6767992f4",
    "name": "Advertise Coin",
    "symbol": "ADCO",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14344/thumb/advertisecoin200x200.png?1696514031"
  },
  {
    "chainId": 1,
    "address": "0x269877f972622d3c293fca595c65cf34b7f527ce",
    "name": "Friends with Benefits Network",
    "symbol": "FWB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33783/thumb/Group.png?1702981073"
  },
  {
    "chainId": 1,
    "address": "0x761d38e5ddf6ccf6cf7c55759d5210750b5d60f3",
    "name": "Dogelon Mars",
    "symbol": "ELON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14962/thumb/6GxcPRo3_400x400.jpg?1696514622"
  },
  {
    "chainId": 1,
    "address": "0x19062190b1925b5b6689d7073fdfc8c2976ef8cb",
    "name": "Swarm",
    "symbol": "BZZ",
    "decimals": 16,
    "logoURI": "https://assets.coingecko.com/coins/images/16509/thumb/Circle_Orange_onWhite.png?1696516071"
  },
  {
    "chainId": 1,
    "address": "0x92868a5255c628da08f550a858a802f5351c5223",
    "name": "Cross Chain Bridge",
    "symbol": "BRIDGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20223/thumb/0x92868A5255C628dA08F550a858A802f5351C5223.png?1696519632"
  },
  {
    "chainId": 1,
    "address": "0xcd2828fc4d8e8a0ede91bb38cf64b1a81de65bf6",
    "name": "Oddz",
    "symbol": "ODDZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14421/thumb/NewLogo.png?1696514112"
  },
  {
    "chainId": 1,
    "address": "0x0fd3822072ad001aac1c90a09d9506f097f24458",
    "name": "NOA PLAY",
    "symbol": "NOA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15723/thumb/XHFjmBTx_400x400.png?1696515350"
  },
  {
    "chainId": 1,
    "address": "0x45804880de22913dafe09f4980848ece6ecbaf78",
    "name": "PAX Gold",
    "symbol": "PAXG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9519/thumb/paxgold.png?1696509604"
  },
  {
    "chainId": 1,
    "address": "0x7cda79830faf07ed696fe220566116951ced36a7",
    "name": "Maya Preferred",
    "symbol": "MAYP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8081/thumb/2u0OXhT9_400x400.png?1696508303"
  },
  {
    "chainId": 1,
    "address": "0x9695e0114e12c0d3a3636fab5a18e6b737529023",
    "name": "Dfyn Network",
    "symbol": "DFYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15368/thumb/SgqhfWz4_400x400_%281%29.jpg?1696515016"
  },
  {
    "chainId": 1,
    "address": "0x5833dbb0749887174b254ba4a5df747ff523a905",
    "name": "XRun",
    "symbol": "XRUN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25272/thumb/19787.png?1696524411"
  },
  {
    "chainId": 1,
    "address": "0xb17c88bda07d28b3838e0c1de6a30eafbcf52d85",
    "name": "Shyft Network",
    "symbol": "SHFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14534/thumb/TOKEN-Gradient.png?1696514218"
  },
  {
    "chainId": 1,
    "address": "0x20bc832ca081b91433ff6c17f85701b6e92486c5",
    "name": "rETH2",
    "symbol": "RETH2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16570/thumb/red256.png?1696516131"
  },
  {
    "chainId": 1,
    "address": "0x74232704659ef37c08995e386a2e26cc27a8d7b1",
    "name": "Strike",
    "symbol": "STRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14607/thumb/Jw-36llq_400x400.jpg?1696514286"
  },
  {
    "chainId": 1,
    "address": "0xe796d6ca1ceb1b022ece5296226bf784110031cd",
    "name": "Blind Boxes",
    "symbol": "BLES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14537/thumb/BLES-Logo-BW.png?1696514221"
  },
  {
    "chainId": 1,
    "address": "0x9ea3b5b4ec044b70375236a281986106457b20ef",
    "name": "Delta Financial",
    "symbol": "DELTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14564/thumb/Delta_logo.png?1696514246"
  },
  {
    "chainId": 1,
    "address": "0xfe2e637202056d30016725477c5da089ab0a043a",
    "name": "sETH2",
    "symbol": "SETH2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16569/thumb/emerald256.png?1696516130"
  },
  {
    "chainId": 1,
    "address": "0x36919a60a2b67b6d2329863093d180d23d5a0308",
    "name": "Kusunoki Samurai",
    "symbol": "KUSUNOKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24688/thumb/68747470733a2f2f7777772e6b7573756e6f6b6973616d757261692e636f6d2f696d616765732f6b7573756e6f6b692d30312e706e67.png?1696523855"
  },
  {
    "chainId": 1,
    "address": "0x741b0428efdf4372a8df6fb54b018db5e5ab7710",
    "name": "ARTX",
    "symbol": "ARTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14652/thumb/ARTX.png?1708198632"
  },
  {
    "chainId": 1,
    "address": "0x88acdd2a6425c3faae4bc9650fd7e27e0bebb7ab",
    "name": "Alchemist",
    "symbol": "MIST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14655/thumb/79158662.png?1696514331"
  },
  {
    "chainId": 1,
    "address": "0x2614f29c39de46468a921fd0b41fdd99a01f2edf",
    "name": "HELIOS",
    "symbol": "HLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35650/thumb/HeliosPNG.png?1709375257"
  },
  {
    "chainId": 1,
    "address": "0x0100546f2cd4c9d97f798ffc9755e47865ff7ee6",
    "name": "Alchemix ETH",
    "symbol": "ALETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16271/thumb/aleth.png?1696515869"
  },
  {
    "chainId": 1,
    "address": "0x04969cd041c0cafb6ac462bd65b536a5bdb3a670",
    "name": "Wrapped ECOMI",
    "symbol": "WOMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14675/thumb/ecomi.jpg?1696514349"
  },
  {
    "chainId": 1,
    "address": "0x30672ae2680c319ec1028b69670a4a786baa0f35",
    "name": "NodeSynapse",
    "symbol": "NS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35679/thumb/NodeSynapse_%281%29.png?1709525297"
  },
  {
    "chainId": 1,
    "address": "0x777fd20c983d6658c1d50b3958b3a1733d1cd1e1",
    "name": "PUBLISH",
    "symbol": "NEWS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/9572/thumb/PUBLISH_Logo_280X280.png?1696509652"
  },
  {
    "chainId": 1,
    "address": "0x7e9d8f07a64e363e97a648904a89fb4cd5fb94cd",
    "name": "Forefront",
    "symbol": "FF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14659/thumb/N2kir6jx_400x400.jpg?1696514335"
  },
  {
    "chainId": 1,
    "address": "0x6dea81c8171d0ba574754ef6f8b412f2ed88c54d",
    "name": "Liquity",
    "symbol": "LQTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14665/thumb/200-lqty-icon.png?1696514340"
  },
  {
    "chainId": 1,
    "address": "0x5f98805a4e8be255a32880fdec7f6728c6568ba0",
    "name": "Liquity USD",
    "symbol": "LUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14666/thumb/Group_3.png?1696514341"
  },
  {
    "chainId": 1,
    "address": "0xf45f6c8bb3d77ea762175b8f7ca4d251941649fa",
    "name": "Lemond",
    "symbol": "LEMD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14964/thumb/D-smP_i-_400x400.png?1696514624"
  },
  {
    "chainId": 1,
    "address": "0x9a0aba393aac4dfbff4333b06c407458002c6183",
    "name": "ACoconut",
    "symbol": "AC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12779/thumb/ac_logo.png?1696512574"
  },
  {
    "chainId": 1,
    "address": "0x1735db6ab5baa19ea55d0adceed7bcdc008b3136",
    "name": "UREEQA",
    "symbol": "URQA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14605/thumb/R_O2enOX_400x400.png?1696514283"
  },
  {
    "chainId": 1,
    "address": "0x0db8d8b76bc361bacbb72e2c491e06085a97ab31",
    "name": "IQeon",
    "symbol": "IQN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1736/thumb/IQN_logo_64x64.png?1696502750"
  },
  {
    "chainId": 1,
    "address": "0x99c6e435ec259a7e8d65e1955c9423db624ba54c",
    "name": "Finminity",
    "symbol": "FMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14696/thumb/finminity.png?1696514368"
  },
  {
    "chainId": 1,
    "address": "0x3c72fca8523686fd9e5740b0826fa4bb376e0241",
    "name": "300FIT",
    "symbol": "FIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9690/thumb/6mkyI4Ss_400x400.png?1696509759"
  },
  {
    "chainId": 1,
    "address": "0xff75ced57419bcaebe5f05254983b013b0646ef5",
    "name": "Cook",
    "symbol": "COOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14603/thumb/logo-200x200.jpg?1696514281"
  },
  {
    "chainId": 1,
    "address": "0x794fcee2c16e9495c3f590958b1209400751ba80",
    "name": "Syntax AI",
    "symbol": "SYNTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35680/thumb/syntax-3.jpeg?1709525443"
  },
  {
    "chainId": 1,
    "address": "0xaf4dce16da2877f8c9e00544c93b62ac40631f16",
    "name": "Monetha",
    "symbol": "MTH",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/902/thumb/LOGO-MTH-200-200.png?1696502024"
  },
  {
    "chainId": 1,
    "address": "0x056c1d42fb1326f57da7f19ebb7dda4673f1ff55",
    "name": "Gains",
    "symbol": "GAINS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14681/thumb/200x200.png?1696514354"
  },
  {
    "chainId": 1,
    "address": "0xec363faa5c4dd0e51f3d9b5d0101263760e7cdeb",
    "name": "Instadapp WBTC",
    "symbol": "IWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25784/thumb/iWBTC-100x100.png?1696524871"
  },
  {
    "chainId": 1,
    "address": "0x083d41d6dd21ee938f0c055ca4fb12268df0efac",
    "name": "GogolCoin",
    "symbol": "GOL",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/14824/thumb/GOL.png?1696514492"
  },
  {
    "chainId": 1,
    "address": "0xbd2949f67dcdc549c6ebe98696449fa79d988a9f",
    "name": "Meter Governance mapped by Meter io",
    "symbol": "EMTRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12175/thumb/Dark-blue-icon-transparent-vector-white-icon200x200.png?1696512012"
  },
  {
    "chainId": 1,
    "address": "0x4fee21439f2b95b72da2f9f901b3956f27fe91d5",
    "name": "FrogSwap",
    "symbol": "FROG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21449/thumb/logo.54d7099f.png?1696520811"
  },
  {
    "chainId": 1,
    "address": "0xbf2179859fc6d5bee9bf9158632dc51678a4100e",
    "name": "aelf",
    "symbol": "ELF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1371/thumb/aelf-logo.png?1696502429"
  },
  {
    "chainId": 1,
    "address": "0xee503d43ad447cdfc719f688207bfcb2fbb9471c",
    "name": "Daomatian",
    "symbol": "DAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30784/thumb/3Vwa_H8f_400x400.jpg?1696529651"
  },
  {
    "chainId": 1,
    "address": "0x2008e3057bd734e10ad13c9eae45ff132abc1722",
    "name": "Zebi",
    "symbol": "ZCO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2756/thumb/zebi.png?1696503529"
  },
  {
    "chainId": 1,
    "address": "0x6ef6610d24593805144d73b13d4405e00a4e4ac7",
    "name": "Die Protocol",
    "symbol": "DIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27903/thumb/newlogo.jpg?1696526924"
  },
  {
    "chainId": 1,
    "address": "0x430ef9263e76dae63c84292c3409d61c598e9682",
    "name": "Vulcan Forged",
    "symbol": "PYR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14770/thumb/1617088937196.png?1696514439"
  },
  {
    "chainId": 1,
    "address": "0x2f109021afe75b949429fe30523ee7c0d5b27207",
    "name": "OccamFi",
    "symbol": "OCC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14801/thumb/occfi.PNG?1696514470"
  },
  {
    "chainId": 1,
    "address": "0x728f30fa2f100742c7949d1961804fa8e0b1387d",
    "name": "GamerCoin",
    "symbol": "GHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14714/thumb/ghx_icon.png?1696514385"
  },
  {
    "chainId": 1,
    "address": "0x87931e7ad81914e7898d07c68f145fc0a553d8fb",
    "name": "WIZARD Vault  NFTX ",
    "symbol": "WIZARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17068/thumb/Wizards.png?1696516630"
  },
  {
    "chainId": 1,
    "address": "0x21381e026ad6d8266244f2a583b35f9e4413fa2a",
    "name": "Formation FI",
    "symbol": "FORM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16145/thumb/FORM.png?1696515750"
  },
  {
    "chainId": 1,
    "address": "0xb39185e33e8c28e0bb3dbbce24da5dea6379ae91",
    "name": "PHUNK Vault  NFTX ",
    "symbol": "PHUNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17074/thumb/Phunks.png?1696516636"
  },
  {
    "chainId": 1,
    "address": "0xed0439eacf4c4965ae4613d77a5c2efe10e5f183",
    "name": "Niftyx Protocol",
    "symbol": "SHROOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12341/thumb/niftyx_logo.jpg?1696512168"
  },
  {
    "chainId": 1,
    "address": "0xa364a9f85f61dc5e3118be282a23edc5e09f62ac",
    "name": "Golden Zen Token",
    "symbol": "GZT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33467/thumb/Zen.jpeg?1701942862"
  },
  {
    "chainId": 1,
    "address": "0x7ca4408137eb639570f8e647d9bd7b7e8717514a",
    "name": "Alpaca City",
    "symbol": "ALPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13070/thumb/alpaca_logo.png?1696512859"
  },
  {
    "chainId": 1,
    "address": "0x397deb686c72384fad502a81f4d7fdb89e1f1280",
    "name": "XELS",
    "symbol": "XELS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14729/thumb/Xels_logo_square_%281%29.png?1696514398"
  },
  {
    "chainId": 1,
    "address": "0xa4e8c3ec456107ea67d3075bf9e3df3a75823db0",
    "name": "Loom Network  OLD ",
    "symbol": "LOOMOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3387/thumb/1_K76UVoLq-FOL7l-_Fag-Qw_2x.png?1696504085"
  },
  {
    "chainId": 1,
    "address": "0x42476f744292107e34519f9c357927074ea3f75d",
    "name": "Loom Network  NEW ",
    "symbol": "LOOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14735/thumb/LOOM.png?1696514405"
  },
  {
    "chainId": 1,
    "address": "0xcf9fbffec9e0e5bbc62e79bf1965f5db76955661",
    "name": "MeshBox",
    "symbol": "MESH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6200/thumb/DPuEPJQ6_400x400.jpg?1696506586"
  },
  {
    "chainId": 1,
    "address": "0x6307b25a665efc992ec1c1bc403c38f3ddd7c661",
    "name": "Global Coin Research",
    "symbol": "GCR",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/14815/thumb/gcr.jpeg?1696514483"
  },
  {
    "chainId": 1,
    "address": "0xb0e99627bc29adef1178f16117bf495351e81997",
    "name": "Dex Trade Coin",
    "symbol": "DXC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15025/thumb/R1A63oDx_400x400.jpg?1696514686"
  },
  {
    "chainId": 1,
    "address": "0x09970aec766b6f3223aca9111555e99dc50ff13a",
    "name": "Levolution",
    "symbol": "LEVL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7370/thumb/L_Icon_Blue_200x200.png?1696507652"
  },
  {
    "chainId": 1,
    "address": "0x8bc2bcb1b1896291942c36f3cca3c1afa0aaa7fd",
    "name": "3SPACE ART",
    "symbol": "PACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33575/thumb/3space-art.png?1702901855"
  },
  {
    "chainId": 1,
    "address": "0x0f17bc9a994b87b5225cfb6a2cd4d667adb4f20b",
    "name": "Jarvis Synthetic Euro",
    "symbol": "JEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15725/thumb/jEUR.png?1696515352"
  },
  {
    "chainId": 1,
    "address": "0x5d858bcd53e085920620549214a8b27ce2f04670",
    "name": "POP Network",
    "symbol": "POP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7281/thumb/Logo_%28Light_Mode%29_%281%29.png?1696507572"
  },
  {
    "chainId": 1,
    "address": "0xa44e5137293e855b1b7bc7e2c6f8cd796ffcb037",
    "name": "Sentinel  OLD ",
    "symbol": "DVPN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/3625/thumb/Sentinel_512X512.png?1696504304"
  },
  {
    "chainId": 1,
    "address": "0x0b5326da634f9270fb84481dd6f94d3dc2ca7096",
    "name": "Etho Protocol",
    "symbol": "ETHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5194/thumb/IconOnly_Transparent_NoBuffer.png?1712452538"
  },
  {
    "chainId": 1,
    "address": "0xc52c326331e9ce41f04484d3b5e5648158028804",
    "name": "Unizen",
    "symbol": "ZCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14830/thumb/logo128px.png?1700169101"
  },
  {
    "chainId": 1,
    "address": "0x321c2fe4446c7c963dc41dd58879af648838f98d",
    "name": "Cryptex Finance",
    "symbol": "CTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14932/thumb/glossy_icon_-_C200px.png?1696514592"
  },
  {
    "chainId": 1,
    "address": "0xd85ad783cc94bd04196a13dc042a3054a9b52210",
    "name": "TribeOne",
    "symbol": "HAKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16575/thumb/USqW1QX.png?1696516136"
  },
  {
    "chainId": 1,
    "address": "0x565d3902d6a5a2d5ce28ff427423e88933334dd2",
    "name": "Adult Playground",
    "symbol": "ADULT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32053/thumb/0x565d3902d6a5a2d5ce28ff427423e88933334dd2.png?1696530850"
  },
  {
    "chainId": 1,
    "address": "0x48c3399719b582dd63eb5aadf12a40b4c3f52fa2",
    "name": "StakeWise",
    "symbol": "SWISE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15044/thumb/stakewise200.png?1696514703"
  },
  {
    "chainId": 1,
    "address": "0x8cc0f052fff7ead7f2edcccac895502e884a8a71",
    "name": "ARTH",
    "symbol": "ARTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16876/thumb/Ik5dhOq.png?1696516444"
  },
  {
    "chainId": 1,
    "address": "0xd62baa6f3c579316b2798262a219b367c549c7dc",
    "name": "Reflect Audit",
    "symbol": "REF",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35681/thumb/ref.jpeg?1709525682"
  },
  {
    "chainId": 1,
    "address": "0x43a96962254855f16b925556f9e97be436a43448",
    "name": "Hord",
    "symbol": "HORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14972/thumb/HORD_TOKEN_%281%29_1.png?1705414356"
  },
  {
    "chainId": 1,
    "address": "0xb5f1457d6fba1956fb8d31b0b7caca14bde0be4b",
    "name": "Stilton",
    "symbol": "STILT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26132/thumb/2vfQHbe9_400x400.png?1696525220"
  },
  {
    "chainId": 1,
    "address": "0x3a82d3111ab5faf39d847d46023d9090261a658f",
    "name": "Tycoon",
    "symbol": "TYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15165/thumb/coingecko.png?1696514820"
  },
  {
    "chainId": 1,
    "address": "0x77fba179c79de5b7653f68b5039af940ada60ce0",
    "name": "Ampleforth Governance",
    "symbol": "FORTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14917/thumb/photo_2021-04-22_00.00.03.jpeg?1696514579"
  },
  {
    "chainId": 1,
    "address": "0x20cd2e7ec8f5d8b337fe46a4f565ccef1561b9a9",
    "name": "ESG",
    "symbol": "ESG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24932/thumb/esg.png?1696524088"
  },
  {
    "chainId": 1,
    "address": "0xedadeb5faa413e6c8623461849dfd0b7c3790c32",
    "name": "Obortech",
    "symbol": "OBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14929/thumb/OBORTECH_200.png?1696514590"
  },
  {
    "chainId": 1,
    "address": "0x27778e14ce36d3b85e1effeb43816a17bbb7088a",
    "name": "Lyfe Gold",
    "symbol": "LGOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14938/thumb/LGOLD-Logo-200.png?1696514597"
  },
  {
    "chainId": 1,
    "address": "0xd64809f5f7d772d9112a6bd379de00a77188199e",
    "name": "Lyfe Silver",
    "symbol": "LSILVER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14939/thumb/LSILVER-Logo-200.png?1696514598"
  },
  {
    "chainId": 1,
    "address": "0xc55c2175e90a46602fd42e931f62b3acc1a013ca",
    "name": "Mogul Productions",
    "symbol": "STARS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14975/thumb/STARS_LOGO_PNG.png?1696514635"
  },
  {
    "chainId": 1,
    "address": "0xb6ee9668771a79be7967ee29a63d4184f8097143",
    "name": "CargoX",
    "symbol": "CXO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2580/thumb/cargox.png?1696503384"
  },
  {
    "chainId": 1,
    "address": "0xf4cd3d3fda8d7fd6c5a500203e38640a70bf9577",
    "name": "YfDAI finance",
    "symbol": "YF-DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12385/thumb/1619048513068.png?1696512208"
  },
  {
    "chainId": 1,
    "address": "0x4a615bb7166210cce20e6642a6f8fb5d4d044496",
    "name": "NAOS Finance",
    "symbol": "NAOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15144/thumb/bafybeibkztkshitabrf7yqqkqtbjqestjknpgv7lsjfzdsa3ufspagqs2e.ipfs.infura-ipfs.io.png?1696514800"
  },
  {
    "chainId": 1,
    "address": "0x61e90a50137e1f645c9ef4a0d3a4f01477738406",
    "name": "League of Kingdoms",
    "symbol": "LOKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22572/thumb/loka_64pix.png?1696521891"
  },
  {
    "chainId": 1,
    "address": "0x0352557b007a4aae1511c114409b932f06f9e2f4",
    "name": "sRUNE",
    "symbol": "SRUNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14953/thumb/sRUNE.png?1696514611"
  },
  {
    "chainId": 1,
    "address": "0x20a62aca58526836165ca53fe67dd884288c8abf",
    "name": "Rentible",
    "symbol": "RNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15176/thumb/rnb_200x200.png?1703145247"
  },
  {
    "chainId": 1,
    "address": "0x253a5b4f0a210b8d33b702a938b1d56d06b5449e",
    "name": "Omni Foundation",
    "symbol": "OMN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36675/thumb/Layer_1_%281%29_%282%29.png?1712062599"
  },
  {
    "chainId": 1,
    "address": "0x808507121b80c02388fad14726482e061b8da827",
    "name": "Pendle",
    "symbol": "PENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15069/thumb/Pendle_Logo_Normal-03.png?1696514728"
  },
  {
    "chainId": 1,
    "address": "0x7697b462a7c4ff5f8b55bdbc2f4076c2af9cf51a",
    "name": "Sarcophagus",
    "symbol": "SARCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15091/thumb/E2S2-CcUcAAyNxD.jpeg?1696514749"
  },
  {
    "chainId": 1,
    "address": "0x5fbc3cb8b428cc00a04808870295d39962cc7fee",
    "name": "Bitazza",
    "symbol": "BTZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24597/thumb/BTZ_Logo_%28200x200%29.png?1696523771"
  },
  {
    "chainId": 1,
    "address": "0xcef46305d096fa876dd23048bf80f9345282e3fc",
    "name": "Banque Universal",
    "symbol": "CBU",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/8112/thumb/png_tr.png?1696508329"
  },
  {
    "chainId": 1,
    "address": "0x4575f41308ec1483f3d399aa9a2826d74da13deb",
    "name": "Orchid Protocol",
    "symbol": "OXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3916/thumb/download_%285%29.png?1696504574"
  },
  {
    "chainId": 1,
    "address": "0xc67b12049c2d0cf6e476bc64c7f82fc6c63cffc5",
    "name": "Globe Derivative Exchange",
    "symbol": "GDT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/15335/thumb/gdt.PNG?1696514983"
  },
  {
    "chainId": 1,
    "address": "0x9d409a0a012cfba9b15f6d4b36ac57a46966ab9a",
    "name": "Yearn Compounding veCRV yVault",
    "symbol": "YVBOOST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15152/thumb/yvBOOST.png?1696514808"
  },
  {
    "chainId": 1,
    "address": "0x358aa737e033f34df7c54306960a38d09aabd523",
    "name": "Ares Protocol",
    "symbol": "ARES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15153/thumb/Ares-logo.png?1696514809"
  },
  {
    "chainId": 1,
    "address": "0xf519381791c03dd7666c142d4e49fd94d3536011",
    "name": "Asia Coin",
    "symbol": "ASIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18589/thumb/Ou7mp_R1TQ5B9vsBiZ8oQnSv36M6hiA2hESxV_7YSw0.png?1696518065"
  },
  {
    "chainId": 1,
    "address": "0xbbbbbbb5aa847a2003fbc6b5c16df0bd1e725f61",
    "name": "B Protocol",
    "symbol": "BPRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15110/thumb/66428641.jpg?1696514768"
  },
  {
    "chainId": 1,
    "address": "0x1614f18fc94f47967a3fbe5ffcd46d4e7da3d787",
    "name": "PAID Network",
    "symbol": "PAID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13761/thumb/512.png?1696513503"
  },
  {
    "chainId": 1,
    "address": "0xc064f4f215b6a1e4e7f39bd8530c4de0fc43ee9d",
    "name": "LeisureMeta",
    "symbol": "LM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25761/thumb/SVG_16533804486374586M.jpg?1696524846"
  },
  {
    "chainId": 1,
    "address": "0x668c50b1c7f46effbe3f242687071d7908aab00a",
    "name": "CoShi Inu",
    "symbol": "COSHI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15219/thumb/200.jpg?1696514874"
  },
  {
    "chainId": 1,
    "address": "0x2d94aa3e47d9d5024503ca8491fce9a2fb4da198",
    "name": "Bankless DAO",
    "symbol": "BANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15227/thumb/j4WEJrwU.png?1696514882"
  },
  {
    "chainId": 1,
    "address": "0xdd1ad9a21ce722c151a836373babe42c868ce9a4",
    "name": "Universal Basic Income",
    "symbol": "UBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15269/thumb/ubi.png?1696514921"
  },
  {
    "chainId": 1,
    "address": "0x0275e1001e293c46cfe158b3702aade0b99f88a5",
    "name": "Oiler",
    "symbol": "OIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15249/thumb/oiler.png?1696514902"
  },
  {
    "chainId": 1,
    "address": "0xac5b038058bcd0424c9c252c6487c25f032e5ddc",
    "name": "EpiK Protocol",
    "symbol": "AIEPK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15188/thumb/AIEPK_logo.png?1696514845"
  },
  {
    "chainId": 1,
    "address": "0x8baef8c9568c21b1a2b2fd048f8b4da835691fd0",
    "name": "USD ZEE",
    "symbol": "USDZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24876/thumb/003.png?1696524035"
  },
  {
    "chainId": 1,
    "address": "0x68c85b9f78f30a0df5ac5723e4e700037f185415",
    "name": "AiWork",
    "symbol": "AWO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15373/thumb/aiwork.PNG?1696515021"
  },
  {
    "chainId": 1,
    "address": "0x0c572544a4ee47904d54aaa6a970af96b6f00e1b",
    "name": "Wasder",
    "symbol": "WAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15374/thumb/wasderBoxedLogoWhite-200x200.png?1696515022"
  },
  {
    "chainId": 1,
    "address": "0xc8807f0f5ba3fa45ffbdc66928d71c5289249014",
    "name": "Ispolink",
    "symbol": "ISP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15283/thumb/isolink.PNG?1696514934"
  },
  {
    "chainId": 1,
    "address": "0xd2adc1c84443ad06f0017adca346bd9b6fc52cab",
    "name": "dFund",
    "symbol": "DFND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15286/thumb/1zbdX36.png?1696514937"
  },
  {
    "chainId": 1,
    "address": "0x464fdb8affc9bac185a7393fd4298137866dcfb8",
    "name": "Realm",
    "symbol": "REALM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18366/thumb/realm.PNG?1696517859"
  },
  {
    "chainId": 1,
    "address": "0x537edd52ebcb9f48ff2f8a28c51fcdb9d6a6e0d4",
    "name": "Small Doge",
    "symbol": "SDOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15377/thumb/small_dog.PNG?1696515024"
  },
  {
    "chainId": 1,
    "address": "0x5d30ad9c6374bf925d0a75454fa327aacf778492",
    "name": "PERI Finance",
    "symbol": "PERI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15313/thumb/6xVEMS1.png?1696514963"
  },
  {
    "chainId": 1,
    "address": "0x368c5290b13caa10284db58b4ad4f3e9ee8bf4c9",
    "name": "KKO Protocol",
    "symbol": "KKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15366/thumb/kko-coingecko.png?1696515014"
  },
  {
    "chainId": 1,
    "address": "0xd084944d3c05cd115c09d072b9f44ba3e0e45921",
    "name": "Manifold Finance",
    "symbol": "FOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15928/thumb/Manifold.png?1696515541"
  },
  {
    "chainId": 1,
    "address": "0x48c276e8d03813224bb1e55f953adb6d02fd3e02",
    "name": "Kuma Inu",
    "symbol": "KUMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15526/thumb/kuma_inu.PNG?1696515168"
  },
  {
    "chainId": 1,
    "address": "0xacfa209fb73bf3dd5bbfb1101b9bc999c49062a5",
    "name": "EvidenZ",
    "symbol": "BCDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2014/thumb/evidenz-512.png?1696502988"
  },
  {
    "chainId": 1,
    "address": "0x993864e43caa7f7f12953ad6feb1d1ca635b875f",
    "name": "SingularityDAO",
    "symbol": "SDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15385/thumb/200x200_logo.png?1696515032"
  },
  {
    "chainId": 1,
    "address": "0xd89310f4baedb33afb36d7cc45bb8847f4463060",
    "name": "MEMEX",
    "symbol": "MEMEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15448/thumb/V46Sq63d_400x400.jpg?1696515092"
  },
  {
    "chainId": 1,
    "address": "0x2a03a891add2dc6d0f7b94419086630ba5cb65b6",
    "name": "Dreamverse",
    "symbol": "DV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21554/thumb/dv.png?1696520916"
  },
  {
    "chainId": 1,
    "address": "0x18c1074845c389907762a71242bfe271c9da2d9c",
    "name": "BULL Token",
    "symbol": "BULL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30785/thumb/BULL200x200.jpg?1696529652"
  },
  {
    "chainId": 1,
    "address": "0x29ef81cc8737f19449d501396d4968d138140675",
    "name": "0xConnect",
    "symbol": "0XCON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33258/thumb/ox200.png?1701318655"
  },
  {
    "chainId": 1,
    "address": "0x74d2d73b455540b037298c0e0925bc702aedbe4a",
    "name": "Schrodinger",
    "symbol": "MEOW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19649/thumb/kitty_dinger.png?1696519077"
  },
  {
    "chainId": 1,
    "address": "0xc0b68eb52c89e3fffa62d78012ac8b661bfaa323",
    "name": "Vixco",
    "symbol": "VIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15405/thumb/vixco.png?1696515051"
  },
  {
    "chainId": 1,
    "address": "0xc28e931814725bbeb9e670676fabbcb694fe7df2",
    "name": "Quadrant Protocol",
    "symbol": "EQUAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4462/thumb/equad.png?1696505053"
  },
  {
    "chainId": 1,
    "address": "0xfc979087305a826c2b2a0056cfaba50aad3e6439",
    "name": "Dafi Protocol",
    "symbol": "DAFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14428/thumb/Dafi_Black_Icon.png?1696514118"
  },
  {
    "chainId": 1,
    "address": "0x81f8f0bb1cb2a06649e51913a151f0e7ef6fa321",
    "name": "VitaDAO",
    "symbol": "VITA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16580/thumb/QmRjwywiAfpzSfQTuWM3zCTghSHN7G6ohQaar7Ht6WANUp.png?1696516140"
  },
  {
    "chainId": 1,
    "address": "0x380bd08929d4ae51de22f708dccb6b4c534ab79a",
    "name": "TINFA",
    "symbol": "TINFA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34542/thumb/TINFA_%281%29.png?1705339483"
  },
  {
    "chainId": 1,
    "address": "0x557b933a7c2c45672b610f8954a3deb39a51a8ca",
    "name": "REVV",
    "symbol": "REVV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12373/thumb/REVV_TOKEN_Refined_2021_%281%29.png?1696512197"
  },
  {
    "chainId": 1,
    "address": "0x5cb888182fbffdb62c08fb4b5a343914f00fdfee",
    "name": "Moneybrain BiPS",
    "symbol": "BIPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15556/thumb/bips_logo_200x200_Trans.png?1696515197"
  },
  {
    "chainId": 1,
    "address": "0x64a77277e37d44957fe5815d6ff442ab8b16cc29",
    "name": "SpaceDawgs",
    "symbol": "DAWGS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17076/thumb/twitter-facebook-Instagram-pfp.png?1696516638"
  },
  {
    "chainId": 1,
    "address": "0xb64ef51c888972c908cfacf59b47c1afbc0ab8ac",
    "name": "Storj",
    "symbol": "STORJ",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/949/thumb/storj.png?1696502065"
  },
  {
    "chainId": 1,
    "address": "0x4a1d542b52a95ad01ddc70c2e7df0c7bbaadc56f",
    "name": "Niftify",
    "symbol": "NIFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20558/thumb/AWPDLxLK_400x400.jpg?1696519967"
  },
  {
    "chainId": 1,
    "address": "0x887168120cb89fb06f3e74dc4af20d67df0977f6",
    "name": "Sekuritance",
    "symbol": "SKRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15495/thumb/skrt.png?1696515139"
  },
  {
    "chainId": 1,
    "address": "0x675bbc7514013e2073db7a919f6e4cbef576de37",
    "name": "Coldstack",
    "symbol": "CLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15499/thumb/logo_200x200.png?1696515143"
  },
  {
    "chainId": 1,
    "address": "0x108a850856db3f85d0269a2693d896b394c80325",
    "name": "THORWallet DEX",
    "symbol": "TGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21843/thumb/tgt_logo.png?1696521198"
  },
  {
    "chainId": 1,
    "address": "0x53dfea0a8cc2a2a2e425e1c174bc162999723ea0",
    "name": "Jarvis Synthetic Swiss Franc",
    "symbol": "JCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15727/thumb/jCHF.png?1696515353"
  },
  {
    "chainId": 1,
    "address": "0x6100dd79fcaa88420750dcee3f735d168abcb771",
    "name": "Ethereans",
    "symbol": "OS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19592/thumb/eherian.jpg?1697038777"
  },
  {
    "chainId": 1,
    "address": "0x6bb61215298f296c55b19ad842d3df69021da2ef",
    "name": "Drops Ownership Power",
    "symbol": "DOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15696/thumb/dop.png?1696515325"
  },
  {
    "chainId": 1,
    "address": "0x997507cc49fbf0cd6ce5e1ee543218556fafdebc",
    "name": "Bitenium",
    "symbol": "BT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14618/thumb/BT-token-logo-200.png?1696514296"
  },
  {
    "chainId": 1,
    "address": "0x814e0908b12a99fecf5bc101bb5d0b8b5cdf7d26",
    "name": "Measurable Data",
    "symbol": "MDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2441/thumb/mdt_icon_120x120.png?1711452723"
  },
  {
    "chainId": 1,
    "address": "0x616ef40d55c0d2c506f4d6873bda8090b79bf8fc",
    "name": "Kounotori",
    "symbol": "KTO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21251/thumb/KTO.png?1696520624"
  },
  {
    "chainId": 1,
    "address": "0x6710c63432a2de02954fc0f851db07146a6c0312",
    "name": "Smart MFG",
    "symbol": "MFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1842/thumb/Smart_MFG_Cropped_Logo.png?1696502841"
  },
  {
    "chainId": 1,
    "address": "0xf55a93b613d172b86c2ba3981a849dae2aecde2f",
    "name": "Your Futures Exchange",
    "symbol": "YFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15654/thumb/yfx.PNG?1696515285"
  },
  {
    "chainId": 1,
    "address": "0xda23d301761e4e2bf474951f978f6dfb6f3c9f14",
    "name": "Tsuki Inu",
    "symbol": "TKINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15681/thumb/tkinu.png?1696515311"
  },
  {
    "chainId": 1,
    "address": "0x72a66e54b66892ae3bbe54df7bb7dd5ae927a6f9",
    "name": "Crypto Village Accelerator CVAG",
    "symbol": "CVAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17124/thumb/cvag.PNG?1696516684"
  },
  {
    "chainId": 1,
    "address": "0xf720e38f678b29b243f7d53b56acbf5de98f2385",
    "name": "Upfire",
    "symbol": "UPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20562/thumb/UPR256.png?1696519971"
  },
  {
    "chainId": 1,
    "address": "0x846c66cf71c43f80403b51fe3906b3599d63336f",
    "name": "PumaPay",
    "symbol": "PMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2307/thumb/pumapay_dark_rounded_256x256.png?1696503200"
  },
  {
    "chainId": 1,
    "address": "0xfc70cbb442d5c115ee1497d22b421b1f9bd9f3da",
    "name": "Biblical Truth",
    "symbol": "BTRU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36677/thumb/BTRU_Logo.png?1712062960"
  },
  {
    "chainId": 1,
    "address": "0x51db5ad35c671a87207d88fc11d593ac0c8415bd",
    "name": "Moeda Loyalty Points",
    "symbol": "MDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/895/thumb/moeda-loyalty-points.png?1696502019"
  },
  {
    "chainId": 1,
    "address": "0x00d8318e44780edeefcf3020a5448f636788883c",
    "name": "dAppstore",
    "symbol": "DAPPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15780/thumb/large-logo.png?1696515404"
  },
  {
    "chainId": 1,
    "address": "0x107c4504cd79c5d2696ea0030a8dd4e92601b82e",
    "name": "Bloom",
    "symbol": "BLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2662/thumb/bloom.png?1696503453"
  },
  {
    "chainId": 1,
    "address": "0x2ebd53d035150f328bd754d6dc66b99b0edb89aa",
    "name": "Metronome",
    "symbol": "MET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3249/thumb/met.png?1696503964"
  },
  {
    "chainId": 1,
    "address": "0xf4d2888d29d722226fafa5d9b24f9164c092421e",
    "name": "LooksRare",
    "symbol": "LOOKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22173/thumb/circle-black-256.png?1696521517"
  },
  {
    "chainId": 1,
    "address": "0xcb84d72e61e383767c4dfeb2d8ff7f4fb89abc6e",
    "name": "Vega Protocol",
    "symbol": "VEGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15870/thumb/vega.PNG?1696515485"
  },
  {
    "chainId": 1,
    "address": "0x6123b0049f904d730db3c36a31167d9d4121fa6b",
    "name": "Ribbon Finance",
    "symbol": "RBN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15823/thumb/ribbon.png?1709186956"
  },
  {
    "chainId": 1,
    "address": "0x47da5456bc2e1ce391b645ce80f2e97192e4976a",
    "name": "PL Gnet",
    "symbol": "PLUG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15824/thumb/PLUG_token.png?1696515442"
  },
  {
    "chainId": 1,
    "address": "0x1ffefd8036409cb6d652bd610de465933b226917",
    "name": "Everscale",
    "symbol": "EVER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/12783/thumb/everscale_badge_main_round_1x.png?1696512578"
  },
  {
    "chainId": 1,
    "address": "0x5eeaa2dcb23056f4e8654a349e57ebe5e76b5e6e",
    "name": "Virtue Poker Points",
    "symbol": "VPP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3386/thumb/vp-logo-200x200.png?1696504084"
  },
  {
    "chainId": 1,
    "address": "0x618e75ac90b12c6049ba3b27f5d5f8651b0037f6",
    "name": "QASH",
    "symbol": "QASH",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/1220/thumb/qash.jpeg?1696502299"
  },
  {
    "chainId": 1,
    "address": "0xcc802c45b55581713cecd1eb17be9ab7fccb0844",
    "name": "SBU Honey",
    "symbol": "BHNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26079/thumb/honey.png?1696525163"
  },
  {
    "chainId": 1,
    "address": "0xf6650117017ffd48b725b4ec5a00b414097108a7",
    "name": "Xido Finance",
    "symbol": "XIDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16161/thumb/KJw4clj.png?1696515765"
  },
  {
    "chainId": 1,
    "address": "0xb67beb26ebeb0dceec354ae0942256d03c01771b",
    "name": "DeStorage",
    "symbol": "DS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15984/thumb/DS_LOG_Transparent_200x200.png?1696515597"
  },
  {
    "chainId": 1,
    "address": "0x5f474906637bdcda05f29c74653f6962bb0f8eda",
    "name": "DeFinity",
    "symbol": "DEFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15875/thumb/definity-listing-logo.png?1696515490"
  },
  {
    "chainId": 1,
    "address": "0x9d7630adf7ab0b0cb00af747db76864df0ec82e4",
    "name": "GATENet",
    "symbol": "GATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16009/thumb/82168105.png?1696515621"
  },
  {
    "chainId": 1,
    "address": "0x2620638eda99f9e7e902ea24a285456ee9438861",
    "name": "Crust Shadow",
    "symbol": "CSM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16037/thumb/2_mnCYZfHmLg2bFrjM3vKtPw.png?1696515647"
  },
  {
    "chainId": 1,
    "address": "0x7b7983967409fce461ea8bbdf9ed37631b1d59c9",
    "name": "KPOP Coin",
    "symbol": "KPOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15923/thumb/kpop.PNG?1696515536"
  },
  {
    "chainId": 1,
    "address": "0x8a001303158670e284950565164933372807cd48",
    "name": "WaifuAI",
    "symbol": "WFAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30606/thumb/waifuai.jpg?1696529476"
  },
  {
    "chainId": 1,
    "address": "0xed6bb4700abd545575f0642bb5ea8e0d3b24e9f3",
    "name": "Digital Files",
    "symbol": "DIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29282/thumb/DIFI_logo.png?1696528234"
  },
  {
    "chainId": 1,
    "address": "0xdd974d5c2e2928dea5f71b9825b8b646686bd200",
    "name": "Kyber Network Crystal Legacy",
    "symbol": "KNCL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/947/thumb/logo-kncl.png?1696502063"
  },
  {
    "chainId": 1,
    "address": "0xaa4e3edb11afa93c41db59842b29de64b72e355b",
    "name": "Marginswap",
    "symbol": "MFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13899/thumb/marginswap_logo.png?1696513642"
  },
  {
    "chainId": 1,
    "address": "0x9aab071b4129b083b01cb5a0cb513ce7eca26fa5",
    "name": "Hunt",
    "symbol": "HUNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7989/thumb/HUNT.png?1696508215"
  },
  {
    "chainId": 1,
    "address": "0x16980b3b4a3f9d89e33311b5aa8f80303e5ca4f8",
    "name": "KIRA Network",
    "symbol": "KEX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13232/thumb/KEX-200x200.png?1705676884"
  },
  {
    "chainId": 1,
    "address": "0x3267c5b73cc15e253b1a90c01366b17d560bc6fb",
    "name": "President Ron DeSantis",
    "symbol": "RON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30620/thumb/Desantis-200x200.jpg?1696529494"
  },
  {
    "chainId": 1,
    "address": "0xc314b0e758d5ff74f63e307a86ebfe183c95767b",
    "name": "Adappter",
    "symbol": "ADP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14203/thumb/logo_on.png?1696513920"
  },
  {
    "chainId": 1,
    "address": "0x8355dbe8b0e275abad27eb843f3eaf3fc855e525",
    "name": "Wolf Game Wool",
    "symbol": "WOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20760/thumb/KM3RwIVx_400x400.jpg?1696520155"
  },
  {
    "chainId": 1,
    "address": "0xf970b8e36e23f7fc3fd752eea86f8be8d83375a6",
    "name": "Ripio Credit Network",
    "symbol": "RCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1057/thumb/ripio-credit-network.png?1696502162"
  },
  {
    "chainId": 1,
    "address": "0x64d91f12ece7362f91a6f8e7940cd55f05060b92",
    "name": "ASH",
    "symbol": "ASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15714/thumb/omnPqaTY.png?1696515341"
  },
  {
    "chainId": 1,
    "address": "0xe1fc4455f62a6e89476f1072530c20cf1a0622da",
    "name": "Phuture",
    "symbol": "PHTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16071/thumb/phtr.png?1696515680"
  },
  {
    "chainId": 1,
    "address": "0x97b65710d03e12775189f0d113202cc1443b0aa2",
    "name": "AstroElon",
    "symbol": "ELONONE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16082/thumb/AstroElon.png?1696515690"
  },
  {
    "chainId": 1,
    "address": "0x4da34f8264cb33a5c9f17081b9ef5ff6091116f4",
    "name": "ELYFI",
    "symbol": "ELFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23733/thumb/elyfi_logo.png?1696522936"
  },
  {
    "chainId": 1,
    "address": "0xfd414e39155f91e94443a9fe97e856569d0f5eec",
    "name": "Shibarium Perpetuals",
    "symbol": "SERP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29465/thumb/photo_2023-03-15_22.44.48-removebg-preview_%281%29.png?1696528411"
  },
  {
    "chainId": 1,
    "address": "0x8f828a0644f12fa352888e645a90333d30f6fd7d",
    "name": "Rinia Inu",
    "symbol": "RINIA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29466/thumb/rinia.png?1696528411"
  },
  {
    "chainId": 1,
    "address": "0x69af81e73a73b40adf4f3d4223cd9b1ece623074",
    "name": "Mask Network",
    "symbol": "MASK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14051/thumb/Mask_Network.jpg?1696513776"
  },
  {
    "chainId": 1,
    "address": "0x13b2f6928d7204328b0e8e4bcd0379aa06ea21fa",
    "name": "Aave AMM WBTC",
    "symbol": "AAMMWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17215/thumb/aAMMWBTC_2x.png?1696516771"
  },
  {
    "chainId": 1,
    "address": "0xd4c435f5b09f855c3317c8524cb1f586e42795fa",
    "name": "Cindicator",
    "symbol": "CND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1006/thumb/cindicator.png?1696502118"
  },
  {
    "chainId": 1,
    "address": "0xcb86c6a22cb56b6cf40cafedb06ba0df188a416e",
    "name": "inSure DeFi",
    "symbol": "SURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10354/thumb/logo-grey-circle.png?1696510355"
  },
  {
    "chainId": 1,
    "address": "0xa1b0edf4460cc4d8bfaa18ed871bff15e5b57eb4",
    "name": "Aave AMM UniBATWETH",
    "symbol": "AAMMUNIBATWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17217/thumb/aAmmUniBATWETH.png?1696516773"
  },
  {
    "chainId": 1,
    "address": "0xe340b25fe32b1011616bb8ec495a4d503e322177",
    "name": "Aave AMM UniDAIUSDC",
    "symbol": "AAMMUNIDAIUSDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17218/thumb/aAmmUniDAIUSDC.png?1696516773"
  },
  {
    "chainId": 1,
    "address": "0x947aeb02304391f8fbe5b25d7d98d649b57b1788",
    "name": "Mandala Exchange",
    "symbol": "MDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13736/thumb/png-cmc.png?1696513480"
  },
  {
    "chainId": 1,
    "address": "0x8d137e3337eb1b58a222fef2b2cc7c423903d9cf",
    "name": "SQGL Vault  NFTX ",
    "symbol": "SQGL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17041/thumb/SQGL.png?1696516604"
  },
  {
    "chainId": 1,
    "address": "0xcfa0885131f602d11d4da248d2c65a62063567a9",
    "name": "TORG",
    "symbol": "TORG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17596/thumb/TORG_Logo_200x200.png?1696517129"
  },
  {
    "chainId": 1,
    "address": "0x391e86e2c002c70dee155eaceb88f7a3c38f5976",
    "name": "Aave AMM UniUSDCWETH",
    "symbol": "AAMMUNIUSDCWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17219/thumb/aAmmUniUSDCWETH.png?1696516774"
  },
  {
    "chainId": 1,
    "address": "0x09ce2b746c32528b7d864a1e3979bd97d2f095ab",
    "name": "DeFIL",
    "symbol": "DFL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17708/thumb/defillogo200200.png?1696517235"
  },
  {
    "chainId": 1,
    "address": "0x0ea20e7ffb006d4cfe84df2f72d8c7bd89247db0",
    "name": "Aave AMM UniCRVWETH",
    "symbol": "AAMMUNICRVWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17220/thumb/aAmmUniCRVWETH.png?1696516775"
  },
  {
    "chainId": 1,
    "address": "0x491e136ff7ff03e6ab097e54734697bb5802fc1c",
    "name": "Kattana",
    "symbol": "KTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14739/thumb/256-256-1.png?1696514409"
  },
  {
    "chainId": 1,
    "address": "0x069f967be0ca21c7d793d8c343f71e597d9a49b3",
    "name": "HZM Coin",
    "symbol": "HZM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/18032/thumb/HZM_new_Logo_PNG_CoinGecko.png?1696517548"
  },
  {
    "chainId": 1,
    "address": "0xb8db81b84d30e2387de0ff330420a4aaa6688134",
    "name": "Aave AMM UniLINKWETH",
    "symbol": "AAMMUNILINKWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17221/thumb/aAmmUniLINKWETH.png?1696516776"
  },
  {
    "chainId": 1,
    "address": "0x93c9175e26f57d2888c7df8b470c9eea5c0b0a93",
    "name": "B cube ai",
    "symbol": "BCUBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15948/thumb/bcube.PNG?1696515560"
  },
  {
    "chainId": 1,
    "address": "0x44108f0223a3c3028f5fe7aec7f9bb2e66bef82f",
    "name": "Across Protocol",
    "symbol": "ACX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28161/thumb/across-200x200.png?1696527165"
  },
  {
    "chainId": 1,
    "address": "0x508df5aa4746be37b5b6a69684dfd8bdc322219d",
    "name": "Crafting Finance",
    "symbol": "CRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21559/thumb/tokenlogo.png?1696520920"
  },
  {
    "chainId": 1,
    "address": "0x70401dfd142a16dc7031c56e862fc88cb9537ce0",
    "name": "Bird Money",
    "symbol": "BIRD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13260/thumb/favicon-180x180.png?1696513035"
  },
  {
    "chainId": 1,
    "address": "0x3e2324342bf5b8a1dca42915f0489497203d640e",
    "name": "Barley Finance",
    "symbol": "BARL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34867/thumb/logo_200.png?1706410358"
  },
  {
    "chainId": 1,
    "address": "0xa9e201a4e269d6cd5e9f0fcbcb78520cf815878b",
    "name": "Aave AMM UniRENWETH",
    "symbol": "AAMMUNIRENWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17223/thumb/aAmmUniRENWETH.png?1696516778"
  },
  {
    "chainId": 1,
    "address": "0x6368e1e18c4c419ddfc608a0bed1ccb87b9250fc",
    "name": "Tap",
    "symbol": "XTP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10291/thumb/0_3SJYkk_400x400.jpg?1696510296"
  },
  {
    "chainId": 1,
    "address": "0x38e491a71291cd43e8de63b7253e482622184894",
    "name": "Aave AMM UniSNXWETH",
    "symbol": "AAMMUNISNXWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17224/thumb/aAmmUniSNXWETH.png?1696516779"
  },
  {
    "chainId": 1,
    "address": "0x3d26dcd840fcc8e4b2193ace8a092e4a65832f9f",
    "name": "Aave AMM UniUNIWETH",
    "symbol": "AAMMUNIUNIWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17225/thumb/aAmmUniUNIWETH.png?1696516780"
  },
  {
    "chainId": 1,
    "address": "0xb009bfaaf85e53f55d8657781eb69feaaed83672",
    "name": "EdgeSwap",
    "symbol": "EGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24878/thumb/egs.png?1696524036"
  },
  {
    "chainId": 1,
    "address": "0x4e3fbd56cd56c3e72c1403e103b45db9da5b9d2b",
    "name": "Convex Finance",
    "symbol": "CVX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15585/thumb/convex.png?1696515221"
  },
  {
    "chainId": 1,
    "address": "0xed0d5747a9ab03a75fbfec3228cd55848245b75d",
    "name": "e Money",
    "symbol": "NGM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13722/thumb/logo-200x200.jpg?1696513466"
  },
  {
    "chainId": 1,
    "address": "0x37a15c92e67686aa268df03d4c881a76340907e8",
    "name": "Pixiu Finance",
    "symbol": "PIXIU",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/17297/thumb/pixiu_picture_.png?1696516850"
  },
  {
    "chainId": 1,
    "address": "0x08c32b0726c5684024ea6e141c50ade9690bbdcc",
    "name": "Stratos",
    "symbol": "STOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16213/thumb/token_512x512.png?1696515814"
  },
  {
    "chainId": 1,
    "address": "0x9f9913853f749b3fe6d6d4e16a1cc3c1656b6d51",
    "name": "BITT",
    "symbol": "BITT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13783/thumb/BITT_Logo_256pixels.png?1696513523"
  },
  {
    "chainId": 1,
    "address": "0x08037036451c768465369431da5c671ad9b37dbc",
    "name": "NFT Stars",
    "symbol": "NFTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16141/thumb/j2KHi8zR_400x400.png?1696515746"
  },
  {
    "chainId": 1,
    "address": "0xd24946147829deaa935be2ad85a3291dbf109c80",
    "name": "Aave AMM USDC",
    "symbol": "AAMMUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17226/thumb/aAMMUSDC_2x.png?1696516780"
  },
  {
    "chainId": 1,
    "address": "0xcd7492db29e2ab436e819b249452ee1bbdf52214",
    "name": "SafeMoon Inu",
    "symbol": "SMI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/16091/thumb/SMI.png?1696515699"
  },
  {
    "chainId": 1,
    "address": "0x559b7bfc48a5274754b08819f75c5f27af53d53b",
    "name": "Qi Dao",
    "symbol": "QI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15329/thumb/qidao.jpeg?1697093140"
  },
  {
    "chainId": 1,
    "address": "0xb8647e90c0645152fccf4d9abb6b59eb4aa99052",
    "name": "KeyFi",
    "symbol": "KEYFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15098/thumb/keyfi_logo.jpg?1696514756"
  },
  {
    "chainId": 1,
    "address": "0xc58f53a8adff2fb4eb16ed56635772075e2ee123",
    "name": "Aave AMM UniWBTCWETH",
    "symbol": "AAMMUNIWBTCWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17231/thumb/aAmmUniWBTCWETH.png?1696516785"
  },
  {
    "chainId": 1,
    "address": "0xf5eda6c581f4373b07ce111baf8d1c4fc21cbaa1",
    "name": "Game Tournament Trophy",
    "symbol": "GTT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x8013266cb5c9dd48be3ad7d1ce832874d64b3ce1",
    "name": "Boop",
    "symbol": "BOOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27918/thumb/boop.png?1696526938"
  },
  {
    "chainId": 1,
    "address": "0xe59d2ff6995a926a574390824a657eed36801e55",
    "name": "Aave AMM UniAAVEWETH",
    "symbol": "AAMMUNIAAVEWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17232/thumb/aAmmUniAAVEWETH.png?1696516786"
  },
  {
    "chainId": 1,
    "address": "0x9196e18bc349b1f64bc08784eae259525329a1ad",
    "name": "Pussy Financial",
    "symbol": "PUSSY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15213/thumb/pussytoken.png?1696514869"
  },
  {
    "chainId": 1,
    "address": "0x9f9643209dcce8d7399d7bf932354768069ebc64",
    "name": "Invest Club Global",
    "symbol": "ICG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34316/thumb/thatone_200%281%29.png?1704621005"
  },
  {
    "chainId": 1,
    "address": "0x450e7f6e3a2f247a51b98c39297a9a5bfbdb3170",
    "name": "Elon GOAT",
    "symbol": "EGT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22301/thumb/ElonGoat.jpg?1696521647"
  },
  {
    "chainId": 1,
    "address": "0x7c28310cc0b8d898c57b93913098e74a3ba23228",
    "name": "PEACE COIN",
    "symbol": "PCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32470/thumb/PEACE.jpg?1698259975"
  },
  {
    "chainId": 1,
    "address": "0x9303eabc860a743aabcc3a1629014cabcc3f8d36",
    "name": "Aave AMM UniDAIWETH",
    "symbol": "AAMMUNIDAIWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17230/thumb/aAmmUniDAIWETH.png?1696516784"
  },
  {
    "chainId": 1,
    "address": "0x37fe0f067fa808ffbdd12891c0858532cfe7361d",
    "name": "Civilization",
    "symbol": "CIV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17626/thumb/civ-200x200.png?1696517157"
  },
  {
    "chainId": 1,
    "address": "0xcdb9d30a3ba48cdfcb0ecbe19317e6cf783672f1",
    "name": "Mondo Community Coin",
    "symbol": "MNDCC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18680/thumb/mndcc_bildmarke_256px.png?1696518149"
  },
  {
    "chainId": 1,
    "address": "0x1a4b46696b2bb4794eb3d4c26f1c55f9170fa4c5",
    "name": "BitDAO",
    "symbol": "BIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17627/thumb/rI_YptK8.png?1696517158"
  },
  {
    "chainId": 1,
    "address": "0x030ba81f1c18d280636f32af80b9aad02cf0854e",
    "name": "Aave WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17238/thumb/aWETH_2x.png?1696516795"
  },
  {
    "chainId": 1,
    "address": "0x641927e970222b10b2e8cdbc96b1b4f427316f16",
    "name": "MEEB Vault  NFTX ",
    "symbol": "MEEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17044/thumb/Meebs.png?1696516607"
  },
  {
    "chainId": 1,
    "address": "0xd1b624f07a4d9b3e3746e33cb58f42df079b5444",
    "name": "NKCL Classic",
    "symbol": "NKCLC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17893/thumb/w02Nksm-.png?1696517413"
  },
  {
    "chainId": 1,
    "address": "0x272f97b7a56a387ae942350bbc7df5700f8a4576",
    "name": "Aave BAL",
    "symbol": "ABAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17242/thumb/aBAL_2x.png?1696516799"
  },
  {
    "chainId": 1,
    "address": "0xac1419ee74f203c6b9daa3635ad7169b7ebb5c1a",
    "name": "Nebula",
    "symbol": "NEBULA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33029/thumb/20231116_120200.jpg?1700347575"
  },
  {
    "chainId": 1,
    "address": "0x409c4d8cd5d2924b9bc5509230d16a61289c8153",
    "name": "TONStarter",
    "symbol": "TOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17878/thumb/tos.PNG?1696517400"
  },
  {
    "chainId": 1,
    "address": "0x841fb148863454a3b3570f515414759be9091465",
    "name": "Shih Tzu",
    "symbol": "SHIH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15309/thumb/shit.PNG?1696514959"
  },
  {
    "chainId": 1,
    "address": "0x99bb69ee1bbfc7706c3ebb79b21c5b698fe58ec0",
    "name": "DeHub",
    "symbol": "DHB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18094/thumb/dehub.PNG?1696517599"
  },
  {
    "chainId": 1,
    "address": "0x320d31183100280ccdf69366cd56180ea442a3e8",
    "name": "Lightcoin",
    "symbol": "LHC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/16381/thumb/lhc.PNG?1696515979"
  },
  {
    "chainId": 1,
    "address": "0xc9bc48c72154ef3e5425641a3c747242112a46af",
    "name": "Aave RAI",
    "symbol": "ARAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17245/thumb/aRAI_2x.png?1696516802"
  },
  {
    "chainId": 1,
    "address": "0xd37ee7e4f452c6638c96536e68090de8cbcdb583",
    "name": "Aave GUSD",
    "symbol": "AGUSD",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/17246/thumb/aGUSD_2x.png?1696516803"
  },
  {
    "chainId": 1,
    "address": "0xffc97d72e13e01096502cb8eb52dee56f74dad7b",
    "name": "Aave AAVE",
    "symbol": "AAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17248/thumb/amAAVE_2x.png?1696516804"
  },
  {
    "chainId": 1,
    "address": "0x0d86eb9f43c57f6ff3bc9e23d8f9d82503f0e84b",
    "name": "Maximus DAO",
    "symbol": "MAXI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25126/thumb/maxilogo200.png?1696524276"
  },
  {
    "chainId": 1,
    "address": "0xb9d7cb55f463405cdfbe4e90a6d2df01c2b92bf1",
    "name": "Aave UNI",
    "symbol": "AUNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17247/thumb/aUNI_2x.png?1696516804"
  },
  {
    "chainId": 1,
    "address": "0xb794ad95317f75c44090f64955954c3849315ffe",
    "name": "Ribbit Meme",
    "symbol": "RIBBIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30145/thumb/ribbit-logo.png?1696529066"
  },
  {
    "chainId": 1,
    "address": "0x03d1e72765545729a035e909edd9371a405f77fb",
    "name": "Nabox",
    "symbol": "NABOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16445/thumb/NyemjVRA_400x400.png?1696516042"
  },
  {
    "chainId": 1,
    "address": "0x20a8cec5fffea65be7122bcab2ffe32ed4ebf03a",
    "name": "DinoX",
    "symbol": "DNXC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17321/thumb/asset_icon_dnxc_200.png?1696516875"
  },
  {
    "chainId": 1,
    "address": "0x9a2af0abb12bee5369b180976be01e8c80d0e7b6",
    "name": "Empire",
    "symbol": "EMPIRE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16390/thumb/pb_ranDy_400x400.png?1696515988"
  },
  {
    "chainId": 1,
    "address": "0x7dc85f00715c9d9ec1b50731a9bcc8df95087c55",
    "name": "Clexy",
    "symbol": "CLEXY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34318/thumb/clexylogopurple_notext_%281%29.png?1704621766"
  },
  {
    "chainId": 1,
    "address": "0xd31695a1d35e489252ce57b129fd4b1b05e6acac",
    "name": "TOKPIE",
    "symbol": "TKP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3731/thumb/tokpie-200x200.png?1696504401"
  },
  {
    "chainId": 1,
    "address": "0xeca82185adce47f39c684352b0439f030f860318",
    "name": "PERL eco",
    "symbol": "PERL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4682/thumb/PERL.eco-Icon-green_6x.png?1696505249"
  },
  {
    "chainId": 1,
    "address": "0xeeee2a622330e6d2036691e983dee87330588603",
    "name": "Asko",
    "symbol": "ASKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11773/thumb/ASKO.png?1696511652"
  },
  {
    "chainId": 1,
    "address": "0xeec2be5c91ae7f8a338e1e5f3b5de49d07afdc81",
    "name": "Dopex",
    "symbol": "DPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16652/thumb/DPX_%281%29.png?1696516213"
  },
  {
    "chainId": 1,
    "address": "0x0ff5a8451a839f5f0bb3562689d9a44089738d11",
    "name": "Dopex Rebate",
    "symbol": "RDPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16659/thumb/rDPX_200x200_Coingecko.png?1696516221"
  },
  {
    "chainId": 1,
    "address": "0x02e7ac540409d32c90bfb51114003a9e1ff0249c",
    "name": "JPG NFT Index",
    "symbol": "JPG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25071/thumb/JPG-token-logo-01.png?1696524219"
  },
  {
    "chainId": 1,
    "address": "0x35e78b3982e87ecfd5b3f3265b601c046cdbe232",
    "name": "SideShift",
    "symbol": "XAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15719/thumb/sideshift-icon_3x.png?1696515346"
  },
  {
    "chainId": 1,
    "address": "0x1341a2257fa7b770420ef70616f888056f90926c",
    "name": "Zoo",
    "symbol": "ZOOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15629/thumb/gyzERsO.png?1696515262"
  },
  {
    "chainId": 1,
    "address": "0x53263d9ef74db583b15fbc6d5d4e8b83833fa134",
    "name": "LEAP Token",
    "symbol": "LEAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27258/thumb/LEAPtoken_LOGO.png?1696526311"
  },
  {
    "chainId": 1,
    "address": "0x0fd67b4ceb9b607ef206904ec73459c4880132c9",
    "name": "ShoeFy",
    "symbol": "SHOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19082/thumb/SHOEFY.jpg?1696518532"
  },
  {
    "chainId": 1,
    "address": "0x8dae6cb04688c62d939ed9b68d32bc62e49970b1",
    "name": "Aave CRV",
    "symbol": "ACRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17250/thumb/aCRV_2x.png?1696516806"
  },
  {
    "chainId": 1,
    "address": "0xde5ed76e7c05ec5e4572cfc88d1acea165109e44",
    "name": "DEUS Finance",
    "symbol": "DEUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18778/thumb/Black_Background_200x200.png?1696518242"
  },
  {
    "chainId": 1,
    "address": "0x618679df9efcd19694bb1daa8d00718eacfa2883",
    "name": "Universe XYZ",
    "symbol": "XYZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15809/thumb/universexyz.png?1696515429"
  },
  {
    "chainId": 1,
    "address": "0x69fa0fee221ad11012bab0fdb45d444d3d2ce71c",
    "name": "Thorstarter",
    "symbol": "XRUNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16835/thumb/thorstarter.jpg?1696516403"
  },
  {
    "chainId": 1,
    "address": "0x3b58c52c03ca5eb619eba171091c86c34d603e5f",
    "name": "Cyclub",
    "symbol": "CYCLUB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/12524/thumb/mcicoin-logo.png?1696512337"
  },
  {
    "chainId": 1,
    "address": "0xc382e04099a435439725bb40647e2b32dc136806",
    "name": "Cogecoin",
    "symbol": "COGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16791/thumb/Dog-Logo_1x.png?1696516362"
  },
  {
    "chainId": 1,
    "address": "0x692accdd8b86692427e0aa4752ae917df01cc56f",
    "name": "Sunrise",
    "symbol": "SUNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17601/thumb/ticker_sunc_200_200.png?1696517134"
  },
  {
    "chainId": 1,
    "address": "0xe76c6c83af64e4c60245d8c7de953df673a7a33d",
    "name": "Railgun",
    "symbol": "RAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16840/thumb/railgun.jpeg?1696516407"
  },
  {
    "chainId": 1,
    "address": "0x53bd789f2cdb846b227d8ffc7b46ed4263231fdf",
    "name": "SimbCoin Swap",
    "symbol": "SMBSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16808/thumb/smb.png?1696516378"
  },
  {
    "chainId": 1,
    "address": "0x0b63128c40737b13647552e0c926bcfeccc35f93",
    "name": "wLITI",
    "symbol": "WLITI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16810/thumb/liticapital_real.png?1696516380"
  },
  {
    "chainId": 1,
    "address": "0xe0a189c975e4928222978a74517442239a0b86ff",
    "name": "Keys",
    "symbol": "KEYS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/20604/thumb/200x200_%2843%29.png?1696520010"
  },
  {
    "chainId": 1,
    "address": "0x1da87b114f35e1dc91f72bf57fc07a768ad40bb0",
    "name": "Equalizer",
    "symbol": "EQZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14741/thumb/X2p5mb2f_400x400.png?1696514410"
  },
  {
    "chainId": 1,
    "address": "0x4691937a7508860f876c9c0a2a617e7d9e945d4b",
    "name": "WOO",
    "symbol": "WOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12921/thumb/WOO_Logos_2023_Profile_Pic_WOO.png?1696512709"
  },
  {
    "chainId": 1,
    "address": "0x3e9bc21c9b189c09df3ef1b824798658d5011937",
    "name": "Linear",
    "symbol": "LINA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12509/thumb/1649227343-linalogo200px.png?1696512324"
  },
  {
    "chainId": 1,
    "address": "0x155ff1a85f440ee0a382ea949f24ce4e0b751c65",
    "name": "Behodler",
    "symbol": "EYE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12804/thumb/etherscan-eye-2-1.png?1696512597"
  },
  {
    "chainId": 1,
    "address": "0xad996a45fd2373ed0b10efa4a8ecb9de445a4302",
    "name": "Shirtum",
    "symbol": "SHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16955/thumb/4fWlpC0.png?1696516523"
  },
  {
    "chainId": 1,
    "address": "0xf0c5831ec3da15f3696b4dad8b21c7ce2f007f28",
    "name": "AXIS",
    "symbol": "AXIS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/16972/thumb/lUeFYUeu_400x400.png?1696516538"
  },
  {
    "chainId": 1,
    "address": "0x269616d549d7e8eaa82dfb17028d0b212d11232a",
    "name": "Punk Vault  NFTX ",
    "symbol": "PUNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17018/thumb/Punk.png?1696516582"
  },
  {
    "chainId": 1,
    "address": "0xd779eea9936b4e323cddff2529eb6f13d0a4d66e",
    "name": "EnterDAO",
    "symbol": "ENTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18684/thumb/entr.png?1696518153"
  },
  {
    "chainId": 1,
    "address": "0x744d70fdbe2ba4cf95131626614a1763df805b9e",
    "name": "Status",
    "symbol": "SNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/779/thumb/status.png?1696501931"
  },
  {
    "chainId": 1,
    "address": "0xc31cebf8f9e825d1d1244d73d0a65e44bd5210db",
    "name": "CRYN",
    "symbol": "CRYN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/24601/thumb/cryn.png?1696523774"
  },
  {
    "chainId": 1,
    "address": "0xced6a1b885ee79899422d3a2f61fa9c77282c573",
    "name": "CodeGenie",
    "symbol": "CODEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36784/thumb/CMC_CG.png?1712337652"
  },
  {
    "chainId": 1,
    "address": "0x96eb50804d0ef2790f2e1a33670feff6040cf89d",
    "name": "Space Xmitter",
    "symbol": "SX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28335/thumb/X200.png?1696527341"
  },
  {
    "chainId": 1,
    "address": "0xceeb07dd26b36287b6d109f0b06d7e8202ce8c1d",
    "name": "Got Guaranteed",
    "symbol": "GOTG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26107/thumb/%EA%B0%93%EC%A7%80%EB%A1%9C%EA%B3%A0.png?1696525197"
  },
  {
    "chainId": 1,
    "address": "0x6b1a8f210ec6b7b6643cea3583fb0c079f367898",
    "name": "Baanx",
    "symbol": "BXX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17108/thumb/BXX_Token_logo.png?1696516669"
  },
  {
    "chainId": 1,
    "address": "0xee06a81a695750e71a662b51066f2c74cf4478a0",
    "name": "Decentral Games  Old ",
    "symbol": "DG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13267/thumb/%28Old%29_DG.png?1696513041"
  },
  {
    "chainId": 1,
    "address": "0xa2085073878152ac3090ea13d1e41bd69e60dc99",
    "name": "Escoin",
    "symbol": "ELG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13566/thumb/escoin-200.png?1696513320"
  },
  {
    "chainId": 1,
    "address": "0x4ece5c5cfb9b960a49aae739e15cdb6cfdcc5782",
    "name": "Doont Buy",
    "symbol": "DBUY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17895/thumb/VaxEiZ3.png?1696517415"
  },
  {
    "chainId": 1,
    "address": "0xc525abf65169e6d443f698276761eb9ea7d921a4",
    "name": "Bitnex AI",
    "symbol": "BTX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33649/thumb/logo-small.png?1702543587"
  },
  {
    "chainId": 1,
    "address": "0xfa99a87b14b02e2240c79240c5a20f945ca5ef76",
    "name": "GG",
    "symbol": "GGTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13666/thumb/ggtk.png?1696513416"
  },
  {
    "chainId": 1,
    "address": "0x72e5390edb7727e3d4e3436451dadaff675dbcc0",
    "name": "Hanu Yokia",
    "symbol": "HANU",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/17161/thumb/Goji_Hanu_Logo_200x200.png?1696516720"
  },
  {
    "chainId": 1,
    "address": "0x2365a4890ed8965e564b7e2d27c38ba67fec4c6f",
    "name": "Aave AMM UniWBTCUSDC",
    "symbol": "AAMMUNIWBTCUSDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17255/thumb/aAmmUniWBTCUSDC.png?1696516811"
  },
  {
    "chainId": 1,
    "address": "0xf256cc7847e919fac9b808cc216cac87ccf2f47a",
    "name": "Aave XSUSHI",
    "symbol": "AXSUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17256/thumb/aXSUSHI_2x.png?1696516812"
  },
  {
    "chainId": 1,
    "address": "0x5165d24277cd063f5ac44efd447b27025e888f37",
    "name": "Aave YFI",
    "symbol": "AYFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17257/thumb/aYFI_2x.png?1696516813"
  },
  {
    "chainId": 1,
    "address": "0xd6caf5bd23cf057f5fccce295dcc50c01c198707",
    "name": "Evanesco Network",
    "symbol": "EVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16722/thumb/evan.PNG?1696516296"
  },
  {
    "chainId": 1,
    "address": "0xa0bed124a09ac2bd941b10349d8d224fe3c955eb",
    "name": "DePay",
    "symbol": "DEPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13183/thumb/DEPAY.png?1696512965"
  },
  {
    "chainId": 1,
    "address": "0xf629cbd94d3791c9250152bd8dfbdf380e2a3b9c",
    "name": "Enjin Coin",
    "symbol": "ENJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1102/thumb/Symbol_Only_-_Purple.png?1709725966"
  },
  {
    "chainId": 1,
    "address": "0xfb130d93e49dca13264344966a611dc79a456bc5",
    "name": "DogeGF",
    "symbol": "DOGEGF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18651/thumb/dogegf_logo_200x200.png?1712082872"
  },
  {
    "chainId": 1,
    "address": "0x2c9aceb63181cd08a093d052ec041e191f229692",
    "name": "Angryb",
    "symbol": "ANB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17270/thumb/AngryBlockchain.png?1696516825"
  },
  {
    "chainId": 1,
    "address": "0x3bbbb6a231d0a1a12c6b79ba5bc2ed6358db5160",
    "name": "Zenith",
    "symbol": "ZEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35861/thumb/o7lSBgbb_400x400.jpg?1709921042"
  },
  {
    "chainId": 1,
    "address": "0x188e817b02e635d482ae4d81e25dda98a97c4a42",
    "name": "Lithium Finance",
    "symbol": "LITH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17278/thumb/Lithium_Logo-03200x200.png?1696516832"
  },
  {
    "chainId": 1,
    "address": "0xa1e770be76bde604f8ebb66f640250a787b9422b",
    "name": "Anduschain",
    "symbol": "DEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20842/thumb/DEB.png?1696520234"
  },
  {
    "chainId": 1,
    "address": "0x8c6bf16c273636523c29db7db04396143770f6a0",
    "name": "Moon Rabbit",
    "symbol": "AAA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17310/thumb/logo_moon_no_inscriptions-01.png?1696516863"
  },
  {
    "chainId": 1,
    "address": "0x226f7b842e0f0120b7e194d05432b3fd14773a9d",
    "name": "UNION Protocol Governance",
    "symbol": "UNN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13408/thumb/unn_finance.png?1696513169"
  },
  {
    "chainId": 1,
    "address": "0x9ab7bb7fdc60f4357ecfef43986818a2a3569c62",
    "name": "Guild of Guardians",
    "symbol": "GOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17362/thumb/V2QDNoLg_400x400.jpg?1696516913"
  },
  {
    "chainId": 1,
    "address": "0x3d9a4d8ab4f5bd1d5d08ae3a95e8ed8bb4d7e3b9",
    "name": "STONKSDAO",
    "symbol": "STONKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29976/thumb/Final_Logo_%281%29_%281%29.png?1696528901"
  },
  {
    "chainId": 1,
    "address": "0xcda4e840411c00a614ad9205caec807c7458a0e3",
    "name": "PureFi",
    "symbol": "UFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17341/thumb/purefi.PNG?1696516893"
  },
  {
    "chainId": 1,
    "address": "0x777e2ae845272a2f540ebf6a3d03734a5a8f618e",
    "name": "Ryoshis Vision",
    "symbol": "RYOSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17614/thumb/OFFICIAL_RYOSHI_LOGO-removebg-preview_2_200x200.png?1696517146"
  },
  {
    "chainId": 1,
    "address": "0x73b708e84837ffccde2933e3a1531fe61d5e80ef",
    "name": "Betero",
    "symbol": "BTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25153/thumb/B_green200.png?1696524302"
  },
  {
    "chainId": 1,
    "address": "0x94804dc4948184ffd7355f62ccbb221c9765886f",
    "name": "Rage Fan",
    "symbol": "RAGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14585/thumb/Copy_of_RAGE_FAN_LOGO_ONLY.png?1696514265"
  },
  {
    "chainId": 1,
    "address": "0x8db1d28ee0d822367af8d220c0dc7cb6fe9dc442",
    "name": "ETHPad",
    "symbol": "ETHPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17520/thumb/tHAbIBQK_400x400.jpg?1696517058"
  },
  {
    "chainId": 1,
    "address": "0xc4f6e93aeddc11dc22268488465babcaf09399ac",
    "name": "hi Dollar",
    "symbol": "HI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17618/thumb/unnamed_%281%29.png?1696517150"
  },
  {
    "chainId": 1,
    "address": "0x0ab902bd4de7521a565f3058fb43d6d01d0d2670",
    "name": "CogniTechAI",
    "symbol": "CTI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36672/thumb/dp.png?1712061998"
  },
  {
    "chainId": 1,
    "address": "0x7815bda662050d84718b988735218cffd32f75ea",
    "name": "Yel Finance",
    "symbol": "YEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17429/thumb/Logo200.png?1696516976"
  },
  {
    "chainId": 1,
    "address": "0xc0ba369c8db6eb3924965e5c4fd0b4c1b91e305f",
    "name": "DLP Duck",
    "symbol": "DUCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13440/thumb/DLP_Duck_Token.png?1696513204"
  },
  {
    "chainId": 1,
    "address": "0x3446dd70b2d52a6bf4a5a192d9b0a161295ab7f9",
    "name": "sudoswap",
    "symbol": "SUDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27151/thumb/sudo.png?1696526202"
  },
  {
    "chainId": 1,
    "address": "0xab306326bc72c2335bd08f42cbec383691ef8446",
    "name": "PPizza",
    "symbol": "PPIZZA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30063/thumb/iconArtboard_1_10x.png?1696528985"
  },
  {
    "chainId": 1,
    "address": "0x0ec9f76202a7061eb9b3a7d6b59d36215a7e37da",
    "name": "BlackPool",
    "symbol": "BPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15887/thumb/uyO7dQzR_400x400.jpg?1696515502"
  },
  {
    "chainId": 1,
    "address": "0x94e496474f1725f1c1824cb5bdb92d7691a4f03a",
    "name": "Banana",
    "symbol": "BANANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17521/thumb/banana-token-cg.png?1696517059"
  },
  {
    "chainId": 1,
    "address": "0xd81b71cbb89b2800cdb000aa277dc1491dc923c3",
    "name": "NFTMart",
    "symbol": "NMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15811/thumb/NFTMART.png?1696515430"
  },
  {
    "chainId": 1,
    "address": "0x9fa69536d1cda4a04cfb50688294de75b505a9ae",
    "name": "DeRace",
    "symbol": "DERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17438/thumb/DERC_logo_coingecko.png?1696516984"
  },
  {
    "chainId": 1,
    "address": "0x2be1e42bf263aab47d27ba92e72c14823e101d7c",
    "name": "Fluid FRAX",
    "symbol": "FFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28475/thumb/fFRAX-200x200.png?1696527469"
  },
  {
    "chainId": 1,
    "address": "0xe7ae6d0c56cacaf007b7e4d312f9af686a9e9a04",
    "name": "Vabble",
    "symbol": "VAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17508/thumb/vabble.png?1705376555"
  },
  {
    "chainId": 1,
    "address": "0xd85a6ae55a7f33b0ee113c234d2ee308edeaf7fd",
    "name": "Cobak",
    "symbol": "CBK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13459/thumb/cbk-128-128.png?1696513222"
  },
  {
    "chainId": 1,
    "address": "0x2e9d63788249371f1dfc918a52f8d799f4a38c94",
    "name": "Tokemak",
    "symbol": "TOKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17495/thumb/tokemak-avatar-200px-black.png?1696517036"
  },
  {
    "chainId": 1,
    "address": "0xcbe771323587ea16dacb6016e269d7f08a7acc4e",
    "name": "Spores Network",
    "symbol": "SPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17431/thumb/SPO_token.png?1696516978"
  },
  {
    "chainId": 1,
    "address": "0x42e70913b53cfcc38b18ffbd124e8f65c706deaf",
    "name": "SALAMANDER",
    "symbol": "SALLY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31826/thumb/Sally200x200.png?1696530640"
  },
  {
    "chainId": 1,
    "address": "0x4c11249814f11b9346808179cf06e71ac328c1b5",
    "name": "Oraichain",
    "symbol": "ORAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12931/thumb/orai.png?1696512718"
  },
  {
    "chainId": 1,
    "address": "0x009668a9691e456972c8ec4cc84e99486308b84d",
    "name": "Terareum  OLD ",
    "symbol": "TERA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23880/thumb/Tera-200x200-1.png?1696523080"
  },
  {
    "chainId": 1,
    "address": "0x6e765d26388a17a6e86c49a8e41df3f58abcd337",
    "name": "Kangal",
    "symbol": "KANGAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14241/thumb/logo-200.png?1696513956"
  },
  {
    "chainId": 1,
    "address": "0x70bef3bb2f001da2fddb207dae696cd9faff3f5d",
    "name": "Ninja Squad",
    "symbol": "NST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22248/thumb/nst-logo-200x200.png?1711559985"
  },
  {
    "chainId": 1,
    "address": "0xb9d4b6dc1e1ee3577cc442de015cc11f238b35ed",
    "name": "Magnum",
    "symbol": "MAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33531/thumb/20231213_112204.jpg?1702499402"
  },
  {
    "chainId": 1,
    "address": "0x6286a9e6f7e745a6d884561d88f94542d6715698",
    "name": "Cryptomeda",
    "symbol": "TECH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17983/thumb/tech.png?1696517501"
  },
  {
    "chainId": 1,
    "address": "0x65def5029a0e7591e46b38742bfedd1fb7b24436",
    "name": "Kanpeki",
    "symbol": "KAE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18833/thumb/kanpeki-logo-square_2.png?1696518294"
  },
  {
    "chainId": 1,
    "address": "0xbeab712832112bd7664226db7cd025b153d3af55",
    "name": "Bright Union",
    "symbol": "BRIGHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17552/thumb/BrightToken_Token-only_200x200-1.png?1696517088"
  },
  {
    "chainId": 1,
    "address": "0x0e192d382a36de7011f795acc4391cd302003606",
    "name": "Futureswap",
    "symbol": "FST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14520/thumb/futureswap_logo.png?1696514206"
  },
  {
    "chainId": 1,
    "address": "0xc668695dcbcf682de106da94bde65c9bc79362d3",
    "name": "Shadow Node",
    "symbol": "SVPN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36124/thumb/Group_13127_copy_%28200p%29.png?1710502856"
  },
  {
    "chainId": 1,
    "address": "0x1559fa1b8f28238fd5d76d9f434ad86fd20d1559",
    "name": "EDEN",
    "symbol": "EDEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17470/thumb/Iyc0XM5-_400x400.jpg?1696517012"
  },
  {
    "chainId": 1,
    "address": "0x4c2e59d098df7b6cbae0848d66de2f8a4889b9c3",
    "name": "Fodl Finance",
    "symbol": "FODL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19040/thumb/Fodl_Symbol_Gradient.png?1696518491"
  },
  {
    "chainId": 1,
    "address": "0x62959c699a52ec647622c91e79ce73344e4099f5",
    "name": "DeFine",
    "symbol": "DFA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17579/thumb/define.PNG?1696517113"
  },
  {
    "chainId": 1,
    "address": "0x090185f2135308bad17527004364ebcc2d37e5f6",
    "name": "Spell",
    "symbol": "SPELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15861/thumb/abracadabra-3.png?1696515477"
  },
  {
    "chainId": 1,
    "address": "0x4463e6a3ded0dbe3f6e15bc8420dfc55e5fea830",
    "name": "TXA",
    "symbol": "TXA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17746/thumb/jwjcN5Kx_400x400.jpg?1696517273"
  },
  {
    "chainId": 1,
    "address": "0x8ce9137d39326ad0cd6491fb5cc0cba0e089b6a9",
    "name": "SXP",
    "symbol": "SXP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9368/thumb/swipe.png?1696509466"
  },
  {
    "chainId": 1,
    "address": "0xe33ae4e795114279721047484e5ad5cc7df24fcb",
    "name": "MCFinance",
    "symbol": "MCF",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/17608/thumb/arFRQ0b.png?1696517140"
  },
  {
    "chainId": 1,
    "address": "0x9992ec3cf6a55b00978cddf2b27bc6882d88d1ec",
    "name": "Polymath",
    "symbol": "POLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2784/thumb/inKkF01.png?1696503553"
  },
  {
    "chainId": 1,
    "address": "0x2a9bdcff37ab68b95a53435adfd8892e86084f93",
    "name": "Alpha Quark",
    "symbol": "AQT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12872/thumb/alpha_quark_logo.png?1696512660"
  },
  {
    "chainId": 1,
    "address": "0x8727c112c712c4a03371ac87a74dd6ab104af768",
    "name": "Jetcoin",
    "symbol": "JET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2487/thumb/jetcoin.png?1696503306"
  },
  {
    "chainId": 1,
    "address": "0xe94b97b6b43639e238c851a7e693f50033efd75c",
    "name": "HaloDAO",
    "symbol": "RNBW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16754/thumb/RNBW-256x256.png?1696516327"
  },
  {
    "chainId": 1,
    "address": "0xf203ca1769ca8e9e8fe1da9d147db68b6c919817",
    "name": "Wrapped NCG",
    "symbol": "WNCG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17747/thumb/WNCG_Color.png?1696517274"
  },
  {
    "chainId": 1,
    "address": "0xa735a3af76cc30791c61c10d585833829d36cbe0",
    "name": "Image Generation AI",
    "symbol": "IMGNAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28666/thumb/imgnai-200x200.png?1698476572"
  },
  {
    "chainId": 1,
    "address": "0xa11bd36801d8fa4448f0ac4ea7a62e3634ce8c7c",
    "name": "Allbridge",
    "symbol": "ABR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18690/thumb/logo.png?1701737985"
  },
  {
    "chainId": 1,
    "address": "0xd4ae236a5080a09c0f7bd6e6b84919523573a43b",
    "name": "Future T I M E Dividend",
    "symbol": "FUTURE",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x302cae5dcf8f051d0177043c3438020b89b33218",
    "name": "Boost",
    "symbol": "BOOST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17657/thumb/02FjG1Og.png?1696517188"
  },
  {
    "chainId": 1,
    "address": "0x012e0e6342308b247f36ee500ecb14dc77a7a8c1",
    "name": "Sukhavati Network",
    "symbol": "SKT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17674/thumb/logo2.png?1696517204"
  },
  {
    "chainId": 1,
    "address": "0x72377f31e30a405282b522d588aebbea202b4f23",
    "name": "Varen",
    "symbol": "VRN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17633/thumb/vrn.png?1696517164"
  },
  {
    "chainId": 1,
    "address": "0xaec65404ddc3af3c897ad89571d5772c1a695f22",
    "name": "Phoenix Finance",
    "symbol": "PHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17675/thumb/phx_logo.png?1696517205"
  },
  {
    "chainId": 1,
    "address": "0xd528cf2e081f72908e086f8800977df826b5a483",
    "name": "Paribus",
    "symbol": "PBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18410/thumb/paribus.PNG?1696517900"
  },
  {
    "chainId": 1,
    "address": "0xbd9908b0cdd50386f92efcc8e1d71766c2782df0",
    "name": "DAOSquare",
    "symbol": "RICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19084/thumb/tw5gu1dW_400x400.jpeg?1696518537"
  },
  {
    "chainId": 1,
    "address": "0xa150db9b1fa65b44799d4dd949d922c0a33ee606",
    "name": "Digital Reserve Currency",
    "symbol": "DRC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/12802/thumb/DRC_Logo.jpg?1696512595"
  },
  {
    "chainId": 1,
    "address": "0xd6327ce1fb9d6020e8c2c0e124a1ec23dcab7536",
    "name": "Cuminu",
    "symbol": "CUMINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15832/thumb/CUMINU.png?1696515450"
  },
  {
    "chainId": 1,
    "address": "0xe362a9415d8c332a78ba6734620f1afc19099bbc",
    "name": "Send Finance",
    "symbol": "SEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33639/thumb/IMG_20231213_084952_753.jpg?1702537949"
  },
  {
    "chainId": 1,
    "address": "0xdfdb7f72c1f195c5951a234e8db9806eb0635346",
    "name": "Feisty Doge NFT",
    "symbol": "NFD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17834/thumb/doge-fractionalized.png?1696517355"
  },
  {
    "chainId": 1,
    "address": "0x55986c1dcbf2fa1f3402c5df256272621624821f",
    "name": "Bank BTC",
    "symbol": "BANKBTC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17827/thumb/logo_200x200.png?1696517349"
  },
  {
    "chainId": 1,
    "address": "0xcfcecfe2bd2fed07a9145222e8a7ad9cf1ccd22a",
    "name": "Adshares",
    "symbol": "ADS",
    "decimals": 11,
    "logoURI": "https://assets.coingecko.com/coins/images/868/thumb/rnO9DyJ.png?1696502001"
  },
  {
    "chainId": 1,
    "address": "0x3757232b55e60da4a8793183ac030cfce4c3865d",
    "name": "YDragon",
    "symbol": "YDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17807/thumb/icon.png?1696517328"
  },
  {
    "chainId": 1,
    "address": "0x8a0cdfab62ed35b836dc0633482798421c81b3ec",
    "name": "Spherium",
    "symbol": "SPHRI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17787/thumb/Group_15.png?1696517311"
  },
  {
    "chainId": 1,
    "address": "0x9d71ce49ab8a0e6d2a1e7bfb89374c9392fd6804",
    "name": "NvirWorld",
    "symbol": "NVIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21359/thumb/Nvir_symbol.png?1696520724"
  },
  {
    "chainId": 1,
    "address": "0xa01199c61841fce3b3dafb83fefc1899715c8756",
    "name": "Cirus",
    "symbol": "CIRUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17798/thumb/8p0Bvi90_400x400.jpg?1696517320"
  },
  {
    "chainId": 1,
    "address": "0x4c5cb5d87709387f8821709f7a6664f00dcf0c93",
    "name": "Raft",
    "symbol": "RAFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30915/thumb/Logomark_Transparent_Background_200x200.png?1696529760"
  },
  {
    "chainId": 1,
    "address": "0x06a00715e6f92210af9d7680b584931faf71a833",
    "name": "Chronicle",
    "symbol": "XNL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18413/thumb/xnl_logo.png?1696517903"
  },
  {
    "chainId": 1,
    "address": "0x0cf0ee63788a0849fe5297f3407f701e122cc023",
    "name": "Streamr XDATA",
    "symbol": "XDATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1115/thumb/streamr.png?1696502210"
  },
  {
    "chainId": 1,
    "address": "0x61107a409fffe1965126aa456af679719695c69c",
    "name": "Umi Digital",
    "symbol": "UMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17874/thumb/Umi-200-x-200.png?1696517396"
  },
  {
    "chainId": 1,
    "address": "0x24ec2ca132abf8f6f8a6e24a1b97943e31f256a7",
    "name": "dotmoovs",
    "symbol": "MOOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15817/thumb/dotmoovs-symbol-gradient.png?1696515436"
  },
  {
    "chainId": 1,
    "address": "0xea7cc765ebc94c4805e3bff28d7e4ae48d06468a",
    "name": "Pad Fi",
    "symbol": "PAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18117/thumb/pad.fi.png?1696517620"
  },
  {
    "chainId": 1,
    "address": "0xc0ae17eb994fa828540ffa53776b3830233a1b02",
    "name": "Element Black",
    "symbol": "ELT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25160/thumb/4TCtUKHs_400x400.jpg?1696524307"
  },
  {
    "chainId": 1,
    "address": "0x1f19f83fc9a25f3c861260143e36c17706257986",
    "name": "DAO Invest",
    "symbol": "VEST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17901/thumb/logo-round-200.png?1696517421"
  },
  {
    "chainId": 1,
    "address": "0x4da0c48376c277cdbd7fc6fdc6936dee3e4adf75",
    "name": "Epik Prime",
    "symbol": "EPIK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17907/thumb/EPIK_Prime_LOGO.jpg?1696517427"
  },
  {
    "chainId": 1,
    "address": "0x665f4709940f557e9dde63df0fd9cf6425852b4d",
    "name": "Halving",
    "symbol": "HALVING",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35732/thumb/halving_%284%29.png?1709691354"
  },
  {
    "chainId": 1,
    "address": "0x1f4cb968b76931c494ff92ed80ccb169ad641cb1",
    "name": "Structure Finance",
    "symbol": "STF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18409/thumb/v4RAXKdq_400x400_%281%29.jpg?1696517899"
  },
  {
    "chainId": 1,
    "address": "0x5e67e49a95b8ccc7a46b68bcdc1c35dc799ef9e0",
    "name": "DigiFund V1",
    "symbol": "DFUND",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32951/thumb/photo_2023-11-08_13-27-36.jpg?1699935822"
  },
  {
    "chainId": 1,
    "address": "0xdb5c3c46e28b53a39c255aa39a411dd64e5fed9c",
    "name": "Neos Credits",
    "symbol": "NCR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17935/thumb/s4C4tVi.png?1696517456"
  },
  {
    "chainId": 1,
    "address": "0x5dd57da40e6866c9fcc34f4b6ddc89f1ba740dfe",
    "name": "BrightID",
    "symbol": "BRIGHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18415/thumb/bright.PNG?1696517905"
  },
  {
    "chainId": 1,
    "address": "0xc8ef1460277ea47d179dec66d1c5f8b7f7ae5a28",
    "name": "Rikkei Finance",
    "symbol": "RIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21309/thumb/rikkei-finance.jpeg?1696520678"
  },
  {
    "chainId": 1,
    "address": "0x5f0bc16d50f72d10b719dbf6845de2e599eb5624",
    "name": "Vent Finance",
    "symbol": "VENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17925/thumb/Artboard_29.png?1696517445"
  },
  {
    "chainId": 1,
    "address": "0x8e870d67f660d95d5be530380d0ec0bd388289e1",
    "name": "Pax Dollar",
    "symbol": "USDP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6013/thumb/Pax_Dollar.png?1696506427"
  },
  {
    "chainId": 1,
    "address": "0x734c90044a0ba31b3f2e640c10dc5d3540499bfd",
    "name": "TradeStars",
    "symbol": "TSX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15229/thumb/WsO9siKG_400x400.png?1696514884"
  },
  {
    "chainId": 1,
    "address": "0x9cf98eb8a8b28c83e8612046cf55701ce3eb0063",
    "name": "Unreal Finance",
    "symbol": "UGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17943/thumb/11292.png?1696517464"
  },
  {
    "chainId": 1,
    "address": "0x2610f0bfc21ef389fe4d03cfb7de9ac1e6c99d6e",
    "name": "Skyrim Finance",
    "symbol": "SKYRIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18003/thumb/Xo3oCb53_400x400.png?1696517519"
  },
  {
    "chainId": 1,
    "address": "0x1e4e46b7bf03ece908c88ff7cc4975560010893a",
    "name": "Internet of Energy Network",
    "symbol": "IOEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19095/thumb/12799.png?1696518548"
  },
  {
    "chainId": 1,
    "address": "0x15b7c0c907e4c6b9adaaaabc300c08991d6cea05",
    "name": "Gelato",
    "symbol": "GEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15026/thumb/Gelato_Icon_Logo_1024x1024.png?1696514687"
  },
  {
    "chainId": 1,
    "address": "0xae697f994fc5ebc000f8e22ebffee04612f98a0d",
    "name": "LGCY Network",
    "symbol": "LGCY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12181/thumb/LGCY_network.jpg?1696512018"
  },
  {
    "chainId": 1,
    "address": "0x1bbf25e71ec48b84d773809b4ba55b6f4be946fb",
    "name": "Vow",
    "symbol": "VOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18202/thumb/72Nd63R0_400x400.png?1696517701"
  },
  {
    "chainId": 1,
    "address": "0xe74be071f3b62f6a4ac23ca68e5e2a39797a3c30",
    "name": "Recharge",
    "symbol": "RCG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18120/thumb/thecharge.PNG?1696517623"
  },
  {
    "chainId": 1,
    "address": "0x4dd28568d05f09b02220b09c2cb307bfd837cb95",
    "name": "FingerprintsDAO",
    "symbol": "PRINTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18143/thumb/prints.png?1696517646"
  },
  {
    "chainId": 1,
    "address": "0xe7f72bc0252ca7b16dbb72eeee1afcdb2429f2dd",
    "name": "NFTLaunch",
    "symbol": "NFTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18140/thumb/nftl.PNG?1696517643"
  },
  {
    "chainId": 1,
    "address": "0x34f797e7190c131cf630524655a618b5bd8738e7",
    "name": "BaconDAO",
    "symbol": "BACON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18059/thumb/xDV_bhdA_400x400.jpg?1696517567"
  },
  {
    "chainId": 1,
    "address": "0xf009f5531de69067435e32c4b9d36077f4c4a673",
    "name": "Unvest",
    "symbol": "UNV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18119/thumb/UNV.jpg?1696517622"
  },
  {
    "chainId": 1,
    "address": "0x799ebfabe77a6e34311eeee9825190b9ece32824",
    "name": "Braintrust",
    "symbol": "BTRST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18100/thumb/braintrust.PNG?1696517605"
  },
  {
    "chainId": 1,
    "address": "0xc6dddb5bc6e61e0841c54f3e723ae1f3a807260b",
    "name": "Aurox",
    "symbol": "URUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14122/thumb/Aurox.png?1696513842"
  },
  {
    "chainId": 1,
    "address": "0x60f63b76e2fc1649e57a3489162732a90acf59fe",
    "name": "Flurry Finance",
    "symbol": "FLURRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16235/thumb/flurry_logo_only_200x200.png?1696515836"
  },
  {
    "chainId": 1,
    "address": "0x7ae1d57b58fa6411f32948314badd83583ee0e8c",
    "name": "Dope Wars Paper",
    "symbol": "PAPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18166/thumb/EQHGcBO__400x400.jpeg?1696517667"
  },
  {
    "chainId": 1,
    "address": "0x39d30828a163713d91c4eadbba2c497a9139ec5c",
    "name": "Happy Birthday Coin",
    "symbol": "HBDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12814/thumb/%EC%A7%84%ED%92%88.png?1696512607"
  },
  {
    "chainId": 1,
    "address": "0x356a5160f2b34bc8d88fb084745465ebbbed0174",
    "name": "INVI",
    "symbol": "INVI",
    "decimals": 13,
    "logoURI": "https://assets.coingecko.com/coins/images/17363/thumb/INVI-Ticker.png?1696516914"
  },
  {
    "chainId": 1,
    "address": "0x42a501903afaa1086b5975773375c80e363f4063",
    "name": "Cryptyk",
    "symbol": "CTK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2729/thumb/th.jpg?1696503507"
  },
  {
    "chainId": 1,
    "address": "0xbb3d7f42c58abd83616ad7c8c72473ee46df2678",
    "name": "VectorChat ai",
    "symbol": "CHAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33969/thumb/gecko.png?1703576146"
  },
  {
    "chainId": 1,
    "address": "0x5516ac1aaca7bb2fd5b7bdde1549ef1ea242953d",
    "name": "Decentralized ETF",
    "symbol": "DETF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24232/thumb/DETF.png?1697123799"
  },
  {
    "chainId": 1,
    "address": "0xc6bdb96e29c38dc43f014eed44de4106a6a8eb5f",
    "name": "Inu Inu",
    "symbol": "INUINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26391/thumb/logo_square_200.png?1696525468"
  },
  {
    "chainId": 1,
    "address": "0xc73c167e7a4ba109e4052f70d5466d0c312a344d",
    "name": "Sanshu Inu  OLD ",
    "symbol": "SANSHU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15370/thumb/m9DgRlXY.jpg?1696515018"
  },
  {
    "chainId": 1,
    "address": "0x794baab6b878467f93ef17e2f2851ce04e3e34c8",
    "name": "YIN Finance",
    "symbol": "YIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18272/thumb/e37387ae6ee756fd.jpg?1696517766"
  },
  {
    "chainId": 1,
    "address": "0x37ac5730cfb1549715647828a05b3a8ed781134b",
    "name": "AiFi Protocol",
    "symbol": "AIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33642/thumb/image_%282%29.png?1702540762"
  },
  {
    "chainId": 1,
    "address": "0x87a92428bbc876d463c21c8e51b903f127d9a9f4",
    "name": "Advanced United Continent",
    "symbol": "AUC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24369/thumb/200x200.png?1696523553"
  },
  {
    "chainId": 1,
    "address": "0xb26c4b3ca601136daf98593feaeff9e0ca702a8d",
    "name": "Aladdin DAO",
    "symbol": "ALD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18277/thumb/78200839.png?1696517771"
  },
  {
    "chainId": 1,
    "address": "0x131157c6760f78f7ddf877c0019eba175ba4b6f6",
    "name": "BigShortBets",
    "symbol": "BIGSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18235/thumb/bigsb-token-logo.png?1696517731"
  },
  {
    "chainId": 1,
    "address": "0x0026dfbd8dbb6f8d0c88303cc1b1596409fda542",
    "name": "SANSHU ",
    "symbol": "SANSHU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35739/thumb/blobid0.png?1710233593"
  },
  {
    "chainId": 1,
    "address": "0xd55210bb6898c021a19de1f58d27b71f095921ee",
    "name": "Chickencoin",
    "symbol": "CHKN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36684/thumb/gekologo.jpg?1712070948"
  },
  {
    "chainId": 1,
    "address": "0x9783b81438c24848f85848f8df31845097341771",
    "name": "Dog Collar",
    "symbol": "COLLAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18324/thumb/dcLogo.png?1696517815"
  },
  {
    "chainId": 1,
    "address": "0xee9e5eff401ee921b138490d00ca8d1f13f67a72",
    "name": "Asian Fintech",
    "symbol": "AFIN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7519/thumb/Afinlogo-200x200.png?1696507786"
  },
  {
    "chainId": 1,
    "address": "0xea26c4ac16d4a5a106820bc8aee85fd0b7b2b664",
    "name": "QuarkChain",
    "symbol": "QKC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3849/thumb/quarkchain.png?1696504513"
  },
  {
    "chainId": 1,
    "address": "0xc52fafdc900cb92ae01e6e4f8979af7f436e2eb2",
    "name": "Settled EthXY Token",
    "symbol": "SEXY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33032/thumb/logo-circle.png?1700348518"
  },
  {
    "chainId": 1,
    "address": "0xfca89d55a768375ab7ca04485a35a964bea828dd",
    "name": "Delrey Inu",
    "symbol": "DELREY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29661/thumb/Delrey_Inu.png?1696528596"
  },
  {
    "chainId": 1,
    "address": "0xaada04204e9e1099daf67cf3d5d137e84e41cf41",
    "name": "Peepo",
    "symbol": "PEEPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30006/thumb/20230425_175833.png?1696528931"
  },
  {
    "chainId": 1,
    "address": "0x232fb065d9d24c34708eedbf03724f2e95abe768",
    "name": "Sheesha Finance  ERC20 ",
    "symbol": "SHEESHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23053/thumb/MLBmh4z0.png?1696522345"
  },
  {
    "chainId": 1,
    "address": "0x51cb253744189f11241becb29bedd3f1b5384fdb",
    "name": "Dimitra",
    "symbol": "DMTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18530/thumb/HqEiru32_400x400.jpg?1696518010"
  },
  {
    "chainId": 1,
    "address": "0x5ca9a71b1d01849c0a95490cc00559717fcf0d1d",
    "name": "Aeternity",
    "symbol": "AE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1091/thumb/aeternity.png?1696502190"
  },
  {
    "chainId": 1,
    "address": "0xd76050f75627e508fa14b84036fbf40b8cc549bd",
    "name": "SCRIV",
    "symbol": "SCRIV",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/5684/thumb/Al7TZrAN_400x400.jpg?1696506143"
  },
  {
    "chainId": 1,
    "address": "0xdb298285fe4c5410b05390ca80e8fbe9de1f259b",
    "name": "handle fi",
    "symbol": "FOREX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18533/thumb/handle.fiFOREXLogoDark200x200.png?1696518013"
  },
  {
    "chainId": 1,
    "address": "0x80d55c03180349fff4a229102f62328220a96444",
    "name": "Opulous",
    "symbol": "OPUL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16548/thumb/opulous.PNG?1696516110"
  },
  {
    "chainId": 1,
    "address": "0x28b5e12cce51f15594b0b91d5b5adaa70f684a02",
    "name": "Napoleon X",
    "symbol": "NPX",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/1471/thumb/napoleon-x.jpg?1696502517"
  },
  {
    "chainId": 1,
    "address": "0x866f8a50a64e68ca66e97e032c5da99538b3f942",
    "name": "eBlockStock",
    "symbol": "EBSO",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/18636/thumb/eBSO_logo_szines.png?1696518108"
  },
  {
    "chainId": 1,
    "address": "0x2ed2cc2c858a8a8219fd2f2d9e170285dbd02756",
    "name": "Sports Bet",
    "symbol": "SBET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25312/thumb/sbet_logo_256.png?1696524449"
  },
  {
    "chainId": 1,
    "address": "0x1410434b0346f5be678d0fb554e5c7ab620f8f4a",
    "name": "BitKan",
    "symbol": "KAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4651/thumb/kan-token.png?1696505219"
  },
  {
    "chainId": 1,
    "address": "0x4da08a1bff50be96bded5c7019227164b49c2bfc",
    "name": "Mononoke Inu",
    "symbol": "MONONOKE-INU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/18769/thumb/z9YDK0f.png?1696518234"
  },
  {
    "chainId": 1,
    "address": "0x922e2708462c7a3d014d8344f7c4d92b27ecf332",
    "name": "Neuroni AI",
    "symbol": "NEURONI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28887/thumb/200X200.png?1696527864"
  },
  {
    "chainId": 1,
    "address": "0xfb782396c9b20e564a64896181c7ac8d8979d5f4",
    "name": "Divergence Protocol",
    "symbol": "DIVER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18666/thumb/DIVER.jpg?1696518136"
  },
  {
    "chainId": 1,
    "address": "0x765f0c16d1ddc279295c1a7c24b0883f62d33f75",
    "name": "DaTa eXchange DTX",
    "symbol": "DTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2920/thumb/dtx.png?1696503667"
  },
  {
    "chainId": 1,
    "address": "0x58f9102bf53cf186682bd9a281d3cd3c616eec41",
    "name": "Triall",
    "symbol": "TRL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18679/thumb/-B7ftfN8_400x400.png?1696518148"
  },
  {
    "chainId": 1,
    "address": "0x16cda4028e9e872a38acb903176719299beaed87",
    "name": "MARS4",
    "symbol": "MARS4",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18709/thumb/mars4_logo.jpg?1696518177"
  },
  {
    "chainId": 1,
    "address": "0xbe1a001fe942f96eea22ba08783140b9dcc09d28",
    "name": "Beta Finance",
    "symbol": "BETA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18715/thumb/beta_finance.jpg?1696518183"
  },
  {
    "chainId": 1,
    "address": "0x96610186f3ab8d73ebee1cf950c750f3b1fb79c2",
    "name": "Enjinstarter",
    "symbol": "EJS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18732/thumb/l-R1nYA0_400x400.jpg?1696518198"
  },
  {
    "chainId": 1,
    "address": "0xe03b2642a5111ad0efc0cbce766498c2dd562ae9",
    "name": "Old Bitcoin",
    "symbol": "BC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28832/thumb/bitcoinlogo.png?1696527808"
  },
  {
    "chainId": 1,
    "address": "0x4674a4f24c5f63d53f22490fb3a08eaaad739ff8",
    "name": "Brokoli",
    "symbol": "BRKL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18763/thumb/brkl.png?1696518228"
  },
  {
    "chainId": 1,
    "address": "0x8eef5a82e6aa222a60f009ac18c24ee12dbf4b41",
    "name": "Autobahn Network",
    "symbol": "TXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12432/thumb/txl.png?1696512252"
  },
  {
    "chainId": 1,
    "address": "0x536381a8628dbcc8c70ac9a30a7258442eab4c92",
    "name": "Pantos",
    "symbol": "PAN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9639/thumb/Pantos-logo-compact.png?1696509709"
  },
  {
    "chainId": 1,
    "address": "0x122303734c898e9d233affc234271f04e42e77ad",
    "name": "Maxwell the spinning cat",
    "symbol": "CAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35859/thumb/maxwell-cg.png?1709896910"
  },
  {
    "chainId": 1,
    "address": "0x1cc7047e15825f639e0752eb1b89e4225f5327f2",
    "name": "Pullix",
    "symbol": "PLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34319/thumb/IMG-20231231-WA0003.jpg?1704625607"
  },
  {
    "chainId": 1,
    "address": "0xd7c302fc3ac829c7e896a32c4bd126f3e8bd0a1f",
    "name": "Bit2Me",
    "symbol": "B2M",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19848/thumb/b2m-circle-solid-default.png?1696519271"
  },
  {
    "chainId": 1,
    "address": "0xdf290b162a7d3e0a328cf198308d421954f08b94",
    "name": "Beyond Protocol",
    "symbol": "BP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4732/thumb/b-logo-200x200.png?1696505293"
  },
  {
    "chainId": 1,
    "address": "0xb31ef9e52d94d4120eb44fe1ddfde5b4654a6515",
    "name": "DOSE",
    "symbol": "DOSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18847/thumb/dose.PNG?1696518308"
  },
  {
    "chainId": 1,
    "address": "0x8947da500eb47f82df21143d0c01a29862a8c3c5",
    "name": "Thales",
    "symbol": "THALES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18388/thumb/CLVZJN_C_400x400.png?1696517879"
  },
  {
    "chainId": 1,
    "address": "0xa3d59c6d24f428dcfedc09724869e7af4d281fdd",
    "name": "DeviantsFactions",
    "symbol": "DEV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32561/thumb/icon.png?1698546192"
  },
  {
    "chainId": 1,
    "address": "0x3625b55f653f8c7c884ac668696881df43631d44",
    "name": "Arti Project",
    "symbol": "ARTI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/18878/thumb/arti.PNG?1696518337"
  },
  {
    "chainId": 1,
    "address": "0x524d524b4c9366be706d3a90dcf70076ca037ae3",
    "name": "RMRK",
    "symbol": "RMRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18656/thumb/download_%281%29_%281%29.png?1696518126"
  },
  {
    "chainId": 1,
    "address": "0x52a047ee205701895ee06a375492490ec9c597ce",
    "name": "PulseMarkets",
    "symbol": "PULSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19011/thumb/photo_2021-11-11_06-24-09.jpg?1696518463"
  },
  {
    "chainId": 1,
    "address": "0x94dd57da120c7c564d9aa438773068705e828781",
    "name": "Journey",
    "symbol": "JRNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33707/thumb/A541vx-O_400x400.jpg?1702854357"
  },
  {
    "chainId": 1,
    "address": "0x785c34312dfa6b74f6f1829f79ade39042222168",
    "name": "Bumper",
    "symbol": "BUMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17822/thumb/Bumper-Icon-Inverse-200.png?1696517342"
  },
  {
    "chainId": 1,
    "address": "0x333a4823466879eef910a04d473505da62142069",
    "name": "Nation3",
    "symbol": "NATION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25106/thumb/orb-white-256.png?1709559789"
  },
  {
    "chainId": 1,
    "address": "0x1337def16f9b486faed0293eb623dc8395dfe46a",
    "name": "ARMOR",
    "symbol": "ARMOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13748/thumb/armor.png?1696513491"
  },
  {
    "chainId": 1,
    "address": "0xeb4c2781e4eba804ce9a9803c67d0893436bb27d",
    "name": "renBTC",
    "symbol": "RENBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11370/thumb/Bitcoin.jpg?1696511287"
  },
  {
    "chainId": 1,
    "address": "0x0aa7efe4945db24d95ca6e117bba65ed326e291a",
    "name": "Ojamu",
    "symbol": "OJA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18947/thumb/ojamu-icon-PNK.png?1696518402"
  },
  {
    "chainId": 1,
    "address": "0xd7dcd9b99787c619b4d57979521258d1a7267ad7",
    "name": "Evrynet",
    "symbol": "EVRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19194/thumb/evry.png?1696518642"
  },
  {
    "chainId": 1,
    "address": "0x23d894fb4a0f551f2f923fc85e09819d1f3894b2",
    "name": "Intellix",
    "symbol": "ITX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34406/thumb/itx.jpeg?1704791991"
  },
  {
    "chainId": 1,
    "address": "0x0f2d719407fdbeff09d87557abb7232601fd9f29",
    "name": "Synapse",
    "symbol": "SYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18024/thumb/synapse_social_icon.png?1696517540"
  },
  {
    "chainId": 1,
    "address": "0x3a1311b8c404629e38f61d566cefefed083b9670",
    "name": "Piccolo Inu",
    "symbol": "PINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19335/thumb/Piccolo-Inu.png?1696518776"
  },
  {
    "chainId": 1,
    "address": "0x549020a9cb845220d66d3e9c6d9f9ef61c981102",
    "name": "Sidus",
    "symbol": "SIDUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21401/thumb/SIDUS_coin_logo.png?1696520765"
  },
  {
    "chainId": 1,
    "address": "0xe7f58a92476056627f9fdb92286778abd83b285f",
    "name": "DecentraWeb",
    "symbol": "DWEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18971/thumb/dweb-logo-transparent.png?1696518425"
  },
  {
    "chainId": 1,
    "address": "0x71ab77b7dbb4fa7e017bc15090b2163221420282",
    "name": "Highstreet",
    "symbol": "HIGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18973/thumb/logosq200200Coingecko.png?1696518427"
  },
  {
    "chainId": 1,
    "address": "0xc30769d445d1cbbc5f85fcdb4294b827bd3e7dda",
    "name": "MissionMars",
    "symbol": "MMARS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33942/thumb/image_2023-12-21_15-40-16__2_-removebg-preview.png?1703509049"
  },
  {
    "chainId": 1,
    "address": "0x064797ac7f833d01faeeae0e69f3af5a52a91fc8",
    "name": "Smol Su",
    "symbol": "SU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30179/thumb/SmolSu_%281%29.jpg?1696529098"
  },
  {
    "chainId": 1,
    "address": "0x02d3a27ac3f55d5d91fb0f52759842696a864217",
    "name": "Charged Particles",
    "symbol": "IONX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15836/thumb/DrKjSQMH_400x400.png?1696515454"
  },
  {
    "chainId": 1,
    "address": "0x2162f572b25f7358db9376ab58a947a4e45cede1",
    "name": "LABEL Foundation",
    "symbol": "LBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19202/thumb/tele_profile_%EB%8C%80%EC%A7%80_1_-_%282%29.png?1696518649"
  },
  {
    "chainId": 1,
    "address": "0x60e683c6514edd5f758a55b6f393bebbafaa8d5e",
    "name": "Page",
    "symbol": "PAGE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/19015/thumb/Page_Icon_LG.png?1696518467"
  },
  {
    "chainId": 1,
    "address": "0x9d65ff81a3c488d585bbfb0bfe3c7707c7917f54",
    "name": "SSV Network",
    "symbol": "SSV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19155/thumb/ssv.png?1696518606"
  },
  {
    "chainId": 1,
    "address": "0xf9fbe825bfb2bf3e387af0dc18cac8d87f29dea8",
    "name": "Radar",
    "symbol": "RADAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13909/thumb/RADAR.png?1696513651"
  },
  {
    "chainId": 1,
    "address": "0x38d9eb07a7b8df7d86f440a4a5c4a4c1a27e1a08",
    "name": "bloXmove",
    "symbol": "BLXM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19310/thumb/blxm_200x200.png?1696518752"
  },
  {
    "chainId": 1,
    "address": "0xf3ae5d769e153ef72b4e3591ac004e89f48107a1",
    "name": "Deeper Network",
    "symbol": "DPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14748/thumb/deeper.png?1696514418"
  },
  {
    "chainId": 1,
    "address": "0xe4dae00bc1c46ea2f44ae71b1beb8b171c15d812",
    "name": "PREMA",
    "symbol": "PRMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27600/thumb/prmx.png?1696526631"
  },
  {
    "chainId": 1,
    "address": "0x910c4da718caf4ee38ce5c2490cddaeca689204e",
    "name": "Waste Digital Coin",
    "symbol": "WACO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19035/thumb/waco.png?1696518487"
  },
  {
    "chainId": 1,
    "address": "0x2ba8349123de45e931a8c8264c332e6e9cf593f9",
    "name": "Blockchain Monster Hunt",
    "symbol": "BCMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19045/thumb/bcmc-coin-200x200.png?1696518496"
  },
  {
    "chainId": 1,
    "address": "0x9d39a5de30e57443bff2a8307a4256c8797a3497",
    "name": "Ethena Staked USDe",
    "symbol": "SUSDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33669/thumb/photo_2023-12-14_17-00-20.jpg?1702696035"
  },
  {
    "chainId": 1,
    "address": "0xa487bf43cf3b10dffc97a9a744cbb7036965d3b9",
    "name": "Deri Protocol",
    "symbol": "DERI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13931/thumb/200vs200.jpg?1696513670"
  },
  {
    "chainId": 1,
    "address": "0xf5b5efc906513b4344ebabcf47a04901f99f09f3",
    "name": "UBIX Network",
    "symbol": "UBX",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/13000/thumb/1.png?1696512789"
  },
  {
    "chainId": 1,
    "address": "0x1b8568fbb47708e9e9d31ff303254f748805bf21",
    "name": "Scarcity",
    "symbol": "SCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19064/thumb/scx_gecko.png?1696518513"
  },
  {
    "chainId": 1,
    "address": "0x1da4858ad385cc377165a298cc2ce3fce0c5fd31",
    "name": "CloutContracts",
    "symbol": "CCS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/19072/thumb/200.png?1696518522"
  },
  {
    "chainId": 1,
    "address": "0x5f64ab1544d28732f0a24f4713c2c8ec0da089f0",
    "name": "Domani Protocol",
    "symbol": "DEXTF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12634/thumb/0qgT0aMu_400x400.jpg?1696512442"
  },
  {
    "chainId": 1,
    "address": "0x5b685863494c33f344081f75e5430c260c224a32",
    "name": "CORE MultiChain",
    "symbol": "CMCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18848/thumb/O4IzY2CQ_400x400.png?1696518309"
  },
  {
    "chainId": 1,
    "address": "0x3c8d2fce49906e11e71cb16fa0ffeb2b16c29638",
    "name": "Nifty League",
    "symbol": "NFTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19224/thumb/NFTL-256x256.png?1696518671"
  },
  {
    "chainId": 1,
    "address": "0x8ee325ae3e54e83956ef2d5952d3c8bc1fa6ec27",
    "name": "Fable Of The Dragon",
    "symbol": "TYRANT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27911/thumb/image_%283%29.png?1696526932"
  },
  {
    "chainId": 1,
    "address": "0x0cec1a9154ff802e7934fc916ed7ca50bde6844e",
    "name": "PoolTogether",
    "symbol": "POOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14003/thumb/PoolTogether.png?1696513732"
  },
  {
    "chainId": 1,
    "address": "0xf56408077487cb879c992909c5b5c66d68c02eb4",
    "name": "Riot Racers",
    "symbol": "RIOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19238/thumb/jyxvIbmJ_400x400.png?1696518684"
  },
  {
    "chainId": 1,
    "address": "0xc5b3d3231001a776123194cf1290068e8b0c783b",
    "name": "LIT",
    "symbol": "LIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21344/thumb/LitLogo_CG.png?1696520710"
  },
  {
    "chainId": 1,
    "address": "0x29de856ee1e2b281c9f6dd3c670272e7c6ea63eb",
    "name": "Groge",
    "symbol": "GROGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33259/thumb/Groge.png?1701318941"
  },
  {
    "chainId": 1,
    "address": "0x8765b1a0eb57ca49be7eacd35b24a574d0203656",
    "name": "MetaGameHub DAO",
    "symbol": "MGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20625/thumb/mgh.PNG?1696520029"
  },
  {
    "chainId": 1,
    "address": "0x9ce115f0341ae5dabc8b477b74e83db2018a6f42",
    "name": "HairDAO",
    "symbol": "HAIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29620/thumb/LinkedIn_Logo.jpg?1696528556"
  },
  {
    "chainId": 1,
    "address": "0xed3d4e446a96dc3b181b64b75c3c70da41dc3cbe",
    "name": "Vodra",
    "symbol": "VDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19338/thumb/vdr.png?1696518779"
  },
  {
    "chainId": 1,
    "address": "0x1fac00ccee478eced6a120a50ed2ab28ee7fe32b",
    "name": "Bitune",
    "symbol": "TUNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14112/thumb/bittune.png?1704793548"
  },
  {
    "chainId": 1,
    "address": "0xba8a621b4a54e61c442f5ec623687e2a942225ef",
    "name": "Sandclock",
    "symbol": "QUARTZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19368/thumb/sandclock.jpg?1696518808"
  },
  {
    "chainId": 1,
    "address": "0x9e5bd9d9fad182ff0a93ba8085b664bcab00fa68",
    "name": "Dinger",
    "symbol": "DINGER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19443/thumb/dinger.png?1696518881"
  },
  {
    "chainId": 1,
    "address": "0x876a87e17d76e1487186e00792985918abadca6b",
    "name": "MemeDefi",
    "symbol": "MEMEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33185/thumb/200x200.png?1700953675"
  },
  {
    "chainId": 1,
    "address": "0x0ab87046fbb341d058f17cbc4c1133f25a20a52f",
    "name": "Governance OHM",
    "symbol": "GOHM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21129/thumb/token_wsOHM_logo.png?1696520508"
  },
  {
    "chainId": 1,
    "address": "0x296233e84c1d7bff11121bf6d60f0ffa39c3f0cf",
    "name": "No One",
    "symbol": "NOONE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19393/thumb/logo-200x200_%289%29.png?1696518833"
  },
  {
    "chainId": 1,
    "address": "0x1782bb88ae0aa07031bc05bbe2bd4150c9da223a",
    "name": "Clickart ai",
    "symbol": "CLICKART",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32985/thumb/2_%282%29.png?1700093960"
  },
  {
    "chainId": 1,
    "address": "0x3ef389f264e07fff3106a3926f2a166d1393086f",
    "name": "Sator",
    "symbol": "SAO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19410/thumb/sator-logo-CMC.png?1696518849"
  },
  {
    "chainId": 1,
    "address": "0xc2544a32872a91f4a553b404c6950e89de901fdb",
    "name": "Frax Price Index Share",
    "symbol": "FPIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24944/thumb/FPIS_icon.png?1696524099"
  },
  {
    "chainId": 1,
    "address": "0x1e053d89e08c24aa2ce5c5b4206744dc2d7bd8f5",
    "name": "ThunderCore",
    "symbol": "TT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4375/thumb/ThunderCore_Logo_Mark_Gradient_%283%29.png?1696504974"
  },
  {
    "chainId": 1,
    "address": "0xe59b14e43479dc30d77f553e4b5c6fc67e1f6caf",
    "name": "LEXA AI",
    "symbol": "LEXA",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x956f47f50a910163d8bf957cf5846d573e7f87ca",
    "name": "Fei USD",
    "symbol": "FEI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14570/thumb/ZqsF51Re_400x400.png?1696514251"
  },
  {
    "chainId": 1,
    "address": "0x1e9d0bb190ac34492aa11b80d28c1c86487a341f",
    "name": "The Neko",
    "symbol": "NEKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22692/thumb/FL6SU9b5_400x400.jpg?1696522003"
  },
  {
    "chainId": 1,
    "address": "0x1f573d6fb3f13d689ff844b4ce37794d79a7ff1c",
    "name": "Bancor Network",
    "symbol": "BNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/736/thumb/Bancor_Token.png?1710479159"
  },
  {
    "chainId": 1,
    "address": "0x389999216860ab8e0175387a0c90e5c52522c945",
    "name": "FEG  OLD ",
    "symbol": "FEG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/13878/thumb/F-Icon-FEG-200x200-288dpi-01_%281%29.png?1696513622"
  },
  {
    "chainId": 1,
    "address": "0x005d1123878fc55fbd56b54c73963b234a64af3c",
    "name": "Kiba Inu",
    "symbol": "KIBA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19525/thumb/kiba.png?1696518959"
  },
  {
    "chainId": 1,
    "address": "0x7f59b64c6ee521c0d7e590cf8e4b843caa236f96",
    "name": "CellMates",
    "symbol": "CELL",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/35114/thumb/D6A3351C-79D3-4658-A67B-4361465162EC.png?1707396881"
  },
  {
    "chainId": 1,
    "address": "0xf1d1a5306daae314af6c5d027a492b313e07e1a0",
    "name": "Envoy",
    "symbol": "ENV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19631/thumb/env.png?1696519059"
  },
  {
    "chainId": 1,
    "address": "0x492798fb464e77cb3cda62b9a2c3c65162db198e",
    "name": "AmpliFi DAO",
    "symbol": "AGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29137/thumb/original-icon-transparent.png?1696528098"
  },
  {
    "chainId": 1,
    "address": "0xaee433adebe0fbb88daa47ef0c1a513caa52ef02",
    "name": "Pontoon",
    "symbol": "TOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19575/thumb/pontoon.PNG?1696519007"
  },
  {
    "chainId": 1,
    "address": "0xd502f487e1841fdc805130e13eae80c61186bc98",
    "name": "Integral",
    "symbol": "ITGR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18381/thumb/Integral_icon_purple.png?1704724515"
  },
  {
    "chainId": 1,
    "address": "0xbededdf2ef49e87037c4fb2ca34d1ff3d3992a11",
    "name": "FEG ETH",
    "symbol": "FEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29643/thumb/feg.png?1696528580"
  },
  {
    "chainId": 1,
    "address": "0x97a3bd8a445cc187c6a751f392e15c3b2134d695",
    "name": "Blockster",
    "symbol": "BXR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19857/thumb/bxr-left2.png?1696519280"
  },
  {
    "chainId": 1,
    "address": "0xa4cf2afd3b165975afffbf7e487cdd40c894ab6b",
    "name": "Shibaken Finance",
    "symbol": "SHIBAKEN",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/15413/thumb/shibak.png?1696515059"
  },
  {
    "chainId": 1,
    "address": "0x0ab39ac604f992aaec3c36de337c3cd3917a7d26",
    "name": "KEEPs Coin",
    "symbol": "KVERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19622/thumb/keeps.PNG?1696519051"
  },
  {
    "chainId": 1,
    "address": "0x8a6d4c8735371ebaf8874fbd518b56edd66024eb",
    "name": "BLOCKS",
    "symbol": "BLOCKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19666/thumb/blocks_logo_200200.png?1696519094"
  },
  {
    "chainId": 1,
    "address": "0x9f52c8ecbee10e00d9faaac5ee9ba0ff6550f511",
    "name": "SIPHER",
    "symbol": "SIPHER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21070/thumb/SipherToken.png?1696520453"
  },
  {
    "chainId": 1,
    "address": "0x505a84a03e382331a1be487b632cf357748b65d6",
    "name": "SHIBGF",
    "symbol": "SHIBGF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19699/thumb/shibgf_logo.png?1696519126"
  },
  {
    "chainId": 1,
    "address": "0xd0d19f52ad8705e60ff31df75a7aca8f1399a69e",
    "name": "404Aliens",
    "symbol": "404A",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35249/thumb/logo-e.jpg?1707988473"
  },
  {
    "chainId": 1,
    "address": "0x44709a920fccf795fbc57baa433cc3dd53c44dbe",
    "name": "DappRadar",
    "symbol": "RADAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20894/thumb/radar.png?1696520286"
  },
  {
    "chainId": 1,
    "address": "0x37fc4b48ce93469dbea9918468993c735049642a",
    "name": "CropBytes",
    "symbol": "CBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19907/thumb/cbx.png?1696519327"
  },
  {
    "chainId": 1,
    "address": "0x00c83aecc790e8a4453e5dd3b0b4b3680501a7a7",
    "name": "SKALE",
    "symbol": "SKL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13245/thumb/SKALE_token_300x300.png?1696513021"
  },
  {
    "chainId": 1,
    "address": "0x0944d5848bd9f60a34ba92aea300d4286696eb76",
    "name": "Palette",
    "symbol": "PLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22153/thumb/262aoVTp_400x400.jpg?1696521498"
  },
  {
    "chainId": 1,
    "address": "0x58cb30368ceb2d194740b144eab4c2da8a917dcb",
    "name": "ZynCoin",
    "symbol": "ZYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35749/thumb/zyn_logo_new_200_x_200.png?1710219535"
  },
  {
    "chainId": 1,
    "address": "0x3656bd0f3f07623bb7f429b390d208f894e44ece",
    "name": "INFLIV",
    "symbol": "IFV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5279/thumb/INFLIV_LOGO_PNG.png?1696505779"
  },
  {
    "chainId": 1,
    "address": "0xa92c49c403386111c1629aee00936eed2a9e74a6",
    "name": "Kollector",
    "symbol": "KLTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19926/thumb/kbase_logo.jpg?1696519345"
  },
  {
    "chainId": 1,
    "address": "0xebf2096e01455108badcbaf86ce30b6e5a72aa52",
    "name": "XIDR",
    "symbol": "XIDR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21126/thumb/XIDR_Logo_256_X_256.png?1696520505"
  },
  {
    "chainId": 1,
    "address": "0xb475332d25d34b59176f5c1d94cb9bc9b5e3954a",
    "name": "Hobbes",
    "symbol": "HOBBES",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30928/thumb/hobbes.jpeg?1696529770"
  },
  {
    "chainId": 1,
    "address": "0x8530b66ca3ddf50e0447eae8ad7ea7d5e62762ed",
    "name": "Meta Doge",
    "symbol": "METADOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19656/thumb/metadoge.png?1696519084"
  },
  {
    "chainId": 1,
    "address": "0xf8e9f10c22840b613cda05a0c5fdb59a4d6cd7ef",
    "name": "Dogs Of Elon",
    "symbol": "DOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19942/thumb/doe.png?1696519360"
  },
  {
    "chainId": 1,
    "address": "0x9be776559fed779cabd67042a7b8987aae592541",
    "name": "Bull Market",
    "symbol": "BULL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29977/thumb/bull.jpg?1696528902"
  },
  {
    "chainId": 1,
    "address": "0x39fbbabf11738317a448031930706cd3e612e1b9",
    "name": "Wrapped XRP",
    "symbol": "WXRP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19952/thumb/WXRP_revision-256px.png?1696519369"
  },
  {
    "chainId": 1,
    "address": "0xf974b5f9ac9c6632fee8b76c61b0242ce69c839d",
    "name": "ZYX",
    "symbol": "ZYX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11964/thumb/zyx.png?1696511823"
  },
  {
    "chainId": 1,
    "address": "0x2920f7d6134f4669343e70122ca9b8f19ef8fa5d",
    "name": "MonoX",
    "symbol": "MONO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20901/thumb/e51c6UaU_400x400.png?1696520292"
  },
  {
    "chainId": 1,
    "address": "0x87869a9789291a6cec99f3c3ef2ff71fceb12a8e",
    "name": "CoinMerge OS",
    "symbol": "CMOS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29327/thumb/cmos.png?1696528277"
  },
  {
    "chainId": 1,
    "address": "0x2da719db753dfa10a62e140f436e1d67f2ddb0d6",
    "name": "Cere Network",
    "symbol": "CERE",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/20008/thumb/cere.png?1696519430"
  },
  {
    "chainId": 1,
    "address": "0x2223bf1d7c19ef7c06dab88938ec7b85952ccd89",
    "name": "Kryxivia Game",
    "symbol": "KXA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21225/thumb/kxa.png?1696520599"
  },
  {
    "chainId": 1,
    "address": "0x4cce605ed955295432958d8951d0b176c10720d5",
    "name": "Novatti Australian Digital Dollar",
    "symbol": "AUDD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33263/thumb/AUDD-Logo-Blue_512.png?1701319895"
  },
  {
    "chainId": 1,
    "address": "0x26c75c7d815efe6bf5a6decd17d20d1fdad96a08",
    "name": "OctopusWallet",
    "symbol": "OCW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36095/thumb/200x200.png?1710481847"
  },
  {
    "chainId": 1,
    "address": "0xfa5b75a9e13df9775cf5b996a049d9cc07c15731",
    "name": "28VCK",
    "symbol": "VCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20063/thumb/9435.png?1696519481"
  },
  {
    "chainId": 1,
    "address": "0xe5597f0723eeaba1b26948e06f008bf0fc1e37e6",
    "name": "GM",
    "symbol": "GM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20066/thumb/PQ29R4D.png?1696519484"
  },
  {
    "chainId": 1,
    "address": "0xd049206fb408a611e543791f2d8f102a8bc253dc",
    "name": "NFTDAO",
    "symbol": "NAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22576/thumb/NAO.png?1696521894"
  },
  {
    "chainId": 1,
    "address": "0x6c862f803ff42a97d4a483ab761256ad8c90f4f8",
    "name": "ELIS",
    "symbol": "XLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13613/thumb/Elis-Symbol-Green.png?1696513363"
  },
  {
    "chainId": 1,
    "address": "0x5d929aa919e489505ccaad8a199619c6dca0c2de",
    "name": "BaaSid",
    "symbol": "BAAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5463/thumb/baasid.png?1696505940"
  },
  {
    "chainId": 1,
    "address": "0xc40f23a3e9613e012944f7957edce97899fa920d",
    "name": "dHealth",
    "symbol": "DHP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20167/thumb/dHealth_Network_Icon_200w.png?1696519580"
  },
  {
    "chainId": 1,
    "address": "0x509a38b7a1cc0dcd83aa9d06214663d9ec7c7f4a",
    "name": "Blocksquare",
    "symbol": "BST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23924/thumb/XpkkqPVR_400x400.jpg?1696523122"
  },
  {
    "chainId": 1,
    "address": "0x5f944b0c4315cb7c3a846b025ab4045da44abf6c",
    "name": "Pancake Games",
    "symbol": "GCAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20217/thumb/6oc-L2UC_400x400.png?1696519627"
  },
  {
    "chainId": 1,
    "address": "0xe66b3aa360bb78468c00bebe163630269db3324f",
    "name": "Merchant",
    "symbol": "MTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17157/thumb/mto.PNG?1696516717"
  },
  {
    "chainId": 1,
    "address": "0xf1ca9cb74685755965c7458528a36934df52a3ef",
    "name": "AVINOC",
    "symbol": "AVINOC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4659/thumb/qX7fh69Q_400x400-1.jpg?1696505226"
  },
  {
    "chainId": 1,
    "address": "0xc4e8a9d47000ab8e59c7031e311762c68215e467",
    "name": "STARX",
    "symbol": "STARX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8417/thumb/starworks.PNG?1696508606"
  },
  {
    "chainId": 1,
    "address": "0x74b988156925937bd4e082f0ed7429da8eaea8db",
    "name": "Meme Inu",
    "symbol": "MEME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20755/thumb/photo_2021-11-20_13-39-45.jpg?1696520151"
  },
  {
    "chainId": 1,
    "address": "0x5c1d9aa868a30795f92fae903edc9eff269044bf",
    "name": "Changer",
    "symbol": "CNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21786/thumb/cng.png?1696521140"
  },
  {
    "chainId": 1,
    "address": "0xbea98c05eeae2f3bc8c3565db7551eb738c8ccab",
    "name": "Geyser",
    "symbol": "GYSR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12995/thumb/logo_padded_shifted.png?1696512785"
  },
  {
    "chainId": 1,
    "address": "0x56de8bc61346321d4f2211e3ac3c0a7f00db9b76",
    "name": "RENA Finance",
    "symbol": "RENA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15667/thumb/photo_2021-05-21_17-31-35.png?1696515297"
  },
  {
    "chainId": 1,
    "address": "0xd1cd47746b8e72359b28c1c84a4f6a19dc1a0ee5",
    "name": "Sonic Suite",
    "symbol": "SONIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28489/thumb/155EE8AF-26AA-4CEC-9EC5-18532E779474.jpeg?1696527481"
  },
  {
    "chainId": 1,
    "address": "0xfc09c7cfd9c175dd9423ca02ae1249579ab12f12",
    "name": "Totoro Inu",
    "symbol": "TOTORO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/20277/thumb/cLogo.png?1696519683"
  },
  {
    "chainId": 1,
    "address": "0x0ba470c6b349d2ae4c93d8c2072b8bfdab95e28a",
    "name": "Sword Bot",
    "symbol": "SWORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33875/thumb/SWORD.jpg?1703144528"
  },
  {
    "chainId": 1,
    "address": "0x5fa3418d828e5cd3c61a66e0fc7fa4a35dadf960",
    "name": "Avatly",
    "symbol": "AVATLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33922/thumb/28738.png?1703370955"
  },
  {
    "chainId": 1,
    "address": "0x1f557fb2aa33dce484902695ca1374f413875519",
    "name": "Vestate",
    "symbol": "VES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35475/thumb/IMG_0759.png?1708772732"
  },
  {
    "chainId": 1,
    "address": "0x62406995cafd18f57e7375e8e0060725acebce58",
    "name": "Firulais Wallet",
    "symbol": "FIWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20716/thumb/FIWT-Logo.png?1696520115"
  },
  {
    "chainId": 1,
    "address": "0xcfeaead4947f0705a14ec42ac3d44129e1ef3ed5",
    "name": "Notional Finance",
    "symbol": "NOTE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/20282/thumb/NOTE-340x340.png?1696519687"
  },
  {
    "chainId": 1,
    "address": "0xe7976c4efc60d9f4c200cc1bcef1a1e3b02c73e7",
    "name": "MAX",
    "symbol": "MAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6454/thumb/MAX_Token.jpg?1696506816"
  },
  {
    "chainId": 1,
    "address": "0x2602278ee1882889b946eb11dc0e810075650983",
    "name": "Vader Protocol",
    "symbol": "VADER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20497/thumb/AcF08Jk1_400x400.jpg?1696519904"
  },
  {
    "chainId": 1,
    "address": "0x6cadf6abbceb53e631c288778daacf125481c1bb",
    "name": "The Citadel",
    "symbol": "CITADEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20445/thumb/2023-03-16_22.55.22.jpg?1696519851"
  },
  {
    "chainId": 1,
    "address": "0x58fad9e3c3ae54c9ba98c3f0e4bf88ab3e8cf3c5",
    "name": "SpaceY 2025",
    "symbol": "SPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20499/thumb/spacey2025.PNG?1696519906"
  },
  {
    "chainId": 1,
    "address": "0x45448e05020576929fcdeabc228e35b420098840",
    "name": "Idavoll DAO",
    "symbol": "IDV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14229/thumb/71922199.png?1696513943"
  },
  {
    "chainId": 1,
    "address": "0xac0968a3e2020ac8ca83e60ccf69081ebc6d3bc3",
    "name": "Cindrum",
    "symbol": "CIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20449/thumb/1_sbv9l6ArIn7oYIhi0oQWoQ.png?1696519854"
  },
  {
    "chainId": 1,
    "address": "0x71fc1f555a39e0b698653ab0b475488ec3c34d57",
    "name": "Rainmaker Games",
    "symbol": "RAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21485/thumb/Final-Flip-Rain-Makers-44-1.png?1696520845"
  },
  {
    "chainId": 1,
    "address": "0x31ea0de8119307aa264bb4b38727aab4e36b074f",
    "name": "Bit Store",
    "symbol": "STORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20463/thumb/bit_store.PNG?1696519868"
  },
  {
    "chainId": 1,
    "address": "0xaea46a60368a7bd060eec7df8cba43b7ef41ad85",
    "name": "Fetch ai",
    "symbol": "FET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5681/thumb/Fetch.jpg?1696506140"
  },
  {
    "chainId": 1,
    "address": "0x685fb6960ea7f7b24f02e17c8d893b8c33239d48",
    "name": "Saiko   The Revival",
    "symbol": "SAIKO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28932/thumb/200x200.png?1696527907"
  },
  {
    "chainId": 1,
    "address": "0x38a94e92a19e970c144ded0b2dd47278ca11cc1f",
    "name": "Falcon Nine",
    "symbol": "F9",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16858/thumb/logo-f9.png?1696516425"
  },
  {
    "chainId": 1,
    "address": "0x31bd628c038f08537e0229f0d8c0a7b18b0cda7b",
    "name": "IntelliQuant",
    "symbol": "INQU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35471/thumb/cropped-INQU-custom-logo.png?1708761830"
  },
  {
    "chainId": 1,
    "address": "0x0af5855a348c58a8bbf65bcc7b9bbacd0e7e2238",
    "name": "Ulanco",
    "symbol": "UAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20662/thumb/udacity_logo_icon_169367.png?1696520064"
  },
  {
    "chainId": 1,
    "address": "0x7a58c0be72be218b41c608b7fe7c5bb630736c71",
    "name": "ConstitutionDAO",
    "symbol": "PEOPLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20612/thumb/GN_UVm3d_400x400.jpg?1696520017"
  },
  {
    "chainId": 1,
    "address": "0x9ab165d795019b6d8b3e971dda91071421305e5a",
    "name": "Aurora Chain",
    "symbol": "AOA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4755/thumb/aurora-logo.png?1696505314"
  },
  {
    "chainId": 1,
    "address": "0xaaaaaa20d9e0e2461697782ef11675f668207961",
    "name": "Aurora",
    "symbol": "AURORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20582/thumb/aurora.jpeg?1696519989"
  },
  {
    "chainId": 1,
    "address": "0xda9fdab21bc4a5811134a6e0ba6ca06624e67c07",
    "name": "Quidd",
    "symbol": "QUIDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19725/thumb/quidd.png?1696519150"
  },
  {
    "chainId": 1,
    "address": "0x2653891204f463fb2a2f4f412564b19e955166ae",
    "name": "Gold Fever Native Gold",
    "symbol": "NGL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20633/thumb/2ypNydrG_400x400.jpg?1696520037"
  },
  {
    "chainId": 1,
    "address": "0x12c51b8f1b8a98c70fe6ae45c158a78931423e4c",
    "name": "XCeption",
    "symbol": "XCEPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33298/thumb/logo_xception_200x200_-1.png?1701402623"
  },
  {
    "chainId": 1,
    "address": "0x8e6cd950ad6ba651f6dd608dc70e5886b1aa6b24",
    "name": "StarLink",
    "symbol": "STARL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16824/thumb/ZxJbRWJ.png?1696516393"
  },
  {
    "chainId": 1,
    "address": "0x517abf1fcdbd76bc75b532683ada9113e313a128",
    "name": "Doke Inu",
    "symbol": "DOKE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29234/thumb/Doke_logo.jpg?1696528191"
  },
  {
    "chainId": 1,
    "address": "0xb549116ac57b47c1b365a890e1d04fd547dfff97",
    "name": "MindAI",
    "symbol": "MDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36180/thumb/logo200.png?1710757731"
  },
  {
    "chainId": 1,
    "address": "0x3a707d56d538e85b783e8ce12b346e7fb6511f90",
    "name": "Inverse Ethereum Volatility Index Token",
    "symbol": "IETHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16715/thumb/logo_-_2021-06-28T085526.083.png?1696516289"
  },
  {
    "chainId": 1,
    "address": "0x62d3c05b9c3d916fbc111819bbd3cee52906c1ae",
    "name": "Every Game",
    "symbol": "EGAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20758/thumb/egame.png?1696520153"
  },
  {
    "chainId": 1,
    "address": "0xae78736cd615f374d3085123a210448e74fc6393",
    "name": "Rocket Pool ETH",
    "symbol": "RETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20764/thumb/reth.png?1696520159"
  },
  {
    "chainId": 1,
    "address": "0xc6e145421fd494b26dcf2bfeb1b02b7c5721978f",
    "name": "Crypto Perx",
    "symbol": "CPRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20768/thumb/8iQSd5YY_400x400.jpg?1696520162"
  },
  {
    "chainId": 1,
    "address": "0x17837004ea685690b32dbead02a274ec4333a26a",
    "name": "Bear Inu",
    "symbol": "BEAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29517/thumb/bear.png?1696528460"
  },
  {
    "chainId": 1,
    "address": "0x1892f6ff5fbe11c31158f8c6f6f6e33106c5b10e",
    "name": "MegaWorld",
    "symbol": "MEGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28456/thumb/Twitter_AVA1.jpg?1696527451"
  },
  {
    "chainId": 1,
    "address": "0xe5ef42d0e5e4aa6b36c613d00db8dad303d505f3",
    "name": "BTour Chain",
    "symbol": "MSOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13141/thumb/msot.png?1696512927"
  },
  {
    "chainId": 1,
    "address": "0xebe4a49df7885d015329c919bf43e6460a858f1e",
    "name": "iShook",
    "symbol": "SHK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3037/thumb/2023-12-20_19.32.44.jpg?1703377166"
  },
  {
    "chainId": 1,
    "address": "0xc01b1979e2244dc94e67891df0af4f7885e57fd4",
    "name": "Lanify",
    "symbol": "LAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36404/thumb/logo200x200.png?1711369836"
  },
  {
    "chainId": 1,
    "address": "0xfeff7b68bc540826da22b296c82a4b8b6b845f41",
    "name": "United Emirates Of Fun",
    "symbol": "UEFN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34353/thumb/LOGO_200.png?1704703932"
  },
  {
    "chainId": 1,
    "address": "0x0d85693b4cb4b8bb4d407e33ba1d9de861db80ca",
    "name": "Privago AI",
    "symbol": "PVGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36836/thumb/bbb2.jpg?1712560493"
  },
  {
    "chainId": 1,
    "address": "0x330528172778cc5196d5f6742886c72505e0613d",
    "name": "XBot",
    "symbol": "XBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31050/thumb/xbot.jpeg?1696529884"
  },
  {
    "chainId": 1,
    "address": "0x4f311c430540db1d64e635eb55f969f1660b2016",
    "name": "Pepe Chain",
    "symbol": "PC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31041/thumb/Pepechain_Token.png?1696529876"
  },
  {
    "chainId": 1,
    "address": "0xddd6a0ecc3c6f6c102e5ea3d8af7b801d1a77ac8",
    "name": "UniX",
    "symbol": "UNIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20674/thumb/unix.png?1696520075"
  },
  {
    "chainId": 1,
    "address": "0x1117ac6ad6cdf1a3bc543bad3b133724620522d5",
    "name": "MODA DAO",
    "symbol": "MODA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20870/thumb/ModaDAO__logomark-primary_3x.png?1696520264"
  },
  {
    "chainId": 1,
    "address": "0x809826cceab68c387726af962713b64cb5cb3cca",
    "name": "Nucleus Vision",
    "symbol": "NCASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3135/thumb/nv.jpg?1696503859"
  },
  {
    "chainId": 1,
    "address": "0x3f68e7b44e9bcb486c2feadb7a2289d9cdfc9088",
    "name": "SportsIcon",
    "symbol": "ICONS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20856/thumb/icons-coin-gold.png?1696520251"
  },
  {
    "chainId": 1,
    "address": "0x9e04f519b094f5f8210441e285f603f4d2b50084",
    "name": "EarthFund",
    "symbol": "1EARTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20822/thumb/earthfund.PNG?1696520215"
  },
  {
    "chainId": 1,
    "address": "0xd2af830e8cbdfed6cc11bab697bb25496ed6fa62",
    "name": "Wrapped OUSD",
    "symbol": "WOUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34355/thumb/wOUSD.png?1704735372"
  },
  {
    "chainId": 1,
    "address": "0xdfc3829b127761a3218bfcee7fc92e1232c9d116",
    "name": "PRivaCY Coin",
    "symbol": "PRCY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14151/thumb/prcy.png?1696513870"
  },
  {
    "chainId": 1,
    "address": "0x70edf1c215d0ce69e7f16fd4e6276ba0d99d4de7",
    "name": "CHEQD Network",
    "symbol": "CHEQ",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/20855/thumb/CHEQ.png?1706906858"
  },
  {
    "chainId": 1,
    "address": "0x9f4909cc95fb870bf48c128c1fdbb5f482797632",
    "name": "Guzzler",
    "symbol": "GZLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20970/thumb/Screenshot-2021-11-27-at-22-57-22.png?1696520357"
  },
  {
    "chainId": 1,
    "address": "0xedb171c18ce90b633db442f2a6f72874093b49ef",
    "name": "Wrapped Ampleforth",
    "symbol": "WAMPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20825/thumb/photo_2021-11-25_02-05-11.jpg?1696520218"
  },
  {
    "chainId": 1,
    "address": "0x5224f552f110ec78e6e0468138950ae5f3040942",
    "name": "Anomus Coin",
    "symbol": "ANOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20865/thumb/w07_TWqn_400x400.png?1696520259"
  },
  {
    "chainId": 1,
    "address": "0xcf91b70017eabde82c9671e30e5502d312ea6eb2",
    "name": "I love puppies",
    "symbol": "PUPPIES",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35708/thumb/1000005823.jpg?1709624357"
  },
  {
    "chainId": 1,
    "address": "0x5eaa69b29f99c84fe5de8200340b4e9b4ab38eac",
    "name": "Raze Network",
    "symbol": "RAZE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14767/thumb/logo-2.png?1696514437"
  },
  {
    "chainId": 1,
    "address": "0x7d29a64504629172a429e64183d6673b9dacbfce",
    "name": "Vectorspace AI",
    "symbol": "VXV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2063/thumb/vectorspace-ai-logo.jpeg?1696503029"
  },
  {
    "chainId": 1,
    "address": "0x066798d9ef0833ccc719076dab77199ecbd178b0",
    "name": "SakeSwap",
    "symbol": "SAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12428/thumb/sake.png?1696512249"
  },
  {
    "chainId": 1,
    "address": "0xe19f85c920b572ca48942315b06d6cac86585c87",
    "name": "PLEB Token",
    "symbol": "PLEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29923/thumb/PLEBToken.png?1696528851"
  },
  {
    "chainId": 1,
    "address": "0xb56a1f3310578f23120182fb2e58c087efe6e147",
    "name": "All Coins Yield Capital",
    "symbol": "ACYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21109/thumb/acyc.png?1696520489"
  },
  {
    "chainId": 1,
    "address": "0x8bbe1a2961b41340468d0548c2cd5b7dfa9b684c",
    "name": "Handy",
    "symbol": "HANDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21944/thumb/Handy_Token_symbol-01.png?1696521293"
  },
  {
    "chainId": 1,
    "address": "0xc36b4311b21fc0c2ead46f1ea6ce97c9c4d98d3d",
    "name": "Creaticles",
    "symbol": "CRE8",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21115/thumb/logo-v3-small.png?1696520495"
  },
  {
    "chainId": 1,
    "address": "0x4554cc10898f92d45378b98d6d6c2dd54c687fb2",
    "name": "Juicebox",
    "symbol": "JBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21120/thumb/CCeIEmvE_400x400.jpg?1696520499"
  },
  {
    "chainId": 1,
    "address": "0xde3ae6f3458db211d298da97fdcf13b6c1c8c87f",
    "name": "Anonym Chain",
    "symbol": "ANC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33670/thumb/200x200_anc.png?1702701458"
  },
  {
    "chainId": 1,
    "address": "0xa9e8acf069c58aec8825542845fd754e41a9489a",
    "name": "PepeCoin",
    "symbol": "PEPECOIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30219/thumb/pepecoin.jpeg?1696529130"
  },
  {
    "chainId": 1,
    "address": "0xd3210f246ae54c5a45a7b4a83315bf718f591bfc",
    "name": "ArkiTech",
    "symbol": "ARKI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33426/thumb/200.png?1711371418"
  },
  {
    "chainId": 1,
    "address": "0x337af08bb6980ecb68389c5ed8876d08643abf8a",
    "name": "NOVAWCHI",
    "symbol": "VACHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29761/thumb/novawchi-infinity-of-nevoverse-coinmarketcap-logo.png?1696528692"
  },
  {
    "chainId": 1,
    "address": "0xe1bda0c3bfa2be7f740f0119b6a34f057bd58eba",
    "name": "Winkies",
    "symbol": "WNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22260/thumb/wnk.png?1696521605"
  },
  {
    "chainId": 1,
    "address": "0xe07f9d810a48ab5c3c914ba3ca53af14e4491e8a",
    "name": "Gyroscope GYD",
    "symbol": "GYD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33428/thumb/GYD-gradient_resized_transparent.png?1702316822"
  },
  {
    "chainId": 1,
    "address": "0x95392f142af1c12f6e39897ff9b09c599666b50c",
    "name": "Impostors Blood",
    "symbol": "BLOOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25527/thumb/RV4nKK2RDO47XzfSLgGs_llgXivhdfpI3tso0.png?1696524659"
  },
  {
    "chainId": 1,
    "address": "0xf17e65822b568b3903685a7c9f496cf7656cc6c2",
    "name": "Biconomy",
    "symbol": "BICO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21061/thumb/biconomy_logo.jpg?1696520444"
  },
  {
    "chainId": 1,
    "address": "0x01597e397605bf280674bf292623460b4204c375",
    "name": "Bent Finance",
    "symbol": "BENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21274/thumb/bent-logo-200x200.png?1696520645"
  },
  {
    "chainId": 1,
    "address": "0x6bd361e10c1afed0d95259e7c0115f3a60e4ea99",
    "name": "BollyCoin",
    "symbol": "BOLLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21610/thumb/IMG-20211208-124337-701.jpg?1696520970"
  },
  {
    "chainId": 1,
    "address": "0x7d5121505149065b562c789a0145ed750e6e8cdd",
    "name": "Victoria VR",
    "symbol": "VR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21178/thumb/vr.png?1696520554"
  },
  {
    "chainId": 1,
    "address": "0x243cacb4d5ff6814ad668c3e225246efa886ad5a",
    "name": "Shina Inu",
    "symbol": "SHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25208/thumb/coingecko-shina-purple-bg.png?1696524352"
  },
  {
    "chainId": 1,
    "address": "0x7c84e62859d0715eb77d1b1c4154ecd6abb21bec",
    "name": "Shping",
    "symbol": "SHPING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2588/thumb/000103_Shping_Logos_Shping_Secondary_Logo.png?1696503392"
  },
  {
    "chainId": 1,
    "address": "0xe632ea2ef2cfd8fc4a2731c76f99078aef6a4b31",
    "name": "THX Network",
    "symbol": "THX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21323/thumb/logo-thx-resized-200-200.png?1696520690"
  },
  {
    "chainId": 1,
    "address": "0x66f73d0fd4161cfad4302dc145ff994375c13475",
    "name": "DexGame",
    "symbol": "DXGM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21084/thumb/dxgm-token.png?1696520466"
  },
  {
    "chainId": 1,
    "address": "0x8db253a1943dddf1af9bcf8706ac9a0ce939d922",
    "name": "Unbound Finance",
    "symbol": "UNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21412/thumb/Unbound_icon_png.png?1696520776"
  },
  {
    "chainId": 1,
    "address": "0x61a35258107563f6b6f102ae25490901c8760b12",
    "name": "Kitty Inu",
    "symbol": "KITTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19385/thumb/KITTY.png?1696518825"
  },
  {
    "chainId": 1,
    "address": "0xd52aae39a2b5cc7812f7b9450ebb61dfef702b15",
    "name": "MetaBrands",
    "symbol": "MAGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21244/thumb/MAGE-CG.png?1696520617"
  },
  {
    "chainId": 1,
    "address": "0xe2f2a5c287993345a840db3b0845fbc70f5935a5",
    "name": "mStable USD",
    "symbol": "MUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11576/thumb/mStable_USD.png?1696511474"
  },
  {
    "chainId": 1,
    "address": "0xda816459f1ab5631232fe5e97a05bbbb94970c95",
    "name": "yvDAI",
    "symbol": "YVDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28743/thumb/yvdai.png?1696527723"
  },
  {
    "chainId": 1,
    "address": "0xb5c578947de0fd71303f71f2c3d41767438bd0de",
    "name": "DeHorizon",
    "symbol": "DEVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21373/thumb/A3RjDqet_400x400.jpg?1696520738"
  },
  {
    "chainId": 1,
    "address": "0x9f9c8ec3534c3ce16f928381372bfbfbfb9f4d24",
    "name": "GraphLinq Chain",
    "symbol": "GLQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14474/thumb/Logo_Dex_4_correct__%282%29.png?1712018526"
  },
  {
    "chainId": 1,
    "address": "0x3300b02efa180c99a2f61f4731665b51e4e254c4",
    "name": "Hitmakr",
    "symbol": "HMKR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36660/thumb/HITMAKR_logo.png?1712042884"
  },
  {
    "chainId": 1,
    "address": "0x34be5b8c30ee4fde069dc878989686abe9884470",
    "name": "SENATE",
    "symbol": "SENATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21272/thumb/siIBOmcs.png?1696520643"
  },
  {
    "chainId": 1,
    "address": "0x8fac8031e079f409135766c7d5de29cf22ef897c",
    "name": "Humans ai",
    "symbol": "HEART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21273/thumb/h_logo_1x.png?1696520644"
  },
  {
    "chainId": 1,
    "address": "0x8f081eb884fd47b79536d28e2dd9d4886773f783",
    "name": "bePAY Finance",
    "symbol": "BECOIN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21275/thumb/logo-becoin.png?1696520646"
  },
  {
    "chainId": 1,
    "address": "0x612e1726435fe38dd49a0b35b4065b56f49c8f11",
    "name": "CryptoCart V2",
    "symbol": "CCV2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15210/thumb/DP7-T6rox-400x400_%281%29.png?1696514866"
  },
  {
    "chainId": 1,
    "address": "0xae6e307c3fe9e922e5674dbd7f830ed49c014c6b",
    "name": "Credefi",
    "symbol": "CREDI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21396/thumb/e1QbZ4qQ_400x400.jpg?1696520760"
  },
  {
    "chainId": 1,
    "address": "0xa0f0546eb5e3ee7e8cfc5da12e5949f3ae622675",
    "name": "Tokoin",
    "symbol": "TOKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8807/thumb/TOKOIN_LOGO_%28no_text%29.png?1696508959"
  },
  {
    "chainId": 1,
    "address": "0xcaabcaa4ca42e1d86de1a201c818639def0ba7a7",
    "name": "Talken",
    "symbol": "TALK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18146/thumb/4NNLx3BF_400x400.jpg?1696517648"
  },
  {
    "chainId": 1,
    "address": "0x9657477ac915f56ca87c253db1320218ec2d5ddd",
    "name": "Jake Newman Enterprises",
    "symbol": "JNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32565/thumb/image-200x200.jpg?1698546675"
  },
  {
    "chainId": 1,
    "address": "0x16cc8367055ae7e9157dbcb9d86fd6ce82522b31",
    "name": "Voxel X Network",
    "symbol": "VXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21419/thumb/Voxel-Logo-200x200-Transparent.png?1696520783"
  },
  {
    "chainId": 1,
    "address": "0xd487892bb4c57edbe7ab401d9fe801c8fe6473f5",
    "name": "Uhive",
    "symbol": "HVE2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3811/thumb/hve2.png?1696504483"
  },
  {
    "chainId": 1,
    "address": "0x1c4853ec0d55e420002c5efabc7ed8e0ba7a4121",
    "name": "Kanagawa Nami",
    "symbol": "OKINAMI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26629/thumb/KANAGAWA_%284%29.png?1696525702"
  },
  {
    "chainId": 1,
    "address": "0x01ba67aac7f75f647d94220cc98fb30fcc5105bf",
    "name": "Lyra Finance",
    "symbol": "LYRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21490/thumb/Add-a-heading-26.png?1696520850"
  },
  {
    "chainId": 1,
    "address": "0xa1817b6d8d890f3943b61648992730373b71f156",
    "name": "Mongoose",
    "symbol": "MONGOOSE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21456/thumb/61b142508fba4235f29d57eb_Frame_24_%281%29.png?1696520818"
  },
  {
    "chainId": 1,
    "address": "0x738865301a9b7dd80dc3666dd48cf034ec42bdda",
    "name": "Agoras  Currency of Tau",
    "symbol": "AGRS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/399/thumb/agrs.png?1696501692"
  },
  {
    "chainId": 1,
    "address": "0x111111517e4929d3dcbdfa7cce55d30d4b6bc4d6",
    "name": "ICHI",
    "symbol": "ICHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13119/thumb/ICHI_%28Round%29.jpg?1696512907"
  },
  {
    "chainId": 1,
    "address": "0x98585dfc8d9e7d48f0b1ae47ce33332cf4237d96",
    "name": "New Order",
    "symbol": "NEWO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21440/thumb/new-order-icon-256px.png?1696520803"
  },
  {
    "chainId": 1,
    "address": "0xdadb4ae5b5d3099dd1f586f990b845f2404a1c4c",
    "name": "Lenny Face",
    "symbol": "",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29138/thumb/Lenny_Face.png?1696528099"
  },
  {
    "chainId": 1,
    "address": "0xaaef88cea01475125522e117bfe45cf32044e238",
    "name": "GuildFi",
    "symbol": "GF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20933/thumb/guildfi-logo-200.png?1696520322"
  },
  {
    "chainId": 1,
    "address": "0x2c642cd3bed9b4e4eeec493611779f13efb502f1",
    "name": "Carbon21",
    "symbol": "C21",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32568/thumb/coin200x200.png?1698549361"
  },
  {
    "chainId": 1,
    "address": "0x505b5eda5e25a67e1c24a2bf1a527ed9eb88bf04",
    "name": "Coinweb",
    "symbol": "CWEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21607/thumb/Logo_200x200.png?1696520967"
  },
  {
    "chainId": 1,
    "address": "0x43d4a3cd90ddd2f8f4f693170c9c8098163502ad",
    "name": "Prime",
    "symbol": "D2D",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21609/thumb/RJD82RrV_400x400.jpg?1696520970"
  },
  {
    "chainId": 1,
    "address": "0x24fcfc492c1393274b6bcd568ac9e225bec93584",
    "name": "Heroes of Mavia",
    "symbol": "MAVIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33895/thumb/2023-12-20_21.21.41_%281%29.jpg?1703230771"
  },
  {
    "chainId": 1,
    "address": "0x7659ce147d0e714454073a5dd7003544234b6aa0",
    "name": "XCAD Network",
    "symbol": "XCAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15857/thumb/xcad_logo.jpg?1707832192"
  },
  {
    "chainId": 1,
    "address": "0x590f820444fa3638e022776752c5eef34e2f89a6",
    "name": "Alephium",
    "symbol": "ALPH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21598/thumb/Alephium-Logo_200x200_listing.png?1696520959"
  },
  {
    "chainId": 1,
    "address": "0x2f141ce366a2462f02cea3d12cf93e4dca49e4fd",
    "name": "FREEdom coin",
    "symbol": "FREE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5585/thumb/free.png?1696506051"
  },
  {
    "chainId": 1,
    "address": "0x78d448a1d1fedaa3e916f467568e86081e0f4df5",
    "name": "MetaBET",
    "symbol": "MBET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22778/thumb/y5TphxL.jpeg?1696522080"
  },
  {
    "chainId": 1,
    "address": "0xb8cb60d07056d54df518785de9600bd4e6b2b53b",
    "name": "Javelin",
    "symbol": "JVL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32109/thumb/999.png?1696530905"
  },
  {
    "chainId": 1,
    "address": "0x77777777772cf0455fb38ee0e75f38034dfa50de",
    "name": "XY Finance",
    "symbol": "XY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21541/thumb/xy.png?1696520900"
  },
  {
    "chainId": 1,
    "address": "0xf0f9d895aca5c8678f706fb8216fa22957685a13",
    "name": "Cult DAO",
    "symbol": "CULT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23331/thumb/quxZPrbC_400x400.jpg?1696522547"
  },
  {
    "chainId": 1,
    "address": "0x900db999074d9277c5da2a43f252d74366230da0",
    "name": "Giveth",
    "symbol": "GIV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21792/thumb/GIVToken_200x200.png?1696521145"
  },
  {
    "chainId": 1,
    "address": "0x3ab6ed69ef663bd986ee59205ccad8a20f98b4c2",
    "name": "Drep",
    "symbol": "DREP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14578/thumb/KotgsCgS_400x400.jpg?1696514258"
  },
  {
    "chainId": 1,
    "address": "0x15874d65e649880c2614e7a480cb7c9a55787ff6",
    "name": "EthereumMax",
    "symbol": "EMAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15540/thumb/EMAX-Coin-Final2000x.png?1696515181"
  },
  {
    "chainId": 1,
    "address": "0x90b831fa3bebf58e9744a14d638e25b4ee06f9bc",
    "name": "Mimo Governance",
    "symbol": "MIMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16449/thumb/mimodefi.PNG?1696516045"
  },
  {
    "chainId": 1,
    "address": "0x0000000de40dfa9b17854cbc7869d80f9f98d823",
    "name": "delta theta",
    "symbol": "DLTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15697/thumb/logo-DT-square.png?1696515325"
  },
  {
    "chainId": 1,
    "address": "0xe803178b48a0e560c2b19f3b3d4e504f79d229ce",
    "name": "Bobcoin",
    "symbol": "BOBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24264/thumb/bobc.png?1696523448"
  },
  {
    "chainId": 1,
    "address": "0x7237c0b30b1355f1b76355582f182f6f04b08740",
    "name": "MetaGaming Guild",
    "symbol": "MGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23287/thumb/mgg.png?1696522506"
  },
  {
    "chainId": 1,
    "address": "0xfe38fd8d710b0ad367f60cb9fc4de5e7ba95633b",
    "name": "Lena",
    "symbol": "LENA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34900/thumb/lena.jpeg?1706583908"
  },
  {
    "chainId": 1,
    "address": "0x19af07b52e5faa0c2b1e11721c52aa23172fe2f5",
    "name": "Memes Street",
    "symbol": "MEMES",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33604/thumb/Meme_St.png?1702510836"
  },
  {
    "chainId": 1,
    "address": "0xa5f1dbb0e55bc31f32c6d032bee330288490e722",
    "name": "Day By Day",
    "symbol": "DBD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21691/thumb/DBD-icon_200x200_%281%29.png?1696521046"
  },
  {
    "chainId": 1,
    "address": "0x383518188c0c6d7730d91b2c03a03c837814a899",
    "name": "Olympus v1",
    "symbol": "OHM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21496/thumb/OHM.jpg?1696520856"
  },
  {
    "chainId": 1,
    "address": "0x0a6e18fb2842855c3af925310b0f50a4bfa17909",
    "name": "CoinPoker",
    "symbol": "CHP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1808/thumb/coinpoker.jpg?1696502810"
  },
  {
    "chainId": 1,
    "address": "0xda7c0810ce6f8329786160bb3d1734cf6661ca6e",
    "name": "Jaypeggers",
    "symbol": "JAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30074/thumb/jay.jpg?1696528999"
  },
  {
    "chainId": 1,
    "address": "0xc73abe8d7a0da644743fe2ad24f4e16bb7ed43f8",
    "name": "Website AI",
    "symbol": "WEBAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32463/thumb/200x200.png?1698242478"
  },
  {
    "chainId": 1,
    "address": "0x4cd0c43b0d53bc318cc5342b77eb6f124e47f526",
    "name": "FreeRossDAO",
    "symbol": "FREE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21648/thumb/free.jpg?1696521006"
  },
  {
    "chainId": 1,
    "address": "0x4b1d0b9f081468d780ca1d5d79132b64301085d1",
    "name": "Lumerin",
    "symbol": "LMR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/24593/thumb/Lumerin-icon.png?1696523767"
  },
  {
    "chainId": 1,
    "address": "0x162bb2bb5fb03976a69dd25bb9afce6140db1433",
    "name": "dog",
    "symbol": "DOG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32464/thumb/200.png?1698242675"
  },
  {
    "chainId": 1,
    "address": "0xd15a1a2a3211b58113e45809f05934252e34e2f8",
    "name": "Woozoo Music",
    "symbol": "WZM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21891/thumb/logo_200_200_%281%29.png?1696521244"
  },
  {
    "chainId": 1,
    "address": "0x08d32b0da63e2c3bcf8019c9c5d849d7a9d791e6",
    "name": "Dentacoin",
    "symbol": "DCN",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/850/thumb/dentacoin.png?1696501986"
  },
  {
    "chainId": 1,
    "address": "0xa3c519683010d59fa54a4a6c4cac0f55cb20bb3f",
    "name": "BetIT",
    "symbol": "BETIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33126/thumb/output-onlinepngtools_%281%29.png?1700795370"
  },
  {
    "chainId": 1,
    "address": "0xe5097d9baeafb89f9bcb78c9290d545db5f9e9cb",
    "name": "Hummingbot",
    "symbol": "HBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21717/thumb/PDPuf0tJ_400x400.jpg?1696521072"
  },
  {
    "chainId": 1,
    "address": "0x470ebf5f030ed85fc1ed4c2d36b9dd02e77cf1b7",
    "name": "TempleDAO",
    "symbol": "TEMPLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20040/thumb/LPK15ZOW_400x400.jpg?1696519459"
  },
  {
    "chainId": 1,
    "address": "0x6f620ec89b8479e97a6985792d0c64f237566746",
    "name": "WePiggy Coin",
    "symbol": "WPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21914/thumb/WPC200.png?1696521265"
  },
  {
    "chainId": 1,
    "address": "0x7717f2828fe4dac8558d23ee4cdfed9544e9321f",
    "name": "OTX EXCHANGE",
    "symbol": "OTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33128/thumb/otx_logo.jpg?1700795823"
  },
  {
    "chainId": 1,
    "address": "0x47b9f01b16e9c9cb99191dca68c9cc5bf6403957",
    "name": "Onston",
    "symbol": "ONSTON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20669/thumb/onston.PNG?1696520070"
  },
  {
    "chainId": 1,
    "address": "0x1fbd3df007eb8a7477a1eab2c63483dcc24effd6",
    "name": "Scaleswap",
    "symbol": "SCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16360/thumb/thumbnail_1170823958_vertical_logo_lateral_radiance.png?1696515959"
  },
  {
    "chainId": 1,
    "address": "0xb113c6cf239f60d380359b762e95c13817275277",
    "name": "BitMEX",
    "symbol": "BMEX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21831/thumb/bitmex-token.jpeg?1696521187"
  },
  {
    "chainId": 1,
    "address": "0xa20f77b7ad5a88badc48800c56507b7274c06fdc",
    "name": "Cherry Network",
    "symbol": "CHER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21855/thumb/cherry.PNG?1696521209"
  },
  {
    "chainId": 1,
    "address": "0x3085154623f51b00dedfc6ceeb5197277a66b17b",
    "name": "NFTY DeFi Protocol",
    "symbol": "NFTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18584/thumb/nfty_logo.png?1696518060"
  },
  {
    "chainId": 1,
    "address": "0x888888888889c00c67689029d7856aac1065ec11",
    "name": "Opium",
    "symbol": "OPIUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13758/thumb/opium-token-black_%281%29.png?1696513500"
  },
  {
    "chainId": 1,
    "address": "0x4daeb4a06f70f4b1a5c329115731fe4b89c0b227",
    "name": "Quasacoin",
    "symbol": "QUA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21965/thumb/coin_200.png?1696521314"
  },
  {
    "chainId": 1,
    "address": "0x80c8c3dcfb854f9542567c8dac3f44d709ebc1de",
    "name": "Spaceswap MILK2",
    "symbol": "MILK2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12771/thumb/milk.png?1696512567"
  },
  {
    "chainId": 1,
    "address": "0xb056c38f6b7dc4064367403e26424cd2c60655e1",
    "name": "CEEK Smart VR",
    "symbol": "CEEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2581/thumb/ceek-smart-vr-token-logo.png?1696503385"
  },
  {
    "chainId": 1,
    "address": "0x9b5161a41b58498eb9c5febf89d60714089d2253",
    "name": "Meta Finance",
    "symbol": "MF1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21954/thumb/eC1-dKKt_400x400.jpg?1696521303"
  },
  {
    "chainId": 1,
    "address": "0x6f2dec5da475333b0af4a3ffc9a33b0211a9a452",
    "name": "CryptoTwitter",
    "symbol": "CT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30237/thumb/200x200px.png?1696529147"
  },
  {
    "chainId": 1,
    "address": "0x0000000005c6b7c1fd10915a05f034f90d524d6e",
    "name": "TRYC",
    "symbol": "TRYC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21985/thumb/16357.png?1696521333"
  },
  {
    "chainId": 1,
    "address": "0xb64fde8f199f073f41c132b9ec7ad5b61de0b1b7",
    "name": "Incognito",
    "symbol": "PRV",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21971/thumb/50738351.png?1696521319"
  },
  {
    "chainId": 1,
    "address": "0x9669890e48f330acd88b78d63e1a6b3482652cd9",
    "name": "Bincentive",
    "symbol": "BCNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9661/thumb/bcnt.png?1696509729"
  },
  {
    "chainId": 1,
    "address": "0xf61bf4d1a948487d61b8fa63808aac06bda55f98",
    "name": "Tr3zor",
    "symbol": "TR3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21980/thumb/9T9OoYXE_400x400.jpg?1696521328"
  },
  {
    "chainId": 1,
    "address": "0x9fc8f0ca1668e87294941b7f627e9c15ea06b459",
    "name": "True PNL",
    "symbol": "PNL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15282/thumb/256x256logo.png?1696514933"
  },
  {
    "chainId": 1,
    "address": "0xe6602b34d8510b033e000975b3322537c7172441",
    "name": "Frontrow",
    "symbol": "FRR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21658/thumb/uZsO4yI7_400x400.jpg?1696521016"
  },
  {
    "chainId": 1,
    "address": "0x6006fc2a849fedaba8330ce36f5133de01f96189",
    "name": "Spaceswap SHAKE",
    "symbol": "SHAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12765/thumb/shake.png?1696512562"
  },
  {
    "chainId": 1,
    "address": "0x6810e776880c02933d47db1b9fc05908e5386b96",
    "name": "Gnosis",
    "symbol": "GNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/662/thumb/logo_square_simple_300px.png?1696501854"
  },
  {
    "chainId": 1,
    "address": "0x9b83f827928abdf18cf1f7e67053572b9bceff3a",
    "name": "Artem",
    "symbol": "ARTEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21998/thumb/ARTM_Logo_MAIN_%281%29.png?1696521345"
  },
  {
    "chainId": 1,
    "address": "0x69b14e8d3cebfdd8196bfe530954a0c226e5008e",
    "name": "SpacePi Token",
    "symbol": "SPACEPI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35375/thumb/logo200.png?1708415311"
  },
  {
    "chainId": 1,
    "address": "0x84342e932797fc62814189f01f0fb05f52519708",
    "name": "Neighbourhoods",
    "symbol": "NHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22055/thumb/social_media_logo_black_bg.png?1696521399"
  },
  {
    "chainId": 1,
    "address": "0x6bba316c48b49bd1eac44573c5c871ff02958469",
    "name": "Gas DAO",
    "symbol": "GAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22086/thumb/SjrgK6sf_400x400.jpg?1696521429"
  },
  {
    "chainId": 1,
    "address": "0x362bc847a3a9637d3af6624eec853618a43ed7d2",
    "name": "PARSIQ",
    "symbol": "PRQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11973/thumb/DsNgK0O.png?1696511831"
  },
  {
    "chainId": 1,
    "address": "0x8254e26e453eb5abd29b3c37ac9e8da32e5d3299",
    "name": "RBX",
    "symbol": "RBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19253/thumb/output-onlinepngtools-9.png?1696518698"
  },
  {
    "chainId": 1,
    "address": "0x3e5d9d8a63cc8a88748f229999cf59487e90721e",
    "name": "MetalSwap",
    "symbol": "XMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22075/thumb/Logo_COIN_-_Gradiente.png?1696521419"
  },
  {
    "chainId": 1,
    "address": "0xe857734840dc188b4283d5af14ab8685467ab87d",
    "name": "ESG Chain",
    "symbol": "ESGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22085/thumb/logo-01.png?1696521428"
  },
  {
    "chainId": 1,
    "address": "0x65ef703f5594d2573eb71aaf55bc0cb548492df4",
    "name": "Multichain",
    "symbol": "MULTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22087/thumb/1_Wyot-SDGZuxbjdkaOeT2-A.png?1696521430"
  },
  {
    "chainId": 1,
    "address": "0x5dd0d493ea59d512efc13d5c1f9d325775192877",
    "name": "Pusuke Inu",
    "symbol": "PUSUKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28674/thumb/Pusuke_Inu.jpg?1696527658"
  },
  {
    "chainId": 1,
    "address": "0xb755d5bc7de83232b9df1886bd5cdb38895933b0",
    "name": "hiMFERS",
    "symbol": "HIMFERS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28719/thumb/hiMFERs.png?1696527700"
  },
  {
    "chainId": 1,
    "address": "0xdd2e93924bdd4e20c3cf4a8736e5955224fa450e",
    "name": "Foho Coin",
    "symbol": "FOHO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17933/thumb/FOHO.Coin_colour-02-1.png?1696517454"
  },
  {
    "chainId": 1,
    "address": "0x7ddc52c4de30e94be3a6a0a2b259b2850f421989",
    "name": "Gomining Token",
    "symbol": "GMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15662/thumb/gmt.png?1696515292"
  },
  {
    "chainId": 1,
    "address": "0x0f7f961648ae6db43c75663ac7e5414eb79b5704",
    "name": "Blockzero Labs",
    "symbol": "XIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10029/thumb/blockzero.jpg?1696510062"
  },
  {
    "chainId": 1,
    "address": "0xda16cf041e2780618c49dbae5d734b89a6bac9b3",
    "name": "Betswap gg",
    "symbol": "BSGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22496/thumb/BSGG_Token_Updated.png?1706248804"
  },
  {
    "chainId": 1,
    "address": "0x2a69655c22eda32ff48d315bb26ed45f150700b4",
    "name": "StakerDAO Wrapped Tezos",
    "symbol": "WXTZ",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14763/thumb/wXTZ-token-FullColor.png?1696514432"
  },
  {
    "chainId": 1,
    "address": "0x970b9bb2c0444f5e81e9d0efb84c8ccdcdcaf84d",
    "name": "Fuse",
    "symbol": "FUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10347/thumb/fuse.png?1696510348"
  },
  {
    "chainId": 1,
    "address": "0x43ab765ee05075d78ad8aa79dcb1978ca3079258",
    "name": "POW",
    "symbol": "POW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22333/thumb/16963.png?1696521677"
  },
  {
    "chainId": 1,
    "address": "0x7f3141c4d6b047fb930991b450f1ed996a51cb26",
    "name": "X",
    "symbol": "X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22334/thumb/x.PNG?1696521678"
  },
  {
    "chainId": 1,
    "address": "0x28dee01d53fed0edf5f6e310bf8ef9311513ae40",
    "name": "BlitzPick",
    "symbol": "XBP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2458/thumb/BlitzPredict.jpg?1696503288"
  },
  {
    "chainId": 1,
    "address": "0xf1b99e3e573a1a9c5e6b2ce818b617f0e664e86b",
    "name": "Opyn Squeeth",
    "symbol": "OSQTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22806/thumb/DyVT5XPV_400x400.jpg?1696522108"
  },
  {
    "chainId": 1,
    "address": "0x1fcdce58959f536621d76f5b7ffb955baa5a672f",
    "name": "ForTube",
    "symbol": "FOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8242/thumb/for.png?1606195375"
  },
  {
    "chainId": 1,
    "address": "0xde4ce5447ce0c67920a1371605a39187cb6847c8",
    "name": "Deesse",
    "symbol": "LOVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22524/thumb/GaqpKHEP_400x400.jpg?1696521847"
  },
  {
    "chainId": 1,
    "address": "0xd7d8f3b8bc8bc48d3acc37879eaba7b85889fa52",
    "name": "ClearDAO",
    "symbol": "CLH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22389/thumb/clh_small_logo_png.png?1696521733"
  },
  {
    "chainId": 1,
    "address": "0xd2287a52287dd64ae2c690d74322e2e345dc8f69",
    "name": "BabyWhale",
    "symbol": "BBW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24168/thumb/babywhale_32.png?1696523357"
  },
  {
    "chainId": 1,
    "address": "0xa15865d9de09cb96aaa3a9081b3dfc8481f07d33",
    "name": "PopeCoin",
    "symbol": "POPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30239/thumb/photo_2023-05-01_11.25.54.jpeg?1696529149"
  },
  {
    "chainId": 1,
    "address": "0x6602e9319f2c5ec0ba31ffcdc4301d7ef03b709e",
    "name": "Wrapped OrdBridge",
    "symbol": "WBRGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33682/thumb/ologo.jpg?1702719798"
  },
  {
    "chainId": 1,
    "address": "0x69d9905b2e5f6f5433212b7f3c954433f23c1572",
    "name": "Onooks",
    "symbol": "OOKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16281/thumb/onooks-logo.png?1696515879"
  },
  {
    "chainId": 1,
    "address": "0x3d0293f06daf4311b482564330d57c8db6c10893",
    "name": "Y Coin",
    "symbol": "YCO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14128/thumb/BIG-POTAT-200x200-removebg-preview.png?1696513849"
  },
  {
    "chainId": 1,
    "address": "0xe138fda441fc31b36171122397a8a11d6cd2c479",
    "name": "Global Trust Coin",
    "symbol": "GTC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/7752/thumb/gtib.png?1696507996"
  },
  {
    "chainId": 1,
    "address": "0x60f5672a271c7e39e787427a18353ba59a4a3578",
    "name": "Pika",
    "symbol": "PIKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14419/thumb/pika-logo-2022-nbg.png?1696514110"
  },
  {
    "chainId": 1,
    "address": "0x72e364f2abdc788b7e918bc238b21f109cd634d7",
    "name": "Metaverse Index",
    "symbol": "MVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14684/thumb/MVI_logo.png?1696514357"
  },
  {
    "chainId": 1,
    "address": "0xa67e9f021b9d208f7e3365b2a155e3c55b27de71",
    "name": "KleeKai",
    "symbol": "KLEE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15548/thumb/Klee-Kai-Logo.png?1696515189"
  },
  {
    "chainId": 1,
    "address": "0x52f4d5ee6c91e01be67ca1f64b11ed0ee370817d",
    "name": "CIA",
    "symbol": "CIA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21569/thumb/Untitled-design-24.png?1696520929"
  },
  {
    "chainId": 1,
    "address": "0xb3cb8d5aeff0f4d1f432f353309f47b885e404e3",
    "name": "MEVerse",
    "symbol": "MEV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24566/thumb/0sp1DWVw_400x400.png?1696523742"
  },
  {
    "chainId": 1,
    "address": "0x95e05e2abbd26943874ac000d87c3d9e115b543c",
    "name": "Raptor",
    "symbol": "BIBLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30866/thumb/bible_logo.png?1696529713"
  },
  {
    "chainId": 1,
    "address": "0x71dc640b99faaf03544bdbce87018c2ae8ebe849",
    "name": "OmniaVerse",
    "symbol": "OMNIA",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25543/thumb/OMNIA.png?1696524676"
  },
  {
    "chainId": 1,
    "address": "0x9c32185b81766a051e08de671207b34466dd1021",
    "name": "BTC Proxy",
    "symbol": "BTCPX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/22630/thumb/MB1aYO7T_400x400.jpg?1696521945"
  },
  {
    "chainId": 1,
    "address": "0x2f4404c4012476929b6503e1397707480bf23b7f",
    "name": "AITravis",
    "symbol": "TAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28933/thumb/ava_%283%29.png?1696527907"
  },
  {
    "chainId": 1,
    "address": "0x375abb85c329753b1ba849a601438ae77eec9893",
    "name": "ParagonsDAO",
    "symbol": "PDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22642/thumb/PDT_LOGO_200.jpg?1696521956"
  },
  {
    "chainId": 1,
    "address": "0xb131f337c45d386ceec234e194b2663d5c3d9dcf",
    "name": "iCommunity",
    "symbol": "ICOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23667/thumb/logo-icommunity.png?1696522869"
  },
  {
    "chainId": 1,
    "address": "0x39207d2e2feef178fbda8083914554c59d9f8c00",
    "name": "MultiPlanetary Inus",
    "symbol": "INUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22648/thumb/logo.png?1696521961"
  },
  {
    "chainId": 1,
    "address": "0x1f31dd60b84ea2a273cfc56876437e069ae80afd",
    "name": "Shinji Inu",
    "symbol": "SHINJI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22846/thumb/Attachment_1645539736.png?1696522147"
  },
  {
    "chainId": 1,
    "address": "0x869dbe51dc214fcb663604b0f7b548592f8c71f5",
    "name": "Okidoki Social",
    "symbol": "DOKI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27018/thumb/Okidoki_Logo.png?1696526070"
  },
  {
    "chainId": 1,
    "address": "0x2af1df3ab0ab157e1e2ad8f88a7d04fbea0c7dc6",
    "name": "Bankless BED Index",
    "symbol": "BED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17175/thumb/BED_Logo_-_No_border.png?1696516734"
  },
  {
    "chainId": 1,
    "address": "0x64df3aab3b21cc275bb76c4a581cf8b726478ee0",
    "name": "Cramer Coin",
    "symbol": "CRAMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27463/thumb/cramercoin.jpg?1696526502"
  },
  {
    "chainId": 1,
    "address": "0x85f6eb2bd5a062f5f8560be93fb7147e16c81472",
    "name": "Franklin",
    "symbol": "FLY",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/14810/thumb/fly_logo_sq_bArtboard_4.png?1696514479"
  },
  {
    "chainId": 1,
    "address": "0x968cbe62c830a0ccf4381614662398505657a2a9",
    "name": "Thrupenny",
    "symbol": "TPY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/26799/thumb/rsz_1mpmojfho_400x400.png?1696525861"
  },
  {
    "chainId": 1,
    "address": "0x5b71bee9d961b1b848f8485eec8d8787f80217f5",
    "name": "Bitforex",
    "symbol": "BF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5330/thumb/BitForex-Logo.png?1696505825"
  },
  {
    "chainId": 1,
    "address": "0x5d74468b69073f809d4fae90afec439e69bf6263",
    "name": "ClayStack Staked ETH",
    "symbol": "CSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33205/thumb/csETH-coingecko.png?1701080702"
  },
  {
    "chainId": 1,
    "address": "0x58f7345b5295e43aa454911571f13be186655be9",
    "name": "Garlicoin",
    "symbol": "GRLC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2699/thumb/garlicoin.png?1696503483"
  },
  {
    "chainId": 1,
    "address": "0xc9b6a17ebb43491635f603a01f8bb3e4b5d22228",
    "name": "MAGA Coin ETH",
    "symbol": "MAGA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27455/thumb/rsz_jimerkya_400x400.png?1696526495"
  },
  {
    "chainId": 1,
    "address": "0x80ce3027a70e0a928d9268994e9b85d03bd4cdcf",
    "name": "Lokr",
    "symbol": "LKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14692/thumb/lokr.png?1696514364"
  },
  {
    "chainId": 1,
    "address": "0x3e828ac5c480069d4765654fb4b8733b910b13b2",
    "name": "Colony Network",
    "symbol": "CLNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23269/thumb/Logo.png?1696522489"
  },
  {
    "chainId": 1,
    "address": "0x3541a5c1b04adaba0b83f161747815cd7b1516bc",
    "name": "CitaDAO",
    "symbol": "KNIGHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22848/thumb/KNIGHT_TOKEN_LOGO.png?1696522149"
  },
  {
    "chainId": 1,
    "address": "0x20d4db1946859e2adb0e5acc2eac58047ad41395",
    "name": "Moon DAO",
    "symbol": "MOONEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22905/thumb/cVWTHdA.png?1696522202"
  },
  {
    "chainId": 1,
    "address": "0x41a3dba3d677e573636ba691a70ff2d606c29666",
    "name": "BlockWallet",
    "symbol": "BLANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14209/thumb/Group_121417608.png?1696513925"
  },
  {
    "chainId": 1,
    "address": "0x64bc2ca1be492be7185faa2c8835d9b824c8a194",
    "name": "Big Time",
    "symbol": "BIGTIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32251/thumb/-6136155493475923781_121.jpg?1696998691"
  },
  {
    "chainId": 1,
    "address": "0xa849eaae994fb86afa73382e9bd88c2b6b18dc71",
    "name": "MVL",
    "symbol": "MVL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3476/thumb/CoinGecko.png?1711620384"
  },
  {
    "chainId": 1,
    "address": "0xbe9ab37a414c517b2be2bfa5945665bb07379054",
    "name": "TomTomCoin",
    "symbol": "TOMS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22864/thumb/toms_logo_200.png?1696522164"
  },
  {
    "chainId": 1,
    "address": "0xc9fe6e1c76210be83dc1b5b20ec7fd010b0b1d15",
    "name": "Fringe Finance",
    "symbol": "FRIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13222/thumb/frin.png?1696513001"
  },
  {
    "chainId": 1,
    "address": "0x7ab7d54f8cb054141142f04ba0b3d41ac4c4d61c",
    "name": "NOTHING",
    "symbol": "NOTHING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33931/thumb/IMG_7800.jpeg?1703418739"
  },
  {
    "chainId": 1,
    "address": "0x721a1b990699ee9d90b6327faad0a3e840ae8335",
    "name": "Lootex",
    "symbol": "LOOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22895/thumb/loot.png?1696522192"
  },
  {
    "chainId": 1,
    "address": "0x2af72850c504ddd3c1876c66a914caee7ff8a46a",
    "name": "WhaleRoom",
    "symbol": "WHL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14328/thumb/ArEX8tkV_400x400.png?1696514016"
  },
  {
    "chainId": 1,
    "address": "0xb90cb79b72eb10c39cbdf86e50b1c89f6a235f2e",
    "name": "Auditchain",
    "symbol": "AUDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6398/thumb/auditchain.png?1696506764"
  },
  {
    "chainId": 1,
    "address": "0xd16fd95d949f996e3808eeea0e3881c59e76ef1e",
    "name": "Para",
    "symbol": "PARA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24652/thumb/para.jpg?1696523821"
  },
  {
    "chainId": 1,
    "address": "0x888888435fde8e7d4c54cab67f206e4199454c60",
    "name": "DFX Finance",
    "symbol": "DFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14091/thumb/DFX.png?1696513813"
  },
  {
    "chainId": 1,
    "address": "0xd94a8f9caed25e63ecc90edfefaf3635ea1e182a",
    "name": "Stablecomp",
    "symbol": "SCOMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33932/thumb/token-icon-big.png?1703422228"
  },
  {
    "chainId": 1,
    "address": "0xc383a3833a87009fd9597f8184979af5edfad019",
    "name": "iETH v1",
    "symbol": "IETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25822/thumb/iETH_100x100.png?1696524907"
  },
  {
    "chainId": 1,
    "address": "0x429881672b9ae42b8eba0e26cd9c73711b891ca5",
    "name": "Pickle Finance",
    "symbol": "PICKLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12435/thumb/0M4W6Yr6_400x400.jpg?1696512255"
  },
  {
    "chainId": 1,
    "address": "0x5d843fa9495d23de997c394296ac7b4d721e841c",
    "name": "Relay Chain",
    "symbol": "RELAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17816/thumb/relay-logo-200.png?1696517336"
  },
  {
    "chainId": 1,
    "address": "0x33840024177a7daca3468912363bed8b425015c5",
    "name": "Ebox",
    "symbol": "EBOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14528/thumb/ebox.png?1696514213"
  },
  {
    "chainId": 1,
    "address": "0x508b27902c6c14972a10a4e413b9cfa449e9cedb",
    "name": "AISignal",
    "symbol": "AISIG",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33671/thumb/IMG_20231214_221614_575.png?1702701782"
  },
  {
    "chainId": 1,
    "address": "0xc631120155621ee625835ec810b9885cdd764cd6",
    "name": "Goldex",
    "symbol": "GLDX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17814/thumb/gldx-logo-200x200.png?1696517334"
  },
  {
    "chainId": 1,
    "address": "0x8a40c222996f9f3431f63bf80244c36822060f12",
    "name": "FINXFLO",
    "symbol": "FXF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13916/thumb/raVh1Iw.png?1696513657"
  },
  {
    "chainId": 1,
    "address": "0x9c6666d5ff4b53b5eb3bd866664c15d0bfcecaa7",
    "name": "Edoverse Zeni",
    "symbol": "ZENI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29238/thumb/Edoverse_Logo.png?1696528195"
  },
  {
    "chainId": 1,
    "address": "0x3c9d6c1c73b31c837832c72e04d3152f051fc1a9",
    "name": "BoringDAO  OLD ",
    "symbol": "BOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12917/thumb/bor_logo.png?1696512705"
  },
  {
    "chainId": 1,
    "address": "0xcdf7028ceab81fa0c6971208e83fa7872994bee5",
    "name": "Threshold Network",
    "symbol": "T",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22228/thumb/nFPNiSbL_400x400.jpg?1696521570"
  },
  {
    "chainId": 1,
    "address": "0x4e352cf164e64adcbad318c3a1e222e9eba4ce42",
    "name": "MUX Protocol",
    "symbol": "MCB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11796/thumb/mux.jpg?1696511672"
  },
  {
    "chainId": 1,
    "address": "0x3a0b022f32b3191d44e5847da12dc0b63fb07c91",
    "name": "Spellfire",
    "symbol": "SPELLFIRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23107/thumb/17316.png?1696522396"
  },
  {
    "chainId": 1,
    "address": "0xc8d3dcb63c38607cb0c9d3f55e8ecce628a01c36",
    "name": "Matrix Labs",
    "symbol": "MATRIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18297/thumb/matrixlabs.png?1696517789"
  },
  {
    "chainId": 1,
    "address": "0xade6fdaba1643e4d1eef68da7170f234470938c6",
    "name": "Harambe",
    "symbol": "HARAMBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23489/thumb/unnamed.jpg?1696522699"
  },
  {
    "chainId": 1,
    "address": "0xbe56ab825fd35678a32dc35bc4eb17e238e1404f",
    "name": "Digits DAO",
    "symbol": "DIGITS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23551/thumb/Logo-Digits-DAO-Icon-01.jpg?1696522758"
  },
  {
    "chainId": 1,
    "address": "0x88800092ff476844f74dc2fc427974bbee2794ae",
    "name": "Ambire Wallet",
    "symbol": "WALLET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23154/thumb/wallet.PNG?1696522445"
  },
  {
    "chainId": 1,
    "address": "0xac9518ba93eeb2336a03137d254d8cc2e4d0fa38",
    "name": "EDUM",
    "symbol": "EDUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32634/thumb/edum_logo_750.png?1698826232"
  },
  {
    "chainId": 1,
    "address": "0xfeef77d3f69374f66429c91d732a244f074bdf74",
    "name": "Convex FXS",
    "symbol": "CVXFXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29001/thumb/cvxfxs.png?1696527973"
  },
  {
    "chainId": 1,
    "address": "0xe9e07fb66976ae5e40cc5f33674725320917b239",
    "name": "Target Protocol",
    "symbol": "TARGET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36286/thumb/dext-icon.png?1711008303"
  },
  {
    "chainId": 1,
    "address": "0x3231cb76718cdef2155fc47b5286d82e6eda273f",
    "name": "Monerium EUR emoney",
    "symbol": "EURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23354/thumb/eur.png?1696522569"
  },
  {
    "chainId": 1,
    "address": "0xb624fde1a972b1c89ec1dad691442d5e8e891469",
    "name": "SporkDAO",
    "symbol": "SPORK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23358/thumb/sporkdao.PNG?1696522573"
  },
  {
    "chainId": 1,
    "address": "0x09395a2a58db45db0da254c7eaa5ac469d8bdc85",
    "name": "SubQuery Network",
    "symbol": "SQT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23359/thumb/photo_2021-07-14_11-31-12.jpg?1696522574"
  },
  {
    "chainId": 1,
    "address": "0x433fce7dfbec729a79999eaf056cb073b2153eba",
    "name": "CoinWealth",
    "symbol": "CNW",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/23769/thumb/cw_logo-4955f59a5c8079f246fa07ac71b2541870ca7d906ca1d9c26d74a3870fafef2f_%281%29.png?1696522970"
  },
  {
    "chainId": 1,
    "address": "0x59d1e836f7b7210a978b25a855085cc46fd090b5",
    "name": "AssangeDAO",
    "symbol": "JUSTICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23555/thumb/JUSTICE_token_logo.png?1696522761"
  },
  {
    "chainId": 1,
    "address": "0x00281dfce4cfd72c0b6fda2aaaf077258743f9e8",
    "name": "NuriFootBall",
    "symbol": "NRFB",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/23360/thumb/pcMlP25e_400x400.jpg?1696522575"
  },
  {
    "chainId": 1,
    "address": "0xcb56b52316041a62b6b5d0583dce4a8ae7a3c629",
    "name": "Cigarette",
    "symbol": "CIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22145/thumb/logo200.png?1696521491"
  },
  {
    "chainId": 1,
    "address": "0x817bbdbc3e8a1204f3691d14bb44992841e3db35",
    "name": "Cudos",
    "symbol": "CUDOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13651/thumb/CudosIconTransparent.png?1696513400"
  },
  {
    "chainId": 1,
    "address": "0xc91a71a1ffa3d8b22ba615ba1b9c01b2bbbf55ad",
    "name": "ZigZag",
    "symbol": "ZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26141/thumb/zig_zag.?1696525229"
  },
  {
    "chainId": 1,
    "address": "0x95aa5d2dbd3c16ee3fdea82d5c6ec3e38ce3314f",
    "name": "PointPay",
    "symbol": "PXP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8950/thumb/6f9823ed-464a-4413-8940-72aa61d30b43.png?1696509097"
  },
  {
    "chainId": 1,
    "address": "0x478156deabfac918369044d52a6bdb5cc5597994",
    "name": "Schrodinger",
    "symbol": "SGR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/36288/thumb/Project_Schrodinger_Logo_Square_200x200.png?1711009156"
  },
  {
    "chainId": 1,
    "address": "0xeeaa40b28a2d1b0b08f6f97bb1dd4b75316c6107",
    "name": "CVI",
    "symbol": "GOVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13875/thumb/GOVI.png?1696513619"
  },
  {
    "chainId": 1,
    "address": "0x4f640f2529ee0cf119a2881485845fa8e61a782a",
    "name": "ORE Network",
    "symbol": "ORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18917/thumb/ORE_FullColor.png?1696518374"
  },
  {
    "chainId": 1,
    "address": "0xe4b91faf8810f8895772e7ca065d4cb889120f94",
    "name": "Cat in a Box Fee Token",
    "symbol": "BOXFEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29928/thumb/boxFEE.png?1696528856"
  },
  {
    "chainId": 1,
    "address": "0x67c5870b4a41d4ebef24d2456547a03f1f3e094b",
    "name": "GoodDollar",
    "symbol": "G",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/14782/thumb/G__Coin_%281%29.png?1696514451"
  },
  {
    "chainId": 1,
    "address": "0x63a0db25159672326ac8fbf512c409598801b145",
    "name": "XPayPro Tech",
    "symbol": "XPPT",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/33306/thumb/cg-xpay-logo.png?1701404908"
  },
  {
    "chainId": 1,
    "address": "0xc502002aeb1b9309fccb016adf50507987fc6c2b",
    "name": "GNFT",
    "symbol": "GNFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23532/thumb/gnft_200x200.png?1696522740"
  },
  {
    "chainId": 1,
    "address": "0x94e9eb8b5ab9fd6b9ea3169d55ffade62a01702e",
    "name": "BreederDAO",
    "symbol": "BREED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25203/thumb/BreederDAO-Breed_Token-FINAL.png?1696524347"
  },
  {
    "chainId": 1,
    "address": "0x70c4430f9d98b4184a4ef3e44ce10c320a8b7383",
    "name": "Gyroscope",
    "symbol": "GYFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23584/thumb/Gyroscope_logo_transparent.png?1696522792"
  },
  {
    "chainId": 1,
    "address": "0x0bf43350076f95e0d16120b4d6bdfa1c9d50bdbd",
    "name": "Antfarm Governance Token",
    "symbol": "AGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28822/thumb/AGT.png?1696527798"
  },
  {
    "chainId": 1,
    "address": "0x77d9046ee15faaceb89439ffdde4be071c2f07bd",
    "name": "Jiyuu",
    "symbol": "JIYUU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28907/thumb/logo_200.png?1696527883"
  },
  {
    "chainId": 1,
    "address": "0x1982b2f5814301d4e9a8b0201555376e62f82428",
    "name": "Aave Interest Bearing STETH",
    "symbol": "ASTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24163/thumb/steth.jpg?1696523352"
  },
  {
    "chainId": 1,
    "address": "0xa3ee21c306a700e682abcdfe9baa6a08f3820419",
    "name": "Creditcoin",
    "symbol": "CTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10569/thumb/ctc.png?1696510552"
  },
  {
    "chainId": 1,
    "address": "0xa2b4c0af19cc16a6cfacce81f192b024d625817d",
    "name": "Kishu Inu",
    "symbol": "KISHU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14890/thumb/uVLzCoP.png?1696514553"
  },
  {
    "chainId": 1,
    "address": "0x5c147e74d63b1d31aa3fd78eb229b65161983b2b",
    "name": "Wrapped Flow",
    "symbol": "WFLOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24045/thumb/wrapped-flow.png?1696523236"
  },
  {
    "chainId": 1,
    "address": "0xc98d64da73a6616c42117b582e832812e7b8d57f",
    "name": "RSS3",
    "symbol": "RSS3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23575/thumb/RSS3.png?1696522783"
  },
  {
    "chainId": 1,
    "address": "0xc690f7c7fcffa6a82b79fab7508c466fefdfc8c5",
    "name": "Lympo",
    "symbol": "LYM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2620/thumb/lympo_token.png?1696503417"
  },
  {
    "chainId": 1,
    "address": "0x865377367054516e17014ccded1e7d814edc9ce4",
    "name": "DOLA",
    "symbol": "DOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14287/thumb/dola.png?1696513984"
  },
  {
    "chainId": 1,
    "address": "0x24a5fe2c2240e58252cdfc43bb94ae67d2ead218",
    "name": "Mone Coin",
    "symbol": "MONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35310/thumb/photo_2024-02-08_23-06-34.jpg?1708158583"
  },
  {
    "chainId": 1,
    "address": "0x3da932456d082cba208feb0b096d49b202bf89c8",
    "name": "Dego Finance",
    "symbol": "DEGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12503/thumb/c185FKx.png?1696512318"
  },
  {
    "chainId": 1,
    "address": "0x1e4ede388cbc9f4b5c79681b7f94d36a11abebc9",
    "name": "X2Y2",
    "symbol": "X2Y2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23633/thumb/logo-60b81ff87b40b11739105acf5ad1e075.png?1696522837"
  },
  {
    "chainId": 1,
    "address": "0x37e83a94c6b1bdb816b59ac71dd02cf154d8111f",
    "name": "PhotoChromic",
    "symbol": "PHCR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23640/thumb/L7vabJibqXHHmGtsIfIEQRaR3SjaE94QSaKWGlqYU7fpuz--nKsWG4BlB6Ddhn5-tGPWo-MDhfZPVAYG7ZM2vpWrVZlSx1Euw78tZcNey6FvpKX6a8w26qBWYeV7wxFzvW7VKieijrMXQ0BkISpt8d6jrCMBvx2ZFKVxF1x31_2X7WhRyqAgIG3epFPNz7v0j0ECaS7HmE0_sTHM4RS3CW1d0a.jpg?1696522844"
  },
  {
    "chainId": 1,
    "address": "0x6e9730ecffbed43fd876a264c982e254ef05a0de",
    "name": "Nord Finance",
    "symbol": "NORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13630/thumb/nord.jpg?1696513378"
  },
  {
    "chainId": 1,
    "address": "0xdac657ffd44a3b9d8aba8749830bf14beb66ff2d",
    "name": "humanDAO",
    "symbol": "HDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21138/thumb/Untitled-2.jpg?1696520516"
  },
  {
    "chainId": 1,
    "address": "0xa6422e3e219ee6d4c1b18895275fe43556fd50ed",
    "name": "Stobox",
    "symbol": "STBU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12637/thumb/123456826.png?1696512445"
  },
  {
    "chainId": 1,
    "address": "0xa350da05405cc204e551c4eed19c3039646528d5",
    "name": "Blocksport",
    "symbol": "BSPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23735/thumb/logo_icon.jpg?1696522938"
  },
  {
    "chainId": 1,
    "address": "0x95b4ef2869ebd94beb4eee400a99824bf5dc325b",
    "name": "cMKR",
    "symbol": "CMKR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17528/thumb/cmkr.PNG?1696517066"
  },
  {
    "chainId": 1,
    "address": "0x7e6f60e237c34307d516ef80218c2b04bccbca40",
    "name": "Trackr",
    "symbol": "TRACKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34433/thumb/photo_2024-01-09_15-28-29_%282%29.jpg?1704869068"
  },
  {
    "chainId": 1,
    "address": "0x8eedefe828a0f16c8fc80e46a87bc0f1de2d960c",
    "name": "DigiMetaverse",
    "symbol": "DGMV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23701/thumb/DigiCorpLabs_token.png?1696522902"
  },
  {
    "chainId": 1,
    "address": "0x3fa729b4548becbad4eab6ef18413470e6d5324c",
    "name": "Mover",
    "symbol": "MOVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13719/thumb/o0KIvs7T_400x400.jpg?1696513464"
  },
  {
    "chainId": 1,
    "address": "0xfe459828c90c0ba4bc8b42f5c5d44f316700b430",
    "name": "BBS Network",
    "symbol": "BBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23715/thumb/Ni13Pg1K_400x400.jpg?1696522915"
  },
  {
    "chainId": 1,
    "address": "0xba6b0dbb2ba8daa8f5d6817946393aef8d3a4487",
    "name": "Hillstone Finance",
    "symbol": "HSF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22335/thumb/logo_-_2022-01-07T094430.368.png?1696521679"
  },
  {
    "chainId": 1,
    "address": "0x0bb217e40f8a5cb79adf04e1aab60e5abd0dfc1e",
    "name": "SWFTCOIN",
    "symbol": "SWFTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2346/thumb/SWFTCoin.jpg?1696503223"
  },
  {
    "chainId": 1,
    "address": "0xea4a2327e75252517535fd013b7c6706609819db",
    "name": "Shibarium Name Service",
    "symbol": "SNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29486/thumb/shibarium.jpg?1696528430"
  },
  {
    "chainId": 1,
    "address": "0xa6586e19ef681b1ac0ed3d46413d199a555dbb95",
    "name": "Lets Go Brandon",
    "symbol": "LETSGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24034/thumb/3zKU0T83_400x400.jpeg?1696523226"
  },
  {
    "chainId": 1,
    "address": "0x3392d8a60b77f8d3eaa4fb58f09d835bd31add29",
    "name": "IndiGG",
    "symbol": "INDI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23857/thumb/https___bucketeer-e05bbc84-baa3-437e-9518-adb32be77984.s3.amazonaws.com_public_images_0cb39c1f-db6d-4dd9-9143-1f30ee38d74d_743x726.jpeg?1696523058"
  },
  {
    "chainId": 1,
    "address": "0x7b18e5d77622ba37173801f77745a0654df825ff",
    "name": "TraderX",
    "symbol": "TX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34051/thumb/logo.jpg?1703744503"
  },
  {
    "chainId": 1,
    "address": "0xb6dc73987848ba6cb3ce297562723c1a78f85fce",
    "name": "Borzoi Coin",
    "symbol": "BORZOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34430/thumb/photo_2024-01-08_05-57-57.jpg?1704868051"
  },
  {
    "chainId": 1,
    "address": "0x4de914f49f04c943dbde7e4f8017d08b51ad657c",
    "name": "Oracle Layer2",
    "symbol": "ORACLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32702/thumb/20231101_235814.png?1698985290"
  },
  {
    "chainId": 1,
    "address": "0xbc6e06778708177a18210181b073da747c88490a",
    "name": "MOBLAND",
    "symbol": "SYNR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22868/thumb/FullML_Yellow_200px.png?1696522168"
  },
  {
    "chainId": 1,
    "address": "0x788b6d2b37aa51d916f2837ae25b05f0e61339d1",
    "name": "Metavault DAO",
    "symbol": "MVD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/23899/thumb/MVD_Coin-simple.png?1696523099"
  },
  {
    "chainId": 1,
    "address": "0xf1290473e210b2108a85237fbcd7b6eb42cc654f",
    "name": "HedgeTrade",
    "symbol": "HEDG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7196/thumb/to3Vj4EZ_400x400.jpg?1696507494"
  },
  {
    "chainId": 1,
    "address": "0x6431fa4b812a2dcc062a38cb55cc7d18135adead",
    "name": "RankerDao",
    "symbol": "RANKER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23902/thumb/ranker.png?1696523102"
  },
  {
    "chainId": 1,
    "address": "0x469084939d1c20fae3c73704fe963941c51be863",
    "name": "Envision",
    "symbol": "VIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23907/thumb/J3JCKVq2.png?1696523107"
  },
  {
    "chainId": 1,
    "address": "0x0173661769325565d4f011b2e5cda688689cc87c",
    "name": "Quantland",
    "symbol": "QLT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24536/thumb/8jYfKBTk_400x400.jpg?1696523713"
  },
  {
    "chainId": 1,
    "address": "0xfb5c6815ca3ac72ce9f5006869ae67f18bf77006",
    "name": "pSTAKE Finance",
    "symbol": "PSTAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23931/thumb/PSTAKE_Dark.png?1696523129"
  },
  {
    "chainId": 1,
    "address": "0x201332bd45c8628d814f870bfb584b385a7c351e",
    "name": "Astra Protocol",
    "symbol": "ASTRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24083/thumb/Astra_%281%29_%281%29.png?1696523277"
  },
  {
    "chainId": 1,
    "address": "0x876a76c80b32e5cfbb27fd840a1a530ef828ebec",
    "name": "Spotted Turtle",
    "symbol": "ST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31581/thumb/20230829_220418.png?1696530398"
  },
  {
    "chainId": 1,
    "address": "0x8a3d77e9d6968b780564936d15b09805827c21fa",
    "name": "Archethic",
    "symbol": "UCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12330/thumb/Archethic_logo.png?1696512158"
  },
  {
    "chainId": 1,
    "address": "0x5b1d655c93185b06b00f7925791106132cb3ad75",
    "name": "DarkMatter",
    "symbol": "DMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23989/thumb/dmt.png?1696523184"
  },
  {
    "chainId": 1,
    "address": "0xf0dc9fc0669f068e04ad79f7d70618d3f9aad439",
    "name": "Oasis Metaverse",
    "symbol": "OASIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32703/thumb/oasis_200x200_loogo.png?1698985354"
  },
  {
    "chainId": 1,
    "address": "0x4fabf135bcf8111671870d4399af739683198f96",
    "name": "Xave Coin",
    "symbol": "XVC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24084/thumb/k8qjP9t9_400x400.jpg?1696523277"
  },
  {
    "chainId": 1,
    "address": "0x3f3cd642e81d030d7b514a2ab5e3a5536beb90ec",
    "name": "Rho",
    "symbol": "RHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25540/thumb/Rho-icon-03.png?1696524673"
  },
  {
    "chainId": 1,
    "address": "0x88aa4a6c5050b9a1b2aa7e34d0582025ca6ab745",
    "name": "Dexpools",
    "symbol": "DXP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25114/thumb/1_0w4P8R5heRFwRomJ4vPCWQ.png?1696524264"
  },
  {
    "chainId": 1,
    "address": "0x4688a8b1f292fdab17e9a90c8bc379dc1dbd8713",
    "name": "Cover Protocol",
    "symbol": "COVER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13563/thumb/1_eWBbDaqpxXqt7WYPSP4qSw.jpeg?1696513317"
  },
  {
    "chainId": 1,
    "address": "0xf8ebf4849f1fa4faf0dff2106a173d3a6cb2eb3a",
    "name": "Troll",
    "symbol": "TROLL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29897/thumb/TROLL.jpeg?1696528821"
  },
  {
    "chainId": 1,
    "address": "0xb44377b74ef1773639b663d0754cb8410a847d02",
    "name": "Dream",
    "symbol": "DREAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24139/thumb/image0.jpeg?1696523330"
  },
  {
    "chainId": 1,
    "address": "0x71a28feaee902966dc8d355e7b8aa427d421e7e0",
    "name": "LunchDAO",
    "symbol": "LUNCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25808/thumb/200x200.png?1696524893"
  },
  {
    "chainId": 1,
    "address": "0x06f3c323f0238c72bf35011071f2b5b7f43a054c",
    "name": "MASQ",
    "symbol": "MASQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13699/thumb/masq.png?1696513446"
  },
  {
    "chainId": 1,
    "address": "0x8a0a9b663693a22235b896f70a229c4a22597623",
    "name": "Scalia Infrastructure",
    "symbol": "SCALE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35653/thumb/Scalia_Infrastructure.png?1709380021"
  },
  {
    "chainId": 1,
    "address": "0xf71a2079566c87e26bcd9766d140d5bfdeeae731",
    "name": "Doge69",
    "symbol": "DOGE69",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33210/thumb/IMG_20231126_121430_856.jpg?1701084935"
  },
  {
    "chainId": 1,
    "address": "0x2f9411088cef82fd9fb904eb8092f28eb485c8f6",
    "name": "Athens",
    "symbol": "ATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24514/thumb/ATH-logo-round.png?1696523693"
  },
  {
    "chainId": 1,
    "address": "0x64323d606cfcb1b50998636a182334ad97637987",
    "name": "xperp",
    "symbol": "XPERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33107/thumb/xperp-logo.png?1700663572"
  },
  {
    "chainId": 1,
    "address": "0x60bb16c4a931b1a0b8a7d945c651dd90f41d42cf",
    "name": "Finance Blocks",
    "symbol": "FBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24117/thumb/financeblocksshadow.png?1696523309"
  },
  {
    "chainId": 1,
    "address": "0xa774ffb4af6b0a91331c084e1aebae6ad535e6f3",
    "name": "flexUSD",
    "symbol": "FLEXUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13323/thumb/flexUSD_2x.png?1696513092"
  },
  {
    "chainId": 1,
    "address": "0xc91b523a59acc63a64f61fc7bbfb4bfc82dd25f2",
    "name": "Multiverse",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16329/thumb/multiverse_logo_full_color_rgb_200px_72ppi.png?1696515930"
  },
  {
    "chainId": 1,
    "address": "0x8a7adc1b690e81c758f1bd0f72dfe27ae6ec56a5",
    "name": "Bolide",
    "symbol": "BLID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25548/thumb/5XvLQULP_400x400.jpg?1703622595"
  },
  {
    "chainId": 1,
    "address": "0x25722cd432d02895d9be45f5deb60fc479c8781e",
    "name": "Sponge  OLD ",
    "symbol": "SPONGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30254/thumb/Sea_Sponge_1.png?1696529163"
  },
  {
    "chainId": 1,
    "address": "0xf725f73caee250ae384ec38bb2c77c38ef2cccea",
    "name": "Ape In Records",
    "symbol": "AIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24194/thumb/gCZZdeR.jpeg?1696523382"
  },
  {
    "chainId": 1,
    "address": "0x961d4921e1718e633bac8ded88c4a1cae44b785a",
    "name": "Thunderhead Staked FLIP",
    "symbol": "STFLIP",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xff3f8227c3b0241848e1420ec77d61aee2ad5f67",
    "name": "TxWorx",
    "symbol": "TX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35644/thumb/IMG_2815.jpeg?1709368757"
  },
  {
    "chainId": 1,
    "address": "0x6247c86b016bc4d9ae141849c0a9eb38c004b742",
    "name": "Hotelium",
    "symbol": "HTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8812/thumb/hotelium.PNG?1696508966"
  },
  {
    "chainId": 1,
    "address": "0x36ac219f90f5a6a3c77f2a7b660e3cc701f68e25",
    "name": "Coinmetro",
    "symbol": "XCM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3125/thumb/XCM_Token_Logo_.png?1696503850"
  },
  {
    "chainId": 1,
    "address": "0x3cbc780d2934d55a06069e837fabd3e6fc23dab0",
    "name": "DBX",
    "symbol": "DBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15682/thumb/logo.png?1696515312"
  },
  {
    "chainId": 1,
    "address": "0xa35923162c49cf95e6bf26623385eb431ad920d3",
    "name": "Turbo",
    "symbol": "TURBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30117/thumb/TurboMark-QL_200.png?1708079597"
  },
  {
    "chainId": 1,
    "address": "0xf939e0a03fb07f59a73314e73794be0e57ac1b4e",
    "name": "crvUSD",
    "symbol": "CRVUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30118/thumb/crvusd.jpeg?1696529040"
  },
  {
    "chainId": 1,
    "address": "0x6dde4ffd6db302bc9a46850f61399e082f6c2122",
    "name": "inheritance Art",
    "symbol": "IAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24221/thumb/iAI.jpg?1696523407"
  },
  {
    "chainId": 1,
    "address": "0x8503a7b00b4b52692cc6c14e5b96f142e30547b7",
    "name": "Meeds DAO",
    "symbol": "MEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24281/thumb/s-YfFWYu_400x400.png?1696523464"
  },
  {
    "chainId": 1,
    "address": "0x61035ed28081c1acc38e399c416bfc08fd6e73a1",
    "name": "FrouBot",
    "symbol": "FROBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32987/thumb/wepik-export-20231029191457FW3V_%281%29.png?1700094955"
  },
  {
    "chainId": 1,
    "address": "0xbfb2b6870501a6ff17121d676a0a45a38c9eed1e",
    "name": "LuckyToad",
    "symbol": "TOAD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28867/thumb/Coingecko_listing.png?1696527840"
  },
  {
    "chainId": 1,
    "address": "0x8f9b4525681f3ea6e43b8e0a57bfff86c0a1dd2e",
    "name": "Zeebu",
    "symbol": "ZBU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31145/thumb/200x200_Front_token.png?1696529974"
  },
  {
    "chainId": 1,
    "address": "0x0f59bfdd1d95a3736d2e219f905ae9b2e69c6638",
    "name": "KIWI DEPLOYER BOT",
    "symbol": "KIWI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35164/thumb/2024-02-10_20.54.15.jpg?1707662977"
  },
  {
    "chainId": 1,
    "address": "0x3ba925fdeae6b46d0bb4d424d829982cb2f7309e",
    "name": "RabbitX",
    "symbol": "RBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30048/thumb/RBX.png?1696528970"
  },
  {
    "chainId": 1,
    "address": "0x144805be43c48ef85435c94e0da4cb4efb1ab4f3",
    "name": "MetaBlox",
    "symbol": "MBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32988/thumb/Artboard_1_copy_4.png?1700099624"
  },
  {
    "chainId": 1,
    "address": "0xed04915c23f00a313a544955524eb7dbd823143d",
    "name": "Alchemy Pay",
    "symbol": "ACH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12390/thumb/ACH_%281%29.png?1696512213"
  },
  {
    "chainId": 1,
    "address": "0x6dc6a27822ae2ca3a47da39a2f2bbd525dd693f8",
    "name": "Cyberdoge",
    "symbol": "CYDOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34085/thumb/CD_Logo_200x200.png?1703910686"
  },
  {
    "chainId": 1,
    "address": "0x62939f78a9a1cc4f9f1ea41cff95cd4f6b912d1b",
    "name": "Numisme2",
    "symbol": "NUME2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31009/thumb/NUME2.jpg?1696529846"
  },
  {
    "chainId": 1,
    "address": "0xa80f2c8f61c56546001f5fc2eb8d6e4e72c45d4c",
    "name": "Unique Utility",
    "symbol": "UNQT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23382/thumb/UniqueCoin_Logo.jpeg?1708335635"
  },
  {
    "chainId": 1,
    "address": "0xa8b919680258d369114910511cc87595aec0be6d",
    "name": "LUKSO  OLD ",
    "symbol": "LYXE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11423/thumb/LYX.png?1696511335"
  },
  {
    "chainId": 1,
    "address": "0xdd782657af86492771bdb369caeb8c246be362f5",
    "name": "RUGAME",
    "symbol": "RUG",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/29925/thumb/rugame.png?1696528853"
  },
  {
    "chainId": 1,
    "address": "0x628ab8b061fea2af1239b68efa5e46135d186666",
    "name": "XBullion Silver",
    "symbol": "SILV",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/24171/thumb/SILV_Logo_%28transparent_circle%29.png?1646816506"
  },
  {
    "chainId": 1,
    "address": "0x09f098b155d561fc9f7bccc97038b7e3d20baf74",
    "name": "ZooDAO",
    "symbol": "ZOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24305/thumb/Zt2BM_8D_400x400.jpg?1696523486"
  },
  {
    "chainId": 1,
    "address": "0x33f391f4c4fe802b70b77ae37670037a92114a7c",
    "name": "Burp",
    "symbol": "BURP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17317/thumb/Big_Town_Chef_-__BURP_-_Avatar_3x.png?1696516871"
  },
  {
    "chainId": 1,
    "address": "0x3c2a309d9005433c1bc2c92ef1be06489e5bf258",
    "name": "Wrapped Paycoin",
    "symbol": "WPCI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/24390/thumb/new_logo_square_%281%29.png?1696523573"
  },
  {
    "chainId": 1,
    "address": "0x1864ce27e9f7517047933caae530674e8c70b8a7",
    "name": "Pibble",
    "symbol": "PIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2972/thumb/pibble.jpg?1696503713"
  },
  {
    "chainId": 1,
    "address": "0xa71d0588eaf47f12b13cf8ec750430d21df04974",
    "name": "Shiba Predator",
    "symbol": "QOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24430/thumb/l1KzMcL.png?1696523611"
  },
  {
    "chainId": 1,
    "address": "0xd43be54c1aedf7ee4099104f2dae4ea88b18a249",
    "name": "Traxx",
    "symbol": "TRAXX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24441/thumb/TT.png?1696523622"
  },
  {
    "chainId": 1,
    "address": "0x01d33fd36ec67c6ada32cf36b31e88ee190b1839",
    "name": "Brazilian Digital",
    "symbol": "BRZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8472/thumb/MicrosoftTeams-image_%286%29.png?1696508657"
  },
  {
    "chainId": 1,
    "address": "0x56694577564fdd577a0abb20fe95c1e2756c2a11",
    "name": "AdaSwap",
    "symbol": "ASW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24453/thumb/rA5cmPtX_400x400.jpg?1696523633"
  },
  {
    "chainId": 1,
    "address": "0x1afb69dbc9f54d08dab1bd3436f8da1af819e647",
    "name": "Melos Studio",
    "symbol": "MELOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24411/thumb/18551.png?1696523593"
  },
  {
    "chainId": 1,
    "address": "0x7bef710a5759d197ec0bf621c3df802c2d60d848",
    "name": "SHOPX",
    "symbol": "SHOPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14600/thumb/shopx_icon_.png?1696514279"
  },
  {
    "chainId": 1,
    "address": "0x436da116249044e8b4464f0cf21dd93311d88190",
    "name": "Colizeum",
    "symbol": "ZEUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24448/thumb/AB0cGpnx_400x400.jpg?1696523629"
  },
  {
    "chainId": 1,
    "address": "0x52a8845df664d76c69d2eea607cd793565af42b8",
    "name": "ApeX",
    "symbol": "APEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25266/thumb/CxpMECpk_400x400_%281%29.png?1696524406"
  },
  {
    "chainId": 1,
    "address": "0xaa6e8127831c9de45ae56bb1b0d4d4da6e5665bd",
    "name": "Index Coop   ETH 2x Flexible Leverage I",
    "symbol": "ETH2X-FLI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14392/thumb/ETH2x-FLI_%281%29.png?1696514083"
  },
  {
    "chainId": 1,
    "address": "0x104f3152d8ebfc3f679392977356962ff36566ac",
    "name": "ChainPort",
    "symbol": "PORTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24490/thumb/VE-tUL-q_400x400.png?1696523670"
  },
  {
    "chainId": 1,
    "address": "0x28cca76f6e8ec81e4550ecd761f899110b060e97",
    "name": "ArGoApp",
    "symbol": "ARGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15477/thumb/Group_118.png?1696515122"
  },
  {
    "chainId": 1,
    "address": "0x4cc19356f2d37338b9802aa8e8fc58b0373296e7",
    "name": "SelfKey",
    "symbol": "KEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2034/thumb/selfkey.png?1696503004"
  },
  {
    "chainId": 1,
    "address": "0x6570ffe19da7e2b425329b157d9109b87f18304b",
    "name": "UNIUM",
    "symbol": "UNM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24567/thumb/Unium_logo.png?1696523743"
  },
  {
    "chainId": 1,
    "address": "0xc8de43bfe33ff496fa14c270d9cb29bda196b9b5",
    "name": "Big Eyes",
    "symbol": "BIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30824/thumb/200-200.jpg?1696529677"
  },
  {
    "chainId": 1,
    "address": "0xc1f33e0cf7e40a67375007104b929e49a581bafe",
    "name": "Spot",
    "symbol": "SPOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28426/thumb/SPOT_Logo_200x200_sq_small_centered.png?1696527423"
  },
  {
    "chainId": 1,
    "address": "0xae36155a55f04a696b8362777620027882b31db5",
    "name": "Kishimoto",
    "symbol": "KISHIMOTO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28011/thumb/kishimoto.png?1696527027"
  },
  {
    "chainId": 1,
    "address": "0x5a3e6a77ba2f983ec0d371ea3b475f8bc0811ad5",
    "name": "0x0 ai  AI Smart Contract",
    "symbol": "0X0",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28880/thumb/0x0.png?1696527857"
  },
  {
    "chainId": 1,
    "address": "0x1d96fd43ee07aa79f8fd003cbdf404fb5ce41ad2",
    "name": "Qawalla",
    "symbol": "QWLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15242/thumb/qwla.png?1696514896"
  },
  {
    "chainId": 1,
    "address": "0x594daad7d77592a2b97b725a7ad59d7e188b5bfa",
    "name": "Apu Apustaja",
    "symbol": "APU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35986/thumb/200x200.png?1710308147"
  },
  {
    "chainId": 1,
    "address": "0xfa5047c9c78b8877af97bdcb85db743fd7313d4a",
    "name": "Rook",
    "symbol": "ROOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13005/thumb/keeper_dao_logo.jpg?1696512794"
  },
  {
    "chainId": 1,
    "address": "0xcd17fa52528f37facb3028688e62ec82d9417581",
    "name": "Materium",
    "symbol": "MTRM",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/24613/thumb/mtrm.png?1696523785"
  },
  {
    "chainId": 1,
    "address": "0x36c79f0b8a2e8a3c0230c254c452973e7a3ba155",
    "name": "BetAI",
    "symbol": "BAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35714/thumb/betai.jpg?1709630399"
  },
  {
    "chainId": 1,
    "address": "0x849c479d7a90eb378dbd00e8f166371176244eb1",
    "name": "MUU",
    "symbol": "MUU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25620/thumb/muu.jpg?1696524754"
  },
  {
    "chainId": 1,
    "address": "0xb70eaf5d316192881aac8786c90b7907b83f02e8",
    "name": "MetaReset",
    "symbol": "RESET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32314/thumb/reset.jpg?1697271138"
  },
  {
    "chainId": 1,
    "address": "0xde075d9adbd0240b4462f124af926452ad0bac91",
    "name": "Bubblefong",
    "symbol": "BBF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24634/thumb/BBF-Token-200x200.png?1696523805"
  },
  {
    "chainId": 1,
    "address": "0x1e2f15302b90edde696593607b6bd444b64e8f02",
    "name": "Shiryo",
    "symbol": "SHIRYO-INU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19652/thumb/shiryo_inu.png?1696519080"
  },
  {
    "chainId": 1,
    "address": "0x5fa54fddf1870c344dbfabb37dfab8700ec0def1",
    "name": "FrogeX",
    "symbol": "FROGEX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14775/thumb/-p8cz7Bk_400x400.png?1696514444"
  },
  {
    "chainId": 1,
    "address": "0x821a278dfff762c76410264303f25bf42e195c0c",
    "name": "pETH",
    "symbol": "PETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28376/thumb/peth.png?1696527377"
  },
  {
    "chainId": 1,
    "address": "0x3209d14ff61766359e64aceff91877cec2ad968e",
    "name": "CouponBay",
    "symbol": "CUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24696/thumb/logo_200.png?1696523862"
  },
  {
    "chainId": 1,
    "address": "0xa318a1277e26dfcdf6f4095b90d137743d900e61",
    "name": "SmartAudit AI",
    "symbol": "AUDIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35567/thumb/SmartAudit_AI.png?1709183104"
  },
  {
    "chainId": 1,
    "address": "0x2596825a84888e8f24b747df29e11b5dd03c81d7",
    "name": "Faith Tribe",
    "symbol": "FTRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23939/thumb/ym1Hf4x2_400x400.jpg?1696523135"
  },
  {
    "chainId": 1,
    "address": "0xaeb3607ec434454ceb308f5cd540875efb54309a",
    "name": "Sturdy",
    "symbol": "STRDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30581/thumb/SF-Dark-Logomark.png?1696529445"
  },
  {
    "chainId": 1,
    "address": "0x4086e77c5e993fdb90a406285d00111a974f877a",
    "name": "Blockchain Brawlers",
    "symbol": "BRWL",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/24701/thumb/fuDdvI8Y_400x400.png?1696523867"
  },
  {
    "chainId": 1,
    "address": "0x2ecba91da63c29ea80fbe7b52632ca2d1f8e5be0",
    "name": "FairERC20",
    "symbol": "FERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30698/thumb/fair.png?1696529568"
  },
  {
    "chainId": 1,
    "address": "0xfbe878ced08132bd8396988671b450793c44bc12",
    "name": "Fox Trading",
    "symbol": "FOXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5182/thumb/foxtrading-logo.png?1547040619"
  },
  {
    "chainId": 1,
    "address": "0x9e976f211daea0d652912ab99b0dc21a7fd728e4",
    "name": "MAP Protocol",
    "symbol": "MAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10085/thumb/53819931.png?1696510114"
  },
  {
    "chainId": 1,
    "address": "0xa8b61cff52564758a204f841e636265bebc8db9b",
    "name": "Yield Protocol",
    "symbol": "YIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14220/thumb/yield.png?1696513935"
  },
  {
    "chainId": 1,
    "address": "0x7712c34205737192402172409a8f7ccef8aa2aec",
    "name": "BlackRock USD Institutional Digital Liq",
    "symbol": "BUIDL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/36291/thumb/blackrock.png?1711013223"
  },
  {
    "chainId": 1,
    "address": "0x6069c9223e8a5da1ec49ac5525d4bb757af72cd8",
    "name": "MUSK Gold",
    "symbol": "MUSK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21696/thumb/musk-icon-200x200.png?1696521051"
  },
  {
    "chainId": 1,
    "address": "0xdc0327d50e6c73db2f8117760592c8bbf1cdcf38",
    "name": "Stronger",
    "symbol": "STRNGR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24922/thumb/200x200_logo.png?1696524078"
  },
  {
    "chainId": 1,
    "address": "0x3cbb7f5d7499af626026e96a2f05df806f2200dc",
    "name": "PandaDAO",
    "symbol": "PANDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24926/thumb/PandaDAO_logo_512.png?1696524082"
  },
  {
    "chainId": 1,
    "address": "0x0018d5e01e53878f90feab02f1b2019a21adf8b1",
    "name": "Shadowcats",
    "symbol": "SHADOWCATS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29140/thumb/Shadowcats.png?1696528101"
  },
  {
    "chainId": 1,
    "address": "0x5bffe8ddff21ca52d8371b14a6c39d6ae3c5d2c7",
    "name": "Hola Token",
    "symbol": "HOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31412/thumb/Hola_Logo.jpg?1696530227"
  },
  {
    "chainId": 1,
    "address": "0x3c48ca59bf2699e51d4974d4b6d284ae52076e5e",
    "name": "Capital DAO Starter",
    "symbol": "CDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20332/thumb/cds.png?1696519736"
  },
  {
    "chainId": 1,
    "address": "0xb840d10d840ef47c233fec1fd040f5b145a6dfa5",
    "name": "STREETH",
    "symbol": "STREETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24954/thumb/streeth.png?1696524108"
  },
  {
    "chainId": 1,
    "address": "0x382a1667c9062f0621362f49076ef6e4fe4c9ec7",
    "name": "RushCoin",
    "symbol": "RUSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25053/thumb/Kax1yBSz_400x400.png?1696524204"
  },
  {
    "chainId": 1,
    "address": "0x9f284e1337a815fe77d2ff4ae46544645b20c5ff",
    "name": "Darwinia Commitment",
    "symbol": "KTON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11895/thumb/KTON.png?1696511761"
  },
  {
    "chainId": 1,
    "address": "0xd78cb66b3affd27569782737fa5b842277e1add7",
    "name": "GTROK",
    "symbol": "GTROK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35643/thumb/20240229_144143.jpg?1709368626"
  },
  {
    "chainId": 1,
    "address": "0xa1a4e303e9c56962f201c5e834abc1e677a3c4f3",
    "name": "CVNX",
    "symbol": "CVNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/862/thumb/CVNX-logo.png?1696501996"
  },
  {
    "chainId": 1,
    "address": "0xb2a63a5dd36c91ec2da59b188ff047f66fac122a",
    "name": "Alpha Impact",
    "symbol": "FOLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19988/thumb/folo.PNG?1696519411"
  },
  {
    "chainId": 1,
    "address": "0xb94acdf8662cd955f137e0c9c9fba535c87b57b4",
    "name": "Mona Token",
    "symbol": "LISA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32376/thumb/monatoken.png?1698035251"
  },
  {
    "chainId": 1,
    "address": "0x19f5d973335480d26270dc498459e2f39cf2da53",
    "name": "Fomo Inu",
    "symbol": "FINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32377/thumb/FINU-Logo.png?1698035474"
  },
  {
    "chainId": 1,
    "address": "0x22987407fd1fc5a971e3fda3b3e74c88666cda91",
    "name": "Smart Reward Token",
    "symbol": "SRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25035/thumb/me-44EyS_400x400.jpg?1696524187"
  },
  {
    "chainId": 1,
    "address": "0x329cf160f30d21006bcd24b67eade561e54cde4c",
    "name": "CareCoin",
    "symbol": "CARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28378/thumb/46FD299C-3DA4-43E6-A6A2-27F492FC7E4D.jpeg?1696527378"
  },
  {
    "chainId": 1,
    "address": "0x0501b9188436e35bb10f35998c40adc079003866",
    "name": "AI Analysis Token",
    "symbol": "AIAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33672/thumb/aia-logo-icon200.png?1702702028"
  },
  {
    "chainId": 1,
    "address": "0x7c5095bb2dae81bb9a21ee9f1b7815cd710194e5",
    "name": "Obama6900",
    "symbol": "OBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32378/thumb/obamasmall.jpg?1698036300"
  },
  {
    "chainId": 1,
    "address": "0x3fab0bbaa03bceaf7c49e2b12877db0142be65fc",
    "name": "Castello Coin",
    "symbol": "CAST",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25216/thumb/19705.png?1696524359"
  },
  {
    "chainId": 1,
    "address": "0x24e89bdf2f65326b94e36978a7edeac63623dafa",
    "name": "Tiger King Coin",
    "symbol": "TKING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15605/thumb/tigerking.png?1696515239"
  },
  {
    "chainId": 1,
    "address": "0xc229c69eb3bb51828d0caa3509a05a51083898dd",
    "name": "Pintu",
    "symbol": "PTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20281/thumb/image_1_8dd79a68aa.png?1696519686"
  },
  {
    "chainId": 1,
    "address": "0x4674672bcddda2ea5300f5207e1158185c944bc0",
    "name": "Gem Exchange and Trading",
    "symbol": "GXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12226/thumb/a.png?1696512059"
  },
  {
    "chainId": 1,
    "address": "0xef952363c1d990a2fa58f8b379a9fa33bad1dfd1",
    "name": "LynKey",
    "symbol": "LYNK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25069/thumb/oOcTsBsg_400x400_%281%29.jpg?1696524217"
  },
  {
    "chainId": 1,
    "address": "0x6a8fee0e33cb65a7e8d21badca62e87639ef74b3",
    "name": "PDX Coin",
    "symbol": "PDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25095/thumb/vBi23nWw_400x400.jpg?1696524246"
  },
  {
    "chainId": 1,
    "address": "0xf67366e83cc9b115ef8cca93baed1f03e6d3ca9a",
    "name": "MindVerse",
    "symbol": "MVERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35875/thumb/photo_2024-03-08_14.24.00.jpeg?1709981737"
  },
  {
    "chainId": 1,
    "address": "0xf81421fc15300c5a8cca9afe12f5cbad502fa756",
    "name": "Cardiocoin",
    "symbol": "CRDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25135/thumb/gQiyJrGcTRjEN-6bI22FOtHOmYF8-_MMYS_U1gIlUm1wf6QxiGnZ64tIeGNNFk9z5xPPch5FMie2grAN4fXQfcjKI9L0EQDVzDEdYDfqWl8ch4ree_q4pypGPNHeSCxnynfOOLAvd9XbKguJHEJNTLAdSKnWCKu92Kho49SplFvDBplt2ruvOz1v5QB795cnjZg4ZemfgfAB_ipJU1YE4TA3uL.jpg?1696524285"
  },
  {
    "chainId": 1,
    "address": "0xd87de4ccef2c2fe651bc4d130cb1a365248f21fa",
    "name": "Lyfe",
    "symbol": "LYFE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28608/thumb/lyfe-32.png?1696527594"
  },
  {
    "chainId": 1,
    "address": "0xf5cfbc74057c610c8ef151a439252680ac68c6dc",
    "name": "Octopus Network",
    "symbol": "OCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18025/thumb/octopus_network.png?1696517541"
  },
  {
    "chainId": 1,
    "address": "0x19e2a43fbbc643c3b2d9667d858d49cad17bc2b5",
    "name": "BNS",
    "symbol": "BNS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25150/thumb/bitbns_logo.png?1696524299"
  },
  {
    "chainId": 1,
    "address": "0x88303fed02b31db9c7a9eafb711da9ef4a03e5d3",
    "name": "Ziktalk",
    "symbol": "ZIK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8991/thumb/ziktalk.logo.jpeg?1696509132"
  },
  {
    "chainId": 1,
    "address": "0xc3dca8f61b275d1e88f3ea31b3e311c49f56b24b",
    "name": "NPick Block",
    "symbol": "NPICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25151/thumb/Logo200_%281%29.png?1696524300"
  },
  {
    "chainId": 1,
    "address": "0xdfddf7a69716124bc346ba556d4b9f9e74c4a8bc",
    "name": "Succession",
    "symbol": "SCCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25377/thumb/sccn.png?1696524510"
  },
  {
    "chainId": 1,
    "address": "0x7f792db54b0e580cdc755178443f0430cf799aca",
    "name": "Volt Inu",
    "symbol": "VOLT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25201/thumb/logo200.png?1696524344"
  },
  {
    "chainId": 1,
    "address": "0xe0c05ec44775e4ad62cdc2eecdf337aa7a143363",
    "name": "Mancium",
    "symbol": "MANC",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/25084/thumb/IKpuMYuGaMHEE0nrDUwpf6DqLQ5zTfZ1sak6ZqeIe2snV4GqQbh4Eic6hHcPl_OpBmlhcrRJ5kqoGzwB1CezMiDYnwR6rDT08_l8zb3-y3aPhTv7_0x5g4IO9JpQXDViEIqTZ7gYzTXqkcP6MH_uCFxQF-l-ohR0-FdT8O-ZTmNUKm--OuqEf9DVa-miAFp4t2pN-kcJ5Jv8woPj51yk-zna8c.jpg?1696524235"
  },
  {
    "chainId": 1,
    "address": "0x4521c9ad6a3d4230803ab752ed238be11f8b342f",
    "name": "Sanin Inu",
    "symbol": "SANI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25222/thumb/B7r0ocfQ_400x400.jpg?1696524365"
  },
  {
    "chainId": 1,
    "address": "0x823556202e86763853b40e9cde725f412e294689",
    "name": "Altered State Machine",
    "symbol": "ASTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24893/thumb/ASTO.png?1696524051"
  },
  {
    "chainId": 1,
    "address": "0x46e98ffe40e408ba6412beb670507e083c8b95ff",
    "name": "Primate",
    "symbol": "PRIMATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25245/thumb/benji-logo-512x512.png?1696524386"
  },
  {
    "chainId": 1,
    "address": "0x5319e86f0e41a06e49eb37046b8c11d78bcad68c",
    "name": "Zelwin",
    "symbol": "ZLW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11547/thumb/5614.png?1696511447"
  },
  {
    "chainId": 1,
    "address": "0x149d8290f653deb8e34c037d239d3d8eee9de5ad",
    "name": "Kingdomverse",
    "symbol": "KING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28607/thumb/_King_logo_%28Color%29.png?1696527593"
  },
  {
    "chainId": 1,
    "address": "0x1122b6a0e00dce0563082b6e2953f3a943855c1f",
    "name": "CENNZnet",
    "symbol": "CENNZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3408/thumb/CENNZ_ticker.png?1696504105"
  },
  {
    "chainId": 1,
    "address": "0xb7b1570e26315baad369b8ea0a943b7f140db9eb",
    "name": "DEEPSPACE",
    "symbol": "DPS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17953/thumb/f1LFu897_400x400.jpg?1696517473"
  },
  {
    "chainId": 1,
    "address": "0x0e29e5abbb5fd88e28b2d355774e73bd47de3bcd",
    "name": "Hakka Finance",
    "symbol": "HAKKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12163/thumb/Hakka-icon.png?1696512001"
  },
  {
    "chainId": 1,
    "address": "0x547b2f82cecfab9c2b1d36fdda96ef9f58c63b8c",
    "name": "Taxa Network",
    "symbol": "TXT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x0262e9374e95b9667b78136c3897cb4e4ef7f0c2",
    "name": "Medifakt",
    "symbol": "FAKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25248/thumb/fxOi9ZYI_400x400.png?1696524389"
  },
  {
    "chainId": 1,
    "address": "0xc18c07a18198a6340cf4d94855fe5eb6dd33b46e",
    "name": "QLINDO",
    "symbol": "QLINDO",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/26851/thumb/Qlindo_Token_Green.png?1696525910"
  },
  {
    "chainId": 1,
    "address": "0xe5caef4af8780e59df925470b050fb23c43ca68c",
    "name": "Ferrum Network",
    "symbol": "FRM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/8251/thumb/FRM.png?1696508455"
  },
  {
    "chainId": 1,
    "address": "0x13a0599c493cc502ed8c4dd26f22f5cedc248fc4",
    "name": "Export Motors Platform",
    "symbol": "EMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24422/thumb/emp_logo_200.png?1696523603"
  },
  {
    "chainId": 1,
    "address": "0x0ede75b5f548e0d37f494368f4fa4982b6d0630a",
    "name": "Meg4mint",
    "symbol": "MEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31714/thumb/MEG_200px.png?1696530536"
  },
  {
    "chainId": 1,
    "address": "0xc353bf07405304aeab75f4c2fac7e88d6a68f98e",
    "name": "Hope money",
    "symbol": "HOPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30971/thumb/HOPE.png?1696529810"
  },
  {
    "chainId": 1,
    "address": "0xa62cc35625b0c8dc1faea39d33625bb4c15bd71c",
    "name": "StormX",
    "symbol": "STMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1369/thumb/StormX.png?1696502427"
  },
  {
    "chainId": 1,
    "address": "0xc775c0c30840cb9f51e21061b054ebf1a00acc29",
    "name": "Pastel",
    "symbol": "PSL",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/14293/thumb/A4qGwEij_400x400.jpg?1696513990"
  },
  {
    "chainId": 1,
    "address": "0x274e7eb07b485cfde53d02270555213447570ac6",
    "name": "SubDAO",
    "symbol": "GOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28379/thumb/gov.png?1696527379"
  },
  {
    "chainId": 1,
    "address": "0xcba78d126f0b1feda0c538bcaf4c852a7a171099",
    "name": "MOE",
    "symbol": "MOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31146/thumb/symbol-200.png?1696529975"
  },
  {
    "chainId": 1,
    "address": "0x08a1c30bbb26425c1031ee9e43fa0b9960742539",
    "name": "LandX Governance Token",
    "symbol": "LNDX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33565/thumb/LNDX-200.png?1702445947"
  },
  {
    "chainId": 1,
    "address": "0x865ec58b06bf6305b886793aa20a2da31d034e68",
    "name": "Mossland",
    "symbol": "MOC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2374/thumb/moc.png?1696503240"
  },
  {
    "chainId": 1,
    "address": "0x88df592f8eb5d7bd38bfef7deb0fbc02cf3778a0",
    "name": "Tellor Tributes",
    "symbol": "TRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9644/thumb/Blk_icon_current.png?1696509713"
  },
  {
    "chainId": 1,
    "address": "0x524ebc93beef838f70b4ae54b675d3e971d5884e",
    "name": "Challenge Coin",
    "symbol": "HERO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26567/thumb/bug-CC-200.png?1696525640"
  },
  {
    "chainId": 1,
    "address": "0x4d4f3715050571a447fffa2cd4cf091c7014ca5c",
    "name": "Summer",
    "symbol": "SUMMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25365/thumb/summer.png?1696524498"
  },
  {
    "chainId": 1,
    "address": "0x4c3bae16c79c30eeb1004fb03c878d89695e3a99",
    "name": "Autumn",
    "symbol": "AUTUMN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25366/thumb/autumn.png?1696524499"
  },
  {
    "chainId": 1,
    "address": "0xccba0b2bc4babe4cbfb6bd2f1edc2a9e86b7845f",
    "name": "Winter",
    "symbol": "WINTER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25367/thumb/winter.png?1696524500"
  },
  {
    "chainId": 1,
    "address": "0x1a3cbda3853494acab67648ee59afeb7ec3e9334",
    "name": "Collateral Network",
    "symbol": "COLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29770/thumb/IMG_20230407_101929_020.png?1696528701"
  },
  {
    "chainId": 1,
    "address": "0x36880f14af2e85cae8e467827fa077d6bf12ea56",
    "name": "Jared From Subway",
    "symbol": "JARED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30890/thumb/Jared_Logo_200_X_200.png?1696529737"
  },
  {
    "chainId": 1,
    "address": "0xf9c53268e9de692ae1b2ea5216e24e1c3ad7cb1e",
    "name": "Idexo",
    "symbol": "IDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18523/thumb/qOiqm7T8_400x400.jpg?1696518004"
  },
  {
    "chainId": 1,
    "address": "0x310c8f00b9de3c31ab95ea68feb6c877538f7947",
    "name": "Undead Blocks",
    "symbol": "UNDEAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25382/thumb/AB0Uemhlw6jR.png?1696524515"
  },
  {
    "chainId": 1,
    "address": "0x14a32f050facf226ec60882398a9bf36d91dbac2",
    "name": "Kyoko",
    "symbol": "KYOKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24454/thumb/kyoko.png?1696523634"
  },
  {
    "chainId": 1,
    "address": "0xb668473944d2e25b6af6d46917eb0233dbac53ae",
    "name": "Neton",
    "symbol": "NTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25418/thumb/lqT0CJNj_400x400.jpg?1696524549"
  },
  {
    "chainId": 1,
    "address": "0x4d4d883f920f7c0c36a1be71a02aa0cde2aa22d1",
    "name": "Opticash",
    "symbol": "OPCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29362/thumb/opticash_logo_256*256.jpg?1696528310"
  },
  {
    "chainId": 1,
    "address": "0x4bd70556ae3f8a6ec6c4080a0c327b24325438f3",
    "name": "HXRO",
    "symbol": "HXRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7805/thumb/X_3D_Black_%2810%29.png?1696508044"
  },
  {
    "chainId": 1,
    "address": "0xa31b1767e09f842ecfd4bc471fe44f830e3891aa",
    "name": "Roobee",
    "symbol": "ROOBEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8791/thumb/Group_11.png?1696508946"
  },
  {
    "chainId": 1,
    "address": "0x83249c6794bca5a77eb8c0af9f1a86e055459cea",
    "name": "GigaSwap",
    "symbol": "GIGA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26919/thumb/final_62fcaacdbcc9640975d3fb7c_822256.png?1696525976"
  },
  {
    "chainId": 1,
    "address": "0x69e8b9528cabda89fe846c67675b5d73d463a916",
    "name": "OPEN Governance",
    "symbol": "OPEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13233/thumb/opendao_logo.png?1696513011"
  },
  {
    "chainId": 1,
    "address": "0xd9adfb67381d392c6e9671f64cdd9014bfcd74f2",
    "name": "Morra",
    "symbol": "MORRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33180/thumb/MORRA.png?1705245739"
  },
  {
    "chainId": 1,
    "address": "0xdecade1c6bf2cd9fb89afad73e4a519c867adcf5",
    "name": "Experty Wisdom",
    "symbol": "WIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13133/thumb/n0MTVBrm_400x400.jpg?1696512920"
  },
  {
    "chainId": 1,
    "address": "0x22b48e1f20043d1db5f2a11cbf1d520a4f20b198",
    "name": "Okuru",
    "symbol": "XOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25473/thumb/xot.png?1696524607"
  },
  {
    "chainId": 1,
    "address": "0x4cff49d0a19ed6ff845a9122fa912abcfb1f68a6",
    "name": "WadzPay",
    "symbol": "WTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13083/thumb/200xWadzToken.png?1696512871"
  },
  {
    "chainId": 1,
    "address": "0x355a824bea1adc22733978a3748271e1bbb34130",
    "name": "Metanept",
    "symbol": "NEPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26798/thumb/OH1UiJ16_400x400.jpeg?1696525860"
  },
  {
    "chainId": 1,
    "address": "0x7c07f7abe10ce8e33dc6c5ad68fe033085256a84",
    "name": "Interest Compounding ETH Index",
    "symbol": "ICETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24483/thumb/icETH-token-logo.png?1696523661"
  },
  {
    "chainId": 1,
    "address": "0x0b498ff89709d3838a063f1dfa463091f9801c2b",
    "name": "BTC 2x Flexible Leverage Index",
    "symbol": "BTC2X-FLI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15406/thumb/Copy_of_BTC2x-FLI_token_logo.png?1696515052"
  },
  {
    "chainId": 1,
    "address": "0xa4cb0dce4849bdcad2d553e9e68644cf40e26cce",
    "name": "Baked",
    "symbol": "BAKED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19178/thumb/rebaked-logo-full.png?1696518627"
  },
  {
    "chainId": 1,
    "address": "0x2be056e595110b30ddd5eaf674bdac54615307d9",
    "name": "Airpuff",
    "symbol": "APUFF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36622/thumb/Airpuff_Icon.jpeg?1711980913"
  },
  {
    "chainId": 1,
    "address": "0x9b2b931d6ab97b6a887b2c5d8529537e6fe73ebe",
    "name": "All In",
    "symbol": "ALLIN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28411/thumb/AllIn_V2_%281%29.png?1696527409"
  },
  {
    "chainId": 1,
    "address": "0x103bb3ebc6f61b3db2d6e01e54ef7d9899a2e16b",
    "name": "Arch Ethereum Div  Yield",
    "symbol": "AEDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30483/thumb/AEDY-512x512.png?1696529369"
  },
  {
    "chainId": 1,
    "address": "0x327673ae6b33bd3d90f0096870059994f30dc8af",
    "name": "Lympo Market",
    "symbol": "LMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15155/thumb/coin_%282%29.png?1696514811"
  },
  {
    "chainId": 1,
    "address": "0x4b5f49487ea7b3609b1ad05459be420548789f1f",
    "name": "LeverFi",
    "symbol": "LEVER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26205/thumb/WI72SpBl_400x400.jpeg?1696525291"
  },
  {
    "chainId": 1,
    "address": "0xf34b1db61aca1a371fe97bad2606c9f534fb9d7d",
    "name": "ArbiSmart",
    "symbol": "RBIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21906/thumb/imgonline-com-ua-Resize-cMjOYOwg12bLazt.png?1696521257"
  },
  {
    "chainId": 1,
    "address": "0x42a7797351dfd281a80807196c8508eb70bb2af9",
    "name": "AISociety",
    "symbol": "AIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32861/thumb/AIS_Symbol.png?1699669819"
  },
  {
    "chainId": 1,
    "address": "0xa393473d64d2f9f026b60b6df7859a689715d092",
    "name": "Lattice",
    "symbol": "LTX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13050/thumb/Lattice.jpg?1696512835"
  },
  {
    "chainId": 1,
    "address": "0xe55d97a97ae6a17706ee281486e98a84095d8aaf",
    "name": "AIPad",
    "symbol": "AIPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28894/thumb/JZadeHu.jpeg?1696527870"
  },
  {
    "chainId": 1,
    "address": "0x616e8bfa43f920657b3497dbf40d6b1a02d4608d",
    "name": "Aura BAL",
    "symbol": "AURABAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26538/thumb/auraBAL.png?1696525612"
  },
  {
    "chainId": 1,
    "address": "0x8eb5bd8c9ab0f8ad28e94693f3c889f490be2ab0",
    "name": "PoS 32",
    "symbol": "POS32",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27358/thumb/90532f66-9158-4bd4-8928-bcb8ec012579.png?1696526404"
  },
  {
    "chainId": 1,
    "address": "0x72e9d9038ce484ee986fea183f8d8df93f9ada13",
    "name": "SmartCredit",
    "symbol": "SMARTCREDIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13036/thumb/smartcredit_logo_02_white_a-1.png?1696512823"
  },
  {
    "chainId": 1,
    "address": "0x96e61422b6a9ba0e068b6c5add4ffabc6a4aae27",
    "name": "Iron Bank EUR",
    "symbol": "IBEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17285/thumb/Iron_Bank_Euro.png?1696516839"
  },
  {
    "chainId": 1,
    "address": "0xfb19075d77a0f111796fb259819830f4780f1429",
    "name": "Fenerbah e",
    "symbol": "FB",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17711/thumb/FB_Logo.png?1696517238"
  },
  {
    "chainId": 1,
    "address": "0x06874f973dc3c96dc22a10ef0d0609f877f335ea",
    "name": "STOA Network",
    "symbol": "STA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16710/thumb/RSloE6X1_400x400.jpeg?1696516284"
  },
  {
    "chainId": 1,
    "address": "0x76dbcba41e11784bfa95ce30e529fa533c3fa4ad",
    "name": "Smart Marketing",
    "symbol": "SMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21905/thumb/smt-logo.png?1696521257"
  },
  {
    "chainId": 1,
    "address": "0x85eee30c52b0b379b046fb0f85f4f3dc3009afec",
    "name": "Keep Network",
    "symbol": "KEEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3373/thumb/IuNzUb5b_400x400.jpg?1696504072"
  },
  {
    "chainId": 1,
    "address": "0x2730d6fdc86c95a74253beffaa8306b40fedecbb",
    "name": "UNICORN",
    "symbol": "UNI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9383/thumb/54231438.png?1696509482"
  },
  {
    "chainId": 1,
    "address": "0xd6c67b93a7b248df608a653d82a100556144c5da",
    "name": "ExNetwork",
    "symbol": "EXNT",
    "decimals": 16,
    "logoURI": "https://assets.coingecko.com/coins/images/12328/thumb/exnt_logo.png?1696512156"
  },
  {
    "chainId": 1,
    "address": "0x71dc40668682a124231301414167e4cf7f55383c",
    "name": "Mimir",
    "symbol": "MIMIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19551/thumb/xaq5Xlzg_400x400.jpg?1696518984"
  },
  {
    "chainId": 1,
    "address": "0x80286b475969e032bdfb85639fed4d4f30ea9526",
    "name": "FREECZ",
    "symbol": "FREECZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33242/thumb/logo.jpeg?1701171276"
  },
  {
    "chainId": 1,
    "address": "0x2d80f5f5328fdcb6eceb7cacf5dd8aedaec94e20",
    "name": "AGA",
    "symbol": "AGA",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/12180/thumb/aga-logo.png?1696512017"
  },
  {
    "chainId": 1,
    "address": "0xd8e2d95c8614f28169757cd6445a71c291dec5bf",
    "name": "Grumpy Cat",
    "symbol": "GRUMPYCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30322/thumb/logo_200x200.png?1696529223"
  },
  {
    "chainId": 1,
    "address": "0x3b7f247f21bf3a07088c2d3423f64233d4b069f7",
    "name": "Dynamite",
    "symbol": "DYNMT",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/8951/thumb/dynamite_logo.jpg?1696509098"
  },
  {
    "chainId": 1,
    "address": "0x5abf88cf3444611d13f6d1b39f3f3ee8575c91a2",
    "name": "Super Athletes Token",
    "symbol": "SAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27761/thumb/SAT_200.png?1696526784"
  },
  {
    "chainId": 1,
    "address": "0x8484e645a054586a6d6af60c0ee911d7b5180e64",
    "name": "DYOR",
    "symbol": "DYOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30149/thumb/dyor.jpeg?1696529070"
  },
  {
    "chainId": 1,
    "address": "0xc01b733b2ef479086b80949676a673346b531aa2",
    "name": "League Bot",
    "symbol": "LEAGUE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31222/thumb/logo_lg.png?1696530048"
  },
  {
    "chainId": 1,
    "address": "0x4e4990e997e1df3f6b39ff49384e2e7e99bc55fe",
    "name": "Saudi Bonk",
    "symbol": "SAUDIBONK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34812/thumb/SaudiBonk.jpg?1706122838"
  },
  {
    "chainId": 1,
    "address": "0x828eb9a1934c143808e325754c54ae2c1741ec49",
    "name": "Addy",
    "symbol": "ADDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36294/thumb/Addy_Logo.jpg?1711016689"
  },
  {
    "chainId": 1,
    "address": "0xd4fa1460f537bb9085d22c7bccb5dd450ef28e3a",
    "name": "Populous",
    "symbol": "PPT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/753/thumb/populous.png?1696501907"
  },
  {
    "chainId": 1,
    "address": "0x86b4dbe5d203e634a12364c0e428fa242a3fba98",
    "name": "poundtoken",
    "symbol": "GBPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26591/thumb/1GBP_logo_black_%281%29.png?1712869127"
  },
  {
    "chainId": 1,
    "address": "0x38b7bf4eecf3eb530b1529c9401fc37d2a71a912",
    "name": "ClayStack Staked MATIC",
    "symbol": "CSMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28352/thumb/csMatic.png?1696527356"
  },
  {
    "chainId": 1,
    "address": "0x332809a8bce4a589da992a4ae495f32fbf4089c6",
    "name": "Black Hole",
    "symbol": "BLH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33312/thumb/photo_2023-11-28_22-54-07.png?1701412780"
  },
  {
    "chainId": 1,
    "address": "0xeb953eda0dc65e3246f43dc8fa13f35623bdd5ed",
    "name": "Raini",
    "symbol": "RAINI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14491/thumb/logo-200x200.png?1696514176"
  },
  {
    "chainId": 1,
    "address": "0xdef1ca1fb7fbcdc777520aa7f396b4e015f497ab",
    "name": "CoW Protocol",
    "symbol": "COW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24384/thumb/cow.png?1696523567"
  },
  {
    "chainId": 1,
    "address": "0x351caa9045d65107b9d311d922d15887cfd634e4",
    "name": "Arowana",
    "symbol": "ARW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17114/thumb/Arowana_LOGO_-_1_.png?1696516675"
  },
  {
    "chainId": 1,
    "address": "0x4057db5bd9f67a566aa10e5587b1a964affc6a16",
    "name": "Truefeedback",
    "symbol": "TFBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8842/thumb/5rd7a55q_400x400.png?1696508994"
  },
  {
    "chainId": 1,
    "address": "0xf028adee51533b1b47beaa890feb54a457f51e89",
    "name": "BMCHAIN",
    "symbol": "BMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7419/thumb/bchain_logo.jpg?1696507693"
  },
  {
    "chainId": 1,
    "address": "0x656c00e1bcd96f256f224ad9112ff426ef053733",
    "name": "Efinity",
    "symbol": "EFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16558/thumb/efi-200px_%281%29.png?1696516119"
  },
  {
    "chainId": 1,
    "address": "0x6243d8cea23066d098a15582d81a598b4e8391f4",
    "name": "Reflexer Ungovernance",
    "symbol": "FLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14123/thumb/EAfYdwgd_400x400.jpg?1696513843"
  },
  {
    "chainId": 1,
    "address": "0x24c19f7101c1731b85f1127eaa0407732e36ecdd",
    "name": "SharedStake Governance v2",
    "symbol": "SGTV2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13948/thumb/sgt-png.png?1696513686"
  },
  {
    "chainId": 1,
    "address": "0x4561de8e0c2bba725d38d266ef62426e62678d82",
    "name": "Coniun",
    "symbol": "CONI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29771/thumb/16801884918921679374349437Only-Amblem-23.png?1696528702"
  },
  {
    "chainId": 1,
    "address": "0x74b1af114274335598da72f5c6ed7b954a016eed",
    "name": "HitBTC",
    "symbol": "HIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16395/thumb/JgMkz95.png?1696515993"
  },
  {
    "chainId": 1,
    "address": "0x97abee33cd075c58bfdd174e0885e08e8f03556f",
    "name": "Sentiment",
    "symbol": "SENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16388/thumb/94SwpUOP_400x400.jpg?1696515986"
  },
  {
    "chainId": 1,
    "address": "0x569424c5ee13884a193773fdc5d1c5f79c443a51",
    "name": "Pine",
    "symbol": "PINE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25660/thumb/Logomark-Colour.png?1696524788"
  },
  {
    "chainId": 1,
    "address": "0xc57f1d079c862b70aa12faab19293f827187aaf6",
    "name": "Gilgeous",
    "symbol": "GLG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30241/thumb/GLG_200x200.jpg?1696529150"
  },
  {
    "chainId": 1,
    "address": "0xab93df617f51e1e415b5b4f8111f122d6b48e55c",
    "name": "Delta Exchange",
    "symbol": "DETO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14610/thumb/deto-logo.png?1696514288"
  },
  {
    "chainId": 1,
    "address": "0x1494ca1f11d487c2bbe4543e90080aeba4ba3c2b",
    "name": "DeFi Pulse Index",
    "symbol": "DPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12465/thumb/defi_pulse_index_set.png?1696512284"
  },
  {
    "chainId": 1,
    "address": "0x65ad6a2288b2dd23e466226397c8f5d1794e58fc",
    "name": "GamyFi",
    "symbol": "GFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14559/thumb/circle-cropped_%281%29.png?1696514241"
  },
  {
    "chainId": 1,
    "address": "0xaa602de53347579f86b996d2add74bb6f79462b2",
    "name": "Zipmex",
    "symbol": "ZMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13866/thumb/ZMT_Token.png?1696513611"
  },
  {
    "chainId": 1,
    "address": "0xa92e7c82b11d10716ab534051b271d2f6aef7df5",
    "name": "Ara",
    "symbol": "ARA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13877/thumb/ara-icon-200.png?1696513621"
  },
  {
    "chainId": 1,
    "address": "0x7714f320adca62b149df2579361afec729c5fe6a",
    "name": "Tenup",
    "symbol": "TUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6254/thumb/uphIf44J_400x400.jpg?1696506637"
  },
  {
    "chainId": 1,
    "address": "0xc0eb85285d83217cd7c891702bcbc0fc401e2d9d",
    "name": "Hiveterminal",
    "symbol": "HVN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/899/thumb/Hiveterminal_token.jpg?1696502022"
  },
  {
    "chainId": 1,
    "address": "0x7c5a0ce9267ed19b22f8cae653f198e3e8daf098",
    "name": "Santiment Network",
    "symbol": "SAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/797/thumb/santiment-network-token.png?1696501947"
  },
  {
    "chainId": 1,
    "address": "0x58002a6b6e659a16de9f02f529b10536e307b0d9",
    "name": "Crypto Holding Frank",
    "symbol": "CHFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11184/thumb/chft.png?1696511117"
  },
  {
    "chainId": 1,
    "address": "0x602f65bb8b8098ad804e99db6760fd36208cd967",
    "name": "Mops",
    "symbol": "MOPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26900/thumb/mops.png?1696525958"
  },
  {
    "chainId": 1,
    "address": "0xf920e4f3fbef5b3ad0a25017514b769bdc4ac135",
    "name": "BABB",
    "symbol": "BAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2168/thumb/babb.png?1696503128"
  },
  {
    "chainId": 1,
    "address": "0x8dbf9a4c99580fc7fd4024ee08f3994420035727",
    "name": "ECO",
    "symbol": "ECO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27961/thumb/_ECO.png?1696526980"
  },
  {
    "chainId": 1,
    "address": "0xfcf8eda095e37a41e002e266daad7efc1579bc0a",
    "name": "FLEX Coin",
    "symbol": "FLEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9108/thumb/coinflex_logo.png?1696509233"
  },
  {
    "chainId": 1,
    "address": "0x24ccedebf841544c9e6a62af4e8c2fa6e5a46fde",
    "name": "BlueSparrow",
    "symbol": "BLUESPARROW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26085/thumb/bluesparrow.?1696525176"
  },
  {
    "chainId": 1,
    "address": "0x255aa6df07540cb5d3d297f0d0d4d84cb52bc8e6",
    "name": "Raiden Network",
    "symbol": "RDN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1132/thumb/raiden-logo.jpg?1696502225"
  },
  {
    "chainId": 1,
    "address": "0xe50365f5d679cb98a1dd62d6f6e58e59321bcddf",
    "name": "LA",
    "symbol": "LA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1051/thumb/LA_token.png?1696502156"
  },
  {
    "chainId": 1,
    "address": "0x910dfc18d6ea3d6a7124a6f8b5458f281060fa4c",
    "name": "X8X",
    "symbol": "X8X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1912/thumb/x8x.png?1696502903"
  },
  {
    "chainId": 1,
    "address": "0x68d57c9a1c35f63e2c83ee8e49a64e9d70528d25",
    "name": "Sirin Labs",
    "symbol": "SRN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1872/thumb/sirin-labs-token.png?1696502868"
  },
  {
    "chainId": 1,
    "address": "0x66761fa41377003622aee3c7675fc7b5c1c2fac5",
    "name": "Clearpool",
    "symbol": "CPOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19252/thumb/photo_2022-08-31_12.45.02.jpeg?1696518697"
  },
  {
    "chainId": 1,
    "address": "0xd98f75b1a3261dab9eed4956c93f33749027a964",
    "name": "Share",
    "symbol": "SHR",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/3609/thumb/74586729_2443914875881351_2785018663453851648_n.png?1696504289"
  },
  {
    "chainId": 1,
    "address": "0xfe5f141bf94fe84bc28ded0ab966c16b17490657",
    "name": "Libra Credit",
    "symbol": "LBA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3673/thumb/libra-credit.png?1696504352"
  },
  {
    "chainId": 1,
    "address": "0xa6ed258e3126be568a5f53ade7b6a1e478e5ef98",
    "name": "Exynos Protocol",
    "symbol": "XYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33652/thumb/photo_2023-12-09_01-43-27_%281%29.jpg?1702544060"
  },
  {
    "chainId": 1,
    "address": "0x4d67edef87a5ff910954899f4e5a0aaf107afd42",
    "name": "BlueSparrow  OLD ",
    "symbol": "BLUESPARROW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19995/thumb/bluesparrow.png?1696519417"
  },
  {
    "chainId": 1,
    "address": "0x08f7be99ed83369541501d60f4e66f8e34c3f736",
    "name": "Cryptoku",
    "symbol": "CKU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25813/thumb/Cryptoku.png?1696524898"
  },
  {
    "chainId": 1,
    "address": "0x11ebe21e9d7bf541a18e1e3ac94939018ce88f0b",
    "name": "Pitch FXS",
    "symbol": "PITCHFXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28414/thumb/pitch.png?1696527412"
  },
  {
    "chainId": 1,
    "address": "0x304645590f197d99fad9fa1d05e7bcdc563e1378",
    "name": "Guardian AI",
    "symbol": "GUARDIAN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33653/thumb/IMG_7686.PNG?1702545311"
  },
  {
    "chainId": 1,
    "address": "0x7df571694b35ef60e9b6651967486c8017491f9b",
    "name": "DOGE 1 Moon Mission",
    "symbol": "DOGE-1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33973/thumb/200x200.jpg?1703577524"
  },
  {
    "chainId": 1,
    "address": "0x1bed97cbc3c24a4fb5c069c6e311a967386131f7",
    "name": "Yearn Ether",
    "symbol": "YETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32220/thumb/yETH-128px.png?1696914909"
  },
  {
    "chainId": 1,
    "address": "0x973e52691176d36453868d9d86572788d27041a9",
    "name": "DxChain",
    "symbol": "DX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4700/thumb/VdZwy0Pv_400x400.png?1696505265"
  },
  {
    "chainId": 1,
    "address": "0x903bef1736cddf2a537176cf3c64579c3867a881",
    "name": "Legacy ICHI",
    "symbol": "ICHI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26004/thumb/legacy.png?1696525081"
  },
  {
    "chainId": 1,
    "address": "0x986ee2b944c42d017f52af21c4c69b84dbea35d8",
    "name": "BitMart",
    "symbol": "BMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5236/thumb/bitmart-token.png?1696505741"
  },
  {
    "chainId": 1,
    "address": "0xd96e84ddbc7cbe1d73c55b6fe8c64f3a6550deea",
    "name": "Gemach",
    "symbol": "GMAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32334/thumb/gmac-200x200.png?1697452788"
  },
  {
    "chainId": 1,
    "address": "0xa97cd1c5bcee878496fadb4edee081d05239051f",
    "name": "AICB",
    "symbol": "AICB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32348/thumb/AICB-20230925162643.png?1697471641"
  },
  {
    "chainId": 1,
    "address": "0xff742d05420b6aca4481f635ad8341f81a6300c2",
    "name": "AscendEx",
    "symbol": "ASD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5003/thumb/bitmax.png?1696505535"
  },
  {
    "chainId": 1,
    "address": "0x41e5560054824ea6b0732e656e3ad64e20e94e45",
    "name": "Civic",
    "symbol": "CVC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/788/thumb/civic-orange.png?1696501939"
  },
  {
    "chainId": 1,
    "address": "0x6e605c269e0c92e70beeb85486f1fc550f9380bd",
    "name": "Catex",
    "symbol": "CATT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6353/thumb/catex.png?1696506725"
  },
  {
    "chainId": 1,
    "address": "0xea60cd69f2b9fd6eb067bddbbf86a5bdeffbbc55",
    "name": "Wecan",
    "symbol": "WECAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32150/thumb/Wecan_W_logo_RGB.png?1702947005"
  },
  {
    "chainId": 1,
    "address": "0x4be10da47a07716af28ad199fbe020501bddd7af",
    "name": "XT com",
    "symbol": "XT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8391/thumb/XT_token.jpg?1696508583"
  },
  {
    "chainId": 1,
    "address": "0xe6d2c3cb986db66818c14c7032db05d1d2a6ee74",
    "name": "Finexbox",
    "symbol": "FNB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/8419/thumb/p1WP-viw_400x400.jpg?1696508608"
  },
  {
    "chainId": 1,
    "address": "0xb4bd4628e6efb0cb521d9ec35050c75840320374",
    "name": "frETH",
    "symbol": "FRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28734/thumb/freth.png?1696527714"
  },
  {
    "chainId": 1,
    "address": "0x77d0cb0ab54f9e74b9405a5b3f60da06a78f1aad",
    "name": "Wrapped Millix",
    "symbol": "WMLX",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/28703/thumb/wrapped_millix-200x200_300-dpi_GC.png?1696527685"
  },
  {
    "chainId": 1,
    "address": "0x1b073382e63411e3bcffe90ac1b9a43fefa1ec6f",
    "name": "Bitpanda Ecosystem",
    "symbol": "BEST",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/8738/thumb/BEST-Coin-Logo.png?1696508897"
  },
  {
    "chainId": 1,
    "address": "0x11eef04c884e24d9b7b4760e7476d06ddf797f36",
    "name": "MX",
    "symbol": "MX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8545/thumb/MEXC_GLOBAL_LOGO.jpeg?1696508719"
  },
  {
    "chainId": 1,
    "address": "0x8ab7404063ec4dbcfd4598215992dc3f8ec853d7",
    "name": "Akropolis",
    "symbol": "AKRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3328/thumb/Akropolis.png?1696504034"
  },
  {
    "chainId": 1,
    "address": "0x426fc8be95573230f6e6bc4af91873f0c67b21b4",
    "name": "BlackPearl",
    "symbol": "BPLC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8931/thumb/EJIpComQ_400x400.png?1696509079"
  },
  {
    "chainId": 1,
    "address": "0x34d6a0f5c2f5d0082141fe73d93b9dd00ca7ce11",
    "name": "Golden",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8939/thumb/4uf1Sim9_400x400.png?1696509087"
  },
  {
    "chainId": 1,
    "address": "0x187d1018e8ef879be4194d6ed7590987463ead85",
    "name": "FUZE",
    "symbol": "FUZE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8941/thumb/logo-fuze-fix-big.png?1696509089"
  },
  {
    "chainId": 1,
    "address": "0xbd89b8d708809e7022135313683663911826977e",
    "name": "O MEE",
    "symbol": "OME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25099/thumb/logo-mark-full-colour-cmyk-693px_300ppi.jpg?1696524249"
  },
  {
    "chainId": 1,
    "address": "0xda5fab7affc6dffd24d60e23153d241a3d9f9603",
    "name": "Cryptiq WEB3",
    "symbol": "CRYPTIQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31583/thumb/cryptiqlogo.png?1702944832"
  },
  {
    "chainId": 1,
    "address": "0xed03ed872159e199065401b6d0d487d78d9464aa",
    "name": "Mexican Peso Tether",
    "symbol": "MXNT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26087/thumb/MXNT_green.jpg?1696525178"
  },
  {
    "chainId": 1,
    "address": "0x23d7ff057c696fee679c60cef61fee6614218f04",
    "name": "Unipoly",
    "symbol": "UNP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32510/thumb/20231018_231410.png?1698398961"
  },
  {
    "chainId": 1,
    "address": "0xe15a66b7b8e385caa6f69fd0d55984b96d7263cf",
    "name": "Arch USD Div  Yield",
    "symbol": "ADDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30484/thumb/ADDY.png?1696529370"
  },
  {
    "chainId": 1,
    "address": "0xa52bffad02b1fe3f86a543a4e81962d3b3bb01a7",
    "name": "Duckereum",
    "symbol": "DUCKER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26325/thumb/coin.png?1696525407"
  },
  {
    "chainId": 1,
    "address": "0x6d57b2e05f26c26b549231c866bdd39779e4a488",
    "name": "VNX Gold",
    "symbol": "VNXAU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28019/thumb/VNX_Token_logo_%283%29.png?1696527035"
  },
  {
    "chainId": 1,
    "address": "0x79b87d52a03ed26a19c1914be4ce37f812e2056c",
    "name": "USP Token",
    "symbol": "USP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28855/thumb/USP_Token.png?1696527829"
  },
  {
    "chainId": 1,
    "address": "0xd9fcd98c322942075a5c3860693e9f4f03aae07b",
    "name": "Euler",
    "symbol": "EUL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26149/thumb/Coingecko_logo_%281%29.png?1706026067"
  },
  {
    "chainId": 1,
    "address": "0xdc247546a6551117c8ea82db2cc0ad6e048e5f6e",
    "name": "Lush AI",
    "symbol": "LUSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35342/thumb/tokenIcon200x200.png?1708317375"
  },
  {
    "chainId": 1,
    "address": "0xab2a7b5876d707e0126b3a75ef7781c77c8877ee",
    "name": "Quadency Token",
    "symbol": "QUAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21364/thumb/GkMXE_7A_400x400.png?1696520730"
  },
  {
    "chainId": 1,
    "address": "0xa8c8cfb141a3bb59fea1e2ea6b79b5ecbcd7b6ca",
    "name": "Syntropy",
    "symbol": "NOIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3269/thumb/Component_1.png?1696503982"
  },
  {
    "chainId": 1,
    "address": "0x1250b98cbde9f99f4c42dcdacee193221f17eb50",
    "name": "ArtCoin",
    "symbol": "AC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26127/thumb/ngiBSUT7_400x400.png?1696525216"
  },
  {
    "chainId": 1,
    "address": "0xadf86e75d8f0f57e0288d0970e7407eaa49b3cab",
    "name": "Apollo Crypto",
    "symbol": "APOLLO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22606/thumb/apollo-icon-green-black-w3-200.png?1696521922"
  },
  {
    "chainId": 1,
    "address": "0x6595b8fd9c920c81500dca94e53cdc712513fb1f",
    "name": "Olyverse",
    "symbol": "OLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13983/thumb/OLY-LOGO.png?1696513715"
  },
  {
    "chainId": 1,
    "address": "0x473037de59cf9484632f4a27b509cfe8d4a31404",
    "name": "STEPN Green Satoshi Token on ETH",
    "symbol": "GST-ETH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/26528/thumb/gst_eth.png?1696525600"
  },
  {
    "chainId": 1,
    "address": "0x1258d60b224c0c5cd888d37bbf31aa5fcfb7e870",
    "name": "NodeAI",
    "symbol": "GPU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35432/thumb/NodeAI.png?1708586757"
  },
  {
    "chainId": 1,
    "address": "0xdc7d16b1e7c54f35a67af95d5a6eecaec27b2a62",
    "name": "Pulse AI",
    "symbol": "PULSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36331/thumb/photo_2024-03-17_22.26.56.jpeg?1711099075"
  },
  {
    "chainId": 1,
    "address": "0x930ae5999210724248b36265e8d3696128115946",
    "name": "HarryPotterRussellSonic1Inu",
    "symbol": "SAITAMA",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31506/thumb/200x200.png?1696530317"
  },
  {
    "chainId": 1,
    "address": "0x3cb48aeb3d1abadc23d2d8a6894b3a68338381c2",
    "name": "PaladinAI",
    "symbol": "PALAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33703/thumb/Design_sans_titre_%288%29_%282%29.png?1702826502"
  },
  {
    "chainId": 1,
    "address": "0xe6f47303032a09c8c0f8ebb713c00e6ed345e8c3",
    "name": "Risitas  OLD ",
    "symbol": "RISITA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30333/thumb/200x200.png?1696529234"
  },
  {
    "chainId": 1,
    "address": "0xaf5191b0de278c7286d6c7cc6ab6bb8a73ba2cd6",
    "name": "Stargate Finance",
    "symbol": "STG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24413/thumb/STG_LOGO.png?1696523595"
  },
  {
    "chainId": 1,
    "address": "0xcccd1ba9f7acd6117834e0d28f25645decb1736a",
    "name": "ECOx",
    "symbol": "ECOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27962/thumb/_ECOx.png?1696526981"
  },
  {
    "chainId": 1,
    "address": "0xd7f0cc50ad69408ae58be033f4f85d2367c2e468",
    "name": "Vera",
    "symbol": "VERA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18519/thumb/JJXTVFOE_400x400.png?1696518000"
  },
  {
    "chainId": 1,
    "address": "0x4ce4c025692b3142dbde1cd432ef55b9a8d18701",
    "name": "Decanect",
    "symbol": "DCNT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27309/thumb/200-x-200-jpeg.jpg?1696526359"
  },
  {
    "chainId": 1,
    "address": "0xdb8d6d3ac21e4efe3675bbb18514010ac9c5558f",
    "name": "Energreen",
    "symbol": "EGRN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30583/thumb/energreen.png?1696529447"
  },
  {
    "chainId": 1,
    "address": "0xbf9e72eeb5adb8b558334c8672950b7a379d4266",
    "name": "CubToken",
    "symbol": "CUBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26197/thumb/photo-2022-04-20-13-35-28.jpg?1696525283"
  },
  {
    "chainId": 1,
    "address": "0xa6089dbfed19d1bcd43146bbdca2b8f9d9f84a9a",
    "name": "UGOLD Inc ",
    "symbol": "UGOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35921/thumb/IMG_20240307_195222_084.jpg?1710221205"
  },
  {
    "chainId": 1,
    "address": "0x8790f2fc7ca2e7db841307fb3f4e72a03baf7b47",
    "name": "Spillways",
    "symbol": "SPILLWAYS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28358/thumb/1669791775144_%281%29.png?1696527362"
  },
  {
    "chainId": 1,
    "address": "0xec12ba5ac0f259e9ac6fc9a3bc23a76ad2fde5d9",
    "name": "HugeWin",
    "symbol": "HUGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35923/thumb/2024-03-01_17.49.27_%281%29.png?1710221459"
  },
  {
    "chainId": 1,
    "address": "0x5a98fcbea516cf06857215779fd812ca3bef1b32",
    "name": "Lido DAO",
    "symbol": "LDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13573/thumb/Lido_DAO.png?1696513326"
  },
  {
    "chainId": 1,
    "address": "0x70715e133f3b98af9dcb4897657cd4608d395d49",
    "name": "NeuralByte",
    "symbol": "NBT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36528/thumb/photo_2024-03-27_10.53.01.jpeg?1711700775"
  },
  {
    "chainId": 1,
    "address": "0x2598c30330d5771ae9f983979209486ae26de875",
    "name": "Any Inu",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34126/thumb/anyinulogo200.png?1704174269"
  },
  {
    "chainId": 1,
    "address": "0x5d0ebc4ec5ac18d30512fb6287886245061b3dbd",
    "name": "Gatsby Inu",
    "symbol": "GATSBY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30578/thumb/gatsby.jpeg?1696529442"
  },
  {
    "chainId": 1,
    "address": "0xa16a609ff4e1a15b6ccb469e7a5dd14e89305283",
    "name": "Spume",
    "symbol": "SPUME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27089/thumb/D6lMeW9k_400x400.jpeg?1696526138"
  },
  {
    "chainId": 1,
    "address": "0x4dbc96275a78de1bf92facb2c165c55de62adab1",
    "name": "Orange BOT",
    "symbol": "ORBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33786/thumb/0x4dbc96275a78de1bf92facb2c165c55de62adab1.png?1702981687"
  },
  {
    "chainId": 1,
    "address": "0x230f5ed78a45452f726365b8ad1d6866f5faa68f",
    "name": "Cryptoforce",
    "symbol": "COF",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32855/thumb/IMG_20231108_221331_196.png?1699665059"
  },
  {
    "chainId": 1,
    "address": "0x32353a6c91143bfd6c7d363b546e62a9a2489a20",
    "name": "Adventure Gold",
    "symbol": "AGLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18125/thumb/lpgblc4h_400x400.jpg?1696517628"
  },
  {
    "chainId": 1,
    "address": "0x416cdaf616a82d7dd46e0dbf36e7d6fe412bc40e",
    "name": "Luna28",
    "symbol": "LUNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34777/thumb/Logo_%281%29_%281%29.png?1706071100"
  },
  {
    "chainId": 1,
    "address": "0xb3ad645db386d7f6d753b2b9c3f4b853da6890b8",
    "name": "Concentrator",
    "symbol": "CTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26364/thumb/%E6%9C%AA%E5%91%BD%E5%90%8D%E7%9A%84%E8%AE%BE%E8%AE%A1.jpg?1696525441"
  },
  {
    "chainId": 1,
    "address": "0x35b08722aa26be119c1608029ccbc976ac5c1082",
    "name": "Eminer",
    "symbol": "EM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9155/thumb/KDbQPKIu_400x400.png?1696509277"
  },
  {
    "chainId": 1,
    "address": "0xc4c75f2a0cb1a9acc33929512dc9733ea1fd6fde",
    "name": "Martin Shkreli Inu",
    "symbol": "MSI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26365/thumb/jEYEUxUI_400x400.jpeg?1696525442"
  },
  {
    "chainId": 1,
    "address": "0x3f382dbd960e3a9bbceae22651e88158d2791550",
    "name": "Aavegotchi",
    "symbol": "GHST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12467/thumb/GHST.png?1696512286"
  },
  {
    "chainId": 1,
    "address": "0xd0cd466b34a24fcb2f87676278af2005ca8a78c4",
    "name": "Popcorn",
    "symbol": "POP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21438/thumb/pop-1_200_x_200.png?1696520801"
  },
  {
    "chainId": 1,
    "address": "0x6e109e9dd7fa1a58bc3eff667e8e41fc3cc07aef",
    "name": "CNH Tether",
    "symbol": "CNHT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/5251/thumb/cny-tether-logo.png?1696505755"
  },
  {
    "chainId": 1,
    "address": "0xff20817765cb7f73d4bde2e66e067e58d11095c2",
    "name": "Amp",
    "symbol": "AMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12409/thumb/amp-200x200.png?1696512231"
  },
  {
    "chainId": 1,
    "address": "0x727f064a78dc734d33eec18d5370aef32ffd46e4",
    "name": "Orion Money",
    "symbol": "ORION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18630/thumb/YtrqPIWc.png?1696518102"
  },
  {
    "chainId": 1,
    "address": "0xd5930c307d7395ff807f2921f12c5eb82131a789",
    "name": "Bolt",
    "symbol": "BOLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4489/thumb/bolt.png?1696505078"
  },
  {
    "chainId": 1,
    "address": "0x6bea7cfef803d1e3d5f7c0103f7ded065644e197",
    "name": "Gamma Strategies",
    "symbol": "GAMMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21975/thumb/gamma-token-200.png?1696521323"
  },
  {
    "chainId": 1,
    "address": "0x3b9be07d622accaed78f479bc0edabfd6397e320",
    "name": "Lossless",
    "symbol": "LSS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15917/thumb/Group_57.png?1696515531"
  },
  {
    "chainId": 1,
    "address": "0x0107c4aaa31940ef88760acb1f32424ca8d8bd80",
    "name": "Authencity",
    "symbol": "AUTH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/26960/thumb/1.png?1696526015"
  },
  {
    "chainId": 1,
    "address": "0xa52b7ea0365310ca87ab73aef4f2f75c9a999af3",
    "name": "Mass Protocol",
    "symbol": "MASS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33788/thumb/Mass_200x200.jpeg?1702988300"
  },
  {
    "chainId": 1,
    "address": "0xc66cdac744916afb6811c71c277d88de90ce8d5b",
    "name": "CDbio",
    "symbol": "MCD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26957/thumb/mcd.png?1696526012"
  },
  {
    "chainId": 1,
    "address": "0x243da8a5561cf5642b852c4c4c3fada3f3116d2d",
    "name": "DefiPal",
    "symbol": "PAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33789/thumb/imresizer-1702861509134.jpg?1702988404"
  },
  {
    "chainId": 1,
    "address": "0x116c4b65e14449947bc6fa1bbe844cb16a162d53",
    "name": "BMAX",
    "symbol": "BMAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26427/thumb/BMAX_logo.png?1696525501"
  },
  {
    "chainId": 1,
    "address": "0xb83c27805aaca5c7082eb45c868d955cf04c337f",
    "name": "JungleKing TigerCoin",
    "symbol": "TIGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26494/thumb/jungleparty.png?1696525568"
  },
  {
    "chainId": 1,
    "address": "0x2a3bff78b79a009976eea096a51a948a3dc00e34",
    "name": "Wilder World",
    "symbol": "WILD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15407/thumb/wld-logo.png?1712273448"
  },
  {
    "chainId": 1,
    "address": "0x5f9123d661459af6f398b6f1566f53222612541e",
    "name": "MaranBet",
    "symbol": "MARAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29787/thumb/token_maran-01_1_%281%29.png?1696528717"
  },
  {
    "chainId": 1,
    "address": "0x0f51bb10119727a7e5ea3538074fb341f56b09ad",
    "name": "DAO Maker",
    "symbol": "DAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13915/thumb/4.png?1696513656"
  },
  {
    "chainId": 1,
    "address": "0xd4126f195a8de772eeffa61a4ab6dd43462f4e39",
    "name": "Hikari Protocol",
    "symbol": "HIKARI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28953/thumb/HIKARI.png?1696527926"
  },
  {
    "chainId": 1,
    "address": "0xf31120603a27a16314effc37a3f32a42028310af",
    "name": "Flack Exchange",
    "symbol": "FLACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34772/thumb/token_%282%29.png?1705993386"
  },
  {
    "chainId": 1,
    "address": "0x171d76d931529384639bc9aad5b77b77041ed604",
    "name": "MetaOctagon",
    "symbol": "MOTG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26496/thumb/iJoylNuO_400x400.jpeg?1696525570"
  },
  {
    "chainId": 1,
    "address": "0x65e9ed59a6c03e97ae984b6c4ff912448ebd3566",
    "name": "Ryoshi Research",
    "symbol": "RYOSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35346/thumb/Ryoshi-4.png?1708319258"
  },
  {
    "chainId": 1,
    "address": "0xe9a95d175a5f4c9369f3b74222402eb1b837693b",
    "name": "ChangeNOW",
    "symbol": "NOW",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/8224/thumb/now.png?1696508430"
  },
  {
    "chainId": 1,
    "address": "0x4fd51cb87ffefdf1711112b5bd8ab682e54988ea",
    "name": "WPT Investing Corp",
    "symbol": "WPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26431/thumb/200200.png?1696525505"
  },
  {
    "chainId": 1,
    "address": "0x6a6aa13393b7d1100c00a57c76c39e8b6c835041",
    "name": "OpenAI ERC",
    "symbol": "OPENAIERC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28412/thumb/OpenAI.jpeg?1696527410"
  },
  {
    "chainId": 1,
    "address": "0x556d4f40982cb95e0714989e0c229c42be8b1499",
    "name": "Golteum",
    "symbol": "GLTM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28597/thumb/_GLTM-Token-Border200.png?1696527584"
  },
  {
    "chainId": 1,
    "address": "0xfcc5c47be19d06bf83eb04298b026f81069ff65b",
    "name": "Yearn CRV",
    "symbol": "YCRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27622/thumb/yearncrvnew_32.png?1696526653"
  },
  {
    "chainId": 1,
    "address": "0x4aa41bc1649c9c3177ed16caaa11482295fc7441",
    "name": "XFai",
    "symbol": "XFIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14904/thumb/XFAI_Logo200x200.jpg?1696514567"
  },
  {
    "chainId": 1,
    "address": "0xc97232527b62efb0d8ed38cf3ea103a6cca4037e",
    "name": "LP Yearn CRV Vault",
    "symbol": "LP-YCRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27624/thumb/yearncrvlp_32.png?1696526654"
  },
  {
    "chainId": 1,
    "address": "0x5f4361971667c8198706dec8c59cf7607e405c6e",
    "name": "Fileshare Platform",
    "symbol": "FSC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28190/thumb/fsc.png?1696527193"
  },
  {
    "chainId": 1,
    "address": "0x53fd2342b43ecd24aef1535bc3797f509616ce8c",
    "name": "Anarchy",
    "symbol": "ANARCHY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26583/thumb/90EWu8yp_400x400.jpg?1696525658"
  },
  {
    "chainId": 1,
    "address": "0x1cf4592ebffd730c7dc92c1bdffdfc3b9efcf29a",
    "name": "Waves",
    "symbol": "WAVES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/425/thumb/waves.png?1696501700"
  },
  {
    "chainId": 1,
    "address": "0xbb126042235e6bd38b17744cb31a8bf4a206c045",
    "name": "fanC",
    "symbol": "FANC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26500/thumb/hJ6MD49P_400x400.jpeg?1696525574"
  },
  {
    "chainId": 1,
    "address": "0x2a7cad775fd9c5c43f996a948660ffc21b4e628c",
    "name": "D Drops",
    "symbol": "DOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26611/thumb/ddrops_%281%29.png?1696525685"
  },
  {
    "chainId": 1,
    "address": "0x8f1cece048cade6b8a05dfa2f90ee4025f4f2662",
    "name": "Galaxy Fox",
    "symbol": "GFOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34882/thumb/GFOX_LOGO.png?1706499381"
  },
  {
    "chainId": 1,
    "address": "0x5283d291dbcf85356a21ba090e6db59121208b44",
    "name": "Blur",
    "symbol": "BLUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28453/thumb/blur.png?1696527448"
  },
  {
    "chainId": 1,
    "address": "0x227c7df69d3ed1ae7574a1a7685fded90292eb48",
    "name": "Milady Vault  NFTX ",
    "symbol": "MILADY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26540/thumb/256x256.png?1696525614"
  },
  {
    "chainId": 1,
    "address": "0xb755506531786c8ac63b756bab1ac387bacb0c04",
    "name": "ZARP Stablecoin",
    "symbol": "ZARP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27333/thumb/zarp_coin.png?1696526381"
  },
  {
    "chainId": 1,
    "address": "0xed1167b6dc64e8a366db86f2e952a482d0981ebd",
    "name": "Lybra",
    "symbol": "LBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29958/thumb/New_LBR_V2.png?1696528884"
  },
  {
    "chainId": 1,
    "address": "0x1bc71108e898586068caba9ee09397a2d275c092",
    "name": "Carbon Crates",
    "symbol": "CARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35454/thumb/Screenshot_2024-02-23_at_1.26.41%E2%80%AFPM.png?1708666068"
  },
  {
    "chainId": 1,
    "address": "0xd2e5decc08a80be6538f89f9ab8ff296e2f724df",
    "name": "STIMA",
    "symbol": "STIMA",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26101/thumb/STIMA_200x200.png?1696525192"
  },
  {
    "chainId": 1,
    "address": "0x49c8efd98ac8114de2fce73d57e2944aebd5613d",
    "name": "AnonTech",
    "symbol": "ATEC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36949/thumb/1000002162.jpg?1712897959"
  },
  {
    "chainId": 1,
    "address": "0xfeb6d5238ed8f1d59dcab2db381aa948e625966d",
    "name": "Doge TV",
    "symbol": "DGTV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28050/thumb/532a5b9e-8294-4be1-be18-f04a2cf5f0a9.png?1696527063"
  },
  {
    "chainId": 1,
    "address": "0xe91598331a36a78f7fefe277ce7c1915da0afb93",
    "name": "RunesBridge",
    "symbol": "RB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36974/thumb/rsz_logo5_1.png?1712911043"
  },
  {
    "chainId": 1,
    "address": "0x075b1bb99792c9e1041ba13afef80c91a1e70fb3",
    "name": "Curve fi renBTC wBTC sBTC",
    "symbol": "CRVRENWSBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11958/thumb/Curvefi_sbtcCrv_32.png?1696511818"
  },
  {
    "chainId": 1,
    "address": "0x20e7125677311fca903a8897042b9983f22ea295",
    "name": "Freeway",
    "symbol": "FWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13012/thumb/S5h7MHR.png?1696512800"
  },
  {
    "chainId": 1,
    "address": "0xbc61e13ca6830fc7f035fd0e90a01cd08be6dcaa",
    "name": "SHOOT",
    "symbol": "SHOOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36553/thumb/icon.png?1711896644"
  },
  {
    "chainId": 1,
    "address": "0x798d1be841a82a273720ce31c822c61a67a601c3",
    "name": "DIGG",
    "symbol": "DIGG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/13737/thumb/digg.PNG?1696513481"
  },
  {
    "chainId": 1,
    "address": "0xbc194e6f748a222754c3e8b9946922c09e7d4e91",
    "name": "Lever Network",
    "symbol": "LEV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15323/thumb/lever.PNG?1696514972"
  },
  {
    "chainId": 1,
    "address": "0x18aaa7115705e8be94bffebde57af9bfc265b998",
    "name": "Audius",
    "symbol": "AUDIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12913/thumb/AudiusCoinLogo_2x.png?1696512701"
  },
  {
    "chainId": 1,
    "address": "0x8fc17671d853341d9e8b001f5fc3c892d09cb53a",
    "name": "BlockGames",
    "symbol": "BLOCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36976/thumb/QTR7pMsP_400x400.jpg?1712911941"
  },
  {
    "chainId": 1,
    "address": "0xda4dd9586d27202a338843dd6b9824d267006783",
    "name": "Echain Network",
    "symbol": "ECT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28133/thumb/1666808362379-8226c3c3451e049842cfe21d65cb91ad.png?1696527140"
  },
  {
    "chainId": 1,
    "address": "0x4fc15c91a9c4a9efb404174464687e8e128730c2",
    "name": "STAT",
    "symbol": "STAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26602/thumb/stat.png?1696525676"
  },
  {
    "chainId": 1,
    "address": "0x664c6e221c77313307467b121528ad563107bd01",
    "name": "Tax Haven Inu",
    "symbol": "TAXHAVENINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27362/thumb/tax_haven_inu.png?1696526406"
  },
  {
    "chainId": 1,
    "address": "0x19de6b897ed14a376dda0fe53a5420d2ac828a28",
    "name": "Bitget Token",
    "symbol": "BGB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11610/thumb/icon_colour.png?1696511504"
  },
  {
    "chainId": 1,
    "address": "0x513c3200f227ebb62e3b3d00b7a83779643a71cf",
    "name": "Uplift",
    "symbol": "LIFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20666/thumb/logo-200x200.png?1696520067"
  },
  {
    "chainId": 1,
    "address": "0x94025780a1ab58868d9b2dbbb775f44b32e8e6e5",
    "name": "BetSwirl",
    "symbol": "BETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26618/thumb/icon_200.png?1696525691"
  },
  {
    "chainId": 1,
    "address": "0xc6956b78e036b87ba2ab9810bf081eb76eedd17a",
    "name": "H",
    "symbol": "H",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28759/thumb/H.png?1696527738"
  },
  {
    "chainId": 1,
    "address": "0xcae0dd4bda7ff3e700355c7629b24d5d728bd2ce",
    "name": "Bowie",
    "symbol": "BOWIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33218/thumb/logo-200x200.png?1701088140"
  },
  {
    "chainId": 1,
    "address": "0x68d009f251ff3a271477f77acb704c3b0f32a0c0",
    "name": "CHAD",
    "symbol": "CHAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35457/thumb/LOGO_RED.png?1708669427"
  },
  {
    "chainId": 1,
    "address": "0x054f76beed60ab6dbeb23502178c52d6c5debe40",
    "name": "DeFiner",
    "symbol": "FIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12780/thumb/PdaW8Lb.png?1696512575"
  },
  {
    "chainId": 1,
    "address": "0x4161725d019690a3e0de50f6be67b07a86a9fae1",
    "name": "TokenPocket Token",
    "symbol": "TPT",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/7603/thumb/pocket.jpg?1696507861"
  },
  {
    "chainId": 1,
    "address": "0x758b4684be769e92eefea93f60dda0181ea303ec",
    "name": "Phonon DAO",
    "symbol": "PHONON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22308/thumb/ezgif-2-e7fb84364d.png?1696521653"
  },
  {
    "chainId": 1,
    "address": "0xb627a1bf727f578384ba18b2a2b46f4fb924ab3b",
    "name": "P3PE HACKER",
    "symbol": "P3PE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33219/thumb/download.png?1701088202"
  },
  {
    "chainId": 1,
    "address": "0xaaaebe6fe48e54f431b0c390cfaf0b017d09d42d",
    "name": "Celsius Network",
    "symbol": "CEL",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/3263/thumb/CEL_logo.png?1696503976"
  },
  {
    "chainId": 1,
    "address": "0xcb69b067d9d8d6dd1209fe4557c43586e54f9045",
    "name": "Possum",
    "symbol": "PSM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33596/thumb/PSM.png?1702521046"
  },
  {
    "chainId": 1,
    "address": "0xa57ed6e54be8125bbe45d6ca330e45ebb71ef11e",
    "name": "NEWPEPE",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31729/thumb/photo_2023-09-12_02-04-48_%282%29.jpg?1696530549"
  },
  {
    "chainId": 1,
    "address": "0xb197a6fdd8ab825332edeba1ab5c4d1bf97fb9a8",
    "name": "Shibarium DAO",
    "symbol": "SHIBDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28881/thumb/200x200.png?1696527858"
  },
  {
    "chainId": 1,
    "address": "0x8888801af4d980682e47f1a9036e589479e835c5",
    "name": "88mph",
    "symbol": "MPH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13137/thumb/yfU-_Tcj_400x400.png?1696512923"
  },
  {
    "chainId": 1,
    "address": "0x92e187a03b6cd19cb6af293ba17f2745fd2357d5",
    "name": "Unit Protocol",
    "symbol": "DUCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13359/thumb/6f38719f-fe83-44ff-af30-7965fd23ac06.png?1696513124"
  },
  {
    "chainId": 1,
    "address": "0xcae72a7a0fd9046cf6b165ca54c9e3a3872109e0",
    "name": "AnRKey X",
    "symbol": "ANRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13415/thumb/anrkey.jpg?1696513176"
  },
  {
    "chainId": 1,
    "address": "0x1045f5ccb01daea4f8eab055f5fcbb7c0e7c89f0",
    "name": "DeFiato",
    "symbol": "DFIAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13386/thumb/Defiato.png?1696513149"
  },
  {
    "chainId": 1,
    "address": "0x1bfce574deff725a3f483c334b790e25c8fa9779",
    "name": "Tao Ce i",
    "symbol": "CETI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36097/thumb/CETI-BLACK-SMALL.png?1710481943"
  },
  {
    "chainId": 1,
    "address": "0xaa2d8c9a8bd0f7945143bfd509be3ff23dd78918",
    "name": "Artizen",
    "symbol": "ATNT",
    "decimals": 3,
    "logoURI": "https://assets.coingecko.com/coins/images/27263/thumb/ATNT-symbol_200.png?1696526315"
  },
  {
    "chainId": 1,
    "address": "0xc770eefad204b5180df6a14ee197d99d808ee52d",
    "name": "ShapeShift FOX",
    "symbol": "FOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9988/thumb/FOX.png?1696510025"
  },
  {
    "chainId": 1,
    "address": "0xc719d010b63e5bbf2c0551872cd5316ed26acd83",
    "name": "Etherisc DIP",
    "symbol": "DIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4586/thumb/dip.png?1696505164"
  },
  {
    "chainId": 1,
    "address": "0x682f3317a8db21ba205dc84a0b7bd5010333bda7",
    "name": "DAOGAME",
    "symbol": "DAOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33320/thumb/logo_purple.png?1709643510"
  },
  {
    "chainId": 1,
    "address": "0x9a7703338730b82a803ba050df55f9b3959f3fb2",
    "name": "Arise Chikun",
    "symbol": "CHIKUN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31043/thumb/WJyGMMxM_400x400.jpg?1696529878"
  },
  {
    "chainId": 1,
    "address": "0x30dcba0405004cf124045793e1933c798af9e66a",
    "name": "Yieldification",
    "symbol": "YDF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26699/thumb/logo.png?1696525772"
  },
  {
    "chainId": 1,
    "address": "0x3f5dd1a1538a4f9f82e543098f01f22480b0a3a8",
    "name": "KumaDex Token",
    "symbol": "DKUMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26693/thumb/dkumaOG02_CoinGecko.png?1696525767"
  },
  {
    "chainId": 1,
    "address": "0x52662717e448be36cb54588499d5a8328bd95292",
    "name": "Tenshi",
    "symbol": "TENSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17170/thumb/TENSHI.png?1696516729"
  },
  {
    "chainId": 1,
    "address": "0x382e57ca8e4c4db9649884ca77b0a355692d14ac",
    "name": "Xyxyx",
    "symbol": "XYXYX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35483/thumb/xyxyx_logo.png?1708850265"
  },
  {
    "chainId": 1,
    "address": "0x8052327f1baf94a9dc8b26b9100f211ee3774f54",
    "name": "A2DAO",
    "symbol": "ATD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14509/thumb/Logo.jpg?1696514195"
  },
  {
    "chainId": 1,
    "address": "0xbd04ccc050058a6a422851fa6c0f92bb65eb06ca",
    "name": "Pre Retogeum",
    "symbol": "PRTG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27397/thumb/IMG_20220921_123729_913.jpg?1696526437"
  },
  {
    "chainId": 1,
    "address": "0x80122c6a83c8202ea365233363d3f4837d13e888",
    "name": "MESSIER",
    "symbol": "M87",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25957/thumb/MESSIERlogonew_%281%29.png?1696525036"
  },
  {
    "chainId": 1,
    "address": "0xc55126051b22ebb829d00368f4b12bde432de5da",
    "name": "Redacted",
    "symbol": "BTRFLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26745/thumb/redacted_v2.jpg?1696525813"
  },
  {
    "chainId": 1,
    "address": "0x6a445e9f40e0b97c92d0b8a3366cef1d67f700bf",
    "name": "Fidu",
    "symbol": "FIDU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25944/thumb/GFI-asset-icon.png?1696525023"
  },
  {
    "chainId": 1,
    "address": "0xb9d7dddca9a4ac480991865efef82e01273f79c3",
    "name": "Boosted LUSD",
    "symbol": "BLUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27681/thumb/bLUSD_logo.png?1696526709"
  },
  {
    "chainId": 1,
    "address": "0x9b81686140e85d28c2236c307dd49b422a663edf",
    "name": "Combustion",
    "symbol": "FIRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31044/thumb/T-wqdlHF_400x400.jpg?1696529879"
  },
  {
    "chainId": 1,
    "address": "0x4b13006980acb09645131b91d259eaa111eaf5ba",
    "name": "Mycelium",
    "symbol": "MYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26874/thumb/MYC_Token.png?1696525933"
  },
  {
    "chainId": 1,
    "address": "0xd291e7a03283640fdc51b121ac401383a46cc623",
    "name": "Rari Governance",
    "symbol": "RGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12900/thumb/Rari_Logo_Transparent.png?1696512688"
  },
  {
    "chainId": 1,
    "address": "0x486f4641ef2b50cc130dadbd27b6f271723873b8",
    "name": "Adventurer Gold",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28427/thumb/anon-npc-pixel.png?1696527424"
  },
  {
    "chainId": 1,
    "address": "0x8e0e57dcb1ce8d9091df38ec1bfc3b224529754a",
    "name": "Moon Tropica",
    "symbol": "CAH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28470/thumb/MTLOGO.png?1696527464"
  },
  {
    "chainId": 1,
    "address": "0x5a8291d0f1836726aea75b979a92a47be1ff0cba",
    "name": "Cremate",
    "symbol": "CRMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33323/thumb/crematelogo.jpeg?1701437525"
  },
  {
    "chainId": 1,
    "address": "0x9c4a4204b79dd291d6b6571c5be8bbcd0622f050",
    "name": "Tracer DAO",
    "symbol": "TCR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18271/thumb/tracer_logo.png?1696517765"
  },
  {
    "chainId": 1,
    "address": "0xa41f142b6eb2b164f8164cae0716892ce02f311f",
    "name": "Avocado DAO",
    "symbol": "AVG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21102/thumb/logo192_%281%29.png?1696520483"
  },
  {
    "chainId": 1,
    "address": "0x954a75564cb355ea2d6fccc6c1212fd01fdcb06f",
    "name": "PLUMPY DRAGONS",
    "symbol": "LOONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35356/thumb/LOONG.png?1711333621"
  },
  {
    "chainId": 1,
    "address": "0xf25304e75026e6a35fedca3b0889ae5c4d3c55d8",
    "name": "Viridis Network",
    "symbol": "VRD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32709/thumb/200x200.png?1698986518"
  },
  {
    "chainId": 1,
    "address": "0x6c6e2c5a4eb108a1f3c985d5a7f4f233483e952f",
    "name": "Etherscape",
    "symbol": "SCAPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35408/thumb/photo_2024-02-20_19.52.34.jpeg?1708500628"
  },
  {
    "chainId": 1,
    "address": "0x9e32b13ce7f2e80a01932b42553652e053d6ed8e",
    "name": "Metis",
    "symbol": "METIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15595/thumb/Metis_Black_Bg.png?1702968192"
  },
  {
    "chainId": 1,
    "address": "0xc2aeedc081d4cb6797a681e9403a82211f97b308",
    "name": "Zeal AI",
    "symbol": "ZAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35487/thumb/photo_6246877151911591329_y_%282%29.jpg?1708878103"
  },
  {
    "chainId": 1,
    "address": "0x434cb4fc4b952872967914d430878eee53ebd502",
    "name": "ALLPAYCOIN",
    "symbol": "APCG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26836/thumb/apcg.png?1696525895"
  },
  {
    "chainId": 1,
    "address": "0xd9c2d319cd7e6177336b0a9c93c21cb48d84fb54",
    "name": "HAPI",
    "symbol": "HAPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14298/thumb/Hapi_logo_square_%281%29.png?1696513995"
  },
  {
    "chainId": 1,
    "address": "0x42496acd2c7b52ce90ed65ada6cafb0e893e2474",
    "name": "BitSwap",
    "symbol": "BITS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36633/thumb/BitSwapLogo_%282%29.png?1712022617"
  },
  {
    "chainId": 1,
    "address": "0x73a83269b9bbafc427e76be0a2c1a1db2a26f4c2",
    "name": "Civfund Stone",
    "symbol": "0NE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25156/thumb/stone.png?1696524304"
  },
  {
    "chainId": 1,
    "address": "0xef19f4e48830093ce5bc8b3ff7f903a0ae3e9fa1",
    "name": "BOTXCOIN",
    "symbol": "BOTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6827/thumb/botx.jpg?1696507152"
  },
  {
    "chainId": 1,
    "address": "0x050d94685c6b0477e1fc555888af6e2bb8dfbda5",
    "name": "Inu ",
    "symbol": "INU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26259/thumb/inulogo_200x200.png?1696525344"
  },
  {
    "chainId": 1,
    "address": "0x03be5c903c727ee2c8c4e9bc0acc860cca4715e2",
    "name": "Ternoa",
    "symbol": "CAPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15921/thumb/e55393fa-7b4d-40f5-9f36-9a8a6bdcb570.png?1696515534"
  },
  {
    "chainId": 1,
    "address": "0x678e840c640f619e17848045d23072844224dd37",
    "name": "Cratos",
    "symbol": "CRTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17322/thumb/cratos.png?1696516876"
  },
  {
    "chainId": 1,
    "address": "0x6dca182ac5e3f99985bc4ee0f726d6472ab1ec55",
    "name": "Ushi",
    "symbol": "USHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27865/thumb/photo_2022-10-17_22-21-01.jpg?1696526883"
  },
  {
    "chainId": 1,
    "address": "0xaa95f26e30001251fb905d264aa7b00ee9df6c18",
    "name": "Kendu Inu",
    "symbol": "KENDU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35722/thumb/photo_2024-03-05_01-23-59_%281%29.jpg?1709632285"
  },
  {
    "chainId": 1,
    "address": "0xca30c93b02514f86d5c86a6e375e3a330b435fb5",
    "name": "Backed IB01   Treasury Bond 0 1yr",
    "symbol": "BIB01",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31755/thumb/b-IB01-200x200.png?1696530574"
  },
  {
    "chainId": 1,
    "address": "0x3c4008eca800ec1283e4cf500e68d06bfabc00a8",
    "name": "HistoryDAO",
    "symbol": "HAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27318/thumb/5961657012978_.pic.jpg?1696526367"
  },
  {
    "chainId": 1,
    "address": "0x0cba60ca5ef4d42f92a5070a8fedd13be93e2861",
    "name": "The Protocol",
    "symbol": "THE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27849/thumb/cSar5sDM_400x400.jpg?1696526868"
  },
  {
    "chainId": 1,
    "address": "0xf12ccd17759367cf139776710b47b00c43d1ac2b",
    "name": "emoji ERC20",
    "symbol": "EMOJI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34635/thumb/8CC1B5C8-A5E2-45E0-BDEE-404DE38B7C8C.jpeg?1705852614"
  },
  {
    "chainId": 1,
    "address": "0x992d339532a9c42f1b0e59a57e95f38da38c66f6",
    "name": "Soulsaver",
    "symbol": "SOUL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27494/thumb/SOUL_LOGO.png?1696526534"
  },
  {
    "chainId": 1,
    "address": "0x20f031e2eca9e9247eb8c57bab5db9a1daa9c5d1",
    "name": "PiuAi",
    "symbol": "PAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35488/thumb/200.png?1708879739"
  },
  {
    "chainId": 1,
    "address": "0x205ed31c867bf715e4182137af95afe9177cd8e7",
    "name": "DEFY",
    "symbol": "DEFY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26877/thumb/8ybr83fb0ca45cm1yvrcaclwbvcp.jpeg?1696525936"
  },
  {
    "chainId": 1,
    "address": "0x244517dc59943e8cdfbd424bdb3262c5f04a1387",
    "name": "Fluid DAI",
    "symbol": "FDAI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28473/thumb/fDAI-200x200.png?1696527467"
  },
  {
    "chainId": 1,
    "address": "0x06576eb3b212d605b797dc15523d9dc9f4f66db4",
    "name": "The Crypto Prophecies",
    "symbol": "TCP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15054/thumb/tcp.PNG?1696514713"
  },
  {
    "chainId": 1,
    "address": "0xadc234a4e90e2045f353f5d4fcde66144d23b458",
    "name": "Fluid USDT",
    "symbol": "FUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28472/thumb/fUSDT-200x200.png?1696527466"
  },
  {
    "chainId": 1,
    "address": "0x3be7bf1a5f23bd8336787d0289b70602f1940875",
    "name": "VIDT DAO",
    "symbol": "VIDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27552/thumb/VIDTDAO_logo.png?1696526588"
  },
  {
    "chainId": 1,
    "address": "0xa683929289b52a0f3f24fef9d6bf1fbbaeb49e9d",
    "name": "VelosBot",
    "symbol": "VELOS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32648/thumb/200x200.jpg?1698894499"
  },
  {
    "chainId": 1,
    "address": "0xbd1848e1491d4308ad18287a745dd4db2a4bd55b",
    "name": "Mochi Market",
    "symbol": "MOMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14993/thumb/mochi.PNG?1696514657"
  },
  {
    "chainId": 1,
    "address": "0xb3e2cb7cccfe139f8ff84013823bf22da6b6390a",
    "name": "Deutsche Digital Assets",
    "symbol": "ICNQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7830/thumb/dda_logo_small.jpg?1696508067"
  },
  {
    "chainId": 1,
    "address": "0xbea269038eb75bdab47a9c04d0f5c572d94b93d5",
    "name": "Wrapped FIO",
    "symbol": "WFIO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29183/thumb/fio_light_favicon_transparent_200x200.png?1696528141"
  },
  {
    "chainId": 1,
    "address": "0xf1f955016ecbcd7321c7266bccfb96c68ea5e49b",
    "name": "Rally",
    "symbol": "RLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12843/thumb/image.png?1696512633"
  },
  {
    "chainId": 1,
    "address": "0x52a163f4cef6a42fa6b17e0f2a773fc149547c9c",
    "name": "BANGER",
    "symbol": "BANGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32881/thumb/banger200x200.png?1699712080"
  },
  {
    "chainId": 1,
    "address": "0x77f9cf0bd8c500cffdf420e72343893aecc2ec0b",
    "name": "Laika",
    "symbol": "LAIKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26717/thumb/Laikacmcfinal.png?1696525787"
  },
  {
    "chainId": 1,
    "address": "0xdd2a36ae937bc134ea694d77fc7e2e36f5d86de0",
    "name": "WELD",
    "symbol": "WELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18544/thumb/weld.png?1696518023"
  },
  {
    "chainId": 1,
    "address": "0x757da0e5c253082b0f2bd5105119f71817fe0911",
    "name": "Very Special Dragon",
    "symbol": "VITO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28527/thumb/Vito200.png?1696527519"
  },
  {
    "chainId": 1,
    "address": "0x4d5f47fa6a74757f35c14fd3a6ef8e3c9bc514e8",
    "name": "Aave v3 WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32882/thumb/WETH_%281%29.png?1699716492"
  },
  {
    "chainId": 1,
    "address": "0x2f1ee92524285012c02a4e638ec010fa7f61fd94",
    "name": "Siphon Life Spell",
    "symbol": "SLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34667/thumb/photo_2024-01-17_04-21-27.jpg?1705654454"
  },
  {
    "chainId": 1,
    "address": "0x426aedbed16726e3f220cb4fed4d4060b95cca46",
    "name": "Bahamas",
    "symbol": "BAHAMAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34089/thumb/Bahamas_Logo_200x200.png?1703911941"
  },
  {
    "chainId": 1,
    "address": "0x5ee5bf7ae06d1be5997a1a72006fe6c607ec6de8",
    "name": "Aave v3 WBTC",
    "symbol": "AWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32883/thumb/wbtc.png?1699719908"
  },
  {
    "chainId": 1,
    "address": "0xfbe6f37d3db3fc939f665cfe21238c11a5447831",
    "name": "ETH 2 0",
    "symbol": "ETH20",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32079/thumb/BSk76OA8_400x400.jpg?1696530878"
  },
  {
    "chainId": 1,
    "address": "0x8eb94a06b4716093dbfe335cbdb098deb2dcde1b",
    "name": "Half Shiba Inu",
    "symbol": "SHIB05",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28957/thumb/halfshib.jpg?1696527930"
  },
  {
    "chainId": 1,
    "address": "0xe88f8313e61a97cec1871ee37fbbe2a8bf3ed1e4",
    "name": "Sora Validator",
    "symbol": "VAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13299/thumb/val-gold-256.png?1696513070"
  },
  {
    "chainId": 1,
    "address": "0xc744df3419a8c9bd4d6b9852a503eb1c5308a326",
    "name": "RED TOKEN",
    "symbol": "RED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27153/thumb/red.png?1696526204"
  },
  {
    "chainId": 1,
    "address": "0xe5a3229ccb22b6484594973a03a3851dcd948756",
    "name": "Receive Access Ecosystem",
    "symbol": "RAE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9428/thumb/Copy_of_raetoken.png?1696509521"
  },
  {
    "chainId": 1,
    "address": "0x8a8116a794744977941c7d3743517410969aacbb",
    "name": "Shiba Punkz",
    "symbol": "SPUNK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27163/thumb/shibapunkzlogo.JPG?1696526213"
  },
  {
    "chainId": 1,
    "address": "0xaa0c5b3567fd1bac8a2a11eb16c3f81a49eea90f",
    "name": "MetamonkeyAi",
    "symbol": "MMAI",
    "decimals": 7,
    "logoURI": "https://assets.coingecko.com/coins/images/27208/thumb/mmai_new_logo.png?1696526256"
  },
  {
    "chainId": 1,
    "address": "0xe9f84d418b008888a992ff8c6d22389c2c3504e0",
    "name": "Maximus BASE",
    "symbol": "BASE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27683/thumb/IMG_1131.PNG?1696526711"
  },
  {
    "chainId": 1,
    "address": "0x1ccf27211e8bf052f6255329ed641b4e94e80603",
    "name": "Metababy",
    "symbol": "BABY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27175/thumb/7psZaqh0_400x400.jpeg?1696526225"
  },
  {
    "chainId": 1,
    "address": "0x465a5a630482f3abd6d3b84b39b29b07214d19e5",
    "name": "Flux USDC",
    "symbol": "FUSDC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29051/thumb/fUSDC_200x200.png?1696528018"
  },
  {
    "chainId": 1,
    "address": "0xf55cd1e399e1cc3d95303048897a680be3313308",
    "name": "Maximus TRIO",
    "symbol": "TRIO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27684/thumb/IMG_1127.PNG?1696526712"
  },
  {
    "chainId": 1,
    "address": "0xeaf61fc150cd5c3bea75744e830d916e60ea5a9f",
    "name": "Typerium",
    "symbol": "TYPE",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/4019/thumb/type.png?1696504657"
  },
  {
    "chainId": 1,
    "address": "0x64b78325d7495d6d4be92f234fa3f3b8d8964b8b",
    "name": "Shopping io",
    "symbol": "SHOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27242/thumb/shop.png?1696526292"
  },
  {
    "chainId": 1,
    "address": "0x0b319db00d07c8fadfaaef13c910141a5da0aa8f",
    "name": "Fluid TUSD",
    "symbol": "FTUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28474/thumb/fTUSD-200x200.png?1696527468"
  },
  {
    "chainId": 1,
    "address": "0x6b0956258ff7bd7645aa35369b55b61b8e6d6140",
    "name": "Maximus LUCKY",
    "symbol": "LUCKY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27685/thumb/IMG_1129.PNG?1696526713"
  },
  {
    "chainId": 1,
    "address": "0x777172d858dc1599914a1c4c6c9fc48c99a60990",
    "name": "Solidly",
    "symbol": "SOLID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28676/thumb/solid_logo_200.png?1701687346"
  },
  {
    "chainId": 1,
    "address": "0xdbecdd726f6ad8e24afc78fe3cc8eb7b73c2d94d",
    "name": "Chaintools",
    "symbol": "CTLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30896/thumb/IMG_0501.png?1696529742"
  },
  {
    "chainId": 1,
    "address": "0xbe92b510007bd3ec0adb3d1fca338dd631e98de7",
    "name": "DegensTogether",
    "symbol": "DEGEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30487/thumb/degen_logo_twit.png?1696529373"
  },
  {
    "chainId": 1,
    "address": "0x291aa47c58558adfc2bcd6f060578fdae1f6570c",
    "name": "Minebase",
    "symbol": "MBASE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27313/thumb/mbase.png?1696526362"
  },
  {
    "chainId": 1,
    "address": "0x5891599664ed15c6e88041b4f5bc08594f026f0e",
    "name": "JPGoldCoin",
    "symbol": "JPGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27321/thumb/JPGC_LOGO-_Coingecko.png?1696526370"
  },
  {
    "chainId": 1,
    "address": "0xb4272071ecadd69d933adcd19ca99fe80664fc08",
    "name": "CryptoFranc",
    "symbol": "XCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8465/thumb/WhuiuJBc_400x400.jpg?1696508650"
  },
  {
    "chainId": 1,
    "address": "0x37f678ad6221a0fd71cda0eca19c8802f4cabf44",
    "name": "YUKKY",
    "symbol": "YUKKY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30512/thumb/200GpSL05ul_400x400.jpg?1696529397"
  },
  {
    "chainId": 1,
    "address": "0x3f817b28da4940f018c6b5c0a11c555ebb1264f9",
    "name": "3A",
    "symbol": "A3A",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33135/thumb/A3A.png?1700801023"
  },
  {
    "chainId": 1,
    "address": "0x762fcf5183ae366c0629d0bcd30b40f331496d0f",
    "name": "Dice Bot",
    "symbol": "DICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32654/thumb/Twitter_pfp.png?1698965261"
  },
  {
    "chainId": 1,
    "address": "0x23b586c0e79fb291ccb0244d468847eae9bb90f6",
    "name": "R4RE",
    "symbol": "R4RE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36559/thumb/R4RE.png?1711902279"
  },
  {
    "chainId": 1,
    "address": "0xbe042e9d09cb588331ff911c2b46fd833a3e5bd6",
    "name": "Pepe Token",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31482/thumb/photo_2023-08-25_07-54-16.jpg?1696530294"
  },
  {
    "chainId": 1,
    "address": "0x142a774e8b52550e88e196cedd7a5835acb646d0",
    "name": "SaitaRealty",
    "symbol": "SRLTY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27289/thumb/s_mksCdB_400x400.jpeg?1696526341"
  },
  {
    "chainId": 1,
    "address": "0xd7c49cee7e9188cca6ad8ff264c1da2e69d4cf3b",
    "name": "Nexus Mutual",
    "symbol": "NXM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11810/thumb/NXMmain.png?1696511684"
  },
  {
    "chainId": 1,
    "address": "0x5c1a467e12150864ea15bd394dac3801c2ca3cb0",
    "name": "Gamma Wallet",
    "symbol": "GAMMA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33221/thumb/GammaWallet.png?1701088418"
  },
  {
    "chainId": 1,
    "address": "0xba93ef534094f8b7001ece2691168140965341ab",
    "name": "Beauty Bakery Linked Operation Transact",
    "symbol": "LOTT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27688/thumb/LOTT.jpg?1696526716"
  },
  {
    "chainId": 1,
    "address": "0x095797fd4297fb79883cc912a5ba6313b15c445d",
    "name": "GOLCOIN",
    "symbol": "GOLC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27341/thumb/logo_in_the_middle_.png?1696526388"
  },
  {
    "chainId": 1,
    "address": "0x238cefec182679c27a3035713416fa0a8198b302",
    "name": "GoodMeme",
    "symbol": "GMEME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30325/thumb/gmeme200smol.png?1696529226"
  },
  {
    "chainId": 1,
    "address": "0xd75f1f81b69bdd4df8efbb70e9c6f4609009d753",
    "name": "YASHA",
    "symbol": "YASHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24040/thumb/oX9E7Q7.png?1696523231"
  },
  {
    "chainId": 1,
    "address": "0x7fd4d7737597e7b4ee22acbf8d94362343ae0a79",
    "name": "Wrapped MistCoin",
    "symbol": "WMC",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/32490/thumb/MistCOin200.png?1698293182"
  },
  {
    "chainId": 1,
    "address": "0x4bdcb66b968060d9390c1d12bd29734496205581",
    "name": "Acquire Fi",
    "symbol": "ACQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27199/thumb/ACQisotypeSept2022_1000x1000px.png?1696526248"
  },
  {
    "chainId": 1,
    "address": "0xe2ba8693ce7474900a045757fe0efca900f6530b",
    "name": "Flux DAI",
    "symbol": "FDAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29052/thumb/fDAI_200x200.png?1696528019"
  },
  {
    "chainId": 1,
    "address": "0x72e4f9f808c49a2a61de9c5896298920dc4eeea9",
    "name": "HarryPotterObamaSonic10Inu  ETH ",
    "symbol": "BITCOIN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30323/thumb/hpos10i_logo_casino_night-dexview.png?1696529224"
  },
  {
    "chainId": 1,
    "address": "0x8526be2379e853d5cf02f9823bb9690e1a6ff9e2",
    "name": "Habibi",
    "symbol": "HABIBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30683/thumb/HABIBI.png?1696529552"
  },
  {
    "chainId": 1,
    "address": "0xf951e335afb289353dc249e82926178eac7ded78",
    "name": "Swell Ethereum",
    "symbol": "SWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30326/thumb/_lB7zEtS_400x400.jpg?1696529227"
  },
  {
    "chainId": 1,
    "address": "0x8326bf664704966c984a3a46fa37d7a80a52dcf4",
    "name": "Dogu Inu",
    "symbol": "DOGU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28492/thumb/Coingecko_Dogu_Inu_logo.jpg?1696527485"
  },
  {
    "chainId": 1,
    "address": "0x2890df158d76e584877a1d17a85fea3aeeb85aa6",
    "name": "Alien Milady Fumo",
    "symbol": "FUMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30812/thumb/miladyfumo.jpg?1696529668"
  },
  {
    "chainId": 1,
    "address": "0xf1376bcef0f78459c0ed0ba5ddce976f1ddf51f4",
    "name": "Universal ETH",
    "symbol": "UNIETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28477/thumb/uniETH_200.png?1696527471"
  },
  {
    "chainId": 1,
    "address": "0xa888d9616c2222788fa19f05f77221a290eef704",
    "name": "Daruma",
    "symbol": "DARUMA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27357/thumb/CGlogo.png?1696526403"
  },
  {
    "chainId": 1,
    "address": "0x6b32022693210cd2cfc466b9ac0085de8fc34ea6",
    "name": "Maximus DECI",
    "symbol": "DECI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27693/thumb/Deci.png?1696526720"
  },
  {
    "chainId": 1,
    "address": "0x0c90c57aaf95a3a87eadda6ec3974c99d786511f",
    "name": "HanChain",
    "symbol": "HAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27374/thumb/logo_200.png?1696526418"
  },
  {
    "chainId": 1,
    "address": "0xc5bcc8ba3f33ab0d64f3473e861bdc0685b19ef5",
    "name": "Mechanium",
    "symbol": "MECHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24374/thumb/w4K4OOMo_400x400.jpg?1696523558"
  },
  {
    "chainId": 1,
    "address": "0x32ea3dc70e2962334864a9665254d2433e4ddbfd",
    "name": "Sportzchain",
    "symbol": "SPN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24915/thumb/Sportzchain_logo_195_195.png?1696524072"
  },
  {
    "chainId": 1,
    "address": "0xa49d7499271ae71cd8ab9ac515e6694c755d400c",
    "name": "Mute",
    "symbol": "MUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14331/thumb/MUTE.png?1696514019"
  },
  {
    "chainId": 1,
    "address": "0xd433138d12beb9929ff6fd583dc83663eea6aaa5",
    "name": "Bitrue Coin",
    "symbol": "BTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8873/thumb/Bittrue_logo.png?1696509025"
  },
  {
    "chainId": 1,
    "address": "0xdc349913d53b446485e98b76800b6254f43df695",
    "name": "Bezoge Earth",
    "symbol": "BEZOGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15639/thumb/token-logo.png?1696515271"
  },
  {
    "chainId": 1,
    "address": "0xe453c3409f8ad2b1fe1ed08e189634d359705a5b",
    "name": "DGI Game",
    "symbol": "DGI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34658/thumb/DGI_Game.png?1705633633"
  },
  {
    "chainId": 1,
    "address": "0xf90c7f66eac7e2130bf677d69a250b2136cf6697",
    "name": "HAKI Token",
    "symbol": "HAKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7945/thumb/haki.png?1696508175"
  },
  {
    "chainId": 1,
    "address": "0xf9d4daae1300cff251979722c4a3c45857973079",
    "name": "bitcastle",
    "symbol": "CASTLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26551/thumb/blue_logo03.png?1696525624"
  },
  {
    "chainId": 1,
    "address": "0xf86cfce1e746456135d7face48c2916d7d3cb676",
    "name": "Everflow Token",
    "symbol": "EFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36307/thumb/everflow-logo.4884c463824dcb062e4b.png?1711090157"
  },
  {
    "chainId": 1,
    "address": "0x8a7664e782860e856031d6c31eb3bde721bc362b",
    "name": "RektSkulls",
    "symbol": "REKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28910/thumb/REKT_icon_200.png?1696527886"
  },
  {
    "chainId": 1,
    "address": "0xf3b9569f82b18aef890de263b84189bd33ebe452",
    "name": "A Hunters Dream",
    "symbol": "CAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25130/thumb/x-Logo-color-10x.png?1696524280"
  },
  {
    "chainId": 1,
    "address": "0x1bf7fd22709733ccd7c45ab27dd02c7ec8e50078",
    "name": "Quiztok",
    "symbol": "QTCON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8208/thumb/QTCON.png?1696508415"
  },
  {
    "chainId": 1,
    "address": "0x7f280dac515121dcda3eac69eb4c13a52392cace",
    "name": "Fancy Games",
    "symbol": "FNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21367/thumb/fnc.png?1696520733"
  },
  {
    "chainId": 1,
    "address": "0x36f8d0d0573ae92326827c4a82fe4ce4c244cab6",
    "name": "Morpho Aave Dai Stablecoin",
    "symbol": "MADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29836/thumb/maDAI.jpg?1696528763"
  },
  {
    "chainId": 1,
    "address": "0x5c0590cde44569bf39ef79e859b367e39cb000f1",
    "name": "hiVALHALLA",
    "symbol": "HIVALHALLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28911/thumb/hiVALHALLA.png?1696527887"
  },
  {
    "chainId": 1,
    "address": "0x2d869ae129e308f94cc47e66eaefb448cee0d03e",
    "name": "OraiDEX",
    "symbol": "ORAIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28104/thumb/oraix.png?1696527113"
  },
  {
    "chainId": 1,
    "address": "0x1ce270557c1f68cfb577b856766310bf8b47fd9c",
    "name": "MongCoin",
    "symbol": "MONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30051/thumb/h3OCGNvn_400x400.jpg?1696528973"
  },
  {
    "chainId": 1,
    "address": "0xf68415be72377611e95d59bc710ccbbbf94c4fa2",
    "name": "alphAI",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30684/thumb/alphAI.png?1696529553"
  },
  {
    "chainId": 1,
    "address": "0xa35b1b31ce002fbf2058d22f30f95d405200a15b",
    "name": "Stader ETHx",
    "symbol": "ETHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30870/thumb/staderx.png?1696529717"
  },
  {
    "chainId": 1,
    "address": "0xb8b295df2cd735b15be5eb419517aa626fc43cd5",
    "name": "Staked LINK",
    "symbol": "STLINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29271/thumb/ST.LINK_tokenl_blue.png?1696528224"
  },
  {
    "chainId": 1,
    "address": "0xd05e3cfe984a00f5d6a1e5b75dee43d7e9ab63c8",
    "name": "Public Violet Fybo",
    "symbol": "PVFYBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32494/thumb/PVFYBO.jpg?1698293586"
  },
  {
    "chainId": 1,
    "address": "0x66580f80a00deafab4519dc33c35bf44d8a12b00",
    "name": "LiqWrap",
    "symbol": "LQW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31584/thumb/logo_1.png?1696530401"
  },
  {
    "chainId": 1,
    "address": "0x1b5036bec1b82d44d52fa953a370b3c6cd9328b5",
    "name": "Elan",
    "symbol": "ELAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27635/thumb/coin_logo_512x512.png?1696526665"
  },
  {
    "chainId": 1,
    "address": "0x6ffcc20d3b4d506870225109edb5a332e6c94529",
    "name": "Agri",
    "symbol": "AGRI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32351/thumb/agri.jpg?1697474949"
  },
  {
    "chainId": 1,
    "address": "0x848578e351d25b6ec0d486e42677891521c3d743",
    "name": "moSOLID",
    "symbol": "MOSOLID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28677/thumb/black_mono_logopng.png?1696527661"
  },
  {
    "chainId": 1,
    "address": "0x1776e1f26f98b1a5df9cd347953a26dd3cb46671",
    "name": "Numeraire",
    "symbol": "NMR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/752/thumb/numeraire.png?1696501906"
  },
  {
    "chainId": 1,
    "address": "0x40986a85b4cfcdb054a6cbfb1210194fee51af88",
    "name": "UniFarm",
    "symbol": "UFARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15247/thumb/ufarm.jpeg?1696514900"
  },
  {
    "chainId": 1,
    "address": "0x6923f9b683111dcc0e20124e9a031deeae5dad93",
    "name": "Crypto Hub",
    "symbol": "HUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32705/thumb/Untitled_design_%285%29.png?1698985521"
  },
  {
    "chainId": 1,
    "address": "0x8971f9fd7196e5cee2c1032b50f656855af7dd26",
    "name": "Lambda",
    "symbol": "LAMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4703/thumb/lambda_200.png?1696505268"
  },
  {
    "chainId": 1,
    "address": "0xa02120696c7b8fe16c09c749e4598819b2b0e915",
    "name": "WXT Token",
    "symbol": "WXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8835/thumb/Wirex.jpg?1696508988"
  },
  {
    "chainId": 1,
    "address": "0x8c088775e4139af116ac1fa6f281bbf71e8c1c73",
    "name": "PUMLx",
    "symbol": "PUMLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27253/thumb/pumlx.png?1696526302"
  },
  {
    "chainId": 1,
    "address": "0xfc4913214444af5c715cc9f7b52655e788a569ed",
    "name": "Icosa",
    "symbol": "ICSA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27708/thumb/icsa.e2b79cbc.png?1696526734"
  },
  {
    "chainId": 1,
    "address": "0x7105e64bf67eca3ae9b123f0e5ca2b83b2ef2da0",
    "name": "X7DAO",
    "symbol": "X7DAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27709/thumb/X7DAO.png?1696526735"
  },
  {
    "chainId": 1,
    "address": "0x70008f18fc58928dce982b0a69c2c21ff80dca54",
    "name": "X7R",
    "symbol": "X7R",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27710/thumb/X7R.png?1696526736"
  },
  {
    "chainId": 1,
    "address": "0xa5269a8e31b93ff27b887b56720a25f844db0529",
    "name": "Morpho Aave USD Coin",
    "symbol": "MAUSDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29835/thumb/maUSDC.jpg?1696528762"
  },
  {
    "chainId": 1,
    "address": "0x07f9702ce093db82dfdc92c2c6e578d6ea8d5e22",
    "name": "Oobit",
    "symbol": "OBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19855/thumb/obt.png?1696519278"
  },
  {
    "chainId": 1,
    "address": "0xb15a5aab2a65745314fcd0d7f5080bfa65bd7c03",
    "name": "tupelothedog",
    "symbol": "TUPELO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34885/thumb/tupelothedog.png?1706538842"
  },
  {
    "chainId": 1,
    "address": "0xc881255e4d639b42e326158c7b8ccb7f33459261",
    "name": "AMMX",
    "symbol": "AMMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34546/thumb/ammx_2.png?1705413208"
  },
  {
    "chainId": 1,
    "address": "0x1376a81fe3ee7d0e431f1ac24286b00f3ccf44e7",
    "name": "Welle",
    "symbol": "WELLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32715/thumb/WELLE-TKN-1.png?1699000890"
  },
  {
    "chainId": 1,
    "address": "0xfa3e941d1f6b7b10ed84a0c211bfa8aee907965e",
    "name": "HayCoin",
    "symbol": "HAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32336/thumb/haycoin.jpeg?1698308493"
  },
  {
    "chainId": 1,
    "address": "0x674c964ac0e89d847d6b0abd144b797bf78ba56b",
    "name": "GreenWorld",
    "symbol": "GWD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27771/thumb/200x200logo.png?1696526792"
  },
  {
    "chainId": 1,
    "address": "0x72953a5c32413614d24c29c84a66ae4b59581bbf",
    "name": "CLever",
    "symbol": "CLEV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27727/thumb/%E6%88%AA%E5%B1%8F2022-10-12_%E4%B8%8A%E5%8D%8810.40.19.png?1696526751"
  },
  {
    "chainId": 1,
    "address": "0x2e516ba5bf3b7ee47fb99b09eadb60bde80a82e0",
    "name": "Eggs",
    "symbol": "EGGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29024/thumb/egg200x200.png?1696527994"
  },
  {
    "chainId": 1,
    "address": "0x71eeba415a523f5c952cc2f06361d5443545ad28",
    "name": "XDAO",
    "symbol": "XDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27363/thumb/token_2.png?1696526408"
  },
  {
    "chainId": 1,
    "address": "0x9dc9d1c18060b45f3dc15fb8d6ab1895022c63b3",
    "name": "BitBullBot",
    "symbol": "BBB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34262/thumb/BitBullBot.jpg?1704387899"
  },
  {
    "chainId": 1,
    "address": "0x9008064e6cf73e27a3aba4b10e69f855a4f8efcc",
    "name": "Gemie",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27745/thumb/icon-transparent.png?1696526767"
  },
  {
    "chainId": 1,
    "address": "0xa6a1cc527d48585538b137e6abc14b2a55489d11",
    "name": "Linework Coin",
    "symbol": "LWC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27739/thumb/Linework-logo_coin_200px-01.png?1696526762"
  },
  {
    "chainId": 1,
    "address": "0xba41ddf06b7ffd89d1267b5a93bfef2424eb2003",
    "name": "Mythos",
    "symbol": "MYTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28045/thumb/Mythos_Logos_200x200.png?1696527059"
  },
  {
    "chainId": 1,
    "address": "0x0ec72cd6690db40b16be166858299f19d4f8e5b0",
    "name": "Teh Golden One",
    "symbol": "GOLD1",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27290/thumb/teh_golden_one_200x200.png?1696526342"
  },
  {
    "chainId": 1,
    "address": "0x892a6f9df0147e5f079b0993f486f9aca3c87881",
    "name": "xFUND",
    "symbol": "XFUND",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/13770/thumb/xfund.png?1696513512"
  },
  {
    "chainId": 1,
    "address": "0xe2cfbbedbce1bd59b1b799c44282e6396d692b84",
    "name": "Dexioprotocol",
    "symbol": "DEXIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27075/thumb/DEXI_cirle_R.png?1696526125"
  },
  {
    "chainId": 1,
    "address": "0x881ba05de1e78f549cc63a8f6cabb1d4ad32250d",
    "name": "00 Token",
    "symbol": "00",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27742/thumb/200x200.png?1696526765"
  },
  {
    "chainId": 1,
    "address": "0x0dcee5f694e492f0dd842a7fbe5bed4c6e4665a6",
    "name": "Catboy",
    "symbol": "CATBOY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33891/thumb/200x200.png?1703213224"
  },
  {
    "chainId": 1,
    "address": "0x95cba664fa19d2c24b6d60b707dfa7879b51fd57",
    "name": "ApeGPT",
    "symbol": "APEGPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34222/thumb/logo200.png?1704298130"
  },
  {
    "chainId": 1,
    "address": "0xb4357054c3da8d46ed642383f03139ac7f090343",
    "name": "Port3 Network",
    "symbol": "PORT3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33383/thumb/port3-bc-200x200.png?1710075114"
  },
  {
    "chainId": 1,
    "address": "0x7cc97bf17c5adabe25f9d19d15a1ec8a1ad65f14",
    "name": "Wolverinu",
    "symbol": "WOLVERINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27872/thumb/LOGO3232.jpg?1696526890"
  },
  {
    "chainId": 1,
    "address": "0xb04bf60e468743418e87291d7c9301a5299d984d",
    "name": "FOREVER SHIBA",
    "symbol": "4SHIBA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29110/thumb/4shiba_logo.png?1696528073"
  },
  {
    "chainId": 1,
    "address": "0x9d7107c8e30617cadc11f9692a19c82ae8bba938",
    "name": "Lucky Roo",
    "symbol": "ROO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27943/thumb/lucky200.png?1696526962"
  },
  {
    "chainId": 1,
    "address": "0xfa93660c3f6a848556bb8e265f994160a1f2b289",
    "name": "Community Business Token",
    "symbol": "CBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14701/thumb/CBT-Token-Small.png?1696514372"
  },
  {
    "chainId": 1,
    "address": "0xd624e5c89466a15812c1d45ce1533be1f16c1702",
    "name": "UBD Network",
    "symbol": "UBDN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33678/thumb/UBD_Network.png?1702717194"
  },
  {
    "chainId": 1,
    "address": "0x42dbbd5ae373fea2fc320f62d44c058522bb3758",
    "name": "Memecoin",
    "symbol": "MEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16370/thumb/mem_gold_200x200_copy.png?1696515969"
  },
  {
    "chainId": 1,
    "address": "0xe9514a6eba63a0bbbe2faea919e773ebe0f527c1",
    "name": "Kekcoin  ETH ",
    "symbol": "KEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33224/thumb/Kekcoin_BTC.png?1701089230"
  },
  {
    "chainId": 1,
    "address": "0x51fa2efd62ee56a493f24ae963eace7d0051929e",
    "name": "Poseidon OCEAN",
    "symbol": "PSDNOCEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27859/thumb/psdnOCEAN.png?1696526878"
  },
  {
    "chainId": 1,
    "address": "0x69ed89ecd35082e031fe52b75123f801db083306",
    "name": "Kaeri",
    "symbol": "KAERI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27977/thumb/63370e2a1f2bea31fc7f6977_512x512_v2-p-500-2.png?1696526995"
  },
  {
    "chainId": 1,
    "address": "0xba50933c268f567bdc86e1ac131be072c6b0b71a",
    "name": "ARPA",
    "symbol": "ARPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8506/thumb/9u0a23XY_400x400.jpg?1696508685"
  },
  {
    "chainId": 1,
    "address": "0x23a2164d482fd2fec9c2d0b66528d42fee7b8817",
    "name": "met a meta metameme",
    "symbol": "METAMEME",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33679/thumb/logo200x200.jpg?1702718320"
  },
  {
    "chainId": 1,
    "address": "0x1e917e764bc34d3bc313fe8159a6bd9d9ffd450d",
    "name": "WEWE",
    "symbol": "WEWE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30458/thumb/IMG_3939.JPG?1696529345"
  },
  {
    "chainId": 1,
    "address": "0xde9804cc479164fa9e9cb59ad4e65012a12aa827",
    "name": "GroceryFi",
    "symbol": "GFI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36673/thumb/GfiPng200.png?1712062167"
  },
  {
    "chainId": 1,
    "address": "0x77a90b04d64189d4d09508612c09219bc6816bdc",
    "name": "Toshi Tools",
    "symbol": "TOSHI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28959/thumb/toshi_logo_200_by_200.png?1696527932"
  },
  {
    "chainId": 1,
    "address": "0x579cea1889991f68acc35ff5c3dd0621ff29b0c9",
    "name": "IQ",
    "symbol": "IQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5010/thumb/YAIS3fUh.png?1696505542"
  },
  {
    "chainId": 1,
    "address": "0x6cd079f296c10eb84a64239b893d4edfa2e1d779",
    "name": "PulseCrypt",
    "symbol": "PLSCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29776/thumb/PulseCrypt.jpg?1696528707"
  },
  {
    "chainId": 1,
    "address": "0xaa68fd12a3b0f7aea66fe8f7111ae3f8d9ac5058",
    "name": "Tweety",
    "symbol": "TWEETY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30861/thumb/tweety_logo.png?1696529708"
  },
  {
    "chainId": 1,
    "address": "0x31adda225642a8f4d7e90d4152be6661ab22a5a2",
    "name": "Hypr Network",
    "symbol": "HYPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33071/thumb/655412078d33fa9fee8e3a1b_hypr_logo_200.png?1700598490"
  },
  {
    "chainId": 1,
    "address": "0xe5d2e173b120341face9e9970889c9fe64081ffd",
    "name": "Bluejay",
    "symbol": "BLU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28085/thumb/Bluejay.png?1696527095"
  },
  {
    "chainId": 1,
    "address": "0xb3207935ff56120f3499e8ad08461dd403bf16b8",
    "name": "dAMM",
    "symbol": "DAMM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28082/thumb/dAMM_logo_round_-_dark.png?1696527092"
  },
  {
    "chainId": 1,
    "address": "0x70d0ff0d3b3f5e69220c09befc70606fa5f89293",
    "name": "hiUNDEAD",
    "symbol": "HIUNDEAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29031/thumb/hiundead.png?1696528001"
  },
  {
    "chainId": 1,
    "address": "0xf433089366899d83a9f26a773d59ec7ecf30355e",
    "name": "Metal DAO",
    "symbol": "MTL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/763/thumb/Metal.png?1696501916"
  },
  {
    "chainId": 1,
    "address": "0x294559fa758c88d639fd085751e463fee7806eab",
    "name": "Metal Blockchain",
    "symbol": "METAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27222/thumb/MB-Logo.png?1710514210"
  },
  {
    "chainId": 1,
    "address": "0xb045f7f363fe4949954811b113bd56d208c67b23",
    "name": "Spider Tanks",
    "symbol": "SILK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28057/thumb/SILK_Logo.png?1696527069"
  },
  {
    "chainId": 1,
    "address": "0x42dae489f1ca3764aabe1907c22bc18776415fcd",
    "name": "VibingCatToken",
    "symbol": "VCT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x109548dc14c0b8d7908a168202a4ab08bb449613",
    "name": "Cloud Binary",
    "symbol": "CBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35733/thumb/CBY-200px.png?1709691425"
  },
  {
    "chainId": 1,
    "address": "0xefab7248d36585e2340e5d25f8a8d243e6e3193f",
    "name": "Dacxi",
    "symbol": "DACXI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4466/thumb/dacxi.png?1696505057"
  },
  {
    "chainId": 1,
    "address": "0x57f12fe6a4e5fe819eec699fadf9db2d06606bb4",
    "name": "Neptune Mutual",
    "symbol": "NPM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28235/thumb/Neptune_Mutual_Logo_Icon_Regular_128.png?1696527238"
  },
  {
    "chainId": 1,
    "address": "0xb3e6ee8d2c586fa03ab70aef96b8ae6d12d64ec7",
    "name": "hiFIDENZA",
    "symbol": "HIFIDENZA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28237/thumb/hifidenza.png?1696527240"
  },
  {
    "chainId": 1,
    "address": "0xac3211a5025414af2866ff09c23fc18bc97e79b1",
    "name": "Dovu  OLD ",
    "symbol": "DOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1072/thumb/dove.png?1696502174"
  },
  {
    "chainId": 1,
    "address": "0xa0ed3c520dc0632657ad2eaaf19e26c4fd431a84",
    "name": "Hippo Wallet",
    "symbol": "HPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28089/thumb/b77170dd-1dd1-4581-91b2-e352794fa045.jpg?1696527099"
  },
  {
    "chainId": 1,
    "address": "0x2b1d36f5b61addaf7da7ebbd11b35fd8cfb0de31",
    "name": "Interport Token",
    "symbol": "ITP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28338/thumb/ITP_Logo_200.png?1696527344"
  },
  {
    "chainId": 1,
    "address": "0xcf4c68db4c2fa0bf58df07b14f45ce7709a716ac",
    "name": "Dejitaru Shirudo",
    "symbol": "SHIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28500/thumb/200x200_3.PNG?1703024361"
  },
  {
    "chainId": 1,
    "address": "0xfc4b4ec763722b71eb1d729749b447a9645f5f30",
    "name": "DumbMoney",
    "symbol": "GME",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32058/thumb/DumbMoney_-_GME_-_Official_Logo_200x200px.png?1696530855"
  },
  {
    "chainId": 1,
    "address": "0x51395ade06eae126f590e7b06dc8f6baf511f13f",
    "name": "hiSQUIGGLE",
    "symbol": "HISQUIGGLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28127/thumb/hisquiggle.png?1696527134"
  },
  {
    "chainId": 1,
    "address": "0x570936840fa942c96ca4c57a170dda494dd9b9c6",
    "name": "Decentral ART",
    "symbol": "ART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29398/thumb/E58AC31F-1B46-4BAD-B8D8-E01551303E09.jpeg?1696528348"
  },
  {
    "chainId": 1,
    "address": "0xe22910d04bdc9d55570f3bc52e786b49dddf37b4",
    "name": "Gatsby Inu  OLD ",
    "symbol": "GATSBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29585/thumb/gatsby_logo_large.png?1696528524"
  },
  {
    "chainId": 1,
    "address": "0xa1d0e215a23d7030842fc67ce582a6afa3ccab83",
    "name": "DFI money",
    "symbol": "YFII",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11902/thumb/YFII-logo.78631676.png?1696511768"
  },
  {
    "chainId": 1,
    "address": "0x4c601dc69affb0d4fc8de1ac303705e432a4a27e",
    "name": "Konnect",
    "symbol": "KCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25756/thumb/EAO_5JhW_400x400.jpg?1696524842"
  },
  {
    "chainId": 1,
    "address": "0x49642110b712c1fd7261bc074105e9e44676c68f",
    "name": "DinoLFG",
    "symbol": "DINO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28239/thumb/F1giO3rXsAAFOsG.jpg?1696527242"
  },
  {
    "chainId": 1,
    "address": "0xd101dcc414f310268c37eeb4cd376ccfa507f571",
    "name": "ResearchCoin",
    "symbol": "RSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28146/thumb/RH_Logo_200x200.png?1696527152"
  },
  {
    "chainId": 1,
    "address": "0x377de38159bf87111c1523055b4a8c3a1a971722",
    "name": "GrokGrow",
    "symbol": "GROKGROW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33680/thumb/GrokGrow.png?1702718755"
  },
  {
    "chainId": 1,
    "address": "0x6d791ff76d2780f7c0403b8ad52de7b369a6154b",
    "name": "AFK Trading Bot",
    "symbol": "AFK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32813/thumb/AFK_Trading_Bot.png?1699578505"
  },
  {
    "chainId": 1,
    "address": "0x5e8422345238f34275888049021821e8e08caa1f",
    "name": "Frax Ether",
    "symbol": "FRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28284/thumb/frxETH_icon.png?1696527284"
  },
  {
    "chainId": 1,
    "address": "0xf21661d0d1d76d3ecb8e1b9f1c923dbfffae4097",
    "name": "Realio",
    "symbol": "RIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12206/thumb/Rio.png?1696512042"
  },
  {
    "chainId": 1,
    "address": "0xf98ab0874b13a7fdc39d7295dedd49850a5d426b",
    "name": "KIRA",
    "symbol": "KIRA",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28196/thumb/KIRA_CMC.png?1696527199"
  },
  {
    "chainId": 1,
    "address": "0xac3e018457b222d93114458476f3e3416abbe38f",
    "name": "Staked Frax Ether",
    "symbol": "SFRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28285/thumb/sfrxETH_icon.png?1696527285"
  },
  {
    "chainId": 1,
    "address": "0x2a8e1e676ec238d8a992307b495b45b3feaa5e86",
    "name": "Origin Dollar",
    "symbol": "OUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12589/thumb/ousd-logo-200x200.png?1696512399"
  },
  {
    "chainId": 1,
    "address": "0x7ca5af5ba3472af6049f63c1abc324475d44efc1",
    "name": "KONDUX",
    "symbol": "KNDX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28244/thumb/kndx.png?1696527247"
  },
  {
    "chainId": 1,
    "address": "0x16594930d16f3970e1a4317c6016555cb2e7b7fc",
    "name": "TokenBot",
    "symbol": "TKB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27106/thumb/tokenbot-logo-icon.png?1696526155"
  },
  {
    "chainId": 1,
    "address": "0x8d610e20481f4c4f3acb87bba9c46bef7795fdfe",
    "name": "Unity Network",
    "symbol": "UNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17143/thumb/unity.PNG?1696516702"
  },
  {
    "chainId": 1,
    "address": "0xe7ef051c6ea1026a70967e8f04da143c67fa4e1f",
    "name": "VetMe",
    "symbol": "VETME",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28787/thumb/VetMe.png?1709837744"
  },
  {
    "chainId": 1,
    "address": "0x96362879529c15c484eabc861c435940e7af22bb",
    "name": "RPG Maker Ai",
    "symbol": "RPGMAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33869/thumb/RPG.jpeg?1703131629"
  },
  {
    "chainId": 1,
    "address": "0xa0246c9032bc3a600820415ae600c6388619a14d",
    "name": "Harvest Finance",
    "symbol": "FARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12304/thumb/FARM_200x200.png?1696512134"
  },
  {
    "chainId": 1,
    "address": "0x5cb3ce6d081fb00d5f6677d196f2d70010ea3f4a",
    "name": "Busy",
    "symbol": "BUSY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14966/thumb/busy.PNG?1696514627"
  },
  {
    "chainId": 1,
    "address": "0x58fcaa970339a9b1f8c0a5b4f3fcd7af2ba3075e",
    "name": "Polar Sync",
    "symbol": "POLAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18102/thumb/polar-sync-logo-200.jpeg?1696517607"
  },
  {
    "chainId": 1,
    "address": "0xdc524e3c6910257744c1f93cf15e9f472b5bd236",
    "name": "Witch Token",
    "symbol": "WITCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17205/thumb/download_%2829%29.png?1696516761"
  },
  {
    "chainId": 1,
    "address": "0xf7e945fce8f19302aacc7e1418b0a0bdef89327b",
    "name": "Galvan",
    "symbol": "IZE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29442/thumb/Galvan_%28IZE%29.png?1696528389"
  },
  {
    "chainId": 1,
    "address": "0xb1191f691a355b43542bea9b8847bc73e7abb137",
    "name": "KIRO",
    "symbol": "KIRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12688/thumb/logo_kirobo-04.png?1696512491"
  },
  {
    "chainId": 1,
    "address": "0x9012744b7a564623b6c3e40b144fc196bdedf1a9",
    "name": "0xNumber",
    "symbol": "OXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33870/thumb/200x200.png?1703131812"
  },
  {
    "chainId": 1,
    "address": "0xba58444c8050ed9385b7417533a73644036d21eb",
    "name": "Lord of Dragons",
    "symbol": "LOGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29471/thumb/sym_1000.png?1696528416"
  },
  {
    "chainId": 1,
    "address": "0x44aad22afbb2606d7828ca1f8f9e5af00e779ae1",
    "name": "Homer",
    "symbol": "SIMPSON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30243/thumb/1111.png?1696529152"
  },
  {
    "chainId": 1,
    "address": "0x8207c1ffc5b6804f6024322ccf34f29c3541ae26",
    "name": "Origin Protocol",
    "symbol": "OGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3296/thumb/op.jpg?1696504006"
  },
  {
    "chainId": 1,
    "address": "0xe04f47ff45576249bc5083dfdf987e03d0550113",
    "name": "Moonsama",
    "symbol": "SAMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28308/thumb/Small.png?1696527312"
  },
  {
    "chainId": 1,
    "address": "0x599955aa9fbc197a1b717d8da6a7012cafe70ab3",
    "name": "Book of Pepe",
    "symbol": "BOPE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36314/thumb/1240A08A-6ABE-4913-9B52-05EC784212B7.jpeg?1711092031"
  },
  {
    "chainId": 1,
    "address": "0xfd0205066521550d7d7ab19da8f72bb004b4c341",
    "name": "Timeless",
    "symbol": "LIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28714/thumb/timeless-logo_3x.png?1696527695"
  },
  {
    "chainId": 1,
    "address": "0x8aec4bbdcfb451aa289bfbd3c2f4e34a44ada1be",
    "name": "dogwifhat Eth",
    "symbol": "DOGWIFHAT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35029/thumb/dogwifhat_Eth.png?1707155488"
  },
  {
    "chainId": 1,
    "address": "0xce391315b414d4c7555956120461d21808a69f3a",
    "name": "Bao Finance V2",
    "symbol": "BAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28287/thumb/bao-logo-mark-full-color-rgb-900px-w-72ppi.png?1696527286"
  },
  {
    "chainId": 1,
    "address": "0x2c5bc2ba3614fd27fcc7022ea71d9172e2632c16",
    "name": "Shib Original Vision",
    "symbol": "SOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29601/thumb/photo_2023-03-19_19-07-40_2.png?1696528539"
  },
  {
    "chainId": 1,
    "address": "0x3496b523e5c00a4b4150d6721320cddb234c3079",
    "name": "Numbers Protocol",
    "symbol": "NUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20495/thumb/NP_Social_media_profile_pic.png?1696519902"
  },
  {
    "chainId": 1,
    "address": "0x5f190f9082878ca141f858c1c90b4c59fe2782c5",
    "name": "Kudoe",
    "symbol": "KDOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28709/thumb/CG.png?1696527691"
  },
  {
    "chainId": 1,
    "address": "0xb9ef770b6a5e12e45983c5d80545258aa38f3b78",
    "name": "Zus",
    "symbol": "ZCN",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/4934/thumb/200x200_transparent.png?1696505474"
  },
  {
    "chainId": 1,
    "address": "0x3106a0a076bedae847652f42ef07fd58589e001f",
    "name": "Alkimi",
    "symbol": "ADS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17979/thumb/ADS.png?1706546465"
  },
  {
    "chainId": 1,
    "address": "0x6d52dfefb16bb9cdc78bfca09061e44574886626",
    "name": "CPUcoin",
    "symbol": "CPU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9544/thumb/uaz.ms.png?1696509626"
  },
  {
    "chainId": 1,
    "address": "0x38cf6cea814aefd01027a0bbf8a78b7aa95a698e",
    "name": "Bypass",
    "symbol": "BYPASS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32716/thumb/0x38cf6cea814aefd01027a0bbf8a78b7aa95a698e.jpg?1699000988"
  },
  {
    "chainId": 1,
    "address": "0x41d5d79431a913c4ae7d69a668ecdfe5ff9dfb68",
    "name": "Inverse Finance",
    "symbol": "INV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14205/thumb/inverse_finance.jpg?1696513922"
  },
  {
    "chainId": 1,
    "address": "0x08f40811c7d6c013744166f3d4cb1a9a92d3d54e",
    "name": "NightVerse Game",
    "symbol": "NVG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29059/thumb/nvg.png?1696528026"
  },
  {
    "chainId": 1,
    "address": "0x64d0f55cd8c7133a9d7102b13987235f486f2224",
    "name": "SwissBorg",
    "symbol": "BORG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2117/thumb/YJUrRy7r_400x400.png?1696503083"
  },
  {
    "chainId": 1,
    "address": "0x048aa6e69d5117bd665af4128383a6982b8367b3",
    "name": "VoxNET",
    "symbol": "VXON",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/28369/thumb/VoxNETlogo.png?1696527372"
  },
  {
    "chainId": 1,
    "address": "0xddcea43727558126c00eea772d660dfbe9c39613",
    "name": "Grizzly Bot",
    "symbol": "GRIZZLY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31611/thumb/gz2.jpg?1696530427"
  },
  {
    "chainId": 1,
    "address": "0xe0b0af14d5eecc2a7d7d723208593379e349a25c",
    "name": "MOZFIRE",
    "symbol": "MOZ",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36851/thumb/2024-04-05_21.03.52.jpg?1712626221"
  },
  {
    "chainId": 1,
    "address": "0x9e46a38f5daabe8683e10793b06749eef7d733d1",
    "name": "PolySwarm",
    "symbol": "NCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2843/thumb/ImcYCVfX_400x400.jpg?1696503602"
  },
  {
    "chainId": 1,
    "address": "0xfc1c93a2507975e98b9d0e9260ded61a00152bf1",
    "name": "Atlas Navi",
    "symbol": "NAVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28389/thumb/token_icon_-_logo_ATLAS_NAVI_200x200_transparent.png?1696527388"
  },
  {
    "chainId": 1,
    "address": "0xe53ec727dbdeb9e2d5456c3be40cff031ab40a55",
    "name": "SuperVerse",
    "symbol": "SUPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14040/thumb/SV-Logo-200x200.png?1706880312"
  },
  {
    "chainId": 1,
    "address": "0x4ee438be38f8682abb089f2bfea48851c5e71eaf",
    "name": "Cryptonovae",
    "symbol": "YAE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14693/thumb/yae.png?1696514365"
  },
  {
    "chainId": 1,
    "address": "0x571042b7138ee957a96a6820fce79c48fe2da816",
    "name": "Escrowed LBR",
    "symbol": "ESLBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34394/thumb/eslbr.png?1704785981"
  },
  {
    "chainId": 1,
    "address": "0xa95c5ebb86e0de73b4fb8c47a45b792cfea28c23",
    "name": "stake link",
    "symbol": "SDL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28406/thumb/stake.png?1696527404"
  },
  {
    "chainId": 1,
    "address": "0xe9fa21e671bcfb04e6868784b89c19d5aa2424ea",
    "name": "EurocoinToken",
    "symbol": "ECTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7634/thumb/logotoken200x200.png?1696507889"
  },
  {
    "chainId": 1,
    "address": "0x651e44d0df89055c00513d0efb9e8d1a0c843378",
    "name": "HappyBear",
    "symbol": "HAPPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29029/thumb/HappyCG.png?1696527999"
  },
  {
    "chainId": 1,
    "address": "0x88a9a52f944315d5b4e917b9689e65445c401e83",
    "name": "FEAR",
    "symbol": "FEAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15825/thumb/fear-logo-400-400.png?1696515443"
  },
  {
    "chainId": 1,
    "address": "0x61b34a012646cd7357f58ee9c0160c6d0021fa41",
    "name": "Elosys",
    "symbol": "ELO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34040/thumb/logo200x200.png?1703693735"
  },
  {
    "chainId": 1,
    "address": "0x0af0e83d064f160376303ac67dd9a7971af88d4c",
    "name": "Match Finance esLBR",
    "symbol": "MESLBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34395/thumb/20240109-160040.png?1704795144"
  },
  {
    "chainId": 1,
    "address": "0x7101a9392eac53b01e7c07ca3baca945a56ee105",
    "name": "X7101",
    "symbol": "X7101",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28419/thumb/X7101_LOGO_black_back_200_x_200_px.png?1696527417"
  },
  {
    "chainId": 1,
    "address": "0x7102dc82ef61bfb0410b1b1bf8ea74575bf0a105",
    "name": "X7102",
    "symbol": "X7102",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28420/thumb/X7102_LOGO_black_back_200_x_200_px.png?1696527417"
  },
  {
    "chainId": 1,
    "address": "0x6d60a8dfb16d09f67d46fcd36a0cd310078257ca",
    "name": "Centurion Invest",
    "symbol": "CIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28483/thumb/PHOTO-2022-05-27-11-34-42.jpg?1696527476"
  },
  {
    "chainId": 1,
    "address": "0x04c17b9d3b29a78f7bd062a57cf44fc633e71f85",
    "name": "IMPT",
    "symbol": "IMPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28486/thumb/IMPT_logo.png?1696527478"
  },
  {
    "chainId": 1,
    "address": "0xe1d7c7a4596b038ced2a84bf65b8647271c53208",
    "name": "NFTY",
    "symbol": "NFTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18741/thumb/coingecko.png?1696518206"
  },
  {
    "chainId": 1,
    "address": "0x8260328d0c405d9ca061d80199102ddc9089e43c",
    "name": "Dojo Supercomputer",
    "symbol": "DOJO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29728/thumb/Dojo_super.jpeg?1696528658"
  },
  {
    "chainId": 1,
    "address": "0xa0ef786bf476fe0810408caba05e536ac800ff86",
    "name": "Myria",
    "symbol": "MYRIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29273/thumb/myria.png?1696528226"
  },
  {
    "chainId": 1,
    "address": "0x9fd5555d0360adc2b5d92179e9bc36802bba8621",
    "name": "APES",
    "symbol": "APES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35030/thumb/mmexport1707027461489.jpg?1707156370"
  },
  {
    "chainId": 1,
    "address": "0x038ed1383763d704d4271fe856ac96b4557e9d06",
    "name": "Alphabet",
    "symbol": "ALPHABET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35172/thumb/ALPHABET_1-1920x970.png?1707748607"
  },
  {
    "chainId": 1,
    "address": "0x04cb855a5e5c4e92dd90bce82548669b4fe88702",
    "name": "BOOK",
    "symbol": "BOOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33034/thumb/book_%281%29.jpg?1700403065"
  },
  {
    "chainId": 1,
    "address": "0x7a3f7f6675514d4d611b442a4b76752f6ab77670",
    "name": "Tora Inu",
    "symbol": "TORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28531/thumb/z9b6m1yS_400x400.png?1696527524"
  },
  {
    "chainId": 1,
    "address": "0x1db11e86fa9b9a87813a4dd3f747eef12ed55a55",
    "name": "Digital Asset Right Token",
    "symbol": "DAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36898/thumb/DARTS_200x200.jpg?1712677273"
  },
  {
    "chainId": 1,
    "address": "0xf17a3fe536f8f7847f1385ec1bc967b2ca9cae8d",
    "name": "Alongside Crypto Market Index",
    "symbol": "AMKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28496/thumb/22999.png?1696527488"
  },
  {
    "chainId": 1,
    "address": "0x792833b894775bd769b3c602ba7172e59a83ab3f",
    "name": "City Boys",
    "symbol": "TOONS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32586/thumb/photo_2023-10-29_02.57.38.jpeg?1698564589"
  },
  {
    "chainId": 1,
    "address": "0x893c47bc1ff55c2269236ac7a4288681532161e0",
    "name": "Error 404",
    "symbol": "ERR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35174/thumb/TOKEN-404---SYMBOL-_-TYPO---PURPLE.jpg?1707749033"
  },
  {
    "chainId": 1,
    "address": "0x67f4c72a50f8df6487720261e188f2abe83f57d7",
    "name": "Wrapped POKT",
    "symbol": "WPOKT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31696/thumb/POKT.jpg?1703257336"
  },
  {
    "chainId": 1,
    "address": "0x33e07f5055173cf8febede8b21b12d1e2b523205",
    "name": "Etherland",
    "symbol": "ELAND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14432/thumb/eland_logo.png?1696514122"
  },
  {
    "chainId": 1,
    "address": "0x428dc22668e6f3468273634067e5545ed5417a3e",
    "name": "MiraQle",
    "symbol": "MQL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12278/thumb/200_200_%EB%AF%B8%EB%9D%BC%ED%81%B4_%EB%A1%9C%EA%B3%A0.png?1696512107"
  },
  {
    "chainId": 1,
    "address": "0x9f94b198ce85c19a846c2b1a4d523f40a747a850",
    "name": "havoc",
    "symbol": "HAVOC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31238/thumb/https___files.gitbook.com_v0_b_gitbook-x-prod.appspot.com_o_organizations_2FUh9uLmLoXru5erqLU4t2_2Flogo_2FQENXQAXAvv7kBdlqdyX2_2F2023-08-01_2013.06.56%281%29.jpg?1696530064"
  },
  {
    "chainId": 1,
    "address": "0xa974bafd35033ef7c3903bb37d109e7f8920c668",
    "name": "Coinmix",
    "symbol": "CM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32587/thumb/Tornado_only_transparent.png?1698565426"
  },
  {
    "chainId": 1,
    "address": "0xe95a203b1a91a908f9b9ce46459d101078c2c3cb",
    "name": "Ankr Staked ETH",
    "symbol": "ANKRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13403/thumb/aETHc.png?1696513165"
  },
  {
    "chainId": 1,
    "address": "0x68037790a0229e9ce6eaa8a99ea92964106c4703",
    "name": "Parallel",
    "symbol": "PAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14153/thumb/par_round_200.png?1696513872"
  },
  {
    "chainId": 1,
    "address": "0xccdb064a41afcd432bcea6f6d0d1e7c371d0b002",
    "name": "Qrolli",
    "symbol": "QR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29681/thumb/logoQrolli.png?1696528615"
  },
  {
    "chainId": 1,
    "address": "0xb23d80f5fefcddaa212212f028021b41ded428cf",
    "name": "Echelon Prime",
    "symbol": "PRIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29053/thumb/prime-logo-small-border_%282%29.png?1696528020"
  },
  {
    "chainId": 1,
    "address": "0x286f851b049ccce1419e09b6468dc3297f86a703",
    "name": "hiSEALS",
    "symbol": "HISEALS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29191/thumb/63f46d084b94830001ea9a60_hiSEALS_-__Logo.png?1696528150"
  },
  {
    "chainId": 1,
    "address": "0x049715c70fdbdd2be4814f76a53dc3d6f4367756",
    "name": "Nezuko",
    "symbol": "NEZUKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30014/thumb/Nezuko_Logo.png?1696528938"
  },
  {
    "chainId": 1,
    "address": "0x2ba592f78db6436527729929aaf6c908497cb200",
    "name": "Cream",
    "symbol": "CREAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11976/thumb/Cream.png?1696511834"
  },
  {
    "chainId": 1,
    "address": "0x8df4197669cf7af9fd29d768c932251a0c2e3e94",
    "name": "Ninja404",
    "symbol": "NINJA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35175/thumb/mascot.png?1707751710"
  },
  {
    "chainId": 1,
    "address": "0x3c6a7ab47b5f058be0e7c7fe1a4b7925b8aca40e",
    "name": "Cajutel",
    "symbol": "CAJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7354/thumb/RuHhMsFd_400x400.png?1696507637"
  },
  {
    "chainId": 1,
    "address": "0xd1d92f1c5524d93ced7bf6f418c8bf8abf68ac66",
    "name": "Bull Frog",
    "symbol": "BULL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31612/thumb/Bull200.png?1696530428"
  },
  {
    "chainId": 1,
    "address": "0x3001f57f8308b189eb412a64322aad5ef9951290",
    "name": "Geometric Energy Corporation",
    "symbol": "GEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33326/thumb/Gec.png?1701437951"
  },
  {
    "chainId": 1,
    "address": "0x0a638f07acc6969abf392bb009f216d22adea36d",
    "name": "Brickken",
    "symbol": "BKN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18474/thumb/brickken-coin.png?1696517960"
  },
  {
    "chainId": 1,
    "address": "0xfd09911130e6930bf87f2b0554c44f400bd80d3e",
    "name": "Ethix",
    "symbol": "ETHIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3031/thumb/ETHIX_icon_256x256-256.png?1696503766"
  },
  {
    "chainId": 1,
    "address": "0x337dd23d05c27bff099d3604938bfc23a9b25820",
    "name": "Marshall Fighting Championship",
    "symbol": "MFC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29390/thumb/91C38363-77C4-4D85-A3B7-EFA1C5745DDF.png?1696528341"
  },
  {
    "chainId": 1,
    "address": "0xf6aeaf0fe66cf2ef2e738ba465fb531ffe39b4e2",
    "name": "Aternos Chain",
    "symbol": "ATR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33399/thumb/ezgif.com-resize_%2830%29.png?1701684875"
  },
  {
    "chainId": 1,
    "address": "0x2d787d4f5005bd66ac910c2e821241625c406ed5",
    "name": "Berry",
    "symbol": "BERRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11016/thumb/Berry_logo.png?1696510963"
  },
  {
    "chainId": 1,
    "address": "0x9469d013805bffb7d3debe5e7839237e535ec483",
    "name": "Darwinia Network",
    "symbol": "RING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9443/thumb/RING.png?1696509535"
  },
  {
    "chainId": 1,
    "address": "0xdf4ef6ee483953fe3b84abd08c6a060445c01170",
    "name": "Wrapped Accumulate",
    "symbol": "WACME",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27207/thumb/accumulate-logo-200x200.png?1696526255"
  },
  {
    "chainId": 1,
    "address": "0x42bbfa2e77757c645eeaad1655e0911a7553efbc",
    "name": "Boba Network",
    "symbol": "BOBA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20285/thumb/Boba-200x200---white.png?1696519690"
  },
  {
    "chainId": 1,
    "address": "0x4591dbff62656e7859afe5e45f6f47d3669fbb28",
    "name": "Prisma mkUSD",
    "symbol": "MKUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31519/thumb/mkUSD_200.png?1696530329"
  },
  {
    "chainId": 1,
    "address": "0x46cca329970b33e1a007dd4ef0594a1cedb3e72a",
    "name": "Yesports",
    "symbol": "YESP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29767/thumb/YESP_TOKEN.png?1696528698"
  },
  {
    "chainId": 1,
    "address": "0x33caf58d14d7cd284cc2d7f2bc878d2d63c8956a",
    "name": "Only Possible On Ethereum",
    "symbol": "OPOE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32097/thumb/download.png?1696530895"
  },
  {
    "chainId": 1,
    "address": "0x5ee84583f67d5ecea5420dbb42b462896e7f8d06",
    "name": "PulseBitcoin",
    "symbol": "PLSB",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/28690/thumb/%E2%82%BF.png?1696527673"
  },
  {
    "chainId": 1,
    "address": "0x341c05c0e9b33c0e38d64de76516b2ce970bb3be",
    "name": "Diversified Staked ETH",
    "symbol": "DSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28751/thumb/dsETH-logo.png?1696527731"
  },
  {
    "chainId": 1,
    "address": "0xedc1004886d010751f74ec0ad223819f9f3b1910",
    "name": "AgriNode",
    "symbol": "AGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28727/thumb/IMG_20230110_070430_486_%281%29.png?1696527707"
  },
  {
    "chainId": 1,
    "address": "0x93eeb426782bd88fcd4b48d7b0368cf061044928",
    "name": "The Rug Game",
    "symbol": "TRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28750/thumb/TRG_Logo_Circular_200px.png?1696527729"
  },
  {
    "chainId": 1,
    "address": "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
    "name": "USDC",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/6319/thumb/usdc.png?1696506694"
  },
  {
    "chainId": 1,
    "address": "0xe7eaec9bca79d537539c00c58ae93117fb7280b9",
    "name": "Doge Protocol",
    "symbol": "DOGEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23689/thumb/LjhKBldd_400x400.png?1696522890"
  },
  {
    "chainId": 1,
    "address": "0x29fa1fee0f4f0ab0e36ef7ab8d7a35439ec6be75",
    "name": "SafeStake",
    "symbol": "DVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30016/thumb/safestake.png?1696528940"
  },
  {
    "chainId": 1,
    "address": "0x1d2d925887f442a27b5393bd5dfbe059910b1ac0",
    "name": "ORMIT",
    "symbol": "ORMIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36771/thumb/1000005541.jpg?1712285663"
  },
  {
    "chainId": 1,
    "address": "0x4295c8556afee00264c0789dde2ddd2dba71acfe",
    "name": "Bidao Smart Chain",
    "symbol": "BISC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29825/thumb/bisc200.png?1696528753"
  },
  {
    "chainId": 1,
    "address": "0xf4b5470523ccd314c6b9da041076e7d79e0df267",
    "name": "blockbank",
    "symbol": "BBANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15081/thumb/blockbank.jpg.png?1696514740"
  },
  {
    "chainId": 1,
    "address": "0x60be1e1fe41c1370adaf5d8e66f07cf1c2df2268",
    "name": "Perion",
    "symbol": "PERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23302/thumb/PERC.png?1696522520"
  },
  {
    "chainId": 1,
    "address": "0x38029c62dfa30d9fd3cadf4c64e9b2ab21dbda17",
    "name": "Dubbz",
    "symbol": "DUBBZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28665/thumb/D8EACA06-18ED-4999-8B3A-6339F9E021CE.jpeg?1696527650"
  },
  {
    "chainId": 1,
    "address": "0x66a3a58f812d0f433daaf1d96e14fd72d1d03d67",
    "name": "YAMA Inu",
    "symbol": "YAMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31636/thumb/Yama_Inu_Coin_3.png?1696530451"
  },
  {
    "chainId": 1,
    "address": "0x0d438f3b5175bebc262bf23753c1e53d03432bde",
    "name": "Wrapped NXM",
    "symbol": "WNXM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11890/thumb/wNXM.png?1696511757"
  },
  {
    "chainId": 1,
    "address": "0x147faf8de9d8d8daae129b187f0d02d819126750",
    "name": "GeoDB",
    "symbol": "GEO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11130/thumb/geodb.png?1696511067"
  },
  {
    "chainId": 1,
    "address": "0xc76d53f988820fe70e01eccb0248b312c2f1c7ca",
    "name": "Inu Token",
    "symbol": "INU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30755/thumb/0xc76d53f988820fe70e01eccb0248b312c2f1c7ca.png?1696529624"
  },
  {
    "chainId": 1,
    "address": "0xcd5fe23c85820f7b72d0926fc9b05b43e359b7ee",
    "name": "Wrapped eETH",
    "symbol": "WEETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33033/thumb/weETH.png?1701438396"
  },
  {
    "chainId": 1,
    "address": "0xcfcffe432a48db53f59c301422d2edd77b2a88d7",
    "name": "Texan",
    "symbol": "TEXAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28731/thumb/texan_logo_200x200.png?1696527711"
  },
  {
    "chainId": 1,
    "address": "0x4385328cc4d643ca98dfea734360c0f596c83449",
    "name": "tomiNet",
    "symbol": "TOMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28730/thumb/logo_for_token.png?1696527710"
  },
  {
    "chainId": 1,
    "address": "0x8ef32a03784c8fd63bbf027251b9620865bd54b6",
    "name": "Bullet Gate Betting Token",
    "symbol": "BULLET",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31242/thumb/2023-08-09_14.53.31.png?1696530068"
  },
  {
    "chainId": 1,
    "address": "0x5957c351946d45f74d1a6f034cd9b6ac6654e0d7",
    "name": "Chart Roulette",
    "symbol": "CR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33111/thumb/IMG_20231121_180645_701.jpg?1700707947"
  },
  {
    "chainId": 1,
    "address": "0x425d105913a04f4f95311788195ba13c3e82c3b9",
    "name": "Apollo Token",
    "symbol": "APOLLO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29861/thumb/Apollo.jpeg?1696528787"
  },
  {
    "chainId": 1,
    "address": "0xde30da39c46104798bb5aa3fe8b9e0e1f348163f",
    "name": "Gitcoin",
    "symbol": "GTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15810/thumb/gitcoin.png?1696515429"
  },
  {
    "chainId": 1,
    "address": "0x94d40b49f020bfebba1a80a0191eb3737b90e8d3",
    "name": "Mintera",
    "symbol": "MNTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28774/thumb/Mintera_Logo_CoinGecko.png?1696527753"
  },
  {
    "chainId": 1,
    "address": "0x6b0b3a982b4634ac68dd83a4dbf02311ce324181",
    "name": "Artificial Liquid Intelligence",
    "symbol": "ALI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22062/thumb/Logo_Circle_720_%281%29.png?1696521405"
  },
  {
    "chainId": 1,
    "address": "0xfb0489e9753b045ddb35e39c6b0cc02ec6b99ac5",
    "name": "AGG",
    "symbol": "AGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36772/thumb/agglogo_200.png?1712286053"
  },
  {
    "chainId": 1,
    "address": "0x91368eef9ab8bdc727e4506897656abb87282003",
    "name": "Network Spirituality",
    "symbol": "NET",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31430/thumb/OGZy3_zy_400x400.jpg?1696530244"
  },
  {
    "chainId": 1,
    "address": "0xd9788f3931ede4d5018184e198699dc6d66c1915",
    "name": "Aave yVault",
    "symbol": "YVAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28784/thumb/yvAAVE-128-0xd9788f3931Ede4D5018184E198699dC6d66C1915.png?1696527763"
  },
  {
    "chainId": 1,
    "address": "0x4a3fe75762017db0ed73a71c9a06db7768db5e66",
    "name": "COMP yVault",
    "symbol": "YVCOMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28786/thumb/yvCOMP-128-0x4A3FE75762017DB0eD73a71C9A06db7768DB5e66.png?1696527765"
  },
  {
    "chainId": 1,
    "address": "0xdb25ca703181e7484a155dd612b06f57e12be5f0",
    "name": "YFI yVault",
    "symbol": "YVYFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28785/thumb/yvYFI-128-0xE14d13d8B3b85aF791b2AADD661cDBd5E6097Db1.png?1696527764"
  },
  {
    "chainId": 1,
    "address": "0xf32aa187d5bc16a2c02a6afb7df1459d0d107574",
    "name": "HachikoInu",
    "symbol": "INU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16392/thumb/hachikoinu-200px.png?1696515990"
  },
  {
    "chainId": 1,
    "address": "0x27b5739e22ad9033bcbf192059122d163b60349d",
    "name": "Staked Yearn CRV Vault",
    "symbol": "ST-YCRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27623/thumb/st-yCRV-128-0x27B5739e22ad9033bcBf192059122d163b60349D.png?1696526654"
  },
  {
    "chainId": 1,
    "address": "0x39142c18b6db2a8a41b7018f49e1478837560cad",
    "name": "STATS",
    "symbol": "STATS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28853/thumb/stats.png?1696527827"
  },
  {
    "chainId": 1,
    "address": "0xa354f35829ae975e850e23e9615b11da1b3dc4de",
    "name": "USDC yVault",
    "symbol": "YVUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28779/thumb/yvUSDC-128-0xe2F6b9773BF3A015E2aA70741Bde1498bdB9425b.png?1696527758"
  },
  {
    "chainId": 1,
    "address": "0x3b27f92c0e212c671ea351827edf93db27cc0c65",
    "name": "USDT yVault",
    "symbol": "YVUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28780/thumb/yvUSDT-128-0x7Da96a3891Add058AdA2E826306D812C638D87a7.png?1696527759"
  },
  {
    "chainId": 1,
    "address": "0xfbeb78a723b8087fd2ea7ef1afec93d35e8bed42",
    "name": "UNI yVault",
    "symbol": "YVUNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28781/thumb/yvUNI-128-0xFBEB78a723b8087fD2ea7Ef1afEc93d35E8Bed42.png?1696527760"
  },
  {
    "chainId": 1,
    "address": "0x00000000051b48047be6dc0ada6de5c3de86a588",
    "name": "Baby Shiba Inu",
    "symbol": "BABYSHIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31350/thumb/100x100_%287%29_%282%29.png?1696530167"
  },
  {
    "chainId": 1,
    "address": "0xbe00734799a67a62af2819825580318ac1b1e4ec",
    "name": "ordinex",
    "symbol": "ORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29084/thumb/tw-logo%281%29.png?1696528048"
  },
  {
    "chainId": 1,
    "address": "0xab85fc558d722a2b7c040ffb77db676bd9e7d322",
    "name": "MOROS NET",
    "symbol": "MOROS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33147/thumb/Moros.jpg?1700822187"
  },
  {
    "chainId": 1,
    "address": "0x42b91f1d05afea671a2da3c780eda2abf0a2a366",
    "name": "Mineable",
    "symbol": "MNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29272/thumb/mnb.png?1696528225"
  },
  {
    "chainId": 1,
    "address": "0x03a0657fc10d7169305a0179eb80d4f568cda705",
    "name": "Blitz Bots",
    "symbol": "BLITZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33854/thumb/photo_2023-12-16_02-17-20.png?1703125111"
  },
  {
    "chainId": 1,
    "address": "0x17c50d62e6e8d20d2dc18e9ad79c43263d0720d9",
    "name": "Not Financial Advice",
    "symbol": "NFAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28758/thumb/NFA_Labs_Logo_%281%29.png?1710245293"
  },
  {
    "chainId": 1,
    "address": "0x8dfc8cc3201425669fae803e1eb125cddd4189ec",
    "name": "Okage Inu",
    "symbol": "OKAGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28829/thumb/Okage_Inu.png?1696527805"
  },
  {
    "chainId": 1,
    "address": "0x0c943300d604c732117db2755873a0a0683ee7a9",
    "name": "Paradise Defi",
    "symbol": "PDF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28827/thumb/photo_2023-01-16_15.08.09.jpeg?1696527803"
  },
  {
    "chainId": 1,
    "address": "0x6bc08509b36a98e829dffad49fde5e412645d0a3",
    "name": "WoofWork io",
    "symbol": "WOOF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28903/thumb/WWlogoTransparent_200x200.png?1696527879"
  },
  {
    "chainId": 1,
    "address": "0x7138eb0d563f3f6722500936a11dcae99d738a2c",
    "name": "Lif3",
    "symbol": "LIF3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31291/thumb/Tokens.png?1696530112"
  },
  {
    "chainId": 1,
    "address": "0xed328e9c1179a30ddc1e7595e036aed8760c22af",
    "name": "Metacade",
    "symbol": "MCADE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29764/thumb/Square_Profile_Photo_1080_x_1080px.png?1699396445"
  },
  {
    "chainId": 1,
    "address": "0x8c1bed5b9a0928467c9b1341da1d7bd5e10b6549",
    "name": "Liquid Staked ETH",
    "symbol": "LSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28848/thumb/LsETH-receipt-token-circle.png?1696527824"
  },
  {
    "chainId": 1,
    "address": "0xd346e8ada104093adcf5f4186087e1ad309bb3b3",
    "name": "Shibnaut",
    "symbol": "SHIBN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28872/thumb/66EDE430-1CD7-4524-B610-FC58A7DCDA3E.jpeg?1696527850"
  },
  {
    "chainId": 1,
    "address": "0x6f3277ad0782a7da3eb676b85a8346a100bf9c1c",
    "name": "DogPad Finance",
    "symbol": "DOGPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28845/thumb/DogPad.png?1696527821"
  },
  {
    "chainId": 1,
    "address": "0x1e4746dc744503b53b4a082cb3607b169a289090",
    "name": "IPOR",
    "symbol": "IPOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28373/thumb/IPOR-token-200x200.png?1696527376"
  },
  {
    "chainId": 1,
    "address": "0x97872eafd79940c7b24f7bcc1eadb1457347adc9",
    "name": "Strips Finance",
    "symbol": "STRP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18327/thumb/Logo-Strips-200-x-200px---without-words.png?1696517818"
  },
  {
    "chainId": 1,
    "address": "0x7a786dac1f315c8a0e9962172ad8ae0c04d9c9b6",
    "name": "MemeDAO",
    "symbol": "MEMD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30468/thumb/memeDAO_200x200.png?1696529354"
  },
  {
    "chainId": 1,
    "address": "0xe9287543684f440b2f29983a3472987bf7b0ed12",
    "name": "Shiba Nodes",
    "symbol": "SHINO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28890/thumb/output-onlinepngtools_%281%29.png?1696527866"
  },
  {
    "chainId": 1,
    "address": "0xfe67a4450907459c3e1fff623aa927dd4e28c67a",
    "name": "Connext",
    "symbol": "NEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31293/thumb/connext.png?1696530113"
  },
  {
    "chainId": 1,
    "address": "0x91be752d438d5f804345b5acb18de0c431ad470f",
    "name": "ETHETF",
    "symbol": "ETHETF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32998/thumb/ETHETF_Logo_200x200.png?1700114369"
  },
  {
    "chainId": 1,
    "address": "0x36e43065e977bc72cb86dbd8405fae7057cdc7fd",
    "name": "ArchAngel",
    "symbol": "ARCHA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/18814/thumb/Archa_200_x_200_PNG.png?1696518276"
  },
  {
    "chainId": 1,
    "address": "0x4236f8aaf2b1f3a28420eb15b8e0ddf63201a95e",
    "name": "Bermuda",
    "symbol": "BMDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28878/thumb/LOGO.png?1696527855"
  },
  {
    "chainId": 1,
    "address": "0x24ae2da0f361aa4be46b48eb19c91e02c5e4f27e",
    "name": "mevETH",
    "symbol": "MEVETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32153/thumb/MEV_logo_2.png?1696598728"
  },
  {
    "chainId": 1,
    "address": "0x76bc677d444f1e9d57daf5187ee2b7dc852745ae",
    "name": "Offshift",
    "symbol": "XFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11977/thumb/CsBrPiA.png?1696511835"
  },
  {
    "chainId": 1,
    "address": "0x2a304fda5a85182dca1d03741bb2f07881b9e095",
    "name": "DCOMY",
    "symbol": "DCO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29007/thumb/15-38-16-200x200.png?1696527978"
  },
  {
    "chainId": 1,
    "address": "0x35fa164735182de50811e8e2e824cfb9b6118ac2",
    "name": "ether fi Staked ETH",
    "symbol": "EETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33049/thumb/ether.fi_eETH.png?1700473063"
  },
  {
    "chainId": 1,
    "address": "0x674c6ad92fd080e4004b2312b45f796a192d27a0",
    "name": "Neutrino Index Token",
    "symbol": "XTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10117/thumb/XTN.png?1712339473"
  },
  {
    "chainId": 1,
    "address": "0x820802fa8a99901f52e39acd21177b0be6ee2974",
    "name": "EUROe Stablecoin",
    "symbol": "EUROE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28913/thumb/euroe-200x200-round.png?1696527889"
  },
  {
    "chainId": 1,
    "address": "0x235c8ee913d93c68d2902a8e0b5a643755705726",
    "name": "tehBag",
    "symbol": "BAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28950/thumb/tehBag-Logo.png?1696527923"
  },
  {
    "chainId": 1,
    "address": "0x138c2f1123cf3f82e4596d097c118eac6684940b",
    "name": "Alpha Coin",
    "symbol": "ALPHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22199/thumb/logo2023.png?1696521542"
  },
  {
    "chainId": 1,
    "address": "0xe1c7e30c42c24582888c758984f6e382096786bd",
    "name": "Curate",
    "symbol": "XCUR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13327/thumb/400x400_%281%29_%283%29_%282%29.png?1696513096"
  },
  {
    "chainId": 1,
    "address": "0xecbee2fae67709f718426ddc3bf770b26b95ed20",
    "name": "Clips",
    "symbol": "CLIPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30151/thumb/clips-logo.png?1696529071"
  },
  {
    "chainId": 1,
    "address": "0xcb50350ab555ed5d56265e096288536e8cac41eb",
    "name": "0xCoco",
    "symbol": "COCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31102/thumb/FL1-g3Ns_400x400.jpg?1696529933"
  },
  {
    "chainId": 1,
    "address": "0x9b110fda4e20db18ad7052f8468a455de7449eb6",
    "name": "Calvaria  DoE",
    "symbol": "RIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28817/thumb/icon200x200.png?1696527793"
  },
  {
    "chainId": 1,
    "address": "0x562e362876c8aee4744fc2c6aac8394c312d215d",
    "name": "Optimus AI",
    "symbol": "OPTI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29328/thumb/photo_2023-11-14_22-41-11.png?1700091127"
  },
  {
    "chainId": 1,
    "address": "0x7d8d7c26179b7a6aebbf66a91c38ed92d5b4996b",
    "name": "Teh Fund",
    "symbol": "FUND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28969/thumb/IMG_20230203_091543_056.png?1696527942"
  },
  {
    "chainId": 1,
    "address": "0x043312456f73d8014d9b84f4337de54995cd2a5b",
    "name": "Bonk2 0",
    "symbol": "BONK20",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33685/thumb/20231211_114717.jpg?1702731710"
  },
  {
    "chainId": 1,
    "address": "0xb870679a7fa65b924026f496de7f27c1dd0e5c5f",
    "name": "Hello Pets",
    "symbol": "PET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14354/thumb/hello_pets.jpg?1696514040"
  },
  {
    "chainId": 1,
    "address": "0x7e5981c2e072f53a0323d3d80baca3e31fb1550c",
    "name": "JovJou",
    "symbol": "JOVJOU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32424/thumb/jj_2.png?1698118523"
  },
  {
    "chainId": 1,
    "address": "0xf134519cbe2042b06ee7ce20df51d09b55559896",
    "name": "Mochi",
    "symbol": "MOCHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28981/thumb/mochi-logo.png?1696527954"
  },
  {
    "chainId": 1,
    "address": "0x8954d907520532c1f0d89d42569232fd0f995fdf",
    "name": "Tradix",
    "symbol": "TX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28997/thumb/Tradix_Logo.png?1696527969"
  },
  {
    "chainId": 1,
    "address": "0x045109cf1be9edec048aa0b3d7a323154a1aea65",
    "name": "Elevate Token",
    "symbol": "ELEV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30085/thumb/elevate.jpeg?1696529009"
  },
  {
    "chainId": 1,
    "address": "0x52b552c4a698c9b1de4c7fe0dd4462af141f7c3e",
    "name": "Alpha Bot Calls",
    "symbol": "ABC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31168/thumb/ABC_Token_CG.png?1696529996"
  },
  {
    "chainId": 1,
    "address": "0x06b884e60794ce02aafab13791b59a2e6a07442f",
    "name": "Unbanked",
    "symbol": "UNBNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20171/thumb/logo_-_2021-11-11T115427.135.png?1696519584"
  },
  {
    "chainId": 1,
    "address": "0x4c05f1e65069a8c0694470720be77e50d2b212d6",
    "name": "0xFreelance",
    "symbol": "0XFREE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32173/thumb/0xfreelance-icon-200px.png?1696742117"
  },
  {
    "chainId": 1,
    "address": "0x39b0e13a29c2a27ce88cebd21262a232b0633613",
    "name": "Grape Governance Token",
    "symbol": "GGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30807/thumb/grape_logo.png?1696529664"
  },
  {
    "chainId": 1,
    "address": "0x3d1e5cf16077f349e999d6b21a4f646e83cd90c5",
    "name": "Stakehouse dETH",
    "symbol": "DETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33657/thumb/dETH.png?1702598011"
  },
  {
    "chainId": 1,
    "address": "0x0d8ca4b20b115d4da5c13dc45dd582a5de3e78bf",
    "name": "Generaitiv",
    "symbol": "GAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29158/thumb/logo-200x200.png?1696528117"
  },
  {
    "chainId": 1,
    "address": "0xf6043bea98b07f0bea7813aeb22d0cb70c91c0c4",
    "name": "MemeDao Ai",
    "symbol": "MDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29021/thumb/MemedaoLogo.png?1696527991"
  },
  {
    "chainId": 1,
    "address": "0xf1fb4cf949277adc3f89c0d6091100789033b7b5",
    "name": "Aibot",
    "symbol": "AIBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31324/thumb/aibot_200.png?1696530143"
  },
  {
    "chainId": 1,
    "address": "0xffd822149fa6749176c7a1424e71a417f26189c8",
    "name": "Nothing Token",
    "symbol": "THING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30327/thumb/fulltrans_%281%29.png?1696529228"
  },
  {
    "chainId": 1,
    "address": "0xdeadface8503399df4b083ef42fa8e02fd39dead",
    "name": "Tribe Token",
    "symbol": "TRIBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33036/thumb/Tribe.jpg?1700403321"
  },
  {
    "chainId": 1,
    "address": "0xa876f27f13a9eb6e621202cefdd5afc4a90e6457",
    "name": "Icy",
    "symbol": "IC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29666/thumb/1200.png?1696528601"
  },
  {
    "chainId": 1,
    "address": "0xfaba6f8e4a5e8ab82f62fe7c39859fa577269be3",
    "name": "Ondo",
    "symbol": "ONDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26580/thumb/ONDO.png?1696525656"
  },
  {
    "chainId": 1,
    "address": "0x97c8321b9434db4110b1d75a551e561036ff2e6a",
    "name": "WagerOn",
    "symbol": "WAGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32385/thumb/logo.png?1698047095"
  },
  {
    "chainId": 1,
    "address": "0x4581af35199bbde87a89941220e04e27ce4b0099",
    "name": "Maximus Pool Party",
    "symbol": "PARTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35369/thumb/Party.png?1708401334"
  },
  {
    "chainId": 1,
    "address": "0x669c01caf0edcad7c2b8dc771474ad937a7ca4af",
    "name": "Wrapped Minima",
    "symbol": "WMINIMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29011/thumb/CoinGeko_icon_%28500x500%29.png?1696527982"
  },
  {
    "chainId": 1,
    "address": "0x207e14389183a94343942de7afbc607f57460618",
    "name": "AngelBlock",
    "symbol": "THOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25058/thumb/Tholos_circle_gradient_200x200.png?1696524208"
  },
  {
    "chainId": 1,
    "address": "0xeb2f5a4e1441df330670dc7b0cf4eac0f7ab5fa5",
    "name": "hiFRIENDS",
    "symbol": "HIFRIENDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29086/thumb/hifriends.png?1696528050"
  },
  {
    "chainId": 1,
    "address": "0x77e06c9eccf2e797fd462a92b6d7642ef85b0a44",
    "name": "Wrapped TAO",
    "symbol": "WTAO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29087/thumb/wtao.png?1696528051"
  },
  {
    "chainId": 1,
    "address": "0x04506dddbf689714487f91ae1397047169afcf34",
    "name": "Unlock Maverick",
    "symbol": "UNKMAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31264/thumb/clean.jpg?1696530088"
  },
  {
    "chainId": 1,
    "address": "0xb685145d7f127b9093d7f9278bae902ef59ff486",
    "name": "FREQAI",
    "symbol": "FREQAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29122/thumb/The_Razors_Edge.png?1696528084"
  },
  {
    "chainId": 1,
    "address": "0x1a6658f40e51b372e593b7d2144c1402d5cf33e8",
    "name": "PUBLC",
    "symbol": "PUBLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29074/thumb/PUBLX_Logo_%28200x200%29.png?1696528040"
  },
  {
    "chainId": 1,
    "address": "0xce20bb92ccf9bbf5091ef85649e71e552819ad8c",
    "name": "Smart Game Finance",
    "symbol": "SMART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29072/thumb/smart.png?1696528038"
  },
  {
    "chainId": 1,
    "address": "0xdd69db25f6d620a7bad3023c5d32761d353d3de9",
    "name": "Goerli ETH",
    "symbol": "GETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29217/thumb/goerli-eth.png?1696528175"
  },
  {
    "chainId": 1,
    "address": "0xe3f03cef497c81d2b28a2fae63ae84b373028718",
    "name": "MARVIN",
    "symbol": "MARVIN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34548/thumb/cg.jpg?1705341719"
  },
  {
    "chainId": 1,
    "address": "0x1032abe2902a23ddcbab085c20e0e69c33ceb8fa",
    "name": "Pepe Predator",
    "symbol": "SNAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30205/thumb/Logo_FOR_CG.png?1696529116"
  },
  {
    "chainId": 1,
    "address": "0x69fa8e7f6bf1ca1fb0de61e1366f7412b827cc51",
    "name": "Enreach",
    "symbol": "NRCH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14694/thumb/enreachdao.jpg?1696514366"
  },
  {
    "chainId": 1,
    "address": "0x06450dee7fd2fb8e39061434babcfc05599a6fb8",
    "name": "XEN Crypto",
    "symbol": "XEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27713/thumb/Xen.jpeg?1696526739"
  },
  {
    "chainId": 1,
    "address": "0x4cf89ca06ad997bc732dc876ed2a7f26a9e7f361",
    "name": "Mysterium",
    "symbol": "MYST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/757/thumb/mysterium.png?1696501911"
  },
  {
    "chainId": 1,
    "address": "0xe70e1d6a2e971ff6debc21dd2540064c25c9bc0d",
    "name": "Spider Spirit",
    "symbol": "SPIDER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30741/thumb/200x2001.png?1696529611"
  },
  {
    "chainId": 1,
    "address": "0xfa4baa6951b6ee382e9ff9af2d523278b99ca6d0",
    "name": "House",
    "symbol": "HOUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31202/thumb/hm_icon_200x200.png?1696530029"
  },
  {
    "chainId": 1,
    "address": "0x4b520c812e8430659fc9f12f6d0c39026c83588d",
    "name": "Decentral Games",
    "symbol": "DG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21173/thumb/Decentral_Games_Logo-1.png?1696520549"
  },
  {
    "chainId": 1,
    "address": "0x7a8adcf432ebcc2311b955d176ee4bfed13bb9a7",
    "name": "MandoX",
    "symbol": "MANDOX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24968/thumb/NmqN8g7.png?1696524121"
  },
  {
    "chainId": 1,
    "address": "0xc5102fe9359fd9a28f877a67e36b0f050d81a3cc",
    "name": "Hop Protocol",
    "symbol": "HOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25445/thumb/hop.png?1696524577"
  },
  {
    "chainId": 1,
    "address": "0x0888fd2960ddf57cfb561b59d73dd2b9524f6a26",
    "name": "Mikawa Inu",
    "symbol": "MIKAWA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29168/thumb/IMG_20230215_172257_635.png?1696528126"
  },
  {
    "chainId": 1,
    "address": "0xddf7fd345d54ff4b40079579d4c4670415dbfd0a",
    "name": "SocialGood",
    "symbol": "SG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3948/thumb/logo_200.png?1696504592"
  },
  {
    "chainId": 1,
    "address": "0xb6d78683a4e54b91031acb41510bd8e144fed025",
    "name": "CoinForge",
    "symbol": "CNFRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35490/thumb/CoinForge.png?1708884142"
  },
  {
    "chainId": 1,
    "address": "0x3540dfcad7cf102a2e44aa0e2132fab1c89d7eae",
    "name": "SchwiftAI",
    "symbol": "SWAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29301/thumb/SchwiftAI.png?1696528253"
  },
  {
    "chainId": 1,
    "address": "0x36c833eed0d376f75d1ff9dfdee260191336065e",
    "name": "Gitcoin Staked ETH Index",
    "symbol": "GTCETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29171/thumb/gtcETH-token-logo.png?1696528129"
  },
  {
    "chainId": 1,
    "address": "0x68b36248477277865c64dfc78884ef80577078f3",
    "name": "Everybody",
    "symbol": "HOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32272/thumb/HOLD_logo_final_%282%29.png?1697179882"
  },
  {
    "chainId": 1,
    "address": "0xe6f4a40156c9e8c7addda66848bbb99fdedecf84",
    "name": "DeTensor",
    "symbol": "DETENSOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36596/thumb/detensor.jpeg?1711950178"
  },
  {
    "chainId": 1,
    "address": "0xbb556b0ee2cbd89ed95ddea881477723a3aa8f8b",
    "name": "AliceNet",
    "symbol": "ALCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29417/thumb/LinkedIn_-_Profile_300_x_300.png?1696528366"
  },
  {
    "chainId": 1,
    "address": "0x68ffb43026cedc070bfc01b2203d90cab62ccf30",
    "name": "Turk Shiba",
    "symbol": "TUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29131/thumb/IMG_20230220_215553_755.jpg?1696528092"
  },
  {
    "chainId": 1,
    "address": "0x584bc13c7d411c00c01a62e8019472de68768430",
    "name": "Hegic",
    "symbol": "HEGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12454/thumb/new.png?1696512274"
  },
  {
    "chainId": 1,
    "address": "0x420698ebc9b7c225731c02d887d0729057339d39",
    "name": "Chuck",
    "symbol": "CHUCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36750/thumb/IMG_0741.jpeg?1712218300"
  },
  {
    "chainId": 1,
    "address": "0x8b0fde007458ee153bd0f66cd448af5fb3d99b43",
    "name": "Mastermind",
    "symbol": "MASTERMIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29205/thumb/Mastermind.png?1696528164"
  },
  {
    "chainId": 1,
    "address": "0xed1273928ba97eed7b49e82c2f39d512d7591112",
    "name": "NerdBot",
    "symbol": "NERD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32386/thumb/nerdbot.png?1698047412"
  },
  {
    "chainId": 1,
    "address": "0x014337b35167b3711195361bb85259009e50a8a4",
    "name": "DecentraNode",
    "symbol": "DNODE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35481/thumb/photo_2024-02-24_18.26.48.jpeg?1708849459"
  },
  {
    "chainId": 1,
    "address": "0xb0ed33f79d89541dfdcb04a8f04bc2c6be025ecc",
    "name": "ZeroLiquid",
    "symbol": "ZERO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29499/thumb/IMG_20230726_094105_915.png?1696528444"
  },
  {
    "chainId": 1,
    "address": "0x2559813bbb508c4c79e9ccce4703bcb1f149edd7",
    "name": "Hourglass",
    "symbol": "WAIT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27516/thumb/Hourglass-Logo.png?1696526555"
  },
  {
    "chainId": 1,
    "address": "0x0c5cb676e38d6973837b9496f6524835208145a2",
    "name": "KaboChan",
    "symbol": "KABO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34631/thumb/KaboChan.png?1705548332"
  },
  {
    "chainId": 1,
    "address": "0x4096fc7119040175589387656f7c6073265f4096",
    "name": "4096",
    "symbol": "4096",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33687/thumb/icon2.png?1702732239"
  },
  {
    "chainId": 1,
    "address": "0x6c22910c6f75f828b305e57c6a54855d8adeabf8",
    "name": "Satoshis Vision",
    "symbol": "SATS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29303/thumb/SATS_123-removebg-preview_%281%29.png?1696528255"
  },
  {
    "chainId": 1,
    "address": "0x80c62fe4487e1351b47ba49809ebd60ed085bf52",
    "name": "Clover Finance",
    "symbol": "CLV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15278/thumb/photo_2022-03-24_10-22-33.jpg?1696514930"
  },
  {
    "chainId": 1,
    "address": "0x8b802513d4aa6f349b197a4ea4c26563cd6fd5b2",
    "name": "HIgher IMO",
    "symbol": "HIGHER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33518/thumb/download.png?1702132819"
  },
  {
    "chainId": 1,
    "address": "0xc32db1d3282e872d98f6437d3bcfa57801ca6d5c",
    "name": "VoldemortTrumpRobotnik 10Neko",
    "symbol": "ETHEREUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31061/thumb/icon_medium.jpg?1696529895"
  },
  {
    "chainId": 1,
    "address": "0x1ea48b9965bb5086f3b468e50ed93888a661fc17",
    "name": "Moneta DAO",
    "symbol": "MON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27493/thumb/logo-moneta.png?1696526534"
  },
  {
    "chainId": 1,
    "address": "0x6d48206b97b164555c8fc7a40d59a7230e055166",
    "name": "Sendpicks",
    "symbol": "SEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32824/thumb/Sendpicks.png?1699582373"
  },
  {
    "chainId": 1,
    "address": "0x23878914efe38d27c4d67ab83ed1b93a74d4086a",
    "name": "Aave v3 USDT",
    "symbol": "AUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32884/thumb/USDT.PNG?1699768611"
  },
  {
    "chainId": 1,
    "address": "0xc97d6c52f3add91fa1c5287a453d7444aecbca83",
    "name": "Degen Zoo",
    "symbol": "DZOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29224/thumb/FAGWW5fe_400x400.jpeg?1696528182"
  },
  {
    "chainId": 1,
    "address": "0xcc9ee9483f662091a1de4795249e24ac0ac2630f",
    "name": "Aave v3 rETH",
    "symbol": "ARETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32885/thumb/reth.png?1699768947"
  },
  {
    "chainId": 1,
    "address": "0xf9ca9523e5b5a42c3018c62b084db8543478c400",
    "name": "Data Lake",
    "symbol": "LAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29279/thumb/VWlCfLlX_400x400.jpeg?1696528232"
  },
  {
    "chainId": 1,
    "address": "0x4f08705fb8f33affc231ed66e626b40e84a71870",
    "name": "Flute",
    "symbol": "FLUT",
    "decimals": 11,
    "logoURI": "https://assets.coingecko.com/coins/images/29148/thumb/TheFlute200x200.png?1696528108"
  },
  {
    "chainId": 1,
    "address": "0x018008bfb33d285247a21d44e50697654f754e63",
    "name": "Aave v3 DAI",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32886/thumb/dai.png?1699769446"
  },
  {
    "chainId": 1,
    "address": "0xc1ecfaf43c53bec9b9143ab274f35603fd10b886",
    "name": "StarShip ERC20",
    "symbol": "SSHIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31686/thumb/photo_2023-09-11_12-27-23_%281%29.jpg?1696530505"
  },
  {
    "chainId": 1,
    "address": "0x86eab36585eddb7a949a0b4771ba733d942a8aa7",
    "name": "Reddit",
    "symbol": "REDDIT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30791/thumb/IMG_0365.jpeg?1696529658"
  },
  {
    "chainId": 1,
    "address": "0xa700b4eb416be35b2911fd5dee80678ff64ff6c9",
    "name": "Aave v3 AAVE",
    "symbol": "AAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32887/thumb/aave.png?1699773604"
  },
  {
    "chainId": 1,
    "address": "0xf66cd2f8755a21d3c8683a10269f795c0532dd58",
    "name": "coreDAO",
    "symbol": "COREDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23745/thumb/coredao.png?1696522947"
  },
  {
    "chainId": 1,
    "address": "0x5e8c8a7243651db1384c0ddfdbe39761e8e7e51a",
    "name": "Aave v3 LINK",
    "symbol": "ALINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32888/thumb/link.png?1699773900"
  },
  {
    "chainId": 1,
    "address": "0xc2456d2118299a2efdfe6522ff79aa48fc5d2b00",
    "name": "MaruTaro",
    "symbol": "MARU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29304/thumb/marutaro_logo.jpg?1696528256"
  },
  {
    "chainId": 1,
    "address": "0xf5d126077096e5b01bc30ffa5d9324d7202d7cb3",
    "name": "CHEW",
    "symbol": "CHEW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29377/thumb/IMG_4701.PNG?1696528324"
  },
  {
    "chainId": 1,
    "address": "0x8a458a9dc9048e005d22849f470891b840296619",
    "name": "Aave v3 MKR",
    "symbol": "AMKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32889/thumb/mkr.png?1699774767"
  },
  {
    "chainId": 1,
    "address": "0x4c612e3b15b96ff9a6faed838f8d07d479a8dd4c",
    "name": "Aave v3 sDAI",
    "symbol": "ASDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32891/thumb/sdai.png?1699785595"
  },
  {
    "chainId": 1,
    "address": "0x5888641e3e6cbea6d84ba81edb217bd691d3be38",
    "name": "Bobo",
    "symbol": "BOBO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29875/thumb/BOBO_200.png?1696528799"
  },
  {
    "chainId": 1,
    "address": "0x3819f64f282bf135d62168c1e513280daf905e06",
    "name": "Hedron",
    "symbol": "HDRN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24208/thumb/hdrn.png?1696523395"
  },
  {
    "chainId": 1,
    "address": "0x53340a1ef3a0ddeba1d94bbd1e2ff55936f0ea60",
    "name": "Bark",
    "symbol": "BARK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29341/thumb/BarkLogo200x200.png?1696528290"
  },
  {
    "chainId": 1,
    "address": "0x295b42684f90c77da7ea46336001010f2791ec8c",
    "name": "Xi",
    "symbol": "XI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18640/thumb/v5NMxPo.png?1696518111"
  },
  {
    "chainId": 1,
    "address": "0xf6d2224916ddfbbab6e6bd0d1b7034f4ae0cab18",
    "name": "Aave v3 UNI",
    "symbol": "AUNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32893/thumb/UNI.png?1699789595"
  },
  {
    "chainId": 1,
    "address": "0xc4ee0aa2d993ca7c9263ecfa26c6f7e13009d2b6",
    "name": "Hoichi",
    "symbol": "HOICHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27227/thumb/3dBs4VvW_400x400.jpeg?1696526278"
  },
  {
    "chainId": 1,
    "address": "0x562e12e1e792643d168c1fa01c1b7198a0f83c9f",
    "name": "BookieBot",
    "symbol": "BB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31546/thumb/photo_2023-08-23_17-48-35_200x200.jpg?1696530359"
  },
  {
    "chainId": 1,
    "address": "0x9a44fd41566876a39655f74971a3a6ea0a17a454",
    "name": "Aave v3 LDO",
    "symbol": "ALDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32895/thumb/LDO.png?1699790151"
  },
  {
    "chainId": 1,
    "address": "0xacf8d5e515ed005655dfefa09c22673a37a7cdee",
    "name": "FunFi",
    "symbol": "FNF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25872/thumb/FNF-200x200.png?1696524955"
  },
  {
    "chainId": 1,
    "address": "0xff770e4c68e35db85c6e0e89a43750ec02bdb2ac",
    "name": "Ordinal BTC",
    "symbol": "OBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29244/thumb/Ordinal_BTC.png?1696528200"
  },
  {
    "chainId": 1,
    "address": "0x393e888b031e000a258a3f646e3d8043c57a1f05",
    "name": "Stakify Finance",
    "symbol": "SIFY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32549/thumb/200x200_-_Copy.png?1698479643"
  },
  {
    "chainId": 1,
    "address": "0x9b9647431632af44be02ddd22477ed94d14aacaa",
    "name": "KOK",
    "symbol": "KOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10502/thumb/logo_navy.png?1696510490"
  },
  {
    "chainId": 1,
    "address": "0x9ce07410673206c693bcec9b07710767637a564c",
    "name": "Monkeys Token",
    "symbol": "MONKEYS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29782/thumb/200x200_Monkeys_Logo.png?1696528712"
  },
  {
    "chainId": 1,
    "address": "0xa53f9dd70055113e13176e291eddb773f85b2650",
    "name": "ZEUS AI",
    "symbol": "ZEUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32896/thumb/cg_size.jpg?1699790661"
  },
  {
    "chainId": 1,
    "address": "0x9040e237c3bf18347bb00957dc22167d0f2b999d",
    "name": "Standard Protocol",
    "symbol": "STND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15100/thumb/STND_logo_black_bg.png?1696514758"
  },
  {
    "chainId": 1,
    "address": "0x09637e374ec5bab8c9e9654a97c9593e3038c5b3",
    "name": "zkProof",
    "symbol": "ZKP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29592/thumb/200.png?1696528531"
  },
  {
    "chainId": 1,
    "address": "0x51a59a02ba906194285e81eb1f98ffa28e7cf4c9",
    "name": "Pepe King Prawn",
    "symbol": "PEPE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31942/thumb/Pepe_King_Prawn.png?1696530749"
  },
  {
    "chainId": 1,
    "address": "0xeb575c45004bd7b61c6a8d3446a62a05a6ce18d8",
    "name": "Ethlas",
    "symbol": "ELS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30331/thumb/ELS_Logo_200x200.png?1696529232"
  },
  {
    "chainId": 1,
    "address": "0xe0151763455a8a021e64880c238ba1cff3787ff0",
    "name": "Aped",
    "symbol": "APED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29906/thumb/Aped.png?1696528835"
  },
  {
    "chainId": 1,
    "address": "0xacdf0dba4b9839b96221a8487e9ca660a48212be",
    "name": "High Yield USD",
    "symbol": "HYUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30899/thumb/hyusdlogo.png?1700809180"
  },
  {
    "chainId": 1,
    "address": "0x76fca1adb104770b38581b64d55e67fa5a0f3966",
    "name": "ZkTsunami",
    "symbol": "ZKT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29253/thumb/Sa72qYMC_400x400.jpg?1696528207"
  },
  {
    "chainId": 1,
    "address": "0x2a7e415c169ce3a580c6f374dc26f6aaad1eccfe",
    "name": "Hachi",
    "symbol": "HACHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29262/thumb/HACHI_main_logo__.png?1696528216"
  },
  {
    "chainId": 1,
    "address": "0x3b0b09f5b14f6d50e6672ae158f9d71893feca18",
    "name": "hiBAKC",
    "symbol": "HIBAKC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30585/thumb/MicrosoftTeams-image_%2812%29.png?1696529448"
  },
  {
    "chainId": 1,
    "address": "0x508e00d5cef397b02d260d035e5ee80775e4c821",
    "name": "Bitcoin Cats",
    "symbol": "1CAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34059/thumb/779e5404024bcd3cec3db54370ea66c.jpg?1703752231"
  },
  {
    "chainId": 1,
    "address": "0x6f3cbe18e9381dec6026e6cb8166c13944fcfee1",
    "name": "FELIX",
    "symbol": "FELIX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32897/thumb/200photo_2023-11-11_02-25-13.jpg?1699790963"
  },
  {
    "chainId": 1,
    "address": "0x5ba19d656b65f1684cfea4af428c23b9f3628f97",
    "name": "AAG",
    "symbol": "AAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21382/thumb/aag-token-icon.png?1696520747"
  },
  {
    "chainId": 1,
    "address": "0xc3681a720605bd6f8fe9a2fabff6a7cdecdc605d",
    "name": "Nihao",
    "symbol": "NIHAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30619/thumb/nihaowhitecat.png?1696529493"
  },
  {
    "chainId": 1,
    "address": "0xdefa4e8a7bcba345f687a2f1456f5edd9ce97202",
    "name": "Kyber Network Crystal",
    "symbol": "KNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14899/thumb/RwdVsGcw_400x400.jpg?1696514562"
  },
  {
    "chainId": 1,
    "address": "0x861950d091704c80b2a0f69d4b3431f0aed7dc7a",
    "name": "Magic BOT",
    "symbol": "MAGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32898/thumb/magic_bot_logo.jpg?1699792367"
  },
  {
    "chainId": 1,
    "address": "0x7c95e7ad2b349dc2f82d0f1117a44b561fa2699a",
    "name": "Gracy",
    "symbol": "GRACY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31943/thumb/gracy_200x200.png?1696530750"
  },
  {
    "chainId": 1,
    "address": "0x6bf765c43030387a983f429c1438e9d2025b7e12",
    "name": "McPepe s",
    "symbol": "PEPES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29265/thumb/pepes-logo_%282%29.png?1696528218"
  },
  {
    "chainId": 1,
    "address": "0x238a1fcc2f65ed50c1e1797cf5857cba3e82a55b",
    "name": "Moeta",
    "symbol": "MOETA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29245/thumb/photo_2023-05-30_21-31-25.jpg?1696528201"
  },
  {
    "chainId": 1,
    "address": "0x09a3ecafa817268f77be1283176b946c4ff2e608",
    "name": "Mirror Protocol",
    "symbol": "MIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13295/thumb/mirror_logo_transparent.png?1696513067"
  },
  {
    "chainId": 1,
    "address": "0x544d230f0362f3843fda5caa99b94cb2b336e384",
    "name": "Magnesium",
    "symbol": "MAG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32056/thumb/WhatsApp_Image_2023-09-29_at_15.29.08.jpeg?1696530853"
  },
  {
    "chainId": 1,
    "address": "0x54012cdf4119de84218f7eb90eeb87e25ae6ebd7",
    "name": "Luffy",
    "symbol": "LUFFY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17736/thumb/LUFFY_LOGO.png?1696517262"
  },
  {
    "chainId": 1,
    "address": "0xbfe178cf147fda6490f69fe7e6ae038ff09d2980",
    "name": "PSI Gate",
    "symbol": "PSI/ACC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35338/thumb/psi-acc_logo.jpg?1708315646"
  },
  {
    "chainId": 1,
    "address": "0xcae7f3aa6d456463bef0e5b508542b69d96f2904",
    "name": "InvectAI",
    "symbol": "INVECTAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29333/thumb/Invect.jpeg?1696528283"
  },
  {
    "chainId": 1,
    "address": "0x80008bcd713c38af90a9930288d446bc3bd2e684",
    "name": "Karate Combat",
    "symbol": "KARATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30384/thumb/karate.jpg?1696529274"
  },
  {
    "chainId": 1,
    "address": "0xd54619e0b9899d74cc9b981354eb6b59732c43b1",
    "name": "GalleryCoin",
    "symbol": "GLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29372/thumb/%EA%B0%A4%EB%9F%AC%EB%A6%AC%EC%BD%94%EC%9D%B8_%EB%A1%9C%EA%B3%A0%28200_200%29.png?1696528319"
  },
  {
    "chainId": 1,
    "address": "0x6ba75d640bebfe5da1197bb5a2aff3327789b5d3",
    "name": "VNX EURO",
    "symbol": "VEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29351/thumb/VNXEUR_%281%29.png?1696528300"
  },
  {
    "chainId": 1,
    "address": "0x6ef460eb3563cfcc73f8147b0a77daffee71f867",
    "name": "Zeus AI",
    "symbol": "ZEUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29593/thumb/ZEUS.png?1696528532"
  },
  {
    "chainId": 1,
    "address": "0xbfc66d8cce39e668fd5d3c10fd1b1eabb82c27b7",
    "name": "OVO",
    "symbol": "OVO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29514/thumb/ovotoken200.png?1696528458"
  },
  {
    "chainId": 1,
    "address": "0x8d96b4ab6c741a4c8679ae323a100d74f085ba8f",
    "name": "Bazaars",
    "symbol": "BZR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26392/thumb/bazaars_logo-color-black-icon.png?1696525469"
  },
  {
    "chainId": 1,
    "address": "0xfac0403a24229d7e2edd994d50f5940624cbeac2",
    "name": "Theopetra",
    "symbol": "THEO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29431/thumb/MicrosoftTeams-image_%2884%29.png?1703417298"
  },
  {
    "chainId": 1,
    "address": "0xefd113cea2f656fe7899a25a06243a6e40406e08",
    "name": "THE9",
    "symbol": "THE9",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29357/thumb/THE9_logo.png?1696528305"
  },
  {
    "chainId": 1,
    "address": "0x530824da86689c9c17cdc2871ff29b058345b44a",
    "name": "Short term T Bill Token",
    "symbol": "STBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29239/thumb/STBT_icon.png?1696528196"
  },
  {
    "chainId": 1,
    "address": "0xa1f410f13b6007fca76833ee7eb58478d47bc5ef",
    "name": "Rejuve AI",
    "symbol": "RJV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29366/thumb/2023_Rejuve_Logo_-_Square_-_Teal.jpg?1696528314"
  },
  {
    "chainId": 1,
    "address": "0x21b8bfbbefc9e2b9a994871ecd742a5132b98aed",
    "name": "Crypto Real Estate",
    "symbol": "CRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29387/thumb/cre-logo-200.png?1696528333"
  },
  {
    "chainId": 1,
    "address": "0x4c0bc0b3c734a573d28c627a96b95a1f1a7596fb",
    "name": "Husbant",
    "symbol": "HUSBANT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32863/thumb/husbant.png?1699670240"
  },
  {
    "chainId": 1,
    "address": "0xcb129aa11ceaa00da1a92d12e26bb776ef3c3628",
    "name": "PowBlocks",
    "symbol": "XPB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32520/thumb/logo.png?1698406955"
  },
  {
    "chainId": 1,
    "address": "0xdcee70654261af21c44c093c300ed3bb97b78192",
    "name": "Wrapped OETH",
    "symbol": "WOETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29734/thumb/wOETH.png?1696528664"
  },
  {
    "chainId": 1,
    "address": "0x03f813f7d066c55e3512985186df3b4e6a08e0d6",
    "name": "Shepherd Inu",
    "symbol": "SINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30366/thumb/SINU.jpg?1696529263"
  },
  {
    "chainId": 1,
    "address": "0x0799de3f4e111b70e845edb7a8646d934a51e210",
    "name": "50cal",
    "symbol": "50CAL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33047/thumb/photo_2023-11-15_01-09-57_%281%29.jpg?1700472328"
  },
  {
    "chainId": 1,
    "address": "0x419d0d8bdd9af5e606ae2232ed285aff190e711b",
    "name": "FUNToken",
    "symbol": "FUN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/761/thumb/FUN.png?1696501914"
  },
  {
    "chainId": 1,
    "address": "0xa4cb3ef5f41a4d89d6fced22ea8a1c57957629aa",
    "name": "MetaRuffy  MR ",
    "symbol": "MR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27831/thumb/mr_logo_2023_200_200.png?1696526850"
  },
  {
    "chainId": 1,
    "address": "0xb69753c06bb5c366be51e73bfc0cc2e3dc07e371",
    "name": "POOH",
    "symbol": "POOH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30091/thumb/Pooh-icon.png?1696529015"
  },
  {
    "chainId": 1,
    "address": "0xbcbf09b4b83eb3893d58f0665d21460c04e39d8e",
    "name": "Sherlock Defi",
    "symbol": "SLOCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30020/thumb/20230425_212153_copy_200x200.jpg?1696528944"
  },
  {
    "chainId": 1,
    "address": "0xfa05a73ffe78ef8f1a739473e462c54bae6567d9",
    "name": "Lunyr",
    "symbol": "LUN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/735/thumb/lunyr.png?1696501891"
  },
  {
    "chainId": 1,
    "address": "0x4df812f6064def1e5e029f1ca858777cc98d2d81",
    "name": "Xaurum",
    "symbol": "XAUR",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/461/thumb/xaurum.png?1696501719"
  },
  {
    "chainId": 1,
    "address": "0x4dd942baa75810a3c1e876e79d5cd35e09c97a76",
    "name": "Dash 2 Trade",
    "symbol": "D2T",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28707/thumb/logo.png?1696527689"
  },
  {
    "chainId": 1,
    "address": "0x5c8190b76e90b4dd0702740cf6eb0f7ee01ab5e9",
    "name": "Archive AI",
    "symbol": "ARCAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28939/thumb/pp.JPG?1696527913"
  },
  {
    "chainId": 1,
    "address": "0x0a58153a0cd1cfaea94ce1f7fdc5d7e679eca936",
    "name": "Internet Money  ETH ",
    "symbol": "IM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28930/thumb/IM_Circle_200.png?1696527905"
  },
  {
    "chainId": 1,
    "address": "0x719e7f0dadfdea25b78595da944f44d15d7e6795",
    "name": "MUSK DAO",
    "symbol": "MUSK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29464/thumb/musk.png?1696528410"
  },
  {
    "chainId": 1,
    "address": "0x5ac83bfbfcebb3397a40fd259dbe7a4be04237d3",
    "name": "Lyfebloc",
    "symbol": "LBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30021/thumb/LBT_Token.png?1696528945"
  },
  {
    "chainId": 1,
    "address": "0x2654e753424a9f02df31cfbc6c5af14a87b6cab5",
    "name": "Kendoll Janner",
    "symbol": "KEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31260/thumb/android-chrome-512x512.png?1696530084"
  },
  {
    "chainId": 1,
    "address": "0x5972a9689ee53481ad6301d60e1e15092754ffa3",
    "name": "Nodebet",
    "symbol": "NBET",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35500/thumb/photo_2024-02-21_20-48-04.jpg?1708926297"
  },
  {
    "chainId": 1,
    "address": "0x8b39b70e39aa811b69365398e0aace9bee238aeb",
    "name": "Red Kite",
    "symbol": "PKF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14422/thumb/red_kite.png?1696514112"
  },
  {
    "chainId": 1,
    "address": "0x431ad2ff6a9c365805ebad47ee021148d6f7dbe0",
    "name": "dForce",
    "symbol": "DF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9709/thumb/xlGxxIjI_400x400.jpg?1696509776"
  },
  {
    "chainId": 1,
    "address": "0x60a26c05c5372dcded66940d2b56076bce925152",
    "name": "SILVER",
    "symbol": "SILVER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32273/thumb/LOGO1.png?1697179980"
  },
  {
    "chainId": 1,
    "address": "0xb2fd1e0478dbf61772996bcce8a2f1151eeeda37",
    "name": "Wrapped Pepe",
    "symbol": "WPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30961/thumb/200.png?1706775966"
  },
  {
    "chainId": 1,
    "address": "0x13dbd5394c2c7e4bdb85b1838286faa66532a262",
    "name": "Sun Tzu",
    "symbol": "TZU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30689/thumb/TZU_CG.png?1696529558"
  },
  {
    "chainId": 1,
    "address": "0xcfc5bd99915aaa815401c5a41a927ab7a38d29cf",
    "name": "Threshold USD",
    "symbol": "THUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32274/thumb/thusd.png?1697180081"
  },
  {
    "chainId": 1,
    "address": "0xb186035490c8602ead853ec98be05e3461521ab2",
    "name": "Pack",
    "symbol": "PACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29556/thumb/Pack.png?1696528496"
  },
  {
    "chainId": 1,
    "address": "0x79d4f0232a66c4c91b89c76362016a1707cfbf4f",
    "name": "VNX Swiss Franc",
    "symbol": "VCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29547/thumb/VNXCHF_%282%29.png?1696528488"
  },
  {
    "chainId": 1,
    "address": "0x4bc3263eb5bb2ef7ad9ab6fb68be80e43b43801f",
    "name": "Voucher Ethereum 2 0",
    "symbol": "VETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29683/thumb/vETH_200.png?1696528617"
  },
  {
    "chainId": 1,
    "address": "0xf36c5f04127f7470834ed6f98bddc1be62aba48d",
    "name": "CryptoAI",
    "symbol": "CAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28899/thumb/200blue.png?1696527875"
  },
  {
    "chainId": 1,
    "address": "0xa7fbd9254f10f8e20a31a593c9e8bc0d041e15f6",
    "name": "Orbeon Protocol",
    "symbol": "ORBN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29566/thumb/logo_%281%29.png?1696528506"
  },
  {
    "chainId": 1,
    "address": "0x9625ce7753ace1fa1865a47aae2c5c2ce4418569",
    "name": "KAP Games",
    "symbol": "KAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27682/thumb/KAP_Logo.png?1696526710"
  },
  {
    "chainId": 1,
    "address": "0x045da4bfe02b320f4403674b3b7d121737727a36",
    "name": "DeFi Franc",
    "symbol": "DCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28108/thumb/22249.png?1696527116"
  },
  {
    "chainId": 1,
    "address": "0xe87e15b9c7d989474cb6d8c56b3db4efad5b21e8",
    "name": "HOKK Finance",
    "symbol": "HOKK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14985/thumb/hokk.png?1696514647"
  },
  {
    "chainId": 1,
    "address": "0x91bdf1eb4215bc1c6d9a6bd83ecfceeb17cd4343",
    "name": "Richard",
    "symbol": "RICHARD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30295/thumb/Richard.png?1696529200"
  },
  {
    "chainId": 1,
    "address": "0x8e81d527f8fa05d82c514401c8144275174557cd",
    "name": " CCB    ",
    "symbol": "CCB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34797/thumb/CCB_200x200px.jpg?1711028027"
  },
  {
    "chainId": 1,
    "address": "0x6aa40d02115090d40dc33c7c5f3cf05112fa4f83",
    "name": "PWRCASH",
    "symbol": "PWRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29571/thumb/our_logo.png?1696528511"
  },
  {
    "chainId": 1,
    "address": "0x059956483753947536204e89bfad909e1a434cc6",
    "name": "Mintlayer",
    "symbol": "ML",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29454/thumb/mintlayer-icon-coingecko200x200.png?1696528401"
  },
  {
    "chainId": 1,
    "address": "0xb5d730d442e1d5b119fb4e5c843c48a64202ef92",
    "name": "Sabai Ecoverse",
    "symbol": "SABAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29573/thumb/Logo_CMC.png?1702661154"
  },
  {
    "chainId": 1,
    "address": "0xfac77a24e52b463ba9857d6b758ba41ae20e31ff",
    "name": "LSDx Finance",
    "symbol": "LSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29519/thumb/logo.png?1696528462"
  },
  {
    "chainId": 1,
    "address": "0xc477d038d5420c6a9e0b031712f61c5120090de9",
    "name": "Boson Protocol",
    "symbol": "BOSON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14710/thumb/boson_logo.png?1696514381"
  },
  {
    "chainId": 1,
    "address": "0xd5df655087d99b7b720a5bc8711f296180a4f44b",
    "name": "Catgirl Optimus",
    "symbol": "OPTIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29669/thumb/Optigpng.png?1696528604"
  },
  {
    "chainId": 1,
    "address": "0x857de36f92330e1b9a21e8745c692f2ce13866cb",
    "name": "Meta Masters Guild Games",
    "symbol": "MEMAGX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36899/thumb/1000039355.jpg?1712677645"
  },
  {
    "chainId": 1,
    "address": "0x89b69f2d1adffa9a253d40840b6baa7fc903d697",
    "name": "Dione",
    "symbol": "DIONE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26931/thumb/dione_200x200.png?1696525987"
  },
  {
    "chainId": 1,
    "address": "0xf915110898d9a455ad2da51bf49520b41655ccea",
    "name": "TAI",
    "symbol": "TAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30574/thumb/tai.png?1696529439"
  },
  {
    "chainId": 1,
    "address": "0xaf8942831f3a096f708b8b31f191b8958cf176c5",
    "name": "Neural Radiance Field",
    "symbol": "NERF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29521/thumb/Neural_Radiance_Field.png?1696528464"
  },
  {
    "chainId": 1,
    "address": "0x1aa51bc7eb181ce48ce626bf62f8956fa9555136",
    "name": "PAWZONE",
    "symbol": "PAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29598/thumb/PAW_Token_Logo.png?1696528536"
  },
  {
    "chainId": 1,
    "address": "0x516e2758b044433371076a48127b8cfa7b0bdb43",
    "name": "Smudge Lord",
    "symbol": "SMUDGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30268/thumb/logo-A.png?1696529175"
  },
  {
    "chainId": 1,
    "address": "0xc89d9aa9d9e54bb196319c6195aea1038d2bc936",
    "name": "Trend X",
    "symbol": "TRENDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29522/thumb/200.png?1696528465"
  },
  {
    "chainId": 1,
    "address": "0xf3bb9f16677f2b86efd1dfca1c141a99783fde58",
    "name": "Crown Token",
    "symbol": "CROWN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30161/thumb/Crown_Token_200_200.png?1696529081"
  },
  {
    "chainId": 1,
    "address": "0x8cb73eb53fa81f808f704bea15a677b6464a1f90",
    "name": "Gauro",
    "symbol": "GAURO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31783/thumb/200_png.png?1696530600"
  },
  {
    "chainId": 1,
    "address": "0x6d3d490964205c8bc8ded39e48e88e8fde45b41f",
    "name": "Gooch",
    "symbol": "GOOCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30290/thumb/gooch.jpg?1696529195"
  },
  {
    "chainId": 1,
    "address": "0x823e1b82ce1dc147bbdb25a203f046afab1ce918",
    "name": "SpiralDAO Coil",
    "symbol": "COIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29773/thumb/200x200.png?1696528704"
  },
  {
    "chainId": 1,
    "address": "0x9559aaa82d9649c7a7b220e7c461d2e74c9a3593",
    "name": "StaFi Staked ETH",
    "symbol": "RETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14452/thumb/rETH.png?1696514140"
  },
  {
    "chainId": 1,
    "address": "0x4e4bffaa8df6f0dc3e5600bbacf7da55f37134fc",
    "name": "BlockStar",
    "symbol": "BST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29524/thumb/BlockStar_Transparent_200_by_200.png?1696528467"
  },
  {
    "chainId": 1,
    "address": "0xa66daa57432024023db65477ba87d4e7f5f95213",
    "name": "Huobi Pool",
    "symbol": "HPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6628/thumb/Huobi_pool_token.png?1696506977"
  },
  {
    "chainId": 1,
    "address": "0xbb19da2482308ec02a242aced4fe0f09d06b12a7",
    "name": "Flash 3 0",
    "symbol": "FLASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31151/thumb/logo_200x200.png?1696529979"
  },
  {
    "chainId": 1,
    "address": "0x1c9a2d6b33b4826757273d47ebee0e2dddcd978b",
    "name": "Flux FRAX",
    "symbol": "FFRAX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29580/thumb/fFRAX-320.png?1696528519"
  },
  {
    "chainId": 1,
    "address": "0x81994b9607e06ab3d5cf3afff9a67374f05f27d7",
    "name": "Flux USDT",
    "symbol": "FUSDT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29581/thumb/fUSDT-320.png?1696528520"
  },
  {
    "chainId": 1,
    "address": "0x0316eb71485b0ab14103307bf65a021042c6d380",
    "name": "Huobi BTC",
    "symbol": "HBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12407/thumb/hbtc.png?1696512229"
  },
  {
    "chainId": 1,
    "address": "0x9db0fb0aebe6a925b7838d16e3993a3976a64aab",
    "name": "Bambi",
    "symbol": "BAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30597/thumb/Frame_24.png?1696529467"
  },
  {
    "chainId": 1,
    "address": "0x9ffc3bcde7b68c46a6dc34f0718009925c1867cb",
    "name": "Huobi Polkadot",
    "symbol": "HDOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14107/thumb/hdot.png?1696513828"
  },
  {
    "chainId": 1,
    "address": "0xbcd4d5ac29e06e4973a1ddcd782cd035d04bc0b7",
    "name": "Quick Intel",
    "symbol": "QKNTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29605/thumb/IMG_6589D0616DF1-1.jpeg?1696528542"
  },
  {
    "chainId": 1,
    "address": "0x9afb950948c2370975fb91a441f36fdc02737cd4",
    "name": "Huobi FIL",
    "symbol": "HFIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14109/thumb/HFIL.png?1696513830"
  },
  {
    "chainId": 1,
    "address": "0x2c000c0093de75a8fa2fccd3d97b314e20b431c3",
    "name": "Huobi Litecoin",
    "symbol": "HLTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14110/thumb/HLTC.png?1696513831"
  },
  {
    "chainId": 1,
    "address": "0x4216663ddc7bd10eaf44609df4dd0f91cd2be7f2",
    "name": "MicroPepe",
    "symbol": "MPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30153/thumb/logo.jpeg?1696529073"
  },
  {
    "chainId": 1,
    "address": "0x7b37a55ffb30c11d95f943672ae98f28cfb7b087",
    "name": "Fufu Token",
    "symbol": "FUFU",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/32225/thumb/6522723afff7e2e198ce38c6_FufuTokenIcon_250x250.png?1696931163"
  },
  {
    "chainId": 1,
    "address": "0x5c0217e4e126d501896594bec409898a9afc5970",
    "name": "Frens Coin",
    "symbol": "FRENS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30470/thumb/Frens_200x200.png?1696529356"
  },
  {
    "chainId": 1,
    "address": "0x9471d30d78a3c9f076ce206d14867a8d8be1efde",
    "name": "ZENEX",
    "symbol": "ZNX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29638/thumb/znx-logo.png?1696528575"
  },
  {
    "chainId": 1,
    "address": "0xf024e6387392a3c3a03ebd0b9d6b747ff5359417",
    "name": "Ordi Launch",
    "symbol": "ORLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34889/thumb/Ordinal_Launch.png?1706540288"
  },
  {
    "chainId": 1,
    "address": "0x00e679ba63b509182c349f5614f0a07cdd0ce0c5",
    "name": "Damex Token",
    "symbol": "DAMEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30053/thumb/damex.jpg?1696528975"
  },
  {
    "chainId": 1,
    "address": "0x198d7387fa97a73f05b8578cdeff8f2a1f34cd1f",
    "name": "Wrapped Jones AURA",
    "symbol": "WJAURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30103/thumb/geckoAura.png?1696529025"
  },
  {
    "chainId": 1,
    "address": "0xd2b274cfbf9534f56b59ad0fb7e645e0354f4941",
    "name": "XDOGE",
    "symbol": "XDOGE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31107/thumb/photo_2023-07-25_13.59.23.jpeg?1696529938"
  },
  {
    "chainId": 1,
    "address": "0x1234567461d3f8db7496581774bd869c83d51c93",
    "name": "BitClave",
    "symbol": "CAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1585/thumb/bitclave.png?1696502618"
  },
  {
    "chainId": 1,
    "address": "0xad6eefb4f4a6ce3e2cd2049c3104f5ce5ed082f0",
    "name": "Simpson6900",
    "symbol": "SIMPSON690",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32175/thumb/1111.png?1696742771"
  },
  {
    "chainId": 1,
    "address": "0xcaeda9650ccd356af7776057a105f9e6ffe68213",
    "name": "Loong",
    "symbol": "LOONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32176/thumb/200x200.png?1696742804"
  },
  {
    "chainId": 1,
    "address": "0xb63b606ac810a52cca15e44bb630fd42d8d1d83d",
    "name": "MCO",
    "symbol": "MCO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/739/thumb/1776.png?1696501894"
  },
  {
    "chainId": 1,
    "address": "0x41b25ff6431074959532db7435dadaca65a21d1c",
    "name": "Claw",
    "symbol": "CLAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29677/thumb/PBixw6K.png?1696528611"
  },
  {
    "chainId": 1,
    "address": "0x69d47846bb7b884062fabc08a4a7deadcd98c45b",
    "name": "AlphaRushAI",
    "symbol": "RUSHAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29536/thumb/logo200x200png.png?1696528478"
  },
  {
    "chainId": 1,
    "address": "0x5de8ab7e27f6e7a1fff3e5b337584aa43961beef",
    "name": "SmarDex",
    "symbol": "SDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29470/thumb/SDEX_logo_transparent_outside_240x240.png?1696930070"
  },
  {
    "chainId": 1,
    "address": "0xffe203b59393593965842439ce1e7d7c78109b46",
    "name": "Satellite Doge 1 Mission",
    "symbol": "DOGE-1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33316/thumb/logo_200-200.png?1701413684"
  },
  {
    "chainId": 1,
    "address": "0x69a95185ee2a045cdc4bcd1b1df10710395e4e23",
    "name": "Poolz Finance  OLD ",
    "symbol": "POOLZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13679/thumb/poolz_logo.png?1696513428"
  },
  {
    "chainId": 1,
    "address": "0xb939da54f9748440a1b279d42be1296942732288",
    "name": "Fonzy",
    "symbol": "FONZY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30245/thumb/FONZY_CROWN_2_HANDS_%282%29.png?1696529154"
  },
  {
    "chainId": 1,
    "address": "0x6732efaf6f39926346bef8b821a04b6361c4f3e5",
    "name": "Simple Asymmetry ETH",
    "symbol": "SAFETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30426/thumb/safETH_Logo_200_200.png?1699479265"
  },
  {
    "chainId": 1,
    "address": "0x4c4a50a61bed3b9024d8ffc1f1d168dc8cb1c689",
    "name": "The Blu Arctic Water Comp",
    "symbol": "BARC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34467/thumb/BARC_200x200.png?1704967819"
  },
  {
    "chainId": 1,
    "address": "0x50eec6d765792dcfb0913c8403ef2a12e1b861a6",
    "name": "Z Cubed",
    "symbol": "Z3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29478/thumb/Z3-BLACK-200x200.png?1696528422"
  },
  {
    "chainId": 1,
    "address": "0xda0c94c73d127ee191955fb46bacd7ff999b2bcd",
    "name": "Construct",
    "symbol": "STANDARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20119/thumb/construct200px.png?1696519536"
  },
  {
    "chainId": 1,
    "address": "0x6fc16de36e040e0c7d1a70d039ef701516516b32",
    "name": "ScopexAI",
    "symbol": "SCOPEX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35151/thumb/scopex_200x200.jpg?1707559235"
  },
  {
    "chainId": 1,
    "address": "0xbb3a8fd6ec4bf0fdc6cd2739b1e41192d12b1873",
    "name": "Orbofi AI",
    "symbol": "OBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30216/thumb/new_V_test2.png?1696529127"
  },
  {
    "chainId": 1,
    "address": "0x15f74458ae0bfdaa1a96ca1aa779d715cc1eefe4",
    "name": "Grai",
    "symbol": "GRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30427/thumb/GRAI_Token.png?1696529315"
  },
  {
    "chainId": 1,
    "address": "0x08f5a9235b08173b7569f83645d2c7fb55e8ccd8",
    "name": "Tierion",
    "symbol": "TNT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/923/thumb/tierion.png?1696502044"
  },
  {
    "chainId": 1,
    "address": "0x5919dea604631016c15c805e3d948a0384879892",
    "name": "The People s Coin",
    "symbol": "PEEP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16777/thumb/peeps.png?1696516349"
  },
  {
    "chainId": 1,
    "address": "0x7cb683151a83c2b10a30cbb003cda9996228a2ba",
    "name": "IYKYK",
    "symbol": "IYKYK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30022/thumb/iykyk200.png?1696528946"
  },
  {
    "chainId": 1,
    "address": "0xec6d73557937974077911a0b6fdc436b0ff70296",
    "name": "Skrimples",
    "symbol": "SKRIMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30080/thumb/skrimples_logo200.png?1696529004"
  },
  {
    "chainId": 1,
    "address": "0xbd15ad921e1b480209af549874a2fcb80dc312bf",
    "name": "Harpoon",
    "symbol": "HRP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29889/thumb/Harpoon.jpeg?1696528813"
  },
  {
    "chainId": 1,
    "address": "0x5de597849cf72c72f073e9085bdd0dadd8e6c199",
    "name": "Finblox",
    "symbol": "FBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27087/thumb/fbx.png?1696526136"
  },
  {
    "chainId": 1,
    "address": "0x2c9715d6d95443c3bdf29e473ad168ff6fa6627d",
    "name": "Crab Rave Token",
    "symbol": "CRABS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30849/thumb/cute-little-crab-cartoon-posing-vector_%281%29.jpg?1696529700"
  },
  {
    "chainId": 1,
    "address": "0x9681ee0d91e737c3b60aceba7fbdae61b5462f42",
    "name": "Crypto Carbon Energy",
    "symbol": "CYCE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29824/thumb/CYCE.jpg?1696528752"
  },
  {
    "chainId": 1,
    "address": "0xf88baf18fab7e330fa0c4f83949e23f52fececce",
    "name": "Granary",
    "symbol": "GRAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29740/thumb/Grain.png?1696528670"
  },
  {
    "chainId": 1,
    "address": "0xd488321ef57ab16b4f334558b1fc4e0213c82db1",
    "name": "WCTrades",
    "symbol": "WCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36125/thumb/WCTrades.png?1710577456"
  },
  {
    "chainId": 1,
    "address": "0x21ead867c8c5181854f6f8ce71f75b173d2bc16a",
    "name": "LSDx Pool",
    "symbol": "ETHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29780/thumb/ethx.png?1696528710"
  },
  {
    "chainId": 1,
    "address": "0x87fd01183ba0235e1568995884a78f61081267ef",
    "name": "Open Mind Network",
    "symbol": "OPMND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35768/thumb/uprptsto_400x400.jpg?1709784632"
  },
  {
    "chainId": 1,
    "address": "0xf4cc8237fc85f54b64975142ec71a9554ecf4c61",
    "name": "Versus",
    "symbol": "VS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32230/thumb/avi.png?1696933323"
  },
  {
    "chainId": 1,
    "address": "0x68b429161ec09a6c1d65ba70727ab1faa5bc4026",
    "name": "Ordinal Doge",
    "symbol": "ODOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29323/thumb/o-DOGE-200x200.png?1696528274"
  },
  {
    "chainId": 1,
    "address": "0x082646b22a3960da69ef7a778c16dd6fb85dd999",
    "name": "Fuck Pepe",
    "symbol": "FKPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29894/thumb/Screenshot_7.png?1696528818"
  },
  {
    "chainId": 1,
    "address": "0x718dab2c7bbd2d9503dc1ce1dbc215abc41a3545",
    "name": "XAIGROK",
    "symbol": "XAIGROK",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/33584/thumb/20231207_155741.jpg?1702465158"
  },
  {
    "chainId": 1,
    "address": "0x0364e6c6f4f9ad90f11a39e681bd6c0156b3efdb",
    "name": "Bimbo The Dog",
    "symbol": "BIMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34551/thumb/photo_2024-01-12_16.35.16.png?1705368435"
  },
  {
    "chainId": 1,
    "address": "0x75430d0782a443bd4f1c92c69009599dea53a206",
    "name": "Pick Or Rick",
    "symbol": "RICK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30653/thumb/Untitled_Artwork_8_%283%29_%281%29.png?1696529523"
  },
  {
    "chainId": 1,
    "address": "0xbc396689893d065f41bc2c6ecbee5e0085233447",
    "name": "Perpetual Protocol",
    "symbol": "PERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12381/thumb/60d18e06844a844ad75901a9_mark_only_03.png?1696512205"
  },
  {
    "chainId": 1,
    "address": "0x049e9f5369358786a1ce6483d668d062cfe547ec",
    "name": "Checks Token",
    "symbol": "CHECKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30367/thumb/checkstoken-200x200.png?1696529264"
  },
  {
    "chainId": 1,
    "address": "0x5f18ea482ad5cc6bc65803817c99f477043dce85",
    "name": "Agility",
    "symbol": "AGI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29781/thumb/14csBMea_400x400.jpg?1696528711"
  },
  {
    "chainId": 1,
    "address": "0x0f5def84ced3e9e295dae28df96d0b846de92c1a",
    "name": "Crypto SDG",
    "symbol": "SDG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29816/thumb/SDG_token_Logo_200.png?1696528745"
  },
  {
    "chainId": 1,
    "address": "0x24bcec1afda63e622a97f17cff9a61ffcfd9b735",
    "name": "Crab Market",
    "symbol": "CRAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28356/thumb/200-icon.png?1696527360"
  },
  {
    "chainId": 1,
    "address": "0xb7bda6a89e724f63572ce68fddc1a6d1d5d24bcf",
    "name": "OGzClub",
    "symbol": "OGZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31121/thumb/25832.png?1696529951"
  },
  {
    "chainId": 1,
    "address": "0xd689933dafeec0041385d278adf5492a2b3b68fb",
    "name": "Decentralized Intelligence Agency",
    "symbol": "DIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32596/thumb/DIA.jpg?1698674064"
  },
  {
    "chainId": 1,
    "address": "0xd015422879a1308ba557510345e944b912b9ab73",
    "name": "FreeTrump",
    "symbol": "TRUMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34699/thumb/photo_2024-01-19_11-00-46.jpg?1705859328"
  },
  {
    "chainId": 1,
    "address": "0x3a4c122243b53135e7f6bf5b4658092c3a3109ab",
    "name": "HustleBot",
    "symbol": "HUSTLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32651/thumb/IMG_20231030_233538_062.jpg?1698895135"
  },
  {
    "chainId": 1,
    "address": "0x10010078a54396f62c96df8532dc2b4847d47ed3",
    "name": "Hundred Finance",
    "symbol": "HND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18445/thumb/hnd.PNG?1696517933"
  },
  {
    "chainId": 1,
    "address": "0x247dc9cbbaadabce6e30e2a84ec6c53a419913ad",
    "name": "Ethereum Message Service",
    "symbol": "EMS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29812/thumb/LogoEMS200x200.png?1696528741"
  },
  {
    "chainId": 1,
    "address": "0x7da2641000cbb407c329310c461b2cb9c70c3046",
    "name": "Delysium",
    "symbol": "AGI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29299/thumb/AGI_logo_200.png?1696528251"
  },
  {
    "chainId": 1,
    "address": "0x519c1001d550c0a1dae7d1fc220f7d14c2a521bb",
    "name": "Polkaswap",
    "symbol": "PSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15475/thumb/pswap-token-logomark-ticker-icon-200px-transparent-optimized.png?1696515120"
  },
  {
    "chainId": 1,
    "address": "0xd0e94bf2466979b81d738c34058d4a6559c4d8be",
    "name": "Aardvark  OLD ",
    "symbol": "ARDVRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31451/thumb/Logo_Trans_CG.png?1696530265"
  },
  {
    "chainId": 1,
    "address": "0x4872208c83acbfd7f6dea5aa6ce6d5d7aed2ac1c",
    "name": "AiMage Tools",
    "symbol": "AIMAGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33585/thumb/AiMAGE.jpg?1702465762"
  },
  {
    "chainId": 1,
    "address": "0x5efcea234f7547de4569aad1215fa5d2adaced38",
    "name": "Clown Pepe",
    "symbol": "HONK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29890/thumb/Honk.png?1696528814"
  },
  {
    "chainId": 1,
    "address": "0x00282fd551d03dc033256c4bf119532e8c735d8a",
    "name": "Biaocoin",
    "symbol": "BIAO",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/30602/thumb/biao_cmc.png?1696529472"
  },
  {
    "chainId": 1,
    "address": "0xc3c7b03335eb950a2a9207ac5cac0571de34d844",
    "name": "BABY WALL STREET MEMES",
    "symbol": "BWSM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32174/thumb/BabyWallStreet-2.png?1696742579"
  },
  {
    "chainId": 1,
    "address": "0x66d4e39c355a911f6d19b3612785b63184a57c72",
    "name": "Aardvark",
    "symbol": "VARK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32678/thumb/VARK.jpg?1698929830"
  },
  {
    "chainId": 1,
    "address": "0x357c915d7c12dc506d13332bb06c932af13e99a0",
    "name": "LuckysLeprecoin",
    "symbol": "LUCKYSLP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32303/thumb/Luckys_Leprecoin_New_Logo_200x200.png?1697187844"
  },
  {
    "chainId": 1,
    "address": "0x6b89b97169a797d94f057f4a0b01e2ca303155e4",
    "name": "Chad Coin",
    "symbol": "CHAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29863/thumb/UfTmUNWv_400x400.png?1696528788"
  },
  {
    "chainId": 1,
    "address": "0xad1a5b8538a866ecd56ddd328b50ed57ced5d936",
    "name": "Good Gensler",
    "symbol": "GENSLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29943/thumb/goodgg-logo-.png?1696528870"
  },
  {
    "chainId": 1,
    "address": "0x40d16fc0246ad3160ccc09b8d0d3a2cd28ae6c2f",
    "name": "GHO",
    "symbol": "GHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30663/thumb/ghoaave.jpeg?1696529533"
  },
  {
    "chainId": 1,
    "address": "0xcc8e21f599995d1c8367054841b8af5024ddf01b",
    "name": "Alpha Gardeners",
    "symbol": "AG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31153/thumb/AG.png?1696529981"
  },
  {
    "chainId": 1,
    "address": "0xf819d9cb1c2a819fd991781a822de3ca8607c3c9",
    "name": "Unibot",
    "symbol": "UNIBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30462/thumb/logonoline_%281%29.png?1696529349"
  },
  {
    "chainId": 1,
    "address": "0x0fe0ed7f146cb12e4b9759aff4fa8d34571802ca",
    "name": "Pool Partyyy",
    "symbol": "PARTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29832/thumb/dioiHjS2_400x400.jpg?1696528760"
  },
  {
    "chainId": 1,
    "address": "0xc6cc3d07c705e39d11c7f60d8836c7c78d4ac5f1",
    "name": "Olea Token",
    "symbol": "OLEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29056/thumb/symbol.png?1696528023"
  },
  {
    "chainId": 1,
    "address": "0xf04af3f4e4929f7cd25a751e6149a3318373d4fe",
    "name": "Spring Token",
    "symbol": "SPRING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25364/thumb/spring.png?1696524497"
  },
  {
    "chainId": 1,
    "address": "0x3ea8ea4237344c9931214796d9417af1a1180770",
    "name": "Flux Protocol",
    "symbol": "FLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21137/thumb/Flux_Logo.jpg?1712087060"
  },
  {
    "chainId": 1,
    "address": "0x30d20208d987713f46dfd34ef128bb16c404d10f",
    "name": "Stader",
    "symbol": "SD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20658/thumb/SD_Token_Logo.png?1696520060"
  },
  {
    "chainId": 1,
    "address": "0x53250b5dfa8c911547afeaf18db025024c8e919a",
    "name": "Kermit",
    "symbol": "KERMIT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29858/thumb/Kermit.png?1696528784"
  },
  {
    "chainId": 1,
    "address": "0xafe7131a57e44f832cb2de78ade38cad644aac2f",
    "name": "Morpho Aave Tether USD",
    "symbol": "MAUSDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29834/thumb/maUSDT_%284%29.png?1696528762"
  },
  {
    "chainId": 1,
    "address": "0x9994e35db50125e0df82e4c2dde62496ce330999",
    "name": "Morpho",
    "symbol": "MORPHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29837/thumb/2022-official-morpho-token.png?1696528764"
  },
  {
    "chainId": 1,
    "address": "0x80f0c1c49891dcfdd40b6e0f960f84e6042bcb6f",
    "name": "DBXen",
    "symbol": "DXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29838/thumb/DBXen.jpg?1696528765"
  },
  {
    "chainId": 1,
    "address": "0xd068c7c941fbbd2300cb2f1841858c2643722dc7",
    "name": "DGNAPP AI",
    "symbol": "DEGAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29745/thumb/logo_200x200.png?1696528677"
  },
  {
    "chainId": 1,
    "address": "0x6982508145454ce325ddbe47a25d4ec3d2311933",
    "name": "Pepe",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29850/thumb/pepe-token.jpeg?1696528776"
  },
  {
    "chainId": 1,
    "address": "0xc8020985a6b30773d866cbef65a7a11f96773413",
    "name": "Envi Coin",
    "symbol": "ENVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32375/thumb/IMG_1970.png?1698035209"
  },
  {
    "chainId": 1,
    "address": "0x159cdaf78be31e730d9e1330adfcfbb79a5fdb95",
    "name": "MixToEarn",
    "symbol": "MTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30893/thumb/200x200x.png?1696529739"
  },
  {
    "chainId": 1,
    "address": "0x427a03fb96d9a94a6727fbcfbba143444090dd64",
    "name": "PIXL",
    "symbol": "PIXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33045/thumb/20220531_190522.jpg?1700450022"
  },
  {
    "chainId": 1,
    "address": "0xea18dc833653172bbe819feb5320d9da9f99799a",
    "name": "Dogelon Mars 2 0",
    "symbol": "ELON20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30963/thumb/ABE04427-81C7-4AF2-9308-AE35C661E66E.jpeg?1696529803"
  },
  {
    "chainId": 1,
    "address": "0x0d505c03d30e65f6e9b4ef88855a47a89e4b7676",
    "name": "Zoomer",
    "symbol": "ZOOMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30894/thumb/zoooooooooomer.jpg?1696529740"
  },
  {
    "chainId": 1,
    "address": "0xf5ec1a08902ae0ae5323466c35ea49a37409e4ca",
    "name": "SAKURA UNITED PLATFORM",
    "symbol": "SUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32213/thumb/apple200%C3%97200.png?1696827569"
  },
  {
    "chainId": 1,
    "address": "0xf2a22b900dde3ba18ec2aef67d4c8c1a0dab6aac",
    "name": "Monkeys",
    "symbol": "MONKEYS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29815/thumb/Monkeys_Elon.png?1696528744"
  },
  {
    "chainId": 1,
    "address": "0x577fee283e776eec29c9e4d258431982780a38a8",
    "name": "Pepa ERC",
    "symbol": "PEPA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29866/thumb/3iGdoyL.png?1696528791"
  },
  {
    "chainId": 1,
    "address": "0xaf75d880b3128981d1fed3292fc02e3fb37acd53",
    "name": "TruthGPT",
    "symbol": "TRUTH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29867/thumb/TruthGPT.png?1696528792"
  },
  {
    "chainId": 1,
    "address": "0x93ad9b819c88d98b4c9641470a96e24769ae7922",
    "name": "KRYZA Exchange",
    "symbol": "KRX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21249/thumb/krx200150.png?1696520622"
  },
  {
    "chainId": 1,
    "address": "0x40803cea2b2a32bda1be61d3604af6a814e70976",
    "name": "Spool",
    "symbol": "SPOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21532/thumb/spool.png?1696520891"
  },
  {
    "chainId": 1,
    "address": "0x5c559f3ee9a81da83e069c0093471cb05d84052a",
    "name": "BabyPepe",
    "symbol": "BABYPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29869/thumb/BabyPepe200.png?1696528794"
  },
  {
    "chainId": 1,
    "address": "0x8bcbef61acd66537362f38167f11875134ffcd63",
    "name": "Pepe Girl",
    "symbol": "PEPEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29896/thumb/pepe_sad_reverse_200X200.png?1696528820"
  },
  {
    "chainId": 1,
    "address": "0x490bbbc2485e99989ba39b34802fafa58e26aba4",
    "name": "Morpho Aave Wrapped Ether",
    "symbol": "MAWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29873/thumb/maWETH.jpg?1696528798"
  },
  {
    "chainId": 1,
    "address": "0x571e21a545842c6ce596663cda5caa8196ac1c7a",
    "name": "Champignons of Arborethia",
    "symbol": "CHAMPZ",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31124/thumb/Champz_logo_square_200x200.png?1704671904"
  },
  {
    "chainId": 1,
    "address": "0x5026f006b85729a8b14553fae6af249ad16c9aab",
    "name": "Wojak",
    "symbol": "WOJAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29856/thumb/wojak.png?1696528782"
  },
  {
    "chainId": 1,
    "address": "0xee772cec929d8430b4fa7a01cd7fbd159a68aa83",
    "name": "Shanghai Inu",
    "symbol": "SHANG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29881/thumb/IMG_20230418_174523_393-removebg-preview.png?1696528806"
  },
  {
    "chainId": 1,
    "address": "0x9dc7094530cb1bcf5442c3b9389ee386738a190c",
    "name": "Morpho Aave Curve DAO Token",
    "symbol": "MACRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29874/thumb/maCRV.jpg?1696528799"
  },
  {
    "chainId": 1,
    "address": "0xc00e94cb662c3520282e6f5717214004a7f26888",
    "name": "Compound",
    "symbol": "COMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10775/thumb/COMP.png?1696510737"
  },
  {
    "chainId": 1,
    "address": "0xd59d7d2e955533fcd21641da8a70eae9624a3c49",
    "name": "MagicRing",
    "symbol": "MRING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36905/thumb/photo_2024-04-08_01-09-25.jpg?1712724842"
  },
  {
    "chainId": 1,
    "address": "0xad8d0de33c43eefe104a279cdb6ae250c12e6214",
    "name": "Naruto",
    "symbol": "NARUTO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29963/thumb/logo.jpg?1696528889"
  },
  {
    "chainId": 1,
    "address": "0xf7790914dc335b20aa19d7c9c9171e14e278a134",
    "name": "Euro Coinvertible",
    "symbol": "EUR-C",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29903/thumb/Screenshot_2023-04-20_at_5.57.42_PM.png?1696528832"
  },
  {
    "chainId": 1,
    "address": "0x18e5f92103d1b34623738ee79214b1659f2ee109",
    "name": "Wrapped CellMates",
    "symbol": "WCELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35154/thumb/755F6B74-7940-475C-9818-4672DEF73BDA.png?1707561272"
  },
  {
    "chainId": 1,
    "address": "0xe46091dce9c67691bcf22768bbee0bc9e20d4beb",
    "name": "WSB Classic",
    "symbol": "WSBC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30165/thumb/photo_2023-05-05_07-44-23.jpg?1696529085"
  },
  {
    "chainId": 1,
    "address": "0x9a6aea7c2b0d563eab701e3255110d30f08b5a64",
    "name": "Metapolitans",
    "symbol": "MAPS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29933/thumb/Logo.png?1696528861"
  },
  {
    "chainId": 1,
    "address": "0x8442e0e292186854bb6875b2a0fc1308b9ded793",
    "name": "Print The Pepe",
    "symbol": "PP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29964/thumb/printthepepe.PNG?1696528890"
  },
  {
    "chainId": 1,
    "address": "0x1a963df363d01eebb2816b366d61c917f20e1ebe",
    "name": "MEMEME",
    "symbol": "MEMEME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29965/thumb/MEMEME_token_image_200px.png?1696528891"
  },
  {
    "chainId": 1,
    "address": "0x1a11ea9d61588d756d9f1014c3cf0d226aedd279",
    "name": "MILEI Token",
    "symbol": "MILEI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34697/thumb/MILEI_CIRCLE_200px.png?1705858944"
  },
  {
    "chainId": 1,
    "address": "0xddd5592cf4759313c649eb4e624a79541ed222ed",
    "name": "PepeXL",
    "symbol": "PEPEXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30343/thumb/plogo200.png?1696529243"
  },
  {
    "chainId": 1,
    "address": "0x5582a479f0c403e207d2578963ccef5d03ba636f",
    "name": "Salad",
    "symbol": "SALD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29936/thumb/RJV4r-hN_400x400.jpg?1696528864"
  },
  {
    "chainId": 1,
    "address": "0x244b797d622d4dee8b188b03546acaabd0cf91a0",
    "name": "FourCoin",
    "symbol": "FOUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30167/thumb/fourcoin.jpeg?1696529087"
  },
  {
    "chainId": 1,
    "address": "0x553afe6468949e0685959022217336717df5fbe8",
    "name": "Palette",
    "symbol": "PLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35615/thumb/output-onlinepngtools-13-1.png?1709282022"
  },
  {
    "chainId": 1,
    "address": "0x4c6e2c495b974b8d4220e370f23c7e0e1da9b644",
    "name": "Smiley Coin",
    "symbol": "SMILEY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30809/thumb/photo_2023-06-21_03-25-14.jpg?1696529666"
  },
  {
    "chainId": 1,
    "address": "0x8e729198d1c59b82bd6bba579310c40d740a11c2",
    "name": "Alvara Protocol",
    "symbol": "ALVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35707/thumb/ALVA.png?1709624242"
  },
  {
    "chainId": 1,
    "address": "0x755582d6db18003e67eccaa7090421a775280a36",
    "name": "FP  JeerGirl",
    "symbol": "JEERGIRL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34108/thumb/JEERGIRL.jpg?1703988353"
  },
  {
    "chainId": 1,
    "address": "0x9d69a8ccb7cbc03d3dbb2c4ab03c0c1fad3e74b0",
    "name": "Creatopy Builder",
    "symbol": "CREATOPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36335/thumb/IMG_20240321_153859_510.jpg?1711100114"
  },
  {
    "chainId": 1,
    "address": "0x2f573070e6090b3264fe707e2c9f201716f123c7",
    "name": "Mumu",
    "symbol": "MUMU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29921/thumb/mumuGreen.jpg?1712079227"
  },
  {
    "chainId": 1,
    "address": "0x15f20f9dfdf96ccf6ac96653b7c0abfe4a9c9f0f",
    "name": "Wall Street Baby",
    "symbol": "WSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15042/thumb/GMih0pSW_400x400.png?1696514701"
  },
  {
    "chainId": 1,
    "address": "0x6bc40d4099f9057b23af309c08d935b890d7adc0",
    "name": "SnailBrook",
    "symbol": "SNAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29922/thumb/snail_200x200.png?1696528851"
  },
  {
    "chainId": 1,
    "address": "0x0adcfdfe9e88d28cc4c1daac9cc021067aff9b0c",
    "name": "AnimalFam",
    "symbol": "TOTOFO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29959/thumb/logo2_%283%29.png?1696528885"
  },
  {
    "chainId": 1,
    "address": "0xec505c81d6a7567b5bde804870b1038832fe6da1",
    "name": "Coinhound",
    "symbol": "CND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30056/thumb/%28200x200%29_Coinhound_Icon.png?1696528977"
  },
  {
    "chainId": 1,
    "address": "0xb8c55c80a1cb7394088a36c6b634dc2bf3c6fb67",
    "name": "Pepe Doge",
    "symbol": "PEPEDOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29968/thumb/Pepedoge.jpeg?1696528894"
  },
  {
    "chainId": 1,
    "address": "0xd7b2c1a7f3c67fb0ea57a7ef29bc1f18d7be3195",
    "name": "VoluMint",
    "symbol": "VMINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33745/thumb/volumint_200x200px_clearbg.png?1702924153"
  },
  {
    "chainId": 1,
    "address": "0x6c4c193bff0a117f0c2b516802abba961a1eeb12",
    "name": "Papa",
    "symbol": "PAPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29971/thumb/logo-dex.png?1696528897"
  },
  {
    "chainId": 1,
    "address": "0x41545f8b9472d758bb669ed8eaeeecd7a9c4ec29",
    "name": "Forta",
    "symbol": "FORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25060/thumb/Forta_lgo_%281%29.png?1696524210"
  },
  {
    "chainId": 1,
    "address": "0x5ad02305ba9a4985390170337582986e419f1a2c",
    "name": "Crypto X",
    "symbol": "CX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31084/thumb/Logo_W_kBG_%281%29.png?1696529917"
  },
  {
    "chainId": 1,
    "address": "0x8a7b7b9b2f7d0c63f66171721339705a6188a7d5",
    "name": "EtherDoge",
    "symbol": "EDOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1013/thumb/etherdoge.png?1696502125"
  },
  {
    "chainId": 1,
    "address": "0xb06b8186cc008a79fd6722b1eefad07c14e97da0",
    "name": "Sign Token",
    "symbol": "SIGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26495/thumb/PWRjbqG9_400x400.png?1696525569"
  },
  {
    "chainId": 1,
    "address": "0x3be65bd36c1ad28d33acc1c9dd8338a2706ca000",
    "name": "Kairos",
    "symbol": "KAIROS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30001/thumb/IMG_1074.png?1696528926"
  },
  {
    "chainId": 1,
    "address": "0x0ba74fb26ca523f2dc22fa4318581cc2452eaba1",
    "name": "Bogdanoff",
    "symbol": "BOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30026/thumb/telegram-cloud-document-1-1809833204887060537_copia_2.png?1696528949"
  },
  {
    "chainId": 1,
    "address": "0xe07c41e9cdf7e0a7800e4bbf90d414654fd6413d",
    "name": "CBDC",
    "symbol": "CBDC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31665/thumb/MThOhXH1_400x400.jpg?1696530482"
  },
  {
    "chainId": 1,
    "address": "0x98e35f5599b57998900e5e0675721c90a5499327",
    "name": "BuddyAI",
    "symbol": "BUDDY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34109/thumb/photo_2023-12-26_10-13-17.jpg?1703988840"
  },
  {
    "chainId": 1,
    "address": "0xa9b1eb5908cfc3cdf91f9b8b3a74108598009096",
    "name": "Bounce",
    "symbol": "AUCTION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13860/thumb/1_KtgpRIJzuwfHe0Rl0avP_g.jpeg?1696513606"
  },
  {
    "chainId": 1,
    "address": "0xc7a2572fa8fdb0f7e81d6d3c4e3ccf78fb0dc374",
    "name": "Ben s Finale",
    "symbol": "FINALE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30661/thumb/tGy9rQvi_400x400.jpg?1696529531"
  },
  {
    "chainId": 1,
    "address": "0xe1ec350ea16d1ddaff57f31387b2d9708eb7ce28",
    "name": "Pepechain",
    "symbol": "PC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30059/thumb/Pepechain.png?1696528980"
  },
  {
    "chainId": 1,
    "address": "0x2047ab3072b52561596ce5e0131bdbb7c848538d",
    "name": "Bored Token",
    "symbol": "BORED",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28515/thumb/Bored.png?1696527508"
  },
  {
    "chainId": 1,
    "address": "0x2f5fa8adf5f09a5f9de05b65fe82a404913f02c4",
    "name": "TROLL 2 0",
    "symbol": "TROLL20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34605/thumb/Troll_2.o.jpg?1705492471"
  },
  {
    "chainId": 1,
    "address": "0x2c7f442aab99d5e18cfae2291c507c0b5f3c1eb5",
    "name": "Keko",
    "symbol": "KEKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29967/thumb/ytX0JKjO_400x400.jpg?1696528893"
  },
  {
    "chainId": 1,
    "address": "0x0414d8c87b271266a5864329fb4932bbe19c0c49",
    "name": "WSB Coin",
    "symbol": "WSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30060/thumb/wsb_pfp.png?1696528981"
  },
  {
    "chainId": 1,
    "address": "0xa3bed4e1c75d00fa6f4e5e6922db7261b5e9acd2",
    "name": "mStable Governance  Meta",
    "symbol": "MTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11846/thumb/mStable.png?1696511717"
  },
  {
    "chainId": 1,
    "address": "0xe7ac8545e34771de3706598abb3db9a19af2a07f",
    "name": "MONKED",
    "symbol": "MONKED",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30011/thumb/monked200.png?1696528935"
  },
  {
    "chainId": 1,
    "address": "0xa589d8868607b8d79ee4288ce192796051263b64",
    "name": "TATE",
    "symbol": "TATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30034/thumb/Tate_logo_Coingecko.png?1696528957"
  },
  {
    "chainId": 1,
    "address": "0x81db1949d0e888557bc632f7c0f6698b1f8c9106",
    "name": "d acc",
    "symbol": "D/ACC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33313/thumb/dddd.png?1701412963"
  },
  {
    "chainId": 1,
    "address": "0x0f7b3f5a8fed821c5eb60049538a548db2d479ce",
    "name": "AirTor Protocol",
    "symbol": "ATOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29690/thumb/AirTor.png?1696528623"
  },
  {
    "chainId": 1,
    "address": "0xb076bda1abc154ddb4ccd9be45542a823aee290e",
    "name": "FlexMeme",
    "symbol": "FLEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30018/thumb/flexmeme.png?1696528942"
  },
  {
    "chainId": 1,
    "address": "0xd08623fb2a1f044025eec65886011cf5d0f06b01",
    "name": "Larry",
    "symbol": "LARRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30460/thumb/Safeimagekit-resized-img.png?1696529347"
  },
  {
    "chainId": 1,
    "address": "0xde8cd13b812bcd82218754a740b27e76ec1e86ad",
    "name": "TRESTLE",
    "symbol": "TRESTLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34992/thumb/PNG_-_Logo_-_3485x.png?1706943532"
  },
  {
    "chainId": 1,
    "address": "0xb4fc1fc74effa5dc15a031eb8159302cfa4f1288",
    "name": "OrdiZK",
    "symbol": "OZK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33619/thumb/IMG_20231214_094021_481.jpg?1702528631"
  },
  {
    "chainId": 1,
    "address": "0xa117000000f279d81a1d3cc75430faa017fa5a2e",
    "name": "Aragon",
    "symbol": "ANT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/681/thumb/Avatar_Circle_x6.png?1696501871"
  },
  {
    "chainId": 1,
    "address": "0xb2cb83e8e1b326373b7f1068d10c50ebfa04f070",
    "name": "Pepito",
    "symbol": "PEPI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30062/thumb/IMG_20230428_101018_444.png?1696528984"
  },
  {
    "chainId": 1,
    "address": "0x8457ca5040ad67fdebbcc8edce889a335bc0fbfb",
    "name": "AltLayer",
    "symbol": "ALT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34608/thumb/altlayer.jpeg?1705497960"
  },
  {
    "chainId": 1,
    "address": "0xd87d72248093597df8d56d2a53c1ab7c1a0cc8da",
    "name": "HAHA",
    "symbol": "HAHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30070/thumb/2.png?1696528991"
  },
  {
    "chainId": 1,
    "address": "0xc5190e7fec4d97a3a3b1ab42dfedac608e2d0793",
    "name": "FX1Sports",
    "symbol": "FXI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30078/thumb/fx1_sports_logo.jpg?1696529003"
  },
  {
    "chainId": 1,
    "address": "0x3d2b66bc4f9d6388bd2d97b95b565be1686aefb3",
    "name": " LAMBO",
    "symbol": "LAMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30126/thumb/IMG_20230502_225617_622.png?1696529048"
  },
  {
    "chainId": 1,
    "address": "0x7f4c5447af6a96d8eeaee1d932338cfc57890dbd",
    "name": "Dave Coin",
    "symbol": "DAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30703/thumb/davelogo_transparent200x200.png?1696529575"
  },
  {
    "chainId": 1,
    "address": "0xbbb32f99e6f2cb29337eebaa43c5069386de6e6c",
    "name": "Phemex Token",
    "symbol": "PT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33314/thumb/phemex_logo.png?1701959611"
  },
  {
    "chainId": 1,
    "address": "0x2c10c0de3362ff21f8ed6bc7f4ac5e391153fd2c",
    "name": "MARKETVIZ",
    "symbol": "VIZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30083/thumb/CG_Logo.png?1696529007"
  },
  {
    "chainId": 1,
    "address": "0xe60779cc1b2c1d0580611c526a8df0e3f870ec48",
    "name": "unshETHing Token",
    "symbol": "USH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29337/thumb/unsheth_large_logo.png?1696528287"
  },
  {
    "chainId": 1,
    "address": "0xb7446b185085e019f0008f1d338de26486a4db10",
    "name": "Bull Moon",
    "symbol": "BULLMOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32790/thumb/bullmoon_logo_200x200.png?1699427210"
  },
  {
    "chainId": 1,
    "address": "0x1f7505f486c22f4338ac2bde67a3e93a547644b9",
    "name": "Cat Intelligence Agency",
    "symbol": "CIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34728/thumb/cia.jpeg?1706632795"
  },
  {
    "chainId": 1,
    "address": "0x5f7827fdeb7c20b443265fc2f40845b715385ff2",
    "name": "Societe Generale FORGE EURCV",
    "symbol": "EURCV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33415/thumb/eurcv_%281%29.png?1701752017"
  },
  {
    "chainId": 1,
    "address": "0x4bb3205bf648b7f59ef90dee0f1b62f6116bc7ca",
    "name": "NBX",
    "symbol": "BYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14746/thumb/NBX.png?1696514416"
  },
  {
    "chainId": 1,
    "address": "0x994a258c7dec633b5b15376f850d5ea701179e79",
    "name": "0xGasless  OLD ",
    "symbol": "0XGAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31345/thumb/0XGAS.jpg?1696530163"
  },
  {
    "chainId": 1,
    "address": "0xd77401a76d6cdb7ac3bb031bf25dec07615509e7",
    "name": "Metropoly",
    "symbol": "METRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30138/thumb/Metropoly_Icon.png?1696529059"
  },
  {
    "chainId": 1,
    "address": "0xe7c8537f92b4feefdc19bd6b4023dfe79400cb30",
    "name": "Bones",
    "symbol": "BONES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32232/thumb/IMG_20231008_075551_623_%281%29.jpg?1696936788"
  },
  {
    "chainId": 1,
    "address": "0xa1faa113cbe53436df28ff0aee54275c13b40975",
    "name": "Stella",
    "symbol": "ALPHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12738/thumb/Stella200x200-06.png?1696512537"
  },
  {
    "chainId": 1,
    "address": "0x0f1e49d6dcfc9eefcce9d5ae3c660f8ead75061a",
    "name": "Vinlink",
    "symbol": "VNLNK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30139/thumb/VNLNK_Logo.png?1696529060"
  },
  {
    "chainId": 1,
    "address": "0x4c19596f5aaff459fa38b0f7ed92f11ae6543784",
    "name": "TrueFi",
    "symbol": "TRU",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13180/thumb/truefi_glyph_color.png?1696512963"
  },
  {
    "chainId": 1,
    "address": "0x5fc111f3fa4c6b32eaf65659cfebdeed57234069",
    "name": "0xGasless",
    "symbol": "0XGAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33756/thumb/Main_LOGO.jpg?1707051665"
  },
  {
    "chainId": 1,
    "address": "0x02795795196f563fdafce8dd97fca4871ded51c3",
    "name": "Winnerz",
    "symbol": "WNZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34924/thumb/wnz.jpeg?1706666326"
  },
  {
    "chainId": 1,
    "address": "0x255f1b39172f65dc6406b8bee8b08155c45fe1b6",
    "name": "HarambeCoin",
    "symbol": "HARAMBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36043/thumb/HarambeCoin_full_cameo_200.png?1710400612"
  },
  {
    "chainId": 1,
    "address": "0x666cbfaa3baa2faccfac8854fea1e5db140fb104",
    "name": "PLUMS",
    "symbol": "PLUMS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30097/thumb/logoPlums.png?1696529021"
  },
  {
    "chainId": 1,
    "address": "0x7d8146cf21e8d7cbe46054e01588207b51198729",
    "name": "BOB Token",
    "symbol": "BOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29929/thumb/bob.png?1696528857"
  },
  {
    "chainId": 1,
    "address": "0x9bf1d7d63dd7a4ce167cf4866388226eeefa702e",
    "name": "Ben",
    "symbol": "BEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30175/thumb/BEN_COIN.png?1696529094"
  },
  {
    "chainId": 1,
    "address": "0x586a7cfe21e55ec0e24f0bfb118f77fe4ca87bab",
    "name": "BetbuInu",
    "symbol": "CRYPTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31298/thumb/Betbu.jpeg?1696530118"
  },
  {
    "chainId": 1,
    "address": "0xe9da5e227e3fa4fc933b5f540be021e7ecc3fd81",
    "name": "GMFAM",
    "symbol": "GMFAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30547/thumb/GMFAM-logo_%282%29.png?1696529419"
  },
  {
    "chainId": 1,
    "address": "0x4e47951508fd4a4126f8ff9cf5e6fa3b7cc8e073",
    "name": "Fluid",
    "symbol": "FLUID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31890/thumb/fluidlogo200x200.png?1696530701"
  },
  {
    "chainId": 1,
    "address": "0xd4f4d0a10bcae123bb6655e8fe93a30d01eebd04",
    "name": "LinqAI",
    "symbol": "LNQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35645/thumb/cmc-cg-linq-logo_%282%29.png?1709368877"
  },
  {
    "chainId": 1,
    "address": "0xa1abecc1b3958da78259fa2793653fc48e976420",
    "name": "Dogggo",
    "symbol": "DOGGGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34634/thumb/DT-Thumbnail-200x200.png?1705549833"
  },
  {
    "chainId": 1,
    "address": "0xea7b7dc089c9a4a916b5a7a37617f59fd54e37e4",
    "name": "HyperCycle",
    "symbol": "HYPC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29917/thumb/HyperCycle-Logo-2022_icon_copy_%281%29.png?1696528845"
  },
  {
    "chainId": 1,
    "address": "0xf03d5fc6e08de6ad886fca34abf9a59ef633b78a",
    "name": "Capybara Token",
    "symbol": "CAPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30223/thumb/capylogo.png?1696529133"
  },
  {
    "chainId": 1,
    "address": "0xeee0fe52299f2de8e2ed5111cd521ab67dcf0faf",
    "name": "The QWAN",
    "symbol": "QWAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30613/thumb/qwan.jpg?1696529482"
  },
  {
    "chainId": 1,
    "address": "0xa8388b8334beb4840d65ed80f858b080dffd7e2b",
    "name": "Son Of Pepe",
    "symbol": "SOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30895/thumb/IMG_20230701_205503_304_%283%29.png?1696529741"
  },
  {
    "chainId": 1,
    "address": "0x675ababd3a210566a5e213547523b740be80041a",
    "name": "Entropy",
    "symbol": "ENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30102/thumb/Entropy_bi_symbol_200x200.png?1696529025"
  },
  {
    "chainId": 1,
    "address": "0x02d7a93829b365b7ad4c582dace1493aac50a290",
    "name": "Scat",
    "symbol": "CAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30429/thumb/output-onlinepngtools.png?1696529317"
  },
  {
    "chainId": 1,
    "address": "0x223028738503838e89fc5fd5b1a42f1d024d9600",
    "name": "DegenInsure",
    "symbol": "DGNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32185/thumb/Frame_12_%282%29.png?1696756306"
  },
  {
    "chainId": 1,
    "address": "0xb2dbf14d0b47ed3ba02bdb7c954e05a72deb7544",
    "name": "MobiFi",
    "symbol": "MOFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14697/thumb/MOFI_Coin_Green_200x200.png?1696514369"
  },
  {
    "chainId": 1,
    "address": "0x8e235f491ae66b82296d58332adc2a021c449c10",
    "name": "Tipja",
    "symbol": "TIPJA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30182/thumb/logo.jpg?1696529101"
  },
  {
    "chainId": 1,
    "address": "0x12970e6868f88f6557b76120662c1b3e50a646bf",
    "name": "Milady Meme Coin",
    "symbol": "LADYS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30194/thumb/logo.png?1696529109"
  },
  {
    "chainId": 1,
    "address": "0xe45dfc26215312edc131e34ea9299fbca53275ca",
    "name": "Relation Native Token",
    "symbol": "REL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30656/thumb/2.png?1696529526"
  },
  {
    "chainId": 1,
    "address": "0x378e1be15be6d6d1f23cfe7090b6a77660dbf14d",
    "name": "FOXE",
    "symbol": "FOXE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30227/thumb/foxe.jpeg?1696529137"
  },
  {
    "chainId": 1,
    "address": "0xc961da88bb5e8ee2ba7dfd4c62a875ef80f7202f",
    "name": "Haram",
    "symbol": "HARAM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30374/thumb/200x200trans.png?1696529268"
  },
  {
    "chainId": 1,
    "address": "0xef8e456967122db4c3c160314bde8d2602ad6199",
    "name": "Wagmi Coin",
    "symbol": "WAGMI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30477/thumb/W.png?1696529363"
  },
  {
    "chainId": 1,
    "address": "0xb435a47ecea7f5366b2520e45b9bed7e01d2ffae",
    "name": "The Nemesis",
    "symbol": "NEMS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30590/thumb/nems.png?1696529458"
  },
  {
    "chainId": 1,
    "address": "0xb10cc888cb2cce7036f4c7ecad8a57da16161338",
    "name": "Switch Token",
    "symbol": "SWITCH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30140/thumb/SWITCH-200x200-Circle_Token.jpg?1696529061"
  },
  {
    "chainId": 1,
    "address": "0x0ebe30595a44e5288c24161ddfc1e9fa08e33a0c",
    "name": "Nyan Meme Coin",
    "symbol": "NYAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30197/thumb/n0z-571z_400x400.png?1696529112"
  },
  {
    "chainId": 1,
    "address": "0x9ff58067bd8d239000010c154c6983a325df138e",
    "name": "Propchain",
    "symbol": "PROPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30451/thumb/prop_%281%29.png?1702322253"
  },
  {
    "chainId": 1,
    "address": "0xe08ef9206a8a7c9337cc6611b4f5226fdafc4772",
    "name": "MESSI COIN",
    "symbol": "MESSI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30762/thumb/x4aUydEr_400x400.jpg?1696529631"
  },
  {
    "chainId": 1,
    "address": "0x4fe8d4775b7cb2546b9ee86182081cdf8f77b053",
    "name": "KAIJUNO8",
    "symbol": "KAIJU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35688/thumb/mmexport1709168349348.jpg?1709533292"
  },
  {
    "chainId": 1,
    "address": "0x730bcbe5cdc1a3061dfe700774b7b8dd1d4173db",
    "name": "DaVinci",
    "symbol": "WTF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35734/thumb/IMG_8820.png?1709693722"
  },
  {
    "chainId": 1,
    "address": "0xc7f950271d118a5bdf250dffc39128dcced8472c",
    "name": "Chainback",
    "symbol": "ARCHIVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30580/thumb/Chainback-glyph-logo-200.png?1696529444"
  },
  {
    "chainId": 1,
    "address": "0x9c0bd34bebc33a0e898554cfc91e8a84c728bf9f",
    "name": "Pisscoin",
    "symbol": "PISS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30459/thumb/A7661040-68D2-4753-B20E-B7416A7F90C2.png?1696529346"
  },
  {
    "chainId": 1,
    "address": "0xc3b36424c70e0e6aee3b91d1894c2e336447dbd3",
    "name": "TokenWatch",
    "symbol": "TOKENWATCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34672/thumb/TW_token.png?1705657486"
  },
  {
    "chainId": 1,
    "address": "0x83f20f44975d03b1b09e64809b757c47f942beea",
    "name": "Savings Dai",
    "symbol": "SDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32254/thumb/sdai.png?1697015278"
  },
  {
    "chainId": 1,
    "address": "0xe0f63a424a4439cbe457d80e4f4b51ad25b2c56c",
    "name": "SPX6900",
    "symbol": "SPX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31401/thumb/sticker_%281%29.jpg?1702371083"
  },
  {
    "chainId": 1,
    "address": "0x2f123cf3f37ce3328cc9b5b8415f9ec5109b45e7",
    "name": "Backed GOVIES 0 6 months EURO",
    "symbol": "BC3M",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31870/thumb/b-C3-M-200x200.png?1696530682"
  },
  {
    "chainId": 1,
    "address": "0x1e2c4fb7ede391d116e6b41cd0608260e8801d59",
    "name": "Backed CSPX Core S P 500",
    "symbol": "BCSPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31891/thumb/b-CSPX-200x200.png?1696530702"
  },
  {
    "chainId": 1,
    "address": "0xfe60fba03048effb4acf3f0088ec2f53d779d3bb",
    "name": "3d3d",
    "symbol": "3D3D",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30232/thumb/200x200_2.PNG?1696529142"
  },
  {
    "chainId": 1,
    "address": "0xbbc7f7a6aadac103769c66cbc69ab720f7f9eae3",
    "name": "INX Token",
    "symbol": "INX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29241/thumb/Tc8Z6xAG_400x400.jpg?1696528197"
  },
  {
    "chainId": 1,
    "address": "0x226bb599a12c826476e3a771454697ea52e9e220",
    "name": "Propy",
    "symbol": "PRO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/869/thumb/propy.png?1696502002"
  },
  {
    "chainId": 1,
    "address": "0xf831938caf837cd505de196bbb408d81a06376ab",
    "name": "Jeff",
    "symbol": "JEFF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30222/thumb/photo_2023-05-06_23.53.02.jpeg?1696529132"
  },
  {
    "chainId": 1,
    "address": "0x370a366f402e2e41cdbbe54ecec12aae0cce1955",
    "name": "Toad Killer",
    "symbol": "TOAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30234/thumb/khLXf12.png?1700669365"
  },
  {
    "chainId": 1,
    "address": "0x1f17d72cbe65df609315df5c4f5f729efbd00ade",
    "name": "GYOSHI",
    "symbol": "GYOSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30002/thumb/99245ad73184347ee7c93bfc4f8ca2c2.png?1696528927"
  },
  {
    "chainId": 1,
    "address": "0x68ddd6812684ff87d483342b69db5a89c2af711b",
    "name": "PINGU",
    "symbol": "NOOTNOOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32275/thumb/PINGU.png?1697180139"
  },
  {
    "chainId": 1,
    "address": "0x5b649c07e7ba0a1c529deaabed0b47699919b4a2",
    "name": "Suzuverse",
    "symbol": "SGT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30549/thumb/Suzuverse.png?1696529421"
  },
  {
    "chainId": 1,
    "address": "0x335f4e66b9b61cee5ceade4e727fcec20156b2f0",
    "name": "ElmoERC",
    "symbol": "ELMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30261/thumb/elmologo-21.png?1696930389"
  },
  {
    "chainId": 1,
    "address": "0x5cd8adec25af71f038c54bf580fc04d06b5f12ab",
    "name": "Suave",
    "symbol": "CCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32476/thumb/suave_logo.png?1698286252"
  },
  {
    "chainId": 1,
    "address": "0x441c3c2f4a92f1b90f916811514ebddbbd3bff4f",
    "name": "LFG coin",
    "symbol": "LFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30278/thumb/T207klVP_400x400.jpg?1696529184"
  },
  {
    "chainId": 1,
    "address": "0x64e8cc11ddccea70cc800231a62f0d9ea46c94ac",
    "name": "XApis",
    "symbol": "WAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32477/thumb/photo_2023-10-18_16.45.53.jpeg?1698286327"
  },
  {
    "chainId": 1,
    "address": "0x3f14920c99beb920afa163031c4e47a3e03b3e4a",
    "name": " send",
    "symbol": "SEND",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/30847/thumb/logo_send_new_africa.png?1709638658"
  },
  {
    "chainId": 1,
    "address": "0xc666081073e8dff8d3d1c2292a29ae1a2153ec09",
    "name": "Digitex",
    "symbol": "DGTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2188/thumb/DGTX.png?1696503145"
  },
  {
    "chainId": 1,
    "address": "0x451fd37983d494bce294295f78a426832376b7df",
    "name": "Xeno",
    "symbol": "XENO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34702/thumb/tokenlogo200x200.jpg?1705861546"
  },
  {
    "chainId": 1,
    "address": "0x9dcd367e2afa8d6e5d6cf0306094e3eb7bbaaf4d",
    "name": "Crypto Bros",
    "symbol": "BROS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30338/thumb/Crypto_Bros_%284%29.png?1696529239"
  },
  {
    "chainId": 1,
    "address": "0x58083b54013631bacc0bbb6d4efa543fee1d9ce0",
    "name": "Force",
    "symbol": "FRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30917/thumb/force_token_1000.png?1696529762"
  },
  {
    "chainId": 1,
    "address": "0xf7168c8abb0ff80116413a8d95396bbdc318a3ff",
    "name": "KEK",
    "symbol": "KEKE",
    "decimals": 7,
    "logoURI": "https://assets.coingecko.com/coins/images/30172/thumb/KEKE.png?1696529092"
  },
  {
    "chainId": 1,
    "address": "0x4743a7a193cdf202035e9bc6830a07f1607630c4",
    "name": "Family Guy",
    "symbol": "GUY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30435/thumb/photo_2023-05-16_18-24-10_%281%29.png?1696529323"
  },
  {
    "chainId": 1,
    "address": "0xf0edac27aa3e85e2d176f689b0025f90c154393a",
    "name": "I LOVE SNOOPY",
    "symbol": "LOVESNOOPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30340/thumb/dAlRqD6r_400x400.jpg?1696529240"
  },
  {
    "chainId": 1,
    "address": "0x04c618cdbc1d59142dfeb4b9864835a06983ec2d",
    "name": "Joseon Mun",
    "symbol": "JSM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30387/thumb/IMG_20230511_130419_023.jpg?1696529276"
  },
  {
    "chainId": 1,
    "address": "0x8387c2d1b0eea6b91b910aff66a7bd2710a5946a",
    "name": "Fomo Eth",
    "symbol": "FOMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30389/thumb/photo_2023-05-13_00-46-17.jpg?1696529278"
  },
  {
    "chainId": 1,
    "address": "0x5dcd6272c3cbb250823f0b7e6c618bce11b21f90",
    "name": "Pear Swap",
    "symbol": "PEAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30409/thumb/9zYlZj3H_400x400.jpg?1696529298"
  },
  {
    "chainId": 1,
    "address": "0x9b4a69de6ca0defdd02c0c4ce6cb84de5202944e",
    "name": "PROOF Platform",
    "symbol": "PROOF",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32478/thumb/PROOF_icon_-_200.png?1698286769"
  },
  {
    "chainId": 1,
    "address": "0x856c4efb76c1d1ae02e20ceb03a2a6a08b0b8dc3",
    "name": "Origin Ether",
    "symbol": "OETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29733/thumb/OETH.png?1696528663"
  },
  {
    "chainId": 1,
    "address": "0xd807f7e2818db8eda0d28b5be74866338eaedb86",
    "name": "Jim",
    "symbol": "JIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30235/thumb/_jim_Logo.png?1696529145"
  },
  {
    "chainId": 1,
    "address": "0x168e209d7b2f58f1f24b8ae7b7d35e662bbf11cc",
    "name": "LayerAI",
    "symbol": "LAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29223/thumb/Favicon_200x200px.png?1696528181"
  },
  {
    "chainId": 1,
    "address": "0x27f103f86070cc639fef262787a16887d22d8415",
    "name": "FOFO Token",
    "symbol": "FOFO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30438/thumb/01_%281%29_copy.png?1696529326"
  },
  {
    "chainId": 1,
    "address": "0x3db045814d0a29d831fe38055cb97a956ef7cafb",
    "name": "BlockRemit",
    "symbol": "REMIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30360/thumb/20230512_204249_copy_200x200.jpg?1696529259"
  },
  {
    "chainId": 1,
    "address": "0x382edfe4c6168858c81893fe00fcb7b68914d929",
    "name": "SJ741 Emeralds",
    "symbol": "EMERALD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35360/thumb/emeraldsCoingecko.png?1708363791"
  },
  {
    "chainId": 1,
    "address": "0xe1a3864dbf62fb94834b108ff6bf439ce70183ac",
    "name": "vXDEFI",
    "symbol": "VXDEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30479/thumb/vXDEFI.jpg?1696529365"
  },
  {
    "chainId": 1,
    "address": "0x9ad37205d608b8b219e6a2573f922094cec5c200",
    "name": "iZUMi Finance",
    "symbol": "IZI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21791/thumb/izumi-logo-symbol.png?1696521144"
  },
  {
    "chainId": 1,
    "address": "0x2b9a49417f9c9c8dd18ef5bb37c20637441ad67a",
    "name": "greg",
    "symbol": "GREG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30415/thumb/photo_2023-05-16_14-37-01.jpg?1696529303"
  },
  {
    "chainId": 1,
    "address": "0x8881562783028f5c1bcb985d2283d5e170d88888",
    "name": "Shuffle",
    "symbol": "SHFL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35148/thumb/shuffle_token_200x200.png?1707552712"
  },
  {
    "chainId": 1,
    "address": "0xcedefe438860d2789da6419b3a19cece2a41038d",
    "name": "Love Hate Inu",
    "symbol": "LHINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30455/thumb/lhinu.png?1696529342"
  },
  {
    "chainId": 1,
    "address": "0x9b01637302b6adfc2c82678e2a8d680cff6337b7",
    "name": "Fit",
    "symbol": "FIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30918/thumb/FIT_Token_1000.png?1696529763"
  },
  {
    "chainId": 1,
    "address": "0x830a8512db4f6fca51968593e2667156c2c483a8",
    "name": "WEN Token",
    "symbol": "WEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30400/thumb/kxtrOzUd_400x400.jpg?1696529290"
  },
  {
    "chainId": 1,
    "address": "0x24249b5a869a445c9b0ce269a08d73c618df9d21",
    "name": "HarryPotterTrumpHomerSimpson777Inu",
    "symbol": "ETHEREUM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31320/thumb/hp777_%281%29.jpg?1696530139"
  },
  {
    "chainId": 1,
    "address": "0x0d248ce39e26fb00f911fb1e7a45a00d8c94341c",
    "name": "Butter",
    "symbol": "BUTTER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30252/thumb/butterlogotrans.png?1696529161"
  },
  {
    "chainId": 1,
    "address": "0x77571a64342667f7818520ef004b2b91f47a266b",
    "name": "SnailMoon",
    "symbol": "SNM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30401/thumb/SnailMoonCoingecko.png?1696529291"
  },
  {
    "chainId": 1,
    "address": "0x7163436b8efffb469f6bb81cc908b1661d4795e6",
    "name": "Esco Coin",
    "symbol": "ESCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30405/thumb/escocoin-06.png?1696529294"
  },
  {
    "chainId": 1,
    "address": "0x3d330b8d4eb25b0933e564d7284d462346d453ef",
    "name": "GROQ",
    "symbol": "GROQ",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33268/thumb/GROQ.png?1701357958"
  },
  {
    "chainId": 1,
    "address": "0xf190dbd849e372ff824e631a1fdf199f38358bcf",
    "name": "Capybara Memecoin",
    "symbol": "BARA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30399/thumb/Capy_logo.png?1696529289"
  },
  {
    "chainId": 1,
    "address": "0xa41d2f8ee4f47d3b860a149765a7df8c3287b7f0",
    "name": "Syncus",
    "symbol": "SYNC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33573/thumb/Syncus.png?1702450708"
  },
  {
    "chainId": 1,
    "address": "0xeb4f5c4bf62fac1be7e4bef9c7c055bf1c52241d",
    "name": "Runic Chain",
    "symbol": "RUNIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36044/thumb/logo.png?1710401096"
  },
  {
    "chainId": 1,
    "address": "0x52284158e02425290f6b627aeb5fff65edf058ad",
    "name": "FlappyMoonbird",
    "symbol": "FMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30659/thumb/200_%282%29.jpg?1696529529"
  },
  {
    "chainId": 1,
    "address": "0xa2e3356610840701bdf5611a53974510ae27e2e1",
    "name": "Wrapped Beacon ETH",
    "symbol": "WBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30061/thumb/wbeth-icon.png?1696528983"
  },
  {
    "chainId": 1,
    "address": "0xb4f8ae8d7d29ac74894cd40ecc24e50f6f146ca6",
    "name": "FriendSpot",
    "symbol": "SPOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31734/thumb/frenspot-logo_%281%29.png?1696530554"
  },
  {
    "chainId": 1,
    "address": "0x43a8f41644a3f07818a43455b975d940ce16a3fa",
    "name": "Fren Nation",
    "symbol": "FREN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30551/thumb/frennation.png?1696529423"
  },
  {
    "chainId": 1,
    "address": "0xbddf903f43dc7d9801f3f0034ba306169074ef8e",
    "name": "Apes Go Bananas",
    "symbol": "AGB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30465/thumb/agb_logo.jpeg?1696529352"
  },
  {
    "chainId": 1,
    "address": "0x62d04c79c1f3a2d7230ffcd3ab01794e1d153239",
    "name": "CryptoPirates",
    "symbol": "OGMF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30442/thumb/cryptopirates.png?1696529329"
  },
  {
    "chainId": 1,
    "address": "0x5c99a6c14f8dc0c2c3655c64a2cef68ff9f771ba",
    "name": "Larry the Llama",
    "symbol": "LARRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32791/thumb/Larry_the_Llama_Logo_200x200.png?1699427698"
  },
  {
    "chainId": 1,
    "address": "0xf79c694605f29ddf3f0eb41319c38672ab6fa89f",
    "name": "Mackerel",
    "symbol": "MACKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33270/thumb/3vQdHnYS_400x400.jpg?1701326383"
  },
  {
    "chainId": 1,
    "address": "0xdd50c053c096cb04a3e3362e2b622529ec5f2e8a",
    "name": "OpenEden TBILL",
    "symbol": "TBILL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30576/thumb/OE_Logo_200x200_Transparent.png?1696529441"
  },
  {
    "chainId": 1,
    "address": "0x01eeffcd9a10266ed00946121df097eed173b43d",
    "name": "XDoge",
    "symbol": "XD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32734/thumb/5365F431-9922-4CC4-BE80-FDB7A86E3764.png?1699219089"
  },
  {
    "chainId": 1,
    "address": "0x7468d234a8db6f1085dbf4e403553bfed41df95c",
    "name": "IO",
    "symbol": "IO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32792/thumb/IMG_8756.jpeg?1699429009"
  },
  {
    "chainId": 1,
    "address": "0x9a0df129e798438a8ad995368bd82baa7eee8913",
    "name": "BEEP Coin",
    "symbol": "BEEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30566/thumb/green.png?1696529437"
  },
  {
    "chainId": 1,
    "address": "0xcae3faa4b6cf660aef18474074949ba0948bc025",
    "name": "Generational Wealth",
    "symbol": "GEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30419/thumb/ethericon200x200.png?1696529307"
  },
  {
    "chainId": 1,
    "address": "0x52e7b23e1faba6d83376725e2ccad75efbd8736e",
    "name": "BURN",
    "symbol": "BURN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30506/thumb/burn.jpg?1696529392"
  },
  {
    "chainId": 1,
    "address": "0xe8438c23157de97bde8bedd2eeabc8e7e44de18a",
    "name": "Lunatics  ETH ",
    "symbol": "LUNAT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30526/thumb/whvem.png?1696529402"
  },
  {
    "chainId": 1,
    "address": "0x2b591e99afe9f32eaa6214f7b7629768c40eeb39",
    "name": "HEX",
    "symbol": "HEX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10103/thumb/HEX-logo.png?1696510130"
  },
  {
    "chainId": 1,
    "address": "0x9352ec6d3e8b54a96b6903a79635c1e2d298b101",
    "name": "Bad Santa",
    "symbol": "BAD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33271/thumb/IMG_20231128_155805_210.png?1701326565"
  },
  {
    "chainId": 1,
    "address": "0x7de91b204c1c737bcee6f000aaa6569cf7061cb7",
    "name": "Robonomics Network",
    "symbol": "XRT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/7024/thumb/Robonomics-Network-logo.png?1696507330"
  },
  {
    "chainId": 1,
    "address": "0x3007083eaa95497cd6b2b809fb97b6a30bdf53d3",
    "name": "PSYOP",
    "symbol": "PSYOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30485/thumb/logo_200.png?1696529371"
  },
  {
    "chainId": 1,
    "address": "0xd1c8fa30fded3e0031dc24c1646d74108b096cc2",
    "name": "Luckyinu",
    "symbol": "LUCKY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34998/thumb/Luckyinu_%281%29.png?1706955605"
  },
  {
    "chainId": 1,
    "address": "0x955d5c14c8d4944da1ea7836bd44d54a8ec35ba1",
    "name": "Refund",
    "symbol": "RFD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30501/thumb/Avatar_%282%29.png?1696529387"
  },
  {
    "chainId": 1,
    "address": "0x21e5c85a5b1f38bddde68307af77e38f747cd530",
    "name": "Doggensnout Skeptic",
    "symbol": "DOGS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35772/thumb/DOG_T200.png?1709838273"
  },
  {
    "chainId": 1,
    "address": "0xb17d69c91135516b0256c67e8bd32cd238b56161",
    "name": "Gravitas",
    "symbol": "GRAVITAS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32995/thumb/200photo_2023-11-12_14-07-05.jpg?1700110430"
  },
  {
    "chainId": 1,
    "address": "0xb5ce43fe2fcffffb2eece95ec413d08def28046f",
    "name": "PepElon",
    "symbol": "PELO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31371/thumb/7gd1S3fr_400x400.jpg?1696530188"
  },
  {
    "chainId": 1,
    "address": "0x514910771af9ca656af840dff83e8264ecf986ca",
    "name": "Chainlink",
    "symbol": "LINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/877/thumb/chainlink-new-logo.png?1696502009"
  },
  {
    "chainId": 1,
    "address": "0x63f7b1b538a78cb699e5399621b3d2e047c40de4",
    "name": "Maxity",
    "symbol": "MAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26933/thumb/MAX_token_200.png?1696525989"
  },
  {
    "chainId": 1,
    "address": "0x6b175474e89094c44da98b954eedeac495271d0f",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 1,
    "address": "0x2048e0d048224381cac3ea06012ced4a6f122d32",
    "name": "BANDS",
    "symbol": "BANDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35693/thumb/Bands_Crypto_Logo_200x200_transparent.png?1709535684"
  },
  {
    "chainId": 1,
    "address": "0x74a85f68cd947d05585f76118e51b2ed1484a838",
    "name": "Diamault",
    "symbol": "DVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32200/thumb/DVT.jpg?1696745951"
  },
  {
    "chainId": 1,
    "address": "0x013062189dc3dcc99e9cee714c513033b8d99e3c",
    "name": "Bware",
    "symbol": "INFRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30546/thumb/infra-token-square.png?1696529418"
  },
  {
    "chainId": 1,
    "address": "0xff8c479134a18918059493243943150776cf8cf2",
    "name": "Renq Finance",
    "symbol": "RENQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29603/thumb/Renq.png?1696528541"
  },
  {
    "chainId": 1,
    "address": "0xdf121180af07cb906d970799627a430d2440c36d",
    "name": "Piggy Bank",
    "symbol": "PIGGY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30594/thumb/piggy_bank_logo_png.png?1696529462"
  },
  {
    "chainId": 1,
    "address": "0xa2cd3d43c775978a96bdbf12d733d5a1ed94fb18",
    "name": "Onyxcoin",
    "symbol": "XCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24210/thumb/onyxlogo.jpg?1696523397"
  },
  {
    "chainId": 1,
    "address": "0x2b896c7f060ae441b76bbe47cddca934ba60f37f",
    "name": "BlackLatexFist",
    "symbol": "BLF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31871/thumb/BLF_Fist_logo_200px.png?1696530683"
  },
  {
    "chainId": 1,
    "address": "0x7acc3f723419fa0c1f789618f798e75c5189c24f",
    "name": "DADA",
    "symbol": "DADA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34999/thumb/Dada.png?1707068797"
  },
  {
    "chainId": 1,
    "address": "0x2ad9addd0d97ec3cdba27f92bf6077893b76ab0b",
    "name": "Planet Token",
    "symbol": "PLANET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30767/thumb/Logo-Planet.png?1696529635"
  },
  {
    "chainId": 1,
    "address": "0xeb31ba344310bc4872c6188ff210d7341a301ea9",
    "name": "GuessOnChain",
    "symbol": "GUESS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32279/thumb/Guess_200px.png?1697180810"
  },
  {
    "chainId": 1,
    "address": "0xc03fbf20a586fa89c2a5f6f941458e1fbc40c661",
    "name": "COMBO",
    "symbol": "COMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4932/thumb/COMBO.jpg?1696505472"
  },
  {
    "chainId": 1,
    "address": "0xf89674f18309a2e97843c6e9b19c07c22caef6d5",
    "name": "cyb3rgam3r420",
    "symbol": "GAMER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32071/thumb/GamerLogo.jpg?1696586649"
  },
  {
    "chainId": 1,
    "address": "0xe0c28a5a2da3920946e8bf821f61f7bea311048b",
    "name": "Stakehouse kETH",
    "symbol": "KETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33656/thumb/kETH.png?1702597766"
  },
  {
    "chainId": 1,
    "address": "0xecf3672a6d2147e2a77f07069fb48d8cf6f6fbf9",
    "name": "Inception mETH",
    "symbol": "INMETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34958/thumb/mETH.png?1706773620"
  },
  {
    "chainId": 1,
    "address": "0x9181f633e9b9f15a32d5e37094f4c93b333e0e92",
    "name": "Inception oETH",
    "symbol": "INOETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34957/thumb/origin-ether-oeth-logo_1.png?1706773605"
  },
  {
    "chainId": 1,
    "address": "0x8365332d4baf69bc24ca2401b90c3853ab9f818e",
    "name": "Wolf of Wall Street",
    "symbol": "WOLF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31461/thumb/Cmc_wolf_2.png?1696530274"
  },
  {
    "chainId": 1,
    "address": "0xba386a4ca26b85fd057ab1ef86e3dc7bdeb5ce70",
    "name": "Jesus Coin",
    "symbol": "JESUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30036/thumb/JESUS_COIN_LOGO.png?1696528959"
  },
  {
    "chainId": 1,
    "address": "0x33bc4e44054fdb21d5a7cea5c03613782d821e45",
    "name": "Xudo",
    "symbol": "XUDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30662/thumb/xudo.png?1696529532"
  },
  {
    "chainId": 1,
    "address": "0xf70ce9ee486106882d3dc43ddbd84e0fa71ac2a5",
    "name": "Ducker",
    "symbol": "DUCKER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30816/thumb/Ducker_Logo_200x200.png?1696529672"
  },
  {
    "chainId": 1,
    "address": "0x15e5d3deff5c3a13adca4f313ff44735881ebd6c",
    "name": "Meta Art Connection",
    "symbol": "MAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32193/thumb/mac.png?1696745471"
  },
  {
    "chainId": 1,
    "address": "0xc4b7af50644c661e270fbb8da770049c9fc0bbe1",
    "name": "Gelios",
    "symbol": "GOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2324/thumb/Gelios_Coingecko.png?1711380304"
  },
  {
    "chainId": 1,
    "address": "0xb2d2e1309db33b38a19ee2a7cd9cb5de39d76663",
    "name": "colR Coin",
    "symbol": "COLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26701/thumb/Untitled_%281%29.png?1696525774"
  },
  {
    "chainId": 1,
    "address": "0xacb47686b92fdde6d233ec6445c1f8d4d0d59c38",
    "name": "Swipe Token",
    "symbol": "SWIPE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30601/thumb/SWIPE_%28200_%C3%97_200_px%29.png?1696529471"
  },
  {
    "chainId": 1,
    "address": "0x3bf954e809620bf2f1fcb667f1c7d2d2e94350d1",
    "name": "Vision City",
    "symbol": "VIZ",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30614/thumb/logo.png?1696529483"
  },
  {
    "chainId": 1,
    "address": "0xda63feff6e6d75cd7a862cd56c625045dcf26e88",
    "name": "ChainMiner",
    "symbol": "CMINER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36901/thumb/1000151140.png?1712685143"
  },
  {
    "chainId": 1,
    "address": "0x7b95ec873268a6bfc6427e7a28e396db9d0ebc65",
    "name": "Aave v3 CRV",
    "symbol": "ACRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32901/thumb/CRV.png?1699802172"
  },
  {
    "chainId": 1,
    "address": "0x9c0241e7538b35454735ae453423daf470a25b3a",
    "name": "UnityBot",
    "symbol": "UNITYBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32864/thumb/UnityBot.png?1699670276"
  },
  {
    "chainId": 1,
    "address": "0x78a0a62fba6fb21a83fe8a3433d44c73a4017a6f",
    "name": "Open Exchange Token",
    "symbol": "OXOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30604/thumb/Logo2.png?1696529473"
  },
  {
    "chainId": 1,
    "address": "0x807534b396919783b7e30383fe57d857bc084338",
    "name": "Test",
    "symbol": "TEST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31188/thumb/IMG_6480.png?1696530015"
  },
  {
    "chainId": 1,
    "address": "0xc7b4c17861357b8abb91f25581e7263e08dcb59c",
    "name": "Aave v3 SNX",
    "symbol": "ASNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32902/thumb/SNX.png?1699802376"
  },
  {
    "chainId": 1,
    "address": "0xee9e7bb7e55bbc86414047b61d65c9c0d91ffbd0",
    "name": "Fracton Protocol",
    "symbol": "FT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26809/thumb/CtH9UlRl_400x400.jpeg?1696525869"
  },
  {
    "chainId": 1,
    "address": "0x2516e7b3f76294e03c42aa4c5b5b4dce9c436fb8",
    "name": "Aave v3 BAL",
    "symbol": "ABAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32903/thumb/BAL_%281%29.png?1699802595"
  },
  {
    "chainId": 1,
    "address": "0x4db57d585fa82ca32d25086ddc069d899f08d455",
    "name": "Enoch",
    "symbol": "ENOCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30810/thumb/20230619_001351.png?1696529667"
  },
  {
    "chainId": 1,
    "address": "0x9609b540e5dedddb147abbf9812ade06b1e61b2c",
    "name": "Steamboat Willie",
    "symbol": "MICKEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35349/thumb/photo_2023-12-30_01.07.19.jpeg?1708340723"
  },
  {
    "chainId": 1,
    "address": "0xd4e245848d6e1220dbe62e155d89fa327e43cb06",
    "name": "Aave v3 FRAX",
    "symbol": "AFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32904/thumb/FRAX.png?1699802848"
  },
  {
    "chainId": 1,
    "address": "0x7fa768e035f956c41d6aeaa3bd857e7e5141cad5",
    "name": "Inception stETH",
    "symbol": "INSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34954/thumb/Group_14.png?1706773546"
  },
  {
    "chainId": 1,
    "address": "0x1ba9843bd4327c6c77011406de5fa8749f7e3479",
    "name": "Aave v3 STG",
    "symbol": "ASTG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32905/thumb/ST.png?1699803503"
  },
  {
    "chainId": 1,
    "address": "0x71aef7b30728b9bb371578f36c5a1f1502a5723e",
    "name": "Aave v3 1INCH",
    "symbol": "A1INCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32906/thumb/1INCH.png?1699804523"
  },
  {
    "chainId": 1,
    "address": "0xfd07fd5ebea6f24888a397997e262179bf494336",
    "name": "Inception osETH",
    "symbol": "INOSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34955/thumb/osETH.png?1706773564"
  },
  {
    "chainId": 1,
    "address": "0xd8e163967fed76806df0097b704ba721b9b37656",
    "name": "Cope Coin",
    "symbol": "COPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29912/thumb/QxzD6QZa_400x400_1_200x200.jpg?1696528840"
  },
  {
    "chainId": 1,
    "address": "0x4ad434b8cdc3aa5ac97932d6bd18b5d313ab0f6f",
    "name": "EverMoon ERC",
    "symbol": "EVERMOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30720/thumb/evermoon.png?1696529591"
  },
  {
    "chainId": 1,
    "address": "0xb5b1b659da79a2507c27aad509f15b4874edc0cc",
    "name": "Dust Protocol",
    "symbol": "DUST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24289/thumb/6388d49d-1f00-448d-92bc-f2db1441bbce.?1696523472"
  },
  {
    "chainId": 1,
    "address": "0x0a44a7ccea34a7563ba1d45a5f757d0b02281124",
    "name": "BlockBlend",
    "symbol": "BBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30680/thumb/mdWl1HlW_400x400.jpg?1696529549"
  },
  {
    "chainId": 1,
    "address": "0x85225ed797fd4128ac45a992c46ea4681a7a15da",
    "name": "Hyperbolic Protocol",
    "symbol": "HYPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30743/thumb/HP_200x200.png?1696529613"
  },
  {
    "chainId": 1,
    "address": "0x369733153e6e08d38f2bc72ae2432e855cfbe221",
    "name": "XALPHA AI",
    "symbol": "XALPHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34503/thumb/XALPHA.png?1705367018"
  },
  {
    "chainId": 1,
    "address": "0x52b43e3aa47612a80114ac5549ff89c6a64d5f22",
    "name": "Harambe Wisdom",
    "symbol": "RAMBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32114/thumb/RAMBE_LOGGG.PNG?1696585568"
  },
  {
    "chainId": 1,
    "address": "0xd8684adc4664bc2a0c78ddc8657dc005e804af15",
    "name": "Copycat DAO",
    "symbol": "CCD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30728/thumb/twitter-logo.jpeg?1696529599"
  },
  {
    "chainId": 1,
    "address": "0x1294f4183763743c7c9519bec51773fb3acd78fd",
    "name": "Fideum",
    "symbol": "FI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33758/thumb/Fideum-token.png?1702955199"
  },
  {
    "chainId": 1,
    "address": "0x2c022e58c5e3ee213f06f975fd8a0d3a6fe9ca1c",
    "name": "FINU",
    "symbol": "FINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29309/thumb/FINU_200x200.png?1696528260"
  },
  {
    "chainId": 1,
    "address": "0x259ce0cb3581995d40cbb03fd4badeaaba1edaff",
    "name": "SphereSXS",
    "symbol": "SXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27924/thumb/IMG_0354.jpeg?1703176663"
  },
  {
    "chainId": 1,
    "address": "0x2b46578b7f06f2b373ad0e0c9b28f800dcc80bf3",
    "name": "Echo Bot",
    "symbol": "ECHO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31266/thumb/hd1qyGUA_200x200.jpg?1696530090"
  },
  {
    "chainId": 1,
    "address": "0xda8263d8ce3f726233f8b5585bcb86a3120a58b6",
    "name": "DogeClub",
    "symbol": "DOGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30733/thumb/200*200.png?1696529603"
  },
  {
    "chainId": 1,
    "address": "0x208bcf9cd1d694d1d3c630362e517940e3724d80",
    "name": "ORACLE",
    "symbol": "ORACLE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32234/thumb/qlasify_%281%29.jpg?1696936966"
  },
  {
    "chainId": 1,
    "address": "0x445bd590a01fe6709d4f13a8f579c1e4846921db",
    "name": "DUMMY",
    "symbol": "DUMMY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30650/thumb/a3miYrC7_400x400-removebg-preview%281%29.png?1696529520"
  },
  {
    "chainId": 1,
    "address": "0x3f17f64f682019599ba51638f74e4b6c127fe725",
    "name": "ReviveEth",
    "symbol": "REVIVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30770/thumb/IMG_1662.png?1696529638"
  },
  {
    "chainId": 1,
    "address": "0x5395df198fcbc9c13bec506004c9a8b6460a7712",
    "name": "Bubu",
    "symbol": "BUBU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30835/thumb/Bubu200x200.png?1696529693"
  },
  {
    "chainId": 1,
    "address": "0xf56842af3b56fd72d17cb103f92d027bba912e89",
    "name": "Bamboo DeFi",
    "symbol": "BAMBOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13856/thumb/LOGO_BAMBOO_PNG.png?1696513602"
  },
  {
    "chainId": 1,
    "address": "0xf043809056ac2492c4aae6317705f751082c0db1",
    "name": "ELONGATE",
    "symbol": "ELONGATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32236/thumb/ElonGate.png?1696937153"
  },
  {
    "chainId": 1,
    "address": "0x64aa3364f17a4d01c6f1751fd97c2bd3d7e7f1d5",
    "name": "Olympus",
    "symbol": "OHM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14483/thumb/token_OHM_%281%29.png?1696514169"
  },
  {
    "chainId": 1,
    "address": "0x68a47fe1cf42eba4a030a10cd4d6a1031ca3ca0a",
    "name": "Tectum",
    "symbol": "TET",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29575/thumb/Icon_TET.png?1696528514"
  },
  {
    "chainId": 1,
    "address": "0xeaa63125dd63f10874f99cdbbb18410e7fc79dd3",
    "name": "Hemule",
    "symbol": "HEMULE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33798/thumb/photo_2023-12-18_04-47-31.jpg?1702990624"
  },
  {
    "chainId": 1,
    "address": "0xf4d861575ecc9493420a3f5a14f85b13f0b50eb3",
    "name": "Fractal",
    "symbol": "FCL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14096/thumb/FractalID_SM_Profile_Image.jpg?1701799215"
  },
  {
    "chainId": 1,
    "address": "0xe9b7b5d5e8d2bcc78884f9f9099bfa42a9e5c1a5",
    "name": "Zenland",
    "symbol": "ZENF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29477/thumb/tokenicon200px.png?1696528421"
  },
  {
    "chainId": 1,
    "address": "0xf941d3aabf2ee0f5589e68ba6047b8329592b366",
    "name": "Good Dog",
    "symbol": "HEEL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27138/thumb/HEEL_New_Logo.png?1696526189"
  },
  {
    "chainId": 1,
    "address": "0x176bc22e1855cd5cf5a840081c6c5b92b55e2210",
    "name": "Gambex",
    "symbol": "GBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32745/thumb/gbe_cg.png?1699262150"
  },
  {
    "chainId": 1,
    "address": "0xcf9560b9e952b195d408be966e4f6cf4ab8206e5",
    "name": "Doctor Evil",
    "symbol": "EVIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31240/thumb/EvilLogo200x200.png?1696530066"
  },
  {
    "chainId": 1,
    "address": "0x017e9db34fc69af0dc7c7b4b33511226971cddc7",
    "name": "On Chain Dynamics",
    "symbol": "OCD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31639/thumb/Screenshot_2023-09-01_010740.png?1696530455"
  },
  {
    "chainId": 1,
    "address": "0x545bd6c032efdde65a377a6719def2796c8e0f2e",
    "name": "Aave v3 ENS",
    "symbol": "AENS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32907/thumb/ens.png?1699816992"
  },
  {
    "chainId": 1,
    "address": "0x5b502e3796385e1e9755d7043b9c945c3accec9c",
    "name": "Aave v3 KNC",
    "symbol": "AKNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32908/thumb/kyber.png?1699817281"
  },
  {
    "chainId": 1,
    "address": "0xe8e531aa894969759d0d4b207a972a3a97d287f0",
    "name": "Watcher AI",
    "symbol": "WAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36843/thumb/photo_6073264381849879042_c_%281%29.jpg?1712582416"
  },
  {
    "chainId": 1,
    "address": "0xad038eb671c44b853887a7e32528fab35dc5d710",
    "name": "DOLA Borrowing Right",
    "symbol": "DBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28530/thumb/DBR.png?1696527523"
  },
  {
    "chainId": 1,
    "address": "0x0e9cc0f7e550bd43bd2af2214563c47699f96479",
    "name": "UnleashClub",
    "symbol": "UNLEASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30804/thumb/UnleashClub.png?1696529662"
  },
  {
    "chainId": 1,
    "address": "0xab97bf5fb097e503bba2c86b7c56c0059ac0dc06",
    "name": "Chanalog",
    "symbol": "CHAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32997/thumb/chanalog_token_logo_200.png?1700111084"
  },
  {
    "chainId": 1,
    "address": "0xbba39fd2935d5769116ce38d46a71bde9cf03099",
    "name": "Choise com",
    "symbol": "CHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25935/thumb/cho_%282%29.png?1696525014"
  },
  {
    "chainId": 1,
    "address": "0x7dedbce5a2e31e4c75f87fea60bf796c17718715",
    "name": "Penpie",
    "symbol": "PNP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30760/thumb/PNP_Token.png?1696529629"
  },
  {
    "chainId": 1,
    "address": "0x5eca7b975e34567d9460fa613013a7a6993ad185",
    "name": "Blacksmith Token",
    "symbol": "BS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30806/thumb/roGLhC57_400x400.jpg?1696529664"
  },
  {
    "chainId": 1,
    "address": "0xb76cf92076adbf1d9c39294fa8e7a67579fde357",
    "name": "Aave v3 RPL",
    "symbol": "ARPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32909/thumb/RPL.png?1699817497"
  },
  {
    "chainId": 1,
    "address": "0xc19b6a4ac7c7cc24459f08984bbd09664af17bd1",
    "name": "SENSO",
    "symbol": "SENSO",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/11064/thumb/senso.png?1696511006"
  },
  {
    "chainId": 1,
    "address": "0x7448c7456a97769f6cd04f1e83a4a23ccdc46abd",
    "name": "Maverick Protocol",
    "symbol": "MAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30850/thumb/MAV_Logo.png?1696529701"
  },
  {
    "chainId": 1,
    "address": "0x6230f552a1c825d02e1140ccc0d3f5eeec81ca84",
    "name": "FusionBot",
    "symbol": "FUSION",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31125/thumb/logo.png?1696529955"
  },
  {
    "chainId": 1,
    "address": "0x38cf11283de05cf1823b7804bc75068bd6296957",
    "name": "MoonBot",
    "symbol": "MBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31198/thumb/IMAGE_2023-07-27_20_46_47.jpg?1696530025"
  },
  {
    "chainId": 1,
    "address": "0x819c1a1568934ee59d9f3c8b9640908556c44140",
    "name": "Hobbes  OLD ",
    "symbol": "HOBBES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29541/thumb/Logo200x200.png?1696528483"
  },
  {
    "chainId": 1,
    "address": "0x41d321e0365a42058a465dc5bf98242a92c2e9f3",
    "name": "XToolsAI",
    "symbol": "XTAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36844/thumb/IMAGE_2024-03-27_02_04_19.jpg?1712582480"
  },
  {
    "chainId": 1,
    "address": "0xea3983fc6d0fbbc41fb6f6091f68f3e08894dc06",
    "name": "Unido",
    "symbol": "UDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14176/thumb/unido.jpg?1696513894"
  },
  {
    "chainId": 1,
    "address": "0xbb4f3ad7a2cf75d8effc4f6d7bd21d95f06165ca",
    "name": "Sheesh",
    "symbol": "SHS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30858/thumb/1111.png?1696529705"
  },
  {
    "chainId": 1,
    "address": "0xf8c76dbea329ec4fa987afc514f805b21b249d79",
    "name": "L",
    "symbol": "L",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30846/thumb/lltokenimagelogo.png?1696529696"
  },
  {
    "chainId": 1,
    "address": "0x198065e69a86cb8a9154b333aad8efe7a3c256f8",
    "name": "Koyo",
    "symbol": "KOY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29484/thumb/koyo200x200.png?1696528428"
  },
  {
    "chainId": 1,
    "address": "0x63e28547f0997e8ab265b24b7e5df302f3498ff3",
    "name": "FP  WorldOfWomen",
    "symbol": "UWOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34111/thumb/WOW.png?1703996347"
  },
  {
    "chainId": 1,
    "address": "0xd567b5f02b9073ad3a982a099a23bf019ff11d1c",
    "name": "Gamestarter",
    "symbol": "GAME",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/17604/thumb/gpMi14-r_400x400.jpg?1696517136"
  },
  {
    "chainId": 1,
    "address": "0xbc8e35221904f61b4200ca44a08e4dac387ac83a",
    "name": "Fair BERC20",
    "symbol": "BERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30860/thumb/BERClogo3.png_alt_media_token_3c04fa07-9756-4037-8950-e30295030131.png?1696529707"
  },
  {
    "chainId": 1,
    "address": "0x113c65707c530502fef959308197353f6df97867",
    "name": "The Joker Coin",
    "symbol": "JOKER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36845/thumb/1000020014.png?1712582992"
  },
  {
    "chainId": 1,
    "address": "0x5510d26c504b21ef22ca85b7125390bc23ca50e7",
    "name": "Homer",
    "symbol": "SIMPSON20",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30882/thumb/1111.png?1696529729"
  },
  {
    "chainId": 1,
    "address": "0x32eca61dc25d0742f238ce523e66b68867625dad",
    "name": "0xFair",
    "symbol": "FAIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35725/thumb/fair-200.png?1709634952"
  },
  {
    "chainId": 1,
    "address": "0x6aa3ecec75ceb388d2e929814ead4fc4cd0648fc",
    "name": "Reversal",
    "symbol": "RVSL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35453/thumb/Reversal_Logo.png?1708665533"
  },
  {
    "chainId": 1,
    "address": "0x8189afbe7b0e81dae735ef027cd31371b3974feb",
    "name": "Bean",
    "symbol": "BEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30884/thumb/coin_logo.png?1696529731"
  },
  {
    "chainId": 1,
    "address": "0x2a5fa016ffb20c70e2ef36058c08547f344677aa",
    "name": "Vitalik Smart Gas",
    "symbol": "VSG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35275/thumb/JW9w9bvc_400x400.jpg?1708053854"
  },
  {
    "chainId": 1,
    "address": "0x089453742936dd35134383aee9d78bee63a69b01",
    "name": "Gold",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31894/thumb/GOLD.png?1696530705"
  },
  {
    "chainId": 1,
    "address": "0xa203eb78fee91c8459c6d4ef3a899d8724ee5b35",
    "name": "Liquid Mercury",
    "symbol": "MERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29297/thumb/cda56b6a-146c-4348-b4ff-5b87980c0f83.jpg?1696528249"
  },
  {
    "chainId": 1,
    "address": "0x5526f0cab9f260023e94cf01e353e70d2f45ed81",
    "name": "CockBoxing",
    "symbol": "COCKS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32699/thumb/IMG_20231031_171642_092.png?1698984813"
  },
  {
    "chainId": 1,
    "address": "0x0763fdccf1ae541a5961815c0872a8c5bc6de4d7",
    "name": "SUKU",
    "symbol": "SUKU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11969/thumb/suku-200.png?1696511828"
  },
  {
    "chainId": 1,
    "address": "0x36948a6809be82d3ba8de9f2be626101e2c9e473",
    "name": "BoltBot",
    "symbol": "BOLT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30877/thumb/boltbot.jpg?1696529725"
  },
  {
    "chainId": 1,
    "address": "0x4ebe70cb942d5af0a18b9126762637e7098ff5fd",
    "name": "G Revolution",
    "symbol": "G",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30869/thumb/G_logo.png?1696529716"
  },
  {
    "chainId": 1,
    "address": "0x32b86b99441480a7e5bd3a26c124ec2373e3f015",
    "name": "Bad Idea AI",
    "symbol": "BAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30871/thumb/Bad_idea.jpeg?1696529718"
  },
  {
    "chainId": 1,
    "address": "0xb9098d3669a78e9afe8b94a97290407400d9da31",
    "name": "Protectorate Protocol",
    "symbol": "PRTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30901/thumb/IMG_1778.png?1696529746"
  },
  {
    "chainId": 1,
    "address": "0x6b3595068778dd592e39a122f4f5a5cf09c90fe2",
    "name": "Sushi",
    "symbol": "SUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12271/thumb/512x512_Logo_no_chop.png?1696512101"
  },
  {
    "chainId": 1,
    "address": "0xf2ec4a773ef90c58d98ea734c0ebdb538519b988",
    "name": "Doge 2 0",
    "symbol": "DOGE20",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30902/thumb/2DOGE_200.jpg?1696529747"
  },
  {
    "chainId": 1,
    "address": "0x5a035e3f1551a15230d0cde3357fb1bf89369261",
    "name": "Wooonen",
    "symbol": "WOOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30909/thumb/SGpzsmu.jpg?1696529754"
  },
  {
    "chainId": 1,
    "address": "0xa62894d5196bc44e4c3978400ad07e7b30352372",
    "name": "X",
    "symbol": "X",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31144/thumb/X.png?1696529972"
  },
  {
    "chainId": 1,
    "address": "0xf482d79ae6e8725c199213fc909d6bc30df62815",
    "name": "Recycle Impact World Association",
    "symbol": "RIWA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32158/thumb/RIWA.jpg?1696601500"
  },
  {
    "chainId": 1,
    "address": "0x8e1308f4808788767a864eee9a4927e38c790352",
    "name": "Physics",
    "symbol": "PHYSICS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32752/thumb/Physics_Logo_200x200_PNG.png?1699265054"
  },
  {
    "chainId": 1,
    "address": "0xed35af169af46a02ee13b9d79eb57d6d68c1749e",
    "name": "ECOMI",
    "symbol": "OMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4428/thumb/ECOMI.png?1696505023"
  },
  {
    "chainId": 1,
    "address": "0x48c87cdacb6bb6bf6e5cd85d8ee5c847084c7410",
    "name": "Hamsters",
    "symbol": "HAMS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31055/thumb/hams_logo_s.png?1696529890"
  },
  {
    "chainId": 1,
    "address": "0x9b44793a0177c84dd01ad81137db696531902871",
    "name": "PVP",
    "symbol": "PVP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31462/thumb/PVPLOGO.JPEG?1696530275"
  },
  {
    "chainId": 1,
    "address": "0x7ba5273d53d8a964b0a338e25def0c082f5c29d3",
    "name": "WEN",
    "symbol": "WEN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31835/thumb/Wen_small.png?1696530649"
  },
  {
    "chainId": 1,
    "address": "0xf257a2783f6633a149b5966e32432b5bb3462c96",
    "name": "RagingElonMarsCoin",
    "symbol": "DOGECOIN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31463/thumb/JgNkdCWn_400x400.jpg?1696530276"
  },
  {
    "chainId": 1,
    "address": "0x3442fbf264b6d723e01775a710850dcef6e6847c",
    "name": "VINU Network",
    "symbol": "VNN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32196/thumb/Vinu_Network.jpg?1696745619"
  },
  {
    "chainId": 1,
    "address": "0x0981d9774a59a703db85f5eaa23672283ea31106",
    "name": "Pepe Inu",
    "symbol": "PEPINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30941/thumb/Pepe_Inu.png?1696529780"
  },
  {
    "chainId": 1,
    "address": "0xffffffff2ba8f66d4e51811c5190992176930278",
    "name": "Furucombo",
    "symbol": "COMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13629/thumb/COMBO_token_ol.png?1696513377"
  },
  {
    "chainId": 1,
    "address": "0x70be04312f5f66d03708f0a1d6353b3e0f80ddbb",
    "name": "BetBot",
    "symbol": "BBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31076/thumb/BetBot.png?1696529910"
  },
  {
    "chainId": 1,
    "address": "0xe28b3b32b6c345a34ff64674606124dd5aceca30",
    "name": "Injective",
    "symbol": "INJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12882/thumb/Secondary_Symbol.png?1696512670"
  },
  {
    "chainId": 1,
    "address": "0x45f93404ae1e4f0411a7f42bc6a5dc395792738d",
    "name": "DGEN",
    "symbol": "DGEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31294/thumb/dgen-new-200-200.png?1696530114"
  },
  {
    "chainId": 1,
    "address": "0xf4aaa9428a881a5c054d0ed041f5749a336c9ab5",
    "name": "Mine AI",
    "symbol": "MAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35773/thumb/Mine_AI.png?1709793186"
  },
  {
    "chainId": 1,
    "address": "0xd0b3a986fff305854a7238a8e099cce1ced01a3d",
    "name": "Nova",
    "symbol": "NOVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30943/thumb/IMG_20230709_210803_010.jpg?1696529782"
  },
  {
    "chainId": 1,
    "address": "0x14da230d6726c50f759bc1838717f8ce6373509c",
    "name": "Kambria",
    "symbol": "KAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4784/thumb/kambria.png?1696505340"
  },
  {
    "chainId": 1,
    "address": "0x111111111117dc0aa78b770fa6a738034120c302",
    "name": "1inch",
    "symbol": "1INCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13469/thumb/1inch-token.png?1696513230"
  },
  {
    "chainId": 1,
    "address": "0x1a88df1cfe15af22b3c4c783d4e6f7f9e0c1885d",
    "name": "Aave stkGHO",
    "symbol": "STKGHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34849/thumb/stkgho.png?1706260099"
  },
  {
    "chainId": 1,
    "address": "0xf31826269ac7f452b1274cc884812f426c18ddca",
    "name": "MILK Coin",
    "symbol": "MILK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35240/thumb/MILK_logo_Background_Removed.png?1707969987"
  },
  {
    "chainId": 1,
    "address": "0xb8112a01744fe88bc8672cd977378169e33eb9a5",
    "name": "Ordinals Inscription Bot",
    "symbol": "OIB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33456/thumb/logov2_%281%29_%281%29.png?1701923425"
  },
  {
    "chainId": 1,
    "address": "0x6c059413686565d5ad6cce6eed7742c42dbc44ca",
    "name": "Laelaps",
    "symbol": "LAELAPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30986/thumb/laelaps.png?1696529825"
  },
  {
    "chainId": 1,
    "address": "0xa0d3707c569ff8c87fa923d3823ec5d81c98be78",
    "name": "Instadapp ETH v2",
    "symbol": "IETHV2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29704/thumb/iETH.png?1696528636"
  },
  {
    "chainId": 1,
    "address": "0xde67d97b8770dc98c746a3fc0093c538666eb493",
    "name": "Bitrock",
    "symbol": "BROCK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30987/thumb/20230713_155626.png?1696529826"
  },
  {
    "chainId": 1,
    "address": "0xd2c869382c7ac9f87ff73548d029d67c0f9dee31",
    "name": "Wagie Bot",
    "symbol": "WAGIEBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30954/thumb/Wagies3%281%29.png?1696529793"
  },
  {
    "chainId": 1,
    "address": "0x66861d5f0fbfb7b2711712fef2172c560d08d0ab",
    "name": "A Fund Baby",
    "symbol": "FUND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30955/thumb/fund_baby.jpg?1696529794"
  },
  {
    "chainId": 1,
    "address": "0x3127294f1fd3c097ef31e54301069346b29d0209",
    "name": "Blast Inu",
    "symbol": "BLAST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33226/thumb/photo_2023-11-26_22-31-53-removebg-preview_%281%29.png?1701118741"
  },
  {
    "chainId": 1,
    "address": "0xa34ee6108fe427f91edce0d6520d9fec0e64f67b",
    "name": "Pepe Le Pew Coin",
    "symbol": "PLPC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30876/thumb/plpc.png?1696529724"
  },
  {
    "chainId": 1,
    "address": "0xb59490ab09a0f526cc7305822ac65f2ab12f9723",
    "name": "Litentry",
    "symbol": "LIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13825/thumb/logo_200x200.png?1696513568"
  },
  {
    "chainId": 1,
    "address": "0xcf0c122c6b73ff809c693db761e7baebe62b6a2e",
    "name": "FLOKI",
    "symbol": "FLOKI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16746/thumb/PNG_image.png?1696516318"
  },
  {
    "chainId": 1,
    "address": "0x3c3a81e81dc49a522a592e7622a7e711c06bf354",
    "name": "Mantle",
    "symbol": "MNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30980/thumb/token-logo.png?1696529819"
  },
  {
    "chainId": 1,
    "address": "0xf6afc05fccea5a53f22a3e39ffee861e016bd9a0",
    "name": "LandWolf",
    "symbol": "WOLF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30982/thumb/landwolf.jpg?1696529821"
  },
  {
    "chainId": 1,
    "address": "0xf07ebf00aa847bbb0057b92cb469747e2c7c8a4f",
    "name": "Echoes",
    "symbol": "ECHOES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30994/thumb/logosmallcircle.png?1696529832"
  },
  {
    "chainId": 1,
    "address": "0xf79c22713f3912be20a324034a91b11e2958a37e",
    "name": "Plan Blui",
    "symbol": "PBLUI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33227/thumb/TqkFW5A2_400x400.png?1701127038"
  },
  {
    "chainId": 1,
    "address": "0xc4e30d504fe3b18423d82a34e7ebe529a56f89c2",
    "name": "Wojak 2 0 Coin",
    "symbol": "WOJAK20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30979/thumb/Wojak_2.0_Logo.png?1696529818"
  },
  {
    "chainId": 1,
    "address": "0xd38bb40815d2b0c2d2c866e0c72c5728ffc76dd9",
    "name": "Symbiosis",
    "symbol": "SIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20805/thumb/SymbiosisFinance_logo-150x150.jpeg?1696520198"
  },
  {
    "chainId": 1,
    "address": "0x3feb4fea5132695542f8ede5076ac43296d17c6d",
    "name": "Bitcoin 2 0",
    "symbol": "BTC20",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30984/thumb/btc2-200x200.png?1696529823"
  },
  {
    "chainId": 1,
    "address": "0x9ce84f6a69986a83d92c324df10bc8e64771030f",
    "name": "CHEX Token",
    "symbol": "CHEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10349/thumb/logo-dark-200.png?1701121978"
  },
  {
    "chainId": 1,
    "address": "0x56978e609f2cab06f77c5c8fd75166fcd8f09bd8",
    "name": "GenieBot",
    "symbol": "GENIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30874/thumb/IMG_0334.jpeg?1696529722"
  },
  {
    "chainId": 1,
    "address": "0x46b2deae6eff3011008ea27ea36b7c27255ddfa9",
    "name": "dYdX",
    "symbol": "WETHDYDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32595/thumb/dydx.png?1698673699"
  },
  {
    "chainId": 1,
    "address": "0x18cc2ba8995c6307e355726244adb023cf00522f",
    "name": "Monke",
    "symbol": "MONKE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30192/thumb/MonkeCG.png?1698675883"
  },
  {
    "chainId": 1,
    "address": "0x99a01a4d6a4d621094983050d9a2f10b2912e53d",
    "name": "VirtuSwap",
    "symbol": "VRSW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30629/thumb/VirtuSwap_Logo_Red_200x200.png?1696529502"
  },
  {
    "chainId": 1,
    "address": "0x99fe3b1391503a1bc1788051347a1324bff41452",
    "name": "SX Network  OLD ",
    "symbol": "SX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13779/thumb/sx.png?1696513520"
  },
  {
    "chainId": 1,
    "address": "0x83031984b45553070a088273f341bff7fb4f2f46",
    "name": "hiBAYC",
    "symbol": "HIBAYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26623/thumb/hibayc.png?1696525696"
  },
  {
    "chainId": 1,
    "address": "0xce9de5365739b1bed5c8546867aee4209fbb8538",
    "name": "Thug Life",
    "symbol": "THUG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31036/thumb/IMG_20230717_172433_167.jpg?1696529871"
  },
  {
    "chainId": 1,
    "address": "0x265f542c1e78068f13d87c6fe0df54f3e9562a48",
    "name": "Proof Of Pepe",
    "symbol": "POP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31000/thumb/ebEOnrjk_400x400.jpg?1696529838"
  },
  {
    "chainId": 1,
    "address": "0xbe9f61555f50dd6167f2772e9cf7519790d96624",
    "name": "SX Network",
    "symbol": "SX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34934/thumb/sx-ntework.jpeg?1706699134"
  },
  {
    "chainId": 1,
    "address": "0x84412819ae69b10250d0d54d58f454018f1c8a42",
    "name": "Scarab Tools",
    "symbol": "DUNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30706/thumb/scarab-cg.png?1696529577"
  },
  {
    "chainId": 1,
    "address": "0xf31698ddad0d11160fe85c500397a470cd3d492e",
    "name": "Wexo",
    "symbol": "WEXO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33801/thumb/wexo_token_200x200.png?1702991908"
  },
  {
    "chainId": 1,
    "address": "0x60eb57d085c59932d5faa6c6026268a4386927d0",
    "name": "LOCG",
    "symbol": "LOCG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15175/thumb/LOC_GAME.png?1696514830"
  },
  {
    "chainId": 1,
    "address": "0x4c0f743928ca8fa7fb24ad89669c8a7838f34917",
    "name": "STACKER AI",
    "symbol": "STACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32865/thumb/IMG_1751.png?1699670330"
  },
  {
    "chainId": 1,
    "address": "0x313cae7ad4454aac7b208c1f089da2b0e5825e46",
    "name": "RepubliK",
    "symbol": "RPK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32950/thumb/RPK_Token_Logo_2x.png?1699935349"
  },
  {
    "chainId": 1,
    "address": "0x14fee680690900ba0cccfc76ad70fd1b95d10e16",
    "name": "PAAL AI",
    "symbol": "PAAL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30815/thumb/PaalX.jpg?1696529671"
  },
  {
    "chainId": 1,
    "address": "0x267eb2a9a13dc304a9deff4277abe850d0852c5f",
    "name": "Trace AI",
    "symbol": "TAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31058/thumb/200x200.png?1696529892"
  },
  {
    "chainId": 1,
    "address": "0x0a266778c113a8f0d3d7aeacd12b74a7da5037a0",
    "name": "OLYMPUS",
    "symbol": "OLAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33251/thumb/200cg.png?1701237177"
  },
  {
    "chainId": 1,
    "address": "0x128ad1ad707c3b36e6f2ac9739f9df7516fdb592",
    "name": "alfa society",
    "symbol": "ALFA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31017/thumb/token_logo_listing.png?1696529853"
  },
  {
    "chainId": 1,
    "address": "0x4104b135dbc9609fc1a9490e61369036497660c8",
    "name": "Spectra",
    "symbol": "APW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15597/thumb/spectra.png?1696515232"
  },
  {
    "chainId": 1,
    "address": "0x0c08638473cafbca3beb113616a1871f4bfad4f9",
    "name": "ZooCoin",
    "symbol": "ZOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31230/thumb/ZOO.png?1696530056"
  },
  {
    "chainId": 1,
    "address": "0x95a4492f028aa1fd432ea71146b433e7b4446611",
    "name": "APY Finance",
    "symbol": "APY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13041/thumb/1*AvkD-OLocausbxqUzezZ0A.png?1696512828"
  },
  {
    "chainId": 1,
    "address": "0x163f8c2467924be0ae7b5347228cabf260318753",
    "name": "Worldcoin",
    "symbol": "WLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31069/thumb/worldcoin.jpeg?1696529903"
  },
  {
    "chainId": 1,
    "address": "0x553c7f9c780316fc1d34b8e14ac2465ab22a090b",
    "name": "Aktionariat RealUnit Schweiz AG Tokeniz",
    "symbol": "REALU",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33331/thumb/REALU.png?1701517686"
  },
  {
    "chainId": 1,
    "address": "0x5aef5bba19e6a1644805bd4f5c93c8557b87c62c",
    "name": "DeepFakeAI",
    "symbol": "FAKEAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30633/thumb/Safeimagekit-resized-img_%286%29.png?1696529506"
  },
  {
    "chainId": 1,
    "address": "0x97d4f49eeb0e2c96d5ebaa71ab8418e563ecd9fd",
    "name": "Liquid Staking Derivative",
    "symbol": "LSD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29188/thumb/LSD.jpg?1696528146"
  },
  {
    "chainId": 1,
    "address": "0xb459f7204a8ac84f9e7758d6d839ebd01670e35c",
    "name": "Lotty",
    "symbol": "LOTTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31018/thumb/download.png?1696529854"
  },
  {
    "chainId": 1,
    "address": "0xfc116ea24f002f600e363bdce4b91715fe5e0392",
    "name": "L3T H1M C00K",
    "symbol": "DOUGH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35850/thumb/Icon-200x200.png?1709895424"
  },
  {
    "chainId": 1,
    "address": "0xc5f0f7b66764f6ec8c8dff7ba683102295e16409",
    "name": "First Digital USD",
    "symbol": "FDUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31079/thumb/firstfigital.jpeg?1696529912"
  },
  {
    "chainId": 1,
    "address": "0x69e37422cb87d963367f73a119c8ce9a4d529b72",
    "name": "Advantis",
    "symbol": "ADVT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28549/thumb/Advantis_Logo_%28ROUND%29.png?1709178371"
  },
  {
    "chainId": 1,
    "address": "0xaa8330fb2b4d5d07abfe7a72262752a8505c6b37",
    "name": "Polkacity",
    "symbol": "POLC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14066/thumb/vykih1Nq_400x400.png?1696513790"
  },
  {
    "chainId": 1,
    "address": "0x0000206329b97db379d5e1bf586bbdb969c63274",
    "name": "USDA",
    "symbol": "USDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34510/thumb/agUSD-coingecko.png?1705288392"
  },
  {
    "chainId": 1,
    "address": "0xd2bdaaf2b9cc6981fd273dcb7c04023bfbe0a7fe",
    "name": "Aviator",
    "symbol": "AVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31024/thumb/avi-200x200png.png?1711119363"
  },
  {
    "chainId": 1,
    "address": "0x1f9840a85d5af5bf1d1762f925bdaddc4201f984",
    "name": "Uniswap",
    "symbol": "UNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12504/thumb/uni.jpg?1696512319"
  },
  {
    "chainId": 1,
    "address": "0x8c130499d33097d4d000d3332e1672f75b431543",
    "name": "Hoppy Token",
    "symbol": "HOPPY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31027/thumb/IMG_1349.jpeg?1696529863"
  },
  {
    "chainId": 1,
    "address": "0xfcdb9e987f9159dab2f507007d5e3d10c510aa70",
    "name": "0x1 tools  AI Multi tool",
    "symbol": "0X1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31026/thumb/a392bc3a-9e36-4e6a-bdfe-254b7bb36918.jpeg?1696529862"
  },
  {
    "chainId": 1,
    "address": "0x3d806324b6df5af3c1a81acba14a8a62fe6d643f",
    "name": "BarbieCrashBandicootRFK88",
    "symbol": "SOLANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31247/thumb/logo_200px.jpg?1696530072"
  },
  {
    "chainId": 1,
    "address": "0xb446566d6d644249d5d82aab5fea8a5b7da3f691",
    "name": "Aktionariat TBo c o Comon Accelerator H",
    "symbol": "TBOS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33332/thumb/TBOS.png?1701518908"
  },
  {
    "chainId": 1,
    "address": "0x17f8e6694080c07d7414c918dd9b9c32a2981ea1",
    "name": "Ascend",
    "symbol": "ASC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31110/thumb/acdw2000.png?1696529940"
  },
  {
    "chainId": 1,
    "address": "0x8747a3114ef7f0eebd3eb337f745e31dbf81a952",
    "name": "Aktionariat ServiceHunter AG Tokenized ",
    "symbol": "DQTS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33335/thumb/DQTS.png?1701520936"
  },
  {
    "chainId": 1,
    "address": "0x33ba0f96980314cf962bd6224b5c3befa5946202",
    "name": "nsurance",
    "symbol": "N",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32392/thumb/n-logo-200x200.png?1698053006"
  },
  {
    "chainId": 1,
    "address": "0x3f2d4708f75de6fb60b687fed326697634774deb",
    "name": "NeoBot",
    "symbol": "NEOBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31033/thumb/20230718_190506_0000.png?1696529869"
  },
  {
    "chainId": 1,
    "address": "0x4e51a6b3cc6d5ae69a0d44db9de846aeb5a582dd",
    "name": "Gyoza",
    "symbol": "GYOZA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31083/thumb/logorecized_new.png?1696529916"
  },
  {
    "chainId": 1,
    "address": "0xf49311af05a4ffb1dbf33d61e9b2d4f0a7d4a71c",
    "name": "CompanionBot",
    "symbol": "CBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31302/thumb/IMG_20230812_173007_931.jpg?1696530122"
  },
  {
    "chainId": 1,
    "address": "0x112b08621e27e10773ec95d250604a041f36c582",
    "name": "Wrapped Kaspa",
    "symbol": "KAS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31037/thumb/WKAS-256-black-ETH-3.png?1696529872"
  },
  {
    "chainId": 1,
    "address": "0xbe579b687fd9cd4fd6cb85abc8fbe069ab0be2cb",
    "name": "Raw Chicken Experiment",
    "symbol": "RCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35852/thumb/Chicken.png?1709895570"
  },
  {
    "chainId": 1,
    "address": "0x9754d5e5ea788c62f11d34a818aeae927aeac84c",
    "name": "YoloNolo",
    "symbol": "NOLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31067/thumb/Nolo_200x200_logo.png?1696529901"
  },
  {
    "chainId": 1,
    "address": "0x56528c1df17fd5451451eb6efde297758bc8f9a1",
    "name": "Aktionariat Alan Frei Company Tokenized",
    "symbol": "AFS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33336/thumb/AFS.png?1701521247"
  },
  {
    "chainId": 1,
    "address": "0xa64efd5fe826f62e310a951332b519e1e3871489",
    "name": "BetaCarbon",
    "symbol": "BCAU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31040/thumb/Safeimagekit-resized-img_%281%29.png?1696529875"
  },
  {
    "chainId": 1,
    "address": "0x848896470d989f30503d8f883c331f63b73b66ea",
    "name": "Medicle",
    "symbol": "MDI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31082/thumb/Medicle%28MDI%29_NEW_.png?1696529915"
  },
  {
    "chainId": 1,
    "address": "0xd17c18979a45c0f3604331831c409f7bff64d3c1",
    "name": "Farmer Friends",
    "symbol": "FRENS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31048/thumb/FRENS_logo.png?1696529883"
  },
  {
    "chainId": 1,
    "address": "0x7bd44cf5c0566aab26150a0cd5c3d20c5535686f",
    "name": "Evil Pepe",
    "symbol": "EVILPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31201/thumb/IMG_20230803_120336_441.png?1696530028"
  },
  {
    "chainId": 1,
    "address": "0x3dbb00c9be5a327e25caf4f650844c5dba81e34b",
    "name": "StaFi Staked MATIC",
    "symbol": "RMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29610/thumb/rMATIC.png?1696528546"
  },
  {
    "chainId": 1,
    "address": "0x7c3ed93b03e67c1fa240395b28e38c4386321b58",
    "name": "Wally Bot",
    "symbol": "WALLY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31046/thumb/IMG_4215.jpeg?1696529881"
  },
  {
    "chainId": 1,
    "address": "0x67d85a291fcdc862a78812a3c26d55e28ffb2701",
    "name": "Asymetrix",
    "symbol": "ASX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29984/thumb/Asymetrix_Logo_Only_Square_w200.png?1696528909"
  },
  {
    "chainId": 1,
    "address": "0x414cbf31c62d99515bfd66497b495a585b52f703",
    "name": "Chappie",
    "symbol": "CHAP",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/31375/thumb/Frame_563.png?1696530192"
  },
  {
    "chainId": 1,
    "address": "0xcb8a95e76a16b58c30b01e39dd6aad5949e5e802",
    "name": "Tetris",
    "symbol": "TETRIS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31100/thumb/tetris.jpeg?1696529931"
  },
  {
    "chainId": 1,
    "address": "0x2c623d3cc9a2cc158951b8093cb94e80cf56deea",
    "name": "NexAI",
    "symbol": "NEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31056/thumb/NexAI-LOGO_200x200.png?1696529891"
  },
  {
    "chainId": 1,
    "address": "0xc9a1f104fbbda8b8752946f7d56d59d28284037f",
    "name": "Doug",
    "symbol": "DOUG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32639/thumb/DOUG_Logo_200x200.png?1698854273"
  },
  {
    "chainId": 1,
    "address": "0x12ed0641242e4c6c220e3ca8f616e9d5470ac99a",
    "name": "Earn Network",
    "symbol": "EARN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33350/thumb/EARN_logo.png?1701561551"
  },
  {
    "chainId": 1,
    "address": "0x3f57c35633cb29834bb7577ba8052eab90f52a02",
    "name": "Defender Bot",
    "symbol": "DFNDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34816/thumb/defenderbott.png?1706173844"
  },
  {
    "chainId": 1,
    "address": "0x8dbd1331b1de57835b24657ed21d0691e2e7362a",
    "name": "SentimentAI",
    "symbol": "SENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34817/thumb/sentlogo.jpg?1706175707"
  },
  {
    "chainId": 1,
    "address": "0x19f8ed44aa2f5580d44ca6ed2a0e9bb33a08922d",
    "name": "X Mass",
    "symbol": "X-MASS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33337/thumb/IMG_9351.jpeg?1701525066"
  },
  {
    "chainId": 1,
    "address": "0x9be236ee350d18aaaf18619c5776d4096e94a0c7",
    "name": "RATIO",
    "symbol": "RATIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32975/thumb/IMG_20231112_075548_885.jpg?1700069403"
  },
  {
    "chainId": 1,
    "address": "0x7ae075546e8042dc263fa0eb6519ce0a04eabb93",
    "name": "Metal Tools",
    "symbol": "METAL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31340/thumb/logonew.png?1696530158"
  },
  {
    "chainId": 1,
    "address": "0xa4be4cdc552891a6702e1ae9645ef445179a4463",
    "name": "INOFi",
    "symbol": "FON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31343/thumb/FON.png?1696530161"
  },
  {
    "chainId": 1,
    "address": "0x27206f5a9afd0c51da95f20972885545d3b33647",
    "name": "KuKu",
    "symbol": "KUKU",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/31177/thumb/IMG_20230801_131536_144.jpg?1696530005"
  },
  {
    "chainId": 1,
    "address": "0xde0220b69ce3e855a0124433a8e8d093f53a6be4",
    "name": "Where Did The ETH Go ",
    "symbol": "WHETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31486/thumb/WHETH_logo_transparent_200px.png?1696530297"
  },
  {
    "chainId": 1,
    "address": "0x6a6bfdd2174ade27258fe96b9afae3758dec14f2",
    "name": "Solareum",
    "symbol": "SOLAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33338/thumb/IMG_20231130_223554_575.jpg?1701525234"
  },
  {
    "chainId": 1,
    "address": "0xe8e8486228753e01dbc222da262aa706bd67e601",
    "name": "Arch Ethereum Web3",
    "symbol": "WEB3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26562/thumb/WEB3-TOKEN.png?1696525635"
  },
  {
    "chainId": 1,
    "address": "0x3d5fa1cf7d356474f72c8cb24f7a6117b40f8c40",
    "name": "Saintbot",
    "symbol": "SAINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31165/thumb/1SDHoUxG_400x400.jpg?1696529993"
  },
  {
    "chainId": 1,
    "address": "0x21413c119b0c11c5d96ae1bd328917bc5c8ed67e",
    "name": "GenomesDAO GENE",
    "symbol": "GENE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20807/thumb/1637683704200x200.png?1696520200"
  },
  {
    "chainId": 1,
    "address": "0x2388821b40f3ab780f09e97b42b7b577d37a6d5e",
    "name": "GROKX",
    "symbol": "GROKX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33339/thumb/IMG_7246.jpeg?1701525504"
  },
  {
    "chainId": 1,
    "address": "0x4551d8ec1257092cd42b85824c45f944083c7885",
    "name": "Cloud Mining Technologies",
    "symbol": "CXM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34622/thumb/CXM.png?1705504480"
  },
  {
    "chainId": 1,
    "address": "0x9c354503c38481a7a7a51629142963f98ecc12d0",
    "name": "Origin DeFi Governance",
    "symbol": "OGV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26353/thumb/ogv-200x200.png?1696525431"
  },
  {
    "chainId": 1,
    "address": "0x4309e88d1d511f3764ee0f154cee98d783b61f09",
    "name": "Onchain AI",
    "symbol": "OCAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31191/thumb/onchain_200.jpeg?1696530019"
  },
  {
    "chainId": 1,
    "address": "0xafe53eea0cfe20198328890b69107d5fd8159a77",
    "name": "XAI",
    "symbol": "X",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31117/thumb/LOGO.jpg?1696529947"
  },
  {
    "chainId": 1,
    "address": "0x7016c6f4dba9a9a48a47a2fe73a1b9271432d1d5",
    "name": "Fanbase",
    "symbol": "WFNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32445/thumb/Fanbase.png?1698238933"
  },
  {
    "chainId": 1,
    "address": "0xb1c064c3f2908f741c9dea4afc5773238b53e6cc",
    "name": "WarioXRPDumbledoreYugioh69Inu",
    "symbol": "XRP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31167/thumb/ETYYcV7I_400x400.jpg?1696529995"
  },
  {
    "chainId": 1,
    "address": "0xa3c31927a092bd54eb9a0b5dfe01d9db5028bd4f",
    "name": "Espresso Bot",
    "symbol": "ESPR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31091/thumb/ESPR.png?1696529923"
  },
  {
    "chainId": 1,
    "address": "0x6ad9a31f02f1e790ff85584ea3c3d0001e45cd64",
    "name": "Ethane",
    "symbol": "C2H6",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31118/thumb/Ethane200x200.png?1696529948"
  },
  {
    "chainId": 1,
    "address": "0xaaee1a9723aadb7afa2810263653a34ba2c21c7a",
    "name": "Mog Coin",
    "symbol": "MOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31059/thumb/MOG_LOGO_200x200.png?1696529893"
  },
  {
    "chainId": 1,
    "address": "0x32d5b155e82c87c491ea14b59477ddd0957e242c",
    "name": "X Bridge Bot",
    "symbol": "XFER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31178/thumb/Xbot.jpeg?1696530006"
  },
  {
    "chainId": 1,
    "address": "0x903ff0ba636e32de1767a4b5eeb55c155763d8b7",
    "name": "None Trading",
    "symbol": "NONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31137/thumb/None-200x200.png?1696529966"
  },
  {
    "chainId": 1,
    "address": "0x0001a500a6b18995b03f44bb040a5ffc28e45cb0",
    "name": "Autonolas",
    "symbol": "OLAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31099/thumb/OLAS-token.png?1696529930"
  },
  {
    "chainId": 1,
    "address": "0xbd7e92cf6f857be8541fca6abfb72aef8e16c307",
    "name": "Prodigy Bot",
    "symbol": "PRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31171/thumb/PRODIGY_LOGO_clean_%283%29.png?1696529999"
  },
  {
    "chainId": 1,
    "address": "0xb131f4a55907b10d1f0a50d8ab8fa09ec342cd74",
    "name": "Memecoin",
    "symbol": "MEME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32528/thumb/memecoin_%282%29.png?1698912168"
  },
  {
    "chainId": 1,
    "address": "0xe0c8b298db4cffe05d1bea0bb1ba414522b33c1b",
    "name": "nuco cloud",
    "symbol": "NCDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8932/thumb/NCDT_Token.png?1712562605"
  },
  {
    "chainId": 1,
    "address": "0xe2f98dd7506807ef82d1988aa77c320bc52f8df4",
    "name": "Hold On for Dear Life",
    "symbol": "HODL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32636/thumb/HODL.jpg?1698850454"
  },
  {
    "chainId": 1,
    "address": "0xba3335588d9403515223f109edc4eb7269a9ab5d",
    "name": "Gearbox",
    "symbol": "GEAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21630/thumb/gear.png?1696520990"
  },
  {
    "chainId": 1,
    "address": "0x94b888e11a9e960a9c3b3528eb6ac807b27ca62e",
    "name": "Inception lsETH",
    "symbol": "INLSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34951/thumb/LSETH.png?1706771493"
  },
  {
    "chainId": 1,
    "address": "0x0a13a5929e5f0ff0eaba4bd9e9512c91fce40280",
    "name": "XAI Corp",
    "symbol": "XAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30992/thumb/XAI.png?1700491072"
  },
  {
    "chainId": 1,
    "address": "0x9138c8779a0ac8a84d69617d5715bd8afa23c650",
    "name": "Floor Cheese Burger",
    "symbol": "FLRBRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31526/thumb/YvPr4oRZ_400x400.jpg?1696530335"
  },
  {
    "chainId": 1,
    "address": "0xaaaf91d9b90df800df4f55c205fd6989c977e73a",
    "name": "Monolith",
    "symbol": "TKN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/675/thumb/Monolith.png?1696501865"
  },
  {
    "chainId": 1,
    "address": "0xeda8b016efa8b1161208cf041cd86972eee0f31e",
    "name": "IHT Real Estate Protocol",
    "symbol": "IHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3158/thumb/iht-real-estate-protocol.png?1696503880"
  },
  {
    "chainId": 1,
    "address": "0x59c6766de1dc50a9c9db86cb0461b5ce07408ab7",
    "name": "Spurdo",
    "symbol": "SPURDO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31342/thumb/photo_2023-08-16_19-23-12_%281%29.png?1696530160"
  },
  {
    "chainId": 1,
    "address": "0xff836a5821e69066c87e268bc51b849fab94240c",
    "name": "Real Smurf Cat",
    "symbol": "SMURFCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31751/thumb/SMURFCAT.png?1708079642"
  },
  {
    "chainId": 1,
    "address": "0x8edc62648236c20e528adb675ad3f3047697186f",
    "name": "ApePudgyCloneXAzukiMilady",
    "symbol": "NFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34703/thumb/IMG_8634.jpeg?1705862379"
  },
  {
    "chainId": 1,
    "address": "0x695afdb42edff97af470a15920a66df81a234c0e",
    "name": "Wrapped MOXY",
    "symbol": "WMOXY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31221/thumb/Moxy_200x_200_Logo.png?1696530047"
  },
  {
    "chainId": 1,
    "address": "0xade00c28244d5ce17d72e40330b1c318cd12b7c3",
    "name": "AdEx",
    "symbol": "ADX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/847/thumb/adex.jpeg?1696501984"
  },
  {
    "chainId": 1,
    "address": "0x419c4db4b9e25d6db2ad9691ccb832c8d9fda05e",
    "name": "Dragonchain",
    "symbol": "DRGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1289/thumb/dragonchain.png?1696502360"
  },
  {
    "chainId": 1,
    "address": "0xb244b3574a5627849fca2057e3854340def63071",
    "name": "Veil Exchange",
    "symbol": "VEIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32624/thumb/HeIMUGIE_400x400.jpg?1698822478"
  },
  {
    "chainId": 1,
    "address": "0x60927b83ddd2096f38f22a8a2d84cf863402d1a1",
    "name": "Eternal AI",
    "symbol": "MIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33340/thumb/download.png?1701525697"
  },
  {
    "chainId": 1,
    "address": "0x3bd7d4f524d09f4e331577247a048d56e4b67a7f",
    "name": "5ire",
    "symbol": "5IRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25246/thumb/16919.png?1696524387"
  },
  {
    "chainId": 1,
    "address": "0x628a3b2e302c7e896acc432d2d0dd22b6cb9bc88",
    "name": "LimeWire",
    "symbol": "LMWR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30356/thumb/LimeWire_Logo_Icon_200x200_%281%29.png?1696529256"
  },
  {
    "chainId": 1,
    "address": "0x3ee4b152824b657644c7a9b50694787e80eb8f4a",
    "name": "Bazed Games",
    "symbol": "BAZED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31195/thumb/IMG_1849.jpeg?1696530023"
  },
  {
    "chainId": 1,
    "address": "0x925206b8a707096ed26ae47c84747fe0bb734f59",
    "name": "WhiteBIT Coin",
    "symbol": "WBT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27045/thumb/wbt_token.png?1696526096"
  },
  {
    "chainId": 1,
    "address": "0x6c3ea9036406852006290770bedfcaba0e23a0e8",
    "name": "PayPal USD",
    "symbol": "PYUSD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31212/thumb/PYUSD_Logo_%282%29.png?1696530039"
  },
  {
    "chainId": 1,
    "address": "0xe1a70b24e109f7a8b39806c554e123efc6769e91",
    "name": "Liquid Savings DAI",
    "symbol": "LSDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31219/thumb/lsdai-logo.png?1696530045"
  },
  {
    "chainId": 1,
    "address": "0x4efce4c758ddfb3911a1a1282a29ce0bdb16ef86",
    "name": "WOW",
    "symbol": "",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33341/thumb/logo_%283%29.png?1701525830"
  },
  {
    "chainId": 1,
    "address": "0x312d43881860807fa04b193d69744d087fc3308a",
    "name": "BRC 20 DEX",
    "symbol": "BD20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35209/thumb/logo.jpg?1711030438"
  },
  {
    "chainId": 1,
    "address": "0xf62ac0fcae17f9195280ced4de978313effe2daa",
    "name": "Nchart Token",
    "symbol": "CHART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31169/thumb/n_200.png?1696529997"
  },
  {
    "chainId": 1,
    "address": "0x343cf59a43bd7ddd38b7236a478139a86a26222b",
    "name": "Caesar s Arena",
    "symbol": "CAESAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32623/thumb/logo32.png?1698821024"
  },
  {
    "chainId": 1,
    "address": "0xb622907fbff6cbf7c3ce355173251e3edb13a606",
    "name": "FortuneBets",
    "symbol": "FRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32966/thumb/fortune-white-on-red.png?1700054670"
  },
  {
    "chainId": 1,
    "address": "0xf4308b0263723b121056938c2172868e408079d0",
    "name": "CryoDAO",
    "symbol": "CRYO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34624/thumb/cryo-icon.png?1705504846"
  },
  {
    "chainId": 1,
    "address": "0xe0a6136f866684c0f19936c0c42a8c181c066f1b",
    "name": "Tidalflats",
    "symbol": "TIDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33250/thumb/TIDE.jpg?1701236279"
  },
  {
    "chainId": 1,
    "address": "0xd075e95423c5c4ba1e122cae0f4cdfa19b82881b",
    "name": "OpesAI",
    "symbol": "WPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16647/thumb/ai.png?1696516208"
  },
  {
    "chainId": 1,
    "address": "0x280df82db83abb0a4c734bb02afc7985a1c8eaf2",
    "name": "Etherpets",
    "symbol": "EPETS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34928/thumb/112.png?1706686505"
  },
  {
    "chainId": 1,
    "address": "0x1cdd2eab61112697626f7b4bb0e23da4febf7b7c",
    "name": "Bridged Tether  Wormhole ",
    "symbol": "USDTSO",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/22881/thumb/USDTso_wh_small.png?1696522179"
  },
  {
    "chainId": 1,
    "address": "0x2d886570a0da04885bfd6eb48ed8b8ff01a0eb7e",
    "name": "Blockchain Bets",
    "symbol": "BCB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28694/thumb/200x200.jpg?1696527676"
  },
  {
    "chainId": 1,
    "address": "0xce0e16a9996af71180c5967bd79885c544b50a55",
    "name": "Pumpkin",
    "symbol": "PUMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32638/thumb/_PUMP_200x200.png?1698852550"
  },
  {
    "chainId": 1,
    "address": "0x4501a82790ef2587dfeb93dc038541228e516597",
    "name": "Hydra",
    "symbol": "HYDRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32062/thumb/GDQ7c1hj_400x400.jpg?1696530859"
  },
  {
    "chainId": 1,
    "address": "0xe0e51a3f1d9c65bba1326e4a0b1d99b737efc2c0",
    "name": "RedneckMountainDew",
    "symbol": "RMD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32395/thumb/mountain_dew_%282%29_%281%29_%281%29.png?1698053570"
  },
  {
    "chainId": 1,
    "address": "0xda47862a83dac0c112ba89c6abc2159b95afd71c",
    "name": "Prisma Governance Token",
    "symbol": "PRISMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31520/thumb/PRISMA_200.png?1696530330"
  },
  {
    "chainId": 1,
    "address": "0x8c6778023c3d4fd79ddd14810079f64c39e9e43d",
    "name": "0xAISwap",
    "symbol": "0XAISWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32739/thumb/0x-AISwap-logo.png?1699236607"
  },
  {
    "chainId": 1,
    "address": "0x423f4e6138e475d85cf7ea071ac92097ed631eea",
    "name": "PondCoin",
    "symbol": "PNDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31215/thumb/pond-coin.jpeg?1696530042"
  },
  {
    "chainId": 1,
    "address": "0xdd23410f1fff27279c7314ff4735e71852084ddf",
    "name": "Volume AI",
    "symbol": "VAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35869/thumb/photo_2024-02-29_17-06-28.jpg?1709973817"
  },
  {
    "chainId": 1,
    "address": "0xed4e879087ebd0e8a77d66870012b5e0dffd0fa4",
    "name": "AstroPepeX",
    "symbol": "APX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31839/thumb/cglogo.png?1699223398"
  },
  {
    "chainId": 1,
    "address": "0x9f2b8d32dd418d5b41e740265929e5d3d5fb102b",
    "name": "Anonbot",
    "symbol": "ANONBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32625/thumb/IMG_6424_%283%29.png?1698822640"
  },
  {
    "chainId": 1,
    "address": "0xbf0b8b7475edb32d103001efd19fdd2753d7b76d",
    "name": "Abachi",
    "symbol": "ABI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31225/thumb/Abachi.png?1696530051"
  },
  {
    "chainId": 1,
    "address": "0x99f618edcfedca1fcc8302e14daa84802114a8c5",
    "name": "DecentraBNB",
    "symbol": "DBNB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26544/thumb/Screenshot_2022-07-22_at_18.16.11.png?1696525617"
  },
  {
    "chainId": 1,
    "address": "0x8e3fa615392688ddd9bf8f25d1f8dc744ac1a12c",
    "name": "DumbMoney",
    "symbol": "GME",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34929/thumb/DUMBMONEYICON.jpg?1706687792"
  },
  {
    "chainId": 1,
    "address": "0x1b5e16c5b20fb5ee87c61fe9afe735cca3b21a65",
    "name": "Index Coop Large Cap Index",
    "symbol": "IC21",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31698/thumb/ic21_token_logo.png?1696530516"
  },
  {
    "chainId": 1,
    "address": "0xabec00542d141bddf58649bfe860c6449807237c",
    "name": "CruxDecussata",
    "symbol": "X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31246/thumb/Asset_1.png?1696530071"
  },
  {
    "chainId": 1,
    "address": "0x3af33bef05c2dcb3c7288b77fe1c8d2aeba4d789",
    "name": "Kromatika",
    "symbol": "KROM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20541/thumb/KROM_Transparent.png?1696519948"
  },
  {
    "chainId": 1,
    "address": "0x12272c264ca580d2190b16db14cea3815f52060c",
    "name": "GROKBOT",
    "symbol": "GROKBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33424/thumb/Group_79.png?1701768966"
  },
  {
    "chainId": 1,
    "address": "0x733b5056a0697e7a4357305fe452999a0c409feb",
    "name": "IMVU",
    "symbol": "VCORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30682/thumb/VCORE_Logo_200x200.png?1696529551"
  },
  {
    "chainId": 1,
    "address": "0x07e0edf8ce600fb51d44f51e3348d77d67f298ae",
    "name": "HarryPotterObamaPacMan8Inu",
    "symbol": "XRP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31241/thumb/Pacman_Logo200x200.png?1696530067"
  },
  {
    "chainId": 1,
    "address": "0xd0d56273290d339aaf1417d9bfa1bb8cfe8a0933",
    "name": "Foom",
    "symbol": "FOOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31308/thumb/f.png?1696530127"
  },
  {
    "chainId": 1,
    "address": "0x32a5fc22e20903e78171977359b500060b7316bf",
    "name": "Paw a Gotchi",
    "symbol": "PAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33116/thumb/pawagotchi_200px.png?1700732363"
  },
  {
    "chainId": 1,
    "address": "0x3f0c858c26775df75e45934b189154c436bd7335",
    "name": "SuperFrank",
    "symbol": "CHFP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32531/thumb/SuperFrank_logo1080x1080_nobackground_noletters_centered.png?1698461197"
  },
  {
    "chainId": 1,
    "address": "0x7137e8a3b069c3f787c4ffbb901b91e4ba47d082",
    "name": "Zeus",
    "symbol": "ZEUS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36847/thumb/_ZEUS.jpg?1712584303"
  },
  {
    "chainId": 1,
    "address": "0x66a39d581961d4cfa9bd3953ccc4e7ec7bd56716",
    "name": "Yellow Team",
    "symbol": "YELLOW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31441/thumb/yellowteamclick.jpeg?1696530255"
  },
  {
    "chainId": 1,
    "address": "0xe86df1970055e9caee93dae9b7d5fd71595d0e18",
    "name": "Bitcoin20",
    "symbol": "BTC20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31284/thumb/BTC20_logo_%281%29.png?1696530107"
  },
  {
    "chainId": 1,
    "address": "0x209a78d23f825950a5df4d6d21288e5212b44f2c",
    "name": "AlphaNova",
    "symbol": "ANVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35210/thumb/LiquidAlpha-Logo.png?1709176634"
  },
  {
    "chainId": 1,
    "address": "0x3e34eabf5858a126cb583107e643080cee20ca64",
    "name": "Linq",
    "symbol": "LINQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31328/thumb/IMG_1185.jpeg?1696530147"
  },
  {
    "chainId": 1,
    "address": "0x57ab1ec28d129707052df4df418d58a2d46d5f51",
    "name": "sUSD",
    "symbol": "SUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5013/thumb/sUSD.png?1696505546"
  },
  {
    "chainId": 1,
    "address": "0xc3960227e41c3f54e9b399ce216149dea5315c34",
    "name": "Changpeng Zhao",
    "symbol": "CZ",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33193/thumb/IMG_20231204_224120_835.jpg?1701798785"
  },
  {
    "chainId": 1,
    "address": "0xc14b4d4ca66f40f352d7a50fd230ef8b2fb3b8d4",
    "name": "Blocktools",
    "symbol": "TOOLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31313/thumb/blocktools.jpeg?1696530132"
  },
  {
    "chainId": 1,
    "address": "0x01e87d74b11f656a673a3e7c441425816213eb0c",
    "name": "Sonic",
    "symbol": "HOTDOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31480/thumb/Sonic-The-Hot-Dog-NO-BG-NO-TEXT-1.png?1696530292"
  },
  {
    "chainId": 1,
    "address": "0xbe1bc2e2338defcd88e7f9f5f745ddb222180ab9",
    "name": "TeSo",
    "symbol": "TESO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31329/thumb/8A255A9F-4860-4D3C-9CDF-A20DC488FD7A.jpeg?1696530148"
  },
  {
    "chainId": 1,
    "address": "0xed354cae598148846aa9694254c854fc7051163c",
    "name": "Shido  OLD ",
    "symbol": "SHIDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26674/thumb/Shido.png?1696525746"
  },
  {
    "chainId": 1,
    "address": "0xca4b70beccabce29efa5bc5c86311e5d38461842",
    "name": "Sybulls",
    "symbol": "SYBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31316/thumb/sybulls.png?1696530135"
  },
  {
    "chainId": 1,
    "address": "0xe2512a2f19f0388ad3d7a5263eaa82acd564827b",
    "name": "Shido",
    "symbol": "SHIDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32070/thumb/SHIDO.png?1696530867"
  },
  {
    "chainId": 1,
    "address": "0x98e1f56b334438e3f0bde22d92f5bfd746e0631f",
    "name": "Illuminati",
    "symbol": "ILUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32362/thumb/ILLUMINATI_200x200.png?1698067399"
  },
  {
    "chainId": 1,
    "address": "0x0f3b904172f937748af7f09353b016219a14562c",
    "name": "Polygon Star",
    "symbol": "POS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31947/thumb/logo.png?1696530753"
  },
  {
    "chainId": 1,
    "address": "0x14778860e937f509e651192a90589de711fb88a9",
    "name": "CyberConnect",
    "symbol": "CYBER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31274/thumb/cyberconnect.png?1696530098"
  },
  {
    "chainId": 1,
    "address": "0x20945ca1df56d237fd40036d47e866c7dccd2114",
    "name": "Nsure Network",
    "symbol": "NSURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12948/thumb/Nsure_token.png?1696512737"
  },
  {
    "chainId": 1,
    "address": "0x7d60de2e7d92cb5c863bc82f8d59b37c59fc0a7a",
    "name": "DeeLance",
    "symbol": "DLANCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31404/thumb/Deelancer-Final-LOGO.png?1696530220"
  },
  {
    "chainId": 1,
    "address": "0x982b50e55394641ca975a0eec630b120b671391a",
    "name": "Ecoterra",
    "symbol": "ECOTERRA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31563/thumb/ECOTERRA.png?1696530376"
  },
  {
    "chainId": 1,
    "address": "0xa9fbcc25435ad713a9468d8c89dd7baae8914e3a",
    "name": "Prophet",
    "symbol": "PROPHET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31908/thumb/Logo_200x200.png?1696530717"
  },
  {
    "chainId": 1,
    "address": "0xa3d4bee77b05d4a0c943877558ce21a763c4fa29",
    "name": "The Root Network",
    "symbol": "ROOT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33122/thumb/6T1Tapl__400x400.jpg?1700740439"
  },
  {
    "chainId": 1,
    "address": "0x1a44e35d5451e0b78621a1b3e7a53dfaa306b1d0",
    "name": "baoETH ETH StablePool",
    "symbol": "B-BAOETH-ETH-BPT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x966e770030209c95f974f37edbde65d98e853354",
    "name": "Maxx",
    "symbol": "MAXX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30488/thumb/D9E6AF16-95F9-4550-87B5-E60C50DB1E02.png?1696529375"
  },
  {
    "chainId": 1,
    "address": "0x5e74c9036fb86bd7ecdcb084a0673efc32ea31cb",
    "name": "sETH",
    "symbol": "SETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8843/thumb/sETH.png?1696508995"
  },
  {
    "chainId": 1,
    "address": "0xfe18be6b3bd88a2d2a7f928d00292e7a9963cfc6",
    "name": "sBTC",
    "symbol": "SBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8838/thumb/sBTC.png?1696508990"
  },
  {
    "chainId": 1,
    "address": "0x9ee8c380e1926730ad89e91665ff27063b13c90a",
    "name": "Coupon Assets",
    "symbol": "CA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32202/thumb/ca.jpg?1696746380"
  },
  {
    "chainId": 1,
    "address": "0xc740181345c65552333e1edc797e03f11852b1c8",
    "name": "Kento",
    "symbol": "KNTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31352/thumb/knto-cg-200.png?1696530169"
  },
  {
    "chainId": 1,
    "address": "0x5044d567f7b30891639d982a05726a6bfe8bae6a",
    "name": "Brainers",
    "symbol": "BRAINERS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36127/thumb/Brainers.png?1710579156"
  },
  {
    "chainId": 1,
    "address": "0x8ed97a637a790be1feff5e888d43629dc05408f6",
    "name": "Non Playable Coin",
    "symbol": "NPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31193/thumb/NPC_200x200.png?1696530021"
  },
  {
    "chainId": 1,
    "address": "0x0ae055097c6d159879521c384f1d2123d1f195e6",
    "name": "STAKE",
    "symbol": "STAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11061/thumb/xdai.png?1696511003"
  },
  {
    "chainId": 1,
    "address": "0xfe6f2e70f30a0894d0aee79e11653275e89c7bd6",
    "name": "Karen Pepe",
    "symbol": "KEPE",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xfae30394a76796dc3df37c2714f5fc12083dfdb0",
    "name": "NakaChain",
    "symbol": "NAKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36269/thumb/1CR4up2C_400x400.jpg?1711003041"
  },
  {
    "chainId": 1,
    "address": "0x8efe7dd5984640537b6596fb28b762f6c000f184",
    "name": "Dark Forest",
    "symbol": "DARK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31377/thumb/Dark_Forest.png?1696530194"
  },
  {
    "chainId": 1,
    "address": "0x476c5e26a75bd202a9683ffd34359c0cc15be0ff",
    "name": "Serum",
    "symbol": "SRM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/11970/thumb/serum-logo.png?1696511829"
  },
  {
    "chainId": 1,
    "address": "0x58fb30a61c218a3607e9273d52995a49ff2697ee",
    "name": "PintSwap",
    "symbol": "PINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32396/thumb/Pint_token_logo.jpg?1698054159"
  },
  {
    "chainId": 1,
    "address": "0x3f3052ccc6ab2337e3d63f5c695c4471a8637a32",
    "name": "OOF",
    "symbol": "OOF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32573/thumb/Oof_logi.jpeg?1698554413"
  },
  {
    "chainId": 1,
    "address": "0x8ff62c1b0868e1a8905eb0d0cbb04f20667dcf88",
    "name": "BTCrewards",
    "symbol": "BTCR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32741/thumb/Bitcoin_Rewards_Logo_-_200x200.png?1699239519"
  },
  {
    "chainId": 1,
    "address": "0x87d73e916d7057945c9bcd8cdd94e42a6f47f776",
    "name": "NFTX",
    "symbol": "NFTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13574/thumb/NFTX_%28Real%29.jpg?1696513327"
  },
  {
    "chainId": 1,
    "address": "0xb1a822ce8c799b0777ed1f260113819247e1bf26",
    "name": "HairyPlotterFTX",
    "symbol": "FTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32116/thumb/HairyPlotterFTX.png?1696585812"
  },
  {
    "chainId": 1,
    "address": "0x3250577e12b9469915c1fa3a71c22817ca44c4d9",
    "name": "Socrates",
    "symbol": "SOC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34895/thumb/socrates.jpeg?1706580533"
  },
  {
    "chainId": 1,
    "address": "0x285db79fa7e0e89e822786f48a7c98c6c1dc1c7d",
    "name": "Magic Internet Cash",
    "symbol": "MIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31405/thumb/MICsmall.png?1696530221"
  },
  {
    "chainId": 1,
    "address": "0x9778ac3d5a2f916aa9abf1eb85c207d990ca2655",
    "name": "OG SMINEM",
    "symbol": "OGSM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32574/thumb/Sminem_portrait__2_-removebg-preview.png?1698554514"
  },
  {
    "chainId": 1,
    "address": "0xfcaf0e4498e78d65526a507360f755178b804ba8",
    "name": "NicCageWaluigiElmo42069Inu",
    "symbol": "SHIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31378/thumb/NicCageWaluigiElmo42069Inu.png?1696530195"
  },
  {
    "chainId": 1,
    "address": "0x03475e14de25908ed484eb99ad38d68cb6399807",
    "name": "Blue Team",
    "symbol": "BLUE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31443/thumb/blueteam.jpeg?1696530257"
  },
  {
    "chainId": 1,
    "address": "0x320ed4c7243e35a00f9ca30a1ae60929d15eae37",
    "name": "The Blox Project",
    "symbol": "BLOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32397/thumb/blox.png?1698054451"
  },
  {
    "chainId": 1,
    "address": "0xca5b0ae1d104030a9b8f879523508efd86c14483",
    "name": "Benji Bananas",
    "symbol": "BENJI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31022/thumb/BenjiTokens_FinalFlat.png?1696529858"
  },
  {
    "chainId": 1,
    "address": "0xebb82c932759b515b2efc1cfbb6bf2f6dbace404",
    "name": "shares finance",
    "symbol": "SHARES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31379/thumb/shares.jpeg?1696530196"
  },
  {
    "chainId": 1,
    "address": "0xc9cb16ac87c43697b838c64ee416d748817f2b77",
    "name": "Groooook",
    "symbol": "GROOOOOK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33351/thumb/20231120143459.png?1701561790"
  },
  {
    "chainId": 1,
    "address": "0xb1f136a74e18e4e2921febbf25820d1bb65b5647",
    "name": "hiPunks",
    "symbol": "HIPUNKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26739/thumb/hipunks.png?1696525808"
  },
  {
    "chainId": 1,
    "address": "0xcaa79bf8b1d00bf3d4f6dbec6221955871c04618",
    "name": "CrocBot",
    "symbol": "CROC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31382/thumb/Frame_123.png?1696530199"
  },
  {
    "chainId": 1,
    "address": "0xb7cfe05915ef0c040c6dde2007c9ddab26259e04",
    "name": "Molly",
    "symbol": "MOLLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34628/thumb/Molly.png?1705547025"
  },
  {
    "chainId": 1,
    "address": "0x7b744eea1deca2f1b7b31f15ba036fa1759452d7",
    "name": "El Hippo",
    "symbol": "HIPP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31384/thumb/HIPP.png?1696530201"
  },
  {
    "chainId": 1,
    "address": "0x670ec8893da9c339d52fa70189756a92cafbe654",
    "name": "Potter Predator",
    "symbol": "VOLDEMORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31394/thumb/Potter_Predator.png?1696530210"
  },
  {
    "chainId": 1,
    "address": "0x0a693a301215aad39d83a32a5b5279f2d238851b",
    "name": "Pepe OG",
    "symbol": "POG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31597/thumb/photo_2023-09-04_17.43.39.jpeg?1696530414"
  },
  {
    "chainId": 1,
    "address": "0x34df29dd880e9fe2cec0f85f7658b75606fb2870",
    "name": "Navy seal",
    "symbol": "NAVYSEAL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31406/thumb/Navyseal.jpg?1696530222"
  },
  {
    "chainId": 1,
    "address": "0x9360c489056b64d5003bf22f4f31458e31cc8028",
    "name": "Bank",
    "symbol": "BANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31407/thumb/Bank.png?1696530222"
  },
  {
    "chainId": 1,
    "address": "0x69cd13d248830602a60b1f20ab11f5049385877d",
    "name": "Baby Pepe",
    "symbol": "BABYPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31417/thumb/output-onlinepngtools_%281%29.png?1696530232"
  },
  {
    "chainId": 1,
    "address": "0x4594cffbfc09bc5e7ecf1c2e1c1e24f0f7d29036",
    "name": "0 Knowledge Network",
    "symbol": "0KN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31626/thumb/200x200.png?1696530442"
  },
  {
    "chainId": 1,
    "address": "0xa72332af4a7efbce221903e7a09175be64f0400d",
    "name": "Gumball Machine",
    "symbol": "GUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35277/thumb/gumball.jpg?1708080490"
  },
  {
    "chainId": 1,
    "address": "0x6988a804c74fd04f37da1ea4781cea68c9c00f86",
    "name": "Tribal Token",
    "symbol": "TRIBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27774/thumb/PFP_black_%281%29.png?1696526795"
  },
  {
    "chainId": 1,
    "address": "0xcd6926193308d3b371fdd6a6219067e550000000",
    "name": "Nest Protocol",
    "symbol": "NEST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11284/thumb/52954052.png?1696511207"
  },
  {
    "chainId": 1,
    "address": "0xa9ad6830180f9c150349f2cecadd710586e35cb7",
    "name": "ETH Stable",
    "symbol": "ETHS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31464/thumb/ETHS.jpeg?1696530277"
  },
  {
    "chainId": 1,
    "address": "0x3c37577f1de12046aea6975862559a50d8f50158",
    "name": "ETH Coin",
    "symbol": "ETHC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31465/thumb/ETHC.jpeg?1696530278"
  },
  {
    "chainId": 1,
    "address": "0xe221db71c5c527149bdf56c4d13a54cf55e543c1",
    "name": "Aktionariat Technologies of Understandi",
    "symbol": "VIDS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33252/thumb/VIDS.PNG?1701254720"
  },
  {
    "chainId": 1,
    "address": "0xa88842ae47dbe87116cf7001dddd1b246fcd8262",
    "name": "hiENS3",
    "symbol": "HIENS3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27262/thumb/hiens3.png?1696526315"
  },
  {
    "chainId": 1,
    "address": "0x33bd66c334274989b673a8e8c8d1a3f1b8de5889",
    "name": "hiODBS",
    "symbol": "HIODBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27296/thumb/hiodbs.png?1696526348"
  },
  {
    "chainId": 1,
    "address": "0xbbeca80a4c99c3b1bd3c10e64595d0fc7dc26ee0",
    "name": "hiMEEBITS",
    "symbol": "HIMEEBITS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28126/thumb/himeebit.png?1696527133"
  },
  {
    "chainId": 1,
    "address": "0x3700adfd26d5bc062cb8b8a77e68fbd43f58ecab",
    "name": "hiDOODLES",
    "symbol": "HIDOODLES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27427/thumb/hidoodles.png?1696526468"
  },
  {
    "chainId": 1,
    "address": "0x12aef5c60c2c86c8ecd3079f22f285f326371340",
    "name": "hiSAND33",
    "symbol": "HISAND33",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27006/thumb/logo.ab86781.png?1696526059"
  },
  {
    "chainId": 1,
    "address": "0x00a7ec2f2b451cb0233e8adbf4c9a951027c2b02",
    "name": "hiENS4",
    "symbol": "HIENS4",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26921/thumb/62f9ec6fd1c44e000102051d_MicrosoftTeams-image.png?1696525978"
  },
  {
    "chainId": 1,
    "address": "0x5362ca75aa3c0e714bc628296640c43dc5cb9ed6",
    "name": "Dejitaru Hoshi",
    "symbol": "HOSHI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31488/thumb/Hoshi_200x200_-_Copy.png?1696530299"
  },
  {
    "chainId": 1,
    "address": "0x7a83ff237e7870d8d6c3485920ebe654d2841315",
    "name": "hiOD",
    "symbol": "HIOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28238/thumb/hiod.png?1696527241"
  },
  {
    "chainId": 1,
    "address": "0x00c5ca160a968f47e7272a0cfcda36428f386cb6",
    "name": "USDEBT",
    "symbol": "USDEBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31448/thumb/200x200.png?1710861880"
  },
  {
    "chainId": 1,
    "address": "0x635f15eb7aa2e62d122f6b1f9f519fdccf4abdda",
    "name": "hiMAYC",
    "symbol": "HIMAYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27602/thumb/himayc.png?1696526633"
  },
  {
    "chainId": 1,
    "address": "0x082716b6734b31791407d7dd0e2a2c41260029b2",
    "name": "hiCOOLCATS",
    "symbol": "HICOOLCATS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28603/thumb/hicoolcats.png?1696527589"
  },
  {
    "chainId": 1,
    "address": "0x764104dc24dadff01150253a58c82337984b4319",
    "name": "hiCLONEX",
    "symbol": "HICLONEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28436/thumb/hiclonex.png?1696527433"
  },
  {
    "chainId": 1,
    "address": "0x15c1cab705b9ddb9ffeeea608ed8bafcdd4c0396",
    "name": "HXAcoin",
    "symbol": "HXA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31315/thumb/HXA_Ticker_image_%281%29.png?1699280752"
  },
  {
    "chainId": 1,
    "address": "0x0b0a8c7c34374c1d0c649917a97eee6c6c929b1b",
    "name": "Shiba V Pepe",
    "symbol": "SHEPE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31599/thumb/photo_2023-08-30_01-36-19_%282%29.png?1696530415"
  },
  {
    "chainId": 1,
    "address": "0x669db4c47f89f21554ebd825a744888725fd9491",
    "name": "hiPENGUINS",
    "symbol": "HIPENGUINS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28381/thumb/63856bebcd8c700001b23c42_hiPENGUINS-_Logo.png?1696527381"
  },
  {
    "chainId": 1,
    "address": "0xc1ad0aa69454603a5dee55cf9bd9341e01328544",
    "name": "hiGAZERS",
    "symbol": "HIGAZERS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28316/thumb/higazers.png?1696527322"
  },
  {
    "chainId": 1,
    "address": "0x4b7ffcb2b92fb4890f22f62a52fb7a180eab818e",
    "name": "DIVA Protocol",
    "symbol": "DIVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31467/thumb/DIVA-200px-square.png?1696530280"
  },
  {
    "chainId": 1,
    "address": "0xc0da2786176dac9eda661c566828950ca571348c",
    "name": "Aktionariat Art Leasing And Invest AG T",
    "symbol": "ARTS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33352/thumb/ART.png?1701562351"
  },
  {
    "chainId": 1,
    "address": "0x8a3c710e41cd95799c535f22dbae371d7c858651",
    "name": "Xccelerate",
    "symbol": "XLRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31425/thumb/Xccelerate.png?1696530240"
  },
  {
    "chainId": 1,
    "address": "0x7721a4cb6190edb11d47f51c20968436eccdafb8",
    "name": "Guiser",
    "symbol": "GUISE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31426/thumb/GUISE.jpg?1696530241"
  },
  {
    "chainId": 1,
    "address": "0xd2ce625515600b7799a5750a853ec8d2a44fbe27",
    "name": "Sugarbaby",
    "symbol": "SUGAR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31491/thumb/_SUGAR_Token_Image.png?1696530302"
  },
  {
    "chainId": 1,
    "address": "0xa995d67fd0187b4b8fea3a60e11f31a08e4ac40b",
    "name": "Aktionariat Pension Dynamics AG Tokeniz",
    "symbol": "PDS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33353/thumb/PD.png?1701563333"
  },
  {
    "chainId": 1,
    "address": "0x4e8de529fe22dee9266b029cdfc52142b82e0e2f",
    "name": "Aktionariat Outlawz Food AG Tokenized S",
    "symbol": "VEGS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33355/thumb/VEG.png?1701563721"
  },
  {
    "chainId": 1,
    "address": "0x43e54c2e7b3e294de3a155785f52ab49d87b9922",
    "name": "Aladdin sdCRV",
    "symbol": "ASDCRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31428/thumb/asdCRV_logo.jpg?1696530243"
  },
  {
    "chainId": 1,
    "address": "0x17729f7ffddd4ed480cf4cfb1668ef90d1e6a24e",
    "name": "EventsX",
    "symbol": "EVEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34397/thumb/evex_logo.jpg?1704787275"
  },
  {
    "chainId": 1,
    "address": "0x24a6a37576377f63f194caa5f518a60f45b42921",
    "name": "Float Protocol",
    "symbol": "BANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14116/thumb/FLOAT-Bank_LOGO-reduced_01.png?1696513836"
  },
  {
    "chainId": 1,
    "address": "0xe4ab0be415e277d82c38625b72bd7dea232c2e7d",
    "name": "XRP20",
    "symbol": "XRP20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31435/thumb/xrp20_logo_resize.png?1696530250"
  },
  {
    "chainId": 1,
    "address": "0x456815812b3129a4389ea4d73e9706697cc91373",
    "name": "0xS",
    "symbol": "0XS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31453/thumb/Logo200x200.png?1696530267"
  },
  {
    "chainId": 1,
    "address": "0x189fc141854bfc6146777406d53fbc89f4e70407",
    "name": "Ceji",
    "symbol": "CEJI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26458/thumb/CEJI.png?1696525531"
  },
  {
    "chainId": 1,
    "address": "0x43d7e65b8ff49698d9550a7f315c87e67344fb59",
    "name": "Shiba Saga",
    "symbol": "SHIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31437/thumb/51nmYxm3_400x400.jpg?1696530252"
  },
  {
    "chainId": 1,
    "address": "0xd5dc24c9ef8a21e6d7caa603f9502e4a0dc7fe8a",
    "name": "Red Team",
    "symbol": "RED",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31439/thumb/red_team_click.jpeg?1696530254"
  },
  {
    "chainId": 1,
    "address": "0x9078deb4736b7e2181beb8db276dfbfa7f21f821",
    "name": "Monk gg",
    "symbol": "MONKGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31502/thumb/Lu4Pg4jC_400x400.jpg?1696530313"
  },
  {
    "chainId": 1,
    "address": "0x70b790d0948a760e80bc3f892b142f7779b538b2",
    "name": "Dora Factory",
    "symbol": "DORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31552/thumb/DORA.jpg?1696530364"
  },
  {
    "chainId": 1,
    "address": "0x73c6a7491d0db90bdb0060308cde0f49dfd1d0b0",
    "name": "DogeBonk",
    "symbol": "DOBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31455/thumb/200x200.png?1696530269"
  },
  {
    "chainId": 1,
    "address": "0xd87996ff3d06858bfc20989aef50cc5fcd4d84ca",
    "name": "Golden Inu",
    "symbol": "GOLDEN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31473/thumb/git-200x200.png?1704456922"
  },
  {
    "chainId": 1,
    "address": "0xfbbe098ee65238e4d9f771404edddcbf89cd689b",
    "name": "Treemeister",
    "symbol": "TREE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31529/thumb/TREELOGO.png?1696530338"
  },
  {
    "chainId": 1,
    "address": "0xd3b9a9e5ee4aab3963c95dd715882def4f22b449",
    "name": "Revhub",
    "symbol": "REVHUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31841/thumb/Revhub.jpg?1696530655"
  },
  {
    "chainId": 1,
    "address": "0x3a6dc7eefef660be5c254c8aa1b710202151e345",
    "name": "dPARROT",
    "symbol": "PARROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31601/thumb/coingecko.jpg?1696530417"
  },
  {
    "chainId": 1,
    "address": "0x297e4e5e59ad72b1b0a2fd446929e76117be0e0a",
    "name": "Smart Valor",
    "symbol": "VALOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7231/thumb/1._SMART_VALOR__Logo_NEW.png?1696507527"
  },
  {
    "chainId": 1,
    "address": "0x15c65da8309153d4d0edb47041555f155d5e4933",
    "name": "Proof of Anon",
    "symbol": "PRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32102/thumb/IMG_3084.jpeg?1696530899"
  },
  {
    "chainId": 1,
    "address": "0x2f8221e82e0d4669ad66eabf02a5baed43ea49e7",
    "name": "Newsly",
    "symbol": "NEWS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31489/thumb/byk_Sy1X_400x400.jpg?1696530300"
  },
  {
    "chainId": 1,
    "address": "0x9baa12a9e3b9dc355f162082762f95626367d087",
    "name": "Handz of Gods",
    "symbol": "HANDZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31531/thumb/40B0eCTs_400x400.jpg?1696530340"
  },
  {
    "chainId": 1,
    "address": "0x97de57ec338ab5d51557da3434828c5dbfada371",
    "name": "eUSD  OLD ",
    "symbol": "EUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30047/thumb/ethereum-eth-logo-colored_4_%281%29_%281%29.png?1696528969"
  },
  {
    "chainId": 1,
    "address": "0xdf3ac4f479375802a821f7b7b46cd7eb5e4262cc",
    "name": "eUSD",
    "symbol": "EUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31603/thumb/eusd.png?1696530419"
  },
  {
    "chainId": 1,
    "address": "0x7ad16874759348f04b6b6119463d66c07ae54899",
    "name": "PIRB",
    "symbol": "PIRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31653/thumb/pirb_geckologo.png?1696530468"
  },
  {
    "chainId": 1,
    "address": "0x53020f42f6da51b50cf6e23e45266ef223122376",
    "name": "BlockSpot Network",
    "symbol": "SPOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33078/thumb/token_%285%29_%281%29.png?1700550027"
  },
  {
    "chainId": 1,
    "address": "0x776280f68ad33c4d49e6846507b7dbaf7811c89f",
    "name": "ZeroLiquid ETH",
    "symbol": "ZETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31533/thumb/IMG_20230828_212341_351.png?1696530342"
  },
  {
    "chainId": 1,
    "address": "0xd04202abd9568a8a8798484ca1bd8778411f7103",
    "name": "FireBot",
    "symbol": "FIREBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32532/thumb/firebot.png?1698461366"
  },
  {
    "chainId": 1,
    "address": "0x51fe05eac152494908ff1ebbd50e116e960baf64",
    "name": "X GPT",
    "symbol": "XGPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31559/thumb/XGPT_Logo.png?1696530372"
  },
  {
    "chainId": 1,
    "address": "0xd82fd4d6d62f89a1e50b1db69ad19932314aa408",
    "name": "Liquis",
    "symbol": "LIQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31587/thumb/Liqblue.png?1696530404"
  },
  {
    "chainId": 1,
    "address": "0xaf2ca40d3fc4459436d11b94d21fa4b8a89fb51d",
    "name": "COTI Governance Token",
    "symbol": "GCOTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32930/thumb/gCOTI_logo_icon200x200.png?1700669092"
  },
  {
    "chainId": 1,
    "address": "0xec21890967a8ceb3e55a3f79dac4e90673ba3c2e",
    "name": "BEBE",
    "symbol": "BEBE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/34932/thumb/1000003129.jpg?1706694024"
  },
  {
    "chainId": 1,
    "address": "0x52d30b949bdbc63780e5a263cf436d4df983fe58",
    "name": "Zenith Wallet",
    "symbol": "ZW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31670/thumb/ZENITH.jpg?1696530486"
  },
  {
    "chainId": 1,
    "address": "0x4208aa4d7a9a10f4f8bb7f6400c1b2161d946969",
    "name": "DongCoin",
    "symbol": "DONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31499/thumb/DONG_LOGO.png?1696530310"
  },
  {
    "chainId": 1,
    "address": "0x03ee5026c07d85ff8ae791370dd0f4c1ae6c97fc",
    "name": "0x Leverage",
    "symbol": "OXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31977/thumb/Ox.jpeg?1696530780"
  },
  {
    "chainId": 1,
    "address": "0xa0117792d4b100fd329b37e8ab4181df8a5b3326",
    "name": "BREPE",
    "symbol": "BREPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31561/thumb/cmc_200x200_logo_transparency.png?1696530374"
  },
  {
    "chainId": 1,
    "address": "0x9532ca064278ce3ba4fcc66cebec6d9f04f58f70",
    "name": "Bot Compiler",
    "symbol": "BOTC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31513/thumb/imageedit_1_8217440916.png?1696530323"
  },
  {
    "chainId": 1,
    "address": "0x0258f474786ddfd37abce6df6bbb1dd5dfc4434a",
    "name": "Orion",
    "symbol": "ORN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11841/thumb/new_logo.jpeg?1696511712"
  },
  {
    "chainId": 1,
    "address": "0x1028083026fe1e8e1e79170ceae0edbad07b052d",
    "name": "MonkCoin",
    "symbol": "MONK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31725/thumb/MonkCoin.jpg?1696530546"
  },
  {
    "chainId": 1,
    "address": "0xe4efdd2eb216a4620cfa55c5cc67bd09dc64ff24",
    "name": " ",
    "symbol": "",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31535/thumb/KB0nEHf__400x400.jpg?1696530344"
  },
  {
    "chainId": 1,
    "address": "0xf6f31b8afbf8e3f7fc8246bef26093f02838da98",
    "name": "Unicorn Metaverse",
    "symbol": "UNIVERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33077/thumb/rar4.JPG?1700549656"
  },
  {
    "chainId": 1,
    "address": "0xa3cb87080e68ad54d00573983d935fa85d168fde",
    "name": "InfinityBit Token",
    "symbol": "IBIT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31562/thumb/ibit-200x200cg.png?1696530375"
  },
  {
    "chainId": 1,
    "address": "0xf47245e9a3ba3dca8b004e34afc1290b1d435a52",
    "name": "Magical Blocks",
    "symbol": "MBLK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33186/thumb/13441700648242_.pic.jpg?1700953813"
  },
  {
    "chainId": 1,
    "address": "0xa946fb6b6b860c68df3c293f1e2c3881b243e08c",
    "name": "Dragon",
    "symbol": "DRAGON",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xb281d84989c06e2a6ccdc5ea7bf1663c79a1c31a",
    "name": "stoicDAO",
    "symbol": "ZETA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33343/thumb/lCisSJAU_400x400.jpg?1701528682"
  },
  {
    "chainId": 1,
    "address": "0x6f8b23296394d20ec048fbdec8ebc0ca90f5c8f1",
    "name": "TUF Token",
    "symbol": "TUF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28410/thumb/2022-12-06_10.10.16.jpg?1696527408"
  },
  {
    "chainId": 1,
    "address": "0x08d0222a206d1aee59a9b66969c04fd1e8a0f864",
    "name": "Momo v2",
    "symbol": "MOMOV2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31515/thumb/momo.jpg?1696530325"
  },
  {
    "chainId": 1,
    "address": "0x07c904d8c04323ef9fe6bf13aaeba05b62c54825",
    "name": "Eeyor",
    "symbol": "EEYOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31565/thumb/200eeyor.png?1696530378"
  },
  {
    "chainId": 1,
    "address": "0x226d6d842d49b4d757bef1632053a198d5d9c8aa",
    "name": "Block Browser",
    "symbol": "BLOCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31538/thumb/photo_2023-08-29_01-46-11.jpg?1696530347"
  },
  {
    "chainId": 1,
    "address": "0xc3ade5ace1bbb033ccae8177c12ecbfa16bd6a9d",
    "name": "Inception swETH",
    "symbol": "INSWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34943/thumb/swETH.png?1706757072"
  },
  {
    "chainId": 1,
    "address": "0x6a1d3d36e1be3da88a2268ebeb1a055758bdd9a9",
    "name": "RFK Coin",
    "symbol": "RFKC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31572/thumb/RFKCoin_100px.png?1696530384"
  },
  {
    "chainId": 1,
    "address": "0xdb81f7b3f0b2baebd5009cddade5c9a9c82378bb",
    "name": "Jjmoji",
    "symbol": "JJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33344/thumb/JJ.jpg?1701529096"
  },
  {
    "chainId": 1,
    "address": "0xfbd5fd3f85e9f4c5e8b40eec9f8b8ab1caaa146b",
    "name": "Treat Token",
    "symbol": "TREAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31518/thumb/treat.jpg?1696530328"
  },
  {
    "chainId": 1,
    "address": "0x19be3a0f1a6cccc99b3cdc13475613e559be551c",
    "name": "FrenBot",
    "symbol": "MEF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31573/thumb/frenbot_logo.png?1697699412"
  },
  {
    "chainId": 1,
    "address": "0x3540abe4f288b280a0740ad5121aec337c404d15",
    "name": "TPRO Network",
    "symbol": "TPRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26694/thumb/tpro-logo-200x200.png?1712870769"
  },
  {
    "chainId": 1,
    "address": "0x06d6f0dd6703a1cfe16025dcc55f36f017887627",
    "name": "Montage Token",
    "symbol": "MTGX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36903/thumb/Montage-200x200.png?1712699912"
  },
  {
    "chainId": 1,
    "address": "0xdcc97d2c1048e8f3f2fc58ace9024ab8b350e4b1",
    "name": "BENCOIN",
    "symbol": "BEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32701/thumb/BEN_COIN_BACK_%281%29.png?1698985208"
  },
  {
    "chainId": 1,
    "address": "0xc0804a4eae02fdf7516f55a12990d282640a4961",
    "name": "OpenPool",
    "symbol": "OPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31541/thumb/0xc0804a4eae02fdf7516f55a12990d282640a4961.png?1696530354"
  },
  {
    "chainId": 1,
    "address": "0xd1b5651e55d4ceed36251c61c50c889b36f6abb5",
    "name": "Stake DAO CRV",
    "symbol": "SDCRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27756/thumb/scCRV-2.png?1696526779"
  },
  {
    "chainId": 1,
    "address": "0x8929e9dbd2785e3ba16175e596cdd61520fee0d1",
    "name": "Altitude",
    "symbol": "ALTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30114/thumb/logo.png?1696529036"
  },
  {
    "chainId": 1,
    "address": "0xd561a593d9dd8b9a0e3a487dfb517c9371d6dda7",
    "name": "Meow Meme",
    "symbol": "MEOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36128/thumb/Meow_Meme_Coin.png?1710579427"
  },
  {
    "chainId": 1,
    "address": "0x8353b92201f19b4812eee32efd325f7ede123718",
    "name": "ScamFari",
    "symbol": "SCM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31267/thumb/Ava_Scamfari_%281%29.png?1696530091"
  },
  {
    "chainId": 1,
    "address": "0x8df60cfc67c1bf51fe3dd61b34b156f14f538f5d",
    "name": "Wojak 2 69",
    "symbol": "WOJAK269",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31660/thumb/IMG_9266.jpeg?1696530477"
  },
  {
    "chainId": 1,
    "address": "0x8423b76be8ef6ca7400a6b4c334d29c1d5d4572c",
    "name": "HarryPotterObamaInu",
    "symbol": "INU",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31607/thumb/harry_logo.png?1696530423"
  },
  {
    "chainId": 1,
    "address": "0xca7013ba4bf76bcdc3ffc71735896682644f47c2",
    "name": "Degen",
    "symbol": "DGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31606/thumb/Degen.png?1696530422"
  },
  {
    "chainId": 1,
    "address": "0x75ce16d11b83605aa039d40d7d846ff23064fb65",
    "name": "DUBX",
    "symbol": "DUB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31605/thumb/dubx200*200.png?1696530421"
  },
  {
    "chainId": 1,
    "address": "0x8e01397163b21f64cec1f06ca6cc7d9aa8f718e9",
    "name": "LIF3 LSHARE",
    "symbol": "LSHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31575/thumb/LSHARE.png?1696530392"
  },
  {
    "chainId": 1,
    "address": "0x2e19067cbeb38d6554d31a1a83aefc4018a1688a",
    "name": "Coinback",
    "symbol": "CBK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31576/thumb/F1090BAA-54E3-4F9A-AFAA-57BA89E144CB.PNG?1696530393"
  },
  {
    "chainId": 1,
    "address": "0xcb27e0b9530d5107302e3e0691dd0f64faf498b0",
    "name": "Pepe Of Wallstreet",
    "symbol": "POW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31579/thumb/photo_2023-09-04_15-13-35_%281%29.jpg?1696530396"
  },
  {
    "chainId": 1,
    "address": "0x1142866f451d9d5281c5c8349a332bd338e552a1",
    "name": "SuperMarioPorsche911Inu",
    "symbol": "SILKROAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31649/thumb/smp911i_logo400x400_%283%29.jpg?1696530464"
  },
  {
    "chainId": 1,
    "address": "0xe925aa77d51746b865e5c05165a879820cb4b720",
    "name": "C Cash",
    "symbol": "CCASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31644/thumb/200x.png?1696530460"
  },
  {
    "chainId": 1,
    "address": "0xf0f67ddb45c90946d717f2225910cb940c28afae",
    "name": "Memes Casino",
    "symbol": "MEMES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32398/thumb/F80JpRIaoAA0ug6-min.png?1698054612"
  },
  {
    "chainId": 1,
    "address": "0x7aa2f174fbc4d0a17b34adfb9b3e1dc029b46d76",
    "name": "RADA Foundation",
    "symbol": "RADA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31645/thumb/1.png?1696530461"
  },
  {
    "chainId": 1,
    "address": "0xc47ef9b19c3e29317a50f5fbe594eba361dada4a",
    "name": "Edelcoin",
    "symbol": "EDLC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31621/thumb/edelcoin_logo_200_%281%29.png?1696530437"
  },
  {
    "chainId": 1,
    "address": "0x6d8f7ac13e8c2944d27e0f9bc6aa0122f2ec1384",
    "name": "SuperMarket",
    "symbol": "SUPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31623/thumb/SuperMarket.png?1696530439"
  },
  {
    "chainId": 1,
    "address": "0x24edded3f03abb2e9d047464294133378bddb596",
    "name": "SECT BOT",
    "symbol": "SECT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33525/thumb/IMG_7387.png?1702138186"
  },
  {
    "chainId": 1,
    "address": "0x9f891b5ecbd89dd8a5ee4d1d80efc3fe78b306cb",
    "name": "SONIK",
    "symbol": "SONIK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31693/thumb/Sonik_Logo_%281%29.png?1696530511"
  },
  {
    "chainId": 1,
    "address": "0x29d7139271398d0c2e22523fda06e023dcb07f8f",
    "name": "KlubCoin",
    "symbol": "KLUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32869/thumb/klub.jpg?1701141365"
  },
  {
    "chainId": 1,
    "address": "0x4e452b391a86c9240e98df7277ce0bea5be08e43",
    "name": "Pax Unitas",
    "symbol": "PAXU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31648/thumb/IMG_11C5EA8BD781-1.jpeg?1696530463"
  },
  {
    "chainId": 1,
    "address": "0x5ebffc53c3b8a71120789a211f9b91c79c78d1e0",
    "name": "SOJUDAO",
    "symbol": "SOJU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31627/thumb/1.png?1696530443"
  },
  {
    "chainId": 1,
    "address": "0x94be6962be41377d5beda8dfe1b100f3bf0eacf3",
    "name": "DORK LORD",
    "symbol": "DORKL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31654/thumb/%E1%97%AAO%E1%96%87K_%E1%92%AAO%E1%96%87%E1%97%AA.png?1696530469"
  },
  {
    "chainId": 1,
    "address": "0x9b0e1c344141fb361b842d397df07174e1cdb988",
    "name": "EmotiCoin",
    "symbol": "EMOTI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31630/thumb/200x200_logo_1.png?1696530446"
  },
  {
    "chainId": 1,
    "address": "0x3294395e62f4eb6af3f1fcf89f5602d90fb3ef69",
    "name": "Celo  Wormhole ",
    "symbol": "CELO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32400/thumb/celo_wh.png?1698055352"
  },
  {
    "chainId": 1,
    "address": "0xbb63e6be33bc5b5386d7ab0529dc6c400f2ac2ec",
    "name": "Cuckadoodledoo",
    "symbol": "CUCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31631/thumb/Cuckman__%281%29.png?1696530447"
  },
  {
    "chainId": 1,
    "address": "0x3593d125a4f7849a1b059e64f4517a86dd60c95d",
    "name": "MANTRA",
    "symbol": "OM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12151/thumb/OM_Token.png?1696511991"
  },
  {
    "chainId": 1,
    "address": "0x6fdb90535c09b82825e38d41edf5e66211d4b442",
    "name": "Yield Magnet",
    "symbol": "MAGNET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32821/thumb/IMG_0410.jpeg?1699582097"
  },
  {
    "chainId": 1,
    "address": "0x7d7513bba1af295574c24bbe425873d0c431ff9f",
    "name": "Worldcore coin",
    "symbol": "WCC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35511/thumb/1000956327.png?1709637554"
  },
  {
    "chainId": 1,
    "address": "0x890a60b4450484847380bddca0c3c3ee2ff0dfbe",
    "name": "X Ratio AI",
    "symbol": "XRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36130/thumb/200x200cmc.png?1710579759"
  },
  {
    "chainId": 1,
    "address": "0x96add417293a49e80f024734e96cfd8b355bcc14",
    "name": "LiquidLayer",
    "symbol": "LILA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32547/thumb/photo_2023-10-08_12-09-02.jpg?1698474657"
  },
  {
    "chainId": 1,
    "address": "0x3adf095d04311d7bf05c838f0d3dc34a83d81ab6",
    "name": "SHILLD",
    "symbol": "SHILLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31675/thumb/200x200.png?1696530491"
  },
  {
    "chainId": 1,
    "address": "0x3ec15c4745e21ab3831d1f51c492e3b5582d6239",
    "name": "PICKLE",
    "symbol": "PICKLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31717/thumb/ezgif.com-resize_%287%29.png?1696530538"
  },
  {
    "chainId": 1,
    "address": "0x37dba54fdc402aff647ce06c66972f5d662c326d",
    "name": "MELON",
    "symbol": "MELON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31701/thumb/Melon_Icon_200px_2.png?1696530525"
  },
  {
    "chainId": 1,
    "address": "0x9863bcc2fb23dfdf5fe275aa4c5575a32a580911",
    "name": "PEPURAI",
    "symbol": "PEPURAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31702/thumb/Pepurai-_LOGO_%282%29.png?1696530526"
  },
  {
    "chainId": 1,
    "address": "0x75c97384ca209f915381755c582ec0e2ce88c1ba",
    "name": "FINE",
    "symbol": "FINE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31703/thumb/05220a_054ff0bb5d7c41db99db6332923ac4fd_mv2.png?1696530527"
  },
  {
    "chainId": 1,
    "address": "0xa9049425b938c46ac3e312d4cdaeccb26282aeb2",
    "name": "WickedBet Casino",
    "symbol": "WIK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31721/thumb/WB_B_W.png?1696530542"
  },
  {
    "chainId": 1,
    "address": "0x69d29f1b0cc37d8d3b61583c99ad0ab926142069",
    "name": "Pepe Inverted",
    "symbol": "",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31948/thumb/photo_2023-09-25_14-05-49.jpg?1696530754"
  },
  {
    "chainId": 1,
    "address": "0x59d9356e565ab3a36dd77763fc0d87feaf85508c",
    "name": "Mountain Protocol USD",
    "symbol": "USDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31719/thumb/usdm.png?1696530540"
  },
  {
    "chainId": 1,
    "address": "0x6213f40e00f4595aa038fa710e3f837b492d6757",
    "name": "The Pond",
    "symbol": "THEPOND",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/31846/thumb/The_Pond_Token.png?1696530660"
  },
  {
    "chainId": 1,
    "address": "0x7d51888c5abb7cdfa9cdd6a50673c7f8afaccd7f",
    "name": "DuckDAO",
    "symbol": "DD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33325/thumb/duckdao_dd_token_icon.png?1701437826"
  },
  {
    "chainId": 1,
    "address": "0xa40640458fbc27b6eefedea1e9c9e17d4cee7a21",
    "name": "Anchored Coins AEUR",
    "symbol": "AEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33469/thumb/aeur-icon2.png?1701944612"
  },
  {
    "chainId": 1,
    "address": "0xddcc69879e1d2376ce799051afa98c689f234cca",
    "name": "SmartMoney",
    "symbol": "SMRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35280/thumb/Smartmoney.jpeg?1708054880"
  },
  {
    "chainId": 1,
    "address": "0x5d6310984776583f58d29e45340f14ad3e8bbb98",
    "name": "Viralsniper",
    "symbol": "VIRAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31756/thumb/Viralsniper.png?1696530575"
  },
  {
    "chainId": 1,
    "address": "0x1063181dc986f76f7ea2dd109e16fc596d0f522a",
    "name": "Cybria",
    "symbol": "CYBA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31694/thumb/Cybria_Logo_200x200.png?1696530512"
  },
  {
    "chainId": 1,
    "address": "0x485c665d8eeaca7987c48057ffd0591e2aaa797e",
    "name": "Metamundo",
    "symbol": "MMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31724/thumb/batch_MM_logo.png?1696530545"
  },
  {
    "chainId": 1,
    "address": "0x4ba942d56a3b95b03c0af7a91b7294e0e95271f4",
    "name": "Super Cycle",
    "symbol": "RICH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32754/thumb/SUPERCYCLE.png?1699265229"
  },
  {
    "chainId": 1,
    "address": "0xadd8abdea5cb95f4dcd8e128eeef64f023615a6a",
    "name": "Cheezburger",
    "symbol": "CHZB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32048/thumb/20230930_071301.jpg?1696530845"
  },
  {
    "chainId": 1,
    "address": "0xbc8b7ff89eb2b73ecdd579f81424a3b582200808",
    "name": "Curve Inu",
    "symbol": "CRVY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31759/thumb/0x7ojNgl_400x400.jpg?1696530578"
  },
  {
    "chainId": 1,
    "address": "0x03049b395147713ae53c0617093675b4b86dde78",
    "name": "BobaCat",
    "symbol": "PSPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31777/thumb/cg.png?1696530595"
  },
  {
    "chainId": 1,
    "address": "0x0c21638d4bcb88568f88bc84a50e317715f8de8a",
    "name": "GrokDogeX",
    "symbol": "GDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32756/thumb/GDX.jpg?1699280011"
  },
  {
    "chainId": 1,
    "address": "0x55a8f6c6b3aa58ad6d1f26f6afeded78f32e19f4",
    "name": "Aegis Ai",
    "symbol": "AEGIS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33169/thumb/UXDa3TDq_400x400.jpg?1700909114"
  },
  {
    "chainId": 1,
    "address": "0x0d99482bb2c436b7183e7bd474cd65fa8711c8ec",
    "name": "NEO PEPE",
    "symbol": "NEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31733/thumb/NEPE.png?1696530553"
  },
  {
    "chainId": 1,
    "address": "0x428487ad75dc9197cccb496629ab516974329dc1",
    "name": "RealAliensEnjoyingLiquidity",
    "symbol": "RAEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31735/thumb/rael_logo_%282%29.jpg?1696530554"
  },
  {
    "chainId": 1,
    "address": "0xdf2c7238198ad8b389666574f2d8bc411a4b7428",
    "name": "Mainframe",
    "symbol": "MFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3293/thumb/Mainframe_Token_Logo.png?1696504003"
  },
  {
    "chainId": 1,
    "address": "0x69d57d410e72eb21fb83fa3dbf090ba07dbe0e76",
    "name": "Ante Casino",
    "symbol": "CHANCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31778/thumb/Chance_Token.jpg?1696530596"
  },
  {
    "chainId": 1,
    "address": "0x6d47a7be5d410910eef7ee2a3d34931f98f36812",
    "name": "INDEX20",
    "symbol": "I20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31737/thumb/200px.jpg?1696530556"
  },
  {
    "chainId": 1,
    "address": "0x4d9b0b7a6db042cb990d36a0df5aa2960e552f16",
    "name": "Hpohs888inu",
    "symbol": "TETHER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31355/thumb/200xlogo.png?1696530172"
  },
  {
    "chainId": 1,
    "address": "0xdefcafe7eac90d31bbba841038df365de3c4e207",
    "name": "0xDEFCAFE",
    "symbol": "CAFE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31738/thumb/200x200.png?1696530557"
  },
  {
    "chainId": 1,
    "address": "0x93999d3fcab15cc052cf96b739580fc11e015944",
    "name": "NoGas",
    "symbol": "NGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34049/thumb/0x93999d3fcab15cc052cf96b739580fc11e015944_%281%29.jpg?1703733938"
  },
  {
    "chainId": 1,
    "address": "0x5fe72ed557d8a02fff49b3b826792c765d5ce162",
    "name": "Shezmu",
    "symbol": "SHEZMU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31772/thumb/ShezmuPlainblk.png?1696530590"
  },
  {
    "chainId": 1,
    "address": "0x503cd987998824192578d0d7950148445667287c",
    "name": "FOGnet",
    "symbol": "FOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31743/thumb/fognet_logo_512-pso36yibc3wmls8sg2l5arxqptos6g1glqowa3rgik.png?1696530562"
  },
  {
    "chainId": 1,
    "address": "0x2c056f9402a0627bc0e580365bb12979fc011e2c",
    "name": "Squid Game 2 0",
    "symbol": "SQUID2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30932/thumb/SQUID2_LOGO_%28200x200%29.png?1696529774"
  },
  {
    "chainId": 1,
    "address": "0xbfabde619ed5c4311811cf422562709710db587d",
    "name": "Diva Staking",
    "symbol": "DIVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31792/thumb/2024-02-17_11.56.27.jpg?1709885558"
  },
  {
    "chainId": 1,
    "address": "0xd115a61a25c059c78cd34b97fab7ab25dfc84bc3",
    "name": "Jesus",
    "symbol": "RAPTOR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30745/thumb/Favicon_%281%29.png?1696529615"
  },
  {
    "chainId": 1,
    "address": "0x4c45bbec2ff7810ef4a77ad7bd4757c446fe4155",
    "name": "Jungle Labz",
    "symbol": "JNGL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31742/thumb/Jungle.png?1696530561"
  },
  {
    "chainId": 1,
    "address": "0x2de7b02ae3b1f11d51ca7b2495e9094874a064c0",
    "name": "SHIB2",
    "symbol": "SHIB2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31776/thumb/IMG_6344_2.jpg?1696530594"
  },
  {
    "chainId": 1,
    "address": "0x33349b282065b0284d756f0577fb39c158f935e6",
    "name": "Maple",
    "symbol": "MPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14097/thumb/Maple_Logo_Mark_Maple_Orange.png?1696513819"
  },
  {
    "chainId": 1,
    "address": "0x18a3563c21062897950bb09339c82b9686a35667",
    "name": "EtherNexus",
    "symbol": "ENXS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29855/thumb/logo-circle.png?1696528781"
  },
  {
    "chainId": 1,
    "address": "0x089729b0786c8803cff972c16e402f3344d079ea",
    "name": "BlockGPT",
    "symbol": "BGPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30872/thumb/BLOCKEM_%282%29.png?1707925988"
  },
  {
    "chainId": 1,
    "address": "0xcdd0d11de0225b528b3a20d6436392c8260969d0",
    "name": "Trade Bionic",
    "symbol": "ONIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32357/thumb/9ca5f792-9bee-4d1f-942b-7ef09234510d_2.JPG?1697476642"
  },
  {
    "chainId": 1,
    "address": "0xde47a2460e4b6c36b26919ef9255b4f3f86de0a0",
    "name": "OoLong",
    "symbol": "",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32627/thumb/rabbit2_200x200.png?1698822914"
  },
  {
    "chainId": 1,
    "address": "0xd27b128dc6536309cdebf7f1aff0cb7717bc0268",
    "name": "Etherempires",
    "symbol": "ETE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31804/thumb/IMG_20230919_125717_505.png?1696530619"
  },
  {
    "chainId": 1,
    "address": "0x1af4f195a8aaa7ffd752c28a13b719b84056f0d6",
    "name": "NFT Track Protocol",
    "symbol": "NTP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27108/thumb/NTP_Symbol_CoinGecko.jpg?1696526161"
  },
  {
    "chainId": 1,
    "address": "0x6f259637dcd74c767781e37bc6133cd6a68aa161",
    "name": "Huobi",
    "symbol": "HT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2822/thumb/huobi-token-logo.png?1696503584"
  },
  {
    "chainId": 1,
    "address": "0xcc09f34accdb36ee3ed98358a3b8a6ae5c29ea07",
    "name": "Anon Web3",
    "symbol": "AW3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31814/thumb/IMG_2336.png?1696530628"
  },
  {
    "chainId": 1,
    "address": "0x339e6c8d204b1aaa3fb74bc7ba15b9fb6648b7d2",
    "name": "Omega Cloud",
    "symbol": "OMEGA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35930/thumb/2024-03-10_00.16.57.jpg?1710225569"
  },
  {
    "chainId": 1,
    "address": "0xcdeb595293511115d9d9d44b189cc0da4a08cfaf",
    "name": "Jellyfish Mobile",
    "symbol": "JFISH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31879/thumb/jelly-fish-mix-white_%282%29.png?1696530691"
  },
  {
    "chainId": 1,
    "address": "0x5faa989af96af85384b8a938c2ede4a7378d9875",
    "name": "Galxe",
    "symbol": "GAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24530/thumb/galaxy.jpg?1696523708"
  },
  {
    "chainId": 1,
    "address": "0xd22a61e8503bea5842e5e0126ca9ffc4dd492084",
    "name": "Strawberry Elephant",
    "symbol": "",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31819/thumb/photo_2023-09-15_23-44-10.jpg?1696530633"
  },
  {
    "chainId": 1,
    "address": "0x561cf9121e89926c27fa1cfc78dfcc4c422937a4",
    "name": "Squid Game",
    "symbol": "SQUID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33074/thumb/photo_2023-11-17_14.18.23.jpeg?1700542677"
  },
  {
    "chainId": 1,
    "address": "0x9e18d5bab2fa94a6a95f509ecb38f8f68322abd3",
    "name": "AMATERASU OMIKAMI",
    "symbol": "OMIKAMI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31821/thumb/Omikami_BTC_AMZN.png?1696530635"
  },
  {
    "chainId": 1,
    "address": "0x5da151b95657e788076d04d56234bd93e409cb09",
    "name": "OTSea",
    "symbol": "OTSEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31900/thumb/logo200x200.png?1712254528"
  },
  {
    "chainId": 1,
    "address": "0xc81b0d02393a956234e24564805a896f14ad1250",
    "name": "BeautifulPrincessDisorder",
    "symbol": "BPD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31824/thumb/6RGuGvnz_400x400.jpg?1696530638"
  },
  {
    "chainId": 1,
    "address": "0xce3ee7435a5bedbe73b92f39828b0cfd9d0ff568",
    "name": "UniDexAI",
    "symbol": "UDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35571/thumb/IMG_2678.png?1709193542"
  },
  {
    "chainId": 1,
    "address": "0xe2cfd7a01ec63875cd9da6c7c1b7025166c2fa2f",
    "name": "Hyper",
    "symbol": "HYPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35931/thumb/C046D3BF-912C-4FE5-AEAD-6F1F2093AD92.png?1710225712"
  },
  {
    "chainId": 1,
    "address": "0x52d134c6db5889fad3542a09eaf7aa90c0fdf9e4",
    "name": "Backed IBTA   Treasury Bond 1 3yr",
    "symbol": "BIBTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31880/thumb/b-IBTA-200x200.png?1696530692"
  },
  {
    "chainId": 1,
    "address": "0x8bf30e9f44e5d068a9d0c20da22660997a532e33",
    "name": "GhostDAG org",
    "symbol": "GDAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33152/thumb/GD256.png?1700825767"
  },
  {
    "chainId": 1,
    "address": "0xc434268603ca8854e0be1a3ff15cad73bd6ec49a",
    "name": "Zapicorn",
    "symbol": "ZAPI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33153/thumb/download.png?1700825821"
  },
  {
    "chainId": 1,
    "address": "0x162433c934aa74ba147e05150b1206b2c922f71d",
    "name": "Creso  OLD ",
    "symbol": "CRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32541/thumb/CRESO.png?1698473415"
  },
  {
    "chainId": 1,
    "address": "0x55b2cfcfe99110c773f00b023560dd9ef6c8a13b",
    "name": "Index Coop CoinDesk ETH Trend Index",
    "symbol": "CDETI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32358/thumb/cdETI-logo.png?1697940129"
  },
  {
    "chainId": 1,
    "address": "0x7d3b4f8d5dd14a0c263c4bee7be434c15e188d3e",
    "name": "Moe",
    "symbol": "MOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32680/thumb/moe.png?1698934350"
  },
  {
    "chainId": 1,
    "address": "0x182c00807f2d4904d02d5e0d179600ff6a3ae67c",
    "name": "BTEX",
    "symbol": "BTEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31884/thumb/BTEX_200x200.png?1696530695"
  },
  {
    "chainId": 1,
    "address": "0xb48361032956d52f1ade21c73c273644f4ea24ca",
    "name": "MOONERIUM",
    "symbol": "MOONERIUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33081/thumb/200.png?1700552508"
  },
  {
    "chainId": 1,
    "address": "0xd6d9f4385bff8670d66a708b258969913619a673",
    "name": "LEETCoin",
    "symbol": "LEET",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32934/thumb/logo100.png?1699870536"
  },
  {
    "chainId": 1,
    "address": "0x244748587f145c0571e92e85429d711d1b4cc43a",
    "name": "Chooky",
    "symbol": "CHOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31833/thumb/Chooky_V2.png?1707776702"
  },
  {
    "chainId": 1,
    "address": "0x00c8555542158fff0feb892c8e000a8d1831762c",
    "name": "Mori Finance",
    "symbol": "MORI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31885/thumb/MORI.jpg?1696530696"
  },
  {
    "chainId": 1,
    "address": "0x41ea5d41eeacc2d5c4072260945118a13bb7ebce",
    "name": "Creso",
    "symbol": "CRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32954/thumb/CRE.jpg?1699941230"
  },
  {
    "chainId": 1,
    "address": "0xa1db57defd15be659cfbc612a13195adf5b237c6",
    "name": "ManaCoin",
    "symbol": "MNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31906/thumb/social-update.png?1696530715"
  },
  {
    "chainId": 1,
    "address": "0x9b5c38cc2d1ba05ed87c8f8a2418475bacb20073",
    "name": "Vector Space Biosciences  Inc ",
    "symbol": "SBIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31837/thumb/Hex.png?1696530651"
  },
  {
    "chainId": 1,
    "address": "0x1b19c19393e2d034d8ff31ff34c81252fcbbee92",
    "name": "OUSG",
    "symbol": "OUSG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29023/thumb/OUSG.png?1696527993"
  },
  {
    "chainId": 1,
    "address": "0x13832bec9b7029a82988017a7f6095bdf0207d62",
    "name": "Milk",
    "symbol": "MILK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32681/thumb/output-onlinepngtools-2.png?1698934572"
  },
  {
    "chainId": 1,
    "address": "0xcc2d17dfa245dc2aa3705fc75d2f7df3fe6440c7",
    "name": "Reserve",
    "symbol": "RSRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32672/thumb/download.png?1698910754"
  },
  {
    "chainId": 1,
    "address": "0xb62e24b747eaa41454857cf6011832117df59cb8",
    "name": "Epiko",
    "symbol": "EPIKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31909/thumb/EPIKO_Light_200x200.png?1703118804"
  },
  {
    "chainId": 1,
    "address": "0xcfb65079138878b9dd29b99c151b286464f81276",
    "name": "Bobi",
    "symbol": "BOBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32682/thumb/200F9d1nhqWwAA4M5r.jpeg?1698936285"
  },
  {
    "chainId": 1,
    "address": "0x2df7d0e4903029717c949cad204075a3d75c8806",
    "name": "Sweet",
    "symbol": "SWEET",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31959/thumb/0x2df7d0e4903029717c949cad204075a3d75c8806.png?1696530765"
  },
  {
    "chainId": 1,
    "address": "0x697a79af2de4af9e9aa0d08905374556ad1353bb",
    "name": "NinaPumps",
    "symbol": "NINA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31910/thumb/pqT9YDIi_400x400-removebg-preview_%281%29.png?1696530718"
  },
  {
    "chainId": 1,
    "address": "0x0176b898e92e814c06cc379e508ceb571f70bd40",
    "name": "Tipcoin",
    "symbol": "TIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31845/thumb/tip.png?1696530659"
  },
  {
    "chainId": 1,
    "address": "0xa6345ffadfa23dfc9014bce72ff2fa8712e54231",
    "name": "Pepe Prophet",
    "symbol": "KEK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31932/thumb/20231009_185537.png?1696963573"
  },
  {
    "chainId": 1,
    "address": "0xa1f830aa68b53fd3ee3bb86d7f8254e604740c8b",
    "name": "ChainSwap",
    "symbol": "CHAINS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32657/thumb/logo-cs_%281%29.png?1698899704"
  },
  {
    "chainId": 1,
    "address": "0x3419875b4d3bca7f3fdda2db7a476a79fd31b4fe",
    "name": "DizzyHavoc",
    "symbol": "DZHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35575/thumb/dizzyhavoc_circle_3_200_200_transparentBg.png?1709194317"
  },
  {
    "chainId": 1,
    "address": "0x5713c26280647adad2f25bb54376943ecaa9d8e3",
    "name": "Elon Xmas",
    "symbol": "XMAS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33721/thumb/Elon_Xmas.png?1702885377"
  },
  {
    "chainId": 1,
    "address": "0x1abaea1f7c830bd89acc67ec4af516284b1bc33c",
    "name": "EURC",
    "symbol": "EURC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26045/thumb/euro.png?1696525125"
  },
  {
    "chainId": 1,
    "address": "0x83e817e1574e2201a005ec0f7e700ed5606f555e",
    "name": "mPendle",
    "symbol": "MPENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31918/thumb/mPendle.png?1696530726"
  },
  {
    "chainId": 1,
    "address": "0xacc48f65f8701f376bc4ba13f4bc103f31dda74e",
    "name": "Swift",
    "symbol": "SWIFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31979/thumb/Safeimagekit-resized-img_%284%29.png?1696530782"
  },
  {
    "chainId": 1,
    "address": "0x5e27e384acbba20982f991893b9970aaf3f43181",
    "name": "Tyrion finance",
    "symbol": "TYRION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31856/thumb/tyrionlogo.jpg?1696530669"
  },
  {
    "chainId": 1,
    "address": "0x1903be033d3e436dd79a8cf9030675bcf97ab589",
    "name": "Be ikta ",
    "symbol": "BJK",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32038/thumb/BJK_200x200_2.png?1696530835"
  },
  {
    "chainId": 1,
    "address": "0x76af4cb74c8d4da51403d672a799e94b5958c230",
    "name": "Feels Good Man",
    "symbol": "GOOD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31980/thumb/Feels_Good_Man.png?1696530783"
  },
  {
    "chainId": 1,
    "address": "0x901ea3606d567f9f1e964639d5cbb8659080be8a",
    "name": "CoinW",
    "symbol": "CWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31999/thumb/cwt_%281%29.jpeg?1696530800"
  },
  {
    "chainId": 1,
    "address": "0x40e9187078032afe1a30cfcf76e4fe3d7ab5c6c5",
    "name": "AIgentX",
    "symbol": "AIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31860/thumb/logo22.jpg?1698917072"
  },
  {
    "chainId": 1,
    "address": "0xddf688e96cb2531a69bf6347c02f069266c1aa81",
    "name": "MEMEVENGERS",
    "symbol": "MMVG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32000/thumb/MMVG.jpg?1696530801"
  },
  {
    "chainId": 1,
    "address": "0x32fd949e1953b21b7a8232ef4259cd708b4e0847",
    "name": "HyperBC",
    "symbol": "HBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32003/thumb/hbt_%281%29.png?1696530803"
  },
  {
    "chainId": 1,
    "address": "0x653aab62056b92641116d63927de6141d780e596",
    "name": "Anchored Coins ACHF",
    "symbol": "ACHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33471/thumb/logo_achf.png?1701946601"
  },
  {
    "chainId": 1,
    "address": "0x667210a731447f8b385e068205759be2311b86d4",
    "name": "ETF The Token",
    "symbol": "ETF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31892/thumb/%E2%96%B2_%282%29.png?1696530703"
  },
  {
    "chainId": 1,
    "address": "0x64c79c8c59a2be17c8d651f73e5ee7942eebdc9e",
    "name": "Elon Cat",
    "symbol": "SCHRODINGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31893/thumb/png_%285%29_%281%29.png?1696530704"
  },
  {
    "chainId": 1,
    "address": "0xf1102d6d2a531124fa043d18a06c394a81aaa866",
    "name": "Shiba Inu Classic",
    "symbol": "SHIBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31941/thumb/Shiba_Inu_Classic.png?1696530748"
  },
  {
    "chainId": 1,
    "address": "0x38f9bb135ea88033f4377b9ea0fb5cfb773fec2f",
    "name": "Alpha Shards",
    "symbol": "ALPHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32002/thumb/alpha_%281%29.jpeg?1696530802"
  },
  {
    "chainId": 1,
    "address": "0x4eea955f63d7e24ea7272651a29c7c70f7c2a9ae",
    "name": "EastGate Pharmaceuticals",
    "symbol": "EGP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31866/thumb/IMG_20230921_002259_489.jpg?1696530678"
  },
  {
    "chainId": 1,
    "address": "0x34ba042827996821cffeb06477d48a2ff9474483",
    "name": "Shib2 0",
    "symbol": "SHIB20",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30940/thumb/unnamed.jpg?1696529779"
  },
  {
    "chainId": 1,
    "address": "0x20561172f791f915323241e885b4f7d5187c36e1",
    "name": "Calcium",
    "symbol": "CAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31971/thumb/callogo.jpg?1696530776"
  },
  {
    "chainId": 1,
    "address": "0x0e74a58557555cb44415df5a960d9eb14e96142a",
    "name": "VegasBot",
    "symbol": "VEGAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31983/thumb/logoblk.jpg?1696530786"
  },
  {
    "chainId": 1,
    "address": "0xda30f261a962d5aae94c9ecd170544600d193766",
    "name": "Orbler",
    "symbol": "ORBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24056/thumb/orbr200.png?1696523246"
  },
  {
    "chainId": 1,
    "address": "0x642ac912a58428767fa14a26a749f9a1b001ba92",
    "name": "XX",
    "symbol": "XX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31972/thumb/200.png?1696530777"
  },
  {
    "chainId": 1,
    "address": "0x9f5e508182e1cbd23ea5ef65d1d6c342beb7d6d3",
    "name": " JONES",
    "symbol": "JONES",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34588/thumb/ALEX_JONES_LOGO_200px_CoinGecko.jpg?1705425283"
  },
  {
    "chainId": 1,
    "address": "0x015628ce9150db1bce2fbb717a09e846f8a32436",
    "name": "Big Bonus Coin  ETH ",
    "symbol": "BBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32660/thumb/BBC-ETH-logo.png?1698900727"
  },
  {
    "chainId": 1,
    "address": "0x2ef8a2ccb058915e00e16aa13cc6e36f19d8893b",
    "name": "ValiDAO",
    "symbol": "VDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33748/thumb/200x200.png?1712694763"
  },
  {
    "chainId": 1,
    "address": "0x242914f264d44e4c5ead31092ec133dcb6c4d73f",
    "name": "Share Friend",
    "symbol": "SFRIEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32407/thumb/photo_2023-10-17_08-34-30.jpg?1698057000"
  },
  {
    "chainId": 1,
    "address": "0xeb935deb517e4c2abc282e5e251ed4d05db79e93",
    "name": "Fort Block Games",
    "symbol": "FBG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35354/thumb/photo_2024-02-18_10-27-38.jpg?1708360479"
  },
  {
    "chainId": 1,
    "address": "0x0b38210ea11411557c13457d4da7dc6ea731b88a",
    "name": "API3",
    "symbol": "API3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13256/thumb/api3.jpg?1696513031"
  },
  {
    "chainId": 1,
    "address": "0xd8c0b13b551718b808fc97ead59499d5ef862775",
    "name": "Gala Music",
    "symbol": "MUSIC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/33472/thumb/gmusic.jpeg?1701947141"
  },
  {
    "chainId": 1,
    "address": "0x356e17967206efb413b60ab0ba44e269063a26c9",
    "name": "Of Course I Still Love You",
    "symbol": "OCISLY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32035/thumb/IMG_20230927_151428_156_%281%29.jpg?1696530831"
  },
  {
    "chainId": 1,
    "address": "0xfd4ca4a692f14d88af3e7ae13cf00d5095213b25",
    "name": "Wiskers",
    "symbol": "WSKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32124/thumb/1.png?1696587384"
  },
  {
    "chainId": 1,
    "address": "0x0058c8581b9fed6864faa654505bc89890cdb2dd",
    "name": "BabySmurf9000",
    "symbol": "BS9000",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32088/thumb/logo2.png?1701897911"
  },
  {
    "chainId": 1,
    "address": "0x79c6ffe2ccbca761e9e289a69432bffb0b744876",
    "name": "Pineapple Owl",
    "symbol": "PINEOWL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32089/thumb/Pineapple_Owl_Logo_Alt.png?1696530887"
  },
  {
    "chainId": 1,
    "address": "0xfd8b9ba4845fb38c779317ec134b298c064937a2",
    "name": "9inch ETH",
    "symbol": "9INCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32663/thumb/9INCH_LOGO_200px.png?1699299985"
  },
  {
    "chainId": 1,
    "address": "0xa9c1eae6b76c09f84a89ff785ee4001a2b7294ce",
    "name": "EchoBlock",
    "symbol": "EBLOCK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36788/thumb/photo_2024-03-26_06-32-10.jpg?1712416695"
  },
  {
    "chainId": 1,
    "address": "0x4730fb1463a6f1f44aeb45f6c5c422427f37f4d0",
    "name": "FOUR",
    "symbol": "FOUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3434/thumb/FOUR.png?1696504127"
  },
  {
    "chainId": 1,
    "address": "0xfd1450a131599ff34f3be1775d8c8bf79e353d8c",
    "name": "Shiba",
    "symbol": "SHIBA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32299/thumb/IMG_20231011_081456_897.jpg?1697186075"
  },
  {
    "chainId": 1,
    "address": "0x6f91d21de4e40b3b80636b6b3ba425b636b798cf",
    "name": "Wall Street Bets",
    "symbol": "WSB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32095/thumb/GOoZ5azT_400x400.jpg?1696530893"
  },
  {
    "chainId": 1,
    "address": "0x8d85b9553896bd8159e608cb958628e01ed10916",
    "name": "STONED",
    "symbol": "STONED",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32108/thumb/TemA4ZCP_400x400.jpg?1696530904"
  },
  {
    "chainId": 1,
    "address": "0xa9d54f37ebb99f83b603cc95fc1a5f3907aaccfd",
    "name": "Pikaboss",
    "symbol": "PIKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32061/thumb/logocg.png?1696530858"
  },
  {
    "chainId": 1,
    "address": "0x87fdd1e031b356bdc1c55a3231cfe266552d8284",
    "name": "PBM Coin",
    "symbol": "PBMC",
    "decimals": 14,
    "logoURI": "https://assets.coingecko.com/coins/images/32063/thumb/PBM_Coin_%28Sep_23%29.png?1696530860"
  },
  {
    "chainId": 1,
    "address": "0x1cfa5641c01406ab8ac350ded7d735ec41298372",
    "name": "Convertible JPY Token",
    "symbol": "CJPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31356/thumb/CJPY_logo.png?1696530173"
  },
  {
    "chainId": 1,
    "address": "0x2c95d751da37a5c1d9c5a7fd465c1d50f3d96160",
    "name": "WASSIE",
    "symbol": "WASSIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30144/thumb/logo-coingecko.png?1696529065"
  },
  {
    "chainId": 1,
    "address": "0x5aa158404fed6b4730c13f49d3a7f820e14a636f",
    "name": "ULTRON",
    "symbol": "ULX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26977/thumb/ULTRON-Profile-Pic.jpg?1696526031"
  },
  {
    "chainId": 1,
    "address": "0x95640a134721475bc78594c8ea66c0182c7b9a50",
    "name": "Metroxynth",
    "symbol": "MXH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34346/thumb/metroxynth-logo-200x200.png?1704698264"
  },
  {
    "chainId": 1,
    "address": "0x1236ea13c7339287cd00ab196aaa8217006b04dc",
    "name": "Epic League",
    "symbol": "EPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32139/thumb/epl_symbol_200.png?1696591947"
  },
  {
    "chainId": 1,
    "address": "0xe063f04f280c60aeca68b38341c2eecbec703ae2",
    "name": "f x  Protocol Leveraged ETH",
    "symbol": "XETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32165/thumb/0xe063f04f280c60aeca68b38341c2eecbec703ae2.png?1696724194"
  },
  {
    "chainId": 1,
    "address": "0xa50709b10c328772ea24a44b13c7e23c75561ecb",
    "name": "28",
    "symbol": "28",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32142/thumb/28.png?1696592290"
  },
  {
    "chainId": 1,
    "address": "0x365accfca291e7d3914637abf1f7635db165bb09",
    "name": "f x  Protocol",
    "symbol": "FXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31889/thumb/FXN_200x200.png?1696530700"
  },
  {
    "chainId": 1,
    "address": "0x00b4651795193daca7e73b5ddd031dfde8ca5720",
    "name": "CHAINMAIL",
    "symbol": "MAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33431/thumb/Z_hlFCgV_400x400.jpg?1701818031"
  },
  {
    "chainId": 1,
    "address": "0x170dec83c7753aaad20c01a0016b5a2e143990d4",
    "name": "Wigger",
    "symbol": "WIGGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32128/thumb/SmartSelect_20231003_234100_Telegram.jpg?1696588180"
  },
  {
    "chainId": 1,
    "address": "0x6cf9788cda282aeb729477f6bda9c4db082b5746",
    "name": "KingdomGame",
    "symbol": "KINGDOM",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/32147/thumb/Kingdom200x200.png?1696596846"
  },
  {
    "chainId": 1,
    "address": "0x405154cfaf5ea4ef57b65b86959c73dd079fa312",
    "name": "Alice AI",
    "symbol": "ALICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36605/thumb/Alice_AI.png?1711964052"
  },
  {
    "chainId": 1,
    "address": "0x2105465ab589b74747b01afdaf606d058fb082be",
    "name": "DEV SMASHED HIS KEYBOARD",
    "symbol": "HIXOKDKEKJCJDKSICND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32630/thumb/DEV_SMASHED_HIS_KEYBOARD.png?1698824159"
  },
  {
    "chainId": 1,
    "address": "0xed1ddc491a2c8b1f7d6e8933580a47e124ea38db",
    "name": "Intelligence On Chain",
    "symbol": "IOC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32418/thumb/IMG_20231017_174915_959.jpg?1698116981"
  },
  {
    "chainId": 1,
    "address": "0x9aaf731799e824a74a4d3a14e6b00bcc28c327db",
    "name": "Jable",
    "symbol": "JAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32131/thumb/Jab.png?1696588461"
  },
  {
    "chainId": 1,
    "address": "0x583019ff0f430721ada9cfb4fac8f06ca104d0b4",
    "name": "Staked Yearn Ether",
    "symbol": "ST-YETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32222/thumb/st-yETH-128px.png?1696914941"
  },
  {
    "chainId": 1,
    "address": "0x34dce75a3d1910cc9d188aa5a75fb9addcae0fcc",
    "name": "XV",
    "symbol": "XV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33432/thumb/XV_Token_logo_black_background_200_by_200.png?1701818327"
  },
  {
    "chainId": 1,
    "address": "0x4f27633e8b1cfd5e043c68cdcd81c0bbfdb1cf14",
    "name": "WhatBot",
    "symbol": "WHAT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xb39364b51d2c97b62b838bc5213b8627eb469101",
    "name": "Swot AI",
    "symbol": "SWOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35281/thumb/HLL21bqf_400x400_%281%29.jpg?1708054966"
  },
  {
    "chainId": 1,
    "address": "0x597981eac8a293054a1826c7b60cbf92972a36c1",
    "name": "Caacon",
    "symbol": "CC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31023/thumb/512x512.png?1709640084"
  },
  {
    "chainId": 1,
    "address": "0x36737b4ac153c762d6a767056e1af7b17573a6b9",
    "name": "GOLD",
    "symbol": "GOLD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32155/thumb/LOGO.jpg?1696598967"
  },
  {
    "chainId": 1,
    "address": "0x53805a76e1f5ebbfe7115f16f9c87c2f7e633726",
    "name": "f x  Protocol Fractional ETH",
    "symbol": "FETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32166/thumb/0x53805a76e1f5ebbfe7115f16f9c87c2f7e633726.png?1696724197"
  },
  {
    "chainId": 1,
    "address": "0xf0610eb7d8ee12d59412da32625d5e273e78ff0b",
    "name": "MasterDEX",
    "symbol": "MDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32311/thumb/MASTERDEX_logo-02_%282%29.png?1697188983"
  },
  {
    "chainId": 1,
    "address": "0x257508ca6341337dd243d54afd964e19375b4af9",
    "name": "Oracle Tools",
    "symbol": "OT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35582/thumb/IMG_2695.png?1709202522"
  },
  {
    "chainId": 1,
    "address": "0x320623b8e4ff03373931769a31fc52a4e78b5d70",
    "name": "Reserve Rights",
    "symbol": "RSR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8365/thumb/rsr.png?1696508558"
  },
  {
    "chainId": 1,
    "address": "0x6c28aef8977c9b773996d0e8376d2ee379446f2f",
    "name": "Quickswap  OLD ",
    "symbol": "QUICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13970/thumb/quick.png?1696513704"
  },
  {
    "chainId": 1,
    "address": "0x940a2db1b7008b6c776d4faaca729d6d4a4aa551",
    "name": "Dusk",
    "symbol": "DUSK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5217/thumb/image_widget_biddfvxd454b1.png?1696505726"
  },
  {
    "chainId": 1,
    "address": "0x10f2cf6ef155460c5b716080eb57928652867f2e",
    "name": "Alan Musk",
    "symbol": "MUSK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32262/thumb/28422.png?1707509906"
  },
  {
    "chainId": 1,
    "address": "0xa338b5a4bbd8053994bb6c55d770fc2447d66b88",
    "name": "Emerging Assets Group",
    "symbol": "EAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32265/thumb/0xa338b5a4bbd8053994bb6c55d770fc2447d66b88_%281%29.png?1697029815"
  },
  {
    "chainId": 1,
    "address": "0xee3c722d177559f73288cec91fa3e4bbfd8c27fc",
    "name": "Douglas Adams",
    "symbol": "HHGTTG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32837/thumb/HHGTTG200.jpg?1706880868"
  },
  {
    "chainId": 1,
    "address": "0x82967568a57625675b260ebab1294038c9accc6e",
    "name": "Nova DAO",
    "symbol": "NOVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34464/thumb/200x200_nova.png?1704965570"
  },
  {
    "chainId": 1,
    "address": "0xbe1dbe6741fb988fb571ab1e28cffb36e3c62629",
    "name": "Massive Protocol",
    "symbol": "MAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25134/thumb/logo.png?1696524284"
  },
  {
    "chainId": 1,
    "address": "0xb504035a11e672e12a099f32b1672b9c4a78b22f",
    "name": "SAFEREUM",
    "symbol": "SAFEREUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32179/thumb/FD173148-2002-420F-B138-E8DB70F754A0.png?1696743020"
  },
  {
    "chainId": 1,
    "address": "0xa922a70569a7555518bf4ded5094661a965e23ca",
    "name": "MN Bridge",
    "symbol": "MNB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32199/thumb/MNB.jpg?1696745894"
  },
  {
    "chainId": 1,
    "address": "0x6b448aeb3bfd1dcbe337d59f6dee159daab52768",
    "name": "Resistor AI",
    "symbol": "TOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36790/thumb/logo-square.png?1712418946"
  },
  {
    "chainId": 1,
    "address": "0x0eb1e72204c42de2263d37db606fb25c21d649b8",
    "name": "Astro X",
    "symbol": "ASTROX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34465/thumb/logo_astro_x-01.png?1704965967"
  },
  {
    "chainId": 1,
    "address": "0xc581b735a1688071a1746c968e0798d642ede491",
    "name": "Euro Tether",
    "symbol": "EURT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17385/thumb/Tether_new.png?1696516934"
  },
  {
    "chainId": 1,
    "address": "0x556bb0b27e855e6f2cebb47174495b9bbeb97ff1",
    "name": "Ctrl",
    "symbol": "CTRL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33356/thumb/ctrl_200x200.png?1701600824"
  },
  {
    "chainId": 1,
    "address": "0x3eef95e483b0a013d34b9482c6f4afcbd4d38146",
    "name": "Multiport",
    "symbol": "PORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32290/thumb/Multiport.png?1697182790"
  },
  {
    "chainId": 1,
    "address": "0x38e382f74dfb84608f3c1f10187f6bef5951de93",
    "name": "Multibit",
    "symbol": "MUBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33051/thumb/multi.jpg?1700473994"
  },
  {
    "chainId": 1,
    "address": "0x34bc13de8e5124a7c47d4b7ff7c5ade6ee34faba",
    "name": "Project Dojo",
    "symbol": "DOJO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33156/thumb/Project_Dojo_logo_transparent_200x200.png?1700827030"
  },
  {
    "chainId": 1,
    "address": "0x3d000462fb9826804a45c0ea869b83b69587f2db",
    "name": "Spatial Computing",
    "symbol": "CMPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35780/thumb/Spatial_%281%29.jpeg?1709812474"
  },
  {
    "chainId": 1,
    "address": "0x8c282c35b5e1088bb208991c151182a782637699",
    "name": "Monai",
    "symbol": "MONAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35661/thumb/2024-03-02_02.02.09.png?1709453630"
  },
  {
    "chainId": 1,
    "address": "0x4aecc899b68f086c008e920eafdc2a66d88aa7f1",
    "name": "SAFEMARS",
    "symbol": "SAFEMARS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32292/thumb/LOGO1.png?1697182930"
  },
  {
    "chainId": 1,
    "address": "0x0a8f4c4f23d72857745e26695dcd8dedf8e349b9",
    "name": "Marksman",
    "symbol": "MARKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33658/thumb/mlogo.png?1702598130"
  },
  {
    "chainId": 1,
    "address": "0x2f30700735697baac1b9177d404303f0cc0e1a74",
    "name": "Startupers",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32325/thumb/star.jpg?1697449239"
  },
  {
    "chainId": 1,
    "address": "0xe410fbd0977c8a2f276283425de488129f7faff9",
    "name": "Sesterce",
    "symbol": "SES",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32327/thumb/Screenshot_2023-10-13_185155-removebg-preview_%281%29.png?1697451481"
  },
  {
    "chainId": 1,
    "address": "0x76e222b07c53d28b89b0bac18602810fc22b49a8",
    "name": "Joe Coin",
    "symbol": "JOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32333/thumb/joe.png?1697452551"
  },
  {
    "chainId": 1,
    "address": "0xd721706581d97ecd202bbab5c71b5a85f0f78e69",
    "name": "DOGE 1",
    "symbol": "DOGE1",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32298/thumb/200.png?1697185878"
  },
  {
    "chainId": 1,
    "address": "0xe0a458bf4acf353cb45e211281a334bb1d837885",
    "name": "4Chan",
    "symbol": "4CHAN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30124/thumb/4CHAN.png?1696529046"
  },
  {
    "chainId": 1,
    "address": "0xebcf2fbe20e7bbbd5232eb186b85c143d362074e",
    "name": "Dream",
    "symbol": "DREAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32665/thumb/DREAM_200x200.png?1698902861"
  },
  {
    "chainId": 1,
    "address": "0xb58e26ac9cc14c0422c2b419b0ca555ee4dcb7cb",
    "name": "Niza Global",
    "symbol": "NIZA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32328/thumb/NizaCoin-logo-transaprent.png?1697451581"
  },
  {
    "chainId": 1,
    "address": "0xb70835d7822ebb9426b56543e391846c107bd32c",
    "name": "Game",
    "symbol": "GTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2046/thumb/game.png?1696503014"
  },
  {
    "chainId": 1,
    "address": "0x8221022401843fecaab247c88dd9960071ce7058",
    "name": "SCRATCH",
    "symbol": "SCRATCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32673/thumb/IMG_8985.jpg?1698911005"
  },
  {
    "chainId": 1,
    "address": "0x87d907568a0761ea45d2917e324557920668f224",
    "name": "Grok2 0",
    "symbol": "GROK20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32942/thumb/G2C_200px.jpeg?1699874312"
  },
  {
    "chainId": 1,
    "address": "0x44971abf0251958492fee97da3e5c5ada88b9185",
    "name": "BasedAI",
    "symbol": "BASEDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36607/thumb/1000004475.png?1711964332"
  },
  {
    "chainId": 1,
    "address": "0x8390a1da07e376ef7add4be859ba74fb83aa02d5",
    "name": "Grok",
    "symbol": "GROK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32788/thumb/GROK.png?1707579427"
  },
  {
    "chainId": 1,
    "address": "0x4507cef57c46789ef8d1a19ea45f4216bae2b528",
    "name": "TokenFi",
    "symbol": "TOKEN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32507/thumb/MAIN_TokenFi_logo_icon.png?1698918427"
  },
  {
    "chainId": 1,
    "address": "0xdddddd4301a082e62e84e43f474f044423921918",
    "name": "Rhino fi",
    "symbol": "DVF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16414/thumb/rhinologo.png?1697736807"
  },
  {
    "chainId": 1,
    "address": "0xfba5aa703939238c4d03cacbbd126fd7107b2082",
    "name": "WhisperBot",
    "symbol": "WSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32365/thumb/WhisperBot_Logo_Transparent_BG_%281%29.png?1698030191"
  },
  {
    "chainId": 1,
    "address": "0x72fd1648f34ddb747acd00ae114e805c7b81422d",
    "name": "PEPEBEAST",
    "symbol": "PEPEBEAST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32366/thumb/pepebeast.jpg?1698030307"
  },
  {
    "chainId": 1,
    "address": "0x8f0f56472c3e5730b1ea2f444e7829288da261e6",
    "name": "Rogue MAV",
    "symbol": "RMAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32384/thumb/rMAV.png?1698045143"
  },
  {
    "chainId": 1,
    "address": "0x5b060754b27844dbc3578382886828be224b804f",
    "name": "Loong",
    "symbol": "LOONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32308/thumb/Ow2kjQo7_400x400.jpg?1697188397"
  },
  {
    "chainId": 1,
    "address": "0x77777feddddffc19ff86db637967013e6c6a116c",
    "name": "Tornado Cash",
    "symbol": "TORN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13496/thumb/ZINt8NSB_400x400.jpg?1696513257"
  },
  {
    "chainId": 1,
    "address": "0xc7c53760375530e5af29fded5e13989325299382",
    "name": "WORLD PEACE COIN",
    "symbol": "WPC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32331/thumb/image01_%282%29_2.jpg?1697452381"
  },
  {
    "chainId": 1,
    "address": "0x7321a13bbf58220bca94a605eff3e24daee86376",
    "name": "Robin Hood",
    "symbol": "HOOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32388/thumb/IMG_20231017_095500_002.jpg?1698051071"
  },
  {
    "chainId": 1,
    "address": "0xcac1277aa6ecb68b84fad070910d37029e810b79",
    "name": "BitDelta",
    "symbol": "BDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36609/thumb/200x200_BDT.png?1711973432"
  },
  {
    "chainId": 1,
    "address": "0x613577bfea8ba6571f6b7a86716d04c80a3fbeb4",
    "name": "LNDRY",
    "symbol": "LNDRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33357/thumb/lndry.jpeg?1701602475"
  },
  {
    "chainId": 1,
    "address": "0x1a2eb478fa07125c9935a77b3c03a82470801e30",
    "name": "Amino",
    "symbol": "AMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32677/thumb/IMG_6956.png?1698920127"
  },
  {
    "chainId": 1,
    "address": "0x7d1afa7b718fb893db30a3abc0cfc608aacfebb0",
    "name": "Polygon",
    "symbol": "MATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4713/thumb/polygon.png?1698233745"
  },
  {
    "chainId": 1,
    "address": "0x276e84edc5bd94731b65c207dc1512fc5391572f",
    "name": "Potion Exchange",
    "symbol": "PTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33475/thumb/output-onlinepngtools_%289%29.png?1702013459"
  },
  {
    "chainId": 1,
    "address": "0xb14ebf566511b9e6002bb286016ab2497b9b9c9d",
    "name": "Hypersign Identity",
    "symbol": "HID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16158/thumb/hypersign.png?1696515762"
  },
  {
    "chainId": 1,
    "address": "0x13c4aa5c3f5bb1109c267e520a87c89684d3e73c",
    "name": "Verify Authentificator Bot",
    "symbol": "VERIFY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32845/thumb/logo.png?1699589566"
  },
  {
    "chainId": 1,
    "address": "0x2534588fa8ecddf160c6c0ae7d62523604df0a75",
    "name": "Arcade Arcoin",
    "symbol": "ARCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33476/thumb/2023-12-06_16.34.14.jpg?1702013632"
  },
  {
    "chainId": 1,
    "address": "0x7a29fb472f0e62bce6127f125e8d5ad2a9fa3839",
    "name": "Rebirth Protocol",
    "symbol": "RBH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32447/thumb/Rebirth.jpg?1698239321"
  },
  {
    "chainId": 1,
    "address": "0x686d1596e5632fe0471961e7977e8efe371b0b21",
    "name": "SurrealVerse",
    "symbol": "AZEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32448/thumb/IMG_3734.jpeg?1698239938"
  },
  {
    "chainId": 1,
    "address": "0x7e84c0890ca316458d41497146b2176f6a226277",
    "name": "RAVE NFT",
    "symbol": "RAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32535/thumb/rave.jpg?1698461782"
  },
  {
    "chainId": 1,
    "address": "0xeda43fa2f35ea174a7e9b73658b18a7da00adf38",
    "name": "Namx",
    "symbol": "NAMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32437/thumb/Screenshot_2023-10-18_at_13.20.36.png?1698226504"
  },
  {
    "chainId": 1,
    "address": "0xf017d3690346eb8234b85f74cee5e15821fee1f4",
    "name": "GEKKO",
    "symbol": "GEKKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32439/thumb/gekko_%281%29.png?1698228341"
  },
  {
    "chainId": 1,
    "address": "0x455e53cbb86018ac2b8092fdcd39d8444affc3f6",
    "name": "Polygon Ecosystem Token",
    "symbol": "POL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32440/thumb/polygon.png?1698233684"
  },
  {
    "chainId": 1,
    "address": "0xae3359ed3c567482fb0102c584c23daa2693eacf",
    "name": "DORK",
    "symbol": "DORK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32704/thumb/Coin_2.png?1698985483"
  },
  {
    "chainId": 1,
    "address": "0x390e61f798267fe7aa9bbe61be8bb1776250d44c",
    "name": "T2T2",
    "symbol": "T2T2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32856/thumb/T2T2.png?1699697712"
  },
  {
    "chainId": 1,
    "address": "0x0257ffd7ea2ebba4aaa090c7adbdd032a08c1f74",
    "name": "ZELIX",
    "symbol": "ZELIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32453/thumb/ZX_Logo_20230724_200x200.png?1701866637"
  },
  {
    "chainId": 1,
    "address": "0x7a8946eda77817126ffe301249f6dc4c7df293c3",
    "name": "Dyl",
    "symbol": "DYL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32550/thumb/Dyl-pfp-200x.png?1698479973"
  },
  {
    "chainId": 1,
    "address": "0xd1f17b7a6bff962659ed608bcd6d318bb5fbb249",
    "name": "Zuzalu Inu",
    "symbol": "ZUZALU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34552/thumb/zuzalu_0xd1f17b7a6bff962659ed608bcd6d318bb5fbb2491705497250930.jpg?1706250673"
  },
  {
    "chainId": 1,
    "address": "0x0ee27a1f959ea7ea2aa171a7e2e48fd9f17bb8eb",
    "name": "First GROK AI",
    "symbol": "GROK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33005/thumb/Untitled_design_%2829%29.png?1700122635"
  },
  {
    "chainId": 1,
    "address": "0x38e4adb44ef08f22f5b5b76a8f0c2d0dcbe7dca1",
    "name": "PowerPool Concentrated Voting Power",
    "symbol": "CVP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12266/thumb/Powerpool.jpg?1696512097"
  },
  {
    "chainId": 1,
    "address": "0xff00644ca76def7a3f7501a281ffe45934aefbfe",
    "name": "Shadow Wizard Money Gang",
    "symbol": "GANG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32452/thumb/logo.png?1698240659"
  },
  {
    "chainId": 1,
    "address": "0xd939212f16560447ed82ce46ca40a63db62419b5",
    "name": "Mega Yacht Cult",
    "symbol": "MYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32456/thumb/Untitled_design_%283%29_%281%29.png?1698241312"
  },
  {
    "chainId": 1,
    "address": "0x27c57f84236780881be694a18e149bb5bb78c21f",
    "name": "LiquidityRush",
    "symbol": "LIQR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34846/thumb/liqr_%281%29.jpg?1706254692"
  },
  {
    "chainId": 1,
    "address": "0x491604c0fdf08347dd1fa4ee062a822a5dd06b5d",
    "name": "Cartesi",
    "symbol": "CTSI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11038/thumb/Cartesi_Logo.png?1696510982"
  },
  {
    "chainId": 1,
    "address": "0x7fc66500c84a76ad7e9c93437bfc5ac33e2ddae9",
    "name": "Aave",
    "symbol": "AAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12645/thumb/AAVE.png?1696512452"
  },
  {
    "chainId": 1,
    "address": "0xbeef8e0982874e0292e6c5751c5a4092b3e1beef",
    "name": "Staked BIFI",
    "symbol": "MOOBIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32597/thumb/319381e63428d3c2ab6e035d5f3abd76.png?1698682355"
  },
  {
    "chainId": 1,
    "address": "0x746dda2ea243400d5a63e0700f190ab79f06489e",
    "name": "BOSagora",
    "symbol": "BOA",
    "decimals": 7,
    "logoURI": "https://assets.coingecko.com/coins/images/9202/thumb/Picture1.png?1696509320"
  },
  {
    "chainId": 1,
    "address": "0x4c746edf20762dc201ac40135e0c13e400d23d58",
    "name": "GOD Coin",
    "symbol": "GOD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33360/thumb/_kJ-LILu_400x400.jpg?1701603268"
  },
  {
    "chainId": 1,
    "address": "0x72831eebef4e3f3697a6b216e3713958210ae8cd",
    "name": "Blob",
    "symbol": "BLOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35309/thumb/Blob.jpeg?1708158035"
  },
  {
    "chainId": 1,
    "address": "0xa444ec96ee01bb219a44b285de47bf33c3447ad5",
    "name": "LEOX",
    "symbol": "LEOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29980/thumb/LEOX-LOGO-BLACK-2.jpg?1696528905"
  },
  {
    "chainId": 1,
    "address": "0x01194726b1b55bbf99cb083ba8e5dcc0834adbc3",
    "name": "ArcadeFi",
    "symbol": "ARCADE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35504/thumb/IMG_8212.jpeg?1708935437"
  },
  {
    "chainId": 1,
    "address": "0xc56aa8032d2cda3e401171b7787205f6640837d9",
    "name": "Alpine",
    "symbol": "ALP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32517/thumb/al_pine200.png?1698406431"
  },
  {
    "chainId": 1,
    "address": "0x7cdbfc86a0bfa20f133748b0cf5cea5b787b182c",
    "name": "TokenSight",
    "symbol": "TKST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32514/thumb/token-coingecko.png?1698401758"
  },
  {
    "chainId": 1,
    "address": "0x1123d17fcf93ed2b41440317503346a0fdfe3ed7",
    "name": "Prometheum Prodigy",
    "symbol": "PMPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32522/thumb/IMG_1277.jpeg?1708596900"
  },
  {
    "chainId": 1,
    "address": "0xa792ff98afd4ccb8d33f2590d0bc9420a062e555",
    "name": "Gold Utility Token",
    "symbol": "AGF",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x9f4a8167ca311a87b0d03aafa44e0d2c3d8a3631",
    "name": "BlockBox",
    "symbol": "BBOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32521/thumb/IMG_0229.jpeg?1698407087"
  },
  {
    "chainId": 1,
    "address": "0xf1dba5c1972778763409447d9adedabbbeb177c1",
    "name": "GEXC FINANCE",
    "symbol": "GEXC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35505/thumb/gexc200.png?1708935648"
  },
  {
    "chainId": 1,
    "address": "0xabd601423a2cd5723cb546acc5c40fb01c3422cf",
    "name": "Baby X",
    "symbol": "BABYX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32525/thumb/topimg.png?1698411093"
  },
  {
    "chainId": 1,
    "address": "0x617ead3c59ded3ea1bb17881118cf310144b450f",
    "name": "The Secret Coin",
    "symbol": "TSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32545/thumb/IMG_2160.jpeg?1698474245"
  },
  {
    "chainId": 1,
    "address": "0x55c5bd14ec3b3e24fd8c5e8b8eb9a52e876e5a06",
    "name": "NitroBots",
    "symbol": "NITRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32526/thumb/Nitro.png?1698411154"
  },
  {
    "chainId": 1,
    "address": "0x67f3086f7823eaf35f5aaadfb2e9b9c5b09578cf",
    "name": "InsightX",
    "symbol": "INX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32552/thumb/android-chrome-192x192.png?1698480297"
  },
  {
    "chainId": 1,
    "address": "0x02e7f808990638e9e67e1f00313037ede2362361",
    "name": "KiboShib",
    "symbol": "KIBSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29335/thumb/foto_no_exif_%2811%29%282%29_%281%29.png?1696528285"
  },
  {
    "chainId": 1,
    "address": "0x80d186b4c786ea66592b2c52e2004ab10cfe4cf3",
    "name": "LYNX",
    "symbol": "LYNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32501/thumb/3C1E4E5C-4B05-4039-B9DD-1DAB463AB44A.png?1698312161"
  },
  {
    "chainId": 1,
    "address": "0x0080428794a79a40ae03cf6e6c1d56bd5467a4a2",
    "name": "LONG  ETH ",
    "symbol": "LONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32559/thumb/IMG_20231005_122433_189.jpg?1698544869"
  },
  {
    "chainId": 1,
    "address": "0x5df2aa6d903410b2a747c90dbf2de0de7b15ac60",
    "name": "0xgambit",
    "symbol": "0XG",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32579/thumb/logo.png?1698558743"
  },
  {
    "chainId": 1,
    "address": "0x2b91c5a73b7c4a70a61536429a2bf45461d85b27",
    "name": "Riggers",
    "symbol": "RIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35159/thumb/Rigger_transparent.png?1707567408"
  },
  {
    "chainId": 1,
    "address": "0x7e877b99897d514da01bd1d177e693ec639961af",
    "name": "Oggy Inu  ETH ",
    "symbol": "OGGY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32619/thumb/Oggy_logo_200x200.png?1698756117"
  },
  {
    "chainId": 1,
    "address": "0xbb9fd9fa4863c03c574007ff3370787b9ce65ff6",
    "name": "HILO",
    "symbol": "HILO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28661/thumb/cmchiloo.png?1696527646"
  },
  {
    "chainId": 1,
    "address": "0x961c8c0b1aad0c0b10a51fef6a867e3091bcef17",
    "name": "Dypius  OLD ",
    "symbol": "DYP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13480/thumb/DYP-200x200px.png?1696513241"
  },
  {
    "chainId": 1,
    "address": "0xd51e4965ad973e8c1e1f22369bb884e6914b012c",
    "name": "ZenoCard",
    "symbol": "ZENO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36529/thumb/IMG_20240324_220958_106_%281%29.png?1711700883"
  },
  {
    "chainId": 1,
    "address": "0xd12b2859227b88c8065b6216e6e1b980d8dfe152",
    "name": "SatNode",
    "symbol": "SND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35214/thumb/New_Project_%283%29_%281%29.png?1707898354"
  },
  {
    "chainId": 1,
    "address": "0x8ed955a2b7d2c3a17a9d05daca95e01818f8c11e",
    "name": "APF coin",
    "symbol": "APFC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30990/thumb/photo_2023-10-28_09-48-25.jpg?1698518912"
  },
  {
    "chainId": 1,
    "address": "0xa8c08d8bb15a2d1a4d3b78def9c635ef1e340e16",
    "name": "SERBIAN DANCING LADY",
    "symbol": "",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32688/thumb/IMG_20231101_105021_023.jpg?1698938977"
  },
  {
    "chainId": 1,
    "address": "0x6c74ebb3d9b4a1e77558f1871cd9c8ef07b4dfb2",
    "name": "The Autism Token",
    "symbol": "TISM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32794/thumb/tism_logo_resized.png?1699445572"
  },
  {
    "chainId": 1,
    "address": "0x745407c86df8db893011912d3ab28e68b62e49b0",
    "name": "MahaDAO",
    "symbol": "MAHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13404/thumb/MAHA_Token.png?1696513166"
  },
  {
    "chainId": 1,
    "address": "0x6e3f7d6f0eab61f161551c60096aa559d2cf46cd",
    "name": "Coinhiba",
    "symbol": "HIBA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32632/thumb/Coinhiba.png?1698826000"
  },
  {
    "chainId": 1,
    "address": "0xe785ec36356b973d8c0a071d478940d6f42c0178",
    "name": "Bigfoot Monster",
    "symbol": "BIGF",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32795/thumb/Safeimagekit-resized-img_%284%29.png?1699445622"
  },
  {
    "chainId": 1,
    "address": "0x6ae2a128cd07d672164ca9f2712ea737d198dd41",
    "name": "Goat Trading",
    "symbol": "GOAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35160/thumb/logo-light1.png?1707569994"
  },
  {
    "chainId": 1,
    "address": "0x39b46b212bdf15b42b166779b9d1787a68b9d0c3",
    "name": "Dypius",
    "symbol": "DYP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33016/thumb/RS-zDhFE_400x400.jpg?1700157370"
  },
  {
    "chainId": 1,
    "address": "0xd1e64bcc904cfdc19d0faba155a9edc69b4bcdae",
    "name": "Pikamoon",
    "symbol": "PIKA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36366/thumb/Pikamoon.png?1711336684"
  },
  {
    "chainId": 1,
    "address": "0xe97db8503dd3e3eb0286aafc50327c598efdb578",
    "name": "Jinko AI",
    "symbol": "JINKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32633/thumb/Jinko_Logo_BY_Colour.png?1698826087"
  },
  {
    "chainId": 1,
    "address": "0xd0a6053f087e87a25dc60701ba6e663b1a548e85",
    "name": "BLOCKLORDS",
    "symbol": "LRDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34775/thumb/LRDS_PNG.png?1706001771"
  },
  {
    "chainId": 1,
    "address": "0x201b5b64438843553e3c3671810ae671c93c685c",
    "name": "Megabot",
    "symbol": "MEGABOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32796/thumb/Megabot.jpg?1699445692"
  },
  {
    "chainId": 1,
    "address": "0x6d5777dce2541175adf6d49cadd666f3ab0ac142",
    "name": "Cartman",
    "symbol": "CARTMAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32797/thumb/2023-11-02_20.50.51.jpg?1699445925"
  },
  {
    "chainId": 1,
    "address": "0x0f4dc5ada841cf5a7652e52d04ae786070cc9472",
    "name": "Aktionariat Green Monkey Club AG Tokeni",
    "symbol": "GMCS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33366/thumb/GMCS.png?1701613728"
  },
  {
    "chainId": 1,
    "address": "0x7199f1dffe926c8fd36567d8df6f41edb01bf632",
    "name": "Okiku",
    "symbol": "OKIKU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32683/thumb/200.png?1698937633"
  },
  {
    "chainId": 1,
    "address": "0xbe6be64e9e5042b6e84e4c27956cce6353efa5f5",
    "name": "Beg",
    "symbol": "BEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32799/thumb/beg100px.png?1699446876"
  },
  {
    "chainId": 1,
    "address": "0xcd54df3c19a7ae672897f2a09821d2c287d36326",
    "name": "Baby Memecoin",
    "symbol": "BABYMEME",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32838/thumb/bd0e18c5-ac94-424b-a6ea-9d654770be1f-18213305.png?1699587642"
  },
  {
    "chainId": 1,
    "address": "0x57e114b691db790c35207b2e685d4a43181e6061",
    "name": "Ethena",
    "symbol": "ENA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36530/thumb/ethena.png?1711701436"
  },
  {
    "chainId": 1,
    "address": "0xb71bdc7014f3740d0267d41d632cab8371f8ba3c",
    "name": "MILEI",
    "symbol": "MILEI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33489/thumb/IMG_7308.png?1702029479"
  },
  {
    "chainId": 1,
    "address": "0x973e00eee6d180b5a0eb08ce3047ac4ea7a45cd5",
    "name": "TOTO",
    "symbol": "TOTO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35215/thumb/200X200.png?1707898793"
  },
  {
    "chainId": 1,
    "address": "0x9d67d207b0cbdbd6dc50f465555cc777bd51159e",
    "name": "Useless Utility",
    "symbol": "UU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33368/thumb/0x9d67d207b0cbdbd6dc50f465555cc777bd51159e.png?1701619663"
  },
  {
    "chainId": 1,
    "address": "0x80592d613a383f2ba3c42e4c247067289ee60152",
    "name": "TurboBot",
    "symbol": "TURBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32747/thumb/FFxY5qST_400x400.jpeg?1699262654"
  },
  {
    "chainId": 1,
    "address": "0x7f9b09f4717072cf4dc18b95d1b09e2b30c76790",
    "name": "VaultTech",
    "symbol": "VAULT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32578/thumb/IMG_6957.jpeg?1698558686"
  },
  {
    "chainId": 1,
    "address": "0x7df18e4efd6e6f73cfb462937dac40fe42533016",
    "name": "ZKitty Bot",
    "symbol": "ZKITTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31474/thumb/AnN0reNi_400x400.jpg?1696530286"
  },
  {
    "chainId": 1,
    "address": "0x614da3b37b6f66f7ce69b4bbbcf9a55ce6168707",
    "name": "MMX",
    "symbol": "MMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33161/thumb/mmx.png?1700830276"
  },
  {
    "chainId": 1,
    "address": "0xbd713f15673b9861b6123840f6e0eba03d6aae51",
    "name": "Eyebot",
    "symbol": "EYEBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33369/thumb/IMG_0672.jpeg?1701619837"
  },
  {
    "chainId": 1,
    "address": "0x26f9111a358385dc46a832cf1a1a021ee72e58a1",
    "name": "Zook Protocol",
    "symbol": "ZOOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32706/thumb/20231102_124343.jpg?1698985803"
  },
  {
    "chainId": 1,
    "address": "0xa2fe5e51729be71261bcf42854012827bc44c044",
    "name": "BURN",
    "symbol": "BURN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32652/thumb/Burn.jpg?1698895268"
  },
  {
    "chainId": 1,
    "address": "0x1783d3979038d986f3959adf7dbfc527f42c4269",
    "name": "Utopia Bot",
    "symbol": "UB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32718/thumb/CG_Transparent_Logo_200x200.png?1699001180"
  },
  {
    "chainId": 1,
    "address": "0xae7ab96520de3a18e5e111b5eaab095312d7fe84",
    "name": "Lido Staked Ether",
    "symbol": "STETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13442/thumb/steth_logo.png?1696513206"
  },
  {
    "chainId": 1,
    "address": "0xf3e66b03d098d0482be9cb3d6999787231a93ed9",
    "name": "PromptIDE",
    "symbol": "PROMPTIDE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32753/thumb/PromptIDE_Logo_%283%29.jpg?1707469846"
  },
  {
    "chainId": 1,
    "address": "0x032c3ec6d4c894844fd855874062a86592801cc9",
    "name": "Converge Bot",
    "symbol": "CONVERGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33519/thumb/cg.png?1712560854"
  },
  {
    "chainId": 1,
    "address": "0x13630da53ac2f3004e6e2a127546935ad932f92f",
    "name": "Wagyu",
    "symbol": "WAGYU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33295/thumb/200x200.png?1701401886"
  },
  {
    "chainId": 1,
    "address": "0xbf19eead55a6b100667f04f8fbc5371e03e8ab2e",
    "name": "Inception cbETH",
    "symbol": "INCBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34948/thumb/cbETH.png?1706770383"
  },
  {
    "chainId": 1,
    "address": "0xb61ebb6bceb7635ecd7e59884ee2e2bcdfd810ba",
    "name": "XSHIB",
    "symbol": "XSHIB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32839/thumb/LOGOSHIB.jpg?1699587693"
  },
  {
    "chainId": 1,
    "address": "0x753cd33a41a7b4857681ab2498ce66ae977aa3de",
    "name": "VaxLabs",
    "symbol": "VLABS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35311/thumb/vaxlab_png-tbg.png?1708158748"
  },
  {
    "chainId": 1,
    "address": "0x99cffb50aad37d17955253f3a4070556b5127a0b",
    "name": "MEGALODON",
    "symbol": "MEGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33371/thumb/mega.jpg?1701620157"
  },
  {
    "chainId": 1,
    "address": "0x1e7a53220c128d100b56970ce88ee963a9511370",
    "name": "PNPCoin",
    "symbol": "PNPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33520/thumb/PNPC-200.png?1702133448"
  },
  {
    "chainId": 1,
    "address": "0x2260fac5e5542a773aa44fbcfedf7c193bc2c599",
    "name": "Wrapped Bitcoin",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7598/thumb/wrapped_bitcoin_wbtc.png?1696507857"
  },
  {
    "chainId": 1,
    "address": "0xda9b11cd701e10c2ec1a284f80820edd128c5246",
    "name": "Inception wbETH",
    "symbol": "INWBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34944/thumb/WBETH.png?1706757148"
  },
  {
    "chainId": 1,
    "address": "0x47879db9e657e644082071b48e2f33d80f369f02",
    "name": "XINU  ETH ",
    "symbol": "XINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32712/thumb/xinu.jpg?1699000456"
  },
  {
    "chainId": 1,
    "address": "0x4e1a609ec87cf6477613f515f6eb64ef2d31089a",
    "name": "Aktionariat Green Consensus AG Tokenize",
    "symbol": "DGCS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33372/thumb/GCS.png?1701648529"
  },
  {
    "chainId": 1,
    "address": "0x68f108fb7141ffe36b832c5c225d9e7e474bd664",
    "name": "Pepinu",
    "symbol": "PEPINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36133/thumb/200200.png?1710581969"
  },
  {
    "chainId": 1,
    "address": "0xd9dcd611bed2be9a4700c725a3810870b9bff094",
    "name": "MaxxChain",
    "symbol": "PWR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31321/thumb/PWR_Logo_200x200.jpg?1696530140"
  },
  {
    "chainId": 1,
    "address": "0x3f48cce3eabb02304f7cc16fd7a35f3a2df7be8e",
    "name": "Truck",
    "symbol": "TRUCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32760/thumb/2023-10-31_19.33.52.jpg?1699328941"
  },
  {
    "chainId": 1,
    "address": "0x83f5b9c25cc8fce0a7d4a1bda904bf13cfcdd9da",
    "name": "FP  DeGods",
    "symbol": "UDEGODS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32765/thumb/degods_%281%29.png?1699332561"
  },
  {
    "chainId": 1,
    "address": "0xaa846b1ec76cdf864ea03e1df665f6c8d5510923",
    "name": "LeaderDAO",
    "symbol": "LDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32770/thumb/LeaderDAO_Logo_200.jpg?1699342988"
  },
  {
    "chainId": 1,
    "address": "0x8ab98330473101309db94b625f9997366a518223",
    "name": "PlasticHero",
    "symbol": "PTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32771/thumb/Ticker_%282%29.png?1699343244"
  },
  {
    "chainId": 1,
    "address": "0xa100eafdef0099700933538ee795cfad5505b689",
    "name": "FP  Captainz",
    "symbol": "UCAPTAINZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32775/thumb/Captains.png?1699347464"
  },
  {
    "chainId": 1,
    "address": "0x6cd7fc3118a8ffa40af0f99f3cbda54b0c6d4d1d",
    "name": "FP  Milady",
    "symbol": "UMIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32768/thumb/milady_logo.9816562c.png?1699337545"
  },
  {
    "chainId": 1,
    "address": "0xbc526a4ae9632d42291ad9d5ef29d36dd4d0ce26",
    "name": "FP  MoonBirds",
    "symbol": "UMOONBIRDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32777/thumb/mmoonbirds_%281%29.png?1699348940"
  },
  {
    "chainId": 1,
    "address": "0xa07dcc1abfe20d29d87a32e2ba89876145dafb0a",
    "name": "FP  Doodle",
    "symbol": "UDOODLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32776/thumb/mdoodle_%281%29.png?1699348365"
  },
  {
    "chainId": 1,
    "address": "0x36a17b52b5322846fcf894ee1535c8aa23df6765",
    "name": "FP  Potatoz",
    "symbol": "UPOTATOZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32779/thumb/mpotatoz_%282%29.png?1699350069"
  },
  {
    "chainId": 1,
    "address": "0xae05559e5bc86ecca132956fa0482dd7d4b2d76d",
    "name": "FP  CloneX",
    "symbol": "UCLONEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32767/thumb/clonex_logo.30687847_%281%29.png?1699336155"
  },
  {
    "chainId": 1,
    "address": "0xedae06a2dbdd21038608adce58fd173afdba5add",
    "name": "FP  LilPudgys",
    "symbol": "ULP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32782/thumb/lp_%281%29.png?1699351240"
  },
  {
    "chainId": 1,
    "address": "0x9f83ed4fdf636e8b0562ac704267346712b44c36",
    "name": "FP  Mfers",
    "symbol": "UMFER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32783/thumb/mfer_%281%29.png?1699351834"
  },
  {
    "chainId": 1,
    "address": "0xf59ae934f6fe444afc309586cc60a84a0f89aaea",
    "name": "Polkadex",
    "symbol": "PDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14833/thumb/dIze5Ztl_400x400.jpg?1696514500"
  },
  {
    "chainId": 1,
    "address": "0x34635280737b5bfe6c7dc2fc3065d60d66e78185",
    "name": "Convex Prisma",
    "symbol": "CVXPRISMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32961/thumb/cvxprisma.png?1700026172"
  },
  {
    "chainId": 1,
    "address": "0xa2120b9e674d3fc3875f415a7df52e382f141225",
    "name": "Automata",
    "symbol": "ATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15985/thumb/ATA.jpg?1696515598"
  },
  {
    "chainId": 1,
    "address": "0xcd24181edaa0394e1d978d5011e36f67fe41a499",
    "name": "ORB WIZZ COUNCIL",
    "symbol": "ORB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32761/thumb/ORB_WORLD_%281%29.png?1699329160"
  },
  {
    "chainId": 1,
    "address": "0xf19308f923582a6f7c465e5ce7a9dc1bec6665b1",
    "name": "TitanX",
    "symbol": "TITANX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32762/thumb/TitanXpng_%281%29.png?1704456654"
  },
  {
    "chainId": 1,
    "address": "0x8c3d11360a70fb708762b394be8ef23e3a633e3f",
    "name": "FP  BoredApeKennelClub",
    "symbol": "UBAKC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32778/thumb/UBAKC_%281%29.png?1699349512"
  },
  {
    "chainId": 1,
    "address": "0x433fb346139e4eb5e513ae5fd77bd98614b153b1",
    "name": "FP  Beanz",
    "symbol": "UBEANZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32784/thumb/beanz_%281%29.png?1699352128"
  },
  {
    "chainId": 1,
    "address": "0x5acd02940d2e56d9402b8d224e56bd800c544466",
    "name": "Real Strawberry Elephant",
    "symbol": "",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32591/thumb/sqQV944d_400x400.jpg?1698632639"
  },
  {
    "chainId": 1,
    "address": "0x4fa5836b58d6f1877a616e7833c7350781e21202",
    "name": "FP  Otherdeed",
    "symbol": "UOTHR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32766/thumb/othr_%281%29.png?1699332911"
  },
  {
    "chainId": 1,
    "address": "0xc57bf43f8ca4458309386f9a3e8246a8cadcca51",
    "name": "DOXcoin",
    "symbol": "DOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36857/thumb/201.png?1712641116"
  },
  {
    "chainId": 1,
    "address": "0xbbbbca6a901c926f240b89eacb641d8aec7aeafd",
    "name": "Loopring",
    "symbol": "LRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/913/thumb/LRC.png?1696502034"
  },
  {
    "chainId": 1,
    "address": "0x904f36d74bed2ef2729eaa1c7a5b70dea2966a02",
    "name": "Blueberry",
    "symbol": "BLB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35312/thumb/Blueberry_Logo.png?1708159808"
  },
  {
    "chainId": 1,
    "address": "0x3a645ff83560231aab0f9c830ba108b06c94e34a",
    "name": "Lunar",
    "symbol": "LUNAR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35510/thumb/image_2024-02-26_165146229.png?1708937508"
  },
  {
    "chainId": 1,
    "address": "0x1e971b5b21367888239f00da16f0a6b0effecb03",
    "name": "LEEROY JENKINS",
    "symbol": "LEEROY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32774/thumb/IMG_20231106_175636_467.jpg?1699343781"
  },
  {
    "chainId": 1,
    "address": "0x569d0e52c3dbe95983bcc2434cb9f69d905be919",
    "name": "Roaring Kitty",
    "symbol": "ROAR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33986/thumb/logo_200_by_200.png?1703585719"
  },
  {
    "chainId": 1,
    "address": "0x3850952491606a0e420eb929b1a2e1a450d013f1",
    "name": "PanoVerse",
    "symbol": "PANO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32812/thumb/0x3850952491606a0e420eb929b1a2e1a450d013f1.png?1699544500"
  },
  {
    "chainId": 1,
    "address": "0xba10085f901ae4048134e556d579cfd1bfaf89cf",
    "name": "FP  Elemental",
    "symbol": "UELEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32785/thumb/elemental_%281%29.png?1699352536"
  },
  {
    "chainId": 1,
    "address": "0x02de007d412266a2e0fa9287c103474170f06560",
    "name": "Exorde",
    "symbol": "EXD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28684/thumb/logo-exorde.png?1696527668"
  },
  {
    "chainId": 1,
    "address": "0x19a71179fd41c19f8dfb1f38343482bec0271e3c",
    "name": "Spectrum Marketplace",
    "symbol": "SPEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32820/thumb/download.png?1699582062"
  },
  {
    "chainId": 1,
    "address": "0x42f1ffeabd9551aec26d70257c8a4d4c31b72180",
    "name": "FP  0N1Force",
    "symbol": "U0N1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32786/thumb/0N1.png?1699353080"
  },
  {
    "chainId": 1,
    "address": "0x97bd0d0a3e9635325ec0b383cff62914a7d50642",
    "name": "FP  SappySeals",
    "symbol": "USAPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32787/thumb/saps_%281%29.png?1699353354"
  },
  {
    "chainId": 1,
    "address": "0x68d1c2a170bac7f73d7680970345fcf8ddec79bb",
    "name": "VirtuBlock",
    "symbol": "VB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36858/thumb/VB_200x200.png?1712642037"
  },
  {
    "chainId": 1,
    "address": "0xa88920b4a35f7d0e81bc586ce1875036fced9154",
    "name": "Kabuni",
    "symbol": "KBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32827/thumb/Kabuni_Coin.1.png?1699585193"
  },
  {
    "chainId": 1,
    "address": "0x8b4d316e40ac4baf08957963210c6cff683a152b",
    "name": "No Decimal",
    "symbol": "SCARCE",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/34153/thumb/Untitled_design_%2857%29.png?1704208481"
  },
  {
    "chainId": 1,
    "address": "0x6b985d38b1fc891bb57bff59573626b1896d4aa1",
    "name": "Fido",
    "symbol": "FIDO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32943/thumb/Fido_circle.png?1701863458"
  },
  {
    "chainId": 1,
    "address": "0xa2d6b59758d307df27d5809426252d91a060cb24",
    "name": "Aktionariat Parknsleep AG Tokenized Sha",
    "symbol": "PNS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33376/thumb/PN.png?1701650053"
  },
  {
    "chainId": 1,
    "address": "0xa89b728708be04f57c7a33c6f790b6f077298e26",
    "name": "ReptilianZuckerBidenBartcoin",
    "symbol": "BART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32599/thumb/bart.png?1698724602"
  },
  {
    "chainId": 1,
    "address": "0x564a80d0123bdd750fb6a9993834968fc595c09a",
    "name": "Super Best Friends",
    "symbol": "SUBF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36368/thumb/1000017215.jpg?1711338058"
  },
  {
    "chainId": 1,
    "address": "0x9047b76d7fb3d8bc3342616c56028430ebc74290",
    "name": "Bishop",
    "symbol": "BISHOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32868/thumb/5YeVwATD_400x400.jpg?1699672895"
  },
  {
    "chainId": 1,
    "address": "0x2c540c3c7be7af98278dc6963e092cd450009d1f",
    "name": "Sparko",
    "symbol": "SPARKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32964/thumb/recursos-cripto-95.png?1700020107"
  },
  {
    "chainId": 1,
    "address": "0xf538296e7dd856af7044deec949489e2f25705bc",
    "name": "Illumicati",
    "symbol": "MILK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32829/thumb/ILLUMICATI_CG.png?1699585511"
  },
  {
    "chainId": 1,
    "address": "0x1e3778dd6dbfdc1c5b89f95f7c098b21e80ec4fa",
    "name": "Victory Impact",
    "symbol": "VIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32814/thumb/1000084688.png?1699578648"
  },
  {
    "chainId": 1,
    "address": "0xa5e0b9118ce23dd1eec52cd93cf59e8ea3b3ff56",
    "name": "JUDAS",
    "symbol": "JUDAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32938/thumb/photo_2023-11-12_10-29-13.jpg?1699872126"
  },
  {
    "chainId": 1,
    "address": "0x183395dbd0b5e93323a7286d1973150697fffcb3",
    "name": "Convex FXN",
    "symbol": "CVXFXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32962/thumb/cvxfxn.png?1700026163"
  },
  {
    "chainId": 1,
    "address": "0x2adcbee886d23eff5adecc7767bf102e4a1ce151",
    "name": "Aktionariat AyurVeda AG Tokenized Share",
    "symbol": "VEDA",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33377/thumb/VEDA.png?1701650144"
  },
  {
    "chainId": 1,
    "address": "0xae9d553c85eef99c0ef9169f0d49ef33f5275f94",
    "name": "Dex Sniffer",
    "symbol": "DS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32944/thumb/DS.jpg?1699887500"
  },
  {
    "chainId": 1,
    "address": "0x7f39c581f595b53c5cb19bd0b3f8da6c935e2ca0",
    "name": "Wrapped stETH",
    "symbol": "WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18834/thumb/wstETH.png?1696518295"
  },
  {
    "chainId": 1,
    "address": "0xc011a73ee8576fb46f5e1c5751ca3b9fe0af2a6f",
    "name": "Synthetix Network",
    "symbol": "SNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3406/thumb/SNX.png?1696504103"
  },
  {
    "chainId": 1,
    "address": "0xd2960d83c53085b5631f4d0be4916806e40ef1f3",
    "name": "OOFP",
    "symbol": "OOFP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34590/thumb/Picsart_24-01-16_09-13-09-109.jpg?1705428823"
  },
  {
    "chainId": 1,
    "address": "0xa0d69e286b938e21cbf7e51d71f6a4c8918f482f",
    "name": "Electronic USD",
    "symbol": "EUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28445/thumb/0xa0d69e286b938e21cbf7e51d71f6a4c8918f482f.png?1696527441"
  },
  {
    "chainId": 1,
    "address": "0x3408636a7825e894ac5521ca55494f89f96df240",
    "name": "PymeDAO",
    "symbol": "PYME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31490/thumb/tokenLogo.png?1696530301"
  },
  {
    "chainId": 1,
    "address": "0x0b61c4f33bcdef83359ab97673cb5961c6435f4e",
    "name": "HOLD",
    "symbol": "EARN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32575/thumb/200.png?1698554742"
  },
  {
    "chainId": 1,
    "address": "0x97d2fc7d16bc34121c3311f2e2e05d298c19956f",
    "name": "WoofOracle",
    "symbol": "WFO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35788/thumb/Android.png?1709799277"
  },
  {
    "chainId": 1,
    "address": "0x2b81945875f892aff04af0a298d35fb2cf848c7b",
    "name": "Web",
    "symbol": "WEB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32957/thumb/WEB300.jpg?1699971838"
  },
  {
    "chainId": 1,
    "address": "0xdc9ac3c20d1ed0b540df9b1fedc10039df13f99c",
    "name": "xMoney",
    "symbol": "UTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1824/thumb/200x200.png?1696723239"
  },
  {
    "chainId": 1,
    "address": "0x1c001d1c9e8c7b8dc717c714d30b31480ab360f5",
    "name": "Gigantix Wallet",
    "symbol": "GTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33285/thumb/logo.png?1701391200"
  },
  {
    "chainId": 1,
    "address": "0x17be403329ced6df4fdb8278f039bc6ffd3d537f",
    "name": "EMMI GG",
    "symbol": "EMMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33286/thumb/EMMI.jpg?1701391439"
  },
  {
    "chainId": 1,
    "address": "0xba25b2281214300e4e649fead9a6d6acd25f1c0a",
    "name": "Tree",
    "symbol": "TREE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33727/thumb/circle_green.png?1709894067"
  },
  {
    "chainId": 1,
    "address": "0x2817cecf94465a9f7becf43d9b7c8025e88a4213",
    "name": "Weave6",
    "symbol": "WX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35789/thumb/400_*_400.png?1709799390"
  },
  {
    "chainId": 1,
    "address": "0x5bb15141bb6def6d2bafeed8ff84bf889c0c573b",
    "name": "LakeViewMeta",
    "symbol": "LVM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27097/thumb/200x200logo.png?1696526146"
  },
  {
    "chainId": 1,
    "address": "0x0b4663216b812e4a2f0fc2029ff1232958f4bf8c",
    "name": "IUCN Coin",
    "symbol": "IUCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32984/thumb/RiCiW7D.png?1700092792"
  },
  {
    "chainId": 1,
    "address": "0x35282d87011f87508d457f08252bc5bfa52e10a0",
    "name": "ULTRA",
    "symbol": "ULTRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35315/thumb/ultra-logo.png?1708161054"
  },
  {
    "chainId": 1,
    "address": "0xaee9ba9ce49fe810417a36408e34d9962b653e78",
    "name": "Snibbu",
    "symbol": "SNIBBU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35933/thumb/photo_2023-05-02_14.17.15.jpeg?1710338968"
  },
  {
    "chainId": 1,
    "address": "0xd3131fb503e9a20261402aec98caf0603cb913f5",
    "name": "Calico Cat",
    "symbol": "CALIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33167/thumb/CALIC.jpg?1700907915"
  },
  {
    "chainId": 1,
    "address": "0x6dda263994aab33f5ed612294e26f2a13df0da05",
    "name": "IQ Protocol",
    "symbol": "IQT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23274/thumb/-zCkqMuQ.jpeg?1700084588"
  },
  {
    "chainId": 1,
    "address": "0x4be2b2c45b432ba362f198c08094017b61e3bdc6",
    "name": "Marswap",
    "symbol": "MSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31280/thumb/Mars-1.png?1696530103"
  },
  {
    "chainId": 1,
    "address": "0x0f5d2fb29fb7d3cfee444a200298f468908cc942",
    "name": "Decentraland",
    "symbol": "MANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/878/thumb/decentraland-mana.png?1696502010"
  },
  {
    "chainId": 1,
    "address": "0x85516e8862ab543ea15972b7809256efec0696ea",
    "name": "ROCK",
    "symbol": "ROCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32976/thumb/ROCK3.png?1700069603"
  },
  {
    "chainId": 1,
    "address": "0xca9d8ef4ba15ae66347b3d22afe2970b89980f88",
    "name": "Innova DeFi",
    "symbol": "INNOVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34023/thumb/Innova.png?1703672390"
  },
  {
    "chainId": 1,
    "address": "0x0d5639a1442730372e2a650a2900ba07937854f4",
    "name": "CallHub",
    "symbol": "CHUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34257/thumb/imgonline-com-ua-Resize-7HrPGgPEWWg9dyT.png?1704359262"
  },
  {
    "chainId": 1,
    "address": "0x9813037ee2218799597d83d4a5b6f3b6778218d9",
    "name": "Bone ShibaSwap",
    "symbol": "BONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16916/thumb/bone_icon.png?1696516487"
  },
  {
    "chainId": 1,
    "address": "0xb4725590574bb8afae4a3f44f05f9c0f5ebd8f4b",
    "name": "Hole Guys",
    "symbol": "HOLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33538/thumb/download.png?1702377725"
  },
  {
    "chainId": 1,
    "address": "0x9b06f3c5de42d4623d7a2bd940ec735103c68a76",
    "name": "Volta Club",
    "symbol": "VOLTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31602/thumb/volta-200x200.png?1696530418"
  },
  {
    "chainId": 1,
    "address": "0xab5d6508e4726141d29c6074ab366afa03f4ec8d",
    "name": "Cybertruck",
    "symbol": "TRUCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33168/thumb/TRUCK.jpg?1700908260"
  },
  {
    "chainId": 1,
    "address": "0x9cf0ed013e67db12ca3af8e7506fe401aa14dad6",
    "name": "Spectre AI",
    "symbol": "SPECTRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33066/thumb/Spectre.png?1710264243"
  },
  {
    "chainId": 1,
    "address": "0x57a5a0567187ff4a8dcc1a9bba86155e355878f2",
    "name": "Inception ETHx",
    "symbol": "INETHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34950/thumb/EthX.png?1706771369"
  },
  {
    "chainId": 1,
    "address": "0xd084b83c305dafd76ae3e1b4e1f1fe2ecccb3988",
    "name": "Virtua",
    "symbol": "TVK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13330/thumb/virtua_original.png?1696513099"
  },
  {
    "chainId": 1,
    "address": "0xa43a7c62d56df036c187e1966c03e2799d8987ed",
    "name": "TruFin Staked MATIC",
    "symbol": "TRUMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32416/thumb/TruMATIC_logo_250px_%281%29.png?1698114190"
  },
  {
    "chainId": 1,
    "address": "0x761a3557184cbc07b7493da0661c41177b2f97fa",
    "name": "ValleyDAO",
    "symbol": "GROW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32991/thumb/valleydao_200x200.png?1700168068"
  },
  {
    "chainId": 1,
    "address": "0x77ebcf0659bbf4e68d8ce6d84bb25c5cde207b97",
    "name": "MOOxMOO",
    "symbol": "MOOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32493/thumb/MOOX.jpg?1698293551"
  },
  {
    "chainId": 1,
    "address": "0xd33526068d116ce69f19a9ee46f0bd304f21a51f",
    "name": "Rocket Pool",
    "symbol": "RPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2090/thumb/rocket_pool_%28RPL%29.png?1696503058"
  },
  {
    "chainId": 1,
    "address": "0xe3944ab788a60ca266f1eec3c26925b95f6370ad",
    "name": "Precipitate ai",
    "symbol": "RAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32405/thumb/RAIN_200-removebg-preview.png?1711371589"
  },
  {
    "chainId": 1,
    "address": "0xfa2629b9cf3998d52726994e0fcdb750224d8b9d",
    "name": "Inception ankrETH",
    "symbol": "INANKRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34952/thumb/ankrETH.png?1706772250"
  },
  {
    "chainId": 1,
    "address": "0x79ca240990ec3f11381a8f80529828aad0628658",
    "name": "DiVinciPay",
    "symbol": "DVNCI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33091/thumb/DVNCI_Token_Logo_Crop.png?1700603214"
  },
  {
    "chainId": 1,
    "address": "0x081a4c60fbc381dc861bae6629c93d835d5f9a89",
    "name": "Modulus Domain Service",
    "symbol": "MODS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29844/thumb/_MODS.png?1696528771"
  },
  {
    "chainId": 1,
    "address": "0xc8168d5665f4418353728ac970713e09c0b7c20e",
    "name": "Monke",
    "symbol": "MONKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33492/thumb/monkelogochipUntitled-1.png?1709173538"
  },
  {
    "chainId": 1,
    "address": "0xe831f96a7a1dce1aa2eb760b1e296c6a74caa9d5",
    "name": "Nexum",
    "symbol": "NEXM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/23472/thumb/200_-_200_coinmarketcap.png?1696522683"
  },
  {
    "chainId": 1,
    "address": "0x0000000000085d4780b73119b644ae5ecd22b376",
    "name": "TrueUSD",
    "symbol": "TUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3449/thumb/tusd.png?1696504140"
  },
  {
    "chainId": 1,
    "address": "0xacdbfcbd4d6d4e9fe72c3ba4280d728ab2ace30f",
    "name": "TargetWatch Bot",
    "symbol": "TWB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33086/thumb/TARGET_%283%29.png?1700602262"
  },
  {
    "chainId": 1,
    "address": "0xa704662ecb62be83f88ca1a3b5277a381cb32dce",
    "name": "Bee Tools",
    "symbol": "BUZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33089/thumb/TOKEN_IMAGE.jpg?1700602823"
  },
  {
    "chainId": 1,
    "address": "0xc1c146e09640fe9e5d670288cc43c6269949d9a1",
    "name": "Robotrade",
    "symbol": "ROBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33256/thumb/photo_2023-11-28_09-42-35.jpg?1701313072"
  },
  {
    "chainId": 1,
    "address": "0xdb8ad3c6d7f9f364d59af9a62c3139cf5c6be88e",
    "name": "XNOVA",
    "symbol": "XNOVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33087/thumb/Transparent_Logo_%28Black-Purple%29_%282%29.png?1700602472"
  },
  {
    "chainId": 1,
    "address": "0xf1df7305e4bab3885cab5b1e4dfc338452a67891",
    "name": "PaLM AI",
    "symbol": "PALM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33097/thumb/PalmProfile01Artboard_6_2x.png?1703862864"
  },
  {
    "chainId": 1,
    "address": "0xd298fe9fa71ad553e35997b19af19fb435038521",
    "name": "Frog Wif Hat",
    "symbol": "FWIF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35583/thumb/Frog_Wif_Hat_Logo_200x200.png?1709202634"
  },
  {
    "chainId": 1,
    "address": "0x60867b79a68abecb78351b2dbb5fdec4282c5acb",
    "name": "VZZN",
    "symbol": "VZZN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33101/thumb/f5040863b9414c388fdad66039175828.png?1700639092"
  },
  {
    "chainId": 1,
    "address": "0x382ac32e9abe17f424a74e8ce0a92c0f18788f9a",
    "name": "Hottel",
    "symbol": "HOTT",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xe0bd989ec7417374a1c0d9d4fb8b692d1a9e27eb",
    "name": "Blackwater Labs",
    "symbol": "BWL",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/34670/thumb/200x200.png?1705656297"
  },
  {
    "chainId": 1,
    "address": "0x6de037ef9ad2725eb40118bb1702ebb27e4aeb24",
    "name": "Render",
    "symbol": "RNDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11636/thumb/rndr.png?1696511529"
  },
  {
    "chainId": 1,
    "address": "0x767fe9edc9e0df98e07454847909b5e959d7ca0e",
    "name": "Illuvium",
    "symbol": "ILV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14468/thumb/logo-200x200.png?1696514154"
  },
  {
    "chainId": 1,
    "address": "0xa0dd6dd7775e93eb842db0aa142c9c581031ed3b",
    "name": "Mind",
    "symbol": "MND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33540/thumb/mind-logo-black-resized.png?1702378297"
  },
  {
    "chainId": 1,
    "address": "0x9abfc0f085c82ec1be31d30843965fcc63053ffe",
    "name": "QSTAR",
    "symbol": "Q",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33177/thumb/C33809C9-D900-437D-A78A-52A8C48423AA.jpeg?1700924002"
  },
  {
    "chainId": 1,
    "address": "0x23dc3f6859e122b25b2cd5f3cf2335310b0f2b77",
    "name": "DeSME",
    "symbol": "DESME",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/33187/thumb/Logo-%28PNG%29%28200%29.png?1700954161"
  },
  {
    "chainId": 1,
    "address": "0x8be8b0dde627dd6be9a4b7fcadd04899aabdb2b1",
    "name": "Rand",
    "symbol": "RND",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33114/thumb/rnd_icon_200.png?1705089933"
  },
  {
    "chainId": 1,
    "address": "0x68592c5c98c4f4a8a4bc6da2121e65da3d1c0917",
    "name": "Stable USDLR",
    "symbol": "USDLR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33115/thumb/0x68592c5c98c4f4a8a4bc6da2121e65da3d1c0917.png?1700731571"
  },
  {
    "chainId": 1,
    "address": "0x7865ec47bef9823ad0010c4970ed90a5e8107e53",
    "name": "NeoAudit AI",
    "symbol": "NAAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33181/thumb/logo-on-black_%282%29.png?1700925393"
  },
  {
    "chainId": 1,
    "address": "0x50d1c9771902476076ecfc8b2a83ad6b9355a4c9",
    "name": "FTX",
    "symbol": "FTT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9026/thumb/F.png?1696509161"
  },
  {
    "chainId": 1,
    "address": "0x1e241521f4767853b376c2fe795a222a07d588ee",
    "name": "Uni Terminal",
    "symbol": "UNIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33458/thumb/Logo.png?1701925230"
  },
  {
    "chainId": 1,
    "address": "0xbb0e17ef65f82ab018d8edd776e8dd940327b28b",
    "name": "Axie Infinity",
    "symbol": "AXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13029/thumb/axie_infinity_logo.png?1696512817"
  },
  {
    "chainId": 1,
    "address": "0x3b6564b5da73a41d3a66e6558a98fd0e9e1e77ad",
    "name": "Unitus",
    "symbol": "UTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33178/thumb/IMG_0051.jpeg?1700924333"
  },
  {
    "chainId": 1,
    "address": "0xf1c9acdc66974dfb6decb12aa385b9cd01190e38",
    "name": "StakeWise Staked ETH",
    "symbol": "OSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33117/thumb/Frame_27513839.png?1700732599"
  },
  {
    "chainId": 1,
    "address": "0x1a8b8e526d093476ac5c488a3ea057f8de9c0dee",
    "name": "JEFFWorld Token",
    "symbol": "JEFF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33188/thumb/output-onlinepngtools_%283%29.png?1700954618"
  },
  {
    "chainId": 1,
    "address": "0x8dd09822e83313adca54c75696ae80c5429697ff",
    "name": "Sifu Vision",
    "symbol": "SIFU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35601/thumb/quewxQN.png?1709267056"
  },
  {
    "chainId": 1,
    "address": "0xd533a949740bb3306d119cc777fa900ba034cd52",
    "name": "Curve DAO",
    "symbol": "CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12124/thumb/Curve.png?1696511967"
  },
  {
    "chainId": 1,
    "address": "0x9f8f72aa9304c8b593d555f12ef6589cc3a579a2",
    "name": "Maker",
    "symbol": "MKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1364/thumb/Mark_Maker.png?1696502423"
  },
  {
    "chainId": 1,
    "address": "0xa2762ba628b962f93498d8893b6e4346140fe96d",
    "name": "Intrepid Token",
    "symbol": "INT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33229/thumb/int.png?1701257932"
  },
  {
    "chainId": 1,
    "address": "0xea068fba19ce95f12d252ad8cb2939225c4ea02d",
    "name": "Fief",
    "symbol": "FIEF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23710/thumb/61f837d2703b649ab0213d45_Black_logo_-_no_background-p-500.png?1696522910"
  },
  {
    "chainId": 1,
    "address": "0xe13cf110176e0dd6590536cd391b8a3522475f82",
    "name": "RentAI",
    "symbol": "RENT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36421/thumb/RENT_TRANSPARENT_200.png?1711428441"
  },
  {
    "chainId": 1,
    "address": "0xc18360217d8f7ab5e7c516566761ea12ce7f9d72",
    "name": "Ethereum Name Service",
    "symbol": "ENS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19785/thumb/acatxTm8_400x400.jpg?1696519207"
  },
  {
    "chainId": 1,
    "address": "0x8f602ee5b1f8de67120c427717cbe2770f3cf320",
    "name": "Starry",
    "symbol": "STARRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33190/thumb/STARRY.jpg?1700968743"
  },
  {
    "chainId": 1,
    "address": "0xe41d2489571d322189246dafa5ebde1f4699f498",
    "name": "0x Protocol",
    "symbol": "ZRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/863/thumb/0x.png?1696501996"
  },
  {
    "chainId": 1,
    "address": "0x4f4a556361b8b4869f97b8709ff47c1b057ea13b",
    "name": "MAGA",
    "symbol": "TRUMP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33235/thumb/logo_200px.png?1701147167"
  },
  {
    "chainId": 1,
    "address": "0xfed802a7e561d6324818358790b176a01d765f19",
    "name": "BlockRock",
    "symbol": "FED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33236/thumb/d4ac260d-4ee6-4450-b302-6dbd7dde85db.jpeg?1701147502"
  },
  {
    "chainId": 1,
    "address": "0xbd617a1359086e33ff339ea0b9c6de479a3f5943",
    "name": "Ide x ai",
    "symbol": "IDE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33240/thumb/logo_ide-1-150x150.png?1701148159"
  },
  {
    "chainId": 1,
    "address": "0x4f604735c1cf31399c6e711d5962b2b3e0225ad3",
    "name": "Glo Dollar",
    "symbol": "USDGLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29319/thumb/glo_logo_coingecko.png?1696528270"
  },
  {
    "chainId": 1,
    "address": "0xcfeb09c3c5f0f78ad72166d55f9e6e9a60e96eec",
    "name": "VEMP",
    "symbol": "VEMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18074/thumb/VEMP_Token_200x200.png?1696517581"
  },
  {
    "chainId": 1,
    "address": "0xf282484234d905d7229a6c22a0e46bb4b0363ee0",
    "name": "Revswap",
    "symbol": "RVS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33437/thumb/20231205_174957.png?1701847754"
  },
  {
    "chainId": 1,
    "address": "0x661ecf29b533e7d09f67f597af77eeaf3cc6b1e7",
    "name": "0xNude",
    "symbol": "NUDE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33459/thumb/0xNude.png?1701925329"
  },
  {
    "chainId": 1,
    "address": "0x1e354f9ab5bcc9fb981f31b794c5fe13f7a89218",
    "name": "Neural Tensor Dynamics",
    "symbol": "NTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36532/thumb/Neural_Tensor_Dynamics.png?1711705100"
  },
  {
    "chainId": 1,
    "address": "0x6f38e0f1a73c96cb3f42598613ea3474f09cb200",
    "name": "Draggable Aktionariat AG",
    "symbol": "DAKS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/28988/thumb/AKS.png?1696527961"
  },
  {
    "chainId": 1,
    "address": "0xc5b490371870201e2fe6308850507db6e9cba732",
    "name": "Baby GROK",
    "symbol": "BROK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33239/thumb/200x200_bork.png?1701147976"
  },
  {
    "chainId": 1,
    "address": "0xb0f92f94d02a4d634fd394c1889fe3cab1fcffc7",
    "name": "DogeBoy",
    "symbol": "DOGB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33241/thumb/IMG_20231127_212135_822.png?1701170913"
  },
  {
    "chainId": 1,
    "address": "0x2e880962a9609aa3eab4def919fe9e917e99073b",
    "name": "Aktionariat Boss Info AG Tokenized Shar",
    "symbol": "BOSS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33248/thumb/BOSS_%281%29.png?1701172435"
  },
  {
    "chainId": 1,
    "address": "0x443f9ff91a72f1482d13dfd40eec107bb84ca1e5",
    "name": "FarmBot",
    "symbol": "FARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33440/thumb/farmbotlogo200px.jpeg?1701849705"
  },
  {
    "chainId": 1,
    "address": "0xbaac2b4491727d78d2b78815144570b9f2fe8899",
    "name": "The Doge NFT",
    "symbol": "DOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18111/thumb/Doge.png?1696517615"
  },
  {
    "chainId": 1,
    "address": "0xabc93b1c8ffa3a69f478068676360021bbaab4bf",
    "name": "DigiFund Capital V2",
    "symbol": "DFUND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33378/thumb/dfund.png?1701660694"
  },
  {
    "chainId": 1,
    "address": "0x9d1a74967eca155782edf8e84782c74db33fc499",
    "name": "AI COM",
    "symbol": "AICOM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33246/thumb/BbLZo3AH_400x400.jpg?1701172033"
  },
  {
    "chainId": 1,
    "address": "0x23feef978f2f8e9aeb26028d09b5d303b6c6a057",
    "name": "Sacabambaspis",
    "symbol": "SACA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33299/thumb/photo_2023-11-30_03.46.16.jpeg?1701402704"
  },
  {
    "chainId": 1,
    "address": "0xb970e14df2161c0a2f32eba35901f2446581b482",
    "name": "Reaktor",
    "symbol": "RKR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33291/thumb/REAKTOR.png?1701400698"
  },
  {
    "chainId": 1,
    "address": "0xa4bdb11dc0a2bec88d24a3aa1e6bb17201112ebe",
    "name": "USDS Classic",
    "symbol": "USDS",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/7596/thumb/USDS_Classic_Standard.png?1702476175"
  },
  {
    "chainId": 1,
    "address": "0x8b12bd54ca9b2311960057c8f3c88013e79316e3",
    "name": "Reach",
    "symbol": "REACH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33776/thumb/reach.jpeg?1702970338"
  },
  {
    "chainId": 1,
    "address": "0x7f716b4777169ebd8ff132acfff59399049a137b",
    "name": "Gameluk",
    "symbol": "GAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35697/thumb/Gameluk-logo-200*200.png?1709549680"
  },
  {
    "chainId": 1,
    "address": "0xbf40440703a3f7092b2e73c2f7868727275dbbda",
    "name": "Keep Finance",
    "symbol": "KEEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35699/thumb/Keep-icon-200*200.png?1709550148"
  },
  {
    "chainId": 1,
    "address": "0x824e35f7a75324f99300afac75ecf7354e17ea26",
    "name": "TIA",
    "symbol": "TIA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/23994/thumb/tia.png?1696523188"
  },
  {
    "chainId": 1,
    "address": "0x2d3590fac07b91c9451e8d16c372ece9758f040f",
    "name": "ratsDAO",
    "symbol": "RAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33385/thumb/200x200.png?1701671914"
  },
  {
    "chainId": 1,
    "address": "0xf995771a957c19319a7d8d58b4082b049420340f",
    "name": "BONK Inu",
    "symbol": "BONKI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33510/thumb/IMG_20231207_184637_498_%282%29.jpg?1702041006"
  },
  {
    "chainId": 1,
    "address": "0x9dc251f27a7477313242a022181d4b5a3a1e4a0f",
    "name": "Unibets AI",
    "symbol": "BETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33388/thumb/_BETS.jpg?1701677276"
  },
  {
    "chainId": 1,
    "address": "0x30b593f8c3ab37615359b4e0e6df2e06d55bb55d",
    "name": "FXDX",
    "symbol": "FXDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32730/thumb/ICONIC_LOGO_BLACK_PNG_%281%29.png?1699217673"
  },
  {
    "chainId": 1,
    "address": "0x4ffe9cc172527df1e40d0b2efe1e9f05884a13da",
    "name": "DEDPRZ",
    "symbol": "USA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33381/thumb/NewDcoin_%283%29.png?1705599002"
  },
  {
    "chainId": 1,
    "address": "0x551d0501cd5df92663c3d12c3201c9d70ba79998",
    "name": "All Your Base",
    "symbol": "YOBASE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33595/thumb/Group_48095820logo_cmc.png?1702476376"
  },
  {
    "chainId": 1,
    "address": "0x44face2e310e543f6d85867eb06fb251e3bfe1fc",
    "name": "404 Bakery",
    "symbol": "BAKE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35130/thumb/404_4_%282%29.png?1707440511"
  },
  {
    "chainId": 1,
    "address": "0xb4c25205f7a76ae08cbe95e27864106e1b93732c",
    "name": "Extropic AI",
    "symbol": "EXTROPIC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33408/thumb/Picsart_23-12-04_05-49-36-727.png?1701700613"
  },
  {
    "chainId": 1,
    "address": "0xd5fa38027462691769b8a8ba6c444890103b5b94",
    "name": "Dawg Coin",
    "symbol": "DAWG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33409/thumb/dawg_%281%29.jpg?1701700702"
  },
  {
    "chainId": 1,
    "address": "0x922d8563631b03c2c4cf817f4d18f6883aba0109",
    "name": "Houdini Swap",
    "symbol": "LOCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35132/thumb/LOCK_Logo.png?1707443455"
  },
  {
    "chainId": 1,
    "address": "0x378878af3228d23891de306cb2b123d837d0dcf1",
    "name": "Anonify",
    "symbol": "ONI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35426/thumb/Token-image_%281%29.png?1708572251"
  },
  {
    "chainId": 1,
    "address": "0xf56b164efd3cfc02ba739b719b6526a6fa1ca32a",
    "name": "Curio Governance Token",
    "symbol": "CGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13607/thumb/QLwpua7.png?1696513357"
  },
  {
    "chainId": 1,
    "address": "0x2fb652314c3d850e9049057bbe9813f1eee882d3",
    "name": "RocketX Exchange",
    "symbol": "RVF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14728/thumb/avatar-correct-size-official.png?1696514397"
  },
  {
    "chainId": 1,
    "address": "0xe3668873d944e4a949da05fc8bde419eff543882",
    "name": "Yearn yPRISMA",
    "symbol": "YPRISMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33481/thumb/Asset_yPRISMA__Type_Main.png?1702020049"
  },
  {
    "chainId": 1,
    "address": "0xe90ce7764d8401d19ed3733a211bd3b06c631bc0",
    "name": "The Other Party",
    "symbol": "POD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33478/thumb/Twitter_header_-_4.png?1702017433"
  },
  {
    "chainId": 1,
    "address": "0x559cc0850361afe1973c0ba5d0a3446c8a5ad678",
    "name": "Avatar404",
    "symbol": "",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35589/thumb/avatar404.jpeg?1709204091"
  },
  {
    "chainId": 1,
    "address": "0x3eb9c7ee5f72e51f61e832137719fe8d1e53a2ce",
    "name": "DecentraMind",
    "symbol": "DMIND",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33315/thumb/IMG_20231130_105535_975.jpg?1701413610"
  },
  {
    "chainId": 1,
    "address": "0xd5f7838f5c461feff7fe49ea5ebaf7728bb0adfa",
    "name": "Mantle Staked Ether",
    "symbol": "METH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33345/thumb/symbol_transparent_bg.png?1701697066"
  },
  {
    "chainId": 1,
    "address": "0xe73cec024b30a7195af80d13f3b6917d80af11d8",
    "name": "GOATED",
    "symbol": "GOAT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34735/thumb/IMG_20240121_025525_360.jpg?1705943971"
  },
  {
    "chainId": 1,
    "address": "0x01bf66becdcfd6d59a5ca18869f494fea086cdfd",
    "name": "KUMA Protocol US KUMA Interest Bearing ",
    "symbol": "USK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33445/thumb/USK.png?1701888523"
  },
  {
    "chainId": 1,
    "address": "0xaf4ce7cd4f8891ecf1799878c3e9a35b8be57e09",
    "name": "KUMA Protocol Wrapped USK",
    "symbol": "WUSK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33446/thumb/USK.png?1701888542"
  },
  {
    "chainId": 1,
    "address": "0x5e72ad4bf50c952b11a63b6769d02bb486a9a897",
    "name": "Lucky8",
    "symbol": "888",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33448/thumb/Frame_742.png?1701856566"
  },
  {
    "chainId": 1,
    "address": "0x79349edd0b8e83ffaa1af2e6ba0c8ce87731c267",
    "name": "WcDonalds",
    "symbol": "WCD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35897/thumb/wcdonalds.png?1710163437"
  },
  {
    "chainId": 1,
    "address": "0x38c2a4a7330b22788374b8ff70bba513c8d848ca",
    "name": "Truflation",
    "symbol": "TRUF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36642/thumb/TRUF_Token_Logo_Blue.png?1712034385"
  },
  {
    "chainId": 1,
    "address": "0x17f7fea5f87d0f289d9827be78fc96027f8ed724",
    "name": "Aktionariat EHC Kloten Sport AG Tokeniz",
    "symbol": "EHCK",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33390/thumb/EHCK.png?1701680177"
  },
  {
    "chainId": 1,
    "address": "0xc47b0443f8608943c596168504bb6d4a062e0edc",
    "name": "Lacostoken",
    "symbol": "LCSN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33282/thumb/20231129_210945.png?1701362709"
  },
  {
    "chainId": 1,
    "address": "0x273b8e7adddcb4de101416300fcd3688c0612a27",
    "name": "Rabbit Games",
    "symbol": "RAIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33807/thumb/%E5%BE%AE%E4%BF%A1%E5%9B%BE%E7%89%87_20231218214054.jpg?1702993200"
  },
  {
    "chainId": 1,
    "address": "0x64d5fea7d2d600918b76159285994d6ed218f264",
    "name": "Tao Accounting System",
    "symbol": "TAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35591/thumb/tas-logo.png?1709204338"
  },
  {
    "chainId": 1,
    "address": "0x6bfdb6f4e65ead27118592a41eb927cea6956198",
    "name": "FAME AI",
    "symbol": "FMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33574/thumb/20231205-153104.png?1702450853"
  },
  {
    "chainId": 1,
    "address": "0xc7937b44532bf4c0a1f0de3a46c79dddb6dd169d",
    "name": "Deez Nuts  ERC404 ",
    "symbol": "DN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35592/thumb/deez-nuts.png?1709204399"
  },
  {
    "chainId": 1,
    "address": "0x490bd60a5d3e1207fba9b699017561434cc8c675",
    "name": "Bugs Bunny",
    "symbol": "BUGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33398/thumb/Bugs_200x200.png?1701684815"
  },
  {
    "chainId": 1,
    "address": "0xad8f047d9b742565bb9e10c7655bd3ee9c1eab75",
    "name": "Grok Chain",
    "symbol": "GROC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33400/thumb/photo_2023-11-29_17-47-05.png?1701684970"
  },
  {
    "chainId": 1,
    "address": "0xbe9f4f6c8dadb2ac61f31eb1f5171e27d8552df7",
    "name": "Chiba Neko",
    "symbol": "CHIBA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32555/thumb/logo_200x200.png?1698502949"
  },
  {
    "chainId": 1,
    "address": "0x1d00e86748573c322f4cc41518aa0e77bd912eb4",
    "name": "Ethereum Reserve Dollar USDE",
    "symbol": "USDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33542/thumb/USDE_200*200.png?1702383206"
  },
  {
    "chainId": 1,
    "address": "0xa408090a36cef0b23c3f518484a4e06c7c7df27c",
    "name": "FuzangLong",
    "symbol": "LONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35134/thumb/photo_2024-02-08_08-07-28.jpg?1707444987"
  },
  {
    "chainId": 1,
    "address": "0xa069add2d093f9df0e82ab64ec7dd0320cb4f65d",
    "name": "GPUBot",
    "symbol": "GPUBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35627/thumb/GPUBot.png?1709291157"
  },
  {
    "chainId": 1,
    "address": "0x9f8ae89b695fd1773175ccb9106a90ce868acab0",
    "name": "AltFi",
    "symbol": "ALT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33418/thumb/altfi_200x200.png?1701762359"
  },
  {
    "chainId": 1,
    "address": "0x470c8950c0c3aa4b09654bc73b004615119a44b5",
    "name": "Kizuna",
    "symbol": "KIZUNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32113/thumb/IMG_9968.png?1696585475"
  },
  {
    "chainId": 1,
    "address": "0xf7ff660cbdcc5d0ba48988119144425974d88792",
    "name": "Scope Sniper",
    "symbol": "SCOPE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33419/thumb/photo_2023-11-30_08-53-48.jpg?1701766212"
  },
  {
    "chainId": 1,
    "address": "0x878fcc2bdcccff8c56812607b9a58f29b274c4f0",
    "name": "Derp Coin",
    "symbol": "DERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30262/thumb/logo-200x200-transparent.png?1696529170"
  },
  {
    "chainId": 1,
    "address": "0x0c7b199ac2bca0dba8d1785480648f0318b9a7b8",
    "name": "OrdiBot",
    "symbol": "ORDIBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33462/thumb/OIG_%281%29.png?1701927925"
  },
  {
    "chainId": 1,
    "address": "0xe3e24b4ea87935e15bbe99a24e9ace9998e4614d",
    "name": "Aidi Finance",
    "symbol": "AIDI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29675/thumb/aidi_finance.png?1696528610"
  },
  {
    "chainId": 1,
    "address": "0xedf978ee4beafbeefd7a0f55faa5a7c6c3cd467d",
    "name": "FP  CoolCats",
    "symbol": "UCOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32781/thumb/cool_%281%29.png?1699350874"
  },
  {
    "chainId": 1,
    "address": "0xd01d133166820557db7138963bcd9009c54e4c33",
    "name": "ChainEx",
    "symbol": "CEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33477/thumb/ChainEx-2.png?1702013835"
  },
  {
    "chainId": 1,
    "address": "0x710287d1d39dcf62094a83ebb3e736e79400068a",
    "name": "enqAI",
    "symbol": "ENQAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29908/thumb/icon.png?1702507913"
  },
  {
    "chainId": 1,
    "address": "0x9870fb2a93fe3c1ad2fc585dba873e5cb77f3e4c",
    "name": "MrsPepe",
    "symbol": "MRSPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30155/thumb/Untitled_design_%2896%29.png?1702747094"
  },
  {
    "chainId": 1,
    "address": "0xbfba57800bce200915499078289299c86af4a379",
    "name": "ORDINAL BITCOIN",
    "symbol": "OBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33940/thumb/IMG_0418.jpeg?1703503503"
  },
  {
    "chainId": 1,
    "address": "0x9e20461bc2c4c980f62f1b279d71734207a6a356",
    "name": "OmniCat",
    "symbol": "OMNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33917/thumb/2023-12-21_21.39.10.jpg?1703321588"
  },
  {
    "chainId": 1,
    "address": "0xe6f28389daf01ae12b33ee5e7f1e130c454e8494",
    "name": "AntiBot",
    "symbol": "ATB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33495/thumb/IMG_5554.png?1702076259"
  },
  {
    "chainId": 1,
    "address": "0x78993f9bee8b68f2531a92427595405f294161db",
    "name": "0xBET",
    "symbol": "0XBET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35790/thumb/V7LdNy1d_400x400.jpg?1709799471"
  },
  {
    "chainId": 1,
    "address": "0xaf69d0518d341af4961b5fcfdb86a3e3453f1dc5",
    "name": "Longchenchen",
    "symbol": "LONG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33505/thumb/longerc20.jpg?1702039017"
  },
  {
    "chainId": 1,
    "address": "0x4a88ebf6f76b04f5e0e71a351a22e573f636affe",
    "name": "Yoyo Market",
    "symbol": "YOYO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33548/thumb/yoyopngcg.png?1702499846"
  },
  {
    "chainId": 1,
    "address": "0x6034e0d6999741f07cb6fb1162cbaa46a1d33d36",
    "name": "Molecules of Korolchuk IP NFT",
    "symbol": "VITA-FAST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33055/thumb/Vita-FAST_1024px.png?1700475313"
  },
  {
    "chainId": 1,
    "address": "0x553531c45eabc2f3343d19885262ac784046fd5f",
    "name": "CyOp",
    "symbol": "CYOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33598/thumb/CYOP.jpg?1702477495"
  },
  {
    "chainId": 1,
    "address": "0xa685406eff334b1368318d0325fc2cdc0e7086f1",
    "name": "PARTY",
    "symbol": "PARTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33550/thumb/party-logo-200x200.png?1702397278"
  },
  {
    "chainId": 1,
    "address": "0x87c22db324b8b0637c8f09d2670ae7777651dbb8",
    "name": "Root Protocol",
    "symbol": "ISME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36564/thumb/root-0313_%282%29.jpg?1711906595"
  },
  {
    "chainId": 1,
    "address": "0x7658b8cd4719080ad377dfe28dfc19c761a3cb75",
    "name": "MevAI",
    "symbol": "MAI",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xa4ffdf3208f46898ce063e25c1c43056fa754739",
    "name": "AthenaDAO",
    "symbol": "ATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33349/thumb/ATH_Token_Black_Logo.png?1701981879"
  },
  {
    "chainId": 1,
    "address": "0xb5c23f694810a8a6a37fede38004150ee0dd3821",
    "name": "0xTools",
    "symbol": "0XT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/33551/thumb/2023-12-10_19.37.28.jpg?1702428574"
  },
  {
    "chainId": 1,
    "address": "0xccc8cb5229b0ac8069c51fd58367fd1e622afd97",
    "name": "Gods Unchained",
    "symbol": "GODS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17139/thumb/10631.png?1696516698"
  },
  {
    "chainId": 1,
    "address": "0x04c154b66cb340f3ae24111cc767e0184ed00cc6",
    "name": "Dinero Staked ETH",
    "symbol": "PXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33608/thumb/pxETH-icon-transbg_72dpi.png?1702512024"
  },
  {
    "chainId": 1,
    "address": "0xf5b339d3912d5c72e16440b63a9c1df6024de93c",
    "name": "Baby Grok",
    "symbol": "BABYGROK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33397/thumb/HZdZYbd2_400x400.jpg?1701684676"
  },
  {
    "chainId": 1,
    "address": "0xb62132e35a6c13ee1ee0f84dc5d40bad8d815206",
    "name": "NEXO",
    "symbol": "NEXO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3695/thumb/nexo.png?1696504370"
  },
  {
    "chainId": 1,
    "address": "0x000000000075f13bcf2e6652e84821e8b544f6f9",
    "name": "Signet",
    "symbol": "SIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33552/thumb/st.png?1702428755"
  },
  {
    "chainId": 1,
    "address": "0x3e316791842a271ab1e138ff7408c015efd9c6be",
    "name": "Vortex AI",
    "symbol": "VXAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33571/thumb/IMG_20231210_193633_029.jpg?1702449851"
  },
  {
    "chainId": 1,
    "address": "0x9ca98c8b217c3b45074834908555d36af2ac6449",
    "name": "SatsBridge",
    "symbol": "SABR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34813/thumb/1000002973.png?1706123473"
  },
  {
    "chainId": 1,
    "address": "0x02020595e6a34a03a8e9c1f5624b1b7713810083",
    "name": "Spectra Chain",
    "symbol": "SPCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36791/thumb/0x02020595e6a34a03a8e9c1f5624b1b7713810083.jpg?1712421016"
  },
  {
    "chainId": 1,
    "address": "0x9ba021b0a9b958b5e75ce9f6dff97c7ee52cb3e6",
    "name": "Dinero apxETH",
    "symbol": "APXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33609/thumb/apxETH-icon-transbg_72dpi.png?1702512027"
  },
  {
    "chainId": 1,
    "address": "0x795a7e7a0f6b84ee1f2bc9e70a74dbbb49cef9a3",
    "name": "Intrinsic Number Up",
    "symbol": "INU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33631/thumb/INU_LOGO_CG___CMC.png?1705766451"
  },
  {
    "chainId": 1,
    "address": "0x5441765d3ab74e0347df52ffab5a69e5146b5d26",
    "name": "BTCMEME",
    "symbol": "BTCMEME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33600/thumb/IMG_20231212_162704_611.jpg?1702509475"
  },
  {
    "chainId": 1,
    "address": "0x668308d77be3533c909a692302cb4d135bf8041c",
    "name": "Inception sfrxETH",
    "symbol": "INSFRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34949/thumb/sfrxETH.png?1706770412"
  },
  {
    "chainId": 1,
    "address": "0x0cdbbc7fe1c8da0cc41ba96d7edb4cce5982f23f",
    "name": "UNITS LIMITED SUPPLY",
    "symbol": "ULS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33560/thumb/dark_token.png?1702429859"
  },
  {
    "chainId": 1,
    "address": "0x01202c9a1adfc1475c960c23bdf7530698330fa0",
    "name": "Judge AI",
    "symbol": "JUDGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34959/thumb/4MUdYI7V_200x200.jpg?1706774333"
  },
  {
    "chainId": 1,
    "address": "0x01e1d7cbd3bc0eb1030485f33708421011459459",
    "name": "TOAD",
    "symbol": "TOAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33716/thumb/TOAD.png?1702897534"
  },
  {
    "chainId": 1,
    "address": "0x102c776ddb30c754ded4fdcc77a19230a60d4e4f",
    "name": "Floor Protocol",
    "symbol": "FLC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32433/thumb/Floor_Protocol_browser_logo_mark.png?1702379738"
  },
  {
    "chainId": 1,
    "address": "0x36705e789d0d1ba53fd9bb93512722018b0a089e",
    "name": "CortexLoop",
    "symbol": "CRTX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35793/thumb/ava-1_%284%29.png?1709800646"
  },
  {
    "chainId": 1,
    "address": "0xa187927c9185108458647aeec193ef4a62d3bd80",
    "name": "Meta Minigames",
    "symbol": "MMG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35380/thumb/MMG_Black_%281%29.png?1708422554"
  },
  {
    "chainId": 1,
    "address": "0x40c3b81fb887016c0ad02436309c2b265d069a05",
    "name": "Chief Troll Officer",
    "symbol": "CTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34736/thumb/IMG_1873.jpeg?1705944614"
  },
  {
    "chainId": 1,
    "address": "0x64a5b80089c0fb4858a8c2ca9c2988a484539fd8",
    "name": "Print Mining",
    "symbol": "PRINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33606/thumb/Print_miner_%28Custom%29.jpg?1702511022"
  },
  {
    "chainId": 1,
    "address": "0x5e05f367a1923b2a886e2f2bc45c2278a0b9b448",
    "name": "BRC20 BOT",
    "symbol": "BRCBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33607/thumb/IMG_20231206_032536_412.jpg?1702511057"
  },
  {
    "chainId": 1,
    "address": "0xe73d2faeeb9dec890c905b707b574627d5302cb3",
    "name": "Defi Radar",
    "symbol": "DRADAR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34304/thumb/Safeimagekit-resized-img_%2819%29.png?1704612554"
  },
  {
    "chainId": 1,
    "address": "0x6a4f9a1372235c3fea4fb07170a635b26417382b",
    "name": "Wonderland Capital",
    "symbol": "ALICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33485/thumb/Wonderland.png?1702028786"
  },
  {
    "chainId": 1,
    "address": "0x146ae546b7a2d676e5ed9e5d92647c8ddb8e2388",
    "name": "Grand Theft Degens",
    "symbol": "GTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33633/thumb/Grand_Theft_Degens.jpg?1702535287"
  },
  {
    "chainId": 1,
    "address": "0xdc5e9445169c73cf21e1da0b270e8433cac69959",
    "name": "Ketaicoin",
    "symbol": "ETHEREUM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31326/thumb/photo_2023-08-15_21-35-43.jpg?1696530145"
  },
  {
    "chainId": 1,
    "address": "0xcc9e0bd9438ca0056653d134de794abeaff8c676",
    "name": "Leslie",
    "symbol": "LESLIE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33189/thumb/IMG_0721_%282%29.png?1700955081"
  },
  {
    "chainId": 1,
    "address": "0x6dd5f0038474dc29a0adc6ad34d37b0ba53e5435",
    "name": "DEXED",
    "symbol": "DEXED",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35617/thumb/coingecko.png?1709283896"
  },
  {
    "chainId": 1,
    "address": "0xf2041be4ea84599818799eed882389a8a30d2226",
    "name": "NeuralBot",
    "symbol": "NEURAL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33717/thumb/Neuralbot_ai.png?1702880084"
  },
  {
    "chainId": 1,
    "address": "0xd9a442856c234a39a81a089c06451ebaa4306a72",
    "name": "pufETH",
    "symbol": "PUFETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35176/thumb/pufETH-200-200-resolution.png?1707753174"
  },
  {
    "chainId": 1,
    "address": "0x695d38eb4e57e0f137e36df7c1f0f2635981246b",
    "name": "Meme AI Coin",
    "symbol": "MEMEAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34356/thumb/MEMEAI.png?1704706407"
  },
  {
    "chainId": 1,
    "address": "0xeb9e3c6cda17c8755a4e8a1cc944afb7fc7d7905",
    "name": "MatchTrade",
    "symbol": "MATCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34520/thumb/Frame_16.png?1705301241"
  },
  {
    "chainId": 1,
    "address": "0x45e412e1878080815d6d51d47b83d17869433459",
    "name": "Chief Troll Officer",
    "symbol": "CTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34518/thumb/Untitled_design-13.png?1705295928"
  },
  {
    "chainId": 1,
    "address": "0xfe1b621235227bfa14ed81be2ba4e5e4447b52c0",
    "name": "ORDINAL TOOLS",
    "symbol": "ORT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33623/thumb/photo_2023-12-13_21-16-24.jpg?1702529153"
  },
  {
    "chainId": 1,
    "address": "0xce3f08e664693ca792cace4af1364d5e220827b2",
    "name": "Saitama",
    "symbol": "SAITAMA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16353/thumb/SOIKDUWf_400x400.jpeg?1696515952"
  },
  {
    "chainId": 1,
    "address": "0xd23b160e4a0db09ca35a79948e04cf6eafb5283a",
    "name": "Blend Protocol",
    "symbol": "BLEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33766/thumb/Blend_Protocol_Logo_tsp.png?1702967995"
  },
  {
    "chainId": 1,
    "address": "0x13c4b558f6663329c13c838cc0b5b796f7fc0531",
    "name": "Horizon",
    "symbol": "HRZN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33740/thumb/moon.jpg?1702903432"
  },
  {
    "chainId": 1,
    "address": "0x2056ec69ac5afaf210b851ff74de4c194fcd986e",
    "name": "Port AI",
    "symbol": "POAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36792/thumb/image_2024-03-25_19-40-43.png?1712421434"
  },
  {
    "chainId": 1,
    "address": "0x12652c6d93fdb6f4f37d48a8687783c782bb0d10",
    "name": "Entangle",
    "symbol": "NGL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34522/thumb/entangle.jpeg?1705302451"
  },
  {
    "chainId": 1,
    "address": "0x455ad1bc4e18fd4e369234b6e11d88acbc416758",
    "name": "BRC App",
    "symbol": "BRCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34244/thumb/BRC_X_Banners-19.jpg?1704338085"
  },
  {
    "chainId": 1,
    "address": "0x9b3a8159e119eb09822115ae08ee1526849e1116",
    "name": "Meme Alliance",
    "symbol": "MMA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33765/thumb/200x200.png?1709978615"
  },
  {
    "chainId": 1,
    "address": "0x19ae49b9f38dd836317363839a5f6bfbfa7e319a",
    "name": "SaitaChain Coin",
    "symbol": "STC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33665/thumb/Saitama_200pxby200px-1.png?1703066750"
  },
  {
    "chainId": 1,
    "address": "0xaf1eeb83c364ad9ffeb5f97f223c1705d4810033",
    "name": "MoneyArk",
    "symbol": "MARK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33987/thumb/LOGO.png?1703692373"
  },
  {
    "chainId": 1,
    "address": "0x57ca1f39b0757b8610617e90fa9b2980ccebfece",
    "name": "OrdinalsFi",
    "symbol": "ORDIFI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33723/thumb/ordinalsfi200.png?1702885610"
  },
  {
    "chainId": 1,
    "address": "0x600d601d8b9eb5de5ac90fefc68d0d08801bfd3f",
    "name": "Element",
    "symbol": "ELMT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/33775/thumb/ELMT_Logo_Mark_Filled_-_Neon_LG.png?1702970207"
  },
  {
    "chainId": 1,
    "address": "0x4f9254c83eb525f9fcf346490bbb3ed28a81c667",
    "name": "Celer Network",
    "symbol": "CELR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4379/thumb/Celr.png?1696504978"
  },
  {
    "chainId": 1,
    "address": "0x1eb7bd905855c483db19f53c8c4d42db42a159fc",
    "name": "Nordic Ai",
    "symbol": "NRDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36793/thumb/NordicAI_logo_%281%29.png?1712421635"
  },
  {
    "chainId": 1,
    "address": "0xdf87270e04bc5ac140e93571d0dd0c6f4a058b41",
    "name": "Moolahverse",
    "symbol": "MLH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33724/thumb/mlh-logo-200x200.png?1702886528"
  },
  {
    "chainId": 1,
    "address": "0x5483dc6abda5f094865120b2d251b5744fc2ecb5",
    "name": "TaoPad",
    "symbol": "TPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33664/thumb/TAOWHITE.png?1702631267"
  },
  {
    "chainId": 1,
    "address": "0x1e610de0d7acfa1d820024948a91d96c5c9ce6b9",
    "name": "FP  BoredApeYachtClub",
    "symbol": "UBAYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32690/thumb/bayctoken.png?1698974036"
  },
  {
    "chainId": 1,
    "address": "0x3acfc40a19520d97648eb7c0891e747b7f2b0283",
    "name": "FP  Azuki",
    "symbol": "UAZUKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32676/thumb/vsF28_QV_400x400.jpg?1698911738"
  },
  {
    "chainId": 1,
    "address": "0x30f7c830e0c2f4bec871df809d73e27ef19eb151",
    "name": "FP  PudgyPenguins",
    "symbol": "UPPG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32691/thumb/pudgy_penguins_64_64.png?1698974130"
  },
  {
    "chainId": 1,
    "address": "0x359108ca299ca693502ef217e2109ad02aa4277c",
    "name": "FP  MutantApeYachtClub",
    "symbol": "UMAYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32675/thumb/mayc_logo.69fd41b5.png?1698911670"
  },
  {
    "chainId": 1,
    "address": "0xec19caef9c046f5f87a497154766742ab9c90820",
    "name": "FP  Y00ts",
    "symbol": "UY00TS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32780/thumb/y00ts_%281%29.png?1699350631"
  },
  {
    "chainId": 1,
    "address": "0xad161afae7db2a048b042bd68e11c38c9c042cc1",
    "name": "FP  Nakamigos",
    "symbol": "UNKMGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32757/thumb/nakamigos.jpg?1699310532"
  },
  {
    "chainId": 1,
    "address": "0xb53b9e28b98c47e87acfd5a85eeb44a0940ecb12",
    "name": "Ordible",
    "symbol": "ORB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34524/thumb/IMG_5471.PNG?1705303671"
  },
  {
    "chainId": 1,
    "address": "0xf827f77422d6dff7b4e1bc87fec88606082b2199",
    "name": "MuskX",
    "symbol": "MUSKX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34650/thumb/output-onlinepngtools.png?1705593083"
  },
  {
    "chainId": 1,
    "address": "0x0e13914f4fe838e96346ce42f588193eb17d92b6",
    "name": "Lemon Terminal",
    "symbol": "LEMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33837/thumb/cmc.png?1703107858"
  },
  {
    "chainId": 1,
    "address": "0x0a6e481cb5f2d976b2edac4f67dcc824ccdae4af",
    "name": "Audify",
    "symbol": "AUDI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34251/thumb/emoji.png?1704339102"
  },
  {
    "chainId": 1,
    "address": "0xe46a1d19962ea120765d3139c588ffd617be04a8",
    "name": "EverETH",
    "symbol": "EETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34525/thumb/EETH_LOGO.png?1705303995"
  },
  {
    "chainId": 1,
    "address": "0x1bb9b64927e0c5e207c9db4093b3738eef5d8447",
    "name": "Vector Reserve",
    "symbol": "VEC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34876/thumb/vec200.png?1709644054"
  },
  {
    "chainId": 1,
    "address": "0xb7be27ba608641f0b6f152c8b432757f3685d2b0",
    "name": "FP  Meebits",
    "symbol": "U",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32769/thumb/meebits_logo.d8ecd01f_%281%29.png?1699337785"
  },
  {
    "chainId": 1,
    "address": "0x2dc917b722184ece1dcb9bb9dddfb1d63ccd25eb",
    "name": "White Rhinoceros",
    "symbol": "WHRH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33311/thumb/20231127_183017.png?1701412616"
  },
  {
    "chainId": 1,
    "address": "0x6a6b457becae7ef1cf9f22510d136c4aac0fc6d2",
    "name": "Molly AI",
    "symbol": "MOLLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33796/thumb/MOLLY.png?1702990017"
  },
  {
    "chainId": 1,
    "address": "0x328a268b191ef593b72498a9e8a481c086eb21be",
    "name": "MetaZero",
    "symbol": "MZERO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33759/thumb/1500x500-3.jpeg?1702960079"
  },
  {
    "chainId": 1,
    "address": "0xde7d85157d9714eadf595045cc12ca4a5f3e2adb",
    "name": "STP",
    "symbol": "STPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8713/thumb/STP.png?1696508875"
  },
  {
    "chainId": 1,
    "address": "0xfc82bb4ba86045af6f327323a46e80412b91b27d",
    "name": "Prom",
    "symbol": "PROM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8825/thumb/Ticker.png?1696508978"
  },
  {
    "chainId": 1,
    "address": "0x43fe2b0c5485c10e772a1843e32a7642ace5b88c",
    "name": "Red Pill",
    "symbol": "RPILL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33896/thumb/2023-12-21_07.20.32.jpg?1703233932"
  },
  {
    "chainId": 1,
    "address": "0xe94e8e29bd3db22917e83d019af7babdf8b9adf6",
    "name": "Dextensor",
    "symbol": "TAOS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36136/thumb/Dextensor.png?1710599523"
  },
  {
    "chainId": 1,
    "address": "0xd3843c6be03520f45871874375d618b3c7923019",
    "name": "Ordinal Bridge",
    "symbol": "ORDIBRIDGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33906/thumb/Ordibridge.jpeg?1703241141"
  },
  {
    "chainId": 1,
    "address": "0x0e6641e62baa87d77e01ab1c7e9d2f323f26942b",
    "name": "AssetLink",
    "symbol": "ASET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33907/thumb/Logo3D-NOBG.png?1703241821"
  },
  {
    "chainId": 1,
    "address": "0xf92d62ed69242d655e685c96b98f32f1409c3262",
    "name": "AlienForm",
    "symbol": "A4M",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34129/thumb/200x200_a4m_new.png?1712695102"
  },
  {
    "chainId": 1,
    "address": "0xb48fe88717c1a6020cc256a7d6f0e4a285bc9660",
    "name": "Scom Coin",
    "symbol": "SCOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33908/thumb/Scom_Coin.jpg?1703241874"
  },
  {
    "chainId": 1,
    "address": "0xf3ff80d631f7eec2c90180a3cddb3b95e87e2612",
    "name": "Coco",
    "symbol": "COCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33949/thumb/IMG_7817.jpeg?1703537940"
  },
  {
    "chainId": 1,
    "address": "0x97d0cfeb4fde54b430307c9482d6f79c761fe9b6",
    "name": "Epoch Island",
    "symbol": "EPOCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33684/thumb/EpochLogo_200x200.png?1702731332"
  },
  {
    "chainId": 1,
    "address": "0x7236a7ad67976ee07961ab26ed6f4cd23f7a9bd1",
    "name": "TXN Club",
    "symbol": "TXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33157/thumb/200x200.png?1700827374"
  },
  {
    "chainId": 1,
    "address": "0xa58a83361936471b50e2d1c7f99d422e6d73d397",
    "name": "X Rise",
    "symbol": "XRISE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33953/thumb/logo2x2.png?1703561044"
  },
  {
    "chainId": 1,
    "address": "0xc256f81d35a54c3599b424171d719e9ae87b2e9b",
    "name": "Zoopia",
    "symbol": "ZOOA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33955/thumb/twitter_D8EKrscj_400x400.jpg?1703561621"
  },
  {
    "chainId": 1,
    "address": "0x04815313e9329e8905a77251a1781cfa7934259a",
    "name": "wagmicatgirlkanye420etfmoon1000x",
    "symbol": "HOOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33843/thumb/Hood.png?1703121051"
  },
  {
    "chainId": 1,
    "address": "0x74588af8de14287e91d89758636d277d66f217b6",
    "name": "0xOS AI",
    "symbol": "0XOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34007/thumb/0xOS_AI.png?1703621360"
  },
  {
    "chainId": 1,
    "address": "0xaedf386b755465871ff874e3e37af5976e247064",
    "name": "Fasttoken",
    "symbol": "FTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28478/thumb/lightenicon_200x200.png?1696527472"
  },
  {
    "chainId": 1,
    "address": "0x4a029f7bcf33acb03547d8fa7be840347973e24e",
    "name": "Mazze",
    "symbol": "MAZZE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35548/thumb/logo200x200.png?1709094900"
  },
  {
    "chainId": 1,
    "address": "0xc28eb2250d1ae32c7e74cfb6d6b86afc9beb6509",
    "name": "OPEN Ticketing Ecosystem",
    "symbol": "OPN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36612/thumb/TOKEN.png?1711975511"
  },
  {
    "chainId": 1,
    "address": "0x8e1a7f380963945061c4460b0f9c6dc0bdafd94f",
    "name": "XMas Santa Rally",
    "symbol": "XMRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33777/thumb/IMG_6546.jpeg?1702970697"
  },
  {
    "chainId": 1,
    "address": "0xc9f00080d96cea3ef92d2e2e563d4cd41fb5bb36",
    "name": "BLOX",
    "symbol": "BLOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33794/thumb/BLOXLOGO200x200.png?1702989708"
  },
  {
    "chainId": 1,
    "address": "0x621879c6239d8ab9b82712fb56e7be880ce0c6ee",
    "name": "OmniBotX",
    "symbol": "OMNIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33588/thumb/Safeimagekit-resized-img_%2812%29.png?1702466793"
  },
  {
    "chainId": 1,
    "address": "0x30d0e4e6fb0330e45a13e1e06260837f27015de5",
    "name": "Year of the Dragon",
    "symbol": "YOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34053/thumb/Year_of_the_Dragon.png?1703746442"
  },
  {
    "chainId": 1,
    "address": "0xdc7da87115d4464e8aac99920a533d401bf72a50",
    "name": "ICE NET",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34056/thumb/ICE-logo-01.png?1703748227"
  },
  {
    "chainId": 1,
    "address": "0x44ff8620b8ca30902395a7bd3f2407e1a091bf73",
    "name": "Virtual Protocol",
    "symbol": "VIRTUAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34057/thumb/LOGOMARK.png?1708356054"
  },
  {
    "chainId": 1,
    "address": "0x0f0bbaf936f7ada2aca5b80bed7b655758d66950",
    "name": "WIFPEPEMOGINU",
    "symbol": "WIFPEPEMOG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35686/thumb/200x200.jpg?1709531939"
  },
  {
    "chainId": 1,
    "address": "0xc71b5f631354be6853efe9c3ab6b9590f8302e81",
    "name": "Polyhedra Network",
    "symbol": "ZK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36198/thumb/ZK.jpg?1710812518"
  },
  {
    "chainId": 1,
    "address": "0x89d584a1edb3a70b3b07963f9a3ea5399e38b136",
    "name": "AIT Protocol",
    "symbol": "AIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33962/thumb/ait-logo-200x200.png?1703567800"
  },
  {
    "chainId": 1,
    "address": "0x8b227d72570d3ead66014bca8305cbef7f90d1ee",
    "name": "LIZA",
    "symbol": "LIZA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32841/thumb/Liza.png?1699589053"
  },
  {
    "chainId": 1,
    "address": "0xcb0d82f4dfa503c9e3b8abc7a3caa01175b2da39",
    "name": "AurusX",
    "symbol": "AX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27871/thumb/AurusX_token_2D.jpg?1696526889"
  },
  {
    "chainId": 1,
    "address": "0xe084d3069a9b6d1b2d09d21ffad4a03d1b6496da",
    "name": "PUSHD",
    "symbol": "PUSHD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34309/thumb/Pushd.jpg?1704614504"
  },
  {
    "chainId": 1,
    "address": "0xbd6323a83b613f668687014e8a5852079494fb68",
    "name": "BlackrockTradingCurrency",
    "symbol": "BTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34656/thumb/imresizer-%282%29.png?1705632165"
  },
  {
    "chainId": 1,
    "address": "0x2ea6cc1ac06fdc01b568fcad8d842dec3f2ce1ad",
    "name": "Blob Protocol",
    "symbol": "BLOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31749/thumb/IMG_C7C8AFB6267E-1.jpeg?1696530568"
  },
  {
    "chainId": 1,
    "address": "0x00c2999c8b2adf4abc835cc63209533973718eb1",
    "name": "New World Order",
    "symbol": "STATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27387/thumb/icon.png?1696526428"
  },
  {
    "chainId": 1,
    "address": "0x7567d006f6be77e3d87aa831855cb4102e37b17d",
    "name": "Three Hundred AI",
    "symbol": "THND",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x72f713d11480dcf08b37e1898670e736688d218d",
    "name": "Nettensor",
    "symbol": "NAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35630/thumb/logoclip_%28200_x_200_piksel%29.png?1709293129"
  },
  {
    "chainId": 1,
    "address": "0x4b1e80cac91e2216eeb63e29b957eb91ae9c2be8",
    "name": "Jupiter Project",
    "symbol": "JUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10351/thumb/logo512.png?1696510352"
  },
  {
    "chainId": 1,
    "address": "0xb9f69c75a3b67425474f8bcab9a3626d8b8249e1",
    "name": "Memefi Toybox 404",
    "symbol": "TOYBOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35147/thumb/200x200_MemeFI_logo.png?1707552368"
  },
  {
    "chainId": 1,
    "address": "0xb84f00b5ab187bc795d61389ac2418d4e7c77bb0",
    "name": "Balance Network Finance",
    "symbol": "BALANCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34530/thumb/20240113_025010.png?1705307118"
  },
  {
    "chainId": 1,
    "address": "0xed9f6aa6532869576211fd6367e3c328810fbeb3",
    "name": "GPTPlus",
    "symbol": "GPTPLUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34194/thumb/gptplus-icon.png?1704272397"
  },
  {
    "chainId": 1,
    "address": "0xcf60870dcedcad18d987af20687ca4717cae0533",
    "name": "Degen Fighting Championship",
    "symbol": "DFC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34211/thumb/DFC-200x200.png?1704276680"
  },
  {
    "chainId": 1,
    "address": "0x3e156234501c41a041a178ff547e295609c4769a",
    "name": "TeleCard",
    "symbol": "TCARD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36073/thumb/IMG_1889_%281%29.png?1710412075"
  },
  {
    "chainId": 1,
    "address": "0xa0cc4428fbb652c396f28dce8868b8743742a71c",
    "name": "Purple AI",
    "symbol": "PAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34193/thumb/Purple_AI.png?1704272332"
  },
  {
    "chainId": 1,
    "address": "0xd05d90a656fc375ac1478689d7bcd31098f2dd1f",
    "name": "ChainFactory",
    "symbol": "FACTORY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34204/thumb/CFicon_200x200.png?1704275410"
  },
  {
    "chainId": 1,
    "address": "0xf2dfdbe1ea71bbdcb5a4662a16dbf5e487be3ebe",
    "name": "DeCloud",
    "symbol": "CLOUD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35135/thumb/Nl1XLwPV_400x400.jpg?1707445040"
  },
  {
    "chainId": 1,
    "address": "0x04fa0d235c4abf4bcf4787af4cf447de572ef828",
    "name": "UMA",
    "symbol": "UMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10951/thumb/UMA.png?1696510900"
  },
  {
    "chainId": 1,
    "address": "0xc3d2b3e23855001508e460a6dbe9f9e3116201af",
    "name": "GATEWAY TO MARS",
    "symbol": "MARS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34083/thumb/MARS-LOGO-BLACK-200x200.png?1703910355"
  },
  {
    "chainId": 1,
    "address": "0x0aa8a7d1fb4c64b3b1dcea9a7ade81c59c25b95b",
    "name": "AstraAI",
    "symbol": "ASTRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33138/thumb/CG_200x200.png?1700810557"
  },
  {
    "chainId": 1,
    "address": "0x2a8d8fcb18d132d77373eb02b22d8e7d378f4437",
    "name": "TeraHertz Capital",
    "symbol": "THZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34283/thumb/CmDc-Q7W_400x400_%281%29.jpg?1704453174"
  },
  {
    "chainId": 1,
    "address": "0x56d811088235f11c8920698a204a5010a788f4b3",
    "name": "bZx Protocol",
    "symbol": "BZRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11811/thumb/bzrx.png?1696511684"
  },
  {
    "chainId": 1,
    "address": "0x9c2b4b0da5ebd20c29ef20758064554a55a88b68",
    "name": "ByteAI",
    "symbol": "BYTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36432/thumb/photo_2024-03-20_20.40.45.jpeg?1711435137"
  },
  {
    "chainId": 1,
    "address": "0x046eee2cc3188071c02bfc1745a6b17c656e3f3d",
    "name": "Rollbit Coin",
    "symbol": "RLB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24552/thumb/unziL6wO_400x400.jpg?1696523729"
  },
  {
    "chainId": 1,
    "address": "0xd7c9f0e536dc865ae858b0c0453fe76d13c3beac",
    "name": "XAI Stablecoin",
    "symbol": "XAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28254/thumb/Xai_logo.png?1696527256"
  },
  {
    "chainId": 1,
    "address": "0xe469699f617bfd0fbffcd575970d34c2cecffa9f",
    "name": "VuzzMind",
    "symbol": "VUZZ",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34294/thumb/tokenlogogecko.png?1704606291"
  },
  {
    "chainId": 1,
    "address": "0xcd24ba0e3364233ee9301c1d608a14753c8739c5",
    "name": "DeepSouth AI",
    "symbol": "SOUTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34348/thumb/CGpng.png?1704701256"
  },
  {
    "chainId": 1,
    "address": "0x0808e6c4400bde1d70db0d02170b67de05e07ef5",
    "name": "Wrapped LYX  SigmaSwap ",
    "symbol": "WLYX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34338/thumb/wLYX.png?1704679928"
  },
  {
    "chainId": 1,
    "address": "0xc5170dd7386247cdb8c48545c803f5d0e3347022",
    "name": "Titanium22",
    "symbol": "TI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31197/thumb/200PX.png?1704048243"
  },
  {
    "chainId": 1,
    "address": "0x43dfc4159d86f3a37a5a4b3d4580b888ad7d4ddd",
    "name": "DODO",
    "symbol": "DODO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12651/thumb/dodo_logo.png?1696512458"
  },
  {
    "chainId": 1,
    "address": "0xe4b4c008ff36e3c50c4299c223504a480de9c833",
    "name": "Secret Society",
    "symbol": "SS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36865/thumb/622BD442-4493-4AF9-8DF9-CF464B7AFC44.png?1712644574"
  },
  {
    "chainId": 1,
    "address": "0xcffcfdada28ab44d5440301470dcd410e75c4765",
    "name": "Parifi",
    "symbol": "PRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34710/thumb/parifi200x200.png?1705894380"
  },
  {
    "chainId": 1,
    "address": "0xeac32927cf2785cbe0d515298cab448538a8cdd7",
    "name": "StudioAi",
    "symbol": "SAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34468/thumb/200x200.png?1704968139"
  },
  {
    "chainId": 1,
    "address": "0x26a37f5a0200ad622f6db7cd687c78b7049ee841",
    "name": "WatchTowers AI",
    "symbol": "WTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34469/thumb/wts.jpeg?1704968199"
  },
  {
    "chainId": 1,
    "address": "0x078c4adf3fee52eb77f6018d9805dfc69e911d39",
    "name": "Eternity GLORY Token",
    "symbol": "GLORY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34903/thumb/glory-200x200.png?1706594822"
  },
  {
    "chainId": 1,
    "address": "0xab14f1262ea8d910756828cbe08e6cedfb66360d",
    "name": "TORO",
    "symbol": "TORO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34442/thumb/T_coin.png?1704874974"
  },
  {
    "chainId": 1,
    "address": "0x3235b13708f178af6f110de7177ed5de10c1093d",
    "name": "Mongol NFT",
    "symbol": "MNFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23718/thumb/swI93LWg_400x400.jpg?1696522918"
  },
  {
    "chainId": 1,
    "address": "0x6733f0283711f225a447e759d859a70b0c0fd2bc",
    "name": "Vector Reserve Staked vETH",
    "symbol": "SVETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35289/thumb/svETH200.png?1709593889"
  },
  {
    "chainId": 1,
    "address": "0x3ecab35b64345bfc472477a653e4a3abe70532d9",
    "name": "EnterButton",
    "symbol": "ENTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19677/thumb/ENTC_symbol.png?1696519104"
  },
  {
    "chainId": 1,
    "address": "0x6c10d1611a5a95cb967e4bcab5791fd101194949",
    "name": "X Project ERC",
    "symbol": "XERS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35009/thumb/xofficiallogobluesky.jpg?1707101426"
  },
  {
    "chainId": 1,
    "address": "0xe80c0cd204d654cebe8dd64a4857cab6be8345a3",
    "name": "JPEG d  OLD ",
    "symbol": "JPEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24025/thumb/et_43CNi_400x400.jpg?1696523217"
  },
  {
    "chainId": 1,
    "address": "0x2f3d0d2317802a65faac6e4cd94067c37b4d4804",
    "name": "DECENTRACARD",
    "symbol": "DCARD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34533/thumb/DC_Logo_5_200x200.png?1705323804"
  },
  {
    "chainId": 1,
    "address": "0x20bc71ddf242b88502d9819e02d45ad0d4a586c8",
    "name": "Gravitron",
    "symbol": "GTRON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33622/thumb/photo_2023-12-13_08-36-33.jpg?1702529102"
  },
  {
    "chainId": 1,
    "address": "0x33f289d91286535c47270c8479f6776fb3adeb3e",
    "name": "BoxBet",
    "symbol": "BXBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32495/thumb/logo.png?1706725462"
  },
  {
    "chainId": 1,
    "address": "0x96f6ef951840721adbf46ac996b59e0235cb985c",
    "name": "Ondo US Dollar Yield",
    "symbol": "USDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31700/thumb/usdy_%281%29.png?1696530524"
  },
  {
    "chainId": 1,
    "address": "0xbded8a4dc74a940eab68703167db89b1712b68ea",
    "name": "KASPAMINING",
    "symbol": "KMN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34451/thumb/logo_200x200.png?1704901486"
  },
  {
    "chainId": 1,
    "address": "0x5c1ac4e697a2814812a284509e1f1b8aa34a25f3",
    "name": "Avante",
    "symbol": "AXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33667/thumb/logo.png.png?1702693688"
  },
  {
    "chainId": 1,
    "address": "0x75858677e27c930fb622759feaffee2b754af07f",
    "name": "Phantasma",
    "symbol": "SOUL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4130/thumb/phantasma.png?1696504758"
  },
  {
    "chainId": 1,
    "address": "0x2a85556a6701a02e75bc4de8ec340c6de1b29f72",
    "name": "TOKA",
    "symbol": "TOKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34418/thumb/photo_2023-12-22_23.51.07_%281%29_2.jpeg?1704821179"
  },
  {
    "chainId": 1,
    "address": "0x3632dc4d741bfa40dbf3b23b63dd3a25a3061de3",
    "name": "0xEngage",
    "symbol": "ENGAGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34420/thumb/200x200.png?1704821589"
  },
  {
    "chainId": 1,
    "address": "0x8ab2ff0116a279a99950c66a12298962d152b83c",
    "name": "Ordiswap",
    "symbol": "ORDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33910/thumb/IMG-20240113-WA0018.jpg.jpg?1705169306"
  },
  {
    "chainId": 1,
    "address": "0xce722f60f35c37ab295adc4e6ba45bcc7ca89dd6",
    "name": "JPEG d",
    "symbol": "JPGD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35303/thumb/jpgd.jpeg?1708095973"
  },
  {
    "chainId": 1,
    "address": "0xae54e48c2263e889399ad29bfff401feaf08fd57",
    "name": "DiversityEquity Inclusion",
    "symbol": "DEI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34535/thumb/Wei_Icon.png?1705324644"
  },
  {
    "chainId": 1,
    "address": "0x15f73a3ab443ee6ebf36c605c7868159ce5d028c",
    "name": "SmartsetToken",
    "symbol": "SST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34385/thumb/IMG_6239.jpeg?1704779818"
  },
  {
    "chainId": 1,
    "address": "0xfc5e4ed56153b57aa8ef769eba3e79e58e19be93",
    "name": "SOLAV",
    "symbol": "SOLAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34483/thumb/White_Background.png?1705042967"
  },
  {
    "chainId": 1,
    "address": "0x0af55d5ff28a3269d69b98680fd034f115dd53ac",
    "name": "BankSocial",
    "symbol": "BSL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/15738/thumb/banksocial_small.png?1696515363"
  },
  {
    "chainId": 1,
    "address": "0x4e241a9ec66832a16bceaeb9156e524487f061d7",
    "name": "ETF Rocks",
    "symbol": "ETF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34633/thumb/logo-etf-color-200x200.png?1705548981"
  },
  {
    "chainId": 1,
    "address": "0x3845badade8e6dff049820680d1f14bd3903a5d0",
    "name": "The Sandbox",
    "symbol": "SAND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12129/thumb/sandbox_logo.jpg?1696511971"
  },
  {
    "chainId": 1,
    "address": "0xd836d22531d810f192ba6bd0ba3c28c35d4606c2",
    "name": "BONKBEST",
    "symbol": "BONKBEST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34421/thumb/IMG_20240108_204110_824.jpg?1704821712"
  },
  {
    "chainId": 1,
    "address": "0x44c7272b212e033d71b694733c150ff8526c1a0e",
    "name": "Crimson",
    "symbol": "CRM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34383/thumb/Crimson.png?1704779607"
  },
  {
    "chainId": 1,
    "address": "0x3b0fccbd5dae0570a70f1fb6d8d666a33c89d71e",
    "name": "FloorDAO",
    "symbol": "FLOOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35217/thumb/1_MH1_7DcG64sDKJMnz_SFNg.png?1707907538"
  },
  {
    "chainId": 1,
    "address": "0x186ef81fd8e77eec8bffc3039e7ec41d5fc0b457",
    "name": "Inspect",
    "symbol": "INSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33422/thumb/200_%281%29.png?1701767509"
  },
  {
    "chainId": 1,
    "address": "0x12ef10a4fc6e1ea44b4ca9508760ff51c647bb71",
    "name": "Restake Finance",
    "symbol": "RSTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33885/thumb/rstk.png?1703148117"
  },
  {
    "chainId": 1,
    "address": "0x8b9b95292f890df47fff5ac9cbe93d5fc242bd51",
    "name": "BeFi Labs",
    "symbol": "BEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34384/thumb/Logo1_Brave.png?1704779725"
  },
  {
    "chainId": 1,
    "address": "0x8b3870df408ff4d7c3a26df852d41034eda11d81",
    "name": "IOI Token",
    "symbol": "IOI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/15952/thumb/IOI_new_logo.png?1696515564"
  },
  {
    "chainId": 1,
    "address": "0x550775e17ed6767621a1aed580e6eb29ede981e9",
    "name": "Agnus AI",
    "symbol": "AGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34387/thumb/photo_2024-01-08_14.11.37.jpeg?1704780575"
  },
  {
    "chainId": 1,
    "address": "0x92098551e613dfdcd4d7c7b2c35615709e4e0397",
    "name": "ArtemisAI",
    "symbol": "ATAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34452/thumb/black_logo_big.png?1704931121"
  },
  {
    "chainId": 1,
    "address": "0x45b3cf56896c4547426a4145ad1d0ae971120214",
    "name": "404Blocks",
    "symbol": "404BLOCKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35219/thumb/404.png?1707918003"
  },
  {
    "chainId": 1,
    "address": "0x25b4f5d4c314bcd5d7962734936c957b947cb7cf",
    "name": "TG Casino",
    "symbol": "TGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34473/thumb/TGC_Logo_200x200.png?1704998035"
  },
  {
    "chainId": 1,
    "address": "0xedb73d4ed90be7a49d06d0d940055e6d181d22fa",
    "name": "Blueprint oBLUE",
    "symbol": "OBLUE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35539/thumb/photo_2024-02-17_18-41-11.jpg?1709089813"
  },
  {
    "chainId": 1,
    "address": "0xcd8c163cc0ec3a53ce6bec89e665fb97cce51c42",
    "name": "beecasinogames",
    "symbol": "BEECASINO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36435/thumb/beecasino-profile.jpg?1711438474"
  },
  {
    "chainId": 1,
    "address": "0x55ff0f50f639e7acfe06694e6d018bd7678e6da9",
    "name": "Mineshield",
    "symbol": "MNS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36625/thumb/MINESHIELD_200x200.png?1711982251"
  },
  {
    "chainId": 1,
    "address": "0xcefde37817da4fc51ddc24e3820ad316784ee04b",
    "name": "Sonata Network",
    "symbol": "SONA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34486/thumb/IMG_20240111_220818_164.png?1705044900"
  },
  {
    "chainId": 1,
    "address": "0x3a429a151ad985e678a834f9db057163181f58e8",
    "name": "ENCRYPT",
    "symbol": "ENCR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34271/thumb/IMG_20240105_001606_962.png?1704391325"
  },
  {
    "chainId": 1,
    "address": "0x03dde9e5bb31ee40a471476e2fccf75c67921062",
    "name": "EML Protocol",
    "symbol": "EML",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30950/thumb/EML_LOGO.png?1696529788"
  },
  {
    "chainId": 1,
    "address": "0x51d29b072ff42c712e4185723a5a7974920995ed",
    "name": "ChartHub",
    "symbol": "CHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34456/thumb/photo_2024-01-08_00-43-08_%282%29.jpg?1704933058"
  },
  {
    "chainId": 1,
    "address": "0x102fa93ef3d2dc5b0b9ed3e7b3bf832796cb7b95",
    "name": "Boss Blockchain",
    "symbol": "BBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34400/thumb/Safeimagekit-resized-img_%2820%29.png?1704788308"
  },
  {
    "chainId": 1,
    "address": "0xcafe001067cdef266afb7eb5a286dcfd277f3de5",
    "name": "ParaSwap",
    "symbol": "PSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20403/thumb/ep7GqM19_400x400.jpg?1696519812"
  },
  {
    "chainId": 1,
    "address": "0xa562912e1328eea987e04c2650efb5703757850c",
    "name": "Drops",
    "symbol": "DROPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35795/thumb/IMG_1170_%282%29.jpeg?1709801958"
  },
  {
    "chainId": 1,
    "address": "0xf073bac22dab7faf4a3dd6c6189a70d54110525c",
    "name": "GenesisLRT Restaked ETH",
    "symbol": "GENETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34127/thumb/image.png?1704174967"
  },
  {
    "chainId": 1,
    "address": "0xfae103dc9cf190ed75350761e95403b7b8afa6c0",
    "name": "Restaked Swell ETH",
    "symbol": "RSWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34489/thumb/rswETH_Icon.png?1706865484"
  },
  {
    "chainId": 1,
    "address": "0x76952fc1087fdae2e6aa4e5a1cec9e1117a60b00",
    "name": "NY Blockchain",
    "symbol": "NYBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34462/thumb/NYBC-200X200_%E7%94%BB%E6%9D%BF_1_%E5%89%AF%E6%9C%AC_29.jpg?1704961756"
  },
  {
    "chainId": 1,
    "address": "0x943af2ece93118b973c95c2f698ee9d15002e604",
    "name": "GameGPT",
    "symbol": "DUEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34491/thumb/duel.jpeg?1705056819"
  },
  {
    "chainId": 1,
    "address": "0x574d22e2555cac0ce71e44778f6de2e7487ae229",
    "name": "SoonVerse",
    "symbol": "SOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29448/thumb/1.png?1696528395"
  },
  {
    "chainId": 1,
    "address": "0xbbbbbbbb46a1da0f0c3f64522c275baa4c332636",
    "name": "ZKBase",
    "symbol": "ZKB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13585/thumb/image_2024-01-16_172847810.png?1705397359"
  },
  {
    "chainId": 1,
    "address": "0x72fc1c1c926bd26712f62e7485392cd405478f05",
    "name": "Bark Gas Token",
    "symbol": "BARK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34562/thumb/Bark_Gas_Token.png?1705400669"
  },
  {
    "chainId": 1,
    "address": "0x8248270620aa532e4d64316017be5e873e37cc09",
    "name": "DevvE",
    "symbol": "DEVVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35136/thumb/DevvE.png?1707445824"
  },
  {
    "chainId": 1,
    "address": "0x91dfbee3965baaee32784c2d546b7a0c62f268c9",
    "name": "Forj",
    "symbol": "BONDLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13322/thumb/FORJ_twitter_twitter-linked_in_profile_%281%29.png?1696513091"
  },
  {
    "chainId": 1,
    "address": "0x9bf3be1e445d5d386f891a1eccc3245e5a3670a4",
    "name": "FACET",
    "symbol": "FACET",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34825/thumb/FACET.jpg?1706191458"
  },
  {
    "chainId": 1,
    "address": "0xf2fdd9c25d7bc8002ce89716d1be484b2d976944",
    "name": "  Payments",
    "symbol": "XPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34745/thumb/xpay_%281%29.jpg?1705949795"
  },
  {
    "chainId": 1,
    "address": "0xc0b6addde76be6a09231edc12a5e3794c11f594f",
    "name": "FlexGPU",
    "symbol": "FGPU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35796/thumb/flexGPU.png?1709802187"
  },
  {
    "chainId": 1,
    "address": "0x7c5b267ed81009aa7374b5ca7e5137da47045ba8",
    "name": "TAIKAI",
    "symbol": "TKAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34827/thumb/TKAI.jpg?1706191900"
  },
  {
    "chainId": 1,
    "address": "0xb551b43af192965f74e3dfaa476c890b403cad95",
    "name": "DataBot",
    "symbol": "DATA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31508/thumb/Databot_LOGO_coingecko.png?1696530318"
  },
  {
    "chainId": 1,
    "address": "0x8d9d725aaa3f6236763ff548051657a342c37623",
    "name": "NED",
    "symbol": "NED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34704/thumb/NED_logo_%281%29.png?1705891856"
  },
  {
    "chainId": 1,
    "address": "0xecd48f326e70388d993694de59b4542ce8af7649",
    "name": "Froggy Friends",
    "symbol": "TADPOLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35138/thumb/tadpole.jpeg?1707447320"
  },
  {
    "chainId": 1,
    "address": "0x6c061d18d2b5bbfbe8a8d1eeb9ee27efd544cc5d",
    "name": "Monarch",
    "symbol": "MNRCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35139/thumb/mnrc.jpeg?1707449200"
  },
  {
    "chainId": 1,
    "address": "0xce246eea10988c495b4a90a905ee9237a0f91543",
    "name": "VaultCraft",
    "symbol": "VCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33237/thumb/VCX_Token_200x200.png?1701147660"
  },
  {
    "chainId": 1,
    "address": "0xb369daca21ee035312176eb8cf9d88ce97e0aa95",
    "name": "Skol",
    "symbol": "SKOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35934/thumb/skollogo_200x200.png?1710264034"
  },
  {
    "chainId": 1,
    "address": "0x218de5e6324c5351c3a2bf0c40d76f585b8de04d",
    "name": "Stake Together",
    "symbol": "STPETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34709/thumb/stake-together.png?1705894276"
  },
  {
    "chainId": 1,
    "address": "0x949d48eca67b17269629c7194f4b727d4ef9e5d6",
    "name": "Merit Circle",
    "symbol": "MC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19304/thumb/Db4XqML.png?1696518747"
  },
  {
    "chainId": 1,
    "address": "0xe2f668f6166eda65620a01c4c116a5f6e5528614",
    "name": "Me Gusta",
    "symbol": "GUSTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30348/thumb/NoMeGusta.jpg?1696529248"
  },
  {
    "chainId": 1,
    "address": "0x525a8f6f3ba4752868cde25164382bfbae3990e1",
    "name": "Nym",
    "symbol": "NYM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/24488/thumb/NYM_Token.png?1696523668"
  },
  {
    "chainId": 1,
    "address": "0xbc391e78b0ea0d1db04890732742494e7fbfc118",
    "name": "NIRVANA",
    "symbol": "VANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32469/thumb/vana.jpg?1698259899"
  },
  {
    "chainId": 1,
    "address": "0xa067237f8016d5e3770cf08b20e343ab9ee813d5",
    "name": "Greelance",
    "symbol": "GRL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31387/thumb/G-logo.jpg?1696530204"
  },
  {
    "chainId": 1,
    "address": "0x9511ff502f982643935a39206f98a6c18d4527cf",
    "name": "BlockDefend AI",
    "symbol": "DEFEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36438/thumb/BlockDefend_AI.png?1711440094"
  },
  {
    "chainId": 1,
    "address": "0x6553565eac5daa9bfc5e2892b36291634c9b2ad6",
    "name": "Rake com",
    "symbol": "RAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33791/thumb/rake_200x200.png?1702988861"
  },
  {
    "chainId": 1,
    "address": "0xfe98796e0af4ababd716508429e51ff9ac1bb4d5",
    "name": "Transactra Finance",
    "symbol": "TRSCT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35636/thumb/logo.png?1709294261"
  },
  {
    "chainId": 1,
    "address": "0x069d89974f4edabde69450f9cf5cf7d8cbd2568d",
    "name": "BVM",
    "symbol": "BVM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35938/thumb/twitter-pfp.png?1710232498"
  },
  {
    "chainId": 1,
    "address": "0x5e21d1ee5cf0077b314c381720273ae82378d613",
    "name": "The Infinite Garden",
    "symbol": "ETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34540/thumb/film.jpg?1705338132"
  },
  {
    "chainId": 1,
    "address": "0xc01154b4ccb518232d6bbfc9b9e6c5068b766f82",
    "name": "NEXUS",
    "symbol": "NEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35797/thumb/LLm7P2IL_400x400.jpg?1709802342"
  },
  {
    "chainId": 1,
    "address": "0xde5d2530a877871f6f0fc240b9fce117246dadae",
    "name": "Juice",
    "symbol": "JUICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35942/thumb/juiceCG.png?1710233232"
  },
  {
    "chainId": 1,
    "address": "0x750c3a0a0ce9984eeb8c5d146dff024b584e5e33",
    "name": "zkHive",
    "symbol": "ZKHIVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35940/thumb/IMG_4449.jpeg?1710233140"
  },
  {
    "chainId": 1,
    "address": "0xf96287ea71abb5c8c58961c7a3cd3982a505e264",
    "name": "Stem AI",
    "symbol": "STEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34660/thumb/Stem_AI.png?1705636636"
  },
  {
    "chainId": 1,
    "address": "0x0f5c78f152152dda52a2ea45b0a8c10733010748",
    "name": "XOX Labs",
    "symbol": "XOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34611/thumb/0164CF6D-8683-418C-9ACB-C6D63B9E1CCB.jpeg?1705499880"
  },
  {
    "chainId": 1,
    "address": "0xe2976a66e8cef3932cdaeb935e114dcd5ce20f20",
    "name": "ChessFish",
    "symbol": "CFSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34623/thumb/TokenLogo.png?1705504570"
  },
  {
    "chainId": 1,
    "address": "0x15e6e0d4ebeac120f9a97e71faa6a0235b85ed12",
    "name": "SatoshiVM",
    "symbol": "SAVM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34659/thumb/SatoshiVM.jpeg?1705635248"
  },
  {
    "chainId": 1,
    "address": "0x8ed2fc62d6850eaadcb717465752dab591286839",
    "name": "Elyssa",
    "symbol": "ELY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31187/thumb/IMG_20230720_174915_364.jpg?1696530014"
  },
  {
    "chainId": 1,
    "address": "0x619e398858a3110df4d89056a15a40338a01e65f",
    "name": "Garbage",
    "symbol": "GARBAGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33624/thumb/CircleLogo_200x200_EH_%281%29.png?1702532702"
  },
  {
    "chainId": 1,
    "address": "0x179f782d7fbe745f40b20e0c7dbb6205d43fa4b9",
    "name": "Inspire AI",
    "symbol": "INSP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35540/thumb/VJwHPSjF_200x200.jpg?1709092810"
  },
  {
    "chainId": 1,
    "address": "0xaa7a9ca87d3694b5755f213b5d04094b8d0f0a6f",
    "name": "OriginTrail",
    "symbol": "TRAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1877/thumb/TRAC.jpg?1696502873"
  },
  {
    "chainId": 1,
    "address": "0x4447743d42680f7fbb56e87b3617463113ca649e",
    "name": "MeshWave",
    "symbol": "MWAVE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36696/thumb/ava-1_%281%29.png?1712117270"
  },
  {
    "chainId": 1,
    "address": "0xa711bcc2b6f5c4fc3dfaccc2a01148765cbbab1c",
    "name": "Grok1 5",
    "symbol": "GROK15",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34838/thumb/IMG_20240123_045540_147.jpg?1706240605"
  },
  {
    "chainId": 1,
    "address": "0xbf1aa3a0e1294b3b996533f5d648dec2e59bad05",
    "name": "Funarcade",
    "symbol": "FAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34767/thumb/FAT_logo_200x200.png?1705992218"
  },
  {
    "chainId": 1,
    "address": "0x2960d71855a521c8414d29a27218efdb67c34180",
    "name": "Leaxcoin",
    "symbol": "LEAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6918/thumb/X_C0tYzl_400x400.jpg?1696507236"
  },
  {
    "chainId": 1,
    "address": "0x1650978997e5898db8eeffdb7c530bb223b71c79",
    "name": "Flashpad Token",
    "symbol": "FLASH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34835/thumb/200.jpg?1706230764"
  },
  {
    "chainId": 1,
    "address": "0xdefb0b264032e4e128b00d02b3fd0aa00331237b",
    "name": "Buddha",
    "symbol": "BUDDHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34079/thumb/logo_CMC.png?1706009621"
  },
  {
    "chainId": 1,
    "address": "0x5d56b6581d2e7e7574adce2dc593f499a53d7505",
    "name": "Church of the Machina",
    "symbol": "MACHINA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34720/thumb/machina_logo.png?1705916393"
  },
  {
    "chainId": 1,
    "address": "0x96a5399d07896f757bd4c6ef56461f58db951862",
    "name": "DragonX win",
    "symbol": "DRAGONX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34724/thumb/DragonX_%281%29.png?1705918646"
  },
  {
    "chainId": 1,
    "address": "0x661013bb8d1c95d86d9c85f76e9004561f1bb36f",
    "name": "DeFi Robot",
    "symbol": "DRBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34725/thumb/defiRobot-logo.jpg?1705918727"
  },
  {
    "chainId": 1,
    "address": "0x435998003ccb7abeaa392494c89f7799fe241db5",
    "name": "Interns",
    "symbol": "INTERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34727/thumb/IMG_1926.jpeg?1705919202"
  },
  {
    "chainId": 1,
    "address": "0x61b57bdc01e3072fab3e9e2f3c7b88d482734e05",
    "name": "MetaZooMee",
    "symbol": "MZM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31708/thumb/MZM-200px_%281%29.png?1696530531"
  },
  {
    "chainId": 1,
    "address": "0x148255a3b10666d9788ec48bc61ea3e48974bf2c",
    "name": "Decentralized Music Chain",
    "symbol": "DMCC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34760/thumb/feed.jpeg?1705985058"
  },
  {
    "chainId": 1,
    "address": "0xd57187e56e5b31b4d7813d7ceea1e9f9b97ee82f",
    "name": "zkInfra",
    "symbol": "ZKIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36517/thumb/IMG_20240327_124751_784.jpg?1711694787"
  },
  {
    "chainId": 1,
    "address": "0x8ccd897ca6160ed76755383b201c1948394328c7",
    "name": "Balance AI",
    "symbol": "BAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34732/thumb/Balance_AI.png?1705921540"
  },
  {
    "chainId": 1,
    "address": "0xebdb54e76bfec9ab4e06ccf6e484e4126f81befc",
    "name": "Soul Society",
    "symbol": "HON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34781/thumb/symbol-soulsociety-new.png?1706015872"
  },
  {
    "chainId": 1,
    "address": "0x6c5ba91642f10282b576d91922ae6448c9d52f4e",
    "name": "Phala",
    "symbol": "PHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12451/thumb/phala.png?1696512270"
  },
  {
    "chainId": 1,
    "address": "0xf4cccfda0781ae019a9d4e1853dcd3e288daaa89",
    "name": "Da Pinchi",
    "symbol": "PINCHI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34739/thumb/CGTlogo.png?1705946139"
  },
  {
    "chainId": 1,
    "address": "0xa1f3aca66403d29b909605040c30ae1f1245d14c",
    "name": "Adapt3r Digital Treasury Bill Fund",
    "symbol": "TFBILL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34761/thumb/s3hkVoBT_400x400.png?1705986984"
  },
  {
    "chainId": 1,
    "address": "0xd8f1460044925d2d5c723c7054cd9247027415b7",
    "name": "Clipper SAIL",
    "symbol": "SAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31380/thumb/SAIL-Logo.png?1696530197"
  },
  {
    "chainId": 1,
    "address": "0x80d69e79258fe9d056c822461c4eb0b4ca8802e2",
    "name": "Inception rETH",
    "symbol": "INRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34956/thumb/rETH.png?1706773577"
  },
  {
    "chainId": 1,
    "address": "0x7ae4f8885f6cfa41a692cb9da3789cfa6a83e9f2",
    "name": "GenomesDAO GENOME",
    "symbol": "GENOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36763/thumb/images.jpg?1712282056"
  },
  {
    "chainId": 1,
    "address": "0x1c2884c71629c7d4e378ec95d03bfaf9f6fa5afe",
    "name": "Bundl Tools",
    "symbol": "BUNDL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34748/thumb/Bundl_Tools.png?1705953621"
  },
  {
    "chainId": 1,
    "address": "0x0b6f3ea2814f3fff804ba5d5c237aebbc364fba9",
    "name": "Unagi Token",
    "symbol": "UNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34733/thumb/unagi.png?1705922276"
  },
  {
    "chainId": 1,
    "address": "0xdc63269ea166b70d4780b3a11f5c825c2b761b01",
    "name": "PAWSWAP",
    "symbol": "PAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28946/thumb/PawLogo.png?1699394612"
  },
  {
    "chainId": 1,
    "address": "0xc3cc3076cb304494775b3193ef1aa080ba6bf962",
    "name": "OrdiGen",
    "symbol": "ODGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34881/thumb/ODGN.jpg?1706499113"
  },
  {
    "chainId": 1,
    "address": "0x50b806c5fe274c07e46b96be8c68d2fd2d9597b4",
    "name": "TUCKER CARLSON",
    "symbol": "TUCKER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34804/thumb/tucker_logo_transparent.png?1706094592"
  },
  {
    "chainId": 1,
    "address": "0x5f48d1fd6814cd1cd38aeb895755e57d519196d1",
    "name": "Trophy",
    "symbol": "WINS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31007/thumb/icon_wins.png?1696529844"
  },
  {
    "chainId": 1,
    "address": "0x7340ea46360576dc46ef49bce99bc5072c32421d",
    "name": "DollarSqueeze",
    "symbol": "DSQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30764/thumb/DS.png?1696529632"
  },
  {
    "chainId": 1,
    "address": "0xd4318fa09c45cfb6355ded6085b0d698b64ec1cd",
    "name": "Federal AI",
    "symbol": "FEDAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/34912/thumb/fed-ai-logo-200px.png?1706604536"
  },
  {
    "chainId": 1,
    "address": "0xb19189fb36c816f3e0f16065057b07b790998fdc",
    "name": "Serum SER",
    "symbol": "SER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27757/thumb/Untitled_%28200_x_200_px%29.png?1699041628"
  },
  {
    "chainId": 1,
    "address": "0xb9f599ce614feb2e1bbe58f180f370d05b39344e",
    "name": "PepeFork",
    "symbol": "PORK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34913/thumb/pork.png?1706606047"
  },
  {
    "chainId": 1,
    "address": "0x576e2bed8f7b46d34016198911cdf9886f78bea7",
    "name": "MAGA",
    "symbol": "TRUMP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31498/thumb/Maga-Trump-Logo-200px.png?1696530309"
  },
  {
    "chainId": 1,
    "address": "0x891de5f139791ddf9dbabf519cfe2a049f8fc6d3",
    "name": "Dibbles",
    "symbol": "DIBBLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34860/thumb/dibbles_logo.png?1706351813"
  },
  {
    "chainId": 1,
    "address": "0x3a97e00b48d56bd5e0502e1a2a8c036a040e1b99",
    "name": "JACKBOT",
    "symbol": "JBOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34177/thumb/JBOT.jpg?1704219980"
  },
  {
    "chainId": 1,
    "address": "0xc4727c419acfb49c8660736506ca023a4e8509e0",
    "name": "NodifiAI",
    "symbol": "NODIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36795/thumb/dexcreener_%281%29.png?1712434294"
  },
  {
    "chainId": 1,
    "address": "0xa74a05b17d72e9b0781d973e7963dfaacd266b94",
    "name": "Oasis",
    "symbol": "OASIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35546/thumb/IMG_20240224_152139_778.png?1709094500"
  },
  {
    "chainId": 1,
    "address": "0xec70ff4a5b09110e4d20ada4f2db4a86ec61fac6",
    "name": "Grape",
    "symbol": "GRP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31106/thumb/grape_logo-removebg-preview_%281%29.png?1696529937"
  },
  {
    "chainId": 1,
    "address": "0xc8bf8bc34874e07f6a0d4abc8be22ba9e372631b",
    "name": "SmartWorld Global Token",
    "symbol": "SWGT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35040/thumb/IMG_20240204_024533_022.png?1707192592"
  },
  {
    "chainId": 1,
    "address": "0x95d8bf2f57cf973251972b496dc6b1d9c6b5bce3",
    "name": "Blue",
    "symbol": "BLUE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35234/thumb/photo_2023-11-08_08-59-01-removebg-preview-removebg-preview.png?1707935380"
  },
  {
    "chainId": 1,
    "address": "0xc106b98c4d0b3f1c92da0e9a6089e9c63ceacbb0",
    "name": "TrendAppend",
    "symbol": "TRND",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35190/thumb/TrendApp_-_Copy.jpg?1707817712"
  },
  {
    "chainId": 1,
    "address": "0x926ff6584b5905cc793cfb19bfc0ad6443671f47",
    "name": "Pablo DeFi",
    "symbol": "PABLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34799/thumb/FSNlYKG.png?1706074430"
  },
  {
    "chainId": 1,
    "address": "0x77be1ba1cd2d7a63bffc772d361168cc327dd8bc",
    "name": "Meow Meow Coin",
    "symbol": "MEOW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35079/thumb/meow.jpeg?1707301706"
  },
  {
    "chainId": 1,
    "address": "0xd555498a524612c67f286df0e0a9a64a73a7cdc7",
    "name": "DeFrogs",
    "symbol": "DEFROGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35080/thumb/defrogs.jpeg?1707303202"
  },
  {
    "chainId": 1,
    "address": "0x6953f27db0701e22616e701dba91acc2e4b6deca",
    "name": "Starbot",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34917/thumb/Logo_transparent_small.png?1706613080"
  },
  {
    "chainId": 1,
    "address": "0x3e362038fd3d08887d498944d489af7909619a9b",
    "name": "CHOW CHOW",
    "symbol": "CHOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34803/thumb/CHOW_CHOW.png?1706094366"
  },
  {
    "chainId": 1,
    "address": "0x595832f8fc6bf59c85c527fec3740a1b7a361269",
    "name": "Powerledger",
    "symbol": "POWR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/1104/thumb/Powerledger_Northstar_colour_digital_%282%29.png?1706702222"
  },
  {
    "chainId": 1,
    "address": "0x5eed99d066a8caf10f3e4327c1b3d8b673485eed",
    "name": "Garden",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34671/thumb/icon.png?1705656915"
  },
  {
    "chainId": 1,
    "address": "0xdfb03da57a3c56124c72a47729a1d0ed54d38ff5",
    "name": "Hedex",
    "symbol": "HEDEX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35236/thumb/IMG_8082.png?1707935959"
  },
  {
    "chainId": 1,
    "address": "0xea81dab2e0ecbc6b5c4172de4c22b6ef6e55bd8f",
    "name": "Plebbit",
    "symbol": "PLEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31903/thumb/WgMdNjT4_400x400.png?1696530713"
  },
  {
    "chainId": 1,
    "address": "0xf091867ec603a6628ed83d274e835539d82e9cc8",
    "name": "ZetaChain",
    "symbol": "ZETA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26718/thumb/Twitter_icon.png?1696525788"
  },
  {
    "chainId": 1,
    "address": "0x9a96ec9b57fb64fbc60b423d1f4da7691bd35079",
    "name": "Ajna Protocol",
    "symbol": "AJNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30187/thumb/AJNA-Icon-200.png?1696529105"
  },
  {
    "chainId": 1,
    "address": "0x944824290cc12f31ae18ef51216a223ba4063092",
    "name": "Masa",
    "symbol": "MASA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35538/thumb/masa.png?1712635899"
  },
  {
    "chainId": 1,
    "address": "0xb3999f658c0391d94a37f7ff328f3fec942bcadc",
    "name": "Hashflow",
    "symbol": "HFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26136/thumb/hashflow-icon-cmc.png?1696525224"
  },
  {
    "chainId": 1,
    "address": "0x5a9261b023692405f2f680240c6b010638e416dd",
    "name": "Storm Warfare",
    "symbol": "JAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35045/thumb/janus_token.jpeg?1707201471"
  },
  {
    "chainId": 1,
    "address": "0xb7037457de15fed6cbecc0c62d5d610834b958ec",
    "name": "Whirl",
    "symbol": "WHIRL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34966/thumb/CG_LOGO.png?1706806621"
  },
  {
    "chainId": 1,
    "address": "0xa8dfdfdf9d288e909a96178e15731a6f4048a7aa",
    "name": "ScanAI",
    "symbol": "SCAN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34970/thumb/1000005375.jpg?1706851346"
  },
  {
    "chainId": 1,
    "address": "0x8110af6bd2af3f5c4586032ff813c8934451abbe",
    "name": "Cypher AI",
    "symbol": "CYPHER",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35946/thumb/IMG_3059.jpeg?1710234192"
  },
  {
    "chainId": 1,
    "address": "0x4c73c1c8c95de5674d53604b15d968485414cb32",
    "name": "Bomb Shelter Inu",
    "symbol": "BOOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35947/thumb/BOOM_LOGO.png?1710234299"
  },
  {
    "chainId": 1,
    "address": "0xbdcd291c32e06bbf2d7b1ffc823959e3258e3583",
    "name": "WASD Studios",
    "symbol": "WASD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36442/thumb/logo200.png?1711440765"
  },
  {
    "chainId": 1,
    "address": "0x7d225c4cc612e61d26523b099b0718d03152edef",
    "name": "FlokiFork",
    "symbol": "FORK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35048/thumb/floki-inu-floki-logo200x200.png?1707206165"
  },
  {
    "chainId": 1,
    "address": "0x39a9728fb398583154e6cc5e3defa60908f58e2f",
    "name": "BEFY",
    "symbol": "BEFY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34964/thumb/Logo-orange_%281%29.png?1706803795"
  },
  {
    "chainId": 1,
    "address": "0xdaa7699352ac8709f3d2fd092226d3dd7da40474",
    "name": "OPCAT",
    "symbol": "OPCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34965/thumb/photo_2024-01-22_14.49.24.png?1706803957"
  },
  {
    "chainId": 1,
    "address": "0x1b54a6fa1360bd71a0f28f77a1d6fba215d498c3",
    "name": "Casinu Inu",
    "symbol": "CASINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34978/thumb/casinuLOGOCG.png?1706861457"
  },
  {
    "chainId": 1,
    "address": "0x344d6117ae0984f3afdd23e593f92d95a83dcd0e",
    "name": "Skibidi Toilet",
    "symbol": "TOILET",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32832/thumb/IMG_1165.jpeg?1705778817"
  },
  {
    "chainId": 1,
    "address": "0xd4568d313fa2d5c6eba37c9f30f6fb8d1c552ddd",
    "name": "Ledgity Token",
    "symbol": "LDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35046/thumb/ldy-token.png?1707204910"
  },
  {
    "chainId": 1,
    "address": "0x0447d3454b25935eed47f65b4bd22b9b23be326a",
    "name": "Opal",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34925/thumb/opal.jpeg?1706671563"
  },
  {
    "chainId": 1,
    "address": "0xd69a0a9682f679f50e34de40105a93bebb2ff43d",
    "name": "Mackerel",
    "symbol": "MACKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33618/thumb/macke-icon.png?1702528445"
  },
  {
    "chainId": 1,
    "address": "0x10703ca5e253306e2ababd68e963198be8887c81",
    "name": "0xScans",
    "symbol": "SCAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34983/thumb/photo_2024-02-01_20-12-00.jpg?1706873450"
  },
  {
    "chainId": 1,
    "address": "0x8c688327c9371bb3bd69f6e1f1a6d8c9ca0880a7",
    "name": "   D D ",
    "symbol": "DADA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34988/thumb/0202_DADA.jpg?1706880001"
  },
  {
    "chainId": 1,
    "address": "0x149cac67f1cd5d80651e7c9bb359ec285d821a05",
    "name": "Minterest",
    "symbol": "MINTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29455/thumb/MINTY.png?1706817774"
  },
  {
    "chainId": 1,
    "address": "0x8dcaec45365e5ada5676073a07b418c2f538145a",
    "name": "Shell Protocol Token",
    "symbol": "SHELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35192/thumb/shellprotocoltoken-reduced.png?1712428374"
  },
  {
    "chainId": 1,
    "address": "0x1bbe973bef3a977fc51cbed703e8ffdefe001fed",
    "name": "Portal",
    "symbol": "PORTAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35436/thumb/portal.jpeg?1708590254"
  },
  {
    "chainId": 1,
    "address": "0xfe39c384d702914127a005523f9915addb9bd59b",
    "name": "Hippocrat",
    "symbol": "HPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4243/thumb/Hippocrat_Token_Logo_ver.1.0.jpg?1707157531"
  },
  {
    "chainId": 1,
    "address": "0xe9572938bcbf08adcee86fd12a7c0d08dc4ab841",
    "name": "Inscribe",
    "symbol": "INS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34960/thumb/nwb4N3Gx_400x400.jpg?1706788569"
  },
  {
    "chainId": 1,
    "address": "0x00aba6fe5557de1a1d565658cbddddf7c710a1eb",
    "name": "EasyFi V2",
    "symbol": "EZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12742/thumb/Logo_Icon.png?1696512540"
  },
  {
    "chainId": 1,
    "address": "0x72fca22c6070b4cf68abdb719fa484d9ef10a73b",
    "name": "Sowa AI",
    "symbol": "SOWA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36798/thumb/sowa_ai_logo.png?1712441273"
  },
  {
    "chainId": 1,
    "address": "0x6b0faca7ba905a86f221ceb5ca404f605e5b3131",
    "name": "DeFi",
    "symbol": "DEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34369/thumb/defi_logo.png?1704905360"
  },
  {
    "chainId": 1,
    "address": "0xfc0b60e0df5dc9d4b72d957ca2d251cee308019a",
    "name": "SLG GAMES",
    "symbol": "SLG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26436/thumb/SLG_LOGO18.jpg?1706730326"
  },
  {
    "chainId": 1,
    "address": "0xc3f8143212871014b472ea83285af7f25928dee4",
    "name": "SOHOTRN",
    "symbol": "SOHOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34719/thumb/SOHOTRN.png?1705916112"
  },
  {
    "chainId": 1,
    "address": "0x8df5066cf67d909eb67b82854cf54026d31fffae",
    "name": "KOI",
    "symbol": "KOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33927/thumb/coingecko.png?1703408786"
  },
  {
    "chainId": 1,
    "address": "0x97effb790f2fbb701d88f89db4521348a2b77be8",
    "name": "Convergence Finance",
    "symbol": "CVG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34989/thumb/AVI.png?1706887801"
  },
  {
    "chainId": 1,
    "address": "0x723696965f47b990dff00064fcaca95f0ee01123",
    "name": "Alpha Radar AI",
    "symbol": "ARBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31873/thumb/ARBOT_logo_200*200.png?1696530685"
  },
  {
    "chainId": 1,
    "address": "0x43415eb6ff9db7e26a15b704e7a3edce97d31c4e",
    "name": "Superstate Short Duration U S  Governme",
    "symbol": "USTB",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35012/thumb/ustb.jpeg?1707102680"
  },
  {
    "chainId": 1,
    "address": "0xb81914f05daf95802eb30726a399733e0696cd79",
    "name": "Hola",
    "symbol": "HOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35050/thumb/Hola.png?1707210683"
  },
  {
    "chainId": 1,
    "address": "0x78223d31298107f3e310b09797b07967832046a6",
    "name": "ROYAL SMART FUTURE TOKEN",
    "symbol": "RSFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26630/thumb/RSF200x200.png?1696525703"
  },
  {
    "chainId": 1,
    "address": "0xbef26bd568e421d6708cca55ad6e35f8bfa0c406",
    "name": "bitsCrunch Token",
    "symbol": "BCUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34832/thumb/logo-256_%281%29.png?1710244462"
  },
  {
    "chainId": 1,
    "address": "0x0305f515fa978cf87226cf8a9776d25bcfb2cc0b",
    "name": "Pepe 2 0",
    "symbol": "PEPE20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30853/thumb/PEPE_2.0.png?1707684909"
  },
  {
    "chainId": 1,
    "address": "0xa849cd6239906f23b63ba34441b73a5c6eba8a00",
    "name": "HashMind",
    "symbol": "HASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35014/thumb/IMG_20240203_223037_717.jpg?1707107441"
  },
  {
    "chainId": 1,
    "address": "0x2c06ba9e7f0daccbc1f6a33ea67e85bb68fbee3a",
    "name": "Lends",
    "symbol": "LENDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35015/thumb/logo-200x200.png?1707107578"
  },
  {
    "chainId": 1,
    "address": "0xe16f2ec94e8a0819eb93022c45e05d582f4e5c15",
    "name": "Anzen Private Credit",
    "symbol": "PCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35018/thumb/Anzen-Logo-Symbol-Color_small_square.png?1707110484"
  },
  {
    "chainId": 1,
    "address": "0xe085ff62e049b5e3a8d12bc6dffc2dfa329a8894",
    "name": "FlexBot",
    "symbol": "FLEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35016/thumb/flexbot.jpeg?1707108134"
  },
  {
    "chainId": 1,
    "address": "0x4752613b11dfc4e735c4853692c43542e87f0cc2",
    "name": "Goofy Inu",
    "symbol": "GOOFY",
    "decimals": 9
  },
  {
    "chainId": 1,
    "address": "0x177c3973b16c16fb5d934ca92b6e6afb03383268",
    "name": "Potion 404",
    "symbol": "P404",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35141/thumb/JYgmhV72_400x400.jpg?1707454768"
  },
  {
    "chainId": 1,
    "address": "0x6ef3d766dfe02dc4bf04aae9122eb9a0ded25615",
    "name": "Prime Staked ETH",
    "symbol": "PRIMEETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34994/thumb/PrimeETH_Token_Logo.png?1706951612"
  },
  {
    "chainId": 1,
    "address": "0x99b600d0a4abdbc4a6796225a160bcf3d5ce2a89",
    "name": "Solareum",
    "symbol": "SRM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30072/thumb/green_lrg_transparent.png?1696528993"
  },
  {
    "chainId": 1,
    "address": "0xc0db17bc219c5ca8746c29ee47862ee3ad742f4a",
    "name": "ScottyTheAi",
    "symbol": "SCOTTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35052/thumb/scotty_logo_200_x_200.png?1707305097"
  },
  {
    "chainId": 1,
    "address": "0x60c0d11c10a0c04acb47c6296156bdffac62ef97",
    "name": "BBCGoldCoin",
    "symbol": "BBCG",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/35064/thumb/Gold_Token_Icon_kl_transp-1.png?1707241154"
  },
  {
    "chainId": 1,
    "address": "0x47e5c76f155083f1aee39578311a2a5faa938a82",
    "name": "Trinique",
    "symbol": "TNQ",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/36533/thumb/TNQ_LOGO.png?1711705168"
  },
  {
    "chainId": 1,
    "address": "0x42069d11a2cc72388a2e06210921e839cfbd3280",
    "name": "GnomeLand",
    "symbol": "GNOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35201/thumb/logopng.png?1707841763"
  },
  {
    "chainId": 1,
    "address": "0x4b7265d153886a7dc717e815862acde6ff7b5bc8",
    "name": "DENCHCOIN",
    "symbol": "DENCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11103/thumb/Dench3-e1582923464559.png?1696511043"
  },
  {
    "chainId": 1,
    "address": "0x0d88ed6e74bbfd96b831231638b66c05571e824f",
    "name": "Aventus",
    "symbol": "AVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/901/thumb/1673958879448.jpeg?1696502024"
  },
  {
    "chainId": 1,
    "address": "0x18084fba666a33d37592fa2633fd49a74dd93a88",
    "name": "tBTC",
    "symbol": "TBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11224/thumb/0x18084fba666a33d37592fa2633fd49a74dd93a88.png?1696511155"
  },
  {
    "chainId": 1,
    "address": "0x6d7497751656618fc38cfb5478994a20f7e235df",
    "name": "SPYRO",
    "symbol": "SPYRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35203/thumb/IMG_8253.jpeg?1707843227"
  },
  {
    "chainId": 1,
    "address": "0x38d64ce1bdf1a9f24e0ec469c9cade61236fb4a0",
    "name": "Vector ETH",
    "symbol": "VETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35085/thumb/vETH200.png?1709644096"
  },
  {
    "chainId": 1,
    "address": "0xa1290d69c65a6fe4df752f95823fae25cb99e5a7",
    "name": "Kelp DAO Restaked ETH",
    "symbol": "RSETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33800/thumb/Icon___Dark.png?1702991855"
  },
  {
    "chainId": 1,
    "address": "0xc82e3db60a52cf7529253b4ec688f631aad9e7c2",
    "name": "Arc",
    "symbol": "ARC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24235/thumb/arc.png?1696523420"
  },
  {
    "chainId": 1,
    "address": "0xf5264e1673c9365e7c5d4d1d8b440bbf131ff435",
    "name": "vitalek buteren",
    "symbol": "VITALEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35949/thumb/small.jpg?1710238389"
  },
  {
    "chainId": 1,
    "address": "0xb5c457ddb4ce3312a6c5a2b056a1652bd542a208",
    "name": "EtherRock404",
    "symbol": "ROCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35095/thumb/0.png?1707372403"
  },
  {
    "chainId": 1,
    "address": "0x6e8908cfa881c9f6f2c64d3436e7b80b1bf0093f",
    "name": "Bistroo",
    "symbol": "BIST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15645/thumb/bistroo.png?1696515277"
  },
  {
    "chainId": 1,
    "address": "0xbe33f57f41a20b2f00dec91dcc1169597f36221f",
    "name": "Rug",
    "symbol": "RUG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35096/thumb/photo_2024-02-06_19-27-44.jpg?1707372728"
  },
  {
    "chainId": 1,
    "address": "0xfc10cd3895f2c66d6639ec33ae6360d6cfca7d6d",
    "name": "YES",
    "symbol": "YES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32939/thumb/yeslogo2.jpg?1707169040"
  },
  {
    "chainId": 1,
    "address": "0xfa219a9bd37712ef04e5b04abce9ace538ffafb7",
    "name": "Pet the Dog",
    "symbol": "DOGPET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35097/thumb/s01PN6T7_400x400.jpg?1707374459"
  },
  {
    "chainId": 1,
    "address": "0x1f0efa15e9cb7ea9596257da63fecc36ba469b30",
    "name": "Anon",
    "symbol": "ANON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35098/thumb/ANON.jpg?1707412821"
  },
  {
    "chainId": 1,
    "address": "0x9e9fbde7c7a83c43913bddc8779158f1368f0413",
    "name": "Pandora",
    "symbol": "PANDORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34996/thumb/PandoraToken.png?1706954340"
  },
  {
    "chainId": 1,
    "address": "0x44d03e7ee25caee19172eaddf5fbfbff07990c7e",
    "name": "Duel Royale",
    "symbol": "ROYALE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35549/thumb/Duel_Royale.png?1709095119"
  },
  {
    "chainId": 1,
    "address": "0x0c48250eb1f29491f1efbeec0261eb556f0973c7",
    "name": "Aimbot AI",
    "symbol": "AIMBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31200/thumb/200x200.jpg?1696530027"
  },
  {
    "chainId": 1,
    "address": "0x626e8036deb333b408be468f951bdb42433cbf18",
    "name": "AIOZ Network",
    "symbol": "AIOZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14631/thumb/aioz-logo-200.png?1696514309"
  },
  {
    "chainId": 1,
    "address": "0x72adadb447784dd7ab1f472467750fc485e4cb2d",
    "name": "Worldcore  OLD ",
    "symbol": "WRC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/1358/thumb/1000956327.png?1708951434"
  },
  {
    "chainId": 1,
    "address": "0xc944e90c64b2c07662a292be6244bdf05cda44a7",
    "name": "The Graph",
    "symbol": "GRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13397/thumb/Graph_Token.png?1696513159"
  },
  {
    "chainId": 1,
    "address": "0xbe59baad09b07086ee6c39bd0fc234c157c31ccc",
    "name": "LONG TOTEM",
    "symbol": "LONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35103/thumb/IMG_20240206_184148_122.jpg?1707380395"
  },
  {
    "chainId": 1,
    "address": "0x57f5e098cad7a3d1eed53991d4d66c45c9af7812",
    "name": "Wrapped USDM",
    "symbol": "WUSDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33785/thumb/wUSDM_PNG_240px.png?1702981552"
  },
  {
    "chainId": 1,
    "address": "0x8b91f277501cf8322ebe34f137dd35b384b353c7",
    "name": "CoreAI",
    "symbol": "CORE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35106/thumb/Black_And_White._Global_Technology_Abstract_Logo.__%287%29_%281%29.png?1707381177"
  },
  {
    "chainId": 1,
    "address": "0xd1d2eb1b1e90b638588728b4130137d262c87cae",
    "name": "GALA",
    "symbol": "GALA",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12493/thumb/GALA_token_image_-_200PNG.png?1709725869"
  },
  {
    "chainId": 1,
    "address": "0xdc1de096ed4ec5b48f4b0496a6ecee3d63e74f53",
    "name": "ERROR404",
    "symbol": "PNF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35144/thumb/IMG_20240209_071114_878.jpg?1707468943"
  },
  {
    "chainId": 1,
    "address": "0x5c77bffc54e33ee4ae303a168a4a0e45b238d09c",
    "name": "WORMZ",
    "symbol": "WORMZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35105/thumb/logo_200x200.png?1707506487"
  },
  {
    "chainId": 1,
    "address": "0x5fd82ad8a72252a7a449aded99efec7dd8efa690",
    "name": "The GameHub",
    "symbol": "GHUB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36718/thumb/photo_2024-04-02_10.33.14.jpeg?1712128679"
  },
  {
    "chainId": 1,
    "address": "0xfd20e1b78c353877a25274c85fb5566277e5f60e",
    "name": "HighNoon",
    "symbol": "NOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33867/thumb/Noon200x200.png?1703130601"
  },
  {
    "chainId": 1,
    "address": "0xf250b1f6193941bb8bff4152d719edf1a59c0e69",
    "name": "AiRealm Tech",
    "symbol": "AIRM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35066/thumb/IMG_20240203_120402_573.png?1707287784"
  },
  {
    "chainId": 1,
    "address": "0xd289ea09aeece390629e9414d41b4d9d9bf43fd9",
    "name": "AI Technology",
    "symbol": "AITEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35107/thumb/a99PT0iF_400x400.jpg?1707381565"
  },
  {
    "chainId": 1,
    "address": "0x74e9fee3fcb56bccac22e726cce7a78ca90185e1",
    "name": "Rizo",
    "symbol": "RIZO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35440/thumb/rizo_200x200_transparent.png?1708614246"
  },
  {
    "chainId": 1,
    "address": "0x354c8cda7e3b737d360513a0dc5abcee8ee1cea3",
    "name": "BABYTRUMP",
    "symbol": "BABYTRUMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34750/thumb/babytrumpNEWlogo200.png?1705954645"
  },
  {
    "chainId": 1,
    "address": "0x2781246fe707bb15cee3e5ea354e2154a2877b16",
    "name": "ELYSIA",
    "symbol": "EL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10887/thumb/CeyRVXPY_400x400.jpg?1696510840"
  },
  {
    "chainId": 1,
    "address": "0x65d72aa8da931f047169112fcf34f52dbaae7d18",
    "name": "f x  rUSD",
    "symbol": "RUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36338/thumb/rusd.png?1711105384"
  },
  {
    "chainId": 1,
    "address": "0x62d0a8458ed7719fdaf978fe5929c6d342b0bfce",
    "name": "Beam",
    "symbol": "BEAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32417/thumb/chain-logo.png?1698114384"
  },
  {
    "chainId": 1,
    "address": "0x249ca82617ec3dfb2589c4c17ab7ec9765350a18",
    "name": "Verse",
    "symbol": "VERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28424/thumb/verselogo.png?1696527421"
  },
  {
    "chainId": 1,
    "address": "0xb8a87405d9a4f2f866319b77004e88dff66c0d92",
    "name": "Sora AI",
    "symbol": "SORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35293/thumb/IMG_20240315_205215_106.jpg?1710729039"
  },
  {
    "chainId": 1,
    "address": "0xb90b2a35c65dbc466b04240097ca756ad2005295",
    "name": "BOBO Coin",
    "symbol": "BOBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30388/thumb/bobologo.png?1696529277"
  },
  {
    "chainId": 1,
    "address": "0x6930450a416252c7206fbce76c01ecc850a36cb9",
    "name": "SHEBOSHIS",
    "symbol": "SHEB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35291/thumb/SHEB_11200.jpg?1708080526"
  },
  {
    "chainId": 1,
    "address": "0x686f2404e77ab0d9070a46cdfb0b7fecdd2318b0",
    "name": "LORDS",
    "symbol": "LORDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22171/thumb/Frame_1.png?1696521515"
  },
  {
    "chainId": 1,
    "address": "0x61db59daf2af68310fffa003d3b2f452260b673c",
    "name": "Laurion 404",
    "symbol": "LAURION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35233/thumb/k9sEsCqR_400x400.png?1707935032"
  },
  {
    "chainId": 1,
    "address": "0x95ad61b0a150d79219dcf64e1e6cc01f0b64c4ce",
    "name": "Shiba Inu",
    "symbol": "SHIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11939/thumb/shiba.png?1696511800"
  },
  {
    "chainId": 1,
    "address": "0xf161cdb9aa33b2f48be273dae3f3bbb2330ad3e5",
    "name": "AZUR Token",
    "symbol": "AZUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35384/thumb/azur_logo_200_%284%29.png?1708446037"
  },
  {
    "chainId": 1,
    "address": "0x1a59eec501745ad6bdfc37558ddacb38ca5a8c48",
    "name": "Cosmic",
    "symbol": "COSMIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35119/thumb/download.png?1707409910"
  },
  {
    "chainId": 1,
    "address": "0xb1f1f47061a7be15c69f378cb3f69423bd58f2f8",
    "name": "Flashstake",
    "symbol": "FLASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13533/thumb/FLASH.png?1696513294"
  },
  {
    "chainId": 1,
    "address": "0xcc4304a31d09258b0029ea7fe63d032f52e44efe",
    "name": "TrustSwap",
    "symbol": "SWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11795/thumb/Untitled_design-removebg-preview.png?1696511671"
  },
  {
    "chainId": 1,
    "address": "0x46305b2ebcd92809d5fcef577c20c28a185af03c",
    "name": "Shadowladys DN404",
    "symbol": "SHADOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35243/thumb/fixed_logo.png?1707986261"
  },
  {
    "chainId": 1,
    "address": "0xc78b628b060258300218740b1a7a5b3c82b3bd9f",
    "name": "Commune AI",
    "symbol": "COMAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35013/thumb/Wrapped_Commune_AI.png?1707103767"
  },
  {
    "chainId": 1,
    "address": "0x9c7d4fb43919def524c1a9d92fe836169eaf0615",
    "name": "Landwolf",
    "symbol": "WOLF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35907/thumb/COINGECKO_NEW_UPDATE.png?1710338755"
  },
  {
    "chainId": 1,
    "address": "0xb7955695d1df86f35bfbda5e5b7d3069a5639a19",
    "name": "Bully",
    "symbol": "BULLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35252/thumb/Bully.png?1707992406"
  },
  {
    "chainId": 1,
    "address": "0x0d8775f648430679a709e98d2b0cb6250d2887ef",
    "name": "Basic Attention",
    "symbol": "BAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/677/thumb/basic-attention-token.png?1696501867"
  },
  {
    "chainId": 1,
    "address": "0x8afe4055ebc86bd2afb3940c0095c9aca511d852",
    "name": "Arbius",
    "symbol": "AIUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35246/thumb/arbius-200x-logo.png?1707987961"
  },
  {
    "chainId": 1,
    "address": "0x60158131416f5e53d55d73a11be2e203cb26abcc",
    "name": "Hyper",
    "symbol": "EON",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35247/thumb/EON_Token_200_x_200.png?1707987981"
  },
  {
    "chainId": 1,
    "address": "0xa3b3beaf9c0a6160a8e47f000c094d34121f1a57",
    "name": "Coffee Club Token",
    "symbol": "COFFEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35248/thumb/Coffee_Club_Token.png?1707988187"
  },
  {
    "chainId": 1,
    "address": "0x40a7df3df8b56147b781353d379cb960120211d7",
    "name": "Moby",
    "symbol": "MOBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35146/thumb/photo_2024-02-13_09-43-04.jpg?1707846594"
  },
  {
    "chainId": 1,
    "address": "0xe99379955b676d5a7ebe3f42f2b684796e48d437",
    "name": "EGG ETH",
    "symbol": "EGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35388/thumb/eggcoingecko.png?1708450304"
  },
  {
    "chainId": 1,
    "address": "0xce6e54daa1ea95fb3530859d69d4bdb978dd821b",
    "name": "Ordibank",
    "symbol": "ORBK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35736/thumb/obank_light_200x200.png?1710247386"
  },
  {
    "chainId": 1,
    "address": "0x31c8eacbffdd875c74b94b077895bd78cf1e64a3",
    "name": "Radworks",
    "symbol": "RAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14013/thumb/radicle.png?1696513741"
  },
  {
    "chainId": 1,
    "address": "0x3005003bda885dee7c74182e5fe336e9e3df87bb",
    "name": "LENX XD",
    "symbol": "XD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34496/thumb/XD.png?1705264686"
  },
  {
    "chainId": 1,
    "address": "0x33e80a92a9ea73dd02f6e732d1702d58c68388ca",
    "name": "XBANKING",
    "symbol": "XB",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/31650/thumb/Web_1920_%E2%80%93_1.png?1696530465"
  },
  {
    "chainId": 1,
    "address": "0x826180541412d574cf1336d22c0c0a287822678a",
    "name": "Chainflip",
    "symbol": "FLIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25576/thumb/kdt_AgmT_400x400.png?1696524709"
  },
  {
    "chainId": 1,
    "address": "0xb60acd2057067dc9ed8c083f5aa227a244044fd6",
    "name": "Tensorplex Staked TAO",
    "symbol": "STTAO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35330/thumb/stTAO_Logo.png?1708268881"
  },
  {
    "chainId": 1,
    "address": "0xdac17f958d2ee523a2206206994597c13d831ec7",
    "name": "Tether",
    "symbol": "USDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/325/thumb/Tether.png?1696501661"
  },
  {
    "chainId": 1,
    "address": "0xf3617e8a04265160b9ee10253a2c78565571cb76",
    "name": "Blepe the Blue",
    "symbol": "BLEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35391/thumb/photo_2024-02-20_03.15.26.jpeg?1708452233"
  },
  {
    "chainId": 1,
    "address": "0xe08cd119df4c2f147f623559d6985b8afe315873",
    "name": "Bacon",
    "symbol": "BACON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35414/thumb/CG_BACON.JPEG?1708504711"
  },
  {
    "chainId": 1,
    "address": "0x19efa7d0fc88ffe461d1091f8cbe56dc2708a84f",
    "name": "UNITAO",
    "symbol": "UNITAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35743/thumb/rsz_logo11.png?1709976563"
  },
  {
    "chainId": 1,
    "address": "0xe2f95ee8b72ffed59bc4d2f35b1d19b909a6e6b3",
    "name": "EGGX",
    "symbol": "EGGX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35372/thumb/Eggx_item_200x200.png?1708401789"
  },
  {
    "chainId": 1,
    "address": "0x909c1b012d1a56584484bc39969da13ee11e8fa6",
    "name": "ARENA SUPPLY CRATE",
    "symbol": "SPLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35395/thumb/SupplyCrate.png?1708454681"
  },
  {
    "chainId": 1,
    "address": "0x47d20c7800906e01adfc890f283c110ba70a6c97",
    "name": "Azure",
    "symbol": "AZR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34898/thumb/azure200.png?1706582792"
  },
  {
    "chainId": 1,
    "address": "0xbc544207ff1c5b2bc47a35f745010b603b97e99e",
    "name": "AI PIN",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34235/thumb/AiPin-200px.png?1704306335"
  },
  {
    "chainId": 1,
    "address": "0xdb82c0d91e057e05600c8f8dc836beb41da6df14",
    "name": "Smart Layer Network",
    "symbol": "SLN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35397/thumb/sln.jpeg?1708488815"
  },
  {
    "chainId": 1,
    "address": "0x9c2dc0c3cc2badde84b0025cf4df1c5af288d835",
    "name": "Coreto",
    "symbol": "COR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12856/thumb/COR.png?1696512646"
  },
  {
    "chainId": 1,
    "address": "0x62126ec407eae34393ab88b1f5d57e8566e570be",
    "name": "Virtumate",
    "symbol": "MATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35520/thumb/IMG_9102.jpeg?1709006618"
  },
  {
    "chainId": 1,
    "address": "0xe8b1e79d937c648ce1fe96e6739ddb2714058a18",
    "name": "ColonizeMars",
    "symbol": "GTM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35278/thumb/GTM_LOGO.jpg?1708054688"
  },
  {
    "chainId": 1,
    "address": "0xeb51b8dc2d43469c0f0b7365c8a18438907bdf21",
    "name": "Shiva Inu",
    "symbol": "SHIV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36461/thumb/Shiv_200200.jpg?1711509775"
  },
  {
    "chainId": 1,
    "address": "0x3c4b6e6e1ea3d4863700d7f76b36b7f3d3f13e3d",
    "name": "Voyager VGX",
    "symbol": "VGX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/794/thumb/Voyager-vgx.png?1696501945"
  },
  {
    "chainId": 1,
    "address": "0x8b8aa7777e18408c07b0ed52ca3dd5bdab34eb7e",
    "name": "Paradox",
    "symbol": "PDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35448/thumb/Paradox.png?1708620624"
  },
  {
    "chainId": 1,
    "address": "0xa599ab9ee3837be7c5f7b8abf0ce10bc49a2b779",
    "name": "Spider",
    "symbol": "SPIDER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35450/thumb/Black_BG_Spider-Logo.png?1708664882"
  },
  {
    "chainId": 1,
    "address": "0x9c05d54645306d4c4ead6f75846000e1554c0360",
    "name": "21 co Wrapped ADA",
    "symbol": "21ADA",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34614/thumb/21ADA.png?1705503285"
  },
  {
    "chainId": 1,
    "address": "0x48afbbd342f64ef8a9ab1c143719b63c2ad81710",
    "name": "mpETH",
    "symbol": "MPETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35401/thumb/mpETH-jpeg.jpg?1708541939"
  },
  {
    "chainId": 1,
    "address": "0x875773784af8135ea0ef43b5a374aad105c5d39e",
    "name": "IDLE",
    "symbol": "IDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13286/thumb/image.png?1696513058"
  },
  {
    "chainId": 1,
    "address": "0x888888848b652b3e3a0f34c96e00eec0f3a23f72",
    "name": "Alien Worlds",
    "symbol": "TLM",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/14676/thumb/kY-C4o7RThfWrDQsLCAG4q4clZhBDDfJQVhWUEKxXAzyQYMj4Jmq1zmFwpRqxhAJFPOa0AsW_PTSshoPuMnXNwq3rU7Imp15QimXTjlXMx0nC088mt1rIwRs75GnLLugWjSllxgzvQ9YrP4tBgclK4_rb17hjnusGj_c0u2fx0AvVokjSNB-v2poTj0xT9BZRCbzRE3-lF1.jpg?1696514350"
  },
  {
    "chainId": 1,
    "address": "0x83e9f223e1edb3486f876ee888d76bfba26c475a",
    "name": "BlockchainSpace",
    "symbol": "GUILD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21271/thumb/BednjMw.png?1696520642"
  },
  {
    "chainId": 1,
    "address": "0x2f57430a6ceda85a67121757785877b4a71b8e6d",
    "name": "DefiPlaza",
    "symbol": "DFP2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19552/thumb/profile-round.png?1696518985"
  },
  {
    "chainId": 1,
    "address": "0xbec771d15f7e67bc0bb4571c7eb409228cc6fef9",
    "name": "BribeAI",
    "symbol": "BRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36494/thumb/bribeai_logo.png?1711613014"
  },
  {
    "chainId": 1,
    "address": "0xbf72ee725f9b06dc564324774801acebad061946",
    "name": "DApp AI",
    "symbol": "DAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35741/thumb/logo_222.jpg?1709712476"
  },
  {
    "chainId": 1,
    "address": "0xba0dda8762c24da9487f5fa026a9b64b695a07ea",
    "name": "OX Coin",
    "symbol": "OX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35365/thumb/logo.png?1708395976"
  },
  {
    "chainId": 1,
    "address": "0xd0bcb2c156a3507670f9bedc319a6409c41ba68e",
    "name": "DeepL",
    "symbol": "DEEPL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35525/thumb/photo_2024-02-23_17-54-05.jpg?1709015938"
  },
  {
    "chainId": 1,
    "address": "0xb93746dc7a8bf5e381ffb278085b1f62bd0a1fc8",
    "name": "Sendex AI",
    "symbol": "SENDEX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35602/thumb/cg.png?1709277111"
  },
  {
    "chainId": 1,
    "address": "0x958a03181806cb8807bef40e864e55f89b3393d3",
    "name": "UTIX",
    "symbol": "UTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35619/thumb/utxlogo.png?1709287098"
  },
  {
    "chainId": 1,
    "address": "0x1a7e4e63778b4f12a199c062f3efdd288afcbce8",
    "name": "EURA",
    "symbol": "EURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19479/thumb/agEUR-4.png?1710726218"
  },
  {
    "chainId": 1,
    "address": "0xd46ba6d942050d489dbd938a2c909a5d5039a161",
    "name": "Ampleforth",
    "symbol": "AMPL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/4708/thumb/Ampleforth.png?1696505273"
  },
  {
    "chainId": 1,
    "address": "0x8d6cebd76f18e1558d4db88138e2defb3909fad6",
    "name": "MAI  Ethereum ",
    "symbol": "MIMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35521/thumb/mimatic-red.png?1709007081"
  },
  {
    "chainId": 1,
    "address": "0x808688c820ab080a6ff1019f03e5ec227d9b522b",
    "name": "Bag",
    "symbol": "BAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35417/thumb/BAG-coingecko.png?1709393428"
  },
  {
    "chainId": 1,
    "address": "0xce5464b006a10d20c5fc56a19618212a129eed45",
    "name": "Egostation",
    "symbol": "ESTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36816/thumb/1000104152.jpg?1712550282"
  },
  {
    "chainId": 1,
    "address": "0x66d5c66e7c83e0682d947176534242c9f19b3365",
    "name": "Staked Vector",
    "symbol": "SVEC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35462/thumb/sVEC200.png?1709644177"
  },
  {
    "chainId": 1,
    "address": "0xf411903cbc70a74d22900a5de66a2dda66507255",
    "name": "Verasity",
    "symbol": "VRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14025/thumb/VRA.jpg?1696513752"
  },
  {
    "chainId": 1,
    "address": "0x68bbed6a47194eff1cf514b50ea91895597fc91e",
    "name": "ANDY the Wisguy",
    "symbol": "ANDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35910/thumb/IMG_20240309_044840_797.jpg?1710179397"
  },
  {
    "chainId": 1,
    "address": "0x1495bc9e44af1f8bcb62278d2bec4540cf0c05ea",
    "name": "Zero1 Labs",
    "symbol": "DEAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36315/thumb/512x512-xion-ava03.png?1711092111"
  },
  {
    "chainId": 1,
    "address": "0x788d86e00ab31db859c3d6b80d5a9375801d7f2a",
    "name": "TENET",
    "symbol": "TENET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30108/thumb/tenet_original.png?1696529030"
  },
  {
    "chainId": 1,
    "address": "0xef3daa5fda8ad7aabff4658f1f78061fd626b8f0",
    "name": "MUZZLE",
    "symbol": "MUZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35527/thumb/muzz-token-200.png?1709024198"
  },
  {
    "chainId": 1,
    "address": "0xae4533189c7281501f04ba4b7c37e3aded402902",
    "name": "World Friendship Cash",
    "symbol": "WFCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31997/thumb/wfca.jpg?1696530798"
  },
  {
    "chainId": 1,
    "address": "0x586ee5df24c5a426e42ed7ea6e3eb0f00a4a2256",
    "name": "AirDAO",
    "symbol": "AMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1041/thumb/amb.png?1696502148"
  },
  {
    "chainId": 1,
    "address": "0x559babc46a6ab52fd11b2790bce01f0a434a8b4e",
    "name": "TrendGuru",
    "symbol": "TRENDGURU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35728/thumb/TrendGuru.png?1709662816"
  },
  {
    "chainId": 1,
    "address": "0xe485e2f1bab389c08721b291f6b59780fec83fd7",
    "name": "Shutter",
    "symbol": "SHU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35524/thumb/Shutter_logo.png?1709014167"
  },
  {
    "chainId": 1,
    "address": "0x2907dcbc51191a5d7494077d1156fffc34f110ab",
    "name": "Dibbles 404",
    "symbol": "ERRDB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35747/thumb/download_%281%29.png?1709715045"
  },
  {
    "chainId": 1,
    "address": "0xd155fa55c40d010335aa152891aa687e2f3090bd",
    "name": "AInalysis",
    "symbol": "AIL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35603/thumb/ainalysis_200x200.png?1709277575"
  },
  {
    "chainId": 1,
    "address": "0x7a65cb87f596caf31a4932f074c59c0592be77d7",
    "name": "Zypto Token",
    "symbol": "ZYPTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19089/thumb/image_2024-02-23_172710251.png?1708680432"
  },
  {
    "chainId": 1,
    "address": "0xc5842df170b8c8d09eb851a8d5db3dfa00669e3f",
    "name": "Xero AI",
    "symbol": "XEROAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36539/thumb/Untitled_design_%285%29.png?1711708210"
  },
  {
    "chainId": 1,
    "address": "0x4d224452801aced8b2f0aebe155379bb5d594381",
    "name": "ApeCoin",
    "symbol": "APE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24383/thumb/apecoin.jpg?1696523566"
  },
  {
    "chainId": 1,
    "address": "0xd979c468a68062e7bdff4ba6df7842dfd3492e0f",
    "name": "Beoble",
    "symbol": "BBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35011/thumb/beoble_200px.png?1707102471"
  },
  {
    "chainId": 1,
    "address": "0xe020b01b6fbd83066aa2e8ee0ccd1eb8d9cc70bf",
    "name": "Arcade",
    "symbol": "ARCD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33112/thumb/OFFICIAL_ARCD_TOKEN_SYMBOL_64x64.png?1711190585"
  },
  {
    "chainId": 1,
    "address": "0xd417144312dbf50465b1c641d016962017ef6240",
    "name": "Covalent",
    "symbol": "CQT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14168/thumb/CQT_Logo.png?1711563318"
  },
  {
    "chainId": 1,
    "address": "0x3429d03c6f7521aec737a0bbf2e5ddcef2c3ae31",
    "name": "Pixels",
    "symbol": "PIXEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35100/thumb/pixel-icon.png?1708339519"
  },
  {
    "chainId": 1,
    "address": "0x8e3a59427b1d87db234dd4ff63b25e4bf94672f4",
    "name": "Kelp Earned Points",
    "symbol": "KEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35560/thumb/kep.png?1709163916"
  },
  {
    "chainId": 1,
    "address": "0x74fe27e70db10147f8b6b38b3c9d12bbdcf3b5af",
    "name": "iRYDE",
    "symbol": "IRYDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35530/thumb/3pIMg6HT.jpg?1709032145"
  },
  {
    "chainId": 1,
    "address": "0xdda31d354a519ecfb0bc2a536b5e7be147c0f7f4",
    "name": "Everlodge",
    "symbol": "ELDG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35532/thumb/1000034240.png?1709032461"
  },
  {
    "chainId": 1,
    "address": "0x54da724e13d64619a36e1ee6f4bf92d5a410da41",
    "name": "Secured On Blockchain  OLD ",
    "symbol": "SOB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32511/thumb/IMG_20230914_104422_780_%281%29.png?1698399012"
  },
  {
    "chainId": 1,
    "address": "0xb813322cd994a2f7808c340ea12e0a2283a7a757",
    "name": "Cosmo Baby",
    "symbol": "CBABY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35533/thumb/ComcoBaby_Icon_%282%29.png?1709032682"
  },
  {
    "chainId": 1,
    "address": "0x7ba441772fdd56dda03e7e91284db2ea2ba237da",
    "name": "ALPHABYTE",
    "symbol": "ALPHA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35537/thumb/Alphabyte.jpeg?1709033596"
  },
  {
    "chainId": 1,
    "address": "0xb08c567824cdcb530b9da4adc3a3001ee2f170ae",
    "name": "Wh1t3h1v3",
    "symbol": "HIVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35622/thumb/hive.jpg?1709289785"
  },
  {
    "chainId": 1,
    "address": "0xd5d86fc8d5c0ea1ac1ac5dfab6e529c9967a45e9",
    "name": "NFT Worlds",
    "symbol": "WRLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22112/thumb/ZyBrRgfO.jpg?1696521454"
  },
  {
    "chainId": 1,
    "address": "0xf7498c98789957f4ee53b3e37ff5b7ef8a6cfc7b",
    "name": "DEVAI",
    "symbol": "0XDEV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33294/thumb/DevAi.jpg?1702294618"
  },
  {
    "chainId": 1,
    "address": "0x239015de0f0121f2c80134558decbba75ea056c6",
    "name": "Wand",
    "symbol": "WAND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35606/thumb/wand_wizard_200x200.png?1709280453"
  },
  {
    "chainId": 1,
    "address": "0xeb4628418e1acbba62bc72c9b7a53b1865ff283d",
    "name": "0x404",
    "symbol": "XFOUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35623/thumb/0x404.png?1709289914"
  },
  {
    "chainId": 1,
    "address": "0xa6c0c097741d55ecd9a3a7def3a8253fd022ceb9",
    "name": "AVA",
    "symbol": "AVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3014/thumb/AVA_Logo_160x160px_Black.png?1696503750"
  },
  {
    "chainId": 1,
    "address": "0x0e4e7f2aecf408aff4f82f067677050239bdc58a",
    "name": "Fungify Token",
    "symbol": "FUNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35607/thumb/FUNG-logo-200x200.png?1709280585"
  },
  {
    "chainId": 1,
    "address": "0x62cf53a8d024c785e6bd49b83d94a74e1f9c48b1",
    "name": "Aimee",
    "symbol": "AIMEE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35608/thumb/aimeelogo200x200.png?1709280851"
  },
  {
    "chainId": 1,
    "address": "0x102dc1840f0c3c179670f21fa63597e82df34e60",
    "name": "Virtucloud",
    "symbol": "VIRTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35610/thumb/1000004130.png?1709281192"
  },
  {
    "chainId": 1,
    "address": "0x95ccffae3eb8767d4a941ec43280961dde89f4de",
    "name": "TaoBank",
    "symbol": "TBANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35612/thumb/200x200.png?1709281616"
  },
  {
    "chainId": 1,
    "address": "0xe2353069f71a27bbbe66eeabff05de109c7d5e19",
    "name": "Bonsai3",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33162/thumb/logo-bonsai3200x200.png?1700830408"
  },
  {
    "chainId": 1,
    "address": "0x92f419fb7a750aed295b0ddf536276bf5a40124f",
    "name": "Tatsu",
    "symbol": "TATSU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35581/thumb/download.png?1709202359"
  },
  {
    "chainId": 1,
    "address": "0xf76c15cbf1fa9d60a7af39e660d6d7c62ef8021e",
    "name": "Wagyu Protocol",
    "symbol": "WAGYU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35698/thumb/200c.png?1709549986"
  },
  {
    "chainId": 1,
    "address": "0x1c3d163219bb74f430411b95d66b72056f366ec1",
    "name": "ENO",
    "symbol": "ENO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26501/thumb/Logo_%281%29.png?1709565574"
  },
  {
    "chainId": 1,
    "address": "0xfd26e39807772251c3bb90fb1fcd9ce5b80c5c24",
    "name": "Codex Multichain",
    "symbol": "CODEX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33980/thumb/200x200.jpg?1703580135"
  },
  {
    "chainId": 1,
    "address": "0x9fdfb933ee990955d3219d4f892fd1f786b47c9b",
    "name": "Meme Kombat",
    "symbol": "MK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35750/thumb/MKName_200.png?1709718615"
  },
  {
    "chainId": 1,
    "address": "0x51015f4fefe3366c6f62539140149f8088888888",
    "name": "Nuk em Loans",
    "symbol": "NUKEM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35239/thumb/it-s-a-blank-te_62950976_%282%29.png?1707936386"
  },
  {
    "chainId": 1,
    "address": "0xe8196181c5fe192b83c5ca34be910d93d691d935",
    "name": "Self Operating AI",
    "symbol": "SOAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35757/thumb/20240305_131444.png?1709724035"
  },
  {
    "chainId": 1,
    "address": "0xf938346d7117534222b48d09325a6b8162b3a9e7",
    "name": "Choppy",
    "symbol": "CHOPPY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35731/thumb/200x200.png?1709691279"
  },
  {
    "chainId": 1,
    "address": "0xe96edd48cf0c6e930ce55f171a721017b28e0f08",
    "name": "NexusAI",
    "symbol": "NEXUSAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36238/thumb/NexusAI.png?1710918464"
  },
  {
    "chainId": 1,
    "address": "0x73454acfddb7a36a3cd8eb171fbea86c6a55e550",
    "name": "BuildAI",
    "symbol": "BUILD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35460/thumb/etV6ayQT_400x400.jpg?1708675470"
  },
  {
    "chainId": 1,
    "address": "0xa0e7626287bd02cbe3531c65148261bf0c0ed98b",
    "name": "Shill Guard Token",
    "symbol": "SGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33176/thumb/SHILL_GUARD_TOKEN_SGT_LOGO.png?1700923841"
  },
  {
    "chainId": 1,
    "address": "0x6b96c78d4472031f1fbed2d1c4bd3895e9212344",
    "name": "TAOx",
    "symbol": "TAOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35842/thumb/logo-taox.jpeg?1709889951"
  },
  {
    "chainId": 1,
    "address": "0x6e2a43be0b1d33b726f0ca3b8de60b3482b8b050",
    "name": "Arkham",
    "symbol": "ARKM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30929/thumb/Arkham_Logo_CG.png?1696529771"
  },
  {
    "chainId": 1,
    "address": "0x9ae380f0272e2162340a5bb646c354271c0f5cfc",
    "name": "Conic",
    "symbol": "CNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24747/thumb/cnc.png?1696523909"
  },
  {
    "chainId": 1,
    "address": "0xdd66781d0e9a08d4fbb5ec7bac80b691be27f21d",
    "name": "AxonDAO Governance Token",
    "symbol": "AXGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35290/thumb/AXGT-logo-7.png?1708076161"
  },
  {
    "chainId": 1,
    "address": "0x9d14bce1daddf408d77295bb1be9b343814f44de",
    "name": "Koi",
    "symbol": "KOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35766/thumb/Koi_logo.png?1709782399"
  },
  {
    "chainId": 1,
    "address": "0x4b6d036d0bc62a633acca6d10956e9dbbb16748f",
    "name": "Blood Crystal",
    "symbol": "BC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35834/thumb/BloodCrystalToken_200x200.png?1709888447"
  },
  {
    "chainId": 1,
    "address": "0x0d827543d120f49f0614ca1cb2fa18f96ac817f0",
    "name": "Samurai Bot",
    "symbol": "SAMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36047/thumb/6523753be331313328630eae_Sambo_logo_stacked_transparent-p-500.png?1710401756"
  },
  {
    "chainId": 1,
    "address": "0x9194337c06405623c0f374e63fa1cc94e2788c58",
    "name": "CYBONK",
    "symbol": "CYBONK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35802/thumb/cybonk_%281%29.png?1709839583"
  },
  {
    "chainId": 1,
    "address": "0x1ef846ce0da79d8d4e111bf8c5117cd1209a0478",
    "name": "Ethereum Inu",
    "symbol": "ETHINU",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/34875/thumb/1000024435.jpg?1706496056"
  },
  {
    "chainId": 1,
    "address": "0x000000000a1c6659ac226dbb1c5bdc648df72e9e",
    "name": "Looter",
    "symbol": "LOOTER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35837/thumb/LMdtc2Xz_400x400_%281%29.jpg?1709889187"
  },
  {
    "chainId": 1,
    "address": "0x142f4330ab3eda738cb373791c2e99cc325bed20",
    "name": "Secured On Blockchain",
    "symbol": "SOB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36725/thumb/sob.jpeg?1712143555"
  },
  {
    "chainId": 1,
    "address": "0x73c69d24ad28e2d43d03cbf35f79fe26ebde1011",
    "name": "Archimedes Finance",
    "symbol": "ARCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28970/thumb/arch.png?1696527943"
  },
  {
    "chainId": 1,
    "address": "0x21bfbda47a0b4b5b1248c767ee49f7caa9b23697",
    "name": "Ovr",
    "symbol": "OVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13429/thumb/LOGO-OVER-ICON-200X200PX.png?1699396496"
  },
  {
    "chainId": 1,
    "address": "0x1f769203d2abcb78f5a77dd15c0078c50fb13287",
    "name": "Azure Wallet",
    "symbol": "AZURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35753/thumb/Azure.jpeg?1709723218"
  },
  {
    "chainId": 1,
    "address": "0xe2808d7298f0d4111ea1773d3320f5d80eab83e8",
    "name": "WAVX Exchange",
    "symbol": "WAVX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35758/thumb/WAVX.png?1709811960"
  },
  {
    "chainId": 1,
    "address": "0xbe7458bc543cf2df43ac109d2f713dffe6417aa4",
    "name": "Athenas AI",
    "symbol": "ATH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35820/thumb/ath200.png?1709886629"
  },
  {
    "chainId": 1,
    "address": "0xefb20cd759f617ab59b758a1f06de02feb5e8bd5",
    "name": "TaoHarvest",
    "symbol": "TAH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35840/thumb/500x500.jpg?1709889331"
  },
  {
    "chainId": 1,
    "address": "0x391fb9429d8685d5d0f14a185fc0820c6ac2817e",
    "name": "NeonAI",
    "symbol": "NEONAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35691/thumb/Untitled_design%282%29.png?1709535075"
  },
  {
    "chainId": 1,
    "address": "0xea55f9d4a1bcea5e3e806f169f33f0092d5fb7f0",
    "name": "Gemtools",
    "symbol": "GEMS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35815/thumb/53121b16-ddea-4b35-b43a-53335d8426ea.png?1709885212"
  },
  {
    "chainId": 1,
    "address": "0x91fbb2503ac69702061f1ac6885759fc853e6eae",
    "name": "K9 Finance DAO",
    "symbol": "KNINE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35824/thumb/k9_200_X_200.png?1709886958"
  },
  {
    "chainId": 1,
    "address": "0xb4b9dc1c77bdbb135ea907fd5a08094d98883a35",
    "name": "Sweat Economy",
    "symbol": "SWEAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25057/thumb/fhD9Xs16_400x400.jpg?1696524208"
  },
  {
    "chainId": 1,
    "address": "0x03aa6298f1370642642415edc0db8b957783e8d6",
    "name": "NetMind Token",
    "symbol": "NMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35328/thumb/NMT-logo.png?1708267799"
  },
  {
    "chainId": 1,
    "address": "0x0e6d98de664a28a32b584c72e606ff7dea898dc6",
    "name": "Produce AI",
    "symbol": "PRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35817/thumb/200_logo.png?1709885406"
  },
  {
    "chainId": 1,
    "address": "0x05f72d4555e71ef3f5862627828fc2f830d3ccb1",
    "name": "DegenMasters AI",
    "symbol": "DMAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35827/thumb/favicon2_200x200.png?1709887422"
  },
  {
    "chainId": 1,
    "address": "0x49fb8ad7578148e17c3ef0c344ce23a66ed372c4",
    "name": "tao bot",
    "symbol": "TAOBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35829/thumb/taobot200.png?1709887868"
  },
  {
    "chainId": 1,
    "address": "0xff055ff1ec9aa85f14e757d769f9d52d023f2e3b",
    "name": "NodeHUB",
    "symbol": "NHUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35846/thumb/photo_2024-03-07_20.54.49.jpeg?1709891015"
  },
  {
    "chainId": 1,
    "address": "0x747e550a7b848ace786c3cfe754aa78febc8a022",
    "name": "DODO",
    "symbol": "DODO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35955/thumb/listings.png?1710252222"
  },
  {
    "chainId": 1,
    "address": "0x6ab98d9e04cfb57c6b6e9800a5a43a9946a6523d",
    "name": "RUNodE",
    "symbol": "RUDES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36500/thumb/200x200.png?1711872443"
  },
  {
    "chainId": 1,
    "address": "0x0de05f6447ab4d22c8827449ee4ba2d5c288379b",
    "name": "Ooki",
    "symbol": "OOKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21719/thumb/Ooki_Token.png?1696521074"
  },
  {
    "chainId": 1,
    "address": "0x07e128e823d2b9b22edbda43820aa1a72de99613",
    "name": "Host AI",
    "symbol": "HOSTAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35854/thumb/200.png?1709895690"
  },
  {
    "chainId": 1,
    "address": "0xaf6aca2769be86b9910165bff78fea643c086c61",
    "name": "Decentra Ecosystem",
    "symbol": "DCE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35499/thumb/photo_2024-02-19_01.png?1708926162"
  },
  {
    "chainId": 1,
    "address": "0xe3b9cfb8ea8a4f1279fbc28d3e15b4d2d86f18a0",
    "name": "Fottie",
    "symbol": "FOTTIE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35860/thumb/fottie_logo_medium_%281%29.png?1709920764"
  },
  {
    "chainId": 1,
    "address": "0x423352f2c6e0e72422b69af03aba259310146d90",
    "name": "Reality Metaverse",
    "symbol": "RMV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29806/thumb/rltm.png?1696528735"
  },
  {
    "chainId": 1,
    "address": "0x3e5d4ed373596f00a5fc78ceb49994d9dc665aa9",
    "name": "Gem AI",
    "symbol": "GEMAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35992/thumb/gem_ai_logo_200x200.png?1710315447"
  },
  {
    "chainId": 1,
    "address": "0x2e1e15c44ffe4df6a0cb7371cd00d5028e571d14",
    "name": "Mettalex",
    "symbol": "MTLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12730/thumb/nrEqFTEW_400x400.jpg?1696512529"
  },
  {
    "chainId": 1,
    "address": "0x02a97fd243b8301bcd6ba3a87693162d6bd130bd",
    "name": "SempsunAi2 0",
    "symbol": "SMAI20",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35898/thumb/Artboard_14.png?1710164614"
  },
  {
    "chainId": 1,
    "address": "0x72cdb56e9d96c64fb52e8cdeda7475f3aa843f59",
    "name": "TAOPlay",
    "symbol": "TAOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35899/thumb/photo_2024-03-08_00-54-55.jpg?1710164992"
  },
  {
    "chainId": 1,
    "address": "0xd9812f24f34e0d727bbf6ea7caaee05b7f7a2603",
    "name": "TensorSpace",
    "symbol": "TPU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35862/thumb/TensorSpace.png?1709969045"
  },
  {
    "chainId": 1,
    "address": "0xb528edbef013aff855ac3c50b381f253af13b997",
    "name": "Aevo",
    "symbol": "AEVO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35893/thumb/aevo.png?1710138340"
  },
  {
    "chainId": 1,
    "address": "0xac51066d7bec65dc4589368da368b212745d63e8",
    "name": "My Neighbor Alice",
    "symbol": "ALICE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14375/thumb/alice_logo.jpg?1696514067"
  },
  {
    "chainId": 1,
    "address": "0x86b5e2642d094078ad7fe4e0a61f1f4eb4e3280c",
    "name": "SynthAI",
    "symbol": "SYNTHAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35865/thumb/Logo_SynthAI_Logogram.png?1709970520"
  },
  {
    "chainId": 1,
    "address": "0xfa816739edaac4ebf329219d4ef575085cc28037",
    "name": "Search",
    "symbol": "0XSEARCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35866/thumb/photo_2024-03-07_21.13.29.jpeg?1709970596"
  },
  {
    "chainId": 1,
    "address": "0x6a7eff1e2c355ad6eb91bebb5ded49257f3fed98",
    "name": "OpSec",
    "symbol": "OPSEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34284/thumb/IMG_8108.jpeg?1704453267"
  },
  {
    "chainId": 1,
    "address": "0xa1147413cbf11737eff228fb0073c5ce101e9d87",
    "name": " CRATE",
    "symbol": "CRATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35652/thumb/CRATE_LOGO.png?1709379820"
  },
  {
    "chainId": 1,
    "address": "0x38604d52d3a89b16a90053ae0c6d009504a9a5b8",
    "name": "Cloud AI",
    "symbol": "CLD",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/35870/thumb/cloudai-transparent.png?1709973949"
  },
  {
    "chainId": 1,
    "address": "0x8e32b8a41f2e86a3ee198912ac8d756c84295b40",
    "name": "Track The Funds Bot",
    "symbol": "TTF",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34337/thumb/ttf-logo-200.png?1704679746"
  },
  {
    "chainId": 1,
    "address": "0xec33d722d014c02cf4efb46d68098e98e027c364",
    "name": " REKT",
    "symbol": "REKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35894/thumb/rekt.jpeg?1710156572"
  },
  {
    "chainId": 1,
    "address": "0x39de85301c78f4d623e5c05cde2fd119a3a92cd9",
    "name": "blobs",
    "symbol": "BLOBS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35964/thumb/blobs200tnps.png?1710259853"
  },
  {
    "chainId": 1,
    "address": "0x57b49219614859176ddb029298486b6c30193cbd",
    "name": "Oracle AI",
    "symbol": "ORACLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35463/thumb/Oracle_AI_200x200.png?1708677748"
  },
  {
    "chainId": 1,
    "address": "0xb36cf340a35f9860d0bb59afb0355580f0000dad",
    "name": "Padre",
    "symbol": "PADRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36079/thumb/favicon_200x200_transparent_png.png?1710414755"
  },
  {
    "chainId": 1,
    "address": "0x7086e014fd59e66567a986e7b401a5810c2361ce",
    "name": "QUANTUM HUB",
    "symbol": "QUANTUM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35965/thumb/2024-03-10_22.19.52.png?1710260969"
  },
  {
    "chainId": 1,
    "address": "0xfe0c30065b384f05761f15d0cc899d4f9f9cc0eb",
    "name": "Ether fi",
    "symbol": "ETHFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35958/thumb/etherfi.jpeg?1710254562"
  },
  {
    "chainId": 1,
    "address": "0xae41b275aaaf484b541a5881a2dded9515184cca",
    "name": "ChainSwap",
    "symbol": "CSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35888/thumb/ChainSwap.png?1710121063"
  },
  {
    "chainId": 1,
    "address": "0xda251891e21e6edb0e6828e77621c7b98ea4e8ba",
    "name": "Meme Moguls",
    "symbol": "MGLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35975/thumb/IMG_20240311_163933_476.jpg?1710298041"
  },
  {
    "chainId": 1,
    "address": "0x70b02a7d6d86c8ed2193d3c167468ebae28b9538",
    "name": "WorldTao",
    "symbol": "WTAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35973/thumb/world_tao.jpeg?1710297866"
  },
  {
    "chainId": 1,
    "address": "0x609c117183c9264af6ee54fe3f290f212b31afbf",
    "name": " CryptoPunks 721",
    "symbol": "721",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35974/thumb/CPunks.png?1710297918"
  },
  {
    "chainId": 1,
    "address": "0x571d9b73dc04ed88b4e273e048c8d4848f83b779",
    "name": "Super Closed Source",
    "symbol": "CLOSEDAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35977/thumb/400x400.jpg?1710306253"
  },
  {
    "chainId": 1,
    "address": "0x134359b7c852c82e4ebdd16a61020e6b81dd6a6b",
    "name": "Bet Lounge",
    "symbol": "BETZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35994/thumb/200px-07.png?1710777520"
  },
  {
    "chainId": 1,
    "address": "0x8dce83eca4af45dbe618da1779f9aaca43201084",
    "name": "AlphaKEK AI",
    "symbol": "AIKEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35445/thumb/alphakek_-_Copy.png?1708620097"
  },
  {
    "chainId": 1,
    "address": "0x9094c15f2f535a765e8a2dac20b05148be7044cd",
    "name": "Fusion Ai",
    "symbol": "FUSION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35980/thumb/Fusion_Ai_200x200.png?1710306750"
  },
  {
    "chainId": 1,
    "address": "0x56b8be7c2d3ffe0d8d6feb4d4eb4650c3ea10bb6",
    "name": "LocalAI",
    "symbol": "LOCAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35984/thumb/photo_6012793510064931894_c.jpg?1710307895"
  },
  {
    "chainId": 1,
    "address": "0xc092a137df3cf2b9e5971ba1874d26487c12626d",
    "name": "Ring AI",
    "symbol": "RING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35995/thumb/Logo_PP_200x200_circle.png?1710315842"
  },
  {
    "chainId": 1,
    "address": "0x3c0bb14e8367c384885a97bac6d5cceab474ed75",
    "name": "Artificial idiot",
    "symbol": "AII",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36082/thumb/AII_200x200.png?1710415710"
  },
  {
    "chainId": 1,
    "address": "0x22514ffb0d7232a56f0c24090e7b68f179faa940",
    "name": "QORPO WORLD",
    "symbol": "QORPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35585/thumb/qorpo.jpeg?1709203322"
  },
  {
    "chainId": 1,
    "address": "0xfd4f2caf941b6d737382dce420b368de3fc7f2d4",
    "name": "Patex",
    "symbol": "PATEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35987/thumb/ajhjO1Bk_400x400.jpg?1710313200"
  },
  {
    "chainId": 1,
    "address": "0x4f45acfd7361e283170e28394bbe18395407e0f0",
    "name": "LinkTensor",
    "symbol": "LTS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35990/thumb/4XIsLD9h_400x400.jpg?1710314291"
  },
  {
    "chainId": 1,
    "address": "0xf94e7d0710709388bce3161c32b4eea56d3f91cc",
    "name": "Destra Network",
    "symbol": "DSYNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35996/thumb/Destra_Network.png?1710316001"
  },
  {
    "chainId": 1,
    "address": "0x8e4d27d772099e18900caf5dcc36cb612dcee886",
    "name": "Architex",
    "symbol": "ARCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36026/thumb/arcx.jpeg?1710387472"
  },
  {
    "chainId": 1,
    "address": "0xe842e272a18625319cc36f64eb9f97e5ad0c32af",
    "name": "YAK",
    "symbol": "YAK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35674/thumb/cg.png?1709509096"
  },
  {
    "chainId": 1,
    "address": "0xb418ded94300913fccbef784a49150f46f0fb827",
    "name": "TERRIER",
    "symbol": "BULL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36001/thumb/IMG_3130.jpeg?1710320533"
  },
  {
    "chainId": 1,
    "address": "0xdd4ce03b97085e5023d3a5fbff6e4f2c4dffb7c3",
    "name": "PORA AI",
    "symbol": "PORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36015/thumb/download.png?1710326560"
  },
  {
    "chainId": 1,
    "address": "0x78e3b2ee11950df78a35fd924e92fbb8d1403780",
    "name": "Helga Inu",
    "symbol": "HELGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36201/thumb/logo2.png?1710832936"
  },
  {
    "chainId": 1,
    "address": "0x2a8bf9e7c91c6e3237095e07eb3defbce358df00",
    "name": "OpClouds",
    "symbol": "OPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36004/thumb/IMG_20240312_002726_458.jpg?1710323036"
  },
  {
    "chainId": 1,
    "address": "0x49e1cc8d19e4c52bde82412f900f35ea5e9b5901",
    "name": "TAOVM",
    "symbol": "TAOVM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36005/thumb/200.png?1710323142"
  },
  {
    "chainId": 1,
    "address": "0xd775997452923437ca96065ba15ed02f4a33ed39",
    "name": "dragon wif hat",
    "symbol": "DWIF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36037/thumb/logo.png?1710397494"
  },
  {
    "chainId": 1,
    "address": "0x793a5d8b30aab326f83d20a9370c827fea8fdc51",
    "name": "Gorilla In A Coupe",
    "symbol": "GIAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36016/thumb/Gorilla_In_A_Coupe.png?1710326800"
  },
  {
    "chainId": 1,
    "address": "0xd2ac55ca3bbd2dd1e9936ec640dcb4b745fde759",
    "name": "Index Coop Bitcoin 2x Index",
    "symbol": "BTC2X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36017/thumb/BTC2x-token-logo.png?1710327454"
  },
  {
    "chainId": 1,
    "address": "0x65c4c0517025ec0843c9146af266a2c5a2d148a2",
    "name": "Index Coop Ethereum 2x Index",
    "symbol": "ETH2X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36018/thumb/ETH2x-token-logo.png?1710327494"
  },
  {
    "chainId": 1,
    "address": "0xbf7bc9e63635dc11b335d52b0349d0100a53a1a7",
    "name": "LinkTao",
    "symbol": "LTAO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35963/thumb/Tao.jpeg?1710258313"
  },
  {
    "chainId": 1,
    "address": "0xe4129c7b229812212f88d1bd6a223c45622e6b85",
    "name": "Looted Network",
    "symbol": "LOOT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35518/thumb/Looted_Network.png?1709006038"
  },
  {
    "chainId": 1,
    "address": "0x93109af5638be68ed2d0e094f618777ff1051d28",
    "name": "UNIT DAO",
    "symbol": "UN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34306/thumb/UNITDao.png?1704614014"
  },
  {
    "chainId": 1,
    "address": "0x085780639cc2cacd35e474e71f4d000e2405d8f6",
    "name": "f x  Protocol fxUSD",
    "symbol": "FXUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36202/thumb/fxUSD.jpg?1710833113"
  },
  {
    "chainId": 1,
    "address": "0x4d243e8f511045f0d5f9d0288bc628737b10c079",
    "name": "Jason  Eth ",
    "symbol": "JASON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36232/thumb/jason_%281%29.jpeg?1710906316"
  },
  {
    "chainId": 1,
    "address": "0x0d3bd40758df4f79aad316707fcb809cd4815ffe",
    "name": "21 co Wrapped XRP",
    "symbol": "21XRP",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34615/thumb/XRP.png?1705503368"
  },
  {
    "chainId": 1,
    "address": "0x81994dfbbe69c9e03226c2ebe62f7352dc3db812",
    "name": "RaidSharksBot",
    "symbol": "SHARX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36056/thumb/raidsharkbot-brand-icon-light-circle-colored.png?1710404378"
  },
  {
    "chainId": 1,
    "address": "0x92d6c1e31e14520e676a687f0a93788b716beff5",
    "name": "dYdX",
    "symbol": "ETHDYDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17500/thumb/hjnIm9bV.jpg?1696517040"
  },
  {
    "chainId": 1,
    "address": "0x693170bd3c37dcd46168d8b399aa7551a32de2af",
    "name": "TAO Subnet Sharding",
    "symbol": "TAOSHARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36051/thumb/LOGO_COIN_2-removebg-preview.png?1710402750"
  },
  {
    "chainId": 1,
    "address": "0x1ae7e1d0ce06364ced9ad58225a1705b3e5db92b",
    "name": "lmeow",
    "symbol": "LMEOW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35794/thumb/Untitled_design_200%281%29.png?1709801192"
  },
  {
    "chainId": 1,
    "address": "0x8c7ac134ed985367eadc6f727d79e8295e11435c",
    "name": "The Balkan Dwarf",
    "symbol": "KEKEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32563/thumb/download-compresskaru.com.png?1698546378"
  },
  {
    "chainId": 1,
    "address": "0x5ca135cb8527d76e932f34b5145575f9d8cbe08e",
    "name": "Frax Price Index",
    "symbol": "FPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24945/thumb/FPI_icon.png?1696524100"
  },
  {
    "chainId": 1,
    "address": "0x0c79e09497c9862479c9e6499c61342b5fe3e6b2",
    "name": "HashPower AI",
    "symbol": "HASH",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36055/thumb/photo_2024-03-10_19-30-59_%282%29.jpg?1710403993"
  },
  {
    "chainId": 1,
    "address": "0x8c213ae332274e6314bf4cf989604e7f61162967",
    "name": "Dongo AI",
    "symbol": "DONGO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36058/thumb/pfp.png?1712164948"
  },
  {
    "chainId": 1,
    "address": "0x967da4048cd07ab37855c090aaf366e4ce1b9f48",
    "name": "Ocean Protocol",
    "symbol": "OCEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3687/thumb/ocean-protocol-logo.jpg?1696504363"
  },
  {
    "chainId": 1,
    "address": "0xaaa9214f675316182eaa21c85f0ca99160cc3aaa",
    "name": "QANplatform",
    "symbol": "QANX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15977/thumb/qanx.png?1696515591"
  },
  {
    "chainId": 1,
    "address": "0x73af41fe7054057218e0eb07fe43ba5f25c7d79f",
    "name": "CASHCAB",
    "symbol": "CAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36062/thumb/s7xKrsPT_400x400.jpg?1710405287"
  },
  {
    "chainId": 1,
    "address": "0xe9eccde9d26fcbb5e93f536cfc4510a7f46274f8",
    "name": "infraX",
    "symbol": "INFRA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36063/thumb/RENDR.png?1710405370"
  },
  {
    "chainId": 1,
    "address": "0xad913dcd987fe54ce823e4b755f90598cd62bb15",
    "name": "MFERS",
    "symbol": "MFERS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36064/thumb/logo200.png?1710405377"
  },
  {
    "chainId": 1,
    "address": "0x410e731c2970dce3add351064acf5ce9e33fdbf0",
    "name": "ONBUFF",
    "symbol": "ONIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12252/thumb/d4suDHSzEZjDXc0uxFGPZXfDFp0L1tfoEusBx03zB1bvMLLbxa7Yq5naXAOutkirIlLXacFRRebX_hF2Ez69_fIPUfr2LuN0Z_UrXTLYds4aT95C6eHGXsZnezMHKAaOmjLpKto0xPTiYkXExvEvXrqL7O_J1wecb4eJsgwCoVprMLpm89BdKO9IyMmpNsfbW7JJFMiHgL8vq9fPhaeEi.jpg?1696512083"
  },
  {
    "chainId": 1,
    "address": "0x6a26edf3bbc9f154ca9175216ceb9812f5305e6e",
    "name": "Pixel",
    "symbol": "PIXE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35251/thumb/Pixel_200x200_transparent.png?1708889564"
  },
  {
    "chainId": 1,
    "address": "0x7d0c49057c09501595a8ce23b773bb36a40b521f",
    "name": "PREME Token",
    "symbol": "PREME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36091/thumb/PREME_Token.png?1710477118"
  },
  {
    "chainId": 1,
    "address": "0x2ae21de576e0fe0367651ddcf76e04dd0608c076",
    "name": "Gambit",
    "symbol": "GAMBIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36070/thumb/oHwe1V3Q_400x400.jpg?1710406381"
  },
  {
    "chainId": 1,
    "address": "0xb87b96868644d99cc70a8565ba7311482edebf6e",
    "name": "OnChain Pepe 404",
    "symbol": "OCP404",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35271/thumb/download_%282%29.png?1708017745"
  },
  {
    "chainId": 1,
    "address": "0xb2606492712d311be8f41d940afe8ce742a52d44",
    "name": "zkLend",
    "symbol": "ZEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35979/thumb/zkLend_ZEND_logo_square_transparent_background_%281%29.png?1710306648"
  },
  {
    "chainId": 1,
    "address": "0x85e0b9d3e7e4dba7e59090c533906d0e9211d8b6",
    "name": "Ishi",
    "symbol": "ISHI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36109/thumb/ishi_medium_%281%29.jpg?1710486321"
  },
  {
    "chainId": 1,
    "address": "0x668d78571f124415581b38d32fa9a16f1aaa8417",
    "name": "1ex",
    "symbol": "1EX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34927/thumb/1ex_logo_200%D1%85200.png?1706686304"
  },
  {
    "chainId": 1,
    "address": "0x994bf19e32584255d82896d1477488e87012f209",
    "name": "ComputingAi",
    "symbol": "CPU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36150/thumb/IMG_20240316_201708_467.jpg?1710740157"
  },
  {
    "chainId": 1,
    "address": "0xa91ac63d040deb1b7a5e4d4134ad23eb0ba07e14",
    "name": "Bella Protocol",
    "symbol": "BEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12478/thumb/Bella.png?1696512296"
  },
  {
    "chainId": 1,
    "address": "0x3d1c949a761c11e4cc50c3ae6bdb0f24fd7a39da",
    "name": "Neurahub",
    "symbol": "NEURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36471/thumb/Game_On_%282%29.png?1711524939"
  },
  {
    "chainId": 1,
    "address": "0x05fe069626543842439ef90d9fa1633640c50cf1",
    "name": "Eve AI",
    "symbol": "EVEAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29830/thumb/200-X-200.png?1696528758"
  },
  {
    "chainId": 1,
    "address": "0xa76cec201e939660f8afb1fb8d5865d069df0750",
    "name": "Wanna Bot",
    "symbol": "WANNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36161/thumb/Wanna.Bot_-_Logo_%28CoinGecko%29.png?1710744457"
  },
  {
    "chainId": 1,
    "address": "0x2d17b511a85b401980cc0fed15a8d57fdb8eec60",
    "name": "Pigeon Park",
    "symbol": "PGENZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36162/thumb/Transparent_200x200.png?1710744848"
  },
  {
    "chainId": 1,
    "address": "0x30c81d711253cf61804bc86d223051cb960d5878",
    "name": "StellaryAI",
    "symbol": "STELAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36170/thumb/200x200.png?1710746923"
  },
  {
    "chainId": 1,
    "address": "0x000000000503be77a5ed27bef2c19943a8b5ae73",
    "name": "Xtremeverse",
    "symbol": "XTREME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36163/thumb/xnet_logo.jpg?1710745012"
  },
  {
    "chainId": 1,
    "address": "0xcb21311d3b91b5324f6c11b4f5a656fcacbff122",
    "name": "QuantixAI",
    "symbol": "QAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36166/thumb/07.png?1710745602"
  },
  {
    "chainId": 1,
    "address": "0x78ba134c3ace18e69837b01703d07f0db6fb0a60",
    "name": "Sentinel Bot Ai",
    "symbol": "SNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36171/thumb/sentinelbotai.jpeg?1710746986"
  },
  {
    "chainId": 1,
    "address": "0x3f95aa88ddbb7d9d484aa3d482bf0a80009c52c9",
    "name": "Backed ERNX   Bond",
    "symbol": "BERNX",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0x7039cd6d7966672f194e8139074c3d5c4e6dcf65",
    "name": "Super Trump",
    "symbol": "STRUMP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36083/thumb/Super_Trump.png?1710418492"
  },
  {
    "chainId": 1,
    "address": "0x018dd3a0dd7f213cc822076b3800816d3ce1ed86",
    "name": "HotKeySwap",
    "symbol": "HOTKEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35614/thumb/HOTKEY.jpg?1709281878"
  },
  {
    "chainId": 1,
    "address": "0x0ec78ed49c2d27b315d462d43b5bab94d2c79bf8",
    "name": "MEOW",
    "symbol": "MEOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21530/thumb/meowchain.jpeg?1710834471"
  },
  {
    "chainId": 1,
    "address": "0x6af53c6ec427525f7240e211941223288a0e7c66",
    "name": "Warped Games",
    "symbol": "WARPED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33307/thumb/Warped-ICON-COLOR-ORANGE.png?1709639013"
  },
  {
    "chainId": 1,
    "address": "0xcdb4a8742ed7d0259b51e3454c46c9d6c48d5e88",
    "name": "GPT Protocol",
    "symbol": "GPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35929/thumb/gpt.png?1710225428"
  },
  {
    "chainId": 1,
    "address": "0xda558a295e8d2c1438f7b4806e283940dec7e6db",
    "name": "Tracker AI",
    "symbol": "TRACK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36131/thumb/IMG_20240305_181940_607_%281%29.png?1710581387"
  },
  {
    "chainId": 1,
    "address": "0x740a5ac14d0096c81d331adc1611cf2fd28ae317",
    "name": "Plebz",
    "symbol": "PLEB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36185/thumb/Unknown-1.jpeg?1712340924"
  },
  {
    "chainId": 1,
    "address": "0x622984873c958e00aa0f004cbdd2b5301cf0b132",
    "name": "Gauss0x",
    "symbol": "GAUSS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36192/thumb/gaussian200x200.png?1710761823"
  },
  {
    "chainId": 1,
    "address": "0x004626a008b1acdc4c74ab51644093b155e59a23",
    "name": "Angle Staked EURA",
    "symbol": "STEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32036/thumb/stEUR-x4.png?1696530832"
  },
  {
    "chainId": 1,
    "address": "0xc8dfb57d83bf561457b1a3f6ce22956bb554bcab",
    "name": "TradeMaster ninja",
    "symbol": "TRDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36190/thumb/Transparent_200x200.png?1710761061"
  },
  {
    "chainId": 1,
    "address": "0xf02c2dc9b3cb7f1ba21ccd82dff4ebc92da8996f",
    "name": "TensorScan AI",
    "symbol": "TSA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36193/thumb/TensorScanAI_200x200.png?1710762625"
  },
  {
    "chainId": 1,
    "address": "0x902169d471b62f22ffadc690ca292ec454d0b260",
    "name": "Reboot World",
    "symbol": "RBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36244/thumb/Reboot-Logo_200-1.png?1710921335"
  },
  {
    "chainId": 1,
    "address": "0x93c5a00b41fb5f3906b421b2399ac64b79fdbd42",
    "name": "Voidz",
    "symbol": "VDZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36474/thumb/XcJAYXs7_400x400.jpg?1711527406"
  },
  {
    "chainId": 1,
    "address": "0x518b63da813d46556fea041a88b52e3caa8c16a8",
    "name": "Antfarm Token",
    "symbol": "ATF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28821/thumb/ATF.png?1696527797"
  },
  {
    "chainId": 1,
    "address": "0x1a3a8cf347b2bf5890d3d6a1b981c4f4432c8661",
    "name": "Flying Avocado Cat",
    "symbol": "FAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34223/thumb/Flying_Avocado_Cat.jpg?1704298390"
  },
  {
    "chainId": 1,
    "address": "0xadd353fb2e2c563383ff3272a500f3e7134dafe4",
    "name": "Tunachain",
    "symbol": "TUNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36213/thumb/Project_Logo_B_Transparent.png?1710836529"
  },
  {
    "chainId": 1,
    "address": "0x722a89f1b925fe41883978219c2176aecc7d6699",
    "name": "Kinka",
    "symbol": "XNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36217/thumb/kinka_logomark.png?1710838184"
  },
  {
    "chainId": 1,
    "address": "0x0022228a2cc5e7ef0274a7baa600d44da5ab5776",
    "name": "Angle Staked USDA",
    "symbol": "STUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34509/thumb/stUSD_copie.png?1705288386"
  },
  {
    "chainId": 1,
    "address": "0x4123a133ae3c521fd134d7b13a2dec35b56c2463",
    "name": "Open Custody Protocol",
    "symbol": "OPEN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17541/thumb/OPEN-Token-Black.png?1712837762"
  },
  {
    "chainId": 1,
    "address": "0x72b886d09c117654ab7da13a14d603001de0b777",
    "name": "XDEFI",
    "symbol": "XDEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19524/thumb/FilledLogo.png?1696518958"
  },
  {
    "chainId": 1,
    "address": "0xccccb68e1a848cbdb5b60a974e07aae143ed40c3",
    "name": "HYCHAIN",
    "symbol": "TOPIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31283/thumb/HYCHAIN_Avatar_Circle.png?1710500962"
  },
  {
    "chainId": 1,
    "address": "0xb2d007293a421471586d6bc3fbfeb1990fdfa2e6",
    "name": "Governance VEC",
    "symbol": "GVEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36215/thumb/200x200.png?1710836830"
  },
  {
    "chainId": 1,
    "address": "0x0d3cbed3f69ee050668adf3d9ea57241cba33a2b",
    "name": "PlayDapp",
    "symbol": "PDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14316/thumb/PDA-symbol.png?1710234068"
  },
  {
    "chainId": 1,
    "address": "0x5ea49ff332b7ad99c486347c1c2bcc73d1e22b9b",
    "name": "Synthetic AI",
    "symbol": "SAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36247/thumb/photo_2024-03-18_22.59.08.jpeg?1710923867"
  },
  {
    "chainId": 1,
    "address": "0x23150e1db43a04ae1ea993b76dd3c37156cabe0f",
    "name": "Teq Network",
    "symbol": "TEQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36282/thumb/200.png?1711006401"
  },
  {
    "chainId": 1,
    "address": "0x6d0bb9b6ce385e28ea4ebb7d76dcb3a1aaf7bc4b",
    "name": "CREAT OR",
    "symbol": "CRET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35470/thumb/CRET_Logo.png?1708759906"
  },
  {
    "chainId": 1,
    "address": "0x08ba718f288c3b12b01146816bef9fa03cc635bc",
    "name": "Centaurify",
    "symbol": "CENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20512/thumb/Centaurify_Symbol_Black-01.png?1696519918"
  },
  {
    "chainId": 1,
    "address": "0xc02aaa39b223fe8d0a0e5c4f27ead9083c756cc2",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 1,
    "address": "0xb831717e73dba259ce632b54cfe233d5ead60a1a",
    "name": "Telenode",
    "symbol": "TNODE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36252/thumb/152A9947-A302-4697-8B16-5364E5C083B8_%281%29.png?1710924819"
  },
  {
    "chainId": 1,
    "address": "0x35ca1e5a9b1c09fa542fa18d1ba4d61c8edff852",
    "name": "Schrodi",
    "symbol": "SCHRODI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36255/thumb/apzs3q26_200x200.jpg?1710927534"
  },
  {
    "chainId": 1,
    "address": "0x17d2628d30f8e9e966c9ba831c9b9b01ea8ea75c",
    "name": "ISKRA Token",
    "symbol": "ISK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27428/thumb/ISKRA_logo.png?1696526469"
  },
  {
    "chainId": 1,
    "address": "0x725024200cd4e1f259fcf2b7153d37fb477e139c",
    "name": "Flovi Inu",
    "symbol": "FLOVI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36734/thumb/1000038565.jpg?1712171719"
  },
  {
    "chainId": 1,
    "address": "0x236501327e701692a281934230af0b6be8df3353",
    "name": "Fluence",
    "symbol": "FLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36339/thumb/fluence.png?1711106498"
  },
  {
    "chainId": 1,
    "address": "0x32b053f2cba79f80ada5078cb6b305da92bde6e1",
    "name": "NEURALAI",
    "symbol": "NEURAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36341/thumb/NeuralAI_Token_Logo.png?1712340484"
  },
  {
    "chainId": 1,
    "address": "0x9afa9999e45484adf5d8eed8d9dfe0693bacd838",
    "name": "Everyworld",
    "symbol": "EVERY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36799/thumb/EW_Favicon.png?1712468081"
  },
  {
    "chainId": 1,
    "address": "0x070e984fda37dd942f5c953f6b2375339adac308",
    "name": "Axe Cap",
    "symbol": "AXE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31339/thumb/axe_cap_icon_200x200.png?1696530157"
  },
  {
    "chainId": 1,
    "address": "0xfca59cd816ab1ead66534d82bc21e7515ce441cf",
    "name": "RARI",
    "symbol": "RARI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11845/thumb/rf_logo_%281%29.jpeg?1711076091"
  },
  {
    "chainId": 1,
    "address": "0x2eead196943d6a140ee337a5536de01979ebac9d",
    "name": "Token Sentry Bot",
    "symbol": "SENTRY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36756/thumb/photo_2024-04-01_13-14-28-removebg-preview.jpg?1712245537"
  },
  {
    "chainId": 1,
    "address": "0x50b275a15e4f5004aa96f972a30e6a9c718b203f",
    "name": "StorageChain",
    "symbol": "WSTOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33844/thumb/coinmarketcaplogo.png?1703121289"
  },
  {
    "chainId": 1,
    "address": "0x83869de76b9ad8125e22b857f519f001588c0f62",
    "name": "EXMO Coin",
    "symbol": "EXM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/9154/thumb/200x200.png?1696509276"
  },
  {
    "chainId": 1,
    "address": "0x486d95c40feba650c38e98cd9d7979d9cd88cea0",
    "name": "0xGPU ai",
    "symbol": "0XG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36373/thumb/0xGPU_200x200.png?1711339982"
  },
  {
    "chainId": 1,
    "address": "0x25f8087ead173b73d6e8b84329989a8eea16cf73",
    "name": "Yield Guild Games",
    "symbol": "YGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17358/thumb/Shield_Mark_-_Colored_-_Iridescent.png?1696516909"
  },
  {
    "chainId": 1,
    "address": "0xc61edb127f58f42f47a8be8aebe83cf602a53878",
    "name": "Castle Of Blackwater",
    "symbol": "COBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36002/thumb/COBEcoinLOGO.png?1711118569"
  },
  {
    "chainId": 1,
    "address": "0x85d19fb57ca7da715695fcf347ca2169144523a7",
    "name": "Conan",
    "symbol": "CONAN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35566/thumb/IMG_20240227_061234_892.jpg?1709182894"
  },
  {
    "chainId": 1,
    "address": "0x443459d45c30a03f90037d011cbe22e2183d3b12",
    "name": "TypeAI",
    "symbol": "TYPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34484/thumb/Type_AI_%28Custom%29.jpg?1705044610"
  },
  {
    "chainId": 1,
    "address": "0x0a3bb08b3a15a19b4de82f8acfc862606fb69a2d",
    "name": "iZUMi Bond USD",
    "symbol": "IUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25388/thumb/iusd-logo-symbol-10k%E5%A4%A7%E5%B0%8F.png?1696524521"
  },
  {
    "chainId": 1,
    "address": "0xd47bdf574b4f76210ed503e0efe81b58aa061f3d",
    "name": "TRVL",
    "symbol": "TRVL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20911/thumb/trvl.jpeg?1696520301"
  },
  {
    "chainId": 1,
    "address": "0xde342a3e269056fc3305f9e315f4c40d917ba521",
    "name": "Byte",
    "symbol": "BYTE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33527/thumb/Byte200.jpeg?1702138460"
  },
  {
    "chainId": 1,
    "address": "0x3472a5a71965499acd81997a54bba8d852c6e53d",
    "name": "Badger",
    "symbol": "BADGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13287/thumb/badger_dao_logo.jpg?1696513059"
  },
  {
    "chainId": 1,
    "address": "0xba100000625a3754423978a60c9317c58a424e3d",
    "name": "Balancer",
    "symbol": "BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11683/thumb/Balancer.png?1696511572"
  },
  {
    "chainId": 1,
    "address": "0x97225fae89b370e7721f961d1145e64df56f2482",
    "name": "Smoked Token Burn",
    "symbol": "BURN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36345/thumb/Smoked_Token_Burn.png?1711187183"
  },
  {
    "chainId": 1,
    "address": "0xe304283c3e60cefaf7ea514007cf4e8fdc3d869d",
    "name": "Gecoin",
    "symbol": "GEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25646/thumb/62162f0d583e5a047dc02e7e_gecoin_200x200.png?1696524779"
  },
  {
    "chainId": 1,
    "address": "0xf65b5c5104c4fafd4b709d9d60a185eae063276c",
    "name": "Truebit Protocol",
    "symbol": "TRU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15053/thumb/Truebit.png?1696514712"
  },
  {
    "chainId": 1,
    "address": "0x2e4e1093b4e80e3cc73c92264715ca984bad7e55",
    "name": "ZionWallet",
    "symbol": "ZION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36381/thumb/IMG_20240323_020534_446.png?1711347294"
  },
  {
    "chainId": 1,
    "address": "0x1b78ffbc66139466c4a432f763afce8d4c991060",
    "name": "Cloudnet Ai",
    "symbol": "CNAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36574/thumb/cloud.jpeg?1711943190"
  },
  {
    "chainId": 1,
    "address": "0x0f76d32cdccdcbd602a55af23eaf58fd1ee17245",
    "name": "Backed ERNA   Bond",
    "symbol": "BERNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36757/thumb/b-ERNA-200x200.png?1712245846"
  },
  {
    "chainId": 1,
    "address": "0x0bc529c00c6401aef6d220be8c6ea1667f6ad93e",
    "name": "yearn finance",
    "symbol": "YFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11849/thumb/yearn.jpg?1696511720"
  },
  {
    "chainId": 1,
    "address": "0x4b9278b94a1112cad404048903b8d343a810b07e",
    "name": "Hifi Finance",
    "symbol": "HIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28712/thumb/hft.png?1696527693"
  },
  {
    "chainId": 1,
    "address": "0x2407d54b42a570aeb63819d433a63676163cb6d6",
    "name": "Renegade",
    "symbol": "RNGD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35327/thumb/ren%28200_x_200_px%29.png?1708267655"
  },
  {
    "chainId": 1,
    "address": "0xc6221ac4e99066ea5443acd67d6108f874e2533d",
    "name": "Decentralized Cloud Infra",
    "symbol": "DCI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36575/thumb/dci.jpeg?1711943349"
  },
  {
    "chainId": 1,
    "address": "0xc4c244f1dbca07083fee35220d2169957c275e68",
    "name": "Steak",
    "symbol": "STEAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36597/thumb/Small_STEAK_Token_Logo_%28200_x_200_px%29.png?1711950525"
  },
  {
    "chainId": 1,
    "address": "0x1a3496c18d558bd9c6c8f609e1b129f67ab08163",
    "name": "DEAPCOIN",
    "symbol": "DEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10970/thumb/DEAPcoin_01.png?1696510917"
  },
  {
    "chainId": 1,
    "address": "0x477a3d269266994f15e9c43a8d9c0561c4928088",
    "name": " ",
    "symbol": "YAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36578/thumb/DP.png?1711944185"
  },
  {
    "chainId": 1,
    "address": "0x680c89c40de9d14aa608a1122363cad18783f837",
    "name": "EPICBOTS",
    "symbol": "EPIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36469/thumb/LOGO01_-_200x200.png?1711518285"
  },
  {
    "chainId": 1,
    "address": "0xb45ad160634c528cc3d2926d9807104fa3157305",
    "name": "sDOLA",
    "symbol": "SDOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35495/thumb/sDOLAlogoFINAL.png?1708922289"
  },
  {
    "chainId": 1,
    "address": "0xfbe44cae91d7df8382208fcdc1fe80e40fbc7e9a",
    "name": "The Next Gem AI",
    "symbol": "GEMAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36515/thumb/GemAI_Logo.png?1711693709"
  },
  {
    "chainId": 1,
    "address": "0xb29dc1703facd2967bb8ade2e392385644c6dca9",
    "name": "Gaga  Pepe ",
    "symbol": "GAGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36546/thumb/gaga200x200.png?1711734371"
  },
  {
    "chainId": 1,
    "address": "0x51573c4c228de0227d3b1e6e598981f8054e8807",
    "name": "NOMADS",
    "symbol": "NOMADS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36758/thumb/nomads.jpg?1712246083"
  },
  {
    "chainId": 1,
    "address": "0xc5ba042bf8832999b17c9036e8212f49dce0501a",
    "name": "YOUR AI",
    "symbol": "YOURAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36260/thumb/yourai-ticker_%281%29.png?1710929542"
  },
  {
    "chainId": 1,
    "address": "0x96884fcaac082db4b32601ada5b177fd6cbffa88",
    "name": "ZkLock",
    "symbol": "ZKLK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36586/thumb/IMG_20240328_215725_691.jpg?1711947986"
  },
  {
    "chainId": 1,
    "address": "0x72a93697a5ac73cfee39ff87298220f77c538611",
    "name": "The Dare",
    "symbol": "DARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36818/thumb/DARE.png?1712550833"
  },
  {
    "chainId": 1,
    "address": "0xb17548c7b510427baac4e267bea62e800b247173",
    "name": "Swarm Markets",
    "symbol": "SMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17488/thumb/swarm-SMT-token-symbol_200x200.png?1696517029"
  },
  {
    "chainId": 1,
    "address": "0xee3200f94a1a2345e6cc486032a5df1d50cb621c",
    "name": "ChartAI",
    "symbol": "CX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31896/thumb/200.png?1696530707"
  },
  {
    "chainId": 1,
    "address": "0x50739bd5b6aff093ba2371365727c48a420a060d",
    "name": "CryptoGPT",
    "symbol": "CRGPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36590/thumb/1000036412.jpg?1711949110"
  },
  {
    "chainId": 1,
    "address": "0x5cc5e64ab764a0f1e97f23984e20fd4528356a6a",
    "name": "XRGB",
    "symbol": "XRGB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35447/thumb/log2.png?1708620430"
  },
  {
    "chainId": 1,
    "address": "0x9cdb8ea1002541a816b271363d64b2787631885f",
    "name": "Wrapped Trade AI",
    "symbol": "WTAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36472/thumb/TradeAI_Logo_200x200.png?1711525843"
  },
  {
    "chainId": 1,
    "address": "0xf3768d6e78e65fc64b8f12ffc824452130bd5394",
    "name": "Kerosene",
    "symbol": "KEROSENE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36603/thumb/kerosene.jpeg?1711960633"
  },
  {
    "chainId": 1,
    "address": "0x02f92800f57bcd74066f5709f1daa1a4302df875",
    "name": "Peapods Finance",
    "symbol": "PEAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33711/thumb/NAzHgbTW_400x400.png?1702856653"
  },
  {
    "chainId": 1,
    "address": "0x13f7b4581df403542286563c2f762077b2a368da",
    "name": "Cloak",
    "symbol": "CLOAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36657/thumb/Dextools_Icon.png?1712042390"
  },
  {
    "chainId": 1,
    "address": "0xf614e54d972bab297172765ca83683288f3eae36",
    "name": "RawBlock",
    "symbol": "RWB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36819/thumb/photo_2024-03-18_04-07-55_%281%29-removebg-preview.jpg?1712551052"
  },
  {
    "chainId": 1,
    "address": "0x66e535e8d2ebf13f49f3d49e5c50395a97c137b1",
    "name": "Molten",
    "symbol": "MOLTEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36726/thumb/moltenmesh.png?1712147407"
  },
  {
    "chainId": 1,
    "address": "0xa67d87058b2f4e958b38717909565b6daefb95fe",
    "name": "Ionic Pocket Token",
    "symbol": "INP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36671/thumb/200x200_%282%29.png?1712061406"
  },
  {
    "chainId": 1,
    "address": "0x7dd9c5cba05e151c895fde1cf355c9a1d5da6429",
    "name": "Golem",
    "symbol": "GLM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/542/thumb/Golem_Submark_Positive_RGB.png?1696501761"
  },
  {
    "chainId": 1,
    "address": "0x8af78f0c818302164f73b2365fe152c2d1fe80e1",
    "name": "Financie Token",
    "symbol": "FNCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29099/thumb/VcJQzCWT_400x400.jpg?1696528062"
  },
  {
    "chainId": 1,
    "address": "0x644192291cc835a93d6330b24ea5f5fedd0eef9e",
    "name": "AllianceBlock Nexera",
    "symbol": "NXRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29181/thumb/nxra.png?1712255778"
  },
  {
    "chainId": 1,
    "address": "0x5488eff1976e4a56b4255e926d419a7054df196a",
    "name": "Citty Meme Coin",
    "symbol": "CITTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36878/thumb/200x200.jpg?1712646838"
  },
  {
    "chainId": 1,
    "address": "0x814a870726edb7dfc4798300ae1ce3e5da0ac467",
    "name": "daCat",
    "symbol": "DACAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34801/thumb/200x200.png?1706075099"
  },
  {
    "chainId": 1,
    "address": "0x82f13ab56cc0d1b727e8253a943f0de75b048b0b",
    "name": "PlayFi",
    "symbol": "PLAYFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36773/thumb/pfiCircleLogo_Alt_Alpha_200.png?1712287250"
  },
  {
    "chainId": 1,
    "address": "0x7fed466b893c716235e1b8d685c913f7d2797463",
    "name": "TensorHub",
    "symbol": "THUB",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36779/thumb/Tensor.jpeg?1712310491"
  },
  {
    "chainId": 1,
    "address": "0x2903bd7db50f300b0884f7a15904baffc77f3ec7",
    "name": "Arcade",
    "symbol": "ARC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36682/thumb/Arcade_200_x_200.png?1712067926"
  },
  {
    "chainId": 1,
    "address": "0x00000000e88649dd6aab90088ca25d772d4607d0",
    "name": "Underworld",
    "symbol": "UDW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36704/thumb/UDW-HR-Ticker.jpg?1712125179"
  },
  {
    "chainId": 1,
    "address": "0x0f17eeccc84739b9450c88de0429020e2dec05eb",
    "name": "Otacon AI",
    "symbol": "OTACON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32475/thumb/otacon_logo-square_200x200.png?1698285813"
  },
  {
    "chainId": 1,
    "address": "0xd2ca93093e05366dbc2bf88c3bb189a5141fb6f6",
    "name": "SignAI",
    "symbol": "SAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36781/thumb/signai_logo_trans_500.png?1712311353"
  },
  {
    "chainId": 1,
    "address": "0x5ca381bbfb58f0092df149bd3d243b08b9a8386e",
    "name": "Moonchain",
    "symbol": "MXC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4604/thumb/M_1-modified.png?1712206949"
  },
  {
    "chainId": 1,
    "address": "0x07bac35846e5ed502aa91adf6a9e7aa210f2dcbe",
    "name": "Sifchain",
    "symbol": "EROWAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14044/thumb/EROWAN.png?1696513770"
  },
  {
    "chainId": 1,
    "address": "0xad53ed3fee57160f026189eb91e73e9317fa92f1",
    "name": "zkTAO",
    "symbol": "ZAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36782/thumb/photo_2024-04-04_20-59-25.jpg?1712329957"
  },
  {
    "chainId": 1,
    "address": "0x6eef75b8f067b5853a36da5561f52a8f22faa9af",
    "name": "TaoStack",
    "symbol": "TST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36720/thumb/Logomark_Taostack.png?1712129260"
  },
  {
    "chainId": 1,
    "address": "0x292fcdd1b104de5a00250febba9bc6a5092a0076",
    "name": "HashAI",
    "symbol": "HASHAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36743/thumb/HashAI.png?1712212747"
  },
  {
    "chainId": 1,
    "address": "0xba5bde662c17e2adff1075610382b9b691296350",
    "name": "SuperRare",
    "symbol": "RARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17753/thumb/RARE.png?1709598473"
  },
  {
    "chainId": 1,
    "address": "0xe5c6f5fef89b64f36bfccb063962820136bac42f",
    "name": "Hoppy The Frog",
    "symbol": "HOPPY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36745/thumb/1000012607.png?1712217627"
  },
  {
    "chainId": 1,
    "address": "0xca14007eff0db1f8135f4c25b34de49ab0d42766",
    "name": "Starknet",
    "symbol": "STRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26433/thumb/starknet.png?1696525507"
  },
  {
    "chainId": 1,
    "address": "0x84018071282d4b2996272659d9c01cb08dd7327f",
    "name": "Blendr Network",
    "symbol": "BLENDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35765/thumb/Blendr_Network.png?1709752266"
  },
  {
    "chainId": 1,
    "address": "0x00b78238925c320159023c2ac9ef89da8f16d007",
    "name": "VPS Ai",
    "symbol": "VPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36250/thumb/200px_Logo.png?1710924510"
  },
  {
    "chainId": 1,
    "address": "0x891e33c8a49ed9f59be3726834ad78cf597b56e9",
    "name": "Green Games",
    "symbol": "BETTED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29287/thumb/ficha_icon_CG.png?1696528239"
  },
  {
    "chainId": 1,
    "address": "0xc0a4df35568f116c370e6a6a6022ceb908eeddac",
    "name": "UX Chain",
    "symbol": "UX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/20635/thumb/yArLkbL.png?1696520039"
  },
  {
    "chainId": 1,
    "address": "0x467719ad09025fcc6cf6f8311755809d45a5e5f3",
    "name": "Axelar",
    "symbol": "AXL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27277/thumb/V-65_xQ1_400x400.jpeg?1696526329"
  },
  {
    "chainId": 1,
    "address": "0xebb1afb0a4ddc9b1f84d9aa72ff956cd1c1eb4be",
    "name": "The Emerald Company",
    "symbol": "EMRLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36554/thumb/emrld_logo_200x200px.png?1711898920"
  },
  {
    "chainId": 1,
    "address": "0xe635efcfac44c5f44508f4d17c3a96cb4ce421dd",
    "name": "Defispot",
    "symbol": "SPOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36813/thumb/Defispot_200x200_icon.jpg?1712548786"
  },
  {
    "chainId": 1,
    "address": "0xb8176941f1b5173dfdbadcc5d387829b27e98138",
    "name": "DeOrbit Network",
    "symbol": "DEORBIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36960/thumb/IMG_20240411_010755_512.jpg?1712906910"
  },
  {
    "chainId": 1,
    "address": "0x8a2279d4a90b6fe1c4b30fa660cc9f926797baa2",
    "name": "Chromia",
    "symbol": "CHR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/5000/thumb/Chromia.png?1696505533"
  },
  {
    "chainId": 1,
    "address": "0x3585c1ee4a9c2144510084182b96af961f4ce10d",
    "name": "GenAi",
    "symbol": "GENAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36811/thumb/-yaP5LZ9_400x400.jpg?1712547717"
  },
  {
    "chainId": 1,
    "address": "0xee372d2b7e7c83de7e345267b5e4efc1899a4fab",
    "name": "Xtrack AI",
    "symbol": "XTRACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36826/thumb/f16d1dda-c158-4fc2-8dad-c3b910ea5958.jpeg?1712556425"
  },
  {
    "chainId": 1,
    "address": "0xf898bae008cd85046431ab0a75f00689d6aa1b1c",
    "name": "0xVPN org",
    "symbol": "VPN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36921/thumb/0xf898bae008cd85046431ab0a75f00689d6aa1b1c.png?1712797462"
  },
  {
    "chainId": 1,
    "address": "0xad5fdc8c3c18d50315331fca7f66efe5033f6c4c",
    "name": "Crazy Frog Coin",
    "symbol": "CRAZY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33451/thumb/photo_2023-12-05_09.13.36.jpeg?1701859131"
  },
  {
    "chainId": 1,
    "address": "0x6630e3a2ec1e7e0a2f9f1d2289a9a89b0551683a",
    "name": "dogwifsaudihat",
    "symbol": "WIFSA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36885/thumb/Qr2lN8au_400x400.jpg?1712649792"
  },
  {
    "chainId": 1,
    "address": "0x4fabb145d64652a948d72533023f6e7a623c7c53",
    "name": "BUSD",
    "symbol": "BUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9576/thumb/BUSDLOGO.jpg?1696509654"
  },
  {
    "chainId": 1,
    "address": "0x32c6f1c1731ff8f98ee2ede8954f696446307846",
    "name": "Bearded Dragon",
    "symbol": "BEARDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34836/thumb/200x200.jpg?1706266367"
  },
  {
    "chainId": 1,
    "address": "0x46947241be767bd05dc843755a991265170dac27",
    "name": "Bandwidth AI",
    "symbol": "BPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36962/thumb/Bandwidth_AI.png?1712907332"
  },
  {
    "chainId": 1,
    "address": "0x89deb6c8918a42457bd6ddbcaaf979216c4d774c",
    "name": "HapticAI",
    "symbol": "HAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36939/thumb/logo.png?1712850711"
  },
  {
    "chainId": 1,
    "address": "0xe973e453977195422b48e1852a207b7ee9c913c7",
    "name": "ADreward",
    "symbol": "AD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29639/thumb/ADreward_AD_Logo_CoinGecko_200X200_%EB%B6%88%ED%88%AC%EB%AA%85.png?1696528576"
  },
  {
    "chainId": 1,
    "address": "0x0b452278223d3954f4ac050949d7998e373e7e43",
    "name": "Shita kiri Suzume",
    "symbol": "SUZUME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26813/thumb/shitakiri_cmc.png?1696525873"
  },
  {
    "chainId": 1,
    "address": "0x5ce6f2c0e2a1b4540894f286254bf13b1110d240",
    "name": "Bricks Exchange",
    "symbol": "BRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36963/thumb/brx_logo_symbol_2024_200x200.png?1712907453"
  },
  {
    "chainId": 1,
    "address": "0x6715515f5aa98e8bd3624922e1ba91e6f5fc4402",
    "name": "SatoshiSync",
    "symbol": "SSNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36777/thumb/ssnc.jpeg?1712654999"
  },
  {
    "chainId": 1,
    "address": "0x4e432a62733a7ee38ad2e16b3cc0731457ea5b55",
    "name": "QChain QDT",
    "symbol": "QDT",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/26502/thumb/qdt_logo.png?1696525575"
  },
  {
    "chainId": 1,
    "address": "0xf1e345ea7c33fd6c05f5512a780eb5839ee31674",
    "name": "Telefy",
    "symbol": "TELE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26485/thumb/WhatsApp_Image_2022-07-20_at_2.51.42_AM.jpeg?1696525558"
  },
  {
    "chainId": 1,
    "address": "0x6e0615a03ed9527a6013fcd5b556e36ef4dab1ff",
    "name": "HNB Protocol",
    "symbol": "HNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26182/thumb/newhnblogo.png?1696525269"
  },
  {
    "chainId": 1,
    "address": "0x3330bfb7332ca23cd071631837dc289b09c33333",
    "name": "Rubic",
    "symbol": "RBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12629/thumb/rubic.png?1696512437"
  },
  {
    "chainId": 1,
    "address": "0xa130e3a33a4d84b04c3918c4e5762223ae252f80",
    "name": "Swash",
    "symbol": "SWASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18774/thumb/Swash_icon_GP.png?1712869197"
  },
  {
    "chainId": 1,
    "address": "0x60e254e35dd712394b3aba7a1d19114732e143dd",
    "name": "RivusDAO",
    "symbol": "RIVUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36965/thumb/IMG_0167.png?1712907932"
  },
  {
    "chainId": 1,
    "address": "0x90b7e285ab6cf4e3a2487669dba3e339db8a3320",
    "name": "Yellow Duckies",
    "symbol": "DUCKIES",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27630/thumb/duckies_logo.png?1706528164"
  },
  {
    "chainId": 1,
    "address": "0xfe80d611c6403f70e5b1b9b722d2b3510b740b2b",
    "name": "Equilibria Finance",
    "symbol": "EQB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30645/thumb/QLLK8pmR_400x400.jpg?1696529516"
  },
  {
    "chainId": 1,
    "address": "0x614577036f0a024dbc1c88ba616b394dd65d105a",
    "name": "GENIUS AI",
    "symbol": "GNUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36914/thumb/IMG_5346.png?1712769235"
  },
  {
    "chainId": 1,
    "address": "0x1901f826dfcbfd9d3138936932366b3493a50893",
    "name": "Sphynx Labs",
    "symbol": "SPHYNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30142/thumb/photo_2023-05-03_21.44.17.jpeg?1696529063"
  },
  {
    "chainId": 1,
    "address": "0x912529007bc0d2a5464a6a211ebfe217dfb75dff",
    "name": "Cadence Protocol",
    "symbol": "CAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34323/thumb/CAD_200_200_transparent.png?1704646661"
  },
  {
    "chainId": 1,
    "address": "0x4af8aa621df6dd3e2d653188a357fc2b35c6a037",
    "name": "SELFCrypto",
    "symbol": "SELF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30805/thumb/Isotype_1_200x200.png?1696529663"
  },
  {
    "chainId": 1,
    "address": "0x414ac1853329b3704df0caf7749cd296c7f3b750",
    "name": "Anzen Private Credit",
    "symbol": "PCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35018/thumb/Anzen-Logo-Symbol-Color_small_square.png?1707110484"
  },
  {
    "chainId": 1,
    "address": "0x77774a06271d6a305caccdbc06f847def05c7777",
    "name": "Gyrowin",
    "symbol": "GW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32329/thumb/Brandlogo_1-200x200.png?1697451818"
  },
  {
    "chainId": 1,
    "address": "0xc4170fd71eced3c80badca77f4e12e8aac1e3436",
    "name": "Kryptomon",
    "symbol": "KMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17886/thumb/kryptomon.png?1696517407"
  },
  {
    "chainId": 1,
    "address": "0xbc6378faae98fb2207bb6c35c0f8ce5846fd4c6c",
    "name": "8Bit Chain",
    "symbol": "W8BIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35459/thumb/8BIT-200x200.PNG?1708675050"
  },
  {
    "chainId": 1,
    "address": "0x438e48ed4ce6beecf503d43b9dbd3c30d516e7fd",
    "name": "UWON",
    "symbol": "UWON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35593/thumb/2024-02-29_10.53.39.jpg?1709221802"
  },
  {
    "chainId": 1,
    "address": "0x9377e3c3180dddfcda4e4217ed21f2f7c3b235a0",
    "name": "Cavada",
    "symbol": "CAVADA",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35476/thumb/cavada_main.png?1708773117"
  },
  {
    "chainId": 1,
    "address": "0x911d86c72155c33993d594b0ec7e6206b4c803da",
    "name": "Wrapped Staked LINK",
    "symbol": "WSTLINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35678/thumb/wstlink_logo.png?1709525182"
  },
  {
    "chainId": 1,
    "address": "0xf0187b76be05c1fcaa24f39c0a3aab4434099c4f",
    "name": "Aether Games",
    "symbol": "AEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29916/thumb/coin_200x198.png?1708678337"
  },
  {
    "chainId": 1,
    "address": "0xed2d13a70acbd61074fc56bd0d0845e35f793e5e",
    "name": "Planet Mojo",
    "symbol": "MOJO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36670/thumb/MOJOIcon.jpg?1712061269"
  },
  {
    "chainId": 1,
    "address": "0x50d5118fb90d572b9d42ba65e0addc4900867809",
    "name": "Osean",
    "symbol": "OSEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30968/thumb/oceana.png?1696529808"
  },
  {
    "chainId": 1,
    "address": "0x01824357d7d7eaf4677bc17786abd26cbdec9ad7",
    "name": "Forward",
    "symbol": "FORWARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20866/thumb/182860872_110384464503974_4805876988713020861_n.png?1696520260"
  },
  {
    "chainId": 1,
    "address": "0x039d2e8f097331278bd6c1415d839310e0d5ece4",
    "name": "Linda",
    "symbol": "LINDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33699/thumb/linda-logo-200.png?1705166731"
  },
  {
    "chainId": 1,
    "address": "0xcb19b6b4971bd4206bab176c75b1efe3e28ee5a8",
    "name": "Seneca",
    "symbol": "SEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32852/thumb/200x200.png?1699663676"
  },
  {
    "chainId": 1,
    "address": "0xb712d62fe84258292d1961b5150a19bc4ab49026",
    "name": "Chainge",
    "symbol": "XCHNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16526/thumb/chainge.jpeg?1708099639"
  },
  {
    "chainId": 1,
    "address": "0xb51b97dd5569fab69495316b5a065cccff4b829d",
    "name": "Astrafer",
    "symbol": "ASTRAFER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26246/thumb/ATSRA_Token.png?1696525331"
  },
  {
    "chainId": 1,
    "address": "0x5052fa4a2a147eaaa4c0242e9cc54a10a4f42070",
    "name": "HANePlatform",
    "symbol": "HANEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35202/thumb/HANeP_200px.png?1707842735"
  },
  {
    "chainId": 1,
    "address": "0xa80505c408c4defd9522981cd77e026f5a49fe63",
    "name": "NEUY",
    "symbol": "NEUY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25870/thumb/logo200x200.png?1696524953"
  },
  {
    "chainId": 1,
    "address": "0x0000000000300dd8b0230efcfef136ecdf6abcde",
    "name": "DegenX",
    "symbol": "DGNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27204/thumb/logo-200-200.png?1705413440"
  },
  {
    "chainId": 1,
    "address": "0xe0dcb3e02798d1c6a9650fe1381015ec34705153",
    "name": "Match Token",
    "symbol": "MATCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34082/thumb/MATCH.jpg?1703910229"
  },
  {
    "chainId": 1,
    "address": "0x92cc36d66e9d739d50673d1f27929a371fb83a67",
    "name": "Wagmi",
    "symbol": "WAGMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31887/thumb/wagmi_token_logo.png?1696530698"
  },
  {
    "chainId": 1,
    "address": "0xf1264873436a0771e440e2b28072fafcc5eebd01",
    "name": "Kenshi",
    "symbol": "KNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30759/thumb/kenshi.jpg?1696529628"
  },
  {
    "chainId": 1,
    "address": "0x27eb7aa2cce204012f7d9cd4d49baa746ec5ab63",
    "name": "GGTKN",
    "symbol": "GGTKN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26275/thumb/ggtokenofficiallogo.png?1696525358"
  },
  {
    "chainId": 1,
    "address": "0x174c47d6a4e548ed2b7d369dc0ffb2e60a6ac0f8",
    "name": "Amulet Protocol",
    "symbol": "AMU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34716/thumb/AMT200x200.png?1705904544"
  },
  {
    "chainId": 1,
    "address": "0x2c2d8a078b33bf7782a16acce2c5ba6653a90d5f",
    "name": "L3USD",
    "symbol": "L3USD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26937/thumb/L3USD.png?1696525993"
  },
  {
    "chainId": 1,
    "address": "0x872952d3c1caf944852c5adda65633f1ef218a26",
    "name": "Liquid Crypto",
    "symbol": "LQDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33627/thumb/LQDX.png?1702533263"
  },
  {
    "chainId": 1,
    "address": "0xd3cc9d8f3689b83c91b7b59cab4946b063eb894a",
    "name": "Venus",
    "symbol": "XVS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12677/thumb/download.jpg?1696512482"
  },
  {
    "chainId": 1,
    "address": "0x95987b0cdc7f65d989a30b3b7132a38388c548eb",
    "name": "Pundi X PURSE",
    "symbol": "PURSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20419/thumb/HobiV7k__400x400.jpg?1696519827"
  },
  {
    "chainId": 1,
    "address": "0x39162d9f64c0f9d64d14eac3b6d5a15d3d569eea",
    "name": "MrWeb Finance",
    "symbol": "AMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27173/thumb/jd0i3xri_400x400.jpeg?1696526223"
  },
  {
    "chainId": 1,
    "address": "0x79f05c263055ba20ee0e814acd117c20caa10e0c",
    "name": "Ice Network",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34674/thumb/ice.jpeg?1705663609"
  },
  {
    "chainId": 1,
    "address": "0xd3ac016b1b8c80eeadde4d186a9138c9324e4189",
    "name": "Okcash",
    "symbol": "OK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/274/thumb/ok-logo-200px.png?1696501624"
  },
  {
    "chainId": 1,
    "address": "0xbbcb0356bb9e6b3faa5cbf9e5f36185d53403ac9",
    "name": "Backed Coinbase Global",
    "symbol": "BCOIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31872/thumb/b-COIN-200x200.png?1696530684"
  },
  {
    "chainId": 1,
    "address": "0x2cb5d9fd89d48c516f11904117c57e3934f39524",
    "name": "PEPE UWU",
    "symbol": "CUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33691/thumb/Resized_AvEWEWA5_200x200_%281%29.png?1702796454"
  },
  {
    "chainId": 1,
    "address": "0x9424fde68418b71ea1d3de95547ef67f4dc8f573",
    "name": "BandZai Token",
    "symbol": "BZAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30825/thumb/BZAI_coin.png?1696529678"
  },
  {
    "chainId": 1,
    "address": "0x75cb71325a44fb102a742626b723054acb7e1394",
    "name": "Anime",
    "symbol": "ANI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13792/thumb/ani.png?1696513541"
  },
  {
    "chainId": 1,
    "address": "0xe20b9e246db5a0d21bf9209e4858bc9a3ff7a034",
    "name": "Wrapped Banano",
    "symbol": "WBAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32617/thumb/WBAN.jpg?1698749253"
  },
  {
    "chainId": 1,
    "address": "0xbb6cf73a00f480d0951ba979a7606857cdde626b",
    "name": "Arix",
    "symbol": "ARIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13485/thumb/ARIX.png?1696513245"
  },
  {
    "chainId": 1,
    "address": "0x30b47e13ccee15d6fa7a80994c94bb9b4a4bb973",
    "name": "Scratch Meme Coin",
    "symbol": "SCRATS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31323/thumb/logo_%283%29.png?1696530142"
  },
  {
    "chainId": 1,
    "address": "0x830eb1204380e9c44434db8700257025358707c6",
    "name": "Goons of Balatroon",
    "symbol": "GOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27104/thumb/TokenLogo200x200.png?1702250134"
  },
  {
    "chainId": 1,
    "address": "0x0a77ef9bf662d62fbf9ba4cf861eaa83f9cc4fec",
    "name": "X World Games",
    "symbol": "XWG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17847/thumb/200_200_%281%29_%281%29.png?1696790226"
  },
  {
    "chainId": 1,
    "address": "0xe35009059cb55ded065027e9832a2c564aff7512",
    "name": "SecureChain AI",
    "symbol": "SCAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31680/thumb/SCAI-icon-3.png?1696530499"
  },
  {
    "chainId": 1,
    "address": "0x137ddb47ee24eaa998a535ab00378d6bfa84f893",
    "name": "Radiant Capital",
    "symbol": "RDNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26536/thumb/Radiant-Logo-200x200.png?1696525610"
  },
  {
    "chainId": 1,
    "address": "0xaf6a1125d4cc55a4110dc63cd2ff6e005afb8676",
    "name": "PunkCity",
    "symbol": "PUNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31049/thumb/F4FDA24D-1B29-4CBB-8346-854D1BFFE75A.png?1696529883"
  },
  {
    "chainId": 1,
    "address": "0x4563554284aa7148d6e6d0351519e954ba3b6e02",
    "name": "Xend Finance",
    "symbol": "RWA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14496/thumb/WeChat_Image_20210325163206.png?1696514181"
  },
  {
    "chainId": 1,
    "address": "0xa663b02cf0a4b149d2ad41910cb81e23e1c41c32",
    "name": "Staked FRAX",
    "symbol": "SFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35383/thumb/sfrax.png?1708445569"
  },
  {
    "chainId": 1,
    "address": "0xa10bf0aba0c7953f279c4cb8192d3b5de5ea56e8",
    "name": "Tarot",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31800/thumb/TAROT.jpg?1696530615"
  },
  {
    "chainId": 1,
    "address": "0xf97e2a78f1f3d1fd438ff7cc3bb7de01e5945b83",
    "name": "holoride",
    "symbol": "RIDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21626/thumb/RIDE_Token_Symbol_MASTER.png?1696520986"
  },
  {
    "chainId": 1,
    "address": "0x9d3d07439069c9bbc8d626397cf98cb343ac0a72",
    "name": "Technology Metal Network Global",
    "symbol": "TMNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32834/thumb/TMN_Logo_dark_silver_2.PNG?1699586186"
  },
  {
    "chainId": 1,
    "address": "0xc60d6662027f5797cf873bfe80bcf048e30fc35e",
    "name": "SORA Synthetics",
    "symbol": "XST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27897/thumb/XST_200x200.png?1696526914"
  },
  {
    "chainId": 1,
    "address": "0x2f11eeee0bf21e7661a22dbbbb9068f4ad191b86",
    "name": "Backed NIU Technologies",
    "symbol": "BNIU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31869/thumb/b-NIU-200x200.png?1696530681"
  },
  {
    "chainId": 1,
    "address": "0x37613d64258c0fe09d5e53eecb091da5b8fa8707",
    "name": "Pi Protocol",
    "symbol": "PIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27476/thumb/LOGO_%281%29.png?1696526514"
  },
  {
    "chainId": 1,
    "address": "0x9fd22a17b4a96da3f83797d122172c450381fb88",
    "name": "Jefe",
    "symbol": "JEFE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25336/thumb/JEFE_200.png?1696524470"
  },
  {
    "chainId": 1,
    "address": "0xf68d4d917592f3a62417ace42592f15296cc33a0",
    "name": "COINHUB",
    "symbol": "CHB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/26818/thumb/a08jRqRh_400x400.jpg?1696525877"
  },
  {
    "chainId": 1,
    "address": "0xa48f322f8b3edff967629af79e027628b9dd1298",
    "name": "Davos Protocol",
    "symbol": "DUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28775/thumb/dusd_logo_200x200.png?1696527754"
  },
  {
    "chainId": 1,
    "address": "0x59f4f336bf3d0c49dbfba4a74ebd2a6ace40539a",
    "name": "Catcoin",
    "symbol": "CAT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25279/thumb/cat5.png?1696524419"
  },
  {
    "chainId": 1,
    "address": "0x5e7f20e72c21f6d0bf0a2814fd4164176401cf8e",
    "name": "Demeter",
    "symbol": "DEO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25582/thumb/7zyskNC.png?1696524714"
  },
  {
    "chainId": 1,
    "address": "0x20b3b07e9c0e37815e2892ab09496559f57c3603",
    "name": "USDV",
    "symbol": "USDV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31806/thumb/200x200.png?1696530621"
  },
  {
    "chainId": 1,
    "address": "0x9bd69bc59118ce0fbce9b03551a765a779bd25cf",
    "name": "zkEra Finance",
    "symbol": "ZKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33928/thumb/zke.png?1703409212"
  },
  {
    "chainId": 1,
    "address": "0x4190818ffeb4c7349fa59eeb08eb6e7ab70d9aa7",
    "name": "LFi",
    "symbol": "LFI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31897/thumb/LFi_Token.png?1696530708"
  },
  {
    "chainId": 1,
    "address": "0xc819617f360f6347d222f260e9f4987ca1d0f879",
    "name": "ICLICK Inu",
    "symbol": "ICLICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32060/thumb/Iclick.png?1696530857"
  },
  {
    "chainId": 1,
    "address": "0x7df4122d3eae29fc8fb6be58d9177e8e560be4fb",
    "name": "BlockChainCoinX",
    "symbol": "XCCX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32276/thumb/xccxx.png?1697180263"
  },
  {
    "chainId": 1,
    "address": "0x000000e29fa2bd3e5c215ffc71aa66b29c9769a2",
    "name": "Ethereum Express",
    "symbol": "ETE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31600/thumb/200X200.png?1696530416"
  },
  {
    "chainId": 1,
    "address": "0x00000000ba2ca30042001abc545871380f570b1f",
    "name": "ArithFi",
    "symbol": "ATF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32122/thumb/2.Profile.png?1696586855"
  },
  {
    "chainId": 1,
    "address": "0x96543ef8d2c75c26387c1a319ae69c0bee6f3fe7",
    "name": "Kujira",
    "symbol": "KUJI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/20685/thumb/kuji-200x200.png?1696520085"
  },
  {
    "chainId": 1,
    "address": "0x1b3c515f58857e141a966b33182f2f3feecc10e9",
    "name": "USK",
    "symbol": "USK",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27274/thumb/usk.png?1696526326"
  },
  {
    "chainId": 1,
    "address": "0x772722b55cdc2a086abd064267a17855eb15e8b3",
    "name": "MintMe com Coin",
    "symbol": "MINTME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5127/thumb/MINTME_logo.png?1696505647"
  },
  {
    "chainId": 1,
    "address": "0xb5b29320d2dde5ba5bafa1ebcd270052070483ec",
    "name": "YieldETH  Sommelier ",
    "symbol": "YIELDETH",
    "decimals": 18
  },
  {
    "chainId": 1,
    "address": "0xb399511642fe1666c6a07f83483e6e4feaed9a00",
    "name": "The Standard EURO",
    "symbol": "EUROS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32231/thumb/EUROs-coingecko.png?1696936278"
  },
  {
    "chainId": 1,
    "address": "0x580e933d90091b9ce380740e3a4a39c67eb85b4c",
    "name": "GameSwift",
    "symbol": "GSWIFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30949/thumb/GameSwift_Token.png?1696529788"
  },
  {
    "chainId": 1,
    "address": "0x4e9e4ab99cfc14b852f552f5fb3aa68617825b6c",
    "name": "Solarcoin",
    "symbol": "SLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/152/thumb/solarcoin.png?1696501524"
  },
  {
    "chainId": 1,
    "address": "0x3a856d4effa670c54585a5d523e96513e148e95d",
    "name": "TriasLab",
    "symbol": "TRIAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13963/thumb/5ELqtwp__400x400_%281%29.jpg?1696513699"
  },
  {
    "chainId": 1,
    "address": "0xb2ee0adbe0ef1281025d0676511bb1df14600f4d",
    "name": "FORE Protocol",
    "symbol": "FORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31001/thumb/EXCHANGE_LOGO_FORE.png?1696529839"
  },
  {
    "chainId": 1,
    "address": "0xa4335da338ec4c07c391fc1a9bf75f306adadc08",
    "name": "ARYZE eUSD",
    "symbol": "EUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32422/thumb/ARYZE_eUSD.png?1698118313"
  },
  {
    "chainId": 1,
    "address": "0xc03b43d492d904406db2d7d57e67c7e8234ba752",
    "name": "Wrapped USDR",
    "symbol": "WUSDR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28800/thumb/wUSDRlogo.png?1696527777"
  },
  {
    "chainId": 1,
    "address": "0xcc503242b574bc01145da7e2a743b43fb395ec91",
    "name": "ROVI Protocol",
    "symbol": "ROVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28936/thumb/rovi-coin.png?1696527910"
  },
  {
    "chainId": 1,
    "address": "0x2512c545a0117353e4e9acddff860342640d3cf6",
    "name": "WHEE",
    "symbol": "WHEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30285/thumb/whee.jpg?1696529190"
  },
  {
    "chainId": 1,
    "address": "0x1f3f677ecc58f6a1f9e2cf410df4776a8546b5de",
    "name": "VNDC",
    "symbol": "VNDC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/9670/thumb/vndc-gold-coin.png?1696509740"
  },
  {
    "chainId": 1,
    "address": "0xf05897cfe3ce9bbbfe0751cbe6b1b2c686848dcb",
    "name": "CateCoin",
    "symbol": "CATE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15364/thumb/logo.png?1696515013"
  },
  {
    "chainId": 1,
    "address": "0xb2492e97a68a6e4b9e9a11b99f6c42e5accd38c7",
    "name": "Veloce",
    "symbol": "VEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31214/thumb/VEXT_Logo.jpg?1696530041"
  },
  {
    "chainId": 1,
    "address": "0xdd1b6b259986571a85da82a84f461e1c212591c0",
    "name": "BlazeX",
    "symbol": "BLAZEX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31194/thumb/200x200.png?1696530022"
  },
  {
    "chainId": 1,
    "address": "0x73fbd93bfda83b111ddc092aa3a4ca77fd30d380",
    "name": "SophiaVerse",
    "symbol": "SOPH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30050/thumb/Icon_200x200.png?1696528972"
  },
  {
    "chainId": 1,
    "address": "0xf2b2f7b47715256ce4ea43363a867fdce9353e3a",
    "name": "Bitgert",
    "symbol": "BRISE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17388/thumb/200x200.png?1696516937"
  },
  {
    "chainId": 1,
    "address": "0x8f006d1e1d9dc6c98996f50a4c810f17a47fbf19",
    "name": "Pleasure Coin",
    "symbol": "NSFW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15315/thumb/pleasurecoin-icon.png?1696514965"
  },
  {
    "chainId": 1,
    "address": "0xd20523b39faf1d6e9023a4d6085f87b7b0de7926",
    "name": "OATH",
    "symbol": "OATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24075/thumb/OATH.png?1698051304"
  },
  {
    "chainId": 1,
    "address": "0x25931894a86d47441213199621f1f2994e1c39aa",
    "name": "ChainGPT",
    "symbol": "CGPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29306/thumb/200x200.png?1696528257"
  },
  {
    "chainId": 1,
    "address": "0x152649ea73beab28c5b49b26eb48f7ead6d4c898",
    "name": "PancakeSwap",
    "symbol": "CAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12632/thumb/pancakeswap-cake-logo_%281%29.png?1696512440"
  },
  {
    "chainId": 1,
    "address": "0x4eea762311be76f9071aa01058c047ad12a017a1",
    "name": "GBURN",
    "symbol": "GBURN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33048/thumb/Untitled_design_-_2023-11-10T174143.724.png?1700472805"
  },
  {
    "chainId": 1,
    "address": "0x39d5313c3750140e5042887413ba8aa6145a9bd2",
    "name": "Empyreal",
    "symbol": "EMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31374/thumb/emplogotransparent_%281%29.png?1696530191"
  },
  {
    "chainId": 1,
    "address": "0x6b66ccd1340c479b07b390d326eadcbb84e726ba",
    "name": "Seamless Protocol",
    "symbol": "SEAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33480/thumb/Seamless_Logo_Black_Transparent.png?1702019657"
  },
  {
    "chainId": 1,
    "address": "0x0b925ed163218f6662a35e0f0371ac234f9e9371",
    "name": "Aave v3 wstETH",
    "symbol": "AWSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32890/thumb/WSTETH.png?1699776336"
  },
  {
    "chainId": 1,
    "address": "0x3fe6a295459fae07df8a0cecc36f37160fe86aa9",
    "name": "Aave v3 LUSD",
    "symbol": "ALUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32894/thumb/LUSD.png?1699789893"
  },
  {
    "chainId": 1,
    "address": "0x582d872a1b094fc48f5de31d3b73f2d9be47def1",
    "name": "Toncoin",
    "symbol": "TON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17980/thumb/ton_symbol.png?1696517498"
  },
  {
    "chainId": 1,
    "address": "0xade6057fcafa57d6d51ffa341c64ce4814995995",
    "name": "Backed ZPR1   1 3 Month T Bill",
    "symbol": "BZPR1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32867/thumb/b-ZPR1-200x200.png?1699671603"
  },
  {
    "chainId": 1,
    "address": "0xd711d7d893de57dc13ff465763218770bd42db1d",
    "name": "ARYZE eGBP",
    "symbol": "EGBP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32989/thumb/ARYZE_eGBP.png?1700099647"
  },
  {
    "chainId": 1,
    "address": "0xb55ee890426341fe45ee6dc788d2d93d25b59063",
    "name": "Love io",
    "symbol": "LOVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33011/thumb/IMG_7458.png?1700124277"
  },
  {
    "chainId": 1,
    "address": "0x31e4efe290973ebe91b3a875a7994f650942d28f",
    "name": "Shrapnel",
    "symbol": "SHRAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32793/thumb/shrapnel.png?1699441661"
  },
  {
    "chainId": 1,
    "address": "0x31a2e08f4232329e4eddb025c0275f43c9cd56d7",
    "name": "LUSD",
    "symbol": "LUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33592/thumb/LUSD.jpg?1702471959"
  },
  {
    "chainId": 1,
    "address": "0x735fa792e731a2e8f83f32eb539841b7b72e6d8f",
    "name": "ARYZE eEUR",
    "symbol": "EEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32421/thumb/ARYZE_eEUR.png?1698118176"
  },
  {
    "chainId": 1,
    "address": "0xe939f011a3d8fc0aa874c97e8156053a903d7176",
    "name": "DOLZ io",
    "symbol": "DOLZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33605/thumb/dolz_200.png?1702510960"
  },
  {
    "chainId": 1,
    "address": "0x75d86078625d1e2f612de2627d34c7bc411c18b8",
    "name": "AGII",
    "symbol": "AGII",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32371/thumb/AGII.png?1699534856"
  },
  {
    "chainId": 1,
    "address": "0xe6828d65bf5023ae1851d90d8783cc821ba7eee1",
    "name": "ApeBond",
    "symbol": "ABOND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33814/thumb/l-_N-wAO_400x400.jpg?1703020501"
  },
  {
    "chainId": 1,
    "address": "0xa2c2c937333165d4c5f2dc5f31a43e1239fecfeb",
    "name": "Hera Finance",
    "symbol": "HERA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24751/thumb/logo-icon-200x.png?1696523913"
  },
  {
    "chainId": 1,
    "address": "0x1f7e5118521b550bb1a9b435727c003eb033fc51",
    "name": "Angola",
    "symbol": "AGLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27547/thumb/8SGgovDI_400x400.png?1696526584"
  },
  {
    "chainId": 1,
    "address": "0xbf27b3163c25113be5439d56f8acf2209ef3e5bd",
    "name": "Hotel of Secrets",
    "symbol": "HOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31567/thumb/Hotel_of_Secrets_icon.png?1696530379"
  },
  {
    "chainId": 1,
    "address": "0x8b9f7e9586633ca4abffb1f746da8daed7849cb2",
    "name": "Cremation Coin",
    "symbol": "CREMAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29793/thumb/photo_2023-02-07_04-43-14.jpg?1696528723"
  },
  {
    "chainId": 1,
    "address": "0x388d819724dd6d71760a38f00dc01d310d879771",
    "name": "JustMoney",
    "symbol": "JM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25450/thumb/jm.png?1696524581"
  },
  {
    "chainId": 1,
    "address": "0x4b19c70da4c6fa4baa0660825e889d2f7eabc279",
    "name": "Gamium",
    "symbol": "GMM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22749/thumb/14304.png?1696522053"
  },
  {
    "chainId": 1,
    "address": "0xdad33e12e61dc2f2692f2c12e6303b5ade7277ba",
    "name": "Brewlabs",
    "symbol": "BREWLABS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21928/thumb/7xgmOCBW_400x400.jpg?1696521278"
  },
  {
    "chainId": 1,
    "address": "0x87da7bbd398d6610ded42617e34069d0083ff8ab",
    "name": "Mind Connect",
    "symbol": "MIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29749/thumb/our_logo.png?1696528681"
  },
  {
    "chainId": 1,
    "address": "0x186d0ba3dfc3386c464eecd96a61fbb1e2da00bf",
    "name": "Trava Finance",
    "symbol": "TRAVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17553/thumb/TRAVA_OFFICIAL_LOGO.png?1696517089"
  },
  {
    "chainId": 1,
    "address": "0x333000333b26ee30214b4af6419d9ab07a450400",
    "name": "MELD",
    "symbol": "MELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30170/thumb/Twitter.jpg?1696529090"
  },
  {
    "chainId": 1,
    "address": "0xa0b73e1ff0b80914ab6fe0444e65848c4c34450b",
    "name": "Cronos",
    "symbol": "CRO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7310/thumb/cro_token_logo.png?1696507599"
  },
  {
    "chainId": 1,
    "address": "0x7cfea0dd176651e7b5a1ced9c4faf8ee295315fd",
    "name": "Prime Numbers Ecosystem",
    "symbol": "PRNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22669/thumb/primenumers_favicon_01.png?1696521981"
  },
  {
    "chainId": 1,
    "address": "0xf33893de6eb6ae9a67442e066ae9abd228f5290c",
    "name": "GroveCoin",
    "symbol": "GRV",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25549/thumb/200x200.png?1696524682"
  },
  {
    "chainId": 1,
    "address": "0x5b8650cd999b23cf39ab12e3213fbc8709c7f5cb",
    "name": "MaziMatic",
    "symbol": "MAZI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30490/thumb/4de906d9d4104a16b4334d8ec8f75284.png?1696529377"
  },
  {
    "chainId": 1,
    "address": "0x9ed7e4b1bff939ad473da5e7a218c771d1569456",
    "name": "Reunit Wallet",
    "symbol": "REUNI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29492/thumb/reunit_200.png?1696528436"
  },
  {
    "chainId": 1,
    "address": "0x91a69021b0baef3445e51726458a0ce601471846",
    "name": "Ethos Reserve Note",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29744/thumb/ERN200x200.png?1696528676"
  },
  {
    "chainId": 1,
    "address": "0x9776191f4ebbba7f358c1663bf82c0a0906c77fa",
    "name": "Phoenix Chain",
    "symbol": "PHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29480/thumb/PHOENlX_%281%29_%281%29_%281%29.png?1696528424"
  },
  {
    "chainId": 1,
    "address": "0x9d1089802ee608ba84c5c98211afe5f37f96b36c",
    "name": "Fluid USDC",
    "symbol": "FUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28471/thumb/fUSDC-200x200.png?1696527465"
  },
  {
    "chainId": 1,
    "address": "0xa75e7928d3de682e3f44da60c26f33117c4e6c5c",
    "name": "Propel",
    "symbol": "PEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21290/thumb/img_circle_1.png?1696520660"
  },
  {
    "chainId": 1,
    "address": "0x15a3081b541e8dad25c4a5e0c4c4b4e8d105b2e8",
    "name": "Pepe Original Version",
    "symbol": "POV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30121/thumb/our_logo.png?1696529043"
  },
  {
    "chainId": 1,
    "address": "0x1bd55090b8878630e57fae6ebd11fd61d16dfc9f",
    "name": "ArchLoot",
    "symbol": "ALT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28919/thumb/7.png?1696527894"
  },
  {
    "chainId": 1,
    "address": "0x0ae38f7e10a43b5b2fb064b42a2f4514cba909ef",
    "name": "unshETH Ether",
    "symbol": "UNSHETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30365/thumb/ush.png?1696529262"
  },
  {
    "chainId": 1,
    "address": "0x490e3f4af13e1616ec97a8c6600c1061a8d0253e",
    "name": "Terran Coin",
    "symbol": "TRR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15351/thumb/TERRAN-500px.png?1696515000"
  },
  {
    "chainId": 1,
    "address": "0x849a226f327b89e3133d9930d927f9eb9346f8c9",
    "name": "Crypto Global United",
    "symbol": "CGU",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/20751/thumb/2022_CGU-MG_RGB.png?1696520148"
  },
  {
    "chainId": 1,
    "address": "0x3c917054e03485808137eb306eafa8da0ab695cd",
    "name": "Orbcity",
    "symbol": "ORB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24332/thumb/OrbCity_enlarged.jpg?1696523517"
  },
  {
    "chainId": 1,
    "address": "0xb8c77482e45f1f44de1745f52c74426c631bdd52",
    "name": "BNB",
    "symbol": "BNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/825/thumb/bnb-icon2_2x.png?1696501970"
  },
  {
    "chainId": 1,
    "address": "0xa670d7237398238de01267472c6f13e5b8010fd1",
    "name": "Sommelier",
    "symbol": "SOMM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/23308/thumb/somm_new.png?1696522525"
  },
  {
    "chainId": 1,
    "address": "0xffa188493c15dfaf2c206c97d8633377847b6a52",
    "name": "Wefi Finance",
    "symbol": "WEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30540/thumb/wefi.png?1696529412"
  },
  {
    "chainId": 1,
    "address": "0xd8fa690304d2b2824d918c0c7376e2823704557a",
    "name": "SquidGrow",
    "symbol": "SQUIDGROW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/26573/thumb/Squidgrow-Logo.png?1696525645"
  },
  {
    "chainId": 1,
    "address": "0x395e925834996e558bdec77cd648435d620afb5b",
    "name": "ThreeFold",
    "symbol": "TFT",
    "decimals": 7,
    "logoURI": "https://assets.coingecko.com/coins/images/6704/thumb/black_logo_on_white_background_%282%29.jpg?1696507044"
  },
  {
    "chainId": 1,
    "address": "0x8db4beaccd1698892821a9a0dc367792c0cb9940",
    "name": "Ignore Fud",
    "symbol": "4TOKEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29626/thumb/200x200.png?1696528562"
  },
  {
    "chainId": 1,
    "address": "0x26aad156ba8efa501b32b42ffcdc8413f90e9c99",
    "name": "Open Campus",
    "symbol": "EDU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29948/thumb/EDU_Logo.png?1696528874"
  },
  {
    "chainId": 1,
    "address": "0xf823fd9ac0abb8d779076a41daa049844299f6ed",
    "name": "Squid Game",
    "symbol": "SQUID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20506/thumb/squid.png?1696519912"
  },
  {
    "chainId": 1,
    "address": "0x837d904a3799c0769079be9ecbaddf1abd4ccd6e",
    "name": "Tarot V1",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17881/thumb/tarot-200px.png?1696517403"
  },
  {
    "chainId": 1,
    "address": "0xdcfa8c46ec015d4bf13d6be492cb7a8eb4580899",
    "name": "impactMarket",
    "symbol": "PACT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21907/thumb/PACT_Token_Ticker_Blue_2x.png?1696521258"
  },
  {
    "chainId": 1,
    "address": "0x4184aa04215e5d716dd4c213fed519acadc68f92",
    "name": "ONUS",
    "symbol": "ONUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24599/thumb/ONUS_200.png?1696523773"
  },
  {
    "chainId": 1,
    "address": "0xb012be90957d70d9a070918027655f998c123a88",
    "name": "Hermes DAO",
    "symbol": "HMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29283/thumb/Hermes_DAO.png?1696528235"
  },
  {
    "chainId": 1,
    "address": "0x0b7f0e51cd1739d6c96982d55ad8fa634dd43a9c",
    "name": "Dream Machine Token",
    "symbol": "DMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30505/thumb/dmt.png?1696529391"
  },
  {
    "chainId": 1,
    "address": "0xcdb37a4fbc2da5b78aa4e41a432792f9533e85cc",
    "name": "CheckDot",
    "symbol": "CDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20370/thumb/token-200x200_%281%29.png?1696519781"
  },
  {
    "chainId": 1,
    "address": "0x0a5e677a6a24b2f1a2bf4f3bffc443231d2fdec8",
    "name": "dForce USD",
    "symbol": "USX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17422/thumb/usx_32.png?1696516969"
  },
  {
    "chainId": 1,
    "address": "0x4c584cd339bdde73b7f5210486dd8bbeee3fde6d",
    "name": "ShibElon",
    "symbol": "SHIBELON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/20434/thumb/shibelon.png?1696519841"
  },
  {
    "chainId": 1,
    "address": "0xa64d5d1eb67748226d84812b45711453f1118c32",
    "name": "Cheems Inu  NEW ",
    "symbol": "CINU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29393/thumb/cinu.png?1696528343"
  },
  {
    "chainId": 1,
    "address": "0x075067a4b236f68f123fc0553f4560311937491d",
    "name": "CATpay",
    "symbol": "CATPAY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24731/thumb/catpay.png?1696523894"
  },
  {
    "chainId": 1,
    "address": "0xf3e07812ebc8604fddb0aa35ff79a03f48f48948",
    "name": "JournArt",
    "symbol": "JART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29412/thumb/JournArt%28200%29.png?1696528361"
  },
  {
    "chainId": 1,
    "address": "0xbccd27062ae1a2bea5731c904b96edfb163aba21",
    "name": "Dogcoin",
    "symbol": "DOGS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29065/thumb/1024_%281%29.png?1696528032"
  },
  {
    "chainId": 1,
    "address": "0x9f90b457dea25ef802e38d470dda7343691d8fe1",
    "name": "Crosschain IOTX",
    "symbol": "CIOTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18331/thumb/iotx.PNG?1696517822"
  },
  {
    "chainId": 1,
    "address": "0xe28027c99c7746ffb56b0113e5d9708ac86fae8f",
    "name": "KING Coin",
    "symbol": "KING",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30183/thumb/king.png?1696529102"
  },
  {
    "chainId": 1,
    "address": "0xbe03e60757f21f4b6fc8f16676ad9d5b1002e512",
    "name": "Raini Studios Token",
    "symbol": "RST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32077/thumb/rst_200x200.png?1696530874"
  },
  {
    "chainId": 1,
    "address": "0x5bdc32663ec75e85ff4abc2cae7ae8b606a2cfca",
    "name": "Cookies Protocol",
    "symbol": "CP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28432/thumb/IMG_20221207_200805_897.jpg?1696527429"
  },
  {
    "chainId": 1,
    "address": "0x25ec98773d7b4ced4cafab96a2a1c0945f145e10",
    "name": "Staked USDT",
    "symbol": "STUSDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31135/thumb/2veJywVy_400x400.jpg?1696529964"
  },
  {
    "chainId": 1,
    "address": "0x188fb5f5ae5bbe4154d5778f2bbb2fb985c94d25",
    "name": "OpenBlox",
    "symbol": "OBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26150/thumb/OBX_token-black_background_preview.png?1696525239"
  },
  {
    "chainId": 1,
    "address": "0x263b6b028f3e4ed8c4329eb2b5f409ee38d97296",
    "name": "World Mobile Token",
    "symbol": "WMT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17333/thumb/Colored_Token.png?1696516885"
  },
  {
    "chainId": 1,
    "address": "0xc0b314a8c08637685fc3dafc477b92028c540cfb",
    "name": "Wombat Exchange",
    "symbol": "WOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26946/thumb/Wombat_Token.png?1696526001"
  },
  {
    "chainId": 1,
    "address": "0x5fab9761d60419c9eeebe3915a8fa1ed7e8d2e1b",
    "name": "DIMO",
    "symbol": "DIMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28383/thumb/Token_Logo.png?1696527383"
  },
  {
    "chainId": 1,
    "address": "0x672f4fa517894496b8a958b4b3fca068ce513a39",
    "name": "DexCheck AI",
    "symbol": "DCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26412/thumb/DexCheck_logo_%282%29.png?1696525488"
  },
  {
    "chainId": 1,
    "address": "0x2dff88a56767223a5529ea5960da7a3f5f766406",
    "name": "SPACE ID",
    "symbol": "ID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29468/thumb/sid_token_logo_%28green2%29.png?1696528413"
  },
  {
    "chainId": 1,
    "address": "0x6731827cb6879a2091ce3ab3423f7bf20539b579",
    "name": "Empower",
    "symbol": "MPWR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25456/thumb/MPWR_200x200.png?1696524587"
  },
  {
    "chainId": 1,
    "address": "0xb50721bcf8d664c30412cfbc6cf7a15145234ad1",
    "name": "Arbitrum",
    "symbol": "ARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16547/thumb/photo_2023-03-29_21.47.00.jpeg?1696516109"
  },
  {
    "chainId": 1,
    "address": "0x77f2be773ca0887ba2b3ef8344c8cf13c98d8ca7",
    "name": "FloraChain",
    "symbol": "FYT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29078/thumb/flora.png?1696528044"
  },
  {
    "chainId": 1,
    "address": "0xa89bf95c5f15a847c8eb8d348cd7fed719ad7d80",
    "name": "Chat AI",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29441/thumb/200.png?1696528389"
  },
  {
    "chainId": 1,
    "address": "0x1c98b54d673c026c8286badca3e840aaf72931a3",
    "name": "Stackswap",
    "symbol": "STSW",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29727/thumb/22.png?1696528657"
  },
  {
    "chainId": 1,
    "address": "0x1151cb3d861920e07a38e03eead12c32178567f6",
    "name": "Bonk",
    "symbol": "BONK",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/28600/thumb/bonk.jpg?1696527587"
  },
  {
    "chainId": 1,
    "address": "0x510975eda48a97e0ca228dd04d1217292487bea6",
    "name": "PROJECT XENO",
    "symbol": "GXE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29000/thumb/gxe.png?1696527972"
  },
  {
    "chainId": 1,
    "address": "0xb2cabf797bc907b049e4ccb5b84d13be3a8cfc21",
    "name": "Arable Protocol",
    "symbol": "ACRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23659/thumb/acre_token-02.png?1696522862"
  },
  {
    "chainId": 1,
    "address": "0x4e15361fd6b4bb609fa63c81a2be19d873717870",
    "name": "Wrapped Fantom",
    "symbol": "WFTM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16036/thumb/Fantom.png?1696515646"
  },
  {
    "chainId": 1,
    "address": "0x56c03b8c4fa80ba37f5a7b60caaaef749bb5b220",
    "name": "CANTO",
    "symbol": "CANTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26959/thumb/canto-network.png?1696526014"
  },
  {
    "chainId": 1,
    "address": "0xc9c4fd7579133701fa2769b6955e7e56bb386db1",
    "name": "Bridge Oracle",
    "symbol": "BRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12512/thumb/brg.png?1696512327"
  },
  {
    "chainId": 1,
    "address": "0xd1420af453fd7bf940573431d416cace7ff8280c",
    "name": "Answer Governance",
    "symbol": "AGOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14383/thumb/AgoV_Logo1.png?1696514074"
  },
  {
    "chainId": 1,
    "address": "0xb4a3b0faf0ab53df58001804dda5bfc6a3d59008",
    "name": "Sperax",
    "symbol": "SPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12232/thumb/sperax_logo.jpg?1696512065"
  },
  {
    "chainId": 1,
    "address": "0x969786c4a8884013d1c9ff18dcca2aedbbbfaa8f",
    "name": "Gamerse",
    "symbol": "LFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19582/thumb/gamerse.PNG?1696519013"
  },
  {
    "chainId": 1,
    "address": "0xb9d27bc093ed0a3b7c18366266704cfe5e7af77b",
    "name": "Carbify",
    "symbol": "CBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29892/thumb/Carbify_logo.png?1696528816"
  },
  {
    "chainId": 1,
    "address": "0x977b6fc5de62598b08c85ac8cf2b745874e8b78c",
    "name": "Aave v3 cbETH",
    "symbol": "ACBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32892/thumb/cbETH.png?1699789253"
  },
  {
    "chainId": 1,
    "address": "0x92df60c51c710a1b1c20e42d85e221f3a1bfc7f2",
    "name": "ApeSwap",
    "symbol": "BANANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14870/thumb/banana.png?1696514534"
  },
  {
    "chainId": 1,
    "address": "0x2aeabde1ab736c59e9a19bed67681869eef39526",
    "name": "DOVU",
    "symbol": "DOVU",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31930/thumb/Dovu_Icon_Black_%281%29.png?1696530738"
  },
  {
    "chainId": 1,
    "address": "0xff9c1f21c621696c4f91cf781ec31bd913ee2c26",
    "name": " com  Ordinals ",
    "symbol": "COM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33700/thumb/com200.jpg?1702823408"
  },
  {
    "chainId": 1,
    "address": "0xe2dc070524a6e305ddb64d8513dc444b6a1ec845",
    "name": "Nash",
    "symbol": "NEX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/3246/thumb/Nash_M.png?1696503962"
  },
  {
    "chainId": 1,
    "address": "0x0f3adc247e91c3c50bc08721355a41037e89bc20",
    "name": "Anchor Protocol",
    "symbol": "ANC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14420/thumb/anchor_protocol_logo.jpg?1696514111"
  },
  {
    "chainId": 1,
    "address": "0x965697b4ef02f0de01384d0d4f9f782b1670c163",
    "name": "Oxygen",
    "symbol": "OXY",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13509/thumb/8DjBZ79V_400x400.jpg?1696513271"
  },
  {
    "chainId": 1,
    "address": "0x499568c250ab2a42292261d6121525d70691894b",
    "name": "KROWN",
    "symbol": "KRW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16530/thumb/KRW_token_logo_200x200.png?1696516093"
  },
  {
    "chainId": 1,
    "address": "0x06ddb3a8bc0abc14f85e974cf1a93a6f8d4909d9",
    "name": "8Pay",
    "symbol": "8PAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14667/thumb/8pay.jpeg?1696514342"
  },
  {
    "chainId": 1,
    "address": "0x6dc02164d75651758ac74435806093e421b64605",
    "name": "XAYA",
    "symbol": "WCHI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2091/thumb/xaya200x200.png?1696503058"
  },
  {
    "chainId": 1,
    "address": "0xfb62ae373aca027177d1c18ee0862817f9080d08",
    "name": "My DeFi Pet",
    "symbol": "DPET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15321/thumb/mydefi.PNG?1696514970"
  },
  {
    "chainId": 1,
    "address": "0x92a42db88ed0f02c71d439e55962ca7cab0168b5",
    "name": "TRDGtoken",
    "symbol": "TRDG",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14371/thumb/200x200.png?1696514063"
  },
  {
    "chainId": 1,
    "address": "0x3079f61704e9efa2bcf1db412f735d8d4cfa26f4",
    "name": "HappyFans",
    "symbol": "HAPPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19027/thumb/O1pUoR8G_400x400.jpeg?1696518478"
  },
  {
    "chainId": 1,
    "address": "0xee1cea7665ba7aa97e982edeaecb26b59a04d035",
    "name": "ORAO Network",
    "symbol": "ORAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15189/thumb/orao.PNG?1696514846"
  },
  {
    "chainId": 1,
    "address": "0x7b4328c127b85369d9f82ca0503b000d09cf9180",
    "name": "Dogechain",
    "symbol": "DC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26828/thumb/dogechain.jpeg?1696525887"
  },
  {
    "chainId": 1,
    "address": "0x05fb86775fd5c16290f1e838f5caaa7342bd9a63",
    "name": "Hacken",
    "symbol": "HAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11081/thumb/hacken-symbol-with-bg.png?1696511022"
  },
  {
    "chainId": 1,
    "address": "0xa771b49064da011df051052848477f18dba1d2ac",
    "name": "Handshake",
    "symbol": "HNS",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/10562/thumb/circle-handshakeLogo.png?1696510545"
  },
  {
    "chainId": 1,
    "address": "0x45fdb1b92a649fb6a64ef1511d3ba5bf60044838",
    "name": "SpiceUSD",
    "symbol": "USDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25697/thumb/USDS.png?1696524824"
  },
  {
    "chainId": 1,
    "address": "0x8bcd06492416a749c9369009b3429861b7f27f6e",
    "name": "BlackHat Coin",
    "symbol": "BLKC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/15987/thumb/logo_light.png?1696515600"
  },
  {
    "chainId": 1,
    "address": "0xae12c5930881c53715b369cec7606b70d8eb229f",
    "name": "Coin98",
    "symbol": "C98",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17117/thumb/logo.png?1696516677"
  },
  {
    "chainId": 1,
    "address": "0x396ec402b42066864c406d1ac3bc86b575003ed8",
    "name": "Buying com",
    "symbol": "BUY",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/5290/thumb/buying_200x200.png?1696505788"
  },
  {
    "chainId": 1,
    "address": "0x180dae91d6d56235453a892d2e56a3e40ba81df8",
    "name": "DOJO",
    "symbol": "DOJO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16396/thumb/074606deafab3872.png?1696515994"
  },
  {
    "chainId": 1,
    "address": "0x8287c7b963b405b7b8d467db9d79eec40625b13a",
    "name": "Swingby",
    "symbol": "SWINGBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11861/thumb/swingby.png?1696511731"
  },
  {
    "chainId": 1,
    "address": "0x1d7ca62f6af49ec66f6680b8606e634e55ef22c1",
    "name": "Starter xyz",
    "symbol": "START",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14301/thumb/logo_poly_sym.png?1696513997"
  },
  {
    "chainId": 1,
    "address": "0x6c7b97c7e09e790d161769a52f155125fac6d5a1",
    "name": "Polylauncher",
    "symbol": "ANGEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17739/thumb/Polylauncher_-_200_x_200.png?1696517266"
  },
  {
    "chainId": 1,
    "address": "0x31f69de127c8a0ff10819c0955490a4ae46fcc2a",
    "name": "Obyte",
    "symbol": "GBYTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/561/thumb/byteball.png?1696501768"
  },
  {
    "chainId": 1,
    "address": "0x8848812bd31aeee33313c10a840ffc3169078c5b",
    "name": "CrossFi",
    "symbol": "CRFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15162/thumb/11901619752967_.pic_hd.png?1696514817"
  },
  {
    "chainId": 1,
    "address": "0x667fd83e24ca1d935d36717d305d54fa0cac991c",
    "name": "Collector Coin",
    "symbol": "AGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16918/thumb/AGS.png?1696516489"
  },
  {
    "chainId": 1,
    "address": "0x9393fdc77090f31c7db989390d43f454b1a6e7f3",
    "name": "Dark Energy Crystals",
    "symbol": "DEC",
    "decimals": 3,
    "logoURI": "https://assets.coingecko.com/coins/images/12923/thumb/DEC_token.png?1696512711"
  },
  {
    "chainId": 1,
    "address": "0x8861cff2366c1128fd699b68304ad99a0764ef9a",
    "name": "Cyclone Protocol",
    "symbol": "CYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14065/thumb/b3_DFjFp_400x400.jpg?1696513790"
  },
  {
    "chainId": 1,
    "address": "0xdb726152680ece3c9291f1016f1d36f3995f6941",
    "name": "Media Network",
    "symbol": "MEDIA",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/15142/thumb/media50x50.png?1696514798"
  },
  {
    "chainId": 1,
    "address": "0x34965f73cfa05bf8d8af37cb4af64fa950605ea8",
    "name": "CoinWind",
    "symbol": "COW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15766/thumb/logo.c3d2c062.png?1696515390"
  },
  {
    "chainId": 1,
    "address": "0x9ac07635ddbde5db18648c360defb00f5f22537e",
    "name": "Museum of Crypto Art",
    "symbol": "MOCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15829/thumb/photo_2021-06-04_09.36.16.jpeg?1696515447"
  },
  {
    "chainId": 1,
    "address": "0x7645ddfeeceda57e41f92679c4acd83c56a81d14",
    "name": "Flux Protocol",
    "symbol": "FLUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15002/thumb/logo.dabc411c.png?1696514665"
  },
  {
    "chainId": 1,
    "address": "0xf99d58e463a2e07e5692127302c20a191861b4d6",
    "name": "Anyswap",
    "symbol": "ANY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12242/thumb/anyswap.jpg?1696512074"
  },
  {
    "chainId": 1,
    "address": "0x9fda7ceec4c18008096c2fe2b85f05dc300f94d0",
    "name": "Gaj Finance",
    "symbol": "GAJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15257/thumb/logo200x200.png?1696514910"
  },
  {
    "chainId": 1,
    "address": "0xe0b0c16038845bed3fcf70304d3e167df81ce225",
    "name": "CrossSwap",
    "symbol": "CSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18002/thumb/cross.png?1696517518"
  },
  {
    "chainId": 1,
    "address": "0x939b462ee3311f8926c047d2b576c389092b1649",
    "name": "LiquidApps",
    "symbol": "DAPP",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/8116/thumb/dapp-logo.jpg?1696508334"
  },
  {
    "chainId": 1,
    "address": "0xef3a930e1ffffacd2fc13434ac81bd278b0ecc8d",
    "name": "Stafi",
    "symbol": "FIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12423/thumb/FIS.png?1696512244"
  },
  {
    "chainId": 1,
    "address": "0x154e35c2b0024b3e079c5c5e4fc31c979c189ccb",
    "name": "Raid",
    "symbol": "RAID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18133/thumb/raid_200_oswlvz.png?1696517636"
  },
  {
    "chainId": 1,
    "address": "0x9d79d5b61de59d882ce90125b18f74af650acb93",
    "name": "Neutrino System Base",
    "symbol": "NSBT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/13268/thumb/NSBT.png?1712107101"
  },
  {
    "chainId": 1,
    "address": "0x3405a1bd46b85c5c029483fbecf2f3e611026e45",
    "name": "WOWswap",
    "symbol": "WOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14101/thumb/Group-423.png?1696513823"
  },
  {
    "chainId": 1,
    "address": "0x12bb890508c125661e03b09ec06e404bc9289040",
    "name": "Radio Caca",
    "symbol": "RACA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17841/thumb/ez44_BSs_400x400.jpg?1696517365"
  },
  {
    "chainId": 1,
    "address": "0xca1262e77fb25c0a4112cfc9bad3ff54f617f2e6",
    "name": "Jax Network",
    "symbol": "WJXN",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/18737/thumb/photo.jpg?1696518203"
  },
  {
    "chainId": 1,
    "address": "0x968f6f898a6df937fc1859b323ac2f14643e3fed",
    "name": "Newscrypto Coin",
    "symbol": "NWC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9805/thumb/Tu1_NI3s_%281%29.png?1696509862"
  },
  {
    "chainId": 1,
    "address": "0xfbc4f3f645c4003a2e4f4e9b51077d2daa9a9341",
    "name": "Zedxion",
    "symbol": "ZEDXION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18841/thumb/ZEDXION.png?1696518302"
  },
  {
    "chainId": 1,
    "address": "0xdb85f6685950e285b1e611037bebe5b34e2b7d78",
    "name": "Zano",
    "symbol": "ZANO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8370/thumb/zano.png?1696508563"
  },
  {
    "chainId": 1,
    "address": "0xf6719e1a8fcbb1b9c290019e37e004966a8916c9",
    "name": "Polygen",
    "symbol": "PGEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21476/thumb/polygen-logo_1.jpeg?1696520837"
  },
  {
    "chainId": 1,
    "address": "0x65e6b60ea01668634d68d0513fe814679f925bad",
    "name": "PixelVerse",
    "symbol": "PIXEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19934/thumb/pixelverse.PNG?1696519352"
  },
  {
    "chainId": 1,
    "address": "0x081131434f93063751813c619ecca9c4dc7862a3",
    "name": "Mines of Dalarnia",
    "symbol": "DAR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/19837/thumb/dar.png?1696519259"
  },
  {
    "chainId": 1,
    "address": "0xac57de9c1a09fec648e93eb98875b212db0d460b",
    "name": "Baby Doge Coin",
    "symbol": "BABYDOGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/16125/thumb/babydoge.jpg?1696515731"
  },
  {
    "chainId": 1,
    "address": "0x55af5865807b196bd0197e0902746f31fbccfa58",
    "name": "Spookyswap",
    "symbol": "BOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15223/thumb/logo_200x200.png?1696514878"
  },
  {
    "chainId": 1,
    "address": "0x9fb83c0635de2e815fd1c21b3a292277540c2e8d",
    "name": "RealFevr",
    "symbol": "FEVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17136/thumb/Fevr-Token.png?1696516695"
  },
  {
    "chainId": 1,
    "address": "0x0343131c0257ac21ea5a8dc83841f071efd9285c",
    "name": "Zenith Chain",
    "symbol": "ZENITH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21266/thumb/Jqtp-OVG_400x400.png?1696520637"
  },
  {
    "chainId": 1,
    "address": "0xd8c1232fcd219286e341271385bd70601503b3d7",
    "name": "Dogira",
    "symbol": "DOGIRA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14634/thumb/IQgaRw0.png?1696514312"
  },
  {
    "chainId": 1,
    "address": "0xb6dd77fd132dcaa10f1858734e838a0fa7431580",
    "name": "Filipcoin",
    "symbol": "FCP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21354/thumb/filip.PNG?1696520719"
  },
  {
    "chainId": 1,
    "address": "0x93e32efafd24973d45f363a76d73ccb9edf59986",
    "name": "Bitlocus",
    "symbol": "BTL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/20913/thumb/btl.png?1696520304"
  },
  {
    "chainId": 1,
    "address": "0x42baf1f659d765c65ade5bb7e08eb2c680360d9d",
    "name": "Cornucopias",
    "symbol": "COPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21452/thumb/g56WwJDA_400x400.jpg?1696520814"
  },
  {
    "chainId": 1,
    "address": "0xe1590a6fa0cff9c960181cb77d8a873601772f64",
    "name": "WallStreetBets DApp",
    "symbol": "WSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15093/thumb/Pe1mrDu.png?1696514751"
  },
  {
    "chainId": 1,
    "address": "0xdf09a216fac5adc3e640db418c0b956076509503",
    "name": "Poken",
    "symbol": "PKN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21574/thumb/G10VNBAR_400x400.jpg?1696520934"
  },
  {
    "chainId": 1,
    "address": "0x45c2f8c9b4c0bdc76200448cc26c48ab6ffef83f",
    "name": "Domi",
    "symbol": "DOMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21633/thumb/Transparent_Circle_Logo_2.png?1696520993"
  },
  {
    "chainId": 1,
    "address": "0xa87135285ae208e22068acdbff64b11ec73eaa5a",
    "name": "LunarCrush",
    "symbol": "LUNR",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/19256/thumb/LunarCrush-Earn-Logo.png?1708183703"
  },
  {
    "chainId": 1,
    "address": "0x965b85d4674f64422c4898c8f8083187f02b32c0",
    "name": "Filecoin Standard Full Hashrate",
    "symbol": "SFIL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/21669/thumb/_70BfuBY_400x400.jpg?1696521026"
  },
  {
    "chainId": 1,
    "address": "0xa47c8bf37f92abed4a126bda807a7b7498661acd",
    "name": "Wrapped USTC",
    "symbol": "USTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15462/thumb/ust.png?1696515108"
  },
  {
    "chainId": 1,
    "address": "0xaad483f97f13c6a20b9d05d07c397ce85c42c393",
    "name": "Woonkly Power",
    "symbol": "WOOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14494/thumb/Icono_Infinito_Circular_Blanco_Fondo_Azul_2x.png?1696514179"
  },
  {
    "chainId": 1,
    "address": "0xbd100d061e120b2c67a24453cf6368e63f1be056",
    "name": "iDypius",
    "symbol": "IDYP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21976/thumb/iDYP-200x200px.png?1696521324"
  },
  {
    "chainId": 1,
    "address": "0x77a1f4e744d810239f465043e35d067ca33de259",
    "name": "Voice Street",
    "symbol": "VST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23147/thumb/LbMUkaD9_400x400.jpg?1696522439"
  },
  {
    "chainId": 1,
    "address": "0x7118057ff0f4fd0994fb9d2d94de8231d5cca79e",
    "name": "ReSource Protocol",
    "symbol": "SOURCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20740/thumb/source.png?1696520138"
  },
  {
    "chainId": 1,
    "address": "0x9ab70e92319f0b9127df78868fd3655fb9f1e322",
    "name": "WeWay",
    "symbol": "WWY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22418/thumb/wwy.png?1696521760"
  },
  {
    "chainId": 1,
    "address": "0x25b24b3c47918b7962b3e49c4f468367f73cc0e0",
    "name": "AXL INU",
    "symbol": "AXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22191/thumb/Original_Logo-01.png?1696521535"
  },
  {
    "chainId": 1,
    "address": "0xa693b19d2931d498c5b318df961919bb4aee87a5",
    "name": "TerraUSD  Wormhole ",
    "symbol": "UST",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21150/thumb/UST_wh_small.png?1696520528"
  },
  {
    "chainId": 1,
    "address": "0x7c9f4c87d911613fe9ca58b579f737911aad2d43",
    "name": "MATIC  Wormhole ",
    "symbol": "MATICPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22947/thumb/MATICpo_wh_small.png?1696522243"
  },
  {
    "chainId": 1,
    "address": "0x845e2e8b42dced7dedcdba9bde32c9e338224f97",
    "name": "Satozhi",
    "symbol": "SATOZ",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14786/thumb/0GeSrIaQ_400x400.jpg?1696514455"
  },
  {
    "chainId": 1,
    "address": "0x85f138bfee4ef8e540890cfb48f620571d67eda3",
    "name": "Avalanche  Wormhole ",
    "symbol": "AVAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22943/thumb/AVAX_wh_small.png?1696522240"
  },
  {
    "chainId": 1,
    "address": "0x566957ef80f9fd5526cd2bef8be67035c0b81130",
    "name": "Bridged USD Coin  Wormhole POS ",
    "symbol": "USDCPO",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/22945/thumb/USDCpo_wh_small.png?1696522242"
  },
  {
    "chainId": 1,
    "address": "0x2f75113b13d136f861d212fa9b572f2c79ac81c4",
    "name": "Ekta",
    "symbol": "EKTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22350/thumb/token.jpg?1696521693"
  },
  {
    "chainId": 1,
    "address": "0xc669928185dbce49d2230cc9b0979be6dc797957",
    "name": "BitTorrent",
    "symbol": "BTT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22457/thumb/btt_logo.png?1696521780"
  },
  {
    "chainId": 1,
    "address": "0xcc8fa225d80b9c7d42f96e9570156c65d6caaa25",
    "name": "Smooth Love Potion",
    "symbol": "SLP",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/10366/thumb/SLP.png?1696510368"
  },
  {
    "chainId": 1,
    "address": "0xc4bb7277a74678f053259cb1f96140347efbfd46",
    "name": "Coin of the champions",
    "symbol": "COC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18478/thumb/COC-Yellow-Transparent-1.png?1696517963"
  },
  {
    "chainId": 1,
    "address": "0x9b17baadf0f21f03e35249e0e59723f34994f806",
    "name": "NFTmall",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16217/thumb/Icon-1000x1000.png?1696515817"
  },
  {
    "chainId": 1,
    "address": "0xee9801669c6138e84bd50deb500827b776777d28",
    "name": "O3 Swap",
    "symbol": "O3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15460/thumb/o3.png?1696515107"
  },
  {
    "chainId": 1,
    "address": "0x7825e833d495f3d1c28872415a4aee339d26ac88",
    "name": "Wrapped Telos",
    "symbol": "WTLOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23952/thumb/TLOS.png?1705598803"
  },
  {
    "chainId": 1,
    "address": "0x2217e5921b7edfb4bb193a6228459974010d2198",
    "name": "Qmall",
    "symbol": "QMALL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23630/thumb/tjVN6bQ5_400x400.jpg?1696522834"
  },
  {
    "chainId": 1,
    "address": "0xd3c51de3e6dd9b53d7f37699afb3ee3bf9b9b3f4",
    "name": "MContent",
    "symbol": "MCONTENT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/16781/thumb/Image_mcontent.jpeg?1696516353"
  },
  {
    "chainId": 1,
    "address": "0x1f36fb2d91d9951cf58ae4c1956c0b77e224f1e9",
    "name": "VCGamers",
    "symbol": "VCG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22371/thumb/VCG-Token-Logo-PNG.png?1696521714"
  },
  {
    "chainId": 1,
    "address": "0xe580074a10360404af3abfe2d524d5806d993ea3",
    "name": "PayBolt",
    "symbol": "PAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24175/thumb/logo.png?1696523363"
  },
  {
    "chainId": 1,
    "address": "0x20d60c6eb195868d4643f2c9b0809e4de6cc003d",
    "name": "PlayNity",
    "symbol": "PLY",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21479/thumb/ply.png?1696520840"
  },
  {
    "chainId": 1,
    "address": "0x809e130e10e787139c54e1d12d3d1971b7a675bf",
    "name": "Minted",
    "symbol": "MTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26893/thumb/MTD_Token_%282%29.png?1696525951"
  },
  {
    "chainId": 1,
    "address": "0x1ab43204a195a0fd37edec621482afd3792ef90b",
    "name": "Aurigami",
    "symbol": "PLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24074/thumb/EbB5N8IN_400x400.jpg?1696523268"
  },
  {
    "chainId": 1,
    "address": "0xf655c8567e0f213e6c634cd2a68d992152161dc6",
    "name": "Impermax",
    "symbol": "IBEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27606/thumb/IqwOmX-c_400x400.jpeg?1696526637"
  },
  {
    "chainId": 1,
    "address": "0x19193f450086d0442157b852081976d41657ad56",
    "name": "Nunu Spirits",
    "symbol": "NNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24378/thumb/NNT_99cc33.png?1696523561"
  },
  {
    "chainId": 1,
    "address": "0xc17c30e98541188614df99239cabd40280810ca3",
    "name": "EverRise",
    "symbol": "RISE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16367/thumb/Logo_EverRise_Icon_logo.png?1696515966"
  },
  {
    "chainId": 1,
    "address": "0x32462ba310e447ef34ff0d15bce8613aa8c4a244",
    "name": "Dohrnii",
    "symbol": "DHN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24966/thumb/dhn.png?1696524119"
  },
  {
    "chainId": 1,
    "address": "0x198d14f2ad9ce69e76ea330b374de4957c3f850a",
    "name": "APENFT",
    "symbol": "NFT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/15687/thumb/apenft.jpg?1696515316"
  },
  {
    "chainId": 1,
    "address": "0x16b3e050e9e2f0ac4f1bea1b3e4fdc43d7f062dd",
    "name": "Sombra",
    "symbol": "SMBR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17884/thumb/SMBR_120x120.png?1703029115"
  },
  {
    "chainId": 1,
    "address": "0x446f2a8a39cc730ef378be759a3c57f1a3fe824c",
    "name": "NanoByte",
    "symbol": "NBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23698/thumb/WpcmdLW.png?1696522899"
  },
  {
    "chainId": 1,
    "address": "0x420a24c9c65bd44c48bfb1cc8d6cd1ea8b1ac840",
    "name": "JumpToken",
    "symbol": "JMPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22603/thumb/200x200.png?1696521919"
  },
  {
    "chainId": 1,
    "address": "0x4550003152f12014558e5ce025707e4dd841100f",
    "name": "Kaizen Finance",
    "symbol": "KZEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24396/thumb/PKl5OVRv_400x400.png?1696523579"
  },
  {
    "chainId": 1,
    "address": "0x0c10bf8fcb7bf5412187a595ab97a3609160b5c6",
    "name": "USDD",
    "symbol": "USDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25380/thumb/UUSD.jpg?1696524513"
  },
  {
    "chainId": 1,
    "address": "0xf6e06b54855eff198a2d9a8686113665499a6134",
    "name": "Celestial",
    "symbol": "CELT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18683/thumb/celt.png?1696518152"
  },
  {
    "chainId": 1,
    "address": "0x9767203e89dcd34851240b3919d4900d3e5069f1",
    "name": "A4 Finance",
    "symbol": "A4",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21992/thumb/ba384ad07217a4be75cb85314f5760f7.jpg?1696521340"
  },
  {
    "chainId": 1,
    "address": "0x40a9a694197a0b4b92f2aad48da6bc1b6ff194e9",
    "name": "LFG",
    "symbol": "LFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30910/thumb/LFG.jpg?1696529755"
  },
  {
    "chainId": 1,
    "address": "0xbd8fdda057de7e0162b7a386bec253844b5e07a5",
    "name": "Jarvis",
    "symbol": "JARVIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35812/thumb/jarvis.png?1709873633"
  },
  {
    "chainId": 1,
    "address": "0x345887cdb19e12833ed376bbf8b8b38269f5f5c8",
    "name": "Carbon Browser",
    "symbol": "CSIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28905/thumb/csix.png?1696527881"
  },
  {
    "chainId": 1,
    "address": "0x06677dc4fe12d3ba3c7ccfd0df8cd45e4d4095bf",
    "name": "Work Quest",
    "symbol": "WQT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30040/thumb/WQT-1_%281%29.png?1696528963"
  },
  {
    "chainId": 1,
    "address": "0x70bc0dc6414eb8974bc70685f798838a87d8cce4",
    "name": "Chirpley",
    "symbol": "CHRP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27243/thumb/Chirpley-Logo_200x200.png?1696526293"
  },
  {
    "chainId": 1,
    "address": "0xbc0899e527007f1b8ced694508fcb7a2b9a46f53",
    "name": "Basket",
    "symbol": "BSKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34661/thumb/BSKT_Logo.png?1705636891"
  },
  {
    "chainId": 1,
    "address": "0xbb63a9b64a80e9338b8ea298c51765e57c4f159c",
    "name": "Picasso",
    "symbol": "PICA",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/28286/thumb/PICA.png?1696527286"
  },
  {
    "chainId": 1,
    "address": "0xcf67815cce72e682eb4429eca46843bed81ca739",
    "name": "GAM3S GG",
    "symbol": "G3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35662/thumb/G3_logo.jpg?1709454143"
  },
  {
    "chainId": 1,
    "address": "0xb92ba0a6a843379499770de82aa936d6ba0fd8ca",
    "name": "Youwho",
    "symbol": "YOU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25353/thumb/youwho_200.png?1696524487"
  },
  {
    "chainId": 1,
    "address": "0xd3c325848d7c6e29b574cb0789998b2ff901f17e",
    "name": "OneArt",
    "symbol": "1ART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19307/thumb/token_light_3x.png?1696518749"
  },
  {
    "chainId": 1,
    "address": "0x6c249b6f6492864d914361308601a7abb32e68f8",
    "name": "Brillion",
    "symbol": "DUA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27976/thumb/Screenshot_2023-06-13_at_11.58.02_AM.png?1696526994"
  },
  {
    "chainId": 1,
    "address": "0xaef420fd77477d9dc8b46d704d44dd09d6c27866",
    "name": "Cogito Finance",
    "symbol": "CGV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30731/thumb/CGV.png?1698667079"
  },
  {
    "chainId": 1,
    "address": "0x50327c6c5a14dcade707abad2e27eb517df87ab5",
    "name": "Wrapped Tron",
    "symbol": "WTRX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/22471/thumb/xOesRfpN_400x400.jpg?1696521795"
  },
  {
    "chainId": 1,
    "address": "0x983d8edb44ca96c0595f3c456ebdd47855911f34",
    "name": "War Legends",
    "symbol": "WAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30672/thumb/wZBMVBr2_400x400.jpg?1696529542"
  },
  {
    "chainId": 1,
    "address": "0x967fb0d760ed3ce53afe2f0a071674cccae73550",
    "name": "XANA",
    "symbol": "XETA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24379/thumb/XANA_Logo_neon_pink.png?1696523562"
  },
  {
    "chainId": 1,
    "address": "0xdd13dedecebda566322195bc4451d672a148752c",
    "name": "PRIMAL",
    "symbol": "PRIMAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28435/thumb/PRIMAL_ICON_200px.jpg?1696527432"
  },
  {
    "chainId": 1,
    "address": "0x560363bda52bc6a44ca6c8c9b4a5fadbda32fa60",
    "name": "Seedify fund",
    "symbol": "SFUND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14614/thumb/Favicon_Icon.png?1696514292"
  },
  {
    "chainId": 1,
    "address": "0xc7026a20a640bc71b9074f7aed52a00cd9147091",
    "name": "Tegro",
    "symbol": "TGR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26564/thumb/512x512.png?1696525637"
  },
  {
    "chainId": 1,
    "address": "0x69c2fcae7e30b429166bd616a322e32bec036bcf",
    "name": "MuratiAI",
    "symbol": "MURATIAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30433/thumb/Muratilogo.png?1696529321"
  },
  {
    "chainId": 1,
    "address": "0xae788f80f2756a86aa2f410c651f2af83639b95b",
    "name": "GensoKishi Metaverse",
    "symbol": "MV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23143/thumb/geno.png?1696522435"
  },
  {
    "chainId": 1,
    "address": "0x1416946162b1c2c871a73b07e932d2fb6c932069",
    "name": "Energi",
    "symbol": "NRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5795/thumb/3218.png?1696506239"
  },
  {
    "chainId": 1,
    "address": "0xb990d93c308a31c737aa91839e8ba8eaf4017d7a",
    "name": "PirateCash",
    "symbol": "PIRATE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7155/thumb/logo_%281%29.png?1696507453"
  },
  {
    "chainId": 1,
    "address": "0x5f1f11a3dd7a0c39da1baa3c7b8585b52a77f435",
    "name": "ASAN VERSE",
    "symbol": "ASAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28189/thumb/200200_copy.png?1696527192"
  },
  {
    "chainId": 1,
    "address": "0x9b25889c493ae6df34ceef1ecb10d77c1ba73318",
    "name": "Mean DAO",
    "symbol": "MEAN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21557/thumb/89934951.png?1696520918"
  },
  {
    "chainId": 1,
    "address": "0xb0c7a3ba49c7a6eaba6cd4a96c55a1391070ac9a",
    "name": "Magic",
    "symbol": "MAGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18623/thumb/magic.png?1696518095"
  },
  {
    "chainId": 1,
    "address": "0xf3a3023e6dede84ad88a656a3269f2a36e83c9a9",
    "name": "Sharbi",
    "symbol": "SHARBI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29075/thumb/Sharbi_New_Logo_200x200.png?1712686416"
  },
  {
    "chainId": 1,
    "address": "0x9f826324bb9bdcf7e7eb274bedc417bd45d74f39",
    "name": "MilkAI",
    "symbol": "MILKAI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29055/thumb/MilkAI.jpeg?1696528022"
  },
  {
    "chainId": 1,
    "address": "0x83d6c8c06ac276465e4c92e7ac8c23740f435140",
    "name": "HMX",
    "symbol": "HMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31206/thumb/HMXlogo_CG.png?1696530033"
  },
  {
    "chainId": 1,
    "address": "0x5b52bfb8062ce664d74bbcd4cd6dc7df53fd7233",
    "name": "ZENIQ",
    "symbol": "ZENIQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29284/thumb/zeniq.png?1696528236"
  },
  {
    "chainId": 1,
    "address": "0x9348e94a447bf8b2ec11f374d3f055fd47d936df",
    "name": "For Loot And Glory",
    "symbol": "FLAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20726/thumb/token_logo.ico?1696520124"
  },
  {
    "chainId": 1,
    "address": "0xc3589f56b6869824804a5ea29f2c9886af1b0fce",
    "name": "Honey",
    "symbol": "HNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12895/thumb/hnys.png?1696512683"
  },
  {
    "chainId": 1,
    "address": "0xe7c3755482d0da522678af05945062d4427e0923",
    "name": "ALEX Lab",
    "symbol": "ALEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25837/thumb/ALEX_Token.png?1696524922"
  },
  {
    "chainId": 1,
    "address": "0xdb0170e2d0c1cc1b2e7a90313d9b9afa4f250289",
    "name": "ADAPad",
    "symbol": "ADAPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18273/thumb/EhSqPTtG_400x400.jpg?1696517767"
  },
  {
    "chainId": 1,
    "address": "0x43044f861ec040db59a7e324c40507addb673142",
    "name": "Cap",
    "symbol": "CAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11775/thumb/cap.png?1696511654"
  },
  {
    "chainId": 1,
    "address": "0xc2708a3a4ba7f64bddc1a49f92f941bc77cad23a",
    "name": "Waves Ducks",
    "symbol": "EGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17298/thumb/200x200_pixel.png?1696516851"
  },
  {
    "chainId": 1,
    "address": "0x8bbf1dccbedd5c70d8e793d432fb56b848dd1698",
    "name": "Ape In",
    "symbol": "APEIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18262/thumb/apein.PNG?1696517757"
  },
  {
    "chainId": 1,
    "address": "0xb8e3bb633f7276cc17735d86154e0ad5ec9928c0",
    "name": "VelasPad",
    "symbol": "VLXPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18535/thumb/11654.png?1696518015"
  },
  {
    "chainId": 1,
    "address": "0x16ba8efe847ebdfef99d399902ec29397d403c30",
    "name": "Oh  Finance",
    "symbol": "OH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19493/thumb/oh-token-logo-200.png?1696518929"
  },
  {
    "chainId": 1,
    "address": "0xd373576a9e738f37dc6882328358ff69c4caf4c6",
    "name": "Zam io",
    "symbol": "ZAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19522/thumb/zam.png?1696518956"
  },
  {
    "chainId": 1,
    "address": "0xe57425f1598f9b0d6219706b77f4b3da573a3695",
    "name": "Bitcoin BR",
    "symbol": "BTCBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21205/thumb/btcbr.png?1696520580"
  },
  {
    "chainId": 1,
    "address": "0xd2ba23de8a19316a638dc1e7a9adda1d74233368",
    "name": "Quickswap",
    "symbol": "QUICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25393/thumb/quickswap.png?1696524525"
  },
  {
    "chainId": 1,
    "address": "0xfceb206e1a80527908521121358b5e26caabaa75",
    "name": "Main",
    "symbol": "MAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25258/thumb/logo_circle.png?1696524398"
  },
  {
    "chainId": 1,
    "address": "0x614d7f40701132e25fe6fc17801fbd34212d2eda",
    "name": "SafeBlast",
    "symbol": "BLAST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15686/thumb/safeblast.png?1696515315"
  },
  {
    "chainId": 1,
    "address": "0x8fc8f8269ebca376d046ce292dc7eac40c8d358a",
    "name": "DeFiChain",
    "symbol": "DFI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11757/thumb/symbol-defi-blockchain_200.png?1696511638"
  },
  {
    "chainId": 1,
    "address": "0x7a5d3a9dcd33cb8d527f7b5f96eb4fef43d55636",
    "name": "RadioShack",
    "symbol": "RADIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25307/thumb/ZVoPiysPJq6dPIZm_Se-6vjmsBepwhHlTQfdYZRILbHyVVTRUYCO-wmJJ4zT10HXCGv1j-ZyWr2u2sBaVlap5Y-ILqeXZuIquWdBDxxG0E0qDpgH7omLqYdgWWLSM_TUK9d1PiiYdu6bERdCDaucgFjlqwmhVQK4uV4jyUiXzchVUnu8Qt6SnxlNxz88G0mQ_tfiwkFv_vKqtgb1CcPycVZVz9.jpg?1696524444"
  },
  {
    "chainId": 1,
    "address": "0x26dcfbfa8bc267b250432c01c982eaf81cc5480c",
    "name": "Ankr Staked MATIC",
    "symbol": "ANKRMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25742/thumb/a-matic-c-da4ec10dc9723e695700e25dbf8c8edf.png?1696524833"
  },
  {
    "chainId": 1,
    "address": "0xafcdd4f666c84fed1d8bd825aa762e3714f652c9",
    "name": "Vita Inu",
    "symbol": "VINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20594/thumb/vita-inu-head-wallet-icon-transparent.png?1696520000"
  },
  {
    "chainId": 1,
    "address": "0x9e6b19874e97fe8e8cad77f2c0ab5e7a693e5dbf",
    "name": "StrongHands Finance",
    "symbol": "ISHND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20158/thumb/ISHND_512x512px.png?1696519571"
  },
  {
    "chainId": 1,
    "address": "0x5f018e73c185ab23647c82bd039e762813877f0e",
    "name": "Shack",
    "symbol": "SHACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25699/thumb/shack_no_bg_no_pad3.png?1696524826"
  },
  {
    "chainId": 1,
    "address": "0x40fed5691e547885cabd7a2990de719dcc8497fc",
    "name": "Safe Haven",
    "symbol": "SHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2584/thumb/safehaven.png?1696503388"
  },
  {
    "chainId": 1,
    "address": "0x18a1ea69a50a85752b7bc204a2c45a95ce6e429d",
    "name": "Spice Trade",
    "symbol": "SPICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25770/thumb/SPICE.png?1696524857"
  },
  {
    "chainId": 1,
    "address": "0x57d579f483854c62fef850b8a5332b0d8424b7e2",
    "name": "OpenSwap One",
    "symbol": "OPENX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19274/thumb/X_color.png?1696518717"
  },
  {
    "chainId": 1,
    "address": "0x93581991f68dbae1ea105233b67f7fa0d6bdee7b",
    "name": "Evmos",
    "symbol": "EVMOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24023/thumb/evmos.png?1696523216"
  },
  {
    "chainId": 1,
    "address": "0x4c3a8eceb656ec63eae80a4ebd565e4887db6160",
    "name": "SokuSwap",
    "symbol": "SOKU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18378/thumb/VCIEHaG.png?1696517870"
  },
  {
    "chainId": 1,
    "address": "0x88536c9b2c4701b8db824e6a16829d5b5eb84440",
    "name": "Atlas USV",
    "symbol": "USV",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22066/thumb/7iUyjg5t.png?1696521410"
  },
  {
    "chainId": 1,
    "address": "0xeeeeeb57642040be42185f49c52f7e9b38f8eeee",
    "name": "Elk Finance",
    "symbol": "ELK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17813/thumb/elk.png?1696517333"
  },
  {
    "chainId": 1,
    "address": "0x2e7b0d4f9b2eaf782ed3d160e3a0a4b1a7930ada",
    "name": "Ceres",
    "symbol": "CERES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17959/thumb/sQLDgqx.png?1696517478"
  },
  {
    "chainId": 1,
    "address": "0x4e568ab95f029e8df1e39b30c9d6d076eaa15945",
    "name": "Flycoin FLY",
    "symbol": "FLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26596/thumb/flatArtboard_20tfr.png?1696525671"
  },
  {
    "chainId": 1,
    "address": "0xc72633f995e98ac3bb8a89e6a9c4af335c3d6e44",
    "name": "Omnisea",
    "symbol": "OSEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26475/thumb/293837892_407994084681555_3167689470652146992_n.png?1696525547"
  },
  {
    "chainId": 1,
    "address": "0x2b867efd2de4ad2b583ca0cb3df9c4040ef4d329",
    "name": "Lucky Block",
    "symbol": "LBLOCK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/23126/thumb/logo-200x200-LB-09.png?1696522417"
  },
  {
    "chainId": 1,
    "address": "0xe3c408bd53c31c085a1746af401a4042954ff740",
    "name": "GMT",
    "symbol": "GMT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/23597/thumb/token-gmt-200x200.png?1703153841"
  },
  {
    "chainId": 1,
    "address": "0x2c4f1df9c7de0c59778936c9b145ff56813f3295",
    "name": "AssetMantle",
    "symbol": "MNTL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25181/thumb/thumbnail.png?1696524326"
  },
  {
    "chainId": 1,
    "address": "0x57b59f981730c6257df57cf6f0d98283749a9eeb",
    "name": "BUILD",
    "symbol": "BUILD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26533/thumb/BUILD.png?1696525607"
  },
  {
    "chainId": 1,
    "address": "0xde16ce60804a881e9f8c4ebb3824646edecd478d",
    "name": "MagicCraft",
    "symbol": "MCRT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21318/thumb/mcrt.png?1696520686"
  },
  {
    "chainId": 1,
    "address": "0x839e71613f9aa06e5701cf6de63e303616b0dde3",
    "name": "VVS Finance",
    "symbol": "VVS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20210/thumb/8glAYOTM_400x400.jpg?1696519620"
  },
  {
    "chainId": 1,
    "address": "0x8616e8ea83f048ab9a5ec513c9412dd2993bce3f",
    "name": "handleUSD",
    "symbol": "FXUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26954/thumb/fxUSDlogo.png?1696526009"
  },
  {
    "chainId": 1,
    "address": "0x256d1fce1b1221e8398f65f9b36033ce50b2d497",
    "name": "Alvey Chain",
    "symbol": "WALV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26964/thumb/logo200x200.png?1696526019"
  },
  {
    "chainId": 1,
    "address": "0xb28f803a8772e6584a65ab6dfc535ae6fef8a0b2",
    "name": "Lunafi",
    "symbol": "LFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24594/thumb/lfi.png?1696523768"
  },
  {
    "chainId": 1,
    "address": "0xc285b7e09a4584d027e5bc36571785b515898246",
    "name": "Coin98 Dollar",
    "symbol": "CUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26588/thumb/CUSD-01.png?1696525663"
  },
  {
    "chainId": 1,
    "address": "0x2f32b39023da7d6a6486a85d12b346eb9c2a0d19",
    "name": "Ferro",
    "symbol": "FER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26111/thumb/FER_Token.png?1696525201"
  },
  {
    "chainId": 1,
    "address": "0xb0b195aefa3650a6908f15cdac7d92f8a5791b0b",
    "name": "BOB",
    "symbol": "BOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27266/thumb/Bob-logo.png?1696526318"
  },
  {
    "chainId": 1,
    "address": "0xa719cb79af39a9c10eda2755e0938bce35e9de24",
    "name": "Starfish Finance",
    "symbol": "SEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27533/thumb/SEAN_Token_icon.png?1696526570"
  },
  {
    "chainId": 1,
    "address": "0x3b79a28264fc52c7b4cea90558aa0b162f7faf57",
    "name": "Wonderful Memories",
    "symbol": "WMEMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22392/thumb/wMEMO.png?1696521735"
  },
  {
    "chainId": 1,
    "address": "0x19b22dbadc298c359a1d1b59e35f352a2b40e33c",
    "name": "tPLATINUM",
    "symbol": "TXPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27830/thumb/tPLATINUM_%28TXPT%29_Token.png?1696526849"
  },
  {
    "chainId": 1,
    "address": "0xe4a6f23fb9e00fca037aa0ea0a6954de0a6c53bf",
    "name": "tGOLD",
    "symbol": "TXAU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27828/thumb/tGOLD_token_2D.jpg?1696526846"
  },
  {
    "chainId": 1,
    "address": "0x34abce75d2f8f33940c721dca0f562617787bff3",
    "name": "tSILVER",
    "symbol": "TXAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27829/thumb/tSILVER_token_2D.jpg?1696526848"
  },
  {
    "chainId": 1,
    "address": "0x6b431b8a964bfcf28191b07c91189ff4403957d0",
    "name": "CorgiAI",
    "symbol": "CORGIAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30933/thumb/Token.png?1696529775"
  },
  {
    "chainId": 1,
    "address": "0x61ec85ab89377db65762e234c946b5c25a56e99e",
    "name": "HTX DAO",
    "symbol": "HTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35491/thumb/Frame_1321318576.png?1708908626"
  },
  {
    "chainId": 1,
    "address": "0x40a32606a4ce9b4f350421642ebf65c052d5389b",
    "name": "UP",
    "symbol": "UP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34038/thumb/5.png?1703693495"
  },
  {
    "chainId": 1,
    "address": "0x12996c7b23c4012149bf9f5663ff9aa08a9cf2e4",
    "name": "White Yorkshire",
    "symbol": "WSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36769/thumb/20240403_001638.png?1712285274"
  },
  {
    "chainId": 1,
    "address": "0x1db06f39c14d813d7b1ccb275a93f5b052de1cac",
    "name": "Xave",
    "symbol": "XAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28299/thumb/Xave_logo_gradient.png?1696527298"
  },
  {
    "chainId": 1,
    "address": "0xac87d4cbb82ce7f4bcf31dbdc0024306cfd3ec5a",
    "name": "KEI Finance",
    "symbol": "KEI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29840/thumb/200x200.png?1696528767"
  },
  {
    "chainId": 1,
    "address": "0xd7b675cd5c84a13d1d0f84509345530f6421b57c",
    "name": "Corridor Finance",
    "symbol": "OOOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36057/thumb/corridor-logo-without-text-8bit.jpg?1710404509"
  },
  {
    "chainId": 1,
    "address": "0xeaa260c1d247135cec29ec10fb6acedf2a55dc6b",
    "name": "TheADA",
    "symbol": "TADA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33782/thumb/200x200.jpg?1702980786"
  },
  {
    "chainId": 1,
    "address": "0xb299751b088336e165da313c33e3195b8c6663a6",
    "name": "StarHeroes",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34147/thumb/Screenshot_2024-03-08_180444.png?1709892309"
  },
  {
    "chainId": 1,
    "address": "0x68449870eea84453044bd430822827e21fd8f101",
    "name": "Zaibot",
    "symbol": "ZAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35781/thumb/zai.jpeg?1709795384"
  },
  {
    "chainId": 1,
    "address": "0x527856315a4bcd2f428ea7fa05ea251f7e96a50a",
    "name": "CeDeFiAi",
    "symbol": "CDFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36863/thumb/CeDeFiAi.png?1712643622"
  },
  {
    "chainId": 1,
    "address": "0x82a605d6d9114f4ad6d5ee461027477eeed31e34",
    "name": "Sensay",
    "symbol": "SNSY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36121/thumb/sensay.jpeg?1710495398"
  },
  {
    "chainId": 1,
    "address": "0x033bbde722ea3cdcec73cffea6581df9f9c257de",
    "name": "Velar",
    "symbol": "VELAR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35763/thumb/200.png?1710245034"
  },
  {
    "chainId": 10,
    "address": "0x1db2466d9f5e10d7090e7152b68d62703a2245f0",
    "name": "Sonne Finance",
    "symbol": "SONNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27540/thumb/Token1.png?1696526577"
  },
  {
    "chainId": 10,
    "address": "0xe4d8701c69b3b94a620ff048e4226c895b67b2c0",
    "name": "Minerva Money",
    "symbol": "MINE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30701/thumb/016-D86-A7-E242-4-DC5-860-A-E09-C92-F79-DD2.png?1696529571"
  },
  {
    "chainId": 10,
    "address": "0x1cef2d62af4cd26673c7416957cc4ec619a696a7",
    "name": "DexToro",
    "symbol": "DTORO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33529/thumb/DexToro_Token_Logo_200x200.png?1702255038"
  },
  {
    "chainId": 10,
    "address": "0x3ee6107d9c93955acbb3f39871d32b02f82b78ab",
    "name": "Staked Ethos Reserve Note",
    "symbol": "STERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31522/thumb/stERN.png?1696530332"
  },
  {
    "chainId": 10,
    "address": "0xc3248a1bd9d72fa3da6e6ba701e58cbf818354eb",
    "name": "HANePlatform",
    "symbol": "HANEP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35202/thumb/HANeP_200px.png?1707842735"
  },
  {
    "chainId": 10,
    "address": "0x6d2e5b8841a6aa5f0f973436357f75d3eeb93312",
    "name": "VMEX",
    "symbol": "VMEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33934/thumb/VMEX_3072x3072.jpg?1703500846"
  },
  {
    "chainId": 10,
    "address": "0x96d17e1301b31556e5e263389583a9331e6749e9",
    "name": "RillaFi",
    "symbol": "RILLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29193/thumb/output-onlinepngtools.png?1696528152"
  },
  {
    "chainId": 10,
    "address": "0xe64b4eb5eec6343887c4683da31f0ca62ea39ce3",
    "name": "Aktionariat BEE Digital Growth AG Token",
    "symbol": "BEES",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33367/thumb/BEES.png?1701613820"
  },
  {
    "chainId": 10,
    "address": "0xdc8840a0a1ebf8be5ace62a7d9360dfcb26adffc",
    "name": "HappyAI",
    "symbol": "SMILEAI",
    "decimals": 13,
    "logoURI": "https://assets.coingecko.com/coins/images/34824/thumb/smileai.jpg?1706189986"
  },
  {
    "chainId": 10,
    "address": "0x4200000000000000000000000000000000000042",
    "name": "Optimism",
    "symbol": "OP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25244/thumb/Optimism.png?1696524385"
  },
  {
    "chainId": 10,
    "address": "0xed9b52e6d2df4ad9fc258254e1e5ef5ad0b3ca3c",
    "name": "Aktionariat Fieldoo AG Tokenized Shares",
    "symbol": "FDOS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33392/thumb/FDOS.png?1701680368"
  },
  {
    "chainId": 10,
    "address": "0x61baadcf22d2565b0f471b291c475db5555e0b76",
    "name": "Aelin",
    "symbol": "AELIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22303/thumb/aelien.jpg?1696521649"
  },
  {
    "chainId": 10,
    "address": "0xc26921b5b9ee80773774d36c84328ccb22c3a819",
    "name": "Wrapped OptiDoge",
    "symbol": "WOPTIDOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31066/thumb/Wrapped_OptiDoge_logo_200x200px_.png?1696529900"
  },
  {
    "chainId": 10,
    "address": "0xe3b3a464ee575e8e25d2508918383b89c832f275",
    "name": "PoolTogether Prize USD Coin",
    "symbol": "PUSDCE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32455/thumb/prizeUSDCIcon_200.png?1698241065"
  },
  {
    "chainId": 10,
    "address": "0x9a601c5bb360811d96a23689066af316a30c3027",
    "name": "Pika Protocol",
    "symbol": "PIKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30279/thumb/Pika_protocol.png?1696529185"
  },
  {
    "chainId": 10,
    "address": "0x513c7e3a9c69ca3e22550ef58ac1c0088e918fff",
    "name": "Aave v3 OP",
    "symbol": "AOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32919/thumb/op.png?1699825542"
  },
  {
    "chainId": 10,
    "address": "0x6d80113e533a2c0fe82eabd35f1875dcea89ea97",
    "name": "Aave v3 sUSD",
    "symbol": "ASUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32918/thumb/asusd.png?1699825436"
  },
  {
    "chainId": 10,
    "address": "0x1e925de1c68ef83bd98ee3e130ef14a50309c01b",
    "name": "Exactly Token",
    "symbol": "EXA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31089/thumb/EXA.png?1696529921"
  },
  {
    "chainId": 10,
    "address": "0xfa436399d0458dbe8ab890c3441256e3e09022a8",
    "name": "ZipSwap",
    "symbol": "ZIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22910/thumb/zip.png?1696522207"
  },
  {
    "chainId": 10,
    "address": "0x5ad323d764301e057614edb0449f470d68ea9485",
    "name": "Aktionariat SIA Swiss Influencer Award ",
    "symbol": "SIAS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33373/thumb/SIA.png?1701649606"
  },
  {
    "chainId": 10,
    "address": "0xf8e943f646816e4b51279b8934753821ed832dca",
    "name": "Cthulhu Finance",
    "symbol": "CTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29487/thumb/logo200.png?1696528431"
  },
  {
    "chainId": 10,
    "address": "0x10b3667304130ecc9c972008459249e8141ced97",
    "name": "Aktionariat Clever Forever Education AG",
    "symbol": "CFES",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33365/thumb/CFES.png?1701613582"
  },
  {
    "chainId": 10,
    "address": "0x0d82b5d3a8420c285c6d353a6bdc30d164bb50f0",
    "name": "Aktionariat Sportsparadise Switzerland ",
    "symbol": "SPOS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33391/thumb/SPOS.png?1701680250"
  },
  {
    "chainId": 10,
    "address": "0x8fb94e08bc984497aaaf1a545ed455be89f8c675",
    "name": "Aktionariat TV PLUS AG Tokenized Shares",
    "symbol": "TVPLS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33375/thumb/TVPLS.png?1701649948"
  },
  {
    "chainId": 10,
    "address": "0x071682832d213638f8eda67a873262e581a4006d",
    "name": "Aktionariat Vereign AG Tokenized Shares",
    "symbol": "VRGNS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33364/thumb/VRGNS.png?1701613337"
  },
  {
    "chainId": 10,
    "address": "0xc5b001dc33727f8f26880b184090d3e252470d45",
    "name": "Ethos Reserve Note",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29744/thumb/ERN200x200.png?1696528676"
  },
  {
    "chainId": 10,
    "address": "0x9560e827af36c94d2ac33a39bce1fe78631088db",
    "name": "Velodrome Finance",
    "symbol": "VELO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25783/thumb/velo.png?1696524870"
  },
  {
    "chainId": 10,
    "address": "0xa430a427bd00210506589906a71b54d6c256cedb",
    "name": "Exactly Optimism",
    "symbol": "EXAOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31173/thumb/ExaOP_1.png?1696530001"
  },
  {
    "chainId": 10,
    "address": "0x6f748fd65d7c71949ba6641b3248c4c191f3b322",
    "name": "Exactly WBTC",
    "symbol": "EXAWBTC",
    "decimals": 8
  },
  {
    "chainId": 10,
    "address": "0x81c9a7b55a4df39a9b7b5f781ec0e53539694873",
    "name": "Exactly USD Coin",
    "symbol": "EXAUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31175/thumb/ExaUSDC_1.png?1696530003"
  },
  {
    "chainId": 10,
    "address": "0x2dad3a13ef0c6366220f989157009e501e7938f8",
    "name": "Extra Finance",
    "symbol": "EXTRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30973/thumb/Ex_logo-white-blue_ring_288x.png?1696529812"
  },
  {
    "chainId": 10,
    "address": "0xc4d4500326981eacd020e20a81b1c479c161c7ef",
    "name": "Exactly Wrapped Ether",
    "symbol": "EXAWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31174/thumb/ExaETH_1.png?1696530002"
  },
  {
    "chainId": 10,
    "address": "0x3b08fcd15280e7b5a6e404c4abb87f7c774d1b2e",
    "name": "Overnight Finance",
    "symbol": "OVN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31970/thumb/OVN.png?1696959174"
  },
  {
    "chainId": 10,
    "address": "0xc3864f98f2a61a7caeb95b039d031b4e2f55e0e9",
    "name": "OpenXSwap",
    "symbol": "OPENX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27663/thumb/openx_BASE.png?1710472693"
  },
  {
    "chainId": 10,
    "address": "0x10398abc267496e49106b07dd6be13364d10dc71",
    "name": "Let s Get HAI",
    "symbol": "HAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35863/thumb/hai-logo-200x200.png?1711190591"
  },
  {
    "chainId": 10,
    "address": "0x4b03afc91295ed778320c2824bad5eb5a1d852dd",
    "name": "NBL",
    "symbol": "NBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34811/thumb/%E6%88%AA%E5%9B%BE-%E5%A4%A7.png?1706122685"
  },
  {
    "chainId": 10,
    "address": "0x22ab31cd55130435b5efbf9224b6a9d5ec36533f",
    "name": "Exactly Wrapped stETH",
    "symbol": "EXAWSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31172/thumb/ExawstETH_1.png?1696530000"
  },
  {
    "chainId": 10,
    "address": "0x29cb69d4780b53c1e5cd4d2b817142d2e9890715",
    "name": "PoolTogether Prize WETH   Aave",
    "symbol": "PWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32454/thumb/PrizeWETHIcon_200.png?1698241047"
  },
  {
    "chainId": 10,
    "address": "0xc02b55bb2fe3643e1955b13515396ce23b110f80",
    "name": "Aktionariat Axelra Early Stage AG Token",
    "symbol": "AXRAS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33333/thumb/AXRAS.png?1701519079"
  },
  {
    "chainId": 10,
    "address": "0xdfa46478f9e5ea86d57387849598dbfb2e964b02",
    "name": "MAI  Optimism ",
    "symbol": "MIMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35519/thumb/mimatic-red.png?1709006627"
  },
  {
    "chainId": 10,
    "address": "0x2513486f18eee1498d7b6281f668b955181dd0d9",
    "name": "OpenXSwap Gov  Token",
    "symbol": "XOPENX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28325/thumb/X_op_red_sm.png?1696527332"
  },
  {
    "chainId": 10,
    "address": "0x3ed9acaac7bd974eb83a8ea6432a239e3c829d5d",
    "name": "LERNITAS",
    "symbol": "2192",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36240/thumb/lernitas.jpg?1710920218"
  },
  {
    "chainId": 10,
    "address": "0xf467c7d5a4a9c4687ffc7986ac6ad5a4c81e1404",
    "name": "Kite",
    "symbol": "KITE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35343/thumb/_KITE.png?1708317581"
  },
  {
    "chainId": 10,
    "address": "0x17aabf6838a6303fc6e9c5a227dc1eb6d95c829a",
    "name": "Magicaltux",
    "symbol": "TUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33909/thumb/1703316796-picsay.png?1703453966"
  },
  {
    "chainId": 10,
    "address": "0x970d50d09f3a656b43e11b0d45241a84e3a6e011",
    "name": "Overnight fi DAI ",
    "symbol": "DAI+",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29634/thumb/DAI_.png?1696528571"
  },
  {
    "chainId": 10,
    "address": "0x48a9f8b4b65a55cc46ea557a610acf227454ab09",
    "name": "OP Chads",
    "symbol": "OPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36580/thumb/smol.png?1711944400"
  },
  {
    "chainId": 10,
    "address": "0xcdb4bb51801a1f399d4402c61bc098a72c382e65",
    "name": "OPX Finance",
    "symbol": "OPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28225/thumb/logo_OPX.png?1696527227"
  },
  {
    "chainId": 10,
    "address": "0xa72f7df6c1454096387dbb74f70b3dac4f0a61f5",
    "name": "Aktionariat Carnault AG Tokenized Share",
    "symbol": "CAS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33334/thumb/CAS.png?1701519269"
  },
  {
    "chainId": 10,
    "address": "0xd6c67ff71a82f1d994d94fa01295467d273d7324",
    "name": "Aktionariat DDC Schweiz AG Tokenized Sh",
    "symbol": "DDCS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33354/thumb/DDC.png?1701563532"
  },
  {
    "chainId": 10,
    "address": "0x065399a1e5522af1c9044c18ce60ec70d64d74a1",
    "name": "Aktionariat Finelli Studios AG Tokenize",
    "symbol": "FNLS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/33374/thumb/FNL.png?1701649817"
  },
  {
    "chainId": 10,
    "address": "0x920cf626a271321c151d027030d5d08af699456b",
    "name": "Kwenta",
    "symbol": "KWENTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27409/thumb/kwenta.png?1696526450"
  },
  {
    "chainId": 10,
    "address": "0x7f5c764cbc14f9669b88837ca1490cca17c31607",
    "name": "Bridged USDC  Optimism ",
    "symbol": "USDCE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31580/thumb/USDC-icon.png?1696530397"
  },
  {
    "chainId": 10,
    "address": "0x94b008aa00579c1307b0ef2c499ad98a8ce58e58",
    "name": "Bridged USDT",
    "symbol": "USDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35001/thumb/logo.png?1706959346"
  },
  {
    "chainId": 10,
    "address": "0xe05a08226c49b636acf99c40da8dc6af83ce5bb3",
    "name": "Ankr Staked ETH",
    "symbol": "ANKRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13403/thumb/aETHc.png?1696513165"
  },
  {
    "chainId": 10,
    "address": "0xc52d7f23a2e460248db6ee192cb23dd12bddcbf6",
    "name": "crvUSD",
    "symbol": "CRVUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30118/thumb/crvusd.jpeg?1696529040"
  },
  {
    "chainId": 10,
    "address": "0x47c337bd5b9344a6f3d6f58c474d9d8cd419d8ca",
    "name": "DackieSwap",
    "symbol": "DACKIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30752/thumb/dackieswap_large.png?1707290196"
  },
  {
    "chainId": 10,
    "address": "0x59debed8d46a0cb823d8be8b957add987ead39aa",
    "name": "Quack Token",
    "symbol": "QUACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31436/thumb/0x639C0D019C257966C4907bD4E68E3F349bB58109.png?1696530251"
  },
  {
    "chainId": 10,
    "address": "0x53ed36b1d07a5f4b01e5f872fd054f8439335460",
    "name": "Gemach",
    "symbol": "GMAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32334/thumb/gmac-200x200.png?1697452788"
  },
  {
    "chainId": 10,
    "address": "0xb962150760f9a3bb00e3e9cf48297ee20ada4a33",
    "name": "Zoomer",
    "symbol": "ZOOMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30894/thumb/zoooooooooomer.jpg?1696529740"
  },
  {
    "chainId": 10,
    "address": "0xbfd5206962267c7b4b4a8b3d76ac2e1b2a5c4d5e",
    "name": "Osaka Protocol",
    "symbol": "OSAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30911/thumb/osak_logo.png?1696529756"
  },
  {
    "chainId": 10,
    "address": "0xb4357054c3da8d46ed642383f03139ac7f090343",
    "name": "Port3 Network",
    "symbol": "PORT3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33383/thumb/port3-bc-200x200.png?1710075114"
  },
  {
    "chainId": 10,
    "address": "0x3b6564b5da73a41d3a66e6558a98fd0e9e1e77ad",
    "name": "Unitus",
    "symbol": "UTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33178/thumb/IMG_0051.jpeg?1700924333"
  },
  {
    "chainId": 10,
    "address": "0xc55e93c62874d8100dbd2dfe307edc1036ad5434",
    "name": "Staked BIFI",
    "symbol": "MOOBIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32597/thumb/319381e63428d3c2ab6e035d5f3abd76.png?1698682355"
  },
  {
    "chainId": 10,
    "address": "0x6c84a8f1c29108f47a79964b5fe888d4f4d0de40",
    "name": "tBTC",
    "symbol": "TBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11224/thumb/0x18084fba666a33d37592fa2633fd49a74dd93a88.png?1696511155"
  },
  {
    "chainId": 10,
    "address": "0x395ae52bb17aef68c2888d941736a71dc6d4e125",
    "name": "PoolTogether",
    "symbol": "POOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14003/thumb/PoolTogether.png?1696513732"
  },
  {
    "chainId": 10,
    "address": "0xecf46257ed31c329f204eb43e254c609dee143b3",
    "name": "RigoBlock",
    "symbol": "GRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1532/thumb/Symbol-RigoblockRGB.png?1696502572"
  },
  {
    "chainId": 10,
    "address": "0x9046d36440290ffde54fe0dd84db8b1cfee9107b",
    "name": "yearn finance",
    "symbol": "YFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11849/thumb/yearn.jpg?1696511720"
  },
  {
    "chainId": 10,
    "address": "0x1f514a61bcde34f94bc39731235690ab9da737f7",
    "name": "Tarot",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31800/thumb/TAROT.jpg?1696530615"
  },
  {
    "chainId": 10,
    "address": "0x820802fa8a99901f52e39acd21177b0be6ee2974",
    "name": "EUROe Stablecoin",
    "symbol": "EUROE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28913/thumb/euroe-200x200-round.png?1696527889"
  },
  {
    "chainId": 10,
    "address": "0x625e7708f30ca75bfd92586e17077590c60eb4cd",
    "name": "Aave v3 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32847/thumb/usdc_%281%29.png?1699619355"
  },
  {
    "chainId": 10,
    "address": "0xaf20f5f19698f1d19351028cd7103b63d30de7d7",
    "name": "Wagmi",
    "symbol": "WAGMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31887/thumb/wagmi_token_logo.png?1696530698"
  },
  {
    "chainId": 10,
    "address": "0x9fd22a17b4a96da3f83797d122172c450381fb88",
    "name": "Jefe",
    "symbol": "JEFE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25336/thumb/JEFE_200.png?1696524470"
  },
  {
    "chainId": 10,
    "address": "0x8413041a7702603d9d991f2c4add29e4e8a241f8",
    "name": "Router Protocol",
    "symbol": "ROUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13709/thumb/route_token_200x200-19.png?1696513454"
  },
  {
    "chainId": 10,
    "address": "0xe50fa9b3c56ffb159cb0fca61f5c9d750e8128c8",
    "name": "Aave v3 WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32882/thumb/WETH_%281%29.png?1699716492"
  },
  {
    "chainId": 10,
    "address": "0xe453d6649643f1f460c371dc3d1da98f7922fe51",
    "name": "Fuse",
    "symbol": "FUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10347/thumb/fuse.png?1696510348"
  },
  {
    "chainId": 10,
    "address": "0xa3b615667cbd33cfc69843bf11fbb2a1d926bd46",
    "name": "mPendle",
    "symbol": "MPENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31918/thumb/mPendle.png?1696530726"
  },
  {
    "chainId": 10,
    "address": "0x9cfb13e6c11054ac9fcb92ba89644f30775436e4",
    "name": "Bridged Wrapped stETH  Axelar ",
    "symbol": "AXL-WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32223/thumb/steth-wei_D_3x.png?1696926841"
  },
  {
    "chainId": 10,
    "address": "0xaeaeed23478c3a4b798e4ed40d8b7f41366ae861",
    "name": "Ankr Network",
    "symbol": "ANKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4324/thumb/U85xTl2.png?1696504928"
  },
  {
    "chainId": 10,
    "address": "0x9b88d293b7a791e40d36a39765ffd5a1b9b5c349",
    "name": "Celo  Wormhole ",
    "symbol": "CELO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32400/thumb/celo_wh.png?1698055352"
  },
  {
    "chainId": 10,
    "address": "0x078f358208685046a11c85e8ad32895ded33a249",
    "name": "Aave v3 WBTC",
    "symbol": "AWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32883/thumb/wbtc.png?1699719908"
  },
  {
    "chainId": 10,
    "address": "0x323665443cef804a3b5206103304bd4872ea4253",
    "name": "Verified USD",
    "symbol": "USDV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32948/thumb/usdv_%281%29.png?1699933314"
  },
  {
    "chainId": 10,
    "address": "0x6ab707aca953edaefbc4fd23ba73294241490620",
    "name": "Aave v3 USDT",
    "symbol": "AUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32884/thumb/USDT.PNG?1699768611"
  },
  {
    "chainId": 10,
    "address": "0x1f32b1c2345538c0c6f582fcb022739c4a194ebb",
    "name": "Wrapped stETH",
    "symbol": "WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18834/thumb/wstETH.png?1696518295"
  },
  {
    "chainId": 10,
    "address": "0x59d9356e565ab3a36dd77763fc0d87feaf85508c",
    "name": "Mountain Protocol USD",
    "symbol": "USDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31719/thumb/usdm.png?1696530540"
  },
  {
    "chainId": 10,
    "address": "0x724dc807b04555b71ed48a6896b6f41593b8c637",
    "name": "Aave v3 rETH",
    "symbol": "ARETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32885/thumb/reth.png?1699768947"
  },
  {
    "chainId": 10,
    "address": "0x82e64f49ed5ec1bc6e43dad4fc8af9bb3a2312ee",
    "name": "Aave v3 DAI",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32886/thumb/dai.png?1699769446"
  },
  {
    "chainId": 10,
    "address": "0xf329e36c7bf6e5e86ce2150875a84ce77f477375",
    "name": "Aave v3 AAVE",
    "symbol": "AAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32887/thumb/aave.png?1699773604"
  },
  {
    "chainId": 10,
    "address": "0x191c10aa4af7c30e871e70c95db0e4eb77237530",
    "name": "Aave v3 LINK",
    "symbol": "ALINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32888/thumb/link.png?1699773900"
  },
  {
    "chainId": 10,
    "address": "0xc45a479877e1e9dfe9fcd4056c699575a1045daa",
    "name": "Aave v3 wstETH",
    "symbol": "AWSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32890/thumb/WSTETH.png?1699776336"
  },
  {
    "chainId": 10,
    "address": "0x35b5a3bf31b67f832230da9424824edc9f7ad98c",
    "name": "Phemex Token",
    "symbol": "PT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33314/thumb/phemex_logo.png?1701959611"
  },
  {
    "chainId": 10,
    "address": "0x8eb270e296023e9d92081fdf967ddd7878724424",
    "name": "Aave v3 LUSD",
    "symbol": "ALUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32894/thumb/LUSD.png?1699789893"
  },
  {
    "chainId": 10,
    "address": "0xb829b68f57cc546da7e5806a929e53be32a4625d",
    "name": "Axelar Wrapped Ether",
    "symbol": "AXLETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28171/thumb/weth-wei_D_3x.png?1696527174"
  },
  {
    "chainId": 10,
    "address": "0x3e5d9d8a63cc8a88748f229999cf59487e90721e",
    "name": "MetalSwap",
    "symbol": "XMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22075/thumb/Logo_COIN_-_Gradiente.png?1696521419"
  },
  {
    "chainId": 10,
    "address": "0x6f0fecbc276de8fc69257065fe47c5a03d986394",
    "name": "Popcorn",
    "symbol": "POP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21438/thumb/pop-1_200_x_200.png?1696520801"
  },
  {
    "chainId": 10,
    "address": "0x2e3d870790dc77a83dd1d18184acc7439a53f475",
    "name": "Frax",
    "symbol": "FRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13422/thumb/FRAX_icon.png?1696513182"
  },
  {
    "chainId": 10,
    "address": "0xcb59a0a753fdb7491d5f3d794316f1ade197b21e",
    "name": "Bridged TrueUSD",
    "symbol": "TUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30837/thumb/tusd.jpeg?1696529695"
  },
  {
    "chainId": 10,
    "address": "0xff733b2a3557a7ed6697007ab5d11b79fdd1b76b",
    "name": "Across Protocol",
    "symbol": "ACX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28161/thumb/across-200x200.png?1696527165"
  },
  {
    "chainId": 10,
    "address": "0xaddb6a0412de1ba0f936dcaeb8aaa24578dcf3b2",
    "name": "Coinbase Wrapped Staked ETH",
    "symbol": "CBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27008/thumb/cbeth.png?1709186989"
  },
  {
    "chainId": 10,
    "address": "0x374ad0f47f4ca39c78e5cc54f1c9e426ff8f231a",
    "name": "Premia",
    "symbol": "PREMIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13962/thumb/apple-touch-icon.png?1696513698"
  },
  {
    "chainId": 10,
    "address": "0xb0ae108669ceb86e9e98e8fe9e40d98b867855fd",
    "name": "OneRing",
    "symbol": "RING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23273/thumb/ring.png?1696522493"
  },
  {
    "chainId": 10,
    "address": "0xd3ac016b1b8c80eeadde4d186a9138c9324e4189",
    "name": "Okcash",
    "symbol": "OK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/274/thumb/ok-logo-200px.png?1696501624"
  },
  {
    "chainId": 10,
    "address": "0xbfd291da8a403daaf7e5e9dc1ec0aceacd4848b9",
    "name": "dForce USD",
    "symbol": "USX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17422/thumb/usx_32.png?1696516969"
  },
  {
    "chainId": 10,
    "address": "0x8ae125e8653821e851f12a49f7765db9a9ce7384",
    "name": "DOLA",
    "symbol": "DOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14287/thumb/dola.png?1696513984"
  },
  {
    "chainId": 10,
    "address": "0x74ccbe53f77b08632ce0cb91d3a545bf6b8e0979",
    "name": "Fantom Bomb",
    "symbol": "BOMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24109/thumb/logo-blue.png?1696523301"
  },
  {
    "chainId": 10,
    "address": "0x9bcef72be871e61ed4fbbc7630889bee758eb81d",
    "name": "Rocket Pool ETH",
    "symbol": "RETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20764/thumb/reth.png?1696520159"
  },
  {
    "chainId": 10,
    "address": "0x15e770b95edd73fd96b02ece0266247d50895e76",
    "name": "Jarvis Reward",
    "symbol": "JRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10390/thumb/cfeii0y.png?1696510389"
  },
  {
    "chainId": 10,
    "address": "0x7f5373ae26c3e8ffc4c77b7255df7ec1a9af52a6",
    "name": "Bridged Tether  Axelar ",
    "symbol": "AXLUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31002/thumb/uusdt_D_3x.png?1696529840"
  },
  {
    "chainId": 10,
    "address": "0xaf8ca653fa2772d58f4368b0a71980e9e3ceb888",
    "name": "Tellor Tributes",
    "symbol": "TRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9644/thumb/Blk_icon_current.png?1696509713"
  },
  {
    "chainId": 10,
    "address": "0x47536f17f4ff30e64a96a7555826b8f9e66ec468",
    "name": "Mummy Finance",
    "symbol": "MMY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28547/thumb/Logo_MMY.png?1696527538"
  },
  {
    "chainId": 10,
    "address": "0x6806411765af15bddd26f8f544a34cc40cb9838b",
    "name": "Frax Ether",
    "symbol": "FRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28284/thumb/frxETH_icon.png?1696527284"
  },
  {
    "chainId": 10,
    "address": "0xb153fb3d196a8eb25522705560ac152eeec57901",
    "name": "Magic Internet Money",
    "symbol": "MIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16786/thumb/mimlogopng.png?1696516358"
  },
  {
    "chainId": 10,
    "address": "0x484c2d6e3cdd945a8b2df735e079178c1036578c",
    "name": "Staked Frax Ether",
    "symbol": "SFRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28285/thumb/sfrxETH_icon.png?1696527285"
  },
  {
    "chainId": 10,
    "address": "0x5c0ea461fe5e6f3b4f90a071e72243c14c6abfd7",
    "name": "Amulet Protocol",
    "symbol": "AMU",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34716/thumb/AMT200x200.png?1705904544"
  },
  {
    "chainId": 10,
    "address": "0x3a18dcc9745edcd1ef33ecb93b0b6eba5671e7ca",
    "name": "Kujira",
    "symbol": "KUJI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/20685/thumb/kuji-200x200.png?1696520085"
  },
  {
    "chainId": 10,
    "address": "0xfd389dc9533717239856190f42475d3f263a270d",
    "name": "Granary",
    "symbol": "GRAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29740/thumb/Grain.png?1696528670"
  },
  {
    "chainId": 10,
    "address": "0x0000206329b97db379d5e1bf586bbdb969c63274",
    "name": "USDA",
    "symbol": "USDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34510/thumb/agUSD-coingecko.png?1705288392"
  },
  {
    "chainId": 10,
    "address": "0xdc6ff44d5d932cbd77b52e5612ba0529dc6226f1",
    "name": "Worldcoin",
    "symbol": "WLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31069/thumb/worldcoin.jpeg?1696529903"
  },
  {
    "chainId": 10,
    "address": "0xc27d9bc194a648fe3069955a5126699c4e49351c",
    "name": "Alongside Crypto Market Index",
    "symbol": "AMKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28496/thumb/22999.png?1696527488"
  },
  {
    "chainId": 10,
    "address": "0xd3594e879b358f430e20f82bea61e83562d49d48",
    "name": "ParaSwap",
    "symbol": "PSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20403/thumb/ep7GqM19_400x400.jpg?1696519812"
  },
  {
    "chainId": 10,
    "address": "0x929b939f8524c3be977af57a4a0ad3fb1e374b50",
    "name": "mStable Governance  Meta",
    "symbol": "MTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11846/thumb/mStable.png?1696511717"
  },
  {
    "chainId": 10,
    "address": "0x9ed7e4b1bff939ad473da5e7a218c771d1569456",
    "name": "Reunit Wallet",
    "symbol": "REUNI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29492/thumb/reunit_200.png?1696528436"
  },
  {
    "chainId": 10,
    "address": "0xf1a0da3367bc7aa04f8d94ba57b862ff37ced174",
    "name": "ShapeShift FOX",
    "symbol": "FOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9988/thumb/FOX.png?1696510025"
  },
  {
    "chainId": 10,
    "address": "0xbc7b1ff1c6989f006a1185318ed4e7b5796e66e1",
    "name": "Pendle",
    "symbol": "PENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15069/thumb/Pendle_Logo_Normal-03.png?1696514728"
  },
  {
    "chainId": 10,
    "address": "0x66e535e8d2ebf13f49f3d49e5c50395a97c137b1",
    "name": "Molten",
    "symbol": "MOLTEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36726/thumb/moltenmesh.png?1712147407"
  },
  {
    "chainId": 10,
    "address": "0xeb466342c4d449bc9f53a865d5cb90586f405215",
    "name": "Axelar Bridged USDC",
    "symbol": "AXLUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26476/thumb/uausdc_D_3x.png?1696525548"
  },
  {
    "chainId": 10,
    "address": "0x6d80113e533a2c0fe82eabd35f1875dcea89ea97",
    "name": "Aave SUSD",
    "symbol": "ASUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14266/thumb/aSUSD.9b00ffc6.png?1696513967"
  },
  {
    "chainId": 10,
    "address": "0xfe8b128ba8c78aabc59d4c64cee7ff28e9379921",
    "name": "Balancer",
    "symbol": "BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11683/thumb/Balancer.png?1696511572"
  },
  {
    "chainId": 10,
    "address": "0x76fb31fb4af56892a25e32cfc43de717950c9278",
    "name": "Aave",
    "symbol": "AAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12645/thumb/AAVE.png?1696512452"
  },
  {
    "chainId": 10,
    "address": "0xf899e3909b4492859d44260e1de41a9e663e70f5",
    "name": "RadioShack",
    "symbol": "RADIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25307/thumb/ZVoPiysPJq6dPIZm_Se-6vjmsBepwhHlTQfdYZRILbHyVVTRUYCO-wmJJ4zT10HXCGv1j-ZyWr2u2sBaVlap5Y-ILqeXZuIquWdBDxxG0E0qDpgH7omLqYdgWWLSM_TUK9d1PiiYdu6bERdCDaucgFjlqwmhVQK4uV4jyUiXzchVUnu8Qt6SnxlNxz88G0mQ_tfiwkFv_vKqtgb1CcPycVZVz9.jpg?1696524444"
  },
  {
    "chainId": 10,
    "address": "0x5a5fff6f753d7c11a56a52fe47a177a87e431655",
    "name": "Synapse",
    "symbol": "SYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18024/thumb/synapse_social_icon.png?1696517540"
  },
  {
    "chainId": 10,
    "address": "0x6f620ec89b8479e97a6985792d0c64f237566746",
    "name": "WePiggy Coin",
    "symbol": "WPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21914/thumb/WPC200.png?1696521265"
  },
  {
    "chainId": 10,
    "address": "0xaf9fe3b5ccdae78188b1f8b9a49da7ae9510f151",
    "name": "dHEDGE DAO",
    "symbol": "DHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12508/thumb/dht.png?1696512323"
  },
  {
    "chainId": 10,
    "address": "0x375488f097176507e39b9653b88fdc52cde736bf",
    "name": "Tarot V1",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17881/thumb/tarot-200px.png?1696517403"
  },
  {
    "chainId": 10,
    "address": "0xee9801669c6138e84bd50deb500827b776777d28",
    "name": "O3 Swap",
    "symbol": "O3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15460/thumb/o3.png?1696515107"
  },
  {
    "chainId": 10,
    "address": "0x10010078a54396f62c96df8532dc2b4847d47ed3",
    "name": "Hundred Finance",
    "symbol": "HND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18445/thumb/hnd.PNG?1696517933"
  },
  {
    "chainId": 10,
    "address": "0x50bce64397c75488465253c0a034b8097fea6578",
    "name": "HanChain",
    "symbol": "HAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27374/thumb/logo_200.png?1696526418"
  },
  {
    "chainId": 10,
    "address": "0x00f932f0fe257456b32deda4758922e56a4f4b42",
    "name": "Dope Wars Paper",
    "symbol": "PAPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18166/thumb/EQHGcBO__400x400.jpeg?1696517667"
  },
  {
    "chainId": 10,
    "address": "0xa00e3a3511aac35ca78530c85007afcd31753819",
    "name": "Kyber Network Crystal",
    "symbol": "KNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14899/thumb/RwdVsGcw_400x400.jpg?1696514562"
  },
  {
    "chainId": 10,
    "address": "0x4200000000000000000000000000000000000006",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 10,
    "address": "0x1da650c3b2daa8aa9ff6f661d4156ce24d08a062",
    "name": "Dentacoin",
    "symbol": "DCN",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/850/thumb/dentacoin.png?1696501986"
  },
  {
    "chainId": 10,
    "address": "0x296f55f8fb28e498b858d0bcda06d955b2cb3f97",
    "name": "Stargate Finance",
    "symbol": "STG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24413/thumb/STG_LOGO.png?1696523595"
  },
  {
    "chainId": 10,
    "address": "0x9e1028f5f1d5ede59748ffcee5532509976840e0",
    "name": "Perpetual Protocol",
    "symbol": "PERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12381/thumb/60d18e06844a844ad75901a9_mark_only_03.png?1696512205"
  },
  {
    "chainId": 10,
    "address": "0xc04fb38f10ad352c5f16bd4546f7456e7f1a2d9e",
    "name": "Minato",
    "symbol": "MNTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24622/thumb/MNTO_200x200.png?1696523794"
  },
  {
    "chainId": 10,
    "address": "0xb12c13e66ade1f72f71834f2fc5082db8c091358",
    "name": "Roobee",
    "symbol": "ROOBEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8791/thumb/Group_11.png?1696508946"
  },
  {
    "chainId": 10,
    "address": "0x3e29d3a9316dab217754d13b28646b76607c5f04",
    "name": "Alchemix ETH",
    "symbol": "ALETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16271/thumb/aleth.png?1696515869"
  },
  {
    "chainId": 10,
    "address": "0x73cb180bf0521828d8849bc8cf2b920918e23032",
    "name": "Overnight fi USD ",
    "symbol": "USD+",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25757/thumb/USD__logo.png?1696524843"
  },
  {
    "chainId": 10,
    "address": "0x3e7ef8f50246f725885102e8238cbba33f276747",
    "name": "BarnBridge",
    "symbol": "BOND",
    "decimals": 17,
    "logoURI": "https://assets.coingecko.com/coins/images/12811/thumb/barnbridge.jpg?1696512604"
  },
  {
    "chainId": 10,
    "address": "0x00a35fd824c717879bf370e70ac6868b95870dfb",
    "name": "Iron Bank",
    "symbol": "IB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22902/thumb/ironbank.png?1696522198"
  },
  {
    "chainId": 10,
    "address": "0xc5102fe9359fd9a28f877a67e36b0f050d81a3cc",
    "name": "Hop Protocol",
    "symbol": "HOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25445/thumb/hop.png?1696524577"
  },
  {
    "chainId": 10,
    "address": "0x0994206dfe8de6ec6920ff4d779b0d950605fb53",
    "name": "Curve DAO",
    "symbol": "CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12124/thumb/Curve.png?1696511967"
  },
  {
    "chainId": 10,
    "address": "0xcb8fa9a76b8e203d8c3797bf438d8fb81ea3326a",
    "name": "Alchemix USD",
    "symbol": "ALUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14114/thumb/Alchemix_USD.png?1696513835"
  },
  {
    "chainId": 10,
    "address": "0xb0b195aefa3650a6908f15cdac7d92f8a5791b0b",
    "name": "BOB",
    "symbol": "BOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27266/thumb/Bob-logo.png?1696526318"
  },
  {
    "chainId": 10,
    "address": "0xc40f949f8a4e094d1b49a23ea9241d289b7b2819",
    "name": "Liquity USD",
    "symbol": "LUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14666/thumb/Group_3.png?1696514341"
  },
  {
    "chainId": 10,
    "address": "0x2561aa2bb1d2eb6629edd7b0938d7679b8b49f9e",
    "name": "Ocean Protocol",
    "symbol": "OCEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3687/thumb/ocean-protocol-logo.jpg?1696504363"
  },
  {
    "chainId": 10,
    "address": "0x181d739a0c016bd0b32bd6e47cbca70d3a0e69d7",
    "name": "WaweSwaps Global Token",
    "symbol": "GBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34205/thumb/gbl.png?1704275447"
  },
  {
    "chainId": 10,
    "address": "0x81ab7e0d570b01411fcc4afd3d50ec8c241cb74b",
    "name": "Equalizer",
    "symbol": "EQZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14741/thumb/X2p5mb2f_400x400.png?1696514410"
  },
  {
    "chainId": 10,
    "address": "0x14778860e937f509e651192a90589de711fb88a9",
    "name": "CyberConnect",
    "symbol": "CYBER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31274/thumb/cyberconnect.png?1696530098"
  },
  {
    "chainId": 10,
    "address": "0x28b42698caf46b4b012cf38b6c75867e0762186d",
    "name": "UniDex",
    "symbol": "UNIDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13178/thumb/unidx.png?1696512961"
  },
  {
    "chainId": 10,
    "address": "0x9c9e5fd8bbc25984b178fdce6117defa39d2db39",
    "name": "Binance Peg BUSD",
    "symbol": "BUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31273/thumb/new_binance-peg-busd.png?1696530096"
  },
  {
    "chainId": 10,
    "address": "0x29faf5905bff9cfcc7cf56a5ed91e0f091f8664b",
    "name": "Bankless DAO",
    "symbol": "BANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15227/thumb/j4WEJrwU.png?1696514882"
  },
  {
    "chainId": 10,
    "address": "0x58b9cb810a68a7f3e1e4f8cb45d1b9b3c79705e8",
    "name": "Connext",
    "symbol": "NEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31293/thumb/connext.png?1696530113"
  },
  {
    "chainId": 10,
    "address": "0x0b2c639c533813f4aa9d7837caf62653d097ff85",
    "name": "USDC",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/6319/thumb/usdc.png?1696506694"
  },
  {
    "chainId": 10,
    "address": "0x23ee2343b892b1bb63503a4fabc840e0e2c6810f",
    "name": "Axelar",
    "symbol": "AXL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27277/thumb/V-65_xQ1_400x400.jpeg?1696526329"
  },
  {
    "chainId": 10,
    "address": "0x94025780a1ab58868d9b2dbbb775f44b32e8e6e5",
    "name": "BetSwirl",
    "symbol": "BETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26618/thumb/icon_200.png?1696525691"
  },
  {
    "chainId": 10,
    "address": "0xb396b31599333739a97951b74652c117be86ee1d",
    "name": "Davos Protocol",
    "symbol": "DUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28775/thumb/dusd_logo_200x200.png?1696527754"
  },
  {
    "chainId": 10,
    "address": "0xc03b43d492d904406db2d7d57e67c7e8234ba752",
    "name": "Wrapped USDR",
    "symbol": "WUSDR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28800/thumb/wUSDRlogo.png?1696527777"
  },
  {
    "chainId": 10,
    "address": "0xb4bc46bc6cb217b59ea8f4530bae26bf69f677f0",
    "name": "Beethoven X",
    "symbol": "BEETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19158/thumb/beets-icon-large.png?1696518608"
  },
  {
    "chainId": 10,
    "address": "0xe4de4b87345815c71aa843ea4841bcdc682637bb",
    "name": "BUILD",
    "symbol": "BUILD",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/26533/thumb/BUILD.png?1696525607"
  },
  {
    "chainId": 10,
    "address": "0x66e8617d1df7ab523a316a6c01d16aa5bed93681",
    "name": "Shack",
    "symbol": "SHACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25699/thumb/shack_no_bg_no_pad3.png?1696524826"
  },
  {
    "chainId": 10,
    "address": "0xda10009cbd5d07dd0cecc66161fc93d7c9000da1",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 10,
    "address": "0x42069d11a2cc72388a2e06210921e839cfbd3280",
    "name": "GnomeLand",
    "symbol": "GNOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35201/thumb/logopng.png?1707841763"
  },
  {
    "chainId": 10,
    "address": "0x2b1d36f5b61addaf7da7ebbd11b35fd8cfb0de31",
    "name": "Interport Token",
    "symbol": "ITP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28338/thumb/ITP_Logo_200.png?1696527344"
  },
  {
    "chainId": 10,
    "address": "0x4f604735c1cf31399c6e711d5962b2b3e0225ad3",
    "name": "Glo Dollar",
    "symbol": "USDGLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29319/thumb/glo_logo_coingecko.png?1696528270"
  },
  {
    "chainId": 10,
    "address": "0x2598c30330d5771ae9f983979209486ae26de875",
    "name": "Any Inu",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34126/thumb/anyinulogo200.png?1704174269"
  },
  {
    "chainId": 10,
    "address": "0x350a791bfc2c21f9ed5d10980dad2e2638ffa7f6",
    "name": "Chainlink",
    "symbol": "LINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/877/thumb/chainlink-new-logo.png?1696502009"
  },
  {
    "chainId": 10,
    "address": "0xec6adef5e1006bb305bb1975333e8fc4071295bf",
    "name": "Cartesi",
    "symbol": "CTSI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11038/thumb/Cartesi_Logo.png?1696510982"
  },
  {
    "chainId": 10,
    "address": "0x7fb688ccf682d58f86d7e38e03f9d22e7705448b",
    "name": "Rai Reflex Index",
    "symbol": "RAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14004/thumb/RAI-logo-coin.png?1696513733"
  },
  {
    "chainId": 10,
    "address": "0xe405de8f52ba7559f9df3c368500b6e6ae6cee49",
    "name": "sETH",
    "symbol": "SETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8843/thumb/sETH.png?1696508995"
  },
  {
    "chainId": 10,
    "address": "0xf98dcd95217e15e05d8638da4c91125e59590b07",
    "name": "Kromatika",
    "symbol": "KROM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20541/thumb/KROM_Transparent.png?1696519948"
  },
  {
    "chainId": 10,
    "address": "0x50c5725949a6f0c72e6c4a641f24049a917db0cb",
    "name": "Lyra Finance",
    "symbol": "LYRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21490/thumb/Add-a-heading-26.png?1696520850"
  },
  {
    "chainId": 10,
    "address": "0x68f180fcce6836688e9084f035309e29bf0a2095",
    "name": "Wrapped Bitcoin",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7598/thumb/wrapped_bitcoin_wbtc.png?1696507857"
  },
  {
    "chainId": 10,
    "address": "0x217d47011b23bb961eb6d93ca9945b7501a5bb11",
    "name": "Thales",
    "symbol": "THALES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18388/thumb/CLVZJN_C_400x400.png?1696517879"
  },
  {
    "chainId": 10,
    "address": "0x2dd1b4d4548accea497050619965f91f78b3b532",
    "name": "Staked FRAX",
    "symbol": "SFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35383/thumb/sfrax.png?1708445569"
  },
  {
    "chainId": 10,
    "address": "0x1574564fcfd15bccb3fe04e9818f61131ea74066",
    "name": "Jarvis",
    "symbol": "JARVIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35812/thumb/jarvis.png?1709873633"
  },
  {
    "chainId": 10,
    "address": "0xbc1735d65290a25b4bc964378828a2a57e33d03d",
    "name": "impactMarket",
    "symbol": "PACT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21907/thumb/PACT_Token_Ticker_Blue_2x.png?1696521258"
  },
  {
    "chainId": 10,
    "address": "0xfdb794692724153d1488ccdbe0c56c252596735f",
    "name": "Lido DAO",
    "symbol": "LDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13573/thumb/Lido_DAO.png?1696513326"
  },
  {
    "chainId": 10,
    "address": "0x6fd9d7ad17242c41f7131d257212c54a0e816691",
    "name": "Uniswap",
    "symbol": "UNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12504/thumb/uni.jpg?1696512319"
  },
  {
    "chainId": 10,
    "address": "0x8c6f28f2f1a3c87f0f938b96d27520d9751ec8d9",
    "name": "sUSD",
    "symbol": "SUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5013/thumb/sUSD.png?1696505546"
  },
  {
    "chainId": 10,
    "address": "0x8700daec35af8ff88c16bdf0418774cb3d7599b4",
    "name": "Synthetix Network",
    "symbol": "SNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3406/thumb/SNX.png?1696504103"
  },
  {
    "chainId": 10,
    "address": "0xe0bb0d3de8c10976511e5030ca403dbf4c25165b",
    "name": "0xBitcoin",
    "symbol": "0XBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4454/thumb/0xbtc.png?1696505045"
  },
  {
    "chainId": 10,
    "address": "0xe3c332a5dce0e1d9bc2cc72a68437790570c28a4",
    "name": "BLOCKv",
    "symbol": "VEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1266/thumb/blockv.png?1696502339"
  },
  {
    "chainId": 10,
    "address": "0x8f69ee043d52161fd29137aedf63f5e70cd504d5",
    "name": "The Doge NFT",
    "symbol": "DOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18111/thumb/Doge.png?1696517615"
  },
  {
    "chainId": 10,
    "address": "0x6af3cb766d6cd37449bfd321d961a61b0515c1bc",
    "name": "Ethereans",
    "symbol": "OS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19592/thumb/eherian.jpg?1697038777"
  },
  {
    "chainId": 10,
    "address": "0x00e1724885473b63bce08a9f0a52f35b0979e35a",
    "name": "OATH",
    "symbol": "OATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24075/thumb/OATH.png?1698051304"
  },
  {
    "chainId": 10,
    "address": "0xd52f94df742a6f4b4c8b033369fe13a41782bf44",
    "name": "Layer2DAO",
    "symbol": "L2DAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23699/thumb/Khp7Y4Sn.png?1696522900"
  },
  {
    "chainId": 137,
    "address": "0x527856315a4bcd2f428ea7fa05ea251f7e96a50a",
    "name": "CeDeFiAi",
    "symbol": "CDFI",
    "decimals": 19,
    "logoURI": "https://assets.coingecko.com/coins/images/36863/thumb/CeDeFiAi.png?1712643622"
  },
  {
    "chainId": 137,
    "address": "0x4ec465d5db20a01bc8404dc84f33bb7a398b8335",
    "name": "Floxypay",
    "symbol": "FXY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30702/thumb/mNfZxsg__400x400.jpg?1696529574"
  },
  {
    "chainId": 137,
    "address": "0x3d468ab2329f296e1b9d8476bb54dd77d8c2320f",
    "name": "20WETH 80BAL",
    "symbol": "20WETH-80BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28005/thumb/0x3d468ab2329f296e1b9d8476bb54dd77d8c2320f.png?1696527022"
  },
  {
    "chainId": 137,
    "address": "0x3ed4c2d63def617f436eb031bacae16f478f3b00",
    "name": "DPEX",
    "symbol": "DPEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31038/thumb/logo-ai.png?1696529873"
  },
  {
    "chainId": 137,
    "address": "0x38a232cabb8a7f745c7d6e0a5bf300e3499aa8a6",
    "name": "PIVN",
    "symbol": "PIVN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29617/thumb/pivn.png?1696528553"
  },
  {
    "chainId": 137,
    "address": "0x3c5a5885f6ee4acc2597069fe3c19f49c6efba96",
    "name": "Krida Fans",
    "symbol": "KRIDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23104/thumb/bIdz6haF_400x400.jpg?1696522393"
  },
  {
    "chainId": 137,
    "address": "0xaaa5b9e6c589642f98a1cda99b9d024b8407285a",
    "name": "IRON Titanium",
    "symbol": "TITAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16031/thumb/titan.png?1696515641"
  },
  {
    "chainId": 137,
    "address": "0x5a7bb7b8eff493625a2bb855445911e63a490e42",
    "name": "TSUBASA Utilitiy Token",
    "symbol": "TSUBASAUT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31796/thumb/TSUBASAUT.png?1696530612"
  },
  {
    "chainId": 137,
    "address": "0xb7486718ea21c79bbd894126f79f504fd3625f68",
    "name": "Poodl Exchange Token",
    "symbol": "PET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29762/thumb/PET_200x200.png?1696528693"
  },
  {
    "chainId": 137,
    "address": "0x7b6838c155f2800a0fb99b0382002dbfeee8ce22",
    "name": "HYBRID TOKEN",
    "symbol": "HBD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/29411/thumb/logo-200x200.png?1696528361"
  },
  {
    "chainId": 137,
    "address": "0xf46cb10e8c5fb9368bbf497a3176b80c0af66d44",
    "name": "Vortex Protocol",
    "symbol": "VP",
    "decimals": 11,
    "logoURI": "https://assets.coingecko.com/coins/images/28122/thumb/vp.png?1696527130"
  },
  {
    "chainId": 137,
    "address": "0x3b56a704c01d650147ade2b8cee594066b3f9421",
    "name": "Affyn",
    "symbol": "FYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23275/thumb/fyn.png?1696522495"
  },
  {
    "chainId": 137,
    "address": "0x860ca4c7a60dcdc045ba1012c27788860cfa565f",
    "name": "Polysport Finance",
    "symbol": "PLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36827/thumb/logo_Polysport.png?1712556782"
  },
  {
    "chainId": 137,
    "address": "0xefee2de82343be622dcb4e545f75a3b9f50c272d",
    "name": "TryHards",
    "symbol": "TRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18624/thumb/TryHards_Tokenticker.png?1696518096"
  },
  {
    "chainId": 137,
    "address": "0x01d35cbc2070a3b76693ce2b6364eae24eb88591",
    "name": "Polygen",
    "symbol": "PGEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21476/thumb/polygen-logo_1.jpeg?1696520837"
  },
  {
    "chainId": 137,
    "address": "0xd838290e877e0188a4a44700463419ed96c16107",
    "name": "Toucan Protocol  Nature Carbon Tonne",
    "symbol": "NCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24484/thumb/NCT-removebg-preview.png?1696523665"
  },
  {
    "chainId": 137,
    "address": "0xce899f26928a2b21c6a2fddd393ef37c61dba918",
    "name": "Museum of Crypto Art",
    "symbol": "MOCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15829/thumb/photo_2021-06-04_09.36.16.jpeg?1696515447"
  },
  {
    "chainId": 137,
    "address": "0x00a0d4f2fe50e023aec0648271ce1a6616c702e2",
    "name": "TOKHIT",
    "symbol": "HITT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29356/thumb/IMG_0634.jpg?1696528304"
  },
  {
    "chainId": 137,
    "address": "0x925fadb35b73720238cc78777d02ed4dd3100816",
    "name": "AutoSingle",
    "symbol": "AUTOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27083/thumb/autos.png?1696526133"
  },
  {
    "chainId": 137,
    "address": "0xe09473ed3c1d317d3601290ae7bc511d7e16e62f",
    "name": "Zpunk",
    "symbol": "ZPT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32305/thumb/zpunklogop2b-resized.png?1697188091"
  },
  {
    "chainId": 137,
    "address": "0xa48ad8b964bbf2c420e964b648146567ceb6d5e1",
    "name": "SpaceShipX aUSDC",
    "symbol": "AUSDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28332/thumb/CRYTOLOGO.png?1696527339"
  },
  {
    "chainId": 137,
    "address": "0xd8513c22dd61161ba3872859a6d10eb1612df742",
    "name": "SURFBOARD",
    "symbol": "BOARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34006/thumb/Surfboard_logo.png?1703618759"
  },
  {
    "chainId": 137,
    "address": "0x21fdb8e43d84420afbd6351d645f50c9138daae0",
    "name": "EcoCREDIT",
    "symbol": "ECO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23363/thumb/Eco-CREDIT-Logo.png?1696522578"
  },
  {
    "chainId": 137,
    "address": "0x02649c1ff4296038de4b9ba8f491b42b940a8252",
    "name": "Exchange Genesis Ethlas Medium",
    "symbol": "XGEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22535/thumb/17200.png?1696521856"
  },
  {
    "chainId": 137,
    "address": "0xc77aea972f862df1f6a520be673df63a01255ff9",
    "name": "EarnTV",
    "symbol": "ETV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33207/thumb/ETV__logo.png?1701407386"
  },
  {
    "chainId": 137,
    "address": "0x578fee9def9a270c20865242cfd4ff86f31d0e5b",
    "name": "Runy",
    "symbol": "RUNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29125/thumb/Coin_Runy_200_200.png?1696528086"
  },
  {
    "chainId": 137,
    "address": "0x0e2c818fea38e7df50410f772b7d59af20589a62",
    "name": "Dominium",
    "symbol": "DOM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24506/thumb/DOM-Token-Icon-200-200-px.png?1696523685"
  },
  {
    "chainId": 137,
    "address": "0xce4e6da9c509cb33c23d748713c681c959f68658",
    "name": "PolyYield",
    "symbol": "YIELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16656/thumb/polylogo.png?1696516218"
  },
  {
    "chainId": 137,
    "address": "0x8eb270e296023e9d92081fdf967ddd7878724424",
    "name": "Aave v3 GHST",
    "symbol": "AGHST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32923/thumb/ghst.png?1699826089"
  },
  {
    "chainId": 137,
    "address": "0xc71a7e46266a9a78212a5d87b030f8d1ce942efd",
    "name": "Coinbet Finance",
    "symbol": "CFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28809/thumb/coinbet-transparent.png?1696527786"
  },
  {
    "chainId": 137,
    "address": "0xa0ead927e6c31646cf1d4cc721705c415e515bd4",
    "name": "TRIMBEX",
    "symbol": "TRIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32338/thumb/mm-removebg-preview_7.png?1697453288"
  },
  {
    "chainId": 137,
    "address": "0xd14d1e501b2b52d6134db1ad0857aa91f9bfe2dd",
    "name": "shuts Wave",
    "symbol": "SWAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28830/thumb/swave_logo.png?1696527806"
  },
  {
    "chainId": 137,
    "address": "0x74dd45dd579cad749f9381d6227e7e02277c944b",
    "name": "CULO",
    "symbol": "CULO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34662/thumb/CULO-logo-inverted_200.png?1705641744"
  },
  {
    "chainId": 137,
    "address": "0x3a29cab2e124919d14a6f735b6033a3aad2b260f",
    "name": "Retro Finance oRETRO",
    "symbol": "ORETRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33668/thumb/Screenshot_2023-08-27_101458-removebg-preview.png?1702694247"
  },
  {
    "chainId": 137,
    "address": "0x1c5ea1a050633dd9136e42ce675211116f465692",
    "name": "MnICorp",
    "symbol": "MNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30411/thumb/logo-mni-2022-white-text_%281%29.png?1696529300"
  },
  {
    "chainId": 137,
    "address": "0xd201b8511aab3e9b094b35abcd5d7863c78d6d0e",
    "name": "PolyShark Finance",
    "symbol": "SHARK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15398/thumb/2KY2RdEo_400x400.png?1696515044"
  },
  {
    "chainId": 137,
    "address": "0x25c498781ca536547b147e2199f572e5393d36f0",
    "name": "AirTnT",
    "symbol": "AIRTNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28328/thumb/airtnt.png?1696527335"
  },
  {
    "chainId": 137,
    "address": "0x2627c26b33f5193da4adfb26df60202479ccd2d3",
    "name": "Crypto Gladiator League",
    "symbol": "CGL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18291/thumb/Scalable.png?1696517783"
  },
  {
    "chainId": 137,
    "address": "0xcd7361ac3307d1c5a46b63086a90742ff44c63b3",
    "name": "Crypto Raiders",
    "symbol": "RAIDER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17638/thumb/Ue3Hfh8.png?1696517168"
  },
  {
    "chainId": 137,
    "address": "0x9b79fe702e2508dc61d6db44574462128e626222",
    "name": "DigiBunnies",
    "symbol": "DGBN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32353/thumb/digi.jpg?1697475198"
  },
  {
    "chainId": 137,
    "address": "0x8f18dc399594b451eda8c5da02d0563c0b2d0f16",
    "name": "moonwolf io",
    "symbol": "WOLF",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14698/thumb/oC37AkkD_400x400.jpg?1696514370"
  },
  {
    "chainId": 137,
    "address": "0xa8c557c7ac1626eacaa0e80fac7b6997346306e8",
    "name": "Enegra",
    "symbol": "EGX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/23082/thumb/imgonline-com-ua-Resize-LHUSvWAko1XuD.png?1696522372"
  },
  {
    "chainId": 137,
    "address": "0xea1132120ddcdda2f119e99fa7a27a0d036f7ac9",
    "name": "Aave v3 stMATIC",
    "symbol": "ASTMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32921/thumb/stmatic.png?1699825800"
  },
  {
    "chainId": 137,
    "address": "0x235737dbb56e8517391473f7c964db31fa6ef280",
    "name": "Kasta",
    "symbol": "KASTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22293/thumb/Kasta.png?1696521639"
  },
  {
    "chainId": 137,
    "address": "0x59a32d4b86ffcce0a7f388d6d858c6a0371915ab",
    "name": "MammothAI",
    "symbol": "MAMAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36211/thumb/IMG_20240318_125233_378.png?1710835712"
  },
  {
    "chainId": 137,
    "address": "0x28767e286113ab01ee819b9398a22d6f27badb6e",
    "name": "RabbitCoin Exchange",
    "symbol": "RABBIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36328/thumb/200x200.png?1711098153"
  },
  {
    "chainId": 137,
    "address": "0xee5bb31fdf28b5d64f5a5605085cc4e3649aa624",
    "name": "Websea",
    "symbol": "WBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33272/thumb/WBS-logo-png.png?1701327451"
  },
  {
    "chainId": 137,
    "address": "0xbc5eb84c052fd012bb902c258c9fd241b17c0005",
    "name": "XNET Mobile",
    "symbol": "XNET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31799/thumb/depin.jpeg?1696530614"
  },
  {
    "chainId": 137,
    "address": "0xcaaf554900e33ae5dbc66ae9f8adc3049b7d31db",
    "name": "Liquid Driver liveRETRO",
    "symbol": "LIVERETRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33076/thumb/Rectangle_23015.png?1700544954"
  },
  {
    "chainId": 137,
    "address": "0xca3c652e994d88740b8ab3b33b4935592ab1dfba",
    "name": "Niko",
    "symbol": "NKO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33834/thumb/resized200.png?1703107281"
  },
  {
    "chainId": 137,
    "address": "0x64afdf9e28946419e325d801fb3053d8b8ffdc23",
    "name": "Meeb Master",
    "symbol": "MEEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17312/thumb/logo_-_2021-07-26T144950.172.png?1696516866"
  },
  {
    "chainId": 137,
    "address": "0x5ceebb0947d58fabde2fc026ffe4b33ccfe1ba8b",
    "name": "4INT",
    "symbol": "4INT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/23444/thumb/logo_ff_200x200.jpg?1696522656"
  },
  {
    "chainId": 137,
    "address": "0x3a3e7650f8b9f667da98f236010fbf44ee4b2975",
    "name": "xDollar Stablecoin",
    "symbol": "XUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16291/thumb/xUSD-web-transparent.png?1696515890"
  },
  {
    "chainId": 137,
    "address": "0x57fcbd6503c8be3b1abad191bc7799ef414a5b31",
    "name": "tSILVER",
    "symbol": "TXAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27829/thumb/tSILVER_token_2D.jpg?1696526848"
  },
  {
    "chainId": 137,
    "address": "0x54cd329fc055ed57d657c3128e16091881a1bcdf",
    "name": "Onmax  OLD ",
    "symbol": "OMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34897/thumb/Onmx-200-200.png?1706582677"
  },
  {
    "chainId": 137,
    "address": "0xa3fa99a148fa48d14ed51d610c367c61876997f1",
    "name": "MAI",
    "symbol": "MIMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15264/thumb/mimatic-red.png?1696514916"
  },
  {
    "chainId": 137,
    "address": "0x300211def2a644b036a9bdd3e58159bb2074d388",
    "name": "Crosschain IOTX",
    "symbol": "CIOTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18331/thumb/iotx.PNG?1696517822"
  },
  {
    "chainId": 137,
    "address": "0x40d9fc77027a281d85de1fa660c887e645ae26c3",
    "name": "MetaTrace",
    "symbol": "TRC",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/31427/thumb/Metatrace.png?1696530242"
  },
  {
    "chainId": 137,
    "address": "0x0da0bd2f57a27a70665d53db4ea71e1f26f77a46",
    "name": "Cropto Wheat Token",
    "symbol": "CROW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35306/thumb/CROW_IMG.jpeg?1708156194"
  },
  {
    "chainId": 137,
    "address": "0x29b4ccd16d630df19f768f68f43a0229eae26250",
    "name": "ATRNO",
    "symbol": "ATRNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33536/thumb/logo_%281%29.png?1702366048"
  },
  {
    "chainId": 137,
    "address": "0x58d70ef99a1d22e1a8f8f0e8f27c1babcf8464f3",
    "name": "Meta Plus Token",
    "symbol": "MTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34779/thumb/rsz_img_20240123_184546_288.png?1706156543"
  },
  {
    "chainId": 137,
    "address": "0x8e677ca17065ed74675bc27bcabadb7eef10a292",
    "name": "Rain Coin",
    "symbol": "RAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34032/thumb/rain_coin_logo.png?1703676333"
  },
  {
    "chainId": 137,
    "address": "0x754e15f1243f4b8bd319eb760774388a22b049e5",
    "name": "Unbound",
    "symbol": "UN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33547/thumb/logo_un-200x200.png?1702389221"
  },
  {
    "chainId": 137,
    "address": "0x8f006d1e1d9dc6c98996f50a4c810f17a47fbf19",
    "name": "Pleasure Coin",
    "symbol": "NSFW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15315/thumb/pleasurecoin-icon.png?1696514965"
  },
  {
    "chainId": 137,
    "address": "0x1599fe55cda767b1f631ee7d414b41f5d6de393d",
    "name": "Cool Cats Milk",
    "symbol": "MILK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23340/thumb/milk_logo.jpg?1696522556"
  },
  {
    "chainId": 137,
    "address": "0x61daecab65ee2a1d5b6032df030f3faa3d116aa7",
    "name": "Dark Magic",
    "symbol": "DMAGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15842/thumb/Dark_Magic.png?1696515459"
  },
  {
    "chainId": 137,
    "address": "0xd7c8469c7ec40f853da5f651de81b45aed47e5ab",
    "name": "Potcoin",
    "symbol": "POT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/51/thumb/potcoin.png?1696501447"
  },
  {
    "chainId": 137,
    "address": "0xfd55e4a6767ff7ffc0b8105262a8cdaabe7fbf3f",
    "name": "Patriot Pay",
    "symbol": "PPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33693/thumb/patriot.jpg?1706622595"
  },
  {
    "chainId": 137,
    "address": "0x5f5f7ca63a9d2c5200bc03ca3335a975d8f771d9",
    "name": "BitStarters",
    "symbol": "BITS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34074/thumb/BitStarters_LOGO.jpg?1703906674"
  },
  {
    "chainId": 137,
    "address": "0xf695f9499d18584363aeed0eba4c381d350f81c3",
    "name": "Reality VR",
    "symbol": "RVR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29827/thumb/1F8AA46F-4EBD-439E-9496-512AE24DE06D.png?1696528755"
  },
  {
    "chainId": 137,
    "address": "0xf1428850f92b87e629c6f3a3b75bffbc496f7ba6",
    "name": "Geopoly",
    "symbol": "GEO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21632/thumb/Geopoly_Token.png?1696520992"
  },
  {
    "chainId": 137,
    "address": "0x1fa2f83ba2df61c3d370071d61b17be01e224f3a",
    "name": "Hive Investments HONEY",
    "symbol": "HNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24797/thumb/hny.png?1696523956"
  },
  {
    "chainId": 137,
    "address": "0x3963a400b42377376d6c3d92ddf2d6288d8ee0d6",
    "name": "Equals9",
    "symbol": "EQ9",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28233/thumb/1648392274941.jpeg?1696527236"
  },
  {
    "chainId": 137,
    "address": "0xc3fdbadc7c795ef1d6ba111e06ff8f16a20ea539",
    "name": "Adamant",
    "symbol": "ADDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15225/thumb/adamant.png?1696514880"
  },
  {
    "chainId": 137,
    "address": "0xe78649874bcdb7a9d1666e665f340723a0187482",
    "name": "BIM",
    "symbol": "BIM",
    "decimals": 17,
    "logoURI": "https://assets.coingecko.com/coins/images/30137/thumb/logo200x200.png?1696529058"
  },
  {
    "chainId": 137,
    "address": "0x2d66953fc2eb650f0fd992dbe1e71d743a4e9fee",
    "name": "Backstage Pass Notes",
    "symbol": "NOTES",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36846/thumb/E44F80B4-2565-4AC3-93FD-139AC3438A2C.png?1712583429"
  },
  {
    "chainId": 137,
    "address": "0xae29ac47a9e3b0a52840e547adf74b912999f7fc",
    "name": "EVE",
    "symbol": "EVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22744/thumb/eve-exchange-logo-200-200-03.png?1696522049"
  },
  {
    "chainId": 137,
    "address": "0x3801c3b3b5c98f88a9c9005966aa96aa440b9afc",
    "name": "GAX Liquidity Token Reward",
    "symbol": "GLTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25790/thumb/gltr-token.png?1696524876"
  },
  {
    "chainId": 137,
    "address": "0xc3211f7eb806e916d54a2a166fc36188cffde25b",
    "name": "Cropto Barley Token",
    "symbol": "CROB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35188/thumb/CROB_IMG.jpeg?1707816006"
  },
  {
    "chainId": 137,
    "address": "0x1d2a0e5ec8e5bbdca5cb219e649b565d8e5c3360",
    "name": "Aave Polygon AAVE",
    "symbol": "AMAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17239/thumb/amAAVE_2x.png?1696516796"
  },
  {
    "chainId": 137,
    "address": "0x6749441fdc8650b5b5a854ed255c82ef361f1596",
    "name": "Lucha",
    "symbol": "LUCHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22794/thumb/lucha.png?1696522097"
  },
  {
    "chainId": 137,
    "address": "0x4550003152f12014558e5ce025707e4dd841100f",
    "name": "Kaizen Finance",
    "symbol": "KZEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24396/thumb/PKl5OVRv_400x400.png?1696523579"
  },
  {
    "chainId": 137,
    "address": "0x483dd3425278c1f79f377f1034d9d2cae55648b6",
    "name": "CrowdSwap",
    "symbol": "CROWD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23928/thumb/icon_192.png?1696523126"
  },
  {
    "chainId": 137,
    "address": "0xc1543024dc71247888a7e139c644f44e75e96d38",
    "name": "Battle World",
    "symbol": "BWO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25799/thumb/battleworld_logo_200x200.png?1696524885"
  },
  {
    "chainId": 137,
    "address": "0x2b3b16826719bf0b494c8ddebaa5e882093ee37e",
    "name": "FibSwap DEX",
    "symbol": "FIBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17306/thumb/coingeckofile.png?1696516859"
  },
  {
    "chainId": 137,
    "address": "0xe3f2b1b2229c0333ad17d03f179b87500e7c5e01",
    "name": "Aether Games",
    "symbol": "AEG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29916/thumb/coin_200x198.png?1708678337"
  },
  {
    "chainId": 137,
    "address": "0x2791bca1f2de4661ed88a30c99a7a9449aa84174",
    "name": "Bridged USDC  Polygon PoS Bridge ",
    "symbol": "USDCE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33000/thumb/usdc.png?1700119918"
  },
  {
    "chainId": 137,
    "address": "0xdab529f40e671a1d4bf91361c21bf9f0c9712ab7",
    "name": "Polygon Bridged BUSD  Polygon ",
    "symbol": "BUSD",
    "decimals": 18
  },
  {
    "chainId": 137,
    "address": "0x0bd49815ea8e2682220bcb41524c0dd10ba71d41",
    "name": "Playermon",
    "symbol": "PYM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20682/thumb/eALRa3rZ_400x400.png?1696520082"
  },
  {
    "chainId": 137,
    "address": "0x62a872d9977db171d9e213a5dc2b782e72ca0033",
    "name": "NEUY",
    "symbol": "NEUY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25870/thumb/logo200x200.png?1696524953"
  },
  {
    "chainId": 137,
    "address": "0xcce87c5b269c94b31ec437b1d7d85bf1413b7804",
    "name": "Oort Digital",
    "symbol": "OORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29553/thumb/512_512.png?1696528493"
  },
  {
    "chainId": 137,
    "address": "0xffa188493c15dfaf2c206c97d8633377847b6a52",
    "name": "Wefi Finance",
    "symbol": "WEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30540/thumb/wefi.png?1696529412"
  },
  {
    "chainId": 137,
    "address": "0xf28164a485b0b2c90639e47b0f377b4a438a16b1",
    "name": "Dragon s Quick",
    "symbol": "DQUICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15185/thumb/quick.png?1696514843"
  },
  {
    "chainId": 137,
    "address": "0x64060ab139feaae7f06ca4e63189d86adeb51691",
    "name": "Unicorn Milk",
    "symbol": "UNIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23321/thumb/SVJI5-cerIpV4P1K3d_LeLKN3ZlE4eaazbnPEzrKhnU.png?1696522537"
  },
  {
    "chainId": 137,
    "address": "0x0e9b89007eee9c958c0eda24ef70723c2c93dd58",
    "name": "Ankr Staked MATIC",
    "symbol": "ANKRMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25742/thumb/a-matic-c-da4ec10dc9723e695700e25dbf8c8edf.png?1696524833"
  },
  {
    "chainId": 137,
    "address": "0xd711d7d893de57dc13ff465763218770bd42db1d",
    "name": "ARYZE eGBP",
    "symbol": "EGBP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32989/thumb/ARYZE_eGBP.png?1700099647"
  },
  {
    "chainId": 137,
    "address": "0x839f1a22a59eaaf26c85958712ab32f80fea23d9",
    "name": "Axion",
    "symbol": "AXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13110/thumb/AXION.png?1696512895"
  },
  {
    "chainId": 137,
    "address": "0x2760e46d9bb43dafcbecaad1f64b93207f9f0ed7",
    "name": "Metavault Trade",
    "symbol": "MVX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25402/thumb/mvx.png?1696524534"
  },
  {
    "chainId": 137,
    "address": "0x46d502fac9aea7c5bc7b13c8ec9d02378c33d36f",
    "name": "WolfSafePoorPeople Polygon",
    "symbol": "WSPP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19514/thumb/wspplogo.png?1696518948"
  },
  {
    "chainId": 137,
    "address": "0x01d1a890d40d890d59795afcce22f5adbb511a3a",
    "name": "KUMA Protocol Wrapped FRK",
    "symbol": "WFRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33450/thumb/FRK.png?1701888552"
  },
  {
    "chainId": 137,
    "address": "0x76bf0c28e604cc3fe9967c83b3c3f31c213cfe64",
    "name": "Crystl Finance",
    "symbol": "CRYSTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17023/thumb/CRYSTL_Rebrand_Logo.png?1696516586"
  },
  {
    "chainId": 137,
    "address": "0xcd86152047e800d67bdf00a4c635a8b6c0e5c4c2",
    "name": "Nacho Finance",
    "symbol": "NACHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21635/thumb/NF_nacho_2_200x200.png?1696520994"
  },
  {
    "chainId": 137,
    "address": "0x930a7dc10ae084fbbddc6537d7df7d4c65a40944",
    "name": "UNLOCK",
    "symbol": "UNLOCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23646/thumb/unlock.png?1696522849"
  },
  {
    "chainId": 137,
    "address": "0x60d55f02a771d515e077c9c2403a1ef324885cec",
    "name": "Aave Polygon USDT",
    "symbol": "AMUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17264/thumb/amUSDT_2x.png?1696516819"
  },
  {
    "chainId": 137,
    "address": "0x72c96c73207936e94066b4c8566c6987c9a1f1de",
    "name": "USD Stable Colb",
    "symbol": "SCB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32670/thumb/USD-Stable-Colb-%28SCB%29-Symbol-Green-400px.png?1698910227"
  },
  {
    "chainId": 137,
    "address": "0xcbce9f77921c2e90522d438df4c5582f4c617768",
    "name": "AGA Carbon Rewards",
    "symbol": "ACAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16729/thumb/acr128.png?1696516303"
  },
  {
    "chainId": 137,
    "address": "0xfe4546fefe124f30788c4cc1bb9aa6907a7987f9",
    "name": "CelsiusX Wrapped ETH",
    "symbol": "CXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23682/thumb/k6iYoed.png?1696522884"
  },
  {
    "chainId": 137,
    "address": "0x80244c2441779361f35803b8c711c6c8fc6054a3",
    "name": "BoneSwap",
    "symbol": "BONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17126/thumb/logo_-_2021-07-16T103422.157.png?1696516686"
  },
  {
    "chainId": 137,
    "address": "0x1ed02954d60ba14e26c230eec40cbac55fa3aeea",
    "name": "MakerX",
    "symbol": "MKX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32152/thumb/TokenMakerX.png?1696598199"
  },
  {
    "chainId": 137,
    "address": "0x63c54c41794e14d283123cd38895f7abdf33c379",
    "name": "Qoodo",
    "symbol": "QDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32840/thumb/Top_Shine_Remove-Recovered_%281%29_3.png?1699588905"
  },
  {
    "chainId": 137,
    "address": "0x16dfb898cf7029303c2376031392cb9bac450f94",
    "name": "Dragoma",
    "symbol": "DMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26244/thumb/DMA.png?1696525329"
  },
  {
    "chainId": 137,
    "address": "0xd6a5c5adc3905b354b8d945d9d73142f54254989",
    "name": "Rezolut",
    "symbol": "ZOLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32720/thumb/logomark-light.png?1699001694"
  },
  {
    "chainId": 137,
    "address": "0xf401e2c1ce8f252947b60bfb92578f84217a1545",
    "name": "Arch Balanced Portfolio",
    "symbol": "ABAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30639/thumb/ABAL.png?1696529511"
  },
  {
    "chainId": 137,
    "address": "0x8343091f2499fd4b6174a46d067a920a3b851ff9",
    "name": "Jarvis Synthetic Japanese Yen",
    "symbol": "JJPY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24327/thumb/jJPY.png?1696523512"
  },
  {
    "chainId": 137,
    "address": "0x255707b70bf90aa112006e1b07b9aea6de021424",
    "name": "TETU",
    "symbol": "TETU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17882/thumb/gradient_icon_926.png?1696517403"
  },
  {
    "chainId": 137,
    "address": "0x97d45ec651d25b90701ec49c959e75efe4e1f36c",
    "name": "NX7",
    "symbol": "NX7",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31791/thumb/nx7-02-.png?1696530608"
  },
  {
    "chainId": 137,
    "address": "0x2e6978ceea865948f4c5685e35aec72652e3cb88",
    "name": "tPLATINUM",
    "symbol": "TXPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27830/thumb/tPLATINUM_%28TXPT%29_Token.png?1696526849"
  },
  {
    "chainId": 137,
    "address": "0x9719d867a500ef117cc201206b8ab51e794d3f82",
    "name": "Matic Aave Interest Bearing USDC",
    "symbol": "MAUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14074/thumb/maUSDC.png?1696513798"
  },
  {
    "chainId": 137,
    "address": "0x1a13f4ca1d028320a707d99520abfefca3998b7f",
    "name": "Aave Polygon USDC",
    "symbol": "AMUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/17249/thumb/amUSDC_2x.png?1696516805"
  },
  {
    "chainId": 137,
    "address": "0xd87af7b418d64ff2cde48d890285ba64fc6e115f",
    "name": "Dtec token",
    "symbol": "DTEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35304/thumb/DTECH_200_200.png?1710514224"
  },
  {
    "chainId": 137,
    "address": "0x6a8ec2d9bfbdd20a7f5a4e89d640f7e7ceba4499",
    "name": "MSquare Global",
    "symbol": "MSQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25305/thumb/19743.png?1696524442"
  },
  {
    "chainId": 137,
    "address": "0x723b17718289a91af252d616de2c77944962d122",
    "name": "Gaia Everworld",
    "symbol": "GAIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19629/thumb/gaia.png?1696519058"
  },
  {
    "chainId": 137,
    "address": "0x7e7737c40878e720b32e7bc9cd096259f876d69f",
    "name": "Catheon Gaming",
    "symbol": "CATHEON",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28052/thumb/catheon.jpeg?1706068372"
  },
  {
    "chainId": 137,
    "address": "0x4e78011ce80ee02d2c3e649fb657e45898257815",
    "name": "KlimaDAO",
    "symbol": "KLIMA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19169/thumb/Klima-Token.png?1696518618"
  },
  {
    "chainId": 137,
    "address": "0xcdb4574adb7c6643153a65ee1a953afd5a189cef",
    "name": "Jasmine Forwards Voluntary REC Front Ha",
    "symbol": "FJLT-F24",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34972/thumb/fJLT-FH24.png?1706852222"
  },
  {
    "chainId": 137,
    "address": "0x7fb09e55d55ccd89929e31ba77f64931087c09cb",
    "name": "Lifti",
    "symbol": "LFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33982/thumb/Group_364.jpg?1703582512"
  },
  {
    "chainId": 137,
    "address": "0xd32dfefd9d00f772db460a3b542f0a736d80662f",
    "name": "LOCKON Passive Index",
    "symbol": "LPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32110/thumb/index_passive_icon.png?1696527501"
  },
  {
    "chainId": 137,
    "address": "0xd125443f38a69d776177c2b9c041f462936f8218",
    "name": "FireBot",
    "symbol": "FBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21713/thumb/fbx200.png?1696521068"
  },
  {
    "chainId": 137,
    "address": "0x12a34a6759c871c4c1e8a0a42cfc97e4d7aaf68d",
    "name": "Tutellus",
    "symbol": "TUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2327/thumb/tutellus-logo.png?1696503215"
  },
  {
    "chainId": 137,
    "address": "0xb0b195aefa3650a6908f15cdac7d92f8a5791b0b",
    "name": "BOB",
    "symbol": "BOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27266/thumb/Bob-logo.png?1696526318"
  },
  {
    "chainId": 137,
    "address": "0x3b7e1ce09afe2bb3a23919afb65a38e627cfbe97",
    "name": "Dragon Soul Token",
    "symbol": "DST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29937/thumb/DST_200x200.jpg?1696528864"
  },
  {
    "chainId": 137,
    "address": "0x6bb45ceac714c52342ef73ec663479da35934bf7",
    "name": "PolyPup Bone",
    "symbol": "BONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16819/thumb/9.png?1696516388"
  },
  {
    "chainId": 137,
    "address": "0x127e47aba094a9a87d084a3a93732909ff031419",
    "name": "GENIUS AI",
    "symbol": "GNUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36914/thumb/IMG_5346.png?1712769235"
  },
  {
    "chainId": 137,
    "address": "0x65517425ac3ce259a34400bb67ceb39ff3ddc0bd",
    "name": "Num ARS",
    "symbol": "NARS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23054/thumb/num_finance.png?1696522347"
  },
  {
    "chainId": 137,
    "address": "0x655304e9cc103b47f0afa0825767d6cce426717c",
    "name": "OSSChain",
    "symbol": "OSS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34301/thumb/OSS.png?1712105500"
  },
  {
    "chainId": 137,
    "address": "0xb6169e71144b4debd9caa2dbb851fc8ffde6119f",
    "name": "DuDe",
    "symbol": "DUDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25924/thumb/dude-logo-png.jpg?1696525004"
  },
  {
    "chainId": 137,
    "address": "0xb7042c40de76cfc607ac05e68f9c28a778f0c8a6",
    "name": "Wistaverse",
    "symbol": "WISTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30599/thumb/LOGO_round.png?1696529469"
  },
  {
    "chainId": 137,
    "address": "0x081ec4c0e30159c8259bad8f4887f83010a681dc",
    "name": "Denet File Token",
    "symbol": "DE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31086/thumb/0x081ec4c0e30159c8259bad8f4887f83010a681dc_%281%29.png?1696529919"
  },
  {
    "chainId": 137,
    "address": "0x301595f6fd5f69fad7a488dacb8971e7c0c2f559",
    "name": "Wrapped ThunderPOKT",
    "symbol": "WTPOKT",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/24835/thumb/vjCQH6sw.jpg?1696523992"
  },
  {
    "chainId": 137,
    "address": "0x5ec03c1f7fa7ff05ec476d19e34a22eddb48acdc",
    "name": "ZED RUN",
    "symbol": "ZED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26607/thumb/zed-run.jpeg?1696525681"
  },
  {
    "chainId": 137,
    "address": "0xedcfb6984a3c70501baa8b7f5421ae795ecc1496",
    "name": "ABCMETA",
    "symbol": "META",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27723/thumb/meta-gecko50.png?1696526747"
  },
  {
    "chainId": 137,
    "address": "0x2b867a0f2c67d13f19178e2fe37a8f3769e1bea7",
    "name": "LunarStorm",
    "symbol": "LUST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36359/thumb/LUST.png?1711303206"
  },
  {
    "chainId": 137,
    "address": "0x2075828cdedc356b5358d79cfd314548842e4a2e",
    "name": "BlockChainCoinX",
    "symbol": "XCCX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32276/thumb/xccxx.png?1697180263"
  },
  {
    "chainId": 137,
    "address": "0x5d089336f95e649e491c054279d64a86565e8b25",
    "name": "MonoLend",
    "symbol": "MLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30143/thumb/MLD.png?1696529064"
  },
  {
    "chainId": 137,
    "address": "0xade6057fcafa57d6d51ffa341c64ce4814995995",
    "name": "Backed ZPR1   1 3 Month T Bill",
    "symbol": "BZPR1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32867/thumb/b-ZPR1-200x200.png?1699671603"
  },
  {
    "chainId": 137,
    "address": "0x0d500b1d8e8ef31e21c99d1db9a6444d3adf1270",
    "name": "Wrapped Matic",
    "symbol": "WMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14073/thumb/matic.png?1696513797"
  },
  {
    "chainId": 137,
    "address": "0x7cc15fef543f205bf21018f038f591c6bada941c",
    "name": "PolyCub",
    "symbol": "POLYCUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24124/thumb/polycub.png?1696523315"
  },
  {
    "chainId": 137,
    "address": "0xe003632d97edd30f87c45b2820fc741af55b2fcd",
    "name": "Ziv4 Labs",
    "symbol": "ZIV4",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33454/thumb/Ziv4-logo-WhiteBlue-Z.png?1701907162"
  },
  {
    "chainId": 137,
    "address": "0x5f0197ba06860dac7e31258bdf749f92b6a636d4",
    "name": "Flare Token",
    "symbol": "1FLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19114/thumb/flr.png?1696518566"
  },
  {
    "chainId": 137,
    "address": "0x82362ec182db3cf7829014bc61e9be8a2e82868a",
    "name": "Meshswap Protocol",
    "symbol": "MESH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25609/thumb/VxtSquTp_400x400.jpeg?1652844613"
  },
  {
    "chainId": 137,
    "address": "0xcebdc775e9f18156ec2e04fb4150f1bc54de690f",
    "name": "Leandro Lopes",
    "symbol": "LOPES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28613/thumb/lopes.png?1696527598"
  },
  {
    "chainId": 137,
    "address": "0xd6d3b4254b4526c3095d8ab00b75f186c56dd72c",
    "name": "Lithium Ventures",
    "symbol": "IONS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15298/thumb/lithium_ions.jpg?1696514948"
  },
  {
    "chainId": 137,
    "address": "0x229b1b6c23ff8953d663c4cbb519717e323a0a84",
    "name": "Bloktopia",
    "symbol": "BLOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18819/thumb/logo-bholdus-6.png?1696518281"
  },
  {
    "chainId": 137,
    "address": "0x2db0db271a10661e7090b6758350e18f6798a49d",
    "name": "Mobius Finance",
    "symbol": "MOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18144/thumb/11322.png?1696517647"
  },
  {
    "chainId": 137,
    "address": "0x78445485a8d5b3be765e3027bc336e3c272a23c9",
    "name": "Africarare",
    "symbol": "UBU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35377/thumb/Full_Vector_Token_Icon.jpg?1709147121"
  },
  {
    "chainId": 137,
    "address": "0xac63686230f64bdeaf086fe6764085453ab3023f",
    "name": "Atlas USV",
    "symbol": "USV",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22066/thumb/7iUyjg5t.png?1696521410"
  },
  {
    "chainId": 137,
    "address": "0xcfe2cf35d2bdde84967e67d00ad74237e234ce59",
    "name": "PolyPup",
    "symbol": "PUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16420/thumb/2_%288%29.png?1696516016"
  },
  {
    "chainId": 137,
    "address": "0x6bd10299f4f1d31b3489dc369ea958712d27c81b",
    "name": "Art de Finance",
    "symbol": "ADF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31348/thumb/200x200.png?1696530165"
  },
  {
    "chainId": 137,
    "address": "0x262b8aa7542004f023b0eb02bc6b96350a02b728",
    "name": "Sway Social",
    "symbol": "SWAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18915/thumb/sway.png?1696518372"
  },
  {
    "chainId": 137,
    "address": "0xab9cb20a28f97e189ca0b666b8087803ad636b3c",
    "name": "MEDIEUS",
    "symbol": "MDUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36012/thumb/%E1%84%86%E1%85%A6%E1%84%83%E1%85%B5%E1%84%8B%E1%85%AE%E1%84%89%E1%85%B3_%E1%84%85%E1%85%A9%E1%84%80%E1%85%A9.png?1711379749"
  },
  {
    "chainId": 137,
    "address": "0x2a07461a493b994c2a32f549fd185524f306ab38",
    "name": "Aree Shards",
    "symbol": "AES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27231/thumb/AES_TICKER_200.png?1696526282"
  },
  {
    "chainId": 137,
    "address": "0x1b8cf8045262663d3e1d7e5cc59cc861fd034bb4",
    "name": "GRNGrid",
    "symbol": "G",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/26126/thumb/MQrn5TGl_400x400.jpg?1696525215"
  },
  {
    "chainId": 137,
    "address": "0xad9f61563b104281b14322fec8b42eb67711bf68",
    "name": "Synergy Land Token",
    "symbol": "SNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30471/thumb/15919.png?1696529357"
  },
  {
    "chainId": 137,
    "address": "0x6ab4e20f36ca48b61ecd66c0450fdf665fa130be",
    "name": "DOLZ io",
    "symbol": "DOLZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33605/thumb/dolz_200.png?1702510960"
  },
  {
    "chainId": 137,
    "address": "0x623a1350f6b749575f92ea54d0f745f9f07d3665",
    "name": "Karmaverse Zombie Serum",
    "symbol": "SERUM",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/26002/thumb/serum.png?1696525079"
  },
  {
    "chainId": 137,
    "address": "0x5c3e6447d97fe80a9818ef3fe14a2bf5bb83e0b8",
    "name": "Tracer",
    "symbol": "TRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28564/thumb/tracer.png?1696527553"
  },
  {
    "chainId": 137,
    "address": "0x42e5e06ef5b90fe15f853f59299fc96259209c5c",
    "name": "Aavegotchi KEK",
    "symbol": "KEK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24739/thumb/kek.png?1696523902"
  },
  {
    "chainId": 137,
    "address": "0xc45a479877e1e9dfe9fcd4056c699575a1045daa",
    "name": "Aave v3 SUSHI",
    "symbol": "ASUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32924/thumb/sushi.png?1699826150"
  },
  {
    "chainId": 137,
    "address": "0x6f06e6bed64cf4c4187c06ee2a4732f6a171bc4e",
    "name": "FoodChain Global",
    "symbol": "FOOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23755/thumb/food.png?1696522956"
  },
  {
    "chainId": 137,
    "address": "0xa9806851eb87ad27e8d75660cec2fe6297f92a59",
    "name": "Drive To Earn",
    "symbol": "DTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32243/thumb/DTE.png?1696953033"
  },
  {
    "chainId": 137,
    "address": "0x0102bbfddffbd8d28d3a1b9c47017f62f42768f2",
    "name": "50Cent",
    "symbol": "50C",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16431/thumb/50cents.png?1696516026"
  },
  {
    "chainId": 137,
    "address": "0x4cd44ced63d9a6fef595f6ad3f7ced13fceac768",
    "name": "tetuQi",
    "symbol": "TETUQI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28314/thumb/tetuQi.png?1696527319"
  },
  {
    "chainId": 137,
    "address": "0x5d301750cc9719f00872e33ee81f9c37aba242f4",
    "name": "Revolt 2 Earn",
    "symbol": "RVLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26360/thumb/revolt2earn_32.png?1696525438"
  },
  {
    "chainId": 137,
    "address": "0x0fced30234c3ea94a7b47cc88006ecb7a39dc30e",
    "name": "BITmarkets Token",
    "symbol": "BTMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31547/thumb/btmt-token_%282%29.png?1696530360"
  },
  {
    "chainId": 137,
    "address": "0x2f11eeee0bf21e7661a22dbbbb9068f4ad191b86",
    "name": "Backed NIU Technologies",
    "symbol": "BNIU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31869/thumb/b-NIU-200x200.png?1696530681"
  },
  {
    "chainId": 137,
    "address": "0x0d0b8488222f7f83b23e365320a4021b12ead608",
    "name": "Next Earth",
    "symbol": "NXTT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23264/thumb/kG1zm83.png?1696522484"
  },
  {
    "chainId": 137,
    "address": "0x4a81f8796e0c6ad4877a51c86693b0de8093f2ef",
    "name": "Iron Finance",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17024/thumb/ice_logo.jpg?1696516587"
  },
  {
    "chainId": 137,
    "address": "0xbbcb0356bb9e6b3faa5cbf9e5f36185d53403ac9",
    "name": "Backed Coinbase Global",
    "symbol": "BCOIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31872/thumb/b-COIN-200x200.png?1696530684"
  },
  {
    "chainId": 137,
    "address": "0x623ebda5fc6b271dd597e20ae99927ea9ef8515e",
    "name": "DUX",
    "symbol": "DUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28836/thumb/photo_2023-01-12_17-36-54.jpg?1696527812"
  },
  {
    "chainId": 137,
    "address": "0xeb7eab87837f4dad1bb80856db9e4506fc441f3d",
    "name": "Medieval Empires",
    "symbol": "MEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27588/thumb/MEE_200x200.png?1696526620"
  },
  {
    "chainId": 137,
    "address": "0x24878dfb65693f975d825e157a0685aec2300ad8",
    "name": "Chainers",
    "symbol": "CHU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32480/thumb/opt_3.png?1698287115"
  },
  {
    "chainId": 137,
    "address": "0x1c356b463ce3e46cf494f5fdd2172bdffe5c688d",
    "name": "Sailwars",
    "symbol": "SWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34055/thumb/SWT_200*200.png?1703747855"
  },
  {
    "chainId": 137,
    "address": "0x5c59d7cb794471a9633391c4927ade06b8787a90",
    "name": "Timeleap Finance",
    "symbol": "TIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18854/thumb/timeleap.PNG?1696518315"
  },
  {
    "chainId": 137,
    "address": "0x883abe4168705d2e5da925d28538b7a6aa9d8419",
    "name": "Ball",
    "symbol": "BALL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17557/thumb/2_%289%29.png?1696517092"
  },
  {
    "chainId": 137,
    "address": "0x81ee105457c4eafc061b8c8fedc7bb45d22286d2",
    "name": "Space Rebase XUSD",
    "symbol": "XUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28311/thumb/XUSD.png?1696527316"
  },
  {
    "chainId": 137,
    "address": "0x198f1d316aad1c0bfd36a79bd1a8e9dba92daa18",
    "name": "DeCats",
    "symbol": "DECATS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33203/thumb/Decats_Logo_200_.png?1701076803"
  },
  {
    "chainId": 137,
    "address": "0x5d066d022ede10efa2717ed3d79f22f949f8c175",
    "name": "Stabl fi CASH",
    "symbol": "CASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27558/thumb/cash.png?1696526594"
  },
  {
    "chainId": 137,
    "address": "0xc4a206a306f0db88f98a3591419bc14832536862",
    "name": "Elefant",
    "symbol": "ELE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34417/thumb/7XBqPLzZ_400x400.jpg?1704820988"
  },
  {
    "chainId": 137,
    "address": "0xffc4b25557bb4327d437c3b3ead6b2b15fab391a",
    "name": "Duel Network",
    "symbol": "DUEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30210/thumb/duel.jpg?1696529120"
  },
  {
    "chainId": 137,
    "address": "0x6acda5e7eb1117733dc7cb6158fc67f226b32022",
    "name": "Carb0n fi",
    "symbol": "ZRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23378/thumb/Signet_Zoom_200px_Transparent.png?1696522593"
  },
  {
    "chainId": 137,
    "address": "0x44a6e0be76e1d9620a7f76588e4509fe4fa8e8c8",
    "name": "Aavegotchi FOMO",
    "symbol": "FOMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24737/thumb/fomo.png?1696523900"
  },
  {
    "chainId": 137,
    "address": "0xbc2659ead8d2d13a157a75bfc5acc156e1ea06df",
    "name": "Ommniverse",
    "symbol": "OMMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29394/thumb/ommi_gold.png?1696528344"
  },
  {
    "chainId": 137,
    "address": "0xf9c1f70f9bf57fad5f63c6e1e25c2e895f04c0a6",
    "name": "DeltaHub Community",
    "symbol": "DHC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12656/thumb/200x200Logo.jpg?1702553679"
  },
  {
    "chainId": 137,
    "address": "0x187ae45f2d361cbce37c6a8622119c91148f261b",
    "name": "Polylastic",
    "symbol": "POLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15331/thumb/POLX_Logo200200.png?1696514979"
  },
  {
    "chainId": 137,
    "address": "0x4c392822d4be8494b798cea17b43d48b2308109c",
    "name": "Polly Finance",
    "symbol": "POLLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18354/thumb/POLLY.png?1696517843"
  },
  {
    "chainId": 137,
    "address": "0x2a75a19a162ef79ed97012e31d8f9c06d3bb4f92",
    "name": "Poldo",
    "symbol": "POLDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34537/thumb/Poldo.png?1705324970"
  },
  {
    "chainId": 137,
    "address": "0xf0b2b2d73a7c070e5f44a4e12b3f6a51949a3459",
    "name": "UNILAPSE",
    "symbol": "UNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32458/thumb/200_200.png?1702850543"
  },
  {
    "chainId": 137,
    "address": "0xbc5b59ea1b6f8da8258615ee38d40e999ec5d74f",
    "name": "Paw V2",
    "symbol": "PAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22836/thumb/v9uvwsJ.png?1696522137"
  },
  {
    "chainId": 137,
    "address": "0x08715f5c743f747de0005ad6c45336c163711137",
    "name": "cLFi",
    "symbol": "CLFI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31454/thumb/CLFI.png?1696530268"
  },
  {
    "chainId": 137,
    "address": "0xfc9fa9771145fbb98d15c8c2cc94b837a56d554c",
    "name": "Slingshot",
    "symbol": "SLING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36111/thumb/apple-touch-icon.png?1710487288"
  },
  {
    "chainId": 137,
    "address": "0x18e73a5333984549484348a94f4d219f4fab7b81",
    "name": "Yellow Duckies",
    "symbol": "DUCKIES",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/27630/thumb/duckies_logo.png?1706528164"
  },
  {
    "chainId": 137,
    "address": "0x04cc80c7665e4d57f57e16a1212a57c0c11927ce",
    "name": "Aniverse Metaverse",
    "symbol": "ANIV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29090/thumb/aniv.png?1696528054"
  },
  {
    "chainId": 137,
    "address": "0x204820b6e6feae805e376d2c6837446186e57981",
    "name": "ROND",
    "symbol": "ROND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27601/thumb/rond.png?1696526632"
  },
  {
    "chainId": 137,
    "address": "0x669ddc70273084ea30e6cd4f28ca6e2c70735065",
    "name": "AGA Carbon Credit",
    "symbol": "AGAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16728/thumb/agac128.png?1696516302"
  },
  {
    "chainId": 137,
    "address": "0xec38621e72d86775a89c7422746de1f52bba5320",
    "name": "Davos Protocol",
    "symbol": "DUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28775/thumb/dusd_logo_200x200.png?1696527754"
  },
  {
    "chainId": 137,
    "address": "0x4455ef8b4b4a007a93daa12de63a47eeac700d9d",
    "name": "Forest Knight",
    "symbol": "KNIGHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18441/thumb/Coin_Forest_Knight_200x200.png?1696517928"
  },
  {
    "chainId": 137,
    "address": "0x0c8c8ae8bc3a69dc8482c01ceacfb588bb516b01",
    "name": "AuroraToken",
    "symbol": "AURORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15345/thumb/aurora_tokenlogo.png?1696514994"
  },
  {
    "chainId": 137,
    "address": "0x4c16f69302ccb511c5fac682c7626b9ef0dc126a",
    "name": "Pancake Bunny Polygon",
    "symbol": "POLYBUNNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16855/thumb/token-bunny.a8b61846.png?1696516422"
  },
  {
    "chainId": 137,
    "address": "0xe261d618a959afffd53168cd07d12e37b26761db",
    "name": "DIMO",
    "symbol": "DIMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28383/thumb/Token_Logo.png?1696527383"
  },
  {
    "chainId": 137,
    "address": "0x7fc9e0aa043787bfad28e29632ada302c790ce33",
    "name": "tetuBAL",
    "symbol": "TETUBAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27969/thumb/tetuBal.png?1696526988"
  },
  {
    "chainId": 137,
    "address": "0x32934cb16da43fd661116468c1b225fc26cf9a8c",
    "name": "StrongNode",
    "symbol": "SNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19303/thumb/sne.png?1696518746"
  },
  {
    "chainId": 137,
    "address": "0x311434160d7537be358930def317afb606c0d737",
    "name": "Nakamoto Games",
    "symbol": "NAKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19073/thumb/flCKDeh6_400x400.jpg?1696518523"
  },
  {
    "chainId": 137,
    "address": "0xe9bc9ad74cca887aff32ba09a121b1256fc9f052",
    "name": "Pigcoin",
    "symbol": "PIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35425/thumb/pigcoin_200.png?1708544734"
  },
  {
    "chainId": 137,
    "address": "0xe8377a076adabb3f9838afb77bee96eac101ffb1",
    "name": "MetaSoccer",
    "symbol": "MSU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21670/thumb/Om_U3jQg_400x400.jpg?1696521027"
  },
  {
    "chainId": 137,
    "address": "0x027d14b07299ee2e74a3b7aba561e5b4adf29966",
    "name": "SpaceShipX SSX",
    "symbol": "SSX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28704/thumb/CRYTOLOGO.png?1696527686"
  },
  {
    "chainId": 137,
    "address": "0x34d4ab47bee066f361fa52d792e69ac7bd05ee23",
    "name": "Raider Aurum",
    "symbol": "AURUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18131/thumb/xcqAUkU.png?1696517634"
  },
  {
    "chainId": 137,
    "address": "0xa6da8c8999c094432c77e7d318951d34019af24b",
    "name": "tGOLD",
    "symbol": "TXAU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27828/thumb/tGOLD_token_2D.jpg?1696526846"
  },
  {
    "chainId": 137,
    "address": "0x6e65ae5572df196fae40be2545ebc2a9a24eace9",
    "name": "Shack",
    "symbol": "SHACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25699/thumb/shack_no_bg_no_pad3.png?1696524826"
  },
  {
    "chainId": 137,
    "address": "0x08e175a1eac9744a0f1ccaeb8f669af6a2bda3ce",
    "name": "Energy8",
    "symbol": "E8",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19707/thumb/OZ52Oms.png?1696519133"
  },
  {
    "chainId": 137,
    "address": "0xdda40cdfe4a0090f42ff49f264a831402adb801a",
    "name": "Dogira",
    "symbol": "DOGIRA",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14634/thumb/IQgaRw0.png?1696514312"
  },
  {
    "chainId": 137,
    "address": "0xff22c94ffb6bb5d1df18beb5fd1dfe7583d3b214",
    "name": "IPMB",
    "symbol": "IPMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32952/thumb/200_x_200.png?1699936302"
  },
  {
    "chainId": 137,
    "address": "0x6d80113e533a2c0fe82eabd35f1875dcea89ea97",
    "name": "Aave v3 WMATIC",
    "symbol": "AWMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32922/thumb/wmatic.png?1699825896"
  },
  {
    "chainId": 137,
    "address": "0xfca466f2fa8e667a517c9c6cfa99cf985be5d9b1",
    "name": "Saiyan PEPE",
    "symbol": "SPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29994/thumb/spepe.png?1696528918"
  },
  {
    "chainId": 137,
    "address": "0x3050731a7ab18d05eaa5e01d66df33e04444e72c",
    "name": "ECS Gold",
    "symbol": "ECG",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27542/thumb/ECOSMART_LOGO_200X200.png?1696526579"
  },
  {
    "chainId": 137,
    "address": "0x232804231de32551f13a57aa3984900428adf990",
    "name": "ZURF",
    "symbol": "ZRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32443/thumb/Zurf_logo.png?1698238062"
  },
  {
    "chainId": 137,
    "address": "0x874e178a2f3f3f9d34db862453cd756e7eab0381",
    "name": "Gravity Finance",
    "symbol": "GFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15871/thumb/GFI-Icon.png?1696515486"
  },
  {
    "chainId": 137,
    "address": "0xbfa35599c7aebb0dace9b5aa3ca5f2a79624d8eb",
    "name": "Retro Finance",
    "symbol": "RETRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31136/thumb/retro.png?1696529965"
  },
  {
    "chainId": 137,
    "address": "0x2e2992688ee4b2b7229118f2f4cfd9b8ab13c520",
    "name": "EraApe",
    "symbol": "EAPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33130/thumb/eraape.logo.png?1700796461"
  },
  {
    "chainId": 137,
    "address": "0x0c087f8d6a1f14f71bb7cc7e1b061ca297af7555",
    "name": "Endblock",
    "symbol": "END",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32057/thumb/logo-endblock-200x200.png?1696530854"
  },
  {
    "chainId": 137,
    "address": "0x7067bebfa1720132dfb9373d65b522afbe3a201e",
    "name": "Floyx",
    "symbol": "FLOYX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29527/thumb/FloyxLogo_200x200.png?1696528469"
  },
  {
    "chainId": 137,
    "address": "0x14ee88cd0e3c1534bff13d6d716b8575532c7138",
    "name": "EDNS Domains",
    "symbol": "EDNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30880/thumb/ednslogo.jpg?1696529727"
  },
  {
    "chainId": 137,
    "address": "0xaa3717090cddc9b227e49d0d84a28ac0a996e6ff",
    "name": "Permission Coin",
    "symbol": "ASK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12521/thumb/rjQjTU3c_400x400.jpg?1696512335"
  },
  {
    "chainId": 137,
    "address": "0x05059b925e1396a78f7375a74bc81085b0694436",
    "name": "MTFi",
    "symbol": "MTFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35308/thumb/mtfi-logo-2.png?1708156525"
  },
  {
    "chainId": 137,
    "address": "0xe7a24ef0c5e95ffb0f6684b813a78f2a3ad7d171",
    "name": "Curve fi amDAI amUSDC amUSDT",
    "symbol": "AM3CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26740/thumb/W1sQNVWo_400x400.jpeg?1696525809"
  },
  {
    "chainId": 137,
    "address": "0x7c6c69bafbe40b2c76a25dcdf9854c77f086e2c6",
    "name": "OVAL3",
    "symbol": "OVL3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34001/thumb/512_512_icon.jpg?1703601480"
  },
  {
    "chainId": 137,
    "address": "0x51869836681bce74a514625c856afb697a013797",
    "name": "Genesis Worlds",
    "symbol": "GENESIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19905/thumb/kKkWmniB_400x400.jpg?1696519325"
  },
  {
    "chainId": 137,
    "address": "0x9b765735c82bb00085e9dbf194f20e3fa754258e",
    "name": "Carnomaly",
    "symbol": "CARR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14403/thumb/tcjOTKE3_400x400.png?1696514096"
  },
  {
    "chainId": 137,
    "address": "0xfd290dab8ec58fb7e9c7ce6e36b9a5a1a90ec164",
    "name": "Metalands",
    "symbol": "PVP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30609/thumb/pvp_token.png?1696529478"
  },
  {
    "chainId": 137,
    "address": "0xc79fc2885e207e1c4cc69cf94402dd1a5642452e",
    "name": "Seamans Token",
    "symbol": "SEAT",
    "decimals": 18
  },
  {
    "chainId": 137,
    "address": "0x63062d0f51fc65d4493c85df0863f631fe2a26b6",
    "name": "WorldToken",
    "symbol": "WORLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33182/thumb/photo_2023-11-23_15-08-30.jpg?1700926259"
  },
  {
    "chainId": 137,
    "address": "0xb5c064f955d8e7f38fe0460c556a72987494ee17",
    "name": "Quickswap",
    "symbol": "QUICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25393/thumb/quickswap.png?1696524525"
  },
  {
    "chainId": 137,
    "address": "0x38b501279f1b78f2aa8f2117bf26f64252fe3f2f",
    "name": "CatsApes",
    "symbol": "CATS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32363/thumb/logo_%282%29_%281%29.png?1698028862"
  },
  {
    "chainId": 137,
    "address": "0x21a00838e6b2d4aa3ac4bbc11111be011e1ca111",
    "name": "GREENGOLD",
    "symbol": "GREENGOLD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33578/thumb/GG_Logo_%283%29.png?1702453059"
  },
  {
    "chainId": 137,
    "address": "0xeb2778f74e5ee038e67aa6c77f0f0451abd748fd",
    "name": "PolyZap",
    "symbol": "PZAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15623/thumb/polyzap.png?1696515256"
  },
  {
    "chainId": 137,
    "address": "0x613a489785c95afeb3b404cc41565ccff107b6e0",
    "name": "RadioShack",
    "symbol": "RADIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25307/thumb/ZVoPiysPJq6dPIZm_Se-6vjmsBepwhHlTQfdYZRILbHyVVTRUYCO-wmJJ4zT10HXCGv1j-ZyWr2u2sBaVlap5Y-ILqeXZuIquWdBDxxG0E0qDpgH7omLqYdgWWLSM_TUK9d1PiiYdu6bERdCDaucgFjlqwmhVQK4uV4jyUiXzchVUnu8Qt6SnxlNxz88G0mQ_tfiwkFv_vKqtgb1CcPycVZVz9.jpg?1696524444"
  },
  {
    "chainId": 137,
    "address": "0x421b9b487d5a9b76e4b81809c0f1b9bb8cb24cb9",
    "name": "Phobos Token",
    "symbol": "PBOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28529/thumb/Phobos_Token_symbol.png?1696527521"
  },
  {
    "chainId": 137,
    "address": "0x30ea765d4dda26e0f89e1b23a7c7b2526b7d29ec",
    "name": "PolyPad",
    "symbol": "POLYPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24905/thumb/BXApUK87_400x400.png?1696524062"
  },
  {
    "chainId": 137,
    "address": "0x6f3cc27e17a0f2e52d8e7693ff0d892cea1854bf",
    "name": "Gooeys",
    "symbol": "GOO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25802/thumb/GOO-200.png?1696524888"
  },
  {
    "chainId": 137,
    "address": "0x981aecc6eb4d382b96a02b75e931900705e95a31",
    "name": "SAVAGE",
    "symbol": "SAVG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22750/thumb/savg_logo.jpg?1696522054"
  },
  {
    "chainId": 137,
    "address": "0xff2382bd52efacef02cc895bcbfc4618608aa56f",
    "name": "OneRare",
    "symbol": "ORARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19696/thumb/Thumbnail_-_500_px_-_Black.png?1696519123"
  },
  {
    "chainId": 137,
    "address": "0xa64a329876a27192c7f8bde4430bdec70ea4b2f9",
    "name": "NaturesGold",
    "symbol": "NGOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32030/thumb/i1Uvl8U.png?1696530827"
  },
  {
    "chainId": 137,
    "address": "0x0735fa49eb7d9ddf3e4d9a9f01229627f67632a1",
    "name": "Cropto Corn Token",
    "symbol": "CROC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35189/thumb/CROC_IMG.jpeg?1707816120"
  },
  {
    "chainId": 137,
    "address": "0xd60deba014459f07bbcc077a5b817f31dafd5229",
    "name": "Croatian FF Fan Token",
    "symbol": "VATRENI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28295/thumb/Vatreni.png?1696527294"
  },
  {
    "chainId": 137,
    "address": "0x52468c88e8b4f5bcca20a6a7813355637dc5e3ad",
    "name": "Power Of Deep Ocean",
    "symbol": "PODO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27645/thumb/PODO_TICKER_200.png?1696526674"
  },
  {
    "chainId": 137,
    "address": "0x4ca7bf91e4384d7c7a48ed4c04b555f350eb6719",
    "name": "Zeemcoin",
    "symbol": "ZEEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30336/thumb/Logo_reducido_Zeemcoin_-_Color.png?1696529237"
  },
  {
    "chainId": 137,
    "address": "0x2d027e55b85429e9f205930a8aff6d8e6c8c3021",
    "name": "RAYS",
    "symbol": "RAYS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26609/thumb/mch_rays_200.png?1696525683"
  },
  {
    "chainId": 137,
    "address": "0x0169ec1f8f639b32eec6d923e24c2a2ff45b9dd6",
    "name": "Algebra",
    "symbol": "ALGB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19580/thumb/13211.png?1696519011"
  },
  {
    "chainId": 137,
    "address": "0xe238ecb42c424e877652ad82d8a939183a04c35f",
    "name": "WiFi Map",
    "symbol": "WIFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29533/thumb/IMG_4643.png?1696528475"
  },
  {
    "chainId": 137,
    "address": "0xf6d5d14c5684fa5853a3be6d217cad19c6ab2164",
    "name": "xMATIC",
    "symbol": "XMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28333/thumb/XMATIC.png?1696527339"
  },
  {
    "chainId": 137,
    "address": "0x086e6f3b29f5ec0ba6539a6fbd5fde5d4b3f530b",
    "name": "Phoenix AI",
    "symbol": "PXAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36258/thumb/PHX_final_logo_black.png?1711030360"
  },
  {
    "chainId": 137,
    "address": "0x688b4231472fde70c1d30f48638aa1661725d3ce",
    "name": "SendCrypto",
    "symbol": "SENDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28334/thumb/SC_Logo_200x200.png?1696527340"
  },
  {
    "chainId": 137,
    "address": "0x57c453e7a4d73ac1db8ae07da5dda3b398d17577",
    "name": "Nodewaves",
    "symbol": "NWS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33160/thumb/logo-4.png?1700829948"
  },
  {
    "chainId": 137,
    "address": "0x689f8e5913c158ffb5ac5aeb83b3c875f5d20309",
    "name": "Snook",
    "symbol": "SNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18197/thumb/snk.png?1696517696"
  },
  {
    "chainId": 137,
    "address": "0x8bff1bd27e2789fe390acabc379c380a83b68e84",
    "name": "TurnUp  LFG",
    "symbol": "LFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35020/thumb/0x8bff1bd27e2789fe390acabc379c380a83b68e84.png?1707135388"
  },
  {
    "chainId": 137,
    "address": "0xacbdc2b7a577299718309ed5c4b703fb5ed7af90",
    "name": "Signed",
    "symbol": "SIGN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24030/thumb/wjnKYes.png?1696523222"
  },
  {
    "chainId": 137,
    "address": "0xdab35042e63e93cc8556c9bae482e5415b5ac4b1",
    "name": "Iris",
    "symbol": "IRIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18361/thumb/hermes.PNG?1696517854"
  },
  {
    "chainId": 137,
    "address": "0x05f52cc483c50c2a7e25a13dac17d736fa50f259",
    "name": "BitcoinX",
    "symbol": "BXC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31400/thumb/BXC.jpg?1700565614"
  },
  {
    "chainId": 137,
    "address": "0xb763f1177e9b2fb66fbe0d50372e3e2575c043e5",
    "name": "Karmaverse",
    "symbol": "KNOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24239/thumb/knot.png?1696523424"
  },
  {
    "chainId": 137,
    "address": "0xf2da37643c25ca14291c2a69f51423ed9b7ff4b0",
    "name": "Farmsent",
    "symbol": "FARMS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32241/thumb/FARMS.png?1696952898"
  },
  {
    "chainId": 137,
    "address": "0x23e8b6a3f6891254988b84da3738d2bfe5e703b9",
    "name": "Fabwelt",
    "symbol": "WELT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20505/thumb/welt.PNG?1696519911"
  },
  {
    "chainId": 137,
    "address": "0x958d208cdf087843e9ad98d23823d32e17d723a1",
    "name": "Dragon s Quick",
    "symbol": "DQUICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33890/thumb/QuickSwap-Logo-Full-Color.png?1703163481"
  },
  {
    "chainId": 137,
    "address": "0xb3bc8dae5a49ed956b5d2207d9040c816c56398f",
    "name": "Robinos  OLD ",
    "symbol": "RBN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18864/thumb/1vZaRdJ__400x400.png?1696518324"
  },
  {
    "chainId": 137,
    "address": "0x411bc96881a62572ff33c9d8ce60df99e3d96cd8",
    "name": "Mars Token",
    "symbol": "MRST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27230/thumb/MRST.png?1696526281"
  },
  {
    "chainId": 137,
    "address": "0x36596a1dc57c695bed1a063470a7802797dca133",
    "name": "Umma Token",
    "symbol": "UMMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31681/thumb/UMMA_200x200.png?1696530500"
  },
  {
    "chainId": 137,
    "address": "0x09cad96bc28f55e9253cfb9a84a3a1ab79061e54",
    "name": "OSHI3",
    "symbol": "OSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32481/thumb/LOGO_Oshi_Token_200-200.png?1698287950"
  },
  {
    "chainId": 137,
    "address": "0xd3144ff5f388d36c0a445686c08540296d8b209b",
    "name": "WolfWorksDAO",
    "symbol": "WWD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24106/thumb/200px-cmc.png?1696523298"
  },
  {
    "chainId": 137,
    "address": "0x12a4cebf81f8671faf1ab0acea4e3429e42869e7",
    "name": "Homeety",
    "symbol": "HOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34358/thumb/Logo_Noir_Full_No_Shade.png?1704708069"
  },
  {
    "chainId": 137,
    "address": "0xafb6e8331355fae99c8e8953bb4c6dc5d11e9f3c",
    "name": "Arch Aggressive Portfolio",
    "symbol": "AAGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30638/thumb/AAGG.png?1696529510"
  },
  {
    "chainId": 137,
    "address": "0xc56d17dd519e5eb43a19c9759b5d5372115220bd",
    "name": "Polywolf",
    "symbol": "MOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16034/thumb/moon-black.png?1696515644"
  },
  {
    "chainId": 137,
    "address": "0xd9d77b113e6ef2696267e20c35f3abd87cdaca2c",
    "name": "TanPin",
    "symbol": "TANPIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34840/thumb/token-tanpin-200px.png?1706357520"
  },
  {
    "chainId": 137,
    "address": "0x7e4c577ca35913af564ee2a24d882a4946ec492b",
    "name": "MoonEdge",
    "symbol": "MOONED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17393/thumb/ME_logo_200_200.png?1696516942"
  },
  {
    "chainId": 137,
    "address": "0xa2c638b78783e9afe26a16ec8b11de54eb169360",
    "name": "Pay It Now",
    "symbol": "PIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22482/thumb/PINlogo.png?1696521805"
  },
  {
    "chainId": 137,
    "address": "0x69166f825ecac553d5cdd85e936191d6191dfd57",
    "name": "Walk Up",
    "symbol": "WUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34713/thumb/1000005200.jpg?1711535367"
  },
  {
    "chainId": 137,
    "address": "0xd1f9c58e33933a993a3891f8acfe05a68e1afc05",
    "name": "Sunflower Land",
    "symbol": "SFL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25514/thumb/download.png?1696524647"
  },
  {
    "chainId": 137,
    "address": "0x0b6afe834dab840335f87d99b45c2a4bd81a93c7",
    "name": "Polylauncher",
    "symbol": "ANGEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17739/thumb/Polylauncher_-_200_x_200.png?1696517266"
  },
  {
    "chainId": 137,
    "address": "0xd57f8b6f3e5d1e0ab85118f5e0dd893a08c43346",
    "name": "Omnisea",
    "symbol": "OSEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26475/thumb/293837892_407994084681555_3167689470652146992_n.png?1696525547"
  },
  {
    "chainId": 137,
    "address": "0x87e28bd5499e6846141a85659bc7f0cf815db11d",
    "name": "Quantum Tech",
    "symbol": "QUA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22465/thumb/Pm9J4C3KD-5W41f2VH2IIWUY3gRCNJqTyu7X4SQt0ptuUYnhm68zPVBxTEgehAE_WSZBSz_TzxLfAWu8P7a8zOSZa8MpuqUmntsWVeK01s7IoEBERumgQq1mcTuxyOQXBH68OZBkC_hqVEiSf4vNw_EgbtCSATx6XTaLUmCO33kTaolgB5CU33MiINZkwvRDsexAQnxT7BWO6F.jpg?1696521787"
  },
  {
    "chainId": 137,
    "address": "0x7aefff599570dec2f3dbbc2ace3cb1f8206749eb",
    "name": "Moonflow",
    "symbol": "MOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34862/thumb/coin_big_white_logo_%281%29.png?1706354814"
  },
  {
    "chainId": 137,
    "address": "0x22f991e509102ed4c2babd82424c46bd3d203bd3",
    "name": "Map Node",
    "symbol": "MNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33013/thumb/logo_mapnode.png?1700124539"
  },
  {
    "chainId": 137,
    "address": "0xc2a45fe7d40bcac8369371b08419ddafd3131b4a",
    "name": "Lucidao",
    "symbol": "LCD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23693/thumb/200x200_LUCIDAO_TOKEN_BLUE.png?1700936891"
  },
  {
    "chainId": 137,
    "address": "0x823cd4264c1b951c9209ad0deaea9988fe8429bf",
    "name": "Matic Aave Interest Bearing AAVE",
    "symbol": "MAAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14077/thumb/maAAVE.png?1696513800"
  },
  {
    "chainId": 137,
    "address": "0xeb51d9a39ad5eef215dc0bf39a8821ff804a0f01",
    "name": "Origin LGNS",
    "symbol": "LGNS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36514/thumb/1000000998.jpg?1711693105"
  },
  {
    "chainId": 137,
    "address": "0xfddba7ca7af59ac1b3cc144c08db9bdde7378dd5",
    "name": "Rubber Ducky",
    "symbol": "RUBBER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36540/thumb/RubberDuckyLogo.png?1711708773"
  },
  {
    "chainId": 137,
    "address": "0x081c280cd5c1c934dbafc97a2ab2e596ffe7dfc4",
    "name": "UFORIKA",
    "symbol": "FORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34243/thumb/TOKEN_VER1_GREEN_FRONT_FINAL-Square-min-compressed.png?1704337681"
  },
  {
    "chainId": 137,
    "address": "0x8a953cfe442c5e8855cc6c61b1293fa648bae472",
    "name": "PolyDoge",
    "symbol": "POLYDOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15146/thumb/p1kSco1h_400x400.jpg?1696514802"
  },
  {
    "chainId": 137,
    "address": "0x27842334c55c01ddfe81bf687425f906816c5141",
    "name": "Veloce",
    "symbol": "VEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31214/thumb/VEXT_Logo.jpg?1696530041"
  },
  {
    "chainId": 137,
    "address": "0xa3d315e8e0a62e9abd4a910d07b4768a1b3ff525",
    "name": "AGOS",
    "symbol": "AGOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31902/thumb/GT_200x200.png?1696530712"
  },
  {
    "chainId": 137,
    "address": "0x96f0a7167ab7ba6e59ffb68707c9d1b049524b0f",
    "name": "Defi World",
    "symbol": "DWC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34154/thumb/Defi_Logo_Icon.png?1704210359"
  },
  {
    "chainId": 137,
    "address": "0x33b6d77c607ea499ab5db7e2201c5a516a78a5db",
    "name": "Aimedis  NEW ",
    "symbol": "AIMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29252/thumb/Aimedis_Logo_tg.png?1696528207"
  },
  {
    "chainId": 137,
    "address": "0x181feaecca4a69a793272ea06df40edf2dd0804c",
    "name": "Ares3 Network",
    "symbol": "ARES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29713/thumb/logo_200.png?1696528645"
  },
  {
    "chainId": 137,
    "address": "0x8df3aad3a84da6b69a4da8aec3ea40d9091b2ac4",
    "name": "Aave Polygon WMATIC",
    "symbol": "AMWMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17267/thumb/amWMATIC_2x.png?1696516822"
  },
  {
    "chainId": 137,
    "address": "0x9111d6446ac5b88a84cf06425c6286658368542f",
    "name": "For Loot And Glory",
    "symbol": "FLAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20726/thumb/token_logo.ico?1696520124"
  },
  {
    "chainId": 137,
    "address": "0xd3f07ea86ddf7baebefd49731d7bbd207fedc53b",
    "name": "Polly DeFi Nest",
    "symbol": "NDEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18317/thumb/nDEFI.png?1696517808"
  },
  {
    "chainId": 137,
    "address": "0x5dfd5edfde4d8ec9e632dca9d09fc7e833f74210",
    "name": "Infinity Skies",
    "symbol": "ISKY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22583/thumb/isky.png?1696521901"
  },
  {
    "chainId": 137,
    "address": "0xaa9654becca45b5bdfa5ac646c939c62b527d394",
    "name": "DinoSwap",
    "symbol": "DINO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17103/thumb/DINO.png?1696516664"
  },
  {
    "chainId": 137,
    "address": "0x0c1eb50007974c4e017a6eccf751110bac287ebb",
    "name": "Metable",
    "symbol": "MTBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31594/thumb/MEtable.jpg?1696530411"
  },
  {
    "chainId": 137,
    "address": "0x2ab0e9e4ee70fff1fb9d67031e44f6410170d00e",
    "name": "Xen Crypto  MATIC ",
    "symbol": "MXEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30005/thumb/1_uMvUPqIbGBqjBL6GfZpp3g.jpg?1696528930"
  },
  {
    "chainId": 137,
    "address": "0xc004e2318722ea2b15499d6375905d75ee5390b8",
    "name": "Kommunitas",
    "symbol": "KOM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17483/thumb/1_f1S3h57YLT1e1cl8g7RJpw_2x.jpeg?1696517024"
  },
  {
    "chainId": 137,
    "address": "0xac0f66379a6d7801d7726d5a943356a172549adb",
    "name": "Geodnet",
    "symbol": "GEOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31608/thumb/Circular_White.png?1696530424"
  },
  {
    "chainId": 137,
    "address": "0x05089c9ebffa4f0aca269e32056b1b36b37ed71b",
    "name": "Polywhale",
    "symbol": "KRILL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15016/thumb/2.png?1696514678"
  },
  {
    "chainId": 137,
    "address": "0x3f374ed3c8e61a0d250f275609be2219005c021e",
    "name": "Arcadium",
    "symbol": "ARCADIUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17717/thumb/4dyLxHe.png?1696517244"
  },
  {
    "chainId": 137,
    "address": "0xa4335da338ec4c07c391fc1a9bf75f306adadc08",
    "name": "ARYZE eUSD",
    "symbol": "EUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32422/thumb/ARYZE_eUSD.png?1698118313"
  },
  {
    "chainId": 137,
    "address": "0x200c234721b5e549c3693ccc93cf191f90dc2af9",
    "name": "Badmad Robots",
    "symbol": "METAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24376/thumb/metal.png?1696523559"
  },
  {
    "chainId": 137,
    "address": "0x1a70807f022cbd3ac5380a6a2af3524e56a6184d",
    "name": "Strix",
    "symbol": "STRIX",
    "decimals": 18
  },
  {
    "chainId": 137,
    "address": "0xf239e69ce434c7fb408b05a0da416b14917d934e",
    "name": "PolyShield",
    "symbol": "SHI3LD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17641/thumb/QRE5xI0.png?1696517172"
  },
  {
    "chainId": 137,
    "address": "0x8c069b6cef3453babb15fa97784affeb95d398fa",
    "name": "People s Money PMXX",
    "symbol": "PMXX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26722/thumb/WhatsApp_Image_2022-08-14_at_11.21.14_AM.jpeg?1696525792"
  },
  {
    "chainId": 137,
    "address": "0xb3bec36e9fb41c88b0f0433a3e5f16cf0dd7e0a3",
    "name": "UZXCoin",
    "symbol": "UZX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33056/thumb/Group-2.png?1700476300"
  },
  {
    "chainId": 137,
    "address": "0x1fd6cf265fd3428f655378a803658942095b4c4e",
    "name": "PolyYeld",
    "symbol": "YELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16401/thumb/pxSN2UtB_400x400.jpg?1696515998"
  },
  {
    "chainId": 137,
    "address": "0x80ca0d8c38d2e2bcbab66aa1648bd1c7160500fe",
    "name": "Aave v3 MaticX",
    "symbol": "AMATICX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32920/thumb/maticx.png?1699825663"
  },
  {
    "chainId": 137,
    "address": "0x8437d7c167dfb82ed4cb79cd44b7a32a1dd95c77",
    "name": "Aave v3 agEUR",
    "symbol": "AAGEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32925/thumb/eur.png?1699826217"
  },
  {
    "chainId": 137,
    "address": "0xaa4fbc6809a8e1924520fc85282ac4c76a7671d7",
    "name": "The Unfettered Ecosystem",
    "symbol": "SOULS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30591/thumb/un.jpg?1696529459"
  },
  {
    "chainId": 137,
    "address": "0x8105f88e77a5d102099bf73db4469d3f1e3b0cd6",
    "name": "JennyCo",
    "symbol": "JCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32006/thumb/jenny.jpg?1696530804"
  },
  {
    "chainId": 137,
    "address": "0x3a3df212b7aa91aa0402b9035b098891d276572b",
    "name": "Polycat Finance",
    "symbol": "FISH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15226/thumb/smallLogo.png?1696514881"
  },
  {
    "chainId": 137,
    "address": "0xf86df9b91f002cfeb2aed0e6d05c4c4eaef7cf02",
    "name": "Porygon",
    "symbol": "PORY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34066/thumb/pory_logo.jpg?1705258081"
  },
  {
    "chainId": 137,
    "address": "0xd0ac5be680bfbc0f1958413f4201f91934f61bea",
    "name": "MILO",
    "symbol": "MILO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32183/thumb/MILO.jpg?1696743309"
  },
  {
    "chainId": 137,
    "address": "0x1e37b3855ca1ef46106baa162bfaf8a1e7666a5d",
    "name": "Tamkin",
    "symbol": "TSLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35956/thumb/Tamkin_S_%282%29.png?1710253564"
  },
  {
    "chainId": 137,
    "address": "0xf9d3d8b25b95bcda979025b74fdfa7ac3f380f9f",
    "name": "Cookies Protocol",
    "symbol": "CP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28432/thumb/IMG_20221207_200805_897.jpg?1696527429"
  },
  {
    "chainId": 137,
    "address": "0x3ce1327867077b551ae9a6987bf10c9fd08edce1",
    "name": "SwissCheese",
    "symbol": "SWCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31344/thumb/IMG_7198.jpeg?1696530162"
  },
  {
    "chainId": 137,
    "address": "0xb2c63830d4478cb331142fac075a39671a5541dc",
    "name": "Bombcrypto Coin",
    "symbol": "BOMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26399/thumb/Bombcrypto_Coin.png?1696525476"
  },
  {
    "chainId": 137,
    "address": "0xd4cce747e623ce2d72322892e5db238e2a5eb4f3",
    "name": "HumansCareFoundationWater",
    "symbol": "HCFW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32645/thumb/hcfw_200_%282%29.png?1698893125"
  },
  {
    "chainId": 137,
    "address": "0x9c17d36bccecafb7d284e8b3d985576d21cf2350",
    "name": "OBS World",
    "symbol": "OBSW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35434/thumb/OBS_WORLD_LOGO.jpg?1708589491"
  },
  {
    "chainId": 137,
    "address": "0x03f61137bfb86be07394f0fd07a33984020f96d8",
    "name": "Xpendium",
    "symbol": "XPND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24520/thumb/2XyH_HyB_400x400.jpg?1696523699"
  },
  {
    "chainId": 137,
    "address": "0xef6ab48ef8dfe984fab0d5c4cd6aff2e54dfda14",
    "name": "CRISP Scored Mangroves",
    "symbol": "CRISP-M",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30555/thumb/CrispM_small.png?1696529426"
  },
  {
    "chainId": 137,
    "address": "0x9b760d11f63cb80bffbcf69421cd46511e429f83",
    "name": "LFi",
    "symbol": "LFI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31897/thumb/LFi_Token.png?1696530708"
  },
  {
    "chainId": 137,
    "address": "0xb7f1219db39ea0cb029e4dcc3daffdcfd233defd",
    "name": "KeySATIN",
    "symbol": "KEYSATIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29703/thumb/1-01.png?1696528635"
  },
  {
    "chainId": 137,
    "address": "0x0835cdd017ea7bc4cc187c6e0f8ea2dbe0fea0dd",
    "name": "KOHENOOR",
    "symbol": "KEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33880/thumb/Screenshot_2023_1126_101414.jpg?1703146967"
  },
  {
    "chainId": 137,
    "address": "0x04f177fcacf6fb4d2f95d41d7d3fee8e565ca1d0",
    "name": "DragonMaster",
    "symbol": "DMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25171/thumb/JNBofoc.png?1696524316"
  },
  {
    "chainId": 137,
    "address": "0xe118e8b6dc166cd83695825eb1d30e792435bb00",
    "name": "FireBall",
    "symbol": "FIRE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/15644/thumb/fireball.png?1696515276"
  },
  {
    "chainId": 137,
    "address": "0x0308a3a9c433256ad7ef24dbef9c49c8cb01300a",
    "name": "GoldPesa Option",
    "symbol": "GPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23813/thumb/15462.png?1696523016"
  },
  {
    "chainId": 137,
    "address": "0x6d830a664e6aa9aab976ab66258eecd0b5fb1dc7",
    "name": "Mammoth",
    "symbol": "WOOLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34742/thumb/Wooly_200-200_PNG.png?1705948563"
  },
  {
    "chainId": 137,
    "address": "0xc7d73cb22add1f2019d68ec4fa033f3e3d622c0e",
    "name": "Huralya",
    "symbol": "LYA",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34595/thumb/200.png?1705482533"
  },
  {
    "chainId": 137,
    "address": "0xe20b9e246db5a0d21bf9209e4858bc9a3ff7a034",
    "name": "Wrapped Banano",
    "symbol": "WBAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32617/thumb/WBAN.jpg?1698749253"
  },
  {
    "chainId": 137,
    "address": "0x372246175d50db4fd42c2aba4e3292a0fe41ce2e",
    "name": "AMAUROT",
    "symbol": "AMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28229/thumb/10b06d6c-379f-4c34-b3f2-196a770a4512.png?1696527230"
  },
  {
    "chainId": 137,
    "address": "0x37b8e1152fb90a867f3dcca6e8d537681b04705e",
    "name": "Proto Gyro Dollar",
    "symbol": "P-GYD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29416/thumb/gyd.png?1696528365"
  },
  {
    "chainId": 137,
    "address": "0xa9c992952c2090a51506c4f3636c1320f8fa93fa",
    "name": "Cropto Hazelnut Token",
    "symbol": "CROF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35244/thumb/WhatsApp_Image_2024-02-09_at_16.40.34-2.jpeg?1707987736"
  },
  {
    "chainId": 137,
    "address": "0xf796969fa47fb0748c80b8b153cbb895e88cbd54",
    "name": "Ocavu Network",
    "symbol": "OCAVU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25781/thumb/ocavu.png?1696524868"
  },
  {
    "chainId": 137,
    "address": "0x0395e3178305e64ff9e8d942eb3a470b5d88fae8",
    "name": "Beebox",
    "symbol": "XBBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31811/thumb/XBBC_logo.png?1696530626"
  },
  {
    "chainId": 137,
    "address": "0x948d0a28b600bdbd77af4ea30e6f338167034181",
    "name": "NSHARE",
    "symbol": "NSHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21692/thumb/NF_nshare_2_200x200.png?1696521047"
  },
  {
    "chainId": 137,
    "address": "0x5f32abeebd3c2fac1e7459a27e1ae9f1c16cccca",
    "name": "FARCANA",
    "symbol": "FAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34690/thumb/logo_far_black_short.png?1705853311"
  },
  {
    "chainId": 137,
    "address": "0x00e8c0e92eb3ad88189e7125ec8825edc03ab265",
    "name": "Wrapped USDR",
    "symbol": "WUSDR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28800/thumb/wUSDRlogo.png?1696527777"
  },
  {
    "chainId": 137,
    "address": "0x724dc807b04555b71ed48a6896b6f41593b8c637",
    "name": "Aave v3 DPI",
    "symbol": "ADPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32926/thumb/dpi.png?1699826275"
  },
  {
    "chainId": 137,
    "address": "0x9af3b7dc29d3c4b1a5731408b6a9656fa7ac3b72",
    "name": "PUSD Polyquity",
    "symbol": "PUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16762/thumb/PUSD-purple-200.png?1696516335"
  },
  {
    "chainId": 137,
    "address": "0x9008d70a5282a936552593f410abcbce2f891a97",
    "name": "Penrose Finance",
    "symbol": "PEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26105/thumb/6iWxdkG.png?1696525196"
  },
  {
    "chainId": 137,
    "address": "0x6d9c0b104e5af90a6d11a13eb77288e533333301",
    "name": "MAXX Finance",
    "symbol": "MAXX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27203/thumb/maxxfinance.jpeg?1696526252"
  },
  {
    "chainId": 137,
    "address": "0x36ec6622b5227e3a7e4d085f1dfc1dc8f5192289",
    "name": "Greenercoin",
    "symbol": "GNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33977/thumb/200_X_200_GNC_LOGO.jpg?1703578885"
  },
  {
    "chainId": 137,
    "address": "0x90f3edc7d5298918f7bb51694134b07356f7d0c7",
    "name": "DDAO Hunters",
    "symbol": "DDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23018/thumb/New-Logo.jpg?1696522313"
  },
  {
    "chainId": 137,
    "address": "0xc6c855ad634dcdad23e64da71ba85b8c51e5ad7c",
    "name": "Decentral Games ICE",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18110/thumb/ice-poker.png?1696517614"
  },
  {
    "chainId": 137,
    "address": "0x77a6f2e9a9e44fd5d5c3f9be9e52831fc1c3c0a0",
    "name": "World tateCoin",
    "symbol": "WC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29636/thumb/wsc_polygon_logo_200.png?1696528573"
  },
  {
    "chainId": 137,
    "address": "0x9f95e17b2668afe01f8fbd157068b0a4405cc08d",
    "name": "Bullieverse",
    "symbol": "BULL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24174/thumb/KR3qVAQe_400x400.jpg?1696523362"
  },
  {
    "chainId": 137,
    "address": "0x3f6b3595ecf70735d3f48d69b09c4e4506db3f47",
    "name": "GameStation",
    "symbol": "GAMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19584/thumb/game_station.PNG?1696519015"
  },
  {
    "chainId": 137,
    "address": "0x3d2bd0e15829aa5c362a4144fdf4a1112fa29b5c",
    "name": "Bonsai Token",
    "symbol": "BONSAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35884/thumb/Bonsai_BW_Coingecko-200x200.jpg?1710071621"
  },
  {
    "chainId": 137,
    "address": "0x649a2da7b28e0d54c13d5eff95d3a660652742cc",
    "name": "IDRX",
    "symbol": "IDRX",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/34883/thumb/LOGO_COIN_2_4x.png?1706522103"
  },
  {
    "chainId": 137,
    "address": "0xf4c7da5d8f3b5190d6176bd240613e59665b3153",
    "name": "Gascoin",
    "symbol": "GCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31901/thumb/GAS_LOGO.jpg?1696530712"
  },
  {
    "chainId": 137,
    "address": "0x9eb8a789ed1bd38d281962b523349d5d17a37d47",
    "name": "Xion Finance",
    "symbol": "XGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14039/thumb/xgt.png?1696513765"
  },
  {
    "chainId": 137,
    "address": "0x60eec374a1ba3907e9bdd8a74ce368d041d89c79",
    "name": "Parrotly",
    "symbol": "PBIRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28219/thumb/PARROTLY_BRAND_ASSETS__PARROTLY_ICON_BLUE_-_Copy_-_Copy_-_Copy.png?1696527221"
  },
  {
    "chainId": 137,
    "address": "0x6a3e7c3c6ef65ee26975b12293ca1aad7e1daed2",
    "name": "Aavegotchi ALPHA",
    "symbol": "ALPHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24738/thumb/alpha.png?1696523901"
  },
  {
    "chainId": 137,
    "address": "0xe57594f829b3d514ee12c41e81df3861b19c26e3",
    "name": "Technology Metal Network Global",
    "symbol": "TMNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32834/thumb/TMN_Logo_dark_silver_2.PNG?1699586186"
  },
  {
    "chainId": 137,
    "address": "0xdfce1e99a31c4597a3f8a8945cbfa9037655e335",
    "name": "Astrafer",
    "symbol": "ASTRAFER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26246/thumb/ATSRA_Token.png?1696525331"
  },
  {
    "chainId": 137,
    "address": "0x3b9e9100db1389c518d47c635d80a90ad4c4f41b",
    "name": "Film io",
    "symbol": "FAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35118/thumb/fan-token-icon-flat.png?1712828150"
  },
  {
    "chainId": 137,
    "address": "0x87f654c4b347230c60cad8d7ea9cf0d7238bcc79",
    "name": "Yamp Finance",
    "symbol": "YAMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17301/thumb/0x87f654c4b347230C60CAD8d7ea9cF0D7238bcc79.png?1696516854"
  },
  {
    "chainId": 137,
    "address": "0x53df32548214f51821cf1fe4368109ac5ddea1ff",
    "name": "Sponge",
    "symbol": "SPONGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35467/thumb/sponge.jpeg?1708688227"
  },
  {
    "chainId": 137,
    "address": "0xdd9ba3b2571bea0854beb0508ce10fed0eca7e3e",
    "name": "EnviDa",
    "symbol": "EDAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20342/thumb/logo1.png?1696519749"
  },
  {
    "chainId": 137,
    "address": "0x65ad509db5924ef0001d977590985f965ef1aaaa",
    "name": "Meta Toy DragonZ SAGA  FXERC20 ",
    "symbol": "FXMETOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32859/thumb/Coin-Line_%281%29.png?1699669121"
  },
  {
    "chainId": 137,
    "address": "0x6002410dda2fb88b4d0dc3c1d562f7761191ea80",
    "name": "The Employment Commons Work",
    "symbol": "WORK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18546/thumb/mXSw8Qw.png?1696518025"
  },
  {
    "chainId": 137,
    "address": "0xcb898b0efb084df14dd8e018da37b4d0f06ab26d",
    "name": "Sing",
    "symbol": "SING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17682/thumb/200_%2810%29.png?1696517211"
  },
  {
    "chainId": 137,
    "address": "0xab3d689c22a2bb821f50a4ff0f21a7980dcb8591",
    "name": "Proxy",
    "symbol": "PRXY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17442/thumb/Proxy_logo_200.png?1696516988"
  },
  {
    "chainId": 137,
    "address": "0x13748d548d95d78a3c83fe3f32604b4796cffa23",
    "name": "KogeCoin",
    "symbol": "KOGECOIN",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17546/thumb/koge.PNG?1696517082"
  },
  {
    "chainId": 137,
    "address": "0x3af2dd7b91d8faceccc26d21a065267817213d37",
    "name": "Ribus",
    "symbol": "RIB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35111/thumb/icon-ribus-200.png?1707391243"
  },
  {
    "chainId": 137,
    "address": "0xfa3c05c2023918a4324fde7163591fe6bebd1692",
    "name": "Cresio",
    "symbol": "XCRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8032/thumb/logoXCRE.png?1696508254"
  },
  {
    "chainId": 137,
    "address": "0xb58458c52b6511dc723d7d6f3be8c36d7383b4a8",
    "name": "FrontFanz",
    "symbol": "FANX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33225/thumb/200x200_ff.png?1701100617"
  },
  {
    "chainId": 137,
    "address": "0x0c47298beee5203358e7bc30b9954b584361eab5",
    "name": "Black Stallion",
    "symbol": "BS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29207/thumb/bs.png?1696528166"
  },
  {
    "chainId": 137,
    "address": "0x40f97ec376ac1c503e755433bf57f21e3a49f440",
    "name": "CarrieVerse",
    "symbol": "CVTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28922/thumb/cvtx.png?1696527897"
  },
  {
    "chainId": 137,
    "address": "0x735fa792e731a2e8f83f32eb539841b7b72e6d8f",
    "name": "ARYZE eEUR",
    "symbol": "EEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32421/thumb/ARYZE_eEUR.png?1698118176"
  },
  {
    "chainId": 137,
    "address": "0x6e8a8726639d12935b3219892155520bdc57366b",
    "name": "GenomesDAO GNOME",
    "symbol": "GNOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20885/thumb/gnome.png?1696520278"
  },
  {
    "chainId": 137,
    "address": "0xcda6c923458ca9fac8e3354999e866feaa80b72f",
    "name": "Polygame",
    "symbol": "PGEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32349/thumb/polygame.jpg?1697471722"
  },
  {
    "chainId": 137,
    "address": "0x8a16d4bf8a0a716017e8d2262c4ac32927797a2f",
    "name": "ViciCoin",
    "symbol": "VCNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31305/thumb/ViciCoin_-_small.png?1696530124"
  },
  {
    "chainId": 137,
    "address": "0x486ffaf06a681bf22b5209e9ffce722662a60e8c",
    "name": "Flycoin FLY",
    "symbol": "FLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26596/thumb/flatArtboard_20tfr.png?1696525671"
  },
  {
    "chainId": 137,
    "address": "0x95093f8348c6678df4812c008248d88cad344069",
    "name": "War Legends",
    "symbol": "WAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30672/thumb/wZBMVBr2_400x400.jpg?1696529542"
  },
  {
    "chainId": 137,
    "address": "0xfcb54da3f4193435184f3f647467e12b50754575",
    "name": "Smell",
    "symbol": "SML",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32191/thumb/SML.jpg?1696744491"
  },
  {
    "chainId": 137,
    "address": "0xbd8005612124dc30601e22d8b5d188a89767c640",
    "name": "Exohood",
    "symbol": "EXO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15590/thumb/LOGO_200x200_CIRCLE.png?1696515225"
  },
  {
    "chainId": 137,
    "address": "0x245e5ddb65efea6522fa913229df1f4957fb2e21",
    "name": "LoserChick EGG",
    "symbol": "EGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17574/thumb/lRAQ2MOTpWqrHhI.png?1696517109"
  },
  {
    "chainId": 137,
    "address": "0x04b33078ea1aef29bf3fb29c6ab7b200c58ea126",
    "name": "Safle",
    "symbol": "SAFLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21674/thumb/56835021.png?1696521030"
  },
  {
    "chainId": 137,
    "address": "0x61f95bd637e3034133335c1baa0148e518d438ad",
    "name": "MetaShooter",
    "symbol": "MHUNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24985/thumb/200x200.png?1696524137"
  },
  {
    "chainId": 137,
    "address": "0xc61f39418cd27820b5d4e9ba4a7197eefaeb8b05",
    "name": "Tamadoge",
    "symbol": "TAMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27501/thumb/CaltNDWu_400x400.jpeg?1696526541"
  },
  {
    "chainId": 137,
    "address": "0xe8d17b127ba8b9899a160d9a07b69bca8e08bfc6",
    "name": "NASDEX",
    "symbol": "NSDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18634/thumb/nadex.PNG?1696518106"
  },
  {
    "chainId": 137,
    "address": "0xd7bb095a60d7666d4a6f236423b47ddd6ae6cfa7",
    "name": "Bridged Wrapped stETH  Axelar ",
    "symbol": "AXL-WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32223/thumb/steth-wei_D_3x.png?1696926841"
  },
  {
    "chainId": 137,
    "address": "0x0f8db146ec1288fc35c05e3b78c2f27d3fa7c4a5",
    "name": "Tradeleaf",
    "symbol": "TLF",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29718/thumb/TLF_200x200.png?1696528649"
  },
  {
    "chainId": 137,
    "address": "0xe2c1efc7322bc478285e39820aec9b79275e521c",
    "name": "Loop Of Infinity",
    "symbol": "LOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35587/thumb/lop.jpeg?1709203782"
  },
  {
    "chainId": 137,
    "address": "0x27f8d03b3a2196956ed754badc28d73be8830a6e",
    "name": "Aave Polygon DAI",
    "symbol": "AMDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17251/thumb/amDAI_2x.png?1696516807"
  },
  {
    "chainId": 137,
    "address": "0x1adcef5c780d8895ac77e6ee9239b4b3ecb76da2",
    "name": "DragonMaster Totem",
    "symbol": "TOTEM",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25056/thumb/C6_miUEU_400x400.jpg?1696524207"
  },
  {
    "chainId": 137,
    "address": "0xf972daced7c6b03223710c11413036d17eb298f6",
    "name": "Impermax",
    "symbol": "IBEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27606/thumb/IqwOmX-c_400x400.jpeg?1696526637"
  },
  {
    "chainId": 137,
    "address": "0xa25610a77077390a75ad9072a084c5fbc7d43a0d",
    "name": "Monsoon Finance",
    "symbol": "MCASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18632/thumb/DdcZZeP_%282%29.png?1696518104"
  },
  {
    "chainId": 137,
    "address": "0xd0cfd20e8bbdb7621b705a4fd61de2e80c2cd02f",
    "name": "Safeswap SSGTX",
    "symbol": "SSGTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20769/thumb/GBpj6TpI.png?1696520163"
  },
  {
    "chainId": 137,
    "address": "0x6d969cea201e427d2875724fd4e8044833fbc7f4",
    "name": "Polygon HBD",
    "symbol": "PHBD",
    "decimals": 3,
    "logoURI": "https://assets.coingecko.com/coins/images/25142/thumb/image.png?1696524291"
  },
  {
    "chainId": 137,
    "address": "0x503836c8c3a453c57f58cc99b070f2e78ec14fc0",
    "name": "SPORT",
    "symbol": "SPORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25823/thumb/SPORT_LOGO_PNG.png?1696524908"
  },
  {
    "chainId": 137,
    "address": "0x3efcd659b7a45d14dda8a102836ce4b765c42324",
    "name": "GIB",
    "symbol": "GIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35292/thumb/_2_mascot.jpg?1708076516"
  },
  {
    "chainId": 137,
    "address": "0x88c949b4eb85a90071f2c0bef861bddee1a7479d",
    "name": "Sheesha Finance Polygon",
    "symbol": "MSHEESHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23494/thumb/sheesha.PNG?1696522704"
  },
  {
    "chainId": 137,
    "address": "0x182f1d39df9460d7aef29afbc80bbd68ed0a41d5",
    "name": "RuufCoin",
    "symbol": "RUUF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17968/thumb/44d23204-3919-4ba6-adaf-69a51b9621d8.png?1696517487"
  },
  {
    "chainId": 137,
    "address": "0x8f15330737184643473348004ef70c1c57c2966d",
    "name": "Web3Games com Token",
    "symbol": "WGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34449/thumb/_WGT_token.png?1704899852"
  },
  {
    "chainId": 137,
    "address": "0x39ab6574c289c3ae4d88500eec792ab5b947a5eb",
    "name": "Dystopia",
    "symbol": "DYST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25786/thumb/dystopialogo.jpg?1696524872"
  },
  {
    "chainId": 137,
    "address": "0x9c891326fd8b1a713974f73bb604677e1e63396d",
    "name": "ISLAMICOIN",
    "symbol": "ISLAMI",
    "decimals": 7,
    "logoURI": "https://assets.coingecko.com/coins/images/24749/thumb/3100-x-3100-px-e1632233028219.png?1696523911"
  },
  {
    "chainId": 137,
    "address": "0xb988bd378a0754957d5d9471c96e0f8051645a26",
    "name": "iNFTspace",
    "symbol": "INS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24681/thumb/4hTuPCCC_400x400.jpg?1696523848"
  },
  {
    "chainId": 137,
    "address": "0x2f800db0fdb5223b3c3f354886d907a671414a7f",
    "name": "Toucan Protocol  Base Carbon Tonne",
    "symbol": "BCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19240/thumb/bct_exchange_enhanced.png?1696518686"
  },
  {
    "chainId": 137,
    "address": "0xbd9c89536e406478d8a42c51b35bcfb6bf8a1384",
    "name": "GrabCoinClub",
    "symbol": "GC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30742/thumb/GrabCoinClub.png?1696529612"
  },
  {
    "chainId": 137,
    "address": "0x51b5619f5180e333d18b6310c8d540aea43a0371",
    "name": "Vault Hill City",
    "symbol": "VHC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23120/thumb/vhclogo.3929f10d.png?1696522411"
  },
  {
    "chainId": 137,
    "address": "0xa1fd25f9d59768dfaa376b25a46df2ab2729fb83",
    "name": "5mc",
    "symbol": "5MC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32862/thumb/5mc_200X200.png?1699670007"
  },
  {
    "chainId": 137,
    "address": "0x13efaa3d5e2c86e7aba168f925fef9827648cff4",
    "name": "MAGIC SHOES",
    "symbol": "MCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31088/thumb/MAGIC_SHOES.png?1696529921"
  },
  {
    "chainId": 137,
    "address": "0x4e67b4598bce5f0b2706407407da0d4d8e1d7433",
    "name": "Aventis Metaverse",
    "symbol": "AVTM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33155/thumb/Logo_Only_%28Small%29.png?1705310155"
  },
  {
    "chainId": 137,
    "address": "0x7d24903b4c4c81456cf9bf66b73ce95dc6914950",
    "name": "VitnixX",
    "symbol": "VTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33863/thumb/VitnixX_Logo_PNG.png?1703129818"
  },
  {
    "chainId": 137,
    "address": "0x0059ca765cc75a9b977293681a1b8add0424f5f8",
    "name": "zbyte",
    "symbol": "DPLAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34210/thumb/200x200logo-2.png?1704579873"
  },
  {
    "chainId": 137,
    "address": "0x3e15cd00b456b0fb33827e3c9b49952bb0ec126c",
    "name": "Hibiki Run",
    "symbol": "HUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32011/thumb/hibiki.png?1696530809"
  },
  {
    "chainId": 137,
    "address": "0x346404079b3792a6c548b072b9c4dddfb92948d5",
    "name": "GeniuX",
    "symbol": "IUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26078/thumb/black_IUX-coin-logo2.png?1696525162"
  },
  {
    "chainId": 137,
    "address": "0x7614bd00e0560ec9214e4fcb36d5ae23b3ecd3c5",
    "name": "Koop360",
    "symbol": "KOOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32312/thumb/KOOP_Logo.png?1697189111"
  },
  {
    "chainId": 137,
    "address": "0x7a6168e5a0ecc9841ad8fad20dcfcc4458fef0fb",
    "name": "Epillo",
    "symbol": "EPILLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28846/thumb/2CyKXwGy_400x400.jpg?1696527822"
  },
  {
    "chainId": 137,
    "address": "0x7238390d5f6f64e67c3211c343a410e2a3dec142",
    "name": "Pearl",
    "symbol": "PEARL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30799/thumb/Yp9H3agr_400x400.jpg?1696529660"
  },
  {
    "chainId": 137,
    "address": "0xfc3559771aa5a0666420f2365cf66139fbf4b37f",
    "name": "Stasis Network",
    "symbol": "STS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30922/thumb/STS.png?1696529765"
  },
  {
    "chainId": 137,
    "address": "0xf287d97b6345bad3d88856b26fb7c0ab3f2c7976",
    "name": "Index Coop   MATIC 2x Flexible Leverage",
    "symbol": "MATIC2X-FLI-P",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23388/thumb/MATIC2x-FLI.png?1696522602"
  },
  {
    "chainId": 137,
    "address": "0xb6a5ae40e79891e4deadad06c8a7ca47396df21c",
    "name": "Carbify",
    "symbol": "CBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29892/thumb/Carbify_logo.png?1696528816"
  },
  {
    "chainId": 137,
    "address": "0x3ad707da309f3845cd602059901e39c4dcd66473",
    "name": "Index Coop   ETH 2x Flexible Leverage I",
    "symbol": "ETH2X-FLI-P",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21183/thumb/ETH2x-FLO_token_logo.jpg?1696520559"
  },
  {
    "chainId": 137,
    "address": "0x22e3f02f86bc8ea0d73718a2ae8851854e62adc5",
    "name": "FireStarter",
    "symbol": "FLAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17359/thumb/WhiteOnBlack_Primary_Logo.png?1696516910"
  },
  {
    "chainId": 137,
    "address": "0x62f594339830b90ae4c084ae7d223ffafd9658a7",
    "name": "Sphere Finance",
    "symbol": "SPHERE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24424/thumb/2iR2JsL.png?1696523606"
  },
  {
    "chainId": 137,
    "address": "0x9a4eb698e5de3d3df0a68f681789072de1e50222",
    "name": "Fidira",
    "symbol": "FID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19446/thumb/nl0qESy3_400x400.jpg?1696518884"
  },
  {
    "chainId": 137,
    "address": "0x656bf6767fa8863ac0dd0b7d2a26602b838a2e70",
    "name": "Fitmint",
    "symbol": "FITT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27021/thumb/logo_black.png?1696526073"
  },
  {
    "chainId": 137,
    "address": "0x162539172b53e9a93b7d98fb6c41682de558a320",
    "name": "Gone",
    "symbol": "GONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33916/thumb/7BA9468A-34AE-4F10-BA78-655A1CC2D328-removebg-preview.png?1703313577"
  },
  {
    "chainId": 137,
    "address": "0x6ae96cc93331c19148541d4d2f31363684917092",
    "name": "Caviar",
    "symbol": "CVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31354/thumb/caviar-token_200x200.png?1696530171"
  },
  {
    "chainId": 137,
    "address": "0x49e6a20f1bbdfeec2a8222e052000bbb14ee6007",
    "name": "Tangible",
    "symbol": "TNGBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25428/thumb/TNGBL.png?1696524558"
  },
  {
    "chainId": 137,
    "address": "0x972999c58bbce63a2e398d4ed3bde414b8349eb3",
    "name": "Purpose",
    "symbol": "PRPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2539/thumb/purpose.png?1696503350"
  },
  {
    "chainId": 137,
    "address": "0xdd2af2e723547088d3846841fbdcc6a8093313d6",
    "name": "GOGOcoin",
    "symbol": "GOGO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21341/thumb/gogo.png?1696520708"
  },
  {
    "chainId": 137,
    "address": "0x2cb7285733a30bb08303b917a7a519c88146c6eb",
    "name": "KUMA Protocol FR KUMA Interest Bearing ",
    "symbol": "FRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33447/thumb/FRK.png?1701888548"
  },
  {
    "chainId": 137,
    "address": "0x94b959c93761835f634b8d6e655070c58e2caa12",
    "name": "MetaHub Finance",
    "symbol": "MEN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33255/thumb/metahub-logo-200_200.png?1704197960"
  },
  {
    "chainId": 137,
    "address": "0xe5417af564e4bfda1c483642db72007871397896",
    "name": "Gains Network",
    "symbol": "GNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19737/thumb/logo.png?1696519161"
  },
  {
    "chainId": 137,
    "address": "0x304243a820d4a3718becc89a3f33513586162cf0",
    "name": "Cri3x",
    "symbol": "CRI3X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29944/thumb/200x200_logo.png?1696528871"
  },
  {
    "chainId": 137,
    "address": "0x56633733fc8baf9f730ad2b6b9956ae22c6d4148",
    "name": "CoinCollect",
    "symbol": "COLLECT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35378/thumb/200logo-1.png?1708420838"
  },
  {
    "chainId": 137,
    "address": "0xd6e274f6899ed9d1ea49fdd1beaae9dcdf3081a6",
    "name": "Ninja Warriors",
    "symbol": "NWT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32483/thumb/logo.png?1698288214"
  },
  {
    "chainId": 137,
    "address": "0x27ab6e82f3458edbc0703db2756391b899ce6324",
    "name": "Reental",
    "symbol": "RNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32508/thumb/RNT-crypto-icon_1.png?1698397364"
  },
  {
    "chainId": 137,
    "address": "0xd86b5923f3ad7b585ed81b448170ae026c65ae9a",
    "name": "Iron",
    "symbol": "IRON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14588/thumb/logo_-_2021-03-31T123525.615.png?1696514268"
  },
  {
    "chainId": 137,
    "address": "0x67eb41a14c0fe5cd701fc9d5a3d6597a72f641a6",
    "name": "Giddy",
    "symbol": "GIDDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24960/thumb/gddy.png?1696524114"
  },
  {
    "chainId": 137,
    "address": "0x9dbfc1cbf7a1e711503a29b4b5f9130ebeccac96",
    "name": "UpOnly",
    "symbol": "UPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24900/thumb/f-WNwLNJ_400x400.jpg?1696524058"
  },
  {
    "chainId": 137,
    "address": "0x5c2ed810328349100a66b82b78a1791b101c9d61",
    "name": "Aave Polygon WBTC",
    "symbol": "AMWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17265/thumb/amWBTC_2x.png?1696516820"
  },
  {
    "chainId": 137,
    "address": "0xc2132d05d31c914a87c6611c10748aeb04b58e8f",
    "name": "Polygon Bridged USDT  Polygon ",
    "symbol": "USDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35023/thumb/USDT.png?1707233644"
  },
  {
    "chainId": 137,
    "address": "0x329f5e8aff351327e63acdb264389c798a46c2d3",
    "name": "PolyGamma Finance",
    "symbol": "GAMMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19590/thumb/gamma.png?1696519021"
  },
  {
    "chainId": 137,
    "address": "0x998013798440d44c1dd4c767bdd9580e16e46e28",
    "name": "ITSBLOC",
    "symbol": "ITSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27268/thumb/ezgif-2-1826f94a8a.jpeg?1696526320"
  },
  {
    "chainId": 137,
    "address": "0x9b8b6a1298d34b3c4bbddce8a7ff0149121592c1",
    "name": "AMBO",
    "symbol": "AMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36481/thumb/SHEERTOPIA-TOKEN.png?1711532697"
  },
  {
    "chainId": 137,
    "address": "0x62395ec568c92973a38230de209abba9de18b9b7",
    "name": "XP",
    "symbol": "T3XP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35609/thumb/XPtoken1Logo.png?1709979704"
  },
  {
    "chainId": 137,
    "address": "0xd6a33f67b733d422c821c36f0f79ca145b930d01",
    "name": "Angola",
    "symbol": "AGLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27547/thumb/8SGgovDI_400x400.png?1696526584"
  },
  {
    "chainId": 137,
    "address": "0x4c665bbafd28ec9e5d792345f470ebfca21e3d15",
    "name": "SKYPlay",
    "symbol": "SKP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27129/thumb/SKP_logo_20x20.png?1696526181"
  },
  {
    "chainId": 137,
    "address": "0x823bbb870b0eb86bd7ec0bcc98c84b46a0f99ac7",
    "name": "Voy Finance",
    "symbol": "VOY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30304/thumb/Profile_Picture_2.png?1696529207"
  },
  {
    "chainId": 137,
    "address": "0x28424507fefb6f7f8e9d3860f56504e4e5f5f390",
    "name": "Aave Polygon WETH",
    "symbol": "AMWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17266/thumb/amWETH_2x.png?1696516821"
  },
  {
    "chainId": 137,
    "address": "0x228b5c21ac00155cf62c57bcc704c0da8187950b",
    "name": "Nexus Dubai",
    "symbol": "NXD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23414/thumb/nexus.PNG?1696522627"
  },
  {
    "chainId": 137,
    "address": "0xf9b2ebdfa24688f0dfb12cc55782e635f91e64a2",
    "name": "Cleo Tech",
    "symbol": "CLEO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33662/thumb/cleotechlogo.jpeg?1702601297"
  },
  {
    "chainId": 137,
    "address": "0x47dae46d31f31f84336ac120b15efda261d484fb",
    "name": "Bund V2",
    "symbol": "BUND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13219/thumb/IMG_20230201_114958_189.png?1696512998"
  },
  {
    "chainId": 137,
    "address": "0x21a77520d0a25eede18a34ad4bc7a769d785c6ff",
    "name": "Stars",
    "symbol": "SRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31157/thumb/Stars.jpg?1696529985"
  },
  {
    "chainId": 137,
    "address": "0xc59132fbdf8de8fbe510f568a5d831c991b4fc38",
    "name": "Borderless Money",
    "symbol": "BOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28399/thumb/Logo_BOM_200x200.png?1696527398"
  },
  {
    "chainId": 137,
    "address": "0x6c526368e1185e4cd8a50468eb3429c5aeb674f3",
    "name": "Selfbar",
    "symbol": "SBAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20599/thumb/sbar-icon.png?1696520005"
  },
  {
    "chainId": 137,
    "address": "0x8f9e8e833a69aa467e42c46cca640da84dd4585f",
    "name": "NFT Champions",
    "symbol": "CHAMP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/19536/thumb/champ.png?1696518969"
  },
  {
    "chainId": 137,
    "address": "0x202655af326de310491cb54f120e02ee0da92b55",
    "name": "Creta World",
    "symbol": "CRETA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29129/thumb/colour_icon.png?1696528090"
  },
  {
    "chainId": 137,
    "address": "0xf5ea626334037a2cf0155d49ea6462fddc6eff19",
    "name": "PolygonFarm Finance",
    "symbol": "SPADE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17364/thumb/Polygon-Farm-Finance-Logo-200x200.png?1696516915"
  },
  {
    "chainId": 137,
    "address": "0x403e967b044d4be25170310157cb1a4bf10bdd0f",
    "name": "Aavegotchi FUD",
    "symbol": "FUD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24736/thumb/fud.png?1696523899"
  },
  {
    "chainId": 137,
    "address": "0x70e29b7e036b14d496431b77e0b6eb0008be6165",
    "name": "Captain Tsubasa",
    "symbol": "TSUGT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31757/thumb/TSUBASAGT.png?1696530576"
  },
  {
    "chainId": 137,
    "address": "0xf868939ee81f04f463010bc52eab91c0839ef08c",
    "name": "Attack Wagon",
    "symbol": "ATK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21333/thumb/attack.PNG?1696520700"
  },
  {
    "chainId": 137,
    "address": "0x87d6f8edeccbcca766d2880d19b2c3777d322c22",
    "name": "Miracle Play",
    "symbol": "MPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32653/thumb/MPT.png?1698895300"
  },
  {
    "chainId": 137,
    "address": "0x40379a439d4f6795b6fc9aa5687db461677a2dba",
    "name": "Real USD",
    "symbol": "USDR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/27973/thumb/USDR-200x200.png?1696526992"
  },
  {
    "chainId": 137,
    "address": "0x76e63a3e7ba1e2e61d3da86a87479f983de89a7e",
    "name": "Augury Finance",
    "symbol": "OMEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16573/thumb/omen.PNG?1696516134"
  },
  {
    "chainId": 137,
    "address": "0x431cd3c9ac9fc73644bf68bf5691f4b83f9e104f",
    "name": "Rainbow Token",
    "symbol": "RBW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24498/thumb/icon_rbw.png?1696523678"
  },
  {
    "chainId": 137,
    "address": "0xe26cda27c13f4f87cffc2f437c5900b27ebb5bbb",
    "name": "Rebel Bots",
    "symbol": "RBLS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/24094/thumb/logo-footer-356.88b37f33.png?1696523287"
  },
  {
    "chainId": 137,
    "address": "0xa7aac53f7c860b1f9d610147341c57579c51403e",
    "name": "Rebel Bots Oil",
    "symbol": "XOIL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32640/thumb/Xoil_coingecko.png?1698888846"
  },
  {
    "chainId": 137,
    "address": "0x43c73b90e0c2a355784dcf0da12f477729b31e77",
    "name": "Soil",
    "symbol": "SOIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32050/thumb/SOIL_sygnet_dark_blue.png?1696530847"
  },
  {
    "chainId": 137,
    "address": "0xba777ae3a3c91fcd83ef85bfe65410592bdd0f7c",
    "name": "BitCone",
    "symbol": "CONE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32278/thumb/BitCone_Logo_V7.5_halo_NO_circle_-200x200_copy_BIGGER.png?1697180757"
  },
  {
    "chainId": 137,
    "address": "0x8b78927048de67b9e8c8f834359f15c3822ed871",
    "name": "Legends of Elysium",
    "symbol": "LOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27247/thumb/loe.png?1696526297"
  },
  {
    "chainId": 137,
    "address": "0xcc1b9517460d8ae86fe576f614d091fca65a28fc",
    "name": "Vigorus",
    "symbol": "VIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21245/thumb/VIS.PNG?1696520618"
  },
  {
    "chainId": 137,
    "address": "0x19ccfe396006ffe7a92ab667b0ef90ce61b66f9f",
    "name": "Reel Token",
    "symbol": "REELT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29559/thumb/reelt-logo.png?1696528499"
  },
  {
    "chainId": 137,
    "address": "0x20c750c57c3bc5145af4b7a33d4fb66a8e79fe05",
    "name": "Orbcity",
    "symbol": "ORB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24332/thumb/OrbCity_enlarged.jpg?1696523517"
  },
  {
    "chainId": 137,
    "address": "0xaa1e97614eaf8d85dbf58f7f0b3080b2affcfefc",
    "name": "Flappy Bird Evolution",
    "symbol": "FEVO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32289/thumb/200x200t.png?1697182279"
  },
  {
    "chainId": 137,
    "address": "0xed88227296943857409a8e0f15ad7134e70d0f73",
    "name": "Lumiii",
    "symbol": "LUMIII",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27424/thumb/lumiii-icon-200x200.png?1696526465"
  },
  {
    "chainId": 137,
    "address": "0xe06bd4f5aac8d0aa337d13ec88db6defc6eaeefe",
    "name": "Planet IX",
    "symbol": "IXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20927/thumb/IXT_LOGO_PNG_RGB_200X.png?1696520316"
  },
  {
    "chainId": 137,
    "address": "0xe77abb1e75d2913b2076dd16049992ffeaca5235",
    "name": "Decentrawood",
    "symbol": "DEOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29308/thumb/LOGO.png?1696528259"
  },
  {
    "chainId": 137,
    "address": "0xb1c5c9b97b35592777091cd34ffff141ae866abd",
    "name": "Ubiq",
    "symbol": "UBQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/181/thumb/ubiq.png?1696501553"
  },
  {
    "chainId": 137,
    "address": "0xd0258a3fd00f38aa8090dfee343f10a9d4d30d3f",
    "name": "Voxies",
    "symbol": "VOXEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21260/thumb/voxies.png?1696520632"
  },
  {
    "chainId": 137,
    "address": "0xed2d13a70acbd61074fc56bd0d0845e35f793e5e",
    "name": "Planet Mojo",
    "symbol": "MOJO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36670/thumb/MOJOIcon.jpg?1712061269"
  },
  {
    "chainId": 137,
    "address": "0x950e1561b7a7deb1a32a6419fd435410daf851b0",
    "name": "Decentralized Universal Basic Income",
    "symbol": "DUBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2606/thumb/decentralized-universal-basic-income.png?1696503407"
  },
  {
    "chainId": 137,
    "address": "0x0184316f58b9a44acdd3e683257259dc0cf2202a",
    "name": "PolyGold",
    "symbol": "POLYGOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16055/thumb/wYRJKCT.png?1696515664"
  },
  {
    "chainId": 137,
    "address": "0x2ad2934d5bfb7912304754479dd1f096d5c807da",
    "name": "Argocoin",
    "symbol": "AGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34256/thumb/Argocoin.png?1704343655"
  },
  {
    "chainId": 137,
    "address": "0xc1c93d475dc82fe72dbc7074d55f5a734f8ceeae",
    "name": "Pegaxy",
    "symbol": "PGX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19305/thumb/CMC_PGX_Square.png?1700486146"
  },
  {
    "chainId": 137,
    "address": "0x82a0e6c02b91ec9f6ff943c0a933c03dbaa19689",
    "name": "Wicrypt",
    "symbol": "WNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21223/thumb/wicrypt.PNG?1696520597"
  },
  {
    "chainId": 137,
    "address": "0x838c9634de6590b96aeadc4bc6db5c28fd17e3c2",
    "name": "Matr1x Fire",
    "symbol": "FIRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34961/thumb/4Lw6OZcs_400x400.jpg?1706795533"
  },
  {
    "chainId": 137,
    "address": "0x071ac29d569a47ebffb9e57517f855cb577dcc4c",
    "name": "Galaxy Fight Club",
    "symbol": "GCOIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22364/thumb/M1oqSwPA_400x400.jpg?1696521707"
  },
  {
    "chainId": 137,
    "address": "0xff76c0b48363a7c7307868a81548d340049b0023",
    "name": "Derby Stars RUN",
    "symbol": "DSRUN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31203/thumb/download.png?1696530030"
  },
  {
    "chainId": 137,
    "address": "0xaa404804ba583c025fa64c9a276a6127ceb355c6",
    "name": "CIPHER",
    "symbol": "CPR",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/22719/thumb/E-97BTDPNxk2PYsd8EwHzDbEpM6w-VoJc3t5DutqiPPydT57269nRUrxk8Pqig4CboNanb2Rd0MA8a0N9CGUmGG1jKdbzhy8pIwiDEGcPBP0mslchklUxO7BWSbDOv-ouncsR0aHjYGlG1hTWueUJOeqSLcVR90UKpCLhEWtJ6mS4o8_XQh57pEAEw_aiZmetQ9z0-rsla4JFo.jpg?1696522023"
  },
  {
    "chainId": 137,
    "address": "0x6396252377f54ad33cff9131708da075b21d9b88",
    "name": "NFTBooks",
    "symbol": "NFTBS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/17291/thumb/76C625EC-11F3-46D2-B1E1-B471ACE02B26.jpeg?1696516845"
  },
  {
    "chainId": 137,
    "address": "0x765af38a6e8fdcb1efef8a3dd2213efd3090b00f",
    "name": "Vendetta",
    "symbol": "VDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35185/thumb/logo_200_blk.png?1707811266"
  },
  {
    "chainId": 137,
    "address": "0x0da6a7a0a2e7525950204bb2c511e47a52235b17",
    "name": "MT Tower",
    "symbol": "MT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27457/thumb/mt_logo_200x200.png?1696526497"
  },
  {
    "chainId": 137,
    "address": "0x25cbe406aa02053fedc955095415fd8f497f072d",
    "name": "W3Gamez Network",
    "symbol": "W3G",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34887/thumb/_W3G_Token.png?1706539140"
  },
  {
    "chainId": 137,
    "address": "0xa3c322ad15218fbfaed26ba7f616249f7705d945",
    "name": "GensoKishi Metaverse",
    "symbol": "MV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23143/thumb/geno.png?1696522435"
  },
  {
    "chainId": 137,
    "address": "0xe9e7c09e82328c3107d367f6c617cf9977e63ed0",
    "name": "A51 Finance",
    "symbol": "A51",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36723/thumb/a51.png?1712141364"
  },
  {
    "chainId": 137,
    "address": "0x35c2b6826b5c94fd2dfede51e7cfd4b94410840a",
    "name": "PubGame Coin",
    "symbol": "PGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31460/thumb/pgc.png?1696530273"
  },
  {
    "chainId": 137,
    "address": "0x140a4e80dd8184536acc45f1c452d7540472e6e1",
    "name": "Polker",
    "symbol": "PKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16803/thumb/200x200-PKR_Chip.png?1696516373"
  },
  {
    "chainId": 137,
    "address": "0x6483de4a2c76a38f288c4922fe2f507b2322ef80",
    "name": "Larissa Blockchain",
    "symbol": "LRS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34527/thumb/lrs_200x200.png?1705306770"
  },
  {
    "chainId": 137,
    "address": "0x0c705862f56cd8ec70337f5f5184fea4158a3c00",
    "name": "Abyss World",
    "symbol": "AWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30679/thumb/black.png?1696529548"
  },
  {
    "chainId": 137,
    "address": "0x428360b02c1269bc1c79fbc399ad31d58c1e8fda",
    "name": "Digital Fitness",
    "symbol": "DEFIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14699/thumb/Defit.png?1696514370"
  },
  {
    "chainId": 137,
    "address": "0x73b29199a8e4c146e893eb95f18dac41738a88c6",
    "name": "Bag",
    "symbol": "BAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35417/thumb/BAG-coingecko.png?1709393428"
  },
  {
    "chainId": 137,
    "address": "0xb962150760f9a3bb00e3e9cf48297ee20ada4a33",
    "name": "Zoomer",
    "symbol": "ZOOMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30894/thumb/zoooooooooomer.jpg?1696529740"
  },
  {
    "chainId": 137,
    "address": "0x4e0da40b9063dc48364c1c0ffb4ae9d091fc2270",
    "name": "Edgeware",
    "symbol": "EDG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8452/thumb/logo-edgeware.png?1696508638"
  },
  {
    "chainId": 137,
    "address": "0xcdf937995a55a9ab551d81b463ac0f7f02795368",
    "name": "Vyvo Smart Chain",
    "symbol": "VSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32105/thumb/Vyvo.png?1696530902"
  },
  {
    "chainId": 137,
    "address": "0x60d01ec2d5e98ac51c8b4cf84dfcce98d527c747",
    "name": "iZUMi Finance",
    "symbol": "IZI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21791/thumb/izumi-logo-symbol.png?1696521144"
  },
  {
    "chainId": 137,
    "address": "0xaff41da975501e5b71848c975834341777d1a473",
    "name": "BlockchainSpace",
    "symbol": "GUILD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21271/thumb/BednjMw.png?1696520642"
  },
  {
    "chainId": 137,
    "address": "0xeb99748e91afca94a6289db3b02e7ef4a8f0a22d",
    "name": "MahaDAO",
    "symbol": "MAHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13404/thumb/MAHA_Token.png?1696513166"
  },
  {
    "chainId": 137,
    "address": "0xcfa0bfa4972423eaf8d3b7af7e2ace41c56515f7",
    "name": "StorageChain",
    "symbol": "WSTOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33844/thumb/coinmarketcaplogo.png?1703121289"
  },
  {
    "chainId": 137,
    "address": "0xe156987a81a9b841c1def6f111ea69bf817fb272",
    "name": "Flux Point Studios SHARDS",
    "symbol": "SHARDS",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33635/thumb/shards.png?1702536029"
  },
  {
    "chainId": 137,
    "address": "0x58001cc1a9e17a20935079ab40b1b8f4fc19efd1",
    "name": "Push Protocol",
    "symbol": "PUSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14769/thumb/aiOxYOJI_400x400.jpeg?1696514438"
  },
  {
    "chainId": 137,
    "address": "0x71eeba415a523f5c952cc2f06361d5443545ad28",
    "name": "XDAO",
    "symbol": "XDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27363/thumb/token_2.png?1696526408"
  },
  {
    "chainId": 137,
    "address": "0x34ad6a664b4112a9c6bac7b54b4094a05e3ff775",
    "name": "MariCoin",
    "symbol": "MCOIN",
    "decimals": 3,
    "logoURI": "https://assets.coingecko.com/coins/images/29600/thumb/01_logoMARICOIN_TM_%281%29_%281%29_%281%29.png?1696528538"
  },
  {
    "chainId": 137,
    "address": "0x709d140925272ee606825781b1bef7be6b1412cd",
    "name": "Crypto Global United",
    "symbol": "CGU",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/20751/thumb/2022_CGU-MG_RGB.png?1696520148"
  },
  {
    "chainId": 137,
    "address": "0xeaf631ac57f3cdddd261770dd47f85066131a156",
    "name": "Equalizer",
    "symbol": "EQZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14741/thumb/X2p5mb2f_400x400.png?1696514410"
  },
  {
    "chainId": 137,
    "address": "0x8bb30e0e67b11b978a5040144c410e1ccddcba30",
    "name": "Zus",
    "symbol": "ZCN",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/4934/thumb/200x200_transparent.png?1696505474"
  },
  {
    "chainId": 137,
    "address": "0x0a16aeab4418cb8dcdef9717e801a1c4fceb2b03",
    "name": "Virtucoin",
    "symbol": "V",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35620/thumb/IMG_20231016_172706_142.jpg?1709288699"
  },
  {
    "chainId": 137,
    "address": "0x0731d0c0d123382c163aae78a09390cad2ffc941",
    "name": "Ink Fantom",
    "symbol": "INK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23511/thumb/v3INKLogo-03.png?1696522720"
  },
  {
    "chainId": 137,
    "address": "0xc0a1adce1c62daedf1b5f07b31266090bc5cc6d2",
    "name": "ONUS",
    "symbol": "ONUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24599/thumb/ONUS_200.png?1696523773"
  },
  {
    "chainId": 137,
    "address": "0xf328b73b6c685831f238c30a23fc19140cb4d8fc",
    "name": "Across Protocol",
    "symbol": "ACX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28161/thumb/across-200x200.png?1696527165"
  },
  {
    "chainId": 137,
    "address": "0xc4ce1d6f5d98d65ee25cf85e9f2e9dcfee6cb5d6",
    "name": "crvUSD",
    "symbol": "CRVUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30118/thumb/crvusd.jpeg?1696529040"
  },
  {
    "chainId": 137,
    "address": "0xc708d6f2153933daa50b2d0758955be0a93a8fec",
    "name": "Verse",
    "symbol": "VERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28424/thumb/verselogo.png?1696527421"
  },
  {
    "chainId": 137,
    "address": "0x91f3b9366801c1fca6184c3bd99d5ab0c43a9033",
    "name": "Icon X World",
    "symbol": "ICNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35133/thumb/iconx-world-icon-wt.png?1707444921"
  },
  {
    "chainId": 137,
    "address": "0xe9d2fa815b95a9d087862a09079549f351dab9bd",
    "name": "Bonsai Token",
    "symbol": "BONSAI",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/35884/thumb/Bonsai_BW_Coingecko-200x200.jpg?1710071621"
  },
  {
    "chainId": 137,
    "address": "0x236aa50979d5f3de3bd1eeb40e81137f22ab794b",
    "name": "tBTC",
    "symbol": "TBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11224/thumb/0x18084fba666a33d37592fa2633fd49a74dd93a88.png?1696511155"
  },
  {
    "chainId": 137,
    "address": "0x714db550b574b3e927af3d93e26127d15721d4c2",
    "name": "GMT",
    "symbol": "GMT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/23597/thumb/token-gmt-200x200.png?1703153841"
  },
  {
    "chainId": 137,
    "address": "0x17a011150e9feb7bec4cfada055c8df436eb730b",
    "name": "TDEX Token",
    "symbol": "TT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32725/thumb/20220929-180435.png?1699018453"
  },
  {
    "chainId": 137,
    "address": "0xfcbb00df1d663eee58123946a30ab2138bf9eb2a",
    "name": "ClayStack Staked MATIC",
    "symbol": "CSMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28352/thumb/csMatic.png?1696527356"
  },
  {
    "chainId": 137,
    "address": "0xb4357054c3da8d46ed642383f03139ac7f090343",
    "name": "Port3 Network",
    "symbol": "PORT3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33383/thumb/port3-bc-200x200.png?1710075114"
  },
  {
    "chainId": 137,
    "address": "0xa380c0b01ad15c8cf6b46890bddab5f0868e87f3",
    "name": "Vyvo US Dollar",
    "symbol": "USDV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31718/thumb/_USDV_Solid%28200x200%29.png?1696530539"
  },
  {
    "chainId": 137,
    "address": "0xa288e965e86ac4e5c03352f199cc7a66022e15a8",
    "name": "Black Whale",
    "symbol": "XXX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32516/thumb/BLK_LOGO%28200x200%29.png?1698406168"
  },
  {
    "chainId": 137,
    "address": "0xbc0bea8e634ec838a2a45f8a43e7e16cd2a8ba99",
    "name": "RigoBlock",
    "symbol": "GRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1532/thumb/Symbol-RigoblockRGB.png?1696502572"
  },
  {
    "chainId": 137,
    "address": "0x6f7c932e7684666c9fd1d44527765433e01ff61d",
    "name": "Maker",
    "symbol": "MKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1364/thumb/Mark_Maker.png?1696502423"
  },
  {
    "chainId": 137,
    "address": "0x57999936fc9a9ec0751a8d146cce11901be8bed0",
    "name": "VirtuSwap",
    "symbol": "VRSW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30629/thumb/VirtuSwap_Logo_Red_200x200.png?1696529502"
  },
  {
    "chainId": 137,
    "address": "0x81382e9693de2afc33f69b70a6c12ca9b3a73f47",
    "name": "DOSE",
    "symbol": "DOSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18847/thumb/dose.PNG?1696518308"
  },
  {
    "chainId": 137,
    "address": "0xc5102fe9359fd9a28f877a67e36b0f050d81a3cc",
    "name": "Hop Protocol",
    "symbol": "HOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25445/thumb/hop.png?1696524577"
  },
  {
    "chainId": 137,
    "address": "0xb75bbd79985a8092b05224f62d7fed25924b075d",
    "name": "Datamine",
    "symbol": "DAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11695/thumb/qxsFH8W.png?1696511584"
  },
  {
    "chainId": 137,
    "address": "0xf689e85988d3a7921e852867ce49f53388985e6d",
    "name": "MobiFi",
    "symbol": "MOFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14697/thumb/MOFI_Coin_Green_200x200.png?1696514369"
  },
  {
    "chainId": 137,
    "address": "0x4985e0b13554fb521840e893574d3848c10fcc6f",
    "name": "PolySwarm",
    "symbol": "NCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2843/thumb/ImcYCVfX_400x400.jpg?1696503602"
  },
  {
    "chainId": 137,
    "address": "0xc553a5a789f1bb56a72847b3fda1de3e16e6763f",
    "name": "CygnusDAO",
    "symbol": "CYG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33521/thumb/cyg.png?1702133561"
  },
  {
    "chainId": 137,
    "address": "0x7961ade0a767c0e5b67dd1a1f78ba44f727642ed",
    "name": "Tegro",
    "symbol": "TGR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26564/thumb/512x512.png?1696525637"
  },
  {
    "chainId": 137,
    "address": "0x79375c41d88f839f551457145066096c5c8944bc",
    "name": "SocialGood",
    "symbol": "SG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3948/thumb/logo_200.png?1696504592"
  },
  {
    "chainId": 137,
    "address": "0x06ddb3a8bc0abc14f85e974cf1a93a6f8d4909d9",
    "name": "8Pay",
    "symbol": "8PAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14667/thumb/8pay.jpeg?1696514342"
  },
  {
    "chainId": 137,
    "address": "0x0e62cadbaeec69b8b0f2e9d56510f925512837d2",
    "name": "ZooCoin",
    "symbol": "ZOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31230/thumb/ZOO.png?1696530056"
  },
  {
    "chainId": 137,
    "address": "0x4ed141110f6eeeaba9a1df36d8c26f684d2475dc",
    "name": "Brazilian Digital",
    "symbol": "BRZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8472/thumb/MicrosoftTeams-image_%286%29.png?1696508657"
  },
  {
    "chainId": 137,
    "address": "0xd3ac016b1b8c80eeadde4d186a9138c9324e4189",
    "name": "Okcash",
    "symbol": "OK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/274/thumb/ok-logo-200px.png?1696501624"
  },
  {
    "chainId": 137,
    "address": "0x3a79241a92a4f06952107308057da1991792d372",
    "name": "Zenland",
    "symbol": "ZENF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29477/thumb/tokenicon200px.png?1696528421"
  },
  {
    "chainId": 137,
    "address": "0x44acd96620b708162af4a90524f29a6839675533",
    "name": "TCG Verse",
    "symbol": "TCGC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28548/thumb/tcgc.png?1696527539"
  },
  {
    "chainId": 137,
    "address": "0x2c89bbc92bd86f8075d1decc58c7f4e0107f286b",
    "name": "Wrapped AVAX",
    "symbol": "WAVAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15075/thumb/wrapped-avax.png?1696514734"
  },
  {
    "chainId": 137,
    "address": "0xf21441f9ec4c1fe69cb7cf186eceab31af2b658d",
    "name": "Vent Finance",
    "symbol": "VENT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17925/thumb/Artboard_29.png?1696517445"
  },
  {
    "chainId": 137,
    "address": "0x9a6a40cdf21a0af417f1b815223fd92c85636c58",
    "name": "Basket",
    "symbol": "BSKT",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/34661/thumb/BSKT_Logo.png?1705636891"
  },
  {
    "chainId": 137,
    "address": "0x444444444444c1a66f394025ac839a535246fcc8",
    "name": "Genius",
    "symbol": "GENI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28621/thumb/GENI200x200.png?1696527606"
  },
  {
    "chainId": 137,
    "address": "0x38029c62dfa30d9fd3cadf4c64e9b2ab21dbda17",
    "name": "Dubbz",
    "symbol": "DUBBZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28665/thumb/D8EACA06-18ED-4999-8B3A-6339F9E021CE.jpeg?1696527650"
  },
  {
    "chainId": 137,
    "address": "0x17d342b29f054030a455b4191f977c3b0aa62fd9",
    "name": "Kanga Exchange",
    "symbol": "KNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21188/thumb/KNG_logo-200.png?1696520564"
  },
  {
    "chainId": 137,
    "address": "0xfb40b1efe90d4b786d2d9d9dc799b18bde92923b",
    "name": "LIF3 LSHARE  OLD ",
    "symbol": "LSHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26038/thumb/LSHARE.png?1696525119"
  },
  {
    "chainId": 137,
    "address": "0x2c2d8a078b33bf7782a16acce2c5ba6653a90d5f",
    "name": "L3USD",
    "symbol": "L3USD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26937/thumb/L3USD.png?1696525993"
  },
  {
    "chainId": 137,
    "address": "0xd24157aa1097486dc9d7cf094a7e15026e566b5d",
    "name": "TPRO Network",
    "symbol": "TPRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26694/thumb/tpro-logo-200x200.png?1712870769"
  },
  {
    "chainId": 137,
    "address": "0xc3f6434df5ef44798610c921942e49822c6f5108",
    "name": "Decentral ART",
    "symbol": "ART",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29398/thumb/E58AC31F-1B46-4BAD-B8D8-E01551303E09.jpeg?1696528348"
  },
  {
    "chainId": 137,
    "address": "0x3fb83a9a2c4408909c058b0bfe5b4823f54fafe2",
    "name": "bitsCrunch Token",
    "symbol": "BCUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34832/thumb/logo-256_%281%29.png?1710244462"
  },
  {
    "chainId": 137,
    "address": "0x2598c30330d5771ae9f983979209486ae26de875",
    "name": "Any Inu",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34126/thumb/anyinulogo200.png?1704174269"
  },
  {
    "chainId": 137,
    "address": "0x382586651f043cbdec7bb586e367d77b26d7d149",
    "name": "Wild Goat Coin",
    "symbol": "WGC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34938/thumb/WGC_coin3_200_x_200.png?1706721230"
  },
  {
    "chainId": 137,
    "address": "0xa9f37d84c856fda3812ad0519dad44fa0a3fe207",
    "name": "Enzyme",
    "symbol": "MLN",
    "decimals": 11,
    "logoURI": "https://assets.coingecko.com/coins/images/605/thumb/Enzyme_Icon_Secondary.png?1696501803"
  },
  {
    "chainId": 137,
    "address": "0xba0dda8762c24da9487f5fa026a9b64b695a07ea",
    "name": "OX Coin",
    "symbol": "OX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35365/thumb/logo.png?1708395976"
  },
  {
    "chainId": 137,
    "address": "0xe292178333fc7424211795895865adac05baf3be",
    "name": "Jarvis",
    "symbol": "JARVIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35812/thumb/jarvis.png?1709873633"
  },
  {
    "chainId": 137,
    "address": "0xfe049f59963545bf5469f968e04c9646d6e2c2c5",
    "name": "Blood Crystal",
    "symbol": "BC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35834/thumb/BloodCrystalToken_200x200.png?1709888447"
  },
  {
    "chainId": 137,
    "address": "0x0f76d32cdccdcbd602a55af23eaf58fd1ee17245",
    "name": "Backed ERNA   Bond",
    "symbol": "BERNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36757/thumb/b-ERNA-200x200.png?1712245846"
  },
  {
    "chainId": 137,
    "address": "0x039d2e8f097331278bd6c1415d839310e0d5ece4",
    "name": "Linda",
    "symbol": "LINDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33699/thumb/linda-logo-200.png?1705166731"
  },
  {
    "chainId": 137,
    "address": "0x4a506181f07da5ddfda4ca4c2fa4c67001db94b4",
    "name": "Dyl",
    "symbol": "DYL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32550/thumb/Dyl-pfp-200x.png?1698479973"
  },
  {
    "chainId": 137,
    "address": "0xba11c5effa33c4d6f8f593cfa394241cfe925811",
    "name": "OUSG",
    "symbol": "OUSG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29023/thumb/OUSG.png?1696527993"
  },
  {
    "chainId": 137,
    "address": "0x95e376390f472fcaa21995169e11d523954b3bbb",
    "name": "LiraT",
    "symbol": "TRYT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32815/thumb/lirat.png?1699579061"
  },
  {
    "chainId": 137,
    "address": "0xd94a8f9caed25e63ecc90edfefaf3635ea1e182a",
    "name": "Stablecomp",
    "symbol": "SCOMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33932/thumb/token-icon-big.png?1703422228"
  },
  {
    "chainId": 137,
    "address": "0x14eb40fb7900185c01adc6a5b8ac506d8a600e3c",
    "name": "Token Teknoloji A    EURO",
    "symbol": "EUROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32816/thumb/eurot.png?1699579428"
  },
  {
    "chainId": 137,
    "address": "0xbfc70507384047aa74c29cdc8c5cb88d0f7213ac",
    "name": "Artificial Liquid Intelligence",
    "symbol": "ALI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22062/thumb/Logo_Circle_720_%281%29.png?1696521405"
  },
  {
    "chainId": 137,
    "address": "0x625e7708f30ca75bfd92586e17077590c60eb4cd",
    "name": "Aave v3 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32847/thumb/usdc_%281%29.png?1699619355"
  },
  {
    "chainId": 137,
    "address": "0xa66818b5bda74c081a9582d8aa8929fae77e214a",
    "name": "KUMA Protocol US KUMA Interest Bearing ",
    "symbol": "USK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33445/thumb/USK.png?1701888523"
  },
  {
    "chainId": 137,
    "address": "0x11cd72f7a4b699c67f225ca8abb20bc9f8db90c7",
    "name": "Osaka Protocol",
    "symbol": "OSAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30911/thumb/osak_logo.png?1696529756"
  },
  {
    "chainId": 137,
    "address": "0xc8bb8eda94931ca2f20ef43ea7dbd58e68400400",
    "name": "VNX Gold",
    "symbol": "VNXAU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28019/thumb/VNX_Token_logo_%283%29.png?1696527035"
  },
  {
    "chainId": 137,
    "address": "0x38a536a31ba4d8c1bcca016abbf786ecd25877e8",
    "name": "AssetMantle",
    "symbol": "MNTL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25181/thumb/thumbnail.png?1696524326"
  },
  {
    "chainId": 137,
    "address": "0xb0b2ef34d412d73b0ff90a709d1779a20655165a",
    "name": "Guardian GUARD",
    "symbol": "GUARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17995/thumb/LS_wolfDen_logo.0025_Light_200x200.png?1696517512"
  },
  {
    "chainId": 137,
    "address": "0xe4095d9372e68d108225c306a4491cacfb33b097",
    "name": "VNX EURO",
    "symbol": "VEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29351/thumb/VNXEUR_%281%29.png?1696528300"
  },
  {
    "chainId": 137,
    "address": "0x2f5de51823e514de04475ba8db1eeba5b244ba84",
    "name": "Token Teknoloji A    USD",
    "symbol": "USDOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32817/thumb/usdot.png?1699579447"
  },
  {
    "chainId": 137,
    "address": "0x1e553688199d99d063c0300a12395f7cfedb03e1",
    "name": "REDANCOIN",
    "symbol": "REDAN",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/8292/thumb/REDAN.png?1696508493"
  },
  {
    "chainId": 137,
    "address": "0xcdb3867935247049e87c38ea270edd305d84c9ae",
    "name": "VNX Swiss Franc",
    "symbol": "VCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29547/thumb/VNXCHF_%282%29.png?1696528488"
  },
  {
    "chainId": 137,
    "address": "0x99083d1b9c6744c71d0cf70b8965faca37684527",
    "name": "FRANCE REV FINANCE",
    "symbol": "FRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22224/thumb/LOGO200_FRF.png?1696521567"
  },
  {
    "chainId": 137,
    "address": "0xb5e0cfe1b4db501ac003b740665bf43192cc7853",
    "name": "Ghost",
    "symbol": "GHOST",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11339/thumb/mxpD9xS.png?1696511260"
  },
  {
    "chainId": 137,
    "address": "0xf549db6b1e0b79c8f2730e7b7c6f2edca2cebaa6",
    "name": "impactMarket",
    "symbol": "PACT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21907/thumb/PACT_Token_Ticker_Blue_2x.png?1696521258"
  },
  {
    "chainId": 137,
    "address": "0x1280830f690d0e65195b3c61b028244c3a49f26d",
    "name": "Axelar Wrapped Ether",
    "symbol": "AXLETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28171/thumb/weth-wei_D_3x.png?1696527174"
  },
  {
    "chainId": 137,
    "address": "0xb87904db461005fc716a6bf9f2d451c33b10b80b",
    "name": "Alongside Crypto Market Index",
    "symbol": "AMKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28496/thumb/22999.png?1696527488"
  },
  {
    "chainId": 137,
    "address": "0xcf66eb3d546f0415b368d98a95eaf56ded7aa752",
    "name": "dForce USD",
    "symbol": "USX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17422/thumb/usx_32.png?1696516969"
  },
  {
    "chainId": 137,
    "address": "0x18ec0a6e18e5bc3784fdd3a3634b31245ab704f6",
    "name": "Monerium EUR emoney",
    "symbol": "EURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23354/thumb/eur.png?1696522569"
  },
  {
    "chainId": 137,
    "address": "0x41b3966b4ff7b427969ddf5da3627d6aeae9a48e",
    "name": "NEXO",
    "symbol": "NEXO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3695/thumb/nexo.png?1696504370"
  },
  {
    "chainId": 137,
    "address": "0x554cd6bdd03214b10aafa3e0d4d42de0c5d2937b",
    "name": "Rupiah Token",
    "symbol": "IDRT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/9441/thumb/57421944_1371636006308255_3647136573922738176_n.jpg?1696509533"
  },
  {
    "chainId": 137,
    "address": "0x612d833c0c7a54cdfbe9a4328b6d658020563ec0",
    "name": "Pine",
    "symbol": "PINE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25660/thumb/Logomark-Colour.png?1696524788"
  },
  {
    "chainId": 137,
    "address": "0xf9cea445c2ac1668f50caa2c2924f93606a4a37d",
    "name": "Calvaria  DoE",
    "symbol": "RIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28817/thumb/icon200x200.png?1696527793"
  },
  {
    "chainId": 137,
    "address": "0x8de5b80a0c1b02fe4976851d030b36122dbb8624",
    "name": "Vanar Chain",
    "symbol": "VANRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33466/thumb/apple-touch-icon.png?1701942541"
  },
  {
    "chainId": 137,
    "address": "0xb092e1bf50f518b3ebf7ed26a40015183ae36ac2",
    "name": "Tarot",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31800/thumb/TAROT.jpg?1696530615"
  },
  {
    "chainId": 137,
    "address": "0x9c1c23e60b72bc88a043bf64afdb16a02540ae8f",
    "name": "Darwinia Network",
    "symbol": "RING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9443/thumb/RING.png?1696509535"
  },
  {
    "chainId": 137,
    "address": "0x07ed33a242bd9c08ca3c198e01189e35265024da",
    "name": "Wagmi",
    "symbol": "WAGMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31887/thumb/wagmi_token_logo.png?1696530698"
  },
  {
    "chainId": 137,
    "address": "0x3c59798620e5fec0ae6df1a19c6454094572ab92",
    "name": "Morpheus Network",
    "symbol": "MNW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2379/thumb/MRPH_CoinGecko.png?1696503245"
  },
  {
    "chainId": 137,
    "address": "0x1e2c4fb7ede391d116e6b41cd0608260e8801d59",
    "name": "Backed CSPX Core S P 500",
    "symbol": "BCSPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31891/thumb/b-CSPX-200x200.png?1696530702"
  },
  {
    "chainId": 137,
    "address": "0x282d8efce846a88b159800bd4130ad77443fa1a1",
    "name": "Ocean Protocol",
    "symbol": "OCEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3687/thumb/ocean-protocol-logo.jpg?1696504363"
  },
  {
    "chainId": 137,
    "address": "0x0ba8a6ce46d369d779299dedade864318097b703",
    "name": "JUSD",
    "symbol": "JUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32302/thumb/200x200.png?1697186770"
  },
  {
    "chainId": 137,
    "address": "0x8226ac9edb26ff16da19151042a8ba3bb2cc237f",
    "name": "Escoin",
    "symbol": "ELG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13566/thumb/escoin-200.png?1696513320"
  },
  {
    "chainId": 137,
    "address": "0x52d134c6db5889fad3542a09eaf7aa90c0fdf9e4",
    "name": "Backed IBTA   Treasury Bond 1 3yr",
    "symbol": "BIBTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31880/thumb/b-IBTA-200x200.png?1696530692"
  },
  {
    "chainId": 137,
    "address": "0x8829d36f6680be993f5444198e8cbfa8f02ede96",
    "name": "TAIKAI",
    "symbol": "TKAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34827/thumb/TKAI.jpg?1706191900"
  },
  {
    "chainId": 137,
    "address": "0xe50fa9b3c56ffb159cb0fca61f5c9d750e8128c8",
    "name": "Aave v3 WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32882/thumb/WETH_%281%29.png?1699716492"
  },
  {
    "chainId": 137,
    "address": "0x078f358208685046a11c85e8ad32895ded33a249",
    "name": "Aave v3 WBTC",
    "symbol": "AWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32883/thumb/wbtc.png?1699719908"
  },
  {
    "chainId": 137,
    "address": "0x31c91d8fb96bff40955dd2dbc909b36e8b104dde",
    "name": "Poison Finance",
    "symbol": "POION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28630/thumb/poisonlogo160x160.png?1696527614"
  },
  {
    "chainId": 137,
    "address": "0x32dc2dd3c2be453a369625e6fe0e438aed814919",
    "name": "SelfKey",
    "symbol": "KEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2034/thumb/selfkey.png?1696503004"
  },
  {
    "chainId": 137,
    "address": "0x2f3e306d9f02ee8e8850f9040404918d0b345207",
    "name": "Dogami",
    "symbol": "DOGA",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/24510/thumb/doga_logo.png?1696523689"
  },
  {
    "chainId": 137,
    "address": "0x6ab707aca953edaefbc4fd23ba73294241490620",
    "name": "Aave v3 USDT",
    "symbol": "AUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32884/thumb/USDT.PNG?1699768611"
  },
  {
    "chainId": 137,
    "address": "0x03b54a6e9a984069379fae1a4fc4dbae93b3bccd",
    "name": "Wrapped stETH",
    "symbol": "WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18834/thumb/wstETH.png?1696518295"
  },
  {
    "chainId": 137,
    "address": "0x3562ddf1f5ce2c02ef109e9d5a72e2fdb702711d",
    "name": "Wrapped Kaspa",
    "symbol": "KAS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31037/thumb/WKAS-256-black-ETH-3.png?1696529872"
  },
  {
    "chainId": 137,
    "address": "0xcb64cdeb45def1c513fd890e7e76a865bae46060",
    "name": "Medifakt",
    "symbol": "FAKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25248/thumb/fxOi9ZYI_400x400.png?1696524389"
  },
  {
    "chainId": 137,
    "address": "0x58c7b2828e7f2b2caa0cc7feef242fa3196d03df",
    "name": "3A",
    "symbol": "A3A",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33135/thumb/A3A.png?1700801023"
  },
  {
    "chainId": 137,
    "address": "0x82617aa52dddf5ed9bb7b370ed777b3182a30fd1",
    "name": "Yield Guild Games",
    "symbol": "YGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17358/thumb/Shield_Mark_-_Colored_-_Iridescent.png?1696516909"
  },
  {
    "chainId": 137,
    "address": "0xb121fcd122daaa153bb8a102754127b2682645cb",
    "name": "PERL eco",
    "symbol": "PERL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4682/thumb/PERL.eco-Icon-green_6x.png?1696505249"
  },
  {
    "chainId": 137,
    "address": "0x3558887f15b5b0074dc4167761de14a6dfcb676e",
    "name": "Netvrk",
    "symbol": "NETVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15721/thumb/netvrk_icon.png?1696515348"
  },
  {
    "chainId": 137,
    "address": "0x57f5e098cad7a3d1eed53991d4d66c45c9af7812",
    "name": "Wrapped USDM",
    "symbol": "WUSDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33785/thumb/wUSDM_PNG_240px.png?1702981552"
  },
  {
    "chainId": 137,
    "address": "0x2b1d36f5b61addaf7da7ebbd11b35fd8cfb0de31",
    "name": "Interport Token",
    "symbol": "ITP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28338/thumb/ITP_Logo_200.png?1696527344"
  },
  {
    "chainId": 137,
    "address": "0x323665443cef804a3b5206103304bd4872ea4253",
    "name": "Verified USD",
    "symbol": "USDV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32948/thumb/usdv_%281%29.png?1699933314"
  },
  {
    "chainId": 137,
    "address": "0x59d9356e565ab3a36dd77763fc0d87feaf85508c",
    "name": "Mountain Protocol USD",
    "symbol": "USDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31719/thumb/usdm.png?1696530540"
  },
  {
    "chainId": 137,
    "address": "0xb1a91036e4a3c144efed953e0b6cc5f6b98ad256",
    "name": "Multibit",
    "symbol": "MUBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33051/thumb/multi.jpg?1700473994"
  },
  {
    "chainId": 137,
    "address": "0xe6828d65bf5023ae1851d90d8783cc821ba7eee1",
    "name": "ApeBond",
    "symbol": "ABOND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33814/thumb/l-_N-wAO_400x400.jpg?1703020501"
  },
  {
    "chainId": 137,
    "address": "0xdb6dae4b87be1289715c08385a6fc1a3d970b09d",
    "name": "Moove Protocol",
    "symbol": "MOOVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31549/thumb/Logo-Circle-Moove.png?1696530362"
  },
  {
    "chainId": 137,
    "address": "0x291aa47c58558adfc2bcd6f060578fdae1f6570c",
    "name": "Minebase",
    "symbol": "MBASE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27313/thumb/mbase.png?1696526362"
  },
  {
    "chainId": 137,
    "address": "0x0b6f3ea2814f3fff804ba5d5c237aebbc364fba9",
    "name": "Unagi Token",
    "symbol": "UNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34733/thumb/unagi.png?1705922276"
  },
  {
    "chainId": 137,
    "address": "0x4f604735c1cf31399c6e711d5962b2b3e0225ad3",
    "name": "Glo Dollar",
    "symbol": "USDGLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29319/thumb/glo_logo_coingecko.png?1696528270"
  },
  {
    "chainId": 137,
    "address": "0x7bebd226154e865954a87650faefa8f485d36081",
    "name": "Zignaly",
    "symbol": "ZIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14796/thumb/zignaly.jpg?1704292004"
  },
  {
    "chainId": 137,
    "address": "0x82e64f49ed5ec1bc6e43dad4fc8af9bb3a2312ee",
    "name": "Aave v3 DAI",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32886/thumb/dai.png?1699769446"
  },
  {
    "chainId": 137,
    "address": "0x281c4746c902a322b9a951f07893ac51a7221acc",
    "name": "Staked Ethos Reserve Note",
    "symbol": "STERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31522/thumb/stERN.png?1696530332"
  },
  {
    "chainId": 137,
    "address": "0xf329e36c7bf6e5e86ce2150875a84ce77f477375",
    "name": "Aave v3 AAVE",
    "symbol": "AAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32887/thumb/aave.png?1699773604"
  },
  {
    "chainId": 137,
    "address": "0x644192291cc835a93d6330b24ea5f5fedd0eef9e",
    "name": "AllianceBlock Nexera",
    "symbol": "NXRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29181/thumb/nxra.png?1712255778"
  },
  {
    "chainId": 137,
    "address": "0x191c10aa4af7c30e871e70c95db0e4eb77237530",
    "name": "Aave v3 LINK",
    "symbol": "ALINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32888/thumb/link.png?1699773900"
  },
  {
    "chainId": 137,
    "address": "0x4867b60ad7c6adc98653f661f1aea31740986ba5",
    "name": "Mean DAO",
    "symbol": "MEAN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21557/thumb/89934951.png?1696520918"
  },
  {
    "chainId": 137,
    "address": "0x20c64dee8fda5269a78f2d5bdba861ca1d83df7a",
    "name": "Backed HIGH   High Yield Corp Bond",
    "symbol": "BHIGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31868/thumb/b-HIGH-200x200.png?1696530680"
  },
  {
    "chainId": 137,
    "address": "0x2f123cf3f37ce3328cc9b5b8415f9ec5109b45e7",
    "name": "Backed GOVIES 0 6 months EURO",
    "symbol": "BC3M",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31870/thumb/b-C3-M-200x200.png?1696530682"
  },
  {
    "chainId": 137,
    "address": "0x0266f4f08d82372cf0fcbccc0ff74309089c74d1",
    "name": "Rocket Pool ETH",
    "symbol": "RETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20764/thumb/reth.png?1696520159"
  },
  {
    "chainId": 137,
    "address": "0x110b25d2b21ee73eb401f3ae7833f7072912a0bf",
    "name": "Lif3",
    "symbol": "LIF3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31291/thumb/Tokens.png?1696530112"
  },
  {
    "chainId": 137,
    "address": "0x1236ea13c7339287cd00ab196aaa8217006b04dc",
    "name": "Epic League",
    "symbol": "EPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32139/thumb/epl_symbol_200.png?1696591947"
  },
  {
    "chainId": 137,
    "address": "0x7c603c3c0c97a565cf202c94ab5298bf8510f7dc",
    "name": "OATH",
    "symbol": "OATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24075/thumb/OATH.png?1698051304"
  },
  {
    "chainId": 137,
    "address": "0xf33687811f3ad0cd6b48dd4b39f9f977bd7165a2",
    "name": "TruFin Staked MATIC",
    "symbol": "TRUMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32416/thumb/TruMATIC_logo_250px_%281%29.png?1698114190"
  },
  {
    "chainId": 137,
    "address": "0x4b85a666dec7c959e88b97814e46113601b07e57",
    "name": "GoCrypto",
    "symbol": "GOC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3181/thumb/gocrypto.png?1696503902"
  },
  {
    "chainId": 137,
    "address": "0xceed2671d8634e3ee65000edbbee66139b132fbf",
    "name": "Bridged Tether  Axelar ",
    "symbol": "AXLUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31002/thumb/uusdt_D_3x.png?1696529840"
  },
  {
    "chainId": 137,
    "address": "0x7f792db54b0e580cdc755178443f0430cf799aca",
    "name": "Volt Inu",
    "symbol": "VOLT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/25201/thumb/logo200.png?1696524344"
  },
  {
    "chainId": 137,
    "address": "0x8626264b6a1b4e920905efd381002aba52ea0eea",
    "name": "BlackHat Coin",
    "symbol": "BLKC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/15987/thumb/logo_light.png?1696515600"
  },
  {
    "chainId": 137,
    "address": "0xb0f61c597bbcc29f3f38396b01f9c0f0c2e8bff0",
    "name": "Etherland",
    "symbol": "ELAND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14432/thumb/eland_logo.png?1696514122"
  },
  {
    "chainId": 137,
    "address": "0x9c9e5fd8bbc25984b178fdce6117defa39d2db39",
    "name": "Binance Peg BUSD",
    "symbol": "BUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31273/thumb/new_binance-peg-busd.png?1696530096"
  },
  {
    "chainId": 137,
    "address": "0xdbb5da27ffcfebea8799a5832d4607714fc6aba8",
    "name": "DGEN",
    "symbol": "DGEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31294/thumb/dgen-new-200-200.png?1696530114"
  },
  {
    "chainId": 137,
    "address": "0x306ee01a6ba3b4a8e993fa2c1adc7ea24462000c",
    "name": "Neopin",
    "symbol": "NPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23912/thumb/logo_design.png?1696523111"
  },
  {
    "chainId": 137,
    "address": "0x0000206329b97db379d5e1bf586bbdb969c63274",
    "name": "USDA",
    "symbol": "USDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34510/thumb/agUSD-coingecko.png?1705288392"
  },
  {
    "chainId": 137,
    "address": "0x4ee438be38f8682abb089f2bfea48851c5e71eaf",
    "name": "Cryptonovae",
    "symbol": "YAE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14693/thumb/yae.png?1696514365"
  },
  {
    "chainId": 137,
    "address": "0xc3a9a54c043f348027fffaac0f2f996123a19bf4",
    "name": "Ethos Reserve Note",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29744/thumb/ERN200x200.png?1696528676"
  },
  {
    "chainId": 137,
    "address": "0xb541a306dd240ef04fb5e7e0db9a3c6cb7ddbb07",
    "name": "WFDP",
    "symbol": "WFDP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23758/thumb/wfdp.png?1696522959"
  },
  {
    "chainId": 137,
    "address": "0x4d4d883f920f7c0c36a1be71a02aa0cde2aa22d1",
    "name": "Opticash",
    "symbol": "OPCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29362/thumb/opticash_logo_256*256.jpg?1696528310"
  },
  {
    "chainId": 137,
    "address": "0x6d1fdbb266fcc09a16a22016369210a15bb95761",
    "name": "Staked Frax Ether",
    "symbol": "SFRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28285/thumb/sfrxETH_icon.png?1696527285"
  },
  {
    "chainId": 137,
    "address": "0x9f28e2455f9ffcfac9ebd6084853417362bc5dbb",
    "name": "StaFi Staked MATIC",
    "symbol": "RMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29610/thumb/rMATIC.png?1696528546"
  },
  {
    "chainId": 137,
    "address": "0xe5b49820e5a1063f6f4ddf851327b5e8b2301048",
    "name": "Bonk",
    "symbol": "BONK",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/28600/thumb/bonk.jpg?1696527587"
  },
  {
    "chainId": 137,
    "address": "0x820802fa8a99901f52e39acd21177b0be6ee2974",
    "name": "EUROe Stablecoin",
    "symbol": "EUROE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28913/thumb/euroe-200x200-round.png?1696527889"
  },
  {
    "chainId": 137,
    "address": "0x8929e9dbd2785e3ba16175e596cdd61520fee0d1",
    "name": "Altitude",
    "symbol": "ALTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30114/thumb/logo.png?1696529036"
  },
  {
    "chainId": 137,
    "address": "0xfa5d5dd2517ee9c1419534a16b132adde2e3d948",
    "name": "ULTRON",
    "symbol": "ULX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26977/thumb/ULTRON-Profile-Pic.jpg?1696526031"
  },
  {
    "chainId": 137,
    "address": "0x70048335a8bcd746a2f45bd4bb15f99da5e0eb05",
    "name": "Crypto Puffs",
    "symbol": "PUFFS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17672/thumb/puff.PNG?1696517202"
  },
  {
    "chainId": 137,
    "address": "0xe3322702bedaaed36cddab233360b939775ae5f1",
    "name": "Tellor Tributes",
    "symbol": "TRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9644/thumb/Blk_icon_current.png?1696509713"
  },
  {
    "chainId": 137,
    "address": "0x590eb2920486486c2d9bb3eb651f73b81df87bcf",
    "name": "Bobcoin",
    "symbol": "BOBC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24264/thumb/bobc.png?1696523448"
  },
  {
    "chainId": 137,
    "address": "0xe78aee6ccb05471a69677fb74da80f5d251c042b",
    "name": "Taki Games",
    "symbol": "TAKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25271/thumb/Social_Icon_1_200x200.png?1707905836"
  },
  {
    "chainId": 137,
    "address": "0x027af1e12a5869ed329be4c05617ad528e997d5a",
    "name": "Arch Ethereum Div  Yield",
    "symbol": "AEDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30483/thumb/AEDY-512x512.png?1696529369"
  },
  {
    "chainId": 137,
    "address": "0x7205705771547cf79201111b4bd8aaf29467b9ec",
    "name": "Rocket Pool",
    "symbol": "RPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2090/thumb/rocket_pool_%28RPL%29.png?1696503058"
  },
  {
    "chainId": 137,
    "address": "0x1a49e59ab7101d6a66d6b2fa6d09932079cc80ec",
    "name": "Trophy",
    "symbol": "WINS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31007/thumb/icon_wins.png?1696529844"
  },
  {
    "chainId": 137,
    "address": "0x8af78f0c818302164f73b2365fe152c2d1fe80e1",
    "name": "Financie Token",
    "symbol": "FNCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29099/thumb/VcJQzCWT_400x400.jpg?1696528062"
  },
  {
    "chainId": 137,
    "address": "0x9ff62d1fc52a907b6dcba8077c2ddca6e6a9d3e1",
    "name": "Forta",
    "symbol": "FORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25060/thumb/Forta_lgo_%281%29.png?1696524210"
  },
  {
    "chainId": 137,
    "address": "0x11a88f949c0592238959142653bb6847c6523d81",
    "name": "WEN Token",
    "symbol": "WEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30400/thumb/kxtrOzUd_400x400.jpg?1696529290"
  },
  {
    "chainId": 137,
    "address": "0x9b3b0703d392321ad24338ff1f846650437a43c9",
    "name": "Boson Protocol",
    "symbol": "BOSON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14710/thumb/boson_logo.png?1696514381"
  },
  {
    "chainId": 137,
    "address": "0xee327f889d5947c1dc1934bb208a1e792f953e96",
    "name": "Frax Ether",
    "symbol": "FRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28284/thumb/frxETH_icon.png?1696527284"
  },
  {
    "chainId": 137,
    "address": "0xab1b1680f6037006e337764547fb82d17606c187",
    "name": "Arch USD Div  Yield",
    "symbol": "ADDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30484/thumb/ADDY.png?1696529370"
  },
  {
    "chainId": 137,
    "address": "0x4b4327db1600b8b1440163f667e199cef35385f5",
    "name": "Coinbase Wrapped Staked ETH",
    "symbol": "CBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27008/thumb/cbeth.png?1709186989"
  },
  {
    "chainId": 137,
    "address": "0x78a0a62fba6fb21a83fe8a3433d44c73a4017a6f",
    "name": "Open Exchange Token",
    "symbol": "OXOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30604/thumb/Logo2.png?1696529473"
  },
  {
    "chainId": 137,
    "address": "0x4987a49c253c38b3259092e9aac10ec0c7ef7542",
    "name": "Dust Protocol",
    "symbol": "DUST",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/24289/thumb/6388d49d-1f00-448d-92bc-f2db1441bbce.?1696523472"
  },
  {
    "chainId": 137,
    "address": "0x0294d8eb7857d43feb1210db72456d41481f9ede",
    "name": "LiquidDriver",
    "symbol": "LQDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15782/thumb/LQDR_Glowing_Icon.png?1696515405"
  },
  {
    "chainId": 137,
    "address": "0x8497842420cfdbc97896c2353d75d89fc8d5be5d",
    "name": "VersaGames",
    "symbol": "VERSA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25491/thumb/versa_token_96_96.png?1696524624"
  },
  {
    "chainId": 137,
    "address": "0x58b9cb810a68a7f3e1e4f8cb45d1b9b3c79705e8",
    "name": "Connext",
    "symbol": "NEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31293/thumb/connext.png?1696530113"
  },
  {
    "chainId": 137,
    "address": "0x4b27cd6e6a5e83d236ead376d256fe2f9e9f0d2e",
    "name": "Bolide",
    "symbol": "BLID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25548/thumb/5XvLQULP_400x400.jpg?1703622595"
  },
  {
    "chainId": 137,
    "address": "0xc27158bb8e08899d38765752f91d7d778e16ab8e",
    "name": "KAP Games",
    "symbol": "KAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27682/thumb/KAP_Logo.png?1696526710"
  },
  {
    "chainId": 137,
    "address": "0x6949804b60dd9dcea43fd8d10ccdc640b55f9f7f",
    "name": "BreederDAO",
    "symbol": "BREED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25203/thumb/BreederDAO-Breed_Token-FINAL.png?1696524347"
  },
  {
    "chainId": 137,
    "address": "0x3235b13708f178af6f110de7177ed5de10c1093d",
    "name": "Mongol NFT",
    "symbol": "MNFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23718/thumb/swI93LWg_400x400.jpg?1696522918"
  },
  {
    "chainId": 137,
    "address": "0xe0bceef36f3a6efdd5eebfacd591423f8549b9d5",
    "name": "Defactor",
    "symbol": "FACTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19201/thumb/jFLSu4U9_400x400.png?1696518648"
  },
  {
    "chainId": 137,
    "address": "0xb03e3b00baf9954bf1604d09a4dbd5cf88e1f695",
    "name": "Open Campus",
    "symbol": "EDU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29948/thumb/EDU_Logo.png?1696528874"
  },
  {
    "chainId": 137,
    "address": "0xb9585ec9d4c97ad9ded7250bb9a199fe8eed0eca",
    "name": "WHALE",
    "symbol": "WHALE",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/11797/thumb/WHALE.png?1696511673"
  },
  {
    "chainId": 137,
    "address": "0x5eab32fe1d104ce0c5436fedc3433b646096e47c",
    "name": "LIF3 LSHARE",
    "symbol": "LSHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31575/thumb/LSHARE.png?1696530392"
  },
  {
    "chainId": 137,
    "address": "0x4ec203dd0699fac6adaf483cdd2519bc05d2c573",
    "name": "Cobak",
    "symbol": "CBK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13459/thumb/cbk-128-128.png?1696513222"
  },
  {
    "chainId": 137,
    "address": "0x2e1ad108ff1d8c782fcbbb89aad783ac49586756",
    "name": "Bridged TrueUSD",
    "symbol": "TUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30837/thumb/tusd.jpeg?1696529695"
  },
  {
    "chainId": 137,
    "address": "0xbbfe0b60de96a189bf09079de86a2db7bf0c7883",
    "name": "LunarCrush",
    "symbol": "LUNR",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/19256/thumb/LunarCrush-Earn-Logo.png?1708183703"
  },
  {
    "chainId": 137,
    "address": "0x66dc5a08091d1968e08c16aa5b27bac8398b02be",
    "name": "Civic",
    "symbol": "CVC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/788/thumb/civic-orange.png?1696501939"
  },
  {
    "chainId": 137,
    "address": "0xc19669a405067927865b40ea045a2baabbbe57f5",
    "name": "Star",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31277/thumb/STAR.png?1696530101"
  },
  {
    "chainId": 137,
    "address": "0xa47150f5c615dcf4f2e47aad1862a0f2b54c3973",
    "name": "LEAP Token",
    "symbol": "LEAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27258/thumb/LEAPtoken_LOGO.png?1696526311"
  },
  {
    "chainId": 137,
    "address": "0xe749ea14a2d18e361ed092ebefba64d77a8b4eac",
    "name": "Digitex",
    "symbol": "DGTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2188/thumb/DGTX.png?1696503145"
  },
  {
    "chainId": 137,
    "address": "0x6899face15c14348e1759371049ab64a3a06bfa6",
    "name": "SmarDex",
    "symbol": "SDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29470/thumb/SDEX_logo_transparent_outside_240x240.png?1696930070"
  },
  {
    "chainId": 137,
    "address": "0x77f56cf9365955486b12c4816992388ee8606f0e",
    "name": "Coin98",
    "symbol": "C98",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17117/thumb/logo.png?1696516677"
  },
  {
    "chainId": 137,
    "address": "0x03cf5d93ca7c70ce0a21a09f4d70779d2c66b25a",
    "name": "JumpToken",
    "symbol": "JMPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22603/thumb/200x200.png?1696521919"
  },
  {
    "chainId": 137,
    "address": "0x9cb74c8032b007466865f060ad2c46145d45553d",
    "name": "IDEX",
    "symbol": "IDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2565/thumb/idexlogo.png?1696503371"
  },
  {
    "chainId": 137,
    "address": "0x47c52f93a359db9f4509005cf982dfc440790561",
    "name": "Empower",
    "symbol": "MPWR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25456/thumb/MPWR_200x200.png?1696524587"
  },
  {
    "chainId": 137,
    "address": "0x94025780a1ab58868d9b2dbbb775f44b32e8e6e5",
    "name": "BetSwirl",
    "symbol": "BETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26618/thumb/icon_200.png?1696525691"
  },
  {
    "chainId": 137,
    "address": "0xaf24765f631c8830b5528b57002241ee7eef1c14",
    "name": "IOI Token",
    "symbol": "IOI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/15952/thumb/IOI_new_logo.png?1696515564"
  },
  {
    "chainId": 137,
    "address": "0xd55fce7cdab84d84f2ef3f99816d765a2a94a509",
    "name": "Chain Games",
    "symbol": "CHAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12257/thumb/chain-pfp-logo.png?1696512087"
  },
  {
    "chainId": 137,
    "address": "0x2a93172c8dccbfbc60a39d56183b7279a2f647b4",
    "name": "Decentral Games  Old ",
    "symbol": "DG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13267/thumb/%28Old%29_DG.png?1696513041"
  },
  {
    "chainId": 137,
    "address": "0x2c92a8a41f4b806a6f6f1f7c9d9dec78dcd8c18e",
    "name": "99Starz",
    "symbol": "STZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21467/thumb/stz.png?1696520828"
  },
  {
    "chainId": 137,
    "address": "0x6ddb31002abc64e1479fc439692f7ea061e78165",
    "name": "Furucombo",
    "symbol": "COMBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13629/thumb/COMBO_token_ol.png?1696513377"
  },
  {
    "chainId": 137,
    "address": "0xa7305ae84519ff8be02484cda45834c4e7d13dd6",
    "name": "UniFarm",
    "symbol": "UFARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15247/thumb/ufarm.jpeg?1696514900"
  },
  {
    "chainId": 137,
    "address": "0x24834bbec7e39ef42f4a75eaf8e5b6486d3f0e57",
    "name": "Wrapped Terra Classic",
    "symbol": "LUNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13628/thumb/wluna.png?1696513376"
  },
  {
    "chainId": 137,
    "address": "0xfc77cc32e570baec16f69dded556a35811b5a6ce",
    "name": "Stackswap",
    "symbol": "STSW",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29727/thumb/22.png?1696528657"
  },
  {
    "chainId": 137,
    "address": "0x0bd820ad2d7ab7305b5c9538ba824c9b9beb0561",
    "name": "Royale",
    "symbol": "ROYA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13602/thumb/roya.png?1696513353"
  },
  {
    "chainId": 137,
    "address": "0xc4bb7277a74678f053259cb1f96140347efbfd46",
    "name": "LunaChow",
    "symbol": "LUCHOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18805/thumb/J-MwYfhD_400x400.jpg?1696518267"
  },
  {
    "chainId": 137,
    "address": "0x66e8617d1df7ab523a316a6c01d16aa5bed93681",
    "name": "Spice Trade",
    "symbol": "SPICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25770/thumb/SPICE.png?1696524857"
  },
  {
    "chainId": 137,
    "address": "0xfbd8a3b908e764dbcd51e27992464b4432a1132b",
    "name": "Index Cooperative",
    "symbol": "INDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12729/thumb/index.png?1696512528"
  },
  {
    "chainId": 137,
    "address": "0x4e1581f01046efdd7a1a2cdb0f82cdd7f71f2e59",
    "name": "Popsicle Finance",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14586/thumb/ice.png?1696514266"
  },
  {
    "chainId": 137,
    "address": "0x49a0400587a7f65072c87c4910449fdcc5c47242",
    "name": "Magic Internet Money",
    "symbol": "MIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16786/thumb/mimlogopng.png?1696516358"
  },
  {
    "chainId": 137,
    "address": "0x8f36cc333f55b09bb71091409a3d7ade399e3b1c",
    "name": "Cherry Network",
    "symbol": "CHER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21855/thumb/cherry.PNG?1696521209"
  },
  {
    "chainId": 137,
    "address": "0xd5fa77a860fea9cff31da91bbf9e0faea9538290",
    "name": "Dog Collar",
    "symbol": "COLLAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18324/thumb/dcLogo.png?1696517815"
  },
  {
    "chainId": 137,
    "address": "0xb140665dde25c644c6b418e417c930de8a8a6ac9",
    "name": "Atari",
    "symbol": "ATRI",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/12992/thumb/AtariLogoPS_200x200_%281%29.png?1696512782"
  },
  {
    "chainId": 137,
    "address": "0xe229b734251dd48dda27bb908d90329f229c3531",
    "name": "Gram Platinum",
    "symbol": "GRAMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34368/thumb/gramp.png?1704727504"
  },
  {
    "chainId": 137,
    "address": "0xeee3371b89fc43ea970e908536fcddd975135d8a",
    "name": "The Doge NFT",
    "symbol": "DOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18111/thumb/Doge.png?1696517615"
  },
  {
    "chainId": 137,
    "address": "0x101a023270368c0d50bffb62780f4afd4ea79c35",
    "name": "Ankr Network",
    "symbol": "ANKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4324/thumb/U85xTl2.png?1696504928"
  },
  {
    "chainId": 137,
    "address": "0xee9a352f6aac4af1a5b9f467f6a93e0ffbe9dd35",
    "name": "MASQ",
    "symbol": "MASQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13699/thumb/masq.png?1696513446"
  },
  {
    "chainId": 137,
    "address": "0x218645f85ff27fc456ef46c3cdacbf5c40b2f9e8",
    "name": "Joystick club",
    "symbol": "JOY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21473/thumb/android-chrome-192x192.png?1696520834"
  },
  {
    "chainId": 137,
    "address": "0xe94845ac6782a2e71c407abe4d5201445c26a62b",
    "name": "BUILD",
    "symbol": "BUILD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26533/thumb/BUILD.png?1696525607"
  },
  {
    "chainId": 137,
    "address": "0x7f280dac515121dcda3eac69eb4c13a52392cace",
    "name": "Fancy Games",
    "symbol": "FNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21367/thumb/fnc.png?1696520733"
  },
  {
    "chainId": 137,
    "address": "0x68ee0d0aad9e1984af85ca224117e4d20eaf68be",
    "name": "Crypto Royale",
    "symbol": "ROY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20668/thumb/ROY_logo_new_design_small.png?1696520069"
  },
  {
    "chainId": 137,
    "address": "0x086373fad3447f7f86252fb59d56107e9e0faafa",
    "name": "Yup",
    "symbol": "YUP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12322/thumb/photo_2021-10-26_00-47-35.jpg?1696512151"
  },
  {
    "chainId": 137,
    "address": "0xdcb72ae4d5dc6ae274461d57e65db8d50d0a33ad",
    "name": "DappRadar",
    "symbol": "RADAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20894/thumb/radar.png?1696520286"
  },
  {
    "chainId": 137,
    "address": "0xf59036caebea7dc4b86638dfa2e3c97da9fccd40",
    "name": "Aave v3 wstETH",
    "symbol": "AWSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32890/thumb/WSTETH.png?1699776336"
  },
  {
    "chainId": 137,
    "address": "0x513c7e3a9c69ca3e22550ef58ac1c0088e918fff",
    "name": "Aave v3 CRV",
    "symbol": "ACRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32901/thumb/CRV.png?1699802172"
  },
  {
    "chainId": 137,
    "address": "0x8ffdf2de812095b1d19cb146e4c004587c0a0692",
    "name": "Aave v3 BAL",
    "symbol": "ABAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32903/thumb/BAL_%281%29.png?1699802595"
  },
  {
    "chainId": 137,
    "address": "0x5d47baba0d66083c52009271faf3f50dcc01023c",
    "name": "ApeSwap",
    "symbol": "BANANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14870/thumb/banana.png?1696514534"
  },
  {
    "chainId": 137,
    "address": "0xc14a7747cfec02cfea62e72bb93538de6b2078e6",
    "name": "WaweSwaps Global Token",
    "symbol": "GBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34205/thumb/gbl.png?1704275447"
  },
  {
    "chainId": 137,
    "address": "0xfef5d947472e72efbb2e388c730b7428406f2f95",
    "name": "Autonolas",
    "symbol": "OLAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31099/thumb/OLAS-token.png?1696529930"
  },
  {
    "chainId": 137,
    "address": "0x1a7e49125a6595588c9556f07a4c006461b24545",
    "name": "Aki Network",
    "symbol": "AKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29359/thumb/Aki-logo_3D.png?1703116985"
  },
  {
    "chainId": 137,
    "address": "0x57f12fe6a4e5fe819eec699fadf9db2d06606bb4",
    "name": "Neptune Mutual",
    "symbol": "NPM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28235/thumb/Neptune_Mutual_Logo_Icon_Regular_128.png?1696527238"
  },
  {
    "chainId": 137,
    "address": "0x9e20461bc2c4c980f62f1b279d71734207a6a356",
    "name": "OmniCat",
    "symbol": "OMNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33917/thumb/2023-12-21_21.39.10.jpg?1703321588"
  },
  {
    "chainId": 137,
    "address": "0xae55ab6a966863cb4c774ba8e6c0a37cfbea01f9",
    "name": "Gram Gold",
    "symbol": "GRAMG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34128/thumb/gramg.png?1704175641"
  },
  {
    "chainId": 137,
    "address": "0x3c499c542cef5e3811e1192ce70d8cc03d5c3359",
    "name": "USDC",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/6319/thumb/usdc.png?1696506694"
  },
  {
    "chainId": 137,
    "address": "0x385eeac5cb85a38a9a07a70c73e0a3271cfb54a7",
    "name": "Aavegotchi",
    "symbol": "GHST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12467/thumb/GHST.png?1696512286"
  },
  {
    "chainId": 137,
    "address": "0x831753dd7087cac61ab5644b308642cc1c33dc13",
    "name": "Quickswap  OLD ",
    "symbol": "QUICK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13970/thumb/quick.png?1696513704"
  },
  {
    "chainId": 137,
    "address": "0x8d1566569d5b695d44a9a234540f68d393cdc40d",
    "name": "GameCredits",
    "symbol": "GAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/193/thumb/XlQmXoU.png?1696501564"
  },
  {
    "chainId": 137,
    "address": "0xb33eaad8d922b1083446dc23f610c2567fb5180f",
    "name": "Uniswap",
    "symbol": "UNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12504/thumb/uni.jpg?1696512319"
  },
  {
    "chainId": 137,
    "address": "0x8f3cf7ad23cd3cadbd9735aff958023239c6a063",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 137,
    "address": "0x840195888db4d6a99ed9f73fcd3b225bb3cb1a79",
    "name": "SX Network  OLD ",
    "symbol": "SX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13779/thumb/sx.png?1696513520"
  },
  {
    "chainId": 137,
    "address": "0x72d6066f486bd0052eefb9114b66ae40e0a6031a",
    "name": "WazirX",
    "symbol": "WRX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10547/thumb/WazirX.png?1696510532"
  },
  {
    "chainId": 137,
    "address": "0x53e0bca35ec356bd5dddfebbd1fc0fd03fabad39",
    "name": "Chainlink",
    "symbol": "LINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/877/thumb/chainlink-new-logo.png?1696502009"
  },
  {
    "chainId": 137,
    "address": "0x0000000000000000000000000000000000001010",
    "name": "Polygon",
    "symbol": "MATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4713/thumb/polygon.png?1698233745"
  },
  {
    "chainId": 137,
    "address": "0x7fbc10850cae055b27039af31bd258430e714c62",
    "name": "Unibright",
    "symbol": "UBT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2707/thumb/UnibrightLogo_colorful_500x500_preview.png?1696503489"
  },
  {
    "chainId": 137,
    "address": "0xa2ca40dbe72028d3ac78b5250a8cb8c404e7fb8c",
    "name": "FEAR",
    "symbol": "FEAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15825/thumb/fear-logo-400-400.png?1696515443"
  },
  {
    "chainId": 137,
    "address": "0xd28449bb9bb659725accad52947677cce3719fd7",
    "name": "Dark Matter",
    "symbol": "DMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14223/thumb/dmt.jpg?1696513938"
  },
  {
    "chainId": 137,
    "address": "0xa1428174f516f527fafdd146b883bb4428682737",
    "name": "SuperVerse",
    "symbol": "SUPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14040/thumb/SV-Logo-200x200.png?1706880312"
  },
  {
    "chainId": 137,
    "address": "0x46f48fbdedaa6f5500993bede9539ef85f4bee8e",
    "name": "Arianee",
    "symbol": "ARIA20",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5054/thumb/Aria_Logo_256.png?1696505581"
  },
  {
    "chainId": 137,
    "address": "0xecf8f2fa183b1c4d2a269bf98a54fce86c812d3e",
    "name": "CyberFi",
    "symbol": "CFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13112/thumb/cyberfi_logo.jpeg?1696512897"
  },
  {
    "chainId": 137,
    "address": "0xab0b2ddb9c7e440fac8e140a89c0dbcbf2d7bbff",
    "name": "iFARM",
    "symbol": "IFARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14472/thumb/ifarm.png?1696514159"
  },
  {
    "chainId": 137,
    "address": "0x7c28f627ea3aec8b882b51eb1935f66e5b875714",
    "name": "MurAll",
    "symbol": "PAINT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14103/thumb/paint_logo_200x200.png?1696513825"
  },
  {
    "chainId": 137,
    "address": "0x521cddc0cba84f14c69c1e99249f781aa73ee0bc",
    "name": "Mover",
    "symbol": "MOVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13719/thumb/o0KIvs7T_400x400.jpg?1696513464"
  },
  {
    "chainId": 137,
    "address": "0x9c78ee466d6cb57a4d01fd887d2b5dfb2d46288f",
    "name": "Must",
    "symbol": "MUST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13688/thumb/must_logo.png?1696513436"
  },
  {
    "chainId": 137,
    "address": "0x1bfd67037b42cf73acf2047067bd4f2c47d9bfd6",
    "name": "Wrapped Bitcoin",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7598/thumb/wrapped_bitcoin_wbtc.png?1696507857"
  },
  {
    "chainId": 137,
    "address": "0x361a5a4993493ce00f61c32d4ecca5512b82ce90",
    "name": "Stake DAO",
    "symbol": "SDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13724/thumb/stakedao_logo.jpg?1696513468"
  },
  {
    "chainId": 137,
    "address": "0x52ede6bba83b7b4ba1d738df0df713d6a2036b71",
    "name": "0xMonero",
    "symbol": "0XMR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11035/thumb/0xmnr.PNG?1696510979"
  },
  {
    "chainId": 137,
    "address": "0x1a3acf6d19267e2d3e7f898f42803e90c9219062",
    "name": "Frax Share",
    "symbol": "FXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13423/thumb/Frax_Shares_icon.png?1696513183"
  },
  {
    "chainId": 137,
    "address": "0x2727ab1c2d22170abc9b595177b2d5c6e1ab7b7b",
    "name": "Cartesi",
    "symbol": "CTSI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11038/thumb/Cartesi_Logo.png?1696510982"
  },
  {
    "chainId": 137,
    "address": "0x1379e8886a944d2d9d440b3d88df536aea08d9f3",
    "name": "Mysterium",
    "symbol": "MYST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/757/thumb/mysterium.png?1696501911"
  },
  {
    "chainId": 137,
    "address": "0xd85d1e945766fea5eda9103f918bd915fbca63e6",
    "name": "Celsius Network",
    "symbol": "CEL",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/3263/thumb/CEL_logo.png?1696503976"
  },
  {
    "chainId": 137,
    "address": "0x23d29d30e35c5e8d321e1dc9a8a61bfd846d4c5c",
    "name": "HEX",
    "symbol": "HEX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10103/thumb/HEX-logo.png?1696510130"
  },
  {
    "chainId": 137,
    "address": "0xa1c57f48f0deb89f569dfbe6e2b7f46d33606fd4",
    "name": "Decentraland",
    "symbol": "MANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/878/thumb/decentraland-mana.png?1696502010"
  },
  {
    "chainId": 137,
    "address": "0xc3ec80343d2bae2f8e680fdadde7c17e71e114ea",
    "name": "MANTRA",
    "symbol": "OM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12151/thumb/OM_Token.png?1696511991"
  },
  {
    "chainId": 137,
    "address": "0x127984b5e6d5c59f81dacc9f1c8b3bdc8494572e",
    "name": "Pepedex",
    "symbol": "PPDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13022/thumb/output-onlinepngtools-1.png?1696512811"
  },
  {
    "chainId": 137,
    "address": "0x3809dcdd5dde24b37abe64a5a339784c3323c44f",
    "name": "TrustSwap",
    "symbol": "SWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11795/thumb/Untitled_design-removebg-preview.png?1696511671"
  },
  {
    "chainId": 137,
    "address": "0x034b2090b579228482520c589dbd397c53fc51cc",
    "name": "APY vision",
    "symbol": "VISION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13288/thumb/apyvisionlogo200circle.png?1696513060"
  },
  {
    "chainId": 137,
    "address": "0xda537104d6a5edd53c6fbba9a898708e465260b6",
    "name": "yearn finance",
    "symbol": "YFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11849/thumb/yearn.jpg?1696511720"
  },
  {
    "chainId": 137,
    "address": "0x033d942a6b495c4071083f4cde1f17e986fe856c",
    "name": "AGA",
    "symbol": "AGA",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/12180/thumb/aga-logo.png?1696512017"
  },
  {
    "chainId": 137,
    "address": "0x6c3b2f402cd7d22ae2c319b9d2f16f57927a4a17",
    "name": "KROWN",
    "symbol": "KRW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16530/thumb/KRW_token_logo_200x200.png?1696516093"
  },
  {
    "chainId": 137,
    "address": "0x34c1b299a74588d6abdc1b85a53345a48428a521",
    "name": "EasyFi V2",
    "symbol": "EZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12742/thumb/Logo_Icon.png?1696512540"
  },
  {
    "chainId": 137,
    "address": "0xfdc26cda2d2440d0e83cd1dee8e8be48405806dc",
    "name": "BTU Protocol",
    "symbol": "BTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3697/thumb/logo-btu-white-background_%281%29.png?1707407754"
  },
  {
    "chainId": 137,
    "address": "0xe82808eaa78339b06a691fd92e1be79671cad8d3",
    "name": "PlotX",
    "symbol": "PLOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12795/thumb/PlotX.png?1696512589"
  },
  {
    "chainId": 137,
    "address": "0xdf7837de1f2fa4631d716cf2502f8b230f1dcc32",
    "name": "Telcoin",
    "symbol": "TEL",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/1899/thumb/tel.png?1696502892"
  },
  {
    "chainId": 137,
    "address": "0xb9638272ad6998708de56bbc0a290a1de534a578",
    "name": "IQ",
    "symbol": "IQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5010/thumb/YAIS3fUh.png?1696505542"
  },
  {
    "chainId": 137,
    "address": "0xd6df932a45c0f255f85145f286ea0b292b21c90b",
    "name": "Aave",
    "symbol": "AAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12645/thumb/AAVE.png?1696512452"
  },
  {
    "chainId": 137,
    "address": "0x0b3f868e0be5597d5db7feb59e1cadbb0fdda50a",
    "name": "Sushi",
    "symbol": "SUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12271/thumb/512x512_Logo_no_chop.png?1696512101"
  },
  {
    "chainId": 137,
    "address": "0xc168e40227e4ebd8c1cae80f7a55a4f0e6d66c97",
    "name": "Dfyn Network",
    "symbol": "DFYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15368/thumb/SgqhfWz4_400x400_%281%29.jpg?1696515016"
  },
  {
    "chainId": 137,
    "address": "0x4fdce518fe527439fe76883e6b51a1c522b61b7c",
    "name": "Coreto",
    "symbol": "COR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12856/thumb/COR.png?1696512646"
  },
  {
    "chainId": 137,
    "address": "0x5b4cf2c120a9702225814e18543ee658c5f8631e",
    "name": "UniLend Finance",
    "symbol": "UFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12819/thumb/UniLend_Finance_logo_PNG.png?1696512611"
  },
  {
    "chainId": 137,
    "address": "0x6109cb051c5c64093830121ed76272ab04bbdd7c",
    "name": "Prosper",
    "symbol": "PROS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13668/thumb/heD6cg22l3sF5VgPh4G1xC6lnKEWXJif-jbaqUpv8CDP6jbWaqn9UjBdkXWNrw1CewaQOxb8zXRdNeNJWWiUDjfsEl_d7E3bPLg4cFoilQF5TGKHfWyJlnpm3UYc9ytvRvOjxOevMuiu8-lusnNoOcwgsJpMkYWHqe322GAxLt0_30kFMVAcjEDUrOlkK6hUYi0m9P433mvNlOm.jpg?1696513418"
  },
  {
    "chainId": 137,
    "address": "0xd99bafe5031cc8b345cb2e8c80135991f12d7130",
    "name": "Ferrum Network",
    "symbol": "FRM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8251/thumb/FRM.png?1696508455"
  },
  {
    "chainId": 137,
    "address": "0x2fe8733dcb25bfbba79292294347415417510067",
    "name": "Exeedme",
    "symbol": "XED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13518/thumb/exeedme.png?1696513279"
  },
  {
    "chainId": 137,
    "address": "0xa5eb60ca85898f8b26e18ff7c7e43623ccba772c",
    "name": "CosmicSwap",
    "symbol": "COSMIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16197/thumb/logo_-_2021-06-10T084120.309.png?1696515798"
  },
  {
    "chainId": 137,
    "address": "0x7ceb23fd6bc0add59e62ac25578270cff1b9f619",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 137,
    "address": "0x42dbbd5ae373fea2fc320f62d44c058522bb3758",
    "name": "Memecoin",
    "symbol": "MEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16370/thumb/mem_gold_200x200_copy.png?1696515969"
  },
  {
    "chainId": 137,
    "address": "0x72a5a58f79ffc2102227b92faeba93b169a3a3f1",
    "name": "Finance Vote",
    "symbol": "FVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13181/thumb/finance.png?1696512964"
  },
  {
    "chainId": 137,
    "address": "0xf629712180bef6f4c569b704e03d0acbe276eb6d",
    "name": "Wrapped Statera",
    "symbol": "WSTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13599/thumb/wsta_logo.png?1696513350"
  },
  {
    "chainId": 137,
    "address": "0xa6b37fc85d870711c56fbcb8afe2f8db049ae774",
    "name": "Pillar",
    "symbol": "PLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/809/thumb/v2logo-1.png?1696501958"
  },
  {
    "chainId": 137,
    "address": "0x9a71012b13ca4d3d0cdc72a177df3ef03b0e76a3",
    "name": "Balancer",
    "symbol": "BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11683/thumb/Balancer.png?1696511572"
  },
  {
    "chainId": 137,
    "address": "0x6aca77cf3bab0c4e8210a09b57b07854a995289a",
    "name": "Meeds DAO",
    "symbol": "MEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24281/thumb/s-YfFWYu_400x400.png?1696523464"
  },
  {
    "chainId": 137,
    "address": "0x9de41aff9f55219d5bf4359f167d1d0c772a396d",
    "name": "CAD Coin",
    "symbol": "CADC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14149/thumb/cadc_2.png?1696513868"
  },
  {
    "chainId": 137,
    "address": "0x1c954e8fe737f99f68fa1ccda3e51ebdb291948c",
    "name": "Kyber Network Crystal",
    "symbol": "KNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14899/thumb/RwdVsGcw_400x400.jpg?1696514562"
  },
  {
    "chainId": 137,
    "address": "0xe3ab61371ecc88534c522922a026f2296116c109",
    "name": "Mochi Market",
    "symbol": "MOMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14993/thumb/mochi.PNG?1696514657"
  },
  {
    "chainId": 137,
    "address": "0x83a6da342099835bcaa9c219dd76a5033c837de5",
    "name": "Basis Share",
    "symbol": "BAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13251/thumb/BAS.png?1696513026"
  },
  {
    "chainId": 137,
    "address": "0x08158a6b5d4018340387d1a302f882e98a8bc5b4",
    "name": "Plasma Finance",
    "symbol": "PPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13340/thumb/Hi9sEGAD.png?1696513107"
  },
  {
    "chainId": 137,
    "address": "0xf501dd45a1198c2e1b5aef5314a68b9006d842e0",
    "name": "mStable Governance  Meta",
    "symbol": "MTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11846/thumb/mStable.png?1696511717"
  },
  {
    "chainId": 137,
    "address": "0x6c0ab120dbd11ba701aff6748568311668f63fe0",
    "name": "Spectra",
    "symbol": "APW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15597/thumb/spectra.png?1696515232"
  },
  {
    "chainId": 137,
    "address": "0xbe5cf150e1ff59ca7f2499eaa13bfc40aae70e78",
    "name": "Glitch Protocol",
    "symbol": "GLCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13712/thumb/glitch_logo.jpeg?1696513457"
  },
  {
    "chainId": 137,
    "address": "0xf88332547c680f755481bf489d890426248bb275",
    "name": "inSure DeFi",
    "symbol": "SURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10354/thumb/logo-grey-circle.png?1696510355"
  },
  {
    "chainId": 137,
    "address": "0x47536f17f4ff30e64a96a7555826b8f9e66ec468",
    "name": "Beluga fi",
    "symbol": "BELUGA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13790/thumb/brand-logo-v2.23e5d279.png?1696513539"
  },
  {
    "chainId": 137,
    "address": "0x6ccf12b480a99c54b23647c995f4525d544a7e72",
    "name": "Starter xyz",
    "symbol": "START",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14301/thumb/logo_poly_sym.png?1696513997"
  },
  {
    "chainId": 137,
    "address": "0x172370d5cd63279efa6d502dab29171933a610af",
    "name": "Curve DAO",
    "symbol": "CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12124/thumb/Curve.png?1696511967"
  },
  {
    "chainId": 137,
    "address": "0x50b728d8d964fd00c2d0aad81718b71311fef68a",
    "name": "Synthetix Network",
    "symbol": "SNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3406/thumb/SNX.png?1696504103"
  },
  {
    "chainId": 137,
    "address": "0xfed16c746cb5bfed009730f9e3e6a673006105c7",
    "name": "Digital Reserve Currency",
    "symbol": "DRC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/12802/thumb/DRC_Logo.jpg?1696512595"
  },
  {
    "chainId": 137,
    "address": "0x4cebdbcb286101a17d3ea1f7fe7bbded2b2053dd",
    "name": "Yield App",
    "symbol": "YLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13365/thumb/Google_Play_Store_Icon.png?1696513130"
  },
  {
    "chainId": 137,
    "address": "0xa0e390e9cea0d0e8cd40048ced9fa9ea10d71639",
    "name": "DSLA Protocol",
    "symbol": "DSLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6694/thumb/dsla_logo-squared_200x200.png?1696507035"
  },
  {
    "chainId": 137,
    "address": "0xb565cf70613ca464d68427106af80c67a8e4b801",
    "name": "YVS Finance",
    "symbol": "YVS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13471/thumb/cu0LSzE.png?1696513232"
  },
  {
    "chainId": 137,
    "address": "0x692597b009d13c4049a947cab2239b7d6517875f",
    "name": "Wrapped USTC",
    "symbol": "USTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15462/thumb/ust.png?1696515108"
  },
  {
    "chainId": 137,
    "address": "0x2b9e7ccdf0f4e5b24757c1e1a80e311e34cb10c7",
    "name": "Mask Network",
    "symbol": "MASK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14051/thumb/Mask_Network.jpg?1696513776"
  },
  {
    "chainId": 137,
    "address": "0x9c6bfedc14b5c23e3900889436edca7805170f01",
    "name": "Phoenix Finance",
    "symbol": "PHX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17675/thumb/phx_logo.png?1696517205"
  },
  {
    "chainId": 137,
    "address": "0xa8fcee762642f156b5d757b6fabc36e06b6d4a1a",
    "name": "Aluna",
    "symbol": "ALN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14379/thumb/uaLoLU8c_400x400_%281%29.png?1696514071"
  },
  {
    "chainId": 137,
    "address": "0x70c006878a5a50ed185ac4c87d837633923de296",
    "name": "REVV",
    "symbol": "REVV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12373/thumb/REVV_TOKEN_Refined_2021_%281%29.png?1696512197"
  },
  {
    "chainId": 137,
    "address": "0xee800b277a96b0f490a1a732e1d6395fad960a26",
    "name": "ARPA",
    "symbol": "ARPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8506/thumb/9u0a23XY_400x400.jpg?1696508685"
  },
  {
    "chainId": 137,
    "address": "0x855d4248672a1fce482165e8dbe1207b94b1968a",
    "name": "WOWswap",
    "symbol": "WOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14101/thumb/Group-423.png?1696513823"
  },
  {
    "chainId": 137,
    "address": "0xab5f7a0e20b0d056aed4aa4528c78da45be7308b",
    "name": "Obyte",
    "symbol": "GBYTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/561/thumb/byteball.png?1696501768"
  },
  {
    "chainId": 137,
    "address": "0xa55870278d6389ec5b524553d03c04f5677c061e",
    "name": "XCAD Network",
    "symbol": "XCAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15857/thumb/xcad_logo.jpg?1707832192"
  },
  {
    "chainId": 137,
    "address": "0x904371845bc56dcbbcf0225ef84a669b2fd6bd0d",
    "name": "Relay Chain",
    "symbol": "RELAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17816/thumb/relay-logo-200.png?1696517336"
  },
  {
    "chainId": 137,
    "address": "0x692ac1e363ae34b6b489148152b12e2785a3d8d6",
    "name": "Polytrade",
    "symbol": "TRADE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16416/thumb/Logo_colored_200.png?1696516012"
  },
  {
    "chainId": 137,
    "address": "0x5fe2b58c013d7601147dcdd68c143a77499f5531",
    "name": "The Graph",
    "symbol": "GRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13397/thumb/Graph_Token.png?1696513159"
  },
  {
    "chainId": 137,
    "address": "0x5c7f7fe4766fe8f0fa9b41e2e4194d939488ff1c",
    "name": "Doki Doki",
    "symbol": "DOKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12759/thumb/doki_logo.png?1696512556"
  },
  {
    "chainId": 137,
    "address": "0x3cef98bb43d732e2f285ee605a8158cde967d219",
    "name": "Basic Attention",
    "symbol": "BAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/677/thumb/basic-attention-token.png?1696501867"
  },
  {
    "chainId": 137,
    "address": "0x9c2c5fd7b07e95ee044ddeba0e97a665f142394f",
    "name": "1inch",
    "symbol": "1INCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13469/thumb/1inch-token.png?1696513230"
  },
  {
    "chainId": 137,
    "address": "0xcfb54a6d2da14abecd231174fc5735b4436965d8",
    "name": "Cyclone Protocol",
    "symbol": "CYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14065/thumb/b3_DFjFp_400x400.jpg?1696513790"
  },
  {
    "chainId": 137,
    "address": "0x1c0a798b5a5273a9e54028eb1524fd337b24145f",
    "name": "Loser Coin",
    "symbol": "LOWB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15378/thumb/loser.PNG?1696515025"
  },
  {
    "chainId": 137,
    "address": "0x11a819beb0aa3327e39f52f90d65cc9bca499f33",
    "name": "Scaleswap",
    "symbol": "SCA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16360/thumb/thumbnail_1170823958_vertical_logo_lateral_radiance.png?1696515959"
  },
  {
    "chainId": 137,
    "address": "0xdbf31df14b66535af65aac99c32e9ea844e14501",
    "name": "renBTC",
    "symbol": "RENBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/11370/thumb/Bitcoin.jpg?1696511287"
  },
  {
    "chainId": 137,
    "address": "0x85955046df4668e1dd369d2de9f3aeb98dd2a369",
    "name": "DeFi Pulse Index",
    "symbol": "DPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12465/thumb/defi_pulse_index_set.png?1696512284"
  },
  {
    "chainId": 137,
    "address": "0x16eccfdbb4ee1a85a33f3a9b21175cd7ae753db4",
    "name": "Router Protocol",
    "symbol": "ROUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13709/thumb/route_token_200x200-19.png?1696513454"
  },
  {
    "chainId": 137,
    "address": "0xf521d590fb1e0b432fd0e020cdbd6c6397d652c2",
    "name": "Parachute",
    "symbol": "PAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7590/thumb/Parachute_Logo.png?1696507850"
  },
  {
    "chainId": 137,
    "address": "0x8505b9d2254a7ae468c0e9dd10ccea3a837aef5c",
    "name": "Compound",
    "symbol": "COMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10775/thumb/COMP.png?1696510737"
  },
  {
    "chainId": 137,
    "address": "0xee7666aacaefaa6efeef62ea40176d3eb21953b9",
    "name": "MCH Coin",
    "symbol": "MCHC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15399/thumb/MCHC.jpg?1696515045"
  },
  {
    "chainId": 137,
    "address": "0xfd4959c06fbcc02250952daebf8e0fb38cf9fd8c",
    "name": "ZeroSwap",
    "symbol": "ZEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12861/thumb/logo.?1696512650"
  },
  {
    "chainId": 137,
    "address": "0xdb725f82818de83e99f1dac22a9b5b51d3d04dd4",
    "name": "GET Protocol",
    "symbol": "GET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1927/thumb/GET_Protocol.png?1696502916"
  },
  {
    "chainId": 137,
    "address": "0xcaf5191fc480f43e4df80106c7695eca56e48b18",
    "name": "DinoX",
    "symbol": "DNXC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17321/thumb/asset_icon_dnxc_200.png?1696516875"
  },
  {
    "chainId": 137,
    "address": "0x381caf412b45dac0f62fbeec89de306d3eabe384",
    "name": "DAO Invest",
    "symbol": "VEST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17901/thumb/logo-round-200.png?1696517421"
  },
  {
    "chainId": 137,
    "address": "0xf4c83080e80ae530d6f8180572cbbf1ac9d5d435",
    "name": "BlockWallet",
    "symbol": "BLANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14209/thumb/Group_121417608.png?1696513925"
  },
  {
    "chainId": 137,
    "address": "0xfc40a4f89b410a1b855b5e205064a38fc29f5eb5",
    "name": "rUSD",
    "symbol": "RUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16184/thumb/rUSD-Logo-200.png?1696515786"
  },
  {
    "chainId": 137,
    "address": "0x598e49f01befeb1753737934a5b11fea9119c796",
    "name": "Adshares",
    "symbol": "ADS",
    "decimals": 11,
    "logoURI": "https://assets.coingecko.com/coins/images/868/thumb/rnO9DyJ.png?1696502001"
  },
  {
    "chainId": 137,
    "address": "0xdc0e17eae3b9651875030244b971fa0223a1764f",
    "name": "PERI Finance",
    "symbol": "PERI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15313/thumb/6xVEMS1.png?1696514963"
  },
  {
    "chainId": 137,
    "address": "0x6fb54ffe60386ac33b722be13d2549dd87bf63af",
    "name": "Polinate",
    "symbol": "POLI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18096/thumb/Polinate_Games___Guilds_Elements-05.png?1696517601"
  },
  {
    "chainId": 137,
    "address": "0x2df54842cd85c60f21b4871e09bcc6047b2dcc4d",
    "name": "Immortl  OLD ",
    "symbol": "IMRTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21983/thumb/ONE.png?1696521331"
  },
  {
    "chainId": 137,
    "address": "0x3f95aa88ddbb7d9d484aa3d482bf0a80009c52c9",
    "name": "Backed ERNX   Bond",
    "symbol": "BERNX",
    "decimals": 18
  },
  {
    "chainId": 137,
    "address": "0xd0e9c8f5fae381459cf07ec506c1d2896e8b5df6",
    "name": "Internet of Energy Network",
    "symbol": "IOEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19095/thumb/12799.png?1696518548"
  },
  {
    "chainId": 137,
    "address": "0x3c720206bfacb2d16fa3ac0ed87d2048dbc401fc",
    "name": "Archethic",
    "symbol": "UCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12330/thumb/Archethic_logo.png?1696512158"
  },
  {
    "chainId": 137,
    "address": "0xd6a5ab46ead26f49b03bbb1f9eb1ad5c1767974a",
    "name": "Ethermon",
    "symbol": "EMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15889/thumb/LtET0reH_400x400.jpg?1696515504"
  },
  {
    "chainId": 137,
    "address": "0xd5d86fc8d5c0ea1ac1ac5dfab6e529c9967a45e9",
    "name": "NFT Worlds",
    "symbol": "WRLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22112/thumb/ZyBrRgfO.jpg?1696521454"
  },
  {
    "chainId": 137,
    "address": "0x2ab4f9ac80f33071211729e45cfc346c1f8446d5",
    "name": "Chain Guardians",
    "symbol": "CGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14326/thumb/cgg_logo.png?1696514015"
  },
  {
    "chainId": 137,
    "address": "0x614389eaae0a6821dc49062d56bda3d9d45fa2ff",
    "name": "Orbs",
    "symbol": "ORBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4630/thumb/Orbs.jpg?1696505200"
  },
  {
    "chainId": 137,
    "address": "0x5198e7cc1640049de37d1bd10b03fa5a3afda120",
    "name": "Kaby Arena",
    "symbol": "KABY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18090/thumb/URPKhnop_400x400.jpg?1696517595"
  },
  {
    "chainId": 137,
    "address": "0x6ab6d61428fde76768d7b45d8bfeec19c6ef91a8",
    "name": "Anyswap",
    "symbol": "ANY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12242/thumb/anyswap.jpg?1696512074"
  },
  {
    "chainId": 137,
    "address": "0x34f380a4e3389e99c0369264453523bbe5af7fab",
    "name": "Kangal",
    "symbol": "KANGAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14241/thumb/logo-200.png?1696513956"
  },
  {
    "chainId": 137,
    "address": "0x00d5149cdf7cec8725bf50073c51c4fa58ecca12",
    "name": "UniPower",
    "symbol": "POWER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11618/thumb/unipower.png?1696511512"
  },
  {
    "chainId": 137,
    "address": "0xf4b0903774532aee5ee567c02aab681a81539e92",
    "name": "Gaj Finance",
    "symbol": "GAJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15257/thumb/logo200x200.png?1696514910"
  },
  {
    "chainId": 137,
    "address": "0xf8f9efc0db77d8881500bb06ff5d6abc3070e695",
    "name": "Synapse",
    "symbol": "SYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18024/thumb/synapse_social_icon.png?1696517540"
  },
  {
    "chainId": 137,
    "address": "0x554f074d9ccda8f483d1812d4874cbebd682644e",
    "name": "AnRKey X",
    "symbol": "ANRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13415/thumb/anrkey.jpg?1696513176"
  },
  {
    "chainId": 137,
    "address": "0xaecebfcf604ad245eaf0d5bd68459c3a7a6399c2",
    "name": "RAMP  OLD ",
    "symbol": "RAMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12837/thumb/RAMP-Logo-v2-1000pxsq.png?1696512628"
  },
  {
    "chainId": 137,
    "address": "0x5fcb9de282af6122ce3518cde28b7089c9f97b26",
    "name": "DeHive",
    "symbol": "DHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14926/thumb/logo_200x200.png?1696514587"
  },
  {
    "chainId": 137,
    "address": "0x0e98c977b943f06075b2d795794238fbfb9b9a34",
    "name": "Tomb",
    "symbol": "TOMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16133/thumb/tomb_icon_noBG.png?1696515739"
  },
  {
    "chainId": 137,
    "address": "0xad684e79ce4b6d464f2ff7c3fd51646892e24b96",
    "name": "Autonio",
    "symbol": "NIOX",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/1122/thumb/NewLogo.png?1696502216"
  },
  {
    "chainId": 137,
    "address": "0xe9993763e0b7f7d915a62a5f22a6e151f91d98a8",
    "name": "TORG",
    "symbol": "TORG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17596/thumb/TORG_Logo_200x200.png?1696517129"
  },
  {
    "chainId": 137,
    "address": "0x280053c54006a624c26989cb8354fa4cb86f14d1",
    "name": "Morpheus Labs",
    "symbol": "MIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3164/thumb/mitx.png?1696503886"
  },
  {
    "chainId": 137,
    "address": "0xdae89da41a96956e9e70320ac9c0dd077070d3a5",
    "name": "Centaur",
    "symbol": "CNTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12743/thumb/logo_%2898%29.png?1696512542"
  },
  {
    "chainId": 137,
    "address": "0x65ad6a2288b2dd23e466226397c8f5d1794e58fc",
    "name": "GamyFi",
    "symbol": "GFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14559/thumb/circle-cropped_%281%29.png?1696514241"
  },
  {
    "chainId": 137,
    "address": "0x8a0e8b4b0903929f47c3ea30973940d4a9702067",
    "name": "InsurAce",
    "symbol": "INSUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14226/thumb/insur.png?1696513941"
  },
  {
    "chainId": 137,
    "address": "0x39fc9e94caeacb435842fadedecb783589f50f5f",
    "name": "Kampay",
    "symbol": "KAMPAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8289/thumb/6130e86152bdb07e2848de00_200_4x.png?1696508490"
  },
  {
    "chainId": 137,
    "address": "0x7f67639ffc8c93dd558d452b8920b28815638c44",
    "name": "iMe Lab",
    "symbol": "LIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16243/thumb/lim_200.2.png?1696515843"
  },
  {
    "chainId": 137,
    "address": "0x8346ab8d5ea7a9db0209aed2d1806afa0e2c4c21",
    "name": "Modefi",
    "symbol": "MOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13980/thumb/modefi_logo.png?1696513713"
  },
  {
    "chainId": 137,
    "address": "0x632b8c4e95b2f8a9309417f8d990ab9c04c77369",
    "name": "Weble Ecosystem",
    "symbol": "WET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17353/thumb/cropped-logo-wombat.png?1696516905"
  },
  {
    "chainId": 137,
    "address": "0x71b821aa52a49f32eed535fca6eb5aa130085978",
    "name": "0xBitcoin",
    "symbol": "0XBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4454/thumb/0xbtc.png?1696505045"
  },
  {
    "chainId": 137,
    "address": "0x4e3decbb3645551b8a19f0ea1678079fcb33fb4c",
    "name": "Jarvis Synthetic Euro",
    "symbol": "JEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15725/thumb/jEUR.png?1696515352"
  },
  {
    "chainId": 137,
    "address": "0xbd1463f02f61676d53fd183c2b19282bff93d099",
    "name": "Jarvis Synthetic Swiss Franc",
    "symbol": "JCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15727/thumb/jCHF.png?1696515353"
  },
  {
    "chainId": 137,
    "address": "0x0d6ae2a429df13e44a07cd2969e085e4833f64a0",
    "name": "PolkaBridge",
    "symbol": "PBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13744/thumb/symbol-whitebg200x200.png?1696513488"
  },
  {
    "chainId": 137,
    "address": "0x4597c8a59ab28b36840b82b3a674994a279593d0",
    "name": "Circuits of Value",
    "symbol": "COVAL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/588/thumb/coval-logo.png?1696501792"
  },
  {
    "chainId": 137,
    "address": "0x2b88ad57897a8b496595925f43048301c37615da",
    "name": "Pickle Finance",
    "symbol": "PICKLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12435/thumb/0M4W6Yr6_400x400.jpg?1696512255"
  },
  {
    "chainId": 137,
    "address": "0x211f4e76fcb811ed2b310a232a24b3445d95e3bc",
    "name": "Matrix Labs",
    "symbol": "MATRIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18297/thumb/matrixlabs.png?1696517789"
  },
  {
    "chainId": 137,
    "address": "0x596ebe76e2db4470966ea395b0d063ac6197a8c5",
    "name": "Jarvis Reward",
    "symbol": "JRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10390/thumb/cfeii0y.png?1696510389"
  },
  {
    "chainId": 137,
    "address": "0x00e5646f60ac6fb446f621d146b6e1886f002905",
    "name": "Rai Reflex Index",
    "symbol": "RAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14004/thumb/RAI-logo-coin.png?1696513733"
  },
  {
    "chainId": 137,
    "address": "0xe111178a87a3bff0c8d18decba5798827539ae99",
    "name": "STASIS EURO",
    "symbol": "EURS",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/5164/thumb/EURS_300x300.png?1696505680"
  },
  {
    "chainId": 137,
    "address": "0xe2aa7db6da1dae97c5f5c6914d285fbfcc32a128",
    "name": "Parallel",
    "symbol": "PAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14153/thumb/par_round_200.png?1696513872"
  },
  {
    "chainId": 137,
    "address": "0xe0ce60af0850bf54072635e66e79df17082a1109",
    "name": "PayRue",
    "symbol": "PROPEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8794/thumb/-RNl00DU_400x400.jpg?1696508949"
  },
  {
    "chainId": 137,
    "address": "0x4287f07cbe6954f9f0decd91d0705c926d8d03a4",
    "name": "Trace Network Labs",
    "symbol": "TRACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23266/thumb/Token_Icon_02.png?1696522486"
  },
  {
    "chainId": 137,
    "address": "0x6286a9e6f7e745a6d884561d88f94542d6715698",
    "name": "Cryptomeda",
    "symbol": "TECH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17983/thumb/tech.png?1696517501"
  },
  {
    "chainId": 137,
    "address": "0xadac33f543267c4d59a8c299cf804c303bc3e4ac",
    "name": "Mimo Governance",
    "symbol": "MIMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16449/thumb/mimodefi.PNG?1696516045"
  },
  {
    "chainId": 137,
    "address": "0x0df0f72ee0e5c9b7ca761ecec42754992b2da5bf",
    "name": "Automata",
    "symbol": "ATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15985/thumb/ATA.jpg?1696515598"
  },
  {
    "chainId": 137,
    "address": "0x709a4b6217584188ddb93c82f5d716d969acce1c",
    "name": "Hanu Yokia",
    "symbol": "HANU",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/17161/thumb/Goji_Hanu_Logo_200x200.png?1696516720"
  },
  {
    "chainId": 137,
    "address": "0xc91c06db0f7bffba61e2a5645cc15686f0a8c828",
    "name": "Razor Network",
    "symbol": "RAZOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13797/thumb/icon.png?1696513545"
  },
  {
    "chainId": 137,
    "address": "0x7e7ff932fab08a0af569f93ce65e7b8b23698ad8",
    "name": "YfDAI finance",
    "symbol": "YF-DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12385/thumb/1619048513068.png?1696512208"
  },
  {
    "chainId": 137,
    "address": "0x4e830f67ec499e69930867f9017aeb5b3f629c73",
    "name": "Oddz",
    "symbol": "ODDZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14421/thumb/NewLogo.png?1696514112"
  },
  {
    "chainId": 137,
    "address": "0xde5ed76e7c05ec5e4572cfc88d1acea165109e44",
    "name": "DEUS Finance",
    "symbol": "DEUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18778/thumb/Black_Background_200x200.png?1696518242"
  },
  {
    "chainId": 137,
    "address": "0x36fe11b6d5c9421f68d235694fe192b35e803903",
    "name": "Xend Finance",
    "symbol": "RWA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14496/thumb/WeChat_Image_20210325163206.png?1696514181"
  },
  {
    "chainId": 137,
    "address": "0xa7051c5a22d963b81d71c2ba64d46a877fbc1821",
    "name": "Sifchain",
    "symbol": "EROWAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14044/thumb/EROWAN.png?1696513770"
  },
  {
    "chainId": 137,
    "address": "0x23f07a1c03e7c6d0c88e0e05e79b6e3511073fd5",
    "name": "Crypto Development Services",
    "symbol": "CDS",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/21304/thumb/JhUZ3Rk.png?1696520673"
  },
  {
    "chainId": 137,
    "address": "0x8c92e38eca8210f4fcbf17f0951b198dd7668292",
    "name": "dHEDGE DAO",
    "symbol": "DHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12508/thumb/dht.png?1696512323"
  },
  {
    "chainId": 137,
    "address": "0x3b1a0c9252ee7403093ff55b4a5886d49a3d837a",
    "name": "Continuum World",
    "symbol": "UM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18798/thumb/Moneda.png?1696518260"
  },
  {
    "chainId": 137,
    "address": "0xce6bf09e5c7a3e65b84f88dcc6475c88d38ba5ef",
    "name": "Opacity",
    "symbol": "OPCT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7237/thumb/Opacity.jpg?1696507532"
  },
  {
    "chainId": 137,
    "address": "0xb41ec2c036f8a42da384dde6ada79884f8b84b26",
    "name": "Tidal Finance",
    "symbol": "TIDAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14460/thumb/Tidal-mono.png?1696514147"
  },
  {
    "chainId": 137,
    "address": "0xf4bb0ed25ac7bcc9c327b88bac5ca288a08ec41e",
    "name": "Rari Governance",
    "symbol": "RGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12900/thumb/Rari_Logo_Transparent.png?1696512688"
  },
  {
    "chainId": 137,
    "address": "0x4d0def42cf57d6f27cd4983042a55dce1c9f853c",
    "name": "DexKit",
    "symbol": "KIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13187/thumb/7739.png?1696512969"
  },
  {
    "chainId": 137,
    "address": "0x92c59f1cc9a322670cca29594e4d994d48bdfd36",
    "name": "PhoenixDAO",
    "symbol": "PHNX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11523/thumb/Token_Icon.png?1696511425"
  },
  {
    "chainId": 137,
    "address": "0xc10358f062663448a3489fc258139944534592ac",
    "name": "Blockchain Monster Hunt",
    "symbol": "BCMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19045/thumb/bcmc-coin-200x200.png?1696518496"
  },
  {
    "chainId": 137,
    "address": "0xd52f6ca48882be8fbaa98ce390db18e1dbe1062d",
    "name": "ORE Network",
    "symbol": "ORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18917/thumb/ORE_FullColor.png?1696518374"
  },
  {
    "chainId": 137,
    "address": "0x25788a1a171ec66da6502f9975a15b609ff54cf6",
    "name": "PoolTogether",
    "symbol": "POOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14003/thumb/PoolTogether.png?1696513732"
  },
  {
    "chainId": 137,
    "address": "0x87847703d4bb4fcd42db887ffdcb94496e77e3ab",
    "name": "Hypersign Identity",
    "symbol": "HID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16158/thumb/hypersign.png?1696515762"
  },
  {
    "chainId": 137,
    "address": "0x49b1be61a8ca3f9a9f178d6550e41e00d9162159",
    "name": "GG",
    "symbol": "GGTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13666/thumb/ggtk.png?1696513416"
  },
  {
    "chainId": 137,
    "address": "0x09211dc67f9fe98fb7bbb91be0ef05f4a12fa2b2",
    "name": "Yieldwatch",
    "symbol": "WATCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14186/thumb/WATCH1.png?1696513904"
  },
  {
    "chainId": 137,
    "address": "0x7cdc0421469398e0f3aa8890693d86c840ac8931",
    "name": "Azuki",
    "symbol": "AZUKI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13091/thumb/bdUBSCo.png?1696512878"
  },
  {
    "chainId": 137,
    "address": "0x0e50bea95fe001a370a4f1c220c49aedcb982dec",
    "name": "Ethernity Chain",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14238/thumb/LOGO_HIGH_QUALITY.png?1696513952"
  },
  {
    "chainId": 137,
    "address": "0xbc2597d3f1f9565100582cde02e3712d03b8b0f6",
    "name": "CheesecakeSwap",
    "symbol": "CCAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14547/thumb/CCAKElogo.png?1696514230"
  },
  {
    "chainId": 137,
    "address": "0x863d6074afaf02d9d41a5f8ea83278df7089aa86",
    "name": "CryptoBlades",
    "symbol": "SKILL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15334/thumb/cryptoblade.PNG?1696514982"
  },
  {
    "chainId": 137,
    "address": "0x1ba17c639bdaecd8dc4aac37df062d17ee43a1b8",
    "name": "IX Swap",
    "symbol": "IXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18069/thumb/ixswap.PNG?1696517577"
  },
  {
    "chainId": 137,
    "address": "0xf1c1a3c2481a3a8a3f173a9ab5ade275292a6fa3",
    "name": "BLOCKv",
    "symbol": "VEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1266/thumb/blockv.png?1696502339"
  },
  {
    "chainId": 137,
    "address": "0xe0339c80ffde91f3e20494df88d4206d86024cdf",
    "name": "Dogelon Mars",
    "symbol": "ELON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14962/thumb/6GxcPRo3_400x400.jpg?1696514622"
  },
  {
    "chainId": 137,
    "address": "0x45c32fa6df82ead1e2ef74d17b76547eddfaff89",
    "name": "Frax",
    "symbol": "FRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13422/thumb/FRAX_icon.png?1696513182"
  },
  {
    "chainId": 137,
    "address": "0x11602a402281974a70c2b4824d58ebede967e2be",
    "name": "NBX",
    "symbol": "BYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14746/thumb/NBX.png?1696514416"
  },
  {
    "chainId": 137,
    "address": "0x32515ffdc3a84cfbf9ad4db14ef8f0a535c7afd6",
    "name": "Baked",
    "symbol": "BAKED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19178/thumb/rebaked-logo-full.png?1696518627"
  },
  {
    "chainId": 137,
    "address": "0xa486c6bc102f409180ccb8a94ba045d39f8fc7cb",
    "name": "Nash",
    "symbol": "NEX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/3246/thumb/Nash_M.png?1696503962"
  },
  {
    "chainId": 137,
    "address": "0x04429fbb948bbd09327763214b45e505a5293346",
    "name": "Allbridge",
    "symbol": "ABR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18690/thumb/logo.png?1701737985"
  },
  {
    "chainId": 137,
    "address": "0x6968105460f67c3bf751be7c15f92f5286fd0ce5",
    "name": "Monavale",
    "symbol": "MONA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13298/thumb/monavale_logo.jpg?1696513070"
  },
  {
    "chainId": 137,
    "address": "0x2fd4d793c1905d82018d75e3b09d3035856890a1",
    "name": "Spherium",
    "symbol": "SPHRI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17787/thumb/Group_15.png?1696517311"
  },
  {
    "chainId": 137,
    "address": "0x8eef5a82e6aa222a60f009ac18c24ee12dbf4b41",
    "name": "Autobahn Network",
    "symbol": "TXL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12432/thumb/txl.png?1696512252"
  },
  {
    "chainId": 137,
    "address": "0x55b1a124c04a54eefdefe5fa2ef5f852fb5f2f26",
    "name": "Ethereum Meta",
    "symbol": "ETHM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6586/thumb/ethereum-meta.png?1696506937"
  },
  {
    "chainId": 137,
    "address": "0x26373ec913876c9e6d38494dde458cb8649cb30c",
    "name": "Ojamu",
    "symbol": "OJA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18947/thumb/ojamu-icon-PNK.png?1696518402"
  },
  {
    "chainId": 137,
    "address": "0x3c1bb39bb696b443a1d80bb2b3a3d950ba9dee87",
    "name": "Wall Street Games  OLD ",
    "symbol": "WSG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15872/thumb/X3Awe42.png?1696515487"
  },
  {
    "chainId": 137,
    "address": "0xfe712251173a2cd5f5be2b46bb528328ea3565e1",
    "name": "Metaverse Index",
    "symbol": "MVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14684/thumb/MVI_logo.png?1696514357"
  },
  {
    "chainId": 137,
    "address": "0xa0cb0ce7c6d93a7ebd72952feb4407dddee8a194",
    "name": "Shibaken Finance",
    "symbol": "SHIBAKEN",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/15413/thumb/shibak.png?1696515059"
  },
  {
    "chainId": 137,
    "address": "0xf0059cc2b3e980065a906940fbce5f9db7ae40a7",
    "name": "Shardus",
    "symbol": "ULT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8383/thumb/final_logo_photoshop.png?1696508575"
  },
  {
    "chainId": 137,
    "address": "0xe20d2df5041f8ed06976846470f727295cdd4d23",
    "name": "0x nodes",
    "symbol": "BIOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15600/thumb/BIOS_01.png?1696515235"
  },
  {
    "chainId": 137,
    "address": "0xdfc3829b127761a3218bfcee7fc92e1232c9d116",
    "name": "PRivaCY Coin",
    "symbol": "PRCY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/14151/thumb/prcy.png?1696513870"
  },
  {
    "chainId": 137,
    "address": "0xffb89d7637cf4860884ed48b57ae5562bf64e10f",
    "name": "Pika",
    "symbol": "PIKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14419/thumb/pika-logo-2022-nbg.png?1696514110"
  },
  {
    "chainId": 137,
    "address": "0x27f485b62c4a7e635f561a87560adf5090239e93",
    "name": "DFX Finance",
    "symbol": "DFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14091/thumb/DFX.png?1696513813"
  },
  {
    "chainId": 137,
    "address": "0x1b599beb7b1f50807dd58fd7e8ffcf073b435e71",
    "name": "Blind Boxes",
    "symbol": "BLES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14537/thumb/BLES-Logo-BW.png?1696514221"
  },
  {
    "chainId": 137,
    "address": "0xe79feaaa457ad7899357e8e2065a3267ac9ee601",
    "name": "XAYA",
    "symbol": "WCHI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/2091/thumb/xaya200x200.png?1696503058"
  },
  {
    "chainId": 137,
    "address": "0x4df071fb2d145be595b767f997c91818694a6ce1",
    "name": "MerchDAO",
    "symbol": "MRCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14540/thumb/logo_256x256.png?1696514224"
  },
  {
    "chainId": 137,
    "address": "0x92868a5255c628da08f550a858a802f5351c5223",
    "name": "Cross Chain Bridge",
    "symbol": "BRIDGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20223/thumb/0x92868A5255C628dA08F550a858A802f5351C5223.png?1696519632"
  },
  {
    "chainId": 137,
    "address": "0x5d5530eb3147152fe78d5c4bfeede054c8d1442a",
    "name": "WalletNow",
    "symbol": "WNOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20149/thumb/walletnow.PNG?1696519563"
  },
  {
    "chainId": 137,
    "address": "0xc25351811983818c9fe6d8c580531819c8ade90f",
    "name": "IDLE",
    "symbol": "IDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13286/thumb/image.png?1696513058"
  },
  {
    "chainId": 137,
    "address": "0xba25b552c8a098afdf276324c32c71fe28e0ad40",
    "name": "MacaronSwap",
    "symbol": "MCRN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14633/thumb/macaron.png?1696514311"
  },
  {
    "chainId": 137,
    "address": "0xd1790c5435b9fb7c9444c749cefe53d40d751eac",
    "name": "Mimir",
    "symbol": "MIMIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19551/thumb/xaq5Xlzg_400x400.jpg?1696518984"
  },
  {
    "chainId": 137,
    "address": "0xc48f61a288a08f1b80c2edd74652e1276b6a168c",
    "name": "Geyser",
    "symbol": "GYSR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12995/thumb/logo_padded_shifted.png?1696512785"
  },
  {
    "chainId": 137,
    "address": "0xeaecc18198a475c921b24b8a6c1c1f0f5f3f7ea0",
    "name": "MetaGame",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13099/thumb/V8phEz8V.png?1696512885"
  },
  {
    "chainId": 137,
    "address": "0xb53ec4ace420a62cfb75afdeba600d284777cd65",
    "name": "Space Token",
    "symbol": "SPACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20676/thumb/jYw3kgsY_400x400.png?1696520076"
  },
  {
    "chainId": 137,
    "address": "0x1796ae0b0fa4862485106a0de9b654efe301d0b2",
    "name": "Polychain Monsters",
    "symbol": "PMON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14604/thumb/polkamon.png?1696514282"
  },
  {
    "chainId": 137,
    "address": "0xa69d14d6369e414a32a5c7e729b7afbafd285965",
    "name": "Global Coin Research",
    "symbol": "GCR",
    "decimals": 4,
    "logoURI": "https://assets.coingecko.com/coins/images/14815/thumb/gcr.jpeg?1696514483"
  },
  {
    "chainId": 137,
    "address": "0x48cbc913de09317df2365e6827df50da083701d5",
    "name": "FOUR",
    "symbol": "FOUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3434/thumb/FOUR.png?1696504127"
  },
  {
    "chainId": 137,
    "address": "0x61299774020da444af134c82fa83e3810b309991",
    "name": "Render",
    "symbol": "RNDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11636/thumb/rndr.png?1696511529"
  },
  {
    "chainId": 137,
    "address": "0x7f426f6dc648e50464a0392e60e1bb465a67e9cf",
    "name": "Auto",
    "symbol": "AUTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13751/thumb/autofarm_icon_200x200.png?1696513494"
  },
  {
    "chainId": 137,
    "address": "0xdb298285fe4c5410b05390ca80e8fbe9de1f259b",
    "name": "handle fi",
    "symbol": "FOREX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18533/thumb/handle.fiFOREXLogoDark200x200.png?1696518013"
  },
  {
    "chainId": 137,
    "address": "0x50bcbc40306230713239ae1bddd5eefeeaa273dc",
    "name": "Asia Coin",
    "symbol": "ASIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18589/thumb/Ou7mp_R1TQ5B9vsBiZ8oQnSv36M6hiA2hESxV_7YSw0.png?1696518065"
  },
  {
    "chainId": 137,
    "address": "0xd8ca34fd379d9ca3c6ee3b3905678320f5b45195",
    "name": "Governance OHM",
    "symbol": "GOHM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21129/thumb/token_wsOHM_logo.png?1696520508"
  },
  {
    "chainId": 137,
    "address": "0xef938b6da8576a896f6e0321ef80996f4890f9c4",
    "name": "Decentral Games",
    "symbol": "DG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21173/thumb/Decentral_Games_Logo-1.png?1696520549"
  },
  {
    "chainId": 137,
    "address": "0x65a05db8322701724c197af82c9cae41195b0aa8",
    "name": "ShapeShift FOX",
    "symbol": "FOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9988/thumb/FOX.png?1696510025"
  },
  {
    "chainId": 137,
    "address": "0xe51e88dd08499762b8e4eb3a9f3da9b8e79608c3",
    "name": "Sekuritance",
    "symbol": "SKRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15495/thumb/skrt.png?1696515139"
  },
  {
    "chainId": 137,
    "address": "0xc3c604f1943b8c619c5d65cd11a876e9c8edcf10",
    "name": "MetaGameHub DAO",
    "symbol": "MGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20625/thumb/mgh.PNG?1696520029"
  },
  {
    "chainId": 137,
    "address": "0xb382c1cfa622795a534e5bd56fac93d59bac8b0d",
    "name": "KIRO",
    "symbol": "KIRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12688/thumb/logo_kirobo-04.png?1696512491"
  },
  {
    "chainId": 137,
    "address": "0x5e430f88d1be82eb3ef92b6ff06125168fd5dcf2",
    "name": "MODA DAO",
    "symbol": "MODA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20870/thumb/ModaDAO__logomark-primary_3x.png?1696520264"
  },
  {
    "chainId": 137,
    "address": "0x51de72b17c7bd12e9e6d69eb506a669eb6b5249e",
    "name": "Waves Ducks",
    "symbol": "EGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17298/thumb/200x200_pixel.png?1696516851"
  },
  {
    "chainId": 137,
    "address": "0x2934b36ca9a4b31e633c5be670c8c8b28b6aa015",
    "name": "THX Network",
    "symbol": "THX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21323/thumb/logo-thx-resized-200-200.png?1696520690"
  },
  {
    "chainId": 137,
    "address": "0x64ee4f41a15d6c431ab6607d4e95462169d50f6c",
    "name": "Unvest",
    "symbol": "UNV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18119/thumb/UNV.jpg?1696517622"
  },
  {
    "chainId": 137,
    "address": "0x7dc47cfb674beb5827283f6140f635680a5ce992",
    "name": "BollyCoin",
    "symbol": "BOLLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21610/thumb/IMG-20211208-124337-701.jpg?1696520970"
  },
  {
    "chainId": 137,
    "address": "0x695fc8b80f344411f34bdbcb4e621aa69ada384b",
    "name": "Nitro League",
    "symbol": "NITRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21668/thumb/_X6vYBDM_400x400.jpg?1696521025"
  },
  {
    "chainId": 137,
    "address": "0x5314ba045a459f63906aa7c76d9f337dcb7d6995",
    "name": "Fodl Finance",
    "symbol": "FODL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19040/thumb/Fodl_Symbol_Gradient.png?1696518491"
  },
  {
    "chainId": 137,
    "address": "0x49690541e3f6e933a9aa3cffee6010a7bb5b72d7",
    "name": "Axia",
    "symbol": "AXIAV3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12906/thumb/axia_200x200.png?1696512694"
  },
  {
    "chainId": 137,
    "address": "0x72b9f88e822cf08b031c2206612b025a82fb303c",
    "name": "Day By Day",
    "symbol": "DBD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21691/thumb/DBD-icon_200x200_%281%29.png?1696521046"
  },
  {
    "chainId": 137,
    "address": "0x59b5654a17ac44f3068b3882f298881433bb07ef",
    "name": "CoinPoker",
    "symbol": "CHP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1808/thumb/coinpoker.jpg?1696502810"
  },
  {
    "chainId": 137,
    "address": "0xbbba073c31bf03b8acf7c28ef0738decf3695683",
    "name": "The Sandbox",
    "symbol": "SAND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12129/thumb/sandbox_logo.jpg?1696511971"
  },
  {
    "chainId": 137,
    "address": "0xc748b2a084f8efc47e086ccddd9b7e67aeb571bf",
    "name": "HUMAN Protocol",
    "symbol": "HMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16412/thumb/human_protocol.PNG?1696516009"
  },
  {
    "chainId": 137,
    "address": "0x1b815d120b3ef02039ee11dc2d33de7aa4a8c603",
    "name": "WOO",
    "symbol": "WOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12921/thumb/WOO_Logos_2023_Profile_Pic_WOO.png?1696512709"
  },
  {
    "chainId": 137,
    "address": "0xeb94a5e2c643403e29fa1d7197e7e0708b09ad84",
    "name": "OnX Finance",
    "symbol": "ONX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13445/thumb/onxlogo-1.png?1696513209"
  },
  {
    "chainId": 137,
    "address": "0x64ca1571d1476b7a21c5aaf9f1a750a193a103c0",
    "name": "Forj",
    "symbol": "BONDLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13322/thumb/FORJ_twitter_twitter-linked_in_profile_%281%29.png?1696513091"
  },
  {
    "chainId": 137,
    "address": "0xa3ed22eee92a3872709823a6970069e12a4540eb",
    "name": "Frontier",
    "symbol": "FRONT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12479/thumb/frontier_logo.png?1696512296"
  },
  {
    "chainId": 137,
    "address": "0xfd0cbddec28a93bb86b9db4a62258f5ef25fefde",
    "name": "BITT",
    "symbol": "BITT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13783/thumb/BITT_Logo_256pixels.png?1696513523"
  },
  {
    "chainId": 137,
    "address": "0x2bf9b864cdc97b08b6d79ad4663e71b8ab65c45c",
    "name": "FUSION",
    "symbol": "FSN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2515/thumb/Fusion_200x200.png?1696503329"
  },
  {
    "chainId": 137,
    "address": "0x72928d5436ff65e57f72d5566dcd3baedc649a88",
    "name": "humanDAO",
    "symbol": "HDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21138/thumb/Untitled-2.jpg?1696520516"
  },
  {
    "chainId": 137,
    "address": "0x42d61d766b85431666b39b89c43011f24451bff6",
    "name": "ParaSwap",
    "symbol": "PSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20403/thumb/ep7GqM19_400x400.jpg?1696519812"
  },
  {
    "chainId": 137,
    "address": "0xe8f157e041df3b28151b667364e9c90789da7923",
    "name": "Opium",
    "symbol": "OPIUM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13758/thumb/opium-token-black_%281%29.png?1696513500"
  },
  {
    "chainId": 137,
    "address": "0x84342e932797fc62814189f01f0fb05f52519708",
    "name": "Neighbourhoods",
    "symbol": "NHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22055/thumb/social_media_logo_black_bg.png?1696521399"
  },
  {
    "chainId": 137,
    "address": "0x123706cdd8e60324e610e9a2cc7012d0f45a5b8e",
    "name": "Quidd",
    "symbol": "QUIDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19725/thumb/quidd.png?1696519150"
  },
  {
    "chainId": 137,
    "address": "0xe46f5128b933e5a6f907fe73ece80059c222db0a",
    "name": "dotmoovs",
    "symbol": "MOOV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15817/thumb/dotmoovs-symbol-gradient.png?1696515436"
  },
  {
    "chainId": 137,
    "address": "0x5e0294af1732498c77f8db015a2d52a76298542b",
    "name": "Orion Money",
    "symbol": "ORION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18630/thumb/YtrqPIWc.png?1696518102"
  },
  {
    "chainId": 137,
    "address": "0x430ef9263e76dae63c84292c3409d61c598e9682",
    "name": "Vulcan Forged",
    "symbol": "PYR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14770/thumb/1617088937196.png?1696514439"
  },
  {
    "chainId": 137,
    "address": "0xb355f4f4cc84a9429a59d5c2b98d77016f7ec482",
    "name": "Bitcoin BR",
    "symbol": "BTCBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21205/thumb/btcbr.png?1696520580"
  },
  {
    "chainId": 137,
    "address": "0xf2ae0038696774d65e67892c9d301c5f2cbbda58",
    "name": "CargoX",
    "symbol": "CXO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2580/thumb/cargox.png?1696503384"
  },
  {
    "chainId": 137,
    "address": "0x9fb9a33956351cf4fa040f65a13b835a3c8764e3",
    "name": "Multichain",
    "symbol": "MULTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22087/thumb/1_Wyot-SDGZuxbjdkaOeT2-A.png?1696521430"
  },
  {
    "chainId": 137,
    "address": "0x42f6bdcfd82547e89f1069bf375aa60e6c6c063d",
    "name": "Civilization",
    "symbol": "CIV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17626/thumb/civ-200x200.png?1696517157"
  },
  {
    "chainId": 137,
    "address": "0xd192bac61522e0bc13543528173e69c44472c93a",
    "name": "VersoView",
    "symbol": "VVT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13380/thumb/HkfxEtWh_400x400.jpg?1696513143"
  },
  {
    "chainId": 137,
    "address": "0x09c5a4bca808bd1ba2b8e6b3aaf7442046b4ca5b",
    "name": "Vesper Finance",
    "symbol": "VSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13527/thumb/vesper_logo.jpg?1696513288"
  },
  {
    "chainId": 137,
    "address": "0x6863bd30c9e313b264657b107352ba246f8af8e0",
    "name": "BlackPool",
    "symbol": "BPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15887/thumb/uyO7dQzR_400x400.jpg?1696515502"
  },
  {
    "chainId": 137,
    "address": "0xa649325aa7c5093d12d6f98eb4378deae68ce23f",
    "name": "Wrapped BNB",
    "symbol": "WBNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12591/thumb/binance-coin-logo.png?1696512401"
  },
  {
    "chainId": 137,
    "address": "0x638df98ad8069a15569da5a6b01181804c47e34c",
    "name": "Dafi Protocol",
    "symbol": "DAFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14428/thumb/Dafi_Black_Icon.png?1696514118"
  },
  {
    "chainId": 137,
    "address": "0x1631244689ec1fecbdd22fb5916e920dfc9b8d30",
    "name": "Ovr",
    "symbol": "OVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13429/thumb/LOGO-OVER-ICON-200X200PX.png?1699396496"
  },
  {
    "chainId": 137,
    "address": "0xdb7cb471dd0b49b29cab4a1c14d070f27216a0ab",
    "name": "Bankless DAO",
    "symbol": "BANK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15227/thumb/j4WEJrwU.png?1696514882"
  },
  {
    "chainId": 137,
    "address": "0xdda7b23d2d72746663e7939743f929a3d85fc975",
    "name": "AdEx",
    "symbol": "ADX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/847/thumb/adex.jpeg?1696501984"
  },
  {
    "chainId": 137,
    "address": "0xd1a5f2a049343fc4d5f8d478f734eba51b22375e",
    "name": "KeyFi",
    "symbol": "KEYFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15098/thumb/keyfi_logo.jpg?1696514756"
  },
  {
    "chainId": 137,
    "address": "0x9c32185b81766a051e08de671207b34466dd1021",
    "name": "BTC Proxy",
    "symbol": "BTCPX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/22630/thumb/MB1aYO7T_400x400.jpg?1696521945"
  },
  {
    "chainId": 137,
    "address": "0xe4bf2864ebec7b7fdf6eeca9bacae7cdfdaffe78",
    "name": "DODO",
    "symbol": "DODO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12651/thumb/dodo_logo.png?1696512458"
  },
  {
    "chainId": 137,
    "address": "0xda5949544aaf6124d06f398bfde4c86cc33b0ee7",
    "name": "CyberFM",
    "symbol": "CYFM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5476/thumb/cyberfm.png?1696505952"
  },
  {
    "chainId": 137,
    "address": "0xe6469ba6d2fd6130788e0ea9c0a0515900563b59",
    "name": "TerraUSD  Wormhole ",
    "symbol": "UST",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21150/thumb/UST_wh_small.png?1696520528"
  },
  {
    "chainId": 137,
    "address": "0xba3cb8329d442e6f9eb70fafe1e214251df3d275",
    "name": "Swash",
    "symbol": "SWASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18774/thumb/Swash_icon_GP.png?1712869197"
  },
  {
    "chainId": 137,
    "address": "0x34667ed7c36cbbbf2d5d5c5c8d6eb76a094edb9f",
    "name": "GenomesDAO GENE",
    "symbol": "GENE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20807/thumb/1637683704200x200.png?1696520200"
  },
  {
    "chainId": 137,
    "address": "0xd93f7e271cb87c23aaa73edc008a79646d1f9912",
    "name": "SOL  Wormhole ",
    "symbol": "SOL",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/22876/thumb/SOL_wh_small.png?1696522175"
  },
  {
    "chainId": 137,
    "address": "0x3553f861dec0257bada9f8ed268bf0d74e45e89c",
    "name": "Bridged Tether  Wormhole ",
    "symbol": "USDTSO",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/22881/thumb/USDTso_wh_small.png?1696522179"
  },
  {
    "chainId": 137,
    "address": "0xecdcb5b88f8e3c15f95c720c51c71c9e2080525d",
    "name": "Binance Coin  Wormhole ",
    "symbol": "BNB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22884/thumb/BNB_wh_small.png?1696522182"
  },
  {
    "chainId": 137,
    "address": "0xe631dabef60c37a37d70d3b4f812871df663226f",
    "name": "Swarm Markets",
    "symbol": "SMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17488/thumb/swarm-SMT-token-symbol_200x200.png?1696517029"
  },
  {
    "chainId": 137,
    "address": "0x7bb11e7f8b10e9e571e5d8eace04735fdfb2358a",
    "name": "Avalanche  Wormhole ",
    "symbol": "AVAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22943/thumb/AVAX_wh_small.png?1696522240"
  },
  {
    "chainId": 137,
    "address": "0x11cd37bb86f65419713f30673a480ea33c826872",
    "name": "Ethereum  Wormhole ",
    "symbol": "ETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22990/thumb/ETH_wh_small.png?1696522286"
  },
  {
    "chainId": 137,
    "address": "0x9417669fbf23357d2774e9d421307bd5ea1006d2",
    "name": "Bridged Tether  Wormhole Ethereum ",
    "symbol": "USDTET",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/23025/thumb/USDTet_wh_small.png?1696522320"
  },
  {
    "chainId": 137,
    "address": "0xb41c43fabd22a6c6ea135e975769e9051f9ee8ad",
    "name": "Ebox",
    "symbol": "EBOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14528/thumb/ebox.png?1696514213"
  },
  {
    "chainId": 137,
    "address": "0x5d49c278340655b56609fdf8976eb0612af3a0c3",
    "name": "Wrapped BTC  Wormhole ",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/23004/thumb/WBTC_wh_small.png?1696522299"
  },
  {
    "chainId": 137,
    "address": "0x4318cb63a2b8edf2de971e2f17f77097e499459d",
    "name": "Bridged USD Coin  Wormhole Ethereum ",
    "symbol": "USDCET",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/23019/thumb/USDCet_wh_small.png?1696522314"
  },
  {
    "chainId": 137,
    "address": "0xe0b52e49357fd4daf2c15e02058dce6bc0057db4",
    "name": "EURA",
    "symbol": "EURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19479/thumb/agEUR-4.png?1710726218"
  },
  {
    "chainId": 137,
    "address": "0xd281af594cbb99e8469f3591d57a0c72eb725bdb",
    "name": "Marvelous NFTs",
    "symbol": "MNFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22613/thumb/mnft.png?1696521929"
  },
  {
    "chainId": 137,
    "address": "0x3a9a81d576d83ff21f26f325066054540720fc34",
    "name": "Streamr",
    "symbol": "DATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17869/thumb/DATA_new_symbol_3x.png?1696517392"
  },
  {
    "chainId": 137,
    "address": "0x4ff0b68abc2b9e4e1401e9b691dba7d66b264ac8",
    "name": "Riot Racers",
    "symbol": "RIOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19238/thumb/jyxvIbmJ_400x400.png?1696518684"
  },
  {
    "chainId": 137,
    "address": "0x4af5ff1a60a6ef6c7c8f9c4e304cd9051fca3ec0",
    "name": "Rigel Protocol",
    "symbol": "RGP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15837/thumb/A_qRYvB2_400x400.png?1696515455"
  },
  {
    "chainId": 137,
    "address": "0x9ca6a77c8b38159fd2da9bd25bc3e259c33f5e39",
    "name": "SporkDAO",
    "symbol": "SPORK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23358/thumb/sporkdao.PNG?1696522573"
  },
  {
    "chainId": 137,
    "address": "0xe58e8391ba17731c5671f9c6e00e420608dca10e",
    "name": "GNFT",
    "symbol": "GNFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23532/thumb/gnft_200x200.png?1696522740"
  },
  {
    "chainId": 137,
    "address": "0x43df9c0a1156c96cea98737b511ac89d0e2a1f46",
    "name": "CVI",
    "symbol": "GOVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13875/thumb/GOVI.png?1696513619"
  },
  {
    "chainId": 137,
    "address": "0x8839e639f210b80ffea73aedf51baed8dac04499",
    "name": "DecentraWeb",
    "symbol": "DWEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18971/thumb/dweb-logo-transparent.png?1696518425"
  },
  {
    "chainId": 137,
    "address": "0x2ebd50ae084e71eed419cb6c620b3bbd3927bf7e",
    "name": "Moon Rabbit",
    "symbol": "AAA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17310/thumb/logo_moon_no_inscriptions-01.png?1696516863"
  },
  {
    "chainId": 137,
    "address": "0x2bc07124d8dac638e290f401046ad584546bc47b",
    "name": "Tower",
    "symbol": "TOWER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14134/thumb/tower-circular-1000.png?1696513854"
  },
  {
    "chainId": 137,
    "address": "0x55555555a687343c6ce28c8e1f6641dc71659fad",
    "name": "XY Finance",
    "symbol": "XY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21541/thumb/xy.png?1696520900"
  },
  {
    "chainId": 137,
    "address": "0x2f1b1662a895c6ba01a99dcaf56778e7d77e5609",
    "name": "SpiceUSD",
    "symbol": "USDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25697/thumb/USDS.png?1696524824"
  },
  {
    "chainId": 137,
    "address": "0x8d520c8e66091cfd6743fe37fbe3a09505616c4b",
    "name": "Cosplay Token",
    "symbol": "COT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21294/thumb/brave_ZxsjzUc8xn.png?1696520664"
  },
  {
    "chainId": 137,
    "address": "0x4a7b9a4589a88a06ca29f99556db08234078d727",
    "name": "NFTmall",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16217/thumb/Icon-1000x1000.png?1696515817"
  },
  {
    "chainId": 137,
    "address": "0x14af1f2f02dccb1e43402339099a05a5e363b83c",
    "name": "Kromatika",
    "symbol": "KROM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20541/thumb/KROM_Transparent.png?1696519948"
  },
  {
    "chainId": 137,
    "address": "0xee9801669c6138e84bd50deb500827b776777d28",
    "name": "O3 Swap",
    "symbol": "O3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15460/thumb/o3.png?1696515107"
  },
  {
    "chainId": 137,
    "address": "0xc46a37fbbe433ef24bc7b9388c8728ddcf3ca87c",
    "name": "Mainstream For The Underground",
    "symbol": "MFTU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5519/thumb/Mainstream_for_the_underground.png?1696505989"
  },
  {
    "chainId": 137,
    "address": "0xa5ff48e326958e0ce6fdf9518de561f2b5f57da3",
    "name": "Lokr",
    "symbol": "LKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14692/thumb/lokr.png?1696514364"
  },
  {
    "chainId": 137,
    "address": "0x1581929770be3275a82068c1135b6dd59c5334ed",
    "name": "Alium Finance",
    "symbol": "ALM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15621/thumb/alium.png?1696515255"
  },
  {
    "chainId": 137,
    "address": "0xd3b71117e6c1558c1553305b44988cd944e97300",
    "name": "Yel Finance",
    "symbol": "YEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17429/thumb/Logo200.png?1696516976"
  },
  {
    "chainId": 137,
    "address": "0xa4ce4a467e51aefec683a649c3f14427f104667f",
    "name": "Onston",
    "symbol": "ONSTON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20669/thumb/onston.PNG?1696520070"
  },
  {
    "chainId": 137,
    "address": "0xcf403036bc139d30080d2cf0f5b48066f98191bb",
    "name": "Stobox",
    "symbol": "STBU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12637/thumb/123456826.png?1696512445"
  },
  {
    "chainId": 137,
    "address": "0x6b021b3f68491974be6d4009fee61a4e3c708fd6",
    "name": "Fuse",
    "symbol": "FUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10347/thumb/fuse.png?1696510348"
  },
  {
    "chainId": 137,
    "address": "0x538d47d142f6993038a667e9d6210d3735749b36",
    "name": "Burp",
    "symbol": "BURP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17317/thumb/Big_Town_Chef_-__BURP_-_Avatar_3x.png?1696516871"
  },
  {
    "chainId": 137,
    "address": "0xcc081220542a60a8ea7963c4f53d522b503272c1",
    "name": "NFTY",
    "symbol": "NFTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18741/thumb/coingecko.png?1696518206"
  },
  {
    "chainId": 137,
    "address": "0xe580074a10360404af3abfe2d524d5806d993ea3",
    "name": "PayBolt",
    "symbol": "PAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24175/thumb/logo.png?1696523363"
  },
  {
    "chainId": 137,
    "address": "0xf6f85b3f9fd581c2ee717c404f7684486f057f95",
    "name": "Nord Finance",
    "symbol": "NORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13630/thumb/nord.jpg?1696513378"
  },
  {
    "chainId": 137,
    "address": "0xbc91347e80886453f3f8bbd6d7ac07c122d87735",
    "name": "Banana",
    "symbol": "BANANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17521/thumb/banana-token-cg.png?1696517059"
  },
  {
    "chainId": 137,
    "address": "0x3c205c8b3e02421da82064646788c82f7bd753b9",
    "name": "PureFi",
    "symbol": "UFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17341/thumb/purefi.PNG?1696516893"
  },
  {
    "chainId": 137,
    "address": "0x2f6f07cdcf3588944bf4c42ac74ff24bf56e7590",
    "name": "Stargate Finance",
    "symbol": "STG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24413/thumb/STG_LOGO.png?1696523595"
  },
  {
    "chainId": 137,
    "address": "0x4fafad147c8cd0e52f83830484d164e960bdc6c3",
    "name": "Qawalla",
    "symbol": "QWLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15242/thumb/qwla.png?1696514896"
  },
  {
    "chainId": 137,
    "address": "0x91c5a5488c0decde1eacd8a4f10e0942fb925067",
    "name": "Auditchain",
    "symbol": "AUDT",
    "decimals": 17,
    "logoURI": "https://assets.coingecko.com/coins/images/6398/thumb/auditchain.png?1696506764"
  },
  {
    "chainId": 137,
    "address": "0x3a58a54c066fdc0f2d55fc9c89f0415c92ebf3c4",
    "name": "Lido Staked Matic",
    "symbol": "STMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24185/thumb/stMATIC.png?1696523373"
  },
  {
    "chainId": 137,
    "address": "0xa1388e73c51b37383b1ab9dce8317ef5a0349cc5",
    "name": "Shibaverse",
    "symbol": "VERSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18407/thumb/logo_200.png?1696517897"
  },
  {
    "chainId": 137,
    "address": "0x91e7e32c710661c44ae44d10aa86135d91c3ed65",
    "name": "Peercoin",
    "symbol": "PPC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/4/thumb/peercoin-icon-green-transparent_6x.png?1696501402"
  },
  {
    "chainId": 137,
    "address": "0x4c9f66b2806538cf00ef596e09fb05bcb0d17dc8",
    "name": "Minato",
    "symbol": "MNTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24622/thumb/MNTO_200x200.png?1696523794"
  },
  {
    "chainId": 137,
    "address": "0x1a13f4ca1d028320a707d99520abfefca3998b7f",
    "name": "Aave v2 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14318/thumb/aUSDC.e260d492.png?1696514006"
  },
  {
    "chainId": 137,
    "address": "0x9a06db14d639796b25a6cec6a1bf614fd98815ec",
    "name": "Panther Protocol",
    "symbol": "ZKP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18611/thumb/panther.jpg?1696518084"
  },
  {
    "chainId": 137,
    "address": "0xe613a914bbb433855378183c3ab13003285da40a",
    "name": "Bit2Me",
    "symbol": "B2M",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19848/thumb/b2m-circle-solid-default.png?1696519271"
  },
  {
    "chainId": 137,
    "address": "0xacd4e2d936be9b16c01848a3742a34b3d5a5bdfa",
    "name": "Mechanium",
    "symbol": "MECHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24374/thumb/w4K4OOMo_400x400.jpg?1696523558"
  },
  {
    "chainId": 137,
    "address": "0xc3c7d422809852031b44ab29eec9f1eff2a58756",
    "name": "Lido DAO",
    "symbol": "LDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13573/thumb/Lido_DAO.png?1696513326"
  },
  {
    "chainId": 137,
    "address": "0x0a307bd521701f9d70fb29bfa9e2e36dc998dadb",
    "name": "CoinWealth",
    "symbol": "CNW",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/23769/thumb/cw_logo-4955f59a5c8079f246fa07ac71b2541870ca7d906ca1d9c26d74a3870fafef2f_%281%29.png?1696522970"
  },
  {
    "chainId": 137,
    "address": "0xc17c30e98541188614df99239cabd40280810ca3",
    "name": "EverRise",
    "symbol": "RISE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16367/thumb/Logo_EverRise_Icon_logo.png?1696515966"
  },
  {
    "chainId": 137,
    "address": "0xeeeeeb57642040be42185f49c52f7e9b38f8eeee",
    "name": "Elk Finance",
    "symbol": "ELK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17813/thumb/elk.png?1696517333"
  },
  {
    "chainId": 137,
    "address": "0xe6b9d092223f39013656702a40dbe6b7decc5746",
    "name": "RealFevr",
    "symbol": "FEVR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17136/thumb/Fevr-Token.png?1696516695"
  },
  {
    "chainId": 137,
    "address": "0x11a1779ae6b02bb8e7ff847919bca3e55bcbb3d5",
    "name": "Paper",
    "symbol": "PAPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23510/thumb/v3PAPERLogo-01.png?1696522719"
  },
  {
    "chainId": 137,
    "address": "0xc3f56d567e7663e8932e65d85ae4be7eb5575ca7",
    "name": "Faith Tribe",
    "symbol": "FTRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23939/thumb/ym1Hf4x2_400x400.jpg?1696523135"
  },
  {
    "chainId": 137,
    "address": "0xf50d05a1402d0adafa880d36050736f9f6ee7dee",
    "name": "Instadapp",
    "symbol": "INST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14688/thumb/30hFM0-n_400x400.jpg?1696514361"
  },
  {
    "chainId": 137,
    "address": "0xafcdd4f666c84fed1d8bd825aa762e3714f652c9",
    "name": "Vita Inu",
    "symbol": "VINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20594/thumb/vita-inu-head-wallet-icon-transparent.png?1696520000"
  },
  {
    "chainId": 137,
    "address": "0xb25e20de2f2ebb4cffd4d16a55c7b395e8a94762",
    "name": "Request",
    "symbol": "REQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1031/thumb/Request_icon_green.png?1696502140"
  },
  {
    "chainId": 137,
    "address": "0x61aee582896064ecd5d85d66a32ddeb5574a699d",
    "name": "Hyve",
    "symbol": "HYVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13072/thumb/bAe1G-lD_400x400.png?1696512861"
  },
  {
    "chainId": 137,
    "address": "0x692c44990e4f408ba0917f5c78a83160c1557237",
    "name": "Thales",
    "symbol": "THALES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18388/thumb/CLVZJN_C_400x400.png?1696517879"
  },
  {
    "chainId": 137,
    "address": "0x4fb71290ac171e1d144f7221d882becac7196eb5",
    "name": "BiLira",
    "symbol": "TRYB",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/10119/thumb/JBs9jiXO_400x400.jpg?1696510144"
  },
  {
    "chainId": 137,
    "address": "0xfa68fb4628dff1028cfec22b4162fccd0d45efb6",
    "name": "Stader MaticX",
    "symbol": "MATICX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25383/thumb/maticx.png?1696524516"
  },
  {
    "chainId": 137,
    "address": "0x0a3bb08b3a15a19b4de82f8acfc862606fb69a2d",
    "name": "iZUMi Bond USD",
    "symbol": "IUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25388/thumb/iusd-logo-symbol-10k%E5%A4%A7%E5%B0%8F.png?1696524521"
  },
  {
    "chainId": 137,
    "address": "0x388d819724dd6d71760a38f00dc01d310d879771",
    "name": "JustMoney",
    "symbol": "JM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/25450/thumb/jm.png?1696524581"
  },
  {
    "chainId": 137,
    "address": "0x9767203e89dcd34851240b3919d4900d3e5069f1",
    "name": "A4 Finance",
    "symbol": "A4",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21992/thumb/ba384ad07217a4be75cb85314f5760f7.jpg?1696521340"
  },
  {
    "chainId": 137,
    "address": "0xeeda694439c6fb56cbaa011cc849650b7273285b",
    "name": "Bankless BED Index",
    "symbol": "BED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17175/thumb/BED_Logo_-_No_border.png?1696516734"
  },
  {
    "chainId": 137,
    "address": "0xb5b8381b67248f832c7961bd265f021cd8d291a4",
    "name": "Zelwin",
    "symbol": "ZLW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11547/thumb/5614.png?1696511447"
  },
  {
    "chainId": 137,
    "address": "0xea73dcf6f49f8d6ad5a129aaede776d78d418cfd",
    "name": "Vabble",
    "symbol": "VAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17508/thumb/vabble.png?1705376555"
  },
  {
    "chainId": 137,
    "address": "0x431d5dff03120afa4bdf332c61a6e1766ef37bdb",
    "name": "JPY Coin",
    "symbol": "JPYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25971/thumb/2023jpyc.png?1696525049"
  },
  {
    "chainId": 137,
    "address": "0x221743dc9e954be4f86844649bf19b43d6f8366d",
    "name": "Obortech",
    "symbol": "OBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14929/thumb/OBORTECH_200.png?1696514590"
  },
  {
    "chainId": 137,
    "address": "0x6ae7dfc73e0dde2aa99ac063dcf7e8a63265108c",
    "name": "JPY Coin v1",
    "symbol": "JPYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17277/thumb/WoZ8rruL_400x400.png?1696516831"
  },
  {
    "chainId": 137,
    "address": "0x9e6b19874e97fe8e8cad77f2c0ab5e7a693e5dbf",
    "name": "StrongHands Finance",
    "symbol": "ISHND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20158/thumb/ISHND_512x512px.png?1696519571"
  },
  {
    "chainId": 137,
    "address": "0x534f39c5f4df9cb13e16b24ca07c7c8c0e2eadb7",
    "name": "Safe Haven",
    "symbol": "SHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2584/thumb/safehaven.png?1696503388"
  },
  {
    "chainId": 137,
    "address": "0x564906ec1df8399f00e4ad32c0ecac0404a27a1c",
    "name": "Ambire Wallet",
    "symbol": "WALLET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23154/thumb/wallet.PNG?1696522445"
  },
  {
    "chainId": 137,
    "address": "0xfafa220145dfa5c3ec85b6fa8a75aee2451cde5e",
    "name": "Roobee",
    "symbol": "ROOBEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8791/thumb/Group_11.png?1696508946"
  },
  {
    "chainId": 137,
    "address": "0x111111517e4929d3dcbdfa7cce55d30d4b6bc4d6",
    "name": "ICHI",
    "symbol": "ICHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13119/thumb/ICHI_%28Round%29.jpg?1696512907"
  },
  {
    "chainId": 137,
    "address": "0x013f9c3fac3e2759d7e90aca4f9540f75194a0d7",
    "name": "Neutrino Index Token",
    "symbol": "XTN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10117/thumb/XTN.png?1712339473"
  },
  {
    "chainId": 137,
    "address": "0x3d1d2afd191b165d140e3e8329e634665ffb0e5e",
    "name": "Deri Protocol",
    "symbol": "DERI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13931/thumb/200vs200.jpg?1696513670"
  },
  {
    "chainId": 137,
    "address": "0x15b7c0c907e4c6b9adaaaabc300c08991d6cea05",
    "name": "Gelato",
    "symbol": "GEL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15026/thumb/Gelato_Icon_Logo_1024x1024.png?1696514687"
  },
  {
    "chainId": 137,
    "address": "0x524d524b4c9366be706d3a90dcf70076ca037ae3",
    "name": "RMRK",
    "symbol": "RMRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18656/thumb/download_%281%29_%281%29.png?1696518126"
  },
  {
    "chainId": 137,
    "address": "0x03c2f6808502ffd4ab2787ad1a98ec13dbd5718b",
    "name": "ClinTex CTi",
    "symbol": "CTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13266/thumb/CTI.png?1696513039"
  },
  {
    "chainId": 137,
    "address": "0xc88640b734fea3b35c132fe757aeb5ca6c8cdc66",
    "name": "Nexum",
    "symbol": "NEXM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/23472/thumb/200_-_200_coinmarketcap.png?1696522683"
  },
  {
    "chainId": 137,
    "address": "0xadbe0eac80f955363f4ff47b0f70189093908c04",
    "name": "MetalSwap",
    "symbol": "XMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22075/thumb/Logo_COIN_-_Gradiente.png?1696521419"
  },
  {
    "chainId": 137,
    "address": "0xc5b57e9a1e7914fda753a88f24e5703e617ee50c",
    "name": "Popcorn",
    "symbol": "POP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21438/thumb/pop-1_200_x_200.png?1696520801"
  },
  {
    "chainId": 137,
    "address": "0xbcd2c5c78000504efbc1ce6489dfcac71835406a",
    "name": "Arch Ethereum Web3",
    "symbol": "WEB3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26562/thumb/WEB3-TOKEN.png?1696525635"
  },
  {
    "chainId": 137,
    "address": "0x1d734a02ef1e1f5886e66b0673b71af5b53ffa94",
    "name": "Stader",
    "symbol": "SD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20658/thumb/SD_Token_Logo.png?1696520060"
  },
  {
    "chainId": 137,
    "address": "0x750e4c4984a9e0f12978ea6742bc1c5d248f40ed",
    "name": "Axelar Bridged USDC",
    "symbol": "AXLUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26476/thumb/uausdc_D_3x.png?1696525548"
  },
  {
    "chainId": 137,
    "address": "0xfe457497a2a71bce1eb93ea9e6a685057dd93dee",
    "name": "BNSD Finance",
    "symbol": "BNSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12368/thumb/bnsd.png?1696512193"
  },
  {
    "chainId": 137,
    "address": "0xd7ecf95cf7ef5256990beaf4ac895cd9e64cb947",
    "name": "pTokens BTC",
    "symbol": "PBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25861/thumb/wMTpRljt_400x400.png?1696524945"
  },
  {
    "chainId": 137,
    "address": "0x411be1e071675df40fe1c08ca760bb7aa707cedf",
    "name": "Gamerse",
    "symbol": "LFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19582/thumb/gamerse.PNG?1696519013"
  },
  {
    "chainId": 137,
    "address": "0x6cd6cb131764c704ba9167c29930fbdc38901ab7",
    "name": "xWIN Finance",
    "symbol": "XWIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15400/thumb/200x200_%28transparent_background%29.png?1696515046"
  },
  {
    "chainId": 137,
    "address": "0xbf9f916bbda29a7f990f5f55c7607d94d7c3a60b",
    "name": "DEFY",
    "symbol": "DEFY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26877/thumb/8ybr83fb0ca45cm1yvrcaclwbvcp.jpeg?1696525936"
  },
  {
    "chainId": 137,
    "address": "0x011734f6ed20e8d011d85cf7894814b897420acf",
    "name": "Arable Protocol",
    "symbol": "ACRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23659/thumb/acre_token-02.png?1696522862"
  },
  {
    "chainId": 137,
    "address": "0xb7b31a6bc18e48888545ce79e83e06003be70930",
    "name": "ApeCoin",
    "symbol": "APE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24383/thumb/apecoin.jpg?1696523566"
  },
  {
    "chainId": 137,
    "address": "0xdc3326e71d45186f113a2f448984ca0e8d201995",
    "name": "XSGD",
    "symbol": "XSGD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/12832/thumb/StraitsX_Singapore_Dollar_%28XSGD%29_Token_Logo.png?1696512623"
  },
  {
    "chainId": 137,
    "address": "0xa91fe5a535967f52d3abebdffb3b306d964ace13",
    "name": "Sandclock",
    "symbol": "QUARTZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19368/thumb/sandclock.jpg?1696518808"
  },
  {
    "chainId": 137,
    "address": "0x77d97db5615dfe8a2d16b38eaa3f8f34524a0a74",
    "name": "Lunafi",
    "symbol": "LFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24594/thumb/lfi.png?1696523768"
  },
  {
    "chainId": 137,
    "address": "0x0c9c7712c83b3c70e7c5e11100d33d9401bdf9dd",
    "name": "Wombat",
    "symbol": "WOMBAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26430/thumb/Project_Page_Icon.png?1696525504"
  },
  {
    "chainId": 137,
    "address": "0x189586b5f6317538ae50c20a976597da38984a24",
    "name": "ChainPort",
    "symbol": "PORTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24490/thumb/VE-tUL-q_400x400.png?1696523670"
  },
  {
    "chainId": 137,
    "address": "0xa3c53b53fe43083aa31c6f32ffe90c4d91b44391",
    "name": "Sportzchain",
    "symbol": "SPN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24915/thumb/Sportzchain_logo_195_195.png?1696524072"
  },
  {
    "chainId": 137,
    "address": "0x2c826035c1c36986117a0e949bd6ad4bab54afe2",
    "name": "XIDR",
    "symbol": "XIDR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/21126/thumb/XIDR_Logo_256_X_256.png?1696520505"
  },
  {
    "chainId": 137,
    "address": "0x6e4e624106cb12e168e6533f8ec7c82263358940",
    "name": "Axelar",
    "symbol": "AXL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27277/thumb/V-65_xQ1_400x400.jpeg?1696526329"
  },
  {
    "chainId": 137,
    "address": "0x1a763170b96f23f15576d0fa0b2619d1254c437d",
    "name": "AurusX",
    "symbol": "AX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27871/thumb/AurusX_token_2D.jpg?1696526889"
  },
  {
    "chainId": 137,
    "address": "0xb35fcbcf1fd489fce02ee146599e893fdcdc60e6",
    "name": "DeRace",
    "symbol": "DERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17438/thumb/DERC_logo_coingecko.png?1696516984"
  },
  {
    "chainId": 137,
    "address": "0x23001f892c0c82b79303edc9b9033cd190bb21c7",
    "name": "Liquity USD",
    "symbol": "LUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14666/thumb/Group_3.png?1696514341"
  },
  {
    "chainId": 137,
    "address": "0x13f76eab3ec634c5479cbb0fb705e867b3e469de",
    "name": "TradeMaster ninja",
    "symbol": "TRDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36190/thumb/Transparent_200x200.png?1710761061"
  },
  {
    "chainId": 137,
    "address": "0x1db06f39c14d813d7b1ccb275a93f5b052de1cac",
    "name": "Xave",
    "symbol": "XAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28299/thumb/Xave_logo_gradient.png?1696527298"
  },
  {
    "chainId": 137,
    "address": "0x26c80854c36ff62bba7414a358c8c23bbb8dec39",
    "name": "CheckDot",
    "symbol": "CDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20370/thumb/token-200x200_%281%29.png?1696519781"
  },
  {
    "chainId": 137,
    "address": "0xd7b675cd5c84a13d1d0f84509345530f6421b57c",
    "name": "Corridor Finance",
    "symbol": "OOOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36057/thumb/corridor-logo-without-text-8bit.jpg?1710404509"
  },
  {
    "chainId": 137,
    "address": "0xd566c529b33ecf15170f600d4b1ab12468c8efc6",
    "name": "UERII",
    "symbol": "UERII",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29388/thumb/Avatar512.png?1696528334"
  },
  {
    "chainId": 137,
    "address": "0x276c9cbaa4bdf57d7109a41e67bd09699536fa3d",
    "name": "Somnium Space CUBEs",
    "symbol": "CUBE",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/10687/thumb/CUBE_icon.png?1696510656"
  },
  {
    "chainId": 137,
    "address": "0x7844f79fc841e4f92d974c417031c76f8578c2d5",
    "name": "OPEN Ticketing Ecosystem",
    "symbol": "OPN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36612/thumb/TOKEN.png?1711975511"
  },
  {
    "chainId": 137,
    "address": "0xbbbbbbbbb7949dcc7d1539c91b81a5bf09e37bdb",
    "name": "crow with knife",
    "symbol": "CAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36067/thumb/200px.png?1710405601"
  },
  {
    "chainId": 8453,
    "address": "0xca7465e2c4e6ffa198e11ddfa2e6c6b3522bf799",
    "name": "Mark Jeffrey  Friend tech ",
    "symbol": "MARK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31372/thumb/mark.jpeg?1696530189"
  },
  {
    "chainId": 8453,
    "address": "0xdc46c1e93b71ff9209a0f8076a9951569dc35855",
    "name": "MYSTCL",
    "symbol": "MYST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35142/thumb/eDBJEu7O_400x400.jpeg?1707464572"
  },
  {
    "chainId": 8453,
    "address": "0x1f23b787053802108fed5b67cf703f0778aebad8",
    "name": "Bass Exchange",
    "symbol": "BASS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31982/thumb/200.png?1696530785"
  },
  {
    "chainId": 8453,
    "address": "0xed1978d01d4a8a9d6a43ac79403d5b8dfbed739b",
    "name": "Sandwich Cat",
    "symbol": "SACA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36876/thumb/200x200_%281%29.png?1712646447"
  },
  {
    "chainId": 8453,
    "address": "0xac1bd2486aaf3b5c0fc3fd868558b082a531b2b4",
    "name": "Toshi",
    "symbol": "TOSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31126/thumb/2023-08-11_13.21.24.png?1697025145"
  },
  {
    "chainId": 8453,
    "address": "0x06c4c7f8011fbfb6a9654ee4142532f9bb3c52b8",
    "name": "RatsBase",
    "symbol": "RATS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33860/thumb/IMG_7534.PNG?1703128584"
  },
  {
    "chainId": 8453,
    "address": "0x435f49a87df832a582a1657421f083a0529edf7c",
    "name": "MawCAT",
    "symbol": "MAW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36894/thumb/200.png?1712674506"
  },
  {
    "chainId": 8453,
    "address": "0xadba4694f5deaf3251d06dec38c86946f8952efb",
    "name": "DIP Exchange",
    "symbol": "DIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31497/thumb/token2.png?1696530308"
  },
  {
    "chainId": 8453,
    "address": "0xa19328fb05ce6fd204d16c2a2a98f7cf434c12f4",
    "name": "L2VE INU",
    "symbol": "L2VE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36329/thumb/photo_2024-03-21_09.46.29.jpeg?1711098200"
  },
  {
    "chainId": 8453,
    "address": "0x7e1a118070bcea660a96a1063a39e0fa93080220",
    "name": "EZ Pepe",
    "symbol": "EZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36936/thumb/EZ_200X200.png?1712811096"
  },
  {
    "chainId": 8453,
    "address": "0x73e58c6f5e7b9def4bdda8f02f59f9ef2a342cca",
    "name": "GrabPenny",
    "symbol": "GP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36674/thumb/Group_10469.png?1712062452"
  },
  {
    "chainId": 8453,
    "address": "0xe8e55a847bb446d967ef92f4580162fb8f2d3f38",
    "name": "Broge",
    "symbol": "BROGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36297/thumb/1000004999.jpg?1711048058"
  },
  {
    "chainId": 8453,
    "address": "0x8e16d46cb2da01cdd49601ec73d7b0344969ae33",
    "name": "Coin on Base",
    "symbol": "COIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36089/thumb/download_%284%29.png?1710466413"
  },
  {
    "chainId": 8453,
    "address": "0x1ccb4b14a11e0f2994a7ecbbd4cc69632f4c7c76",
    "name": "Cute Cat Token",
    "symbol": "CCC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36930/thumb/1000017879.jpg?1712803401"
  },
  {
    "chainId": 8453,
    "address": "0x7aef64e5997a81f695e0bc951ca874afccd4d1e1",
    "name": "It s so over",
    "symbol": "OVER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36635/thumb/over-icon.png?1712254709"
  },
  {
    "chainId": 8453,
    "address": "0xc19669a405067927865b40ea045a2baabbbe57f5",
    "name": "Preon Finance STAR",
    "symbol": "STAR",
    "decimals": 18
  },
  {
    "chainId": 8453,
    "address": "0xecaf81eb42cd30014eb44130b89bcd6d4ad98b92",
    "name": "Based Chad",
    "symbol": "CHAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36285/thumb/logo_200_200_square.jpg?1712167823"
  },
  {
    "chainId": 8453,
    "address": "0xd11a584de5fa50a4ee560c48ab44dbb31823d9bc",
    "name": "Buttman",
    "symbol": "BUTT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36326/thumb/kqDFdKtJ_400x400.jpeg?1711097504"
  },
  {
    "chainId": 8453,
    "address": "0xd6bbac6486d93cb576cacf085d307c8ee69bbfe9",
    "name": "SURGE",
    "symbol": "SURGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36405/thumb/1000191048.jpg?1711369900"
  },
  {
    "chainId": 8453,
    "address": "0x2f3b1a07e3efb1fcc64bd09b86bd0fa885d93552",
    "name": "Meridian MST",
    "symbol": "MST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34343/thumb/MST.png?1704697252"
  },
  {
    "chainId": 8453,
    "address": "0x4a36c161aa762f7ecf7517440db2cb53ff1cef32",
    "name": "Pace Bot",
    "symbol": "PACE",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/36036/thumb/Untitled-1-01.png?1710397010"
  },
  {
    "chainId": 8453,
    "address": "0xbf5495efe5db9ce00f80364c8b423567e58d2110",
    "name": "Renzo Restaked ETH",
    "symbol": "EZETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34753/thumb/eth_renzo_logo_%281%29.png?1705956747"
  },
  {
    "chainId": 8453,
    "address": "0x469fda1fb46fcb4befc0d8b994b516bd28c87003",
    "name": "Dino Poker",
    "symbol": "RAWR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35695/thumb/twitter_logo.png?1709546744"
  },
  {
    "chainId": 8453,
    "address": "0x4e98b3917310b0e1f0d53c0619f87fe48deb804b",
    "name": "LOBO",
    "symbol": "LOBO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36187/thumb/lobo200.png?1710760420"
  },
  {
    "chainId": 8453,
    "address": "0x6d6080492d0bd40f1e44cc16791cc1664357f685",
    "name": "ChocoBase",
    "symbol": "CHOCO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31370/thumb/logo_simple_choco_%282%29.png?1696530187"
  },
  {
    "chainId": 8453,
    "address": "0x80f45eacf6537498ecc660e4e4a2d2f99e195cf4",
    "name": "Pepe on Base",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36639/thumb/Pepe_on_Base.png?1712030799"
  },
  {
    "chainId": 8453,
    "address": "0x3c3aa127e6ee3d2f2e432d0184dd36f2d2076b52",
    "name": "Ducky City Earn",
    "symbol": "DCE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35158/thumb/logo_DCE_200.png?1707565577"
  },
  {
    "chainId": 8453,
    "address": "0x0e0c9756a3290cd782cf4ab73ac24d25291c9564",
    "name": "Anime",
    "symbol": "ANIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36565/thumb/anime.png?1711935252"
  },
  {
    "chainId": 8453,
    "address": "0xda761a290e01c69325d12d82ac402e5a73d62e81",
    "name": "Base Pro Shops",
    "symbol": "BPS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36855/thumb/BPS_2.png?1712634757"
  },
  {
    "chainId": 8453,
    "address": "0x2af864fb54b55900cd58d19c7102d9e4fa8d84a3",
    "name": "Grand Base",
    "symbol": "GB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33257/thumb/grandbase.jpeg?1709895415"
  },
  {
    "chainId": 8453,
    "address": "0xd5046b976188eb40f6de40fb527f89c05b323385",
    "name": "BaseX",
    "symbol": "BSX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31419/thumb/bsxlogo.png?1696530234"
  },
  {
    "chainId": 8453,
    "address": "0xe6f47303032a09c8c0f8ebb713c00e6ed345e8c3",
    "name": "Risitas",
    "symbol": "RISITA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36943/thumb/risitas.jpeg?1712890887"
  },
  {
    "chainId": 8453,
    "address": "0x478e03d45716dda94f6dbc15a633b0d90c237e2f",
    "name": "Shaka",
    "symbol": "SHAKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36850/thumb/sharetheshaka.png?1712585688"
  },
  {
    "chainId": 8453,
    "address": "0x532f27101965dd16442e59d40670faf5ebb142e4",
    "name": "Brett",
    "symbol": "BRETT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35529/thumb/1000050750.png?1709031995"
  },
  {
    "chainId": 8453,
    "address": "0x0028e1e60167b48a938b785aa5292917e7eaca8b",
    "name": "Coinye West",
    "symbol": "COINYE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36403/thumb/Coinye_West.png?1711369153"
  },
  {
    "chainId": 8453,
    "address": "0xdda98a036e03611aa50ff457fffbbe9163981529",
    "name": "FRENZ",
    "symbol": "FRENZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36475/thumb/FRENZ.png?1711527630"
  },
  {
    "chainId": 8453,
    "address": "0x7d9ce55d54ff3feddb611fc63ff63ec01f26d15f",
    "name": "Fungi",
    "symbol": "FUNGI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36690/thumb/image_2024-03-29_20-10-19_%282%29_%283%29.png?1712091884"
  },
  {
    "chainId": 8453,
    "address": "0xaa3ecad0cb644c0de72110a905a57667c1a1ca96",
    "name": "weirdo",
    "symbol": "WEIRDO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36290/thumb/weirdo.png?1711012834"
  },
  {
    "chainId": 8453,
    "address": "0x8300e8e6b258147972972dbcf87719da7b817a9c",
    "name": "Pat",
    "symbol": "PAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36545/thumb/Pfp_%282%29.png?1711734258"
  },
  {
    "chainId": 8453,
    "address": "0x940181a94a35a4569e4529a3cdfb74e38fd98631",
    "name": "Aerodrome Finance",
    "symbol": "AERO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31745/thumb/token.png?1696530564"
  },
  {
    "chainId": 8453,
    "address": "0xef80422317dfe797ef7cf51b043226987877d34a",
    "name": "BicOnBase",
    "symbol": "BIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36913/thumb/ticker.jpeg?1712767591"
  },
  {
    "chainId": 8453,
    "address": "0xdfb20f7297aad51e883bba1d21a46defb097c821",
    "name": "Long Johnson",
    "symbol": "OLONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36841/thumb/1000010529.jpg?1712562428"
  },
  {
    "chainId": 8453,
    "address": "0xfea9dcdc9e23a9068bf557ad5b186675c61d33ea",
    "name": "Based Shiba Inu",
    "symbol": "BSHIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36092/thumb/logocg.png?1710478223"
  },
  {
    "chainId": 8453,
    "address": "0x4e496c0256fb9d4cc7ba2fdf931bc9cbb7731660",
    "name": "BOGE",
    "symbol": "BOGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35545/thumb/boge.jpeg?1709094269"
  },
  {
    "chainId": 8453,
    "address": "0x00e57ec29ef2ba7df07ad10573011647b2366f6d",
    "name": "TOSHE",
    "symbol": "TOSHE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23311/thumb/TOSHE.png?1704856679"
  },
  {
    "chainId": 8453,
    "address": "0x063d2db6d348fa5957b78b6141d144426486c68c",
    "name": "SocioCat",
    "symbol": "CAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32679/thumb/Avatar-latest.png?1698934074"
  },
  {
    "chainId": 8453,
    "address": "0x1b6a569dd61edce3c383f6d565e2f79ec3a12980",
    "name": "Young Peezy AKA Pepe",
    "symbol": "PEEZY",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/36649/thumb/For_CoinGecko_200_x_200.jpeg?1712628279"
  },
  {
    "chainId": 8453,
    "address": "0xd07379a755a8f11b57610154861d694b2a0f615a",
    "name": "Base",
    "symbol": "BASE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31199/thumb/59302ba8-022e-45a4-8d00-e29fe2ee768c-removebg-preview.png?1696530026"
  },
  {
    "chainId": 8453,
    "address": "0x6432096f054288ee45b7f6ad8863a1f4a8e1201c",
    "name": "FOMO Base",
    "symbol": "FOMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36468/thumb/FOMO_200X200.jpg?1711518227"
  },
  {
    "chainId": 8453,
    "address": "0xbe3111856e4aca828593274ea6872f27968c8dd6",
    "name": "Krav",
    "symbol": "KRAV",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/31243/thumb/Screen_Shot_2023-08-05_at_2.13.21_PM.png?1696530069"
  },
  {
    "chainId": 8453,
    "address": "0x4a4200947ad71ba4eb5ffeff10cc603a17af2f95",
    "name": "Bell Curve Money",
    "symbol": "BELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36821/thumb/bcm_200x200.png?1712552467"
  },
  {
    "chainId": 8453,
    "address": "0xc4655eb36aa7f1e476a3059a609443ded02ab61f",
    "name": "Particles Money xETH",
    "symbol": "XETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33163/thumb/xETH.png?1700831188"
  },
  {
    "chainId": 8453,
    "address": "0xba71cb8ef2d59de7399745793657838829e0b147",
    "name": "Siamese",
    "symbol": "SIAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36489/thumb/Untitled_design_-_2024-03-25T232153.157.png?1711567424"
  },
  {
    "chainId": 8453,
    "address": "0x54016a4848a38f257b6e96331f7404073fd9c32c",
    "name": "Equalizer  BASE ",
    "symbol": "SCALE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32221/thumb/SCALE_icon_200x200.png?1696835640"
  },
  {
    "chainId": 8453,
    "address": "0xf6e932ca12afa26665dc4dde7e27be02a7c02e50",
    "name": "Mochi",
    "symbol": "MOCHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33083/thumb/mochi200.png?1703024216"
  },
  {
    "chainId": 8453,
    "address": "0x356177b452dbae18b5a4b43b7b5324bfbae235be",
    "name": "DeFido",
    "symbol": "DFD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35776/thumb/logo-defido-200.png?1709794647"
  },
  {
    "chainId": 8453,
    "address": "0x7ed613ab8b2b4c6a781ddc97ea98a666c6437511",
    "name": "All Your Base",
    "symbol": "AYB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35777/thumb/logo-aby.png?1709794884"
  },
  {
    "chainId": 8453,
    "address": "0xd5b70ed3f2ba01bfaaa3beb09e31fe11f833b85f",
    "name": "Based Baby",
    "symbol": "BBB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35515/thumb/bbb.jpg?1708964629"
  },
  {
    "chainId": 8453,
    "address": "0x9d6b8b6fb293c757e05073b84a583ecfaef8d8a7",
    "name": "MUTATIO XCOPYFLIES",
    "symbol": "FLIES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36753/thumb/xcopyflies.jpeg?1712219143"
  },
  {
    "chainId": 8453,
    "address": "0xbcda0bd6cd83558dfb0eec9153ed9c9cfa87782e",
    "name": "WILDx",
    "symbol": "WILD",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/32306/thumb/photo_2023-10-05_00-26-07.png?1697188142"
  },
  {
    "chainId": 8453,
    "address": "0xebff2db643cf955247339c8c6bcd8406308ca437",
    "name": "ChompCoin",
    "symbol": "CHOMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36942/thumb/chomplogo200.png?1712864472"
  },
  {
    "chainId": 8453,
    "address": "0xe3086852a4b125803c815a158249ae468a3254ca",
    "name": "mfercoin",
    "symbol": "MFER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36550/thumb/mfercoin-logo.png?1711876821"
  },
  {
    "chainId": 8453,
    "address": "0x1f3ba804efb9cfe17d595e7262cea4782dbf6e4e",
    "name": "Happy Puppy Club",
    "symbol": "HPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35822/thumb/hpc-token-200.png?1709886836"
  },
  {
    "chainId": 8453,
    "address": "0x27d2decb4bfc9c76f0309b8e88dec3a601fe25a8",
    "name": "Bald",
    "symbol": "BALD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31119/thumb/cdjxKSjo_400x400.jpg?1696529949"
  },
  {
    "chainId": 8453,
    "address": "0xd260115030b9fb6849da169a01ed80b6496d1e99",
    "name": "Based Rate",
    "symbol": "BRATE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32233/thumb/IMG_20231005_222330_591.jpg?1696936841"
  },
  {
    "chainId": 8453,
    "address": "0xb79dd08ea68a908a97220c76d19a6aa9cbde4376",
    "name": "Overnight fi USD ",
    "symbol": "USD+",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25757/thumb/USD__logo.png?1696524843"
  },
  {
    "chainId": 8453,
    "address": "0xff62ddfa80e513114c3a0bf4d6ffff1c1d17aadf",
    "name": "Boe",
    "symbol": "BOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36712/thumb/79jlrE4M_400x400_%282%29.jpg?1712127456"
  },
  {
    "chainId": 8453,
    "address": "0xb661933d9f24ec34dc0fd6862f086079f864b26f",
    "name": "Base Street",
    "symbol": "STREET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36611/thumb/0xb661933d9f24ec34dc0fd6862f086079f864b26f_%281%29.png?1711975109"
  },
  {
    "chainId": 8453,
    "address": "0xb85340f227f4904f5e884a64fa36b39ab493943c",
    "name": "Slerf Cat",
    "symbol": "SLERFCAT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36925/thumb/heroo_%281%29.png?1712798579"
  },
  {
    "chainId": 8453,
    "address": "0x295243647f3efe3bea315d548dcc3d25864ba265",
    "name": "Tochi Base",
    "symbol": "TOCHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33752/thumb/Screenshot_2023-12-17_at_5.04.39%E2%80%AFpm.png?1702946467"
  },
  {
    "chainId": 8453,
    "address": "0x546d239032b24eceee0cb05c92fc39090846adc7",
    "name": "SEED",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35204/thumb/PixotchiLogo.png?1707845029"
  },
  {
    "chainId": 8453,
    "address": "0xd1412d909f67b8db7505ddfcf26cf2303f4b1bb4",
    "name": "RIKU",
    "symbol": "RIKU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35924/thumb/riku.jpeg?1710224102"
  },
  {
    "chainId": 8453,
    "address": "0x9ee4ed687d1ae6fc3a7a427add25c6356876af6c",
    "name": "Based Fink",
    "symbol": "FINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36399/thumb/gigafink.jpg?1711367883"
  },
  {
    "chainId": 8453,
    "address": "0x639c0d019c257966c4907bd4e68e3f349bb58109",
    "name": "Quack Token",
    "symbol": "QUACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31436/thumb/0x639C0D019C257966C4907bD4E68E3F349bB58109.png?1696530251"
  },
  {
    "chainId": 8453,
    "address": "0x28ed8909de1b3881400413ea970cce377a004cca",
    "name": "Punk X",
    "symbol": "PUNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31299/thumb/punkx.jpeg?1696530119"
  },
  {
    "chainId": 8453,
    "address": "0x0a074378461fb7ed3300ea638c6cc38246db4434",
    "name": "El Dorado Exchange  Base ",
    "symbol": "EDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31296/thumb/ede.jpeg?1696530116"
  },
  {
    "chainId": 8453,
    "address": "0xec7acba6fedb7ebfbc0f44f8f0bc6fc39543e28a",
    "name": "AxioDex",
    "symbol": "AXN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33275/thumb/Axiodex_Icon_PNG%282%29.png?1701329198"
  },
  {
    "chainId": 8453,
    "address": "0x4a0a76645941d8c7ba059940b3446228f0db8972",
    "name": "CAROLToken",
    "symbol": "CAROL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32442/thumb/logo_for_social_media_white___blue_logo-2.png?1698237637"
  },
  {
    "chainId": 8453,
    "address": "0x52de0096836bada37c83d6648e0615c40f2d9e51",
    "name": "Baby Brett",
    "symbol": "BABYBRETT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35913/thumb/BB200.png?1710183575"
  },
  {
    "chainId": 8453,
    "address": "0xc5bebd5a6ae6be4fcf383eea014fb1dbd43789ca",
    "name": "Base Name Service",
    "symbol": "BNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31543/thumb/4okORXcg_400x400.png?1696530356"
  },
  {
    "chainId": 8453,
    "address": "0x18a8bd1fe17a1bb9ffb39ecd83e9489cfd17a022",
    "name": "Andy",
    "symbol": "ANDY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36552/thumb/ANDY.jpg?1711896298"
  },
  {
    "chainId": 8453,
    "address": "0x2816a491dd0b7a88d84cbded842a618e59016888",
    "name": "LONG",
    "symbol": "LONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36888/thumb/coin9.png?1712652624"
  },
  {
    "chainId": 8453,
    "address": "0x1dedb41feb0e12637a820c22c1b739ca2e75b48c",
    "name": "BASE BOOK",
    "symbol": "BBOOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36283/thumb/200x200book.png?1711006659"
  },
  {
    "chainId": 8453,
    "address": "0x8129b94753f22ec4e62e2c4d099ffe6773969ebc",
    "name": "Vex Aeterna",
    "symbol": "VEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35775/thumb/webclip_%281%29.png?1709794487"
  },
  {
    "chainId": 8453,
    "address": "0xba5b9b2d2d06a9021eb3190ea5fb0e02160839a4",
    "name": "Sendit",
    "symbol": "SENDIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36789/thumb/Sendit-token-logo.png?1712416889"
  },
  {
    "chainId": 8453,
    "address": "0xfd008f937b4d73eeb00cf74ce90c392be5f07f96",
    "name": "MOON INU",
    "symbol": "MOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36493/thumb/moon-200.png?1711612557"
  },
  {
    "chainId": 8453,
    "address": "0x548f93779fbc992010c07467cbaf329dd5f059b7",
    "name": "BMX",
    "symbol": "BMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31699/thumb/bmx_white.png?1696530517"
  },
  {
    "chainId": 8453,
    "address": "0x4dfbd8a695baf78ebf8f905a4527020e844b278d",
    "name": "We re so back",
    "symbol": "BACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36636/thumb/image_66.png?1712025923"
  },
  {
    "chainId": 8453,
    "address": "0x93e6407554b2f02640ab806cd57bd83e848ec65d",
    "name": "FarLaunch",
    "symbol": "FAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35335/thumb/%E9%BB%91%E5%BA%95.png?1708311337"
  },
  {
    "chainId": 8453,
    "address": "0x4e74d4db6c0726ccded4656d0bce448876bb4c7a",
    "name": "Wrapped BMX Liquidity Token",
    "symbol": "WBLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31532/thumb/wBLT_200x200.png?1696530341"
  },
  {
    "chainId": 8453,
    "address": "0x3eeec801cef575b876d253ab06d75251f67d827d",
    "name": "Ducky City",
    "symbol": "DCM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36664/thumb/logo_duck_200x200_none_bor.png?1712046450"
  },
  {
    "chainId": 8453,
    "address": "0xbe92452bb46485af3308e6d77786bfbe3557808d",
    "name": "Phase Dollar",
    "symbol": "CASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32482/thumb/cash_cg.png?1698288106"
  },
  {
    "chainId": 8453,
    "address": "0xc9e0dc5b3a89c1d55760cbee2b5973957fb9ca11",
    "name": "gembox",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36849/thumb/white_logo.png?1712687544"
  },
  {
    "chainId": 8453,
    "address": "0x67ac031540320886f2706f3e8e680919d7f547a1",
    "name": "Troves",
    "symbol": "TROVES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35140/thumb/troves.jpeg?1707449560"
  },
  {
    "chainId": 8453,
    "address": "0xd2faa0caee8421959aa13fbd20a7ed7e93702ffe",
    "name": "Bald Dog",
    "symbol": "BALDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36287/thumb/0xD2FAA0cAeE8421959aa13Fbd20A7ed7e93702FFE.png?1711008902"
  },
  {
    "chainId": 8453,
    "address": "0x0c659734f1eef9c63b7ebdf78a164cdd745586db",
    "name": "Balancer USDC USDbC axlUSDC",
    "symbol": "USDC-USDBC-AXLUSDC",
    "decimals": 18
  },
  {
    "chainId": 8453,
    "address": "0xbd15d0c77133d3200756dc4d7a4f577dbb2cf6a3",
    "name": "BaseSafe",
    "symbol": "SAFE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36796/thumb/200x200.png?1712628693"
  },
  {
    "chainId": 8453,
    "address": "0xc2fe011c3885277c7f0e7ffd45ff90cadc8ecd12",
    "name": "Poncho",
    "symbol": "PONCHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36160/thumb/ponchologo.PNG?1710744293"
  },
  {
    "chainId": 8453,
    "address": "0x7f12d13b34f5f4f0a9449c16bcd42f0da47af200",
    "name": "NORMIE",
    "symbol": "NORMIE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35880/thumb/NORMIEsite.png?1709983341"
  },
  {
    "chainId": 8453,
    "address": "0x8853f0c059c27527d33d02378e5e4f6d5afb574a",
    "name": "AI INU",
    "symbol": "AIINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36902/thumb/aiinu.png?1712699681"
  },
  {
    "chainId": 8453,
    "address": "0x4acc81dc9c03e5329a2c19763a1d10ba9308339f",
    "name": "Base Baboon",
    "symbol": "BOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36350/thumb/BaboonToken.png?1711191515"
  },
  {
    "chainId": 8453,
    "address": "0x89eb6680c8d85d384c02059e4b07e4dd8535e549",
    "name": "CRAZY FROG",
    "symbol": "FROG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36926/thumb/photo_2024-04-01_16-12-15.jpg?1712800382"
  },
  {
    "chainId": 8453,
    "address": "0x6223901ea64608c75da8497d5eff15d19a1d8fd5",
    "name": "Corgi",
    "symbol": "CORGI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36929/thumb/corgi_-_200x200.png?1712803178"
  },
  {
    "chainId": 8453,
    "address": "0x5babfc2f240bc5de90eb7e19d789412db1dec402",
    "name": "Burning Circle",
    "symbol": "CIRCLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35274/thumb/circle.png?1708048159"
  },
  {
    "chainId": 8453,
    "address": "0x4c584cbc3a221998dc003349e1c12a4179e97b25",
    "name": "Brett Is Based",
    "symbol": "BMONEY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35626/thumb/Brett_logo_480x480_%28200_x_200_px%29.png?1709291062"
  },
  {
    "chainId": 8453,
    "address": "0x314da69de85145fdd5b7580971e9db0388a2cdc4",
    "name": "BasedSwap",
    "symbol": "BSW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36940/thumb/logopng.jpg?1712851900"
  },
  {
    "chainId": 8453,
    "address": "0xff0c532fdb8cd566ae169c1cb157ff2bdc83e105",
    "name": "Fren Pet",
    "symbol": "FP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33022/thumb/token.png?1700274697"
  },
  {
    "chainId": 8453,
    "address": "0xf45116c2e5be608152ea8a6a73917dfe75b707dc",
    "name": "BaseYield",
    "symbol": "BAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33194/thumb/vQ-pZz1S_400x400.jpg?1700971754"
  },
  {
    "chainId": 8453,
    "address": "0xba3b38581f96b04f75ca4ad51296fff3806d7626",
    "name": "Based Potato",
    "symbol": "POTATO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36237/thumb/32orUt2z_400x400.jpeg?1710918375"
  },
  {
    "chainId": 8453,
    "address": "0xcde90558fc317c69580deeaf3efc509428df9080",
    "name": "Normilio",
    "symbol": "NORMILIO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36915/thumb/cg.png?1712769420"
  },
  {
    "chainId": 8453,
    "address": "0xf2d012f604f43e927da3b3576c9c0cafe301428b",
    "name": "Degen Base",
    "symbol": "DB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36420/thumb/DB_Token_Logo.png?1711422014"
  },
  {
    "chainId": 8453,
    "address": "0x0d30be9d9c2cf90aeff4fef5b2e8c3d0b02596a0",
    "name": "BLARB",
    "symbol": "BLARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36275/thumb/blarb.png?1711005265"
  },
  {
    "chainId": 8453,
    "address": "0xaaf7c59712fb510857ff9bbd37885fb88a82849a",
    "name": "Anty the Anteater",
    "symbol": "ANTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36426/thumb/431310940_433519209041733_2725273123295073194_n.png?1711432802"
  },
  {
    "chainId": 8453,
    "address": "0x7d1b04f97dda1fddf0ad9ef5a606e4a956261ac8",
    "name": "Based Money Finance",
    "symbol": "BASED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31388/thumb/BASED256.png?1696530205"
  },
  {
    "chainId": 8453,
    "address": "0x9252384e10ba6beae1a52dbb2a3beca2592268d9",
    "name": "Torch",
    "symbol": "TORCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31788/thumb/Coin_TORCH_FullColor_circle_200.png?1696530605"
  },
  {
    "chainId": 8453,
    "address": "0xca72827a3d211cfd8f6b00ac98824872b72cab49",
    "name": "Cygnus Finance Global USD",
    "symbol": "CGUSD",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35628/thumb/cgUSD.png?1709291316"
  },
  {
    "chainId": 8453,
    "address": "0x58dd173f30ecffdfebcd242c71241fb2f179e9b9",
    "name": "Toup e Tech",
    "symbol": "WIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33279/thumb/token_logo-01.png?1701334931"
  },
  {
    "chainId": 8453,
    "address": "0x37defbc399e5737d53dfb5533d9954572f5b19bf",
    "name": "BlazeBot",
    "symbol": "BLAZE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31279/thumb/blazebot.jpeg?1696530102"
  },
  {
    "chainId": 8453,
    "address": "0x3e05d37cfbd8caaad9e3322d35cc727afaff63e3",
    "name": "Based Peng",
    "symbol": "BENG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36741/thumb/0.5_%281%29_%281%29.png?1712211668"
  },
  {
    "chainId": 8453,
    "address": "0x42069de48741db40aef864f8764432bbccbd0b69",
    "name": "All Street Bets",
    "symbol": "BETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36906/thumb/Screenshot_2024-03-17_034831_no_background.png?1712728634"
  },
  {
    "chainId": 8453,
    "address": "0xef8a84eb92afd22a115d5e81b2c3c605b866f044",
    "name": "Bebe on Base",
    "symbol": "BEBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36839/thumb/logo.png?1712561707"
  },
  {
    "chainId": 8453,
    "address": "0x686b1209b2de12818aa69dd139530448d0c792b3",
    "name": "Poopcoin",
    "symbol": "POOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36549/thumb/poopcoin.png?1711876394"
  },
  {
    "chainId": 8453,
    "address": "0x85e90a5430af45776548adb82ee4cd9e33b08077",
    "name": "Coding Dino",
    "symbol": "DINO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36637/thumb/dino_icon_copy.png?1712029544"
  },
  {
    "chainId": 8453,
    "address": "0x88faea256f789f8dd50de54f9c807eef24f71b16",
    "name": "Landwolf",
    "symbol": "WOLF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36803/thumb/IMG_8678.jpeg?1712469842"
  },
  {
    "chainId": 8453,
    "address": "0x5eb7edfcad75415abb54c345a4ea6ec390f77207",
    "name": "Fortune Bets",
    "symbol": "FORTUNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31286/thumb/200200.png?1696530108"
  },
  {
    "chainId": 8453,
    "address": "0xa68482c74367ff7e7d8aed23553e4c7c2af231f1",
    "name": "NOJEET",
    "symbol": "NOJEET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36895/thumb/nojeet_icon.png?1712674991"
  },
  {
    "chainId": 8453,
    "address": "0x8b67f2e56139ca052a7ec49cbcd1aa9c83f2752a",
    "name": "MUTATIO FLIES",
    "symbol": "FLIES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36752/thumb/MUTATIO.png?1712218952"
  },
  {
    "chainId": 8453,
    "address": "0x9e53e88dcff56d3062510a745952dec4cefdff9e",
    "name": "Basic Dog Meme",
    "symbol": "DOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33749/thumb/Round_Logo.png?1703245893"
  },
  {
    "chainId": 8453,
    "address": "0x4d25e94291fe8dcfbfa572cbb2aaa7b755087c91",
    "name": "High",
    "symbol": "HIGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36318/thumb/high200x200.png?1711092521"
  },
  {
    "chainId": 8453,
    "address": "0x8319767a7b602f88e376368dca1b92d38869b9b4",
    "name": "Based Peaches",
    "symbol": "PEACH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36762/thumb/Based-Peaches-200x200.png?1712260997"
  },
  {
    "chainId": 8453,
    "address": "0x0fa70e156cd3b03ac4080bfe55bd8ab50f5bcb98",
    "name": "Youcoin",
    "symbol": "YOU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31218/thumb/Group_4.png?1696530044"
  },
  {
    "chainId": 8453,
    "address": "0xbefd5c25a59ef2c1316c5a4944931171f30cd3e4",
    "name": "GoldenBoys",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31353/thumb/goldenboys.png?1696530170"
  },
  {
    "chainId": 8453,
    "address": "0x776aaef8d8760129a0398cf8674ee28cefc0eab9",
    "name": "Floppa Cat",
    "symbol": "FLOPPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36969/thumb/IMG_8717.jpeg?1712909755"
  },
  {
    "chainId": 8453,
    "address": "0x17931cfc3217261ce0fa21bb066633c463ed8634",
    "name": "BASEDChad",
    "symbol": "BASED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36378/thumb/based_chad_logo_1.png?1711342570"
  },
  {
    "chainId": 8453,
    "address": "0x64cc19a52f4d631ef5be07947caba14ae00c52eb",
    "name": "Kibble",
    "symbol": "KIBBLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36444/thumb/logo-32x32.png?1711441503"
  },
  {
    "chainId": 8453,
    "address": "0xf395680803b269b62702554723e05b373171b07b",
    "name": "Son of Brett",
    "symbol": "BRATT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36078/thumb/1000005936.jpg?1710414421"
  },
  {
    "chainId": 8453,
    "address": "0x608d5401d377228e465ba6113517dcf9bd1f95ca",
    "name": "Based Rate Share",
    "symbol": "BSHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32127/thumb/input-onlinepngtools.png.jpg?1696588121"
  },
  {
    "chainId": 8453,
    "address": "0xcf3d55c10db69f28fd1a75bd73f3d8a2d9c595ad",
    "name": "Aave v3 cbETH",
    "symbol": "ACBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32892/thumb/cbETH.png?1699789253"
  },
  {
    "chainId": 8453,
    "address": "0xe71f31be318ec604d5ce51879ff1e2d13a5c5847",
    "name": "DHT",
    "symbol": "DHT",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36441/thumb/500x.png?1711440678"
  },
  {
    "chainId": 8453,
    "address": "0x8a24d7260cd02d3dfd8eefb66bc17ad4b17d494c",
    "name": "Based Street Bets",
    "symbol": "BSB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36558/thumb/Logo_BSB.png?1711902137"
  },
  {
    "chainId": 8453,
    "address": "0x17d8217c0f4b4742aaace053281f42eb05ab211d",
    "name": "CloudBase",
    "symbol": "CLOUD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31391/thumb/CloudBaselogo_%281%29.png?1696530207"
  },
  {
    "chainId": 8453,
    "address": "0x614747c53cb1636b4b962e15e1d66d3214621100",
    "name": "CookieBase",
    "symbol": "COOKIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31217/thumb/logo.png?1696530043"
  },
  {
    "chainId": 8453,
    "address": "0x9721b1ce4ffd8af047bbdfd87e4e20ddc544513a",
    "name": "BaseBros",
    "symbol": "BROS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36975/thumb/BB200px.JPG?1712911132"
  },
  {
    "chainId": 8453,
    "address": "0xc5396321805c7f1bce608cda194aa9155fb20f7d",
    "name": "Mooncats on Base",
    "symbol": "MOONCATS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36766/thumb/mooncats.png?1712283962"
  },
  {
    "chainId": 8453,
    "address": "0xcb327b99ff831bf8223cced12b1338ff3aa322ff",
    "name": "Based ETH",
    "symbol": "BSDETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35774/thumb/Icon_White_on_Blue.png?1709793654"
  },
  {
    "chainId": 8453,
    "address": "0x8d8d99dfa6814f694a7edecc18849684fbd3b645",
    "name": "Xidol tech",
    "symbol": "XID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34565/thumb/Logo-Submit-CMC.png?1705501671"
  },
  {
    "chainId": 8453,
    "address": "0x52c45d3068c937cb1e6b4a7f2c2a66b85056dd24",
    "name": "Landtorn Shard",
    "symbol": "SHARD",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31268/thumb/shard.jpg?1696530092"
  },
  {
    "chainId": 8453,
    "address": "0x1d7116f3198421d18bffd5f3dedeebecb4a75be1",
    "name": "Timothy Dexter",
    "symbol": "LORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36764/thumb/0x1d7116f3198421d18bffd5f3dedeebecb4a75be1.jpg?1712282209"
  },
  {
    "chainId": 8453,
    "address": "0x9a3b7959e998bf2b50ef1969067d623877050d92",
    "name": "Pepe But Blue",
    "symbol": "PBB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36206/thumb/200px.png?1710833805"
  },
  {
    "chainId": 8453,
    "address": "0x42069babe14fb1802c5cb0f50bb9d2ad6fef55e2",
    "name": "Frok ai",
    "symbol": "FROK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36028/thumb/Frok_logo.png?1710389187"
  },
  {
    "chainId": 8453,
    "address": "0x2dc1cda9186a4993bd36de60d08787c0c382bead",
    "name": "Magnate Finance",
    "symbol": "MAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31278/thumb/192.png?1696530101"
  },
  {
    "chainId": 8453,
    "address": "0x3159fb5589acd6bf9f82eb0efe8382ed55aed8fd",
    "name": "Apu Apustaja",
    "symbol": "APU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36958/thumb/200X200.png?1712904423"
  },
  {
    "chainId": 8453,
    "address": "0x9a66dff46f5d0d761186936ced6fa4597753a7ad",
    "name": "based bober",
    "symbol": "BOBER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36502/thumb/0bd3eafd-42db-4891-b8d7-5ce63ab61176.jpeg?1711618754"
  },
  {
    "chainId": 8453,
    "address": "0x21eceaf3bf88ef0797e3927d855ca5bb569a47fc",
    "name": "The Void",
    "symbol": "VOID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36669/thumb/void4_%281%29.png?1712051209"
  },
  {
    "chainId": 8453,
    "address": "0x63f844a81571588536614b3fa9260bec3e897126",
    "name": "BREX",
    "symbol": "BREX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36498/thumb/0x63f844a81571588536614b3fa9260bec3e897126.png?1711616623"
  },
  {
    "chainId": 8453,
    "address": "0x5698bba26d8668d781cdcfd2d6d6881226dd0f52",
    "name": "Bool",
    "symbol": "BOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36367/thumb/vM_6qNMg_400x400.jpg?1711337777"
  },
  {
    "chainId": 8453,
    "address": "0x6653dd4b92a0e5bf8ae570a98906d9d6fd2eec09",
    "name": "RocketSwap",
    "symbol": "RCKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31281/thumb/200_logo.png?1696530104"
  },
  {
    "chainId": 8453,
    "address": "0x4ed4e862860bed51a9570b96d89af5e1b0efefed",
    "name": "Degen  Base ",
    "symbol": "DEGEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34515/thumb/android-chrome-512x512.png?1706198225"
  },
  {
    "chainId": 8453,
    "address": "0xcc7ff230365bd730ee4b352cc2492cedac49383e",
    "name": "High Yield USD  Base ",
    "symbol": "HYUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33636/thumb/hyusdlogo.png?1702536133"
  },
  {
    "chainId": 8453,
    "address": "0xc2242d173b610220a5921bc16bd8d9cb31156142",
    "name": "Baseic",
    "symbol": "BASEIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35102/thumb/baseiclogo-3.png?1707379877"
  },
  {
    "chainId": 8453,
    "address": "0x78a087d713be963bf307b18f2ff8122ef9a63ae9",
    "name": "BaseSwap",
    "symbol": "BSWAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31245/thumb/Baseswap_LogoNew.jpg?1696530070"
  },
  {
    "chainId": 8453,
    "address": "0x6d22d3ed82c947be8860a86a69c4b0cb0f65589e",
    "name": "Chonk",
    "symbol": "CHONK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36695/thumb/CG_CHONK.png?1712116693"
  },
  {
    "chainId": 8453,
    "address": "0x7bbab17f79b99708f8cc3be0f4c33024fb403f81",
    "name": "Wild Base",
    "symbol": "BWILD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33293/thumb/output-onlinepngtools_%283%29.png?1701401446"
  },
  {
    "chainId": 8453,
    "address": "0xd9aaec86b65d86f6a7b5b1b0c42ffa531710b6ca",
    "name": "Bridged USD Coin  Base ",
    "symbol": "USDBC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31164/thumb/baseusdc.jpg?1696529993"
  },
  {
    "chainId": 8453,
    "address": "0xd064c53f043d5aee2ac9503b13ee012bf2def1d0",
    "name": "DeFido",
    "symbol": "DEFIDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18861/thumb/coingecko.png?1712043959"
  },
  {
    "chainId": 8453,
    "address": "0xaa6cccdce193698d33deb9ffd4be74eaa74c4898",
    "name": "ElonRWA",
    "symbol": "ELONRWA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36970/thumb/elonrwa.png?1712910039"
  },
  {
    "chainId": 8453,
    "address": "0x5f819f701d6ca4a38ceca5c23e72528580b01977",
    "name": "ZED",
    "symbol": "ZED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32707/thumb/image_%284%29-modified.png?1698986264"
  },
  {
    "chainId": 8453,
    "address": "0x60cbb875d3c6314280dd90a4fe6cafc1d7ce5dfd",
    "name": "eBASE",
    "symbol": "EBASE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/32749/thumb/download.png?1699264126"
  },
  {
    "chainId": 8453,
    "address": "0xafe5451185513925f5e757f001425338ff93412d",
    "name": "Particles Money",
    "symbol": "PARTICLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33164/thumb/PARTICLE.png?1700831300"
  },
  {
    "chainId": 8453,
    "address": "0x5ff986de80fc8502ea9293b8c06ef22b1e3f11e9",
    "name": "Base Inu",
    "symbol": "BINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36541/thumb/baseinu-logo_%281%29.png?1711708876"
  },
  {
    "chainId": 8453,
    "address": "0x09f29e2acdb76a831668b03ce2e3ad7bb41aaa5c",
    "name": "Baseape",
    "symbol": "BAPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31211/thumb/bapeicon.png?1696530038"
  },
  {
    "chainId": 8453,
    "address": "0x9d8a9aec7adcb2b203d285b9c6f91abbcdf1d7e4",
    "name": "DOLP",
    "symbol": "DOLP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36352/thumb/logo_min-min_%281%29.png?1711451584"
  },
  {
    "chainId": 8453,
    "address": "0x347f500323d51e9350285daf299ddb529009e6ae",
    "name": "BLERF",
    "symbol": "BLERF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36473/thumb/BLERF.png?1711527025"
  },
  {
    "chainId": 8453,
    "address": "0x0a1d576f3efef75b330424287a95a366e8281d54",
    "name": "Aave v3 USDbC",
    "symbol": "AUSDBC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32910/thumb/base.png?1699824128"
  },
  {
    "chainId": 8453,
    "address": "0x8ca99727813af2a45df365e920f59692df436ee0",
    "name": "SHADOW",
    "symbol": "SHDW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36132/thumb/icon3_2x.png?1710581601"
  },
  {
    "chainId": 8453,
    "address": "0xd022723a5005f53c95b51d1822f42b1a3366ee4d",
    "name": "COIN",
    "symbol": "COIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31210/thumb/android-chrome-512x512.png?1696530037"
  },
  {
    "chainId": 8453,
    "address": "0xcfb038913d2f93409019e4258244564ad4357be9",
    "name": "Ghozali 404",
    "symbol": "GHZLI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36509/thumb/ghozali.jpeg?1711670958"
  },
  {
    "chainId": 8453,
    "address": "0x85483696cc9970ad9edd786b2c5ef735f38d156f",
    "name": "Overnight fi USDC ",
    "symbol": "USDC+",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34763/thumb/USDC_.png?1705990720"
  },
  {
    "chainId": 8453,
    "address": "0x91f45aa2bde7393e0af1cc674ffe75d746b93567",
    "name": "Frame Token",
    "symbol": "FRAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35034/thumb/Artboard_1_copyd.png?1707189782"
  },
  {
    "chainId": 8453,
    "address": "0xa4af354d466e8a68090dd9eb2cb7caf162f4c8c2",
    "name": "Maid Sweepers",
    "symbol": "SWPRS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35586/thumb/maid-sweeper.png?1709203708"
  },
  {
    "chainId": 8453,
    "address": "0x7b8d415f5239ae5e0f485971529b4f798e63b0b4",
    "name": "Blubi",
    "symbol": "BLUBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36431/thumb/blubi.png?1711434742"
  },
  {
    "chainId": 8453,
    "address": "0xcde172dc5ffc46d228838446c57c1227e0b82049",
    "name": "Boomer",
    "symbol": "BOOMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36477/thumb/Boomer_head_200.png?1711719680"
  },
  {
    "chainId": 8453,
    "address": "0x60359a0dd148b18d5cf1ddf8aa1916221ed0cbcd",
    "name": "NetherFi",
    "symbol": "NFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31557/thumb/Logo_2.png?1696530369"
  },
  {
    "chainId": 8453,
    "address": "0x43e94bd32ac8e57a6009cde6790d95761120d4e9",
    "name": "Dewn",
    "symbol": "DEWN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36379/thumb/dewn_logo.jpg?1711342993"
  },
  {
    "chainId": 8453,
    "address": "0xd386a121991e51eab5e3433bf5b1cf4c8884b47a",
    "name": "Base Velocimeter",
    "symbol": "BVM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31336/thumb/bvm_%281%29.png?1696530154"
  },
  {
    "chainId": 8453,
    "address": "0xbef5d404548fab05820e64f92cf043b6a06f9c72",
    "name": "Pod Finance",
    "symbol": "POD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36699/thumb/Pod_pfp_%282%29.png?1712117891"
  },
  {
    "chainId": 8453,
    "address": "0xbd2dbb8ecea9743ca5b16423b4eaa26bdcfe5ed2",
    "name": "Synthswap",
    "symbol": "SYNTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31190/thumb/synth.png?1696530018"
  },
  {
    "chainId": 8453,
    "address": "0x5b5dee44552546ecea05edea01dcd7be7aa6144a",
    "name": "TN100x",
    "symbol": "TN100X",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35729/thumb/Ham_02-patch.png?1710075243"
  },
  {
    "chainId": 8453,
    "address": "0x45096bd2871911ee82a3084d77a01efa3c9c6733",
    "name": "BUBCAT",
    "symbol": "BUB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36721/thumb/0x45096bd2871911ee82a3084d77a01efa3c9c6733.png?1712129572"
  },
  {
    "chainId": 8453,
    "address": "0x8011eef8fc855df1c4f421443f306e19818e60d3",
    "name": "Duh",
    "symbol": "DUH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36456/thumb/IMG_7076.jpeg?1711474825"
  },
  {
    "chainId": 8453,
    "address": "0xbf1aea8670d2528e08334083616dd9c5f3b087ae",
    "name": "MAI  Base ",
    "symbol": "MIMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35466/thumb/mimatic-red.png?1708687857"
  },
  {
    "chainId": 8453,
    "address": "0x3347453ced85bd288d783d85cdec9b01ab90f9d8",
    "name": "FriendTech33",
    "symbol": "FTW",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31958/thumb/ft33.png?1696530764"
  },
  {
    "chainId": 8453,
    "address": "0x1c1ec1bb5f12f24c97231165b13f3eab9d4ec00e",
    "name": "2FAI",
    "symbol": "2FAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36348/thumb/logo192.png?1711188000"
  },
  {
    "chainId": 8453,
    "address": "0x6f10607507aff450b8cf0a6fa2641a781889ddfa",
    "name": "MILO",
    "symbol": "MILO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/36439/thumb/MILO.png?1711440161"
  },
  {
    "chainId": 8453,
    "address": "0x602417ee10667d0eb216b4a4092aab0bbe4d0511",
    "name": "Xolo",
    "symbol": "XOLO",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/36448/thumb/5CD95348-2CCE-4B79-9AB3-BB2A963B7C97.png?1711445995"
  },
  {
    "chainId": 8453,
    "address": "0x9cbd543f1b1166b2df36b68eb6bb1dce24e6abdf",
    "name": "Based Farm",
    "symbol": "BASED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31641/thumb/Based_Token_Icon.png?1696530457"
  },
  {
    "chainId": 8453,
    "address": "0x1b5d3a85ef27a213c73c610352a0912fd7031637",
    "name": "BaseXchange",
    "symbol": "BEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34826/thumb/BEX.jpg?1706191658"
  },
  {
    "chainId": 8453,
    "address": "0xa7152fa3b1fa9ec858e959164758a66f1ecf33f7",
    "name": "Chicky",
    "symbol": "CHICKY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36451/thumb/0xa7152fa3b1fa9ec858e959164758a66f1ecf33f7_%281%29.png?1711471150"
  },
  {
    "chainId": 8453,
    "address": "0xea930c80d8eac45993d127055e718555e7155fb1",
    "name": "Infinite",
    "symbol": "INF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36156/thumb/Design_sans_titre_25.png?1710743342"
  },
  {
    "chainId": 8453,
    "address": "0x22dc834c3ff3e45f484bf24b9b07b851b981900f",
    "name": "Smudge Cat",
    "symbol": "SMUDCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31209/thumb/logo192.png?1696530036"
  },
  {
    "chainId": 8453,
    "address": "0x63e71271719f03d7233f4fa306b6ea868d0f52ff",
    "name": "BaseInu",
    "symbol": "BINU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31229/thumb/Safeimagekit-resized-img.png?1696530055"
  },
  {
    "chainId": 8453,
    "address": "0x30ed6eb751bb1f94a35256dd68b01d1d27b576a5",
    "name": "Roach Rally",
    "symbol": "ROACH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36387/thumb/Roach_Rally_Logo.png?1711353469"
  },
  {
    "chainId": 8453,
    "address": "0xba5e6fa2f33f3955f0cef50c63dcc84861eab663",
    "name": "based markets",
    "symbol": "BASED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31265/thumb/based_logo_200.png?1696530089"
  },
  {
    "chainId": 8453,
    "address": "0xd473475958d4c1538418224a52e5c0a6c997835a",
    "name": "Based Brians",
    "symbol": "CAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36029/thumb/based_brian_coingecko.png?1710389319"
  },
  {
    "chainId": 8453,
    "address": "0x7d89e05c0b93b24b5cb23a073e60d008fed1acf9",
    "name": "member",
    "symbol": "MEMBER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36041/thumb/member-coingecko.png?1710400496"
  },
  {
    "chainId": 8453,
    "address": "0x981d41c115a2d48cb1215d13bda8f989d407c9c5",
    "name": "Xena Finance",
    "symbol": "XEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32253/thumb/Xena_Logo.png?1697010168"
  },
  {
    "chainId": 8453,
    "address": "0xc2bc7a73613b9bd5f373fe10b55c59a69f4d617b",
    "name": "DackieSwap",
    "symbol": "DACKIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30752/thumb/dackieswap_large.png?1707290196"
  },
  {
    "chainId": 8453,
    "address": "0xde7a416ac821c77478340eebaa21b68297025ef3",
    "name": "Beni",
    "symbol": "BENI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36691/thumb/1000017459.jpg?1712092250"
  },
  {
    "chainId": 8453,
    "address": "0x23e1a3bcdcee4c59209d8871140eb7dd2bd9d1ce",
    "name": "Baso Finance",
    "symbol": "BASO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31327/thumb/baso.jpeg?1696530146"
  },
  {
    "chainId": 8453,
    "address": "0xef0b2ccb53a683fa48799245f376d6a60929f003",
    "name": "MoonBase",
    "symbol": "MOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31867/thumb/moon-logo-200x200.png?1696530679"
  },
  {
    "chainId": 8453,
    "address": "0x0d97f261b1e88845184f678e2d1e7a98d9fd38de",
    "name": "Base God",
    "symbol": "TYBG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34563/thumb/tybg.png?1705400778"
  },
  {
    "chainId": 8453,
    "address": "0x7bc401227777f173ff871993b198a8632741b9bb",
    "name": "Detto Finance",
    "symbol": "DETO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33386/thumb/200_200.png?1701673603"
  },
  {
    "chainId": 8453,
    "address": "0x3421cc14f0e3822cf3b73c3a4bec2a1023b8d9cf",
    "name": "Rebase",
    "symbol": "REBASE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/31444/thumb/R_Transparent.png?1696530258"
  },
  {
    "chainId": 8453,
    "address": "0x9a26f5433671751c3276a065f57e5a02d2817973",
    "name": "Keyboard Cat  Base ",
    "symbol": "KEYCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36608/thumb/keyboard_cat.jpeg?1711965348"
  },
  {
    "chainId": 8453,
    "address": "0x41e99e0f73a88947c52070ff67c19b7abc171a54",
    "name": "Redemption Finance",
    "symbol": "RDMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32332/thumb/IMG_20231011_221107_747.jpg?1697452454"
  },
  {
    "chainId": 8453,
    "address": "0x7233062d88133b5402d39d62bfa23a1b6c8d0898",
    "name": "Citadel",
    "symbol": "FORT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33151/thumb/output-onlinepngtools.png?1700825422"
  },
  {
    "chainId": 8453,
    "address": "0x9edb1571f99ea88e0f8b8409648516a4c32d8541",
    "name": "Don Catblueone",
    "symbol": "DONCAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36731/thumb/IMG_20240401_194107_316.jpg?1712161884"
  },
  {
    "chainId": 8453,
    "address": "0x6921b130d297cc43754afba22e5eac0fbf8db75b",
    "name": "doginme",
    "symbol": "DOGINME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35123/thumb/doginme-logo1-transparent200.png?1710856784"
  },
  {
    "chainId": 8453,
    "address": "0x8c81b4c816d66d36c4bf348bdec01dbcbc70e987",
    "name": "Briun Armstrung",
    "symbol": "BRIUN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36183/thumb/200x200.png?1710758416"
  },
  {
    "chainId": 8453,
    "address": "0x0578d8a44db98b23bf096a382e016e29a5ce0ffe",
    "name": "higher",
    "symbol": "HIGHER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36084/thumb/200x200logo.png?1710427814"
  },
  {
    "chainId": 8453,
    "address": "0x4229c271c19ca5f319fb67b4bc8a40761a6d6299",
    "name": "Neged",
    "symbol": "NEGED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36424/thumb/neged.jpeg?1711429808"
  },
  {
    "chainId": 8453,
    "address": "0x76baa16ff15d61d32e6b3576c3a8c83a25c2f180",
    "name": "Pleb",
    "symbol": "PLEB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36464/thumb/Logo_200x200.png?1711510780"
  },
  {
    "chainId": 8453,
    "address": "0x5399d094b5ec64bdeee130206098ef0e385dc2e9",
    "name": "Hive Protocol",
    "symbol": "HIP",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/36382/thumb/hive.jpeg?1711347525"
  },
  {
    "chainId": 8453,
    "address": "0xac17ee2bee7e06d3e98f7f99818572ed9ea5774d",
    "name": "EARN FINANCE",
    "symbol": "EARNFI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35876/thumb/1000180299.jpg?1709981866"
  },
  {
    "chainId": 8453,
    "address": "0x1c7a460413dd4e964f96d8dfc56e7223ce88cd85",
    "name": "Seamless Protocol",
    "symbol": "SEAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33480/thumb/Seamless_Logo_Black_Transparent.png?1702019657"
  },
  {
    "chainId": 8453,
    "address": "0xece7b98bd817ee5b1f2f536daf34d0b6af8bb542",
    "name": "Just a Black Rock on Base",
    "symbol": "ROCK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36495/thumb/photo_2024-04-05_12-19-47.jpg?1712427724"
  },
  {
    "chainId": 8453,
    "address": "0xb8d98a102b0079b69ffbc760c8d857a31653e56e",
    "name": "Toby ToadGod",
    "symbol": "TOBY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36615/thumb/toby1.png?1711976453"
  },
  {
    "chainId": 8453,
    "address": "0x968d6a288d7b024d5012c0b25d67a889e4e3ec19",
    "name": "Internet Token",
    "symbol": "INT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36593/thumb/internettoken_logo.png?1711949672"
  },
  {
    "chainId": 8453,
    "address": "0x15ac90165f8b45a80534228bdcb124a011f62fee",
    "name": "MOEW",
    "symbol": "MOEW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36737/thumb/MOEW.jpeg?1712210114"
  },
  {
    "chainId": 8453,
    "address": "0xe1abd004250ac8d1f199421d647e01d094faa180",
    "name": "Roost",
    "symbol": "ROOST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36458/thumb/roost.jpeg?1711493580"
  },
  {
    "chainId": 8453,
    "address": "0x10f434b3d1cc13a4a79b062dcc25706f64d10d47",
    "name": "BEPE",
    "symbol": "BEPE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35989/thumb/bepe.jpeg?1710314080"
  },
  {
    "chainId": 8453,
    "address": "0xbc45647ea894030a4e9801ec03479739fa2485f0",
    "name": "Basenji",
    "symbol": "BENJI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36416/thumb/ben200x200.jpg?1711420807"
  },
  {
    "chainId": 8453,
    "address": "0x1dd2d631c92b1acdfcdd51a0f7145a50130050c4",
    "name": "AlienBase",
    "symbol": "ALB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31301/thumb/alb_token.jpg?1696530121"
  },
  {
    "chainId": 8453,
    "address": "0xc2464fb6edc12e4c0d3aa9b201497a27e6b7ef04",
    "name": "BaseAI",
    "symbol": "BASEAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36614/thumb/photo_2024-03-29_12.08.45.jpeg?1711975700"
  },
  {
    "chainId": 8453,
    "address": "0xfa7f2e933c09cd113ef6e72b76d61fb4b27496a1",
    "name": "Basetama",
    "symbol": "BTAMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35818/thumb/1000005865.jpg?1709885504"
  },
  {
    "chainId": 8453,
    "address": "0x655325df2528d3896c6be2d8f5b98cd4020f3191",
    "name": "Ether ORB",
    "symbol": "ORB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36600/thumb/Ether_ORB.png?1711953310"
  },
  {
    "chainId": 8453,
    "address": "0xafb89a09d82fbde58f18ac6437b3fc81724e4df6",
    "name": "The Doge NFT",
    "symbol": "DOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18111/thumb/Doge.png?1696517615"
  },
  {
    "chainId": 8453,
    "address": "0x22a2488fe295047ba13bd8cccdbc8361dbd8cf7c",
    "name": "Sonne Finance",
    "symbol": "SONNE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27540/thumb/Token1.png?1696526577"
  },
  {
    "chainId": 8453,
    "address": "0x9a54ed8d4343e8d89477285a5a0ca06be4051dda",
    "name": "Quadency Token",
    "symbol": "QUAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21364/thumb/GkMXE_7A_400x400.png?1696520730"
  },
  {
    "chainId": 8453,
    "address": "0x5cc5e64ab764a0f1e97f23984e20fd4528356a6a",
    "name": "XRGB",
    "symbol": "XRGB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35447/thumb/log2.png?1708620430"
  },
  {
    "chainId": 8453,
    "address": "0x60d01ec2d5e98ac51c8b4cf84dfcce98d527c747",
    "name": "iZUMi Finance",
    "symbol": "IZI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21791/thumb/izumi-logo-symbol.png?1696521144"
  },
  {
    "chainId": 8453,
    "address": "0xd857af86a2c5b4f46fc7cb8032bd4f5625577eeb",
    "name": "Hokkaidu Inu",
    "symbol": "HOKK",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/34890/thumb/IMG29529FNM3.png?1710943740"
  },
  {
    "chainId": 8453,
    "address": "0x7f62ac1e974d65fab4a81821ca6af659a5f46298",
    "name": "Ethlas",
    "symbol": "ELS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30331/thumb/ELS_Logo_200x200.png?1696529232"
  },
  {
    "chainId": 8453,
    "address": "0x24fcfc492c1393274b6bcd568ac9e225bec93584",
    "name": "Heroes of Mavia",
    "symbol": "MAVIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33895/thumb/2023-12-20_21.21.41_%281%29.jpg?1703230771"
  },
  {
    "chainId": 8453,
    "address": "0xf578ad8809f13dabf921bdd3fcfbe194d0ab5628",
    "name": "Virtue Poker Points",
    "symbol": "VPP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3386/thumb/vp-logo-200x200.png?1696504084"
  },
  {
    "chainId": 8453,
    "address": "0x3419875b4d3bca7f3fdda2db7a476a79fd31b4fe",
    "name": "DizzyHavoc",
    "symbol": "DZHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35575/thumb/dizzyhavoc_circle_3_200_200_transparentBg.png?1709194317"
  },
  {
    "chainId": 8453,
    "address": "0x13f4196cc779275888440b3000ae533bbbbc3166",
    "name": "Alongside Crypto Market Index",
    "symbol": "AMKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28496/thumb/22999.png?1696527488"
  },
  {
    "chainId": 8453,
    "address": "0xb4e1b230dd0476238fc64c99ff9d6ccdfdb2258d",
    "name": "Florence Finance Medici",
    "symbol": "FFM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34382/thumb/M_PNG_200x200_copy.png?1704779326"
  },
  {
    "chainId": 8453,
    "address": "0x417ac0e078398c154edfadd9ef675d30be60af93",
    "name": "crvUSD",
    "symbol": "CRVUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30118/thumb/crvusd.jpeg?1696529040"
  },
  {
    "chainId": 8453,
    "address": "0xc7dcca0a3e69bd762c8db257f868f76be36c8514",
    "name": "KiboShib",
    "symbol": "KIBSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29335/thumb/foto_no_exif_%2811%29%282%29_%281%29.png?1696528285"
  },
  {
    "chainId": 8453,
    "address": "0xe095780ba2a64a4efa7a74830f0b71656f0b0ad4",
    "name": "Byte",
    "symbol": "BYTE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33527/thumb/Byte200.jpeg?1702138460"
  },
  {
    "chainId": 8453,
    "address": "0x0b3e328455c4059eeb9e3f84b5543f74e24e7e1b",
    "name": "Virtual Protocol",
    "symbol": "VIRTUAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34057/thumb/LOGOMARK.png?1708356054"
  },
  {
    "chainId": 8453,
    "address": "0x2974dc646e375e83bd1c0342625b49f288987fa4",
    "name": "Swarm Markets",
    "symbol": "SMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17488/thumb/swarm-SMT-token-symbol_200x200.png?1696517029"
  },
  {
    "chainId": 8453,
    "address": "0x7ccdba6198db389cf37b714fd6573b73f3670236",
    "name": "Basket",
    "symbol": "BSKT",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/34661/thumb/BSKT_Logo.png?1705636891"
  },
  {
    "chainId": 8453,
    "address": "0x50da645f148798f68ef2d7db7c1cb22a6819bb2c",
    "name": "SPX6900",
    "symbol": "SPX",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31401/thumb/sticker_%281%29.jpg?1702371083"
  },
  {
    "chainId": 8453,
    "address": "0x42069d11a2cc72388a2e06210921e839cfbd3280",
    "name": "GnomeLand",
    "symbol": "GNOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35201/thumb/logopng.png?1707841763"
  },
  {
    "chainId": 8453,
    "address": "0x2da56acb9ea78330f947bd57c54119debda7af71",
    "name": "Mog Coin",
    "symbol": "MOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31059/thumb/MOG_LOGO_200x200.png?1696529893"
  },
  {
    "chainId": 8453,
    "address": "0x777b2839832982b35213063d850848369390ee16",
    "name": "Jarvis",
    "symbol": "JARVIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35812/thumb/jarvis.png?1709873633"
  },
  {
    "chainId": 8453,
    "address": "0x66e535e8d2ebf13f49f3d49e5c50395a97c137b1",
    "name": "Molten",
    "symbol": "MOLTEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36726/thumb/moltenmesh.png?1712147407"
  },
  {
    "chainId": 8453,
    "address": "0x0f76d32cdccdcbd602a55af23eaf58fd1ee17245",
    "name": "Backed ERNA   Bond",
    "symbol": "BERNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36757/thumb/b-ERNA-200x200.png?1712245846"
  },
  {
    "chainId": 8453,
    "address": "0x715da73449049d131da72d49fb1e59f5d0d49cf0",
    "name": "impactMarket",
    "symbol": "PACT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21907/thumb/PACT_Token_Ticker_Blue_2x.png?1696521258"
  },
  {
    "chainId": 8453,
    "address": "0x6e51b3a19f114013e5dc09d0477a536c7e4e0207",
    "name": "Media Network",
    "symbol": "MEDIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15142/thumb/media50x50.png?1696514798"
  },
  {
    "chainId": 8453,
    "address": "0x54bc229d1cb15f8b6415efeab4290a40bc8b7d84",
    "name": "dHEDGE DAO",
    "symbol": "DHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12508/thumb/dht.png?1696512323"
  },
  {
    "chainId": 8453,
    "address": "0x4158734d47fc9692176b5085e0f52ee0da5d47f1",
    "name": "Balancer",
    "symbol": "BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11683/thumb/Balancer.png?1696511572"
  },
  {
    "chainId": 8453,
    "address": "0x1a35ee4640b0a3b87705b0a4b45d227ba60ca2ad",
    "name": "axlWBTC",
    "symbol": "AXLWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28173/thumb/wbtc-satoshi_D_3x.png?1696986732"
  },
  {
    "chainId": 8453,
    "address": "0x2ae3f1ec7f1f5012cfeab0185bfc7aa3cf0dec22",
    "name": "Coinbase Wrapped Staked ETH",
    "symbol": "CBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27008/thumb/cbeth.png?1709186989"
  },
  {
    "chainId": 8453,
    "address": "0x65a2508c429a6078a7bc2f7df81ab575bd9d9275",
    "name": "Overnight fi DAI ",
    "symbol": "DAI+",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29634/thumb/DAI_.png?1696528571"
  },
  {
    "chainId": 8453,
    "address": "0xd1db4851bcf5b41442caa32025ce0afe6b8eabc2",
    "name": "Zoomer",
    "symbol": "ZOOMER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30894/thumb/zoooooooooomer.jpg?1696529740"
  },
  {
    "chainId": 8453,
    "address": "0x8fbd0648971d56f1f2c35fa075ff5bc75fb0e39d",
    "name": "UNKJD",
    "symbol": "MBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20841/thumb/monkeyball.png?1696520233"
  },
  {
    "chainId": 8453,
    "address": "0xf7c1cefcf7e1dd8161e00099facd3e1db9e528ee",
    "name": "Tower",
    "symbol": "TOWER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14134/thumb/tower-circular-1000.png?1696513854"
  },
  {
    "chainId": 8453,
    "address": "0x50c5725949a6f0c72e6c4a641f24049a917db0cb",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 8453,
    "address": "0x8f0f56472c3e5730b1ea2f444e7829288da261e6",
    "name": "Rogue MAV",
    "symbol": "RMAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32384/thumb/rMAV.png?1698045143"
  },
  {
    "chainId": 8453,
    "address": "0x236aa50979d5f3de3bd1eeb40e81137f22ab794b",
    "name": "tBTC",
    "symbol": "TBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11224/thumb/0x18084fba666a33d37592fa2633fd49a74dd93a88.png?1696511155"
  },
  {
    "chainId": 8453,
    "address": "0xfa980ced6895ac314e7de34ef1bfae90a5add21b",
    "name": "Echelon Prime",
    "symbol": "PRIME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29053/thumb/prime-logo-small-border_%282%29.png?1696528020"
  },
  {
    "chainId": 8453,
    "address": "0xab36452dbac151be02b16ca17d8919826072f64a",
    "name": "Reserve Rights",
    "symbol": "RSR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8365/thumb/rsr.png?1696508558"
  },
  {
    "chainId": 8453,
    "address": "0x489fe42c267fe0366b16b0c39e7aeef977e841ef",
    "name": "Wrapped Ampleforth",
    "symbol": "WAMPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20825/thumb/photo_2021-11-25_02-05-11.jpg?1696520218"
  },
  {
    "chainId": 8453,
    "address": "0x97c806e7665d3afd84a8fe1837921403d59f3dcc",
    "name": "Artificial Liquid Intelligence",
    "symbol": "ALI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22062/thumb/Logo_Circle_720_%281%29.png?1696521405"
  },
  {
    "chainId": 8453,
    "address": "0xa334884bf6b0a066d553d19e507315e839409e62",
    "name": "Ethos Reserve Note",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29744/thumb/ERN200x200.png?1696528676"
  },
  {
    "chainId": 8453,
    "address": "0xc5102fe9359fd9a28f877a67e36b0f050d81a3cc",
    "name": "Hop Protocol",
    "symbol": "HOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25445/thumb/hop.png?1696524577"
  },
  {
    "chainId": 8453,
    "address": "0x09188484e1ab980daef53a9755241d759c5b7d60",
    "name": "RigoBlock",
    "symbol": "GRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1532/thumb/Symbol-RigoblockRGB.png?1696502572"
  },
  {
    "chainId": 8453,
    "address": "0x9eaf8c1e34f05a589eda6bafdf391cf6ad3cb239",
    "name": "yearn finance",
    "symbol": "YFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11849/thumb/yearn.jpg?1696511720"
  },
  {
    "chainId": 8453,
    "address": "0xa23dd9379f2e12d9ce76ec738b9f5e520d1d861c",
    "name": "USDEBT",
    "symbol": "USDEBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31448/thumb/200x200.png?1710861880"
  },
  {
    "chainId": 8453,
    "address": "0xcfa3ef56d303ae4faaba0592388f19d7c3399fb4",
    "name": "Electronic USD",
    "symbol": "EUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28445/thumb/0xa0d69e286b938e21cbf7e51d71f6a4c8918f482f.png?1696527441"
  },
  {
    "chainId": 8453,
    "address": "0x430c67ed118f0637ffe74c98e9b90919c93b7233",
    "name": "Squid Game",
    "symbol": "SQUID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20506/thumb/squid.png?1696519912"
  },
  {
    "chainId": 8453,
    "address": "0x30b593f8c3ab37615359b4e0e6df2e06d55bb55d",
    "name": "FXDX",
    "symbol": "FXDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32730/thumb/ICONIC_LOGO_BLACK_PNG_%281%29.png?1699217673"
  },
  {
    "chainId": 8453,
    "address": "0xff8adec2221f9f4d8dfbafa6b9a297d17603493d",
    "name": "Moonwell",
    "symbol": "WELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26133/thumb/WELL.png?1696525221"
  },
  {
    "chainId": 8453,
    "address": "0x6b4712ae9797c199edd44f897ca09bc57628a1cf",
    "name": "UniDex",
    "symbol": "UNIDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13178/thumb/unidx.png?1696512961"
  },
  {
    "chainId": 8453,
    "address": "0xb6fe221fe9eef5aba221c348ba20a1bf5e73624c",
    "name": "Rocket Pool ETH",
    "symbol": "RETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20764/thumb/reth.png?1696520159"
  },
  {
    "chainId": 8453,
    "address": "0xf34e0cff046e154cafcae502c7541b9e5fd8c249",
    "name": "Thales",
    "symbol": "THALES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18388/thumb/CLVZJN_C_400x400.png?1696517879"
  },
  {
    "chainId": 8453,
    "address": "0x8888888888f004100c0353d657be6300587a6ccd",
    "name": "ACryptoS",
    "symbol": "ACS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32721/thumb/ACS.jpg?1699009686"
  },
  {
    "chainId": 8453,
    "address": "0xfd4330b0312fdeec6d4225075b82e00493ff2e3f",
    "name": "SmarDex",
    "symbol": "SDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29470/thumb/SDEX_logo_transparent_outside_240x240.png?1696930070"
  },
  {
    "chainId": 8453,
    "address": "0x833589fcd6edb6e08f4c7c32d4f71b54bda02913",
    "name": "USDC",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/6319/thumb/usdc.png?1696506694"
  },
  {
    "chainId": 8453,
    "address": "0x2dad3a13ef0c6366220f989157009e501e7938f8",
    "name": "Extra Finance",
    "symbol": "EXTRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30973/thumb/Ex_logo-white-blue_ring_288x.png?1696529812"
  },
  {
    "chainId": 8453,
    "address": "0xd1f398d6b3e5e2387e413831e206cfeb5fc1dcee",
    "name": "PUBLC",
    "symbol": "PUBLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29074/thumb/PUBLX_Logo_%28200x200%29.png?1696528040"
  },
  {
    "chainId": 8453,
    "address": "0xdcf5130274753c8050ab061b1a1dcbf583f5bfd0",
    "name": "ViciCoin",
    "symbol": "VCNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31305/thumb/ViciCoin_-_small.png?1696530124"
  },
  {
    "chainId": 8453,
    "address": "0x4200000000000000000000000000000000000006",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 8453,
    "address": "0x74ccbe53f77b08632ce0cb91d3a545bf6b8e0979",
    "name": "Fantom Bomb",
    "symbol": "BOMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24109/thumb/logo-blue.png?1696523301"
  },
  {
    "chainId": 8453,
    "address": "0xa617c0c739845b2941bd8edd05c9f993ecc97c18",
    "name": "GAMER",
    "symbol": "GMR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21288/thumb/ezgif-1-7f6a016717.jpg?1696520658"
  },
  {
    "chainId": 8453,
    "address": "0x23ee2343b892b1bb63503a4fabc840e0e2c6810f",
    "name": "Axelar",
    "symbol": "AXL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27277/thumb/V-65_xQ1_400x400.jpeg?1696526329"
  },
  {
    "chainId": 8453,
    "address": "0xeb466342c4d449bc9f53a865d5cb90586f405215",
    "name": "Axelar Bridged USDC",
    "symbol": "AXLUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26476/thumb/uausdc_D_3x.png?1696525548"
  },
  {
    "chainId": 8453,
    "address": "0x0a1d576f3efef75b330424287a95a366e8281d54",
    "name": "Aave v3 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32847/thumb/usdc_%281%29.png?1699619355"
  },
  {
    "chainId": 8453,
    "address": "0xbfd5206962267c7b4b4a8b3d76ac2e1b2a5c4d5e",
    "name": "Osaka Protocol",
    "symbol": "OSAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30911/thumb/osak_logo.png?1696529756"
  },
  {
    "chainId": 8453,
    "address": "0x5a76a56ad937335168b30df3aa1327277421c6ae",
    "name": "Vela Token",
    "symbol": "VELA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28739/thumb/VELA_logo_-_no_background.png?1696527719"
  },
  {
    "chainId": 8453,
    "address": "0xf544251d25f3d243a36b07e7e7962a678f952691",
    "name": "Tarot",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31800/thumb/TAROT.jpg?1696530615"
  },
  {
    "chainId": 8453,
    "address": "0x703d57164ca270b0b330a87fd159cfef1490c0a5",
    "name": "RAI Finance",
    "symbol": "SOFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14686/thumb/sofi.png?1696514359"
  },
  {
    "chainId": 8453,
    "address": "0xde5ed76e7c05ec5e4572cfc88d1acea165109e44",
    "name": "DEUS Finance",
    "symbol": "DEUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18778/thumb/Black_Background_200x200.png?1696518242"
  },
  {
    "chainId": 8453,
    "address": "0xa3d1a8deb97b111454b294e2324efad13a9d8396",
    "name": "Overnight Finance",
    "symbol": "OVN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31970/thumb/OVN.png?1696959174"
  },
  {
    "chainId": 8453,
    "address": "0xc19669a405067927865b40ea045a2baabbbe57f5",
    "name": "Star",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31277/thumb/STAR.png?1696530101"
  },
  {
    "chainId": 8453,
    "address": "0xd4a0e0b9149bcee3c920d2e00b5de09138fd8bb7",
    "name": "Aave v3 WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32882/thumb/WETH_%281%29.png?1699716492"
  },
  {
    "chainId": 8453,
    "address": "0x2b1d36f5b61addaf7da7ebbd11b35fd8cfb0de31",
    "name": "Interport Token",
    "symbol": "ITP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28338/thumb/ITP_Logo_200.png?1696527344"
  },
  {
    "chainId": 8453,
    "address": "0x4621b7a9c75199271f773ebd9a499dbd165c3191",
    "name": "DOLA",
    "symbol": "DOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14287/thumb/dola.png?1696513984"
  },
  {
    "chainId": 8453,
    "address": "0xade6057fcafa57d6d51ffa341c64ce4814995995",
    "name": "Backed ZPR1   1 3 Month T Bill",
    "symbol": "BZPR1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32867/thumb/b-ZPR1-200x200.png?1699671603"
  },
  {
    "chainId": 8453,
    "address": "0x4a3a6dd60a34bb2aba60d73b4c88315e9ceb6a3d",
    "name": "Magic Internet Money",
    "symbol": "MIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16786/thumb/mimlogopng.png?1696516358"
  },
  {
    "chainId": 8453,
    "address": "0xebb78043e29f4af24e6266a7d142f5a08443969e",
    "name": "Derp",
    "symbol": "DERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33069/thumb/derpdex_%281%29.png?1700793428"
  },
  {
    "chainId": 8453,
    "address": "0xe85411c030fb32a9d8b14bbbc6cb19417391f711",
    "name": "Sumer Money suBTC",
    "symbol": "SUBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33264/thumb/Sumer_Money_Logo.jpg?1701321397"
  },
  {
    "chainId": 8453,
    "address": "0x1c22531aa9747d76fff8f0a43b37954ca67d28e0",
    "name": "Sumer Money suETH",
    "symbol": "SUETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33265/thumb/Sumer_Money_Logo.jpg?1701321407"
  },
  {
    "chainId": 8453,
    "address": "0x8bf591eae535f93a242d5a954d3cde648b48a5a8",
    "name": "Sumer Money suUSD",
    "symbol": "SUUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33266/thumb/Sumer_Money_Logo.jpg?1701321416"
  },
  {
    "chainId": 8453,
    "address": "0x4f604735c1cf31399c6e711d5962b2b3e0225ad3",
    "name": "Glo Dollar",
    "symbol": "USDGLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29319/thumb/glo_logo_coingecko.png?1696528270"
  },
  {
    "chainId": 8453,
    "address": "0xb166e8b140d35d9d8226e40c09f757bac5a4d87d",
    "name": "Non Playable Coin",
    "symbol": "NPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31193/thumb/NPC_200x200.png?1696530021"
  },
  {
    "chainId": 8453,
    "address": "0x59d9356e565ab3a36dd77763fc0d87feaf85508c",
    "name": "Mountain Protocol USD",
    "symbol": "USDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31719/thumb/usdm.png?1696530540"
  },
  {
    "chainId": 8453,
    "address": "0xf7a0dd3317535ec4f4d29adf9d620b3d8d5d5069",
    "name": "Staked Ethos Reserve Note",
    "symbol": "STERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31522/thumb/stERN.png?1696530332"
  },
  {
    "chainId": 8453,
    "address": "0xc48e605c7b722a57277e087a6170b9e227e5ac0a",
    "name": "OmniCat",
    "symbol": "OMNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33917/thumb/2023-12-21_21.39.10.jpg?1703321588"
  },
  {
    "chainId": 8453,
    "address": "0xa067436db77ab18b1a315095e4b816791609897c",
    "name": "WASSIE",
    "symbol": "WASSIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30144/thumb/logo-coingecko.png?1696529065"
  },
  {
    "chainId": 8453,
    "address": "0x2598c30330d5771ae9f983979209486ae26de875",
    "name": "Any Inu",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34126/thumb/anyinulogo200.png?1704174269"
  },
  {
    "chainId": 8453,
    "address": "0x881ed0fcef78120a135ec6cc66cef2779fe95bba",
    "name": "DogeGF",
    "symbol": "DOGEGF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18651/thumb/dogegf_logo_200x200.png?1712082872"
  },
  {
    "chainId": 8453,
    "address": "0x524d524b4c9366be706d3a90dcf70076ca037ae3",
    "name": "RMRK",
    "symbol": "RMRK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18656/thumb/download_%281%29_%281%29.png?1696518126"
  },
  {
    "chainId": 8453,
    "address": "0x255f1b39172f65dc6406b8bee8b08155c45fe1b6",
    "name": "HarambeCoin",
    "symbol": "HARAMBE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36043/thumb/HarambeCoin_full_cameo_200.png?1710400612"
  },
  {
    "chainId": 8453,
    "address": "0x3f95aa88ddbb7d9d484aa3d482bf0a80009c52c9",
    "name": "Backed ERNX   Bond",
    "symbol": "BERNX",
    "decimals": 18
  },
  {
    "chainId": 8453,
    "address": "0x8f019931375454fe4ee353427eb94e2e0c9e0a8c",
    "name": "KOMPETE",
    "symbol": "KOMPETE",
    "decimals": 10,
    "logoURI": "https://assets.coingecko.com/coins/images/24298/thumb/KOMPETE_LOGO.?1696523480"
  },
  {
    "chainId": 8453,
    "address": "0x4a506181f07da5ddfda4ca4c2fa4c67001db94b4",
    "name": "Dyl",
    "symbol": "DYL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32550/thumb/Dyl-pfp-200x.png?1698479973"
  },
  {
    "chainId": 8453,
    "address": "0xd7b675cd5c84a13d1d0f84509345530f6421b57c",
    "name": "Corridor Finance",
    "symbol": "OOOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36057/thumb/corridor-logo-without-text-8bit.jpg?1710404509"
  },
  {
    "chainId": 8453,
    "address": "0x1f1aa4d239002bb818536c95e9b762a1fc8484c1",
    "name": "Precipitate ai",
    "symbol": "RAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32405/thumb/RAIN_200-removebg-preview.png?1711371589"
  },
  {
    "chainId": 8453,
    "address": "0xfbb75a59193a3525a8825bebe7d4b56899e2f7e1",
    "name": "ResearchCoin",
    "symbol": "RSC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28146/thumb/RH_Logo_200x200.png?1696527152"
  },
  {
    "chainId": 8453,
    "address": "0xdfbea88c4842d30c26669602888d746d30f9d60d",
    "name": "crow with knife",
    "symbol": "CAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36067/thumb/200px.png?1710405601"
  },
  {
    "chainId": 8453,
    "address": "0x2416092f143378750bb29b79ed961ab195cceea5",
    "name": "Renzo Restaked ETH",
    "symbol": "EZETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34753/thumb/eth_renzo_logo_%281%29.png?1705956747"
  },
  {
    "chainId": 8453,
    "address": "0xf4435cc8b478d54313f04c956882be3d9acf9f6f",
    "name": "Lucha",
    "symbol": "LUCHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22794/thumb/lucha.png?1696522097"
  },
  {
    "chainId": 42161,
    "address": "0x6533afac2e7bccb20dca161449a13a32d391fb00",
    "name": "Aave v3 ARB",
    "symbol": "AARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32912/thumb/arbitrum.png?1699824441"
  },
  {
    "chainId": 42161,
    "address": "0x3a74a9032f731d504f10e02d7247d23b939ea4f6",
    "name": "CEX AI",
    "symbol": "CEX-AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29231/thumb/200x200_black.png?1696528188"
  },
  {
    "chainId": 42161,
    "address": "0x17176a9868f321411b15ccb9b934cf95597e89c4",
    "name": "Good Entry",
    "symbol": "GOOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33407/thumb/photo_2023-12-04_22-16-42.jpg?1701699782"
  },
  {
    "chainId": 42161,
    "address": "0xe3b3fe7bca19ca77ad877a5bebab186becfad906",
    "name": "Staked FRAX",
    "symbol": "SFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35383/thumb/sfrax.png?1708445569"
  },
  {
    "chainId": 42161,
    "address": "0xc19669a405067927865b40ea045a2baabbbe57f5",
    "name": "Star",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31277/thumb/STAR.png?1696530101"
  },
  {
    "chainId": 42161,
    "address": "0x8e50d11a54cff859b202b7fe5225353be0646410",
    "name": "Dinari GOOGL",
    "symbol": "GOOGLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32428/thumb/token-icon.png?1698120046"
  },
  {
    "chainId": 42161,
    "address": "0xa06d505ec28fd9756a200d2b503a66103db98d09",
    "name": "KEI Finance",
    "symbol": "KEI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29840/thumb/200x200.png?1696528767"
  },
  {
    "chainId": 42161,
    "address": "0x51f9f9ff6cb2266d68c04ec289c7aba81378a383",
    "name": "iGameS",
    "symbol": "IGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31416/thumb/igs_logo.png?1696530231"
  },
  {
    "chainId": 42161,
    "address": "0xbcc9c1763d54427bdf5efb6e9eb9494e5a1fbabf",
    "name": "Honor World Token",
    "symbol": "HWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28559/thumb/hwt.png?1696527549"
  },
  {
    "chainId": 42161,
    "address": "0x4e05d153305bc472e220ec3f7a7894b38f74741f",
    "name": "Smart Aliens",
    "symbol": "SAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30710/thumb/SAS200x200.png?1699220256"
  },
  {
    "chainId": 42161,
    "address": "0xb5b5b428e4de365f809ced8271d202449e5c2f72",
    "name": "BRUH",
    "symbol": "BRUH",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32227/thumb/BRUH.jpg?1696931468"
  },
  {
    "chainId": 42161,
    "address": "0x000f1720a263f96532d1ac2bb9cdc12b72c6f386",
    "name": "Fluidity",
    "symbol": "FLY",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/36086/thumb/FLY_2D_Old_Map_Double_Border.png?1710434215"
  },
  {
    "chainId": 42161,
    "address": "0x5575552988a3a80504bbaeb1311674fcfd40ad4b",
    "name": "Sperax",
    "symbol": "SPA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12232/thumb/sperax_logo.jpg?1696512065"
  },
  {
    "chainId": 42161,
    "address": "0xb2d948be3a74ecce80378d4093e6cd7f4dc1cf9c",
    "name": "Alt Markets",
    "symbol": "AMX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29025/thumb/logo-200.png?1696527995"
  },
  {
    "chainId": 42161,
    "address": "0x6fd58f5a2f3468e35feb098b5f59f04157002407",
    "name": "POGAI",
    "symbol": "POGAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30116/thumb/pogai.jpeg?1696529039"
  },
  {
    "chainId": 42161,
    "address": "0x45d780cfe839b8a8f292c9e26aa5d1805bc536dc",
    "name": "Zoci",
    "symbol": "ZOCI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31809/thumb/200x.png?1696530624"
  },
  {
    "chainId": 42161,
    "address": "0x7f465507f058e17ad21623927a120ac05ca32741",
    "name": "Arcadeum",
    "symbol": "ARC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29044/thumb/arc.png?1696528012"
  },
  {
    "chainId": 42161,
    "address": "0x54bdbf3ce36f451ec61493236b8e6213ac87c0f6",
    "name": "Radpie",
    "symbol": "RDP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34052/thumb/Radpie_token_3_2.png?1704672743"
  },
  {
    "chainId": 42161,
    "address": "0xc4a0e0915d538b5ec2e6895261d34f70c64fb13d",
    "name": "Bitago",
    "symbol": "XBIT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33961/thumb/photo_2023-12-26_21-57-47.jpg?1703622923"
  },
  {
    "chainId": 42161,
    "address": "0x0d8f04522c5792c7378e39c92ab348f315f4fc4f",
    "name": "AIENGLISH",
    "symbol": "AIEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30291/thumb/200.png?1696529196"
  },
  {
    "chainId": 42161,
    "address": "0x65c936f008bc34fe819bce9fa5afd9dc2d49977f",
    "name": "Y2K",
    "symbol": "Y2K",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28542/thumb/l7jRo-5-_400x400.jpg?1696527534"
  },
  {
    "chainId": 42161,
    "address": "0xa6ef0ad746d1c35d6ff4d66ceeae0e596d742924",
    "name": "LumiTerra Totem 404",
    "symbol": "LTM04",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36209/thumb/Totem_Token_ICON.png?1710835365"
  },
  {
    "chainId": 42161,
    "address": "0xe7f6c3c1f0018e4c08acc52965e5cbff99e34a44",
    "name": "plsJONES",
    "symbol": "PLSJONES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30553/thumb/plsJONES.png?1696529424"
  },
  {
    "chainId": 42161,
    "address": "0xf1264873436a0771e440e2b28072fafcc5eebd01",
    "name": "Kenshi",
    "symbol": "KNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30759/thumb/kenshi.jpg?1696529628"
  },
  {
    "chainId": 42161,
    "address": "0x031d35296154279dc1984dcd93e392b1f946737b",
    "name": "Cap",
    "symbol": "CAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11775/thumb/cap.png?1696511654"
  },
  {
    "chainId": 42161,
    "address": "0xf36a65fd3b7df848860d174115f1864e6aa2db5e",
    "name": "Seneca USD",
    "symbol": "SENUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35621/thumb/senUSD_logo_200x200.png?1709288877"
  },
  {
    "chainId": 42161,
    "address": "0xc56dc613c8ef3c57314171d73eaefe87aa469186",
    "name": "TLSD Coin",
    "symbol": "TLSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30748/thumb/Icon-200.png?1696529617"
  },
  {
    "chainId": 42161,
    "address": "0x842216e0aa2ae608699f7b1063f26ce6b30c5311",
    "name": "UrDEX Finance",
    "symbol": "URD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29970/thumb/UrDEXLogo.png?1696528896"
  },
  {
    "chainId": 42161,
    "address": "0xf9ca0ec182a94f6231df9b14bd147ef7fb9fa17c",
    "name": "BONER",
    "symbol": "BONER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33968/thumb/unnamed.png?1703575844"
  },
  {
    "chainId": 42161,
    "address": "0x0b5c6ac0e1082f2d81e829b8c2957886e6bb3994",
    "name": "Prism",
    "symbol": "PRISM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31081/thumb/download_%2818%29.png?1696529914"
  },
  {
    "chainId": 42161,
    "address": "0x32df62dc3aed2cd6224193052ce665dc18165841",
    "name": "Balancer 80 RDNT 20 WETH",
    "symbol": "DLP",
    "decimals": 18
  },
  {
    "chainId": 42161,
    "address": "0xc84fae1141b92fa5bf847276828f69caf651cb7f",
    "name": "XNF",
    "symbol": "XNF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33406/thumb/XNF-Light_200px.png?1706018312"
  },
  {
    "chainId": 42161,
    "address": "0x7ddf25cb4861590578f1fb85fcf1c5afd00a01de",
    "name": "PEPER",
    "symbol": "PEPER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30603/thumb/peper_logo.jpg?1696529473"
  },
  {
    "chainId": 42161,
    "address": "0x42972edecd94bdd19a622a6a419bdded2de56e08",
    "name": "Wonderly Finance",
    "symbol": "AFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28973/thumb/afx.png?1696527946"
  },
  {
    "chainId": 42161,
    "address": "0x077bc655c8d81b7de71c79318d0eb8f987e12820",
    "name": "Yuri",
    "symbol": "YURI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31032/thumb/Yuri_logo_200x200.jpg?1696529868"
  },
  {
    "chainId": 42161,
    "address": "0x1f6e4b5ffc94cca08cf6bb1479148d6329d4baf5",
    "name": "Ether Wars",
    "symbol": "WAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30665/thumb/l_logo_blue.png?1696529535"
  },
  {
    "chainId": 42161,
    "address": "0x771146816a0c7d74daf652252d646ab0bff7c21a",
    "name": "Wheat",
    "symbol": "WHEAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29190/thumb/LBdDnLA__400x400.jpg?1696528149"
  },
  {
    "chainId": 42161,
    "address": "0x375da692c2743e3aa18c51fdce139d08490bfa42",
    "name": "Bitbama",
    "symbol": "BAMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36686/thumb/photo_2024-03-31_14-20-41.jpg?1712073912"
  },
  {
    "chainId": 42161,
    "address": "0x76ce14237110c865f431e18f91fc1b225fb6fe99",
    "name": "TraderDAO Proof Of Trade",
    "symbol": "POT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30493/thumb/TraderDAO-POT-Coin_%282%29.png?1696529380"
  },
  {
    "chainId": 42161,
    "address": "0x6d9d8b50a0cccdf510f681a85472626c26377876",
    "name": "Quantum Chaos",
    "symbol": "CHAOS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32973/thumb/QUANTUM_CHAOS.jpg?1700068286"
  },
  {
    "chainId": 42161,
    "address": "0xd5954c3084a1ccd70b4da011e67760b8e78aee84",
    "name": "Arbidex",
    "symbol": "ARX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29506/thumb/tokenlogo.png?1696528451"
  },
  {
    "chainId": 42161,
    "address": "0x93c15cd7de26f07265f0272e0b831c5d7fab174f",
    "name": "Liquid Finance",
    "symbol": "LIQD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27056/thumb/liqd.png?1696526107"
  },
  {
    "chainId": 42161,
    "address": "0x43df266501dff4773f8f33179e3b96ab94dbc28d",
    "name": "GLend",
    "symbol": "GLEND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34850/thumb/GLend_Logo.png?1706261233"
  },
  {
    "chainId": 42161,
    "address": "0xfbbb21d8e7a461f06e5e27efd69703acb5c732a8",
    "name": "Kunji Finance",
    "symbol": "KNJ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31651/thumb/logo_mark_Main_%283%29.png?1696530466"
  },
  {
    "chainId": 42161,
    "address": "0x4a7779abed707a9c7deadbbef5c15f3e52370a99",
    "name": "Flame",
    "symbol": "FLAME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34280/thumb/20240104-164053.jpg?1704452348"
  },
  {
    "chainId": 42161,
    "address": "0xb49b6a3fd1f4bb510ef776de7a88a9e65904478a",
    "name": "Arbitrove ALP",
    "symbol": "ALP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30769/thumb/alp.jpg?1696529637"
  },
  {
    "chainId": 42161,
    "address": "0x6bb7a17acc227fd1f6781d1eedeae01b42047ee0",
    "name": "LEXER Markets",
    "symbol": "LEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29888/thumb/LEX.png?1696528812"
  },
  {
    "chainId": 42161,
    "address": "0xd1c533a00548dd9c1e7b0f8ea834f65383b116de",
    "name": "MetaPocket",
    "symbol": "MPCKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31163/thumb/android-chrome-192x192.png?1696529992"
  },
  {
    "chainId": 42161,
    "address": "0xd56734d7f9979dd94fae3d67c7e928234e71cd4c",
    "name": "Bridged TIA  Hyperlane ",
    "symbol": "TIAN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33199/thumb/tia.n.png?1701047832"
  },
  {
    "chainId": 42161,
    "address": "0xb5a628803ee72d82098d4bcaf29a42e63531b441",
    "name": "Unstoppable DeFi",
    "symbol": "UND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29819/thumb/cglogocen.png?1707169718"
  },
  {
    "chainId": 42161,
    "address": "0xb0ecc6ac0073c063dcfc026ccdc9039cae2998e1",
    "name": "A3S",
    "symbol": "AA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32133/thumb/A3S.jpg?1696588511"
  },
  {
    "chainId": 42161,
    "address": "0x22ab6bc52cad84aae84bd5781258677a6e0cfab0",
    "name": "ELSD Coin",
    "symbol": "ELSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30746/thumb/Icon-200.png?1696529616"
  },
  {
    "chainId": 42161,
    "address": "0x17afc249db786fc300f2d26381fec5cf0c3913fb",
    "name": "Shiba Cartel",
    "symbol": "PESOS",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29512/thumb/Shiba_cartel.jpeg?1696528456"
  },
  {
    "chainId": 42161,
    "address": "0x51f6ee60108cbcb3b613093bd3f224cb49aa1610",
    "name": "Brightpool",
    "symbol": "BRI",
    "decimals": 9
  },
  {
    "chainId": 42161,
    "address": "0x244ae62439c1ef3187f244d8604ac2c391ef2b53",
    "name": "Modular Wallet",
    "symbol": "MOD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29267/thumb/Captura_de_Pantalla_2023-03-01_a_la%28s%29_14.22.57.png?1696528220"
  },
  {
    "chainId": 42161,
    "address": "0x1bebd15adbc64f2f1eea7c3604d7a4b13a28c89c",
    "name": "Dao Glas",
    "symbol": "DGS",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/34822/thumb/daoglas.jpg?1706189265"
  },
  {
    "chainId": 42161,
    "address": "0x5b6424769823e82a1829b0a8bcaf501bffd90d25",
    "name": "Dinari PYPL",
    "symbol": "PYPLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32426/thumb/token-icon.png?1698119995"
  },
  {
    "chainId": 42161,
    "address": "0x13461c85887e85fdc942ac94c4d2699995ad1960",
    "name": "Cradles",
    "symbol": "CRDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33084/thumb/cradles_icon.png?1700601703"
  },
  {
    "chainId": 42161,
    "address": "0x1b8d516e2146d7a32aca0fcbf9482db85fd42c3a",
    "name": "AlphaScan AI",
    "symbol": "ASCN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31180/thumb/AlphaScan_logo.jpg?1696530008"
  },
  {
    "chainId": 42161,
    "address": "0x8ffdf2de812095b1d19cb146e4c004587c0a0692",
    "name": "Aave v3 LUSD",
    "symbol": "ALUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32894/thumb/LUSD.png?1699789893"
  },
  {
    "chainId": 42161,
    "address": "0x8c6bd546fb8b53fe371654a0e54d7a5bd484b319",
    "name": "Goat Protocol",
    "symbol": "GOA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35982/thumb/256x256_Main_Logo_-_Rounded.png?1710338882"
  },
  {
    "chainId": 42161,
    "address": "0x6609be1547166d1c4605f3a243fdcff467e600c3",
    "name": "Neutra Finance",
    "symbol": "NEU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28813/thumb/NEU_Token_logo.png?1696527789"
  },
  {
    "chainId": 42161,
    "address": "0x5fd71280b6385157b291b9962f22153fc9e79000",
    "name": "Garbi Protocol",
    "symbol": "GRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29142/thumb/garbi.png?1696528102"
  },
  {
    "chainId": 42161,
    "address": "0xaae0c3856e665ff9b3e2872b6d75939d810b7e40",
    "name": "YieldFarming Index",
    "symbol": "YFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29399/thumb/l1v2_transparent.png?1696528349"
  },
  {
    "chainId": 42161,
    "address": "0x577fd586c9e6ba7f2e85e025d5824dbe19896656",
    "name": "Synonym Finance",
    "symbol": "SYNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35065/thumb/SYNO-token.png?1707241258"
  },
  {
    "chainId": 42161,
    "address": "0x05eaea39f69b24f8f2da13af2d8ee0853889f2a8",
    "name": "CygnusDAO",
    "symbol": "CYG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33521/thumb/cyg.png?1702133561"
  },
  {
    "chainId": 42161,
    "address": "0x13a7dedb7169a17be92b0e3c7c2315b46f4772b3",
    "name": "Boop",
    "symbol": "BOOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33874/thumb/Boop_resized.png?1703144302"
  },
  {
    "chainId": 42161,
    "address": "0xd74f5255d557944cf7dd0e45ff521520002d5748",
    "name": "Sperax USD",
    "symbol": "USDS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21973/thumb/USDs_logo_1000X1000.png?1696521321"
  },
  {
    "chainId": 42161,
    "address": "0xab10645297d5b2b124523b5181df2e8815190aeb",
    "name": "Forge",
    "symbol": "FORGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29768/thumb/logo_200x200.png?1696528699"
  },
  {
    "chainId": 42161,
    "address": "0xbbea044f9e7c0520195e49ad1e561572e7e1b948",
    "name": "Mizar",
    "symbol": "MZR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23272/thumb/V1DTptkT_400x400.jpg?1696522492"
  },
  {
    "chainId": 42161,
    "address": "0x743b077b103d91109082a7a48e19ffc1093137a5",
    "name": "CharacterAI",
    "symbol": "CHAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29473/thumb/Ai200px.png?1696528418"
  },
  {
    "chainId": 42161,
    "address": "0x67bad479f77488f0f427584e267e66086a7da43a",
    "name": "Dinari ARM",
    "symbol": "ARMD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32982/thumb/Frame_16189.png?1700091554"
  },
  {
    "chainId": 42161,
    "address": "0xb4e3eeec43ebc04f28d31f2c021d15e9f56f0e54",
    "name": "MAW",
    "symbol": "MAW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31152/thumb/MAW._LOGO.png?1696529980"
  },
  {
    "chainId": 42161,
    "address": "0x32e4d98d3010ac12d75019c484caa78665b03986",
    "name": "GBOT",
    "symbol": "GBOT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31429/thumb/fW3tA1ZL_400x400.jpg?1696530244"
  },
  {
    "chainId": 42161,
    "address": "0x93ca0d85837ff83158cd14d65b169cdb223b1921",
    "name": "Eclipse Fi",
    "symbol": "ECLIP",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31815/thumb/ECLIP_token_logo.png?1696530629"
  },
  {
    "chainId": 42161,
    "address": "0x1f52145666c862ed3e2f1da213d479e61b2892af",
    "name": "Funny Coin",
    "symbol": "FUC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30081/thumb/fuc.png?1696529005"
  },
  {
    "chainId": 42161,
    "address": "0xfa58c669b855b29f99374d0f160db286849d139b",
    "name": "MonkeyCoin",
    "symbol": "MKC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36759/thumb/MonkeyCoin_Logo_256x256.png?1712247022"
  },
  {
    "chainId": 42161,
    "address": "0xc7d91ec2b29cd22c45c15760abc189c4e78e8036",
    "name": "Uramaki",
    "symbol": "MAKI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28909/thumb/Maki_Logo.png?1696527885"
  },
  {
    "chainId": 42161,
    "address": "0x29c1ea5ed7af53094b1a79ef60d20641987c867e",
    "name": "Acid",
    "symbol": "ACID",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29418/thumb/acid_200_200.png?1696528367"
  },
  {
    "chainId": 42161,
    "address": "0xb56c29413af8778977093b9b4947efeea7136c36",
    "name": "Torque",
    "symbol": "TORQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33549/thumb/torque-200px.png?1702389376"
  },
  {
    "chainId": 42161,
    "address": "0x118346c2bb9d24412ed58c53bf9bb6f61a20d7ec",
    "name": "Dinari PLD",
    "symbol": "PLDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32980/thumb/Frame_16189.png?1700091507"
  },
  {
    "chainId": 42161,
    "address": "0x3b475f6f2f41853706afc9fa6a6b8c5df1a2724c",
    "name": "Zyberswap",
    "symbol": "ZYB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28943/thumb/logo_with_bg.png?1696527917"
  },
  {
    "chainId": 42161,
    "address": "0x1d987200df3b744cfa9c14f713f5334cb4bc4d5d",
    "name": "REKT",
    "symbol": "REKT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29954/thumb/New_Project_%288%29.png?1696528881"
  },
  {
    "chainId": 42161,
    "address": "0xfc675adfdd721064ba923d07a8a238a9e52d8ace",
    "name": "escrowed PRF",
    "symbol": "ESPRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36646/thumb/esprf_200x200.png?1712037087"
  },
  {
    "chainId": 42161,
    "address": "0x13a78809528b02ad5e7c42f39232d332761dfb1d",
    "name": "Parifi WETH",
    "symbol": "PFWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36644/thumb/200x200weth.png?1712339446"
  },
  {
    "chainId": 42161,
    "address": "0xb21be1caf592a5dc1e75e418704d1b6d50b0d083",
    "name": "MIND Games CORTEX",
    "symbol": "CRX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29446/thumb/CRX200x200.png?1696528393"
  },
  {
    "chainId": 42161,
    "address": "0x34772c4d12f288368aa35ae7bc527a6b2b3e8906",
    "name": "Monopoly Layer 3 POLY",
    "symbol": "POLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30758/thumb/ZkdFe8ym_400x400.jpg?1696529627"
  },
  {
    "chainId": 42161,
    "address": "0x4945970efeec98d393b4b979b9be265a3ae28a8b",
    "name": "GMD",
    "symbol": "GMD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28088/thumb/gmd.png?1696527098"
  },
  {
    "chainId": 42161,
    "address": "0x05d35769a222affd6185e20f3f3676abde56c25f",
    "name": "Unlimited Network Token",
    "symbol": "UWU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30829/thumb/_UWU_new_logo.png?1696529682"
  },
  {
    "chainId": 42161,
    "address": "0x306634bd3a7982afdb42dec1c6b15176cd9e02e0",
    "name": "CEX Index",
    "symbol": "CEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32492/thumb/CEX.jpg?1698293422"
  },
  {
    "chainId": 42161,
    "address": "0xde70aed3d14d39b4955147efcf272334bdb75ab5",
    "name": "YachtingVerse",
    "symbol": "YACHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32858/thumb/aMmNwBTH_400x400.jpg?1699666370"
  },
  {
    "chainId": 42161,
    "address": "0x2f27118e3d2332afb7d165140cf1bb127ea6975d",
    "name": "Gridex",
    "symbol": "GDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29182/thumb/gridex-token.png?1696528140"
  },
  {
    "chainId": 42161,
    "address": "0xf50874f8246776ca4b89eef471e718f70f38458f",
    "name": "Arbswap",
    "symbol": "ARBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30565/thumb/Arb-Logo_Circle_DARK_LOGO_ONLY.png?1696529436"
  },
  {
    "chainId": 42161,
    "address": "0x4810e5a7741ea5fdbb658eda632ddfac3b19e3c6",
    "name": "Arbitrum Ecosystem Index",
    "symbol": "ARBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32042/thumb/Frame_2881_%281%29.png?1696530839"
  },
  {
    "chainId": 42161,
    "address": "0xf8388c2b6edf00e2e27eef5200b1befb24ce141d",
    "name": "Nola",
    "symbol": "NOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33708/thumb/nola.jpg?1702854587"
  },
  {
    "chainId": 42161,
    "address": "0xc701f86e6d242b97d3035f6c2cecaf8e7087e898",
    "name": "RunBlox  Arbitrum ",
    "symbol": "RUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29766/thumb/sh5z8C-0_400x400.jpg?1696528697"
  },
  {
    "chainId": 42161,
    "address": "0x6f9590958ce2beaf9c92a3a8fca6d1ddf310e052",
    "name": "Gas Turbo",
    "symbol": "GAST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35348/thumb/Xg34kjR0_400x400.png?1708321178"
  },
  {
    "chainId": 42161,
    "address": "0xa15907f3e7fdd0a83ec56f4ac543cb10f4d783eb",
    "name": "Kewl",
    "symbol": "KEWL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30534/thumb/Group_599.png?1696529407"
  },
  {
    "chainId": 42161,
    "address": "0x954ac1c73e16c77198e83c088ade88f6223f3d44",
    "name": "LeverageInu",
    "symbol": "LEVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28147/thumb/IMG_20221107_091938_841.png?1696527152"
  },
  {
    "chainId": 42161,
    "address": "0xf7728582002ef82908c8242cf552e969ba863ffa",
    "name": "Savvy ETH",
    "symbol": "SVETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31005/thumb/logo_%288%29.png?1696529842"
  },
  {
    "chainId": 42161,
    "address": "0x9578a9937dff45b4e29e49120ab83cb806f1aa4f",
    "name": "AITom",
    "symbol": "AITOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32013/thumb/aitom.png?1696530811"
  },
  {
    "chainId": 42161,
    "address": "0x0b63c61bba4a876a6eb8b5e596800f7649a9b71e",
    "name": "Sector",
    "symbol": "SECT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29672/thumb/ZVLrbTWD_400x400.jpg?1696528607"
  },
  {
    "chainId": 42161,
    "address": "0x8b0e6f19ee57089f7649a455d89d7bc6314d04e8",
    "name": "Dream Machine Token",
    "symbol": "DMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30505/thumb/dmt.png?1696529391"
  },
  {
    "chainId": 42161,
    "address": "0x15b6ec24f59fea164c6e235941aa00fb0d4a32f6",
    "name": "Lumi Finance LUA Option",
    "symbol": "LUAOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33507/thumb/LUAOP.png?1702039814"
  },
  {
    "chainId": 42161,
    "address": "0x0a1694716de67c98f61942b2cab7df7fe659c87a",
    "name": "Crescentswap Moonlight",
    "symbol": "MNLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30687/thumb/Untitled_design_%282%29.png?1696529556"
  },
  {
    "chainId": 42161,
    "address": "0xe4421566a501045ae4285996577a36f6cf074190",
    "name": "Ice",
    "symbol": "ICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35093/thumb/ICE_LOGO.jpg?1707363813"
  },
  {
    "chainId": 42161,
    "address": "0x6d80113e533a2c0fe82eabd35f1875dcea89ea97",
    "name": "Aave v3 EURS",
    "symbol": "AEURS",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/32913/thumb/eurs.png?1699824583"
  },
  {
    "chainId": 42161,
    "address": "0xe9a5af50874c0ef2748b5db70104b5ccb5557f6d",
    "name": "GMBL COMPUTER CHiP",
    "symbol": "GMBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31659/thumb/GMBL.png?1696530474"
  },
  {
    "chainId": 42161,
    "address": "0x910d3c72c5177c3f1bfb0863b793ec23fa7f6990",
    "name": "NitroShiba",
    "symbol": "NISHIB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27288/thumb/nishib_logo.png?1696526340"
  },
  {
    "chainId": 42161,
    "address": "0x83d6c8c06ac276465e4c92e7ac8c23740f435140",
    "name": "HMX",
    "symbol": "HMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31206/thumb/HMXlogo_CG.png?1696530033"
  },
  {
    "chainId": 42161,
    "address": "0xd07d35368e04a839dee335e213302b21ef14bb4a",
    "name": "Crystal",
    "symbol": "CRYSTAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35099/thumb/CRYSTAL.jpg?1707412769"
  },
  {
    "chainId": 42161,
    "address": "0x9c8ef67c9168587134e1f29a71405dc753500a45",
    "name": "Legends Token",
    "symbol": "LG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30079/thumb/004_%281%29.png?1696529003"
  },
  {
    "chainId": 42161,
    "address": "0x654c908305021b2eaf881cee774ece1d2bcac5fc",
    "name": "GMCash",
    "symbol": "GMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29722/thumb/Token.png?1696528653"
  },
  {
    "chainId": 42161,
    "address": "0x0f96d8c1277bd75a251238af952a7a99db1320e3",
    "name": "GMCash Share",
    "symbol": "GSHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29730/thumb/logoABS.ac04548d_2.png?1696528660"
  },
  {
    "chainId": 42161,
    "address": "0xbb5b3d9f6b57077b4545ea9879ee7fd0bdb08db0",
    "name": "Sharp Portfolio Index",
    "symbol": "SPI",
    "decimals": 17,
    "logoURI": "https://assets.coingecko.com/coins/images/34936/thumb/spi_eclipse.png?1706720688"
  },
  {
    "chainId": 42161,
    "address": "0xec13336bbd50790a00cdc0feddf11287eaf92529",
    "name": "gmUSD",
    "symbol": "GMUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30345/thumb/Logo.jpg?1696529245"
  },
  {
    "chainId": 42161,
    "address": "0x6bcc14b02cd624ebe1a8a665cb6d4067aa097230",
    "name": "Foxify",
    "symbol": "FOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32354/thumb/Foxify_200x200.png?1697475371"
  },
  {
    "chainId": 42161,
    "address": "0xfb9fed8cb962548a11fe7f6f282949061395c7f5",
    "name": "Nuon",
    "symbol": "NUON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29739/thumb/Nuon_image.jpg?1696528669"
  },
  {
    "chainId": 42161,
    "address": "0xf34450d1f23902657cffb2636153677be7d38750",
    "name": "Equinox Ecosystem",
    "symbol": "NOX",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35266/thumb/ex_logo_coin.png?1708012097"
  },
  {
    "chainId": 42161,
    "address": "0x7e325091e5525d3ea4d54a1488ecca8d1df732f3",
    "name": "Pacman Native Token",
    "symbol": "PAC",
    "decimals": 18
  },
  {
    "chainId": 42161,
    "address": "0x2c7941a0fe9c52223b229747322af16160161c98",
    "name": "Jarvis",
    "symbol": "JARVIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35812/thumb/jarvis.png?1709873633"
  },
  {
    "chainId": 42161,
    "address": "0xd67a097dce9d4474737e6871684ae3c05460f571",
    "name": "GND Protocol",
    "symbol": "GND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30043/thumb/image-removebg-preview_%281%29.png?1696528965"
  },
  {
    "chainId": 42161,
    "address": "0x56b251d4b493ee3956e3f899d36b7290902d2326",
    "name": "MMFinance  Arbitrum ",
    "symbol": "MMF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29729/thumb/MMF_Arb_org.png?1696528659"
  },
  {
    "chainId": 42161,
    "address": "0x2ef354c71caab6dc7469bb3c99642878ccd1143f",
    "name": "Grail Inu",
    "symbol": "IGRAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33487/thumb/iGRAIL_Ticker_LOGO_200_by_200.png?1702028919"
  },
  {
    "chainId": 42161,
    "address": "0x4117ec0a779448872d3820f37ba2060ae0b7c34b",
    "name": "USDEX ",
    "symbol": "USDEX+",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30670/thumb/USDEX_-icon-200x200.png?1696529540"
  },
  {
    "chainId": 42161,
    "address": "0xe018c227bc84e44c96391d3067fab5a9a46b7e62",
    "name": "Chromium Dollar",
    "symbol": "CR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28310/thumb/Chromium.png?1696527315"
  },
  {
    "chainId": 42161,
    "address": "0x9ed7e4b1bff939ad473da5e7a218c771d1569456",
    "name": "Reunit Wallet",
    "symbol": "REUNI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29492/thumb/reunit_200.png?1696528436"
  },
  {
    "chainId": 42161,
    "address": "0xe1d3495717f9534db67a6a8d4940dd17435b6a9e",
    "name": "Locus Finance",
    "symbol": "LOCUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33441/thumb/locus_finance_logo.jpeg?1701854209"
  },
  {
    "chainId": 42161,
    "address": "0x1a5b0aaf478bf1fda7b934c76e7692d722982a6d",
    "name": "Buffer Token",
    "symbol": "BFR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18540/thumb/Qk6pjeZ3_400x400.jpg?1696518020"
  },
  {
    "chainId": 42161,
    "address": "0x0ae1bb2bc04308765a6b1215236cea8cfee8cab9",
    "name": "TDEX Token",
    "symbol": "TT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32725/thumb/20220929-180435.png?1699018453"
  },
  {
    "chainId": 42161,
    "address": "0x73db621cc8f0a767bed0e5c0a2920f480d77b765",
    "name": "GTA Token",
    "symbol": "GTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35245/thumb/200%D1%85200.png?1707987797"
  },
  {
    "chainId": 42161,
    "address": "0xcb6460d56825ddc12229c7a7d94b6b26a9f9c867",
    "name": "DefiTankLand",
    "symbol": "DFTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29136/thumb/tank_%282%29.png?1696528097"
  },
  {
    "chainId": 42161,
    "address": "0x4568ca00299819998501914690d6010ae48a59ba",
    "name": "Army of Fortune Metaverse",
    "symbol": "AFC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/32577/thumb/UI_Icon_Coin_Token_AFC_200x200.png?1698558434"
  },
  {
    "chainId": 42161,
    "address": "0xfb17d52f77db6e32b5a082ed4307fcfb0a86beee",
    "name": "Pizon",
    "symbol": "PZT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30120/thumb/pizon.jpeg?1696529042"
  },
  {
    "chainId": 42161,
    "address": "0xe4177c1400a8eee1799835dcde2489c6f0d5d616",
    "name": "CrazyRabbit",
    "symbol": "CRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33468/thumb/logo-no-background.png?1701943008"
  },
  {
    "chainId": 42161,
    "address": "0x640278bada847b7ce71bb22f20517a009a049640",
    "name": "GoSleep NGT",
    "symbol": "NGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29826/thumb/NGT200_200.png?1696528754"
  },
  {
    "chainId": 42161,
    "address": "0x3106e2e148525b3db36795b04691d444c24972fb",
    "name": "Wrapped Staked LINK",
    "symbol": "WSTLINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35678/thumb/wstlink_logo.png?1709525182"
  },
  {
    "chainId": 42161,
    "address": "0x20547341e58fb558637fa15379c92e11f7b7f710",
    "name": "Mozaic",
    "symbol": "MOZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30100/thumb/Main_Logo_1-200x200jpg.jpg?1696529024"
  },
  {
    "chainId": 42161,
    "address": "0x463913d3a3d3d291667d53b8325c598eb88d3b0e",
    "name": "SolidLizard",
    "symbol": "SLIZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28968/thumb/sliz-logo.png?1696527941"
  },
  {
    "chainId": 42161,
    "address": "0x1dd6b5f9281c6b4f043c02a83a46c2772024636c",
    "name": "Lumi Finance LUAUSD",
    "symbol": "LUAUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33503/thumb/LUAUSD.png?1702038709"
  },
  {
    "chainId": 42161,
    "address": "0x643b34980e635719c15a2d4ce69571a258f940e9",
    "name": "The Standard EURO",
    "symbol": "EUROS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32231/thumb/EUROs-coingecko.png?1696936278"
  },
  {
    "chainId": 42161,
    "address": "0xfd086bc7cd5c481dcc9c85ebe478a1c0b69fcbb9",
    "name": "Arbitrum Bridged USDT  Arbitrum ",
    "symbol": "USDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35073/thumb/logo.png?1707292836"
  },
  {
    "chainId": 42161,
    "address": "0x1922c36f3bc762ca300b4a46bb2102f84b1684ab",
    "name": "Compounded Marinated UMAMI",
    "symbol": "CMUMAMI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28433/thumb/Cqx8QjSL_400x400.jpeg?1696527429"
  },
  {
    "chainId": 42161,
    "address": "0x000d636bd52bfc1b3a699165ef5aa340bea8939c",
    "name": "Open Dollar Governance",
    "symbol": "ODG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34689/thumb/ODG_Token_200_x_200.png?1705853107"
  },
  {
    "chainId": 42161,
    "address": "0x000000000026839b3f4181f2cf69336af6153b99",
    "name": "Reboot",
    "symbol": "GG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31228/thumb/gg_cyan_black_square.png?1703749951"
  },
  {
    "chainId": 42161,
    "address": "0xd29f8a9e76ef42ba9f749583ed07cae7bfaec389",
    "name": "opxSliz",
    "symbol": "OPXVESLIZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29291/thumb/62833FE7-F28E-4D33-886E-9E4F8D72EEB7.png?1696528243"
  },
  {
    "chainId": 42161,
    "address": "0x358e5d2378f5fc3fa5504aebb728c4c568f973a4",
    "name": "SpaceAI Finance",
    "symbol": "SPAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33563/thumb/20231211_004306.png?1702443934"
  },
  {
    "chainId": 42161,
    "address": "0xf1a82bfa7fceb8b8741e7e04a6b8efd348ca6393",
    "name": "ArbShib",
    "symbol": "AISHIB",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29945/thumb/200200.jpg?1696528872"
  },
  {
    "chainId": 42161,
    "address": "0x40301951af3f80b8c1744ca77e55111dd3c1dba1",
    "name": "The Ennead",
    "symbol": "NEADRAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29748/thumb/Ennead.png?1696528680"
  },
  {
    "chainId": 42161,
    "address": "0x14e8687af6c94c7c68590bacf0f1d1e338f2862b",
    "name": "Firepot Finance",
    "symbol": "HOTT",
    "decimals": 18
  },
  {
    "chainId": 42161,
    "address": "0xd77b108d4f6cefaa0cae9506a934e825becca46e",
    "name": "WINR Protocol",
    "symbol": "WINR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29340/thumb/WINR.png?1696528290"
  },
  {
    "chainId": 42161,
    "address": "0x2389f6a46562ae5f1557db8562c39ef553f3832b",
    "name": "KERC",
    "symbol": "KERC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35976/thumb/KERC_Logo_1.png?1710306026"
  },
  {
    "chainId": 42161,
    "address": "0x322f15d4bedaa20c178fb75b2628663d2da19736",
    "name": "Monopoly Layer2 DUO",
    "symbol": "DUO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29843/thumb/DUO_ticker_200x200.png?1696528770"
  },
  {
    "chainId": 42161,
    "address": "0x509fd25ee2ac7833a017f17ee8a6fb4aaf947876",
    "name": "Magpie WOM",
    "symbol": "MWOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29924/thumb/mWOM.png?1696528852"
  },
  {
    "chainId": 42161,
    "address": "0x539bde0d7dbd336b79148aa742883198bbf60342",
    "name": "Magic",
    "symbol": "MAGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18623/thumb/magic.png?1696518095"
  },
  {
    "chainId": 42161,
    "address": "0xee9857de0e55d4a54d36a5a5a73a15e57435fdca",
    "name": "AsgardX",
    "symbol": "ODIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29157/thumb/IMG_20230222_053621_351.jpg?1696528116"
  },
  {
    "chainId": 42161,
    "address": "0x964cbf436ddb782d09465930ce8b431311cb54f4",
    "name": "Pingu Exchange",
    "symbol": "PINGU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34790/thumb/icon_primary.png?1706030846"
  },
  {
    "chainId": 42161,
    "address": "0x36d37b6cbca364cf1d843eff8c2f6824491bcf81",
    "name": "Dinari TSLA",
    "symbol": "TSLAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32536/thumb/token-icon.png?1698469155"
  },
  {
    "chainId": 42161,
    "address": "0x154388a4650d63acc823e06ef9e47c1eddd3cbb2",
    "name": "Seneca",
    "symbol": "SEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32852/thumb/200x200.png?1699663676"
  },
  {
    "chainId": 42161,
    "address": "0xc29b9c4f45fd4f07768cf913acde0c7772b47124",
    "name": "Post Tech",
    "symbol": "POST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34260/thumb/bird-logo-color.png?1704360008"
  },
  {
    "chainId": 42161,
    "address": "0x417a1afd44250314bffb11ff68e989775e990ab6",
    "name": "Volta Protocol",
    "symbol": "VOLTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29130/thumb/real_VoltaLogo.png?1696528091"
  },
  {
    "chainId": 42161,
    "address": "0xed7f000ee335b8199b004cca1c6f36d188cf6cb8",
    "name": "D2 Finance",
    "symbol": "D2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34914/thumb/D2_Token_Logo.png?1706610408"
  },
  {
    "chainId": 42161,
    "address": "0x088cd8f5ef3652623c22d48b1605dcfe860cd704",
    "name": "Vela Token",
    "symbol": "VELA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28739/thumb/VELA_logo_-_no_background.png?1696527719"
  },
  {
    "chainId": 42161,
    "address": "0x1824a51c106efc27d35a74efb56d9bf54ddb22d4",
    "name": "Perpy Finance",
    "symbol": "PRY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29525/thumb/perpy-icon-dark.png?1696528468"
  },
  {
    "chainId": 42161,
    "address": "0xea1e79d4a0d347edcadd78d98a7da65d15e8cba1",
    "name": "RocketVerse",
    "symbol": "RKV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30004/thumb/RVK.jpg?1696528929"
  },
  {
    "chainId": 42161,
    "address": "0xdb13df2ea134e7df2208d74b96db063837db5b5c",
    "name": "LAMB by OPNX",
    "symbol": "LAMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36164/thumb/IMG_0888.jpg?1710745203"
  },
  {
    "chainId": 42161,
    "address": "0x00cbcf7b3d37844e44b888bc747bdd75fcf4e555",
    "name": "xPet tech",
    "symbol": "XPET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33553/thumb/xpet_token.jpeg?1702428894"
  },
  {
    "chainId": 42161,
    "address": "0xccd3891c1452b7cb0e4632774b9365dc4ee24f20",
    "name": "El Dorado Exchange  Arb ",
    "symbol": "EDE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29386/thumb/Z6eU0fpI_400x400.jpg?1696528332"
  },
  {
    "chainId": 42161,
    "address": "0x09e18590e8f76b6cf471b3cd75fe1a1a9d2b2c2b",
    "name": "ArbDoge AI",
    "symbol": "AIDOGE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29852/thumb/photo_2023-04-18_14-25-28.jpg?1696528778"
  },
  {
    "chainId": 42161,
    "address": "0x9da913f4dca9b210a232d588113047685a4ed4b1",
    "name": "Dinari BRK A",
    "symbol": "BRKAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32979/thumb/Frame_16189.png?1700091475"
  },
  {
    "chainId": 42161,
    "address": "0xfc5a1a6eb076a2c7ad06ed22c90d7e710e35ad0a",
    "name": "GMX",
    "symbol": "GMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18323/thumb/arbit.png?1696517814"
  },
  {
    "chainId": 42161,
    "address": "0x9c7049d2ddbda7df5da9a9069926f517d2d9e6ff",
    "name": "X Travel Space",
    "symbol": "XTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33370/thumb/t%E1%BA%A3i_xu%E1%BB%91ng.png?1701619978"
  },
  {
    "chainId": 42161,
    "address": "0xc8ccbd97b96834b976c995a67bf46e5754e2c48e",
    "name": "Parallax",
    "symbol": "PLX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31536/thumb/Parallax_round_logo_-_transparent_bg.png?1696530345"
  },
  {
    "chainId": 42161,
    "address": "0xf236ea74b515ef96a9898f5a4ed4aa591f253ce1",
    "name": "Plutus DPX",
    "symbol": "PLSDPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29652/thumb/plsDPX.png?1696528588"
  },
  {
    "chainId": 42161,
    "address": "0xd41f1f0cf89fd239ca4c1f8e8ada46345c86b0a4",
    "name": "Fiat24 CHF",
    "symbol": "CHF24",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/33296/thumb/CHF24.png?1701402516"
  },
  {
    "chainId": 42161,
    "address": "0x5117f4ad0bc70dbb3b05bf39a1ec1ee40dd67654",
    "name": "Avive",
    "symbol": "AVIVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33697/thumb/avive_token.png?1702814979"
  },
  {
    "chainId": 42161,
    "address": "0x847db21ef9de627c23d3dff470af6599b1a30d3f",
    "name": "Black Rabbit AI",
    "symbol": "BRAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28966/thumb/200x200.png?1696527939"
  },
  {
    "chainId": 42161,
    "address": "0x2e516ba5bf3b7ee47fb99b09eadb60bde80a82e0",
    "name": "aEGGS",
    "symbol": "AEGGS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29520/thumb/aeggs.png?1696528463"
  },
  {
    "chainId": 42161,
    "address": "0xd3188e0df68559c0b63361f6160c57ad88b239d8",
    "name": "Astra DAO",
    "symbol": "ASTRADAO",
    "decimals": 17,
    "logoURI": "https://assets.coingecko.com/coins/images/32870/thumb/ASTRADAO.jpg?1699673097"
  },
  {
    "chainId": 42161,
    "address": "0x5db7b150c5f38c5f5db11dcbdb885028fcc51d68",
    "name": "Sterling Finance",
    "symbol": "STR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29167/thumb/FullLogo_Transparent_NoBuffer.png?1696528125"
  },
  {
    "chainId": 42161,
    "address": "0x8240affe697cde618ad05c3c8963f5bfe152650b",
    "name": "Dinari AMZN",
    "symbol": "AMZND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32529/thumb/token-icon.png?1698460876"
  },
  {
    "chainId": 42161,
    "address": "0xb2498cde54810424a24fdd23579164265a0c14c5",
    "name": "Taikula Coin",
    "symbol": "TAIKULA",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30265/thumb/8MV4Pr9j_400x400.png?1696529173"
  },
  {
    "chainId": 42161,
    "address": "0x4dafffddea93ddf1e0e7b61e844331455053ce5c",
    "name": "Dinari NVDA",
    "symbol": "NVDAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32401/thumb/token-icon.png?1698055556"
  },
  {
    "chainId": 42161,
    "address": "0x7fb7ede54259cb3d4e1eaf230c7e2b1ffc951e9a",
    "name": "Numa",
    "symbol": "NUMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35807/thumb/numa200.png?1709845645"
  },
  {
    "chainId": 42161,
    "address": "0x655a6beebf2361a19549a99486ff65f709bd2646",
    "name": "LilAI",
    "symbol": "LILAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30621/thumb/LogoLilai_CoinGecko.png?1696529495"
  },
  {
    "chainId": 42161,
    "address": "0x2c110867ca90e43d372c1c2e92990b00ea32818b",
    "name": "Stabilize",
    "symbol": "STBZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12753/thumb/icon.png?1696512551"
  },
  {
    "chainId": 42161,
    "address": "0xfb1f65955e168e0ef500b170eed4a4efeb99ae32",
    "name": "LiquiCats",
    "symbol": "MEOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28810/thumb/AdZJegQB_400x400.jpg?1696527787"
  },
  {
    "chainId": 42161,
    "address": "0x67c31056358b8977ea95a3a899dd380d4bced706",
    "name": "ETHforestAI",
    "symbol": "ETHFAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29515/thumb/ethfai_logo.png?1696528459"
  },
  {
    "chainId": 42161,
    "address": "0x4d010dbe0c6cdc0571bba026d6dfabbf40c9af28",
    "name": "GemDrop",
    "symbol": "GEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32533/thumb/logogemdrop.png?1698461472"
  },
  {
    "chainId": 42161,
    "address": "0x3d9907f9a368ad0a51be60f7da3b97cf940982d8",
    "name": "Camelot Token",
    "symbol": "GRAIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28416/thumb/vj5DIMhP_400x400.jpeg?1696527414"
  },
  {
    "chainId": 42161,
    "address": "0xb1070c40d5b3d10e1f1c8876c3d2ff5ffed4b3ac",
    "name": "FengLvZiV2",
    "symbol": "FENGLVZIV2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30324/thumb/logo.png?1696529225"
  },
  {
    "chainId": 42161,
    "address": "0x903ca00944d0b51e50d9f4fc96167c89f211542a",
    "name": "Poolshark",
    "symbol": "FIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33964/thumb/finTokenLogo_1_%281%29.png?1703570409"
  },
  {
    "chainId": 42161,
    "address": "0x54c1b12da9594c7cd8fc9a172bb2cc6f83ecf6d4",
    "name": "Speculate DAO",
    "symbol": "SPEC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33617/thumb/Goin_Gecko.png?1702515106"
  },
  {
    "chainId": 42161,
    "address": "0x2c7513f299778a84797f293503db1a73a15fd48d",
    "name": "OpenbetAI",
    "symbol": "OPENBET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29774/thumb/openbetlogo200x200.png?1696528705"
  },
  {
    "chainId": 42161,
    "address": "0xaa54e84a3e6e5a80288d2c2f8e36ea5ca3a3ca30",
    "name": "Sharbi",
    "symbol": "SHARBI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29075/thumb/Sharbi_New_Logo_200x200.png?1712686416"
  },
  {
    "chainId": 42161,
    "address": "0x982239d38af50b0168da33346d85fb12929c4c07",
    "name": "Arbitrove Governance Token",
    "symbol": "TROVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29017/thumb/trove.png?1696527988"
  },
  {
    "chainId": 42161,
    "address": "0x500756c7d239aee30f52c7e52af4f4f008d1a98f",
    "name": "Petroleum OIL",
    "symbol": "OIL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28350/thumb/OIL_200.png?1696527354"
  },
  {
    "chainId": 42161,
    "address": "0x580e933d90091b9ce380740e3a4a39c67eb85b4c",
    "name": "GameSwift",
    "symbol": "GSWIFT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30949/thumb/GameSwift_Token.png?1696529788"
  },
  {
    "chainId": 42161,
    "address": "0xb50a8e92cb9782c9b8f3c88e4ee8a1d0aa2221d7",
    "name": "Yak DAO",
    "symbol": "YAKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36102/thumb/photo_2023-03-24_10-37-37.jpg?1710483450"
  },
  {
    "chainId": 42161,
    "address": "0x296a0b8847bd4ed9af71a9ef238fa5be0778b611",
    "name": "Atlas Aggregator",
    "symbol": "ATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29570/thumb/1024.png?1696528510"
  },
  {
    "chainId": 42161,
    "address": "0xd8f728adb72a46ae2c92234ae8870d04907786c5",
    "name": "Dinari AMD",
    "symbol": "AMDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32981/thumb/Frame_16189.png?1700091539"
  },
  {
    "chainId": 42161,
    "address": "0x15b2fb8f08e4ac1ce019eadae02ee92aedf06851",
    "name": "Chronos Finance",
    "symbol": "CHR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29622/thumb/chronos_icono-01.png?1696528558"
  },
  {
    "chainId": 42161,
    "address": "0x3c9f23db4ddc5655f7be636358d319a3de1ff0c4",
    "name": "Dinari DIS",
    "symbol": "DISD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32538/thumb/token-icon.png?1698469167"
  },
  {
    "chainId": 42161,
    "address": "0x54355cc6913b26a15cca1f820cf17d362fa65db5",
    "name": "SolidLizard synthetic USD",
    "symbol": "SLZUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29900/thumb/slzUSDC2.png?1696528829"
  },
  {
    "chainId": 42161,
    "address": "0xcbe94d75ec713b7ead84f55620dc3174beeb1cfe",
    "name": "FORE Protocol",
    "symbol": "FORE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31001/thumb/EXCHANGE_LOGO_FORE.png?1696529839"
  },
  {
    "chainId": 42161,
    "address": "0x31c91d8fb96bff40955dd2dbc909b36e8b104dde",
    "name": "Poison Finance",
    "symbol": "POION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28630/thumb/poisonlogo160x160.png?1696527614"
  },
  {
    "chainId": 42161,
    "address": "0x033f193b3fceb22a440e89a2867e8fee181594d9",
    "name": "Rodeo Finance",
    "symbol": "RDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30905/thumb/RDO_ticker_logo_-_color-01-01.png?1696529750"
  },
  {
    "chainId": 42161,
    "address": "0xa684cd057951541187f288294a1e1c2646aa2d24",
    "name": "Vesta Finance",
    "symbol": "VSTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23306/thumb/vesta-finance.png?1696522523"
  },
  {
    "chainId": 42161,
    "address": "0xb4b07b60455a2f38cba98a8f3dd161f7ca396a9c",
    "name": "Onchain Trade Protocol",
    "symbol": "OT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29133/thumb/OTCMC.png?1696528094"
  },
  {
    "chainId": 42161,
    "address": "0x7c8a1a80fdd00c9cccd6ebd573e9ecb49bfa2a59",
    "name": "AI CODE",
    "symbol": "AICODE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30057/thumb/AICODE.png?1696528979"
  },
  {
    "chainId": 42161,
    "address": "0x635d0e13f98e107cf6c5cdfbf52c19843f87e76a",
    "name": "Dominator Domains",
    "symbol": "DOMDOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29731/thumb/IMG_20230406_230429_102.jpg?1696528661"
  },
  {
    "chainId": 42161,
    "address": "0x4cfa50b7ce747e2d61724fcac57f24b748ff2b2a",
    "name": "Fluid USDC",
    "symbol": "FUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28471/thumb/fUSDC-200x200.png?1696527465"
  },
  {
    "chainId": 42161,
    "address": "0x17eb7c08c4057b6c270dc0549745adbc874eb15b",
    "name": "Satoshi Airlines Token",
    "symbol": "SAT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/34036/thumb/IMG_20231225_194035_093.jpg?1703690482"
  },
  {
    "chainId": 42161,
    "address": "0x3a33473d7990a605a88ac72a78ad4efc40a54adb",
    "name": "Tigris",
    "symbol": "TIG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30641/thumb/logo.png?1696529513"
  },
  {
    "chainId": 42161,
    "address": "0x9c46e1b70d447b770dbfc8d450543a431af6df3a",
    "name": "Dinari USFR",
    "symbol": "USFRD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32402/thumb/token-icon.png?1698055558"
  },
  {
    "chainId": 42161,
    "address": "0x7a5d193fe4ed9098f7eadc99797087c96b002907",
    "name": "Plutus ARB",
    "symbol": "PLSARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30636/thumb/plsARB.png?1696529509"
  },
  {
    "chainId": 42161,
    "address": "0x6fc2680d8ad8e8312191441b4eca9eff8d06b45a",
    "name": "Artichoke",
    "symbol": "CHOKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30354/thumb/Choke_icon_-_Small.png?1696529254"
  },
  {
    "chainId": 42161,
    "address": "0x11f98c7e42a367dab4f200d2fdc460fb445ce9a8",
    "name": "SpartaDEX",
    "symbol": "SPARTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30836/thumb/SpartaDex_200x200.jpg?1696529694"
  },
  {
    "chainId": 42161,
    "address": "0xe66998533a1992ece9ea99cdf47686f4fc8458e0",
    "name": "Jones USDC",
    "symbol": "JUSDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28893/thumb/jUSDC.png?1696527869"
  },
  {
    "chainId": 42161,
    "address": "0x1fc01117e196800f416a577350cb1938d10501c2",
    "name": "Lumin",
    "symbol": "LUMIN",
    "decimals": 12,
    "logoURI": "https://assets.coingecko.com/coins/images/36594/thumb/Lumin_Logo_Circle_-_Balancer.png?1711949756"
  },
  {
    "chainId": 42161,
    "address": "0xf1f18f765f118c3598cc54dcac1d0e12066263fe",
    "name": "Dinari PFE",
    "symbol": "PFED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32537/thumb/token-icon.png?1698469161"
  },
  {
    "chainId": 42161,
    "address": "0x9eb6299e4bb6669e42cb295a254c8492f67ae2c6",
    "name": "The Bet",
    "symbol": "BET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29545/thumb/200.png?1696528486"
  },
  {
    "chainId": 42161,
    "address": "0x7f7d7806f4eb90d63b0b278daf32a2db2c2001bd",
    "name": "BonusBlock",
    "symbol": "BONUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36364/thumb/Coingecko_logo.png?1711336118"
  },
  {
    "chainId": 42161,
    "address": "0x352f4bf396a7353a0877f99e99757e5d294df374",
    "name": "Sundae the Dog",
    "symbol": "SUNDAE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32010/thumb/sundae.jpg?1696530808"
  },
  {
    "chainId": 42161,
    "address": "0xeca66820ed807c096e1bd7a1a091cd3d3152cc79",
    "name": "Ghast",
    "symbol": "GHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30497/thumb/gha.png?1696529383"
  },
  {
    "chainId": 42161,
    "address": "0xd26b0c6ef8581e921ae41c66e508c62a581b709d",
    "name": "Sexone",
    "symbol": "SEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32135/thumb/IMG_20231004_163306_491.jpg?1696589011"
  },
  {
    "chainId": 42161,
    "address": "0x02150e97271fdc0d6e3a16d9094a0948266f07dd",
    "name": "Hamachi Finance",
    "symbol": "HAMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28783/thumb/icon.png?1696527762"
  },
  {
    "chainId": 42161,
    "address": "0x92a212d9f5eef0b262ac7d84aea64a0d0758b94f",
    "name": "DexFi Governance",
    "symbol": "GDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30671/thumb/gDEX-icon-200x200.png?1696529541"
  },
  {
    "chainId": 42161,
    "address": "0x912ce59144191c1204e64559fe8253a0e49e6548",
    "name": "Arbitrum",
    "symbol": "ARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16547/thumb/photo_2023-03-29_21.47.00.jpeg?1696516109"
  },
  {
    "chainId": 42161,
    "address": "0xff970a61a04b1ca14834a43f5de4533ebddb5cc8",
    "name": "Bridged USDC  Arbitrum ",
    "symbol": "USDCE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30691/thumb/usdc.png?1696529560"
  },
  {
    "chainId": 42161,
    "address": "0x7a2c1b8e26c48a5b73816b7ec826fd4053f5f34b",
    "name": "GoSleep ZZZ",
    "symbol": "ZZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29901/thumb/ZZZ200_200.png?1696528830"
  },
  {
    "chainId": 42161,
    "address": "0xce38e140fc3982a6bcebc37b040913ef2cd6c5a7",
    "name": "Dinari AAPL",
    "symbol": "AAPLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32393/thumb/token-icon.png?1698053181"
  },
  {
    "chainId": 42161,
    "address": "0x53bcf6698c911b2a7409a740eacddb901fc2a2c6",
    "name": "Kabosu  Arbitrum ",
    "symbol": "KABOSU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30320/thumb/IMG_20230510_204814_554_copy_200x200.jpg?1696529222"
  },
  {
    "chainId": 42161,
    "address": "0x802124eb78e43fd8d3d4e6daaaa4be28dc7993dc",
    "name": "Spinaq",
    "symbol": "SPINAQ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33626/thumb/SPINAQ_200x200.png?1702532970"
  },
  {
    "chainId": 42161,
    "address": "0x73ed68b834e44096eb4bea6edead038c945722f1",
    "name": "Sharky Swap",
    "symbol": "SHARKY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29152/thumb/SharkySwapPNG.png?1696528111"
  },
  {
    "chainId": 42161,
    "address": "0x46b979440ac257151ee5a5bc9597b76386907fa1",
    "name": "Dinari COIN",
    "symbol": "COIND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32983/thumb/Frame_16189.png?1700091568"
  },
  {
    "chainId": 42161,
    "address": "0x7f90122bf0700f9e7e1f688fe926940e8839f353",
    "name": "Curve fi USDC USDT",
    "symbol": "2CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28365/thumb/curve2.png?1696527368"
  },
  {
    "chainId": 42161,
    "address": "0x816e21c33fa5f8440ebcdf6e01d39314541bea72",
    "name": "LiquidDriver",
    "symbol": "LQDR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15782/thumb/LQDR_Glowing_Icon.png?1696515405"
  },
  {
    "chainId": 42161,
    "address": "0xd3f1da62cafb7e7bc6531ff1cef6f414291f03d3",
    "name": "Doubloon",
    "symbol": "DBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23660/thumb/galleon-sky.png?1696522863"
  },
  {
    "chainId": 42161,
    "address": "0x1bd013bd089c2b6b2d30a0e0b545810a5844e761",
    "name": "OtterHome",
    "symbol": "HOME",
    "decimals": 16,
    "logoURI": "https://assets.coingecko.com/coins/images/30740/thumb/LOGO.png?1696529610"
  },
  {
    "chainId": 42161,
    "address": "0x458a2df1a5c74c5dc9ed6e01dd1178e6d353243b",
    "name": "Adv3nture xyz Gemstone",
    "symbol": "GEM",
    "decimals": 18
  },
  {
    "chainId": 42161,
    "address": "0x284592a004d945f98de5b040808578c61a4bb39a",
    "name": "AIR",
    "symbol": "AIR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36113/thumb/Air_%28AIR%29_1.png?1710487829"
  },
  {
    "chainId": 42161,
    "address": "0x85667409a723684fe1e57dd1abde8d88c2f54214",
    "name": "MagicGLP",
    "symbol": "MAGICGLP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29071/thumb/Token-mGLP.png?1696528037"
  },
  {
    "chainId": 42161,
    "address": "0x77308f8b63a99b24b262d930e0218ed2f49f8475",
    "name": "Dinari MSFT",
    "symbol": "MSFTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32427/thumb/token-icon.png?1698120021"
  },
  {
    "chainId": 42161,
    "address": "0xf5a27e55c748bcddbfea5477cb9ae924f0f7fd2e",
    "name": "The Standard Token",
    "symbol": "TST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20598/thumb/TheStandard-logo_variation-01.png?1696520005"
  },
  {
    "chainId": 42161,
    "address": "0x6dbf2155b0636cb3fd5359fccefb8a2c02b6cb51",
    "name": "Plutus RDNT",
    "symbol": "PLSRDNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30635/thumb/plsRDNT.png?1696529508"
  },
  {
    "chainId": 42161,
    "address": "0xd2eb76feff5fbd549ca078908fb5efce7533a09d",
    "name": "TronAI",
    "symbol": "TAI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29117/thumb/100.png?1696528079"
  },
  {
    "chainId": 42161,
    "address": "0x07dd5beaffb65b8ff2e575d500bdf324a05295dc",
    "name": "ArbiPad",
    "symbol": "ARBI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30275/thumb/ARBI_Logo.png?1696529181"
  },
  {
    "chainId": 42161,
    "address": "0xd4939d69b31fbe981ed6904a3af43ee1dc777aab",
    "name": "Ethereum   Overnight ",
    "symbol": "ETH+",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32326/thumb/ETH__200x200.png?1700298095"
  },
  {
    "chainId": 42161,
    "address": "0xef261714f7e5ba6b86f4780eb6e3bf26b10729cf",
    "name": "White Lotus",
    "symbol": "LOTUS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30206/thumb/logo2.png?1696529116"
  },
  {
    "chainId": 42161,
    "address": "0x935a33e05ecbd424c1691623c99e501ae22400e6",
    "name": "Liquidify",
    "symbol": "LIQUID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29882/thumb/F25AB89F-C960-41A3-8093-06B3EAE88997.jpeg?1696528807"
  },
  {
    "chainId": 42161,
    "address": "0x810cf1757b51963102e0b4fe9fdfcdf749dea98a",
    "name": "Sanctum Coin",
    "symbol": "SANCTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30135/thumb/image_%289%29.png?1696529056"
  },
  {
    "chainId": 42161,
    "address": "0x519062155b0591627c8a0c0958110a8c5639dca6",
    "name": "Dinari META",
    "symbol": "METAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32394/thumb/token-icon.png?1698053183"
  },
  {
    "chainId": 42161,
    "address": "0xfc0c4114ccba4a91d05bdab4a8480cbec46654dd",
    "name": "RB Finance",
    "symbol": "RB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29623/thumb/Main_token.png?1696528559"
  },
  {
    "chainId": 42161,
    "address": "0x700e4edb5c7d8f53ccb0cf212b81a121728e1d5b",
    "name": "LOPO",
    "symbol": "LOPO",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29250/thumb/IMG_20230220_222454_192.png?1696528205"
  },
  {
    "chainId": 42161,
    "address": "0xf93fc7d6508ae2faf8fc5675e896bc38d6e7212c",
    "name": "U Protocol",
    "symbol": "YOU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34553/thumb/YOU_logo.png?1705368741"
  },
  {
    "chainId": 42161,
    "address": "0xfa296fca3c7dba4a92a42ec0b5e2138da3b29050",
    "name": "AiShiba",
    "symbol": "SHIBAI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29898/thumb/Fotor_AI%EF%BC%882%EF%BC%89.png?1696528822"
  },
  {
    "chainId": 42161,
    "address": "0xf4bd09b048248876e39fcf2e0cdf1aee1240a9d2",
    "name": "Dinari SPY",
    "symbol": "SPYD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32403/thumb/token-icon.png?1698055561"
  },
  {
    "chainId": 42161,
    "address": "0xab9ec601b82a22fbf2b030ac0dd7c1c34f52fa04",
    "name": "1minBET",
    "symbol": "1MB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29606/thumb/CGK.png?1696528542"
  },
  {
    "chainId": 42161,
    "address": "0x76bc2e765414e6c8b596c0f52c4240f80268f41d",
    "name": "Unibit",
    "symbol": "UIBT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36911/thumb/Unibit.png?1712764033"
  },
  {
    "chainId": 42161,
    "address": "0x3269a3c00ab86c753856fd135d97b87facb0d848",
    "name": "Florence Finance Medici",
    "symbol": "FFM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34382/thumb/M_PNG_200x200_copy.png?1704779326"
  },
  {
    "chainId": 42161,
    "address": "0xe6be2f5ab2ad4b8b146e3008f81832cf7b279f43",
    "name": "ToxicGarden finance SEED",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30736/thumb/SEED200x200.png?1696529606"
  },
  {
    "chainId": 42161,
    "address": "0xe11508d3e0cf09e6fd6e94fdf41e83836d83ce50",
    "name": "Parifi USDC",
    "symbol": "PFUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/36645/thumb/200x200pfusdc.png?1712339425"
  },
  {
    "chainId": 42161,
    "address": "0xb98058640970d8aa7bbce3b067b2d63c14143786",
    "name": "Baby Arbitrum",
    "symbol": "BARB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29671/thumb/BabyArbitrum200x200.png?1696528606"
  },
  {
    "chainId": 42161,
    "address": "0x8ebb85d53e6955e557b7c53acde1d42fd68561ec",
    "name": "LionDEX",
    "symbol": "LION",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29685/thumb/Lion_Token.png?1696528619"
  },
  {
    "chainId": 42161,
    "address": "0x8b21e9b7daf2c4325bf3d18c1beb79a347fe902a",
    "name": "Collab Land",
    "symbol": "COLLAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29098/thumb/collab.png?1696528061"
  },
  {
    "chainId": 42161,
    "address": "0x86d80fbb0701dd500767b7bc3af72d002eb5de77",
    "name": "Xfish",
    "symbol": "XFISH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36075/thumb/photo_2024-03-14_01-30-06.jpg?1710413243"
  },
  {
    "chainId": 42161,
    "address": "0x3f56e0c36d275367b8c502090edf38289b3dea0d",
    "name": "MAI  Arbitrum ",
    "symbol": "MIMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35517/thumb/mimatic-red.png?1709005586"
  },
  {
    "chainId": 42161,
    "address": "0x5f320aae9b786a9f329a39e41a74e88e14783067",
    "name": "PEPEX",
    "symbol": "PEPEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30028/thumb/COIN.png?1696528951"
  },
  {
    "chainId": 42161,
    "address": "0x43ab8f7d2a8dd4102ccea6b438f6d747b1b9f034",
    "name": "Savvy",
    "symbol": "SVY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31253/thumb/SAVVY.png?1696530077"
  },
  {
    "chainId": 42161,
    "address": "0x2c852d3334188be136bfc540ef2bb8c37b590bad",
    "name": "MagicLand",
    "symbol": "MAGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18844/thumb/logo_-_2021-10-07T141625.399.png?1696518305"
  },
  {
    "chainId": 42161,
    "address": "0xc24a365a870821eb83fd216c9596edd89479d8d7",
    "name": "GAM3S GG",
    "symbol": "G3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35662/thumb/G3_logo.jpg?1709454143"
  },
  {
    "chainId": 42161,
    "address": "0xdc8184ba488e949815d4aafb35b3c56ad03b4179",
    "name": "Roseon",
    "symbol": "ROSX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29698/thumb/roseon.png?1696528631"
  },
  {
    "chainId": 42161,
    "address": "0x50aa7a13b28eea97dc6c3f5e8aaa7fe512e7306d",
    "name": "XBlue Finance",
    "symbol": "XB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29375/thumb/photo_2023-03-04_15-07-08.png?1696528322"
  },
  {
    "chainId": 42161,
    "address": "0x4c70d32295de15fdf302d3bcbb7fc2631ace1c91",
    "name": "LSDoge",
    "symbol": "LSDOGE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/29999/thumb/LSDoge_200_200.png?1696528924"
  },
  {
    "chainId": 42161,
    "address": "0x8616e8ea83f048ab9a5ec513c9412dd2993bce3f",
    "name": "handleUSD",
    "symbol": "FXUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26954/thumb/fxUSDlogo.png?1696526009"
  },
  {
    "chainId": 42161,
    "address": "0xc8a1c0d8255bc2eb5f6236e119e3428fd0c33a74",
    "name": "ArbiTen 10SHARE",
    "symbol": "10SHARE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29798/thumb/arbiten-s-200.png?1696528727"
  },
  {
    "chainId": 42161,
    "address": "0x1622bf67e6e5747b81866fe0b85178a93c7f86e3",
    "name": "Umami",
    "symbol": "UMAMI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/21159/thumb/_UMAMI-200px.jpg?1696520536"
  },
  {
    "chainId": 42161,
    "address": "0x2cab3abfc1670d1a452df502e216a66883cdf079",
    "name": "Layer2DAO",
    "symbol": "L2DAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23699/thumb/Khp7Y4Sn.png?1696522900"
  },
  {
    "chainId": 42161,
    "address": "0x21ccbc5e7f353ec43b2f5b1fb12c3e9d89d30dca",
    "name": "BlackDragon",
    "symbol": "BDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13426/thumb/Black-Dragon-Black.png?1696513186"
  },
  {
    "chainId": 42161,
    "address": "0xafa5676a6ef790f08290dd4a45e0ec2a5cc5cdab",
    "name": "Cat   Mouse",
    "symbol": "CATMOUSE",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/29947/thumb/favicon200.png?1696528874"
  },
  {
    "chainId": 42161,
    "address": "0x3404149e9ee6f17fb41db1ce593ee48fbdcd9506",
    "name": "Hydranet",
    "symbol": "HDN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25177/thumb/HDXdarkblueInv.png?1696524322"
  },
  {
    "chainId": 42161,
    "address": "0x9925130735a30cae869230a9f19eedbaa71a2c02",
    "name": "Horny Hyenas",
    "symbol": "HORNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28979/thumb/horny.png?1696527952"
  },
  {
    "chainId": 42161,
    "address": "0xa2f9ecf83a48b86265ff5fd36cdbaaa1f349916c",
    "name": "Goons of Balatroon",
    "symbol": "GOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27104/thumb/TokenLogo200x200.png?1702250134"
  },
  {
    "chainId": 42161,
    "address": "0x625e7708f30ca75bfd92586e17077590c60eb4cd",
    "name": "Aave v3 USDC e",
    "symbol": "AUSDCE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32911/thumb/usdc.png?1699824302"
  },
  {
    "chainId": 42161,
    "address": "0x3096e7bfd0878cc65be71f8899bc4cfb57187ba3",
    "name": "Xend Finance",
    "symbol": "RWA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14496/thumb/WeChat_Image_20210325163206.png?1696514181"
  },
  {
    "chainId": 42161,
    "address": "0x602eb0d99a5e3e76d1510372c4d2020e12eaea8a",
    "name": "TridentDAO",
    "symbol": "PSI",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28955/thumb/trident.jpg?1696527928"
  },
  {
    "chainId": 42161,
    "address": "0x1aae7de64d9ae1487e95858bbf98185f21e926fd",
    "name": "AICORE",
    "symbol": "AICORE",
    "decimals": 18
  },
  {
    "chainId": 42161,
    "address": "0xcb55d61e6299597c39feec3d4036e727afbe11be",
    "name": "Lumi Finance Governance Token",
    "symbol": "LUAG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33506/thumb/LUAG.png?1702039637"
  },
  {
    "chainId": 42161,
    "address": "0x87aaffdf26c6885f6010219208d5b161ec7609c0",
    "name": "Equation",
    "symbol": "EQU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32592/thumb/equation_logo.png?1699220377"
  },
  {
    "chainId": 42161,
    "address": "0x4c4b907bd5c38d14a084aac4f511a9b46f7ec429",
    "name": "RB Share",
    "symbol": "RBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29660/thumb/Share_token.png?1696528595"
  },
  {
    "chainId": 42161,
    "address": "0x8a66794e1aaebc7018a7b75f61f384e30ae3b159",
    "name": "BlueSale",
    "symbol": "BLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30300/thumb/logo_BLS_200_bg.png?1696529204"
  },
  {
    "chainId": 42161,
    "address": "0x51318b7d00db7acc4026c88c3952b66278b6a67f",
    "name": "PlutusDAO",
    "symbol": "PLS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25326/thumb/M6nUndNU_400x400.jpg?1696524461"
  },
  {
    "chainId": 42161,
    "address": "0xb4bbfe92702730ef7f1d28e4b9e42381182f48a5",
    "name": "Hold VIP",
    "symbol": "HOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30129/thumb/RteOSWQ__400x400.jpg?1696529050"
  },
  {
    "chainId": 42161,
    "address": "0x513c7e3a9c69ca3e22550ef58ac1c0088e918fff",
    "name": "Aave v3 wstETH",
    "symbol": "AWSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32890/thumb/WSTETH.png?1699776336"
  },
  {
    "chainId": 42161,
    "address": "0xf7693c6fd9a7172d537fa75d133d309501cbd657",
    "name": "Web3 No Value",
    "symbol": "W3N",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32083/thumb/WechatIMG718.jpg?1710596218"
  },
  {
    "chainId": 42161,
    "address": "0x98c435070c24e5152fc14d130937a38810c104b9",
    "name": "Kobe",
    "symbol": "BEEF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35004/thumb/beef-icon-1.png?1707227494"
  },
  {
    "chainId": 42161,
    "address": "0x93d504070ab0eede5449c89c5ea0f5e34d8103f8",
    "name": "Archi Token",
    "symbol": "ARCHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30823/thumb/ARCHILOGO.png?1696529677"
  },
  {
    "chainId": 42161,
    "address": "walc.near",
    "name": "WALC",
    "symbol": "WALC",
    "decimals": 24,
    "logoURI": "https://assets.coingecko.com/coins/images/34332/thumb/WALC_-_Logo.png?1704668584"
  },
  {
    "chainId": 42161,
    "address": "0xbfbcfe8873fe28dfa25f1099282b088d52bbad9c",
    "name": "Equilibria Finance",
    "symbol": "EQB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30645/thumb/QLLK8pmR_400x400.jpg?1696529516"
  },
  {
    "chainId": 42161,
    "address": "0xf16585365bfe050eff7c926f22df87f46da2cafe",
    "name": "Adv3nture xyz Gold",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29126/thumb/gold.png?1696528087"
  },
  {
    "chainId": 42161,
    "address": "0x64343594ab9b56e99087bfa6f2335db24c2d1f17",
    "name": "Vesta Stable",
    "symbol": "VST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23621/thumb/vesta.png?1696522826"
  },
  {
    "chainId": 42161,
    "address": "0xe2db86ce824cf3dcc0c31a073de294239fdde060",
    "name": "OMO Exchange",
    "symbol": "OMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29270/thumb/omo.png?1696528223"
  },
  {
    "chainId": 42161,
    "address": "0x27d8de4c30ffde34e982482ae504fc7f23061f61",
    "name": "My MetaTrader",
    "symbol": "MMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29161/thumb/MMT_logo_Vert.png?1696528120"
  },
  {
    "chainId": 42161,
    "address": "0x1772d876f9df830693b8004322cb8885b77e47e5",
    "name": "AuradX",
    "symbol": "DALLE2",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29511/thumb/200_pix.png?1696528455"
  },
  {
    "chainId": 42161,
    "address": "0x3ad63b3c0ea6d7a093ff98fde040baddc389ecdc",
    "name": "Dinari NFLX",
    "symbol": "NFLXD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32425/thumb/token-icon.png?1698119029"
  },
  {
    "chainId": 42161,
    "address": "0xb2dcbd5258a22385240e4ac13fc6726b66f0de96",
    "name": "ZenithSwap",
    "symbol": "ZSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29658/thumb/zenithswaplogo.png?1696528593"
  },
  {
    "chainId": 42161,
    "address": "0xbe00f3db78688d9704bcb4e0a827aea3a9cc0d62",
    "name": "Fiat24 USD",
    "symbol": "USD24",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/25598/thumb/USD24.png?1696524732"
  },
  {
    "chainId": 42161,
    "address": "0x6740acb82ac5c63a7ad2397ee1faed7c788f5f8c",
    "name": "AlienFi",
    "symbol": "ALIEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29175/thumb/alien.png?1696528133"
  },
  {
    "chainId": 42161,
    "address": "0xaf5db6e1cc585ca312e8c8f7c499033590cf5c98",
    "name": "Arken Finance",
    "symbol": "ARKEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26137/thumb/arken.png?1696525225"
  },
  {
    "chainId": 42161,
    "address": "0x6b5b5eac259e883b484ed879d43dd4d616a90e65",
    "name": "The Knowers",
    "symbol": "KNOW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36964/thumb/63_%281%29.png?1712907583"
  },
  {
    "chainId": 42161,
    "address": "0x764bfc309090e7f93edce53e5befa374cdcb7b8e",
    "name": "Grimace",
    "symbol": "GRIMACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30957/thumb/Grimace.png?1696529796"
  },
  {
    "chainId": 42161,
    "address": "0x7241bc8035b65865156ddb5edef3eb32874a3af6",
    "name": "Jones GLP",
    "symbol": "JGLP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28892/thumb/hatGLP.png?1696527868"
  },
  {
    "chainId": 42161,
    "address": "0xc136e6b376a9946b156db1ed3a34b08afdaed76d",
    "name": "CreDA",
    "symbol": "CREDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22551/thumb/gZ4EgSXD_400x400.jpg?1696521871"
  },
  {
    "chainId": 42161,
    "address": "0xee0b14e8fc86691cf6ee42b9954985b4cf968534",
    "name": "ZenPandaCoin",
    "symbol": "ZPC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30269/thumb/panda.png?1696529176"
  },
  {
    "chainId": 42161,
    "address": "0xd7a892f28dedc74e6b7b33f93be08abfc394a360",
    "name": "Crypto Index Pool",
    "symbol": "CIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32504/thumb/photo_6176746894382970935_y_%281%29.jpg?1698313185"
  },
  {
    "chainId": 42161,
    "address": "0x493070ef5280e7a275a106a30b0414dbdb21febd",
    "name": "AI Supreme",
    "symbol": "AISP",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/30900/thumb/200.png?1696529746"
  },
  {
    "chainId": 42161,
    "address": "0x2c5d06f591d0d8cd43ac232c2b654475a142c7da",
    "name": "Fiat24 EUR",
    "symbol": "EUR24",
    "decimals": 2,
    "logoURI": "https://assets.coingecko.com/coins/images/33297/thumb/EUR24.png?1701402541"
  },
  {
    "chainId": 42161,
    "address": "0xccd05a0fcfc1380e9da27862adb2198e58e0d66f",
    "name": "ANIMA",
    "symbol": "ANIMA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30377/thumb/anima_logo_yellow.png?1696529271"
  },
  {
    "chainId": 42161,
    "address": "0x617b76412bd9f3f80fe87d1533dc7017defa8ad1",
    "name": "XRender",
    "symbol": "XRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32364/thumb/image_2023-10-14_16-00-43.png?1698029814"
  },
  {
    "chainId": 42161,
    "address": "0xef04804e1e474d3f9b73184d7ef5d786f3fce930",
    "name": "Wall Street Games",
    "symbol": "WSG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36483/thumb/wsg.jpeg?1711539007"
  },
  {
    "chainId": 42161,
    "address": "0xe8e87b8f47bda705ce1a5286fbddd26e93d8dffe",
    "name": "Certicos",
    "symbol": "CERT",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/35078/thumb/C62C9ou.png?1707301357"
  },
  {
    "chainId": 42161,
    "address": "0xd6e7c8af05d2d06a76811d44917613bdd258b995",
    "name": "Brr Protocol",
    "symbol": "BRR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29558/thumb/BRR_Protocol.png?1696528498"
  },
  {
    "chainId": 42161,
    "address": "0x9e64d3b9e8ec387a9a58ced80b71ed815f8d82b5",
    "name": "Smolcoin",
    "symbol": "SMOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33951/thumb/Smol_Coin_Icon.png?1703558329"
  },
  {
    "chainId": 42161,
    "address": "0xf9df075716b2d9b95616341dc6bc64c85e56645c",
    "name": "Mayfair",
    "symbol": "MAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30548/thumb/a1VkJI0r_400x400.jpg?1696529420"
  },
  {
    "chainId": 42161,
    "address": "0xf5db7c04bcec3bea25bd4fdc501cff0864f726e7",
    "name": "YSL",
    "symbol": "YSL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34512/thumb/YSL-Light_200px.png?1705288632"
  },
  {
    "chainId": 42161,
    "address": "0x894a3abb764a0ef5da69c62336ac3c15b88bf106",
    "name": "Chunks",
    "symbol": "CHUNKS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30163/thumb/Chunks_Logo_v2_200x200.png?1696529083"
  },
  {
    "chainId": 42161,
    "address": "0xce3a75572bc375476438206f6f29f46f8466ecc2",
    "name": "XLSD Coin",
    "symbol": "XLSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30747/thumb/Icon-200.png?1696529616"
  },
  {
    "chainId": 42161,
    "address": "0x2a35c341f4dcf2d18e0fb38e0df50e8a47af1368",
    "name": "ArbitrumPad",
    "symbol": "ARBPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29629/thumb/xconvert.com.png?1696528567"
  },
  {
    "chainId": 42161,
    "address": "0x80dd74145b8bb10cef01bf914f796bd8b54d7809",
    "name": "Hepton",
    "symbol": "HTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30274/thumb/HTECoingecko.png?1696529180"
  },
  {
    "chainId": 42161,
    "address": "0xf19547f9ed24aa66b03c3a552d181ae334fbb8db",
    "name": "Lodestar",
    "symbol": "LODE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28218/thumb/Lodestar_PFP_350x350px.jpg?1696527220"
  },
  {
    "chainId": 42161,
    "address": "0x5326e71ff593ecc2cf7acae5fe57582d6e74cff1",
    "name": "plvGLP",
    "symbol": "PLVGLP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30554/thumb/plvGLP.png?1696529425"
  },
  {
    "chainId": 42161,
    "address": "0xc3abc47863524ced8daf3ef98d74dd881e131c38",
    "name": "Lumi Finance",
    "symbol": "LUA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33502/thumb/LUA.png?1702038548"
  },
  {
    "chainId": 42161,
    "address": "0xeeac5e75216571773c0064b3b591a86253791db6",
    "name": "ELLERIUM",
    "symbol": "ELM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28153/thumb/elm.png?1696527158"
  },
  {
    "chainId": 42161,
    "address": "0x91c0ab2e50db1afe9e673bbb1fd5a3e0e4adf1f9",
    "name": "BLOBCOIN",
    "symbol": "BLOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34183/thumb/blobby%282%29.png?1704257061"
  },
  {
    "chainId": 42161,
    "address": "0x619c82392cb6e41778b7d088860fea8447941f4c",
    "name": "Army of Fortune Gem",
    "symbol": "AFG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35084/thumb/Token_GEM_AOFVERSE.png?1707317138"
  },
  {
    "chainId": 42161,
    "address": "0x6daf586b7370b14163171544fca24abcc0862ac5",
    "name": "xPet tech BPET",
    "symbol": "BPET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33848/thumb/BPET_logo.png?1703122571"
  },
  {
    "chainId": 42161,
    "address": "0xd2f8b81cfff297c70e5d58ac58762dd5d213ea20",
    "name": "artGPT",
    "symbol": "AGPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29111/thumb/IMG_20230217_063445_776.png?1696528073"
  },
  {
    "chainId": 42161,
    "address": "0x772598e9e62155d7fdfe65fdf01eb5a53a8465be",
    "name": "Empyreal",
    "symbol": "EMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31374/thumb/emplogotransparent_%281%29.png?1696530191"
  },
  {
    "chainId": 42161,
    "address": "0xcb24b141ecaaad0d2c255d6f99d6f4790546a75c",
    "name": "Exponential Capital",
    "symbol": "EXPO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29646/thumb/expo_capital.jpg?1696528582"
  },
  {
    "chainId": 42161,
    "address": "0x319e222de462ac959baf2aec848697aec2bbd770",
    "name": "OreoSwap",
    "symbol": "OREO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28582/thumb/oreoswapx.png?1696527570"
  },
  {
    "chainId": 42161,
    "address": "0x6dd963c510c2d2f09d5eddb48ede45fed063eb36",
    "name": "FactorDAO",
    "symbol": "FCTR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29018/thumb/FactorLogo.png?1696527989"
  },
  {
    "chainId": 42161,
    "address": "0xafccb724e3aec1657fc9514e3e53a0e71e80622d",
    "name": "Vaultka",
    "symbol": "VKA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32408/thumb/VKA_logo_PNG.png?1698057073"
  },
  {
    "chainId": 42161,
    "address": "0x95146881b86b3ee99e63705ec87afe29fcc044d9",
    "name": "Vertex",
    "symbol": "VRTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27927/thumb/vrtx.png?1696526947"
  },
  {
    "chainId": 42161,
    "address": "0x3082cc23568ea640225c2467653db90e9250aaa0",
    "name": "Radiant Capital",
    "symbol": "RDNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26536/thumb/Radiant-Logo-200x200.png?1696525610"
  },
  {
    "chainId": 42161,
    "address": "0x155f0dd04424939368972f4e1838687d6a831151",
    "name": "ArbiDoge",
    "symbol": "ADOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18333/thumb/Screen-Shot-2021-09-04-at-11-59-16-AM.png?1696517824"
  },
  {
    "chainId": 42161,
    "address": "0xb9af4762c039d63e30039f1712dfab77026408c7",
    "name": "BullBear AI",
    "symbol": "AIBB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30096/thumb/ICON_200.png?1696529020"
  },
  {
    "chainId": 42161,
    "address": "0x1c43d05be7e5b54d506e3ddb6f0305e8a66cd04e",
    "name": "ZTX",
    "symbol": "ZTX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32340/thumb/CoinGecko_200_x_200.jpg?1697466589"
  },
  {
    "chainId": 42161,
    "address": "0xf0a479c9c3378638ec603b8b6b0d75903902550b",
    "name": "SpaceCatch",
    "symbol": "CATCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36632/thumb/Logomark_colours.png?1712022405"
  },
  {
    "chainId": 42161,
    "address": "0x872bad41cfc8ba731f811fea8b2d0b9fd6369585",
    "name": "BattleFly",
    "symbol": "GFLY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28828/thumb/GFLY_LOGO.png?1696527804"
  },
  {
    "chainId": 42161,
    "address": "0xaaa6c1e32c55a7bfa8066a6fae9b42650f262418",
    "name": "Ramses Exchange",
    "symbol": "RAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29420/thumb/newram.png?1708427055"
  },
  {
    "chainId": 42161,
    "address": "0x0a84edf70f30325151631ce7a61307d1f4d619a3",
    "name": "BonzAI DePIN",
    "symbol": "BONZAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36032/thumb/bonzai.jpeg?1710394145"
  },
  {
    "chainId": 42161,
    "address": "0x299142a6370e1912156e53fbd4f25d7ba49ddcc5",
    "name": "AI Meta Club",
    "symbol": "AMC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31919/thumb/aimeta.jpg?1696530728"
  },
  {
    "chainId": 42161,
    "address": "0x50e401255275dc405a99d3281f396cca681eea9d",
    "name": "Kortana",
    "symbol": "KORA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30966/thumb/Logo_Kortana_Redondo-4_%281%29.png?1696529806"
  },
  {
    "chainId": 42161,
    "address": "0xd79bb960dc8a206806c3a428b31bca49934d18d7",
    "name": "Rosnet",
    "symbol": "ROSNET",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/35279/thumb/RosnetLogo200x200.png?1708054752"
  },
  {
    "chainId": 42161,
    "address": "0x4cb9a7ae498cedcbb5eae9f25736ae7d428c9d66",
    "name": "Xai",
    "symbol": "XAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34258/thumb/2024-01-09_16.31.28.jpg?1704789138"
  },
  {
    "chainId": 42161,
    "address": "0xacc51ffdef63fb0c014c882267c3a17261a5ed50",
    "name": "Stryke",
    "symbol": "SYK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36531/thumb/stryke.jpeg?1711704094"
  },
  {
    "chainId": 42161,
    "address": "0x10393c20975cf177a3513071bc110f7962cd67da",
    "name": "Jones DAO",
    "symbol": "JONES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23290/thumb/3c8c2ed8-afb3-4b67-9937-5493acd88b50.jpg?1696522508"
  },
  {
    "chainId": 42161,
    "address": "0xed3fb761414da74b74f33e5c5a1f78104b188dfc",
    "name": "ArbiNYAN",
    "symbol": "NYAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18309/thumb/Qmf8DSV4RbSuVRLJvmj4WwEMjwtYcCd53EFXPPjkEwzx4B.png?1696517801"
  },
  {
    "chainId": 42161,
    "address": "0x2b45e21c35a33c58e4c5ce82a82466b0754fd154",
    "name": "Brokkr",
    "symbol": "BRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27813/thumb/bro.png?1696526832"
  },
  {
    "chainId": 42161,
    "address": "0x34229b3f16fbcdfa8d8d9d17c0852f9496f4c7bb",
    "name": "IPOR",
    "symbol": "IPOR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28373/thumb/IPOR-token-200x200.png?1696527376"
  },
  {
    "chainId": 42161,
    "address": "0x2b41806cbf1ffb3d9e31a9ece6b738bf9d6f645f",
    "name": "ENO",
    "symbol": "ENO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26501/thumb/Logo_%281%29.png?1709565574"
  },
  {
    "chainId": 42161,
    "address": "0x6604b5da093f3f35066c6c79e51d581a44c35288",
    "name": "Uranium3o8",
    "symbol": "U",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32571/thumb/Logo_Black_Background.png?1698608787"
  },
  {
    "chainId": 42161,
    "address": "0x7e40b3e0fd0473e0240288d2b7618ac6f0c17e1b",
    "name": "Stable USDLR",
    "symbol": "USDLR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33115/thumb/0x68592c5c98c4f4a8a4bc6da2121e65da3d1c0917.png?1700731571"
  },
  {
    "chainId": 42161,
    "address": "0x60d01ec2d5e98ac51c8b4cf84dfcce98d527c747",
    "name": "iZUMi Finance",
    "symbol": "IZI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21791/thumb/izumi-logo-symbol.png?1696521144"
  },
  {
    "chainId": 42161,
    "address": "0x35ca1e5a9b1c09fa542fa18d1ba4d61c8edff852",
    "name": "Schrodi",
    "symbol": "SCHRODI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36255/thumb/apzs3q26_200x200.jpg?1710927534"
  },
  {
    "chainId": 42161,
    "address": "0x3419875b4d3bca7f3fdda2db7a476a79fd31b4fe",
    "name": "DizzyHavoc",
    "symbol": "DZHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35575/thumb/dizzyhavoc_circle_3_200_200_transparentBg.png?1709194317"
  },
  {
    "chainId": 42161,
    "address": "0x498bf2b1e120fed3ad3d42ea2165e9b73f99c1e5",
    "name": "crvUSD",
    "symbol": "CRVUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30118/thumb/crvusd.jpeg?1696529040"
  },
  {
    "chainId": 42161,
    "address": "0x0000206329b97db379d5e1bf586bbdb969c63274",
    "name": "USDA",
    "symbol": "USDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34510/thumb/agUSD-coingecko.png?1705288392"
  },
  {
    "chainId": 42161,
    "address": "0xa9011ee5796be43123651181dc75c0e72bb1191c",
    "name": "NeuroPulse AI",
    "symbol": "NPAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31848/thumb/NPAI.png?1700257725"
  },
  {
    "chainId": 42161,
    "address": "0x47c337bd5b9344a6f3d6f58c474d9d8cd419d8ca",
    "name": "DackieSwap",
    "symbol": "DACKIE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30752/thumb/dackieswap_large.png?1707290196"
  },
  {
    "chainId": 42161,
    "address": "0x59debed8d46a0cb823d8be8b957add987ead39aa",
    "name": "Quack Token",
    "symbol": "QUACK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31436/thumb/0x639C0D019C257966C4907bD4E68E3F349bB58109.png?1696530251"
  },
  {
    "chainId": 42161,
    "address": "0x0cdbbc7fe1c8da0cc41ba96d7edb4cce5982f23f",
    "name": "UNITS LIMITED SUPPLY",
    "symbol": "ULS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33560/thumb/dark_token.png?1702429859"
  },
  {
    "chainId": 42161,
    "address": "0xdc8b6b6beab4d5034ae91b7a1cf7d05a41f0d239",
    "name": "Gemach",
    "symbol": "GMAC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32334/thumb/gmac-200x200.png?1697452788"
  },
  {
    "chainId": 42161,
    "address": "0x27f485b62c4a7e635f561a87560adf5090239e93",
    "name": "DFX Finance",
    "symbol": "DFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14091/thumb/DFX.png?1696513813"
  },
  {
    "chainId": 42161,
    "address": "0x9f6abbf0ba6b5bfa27f4deb6597cc6ec20573fda",
    "name": "Ferrum Network",
    "symbol": "FRM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8251/thumb/FRM.png?1696508455"
  },
  {
    "chainId": 42161,
    "address": "0xeb4d25db65dcef52380c99ba7e1344c820ecb1fc",
    "name": "X World Games",
    "symbol": "XWG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17847/thumb/200_200_%281%29_%281%29.png?1696790226"
  },
  {
    "chainId": 42161,
    "address": "0xe20b9e246db5a0d21bf9209e4858bc9a3ff7a034",
    "name": "Wrapped Banano",
    "symbol": "WBAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32617/thumb/WBAN.jpg?1698749253"
  },
  {
    "chainId": 42161,
    "address": "0x71eeba415a523f5c952cc2f06361d5443545ad28",
    "name": "XDAO",
    "symbol": "XDAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27363/thumb/token_2.png?1696526408"
  },
  {
    "chainId": 42161,
    "address": "0xf525e73bdeb4ac1b0e741af3ed8a8cbb43ab0756",
    "name": "KiboShib",
    "symbol": "KIBSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29335/thumb/foto_no_exif_%2811%29%282%29_%281%29.png?1696528285"
  },
  {
    "chainId": 42161,
    "address": "0x4e0da40b9063dc48364c1c0ffb4ae9d091fc2270",
    "name": "Edgeware",
    "symbol": "EDG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8452/thumb/logo-edgeware.png?1696508638"
  },
  {
    "chainId": 42161,
    "address": "0x7f4db37d7beb31f445307782bc3da0f18df13696",
    "name": "Yield Yak",
    "symbol": "YAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17654/thumb/yieldyak.png?1696517185"
  },
  {
    "chainId": 42161,
    "address": "0x8888888888f004100c0353d657be6300587a6ccd",
    "name": "ACryptoS",
    "symbol": "ACS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32721/thumb/ACS.jpg?1699009686"
  },
  {
    "chainId": 42161,
    "address": "0x004626a008b1acdc4c74ab51644093b155e59a23",
    "name": "Angle Staked EURA",
    "symbol": "STEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32036/thumb/stEUR-x4.png?1696530832"
  },
  {
    "chainId": 42161,
    "address": "0x429fed88f10285e61b12bdf00848315fbdfcc341",
    "name": "THORWallet DEX",
    "symbol": "TGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21843/thumb/tgt_logo.png?1696521198"
  },
  {
    "chainId": 42161,
    "address": "0xf18c263ec50cc211ef3f172228549b6618f10613",
    "name": "Collab Land",
    "symbol": "COLLAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29098/thumb/collab.png?1696528061"
  },
  {
    "chainId": 42161,
    "address": "0xd1e094cabc5acb9d3b0599c3f76f2d01ff8d3563",
    "name": "VirtuSwap",
    "symbol": "VRSW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30629/thumb/VirtuSwap_Logo_Red_200x200.png?1696529502"
  },
  {
    "chainId": 42161,
    "address": "0xe47ba52f326806559c1dec7ddd997f6957d0317d",
    "name": "Shell Protocol Token",
    "symbol": "SHELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35192/thumb/shellprotocoltoken-reduced.png?1712428374"
  },
  {
    "chainId": 42161,
    "address": "0x24404dc041d74cd03cfe28855f555559390c931b",
    "name": "r CryptoCurrency Moons",
    "symbol": "MOON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11222/thumb/Moons.png?1696511153"
  },
  {
    "chainId": 42161,
    "address": "0xb4357054c3da8d46ed642383f03139ac7f090343",
    "name": "Port3 Network",
    "symbol": "PORT3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33383/thumb/port3-bc-200x200.png?1710075114"
  },
  {
    "chainId": 42161,
    "address": "0xbcd4d5ac29e06e4973a1ddcd782cd035d04bc0b7",
    "name": "Quick Intel",
    "symbol": "QKNTL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29605/thumb/IMG_6589D0616DF1-1.jpeg?1696528542"
  },
  {
    "chainId": 42161,
    "address": "0x6c84a8f1c29108f47a79964b5fe888d4f4d0de40",
    "name": "tBTC",
    "symbol": "TBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11224/thumb/0x18084fba666a33d37592fa2633fd49a74dd93a88.png?1696511155"
  },
  {
    "chainId": 42161,
    "address": "0x6d30ab63fc4cddf268edd294a8d1b07bdb282976",
    "name": "ISSUAA",
    "symbol": "ISS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33324/thumb/ISSUAA_Logo2023_200x200px.png?1701437586"
  },
  {
    "chainId": 42161,
    "address": "0xe50fa9b3c56ffb159cb0fca61f5c9d750e8128c8",
    "name": "Aave v3 WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32882/thumb/WETH_%281%29.png?1699716492"
  },
  {
    "chainId": 42161,
    "address": "0x51c601dc278eb2cfea8e52c4caa35b3d6a9a2c26",
    "name": "Chainge",
    "symbol": "XCHNG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16526/thumb/chainge.jpeg?1708099639"
  },
  {
    "chainId": 42161,
    "address": "0x86f65121804d2cdbef79f9f072d4e0c2eebabc08",
    "name": "Garden",
    "symbol": "SEED",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34671/thumb/icon.png?1705656915"
  },
  {
    "chainId": 42161,
    "address": "0xa3210cd727fe6daf8386af5623ba51a367e46263",
    "name": "Basket",
    "symbol": "BSKT",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/34661/thumb/BSKT_Logo.png?1705636891"
  },
  {
    "chainId": 42161,
    "address": "0xbfd5206962267c7b4b4a8b3d76ac2e1b2a5c4d5e",
    "name": "Osaka Protocol",
    "symbol": "OSAK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30911/thumb/osak_logo.png?1696529756"
  },
  {
    "chainId": 42161,
    "address": "0xb827710314a05bcbee9180e11c2abe5823289422",
    "name": "Abachi",
    "symbol": "ABI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31225/thumb/Abachi.png?1696530051"
  },
  {
    "chainId": 42161,
    "address": "0x2b1d36f5b61addaf7da7ebbd11b35fd8cfb0de31",
    "name": "Interport Token",
    "symbol": "ITP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28338/thumb/ITP_Logo_200.png?1696527344"
  },
  {
    "chainId": 42161,
    "address": "0x4e47951508fd4a4126f8ff9cf5e6fa3b7cc8e073",
    "name": "Fluid",
    "symbol": "FLUID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31890/thumb/fluidlogo200x200.png?1696530701"
  },
  {
    "chainId": 42161,
    "address": "0x8929e9dbd2785e3ba16175e596cdd61520fee0d1",
    "name": "Altitude",
    "symbol": "ALTD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30114/thumb/logo.png?1696529036"
  },
  {
    "chainId": 42161,
    "address": "0xa23e44aea714fbbc08ef28340d78067b9a8cad73",
    "name": "Lybra",
    "symbol": "LBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29958/thumb/New_LBR_V2.png?1696528884"
  },
  {
    "chainId": 42161,
    "address": "0x35751007a407ca6feffe80b3cb397736d2cf4dbe",
    "name": "Wrapped eETH",
    "symbol": "WEETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33033/thumb/weETH.png?1701438396"
  },
  {
    "chainId": 42161,
    "address": "0x42069d11a2cc72388a2e06210921e839cfbd3280",
    "name": "GnomeLand",
    "symbol": "GNOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35201/thumb/logopng.png?1707841763"
  },
  {
    "chainId": 42161,
    "address": "0x58b9cb810a68a7f3e1e4f8cb45d1b9b3c79705e8",
    "name": "Connext",
    "symbol": "NEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31293/thumb/connext.png?1696530113"
  },
  {
    "chainId": 42161,
    "address": "0xdc1bb8018e711f71293a6edfb5c148187008ebdc",
    "name": "WALC",
    "symbol": "WALC",
    "decimals": 24,
    "logoURI": "https://assets.coingecko.com/coins/images/34332/thumb/WALC_-_Logo.png?1704668584"
  },
  {
    "chainId": 42161,
    "address": "0x965d00aa7abc62ca10132e641d08593435ac811d",
    "name": "KAP Games",
    "symbol": "KAP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27682/thumb/KAP_Logo.png?1696526710"
  },
  {
    "chainId": 42161,
    "address": "0x894134a25a5fac1c2c26f1d8fbf05111a3cb9487",
    "name": "Grai",
    "symbol": "GRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30427/thumb/GRAI_Token.png?1696529315"
  },
  {
    "chainId": 42161,
    "address": "0x66e535e8d2ebf13f49f3d49e5c50395a97c137b1",
    "name": "Molten",
    "symbol": "MOLTEN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36726/thumb/moltenmesh.png?1712147407"
  },
  {
    "chainId": 42161,
    "address": "0xba0dda8762c24da9487f5fa026a9b64b695a07ea",
    "name": "OX Coin",
    "symbol": "OX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35365/thumb/logo.png?1708395976"
  },
  {
    "chainId": 42161,
    "address": "0xdcbf4cb83d27c408b30dd7f39bfcabd7176b1ba3",
    "name": "OpenOcean",
    "symbol": "OOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17014/thumb/ooe_log.png?1696516578"
  },
  {
    "chainId": 42161,
    "address": "0xf42e2b8bc2af8b110b65be98db1321b1ab8d44f5",
    "name": "Donut",
    "symbol": "DONUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7538/thumb/Donut.png?1696507804"
  },
  {
    "chainId": 42161,
    "address": "0x4fb21b1dbd1da7616003ff3a8eb57aeab3d241f0",
    "name": "impactMarket",
    "symbol": "PACT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21907/thumb/PACT_Token_Ticker_Blue_2x.png?1696521258"
  },
  {
    "chainId": 42161,
    "address": "0xe22c452bd2ade15dfc8ad98286bc6bdf0c9219b7",
    "name": "Polytrade",
    "symbol": "TRADE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16416/thumb/Logo_colored_200.png?1696516012"
  },
  {
    "chainId": 42161,
    "address": "0x0341c0c0ec423328621788d4854119b97f44e391",
    "name": "Silo Finance",
    "symbol": "SILO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21454/thumb/y0iYKZOv_400x400.png?1696520816"
  },
  {
    "chainId": 42161,
    "address": "0x039d2e8f097331278bd6c1415d839310e0d5ece4",
    "name": "Linda",
    "symbol": "LINDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33699/thumb/linda-logo-200.png?1705166731"
  },
  {
    "chainId": 42161,
    "address": "0xa3d1a8deb97b111454b294e2324efad13a9d8396",
    "name": "Overnight Finance",
    "symbol": "OVN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31970/thumb/OVN.png?1696959174"
  },
  {
    "chainId": 42161,
    "address": "0x57f12fe6a4e5fe819eec699fadf9db2d06606bb4",
    "name": "Neptune Mutual",
    "symbol": "NPM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28235/thumb/Neptune_Mutual_Logo_Icon_Regular_128.png?1696527238"
  },
  {
    "chainId": 42161,
    "address": "0x94025780a1ab58868d9b2dbbb775f44b32e8e6e5",
    "name": "BetSwirl",
    "symbol": "BETS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26618/thumb/icon_200.png?1696525691"
  },
  {
    "chainId": 42161,
    "address": "0x11bf4f05eb28b802ed3ab672594decb20ffe2313",
    "name": "Aurory",
    "symbol": "AURY",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/19324/thumb/Ico_Blanc.png?1696518766"
  },
  {
    "chainId": 42161,
    "address": "0xad4b9c1fbf4923061814dd9d5732eb703faa53d4",
    "name": "Wicrypt",
    "symbol": "WNT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21223/thumb/wicrypt.PNG?1696520597"
  },
  {
    "chainId": 42161,
    "address": "0xa6099b214e8d069911702bc92ef274f63c476c5a",
    "name": "Zenland",
    "symbol": "ZENF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29477/thumb/tokenicon200px.png?1696528421"
  },
  {
    "chainId": 42161,
    "address": "0x724dc807b04555b71ed48a6896b6f41593b8c637",
    "name": "Aave v3 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32847/thumb/usdc_%281%29.png?1699619355"
  },
  {
    "chainId": 42161,
    "address": "0x8ec1877698acf262fe8ad8a295ad94d6ea258988",
    "name": "Davos Protocol",
    "symbol": "DUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28775/thumb/dusd_logo_200x200.png?1696527754"
  },
  {
    "chainId": 42161,
    "address": "0xb829b68f57cc546da7e5806a929e53be32a4625d",
    "name": "Axelar Wrapped Ether",
    "symbol": "AXLETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28171/thumb/weth-wei_D_3x.png?1696527174"
  },
  {
    "chainId": 42161,
    "address": "0x3b60ff35d3f7f62d636b067dd0dc0dfdad670e4e",
    "name": "Milady Meme Coin",
    "symbol": "LADYS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30194/thumb/logo.png?1696529109"
  },
  {
    "chainId": 42161,
    "address": "0x4d15a3a2286d883af0aa1b3f21367843fac63e07",
    "name": "Bridged TrueUSD",
    "symbol": "TUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30837/thumb/tusd.jpeg?1696529695"
  },
  {
    "chainId": 42161,
    "address": "0x7f5373ae26c3e8ffc4c77b7255df7ec1a9af52a6",
    "name": "Bridged Tether  Axelar ",
    "symbol": "AXLUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/31002/thumb/uusdt_D_3x.png?1696529840"
  },
  {
    "chainId": 42161,
    "address": "0x7f4638a58c0615037decc86f1dae60e55fe92874",
    "name": "RigoBlock",
    "symbol": "GRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/1532/thumb/Symbol-RigoblockRGB.png?1696502572"
  },
  {
    "chainId": 42161,
    "address": "0x30dcba0405004cf124045793e1933c798af9e66a",
    "name": "Yieldification",
    "symbol": "YDF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26699/thumb/logo.png?1696525772"
  },
  {
    "chainId": 42161,
    "address": "0x53691596d1bce8cea565b84d4915e69e03d9c99d",
    "name": "Across Protocol",
    "symbol": "ACX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28161/thumb/across-200x200.png?1696527165"
  },
  {
    "chainId": 42161,
    "address": "0xef00278d7eadf3b2c05267a2f185e468ad7eab7d",
    "name": "PepePAD",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30321/thumb/logo200.png?1696529222"
  },
  {
    "chainId": 42161,
    "address": "0x09199d9a5f4448d0848e4395d065e1ad9c4a1f74",
    "name": "Bonk",
    "symbol": "BONK",
    "decimals": 5,
    "logoURI": "https://assets.coingecko.com/coins/images/28600/thumb/bonk.jpg?1696527587"
  },
  {
    "chainId": 42161,
    "address": "0x680447595e8b7b3aa1b43beb9f6098c79ac2ab3f",
    "name": "USDD",
    "symbol": "USDD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25380/thumb/UUSD.jpg?1696524513"
  },
  {
    "chainId": 42161,
    "address": "0xe85411c030fb32a9d8b14bbbc6cb19417391f711",
    "name": "Sumer Money suBTC",
    "symbol": "SUBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33264/thumb/Sumer_Money_Logo.jpg?1701321397"
  },
  {
    "chainId": 42161,
    "address": "0x1b01514a2b3cdef16fd3c680a818a0ab97da8a09",
    "name": "Frax Price Index",
    "symbol": "FPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24945/thumb/FPI_icon.png?1696524100"
  },
  {
    "chainId": 42161,
    "address": "0x0d81e50bc677fa67341c44d7eaa9228dee64a4e1",
    "name": "BarnBridge",
    "symbol": "BOND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12811/thumb/barnbridge.jpg?1696512604"
  },
  {
    "chainId": 42161,
    "address": "0x55678cd083fcdc2947a0df635c93c838c89454a3",
    "name": "Tokenlon",
    "symbol": "LON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13454/thumb/lon_logo.png?1696513217"
  },
  {
    "chainId": 42161,
    "address": "0x07e49d5de43dda6162fa28d24d5935c151875283",
    "name": "CVI",
    "symbol": "GOVI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13875/thumb/GOVI.png?1696513619"
  },
  {
    "chainId": 42161,
    "address": "0xc4be0798e5b5b1c15eda36d9b2d8c1a60717fa92",
    "name": "Adventurer Gold",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28427/thumb/anon-npc-pixel.png?1696527424"
  },
  {
    "chainId": 42161,
    "address": "0xa58663faef461761e44066ea26c1fcddf2927b80",
    "name": "Kommunitas",
    "symbol": "KOM",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/17483/thumb/1_f1S3h57YLT1e1cl8g7RJpw_2x.jpeg?1696517024"
  },
  {
    "chainId": 42161,
    "address": "0x753d224bcf9aafacd81558c32341416df61d3dac",
    "name": "Perpetual Protocol",
    "symbol": "PERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12381/thumb/60d18e06844a844ad75901a9_mark_only_03.png?1696512205"
  },
  {
    "chainId": 42161,
    "address": "0x2b089381f53525451fe5115f23e9d2cc92d7ff1d",
    "name": "Digital Reserve Currency",
    "symbol": "DRC",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/12802/thumb/DRC_Logo.jpg?1696512595"
  },
  {
    "chainId": 42161,
    "address": "0xf29fdf6b7bdffb025d7e6dfdf344992d2d16e249",
    "name": "Genius X",
    "symbol": "GENSX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30904/thumb/GX_short_logo.png?1696529749"
  },
  {
    "chainId": 42161,
    "address": "0x7ce746b45eabd0c4321538dec1b849c79a9a8476",
    "name": "DSLA Protocol",
    "symbol": "DSLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/6694/thumb/dsla_logo-squared_200x200.png?1696507035"
  },
  {
    "chainId": 42161,
    "address": "0xfa5ed56a203466cbbc2430a43c66b9d8723528e7",
    "name": "EURA",
    "symbol": "EURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19479/thumb/agEUR-4.png?1710726218"
  },
  {
    "chainId": 42161,
    "address": "0x0c880f6761f1af8d9aa9c466984b80dab9a8c9e8",
    "name": "Pendle",
    "symbol": "PENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15069/thumb/Pendle_Logo_Normal-03.png?1696514728"
  },
  {
    "chainId": 42161,
    "address": "0xeb466342c4d449bc9f53a865d5cb90586f405215",
    "name": "Axelar Bridged USDC",
    "symbol": "AXLUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/26476/thumb/uausdc_D_3x.png?1696525548"
  },
  {
    "chainId": 42161,
    "address": "0xbad58ed9b5f26a002ea250d7a60dc6729a4a2403",
    "name": "Paribus",
    "symbol": "PBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18410/thumb/paribus.PNG?1696517900"
  },
  {
    "chainId": 42161,
    "address": "0xd3ac016b1b8c80eeadde4d186a9138c9324e4189",
    "name": "Okcash",
    "symbol": "OK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/274/thumb/ok-logo-200px.png?1696501624"
  },
  {
    "chainId": 42161,
    "address": "0x078f358208685046a11c85e8ad32895ded33a249",
    "name": "Aave v3 WBTC",
    "symbol": "AWBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/32883/thumb/wbtc.png?1699719908"
  },
  {
    "chainId": 42161,
    "address": "0x7be5dd337cc6ce3e474f64e2a92a566445290864",
    "name": "OpenLeverage",
    "symbol": "OLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26098/thumb/256x256_OLE_Token_Logo.png?1696525189"
  },
  {
    "chainId": 42161,
    "address": "0x323665443cef804a3b5206103304bd4872ea4253",
    "name": "Verified USD",
    "symbol": "USDV",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32948/thumb/usdv_%281%29.png?1699933314"
  },
  {
    "chainId": 42161,
    "address": "0xe10d4a4255d2d35c9e23e2c4790e073046fbaf5c",
    "name": "LandX Governance Token",
    "symbol": "LNDX",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33565/thumb/LNDX-200.png?1702445947"
  },
  {
    "chainId": 42161,
    "address": "0x58e50e24d5160def294b6b6410d12c597054b79e",
    "name": "Fuzz Finance",
    "symbol": "FUZZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18395/thumb/Fuzz-200x200.png?1696517886"
  },
  {
    "chainId": 42161,
    "address": "0x1c22531aa9747d76fff8f0a43b37954ca67d28e0",
    "name": "Sumer Money suETH",
    "symbol": "SUETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33265/thumb/Sumer_Money_Logo.jpg?1701321407"
  },
  {
    "chainId": 42161,
    "address": "0x6ab707aca953edaefbc4fd23ba73294241490620",
    "name": "Aave v3 USDT",
    "symbol": "AUSDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32884/thumb/USDT.PNG?1699768611"
  },
  {
    "chainId": 42161,
    "address": "0x5979d7b546e38e414f7e9822514be443a4800529",
    "name": "Wrapped stETH",
    "symbol": "WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18834/thumb/wstETH.png?1696518295"
  },
  {
    "chainId": 42161,
    "address": "0x9b06f3c5de42d4623d7a2bd940ec735103c68a76",
    "name": "Volta Club",
    "symbol": "VOLTA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31602/thumb/volta-200x200.png?1696530418"
  },
  {
    "chainId": 42161,
    "address": "0xc218a06a17feb66b1a730889fb9624326fabff4b",
    "name": "ChessFish",
    "symbol": "CFSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34623/thumb/TokenLogo.png?1705504570"
  },
  {
    "chainId": 42161,
    "address": "0x8bf591eae535f93a242d5a954d3cde648b48a5a8",
    "name": "Sumer Money suUSD",
    "symbol": "SUUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33266/thumb/Sumer_Money_Logo.jpg?1701321416"
  },
  {
    "chainId": 42161,
    "address": "0x9dce8e754913d928eb39bc4fc3cf047e364f7f2c",
    "name": "Bloktopia",
    "symbol": "BLOK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18819/thumb/logo-bholdus-6.png?1696518281"
  },
  {
    "chainId": 42161,
    "address": "0xd4848211b699503c772aa1bc7d33b433c4242ac3",
    "name": "Equilibria Finance ePENDLE",
    "symbol": "EPENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33379/thumb/photo_2023-12-02_19-45-14.jpg?1701665986"
  },
  {
    "chainId": 42161,
    "address": "0x6688b00f0c23a4a546beaae51a7c90c439895d48",
    "name": "Tarot V1",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17881/thumb/tarot-200px.png?1696517403"
  },
  {
    "chainId": 42161,
    "address": "0x17a8541b82bf67e10b0874284b4ae66858cb1fd5",
    "name": "Possum",
    "symbol": "PSM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33596/thumb/PSM.png?1702521046"
  },
  {
    "chainId": 42161,
    "address": "0x1426cf37caa89628c4da2864e40cf75e6d66ac6b",
    "name": "Relay Chain",
    "symbol": "RELAY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17816/thumb/relay-logo-200.png?1696517336"
  },
  {
    "chainId": 42161,
    "address": "0xf763fa322dc58dee588252fafee5f448e863b633",
    "name": "Carbon Protocol",
    "symbol": "SWTH",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/3645/thumb/SWTH_Symbol_Origin.png?1696504327"
  },
  {
    "chainId": 42161,
    "address": "0x4f604735c1cf31399c6e711d5962b2b3e0225ad3",
    "name": "Glo Dollar",
    "symbol": "USDGLO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29319/thumb/glo_logo_coingecko.png?1696528270"
  },
  {
    "chainId": 42161,
    "address": "0x74ccbe53f77b08632ce0cb91d3a545bf6b8e0979",
    "name": "Fantom Bomb",
    "symbol": "BOMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24109/thumb/logo-blue.png?1696523301"
  },
  {
    "chainId": 42161,
    "address": "0x079504b86d38119f859c4194765029f692b7b7aa",
    "name": "Lyra Finance",
    "symbol": "LYRA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21490/thumb/Add-a-heading-26.png?1696520850"
  },
  {
    "chainId": 42161,
    "address": "0x10663b695b8f75647bd3ff0ff609e16d35bbd1ec",
    "name": "AmpliFi DAO",
    "symbol": "AGG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29137/thumb/original-icon-transparent.png?1696528098"
  },
  {
    "chainId": 42161,
    "address": "0x9e523234d36973f9e38642886197d023c88e307e",
    "name": "Darwinia Network",
    "symbol": "RING",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9443/thumb/RING.png?1696509535"
  },
  {
    "chainId": 42161,
    "address": "0x2fac624899a844e0628bfdcc70efcd25f6e90b95",
    "name": "Youwho",
    "symbol": "YOU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25353/thumb/youwho_200.png?1696524487"
  },
  {
    "chainId": 42161,
    "address": "0xec70dcb4a1efa46b8f2d97c310c9c4790ba5ffa8",
    "name": "Rocket Pool ETH",
    "symbol": "RETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20764/thumb/reth.png?1696520159"
  },
  {
    "chainId": 42161,
    "address": "0x7dd747d63b094971e6638313a6a2685e80c7fb2e",
    "name": "STFX",
    "symbol": "STFX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28631/thumb/stfx.png?1696527615"
  },
  {
    "chainId": 42161,
    "address": "0x59a729658e9245b0cf1f8cb9fb37945d2b06ea27",
    "name": "GenomesDAO GENE",
    "symbol": "GENE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20807/thumb/1637683704200x200.png?1696520200"
  },
  {
    "chainId": 42161,
    "address": "0x7698ac5d15bb3ba7185adcbff32a80ebd9d0709b",
    "name": "GenomesDAO GNOME",
    "symbol": "GNOME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20885/thumb/gnome.png?1696520278"
  },
  {
    "chainId": 42161,
    "address": "0xa68ec98d7ca870cf1dd0b00ebbb7c4bf60a8e74d",
    "name": "Biconomy",
    "symbol": "BICO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21061/thumb/biconomy_logo.jpg?1696520444"
  },
  {
    "chainId": 42161,
    "address": "0xafdefd913286a1cec75aca503938c6bba01a5445",
    "name": "JPG NFT Index",
    "symbol": "JPG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25071/thumb/JPG-token-logo-01.png?1696524219"
  },
  {
    "chainId": 42161,
    "address": "0xbcf339df10d78f2b44aa760ead0f715a7a7d7269",
    "name": "Guardian GUARD",
    "symbol": "GUARD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17995/thumb/LS_wolfDen_logo.0025_Light_200x200.png?1696517512"
  },
  {
    "chainId": 42161,
    "address": "0x6aa395f06986ea4efe0a4630c7865c1eb08d5e7e",
    "name": "Jarvis Reward",
    "symbol": "JRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10390/thumb/cfeii0y.png?1696510389"
  },
  {
    "chainId": 42161,
    "address": "0x9e758b8a98a42d612b3d38b66a22074dc03d7370",
    "name": "Symbiosis",
    "symbol": "SIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20805/thumb/SymbiosisFinance_logo-150x150.jpeg?1696520198"
  },
  {
    "chainId": 42161,
    "address": "0xb0b195aefa3650a6908f15cdac7d92f8a5791b0b",
    "name": "BOB",
    "symbol": "BOB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27266/thumb/Bob-logo.png?1696526318"
  },
  {
    "chainId": 42161,
    "address": "0x93b346b6bc2548da6a1e7d98e9a421b42541425b",
    "name": "Liquity USD",
    "symbol": "LUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14666/thumb/Group_3.png?1696514341"
  },
  {
    "chainId": 42161,
    "address": "0x24ef78c7092d255ed14a0281ac1800c359af3afe",
    "name": "Rabbit Wallet",
    "symbol": "RAB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29433/thumb/200x200.png?1696528381"
  },
  {
    "chainId": 42161,
    "address": "0x6a7661795c374c0bfc635934efaddff3a7ee23b6",
    "name": "DOLA",
    "symbol": "DOLA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14287/thumb/dola.png?1696513984"
  },
  {
    "chainId": 42161,
    "address": "0xe2105bea3819d3131b5b2306d44eb3271bfcfe0e",
    "name": "Fief",
    "symbol": "FIEF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23710/thumb/61f837d2703b649ab0213d45_Black_logo_-_no_background-p-500.png?1696522910"
  },
  {
    "chainId": 42161,
    "address": "0x43c25f828390de5a3648864eb485cc523e039e67",
    "name": "Hello Pets",
    "symbol": "PET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14354/thumb/hello_pets.jpg?1696514040"
  },
  {
    "chainId": 42161,
    "address": "0x1689a6e1f09658ff37d0bb131514e701045876da",
    "name": "ZooDAO",
    "symbol": "ZOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24305/thumb/Zt2BM_8D_400x400.jpg?1696523486"
  },
  {
    "chainId": 42161,
    "address": "0xeca14f81085e5b8d1c9d32dcb596681574723561",
    "name": "Spool",
    "symbol": "SPOOL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21532/thumb/spool.png?1696520891"
  },
  {
    "chainId": 42161,
    "address": "0xabd587f2607542723b17f14d00d99b987c29b074",
    "name": "SmarDex",
    "symbol": "SDEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29470/thumb/SDEX_logo_transparent_outside_240x240.png?1696930070"
  },
  {
    "chainId": 42161,
    "address": "0xd58d345fd9c82262e087d2d0607624b410d88242",
    "name": "Tellor Tributes",
    "symbol": "TRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9644/thumb/Blk_icon_current.png?1696509713"
  },
  {
    "chainId": 42161,
    "address": "0x188fb5f5ae5bbe4154d5778f2bbb2fb985c94d25",
    "name": "OpenBlox",
    "symbol": "OBX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26150/thumb/OBX_token-black_background_preview.png?1696525239"
  },
  {
    "chainId": 42161,
    "address": "0xe85b662fe97e8562f4099d8a1d5a92d4b453bf30",
    "name": "Thales",
    "symbol": "THALES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18388/thumb/CLVZJN_C_400x400.png?1696517879"
  },
  {
    "chainId": 42161,
    "address": "0xc701f86e6d242b97d3035f6c2cecaf8e7087e898",
    "name": "RunBlox",
    "symbol": "RUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26156/thumb/img_v2_37fc1523-649d-45bc-a75f-387f9de1bfch.png?1696525244"
  },
  {
    "chainId": 42161,
    "address": "0x178412e79c25968a32e89b11f63b33f733770c2a",
    "name": "Frax Ether",
    "symbol": "FRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28284/thumb/frxETH_icon.png?1696527284"
  },
  {
    "chainId": 42161,
    "address": "0xa61f74247455a40b01b0559ff6274441fafa22a3",
    "name": "Magpie",
    "symbol": "MGP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27972/thumb/MagpieLogo.png?1696526991"
  },
  {
    "chainId": 42161,
    "address": "0xc628534100180582e43271448098cb2c185795bd",
    "name": "Flashstake",
    "symbol": "FLASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13533/thumb/FLASH.png?1696513294"
  },
  {
    "chainId": 42161,
    "address": "0xd45e486a90ebb84e9336d371a35dcb021424b96c",
    "name": "Superpower Squad",
    "symbol": "SQUAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28466/thumb/SQUAD200X200.png?1696527460"
  },
  {
    "chainId": 42161,
    "address": "0xf80d589b3dbe130c270a69f1a69d050f268786df",
    "name": "Datamine FLUX",
    "symbol": "FLUX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11756/thumb/fluxres.png?1696511637"
  },
  {
    "chainId": 42161,
    "address": "0x6cc5b1fb8c2fd7af1d2858f916f274b8faac82e1",
    "name": "Ojamu",
    "symbol": "OJA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18947/thumb/ojamu-icon-PNK.png?1696518402"
  },
  {
    "chainId": 42161,
    "address": "0x95ab45875cffdba1e5f451b950bc2e42c0053f39",
    "name": "Staked Frax Ether",
    "symbol": "SFRXETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28285/thumb/sfrxETH_icon.png?1696527285"
  },
  {
    "chainId": 42161,
    "address": "0x56659245931cb6920e39c189d2a0e7dd0da2d57b",
    "name": "Impermax",
    "symbol": "IBEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27606/thumb/IqwOmX-c_400x400.jpeg?1696526637"
  },
  {
    "chainId": 42161,
    "address": "0x0cbd6fadcf8096cc9a43d90b45f65826102e3ece",
    "name": "CheckDot",
    "symbol": "CDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20370/thumb/token-200x200_%281%29.png?1696519781"
  },
  {
    "chainId": 42161,
    "address": "0xadf5dd3e51bf28ab4f07e684ecf5d00691818790",
    "name": "ICHI",
    "symbol": "ICHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13119/thumb/ICHI_%28Round%29.jpg?1696512907"
  },
  {
    "chainId": 42161,
    "address": "0x589d35656641d6ab57a545f08cf473ecd9b6d5f7",
    "name": "GYEN",
    "symbol": "GYEN",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/14191/thumb/icon_gyen_200_200.png?1696513909"
  },
  {
    "chainId": 42161,
    "address": "0xfb9fbcb328317123f5275cda30b6589d5841216b",
    "name": "Antfarm Token",
    "symbol": "ATF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28821/thumb/ATF.png?1696527797"
  },
  {
    "chainId": 42161,
    "address": "0x15a808ed3846d25e88ae868de79f1bcb1ac382b5",
    "name": "Metavault DAO",
    "symbol": "MVD",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/23899/thumb/MVD_Coin-simple.png?1696523099"
  },
  {
    "chainId": 42161,
    "address": "0x37596f20adef5cc9618c8b6ddfa9dcb6329cb0fd",
    "name": "Starfish Finance",
    "symbol": "SEAN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27533/thumb/SEAN_Token_icon.png?1696526570"
  },
  {
    "chainId": 42161,
    "address": "0x80bb30d62a16e1f2084deae84dc293531c3ac3a1",
    "name": "Granary",
    "symbol": "GRAIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29740/thumb/Grain.png?1696528670"
  },
  {
    "chainId": 42161,
    "address": "0xc9c4fd7579133701fa2769b6955e7e56bb386db1",
    "name": "Bridge Oracle",
    "symbol": "BRG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12512/thumb/brg.png?1696512327"
  },
  {
    "chainId": 42161,
    "address": "0x3405e88af759992937b84e58f2fe691ef0eea320",
    "name": "Frax Price Index Share",
    "symbol": "FPIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24944/thumb/FPIS_icon.png?1696524099"
  },
  {
    "chainId": 42161,
    "address": "0x3a18dcc9745edcd1ef33ecb93b0b6eba5671e7ca",
    "name": "Kujira",
    "symbol": "KUJI",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/20685/thumb/kuji-200x200.png?1696520085"
  },
  {
    "chainId": 42161,
    "address": "0x9483ab65847a447e36d21af1cab8c87e9712ff93",
    "name": "Wrapped USDR",
    "symbol": "WUSDR",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/28800/thumb/wUSDRlogo.png?1696527777"
  },
  {
    "chainId": 42161,
    "address": "0x18c11fd286c5ec11c3b683caa813b77f5163a122",
    "name": "Gains Network",
    "symbol": "GNS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19737/thumb/logo.png?1696519161"
  },
  {
    "chainId": 42161,
    "address": "0x371c7ec6d8039ff7933a2aa28eb827ffe1f52f07",
    "name": "JOE",
    "symbol": "JOE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17569/thumb/JoeToken.png?1703732357"
  },
  {
    "chainId": 42161,
    "address": "0xcaa38bcc8fb3077975bbe217acfaa449e6596a84",
    "name": "DAO Maker",
    "symbol": "DAO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13915/thumb/4.png?1696513656"
  },
  {
    "chainId": 42161,
    "address": "0x431402e8b9de9aa016c743880e04e517074d8cec",
    "name": "Hegic",
    "symbol": "HEGIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12454/thumb/new.png?1696512274"
  },
  {
    "chainId": 42161,
    "address": "0xe4dddfe67e7164b0fe14e218d80dc4c08edc01cb",
    "name": "Kyber Network Crystal",
    "symbol": "KNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14899/thumb/RwdVsGcw_400x400.jpg?1696514562"
  },
  {
    "chainId": 42161,
    "address": "0x289ba1701c2f088cf0faf8b3705246331cb8a839",
    "name": "Livepeer",
    "symbol": "LPT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7137/thumb/logo-circle-green.png?1696507437"
  },
  {
    "chainId": 42161,
    "address": "0x7b5eb3940021ec0e8e463d5dbb4b7b09a89ddf96",
    "name": "Wombat Exchange",
    "symbol": "WOM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26946/thumb/Wombat_Token.png?1696526001"
  },
  {
    "chainId": 42161,
    "address": "0xfb9e5d956d889d91a82737b9bfcdac1dce3e1449",
    "name": "Liquity",
    "symbol": "LQTY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14665/thumb/200-lqty-icon.png?1696514340"
  },
  {
    "chainId": 42161,
    "address": "0xcf985aba4647a432e60efceeb8054bbd64244305",
    "name": "EUROe Stablecoin",
    "symbol": "EUROE",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/28913/thumb/euroe-200x200-round.png?1696527889"
  },
  {
    "chainId": 42161,
    "address": "0x5f006745a9a192a7cd1236089f704f9b35d3b9cd",
    "name": "Arable Protocol",
    "symbol": "ACRE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23659/thumb/acre_token-02.png?1696522862"
  },
  {
    "chainId": 42161,
    "address": "0x2297aebd383787a160dd0d9f71508148769342e3",
    "name": "Bitcoin Avalanche Bridged  BTC b ",
    "symbol": "BTCB",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/26115/thumb/btcb.png?1696525205"
  },
  {
    "chainId": 42161,
    "address": "0x8eb270e296023e9d92081fdf967ddd7878724424",
    "name": "Aave v3 rETH",
    "symbol": "ARETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32885/thumb/reth.png?1699768947"
  },
  {
    "chainId": 42161,
    "address": "0x739e93844adabfd58b00b2bed540d1661d9af682",
    "name": "NoriGO ",
    "symbol": "GO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29300/thumb/go-token-medium.png?1696528252"
  },
  {
    "chainId": 42161,
    "address": "0x097bf766e427f5ccd306f21ba05632e0be849b0e",
    "name": "USDS Classic",
    "symbol": "USDS",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/7596/thumb/USDS_Classic_Standard.png?1702476175"
  },
  {
    "chainId": 42161,
    "address": "0x7d05d38e6109a3aeeebf0a570eb8f6856cb4b55e",
    "name": "MintMe com Coin",
    "symbol": "MINTME",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5127/thumb/MINTME_logo.png?1696505647"
  },
  {
    "chainId": 42161,
    "address": "0x61a1ff55c5216b636a294a07d77c6f4df10d3b56",
    "name": "ApeX",
    "symbol": "APEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25266/thumb/CxpMECpk_400x400_%281%29.png?1696524406"
  },
  {
    "chainId": 42161,
    "address": "0x88a269df8fe7f53e590c561954c52fccc8ec0cfb",
    "name": "Ninja Squad",
    "symbol": "NST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22248/thumb/nst-logo-200x200.png?1711559985"
  },
  {
    "chainId": 42161,
    "address": "0xd8724322f44e5c58d7a815f542036fb17dbbf839",
    "name": "Wrapped OETH",
    "symbol": "WOETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29734/thumb/wOETH.png?1696528664"
  },
  {
    "chainId": 42161,
    "address": "0x498c620c7c91c6eba2e3cd5485383f41613b7eb6",
    "name": "Alongside Crypto Market Index",
    "symbol": "AMKT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28496/thumb/22999.png?1696527488"
  },
  {
    "chainId": 42161,
    "address": "0x7d86f1eaff29f076576b2ff09ce3bcc7533fd2c5",
    "name": "Risitas  OLD ",
    "symbol": "RISITA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30333/thumb/200x200.png?1696529234"
  },
  {
    "chainId": 42161,
    "address": "0xda0a57b710768ae17941a9fa33f8b720c8bd9ddd",
    "name": "Marlin",
    "symbol": "POND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8903/thumb/200x200.png?1706115827"
  },
  {
    "chainId": 42161,
    "address": "0x5190f06eacefa2c552dc6bd5e763b81c73293293",
    "name": "Wombex",
    "symbol": "WMX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27844/thumb/WMX_logo.png?1696526863"
  },
  {
    "chainId": 42161,
    "address": "0x25d887ce7a35172c62febfd67a1856f20faebb00",
    "name": "Pepe",
    "symbol": "PEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29850/thumb/pepe-token.jpeg?1696528776"
  },
  {
    "chainId": 42161,
    "address": "0xf0cb2dc0db5e6c66b9a70ac27b06b878da017028",
    "name": "Olympus",
    "symbol": "OHM",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/14483/thumb/token_OHM_%281%29.png?1696514169"
  },
  {
    "chainId": 42161,
    "address": "0xadd5620057336f868eae78a451c503ae7b576bad",
    "name": "enqAI",
    "symbol": "ENQAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29908/thumb/icon.png?1702507913"
  },
  {
    "chainId": 42161,
    "address": "0x51a80238b5738725128d3a3e06ab41c1d4c05c74",
    "name": "unshETHing Token",
    "symbol": "USH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29337/thumb/unsheth_large_logo.png?1696528287"
  },
  {
    "chainId": 42161,
    "address": "0x2f11eeee0bf21e7661a22dbbbb9068f4ad191b86",
    "name": "Backed NIU Technologies",
    "symbol": "BNIU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31869/thumb/b-NIU-200x200.png?1696530681"
  },
  {
    "chainId": 42161,
    "address": "0xeb8e93a0c7504bffd8a8ffa56cd754c63aaebfe8",
    "name": "Overnight fi DAI ",
    "symbol": "DAI+",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29634/thumb/DAI_.png?1696528571"
  },
  {
    "chainId": 42161,
    "address": "0xb766039cc6db368759c1e56b79affe831d0cc507",
    "name": "Rocket Pool",
    "symbol": "RPL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2090/thumb/rocket_pool_%28RPL%29.png?1696503058"
  },
  {
    "chainId": 42161,
    "address": "0xf061956612b3dc79fd285d3d51bc128f2ea87740",
    "name": "YfDAI finance",
    "symbol": "YF-DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12385/thumb/1619048513068.png?1696512208"
  },
  {
    "chainId": 42161,
    "address": "0x0ae38f7e10a43b5b2fb064b42a2f4514cba909ef",
    "name": "unshETH Ether",
    "symbol": "UNSHETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30365/thumb/ush.png?1696529262"
  },
  {
    "chainId": 42161,
    "address": "0xe80772eaf6e2e18b651f160bc9158b2a5cafca65",
    "name": "Overnight fi USD ",
    "symbol": "USD+",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25757/thumb/USD__logo.png?1696524843"
  },
  {
    "chainId": 42161,
    "address": "0x23ee2343b892b1bb63503a4fabc840e0e2c6810f",
    "name": "Axelar",
    "symbol": "AXL",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/27277/thumb/V-65_xQ1_400x400.jpeg?1696526329"
  },
  {
    "chainId": 42161,
    "address": "0xb64e280e9d1b5dbec4accedb2257a87b400db149",
    "name": "Level",
    "symbol": "LVL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28628/thumb/Token.png?1696527613"
  },
  {
    "chainId": 42161,
    "address": "0x1debd73e752beaf79865fd6446b0c970eae7732f",
    "name": "Coinbase Wrapped Staked ETH",
    "symbol": "CBETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27008/thumb/cbeth.png?1709186989"
  },
  {
    "chainId": 42161,
    "address": "0x8626264b6a1b4e920905efd381002aba52ea0eea",
    "name": "BlackHat Coin",
    "symbol": "BLKC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/15987/thumb/logo_light.png?1696515600"
  },
  {
    "chainId": 42161,
    "address": "0x9dca587dc65ac0a043828b0acd946d71eb8d46c1",
    "name": "iFARM",
    "symbol": "IFARM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14472/thumb/ifarm.png?1696514159"
  },
  {
    "chainId": 42161,
    "address": "0xfdad8edc724277e975f4955d288c6eb5b20a3146",
    "name": "Nulswap",
    "symbol": "NSWAP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/30099/thumb/nswap-circ_%283%29.png?1696529023"
  },
  {
    "chainId": 42161,
    "address": "0x0c5fa0e07949f941a6c2c29a008252db1527d6ee",
    "name": "Opulous",
    "symbol": "OPUL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16548/thumb/opulous.PNG?1696516110"
  },
  {
    "chainId": 42161,
    "address": "0xffa188493c15dfaf2c206c97d8633377847b6a52",
    "name": "Wefi Finance",
    "symbol": "WEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30540/thumb/wefi.png?1696529412"
  },
  {
    "chainId": 42161,
    "address": "0xe05a08226c49b636acf99c40da8dc6af83ce5bb3",
    "name": "Ankr Staked ETH",
    "symbol": "ANKRETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13403/thumb/aETHc.png?1696513165"
  },
  {
    "chainId": 42161,
    "address": "0x81de4945807bb31425362f8f7109c18e3dc4f8f0",
    "name": "Bolide",
    "symbol": "BLID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25548/thumb/5XvLQULP_400x400.jpg?1703622595"
  },
  {
    "chainId": 42161,
    "address": "0x13278cd824d33a7adb9f0a9a84aca7c0d2deebf7",
    "name": "Tarot",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31800/thumb/TAROT.jpg?1696530615"
  },
  {
    "chainId": 42161,
    "address": "0xa334884bf6b0a066d553d19e507315e839409e62",
    "name": "Ethos Reserve Note",
    "symbol": "ERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29744/thumb/ERN200x200.png?1696528676"
  },
  {
    "chainId": 42161,
    "address": "0x2ac2b254bc18cd4999f64773a966e4f4869c34ee",
    "name": "Penpie",
    "symbol": "PNP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30760/thumb/PNP_Token.png?1696529629"
  },
  {
    "chainId": 42161,
    "address": "0xe6045890b20945d00e6f3c01878265c03c5435d3",
    "name": "Impossible Finance Launchpad",
    "symbol": "IDIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17803/thumb/IDIA.png?1696517325"
  },
  {
    "chainId": 42161,
    "address": "0xcafcd85d8ca7ad1e1c6f82f651fa15e33aefd07b",
    "name": "WOO",
    "symbol": "WOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12921/thumb/WOO_Logos_2023_Profile_Pic_WOO.png?1696512709"
  },
  {
    "chainId": 42161,
    "address": "0x32eb7902d4134bf98a28b963d26de779af92a212",
    "name": "Dopex Rebate",
    "symbol": "RDPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16659/thumb/rDPX_200x200_Coingecko.png?1696516221"
  },
  {
    "chainId": 42161,
    "address": "0x6c2c06790b3e3e3c38e12ee22f8183b37a13ee55",
    "name": "Dopex",
    "symbol": "DPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16652/thumb/DPX_%281%29.png?1696516213"
  },
  {
    "chainId": 42161,
    "address": "0x560363bda52bc6a44ca6c8c9b4a5fadbda32fa60",
    "name": "Seedify fund",
    "symbol": "SFUND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14614/thumb/Favicon_Icon.png?1696514292"
  },
  {
    "chainId": 42161,
    "address": "0xea986d33ef8a20a96120ecc44dbdd49830192043",
    "name": "Auctus",
    "symbol": "AUC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2165/thumb/Auc_Discord_Avatar1.png?1696503126"
  },
  {
    "chainId": 42161,
    "address": "0xbfa641051ba0a0ad1b0acf549a89536a0d76472e",
    "name": "Badger",
    "symbol": "BADGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13287/thumb/badger_dao_logo.jpg?1696513059"
  },
  {
    "chainId": 42161,
    "address": "0x69eb4fa4a2fbd498c257c57ea8b7655a2559a581",
    "name": "DODO",
    "symbol": "DODO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12651/thumb/dodo_logo.png?1696512458"
  },
  {
    "chainId": 42161,
    "address": "0xa0b862f60edef4452f25b4160f177db44deb6cf1",
    "name": "Gnosis",
    "symbol": "GNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/662/thumb/logo_square_simple_300px.png?1696501854"
  },
  {
    "chainId": 42161,
    "address": "0xc3ae0333f0f34aa734d5493276223d95b8f9cb37",
    "name": "DXdao",
    "symbol": "DXD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11148/thumb/dxdao.png?1696511083"
  },
  {
    "chainId": 42161,
    "address": "0xf4d48ce3ee1ac3651998971541badbb9a14d7234",
    "name": "Cream",
    "symbol": "CREAM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11976/thumb/Cream.png?1696511834"
  },
  {
    "chainId": 42161,
    "address": "0xd4d42f0b6def4ce0383636770ef773390d85c61a",
    "name": "Sushi",
    "symbol": "SUSHI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12271/thumb/512x512_Logo_no_chop.png?1696512101"
  },
  {
    "chainId": 42161,
    "address": "0xda10009cbd5d07dd0cecc66161fc93d7c9000da1",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 42161,
    "address": "0x51fc0f6660482ea73330e414efd7808811a57fa2",
    "name": "Premia",
    "symbol": "PREMIA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13962/thumb/apple-touch-icon.png?1696513698"
  },
  {
    "chainId": 42161,
    "address": "0xef888bca6ab6b1d26dbec977c455388ecd794794",
    "name": "Rari Governance",
    "symbol": "RGT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12900/thumb/Rari_Logo_Transparent.png?1696512688"
  },
  {
    "chainId": 42161,
    "address": "0x3509f19581afedeff07c53592bc0ca84e4855475",
    "name": "xDollar Stablecoin",
    "symbol": "XUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16291/thumb/xUSD-web-transparent.png?1696515890"
  },
  {
    "chainId": 42161,
    "address": "0xa970af1a584579b618be4d69ad6f73459d112f95",
    "name": "sUSD",
    "symbol": "SUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5013/thumb/sUSD.png?1696505546"
  },
  {
    "chainId": 42161,
    "address": "0xa72159fc390f0e3c6d415e658264c7c4051e9b87",
    "name": "Tracer DAO",
    "symbol": "TCR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18271/thumb/tracer_logo.png?1696517765"
  },
  {
    "chainId": 42161,
    "address": "0xde903e2712288a1da82942dddf2c20529565ac30",
    "name": "Swapr",
    "symbol": "SWPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18740/thumb/swapr.jpg?1696518206"
  },
  {
    "chainId": 42161,
    "address": "0x7ba4a00d54a07461d9db2aef539e91409943adc9",
    "name": "Stake DAO",
    "symbol": "SDT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13724/thumb/stakedao_logo.jpg?1696513468"
  },
  {
    "chainId": 42161,
    "address": "0x99f40b01ba9c469193b360f72740e416b17ac332",
    "name": "MATH",
    "symbol": "MATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11335/thumb/2020-05-19-token-200.png?1696511257"
  },
  {
    "chainId": 42161,
    "address": "0x46d0ce7de6247b0a95f67b43b589b4041bae7fbe",
    "name": "Loopring",
    "symbol": "LRC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/913/thumb/LRC.png?1696502034"
  },
  {
    "chainId": 42161,
    "address": "0x7cb16cb78ea464ad35c8a50abf95dff3c9e09d5d",
    "name": "0xBitcoin",
    "symbol": "0XBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/4454/thumb/0xbtc.png?1696505045"
  },
  {
    "chainId": 42161,
    "address": "0xa7aa2921618e3d63da433829d448b58c9445a4c3",
    "name": "Rhino fi",
    "symbol": "DVF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16414/thumb/rhinologo.png?1697736807"
  },
  {
    "chainId": 42161,
    "address": "0x4425742f1ec8d98779690b5a3a6276db85ddc01a",
    "name": "The Doge NFT",
    "symbol": "DOG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18111/thumb/Doge.png?1696517615"
  },
  {
    "chainId": 42161,
    "address": "0x8038f3c971414fd1fc220ba727f2d4a0fc98cb65",
    "name": "dHEDGE DAO",
    "symbol": "DHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12508/thumb/dht.png?1696512323"
  },
  {
    "chainId": 42161,
    "address": "0xaeaeed23478c3a4b798e4ed40d8b7f41366ae861",
    "name": "Ankr Network",
    "symbol": "ANKR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4324/thumb/U85xTl2.png?1696504928"
  },
  {
    "chainId": 42161,
    "address": "0x59d9356e565ab3a36dd77763fc0d87feaf85508c",
    "name": "Mountain Protocol USD",
    "symbol": "USDM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31719/thumb/usdm.png?1696530540"
  },
  {
    "chainId": 42161,
    "address": "0xb1bc21f748ae2be95674876710bc6d78235480e0",
    "name": "Hord",
    "symbol": "HORD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14972/thumb/HORD_TOKEN_%281%29_1.png?1705414356"
  },
  {
    "chainId": 42161,
    "address": "0xae6aab43c4f3e0cea4ab83752c278f8debaba689",
    "name": "dForce",
    "symbol": "DF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9709/thumb/xlGxxIjI_400x400.jpg?1696509776"
  },
  {
    "chainId": 42161,
    "address": "0xf97f4df75117a78c1a5a0dbb814af92458539fb4",
    "name": "Chainlink",
    "symbol": "LINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/877/thumb/chainlink-new-logo.png?1696502009"
  },
  {
    "chainId": 42161,
    "address": "0xfa7f8980b0f1e64a2062791cc3b0871572f1f7f0",
    "name": "Uniswap",
    "symbol": "UNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12504/thumb/uni.jpg?1696512319"
  },
  {
    "chainId": 42161,
    "address": "0xaf88d065e77c8cc2239327c5edb3a432268e5831",
    "name": "USDC",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/6319/thumb/usdc.png?1696506694"
  },
  {
    "chainId": 42161,
    "address": "0x17fc002b466eec40dae837fc4be5c67993ddbd6f",
    "name": "Frax",
    "symbol": "FRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13422/thumb/FRAX_icon.png?1696513182"
  },
  {
    "chainId": 42161,
    "address": "0x9d2f299715d94d8a7e6f5eaa8e654e8c74a988a7",
    "name": "Frax Share",
    "symbol": "FXS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13423/thumb/Frax_Shares_icon.png?1696513183"
  },
  {
    "chainId": 42161,
    "address": "0x5429706887fcb58a595677b73e9b0441c25d993d",
    "name": "UniDex",
    "symbol": "UNIDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13178/thumb/unidx.png?1696512961"
  },
  {
    "chainId": 42161,
    "address": "0x8d9ba570d6cb60c7e3e0f31343efe75ab8e65fb1",
    "name": "Governance OHM",
    "symbol": "GOHM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21129/thumb/token_wsOHM_logo.png?1696520508"
  },
  {
    "chainId": 42161,
    "address": "0x641441c631e2f909700d2f41fd87f0aa6a6b4edb",
    "name": "dForce USD",
    "symbol": "USX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17422/thumb/usx_32.png?1696516969"
  },
  {
    "chainId": 42161,
    "address": "0x6694340fc020c5e6b96567843da2df01b2ce1eb6",
    "name": "Stargate Finance",
    "symbol": "STG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24413/thumb/STG_LOGO.png?1696523595"
  },
  {
    "chainId": 42161,
    "address": "0x319f865b287fcc10b30d8ce6144e8b6d1b476999",
    "name": "Cartesi",
    "symbol": "CTSI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11038/thumb/Cartesi_Logo.png?1696510982"
  },
  {
    "chainId": 42161,
    "address": "0xf0dfad1817b5ba73726b02ab34dd4b4b00bcd392",
    "name": "Minato",
    "symbol": "MNTO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24622/thumb/MNTO_200x200.png?1696523794"
  },
  {
    "chainId": 42161,
    "address": "0x3bd2dfd03bc7c3011ed7fb8c4d0949b382726cee",
    "name": "Roobee",
    "symbol": "ROOBEE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/8791/thumb/Group_11.png?1696508946"
  },
  {
    "chainId": 42161,
    "address": "0x21e60ee73f17ac0a411ae5d690f908c3ed66fe12",
    "name": "Deri Protocol",
    "symbol": "DERI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13931/thumb/200vs200.jpg?1696513670"
  },
  {
    "chainId": 42161,
    "address": "0x68ead55c258d6fa5e46d67fc90f53211eab885be",
    "name": "Popcorn",
    "symbol": "POP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21438/thumb/pop-1_200_x_200.png?1696520801"
  },
  {
    "chainId": 42161,
    "address": "0xc72633f995e98ac3bb8a89e6a9c4af335c3d6e44",
    "name": "Omnisea",
    "symbol": "OSEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26475/thumb/293837892_407994084681555_3167689470652146992_n.png?1696525547"
  },
  {
    "chainId": 42161,
    "address": "0x115d8bf0a53e751f8a472f88d587944ec1c8ca6d",
    "name": "pTokens BTC",
    "symbol": "PBTC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25861/thumb/wMTpRljt_400x400.png?1696524945"
  },
  {
    "chainId": 42161,
    "address": "0x5441695f4445e40900b4c4b0fb3ed2b9e51601a6",
    "name": "ARTH",
    "symbol": "ARTH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16876/thumb/Ik5dhOq.png?1696516444"
  },
  {
    "chainId": 42161,
    "address": "0x3b6564b5da73a41d3a66e6558a98fd0e9e1e77ad",
    "name": "Unitus",
    "symbol": "UTS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33178/thumb/IMG_0051.jpeg?1700924333"
  },
  {
    "chainId": 42161,
    "address": "0xf3c091ed43de9c270593445163a41a876a0bb3dd",
    "name": "Orbs",
    "symbol": "ORBS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4630/thumb/Orbs.jpg?1696505200"
  },
  {
    "chainId": 42161,
    "address": "0x8b5e4c9a188b1a187f2d1e80b1c2fb17fa2922e1",
    "name": "GoldenBoys",
    "symbol": "GOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31353/thumb/goldenboys.png?1696530170"
  },
  {
    "chainId": 42161,
    "address": "0x11bbf12363dc8375b78d2719395d505f52a02f68",
    "name": "Router Protocol",
    "symbol": "ROUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13709/thumb/route_token_200x200-19.png?1696513454"
  },
  {
    "chainId": 42161,
    "address": "0x6b021b3f68491974be6d4009fee61a4e3c708fd6",
    "name": "Fuse",
    "symbol": "FUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10347/thumb/fuse.png?1696510348"
  },
  {
    "chainId": 42161,
    "address": "0xdce765f021410b3266aa0053c93cb4535f1e12e0",
    "name": "peg eUSD",
    "symbol": "PEUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31521/thumb/peUSD_200x200.png?1696530331"
  },
  {
    "chainId": 42161,
    "address": "0xb688ba096b7bb75d7841e47163cd12d18b36a5bf",
    "name": "mPendle",
    "symbol": "MPENDLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31918/thumb/mPendle.png?1696530726"
  },
  {
    "chainId": 42161,
    "address": "0xbc011a12da28e8f0f528d9ee5e7039e22f91cf18",
    "name": "Swell Ethereum",
    "symbol": "SWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30326/thumb/_lB7zEtS_400x400.jpg?1696529227"
  },
  {
    "chainId": 42161,
    "address": "0xaf20f5f19698f1d19351028cd7103b63d30de7d7",
    "name": "Wagmi",
    "symbol": "WAGMI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31887/thumb/wagmi_token_logo.png?1696530698"
  },
  {
    "chainId": 42161,
    "address": "0xb1084db8d3c05cebd5fa9335df95ee4b8a0edc30",
    "name": "Overnight fi USDT ",
    "symbol": "USDT+",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/30168/thumb/USDT_.png?1696529088"
  },
  {
    "chainId": 42161,
    "address": "0x9cfb13e6c11054ac9fcb92ba89644f30775436e4",
    "name": "Bridged Wrapped stETH  Axelar ",
    "symbol": "AXL-WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32223/thumb/steth-wei_D_3x.png?1696926841"
  },
  {
    "chainId": 42161,
    "address": "0x00e1724885473b63bce08a9f0a52f35b0979e35a",
    "name": "OATH",
    "symbol": "OATH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24075/thumb/OATH.png?1698051304"
  },
  {
    "chainId": 42161,
    "address": "0x4e51ac49bc5e2d87e0ef713e9e5ab2d71ef4f336",
    "name": "Celo  Wormhole ",
    "symbol": "CELO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32400/thumb/celo_wh.png?1698055352"
  },
  {
    "chainId": 42161,
    "address": "0x82e64f49ed5ec1bc6e43dad4fc8af9bb3a2312ee",
    "name": "Aave v3 DAI",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32886/thumb/dai.png?1699769446"
  },
  {
    "chainId": 42161,
    "address": "0xf7a0dd3317535ec4f4d29adf9d620b3d8d5d5069",
    "name": "Staked Ethos Reserve Note",
    "symbol": "STERN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31522/thumb/stERN.png?1696530332"
  },
  {
    "chainId": 42161,
    "address": "0xf329e36c7bf6e5e86ce2150875a84ce77f477375",
    "name": "Aave v3 AAVE",
    "symbol": "AAAVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32887/thumb/aave.png?1699773604"
  },
  {
    "chainId": 42161,
    "address": "0x191c10aa4af7c30e871e70c95db0e4eb77237530",
    "name": "Aave v3 LINK",
    "symbol": "ALINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32888/thumb/link.png?1699773900"
  },
  {
    "chainId": 42161,
    "address": "0x38d693ce1df5aadf7bc62595a37d667ad57922e5",
    "name": "Aave v3 FRAX",
    "symbol": "AFRAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32904/thumb/FRAX.png?1699802848"
  },
  {
    "chainId": 42161,
    "address": "0xd978f8489e1245568704407a479a71fcce2afe8f",
    "name": "ApeSwap",
    "symbol": "BANANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14870/thumb/banana.png?1696514534"
  },
  {
    "chainId": 42161,
    "address": "0xc14a7747cfec02cfea62e72bb93538de6b2078e6",
    "name": "WaweSwaps Global Token",
    "symbol": "GBL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34205/thumb/gbl.png?1704275447"
  },
  {
    "chainId": 42161,
    "address": "0x9e20461bc2c4c980f62f1b279d71734207a6a356",
    "name": "OmniCat",
    "symbol": "OMNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33917/thumb/2023-12-21_21.39.10.jpg?1703321588"
  },
  {
    "chainId": 42161,
    "address": "0x2598c30330d5771ae9f983979209486ae26de875",
    "name": "Any Inu",
    "symbol": "AI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34126/thumb/anyinulogo200.png?1704174269"
  },
  {
    "chainId": 42161,
    "address": "0xedd6ca8a4202d4a36611e2fff109648c4863ae19",
    "name": "MahaDAO",
    "symbol": "MAHA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13404/thumb/MAHA_Token.png?1696513166"
  },
  {
    "chainId": 42161,
    "address": "0x080f6aed32fc474dd5717105dba5ea57268f46eb",
    "name": "Synapse",
    "symbol": "SYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18024/thumb/synapse_social_icon.png?1696517540"
  },
  {
    "chainId": 42161,
    "address": "0x848e0ba28b637e8490d88bae51fa99c87116409b",
    "name": "Agave",
    "symbol": "AGVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14146/thumb/agve.png?1696513865"
  },
  {
    "chainId": 42161,
    "address": "0x9623063377ad1b27544c965ccd7342f7ea7e88c7",
    "name": "The Graph",
    "symbol": "GRT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13397/thumb/Graph_Token.png?1696513159"
  },
  {
    "chainId": 42161,
    "address": "0x82af49447d8a07e3bd95bd0d56f35241523fbab1",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 42161,
    "address": "0x3a8b787f78d775aecfeea15706d4221b40f345ab",
    "name": "Celer Network",
    "symbol": "CELR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4379/thumb/Celr.png?1696504978"
  },
  {
    "chainId": 42161,
    "address": "0x11cdb42b0eb46d95f990bedd4695a6e3fa034978",
    "name": "Curve DAO",
    "symbol": "CRV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12124/thumb/Curve.png?1696511967"
  },
  {
    "chainId": 42161,
    "address": "0x4e352cf164e64adcbad318c3a1e222e9eba4ce42",
    "name": "MUX Protocol",
    "symbol": "MCB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11796/thumb/mux.jpg?1696511672"
  },
  {
    "chainId": 42161,
    "address": "0x3e6648c5a70a150a88bce65f4ad4d506fe15d2af",
    "name": "Spell",
    "symbol": "SPELL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15861/thumb/abracadabra-3.png?1696515477"
  },
  {
    "chainId": 42161,
    "address": "0x82e3a8f066a6989666b031d916c43672085b1582",
    "name": "yearn finance",
    "symbol": "YFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11849/thumb/yearn.jpg?1696511720"
  },
  {
    "chainId": 42161,
    "address": "0x965772e0e9c84b6f359c8597c891108dcf1c5b1a",
    "name": "Pickle Finance",
    "symbol": "PICKLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12435/thumb/0M4W6Yr6_400x400.jpg?1696512255"
  },
  {
    "chainId": 42161,
    "address": "0x040d1edc9569d4bab2d15287dc5a4f10f56a56b8",
    "name": "Balancer",
    "symbol": "BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11683/thumb/Balancer.png?1696511572"
  },
  {
    "chainId": 42161,
    "address": "0x354a6da3fcde098f8389cad84b0182725c6c91de",
    "name": "Compound",
    "symbol": "COMP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10775/thumb/COMP.png?1696510737"
  },
  {
    "chainId": 42161,
    "address": "0xb965029343d55189c25a7f3e0c9394dc0f5d41b1",
    "name": "Indexed Finance",
    "symbol": "NDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13546/thumb/indexed-light.74bb5471.png?1696513306"
  },
  {
    "chainId": 42161,
    "address": "0x10010078a54396f62c96df8532dc2b4847d47ed3",
    "name": "Hundred Finance",
    "symbol": "HND",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18445/thumb/hnd.PNG?1696517933"
  },
  {
    "chainId": 42161,
    "address": "0xfea7a6a0b346362bf88a9e4a88416b77a57d6c2a",
    "name": "Magic Internet Money",
    "symbol": "MIM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16786/thumb/mimlogopng.png?1696516358"
  },
  {
    "chainId": 42161,
    "address": "0xdb298285fe4c5410b05390ca80e8fbe9de1f259b",
    "name": "handle fi",
    "symbol": "FOREX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18533/thumb/handle.fiFOREXLogoDark200x200.png?1696518013"
  },
  {
    "chainId": 42161,
    "address": "0x377c6e37633e390aef9afb4f5e0b16689351eed4",
    "name": "ZYX",
    "symbol": "ZYX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11964/thumb/zyx.png?1696511823"
  },
  {
    "chainId": 42161,
    "address": "0x55ff62567f09906a85183b866df84bf599a4bf70",
    "name": "Kromatika",
    "symbol": "KROM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20541/thumb/KROM_Transparent.png?1696519948"
  },
  {
    "chainId": 42161,
    "address": "0xf0a5717ec0883ee56438932b0fe4a20822735fba",
    "name": "xToken",
    "symbol": "XTK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14089/thumb/xtk.png?1696513811"
  },
  {
    "chainId": 42161,
    "address": "0x9b3fa2a7c3eb36d048a5d38d81e7fafc6bc47b25",
    "name": "Aluna",
    "symbol": "ALN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14379/thumb/uaLoLU8c_400x400_%281%29.png?1696514071"
  },
  {
    "chainId": 42161,
    "address": "0x9fb9a33956351cf4fa040f65a13b835a3c8764e3",
    "name": "Multichain",
    "symbol": "MULTI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22087/thumb/1_Wyot-SDGZuxbjdkaOeT2-A.png?1696521430"
  },
  {
    "chainId": 42161,
    "address": "0x488cc08935458403a0458e45e20c0159c8ab2c92",
    "name": "Futureswap",
    "symbol": "FST",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14520/thumb/futureswap_logo.png?1696514206"
  },
  {
    "chainId": 42161,
    "address": "0x6f5401c53e2769c858665621d22ddbf53d8d27c5",
    "name": "Connect Financial",
    "symbol": "CNFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13592/thumb/cf-logo-iconic-black.png?1696513344"
  },
  {
    "chainId": 42161,
    "address": "0x326c33fd1113c1f29b35b4407f3d6312a8518431",
    "name": "Strips Finance",
    "symbol": "STRP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18327/thumb/Logo-Strips-200-x-200px---without-words.png?1696517818"
  },
  {
    "chainId": 42161,
    "address": "0xee9801669c6138e84bd50deb500827b776777d28",
    "name": "O3 Swap",
    "symbol": "O3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15460/thumb/o3.png?1696515107"
  },
  {
    "chainId": 42161,
    "address": "0x123389c2f0e9194d9ba98c21e63c375b67614108",
    "name": "EthereumMax",
    "symbol": "EMAX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15540/thumb/EMAX-Coin-Final2000x.png?1696515181"
  },
  {
    "chainId": 42161,
    "address": "0x0a3bb08b3a15a19b4de82f8acfc862606fb69a2d",
    "name": "iZUMi Bond USD",
    "symbol": "IUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25388/thumb/iusd-logo-symbol-10k%E5%A4%A7%E5%B0%8F.png?1696524521"
  },
  {
    "chainId": 42161,
    "address": "0x39a49bc5017fc668299cd32e734c9269acc35295",
    "name": "Phonon DAO",
    "symbol": "PHONON",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22308/thumb/ezgif-2-e7fb84364d.png?1696521653"
  },
  {
    "chainId": 42161,
    "address": "0xc74fe4c715510ec2f8c61d70d397b32043f55abe",
    "name": "Mycelium",
    "symbol": "MYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26874/thumb/MYC_Token.png?1696525933"
  },
  {
    "chainId": 42161,
    "address": "0x13ad51ed4f1b7e9dc168d8a00cb3f4ddd85efa60",
    "name": "Lido DAO",
    "symbol": "LDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13573/thumb/Lido_DAO.png?1696513326"
  },
  {
    "chainId": 42161,
    "address": "0xc5102fe9359fd9a28f877a67e36b0f050d81a3cc",
    "name": "Hop Protocol",
    "symbol": "HOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25445/thumb/hop.png?1696524577"
  },
  {
    "chainId": 42161,
    "address": "0xecf2adaff1de8a512f6e8bfe67a2c836edb25da3",
    "name": "Wonderful Memories",
    "symbol": "WMEMO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/22392/thumb/wMEMO.png?1696521735"
  },
  {
    "chainId": 42161,
    "address": "0x2f2a2543b76a4166549f7aab2e75bef0aefc5b0f",
    "name": "Wrapped Bitcoin",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7598/thumb/wrapped_bitcoin_wbtc.png?1696507857"
  },
  {
    "chainId": 42161,
    "address": "0x180f7cf38805d1be95c7632f653e26b0838e2969",
    "name": "XDEFI",
    "symbol": "XDEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19524/thumb/FilledLogo.png?1696518958"
  },
  {
    "chainId": 42161,
    "address": "0x1310952bc5594852459ee45bfd0df70b34ac5509",
    "name": "Parifi",
    "symbol": "PRF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34710/thumb/parifi200x200.png?1705894380"
  },
  {
    "chainId": 42161,
    "address": "0x1db06f39c14d813d7b1ccb275a93f5b052de1cac",
    "name": "Xave",
    "symbol": "XAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28299/thumb/Xave_logo_gradient.png?1696527298"
  },
  {
    "chainId": 42161,
    "address": "0xd7b675cd5c84a13d1d0f84509345530f6421b57c",
    "name": "Corridor Finance",
    "symbol": "OOOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36057/thumb/corridor-logo-without-text-8bit.jpg?1710404509"
  },
  {
    "chainId": 42161,
    "address": "0x847503fbf003ce8b005546aa3c03b80b7c2f9771",
    "name": "Byte",
    "symbol": "BYTE",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33527/thumb/Byte200.jpeg?1702138460"
  },
  {
    "chainId": 42161,
    "address": "0xb299751b088336e165da313c33e3195b8c6663a6",
    "name": "StarHeroes",
    "symbol": "STAR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34147/thumb/Screenshot_2024-03-08_180444.png?1709892309"
  },
  {
    "chainId": 42161,
    "address": "0x2416092f143378750bb29b79ed961ab195cceea5",
    "name": "Renzo Restaked ETH",
    "symbol": "EZETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34753/thumb/eth_renzo_logo_%281%29.png?1705956747"
  },
  {
    "chainId": 42161,
    "address": "0x2680e82fb8beb5a153a67fe687ffa67abb6b9013",
    "name": "Swarm Markets",
    "symbol": "SMT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17488/thumb/swarm-SMT-token-symbol_200x200.png?1696517029"
  },
  {
    "chainId": 42161,
    "address": "0xed5740209fcf6974d6f3a5f11e295b5e468ac27c",
    "name": "KEWL EXCHANGE",
    "symbol": "KWL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36883/thumb/yamuk.png?1712649510"
  },
  {
    "chainId": 42161,
    "address": "0x16f1967565aad72dd77588a332ce445e7cef752b",
    "name": "crow with knife",
    "symbol": "CAW",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/36067/thumb/200px.png?1710405601"
  },
  {
    "chainId": 100,
    "address": "0xe91d153e0b41518a2ce8dd3d7944fa863463a97d",
    "name": "XDAI",
    "symbol": "XDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11062/thumb/Identity-Primary-DarkBG.png?1696511004"
  },
  {
    "chainId": 100,
    "address": "0x2086f52651837600180de173b09470f54ef74910",
    "name": "Balancer Stable USD",
    "symbol": "STABAL3",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32112/thumb/bal3.png?1696561317"
  },
  {
    "chainId": 100,
    "address": "0x18e9262e68cc6c6004db93105cc7c001bb103e49",
    "name": "Raid",
    "symbol": "RAID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18133/thumb/raid_200_oswlvz.png?1696517636"
  },
  {
    "chainId": 100,
    "address": "0xb5d592f85ab2d955c25720ebe6ff8d4d1e1be300",
    "name": "Particle",
    "symbol": "PRTCLE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14395/thumb/unknown.png?1696514085"
  },
  {
    "chainId": 100,
    "address": "0xddafbb505ad214d7b80b1f830fccc89b60fb7a83",
    "name": "Gnosis xDAI Bridged USDC  Gnosis ",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35300/thumb/USDC_Icon.png?1708083625"
  },
  {
    "chainId": 100,
    "address": "0x3a97704a1b25f08aa230ae53b352e2e72ef52843",
    "name": "Agave",
    "symbol": "AGVE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14146/thumb/agve.png?1696513865"
  },
  {
    "chainId": 100,
    "address": "0x6c76971f98945ae98dd7d4dfca8711ebea946ea6",
    "name": "Bridged Wrapped stETH  Gnosis ",
    "symbol": "WSTETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33017/thumb/wStETH.png?1700190250"
  },
  {
    "chainId": 100,
    "address": "0x4ecaba5870353805a9f068101a40e0f32ed605c6",
    "name": "Gnosis xDai Bridged USDT  Gnosis ",
    "symbol": "USDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35072/thumb/logo.png?1707292488"
  },
  {
    "chainId": 100,
    "address": "0xedbc7449a9b594ca4e053d9737ec5dc4cbccbfb2",
    "name": "Aave v3 EURe",
    "symbol": "AEURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32929/thumb/monerium.png?1699826642"
  },
  {
    "chainId": 100,
    "address": "0x38fb649ad3d6ba1113be5f57b927053e97fc5bf7",
    "name": "xDai Native Comb",
    "symbol": "XCOMB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/16012/thumb/logo.png?1696515623"
  },
  {
    "chainId": 100,
    "address": "0x4291f029b9e7acb02d49428458cf6fceac545f81",
    "name": "1Hive Water",
    "symbol": "WATER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26089/thumb/water_logo.png?1696525180"
  },
  {
    "chainId": 100,
    "address": "0x71850b7e9ee3f13ab46d67167341e4bdc905eef9",
    "name": "Honey",
    "symbol": "HNY",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12895/thumb/hnys.png?1696512683"
  },
  {
    "chainId": 100,
    "address": "0x63e62989d9eb2d37dfdb1f93a22f063635b07d51",
    "name": "Minerva Wallet",
    "symbol": "MIVA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14732/thumb/MIVA-Token_200x200.png?1696514401"
  },
  {
    "chainId": 100,
    "address": "0xaf204776c7245bf4147c2612bf6e5972ee483701",
    "name": "Savings xDAI",
    "symbol": "SDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32066/thumb/sDAI_Logo_%281%29.png?1696530863"
  },
  {
    "chainId": 100,
    "address": "0x54e4cb2a4fa0ee46e3d9a98d13bea119666e09f6",
    "name": "Gnosis Bridged EURC  Gnosis ",
    "symbol": "EURC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/36068/thumb/EURC_Icon.png?1710406183"
  },
  {
    "chainId": 100,
    "address": "0x3a3e9715018d80916740e8ac300713fdf6614d19",
    "name": "hiveWater",
    "symbol": "HIVEWATER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30424/thumb/hivewater_transparent.png?1696529312"
  },
  {
    "chainId": 100,
    "address": "0xa1fa064a85266e2ca82dee5c5ccec84df445760e",
    "name": "Aave v3 GNO",
    "symbol": "AGNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32928/thumb/gno.png?1699826574"
  },
  {
    "chainId": 100,
    "address": "0x48b1b0d077b4919b65b4e4114806dd803901e1d9",
    "name": "Etherisc DIP",
    "symbol": "DIP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/4586/thumb/dip.png?1696505164"
  },
  {
    "chainId": 100,
    "address": "0x4537e328bf7e4efa29d05caea260d7fe26af9d74",
    "name": "Uniswap",
    "symbol": "UNI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12504/thumb/uni.jpg?1696512319"
  },
  {
    "chainId": 100,
    "address": "0xdfc20ae04ed70bd9c7d720f449eedae19f659d65",
    "name": "Badger",
    "symbol": "BADGER",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13287/thumb/badger_dao_logo.jpg?1696513059"
  },
  {
    "chainId": 100,
    "address": "0x6a8cb6714b1ee5b471a7d2ec4302cb4f5ff25ec2",
    "name": "Energy Web",
    "symbol": "EWT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10886/thumb/R9gQTJV__400x400.png?1696510839"
  },
  {
    "chainId": 100,
    "address": "0x07279343ce61c11cb091142c8a20a1f8194d0ca8",
    "name": "Zenland",
    "symbol": "ZENF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29477/thumb/tokenicon200px.png?1696528421"
  },
  {
    "chainId": 100,
    "address": "0xbf65bfcb5da067446cee6a706ba3fe2fb1a9fdfd",
    "name": "yearn finance",
    "symbol": "YFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11849/thumb/yearn.jpg?1696511720"
  },
  {
    "chainId": 100,
    "address": "0x524b969793a64a602342d89bc2789d43a016b13a",
    "name": "Donut",
    "symbol": "DONUT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/7538/thumb/Donut.png?1696507804"
  },
  {
    "chainId": 100,
    "address": "0xabef652195f98a91e490f047a5006b71c85f058d",
    "name": "crvUSD",
    "symbol": "CRVUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30118/thumb/crvusd.jpeg?1696529040"
  },
  {
    "chainId": 100,
    "address": "0x270de58f54649608d316faa795a9941b355a2bd0",
    "name": "Freedom Reserve",
    "symbol": "FR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/13342/thumb/J6uNL2FS_400x400.jpg?1696513109"
  },
  {
    "chainId": 100,
    "address": "0xb0c5f3100a4d9d9532a4cfd68c55f1ae8da987eb",
    "name": "DAOhaus",
    "symbol": "HAUS",
    "decimals": 35,
    "logoURI": "https://assets.coingecko.com/coins/images/14551/thumb/jN3kkqke_400x400.png?1696514234"
  },
  {
    "chainId": 100,
    "address": "0x8c88ea1fd60462ef7004b9e288afcb4680a3c50c",
    "name": "0xMonero",
    "symbol": "0XMR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11035/thumb/0xmnr.PNG?1696510979"
  },
  {
    "chainId": 100,
    "address": "0xbbcb0356bb9e6b3faa5cbf9e5f36185d53403ac9",
    "name": "Backed Coinbase Global",
    "symbol": "BCOIN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31872/thumb/b-COIN-200x200.png?1696530684"
  },
  {
    "chainId": 100,
    "address": "0xce11e14225575945b8e6dc0d4f2dd4c570f79d9f",
    "name": "Autonolas",
    "symbol": "OLAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31099/thumb/OLAS-token.png?1696529930"
  },
  {
    "chainId": 100,
    "address": "0x58b9cb810a68a7f3e1e4f8cb45d1b9b3c79705e8",
    "name": "Connext",
    "symbol": "NEXT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31293/thumb/connext.png?1696530113"
  },
  {
    "chainId": 100,
    "address": "0x84e2c67cbefae6b5148fca7d02b341b12ff4b5bb",
    "name": "Swash",
    "symbol": "SWASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18774/thumb/Swash_icon_GP.png?1712869197"
  },
  {
    "chainId": 100,
    "address": "0x21a42669643f45bc0e086b8fc2ed70c23d67509d",
    "name": "ShapeShift FOX",
    "symbol": "FOX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9988/thumb/FOX.png?1696510025"
  },
  {
    "chainId": 100,
    "address": "0x2d5563da42b06fbbf9c67b7dc073cf6a7842239e",
    "name": "Jarvis Synthetic Swiss Franc",
    "symbol": "JCHF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15727/thumb/jCHF.png?1696515353"
  },
  {
    "chainId": 100,
    "address": "0x532801ed6f82fffd2dab70a19fc2d7b2772c4f4b",
    "name": "Swapr",
    "symbol": "SWPR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18740/thumb/swapr.jpg?1696518206"
  },
  {
    "chainId": 100,
    "address": "0x431d5dff03120afa4bdf332c61a6e1766ef37bdb",
    "name": "JPY Coin",
    "symbol": "JPYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25971/thumb/2023jpyc.png?1696525049"
  },
  {
    "chainId": 100,
    "address": "0x4b1e2c2762667331bc91648052f646d1b0d35984",
    "name": "EURA",
    "symbol": "EURA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19479/thumb/agEUR-4.png?1710726218"
  },
  {
    "chainId": 100,
    "address": "0x7ecf26cd9a36990b8ea477853663092333f59979",
    "name": "Perpetual Protocol",
    "symbol": "PERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12381/thumb/60d18e06844a844ad75901a9_mark_only_03.png?1696512205"
  },
  {
    "chainId": 100,
    "address": "0x7ef541e2a22058048904fe5744f9c7e4c57af717",
    "name": "Balancer",
    "symbol": "BAL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11683/thumb/Balancer.png?1696511572"
  },
  {
    "chainId": 100,
    "address": "0xc6b7aca6de8a6044e0e32d0c841a89244a10d284",
    "name": "Aave v3 USDC",
    "symbol": "AUSDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/32847/thumb/usdc_%281%29.png?1699619355"
  },
  {
    "chainId": 100,
    "address": "0xdbf3ea6f5bee45c02255b2c26a16f300502f68da",
    "name": "Swarm",
    "symbol": "BZZ",
    "decimals": 16,
    "logoURI": "https://assets.coingecko.com/coins/images/16509/thumb/Circle_Orange_onWhite.png?1696516071"
  },
  {
    "chainId": 100,
    "address": "0xcb444e90d8198415266c6a2724b7900fb12fc56e",
    "name": "Monerium EUR emoney",
    "symbol": "EURE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/23354/thumb/eur.png?1696522569"
  },
  {
    "chainId": 100,
    "address": "0xaad66432d27737ecf6ed183160adc5ef36ab99f2",
    "name": "Tellor Tributes",
    "symbol": "TRB",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9644/thumb/Blk_icon_current.png?1696509713"
  },
  {
    "chainId": 100,
    "address": "0x1e2c4fb7ede391d116e6b41cd0608260e8801d59",
    "name": "Backed CSPX Core S P 500",
    "symbol": "BCSPX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31891/thumb/b-CSPX-200x200.png?1696530702"
  },
  {
    "chainId": 100,
    "address": "0x004626a008b1acdc4c74ab51644093b155e59a23",
    "name": "Angle Staked EURA",
    "symbol": "STEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32036/thumb/stEUR-x4.png?1696530832"
  },
  {
    "chainId": 100,
    "address": "0xd589f00fa2eb83367f732ab3cda92ee0940389cf",
    "name": "Fuse",
    "symbol": "FUSE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/10347/thumb/fuse.png?1696510348"
  },
  {
    "chainId": 100,
    "address": "0x9fb1d52596c44603198fb0aee434fac3a679f702",
    "name": "Jarvis Synthetic Euro",
    "symbol": "JEUR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15725/thumb/jEUR.png?1696515352"
  },
  {
    "chainId": 100,
    "address": "0x52d134c6db5889fad3542a09eaf7aa90c0fdf9e4",
    "name": "Backed IBTA   Treasury Bond 1 3yr",
    "symbol": "BIBTA",
    "decimals": 15,
    "logoURI": "https://assets.coingecko.com/coins/images/31880/thumb/b-IBTA-200x200.png?1696530692"
  },
  {
    "chainId": 100,
    "address": "0x20c64dee8fda5269a78f2d5bdba861ca1d83df7a",
    "name": "Backed HIGH   High Yield Corp Bond",
    "symbol": "BHIGH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31868/thumb/b-HIGH-200x200.png?1696530680"
  },
  {
    "chainId": 100,
    "address": "0x2f123cf3f37ce3328cc9b5b8415f9ec5109b45e7",
    "name": "Backed GOVIES 0 6 months EURO",
    "symbol": "BC3M",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31870/thumb/b-C3-M-200x200.png?1696530682"
  },
  {
    "chainId": 100,
    "address": "0x2f11eeee0bf21e7661a22dbbbb9068f4ad191b86",
    "name": "Backed NIU Technologies",
    "symbol": "BNIU",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31869/thumb/b-NIU-200x200.png?1696530681"
  },
  {
    "chainId": 100,
    "address": "0xa818f1b57c201e092c4a2017a91815034326efd1",
    "name": "Aave v3 WETH",
    "symbol": "AWETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32882/thumb/WETH_%281%29.png?1699716492"
  },
  {
    "chainId": 100,
    "address": "0x0000206329b97db379d5e1bf586bbdb969c63274",
    "name": "USDA",
    "symbol": "USDA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34510/thumb/agUSD-coingecko.png?1705288392"
  },
  {
    "chainId": 100,
    "address": "0xade6057fcafa57d6d51ffa341c64ce4814995995",
    "name": "Backed ZPR1   1 3 Month T Bill",
    "symbol": "BZPR1",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32867/thumb/b-ZPR1-200x200.png?1699671603"
  },
  {
    "chainId": 100,
    "address": "0xd0dd6cef72143e22cced4867eb0d5f2328715533",
    "name": "Aave v3 DAI",
    "symbol": "ADAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32886/thumb/dai.png?1699769446"
  },
  {
    "chainId": 100,
    "address": "0x7a5c3860a77a8dc1b225bd46d0fb2ac1c6d191bc",
    "name": "Aave v3 sDAI",
    "symbol": "ASDAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32891/thumb/sdai.png?1699785595"
  },
  {
    "chainId": 100,
    "address": "0x3f95aa88ddbb7d9d484aa3d482bf0a80009c52c9",
    "name": "Backed ERNX   Bond",
    "symbol": "BERNX",
    "decimals": 18
  },
  {
    "chainId": 100,
    "address": "0x6a023ccd1ff6f2045c3309768ead9e68f978f6e1",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 100,
    "address": "0x0116e28b43a358162b96f70b4de14c98a4465f25",
    "name": "UNCX Network",
    "symbol": "UNCX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12871/thumb/uncx.png?1696512659"
  },
  {
    "chainId": 100,
    "address": "0xe2e73a1c69ecf83f464efce6a5be353a37ca09b2",
    "name": "Chainlink",
    "symbol": "LINK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/877/thumb/chainlink-new-logo.png?1696502009"
  },
  {
    "chainId": 100,
    "address": "0xa899e4476ff5da39dcd9ddec7a6e2c651d8614bf",
    "name": "Bitcoin BR",
    "symbol": "BTCBR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21205/thumb/btcbr.png?1696520580"
  },
  {
    "chainId": 100,
    "address": "0xb90d6bec20993be5d72a5ab353343f7a0281f158",
    "name": "DXdao",
    "symbol": "DXD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11148/thumb/dxdao.png?1696511083"
  },
  {
    "chainId": 100,
    "address": "0xb17d999e840e0c1b157ca5ab8039bd958b5fa317",
    "name": "Etho Protocol",
    "symbol": "ETHO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/5194/thumb/IconOnly_Transparent_NoBuffer.png?1712452538"
  },
  {
    "chainId": 100,
    "address": "0xb7d311e2eb55f2f68a9440da38e7989210b9a05e",
    "name": "STAKE",
    "symbol": "STAKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/11061/thumb/xdai.png?1696511003"
  },
  {
    "chainId": 100,
    "address": "0x364b5975385b4de226cf450df82d7ed69902ac5a",
    "name": "TORG",
    "symbol": "TORG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17596/thumb/TORG_Logo_200x200.png?1696517129"
  },
  {
    "chainId": 100,
    "address": "0x8b8407c6184f1f0fd1082e83d6a3b8349caced12",
    "name": "Circuits of Value",
    "symbol": "COVAL",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/588/thumb/coval-logo.png?1696501792"
  },
  {
    "chainId": 100,
    "address": "0x83ff60e2f93f8edd0637ef669c69d5fb4f64ca8e",
    "name": "BrightID",
    "symbol": "BRIGHT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18415/thumb/bright.PNG?1696517905"
  },
  {
    "chainId": 100,
    "address": "0xfa57aa7beed63d03aaf85ffd1753f5f6242588fb",
    "name": "Mt Pelerin Shares",
    "symbol": "MPS",
    "decimals": 0,
    "logoURI": "https://assets.coingecko.com/coins/images/11471/thumb/MPS.png?1696511378"
  },
  {
    "chainId": 100,
    "address": "0x9c58bacc331c9aa871afd802db6379a98e80cedb",
    "name": "Gnosis",
    "symbol": "GNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/662/thumb/logo_square_simple_300px.png?1696501854"
  },
  {
    "chainId": 100,
    "address": "0x4f4f9b8d5b4d0dc10506e5551b0513b61fd59e75",
    "name": "Giveth",
    "symbol": "GIV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21792/thumb/GIVToken_200x200.png?1696521145"
  },
  {
    "chainId": 100,
    "address": "0x8e5bbbb09ed1ebde8674cda39a0c169401db4252",
    "name": "Wrapped Bitcoin",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7598/thumb/wrapped_bitcoin_wbtc.png?1696507857"
  },
  {
    "chainId": 100,
    "address": "0x256eb8a51f382650b2a1e946b8811953640ee47d",
    "name": "Streamr",
    "symbol": "DATA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17869/thumb/DATA_new_symbol_3x.png?1696517392"
  },
  {
    "chainId": 100,
    "address": "0x3f56e0c36d275367b8c502090edf38289b3dea0d",
    "name": "MAI",
    "symbol": "MIMATIC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/15264/thumb/mimatic-red.png?1696514916"
  },
  {
    "chainId": 100,
    "address": "0x177127622c4a00f3d409b75571e12cb3c8973d3c",
    "name": "CoW Protocol",
    "symbol": "COW",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/24384/thumb/cow.png?1696523567"
  },
  {
    "chainId": 100,
    "address": "0xec3f3e6d7907acda3a7431abd230196cda3fbb19",
    "name": "Ethix",
    "symbol": "ETHIX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3031/thumb/ETHIX_icon_256x256-256.png?1696503766"
  },
  {
    "chainId": 100,
    "address": "0xfbdd194376de19a88118e84e279b977f165d01b8",
    "name": "DeHive",
    "symbol": "DHV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14926/thumb/logo_200x200.png?1696514587"
  },
  {
    "chainId": 100,
    "address": "0x97edc0e345fbbbd8460847fcfa3bc2a13bf8641f",
    "name": "DAOSquare",
    "symbol": "RICE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/19084/thumb/tw5gu1dW_400x400.jpeg?1696518537"
  },
  {
    "chainId": 100,
    "address": "0x417602f4fbdd471a431ae29fb5fe0a681964c11b",
    "name": "JPY Coin v1",
    "symbol": "JPYC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17277/thumb/WoZ8rruL_400x400.png?1696516831"
  },
  {
    "chainId": 100,
    "address": "0xd3d47d5578e55c880505dc40648f7f9307c3e7a8",
    "name": "DeFi Pulse Index",
    "symbol": "DPI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12465/thumb/defi_pulse_index_set.png?1696512284"
  },
  {
    "chainId": 100,
    "address": "0x37b60f4e9a31a64ccc0024dce7d0fd07eaa0f7b3",
    "name": "Kleros",
    "symbol": "PNK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/3833/thumb/kleros.png?1696504500"
  },
  {
    "chainId": 100,
    "address": "0xeeeeeb57642040be42185f49c52f7e9b38f8eeee",
    "name": "Elk Finance",
    "symbol": "ELK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/17813/thumb/elk.png?1696517333"
  },
  {
    "chainId": 100,
    "address": "0x44fa8e6f47987339850636f88629646662444217",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 100,
    "address": "0xc5102fe9359fd9a28f877a67e36b0f050d81a3cc",
    "name": "Hop Protocol",
    "symbol": "HOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25445/thumb/hop.png?1696524577"
  },
  {
    "chainId": 100,
    "address": "0x0f76d32cdccdcbd602a55af23eaf58fd1ee17245",
    "name": "Backed ERNA   Bond",
    "symbol": "BERNA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36757/thumb/b-ERNA-200x200.png?1712245846"
  },
  {
    "chainId": 100,
    "address": "0xdd96b45877d0e8361a4ddb732da741e97f3191ff",
    "name": "Bridged BUSD",
    "symbol": "BUSD",
    "decimals": 18
  },
  {
    "chainId": 324,
    "address": "0xf6d9a093a1c69a152d87e269a7d909e9d76b1815",
    "name": "Superlauncher",
    "symbol": "LAUNCH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14817/thumb/SL-V3-1024.png?1696514485"
  },
  {
    "chainId": 324,
    "address": "0x90059c32eeeb1a2aa1351a58860d98855f3655ad",
    "name": "ONEZ",
    "symbol": "ONEZ",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32970/thumb/onez.jpg?1700056810"
  },
  {
    "chainId": 324,
    "address": "0xcab3f741fa54e79e34753b95717b23018332b8ac",
    "name": "Tiny Era Shard",
    "symbol": "TES",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31204/thumb/TES250.png?1696530031"
  },
  {
    "chainId": 324,
    "address": "0x60e7fe7ae4461b535bb9eb40c20424c7c61063d0",
    "name": "Zeekwifhat",
    "symbol": "ZWIF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36371/thumb/zeekCG.png?1711338998"
  },
  {
    "chainId": 324,
    "address": "0xbfb4b5616044eded03e5b1ad75141f0d9cb1499b",
    "name": "zkDoge",
    "symbol": "ZKDOGE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29619/thumb/zkdoge.png?1696528555"
  },
  {
    "chainId": 324,
    "address": "0x5165ec33b491d7b67260b3143f96bb4ac4736398",
    "name": "Long",
    "symbol": "LONG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36497/thumb/Long_logo_-_200x200_final.png?1711616558"
  },
  {
    "chainId": 324,
    "address": "0x5756a28e2aae01f600fc2c01358395f5c1f8ad3a",
    "name": "veSync",
    "symbol": "VS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30611/thumb/veSync-Logo-PNG-200X200.png?1696529480"
  },
  {
    "chainId": 324,
    "address": "0x47ef4a5641992a72cfd57b9406c9d9cefee8e0c4",
    "name": "zkApes Token",
    "symbol": "ZAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30316/thumb/zkApes.png?1696529217"
  },
  {
    "chainId": 324,
    "address": "0x6d27b6252b980ba4350a1e349a3b684237d5e986",
    "name": "Hush cash",
    "symbol": "HUSH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29932/thumb/coingecko.png?1696528860"
  },
  {
    "chainId": 324,
    "address": "0x06662147240414cc0dc25e6414100750d346bc44",
    "name": "Flashdash",
    "symbol": "FLASHDASH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33274/thumb/logo200.png?1701329106"
  },
  {
    "chainId": 324,
    "address": "0x60143f39b54a731ac790708c8719211878e638c3",
    "name": "Pepera",
    "symbol": "PEPERA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30199/thumb/PEPERA.png?1696529114"
  },
  {
    "chainId": 324,
    "address": "0xed4040fd47629e7c8fbb7da76bb50b3e7695f0f2",
    "name": "Holdstation",
    "symbol": "HOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32726/thumb/HOLD.png?1699018663"
  },
  {
    "chainId": 324,
    "address": "0xc3a3aa99b80845d22cc41dea3bdb9b2a9ebe199a",
    "name": "ZKDX",
    "symbol": "ZKDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34802/thumb/logo-png.png?1706092934"
  },
  {
    "chainId": 324,
    "address": "0x941fc398d9faebdd9f311011541045a1d66c748e",
    "name": "Nop App",
    "symbol": "NOP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36376/thumb/191.png?1711342125"
  },
  {
    "chainId": 324,
    "address": "0xcd2cfa60f04f3421656d6eebee122b3973b3f60c",
    "name": "MetaElfLand",
    "symbol": "MELD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31469/thumb/MELD_LOGO.png?1696530282"
  },
  {
    "chainId": 324,
    "address": "0x3355df6d4c9c3035724fd0e3914de96a5a83aaf4",
    "name": "zkSync Bridged USDC  zkSync ",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35262/thumb/USDC_Icon.png?1708008941"
  },
  {
    "chainId": 324,
    "address": "0x8aaaf6e6167825829ab29f260f246afe742fb243",
    "name": "zkCULT",
    "symbol": "ZCULT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29807/thumb/zkCULT_Logo_200x200.jpg?1696528736"
  },
  {
    "chainId": 324,
    "address": "0xaf08a9d918f16332f22cf8dc9abe9d9e14ddcbc2",
    "name": "Holdstation USDC",
    "symbol": "HSUSDC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32646/thumb/hsUSDC.png?1698893340"
  },
  {
    "chainId": 324,
    "address": "0xe027d939f7de6f521675907cf086f59e4d75b876",
    "name": "Solarcoin",
    "symbol": "SLR",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/152/thumb/solarcoin.png?1696501524"
  },
  {
    "chainId": 324,
    "address": "0x0231b3de40b6b3bdd28dcef037f1b7a3fcf5a95a",
    "name": "Zynergy",
    "symbol": "ZYN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29674/thumb/logo_200px.png?1696528608"
  },
  {
    "chainId": 324,
    "address": "0x31c2c031fdc9d33e974f327ab0d9883eae06ca4a",
    "name": "zkSwap Finance",
    "symbol": "ZF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31633/thumb/zf.png?1696530449"
  },
  {
    "chainId": 324,
    "address": "0x71fd7dfa7db7094e0f857ad3040f1afef76fef85",
    "name": "Bullets",
    "symbol": "BLT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30475/thumb/Logo_BLT_Token_%28200x200%29.png?1696529362"
  },
  {
    "chainId": 324,
    "address": "0x7b3e1236c39ddd2e61cf6da6ac6d11193238ccb0",
    "name": "zkEra Finance",
    "symbol": "ZKE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33928/thumb/zke.png?1703409212"
  },
  {
    "chainId": 324,
    "address": "0x0c6eaaab86e8374a91e3f42c726b6fd1abacb54c",
    "name": "Zino Pet",
    "symbol": "ZPET",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33753/thumb/%E5%B8%81%E6%A0%87.png?1703024400"
  },
  {
    "chainId": 324,
    "address": "0x2b64237277c605d07f17b96f9627712340c32981",
    "name": "GemSwap",
    "symbol": "ZGEM",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29630/thumb/zgem.png?1696528568"
  },
  {
    "chainId": 324,
    "address": "0x94466dcfe241db963af1bc9741a94daa87ab2169",
    "name": "zkLotto",
    "symbol": "ZKLOTTO",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/29694/thumb/logo-sm.png?1696528627"
  },
  {
    "chainId": 324,
    "address": "0x26ac1d9945f65392b8e4e6b895969b5c01a7b414",
    "name": "Kannagi Finance",
    "symbol": "KANA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30713/thumb/200_200.png?1696529584"
  },
  {
    "chainId": 324,
    "address": "0x160e07e42adbc1fce92d505b579bcd8a3fbea77d",
    "name": "Dracula Fi",
    "symbol": "FANG",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31028/thumb/fang.png?1696529864"
  },
  {
    "chainId": 324,
    "address": "0x959ab3394246669914bddeaeb50f8ac85648615e",
    "name": "zkLaunchpad",
    "symbol": "ZKPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29720/thumb/zkpad.jpeg?1696528650"
  },
  {
    "chainId": 324,
    "address": "0x7d3cfc3e9eee2aa5feff52d6193a0dbb08492279",
    "name": "Tektias",
    "symbol": "TEKTIAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35685/thumb/T.jpg?1709527547"
  },
  {
    "chainId": 324,
    "address": "0x244c238325fc1bdf6eded726ee1b47d55895d944",
    "name": "Zorro",
    "symbol": "ZORRO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34381/thumb/ZORRO_TP.png?1704777748"
  },
  {
    "chainId": 324,
    "address": "0x3a4fea0cf14c18c21975ff08efe11c08cccefe10",
    "name": "Orbitpad",
    "symbol": "OPAD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33113/thumb/logo200px.png?1700731139"
  },
  {
    "chainId": 324,
    "address": "0x7ecf006f7c45149b457e9116392279cc8a630f14",
    "name": "zkSwap",
    "symbol": "ZKSP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30582/thumb/zkswap.jpeg?1696529446"
  },
  {
    "chainId": 324,
    "address": "0xc8ec5b0627c794de0e4ea5d97ad9a556b361d243",
    "name": "Whisper",
    "symbol": "WISP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29650/thumb/wisp_200.png?1696528586"
  },
  {
    "chainId": 324,
    "address": "0x10d967f46b06580c4a87b05c78f04e4df25c0db0",
    "name": "Holdstation Utility GOLD",
    "symbol": "UGOLD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32647/thumb/uGOLD.png?1698893439"
  },
  {
    "chainId": 324,
    "address": "0x3a34fa9a1288597ad6c1da709f001d37fef8b19e",
    "name": "SyncDex",
    "symbol": "SYDX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29746/thumb/Logo200x200.png?1696528678"
  },
  {
    "chainId": 324,
    "address": "0xcdb7d260c107499c80b4b748e8331c64595972a1",
    "name": "Karat",
    "symbol": "KAT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31834/thumb/coin1.png?1696530647"
  },
  {
    "chainId": 324,
    "address": "0x2039bb4116b4efc145ec4f0e2ea75012d6c0f181",
    "name": "Celer Bridged BUSD  zkSync ",
    "symbol": "BUSD",
    "decimals": 18
  },
  {
    "chainId": 324,
    "address": "0x971b79ef5e76088af5e52c6d6aa3276c2190e45c",
    "name": "Synapse Network",
    "symbol": "ZKSNP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34990/thumb/blobid0_%281%29.png?1707451143"
  },
  {
    "chainId": 324,
    "address": "0xc6dac3a53d5d6de9d1d05aa6e28b8e9e41722601",
    "name": "Wrapped LIBERTAS OMNIBUS",
    "symbol": "LIBERTAS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35181/thumb/libertas.png?1707796268"
  },
  {
    "chainId": 324,
    "address": "0x5f7cbcb391d33988dad74d6fd683aadda1123e4d",
    "name": "ReactorFusion",
    "symbol": "RF",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29975/thumb/RF_Token_logo.png?1696528900"
  },
  {
    "chainId": 324,
    "address": "0x90eea899185105d583d04b7bdbfb672fce902a53",
    "name": "ZKPepe",
    "symbol": "ZKPEPE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29870/thumb/241.png?1696528795"
  },
  {
    "chainId": 324,
    "address": "0x47260090ce5e83454d5f05a0abbb2c953835f777",
    "name": "SpaceFi",
    "symbol": "SPACE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30822/thumb/aOwhiUUo_400x400.png?1696529676"
  },
  {
    "chainId": 324,
    "address": "0xe7895ed01a1a6aacf1c2e955af14e7cf612e7f9d",
    "name": "Veno Finance Staked ETH",
    "symbol": "LETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/34945/thumb/LETH_%281%29.png?1708020787"
  },
  {
    "chainId": 324,
    "address": "0x2141d7fe06a1d69c016fc638ba75b6ef92fa1435",
    "name": "zkSync id",
    "symbol": "ZKID",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30115/thumb/logo.png?1696529037"
  },
  {
    "chainId": 324,
    "address": "0x144b83555d8a3119b0a69a7bc2f0a0388308fee3",
    "name": "AutoAir AI",
    "symbol": "AAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/36165/thumb/AutoAir_AI_-_200x200.png?1712108994"
  },
  {
    "chainId": 324,
    "address": "0xa995ad25ce5eb76972ab356168f5e1d9257e4d05",
    "name": "Koi",
    "symbol": "KOI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/35766/thumb/Koi_logo.png?1709782399"
  },
  {
    "chainId": 324,
    "address": "0x5aea5775959fbc2557cc8789bc1bf90a239d9a91",
    "name": "WETH",
    "symbol": "WETH",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/2518/thumb/weth.png?1696503332"
  },
  {
    "chainId": 324,
    "address": "0x493257fd37edb34451f62edf8d2a0c418852ba4c",
    "name": "Bridged USDT",
    "symbol": "USDT",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/35001/thumb/logo.png?1706959346"
  },
  {
    "chainId": 324,
    "address": "0xafe4ca0bbe6215cbda12857e723134bc3809f766",
    "name": "MacaronSwap",
    "symbol": "MCRN",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14633/thumb/macaron.png?1696514311"
  },
  {
    "chainId": 324,
    "address": "0x503234f203fc7eb888eec8513210612a43cf6115",
    "name": "Liquity USD",
    "symbol": "LUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14666/thumb/Group_3.png?1696514341"
  },
  {
    "chainId": 324,
    "address": "0x0bf4cb727b3f8092534d793893b2cc3348963dbf",
    "name": "Derp",
    "symbol": "DERP",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33069/thumb/derpdex_%281%29.png?1700793428"
  },
  {
    "chainId": 324,
    "address": "0x4b9eb6c0b6ea15176bbf62841c6b2a8a398cb656",
    "name": "Dai",
    "symbol": "DAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/9956/thumb/Badge_Dai.png?1696509996"
  },
  {
    "chainId": 324,
    "address": "0x9929bcac4417a21d7e6fc86f6dae1cc7f27a2e41",
    "name": "Domani Protocol",
    "symbol": "DEXTF",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/12634/thumb/0qgT0aMu_400x400.jpg?1696512442"
  },
  {
    "chainId": 324,
    "address": "0x5fc44e95eaa48f9eb84be17bd3ac66b6a82af709",
    "name": "Grai",
    "symbol": "GRAI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30427/thumb/GRAI_Token.png?1696529315"
  },
  {
    "chainId": 324,
    "address": "0x9e22d758629761fc5708c171d06c2fabb60b5159",
    "name": "WOO",
    "symbol": "WOO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/12921/thumb/WOO_Logos_2023_Profile_Pic_WOO.png?1696512709"
  },
  {
    "chainId": 324,
    "address": "0x787c09494ec8bcb24dcaf8659e7d5d69979ee508",
    "name": "Maverick Protocol",
    "symbol": "MAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30850/thumb/MAV_Logo.png?1696529701"
  },
  {
    "chainId": 324,
    "address": "0x2d20b8891f2f9ed0ebf1b179b2279f936dec9282",
    "name": "Syncus",
    "symbol": "SYNC",
    "decimals": 9,
    "logoURI": "https://assets.coingecko.com/coins/images/33573/thumb/Syncus.png?1702450708"
  },
  {
    "chainId": 324,
    "address": "0xc059767cb62f003e863f9e7bd1fc813beff9693c",
    "name": "TiUSD",
    "symbol": "TIUSD",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30697/thumb/TIUDS.png?1696529566"
  },
  {
    "chainId": 324,
    "address": "0x0e97c7a0f8b2c9885c8ac9fc6136e829cbc21d42",
    "name": "Mute",
    "symbol": "MUTE",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14331/thumb/MUTE.png?1696514019"
  },
  {
    "chainId": 324,
    "address": "0xbe9f8c0d6f0fd7e46cdacca340747ea2f247991d",
    "name": "Impermax",
    "symbol": "IBEX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/27606/thumb/IqwOmX-c_400x400.jpeg?1696526637"
  },
  {
    "chainId": 324,
    "address": "0xd6c008fb375648972206518fb0b1a1b65f371d05",
    "name": "Omnisea",
    "symbol": "OSEA",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/26475/thumb/293837892_407994084681555_3167689470652146992_n.png?1696525547"
  },
  {
    "chainId": 324,
    "address": "0x7f2fd959013eec5144269ac6edd0015cb10968fc",
    "name": "Tarot",
    "symbol": "TAROT",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31800/thumb/TAROT.jpg?1696530615"
  },
  {
    "chainId": 324,
    "address": "0x16a9494e257703797d747540f01683952547ee5b",
    "name": "iZUMi Finance",
    "symbol": "IZI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/21791/thumb/izumi-logo-symbol.png?1696521144"
  },
  {
    "chainId": 324,
    "address": "0x6ee46cb7cd2f15ee1ec9534cf29a5b51c83283e6",
    "name": "Kyber Network Crystal",
    "symbol": "KNC",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/14899/thumb/RwdVsGcw_400x400.jpg?1696514562"
  },
  {
    "chainId": 324,
    "address": "0xd03465338226ea0178337f4abb16fdd6df529f57",
    "name": "Interport Token",
    "symbol": "ITP",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/28338/thumb/ITP_Logo_200.png?1696527344"
  },
  {
    "chainId": 324,
    "address": "0xd63ef5e9c628c8a0e8984cdfb7444aee44b09044",
    "name": "CVI",
    "symbol": "GOVI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/13875/thumb/GOVI.png?1696513619"
  },
  {
    "chainId": 324,
    "address": "0x81fe5d73a48336129888d71bf164209cf646540f",
    "name": "Unlock Maverick",
    "symbol": "UNKMAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/31264/thumb/clean.jpg?1696530088"
  },
  {
    "chainId": 324,
    "address": "0x8e86e46278518efc1c5ced245cba2c7e3ef11557",
    "name": "Overnight fi USD ",
    "symbol": "USD+",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/25757/thumb/USD__logo.png?1696524843"
  },
  {
    "chainId": 324,
    "address": "0xe593853b4d603d5b8f21036bb4ad0d1880097a6e",
    "name": "Fulcrom",
    "symbol": "FUL",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/29684/thumb/FUL_Token_%281%29.png?1696528618"
  },
  {
    "chainId": 324,
    "address": "0x81e7186947fb59aaaaeb476a47daac60680cbbaf",
    "name": "Wefi Finance",
    "symbol": "WEFI",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/30540/thumb/wefi.png?1696529412"
  },
  {
    "chainId": 324,
    "address": "0xc8ac6191cdc9c7bf846ad6b52aaaa7a0757ee305",
    "name": "Metavault Trade",
    "symbol": "MVX",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/25402/thumb/mvx.png?1696524534"
  },
  {
    "chainId": 324,
    "address": "0xb7f5b452b381a90ba10bdde6d75c6a4fd65a4239",
    "name": "Rogue MAV",
    "symbol": "RMAV",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/32384/thumb/rMAV.png?1698045143"
  },
  {
    "chainId": 324,
    "address": "0xdd9f72afed3631a6c85b5369d84875e6c42f1827",
    "name": "Symbiosis",
    "symbol": "SIS",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/20805/thumb/SymbiosisFinance_logo-150x150.jpeg?1696520198"
  },
  {
    "chainId": 324,
    "address": "0x3613ad277df1d5935d41400a181aa9ec1dc2dc9e",
    "name": "Wagmi",
    "symbol": "WAGMI",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/31887/thumb/wagmi_token_logo.png?1696530698"
  },
  {
    "chainId": 324,
    "address": "0xdea6d5161978d36b5c0fa6a491faa754f4bc809c",
    "name": "Idexo",
    "symbol": "IDO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/18523/thumb/qOiqm7T8_400x400.jpg?1696518004"
  },
  {
    "chainId": 324,
    "address": "0xbbeb516fb02a01611cbbe0453fe3c580d7281011",
    "name": "Wrapped Bitcoin",
    "symbol": "WBTC",
    "decimals": 8,
    "logoURI": "https://assets.coingecko.com/coins/images/7598/thumb/wrapped_bitcoin_wbtc.png?1696507857"
  },
  {
    "chainId": 324,
    "address": "0xb6841b40baafedf4e37935270357ff89df42e68e",
    "name": "Stable USDLR",
    "symbol": "USDLR",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/33115/thumb/0x68592c5c98c4f4a8a4bc6da2121e65da3d1c0917.png?1700731571"
  },
  {
    "chainId": 324,
    "address": "0x1d17cbcf0d6d143135ae902365d2e5e2a16538d4",
    "name": "USDC",
    "symbol": "USDC",
    "decimals": 6,
    "logoURI": "https://assets.coingecko.com/coins/images/6319/thumb/usdc.png?1696506694"
  },
  {
    "chainId": 324,
    "address": "0x75c568a970e54a075b2a5475264cfcff672d2b89",
    "name": "ZKGROK",
    "symbol": "ZKGROK",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/33806/thumb/zkgrok_200_200.png?1703117664"
  },
  {
    "chainId": 324,
    "address": "0xe75a17b4f5c4f844688d5670b684515d7c785e63",
    "name": "Veno Finance",
    "symbol": "VNO",
    "decimals": 18,
    "logoURI": "https://assets.coingecko.com/coins/images/28799/thumb/Veno_Token_dark.png?1696527777"
  }
]
